(function(mod) {
  if (typeof exports == "object" && typeof module == "object") // CommonJS
    return mod(require("tern/lib/infer"), require("tern/lib/tern"));
  if (typeof define == "function" && define.amd) // AMD
    return define([ "tern/lib/infer", "tern/lib/tern" ], mod);
  mod(tern, tern);
})(function(infer, tern) {
  "use strict";

  tern.registerPlugin("extjs5.0.0", function(server, options) {
    server._extjs = {};
    return {
      defs : defs
    };
  });

  var defs = {"!name":"extjs","!define":{},"Ext":{"Action":{"!doc":"<p>An Action is a piece of reusable functionality that can be abstracted out of any particular component so that it\ncan be usefully shared among multiple components.  Actions let you share handlers, configuration options and UI\nupdates across any components that support the Action interface (primarily <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Ext.toolbar.Toolbar</a>,\n<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a> and <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a> components).</p>\n\n<p>Use a single Action instance as the config object for any number of UI Components which share the same configuration. The\nAction not only supplies the configuration, but allows all Components based upon it to have a common set of methods\ncalled at once through a single call to the Action.</p>\n\n<p>Any Component that is to be configured with an Action must also support\nthe following methods:</p>\n\n<ul>\n<li>setText(string)</li>\n<li>setIconCls(string)</li>\n<li>setDisabled(boolean)</li>\n<li>setVisible(boolean)</li>\n<li>setHandler(function)</li>\n</ul>\n\n\n<p>This allows the Action to control its associated Components.</p>\n\n<p>Example usage:</p>\n\n<pre><code>// Define the shared Action.  Each Component below will have the same\n// display text and icon, and will display the same message on click.\nvar action = new <a href=\"#!/api/Ext.Action\" rel=\"Ext.Action\" class=\"docClass\">Ext.Action</a>({\n    <a href=\"#!/api/Ext.Action-cfg-text\" rel=\"Ext.Action-cfg-text\" class=\"docClass\">text</a>: 'Do something',\n    <a href=\"#!/api/Ext.Action-cfg-handler\" rel=\"Ext.Action-cfg-handler\" class=\"docClass\">handler</a>: function(){\n        <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Click', 'You did something.');\n    },\n    <a href=\"#!/api/Ext.Action-cfg-iconCls\" rel=\"Ext.Action-cfg-iconCls\" class=\"docClass\">iconCls</a>: 'do-something',\n    <a href=\"#!/api/Ext.Action-cfg-itemId\" rel=\"Ext.Action-cfg-itemId\" class=\"docClass\">itemId</a>: 'myAction'\n});\n\nvar panel = new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({\n    title: 'Actions',\n    width: 500,\n    height: 300,\n    tbar: [\n        // Add the Action directly to a toolbar as a menu button\n        action,\n        {\n            text: 'Action Menu',\n            // Add the Action to a menu as a text item\n            menu: [action]\n        }\n    ],\n    items: [\n        // Add the Action to the panel body as a standard button\n        new <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>(action)\n    ],\n    renderTo: Ext.getBody()\n});\n\n// Change the text for all components using the Action\naction.setText('Something else');\n\n// Reference an Action through a container using the itemId\nvar btn = panel.getComponent('myAction');\nvar aRef = btn.baseAction;\naRef.setText('New text');\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"disabled":{"!type":"bool","!doc":"<p>True to disable all components configured by this Action, false to enable them.</p>\n"},"handler":{"!doc":"<p>The function that will be invoked by each component tied to this Action\nwhen the component's primary event is triggered.</p>\n"},"hidden":{"!type":"bool","!doc":"<p>True to hide all components configured by this Action, false to show them.</p>\n"},"iconCls":{"!type":"string","!doc":"<p>The CSS class selector that specifies a background image to be used as the header icon for\nall components configured by this Action.</p>\n\n<p>An example of specifying a custom icon class would be something like:</p>\n\n<pre><code>// specify the property in the config for the class:\n     ...\n     iconCls: 'do-something'\n\n// css class that specifies background image to be used as the icon image:\n.do-something { background-image: url(../images/my-icon.gif) 0 6px no-repeat !important; }\n</code></pre>\n\n<p>There are no default icon classes that come with Ext JS.</p>\n"},"itemId":{"!type":"string","!doc":"<p>See <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.<a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">itemId</a>.</p>\n"},"scope":{"!doc":"<p>The scope (this reference) in which the <a href=\"#!/api/Ext.Action-cfg-handler\" rel=\"Ext.Action-cfg-handler\" class=\"docClass\">handler</a> is executed.\nDefaults to the browser window.</p>\n"},"text":{"!type":"string","!doc":"<p>The text to set for all components configured by this Action.</p>\n"},"isAction":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Action, or subclass thereof.</p>\n"},"addComponent":{"!type":"fn(comp: ?) -> !this"},"callEach":{"!type":"fn(fnName: ?, args: ?) -> !this"},"disable":{"!type":"fn() -> !this","!doc":"<p>Disables all components configured by this Action.</p>\n"},"each":{"!type":"fn(fn: ?, scope: ?) -> !this","!doc":"<p>Executes the specified function once for each Component currently tied to this Action.  The function passed\nin should accept a single argument that will be an object that supports the basic Action config/method interface.</p>\n"},"enable":{"!type":"fn() -> !this","!doc":"<p>Enables all components configured by this Action.</p>\n"},"execute":{"!type":"fn(args: ?) -> !this","!doc":"<p>Executes this Action manually using the handler function specified in the original config object\nor the handler function set with <a href=\"#!/api/Ext.Action-method-setHandler\" rel=\"Ext.Action-method-setHandler\" class=\"docClass\">setHandler</a>.  Any arguments passed to this\nfunction will be passed on to the handler function.</p>\n"},"getIconCls":{"!type":"fn() -> !this","!doc":"<p>Gets the icon CSS class currently used by all components configured by this Action.</p>\n"},"getText":{"!type":"fn() -> !this","!doc":"<p>Gets the text currently displayed by all components configured by this Action.</p>\n"},"hide":{"!type":"fn() -> !this","!doc":"<p>Hides all components configured by this Action.</p>\n"},"isDisabled":{"!type":"fn() -> !this","!doc":"<p>Returns true if the components using this Action are currently disabled, else returns false.</p>\n"},"isHidden":{"!type":"fn() -> !this","!doc":"<p>Returns true if the components configured by this Action are currently hidden, else returns false.</p>\n"},"removeComponent":{"!type":"fn(comp: ?) -> !this"},"setDisabled":{"!type":"fn(disabled: ?) -> !this","!doc":"<p>Sets the disabled state of all components configured by this Action.  Shortcut method\nfor <a href=\"#!/api/Ext.Action-method-enable\" rel=\"Ext.Action-method-enable\" class=\"docClass\">enable</a> and <a href=\"#!/api/Ext.Action-method-disable\" rel=\"Ext.Action-method-disable\" class=\"docClass\">disable</a>.</p>\n"},"setHandler":{"!type":"fn(fn: ?, scope: ?) -> !this","!doc":"<p>Sets the function that will be called by each Component using this action when its primary event is triggered.</p>\n"},"setHidden":{"!type":"fn(hidden: ?) -> !this","!doc":"<p>Sets the hidden state of all components configured by this Action.  Shortcut method\nfor <code><a href=\"#!/api/Ext.Action-method-hide\" rel=\"Ext.Action-method-hide\" class=\"docClass\">hide</a></code> and <code><a href=\"#!/api/Ext.Action-method-show\" rel=\"Ext.Action-method-show\" class=\"docClass\">show</a></code>.</p>\n"},"setIconCls":{"!type":"fn(cls: ?) -> !this","!doc":"<p>Sets the icon CSS class for all components configured by this Action.  The class should supply\na background image that will be used as the icon image.</p>\n"},"setText":{"!type":"fn(text: ?) -> !this","!doc":"<p>Sets the text to be displayed by all components configured by this Action.</p>\n"},"show":{"!type":"fn() -> !this","!doc":"<p>Shows all components configured by this Action.</p>\n"}}},"app":{"Application":{"!doc":"<p>Represents an Ext JS application, which is typically a single page app using a\n<a href=\"#!/api/Ext.container.Viewport\" rel=\"Ext.container.Viewport\" class=\"docClass\">Viewport</a>.</p>\n\n<p>An application consists of one or more Views. The behavior of a View is managed by its\ncorresponding <a href=\"#!/api/Ext.app.ViewController\" rel=\"Ext.app.ViewController\" class=\"docClass\">ViewController</a> and ViewModel.</p>\n\n<p>Global activities are coordinated by <a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Controllers</a> which are\nultimately instantiated by an instance of this (or a derived) class.</p>\n\n<pre><code>Ext.application({\n    name: 'MyApp',\n\n    // An instance of this view is created and set as the Viewport:\n    autoCreateViewport: 'MyApp.view.Main'\n});\n</code></pre>\n\n<p>This does several things. First it creates a global variable called 'MyApp' - all of\nyour Application's classes (such as its Models, Views and Controllers) will reside under\nthis single namespace, which drastically lowers the chances of colliding global variables.</p>\n\n<p>The MyApp global will also have a getApplication method to get a reference to the current\napplication:</p>\n\n<pre><code>var app = MyApp.getApplication();\n</code></pre>\n\n<h1>Telling Application about the rest of the app</h1>\n\n<p>Because an <a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Ext.app.Application</a> represents an entire app, we should tell it about the other\nparts of the app - namely the Models, Views and Controllers that are bundled with the application. Let's say we have a blog management app; we\nmight have Models and Controllers for Posts and Comments, and Views for listing, adding and editing Posts and Comments.\nHere's how we'd tell our Application about all these things:</p>\n\n<pre><code>Ext.application({\n    name: 'Blog',\n\n    models: ['Post', 'Comment'],\n\n    controllers: ['Posts', 'Comments'],\n\n    launch: function() {\n        ...\n    }\n});\n</code></pre>\n\n<p>Note that we didn't actually list the Views directly in the Application itself. This is because Views are managed by\nControllers, so it makes sense to keep those dependencies there. The Application will load each of the specified\nControllers using the pathing conventions laid out in the <a href=\"#/guide/application_architecture\">application architecture guide</a> - in this case\nexpecting the controllers to reside in app/controller/Posts.js and app/controller/Comments.js. In turn, each\nController simply needs to list the Views it uses and they will be automatically loaded. Here's how our Posts\ncontroller like be defined:</p>\n\n<pre><code>Ext.define('MyApp.controller.Posts', {\n    extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n    views: ['posts.List', 'posts.Edit'],\n\n    //the rest of the Controller here\n});\n</code></pre>\n\n<p>Because we told our Application about our Models and Controllers, and our Controllers about their Views, Ext JS will\nautomatically load all of our app files for us. This means we don't have to manually add script tags into our html\nfiles whenever we add a new class, but more importantly it enables us to create a minimized build of our entire\napplication using Sencha Cmd.</p>\n\n<h1>Deriving from <a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Ext.app.Application</a></h1>\n\n<p>Typically, applications do not derive directly from <a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Ext.app.Application</a>. Rather, the\nconfiguration passed to <code>Ext.application</code> mimics what you might do in a derived class.\nIn some cases, however, it can be desirable to share logic by using a derived class\nfrom <code><a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Ext.app.Application</a></code>.</p>\n\n<p>Derivation works as you would expect, but using the derived class should still be the\njob of the <code>Ext.application</code> method.</p>\n\n<pre><code>Ext.define('MyApp.app.Application', {\n    extend: '<a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Ext.app.Application</a>',\n    name: 'MyApp',\n    ...\n});\n\nExt.application('MyApp.app.Application');\n</code></pre>\n\n<p>For more information about writing Ext JS applications, please see the <a href=\"#/guide/application_architecture\">application architecture guide</a>.</p>\n","!type":"fn(config?: ?)","prototype":{"appFolder":{"!type":"string","!doc":"<p>The path to the directory which contains all application's classes.\nThis path will be registered via Ext.Loader.setPath for the namespace specified\nin the <a href=\"#!/api/Ext.app.Application-cfg-name\" rel=\"Ext.app.Application-cfg-name\" class=\"docClass\">name</a> config.</p>\n"},"appProperty":{"!type":"string","!doc":"<p>The name of a property to be assigned to the main namespace to gain a reference to\nthis application. Can be set to an empty value to prevent the reference from\nbeing created</p>\n\n<pre><code>Ext.application({\n    name: 'MyApp',\n    appProperty: 'myProp',\n\n    launch: function() {\n        console.log(MyApp.myProp === this);\n    }\n});\n</code></pre>\n"},"autoCreateViewport":{"!doc":"<p>Set to <code>true</code> to automatically load and instantiate <code>AppName.view.Viewport</code>\nbefore firing the launch function. Otherwise this is the name of the view to\ncreate and apply the <code>viewport</code> plugin.</p>\n\n<p>@cmd-auto-dependency {aliasPrefix: \"view.\", mvc: true, requires: [\"<a href=\"#!/api/Ext.plugin.Viewport\" rel=\"Ext.plugin.Viewport\" class=\"docClass\">Ext.plugin.Viewport</a>\"]}</p>\n"},"controllers":{"!doc":"<p>Names of controllers that the app uses.</p>\n"},"defaultToken":{"!type":"string","!doc":"<p>The default token to be used at application launch\nif one is not present. Often this is set to something like <code>'home'</code>.</p>\n"},"enableQuickTips":{"!type":"bool","!doc":"<p>True to automatically set up <a href=\"#!/api/Ext.tip.QuickTip\" rel=\"Ext.tip.QuickTip\" class=\"docClass\">Ext.tip.QuickTip</a> support.</p>\n"},"extend":{"!type":"string","!doc":"<p>A class name to use with the <code>Ext.application</code> call. The class must also extend <a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Ext.app.Application</a>.</p>\n\n<pre><code>Ext.define('MyApp.Application', {\n    extend : '<a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Ext.app.Application</a>',\n\n    launch : function() {\n        Ext.direct.Manager.addProvider(Ext.app.REMOTING_API);\n    }\n});\n\nExt.application({\n    extend : 'MyApp.Application'\n});\n</code></pre>\n"},"glyphFontFamily":{"!type":"string","!doc":"<p>The glyphFontFamily to use for this application.  Used as the default font-family\nfor all components that support a <code>glyph</code> config.</p>\n"},"name":{"!type":"string","!doc":"<p>The name of your application. This will also be the namespace for your views, controllers\nmodels and stores. Don't use spaces or special characters in the name. <strong>Application name\nis mandatory</strong>.</p>\n"},"namespaces":{"!doc":"<p>The list of namespace prefixes used in the application to resolve dependencies\nlike Views and Stores:</p>\n\n<pre><code> Ext.application({\n     name: 'MyApp',\n\n     namespaces: ['Common.code'],\n\n     controllers: [ 'Common.code.controller.Foo', 'Bar' ]\n });\n\n Ext.define('Common.code.controller.Foo', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     models: ['Foo'],    // Loads Common.code.model.Foo\n     views:  ['Bar']     // Loads Common.code.view.Bar\n });\n\n Ext.define('MyApp.controller.Bar', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     models: ['Foo'],    // Loads MyApp.model.Foo\n     views:  ['Bar']     // Loads MyApp.view.Bar\n });\n</code></pre>\n\n<p>You don't need to include main namespace (MyApp), it will be added to the list\nautomatically.</p>\n"},"paths":{"!doc":"<p>Additional load paths to add to Ext.Loader.\nSee Ext.Loader.paths config for more details.</p>\n"},"scope":{"!doc":"<p>The scope to execute the <a href=\"#!/api/Ext.app.Application-method-launch\" rel=\"Ext.app.Application-method-launch\" class=\"docClass\">launch</a> function in. Defaults to the Application instance.</p>\n"},"isApplication":{"!type":"bool"},"createController":{"!type":"fn(name: ?) -> ?","!doc":"<p>Create an instance of a controller by name.</p>\n"},"destroy":{"!type":"fn(destroyRefs: ?) -> !this","!doc":"<p>Inherit docs</p>\n"},"destroyController":{"!type":"fn(controller: ?) -> !this","!doc":"<p>Destroys a controller, any listeners are unbound.</p>\n"},"finishInitControllers":{"!type":"fn() -> !this"},"getAppProperty":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.Application-cfg-appProperty\" rel=\"Ext.app.Application-cfg-appProperty\" class=\"docClass\">appProperty</a>.</p>\n"},"getApplication":{"!type":"fn() -> ?","!doc":"<p>Inherit docs</p>\n"},"getAutoCreateViewport":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.Application-cfg-autoCreateViewport\" rel=\"Ext.app.Application-cfg-autoCreateViewport\" class=\"docClass\">autoCreateViewport</a>.</p>\n"},"getController":{"!type":"fn(name: ?, preventCreate: ?) -> ?","!doc":"<p>Returns instance of a <a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Controller</a> with the given id.\nWhen controller doesn't exist yet, it's created. Note that this method depends\non Application instance and will return undefined when Application is not\naccessible. The only exception is when this Controller instance's id is requested;\nin that case we always return the instance even if Application is no available.</p>\n"},"getDefaultToken":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.Application-cfg-defaultToken\" rel=\"Ext.app.Application-cfg-defaultToken\" class=\"docClass\">defaultToken</a>.</p>\n"},"getEnableQuickTips":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.Application-cfg-enableQuickTips\" rel=\"Ext.app.Application-cfg-enableQuickTips\" class=\"docClass\">enableQuickTips</a>.</p>\n"},"getGlyphFontFamily":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.Application-cfg-glyphFontFamily\" rel=\"Ext.app.Application-cfg-glyphFontFamily\" class=\"docClass\">glyphFontFamily</a>.</p>\n"},"getModuleClassName":{"!type":"fn(name: ?, kind: ?) -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.Application-cfg-moduleClassName\" rel=\"Ext.app.Application-cfg-moduleClassName\" class=\"docClass\">moduleClassName</a>.</p>\n"},"getName":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.Application-cfg-name\" rel=\"Ext.app.Application-cfg-name\" class=\"docClass\">name</a>.</p>\n"},"getViewportName":{"!type":"fn() -> !this"},"initControllers":{"!type":"fn() -> !this"},"initNamespace":{"!type":"fn() -> !this"},"initQuickTips":{"!type":"fn() -> !this"},"initViewport":{"!type":"fn() -> !this"},"launch":{"!type":"fn(profile: ?) -> bool","!doc":"<p>Called automatically when the page has completely loaded. This is an empty function that should be\noverridden by each application that needs to take action on page load.</p>\n"},"onBeforeLaunch":{"!type":"fn() -> !this"},"onClassExtended":{"!type":"fn(cls: ?, data: ?, hooks: ?) -> !this"},"setAppProperty":{"!type":"fn(appProperty: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.Application-cfg-appProperty\" rel=\"Ext.app.Application-cfg-appProperty\" class=\"docClass\">appProperty</a>.</p>\n"},"setAutoCreateViewport":{"!type":"fn(autoCreateViewport: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.Application-cfg-autoCreateViewport\" rel=\"Ext.app.Application-cfg-autoCreateViewport\" class=\"docClass\">autoCreateViewport</a>.</p>\n"},"setDefaultToken":{"!type":"fn(defaultToken: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.Application-cfg-defaultToken\" rel=\"Ext.app.Application-cfg-defaultToken\" class=\"docClass\">defaultToken</a>.</p>\n"},"setEnableQuickTips":{"!type":"fn(enableQuickTips: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.Application-cfg-enableQuickTips\" rel=\"Ext.app.Application-cfg-enableQuickTips\" class=\"docClass\">enableQuickTips</a>.</p>\n"},"setGlyphFontFamily":{"!type":"fn(glyphFontFamily: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.Application-cfg-glyphFontFamily\" rel=\"Ext.app.Application-cfg-glyphFontFamily\" class=\"docClass\">glyphFontFamily</a>.</p>\n"},"setName":{"!type":"fn(name: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.Application-cfg-name\" rel=\"Ext.app.Application-cfg-name\" class=\"docClass\">name</a>.</p>\n"},"unregister":{"!type":"fn(controller: ?) -> !this","!doc":"<p>Unregister a controller from the application.</p>\n"},"updateGlyphFontFamily":{"!type":"fn(fontFamily: ?) -> !this"}}},"Controller":{"prototype":{"active":{"!type":"bool"},"application":{"!doc":"<p>The <a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Ext.app.Application</a> for this controller accessible via the getApplication method.</p>\n"},"models":{"!doc":"<p>Array of models to require from AppName.model namespace. For example:</p>\n\n<pre><code> Ext.define(\"MyApp.controller.Foo\", {\n     extend: \"<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>\",\n     models: ['User', 'Vehicle']\n });\n</code></pre>\n\n<p>This is equivalent of:</p>\n\n<pre><code> Ext.define(\"MyApp.controller.Foo\", {\n     extend: \"<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>\",\n     requires: ['MyApp.model.User', 'MyApp.model.Vehicle'],\n\n     getUserModel: function() {\n         return this.getModel(\"User\");\n     },\n\n     getVehicleModel: function() {\n         return this.getModel(\"Vehicle\");\n     }\n });\n</code></pre>\n"},"moduleClassName":{"!doc":"<p>private</p>\n"},"refs":{"!doc":"<p>You can specify refs with either an Object or an Array:</p>\n\n<pre><code> Ext.define('MyApp.controller.Foo', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     config: {\n         refs: {\n             list: 'grid',\n             user: {\n                 autoCreate: true,\n                 selector: 'form',\n                 xtype: 'form'\n             }\n         }\n     }\n });\n</code></pre>\n\n<p>This will add the <code>getList</code> and <code>getUser</code> methods to the controller which will internally use\nExt.ComponentQuery to reference the resolved component.</p>\n\n<pre><code> Ext.define('MyApp.controller.Foo', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     config : {\n         refs: [{\n             ref: 'list',\n             selector: 'grid'\n         }]\n     }\n });\n</code></pre>\n\n<p>This will add method <code>getList</code> to the controller which will internally use\nExt.ComponentQuery to reference the grid component on page.</p>\n\n<p>The recommended way to use refs is within the config object but legacy means of specifying\nrefs as a sibling of the config object is still supported.</p>\n\n<p>The following fields can be used in ref definition:</p>\n\n<ul>\n<li><code>ref</code> - name of the reference.</li>\n<li><code>selector</code> - Ext.ComponentQuery selector to access the component.</li>\n<li><code>autoCreate</code> - True to create the component automatically if not found on page.</li>\n<li><code>forceCreate</code> - Forces the creation of the component every time reference is accessed\n(when <code>get&lt;REFNAME&gt;</code> is called).</li>\n<li><code>xtype</code> - Used to create component by its xtype with autoCreate or forceCreate. If\nyou don't provide xtype, an <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> instance will be created.</li>\n</ul>\n\n"},"stores":{"!doc":"<p>Array of stores to require from AppName.store namespace and to generate getter methods for.\nFor example:</p>\n\n<pre><code> Ext.define(\"MyApp.controller.Foo\", {\n     extend: \"<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>\",\n     stores: ['Users', 'Vehicles']\n });\n</code></pre>\n\n<p>This is equivalent to:</p>\n\n<pre><code> Ext.define(\"MyApp.controller.Foo\", {\n     extend: \"<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>\",\n\n     requires: [\n         'MyApp.store.Users',\n         'MyApp.store.Vehicles'\n     ]\n\n     getUsersStore: function() {\n         return this.getStore(\"Users\");\n     },\n\n     getVehiclesStore: function() {\n         return this.getStore(\"Vehicles\");\n     }\n });\n</code></pre>\n"},"views":{"!doc":"<p>Array of views to require from AppName.view namespace and to generate getter methods for.\nFor example:</p>\n\n<pre><code> Ext.define(\"MyApp.controller.Foo\", {\n     extend: \"<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>\",\n     views: ['List', 'Detail']\n });\n</code></pre>\n\n<p>This is equivalent of:</p>\n\n<pre><code> Ext.define(\"MyApp.controller.Foo\", {\n     extend: \"<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>\",\n     requires: ['MyApp.view.List', 'MyApp.view.Detail'],\n\n     getListView: function() {\n         return this.getView(\"List\");\n     },\n\n     getDetailView: function() {\n         return this.getView(\"Detail\");\n     }\n });\n</code></pre>\n"},"activate":{"!type":"fn() -> !this","!doc":"<p>Allow the controller to resume receiving events from the event bus.\nRoutes will also be able to begin firing on this controller.\nAlso see <a href=\"#!/api/Ext.app.Controller-method-deactivate\" rel=\"Ext.app.Controller-method-deactivate\" class=\"docClass\">deactivate</a>.</p>\n"},"addRef":{"!type":"fn(refs: ?) -> !this","!doc":"<p>Registers one or more <a href=\"#!/api/Ext.app.Controller-cfg-refs\" rel=\"Ext.app.Controller-cfg-refs\" class=\"docClass\">references</a>.</p>\n"},"applyRefs":{"!type":"fn(refs: ?) -> !this"},"deactivate":{"!type":"fn() -> !this","!doc":"<p>Prevent this controller from receiving events from the event bus.\nRoutes will also not be triggered on inactive controllers unless\nthe <a href=\"#!/api/Ext.app.route.Route-cfg-allowInactive\" rel=\"Ext.app.route.Route-cfg-allowInactive\" class=\"docClass\">Ext.app.route.Route.allowInactive</a> flag is set.\nAlso see <a href=\"#!/api/Ext.app.Controller-method-activate\" rel=\"Ext.app.Controller-method-activate\" class=\"docClass\">activate</a>.</p>\n"},"doInit":{"!type":"fn(app: ?) -> !this"},"ensureId":{"!type":"fn() -> !this"},"finishInit":{"!type":"fn(app: ?) -> !this"},"getActive":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.Controller-cfg-active\" rel=\"Ext.app.Controller-cfg-active\" class=\"docClass\">active</a>.</p>\n"},"getModel":{"!type":"fn(name: ?) -> ?","!doc":"<p>Returns a Model class with the given name.</p>\n"},"getRef":{"!type":"fn(ref: ?, info: ?, config: ?) -> !this"},"getRefs":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.Controller-cfg-refs\" rel=\"Ext.app.Controller-cfg-refs\" class=\"docClass\">refs</a>.</p>\n"},"getStore":{"!type":"fn(name: ?) -> ?","!doc":"<p>Returns instance of a Store with the given name.\nWhen store doesn't exist yet, it's created.</p>\n"},"getView":{"!type":"fn(view: ?) -> ?","!doc":"<p>Returns a View class with the given name.  To create an instance of the view,\nyou can use it like it's used by Application to create the Viewport:</p>\n\n<pre><code>this.getView('Viewport').create();\n</code></pre>\n"},"hasRef":{"!type":"fn(ref: ?) -> bool","!doc":"<p>Returns <code>true</code> if a <a href=\"#!/api/Ext.app.Controller-cfg-refs\" rel=\"Ext.app.Controller-cfg-refs\" class=\"docClass\">reference</a> is registered.</p>\n"},"init":{"!type":"fn(application: ?) -> !this","!doc":"<p>A template method that is called when your application boots. It is called before the\n<a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Application</a>'s launch function is executed so gives a hook point\nto run any code before your Viewport is created.</p>\n"},"initAutoGetters":{"!type":"fn() -> !this"},"isActive":{"!type":"fn() -> bool","!doc":"<p>Checks if this controller is active. See <a href=\"#!/api/Ext.app.Controller-method-activate\" rel=\"Ext.app.Controller-method-activate\" class=\"docClass\">activate</a> &amp;\n<a href=\"#!/api/Ext.app.Controller-method-deactivate\" rel=\"Ext.app.Controller-method-deactivate\" class=\"docClass\">deactivate</a>.</p>\n"},"normalizeRefs":{"!type":"fn(refs: ?, newRefs: ?) -> ?","!doc":"<p>Takes either an object and transforms it into an array. The following are valid refs values:</p>\n\n<pre><code>refs: {\n    myComponent: 'container'\n}\n\nrefs: {\n    myComponent: {\n        selector: 'container'\n    }\n}\n\nrefs: [\n    {\n        ref: 'myComponent',\n        selector: 'container'\n    }\n]\n</code></pre>\n"},"onLaunch":{"!type":"fn(application: ?) -> !this","!doc":"<p>A template method like <a href=\"#!/api/Ext.app.Controller-method-init\" rel=\"Ext.app.Controller-method-init\" class=\"docClass\">init</a>, but called after the viewport is created.\nThis is called after the <a href=\"#!/api/Ext.app.Application-method-launch\" rel=\"Ext.app.Application-method-launch\" class=\"docClass\">launch</a> method of Application\nis executed.</p>\n"},"ref":{"!type":"fn(refs: ?) -> !this"},"setActive":{"!type":"fn(active: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.Controller-cfg-active\" rel=\"Ext.app.Controller-cfg-active\" class=\"docClass\">active</a>.</p>\n"},"setApplication":{"!type":"fn(application: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.Controller-cfg-application\" rel=\"Ext.app.Controller-cfg-application\" class=\"docClass\">application</a>.</p>\n"},"setModuleClassName":{"!type":"fn(moduleClassName: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.Controller-cfg-moduleClassName\" rel=\"Ext.app.Controller-cfg-moduleClassName\" class=\"docClass\">moduleClassName</a>.</p>\n"},"setRefs":{"!type":"fn(refs: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.Controller-cfg-refs\" rel=\"Ext.app.Controller-cfg-refs\" class=\"docClass\">refs</a>.</p>\n"},"updateRefs":{"!type":"fn(refs: ?) -> !this"},"controllerRegex":{},"strings":{},"destroy":{"!type":"fn(destroyRefs: ?, fromApp: ?) -> !this"},"getApplication":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.Controller-cfg-application\" rel=\"Ext.app.Controller-cfg-application\" class=\"docClass\">application</a>.</p>\n"},"getController":{"!type":"fn(id: ?) -> ?","!doc":"<p>Returns instance of a <a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Controller</a> with the given id.\nWhen controller doesn't exist yet, it's created. Note that this method depends\non Application instance and will return undefined when Application is not\naccessible. The only exception is when this Controller instance's id is requested;\nin that case we always return the instance even if Application is no available.</p>\n"},"getModuleClassName":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.Controller-cfg-moduleClassName\" rel=\"Ext.app.Controller-cfg-moduleClassName\" class=\"docClass\">moduleClassName</a>.</p>\n"},"onClassExtended":{"!type":"fn(cls: ?, data: ?, hooks: ?) -> !this"},"createGetter":{"!type":"fn(baseGetter: ?, name: ?) -> !this"},"getFullName":{"!type":"fn(name: ?, kind: ?, namespace: ?) -> !this"},"getGetterName":{"!type":"fn(name: ?, kindUpper: ?) -> !this"},"hasRegisteredPrefix":{"!type":"fn(className: ?) -> !this"},"processDependencies":{"!type":"fn(cls: ?, requires: ?, namespace: ?, kind: ?, names: ?) -> !this","!doc":"<p>This method is called like so:</p>\n\n<pre><code> <a href=\"#!/api/Ext.app.Controller-static-method-processDependencies\" rel=\"Ext.app.Controller-static-method-processDependencies\" class=\"docClass\">Ext.app.Controller.processDependencies</a>(proto, requiresArray, 'MyApp', 'model', [\n     'User',\n     'Item',\n     'Foo@Common.model',\n     'Bar.Baz@Common.model'\n ]);\n</code></pre>\n\n<p>Required dependencies are added to requiresArray.</p>\n"}},"!doc":"<p>Controllers are the glue that binds an application together. All they really do is listen for events (usually from\nviews) and take some action. Here's how we might create a Controller to manage Users:</p>\n\n<pre><code> Ext.define('MyApp.controller.Users', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     init: function() {\n         console.log('Initialized Users! This happens before ' +\n                     'the Application launch() function is called');\n     }\n });\n</code></pre>\n\n<p>The init function is a special method that is called when your application boots. It is called before the\n<a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Application</a>'s launch function is executed so gives a hook point to run any code before\nyour Viewport is created.</p>\n\n<p>The init function is a great place to set up how your controller interacts with the view, and is usually used in\nconjunction with another Controller function - <a href=\"#!/api/Ext.app.BaseController-method-control\" rel=\"Ext.app.BaseController-method-control\" class=\"docClass\">control</a>. The control function\nmakes it easy to listen to events on your view classes and take some action with a handler function. Let's update\nour Users controller to tell us when the panel is rendered:</p>\n\n<pre><code> Ext.define('MyApp.controller.Users', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     init: function() {\n         this.control({\n             'viewport &gt; panel': {\n                 render: this.onPanelRendered\n             }\n         });\n     },\n\n     onPanelRendered: function() {\n         console.log('The panel was rendered');\n     }\n });\n</code></pre>\n\n<p>We've updated the init function to use <a href=\"#!/api/Ext.app.BaseController-method-control\" rel=\"Ext.app.BaseController-method-control\" class=\"docClass\">control method</a> to set up listeners on views\nin our application. The control method uses the ComponentQuery engine to quickly and easily get references to components\non the page. If you are not familiar with ComponentQuery yet, be sure to check out the\ndocumentation. In brief though, it allows us to pass a CSS-like selector that will find\nevery matching component on the page.</p>\n\n<p>In our init function above we supplied 'viewport > panel', which translates to \"find me every Panel that is a direct\nchild of a Viewport\". We then supplied an object that maps event names (just 'render' in this case) to handler\nfunctions. The overall effect is that whenever any component that matches our selector fires a 'render' event, our\nonPanelRendered function is called.</p>\n\n<h2>Event domains</h2>\n\n<p>In Ext JS 4.2, we introduced the concept of event domains. In terms of MVC, an event domain\nis one or more base classes that fire events to which a Controller wants to listen. Besides\nComponent event domain that encompass <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>-descended Views, Controllers now\ncan listen to events from data Stores, Ext.Direct Providers, other Controllers, and Ext.GlobalEvents.\nThis feature provides a way to communicate between parts of the whole application without the need\nto bind controllers together tightly, and allows to develop and test application parts in isolation.</p>\n\n<p>See usage examples in <a href=\"#!/api/Ext.app.BaseController-method-listen\" rel=\"Ext.app.BaseController-method-listen\" class=\"docClass\">listen</a> method documentation.</p>\n\n<h2>Using refs</h2>\n\n<p>One of the most useful parts of Controllers is the ref system. These use the Ext.ComponentQuery to\nmake it really easy to get references to Views on your page. Let's look at an example of this now:</p>\n\n<pre><code> Ext.define('MyApp.controller.Users', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     refs: [{\n         ref: 'list',\n         selector: 'grid'\n     }],\n\n     init: function() {\n         this.control({\n             'button': {\n                 click: this.refreshGrid\n             }\n         });\n     },\n\n     refreshGrid: function() {\n         this.getList().store.load();\n     }\n });\n</code></pre>\n\n<p>This example assumes the existence of a <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Grid</a> on the page, which contains a single button to\nrefresh the Grid when clicked. In our refs array, we set up a reference to the grid. There are two parts to this -\nthe 'selector', which is a ComponentQuery selector which finds any grid on the page and\nassigns it to the reference 'list'.</p>\n\n<p>By giving the reference a name, we get a number of things for free. The first is the getList function that we use in\nthe refreshGrid method above. This is generated automatically by the Controller based on the name of our ref, which\nwas capitalized and prepended with get to go from 'list' to 'getList'.</p>\n\n<p>The way this works is that the first time getList is called by your code, the ComponentQuery selector is run and the\nfirst component that matches the selector ('grid' in this case) will be returned. All future calls to getList will\nuse a cached reference to that grid. Usually it is advised to use a specific ComponentQuery selector that will only\nmatch a single View in your application (in the case above our selector will match any grid on the page).</p>\n\n<p>Bringing it all together, our init function is called when the application boots, at which time we call this.control\nto listen to any click on a <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">button</a> and call our refreshGrid function (again, this will\nmatch any button on the page so we advise a more specific selector than just 'button', but have left it this way for\nsimplicity). When the button is clicked we use out getList function to refresh the grid.</p>\n\n<p>You can create any number of refs and control any number of components this way, simply adding more functions to\nyour Controller as you go. For an example of real-world usage of Controllers see the Feed Viewer example in the\nexamples/app/feed-viewer folder in the SDK download.</p>\n\n<h2>Generated getter methods</h2>\n\n<p>Refs aren't the only thing that generate convenient getter methods. Controllers often have to deal with Models and\nStores so the framework offers a couple of easy ways to get access to those too. Let's look at another example:</p>\n\n<pre><code> Ext.define('MyApp.controller.Users', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     models: ['User'],\n     stores: ['AllUsers', 'AdminUsers'],\n\n     init: function() {\n         var User, allUsers, ed;\n\n         User = this.getUserModel();\n         allUsers = this.getAllUsersStore();\n\n         ed = new User({ name: 'Ed' });\n         allUsers.add(ed);\n     }\n });\n</code></pre>\n\n<p>By specifying Models and Stores that the Controller cares about, it again dynamically loads them from the appropriate\nlocations (app/model/User.js, app/store/AllUsers.js and app/store/AdminUsers.js in this case) and creates getter\nfunctions for them all. The example above will create a new User model instance and add it to the AllUsers Store.\nOf course, you could do anything in this function but in this case we just did something simple to demonstrate the\nfunctionality.</p>\n\n<h2>Further Reading</h2>\n\n<p>For more information about writing Ext JS 4 applications, please see the\n<a href=\"#/guide/application_architecture\">application architecture guide</a>. Also see the <a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Ext.app.Application</a>\ndocumentation.</p>\n","!type":"fn(config?: ?)"},"BaseController":{"prototype":{"before":{},"control":{"!doc":"<p>Adds listeners to components selected via Ext.ComponentQuery. Accepts an\nobject containing component paths mapped to a hash of listener functions.</p>\n\n<p>In the following example the <code>updateUser</code> function is mapped to to the <code>click</code>\nevent on a button component, which is a child of the <code>useredit</code> component.</p>\n\n<pre><code> Ext.define('MyApp.controller.Users', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     config: {\n         control: {\n             'useredit button[action=save]': {\n                 click: 'updateUser'\n             }\n          }\n     },\n\n     updateUser: function(button) {\n         console.log('clicked the Save button');\n     }\n });\n</code></pre>\n\n<p>The method you pass to the listener will automatically be resolved on the controller.\nIn this case, the <code>updateUser</code> method that will get executed on the <code>button</code> <code>click</code>\nevent will resolve to the <code>updateUser</code> method on the controller,</p>\n\n<p>See Ext.ComponentQuery for more information on component selectors.</p>\n"},"id":{"!type":"string","!doc":"<p>The id of this controller. You can use this id when dispatching.</p>\n"},"listen":{"!doc":"<p>Adds listeners to different event sources (also called \"event domains\"). The\nprimary event domain is that of components, but there are also other event domains:\n<a href=\"#!/api/Ext.app.domain.Global\" rel=\"Ext.app.domain.Global\" class=\"docClass\">Global</a> domain that intercepts events fired from\nExt.GlobalEvents Observable instance, <a href=\"#!/api/Ext.app.domain.Controller\" rel=\"Ext.app.domain.Controller\" class=\"docClass\">Controller</a>\ndomain can be used to listen to events fired by other Controllers,\n<a href=\"#!/api/Ext.app.domain.Store\" rel=\"Ext.app.domain.Store\" class=\"docClass\">Store</a> domain gives access to Store events, and\n<a href=\"#!/api/Ext.app.domain.Direct\" rel=\"Ext.app.domain.Direct\" class=\"docClass\">Direct</a> domain can be used with Ext.Direct Providers\nto listen to their events.</p>\n\n<p>To listen to \"bar\" events fired by a controller with id=\"foo\":</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     config: {\n         listen: {\n             controller: {\n                 '#foo': {\n                    bar: 'onFooBar'\n                 }\n             }\n         }\n     }\n });\n</code></pre>\n\n<p>To listen to \"bar\" events fired by any controller, and \"baz\" events\nfired by Store with storeId=\"baz\":</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     config: {\n         listen: {\n             controller: {\n                 '*': {\n                    bar: 'onAnyControllerBar'\n                 }\n             },\n             store: {\n                 '#baz': {\n                     baz: 'onStoreBaz'\n                 }\n             }\n         }\n     }\n });\n</code></pre>\n\n<p>To listen to \"idle\" events fired by Ext.GlobalEvents when other event\nprocessing is complete and Ext JS is about to return control to the browser:</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     config: {\n         listen: {\n             global: {            // Global events are always fired\n                 idle: 'onIdle'   // from the same object, so there\n             }                    // are no selectors\n         }\n     }\n });\n</code></pre>\n\n<p>As this relates to components, the following example:</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     config: {\n         listen: {\n             component: {\n                 'useredit button[action=save]': {\n                    click: 'updateUser'\n                 }\n             }\n         }\n     }\n });\n</code></pre>\n\n<p>Is equivalent to:</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     config: {\n         control: {\n             'useredit button[action=save]': {\n                click: 'updateUser'\n             }\n         }\n     }\n });\n</code></pre>\n\n<p>Of course, these can all be combined in a single call and used instead of\n<code>control</code>, like so:</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     extend: '<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>',\n\n     config: {\n         listen: {\n             global: {\n                 idle: 'onIdle'\n             },\n             controller: {\n                 '*': {\n                    foobar: 'onAnyFooBar'\n                 },\n                 '#foo': {\n                    bar: 'onFooBar'\n                 }\n             },\n             component: {\n                 'useredit button[action=save]': {\n                    click: 'updateUser'\n                 }\n             },\n             store: {\n                 '#qux': {\n                     load: 'onQuxLoad'\n                 }\n             }\n         }\n     }\n });\n</code></pre>\n"},"routes":{"!doc":"<p>An object of routes to handle hash changes. A route can be defined in a simple way:</p>\n\n<pre><code>routes : {\n    'foo/bar'  : 'handleFoo',\n    'user/:id' : 'showUser'\n}\n</code></pre>\n\n<p>Where the property is the hash (which can accept a parameter defined by a colon) and the value\nis the method on the controller to execute. The parameters will get sent in the action method.</p>\n\n<p>At the application level, you can define a event that will be executed when no matching\nroutes are found.</p>\n\n<pre><code>Ext.application({\n    name: 'MyApp',\n    listen: {\n        controller: {\n            '#': {\n                unmatchedroute: 'onUnmatchedRoute'\n            }\n        }\n    },\n\n    onUnmatchedRoute: function(hash) {\n        console.log('Unmatched', hash);\n        // Do something...\n    }\n});\n</code></pre>\n\n<p>There is also a complex means of defining a route where you can use a before action and even\nspecify your own RegEx for the parameter:</p>\n\n<pre><code>routes : {\n    'foo/bar'  : {\n        action  : 'handleFoo',\n        before  : 'beforeHandleFoo'\n    },\n    'user/:id' : {\n        action     : 'showUser',\n        before     : 'beforeShowUser',\n        conditions : {\n            ':id' : '([0-9]+)'\n        }\n    }\n}\n</code></pre>\n\n<p>This will only match if the <code>id</code> parameter is a number.</p>\n\n<p>The before action allows you to cancel an action. Every before action will get passed an <code>action</code> argument with\na <code>resume</code> and <code>stop</code> methods as the last argument of the method and you <em>MUST</em> execute either method:</p>\n\n<pre><code>beforeHandleFoo : function(action) {\n    //some logic here\n\n    //this will allow the handleFoo action to be executed\n    action.resume();\n},\nhandleFoo : function() {\n    //will get executed due to true being passed in callback in beforeHandleFoo\n},\nbeforeShowUser : function(id, action) {\n    //allows for async process like an Ajax\n    Ext.Ajax.request({\n        url     : 'foo.php',\n        success : function() {\n            //will not allow the showUser method to be executed but will continue other queued actions.\n            action.stop();\n        },\n        failure : function() {\n            //will not allow the showUser method to be executed and will not allow other queued actions to be executed.\n            action.stop(true);\n        }\n    });\n},\nshowUser : function(id) {\n    //will not get executed due to false being passed in callback in beforeShowUser\n}\n</code></pre>\n\n<p>You <em>MUST</em> execute the <code>resume</code> or <code>stop</code> method on the <code>action</code> argument. Executing <code>action.resume();</code> will continue\nthe action, <code>action.stop();</code> will not allow the action to resume but will allow other queued actions to resume,\n<code>action.stop(true);</code> will not allow the action and any other queued actions to resume.</p>\n\n<p>The default RegEx that will be used is <code>([%a-zA-Z0-9\\\\-\\\\_\\\\s,]+)</code> but you can specify any\nthat may suit what you need to accomplish. An example of an advanced condition may be to make\na parameter optional and case-insensitive:</p>\n\n<pre><code>routes : {\n    'user:id' : {\n        action     : 'showUser',\n        before     : 'beforeShowUser',\n        conditions : {\n            ':id' : '(?:(?:\\/){1}([%a-z0-9_,\\s\\-]+))?'\n        }\n    }\n}\n</code></pre>\n"},"isController":{"!type":"bool"},"applyControl":{"!type":"fn(control: ?) -> !this"},"applyListen":{"!type":"fn(listen: ?) -> !this"},"getBefore":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.BaseController-cfg-before\" rel=\"Ext.app.BaseController-cfg-before\" class=\"docClass\">before</a>.</p>\n"},"getControl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.BaseController-cfg-control\" rel=\"Ext.app.BaseController-cfg-control\" class=\"docClass\">control</a>.</p>\n"},"getId":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.BaseController-cfg-id\" rel=\"Ext.app.BaseController-cfg-id\" class=\"docClass\">id</a>.</p>\n"},"getListen":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.BaseController-cfg-listen\" rel=\"Ext.app.BaseController-cfg-listen\" class=\"docClass\">listen</a>.</p>\n"},"getRoutes":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.BaseController-cfg-routes\" rel=\"Ext.app.BaseController-cfg-routes\" class=\"docClass\">routes</a>.</p>\n"},"redirectTo":{"!type":"fn(token: ?, force: ?) -> bool","!doc":"<p>Update the hash. By default, it will not execute the routes if the current token and the\ntoken passed are the same.</p>\n"},"setBefore":{"!type":"fn(before: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.BaseController-cfg-before\" rel=\"Ext.app.BaseController-cfg-before\" class=\"docClass\">before</a>.</p>\n"},"setControl":{"!type":"fn(control: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.BaseController-cfg-control\" rel=\"Ext.app.BaseController-cfg-control\" class=\"docClass\">control</a>.</p>\n"},"setId":{"!type":"fn(id: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.BaseController-cfg-id\" rel=\"Ext.app.BaseController-cfg-id\" class=\"docClass\">id</a>.</p>\n"},"setListen":{"!type":"fn(listen: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.BaseController-cfg-listen\" rel=\"Ext.app.BaseController-cfg-listen\" class=\"docClass\">listen</a>.</p>\n"},"setRoutes":{"!type":"fn(routes: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.BaseController-cfg-routes\" rel=\"Ext.app.BaseController-cfg-routes\" class=\"docClass\">routes</a>.</p>\n"},"updateControl":{"!type":"fn(control: ?) -> !this"},"updateListen":{"!type":"fn(listen: ?) -> !this"},"updateRoutes":{"!type":"fn(routes: ?) -> !this"},"destroy":{"!type":"fn() -> !this"},"isActive":{"!type":"fn() -> !this"}},"!doc":"<p>Base class for Controllers.</p>\n","!type":"fn(config?: ?)"},"bindinspector":{"Environment":{"prototype":{"buildBind":{"!type":"fn(bind: ?) -> !this"},"buildComponent":{"!type":"fn(comp: ?) -> !this"},"buildStub":{"!type":"fn(stub: ?, isLinkChild: ?) -> !this"},"buildStubName":{"!type":"fn(stub: ?) -> !this"},"buildViewModel":{"!type":"fn(vm: ?, comp: ?) -> !this"},"captureSnapshot":{"!type":"fn() -> !this","!doc":"<ul>\n<li>Capture methods</li>\n</ul>\n\n"},"createModel":{"!type":"fn(entityName: ?, data: ?) -> !this"},"deserializeStub":{"!type":"fn(stub: ?) -> !this"},"deserializeValue":{"!type":"fn(info: ?) -> !this"},"getCmp":{"!type":"fn(id: ?) -> !this","!doc":"<ul>\n<li>Utility methods</li>\n</ul>\n\n"},"getInheritedVM":{"!type":"fn(comp: ?) -> !this"},"getVM":{"!type":"fn(id: ?) -> !this"},"isRootComponent":{"!type":"fn(c: ?) -> !this"},"serializeModel":{"!type":"fn(Model: ?) -> !this"},"serializeValue":{"!type":"fn(value: ?, checkHasOwn: ?) -> !this"},"unpackComponent":{"!type":"fn(comp: ?, allComponents: ?, allViewModels: ?) -> !this"},"unpackSnapshot":{"!type":"fn(data: ?) -> !this"}}},"Util":{"buildBindData":{"!type":"fn(bind: ?) -> !this"},"getChildStub":{"!type":"fn(name: ?, parent: ?) -> !this"},"valueRenderer":{"!type":"fn(v: ?) -> !this"}}},"domain":{"Component":{"!doc":"<p>This class implements the component event domain. All classes extending from\n<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> are included in this domain. The matching criteria uses\nExt.ComponentQuery.</p>\n","!type":"fn()","type":{"!type":"string"},"dispatch":{"!type":"fn(target: ?, ev: ?, args: ?) -> bool","!doc":"<p>This method dispatches an event fired by an object monitored by this domain. This\nis not called directly but is called by interceptors injected by the <code>monitor</code> method.</p>\n"},"match":{"!type":"fn(target: ?, selector: ?) -> bool","!doc":"<p>This method matches the firer of the event (the <code>target</code>) to the given <code>selector</code>.\nDefault matching is very simple: a match is true when selector equals target's\n<a href=\"#!/api/Ext.app.domain.Component-cfg-idProperty\" rel=\"Ext.app.domain.Component-cfg-idProperty\" class=\"docClass\">idProperty</a>, or when selector is '*' wildcard to match any\ntarget.</p>\n"}},"Controller":{"!doc":"<p>This class implements the controller event domain. All classes extending from\n<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a> are included in this domain. The selectors are simply id's or the\nwildcard \"*\" to match any controller.</p>\n","!type":"fn()","idMatchRe":{},"prefix":{"!type":"string"},"type":{"!type":"string"},"match":{"!type":"fn(target: ?, selector: ?) -> bool","!doc":"<p>This method matches the firer of the event (the <code>target</code>) to the given <code>selector</code>.\nDefault matching is very simple: a match is true when selector equals target's\n<a href=\"#!/api/Ext.app.domain.Controller-cfg-idProperty\" rel=\"Ext.app.domain.Controller-cfg-idProperty\" class=\"docClass\">idProperty</a>, or when selector is '*' wildcard to match any\ntarget.</p>\n"}},"Direct":{"!doc":"<p>This class implements the Ext.Direct event domain. All classes extending from\nExt.direct.Provider are included in this domain. The selectors are simply provider\nid's or the wildcard \"*\" to match any provider.</p>\n","!type":"fn()","idProperty":{"!type":"string","!doc":"<p>Name of the identifier property for this event domain.</p>\n"},"type":{"!type":"string"}},"Global":{"!doc":"<p>This class implements the global event domain. This domain represents event fired from\nExt.GlobalEvents Observable instance. No selectors are supported for this domain.</p>\n","!type":"fn()","match":{},"type":{"!type":"string"},"listen":{"!type":"fn(listeners: ?, controller: ?) -> !this","!doc":"<p>This method adds listeners on behalf of a controller. Since Global domain does not\nsupport selectors, we skip this layer and just accept an object keyed by events.\nFor example:</p>\n\n<pre><code> domain.listen({\n     idle: function() { ... },\n     afterlayout: {\n         fn: function() { ... },\n         delay: 10\n     }\n });\n</code></pre>\n"}},"Store":{"!doc":"<p>This class implements the data store event domain. All classes extending from\nExt.data.AbstractStore are included in this domain. The selectors are simply\nstore id's or the wildcard \"*\" to match any store.</p>\n","!type":"fn()","idProperty":{"!type":"string","!doc":"<p>Name of the identifier property for this event domain.</p>\n"},"type":{"!type":"string"}},"View":{"!type":"fn(controller: ?)","prototype":{"isInstance":{"!type":"bool"},"destroy":{"!type":"fn() -> !this"},"match":{"!type":"fn(target: ?, selector: ?, controller: ?) -> bool","!doc":"<p>This method matches the firer of the event (the <code>target</code>) to the given <code>selector</code>.\nDefault matching is very simple: a match is true when selector equals target's\n<a href=\"#!/api/Ext.app.domain.View-cfg-idProperty\" rel=\"Ext.app.domain.View-cfg-idProperty\" class=\"docClass\">idProperty</a>, or when selector is '*' wildcard to match any\ntarget.</p>\n"}}}},"EventDomain":{"idProperty":{"!type":"string","!doc":"<p>Name of the identifier property for this event domain.</p>\n"},"isInstance":{"!type":"bool"},"destroy":{"!type":"fn() -> !this"},"listen":{"!type":"fn(selectors: ?) -> !this","!doc":"<p>This method adds listeners on behalf of a controller. This method is passed an\nobject that is keyed by selectors. The value of these is also an object but now\nkeyed by event name. For example:</p>\n\n<pre><code> domain.listen({\n     'some[selector]': {\n         click: function() { ... }\n     },\n\n     'other selector': {\n         change: {\n             fn: function() { ... },\n             delay: 10\n         }\n     }\n\n }, controller);\n</code></pre>\n"},"monitor":{"!type":"fn(observable: ?) -> !this","!doc":"<p>This method is called by the derived class to monitor <code>fireEvent</code> calls. Any call\nto <code>fireEvent</code> on the target Observable will be intercepted and dispatched to any\nlistening Controllers. Assuming the original <code>fireEvent</code> method does not return\n<code>false</code>, the event is passed to the <code>dispatch</code> method of this object.</p>\n\n<p>This is typically called in the <code>constructor</code> of derived classes.</p>\n"},"unlisten":{"!type":"fn(controllerId: ?) -> !this","!doc":"<p>Removes all of a controller's attached listeners.</p>\n"},"dispatch":{"!type":"fn(target: ?, ev: ?, args: ?) -> bool","!doc":"<p>This method dispatches an event fired by an object monitored by this domain. This\nis not called directly but is called by interceptors injected by the <code>monitor</code> method.</p>\n"},"match":{"!type":"fn(target: ?, selector: ?) -> bool","!doc":"<p>This method matches the firer of the event (the <code>target</code>) to the given <code>selector</code>.\nDefault matching is very simple: a match is true when selector equals target's\n<a href=\"#!/api/Ext.app.EventDomain-cfg-idProperty\" rel=\"Ext.app.EventDomain-cfg-idProperty\" class=\"docClass\">idProperty</a>, or when selector is '*' wildcard to match any\ntarget.</p>\n"},"prototype":{"idProperty":{"!type":"string","!doc":"<p>Name of the identifier property for this event domain.</p>\n"},"dispatch":{"!type":"fn(target: ?, ev: ?, args: ?) -> bool","!doc":"<p>This method dispatches an event fired by an object monitored by this domain. This\nis not called directly but is called by interceptors injected by the <code>monitor</code> method.</p>\n"},"listen":{"!type":"fn(selectors: ?) -> !this","!doc":"<p>This method adds listeners on behalf of a controller. This method is passed an\nobject that is keyed by selectors. The value of these is also an object but now\nkeyed by event name. For example:</p>\n\n<pre><code> domain.listen({\n     'some[selector]': {\n         click: function() { ... }\n     },\n\n     'other selector': {\n         change: {\n             fn: function() { ... },\n             delay: 10\n         }\n     }\n\n }, controller);\n</code></pre>\n"},"monitor":{"!type":"fn(observable: ?) -> !this","!doc":"<p>This method is called by the derived class to monitor <code>fireEvent</code> calls. Any call\nto <code>fireEvent</code> on the target Observable will be intercepted and dispatched to any\nlistening Controllers. Assuming the original <code>fireEvent</code> method does not return\n<code>false</code>, the event is passed to the <code>dispatch</code> method of this object.</p>\n\n<p>This is typically called in the <code>constructor</code> of derived classes.</p>\n"},"unlisten":{"!type":"fn(controllerId: ?) -> !this","!doc":"<p>Removes all of a controller's attached listeners.</p>\n"},"isInstance":{"!type":"bool"},"instances":{"!doc":"<p>An object map containing <code><a href=\"#!/api/Ext.app.EventDomain\" rel=\"Ext.app.EventDomain\" class=\"docClass\">Ext.app.EventDomain</a></code> instances keyed by the value\nof their <code>type</code> property.</p>\n"},"destroy":{"!type":"fn() -> !this"},"match":{"!type":"fn(target: ?, selector: ?) -> bool","!doc":"<p>This method matches the firer of the event (the <code>target</code>) to the given <code>selector</code>.\nDefault matching is very simple: a match is true when selector equals target's\n<a href=\"#!/api/Ext.app.EventDomain-cfg-idProperty\" rel=\"Ext.app.EventDomain-cfg-idProperty\" class=\"docClass\">idProperty</a>, or when selector is '*' wildcard to match any\ntarget.</p>\n"}},"!doc":"<p>This class is a base class for an event domain. In the context of MVC, an \"event domain\"\nis one or more base classes that fire events to which a Controller wants to listen. A\ncontroller listens to events by describing the selectors for events of interest to it.</p>\n\n<p>Matching selectors to the firer of an event is one key aspect that defines an event\ndomain. All event domain instances must provide a <code>match</code> method that tests selectors\nagainst the event firer.</p>\n\n<p>When an event domain instance is created (typically as a <code>singleton</code>), its <code>type</code>\nproperty is used to catalog the domain in the\n<a href=\"#!/api/Ext.app.EventDomain-static-property-instances\" rel=\"Ext.app.EventDomain-static-property-instances\" class=\"docClass\">Ext.app.EventDomain.instances</a> map.</p>\n\n<p>There are five event domains provided by default:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.app.domain.Component\" rel=\"Ext.app.domain.Component\" class=\"docClass\">Component domain</a>. This is the primary event domain that\nhas been available since Ext JS MVC was introduced. This domain is defined as any class that\nextends <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>, where the selectors use\nExt.ComponentQuery.</li>\n<li><a href=\"#!/api/Ext.app.domain.Global\" rel=\"Ext.app.domain.Global\" class=\"docClass\">Global domain</a>. This domain provides Controllers with access\nto events fired from Ext.GlobalEvents Observable instance. These events represent\nthe state of the application as a whole, and are always anonymous. Because of this, Global\ndomain does not provide selectors at all.</li>\n<li><a href=\"#!/api/Ext.app.domain.Controller\" rel=\"Ext.app.domain.Controller\" class=\"docClass\">Controller domain</a>. This domain includes all classes\nthat extend <a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Ext.app.Controller</a>. Events fired by Controllers will be available\nwithin this domain; selectors are either Controller's <a href=\"#!/api/Ext.app.Controller-cfg-id\" rel=\"Ext.app.Controller-cfg-id\" class=\"docClass\">id</a> or\n'*' wildcard for any Controller.</li>\n<li><a href=\"#!/api/Ext.app.domain.Store\" rel=\"Ext.app.domain.Store\" class=\"docClass\">Store domain</a>. This domain is for classes extending\nExt.data.AbstractStore. Selectors are either Store's\nstoreId or '*' wildcard for any Store.</li>\n<li><a href=\"#!/api/Ext.app.domain.Direct\" rel=\"Ext.app.domain.Direct\" class=\"docClass\">Direct domain</a>. This domain includes all classes that extend\nExt.direct.Provider. Selectors are either Provider's id\nor '*' wildcard for any Provider. This domain is optional and will be loaded only if\nExt.direct.Manager singleton is required in your application.</li>\n</ul>\n\n","!type":"fn()"},"EventBus":{"!doc":"<p>This class manages event dispatching for Controllers. The details of connecting classes\nto this dispatching mechanism is delegated to <a href=\"#!/api/Ext.app.EventDomain\" rel=\"Ext.app.EventDomain\" class=\"docClass\">Ext.app.EventDomain</a> instances.</p>\n","!type":"fn()","control":{"!type":"fn(selectors: ?, controller: ?) -> !this","!doc":"<p>Adds a set of component event listeners for a controller. To work with event domains\nother than component, see <a href=\"#!/api/Ext.app.EventBus-method-listen\" rel=\"Ext.app.EventBus-method-listen\" class=\"docClass\">listen</a>.</p>\n"},"listen":{"!type":"fn(to: ?, controller: ?) -> !this","!doc":"<p>Adds a set of event domain listeners for a controller. For more information on event\ndomains, see <a href=\"#!/api/Ext.app.EventDomain\" rel=\"Ext.app.EventDomain\" class=\"docClass\">Ext.app.EventDomain</a> and <a href=\"#!/api/Ext.app.BaseController\" rel=\"Ext.app.BaseController\" class=\"docClass\">Ext.app.BaseController</a>.</p>\n"},"unlisten":{"!type":"fn(controllerId: ?) -> !this","!doc":"<p>Removes all of a controller's attached listeners.</p>\n"}},"route":{"Queue":{"!doc":"<p>A Queue is a queue of <a href=\"#!/api/Ext.app.route.Route\" rel=\"Ext.app.route.Route\" class=\"docClass\">Ext.app.route.Route</a> instances managed by the\n<a href=\"#!/api/Ext.app.route.Router\" rel=\"Ext.app.route.Router\" class=\"docClass\">Ext.app.route.Router</a> singleton if queueActions is set to <code>true</code>.</p>\n\n<p>A developer shouldn't need to use this class as <a href=\"#!/api/Ext.app.route.Router\" rel=\"Ext.app.route.Router\" class=\"docClass\">Ext.app.route.Router</a> should\nmanage this class. When a <a href=\"#!/api/Ext.app.route.Route\" rel=\"Ext.app.route.Route\" class=\"docClass\">Ext.app.route.Route</a> is executed,it will automatically\nkeep running the queue until the queue is empty.</p>\n","!type":"fn(config: ?)","prototype":{"queue":{"!doc":"<p>The Ext.util.MixedCollection that will hold the queued\n<a href=\"#!/api/Ext.app.route.Route\" rel=\"Ext.app.route.Route\" class=\"docClass\">Ext.app.route.Route</a> and recognized arguments.</p>\n"},"token":{"!doc":"<p>The token from the <a href=\"#!/api/Ext.app.route.Router\" rel=\"Ext.app.route.Router\" class=\"docClass\">Ext.app.route.Router</a> that is being enacted on.</p>\n"},"clearQueue":{"!type":"fn() -> !this","!doc":"<p>Clear all queued actions.</p>\n"},"onActionExecute":{"!type":"fn(clearQueue: ?) -> !this","!doc":"<p>Handle the execution of a queued action and optionally clear all queued actions.</p>\n"},"queueAction":{"!type":"fn(route: ?, args: ?) -> !this","!doc":"<p>Add a <a href=\"#!/api/Ext.app.route.Route\" rel=\"Ext.app.route.Route\" class=\"docClass\">Ext.app.route.Route</a> to the queue.</p>\n"},"runQueue":{"!type":"fn() -> !this","!doc":"<p>Run the queue one by one.</p>\n"}}},"Route":{"!doc":"<p>Represents a mapping between a url and a controller/action pair. May also contain\nadditional params.</p>\n\n<p>This is a private internal class that should not need to be used by end-developer code.\nIts API and existence are subject to change so use at your own risk.</p>\n","!type":"fn(config: ?)","prototype":{"action":{"!type":"string","!doc":"<p>The name of the action that will be called on the\n<a href=\"#!/api/Ext.app.route.Route-cfg-controller\" rel=\"Ext.app.route.Route-cfg-controller\" class=\"docClass\">controller</a> if this route is matched.</p>\n"},"allowInactive":{"!type":"bool","!doc":"<p><code>true</code> to allow this route to be triggered on\na controller that is not active.</p>\n"},"before":{"!doc":"<p>An optional function to use to intercept the <a href=\"#!/api/Ext.app.route.Route-cfg-action\" rel=\"Ext.app.route.Route-cfg-action\" class=\"docClass\">action</a>\nto do extra means and possibly stop the execution. An example is if the route is\nfor editing a user and you need to verify the current user has permission you can\nsend an Ext.Ajax request to a server or some arbitrary code.</p>\n"},"caseInsensitive":{"!type":"bool","!doc":"<p><code>true</code> to allow the tokens to be matched with\ncase-insensitive. Defaults to <code>false</code> which will force case matching.</p>\n"},"conditions":{"!doc":"<p>Optional set of conditions for each token in the url\nstring. Each key should be one of the tokens, each value should be a regex that the\ntoken should accept. For example, if you have a Route with a url like\n<code>\"files/:fileName\"</code> and you want it to match urls like \"files/someImage.jpg\" then\nyou can set these conditions to allow the :fileName token to accept strings\ncontaining a period (\".\"):</p>\n\n<pre><code>conditions: {\n    ':fileName': \"[0-9a-zA-Z\\.]+\"\n}\n</code></pre>\n"},"controller":{"!type":"string","!doc":"<p>The name of the Controller whose <a href=\"#!/api/Ext.app.route.Route-cfg-action\" rel=\"Ext.app.route.Route-cfg-action\" class=\"docClass\">action</a> will be\ncalled if this route is matched.</p>\n"},"url":{"!type":"string","!doc":"<p>The url regex to match against.</p>\n"},"matcherRegex":{"!doc":"<p>A regular expression to match the token to the configured <a href=\"#!/api/Ext.app.route.Route-cfg-url\" rel=\"Ext.app.route.Route-cfg-url\" class=\"docClass\">url</a>.</p>\n"},"paramMatchingRegex":{"!doc":"<p>A regular expression to check if there are parameters in the configured <a href=\"#!/api/Ext.app.route.Route-cfg-url\" rel=\"Ext.app.route.Route-cfg-url\" class=\"docClass\">url</a>.</p>\n"},"paramsInMatchString":{"!doc":"<p>An array of parameters in the configured <a href=\"#!/api/Ext.app.route.Route-cfg-url\" rel=\"Ext.app.route.Route-cfg-url\" class=\"docClass\">url</a>.</p>\n"},"createCallback":{"!type":"fn(args: ?, callback: ?, scope: ?) -> ?","!doc":"<p>Creates the callback function to execute in the configured <a href=\"#!/api/Ext.app.route.Route-cfg-before\" rel=\"Ext.app.route.Route-cfg-before\" class=\"docClass\">before</a> function.</p>\n"},"createMatcherRegex":{"!type":"fn(url: ?) -> ?","!doc":"<p>Takes the configured url string including wildcards and returns a regex that can be\nused to match against a url.</p>\n"},"execute":{"!type":"fn(token: ?, argConfig: ?, callback: ?, scope: ?) -> !this","!doc":"<p>The method to execute the action using the configured before function which will\nkick off the actual <a href=\"#!/api/Ext.app.route.Route-cfg-action\" rel=\"Ext.app.route.Route-cfg-action\" class=\"docClass\">action</a> on the <a href=\"#!/api/Ext.app.route.Route-cfg-controller\" rel=\"Ext.app.route.Route-cfg-controller\" class=\"docClass\">controller</a>.</p>\n"},"matchesFor":{"!type":"fn(url: ?) -> ?","!doc":"<p>Returns a hash of matching url segments for the given url.</p>\n"},"recognize":{"!type":"fn(url: ?) -> ?","!doc":"<p>Attempts to recognize a given url string and return controller/action pair for it.</p>\n"},"recognizes":{"!type":"fn(url: ?) -> bool","!doc":"<p>Returns true if this <a href=\"#!/api/Ext.app.route.Route\" rel=\"Ext.app.route.Route\" class=\"docClass\">Ext.app.route.Route</a> matches the given url string.</p>\n"}}},"Router":{"!doc":"<p>The Router is an ordered set of <a href=\"#!/api/Ext.app.route.Route\" rel=\"Ext.app.route.Route\" class=\"docClass\">Ext.app.route.Route</a> definitions that decode a\nurl into a controller function to execute. Each <code>route</code> defines a type of url to match,\nalong with the controller function to call if it is matched. The Router uses the\n<a href=\"#!/api/Ext.util.History\" rel=\"Ext.util.History\" class=\"docClass\">Ext.util.History</a> singleton to find out when the browser's url has changed.</p>\n\n<p>Routes are almost always defined inside a <a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Controller</a>, as\nopposed to on the Router itself. End-developers should not usually need to interact\ndirectly with the Router as the Controllers manage everything automatically. See the\n<a href=\"#!/api/Ext.app.Controller\" rel=\"Ext.app.Controller\" class=\"docClass\">Controller documentation</a> for more information on specifying\nroutes.</p>\n","multipleToken":{"!type":"string","!doc":"<p>The token to split the routes to support multiple routes.</p>\n"},"queueRoutes":{"!type":"bool","!doc":"<p>True to queue routes to be executed one after the\nother, false to execute routes immediately.</p>\n"},"routes":{"!doc":"<p>The connected <a href=\"#!/api/Ext.app.route.Route\" rel=\"Ext.app.route.Route\" class=\"docClass\">Ext.app.route.Route</a>\ninstances.</p>\n"},"clear":{"!type":"fn() -> !this","!doc":"<p>Clear all the recognized routes.</p>\n"},"connect":{"!type":"fn(url: ?, action: ?, controller: ?) -> !this","!doc":"<p>Create the <a href=\"#!/api/Ext.app.route.Route\" rel=\"Ext.app.route.Route\" class=\"docClass\">Ext.app.route.Route</a> instance and connect to the\n<a href=\"#!/api/Ext.app.route.Router\" rel=\"Ext.app.route.Router\" class=\"docClass\">Ext.app.route.Router</a> singleton.</p>\n"},"disconnectAll":{"!type":"fn(controller: ?) -> !this","!doc":"<p>Disconnects all routes for a controller.</p>\n"},"draw":{"!type":"fn(fn: ?) -> !this","!doc":"<p>Convenience method which just calls the supplied function with the\n<a href=\"#!/api/Ext.app.route.Router\" rel=\"Ext.app.route.Router\" class=\"docClass\">Ext.app.route.Router</a> singleton. Example usage:</p>\n\n<pre><code><a href=\"#!/api/Ext.app.route.Router-method-draw\" rel=\"Ext.app.route.Router-method-draw\" class=\"docClass\">Ext.app.route.Router.draw</a>(function(map) {\n    map.connect('activate/:token', {controller: 'users', action: 'activate'});\n    map.connect('home',            {controller: 'index', action: 'home'});\n});\n</code></pre>\n"},"onStateChange":{"!type":"fn(token: ?) -> !this","!doc":"<p>React to a token</p>\n"},"recognize":{"!type":"fn(url: ?) -> ?","!doc":"<p>Recognizes a url string connected to the Router, return the controller/action pair\nplus any additional config associated with it.</p>\n"}}},"Util":{"prototype":{"addNamespaces":{"!type":"fn(namespace: ?) -> !this","!doc":"<p>Adds namespace(s) to known list.</p>\n"},"clearNamespaces":{"!type":"fn() -> !this","!doc":"<p>Clear all namespaces from known list.</p>\n"},"getNamespace":{"!type":"fn(className: ?) -> string","!doc":"<p>Get namespace prefix for a class name.</p>\n"}}},"ViewController":{"!doc":"<p>A view controller is a controller that can be attached to a specific view\ninstance so it can manage the view and it's child components. Each instance of the view\nwill have a new view controller, so the instances are isolated.</p>\n\n<p>When a controller is specified on a view, the view automatically becomes a <a href=\"#!/api/Ext.container.Container-cfg-referenceHolder\" rel=\"Ext.container.Container-cfg-referenceHolder\" class=\"docClass\">Ext.container.Container.referenceHolder</a>,\nso it will receive inline events declared on the view. Sample usage:</p>\n\n<pre class='inline-example '><code>Ext.define('User', {\n   extend: 'Ext.data.Model',\n   fields: ['name', 'phone']    \n});\n\nExt.define('UserListController', {\n    extend : '<a href=\"#!/api/Ext.app.ViewController\" rel=\"Ext.app.ViewController\" class=\"docClass\">Ext.app.ViewController</a>',\n    alias: 'controller.userlist',\n\n    init: function() {\n        this.userCount = 0;\n        var users = [],\n            i;\n\n        for (i = 0; i &lt; 5; ++i) {\n            users.push(this.getUser());\n        }  \n        this.getView().getStore().add(users);\n    },\n\n    onAddClick: function() {\n        this.addUser();\n    },\n\n    onDeleteClick: function() {\n        var view = this.getView(),\n            selected = view.getSelectionModel().getSelection()[0],\n            store = view.getStore();\n\n        store.remove(selected);\n    },\n\n    onSelectionChange: function(selModel, selections) {\n        this.lookupReference('delete').setDisabled(selections.length === 0);\n    },\n\n    getUser: function() {\n        ++this.userCount;\n        return {\n            name: 'User ' + this.userCount,\n            phone: this.generatePhone()\n        };\n    },\n\n    addUser: function() {\n        this.getView().getStore().add(this.getUser());    \n    },\n\n    generatePhone: function() {\n        var num = '',\n            i;\n\n        for (i = 0; i &lt; 7; ++i) {\n            num += Ext.Number.randomInt(0, 9);\n            if (num.length === 3) {\n                num += '-';\n            }\n        }    \n        return num;\n    }\n});\n\nExt.define('UserList', {\n    extend: '<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>',\n    controller: 'userlist',\n\n    tbar: [{\n        text: 'Add',\n        listeners: {\n            click: 'onAddClick'\n        }    \n    }, {\n        text: 'Delete',\n        reference: 'delete',\n        listeners: {\n            click: 'onDeleteClick'\n        }\n    }],\n    store: {\n        model: 'User'\n    },\n    selModel: {\n        type: 'rowmodel',\n        listeners: {\n            selectionchange: 'onSelectionChange'\n        }    \n    },\n    columns: [{\n        flex: 1,\n        dataIndex: 'name',\n        text: 'Name'\n    }, {\n        flex: 1,\n        dataIndex: 'phone',\n        text: 'Phone'\n    }]\n});\n\nExt.onReady(function() {\n    new UserList({\n        renderTo: Ext.getBody(),\n        width: 400,\n        height: 200\n    });\n});\n</code></pre>\n","!type":"fn()","prototype":{"closeViewAction":{"!type":"string"},"factoryConfig":{},"isViewController":{"!type":"bool"},"privates":{"!doc":"<hr />\n"},"attachReference":{"!type":"fn(component: ?) -> !this","!doc":"<p>Set a reference to a component.</p>\n"},"beforeInit":{"!type":"fn(view: ?) -> !this","!doc":"<p>Called before the view initializes. This is called before the view's\ninitComponent method has been called.</p>\n"},"clearReference":{"!type":"fn(ref: ?) -> !this","!doc":"<p>Clear a reference to a component</p>\n"},"clearReferences":{"!type":"fn() -> !this","!doc":"<p>Invalidates the references collection. Typically called when\nremoving a container from this container, since it's difficult\nto know what references got removed.</p>\n"},"closeView":{"!type":"fn() -> !this","!doc":"<p>This method closes the associated view. The manner in which this is done (that is,\nthe method called to close the view) is specified by <code>closeViewAction</code>.</p>\n\n<p>It is common for views to map one or more events to this method to allow the view\nto be closed.</p>\n"},"control":{"!type":"fn(selectors: ?, listeners: ?) -> !this","!doc":"<p>Adds listeners to components selected via Ext.ComponentQuery. Accepts an\nobject containing component paths mapped to a hash of listener functions.</p>\n\n<p>In the following example the <code>updateUser</code> function is mapped to to the <code>click</code>\nevent on a button component, which is a child of the <code>useredit</code> component.</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     init: function() {\n         this.control({\n             'useredit button[action=save]': {\n                 click: this.updateUser\n             }\n         });\n     },\n\n     updateUser: function(button) {\n         console.log('clicked the Save button');\n     }\n });\n</code></pre>\n\n<p>Or alternatively one call <code>control</code> with two arguments:</p>\n\n<pre><code> this.control('useredit button[action=save]', {\n     click: this.updateUser\n });\n</code></pre>\n\n<p>See Ext.ComponentQuery for more information on component selectors.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroy the view controller.</p>\n"},"fireViewEvent":{"!type":"fn(eventName: ?, args: ?) -> bool","!doc":"<p>Fires an event on the view. See Ext.Component.fireEvent.</p>\n"},"getCloseViewAction":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.app.ViewController-cfg-closeViewAction\" rel=\"Ext.app.ViewController-cfg-closeViewAction\" class=\"docClass\">closeViewAction</a>.</p>\n"},"getReferences":{"!type":"fn() -> ?","!doc":"<p>Returns an object holding the descendants of this view keyed by their\n<code>reference</code>. This object should not be held\npast the scope of the function calling this method. It will not be valid if items\nare added or removed from this or any sub-container.</p>\n\n<p>The intended usage is shown here (assume there are 3 components with reference\nvalues of \"foo\", \"bar\" and \"baz\" at some level below this container):</p>\n\n<pre><code> onClick: function () {\n     var refs = this.getReferences();\n\n     // using \"refs\" we can access any descendant by its \"reference\"\n\n     refs.foo.getValue() + refs.bar.getValue() + refs.baz.getValue();\n }\n</code></pre>\n\n<p>If <code>this</code> component has a <code>reference</code> assigned\nto it, that is <strong>not</strong> included in this object. That reference is understood to\nbelong to the ancestor container configured as the <code>referenceHolder</code>.</p>\n"},"getSession":{"!type":"fn() -> !this"},"getStore":{"!type":"fn(name: ?) -> !this"},"getView":{"!type":"fn() -> ?","!doc":"<p>Get the view for this controller.</p>\n"},"getViewModel":{"!type":"fn() -> !this"},"init":{"!type":"fn(view: ?) -> !this","!doc":"<p>Called when the view initializes. This is called after the view's initComponent\nmethod has been called.</p>\n"},"initViewModel":{"!type":"fn(viewModel: ?) -> !this","!doc":"<p>Called when the view model instance for an attached view is first created.</p>\n"},"listen":{"!type":"fn(to: ?, controller: ?) -> !this","!doc":"<p>Adds listeners to different event sources (also called \"event domains\"). The\nprimary event domain is that of components, but there are also other event domains:\n<a href=\"#!/api/Ext.app.domain.Global\" rel=\"Ext.app.domain.Global\" class=\"docClass\">Global</a> domain that intercepts events fired from\nExt.GlobalEvents Observable instance, <a href=\"#!/api/Ext.app.domain.Controller\" rel=\"Ext.app.domain.Controller\" class=\"docClass\">Controller</a>\ndomain can be used to listen to events fired by other Controllers,\n<a href=\"#!/api/Ext.app.domain.Store\" rel=\"Ext.app.domain.Store\" class=\"docClass\">Store</a> domain gives access to Store events, and\n<a href=\"#!/api/Ext.app.domain.Direct\" rel=\"Ext.app.domain.Direct\" class=\"docClass\">Direct</a> domain can be used with Ext.Direct Providers\nto listen to their events.</p>\n\n<p>To listen to \"bar\" events fired by a controller with id=\"foo\":</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     init: function() {\n         this.listen({\n             controller: {\n                 '#foo': {\n                    bar: this.onFooBar\n                 }\n             }\n         });\n     },\n     ...\n });\n</code></pre>\n\n<p>To listen to \"bar\" events fired by any controller, and \"baz\" events\nfired by Store with storeId=\"baz\":</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     init: function() {\n         this.listen({\n             controller: {\n                 '*': {\n                    bar: this.onAnyControllerBar\n                 }\n             },\n             store: {\n                 '#baz': {\n                     baz: this.onStoreBaz\n                 }\n             }\n         });\n     },\n     ...\n });\n</code></pre>\n\n<p>To listen to \"idle\" events fired by Ext.GlobalEvents when other event\nprocessing is complete and Ext JS is about to return control to the browser:</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     init: function() {\n         this.listen({\n             global: {               // Global events are always fired\n                 idle: this.onIdle   // from the same object, so there\n             }                       // are no selectors\n         });\n     }\n });\n</code></pre>\n\n<p>As this relates to components, the following example:</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     init: function() {\n         this.listen({\n             component: {\n                 'useredit button[action=save]': {\n                    click: this.updateUser\n                 }\n             }\n         });\n     },\n     ...\n });\n</code></pre>\n\n<p>Is equivalent to:</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     init: function() {\n         this.control({\n             'useredit button[action=save]': {\n                click: this.updateUser\n             }\n         });\n     },\n     ...\n });\n</code></pre>\n\n<p>Of course, these can all be combined in a single call and used instead of\n<code>control</code>, like so:</p>\n\n<pre><code> Ext.define('AM.controller.Users', {\n     init: function() {\n         this.listen({\n             global: {\n                 idle: this.onIdle\n             },\n             controller: {\n                 '*': {\n                    foobar: this.onAnyFooBar\n                 },\n                 '#foo': {\n                    bar: this.onFooBar\n                 }\n             },\n             component: {\n                 'useredit button[action=save]': {\n                    click: this.updateUser\n                 }\n             },\n             store: {\n                 '#qux': {\n                     load: this.onQuxLoad\n                 }\n             }\n         });\n     },\n     ...\n });\n</code></pre>\n"},"lookupReference":{"!type":"fn(key: ?) -> ?","!doc":"<p>Get a reference to a component set with the Ext.Component.reference\nconfiguration.</p>\n"},"lookupReferenceHolder":{"!type":"fn() -> !this"},"setCloseViewAction":{"!type":"fn(closeViewAction: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.app.ViewController-cfg-closeViewAction\" rel=\"Ext.app.ViewController-cfg-closeViewAction\" class=\"docClass\">closeViewAction</a>.</p>\n"},"setView":{"!type":"fn(view: ?) -> !this","!doc":"<p>Sets the view for this controller. To be called by the view\nwhen it initializes.</p>\n"}}}},"button":{"Button":{"!doc":"<p>Create simple buttons with this component. Customisations include <a href=\"#!/api/Ext.button.Button-cfg-iconAlign\" rel=\"Ext.button.Button-cfg-iconAlign\" class=\"docClass\">aligned</a>\n<a href=\"#!/api/Ext.button.Button-cfg-iconCls\" rel=\"Ext.button.Button-cfg-iconCls\" class=\"docClass\">icons</a>, <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">dropdown menus</a>, <a href=\"#!/api/Ext.button.Button-cfg-tooltip\" rel=\"Ext.button.Button-cfg-tooltip\" class=\"docClass\">tooltips</a>\nand <a href=\"#!/api/Ext.button.Button-cfg-scale\" rel=\"Ext.button.Button-cfg-scale\" class=\"docClass\">sizing options</a>. Specify a <a href=\"#!/api/Ext.button.Button-cfg-handler\" rel=\"Ext.button.Button-cfg-handler\" class=\"docClass\">handler</a> to run code when\na user clicks the button, or use listeners for other events such as\n<a href=\"#!/api/Ext.button.Button-event-mouseover\" rel=\"Ext.button.Button-event-mouseover\" class=\"docClass\">mouseover</a>. Example usage:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.Button</a>', {\n    text: 'Click me',\n    renderTo: Ext.getBody(),\n    handler: function() {\n        alert('You clicked the button!');\n    }\n});\n</code></pre>\n\n<p>The <a href=\"#!/api/Ext.button.Button-cfg-handler\" rel=\"Ext.button.Button-cfg-handler\" class=\"docClass\">handler</a> configuration can also be updated dynamically using the <a href=\"#!/api/Ext.button.Button-method-setHandler\" rel=\"Ext.button.Button-method-setHandler\" class=\"docClass\">setHandler</a>\nmethod.  Example usage:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.Button</a>', {\n    text    : 'Dynamic Handler Button',\n    renderTo: Ext.getBody(),\n    handler : function() {\n        // this button will spit out a different number every time you click it.\n        // so firstly we must check if that number is already set:\n        if (this.clickCount) {\n            // looks like the property is already set, so lets just add 1 to that number and alert the user\n            this.clickCount++;\n            alert('You have clicked the button \"' + this.clickCount + '\" times.\\n\\nTry clicking it again..');\n        } else {\n            // if the clickCount property is not set, we will set it and alert the user\n            this.clickCount = 1;\n            alert('You just clicked the button for the first time!\\n\\nTry pressing it again..');\n        }\n    }\n});\n</code></pre>\n\n<p>A button within a container:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.Container</a>', {\n    renderTo: Ext.getBody(),\n    items   : [\n        {\n            xtype: 'button',\n            text : 'My Button'\n        }\n    ]\n});\n</code></pre>\n\n<p>A useful option of Button is the <a href=\"#!/api/Ext.button.Button-cfg-scale\" rel=\"Ext.button.Button-cfg-scale\" class=\"docClass\">scale</a> configuration. This configuration has three different options:</p>\n\n<ul>\n<li><code>'small'</code></li>\n<li><code>'medium'</code></li>\n<li><code>'large'</code></li>\n</ul>\n\n\n<p>Example usage:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.Button</a>', {\n    renderTo: document.body,\n    text    : 'Click me',\n    scale   : 'large'\n});\n</code></pre>\n\n<p>Buttons can also be toggled. To enable this, you simple set the <a href=\"#!/api/Ext.button.Button-cfg-enableToggle\" rel=\"Ext.button.Button-cfg-enableToggle\" class=\"docClass\">enableToggle</a> property to <code>true</code>.\nExample usage:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.Button</a>', {\n    renderTo: Ext.getBody(),\n    text: 'Click Me',\n    enableToggle: true\n});\n</code></pre>\n\n<p>You can assign a menu to a button by using the <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a> configuration. This standard configuration\ncan either be a reference to a <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">menu</a> object, a <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">menu</a> id or a\n<a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">menu</a> config blob. When assigning a menu to a button, an arrow is automatically\nadded to the button.  You can change the alignment of the arrow using the <a href=\"#!/api/Ext.button.Button-cfg-arrowAlign\" rel=\"Ext.button.Button-cfg-arrowAlign\" class=\"docClass\">arrowAlign</a> configuration\non button.  Example usage:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.Button</a>', {\n    text      : 'Menu button',\n    renderTo  : Ext.getBody(),\n    arrowAlign: 'bottom',\n    menu      : [\n        {text: 'Item 1'},\n        {text: 'Item 2'},\n        {text: 'Item 3'},\n        {text: 'Item 4'}\n    ]\n});\n</code></pre>\n\n<p>Using listeners, you can easily listen to events fired by any component, using the listeners\nconfiguration or using the <a href=\"#!/api/Ext.button.Button-method-addListener\" rel=\"Ext.button.Button-method-addListener\" class=\"docClass\">addListener</a> method.  Button has a variety of different listeners:</p>\n\n<ul>\n<li><code>click</code></li>\n<li><code>toggle</code></li>\n<li><code>mouseover</code></li>\n<li><code>mouseout</code></li>\n<li><code>mouseshow</code></li>\n<li><code>menuhide</code></li>\n<li><code>menutriggerover</code></li>\n<li><code>menutriggerout</code></li>\n</ul>\n\n\n<p>Example usage:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.Button</a>', {\n    text     : 'Button',\n    renderTo : Ext.getBody(),\n    listeners: {\n        click: function() {\n            // this == the button, as we are in the local scope\n            this.setText('I was clicked!');\n        },\n        mouseover: function() {\n            // set a new config which says we moused over, if not already set\n            if (!this.mousedOver) {\n                this.mousedOver = true;\n                alert('You moused over a button!\\n\\nI wont do this again.');\n            }\n        }\n    }\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"allowDepress":{"!type":"bool","!doc":"<p>False to not allow a pressed Button to be depressed. Only valid when <a href=\"#!/api/Ext.button.Button-cfg-enableToggle\" rel=\"Ext.button.Button-cfg-enableToggle\" class=\"docClass\">enableToggle</a> is true.</p>\n"},"arrowAlign":{"!type":"string","!doc":"<p>The side of the Button box to render the arrow if the button has an associated <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a>. Two\nvalues are allowed:</p>\n\n<ul>\n<li>'right'</li>\n<li>'bottom'</li>\n</ul>\n\n"},"arrowCls":{"!type":"string","!doc":"<p>The className used for the inner arrow element if the button has a menu.</p>\n"},"arrowVisible":{"!type":"bool","!doc":"<p><code>false</code> to hide the button arrow.  Only applicable for <a href=\"#!/api/Ext.button.Split\" rel=\"Ext.button.Split\" class=\"docClass\">Split Buttons</a> and buttons configured with a <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a>.</p>\n"},"autoEl":{"!doc":"<p>A tag name or DomHelper spec used to create the <a href=\"#!/api/Ext.button.Button-method-getEl\" rel=\"Ext.button.Button-method-getEl\" class=\"docClass\">Element</a> which will\nencapsulate this Component.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong>'div'</strong>. The more complex Sencha classes use a more\ncomplex DOM structure specified by their own <a href=\"#!/api/Ext.button.Button-cfg-renderTpl\" rel=\"Ext.button.Button-cfg-renderTpl\" class=\"docClass\">renderTpl</a>s.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components encapsulated by\ndifferent DOM elements. Example usage:</p>\n\n<pre><code>{\n    xtype: 'component',\n    autoEl: {\n        tag: 'img',\n        src: 'http://www.example.com/example.jpg'\n    }\n}, {\n    xtype: 'component',\n    autoEl: {\n        tag: 'blockquote',\n        html: 'autoEl is cool!'\n    }\n}, {\n    xtype: 'container',\n    autoEl: 'ul',\n    cls: 'ux-unordered-list',\n    items: {\n        xtype: 'component',\n        autoEl: 'li',\n        html: 'First list item'\n    }\n}\n</code></pre>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to add to all buttons.</p>\n"},"baseParams":{"!doc":"<p>An object literal of parameters to pass to the url when the <a href=\"#!/api/Ext.button.Button-cfg-href\" rel=\"Ext.button.Button-cfg-href\" class=\"docClass\">href</a> property is specified.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"clickEvent":{"!type":"string","!doc":"<p>The DOM event that will fire the handler of the button. This can be any valid event name (dblclick, contextmenu).</p>\n"},"cls":{"!type":"string","!doc":"<p>A CSS class string to apply to the button's main element.</p>\n"},"destroyMenu":{"!type":"bool","!doc":"<p>Whether or not to destroy any associated menu when this button is destroyed.\nIn addition, a value of <code>true</code> for this config will destroy the currently bound menu when a new\nmenu is set in <a href=\"#!/api/Ext.button.Button-method-setMenu\" rel=\"Ext.button.Button-method-setMenu\" class=\"docClass\">setMenu</a> unless overridden by that method's destroyMenu function argument.</p>\n"},"disabled":{"!type":"bool","!doc":"<p>True if this button is disabled.</p>\n"},"enableToggle":{"!type":"bool","!doc":"<p>True to enable pressed/not pressed toggling. If a <a href=\"#!/api/Ext.button.Button-cfg-toggleGroup\" rel=\"Ext.button.Button-cfg-toggleGroup\" class=\"docClass\">toggleGroup</a> is specified, this\noption will be set to true.</p>\n"},"frame":{"!type":"bool","!doc":"<p>Specify as <code>true</code> to have the Component inject framing elements within the Component at render time to provide a\ngraphical rounded frame around the Component content.</p>\n\n<p>This is only necessary when running on outdated, or non standard-compliant browsers such as Microsoft's Internet\nExplorer prior to version 9 which do not support rounded corners natively.</p>\n\n<p>The extra space taken up by this framing is available from the read only property <a href=\"#!/api/Ext.button.Button-property-frameSize\" rel=\"Ext.button.Button-property-frameSize\" class=\"docClass\">frameSize</a>.</p>\n"},"glyph":{"!doc":"<p>A numeric unicode character code to use as the icon for this button. The default\nfont-family for glyphs can be set globally using\n<a href=\"#!/api/Ext-method-setGlyphFontFamily\" rel=\"Ext-method-setGlyphFontFamily\" class=\"docClass\">Ext.setGlyphFontFamily()</a>. Alternatively, this\nconfig option accepts a string with the charCode and font-family separated by the\n<code>@</code> symbol. For example '65@My Font Family'.</p>\n"},"handleMouseEvents":{"!type":"bool","!doc":"<p>False to disable visual cues on mouseover, mouseout and mousedown.</p>\n"},"handler":{"!doc":"<p>A function called when the button is clicked (can be used instead of click event).</p>\n"},"hidden":{"!type":"bool","!doc":"<p>True if this button is hidden.</p>\n"},"href":{"!type":"string","!doc":"<p>The URL to open when the button is clicked. Specifying this config causes the Button to be\nrendered with the specified URL as the <code>href</code> attribute of its <code>&lt;a&gt;</code> Element.</p>\n\n<p>This is better than specifying a click handler of</p>\n\n<pre><code>function() { window.location = \"http://www.sencha.com\" }\n</code></pre>\n\n<p>because the UI will provide meaningful hints to the user as to what to expect upon clicking\nthe button, and will also allow the user to open in a new tab or window, bookmark or drag the URL, or directly save\nthe URL stream to disk.</p>\n\n<p>See also the <a href=\"#!/api/Ext.button.Button-cfg-hrefTarget\" rel=\"Ext.button.Button-cfg-hrefTarget\" class=\"docClass\">hrefTarget</a> config.</p>\n"},"hrefTarget":{"!type":"string","!doc":"<p>The target attribute to use for the underlying anchor. Only used if the <a href=\"#!/api/Ext.button.Button-cfg-href\" rel=\"Ext.button.Button-cfg-href\" class=\"docClass\">href</a>\nproperty is specified.</p>\n"},"icon":{"!type":"string","!doc":"<p>The path to an image to display in the button.</p>\n\n<p>There are no default icons that come with Ext JS.</p>\n"},"iconAlign":{"!type":"string","!doc":"<p>The side of the Button box to render the icon. Four values are allowed:</p>\n\n<ul>\n<li>'top'</li>\n<li>'right'</li>\n<li>'bottom'</li>\n<li>'left'</li>\n</ul>\n\n"},"iconCls":{"!type":"string","!doc":"<p>A css class which sets a background image to be used as the icon for this button.</p>\n\n<p>There are no default icon classes that come with Ext JS.</p>\n"},"menu":{"!doc":"<p>The <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Menu</a> object associated with this Button when configured with the <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a> config\noption.</p>\n"},"menuAlign":{"!type":"string","!doc":"<p>The position to align the menu to (see Ext.util.Positionable.alignTo for more details).</p>\n"},"minWidth":{"!type":"number","!doc":"<p>The minimum width for this button (used to give a set of buttons a common width).\nSee also <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>.<a href=\"#!/api/Ext.panel.Panel-cfg-minButtonWidth\" rel=\"Ext.panel.Panel-cfg-minButtonWidth\" class=\"docClass\">minButtonWidth</a>.</p>\n"},"overCls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element when the mouse moves over the Element,\nand removed when the mouse moves out. This can be useful for adding customized 'active' or 'hover' styles to the\ncomponent or any of its children using standard CSS rules.</p>\n"},"overflowText":{"!type":"string","!doc":"<p>If used in a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>, the text to be used if this item is shown in the overflow menu.\nSee also <a href=\"#!/api/Ext.toolbar.Item\" rel=\"Ext.toolbar.Item\" class=\"docClass\">Ext.toolbar.Item</a>.<code><a href=\"#!/api/Ext.toolbar.Item-cfg-overflowText\" rel=\"Ext.toolbar.Item-cfg-overflowText\" class=\"docClass\">overflowText</a></code>.</p>\n"},"params":{"!doc":"<p>An object literal of parameters to pass to the url when the <a href=\"#!/api/Ext.button.Button-cfg-href\" rel=\"Ext.button.Button-cfg-href\" class=\"docClass\">href</a> property is specified. Any params\noverride <a href=\"#!/api/Ext.button.Button-cfg-baseParams\" rel=\"Ext.button.Button-cfg-baseParams\" class=\"docClass\">baseParams</a>. New params can be set using the <a href=\"#!/api/Ext.button.Button-method-setParams\" rel=\"Ext.button.Button-method-setParams\" class=\"docClass\">setParams</a> method.</p>\n"},"pressed":{"!type":"bool","!doc":"<p>True if this button is pressed (only if enableToggle = true).</p>\n"},"preventDefault":{"!type":"bool","!doc":"<p>True to prevent the default action when the <a href=\"#!/api/Ext.button.Button-cfg-clickEvent\" rel=\"Ext.button.Button-cfg-clickEvent\" class=\"docClass\">clickEvent</a> is processed.</p>\n"},"renderTpl":{"!doc":"<p>We have to keep \"unselectable\" attribute on all elements because it's not inheritable.\nWithout it, clicking anywhere on a button disrupts current selection and cursor position\nin HtmlEditor.</p>\n"},"repeat":{"!doc":"<p>True to repeat fire the click event while the mouse is down. This can also be a\n<a href=\"#!/api/Ext.util.ClickRepeater\" rel=\"Ext.util.ClickRepeater\" class=\"docClass\">ClickRepeater</a> config object.</p>\n"},"scale":{"!doc":"<p>The size of the Button. Three values are allowed:</p>\n\n<ul>\n<li>'small' - Results in the button element being 16px high.</li>\n<li>'medium' - Results in the button element being 24px high.</li>\n<li>'large' - Results in the button element being 32px high.</li>\n</ul>\n\n"},"scope":{"!doc":"<p>The scope (<strong>this</strong> reference) in which the <code><a href=\"#!/api/Ext.button.Button-cfg-handler\" rel=\"Ext.button.Button-cfg-handler\" class=\"docClass\">handler</a></code> and <code><a href=\"#!/api/Ext.button.Button-cfg-toggleHandler\" rel=\"Ext.button.Button-cfg-toggleHandler\" class=\"docClass\">toggleHandler</a></code> is executed.\nDefaults to this Button.</p>\n"},"showEmptyMenu":{"!type":"bool","!doc":"<p>True to force an attached <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a> with no items to be shown when clicking\nthis button. By default, the menu will not show if it is empty.</p>\n"},"shrinkWrap":{"!doc":"<p>If this property is a number, it is interpreted as follows:</p>\n\n<ul>\n<li>0 (or <code>false</code>): Neither width nor height depend on content.</li>\n<li>1: Width depends on content (shrink wraps), but height does not.</li>\n<li>2 (default): Height depends on content (shrink wraps), but width does not.</li>\n<li>3 (or <code>true</code>): Both width and height depend on content (shrink wrap).</li>\n</ul>\n\n\n<p>In CSS terms, shrink-wrap width is analogous to an inline-block element as opposed\nto a block-level element. Some container layouts always shrink-wrap their children,\neffectively ignoring this property (e.g., <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">Ext.layout.container.HBox</a>,\n<a href=\"#!/api/Ext.layout.container.VBox\" rel=\"Ext.layout.container.VBox\" class=\"docClass\">Ext.layout.container.VBox</a>, <a href=\"#!/api/Ext.layout.component.Dock\" rel=\"Ext.layout.component.Dock\" class=\"docClass\">Ext.layout.component.Dock</a>).</p>\n"},"tabIndex":{"!type":"number","!doc":"<p>Set a DOM tabIndex for this button.</p>\n"},"text":{"!type":"string","!doc":"<p>The button text to be used as innerHTML (html tags are accepted).</p>\n"},"textAlign":{"!type":"string","!doc":"<p>The text alignment for this button (center, left, right).</p>\n"},"toggleGroup":{"!type":"string","!doc":"<p>The group this toggle button is a member of (only 1 per group can be pressed). If a toggleGroup\nis specified, the <a href=\"#!/api/Ext.button.Button-cfg-enableToggle\" rel=\"Ext.button.Button-cfg-enableToggle\" class=\"docClass\">enableToggle</a> configuration will automatically be set to true.</p>\n"},"toggleHandler":{"!doc":"<p>Function called when a Button with <a href=\"#!/api/Ext.button.Button-cfg-enableToggle\" rel=\"Ext.button.Button-cfg-enableToggle\" class=\"docClass\">enableToggle</a> set to true is clicked.</p>\n"},"tooltip":{"!doc":"<p>The tooltip for the button - can be a string to be used as innerHTML (html tags are accepted) or\nQuickTips config object.</p>\n"},"tooltipType":{"!type":"string","!doc":"<p>The type of tooltip to use. Either 'qtip' for QuickTips or 'title' for title attribute.</p>\n"},"value":{"!doc":"<p>The value of this button.  Only applicable when used as an item of a <a href=\"#!/api/Ext.button.Segmented\" rel=\"Ext.button.Segmented\" class=\"docClass\">Segmented Button</a>.</p>\n"},"_baseIconCls":{"!type":"string"},"_btnCls":{"!type":"string"},"_btnWrapCls":{"!type":"string"},"_disabledCls":{"!type":"string"},"_glyphCls":{"!type":"string"},"_hasIconCls":{"!type":"string"},"_innerCls":{"!type":"string"},"_menuActiveCls":{"!type":"string"},"_noTextCls":{"!type":"string"},"_operaArrowCls":{"!type":"string","!doc":"<p><feature legacyBrowser>\n extra class to work around broken display:table impl in opera12</feature></p>\n"},"_pressedCls":{"!type":"string"},"_syncFrameHeight":{"!type":"bool","!doc":"<p><code>true</code> to keep height of the frame's \"MC\" element in sync.  This is needed in IE8\nso that the button's inner element(s) can use height:100% to fill the button when\nit not in shrinkWrap mode</p>\n"},"_textCls":{"!type":"string"},"_triggerRegion":{"!doc":"<p>A reusable object used by getTriggerRegion to avoid excessive object creation.</p>\n"},"allowedScales":{"!doc":"<p>An array of allowed scales.</p>\n"},"defaultBindProperty":{"!type":"string"},"focusCls":{"!type":"string"},"iconTpl":{"!type":"string"},"isButton":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Button, or subclass thereof.</p>\n"},"liquidLayout":{"!type":"bool","!doc":"<p>private, readonly</p>\n"},"menuClickBuffer":{"!type":"number","!doc":"<p>private</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"publishes":{},"template":{"!doc":"<p>A Template used to create the Button's DOM structure.</p>\n\n<p>Instances, or subclasses which need a different DOM structure may provide a different template layout in\nconjunction with an implementation of <a href=\"#!/api/Ext.button.Button-method-getTemplateArgs\" rel=\"Ext.button.Button-method-getTemplateArgs\" class=\"docClass\">getTemplateArgs</a>.</p>\n"},"_hasIcon":{"!type":"fn() -> bool","!doc":"<p>Returns true if this button has an icon (either icon, iconCls, or glyph)</p>\n"},"applyText":{"!type":"fn(text: ?) -> !this"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"clearTip":{"!type":"fn() -> !this"},"didIconStateChange":{"!type":"fn(old: ?, current: ?) -> bool","!doc":"<p>Checks if the icon/iconCls changed from being empty to having a value, or having a value to being empty.</p>\n"},"disable":{"!type":"fn(silent: ?) -> !this","!doc":"<p>Disable the component.</p>\n"},"doNavigate":{"!type":"fn() -> !this"},"doPreventDefault":{"!type":"fn(e: ?) -> !this"},"doToggle":{"!type":"fn() -> !this"},"enable":{"!type":"fn(silent: ?) -> !this","!doc":"<p>Enable the component</p>\n"},"fireHandler":{"!type":"fn(e: ?) -> !this"},"frameTableListener":{"!type":"fn() -> !this"},"getArrowVisible":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.button.Button-cfg-arrowVisible\" rel=\"Ext.button.Button-cfg-arrowVisible\" class=\"docClass\">arrowVisible</a>.</p>\n"},"getElConfig":{"!type":"fn() -> !this"},"getHref":{"!type":"fn() -> ?","!doc":"<p>If there is a configured href for this Button, returns the href with parameters appended.</p>\n"},"getIconAlign":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.button.Button-cfg-iconAlign\" rel=\"Ext.button.Button-cfg-iconAlign\" class=\"docClass\">iconAlign</a>.</p>\n"},"getMenu":{"!type":"fn() -> ?","!doc":"<p>Get the <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a> for this button.</p>\n"},"getRefItems":{"!type":"fn(deep: ?) -> !this"},"getSplitCls":{"!type":"fn() -> !this"},"getTdCls":{"!type":"fn() -> !this","!doc":"<p>Needed for when widget is rendered into a grid cell. The class to add to the cell element.\nOverride needed to add scale to the mix which is poart of the ui name in the mixin and the CSS rule</p>\n"},"getTemplateArgs":{"!type":"fn() -> ?","!doc":"<p>This method returns an object which provides substitution parameters for the <a href=\"#!/api/Ext.button.Button-cfg-renderTpl\" rel=\"Ext.button.Button-cfg-renderTpl\" class=\"docClass\">XTemplate</a> used to\ncreate this Button's DOM structure.</p>\n\n<p>Instances or subclasses which use a different Template to create a different DOM structure may need to provide\ntheir own implementation of this method.</p>\n"},"getText":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.button.Button-cfg-text\" rel=\"Ext.button.Button-cfg-text\" class=\"docClass\">text</a>.</p>\n"},"getTextAlign":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.button.Button-cfg-textAlign\" rel=\"Ext.button.Button-cfg-textAlign\" class=\"docClass\">textAlign</a>.</p>\n"},"getTipAttr":{"!type":"fn() -> !this"},"getTriggerRegion":{"!type":"fn() -> ?","!doc":"<p>Returns an object containing <code>begin</code> and <code>end</code> properties that indicate the\nleft/right bounds of a right trigger or the top/bottom bounds of a bottom trigger.</p>\n"},"hasFrameTable":{"!type":"fn() -> !this"},"hasVisibleMenu":{"!type":"fn() -> bool","!doc":"<p>Returns true if the button has a menu and it is visible</p>\n"},"hideMenu":{"!type":"fn() -> ?","!doc":"<p>Hides this button's menu (if it has one)</p>\n"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"isWithinTrigger":{"!type":"fn(e: ?) -> !this","!doc":"<p>Returns true if the passed event's x/y coordinates are within the trigger region</p>\n"},"maybeShowMenu":{"!type":"fn() -> !this"},"onClick":{"!type":"fn(e: ?) -> !this"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onDownKey":{"!type":"fn(k: ?, e: ?) -> !this"},"onMenuHide":{"!type":"fn(e: ?) -> !this"},"onMenuShow":{"!type":"fn() -> !this"},"onMenuTriggerOut":{"!type":"fn(e: ?) -> !this","!doc":"<p>virtual mouseleave handler called when it is detected that the mouseout event\nsignified the mouse leaving the arrow area of the button - the <code>&lt;em&gt;</code>.</p>\n"},"onMenuTriggerOver":{"!type":"fn(e: ?) -> !this","!doc":"<p>virtual mouseenter handler called when it is detected that the mouseover event\nsignified the mouse entering the arrow area of the button - the <code>&lt;em&gt;</code>.</p>\n"},"onMouseDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Mousedown brings to front, and programatically grabs focus\nunless the mousedown was on a focusable element*</p>\n"},"onMouseEnter":{"!type":"fn(e: ?) -> !this","!doc":"<p>virtual mouseenter handler called when it is detected that the mouseout event\nsignified the mouse entering the encapsulating element.</p>\n"},"onMouseLeave":{"!type":"fn(e: ?) -> !this","!doc":"<p>virtual mouseleave handler called when it is detected that the mouseover event\nsignified the mouse entering the encapsulating element.</p>\n"},"onMouseMove":{"!type":"fn(e: ?) -> !this","!doc":"<p>mousemove handler called when the mouse moves anywhere within the encapsulating element.\nThe position is checked to determine if the mouse is entering or leaving the trigger area. Using\nmousemove to check this is more resource intensive than we'd like, but it is necessary because\nthe trigger area does not line up exactly with sub-elements so we don't always get mouseover/out\nevents when needed. In the future we should consider making the trigger a separate element that\nis absolutely positioned and sized over the trigger area.</p>\n"},"onMouseOut":{"!type":"fn(e: ?) -> !this","!doc":"<p>mouseout handler called when a mouseout event occurs anywhere within the encapsulating element -\nor the mouse leaves the encapsulating element.\nThe targets are interrogated to see what is being exited to where.</p>\n"},"onMouseOver":{"!type":"fn(e: ?) -> !this","!doc":"<p>mouseover handler called when a mouseover event occurs anywhere within the encapsulating element.\nThe targets are interrogated to see what is being entered from where.</p>\n"},"onMouseUp":{"!type":"fn(e: ?) -> !this"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"onRepeatClick":{"!type":"fn(repeat: ?, e: ?) -> !this"},"onTouchStart":{"!type":"fn(e: ?) -> !this"},"renderIcon":{"!type":"fn(values: ?) -> !this"},"restoreClick":{"!type":"fn() -> !this"},"setArrowVisible":{"!type":"fn(arrowVisible: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.button.Button-cfg-arrowVisible\" rel=\"Ext.button.Button-cfg-arrowVisible\" class=\"docClass\">arrowVisible</a>.</p>\n"},"setGlyph":{"!type":"fn(glyph: ?) -> ?","!doc":"<p>Sets this button's glyph</p>\n"},"setHandler":{"!type":"fn(handler: ?, scope?: ?) -> ?","!doc":"<p>Assigns this Button's click handler</p>\n"},"setHref":{"!type":"fn(href: ?) -> !this","!doc":"<p>Sets the href of the embedded anchor element to the passed URL.</p>\n\n<p>Also appends any configured <a href=\"#!/api/Ext.button.Button-cfg-baseParams\" rel=\"Ext.button.Button-cfg-baseParams\" class=\"docClass\">baseParams</a> and parameters set through <a href=\"#!/api/Ext.button.Button-method-setParams\" rel=\"Ext.button.Button-method-setParams\" class=\"docClass\">setParams</a>.</p>\n"},"setIcon":{"!type":"fn(icon: ?) -> ?","!doc":"<p>Sets the background image (inline style) of the button. This method also changes the value of the <a href=\"#!/api/Ext.button.Button-cfg-icon\" rel=\"Ext.button.Button-cfg-icon\" class=\"docClass\">icon</a>\nconfig internally.</p>\n"},"setIconAlign":{"!type":"fn(iconAlign: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.button.Button-cfg-iconAlign\" rel=\"Ext.button.Button-cfg-iconAlign\" class=\"docClass\">iconAlign</a>.</p>\n"},"setIconCls":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Sets the CSS class that provides a background image to use as the button's icon. This method also changes the\nvalue of the <a href=\"#!/api/Ext.button.Button-cfg-iconCls\" rel=\"Ext.button.Button-cfg-iconCls\" class=\"docClass\">iconCls</a> config internally.</p>\n"},"setMenu":{"!type":"fn(menu: ?, destroyMenu: ?) -> !this","!doc":"<p>Sets a new menu for this button. Pass a falsy value to unset the current menu.\nTo destroy the previous menu for this button, explicitly pass <code>false</code> as the second argument. If this is not set, the destroy will depend on the\nvalue of <a href=\"#!/api/Ext.button.Button-cfg-destroyMenu\" rel=\"Ext.button.Button-cfg-destroyMenu\" class=\"docClass\">destroyMenu</a>.</p>\n"},"setParams":{"!type":"fn(params: ?) -> !this","!doc":"<p>Sets the href of the link dynamically according to the params passed, and any <a href=\"#!/api/Ext.button.Button-cfg-baseParams\" rel=\"Ext.button.Button-cfg-baseParams\" class=\"docClass\">baseParams</a> configured.</p>\n\n<pre><code>var button = Ext.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>', {\n    renderTo   : document.body,\n    text       : 'Open',\n    href       : 'http://www.sencha.com',\n    baseParams : {\n        foo : 'bar'\n    }\n});\n\nbutton.setParams({\n    company : 'Sencha'\n});\n</code></pre>\n\n<p>When clicked, this button will open a new window with the url http://www.sencha.com/?foo=bar&amp;company=Sencha because\nthe button wased configured with the <a href=\"#!/api/Ext.button.Button-cfg-baseParams\" rel=\"Ext.button.Button-cfg-baseParams\" class=\"docClass\">baseParams</a> to have <code>foo</code> = <code>'bar'</code> and then used <a href=\"#!/api/Ext.button.Button-method-setParams\" rel=\"Ext.button.Button-method-setParams\" class=\"docClass\">setParams</a> to set the <code>company</code> parameter to <code>'Sencha'</code>.</p>\n\n<p><strong>Only valid if the Button was originally configured with a <a href=\"#!/api/Ext.button.Button-cfg-href\" rel=\"Ext.button.Button-cfg-href\" class=\"docClass\">href</a></strong></p>\n"},"setPressed":{"!type":"fn(pressed?: ?) -> ?","!doc":"<p>Sets the <code>pressed</code> state of this button.</p>\n"},"setScale":{"!type":"fn(scale: ?) -> !this","!doc":"<p>Method to change the scale of the button. See <a href=\"#!/api/Ext.button.Button-cfg-scale\" rel=\"Ext.button.Button-cfg-scale\" class=\"docClass\">scale</a> for allowed configurations.</p>\n"},"setText":{"!type":"fn(text: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.button.Button-cfg-text\" rel=\"Ext.button.Button-cfg-text\" class=\"docClass\">text</a>.</p>\n"},"setTextAlign":{"!type":"fn(textAlign: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.button.Button-cfg-textAlign\" rel=\"Ext.button.Button-cfg-textAlign\" class=\"docClass\">textAlign</a>.</p>\n"},"setTooltip":{"!type":"fn(tooltip: ?) -> ?","!doc":"<p>Sets the tooltip for this Button.</p>\n"},"setUI":{"!type":"fn(ui: ?) -> !this","!doc":"<p>Sets the UI for the component. This will remove any existing UIs on the component. It will also loop through any\n<code>uiCls</code> set on the component and rename them so they include the new UI.</p>\n"},"showMenu":{"!type":"fn(fromEvent: ?) -> !this","!doc":"<p>Shows this button's menu (if it has one)</p>\n"},"toggle":{"!type":"fn(this: ?, pressed: ?, eOpts: ?)","!doc":"<p>Fires when the 'pressed' state of this button changes (only if enableToggle = true)</p>\n"},"updateArrowVisible":{"!type":"fn(visible: ?) -> !this"},"updateIconAlign":{"!type":"fn(align: ?, oldAlign: ?) -> !this"},"updateTextAlign":{"!type":"fn(align: ?, oldAlign: ?) -> !this"},"click":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when this button is clicked, before the configured <a href=\"#!/api/Ext.button.Button-cfg-handler\" rel=\"Ext.button.Button-cfg-handler\" class=\"docClass\">handler</a> is invoked. Execution of the\n<a href=\"#!/api/Ext.button.Button-cfg-handler\" rel=\"Ext.button.Button-cfg-handler\" class=\"docClass\">handler</a> may be vetoed by returning <code>false</code> to this event.</p>\n"},"glyphchange":{"!type":"fn(this: ?, newGlyph: ?, oldGlyph: ?, eOpts: ?)","!doc":"<p>Fired when the button's glyph is changed by the <a href=\"#!/api/Ext.button.Button-method-setGlyph\" rel=\"Ext.button.Button-method-setGlyph\" class=\"docClass\">setGlyph</a> method.</p>\n"},"iconchange":{"!type":"fn(this: ?, oldIcon: ?, newIcon: ?, eOpts: ?)","!doc":"<p>Fired when the button's icon is changed by the <a href=\"#!/api/Ext.button.Button-method-setIcon\" rel=\"Ext.button.Button-method-setIcon\" class=\"docClass\">setIcon</a> or <a href=\"#!/api/Ext.button.Button-method-setIconCls\" rel=\"Ext.button.Button-method-setIconCls\" class=\"docClass\">setIconCls</a> methods.</p>\n"},"menuhide":{"!type":"fn(this: ?, menu: ?, eOpts: ?)","!doc":"<p>If this button has a menu, this event fires when it is hidden</p>\n"},"menushow":{"!type":"fn(this: ?, menu: ?, eOpts: ?)","!doc":"<p>If this button has a menu, this event fires when it is shown</p>\n"},"menutriggerout":{"!type":"fn(this: ?, menu: ?, e: ?, eOpts: ?)","!doc":"<p>If this button has a menu, this event fires when the mouse leaves the menu triggering element</p>\n"},"menutriggerover":{"!type":"fn(this: ?, menu: ?, e: ?, eOpts: ?)","!doc":"<p>If this button has a menu, this event fires when the mouse enters the menu triggering element</p>\n"},"mouseout":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse exits the button</p>\n"},"mouseover":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse hovers over the button</p>\n"},"textchange":{"!type":"fn(this: ?, oldText: ?, newText: ?, eOpts: ?)","!doc":"<p>Fired when the button's text is changed by the <a href=\"#!/api/Ext.button.Button-method-setText\" rel=\"Ext.button.Button-method-setText\" class=\"docClass\">setText</a> method.</p>\n"}}},"Cycle":{"!doc":"<p>A specialized SplitButton that contains a menu of <a href=\"#!/api/Ext.menu.CheckItem\" rel=\"Ext.menu.CheckItem\" class=\"docClass\">Ext.menu.CheckItem</a> elements. The button automatically\ncycles through each menu item on click, raising the button's <a href=\"#!/api/Ext.button.Cycle-event-change\" rel=\"Ext.button.Cycle-event-change\" class=\"docClass\">change</a> event (or calling the button's\n<a href=\"#!/api/Ext.button.Cycle-cfg-changeHandler\" rel=\"Ext.button.Cycle-cfg-changeHandler\" class=\"docClass\">changeHandler</a> function, if supplied) for the active menu item. Clicking on the arrow section of the\nbutton displays the dropdown menu just like a normal SplitButton.  Example usage:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.button.Cycle\" rel=\"Ext.button.Cycle\" class=\"docClass\">Ext.button.Cycle</a>', {\n    showText: true,\n    prependText: 'View as ',\n    renderTo: Ext.getBody(),\n    menu: {\n        id: 'view-type-menu',\n        items: [{\n            text: 'text only',\n            iconCls: 'view-text',\n            checked: true\n        },{\n            text: 'HTML',\n            iconCls: 'view-html'\n        }]\n    },\n    changeHandler: function(cycleBtn, activeItem) {\n        <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Change View', activeItem.text);\n    }\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"changeHandler":{"!doc":"<p>A callback function that will be invoked each time the active menu item in the button's menu has changed. If this\ncallback is not supplied, the SplitButton will instead fire the <a href=\"#!/api/Ext.button.Cycle-event-change\" rel=\"Ext.button.Cycle-event-change\" class=\"docClass\">change</a> event on active item change. The\nchangeHandler function will be called with the following argument list: (SplitButton this, <a href=\"#!/api/Ext.menu.CheckItem\" rel=\"Ext.menu.CheckItem\" class=\"docClass\">Ext.menu.CheckItem</a>\nitem)</p>\n"},"forceGlyph":{"!doc":"<p>The charCode to be used as the static icon for this button.  This icon will always be\ndisplayed regardless of which item is selected in the dropdown list. This override\nthe default behavior of changing the button's icon to match the selected item's icon\non change. This property expects a format consistent with that of <a href=\"#!/api/Ext.button.Cycle-cfg-glyph\" rel=\"Ext.button.Cycle-cfg-glyph\" class=\"docClass\">glyph</a></p>\n"},"forceIcon":{"!type":"string","!doc":"<p>A css class which sets an image to be used as the static icon for this button. This icon will always be displayed\nregardless of which item is selected in the dropdown list. This overrides the default behavior of changing the\nbutton's icon to match the selected item's icon on change.</p>\n"},"items":{"!doc":"<p>An array of <a href=\"#!/api/Ext.menu.CheckItem\" rel=\"Ext.menu.CheckItem\" class=\"docClass\">Ext.menu.CheckItem</a> <strong>config</strong> objects to be used when creating the button's menu items (e.g.,\n<code>{text:'Foo', iconCls:'foo-icon'}</code>)</p>\n"},"prependText":{"!type":"string","!doc":"<p>A static string to prepend before the active item's text when displayed as the button's text (only applies when\nshowText = true).</p>\n"},"showText":{"!type":"bool","!doc":"<p>True to display the active item's text as the button text. The Button will show its\nconfigured <a href=\"#!/api/Ext.button.Cycle-cfg-text\" rel=\"Ext.button.Cycle-cfg-text\" class=\"docClass\">text</a> if this config is omitted.</p>\n"},"menu":{"!doc":"<p>The <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Menu</a> object used to display the <a href=\"#!/api/Ext.menu.CheckItem\" rel=\"Ext.menu.CheckItem\" class=\"docClass\">CheckItems</a> representing the\navailable choices.</p>\n"},"checkHandler":{"!type":"fn(item: ?, pressed: ?) -> !this"},"getActiveItem":{"!type":"fn() -> ?","!doc":"<p>Gets the currently active menu item.</p>\n"},"getButtonText":{"!type":"fn(item: ?) -> !this"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"setActiveItem":{"!type":"fn(item: ?, suppressEvent?: ?) -> !this","!doc":"<p>Sets the button's active menu item.</p>\n"},"toggleSelected":{"!type":"fn() -> !this","!doc":"<p>This is normally called internally on button click, but can be called externally to advance the button's active\nitem programmatically to the next one in the menu. If the current item is the last one in the menu the active\nitem will be set to the first item in the menu.</p>\n"},"change":{"!type":"fn(this: ?, item: ?, eOpts: ?)","!doc":"<p>Fires after the button's active menu item has changed. Note that if a <a href=\"#!/api/Ext.button.Cycle-cfg-changeHandler\" rel=\"Ext.button.Cycle-cfg-changeHandler\" class=\"docClass\">changeHandler</a> function is\nset on this CycleButton, it will be called instead on active item change and this change event will not\nbe fired.</p>\n"}}},"Split":{"prototype":{"arrowCls":{"!type":"string","!doc":"<p>The className used for the inner arrow element if the button has a menu.</p>\n"},"arrowHandler":{"!doc":"<p>A function called when the arrow button is clicked (can be used instead of click event)</p>\n"},"arrowTooltip":{"!type":"string","!doc":"<p>The title attribute of the arrow.</p>\n"},"isSplitButton":{"!type":"bool","!doc":"<p>End Definitions</p>\n"},"split":{"!type":"bool"},"onClick":{"!type":"fn(e: ?) -> !this"},"setArrowHandler":{"!type":"fn(handler: ?, scope?: ?) -> !this","!doc":"<p>Sets this button's arrow click handler.</p>\n"},"arrowclick":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when this button's arrow is clicked.</p>\n"}},"!doc":"<p>A split button that provides a built-in dropdown arrow that can fire an event separately from the default click event\nof the button. Typically this would be used to display a dropdown menu that provides additional options to the\nprimary button action, but any custom handler can provide the arrowclick implementation.  Example usage:</p>\n\n<pre class='inline-example '><code>// display a dropdown menu:\nExt.create('<a href=\"#!/api/Ext.button.Split\" rel=\"Ext.button.Split\" class=\"docClass\">Ext.button.Split</a>', {\n    renderTo: Ext.getBody(),\n    text: 'Options',\n    // handle a click on the button itself\n    handler: function() {\n        alert(\"The button was clicked\");\n    },\n    menu: new <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a>({\n        items: [\n            // these will render as dropdown menu items when the arrow is clicked:\n            {text: 'Item 1', handler: function(){ alert(\"Item 1 clicked\"); }},\n            {text: 'Item 2', handler: function(){ alert(\"Item 2 clicked\"); }}\n        ]\n    })\n});\n</code></pre>\n\n<p>Instead of showing a menu, you can provide any type of custom functionality you want when the dropdown\narrow is clicked:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.button.Split\" rel=\"Ext.button.Split\" class=\"docClass\">Ext.button.Split</a>', {\n    renderTo: 'button-ct',\n    text: 'Options',\n    handler: optionsHandler,\n    arrowHandler: myCustomHandler\n});\n</code></pre>\n","!type":"fn(config: ?)"},"Manager":{"buttonSelector":{"!type":"string"},"groups":{},"pressedButton":{},"getPressed":{"!type":"fn(groupName: ?) -> ?","!doc":"<p>Gets the pressed button in the passed group or null</p>\n"},"init":{"!type":"fn() -> !this"},"onButtonMousedown":{"!type":"fn(button: ?, e: ?) -> !this","!doc":"<p>Called by buton instances.\nTrack the button which was mousedowned upon so that the next <em>document</em> mouseup can be delivered to it\nin case mouse is moved outside of button element.</p>\n"},"onDocumentKeyDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Buttons must react to SPACE and ENTER to trigger the click handler.\nNow that they are <code>&lt;a&gt;</code> elements, we use a keydown listener.</p>\n"},"onDocumentMouseUp":{"!type":"fn(e: ?) -> !this"},"register":{"!type":"fn(btn: ?) -> !this"},"toggleGroup":{"!type":"fn(btn: ?, state: ?) -> !this"},"unregister":{"!type":"fn(btn: ?) -> !this"}},"Segmented":{"!doc":"<p>SegmentedButton is a container for a group of <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Button</a>s.</p>\n","!type":"fn(config: ?)","prototype":{"allowDepress":{"!type":"bool","!doc":"<p>Allow toggling the pressed state of each button.\nOnly applicable when <a href=\"#!/api/Ext.button.Segmented-cfg-allowMultiple\" rel=\"Ext.button.Segmented-cfg-allowMultiple\" class=\"docClass\">allowMultiple</a> is <code>false</code>.</p>\n"},"allowMultiple":{"!type":"bool","!doc":"<p>Allow multiple pressed buttons.</p>\n"},"allowToggle":{"!type":"bool","!doc":"<p>True to enable pressed/not pressed toggling.</p>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"defaultType":{"!type":"string","!doc":"<p>The default <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">xtype</a> of child Components to create in this Container when\na child item is specified as a raw configuration object, rather than as an instantiated Component.</p>\n"},"defaultUI":{"!type":"string","!doc":"<p>Default <a href=\"#!/api/Ext.Component-cfg-ui\" rel=\"Ext.Component-cfg-ui\" class=\"docClass\">ui</a> to use for buttons in this segmented button.\nButtons can override this default by specifying their own UI</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.button.Segmented-cfg-items\" rel=\"Ext.button.Segmented-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"value":{"!doc":"<p>The value of this button.  When <a href=\"#!/api/Ext.button.Segmented-cfg-allowMultiple\" rel=\"Ext.button.Segmented-cfg-allowMultiple\" class=\"docClass\">allowMultiple</a> is <code>false</code>, value is a\nString or Number.  When <a href=\"#!/api/Ext.button.Segmented-cfg-allowMultiple\" rel=\"Ext.button.Segmented-cfg-allowMultiple\" class=\"docClass\">is &lt;code&gt;true&lt;/code&gt;, value is an array\nof values.  A value corresponds to a child button's {@link Ext.button.Button#value\nvalue</a>, or its index if no child button values match the given value.</p>\n\n<p>Using the <code>value</code> config of the child buttons with single toggle:</p>\n\n<pre class='inline-example '><code>var button = Ext.create('<a href=\"#!/api/Ext.button.Segmented\" rel=\"Ext.button.Segmented\" class=\"docClass\">Ext.button.Segmented</a>', {\n    renderTo: Ext.getBody(),\n    value: 'optTwo', // begin with \"Option Two\" selected\n    items: [{\n        text: 'Option One',\n        value: 'optOne'\n    }, {\n        text: 'Option Two',\n        value: 'optTwo'\n    }, {\n        text: 'Option Three',\n        value:  'optThree'\n    }]\n});\n\nconsole.log(button.getValue()); // optTwo\n\n// Sets the value to optOne, and sets the pressed state of the \"Option One\" button\nbutton.setValue('optOne');\n\nconsole.log(button.getValue()); // optOne\n</code></pre>\n\n<p>Using multiple toggle, and index-based values:</p>\n\n<pre class='inline-example '><code>var button = Ext.create('<a href=\"#!/api/Ext.button.Segmented\" rel=\"Ext.button.Segmented\" class=\"docClass\">Ext.button.Segmented</a>', {\n    renderTo: Ext.getBody(),\n    allowMultiple: true\n    value: [1, 2], // begin with \"Option Two\" and \"Option Three\" selected\n    items: [{\n        text: 'Option One'\n    }, {\n        text: 'Option Two'\n    }, {\n        text: 'Option Three'\n    }]\n});\n\n// Sets value to [0, 2], and sets pressed state of \"Option One\" and \"Option Three\"\nbutton.setValue([0, 2]);\n\nconsole.log(button.getValue()); // [0, 2]\n\n// Remove all pressed buttons, and set value to null\nbutton.setValue(null);\n</code></pre>\n"},"vertical":{"!type":"bool","!doc":"<p>True to align the buttons vertically</p>\n"},"_firstCls":{"!type":"string","!doc":"<p>private</p>\n"},"_lastCls":{"!type":"string","!doc":"<p>private</p>\n"},"_middleCls":{"!type":"string","!doc":"<p>private</p>\n"},"beforeRenderConfig":{},"isSegmentedButton":{"!type":"bool"},"itemCls":{"!type":"string"},"maskOnDisable":{"!type":"bool","!doc":"<p>This is an internal flag that you use when creating custom components. By default this is set to <code>true</code> which means\nthat every component gets a mask when it's disabled. Components like FieldContainer, FieldSet, Field, Button, Tab\noverride this property to <code>false</code> since they want to implement custom disable logic.</p>\n"},"privates":{"!doc":"<p></p>\n"},"publishes":{},"twoWayBindable":{},"_lookupButtonByValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Looks up a child button by its value</p>\n"},"_onItemToggle":{"!type":"fn(button: ?, pressed: ?) -> !this","!doc":"<p>Handles the \"toggle\" event of the child buttons.</p>\n"},"_syncItemClasses":{"!type":"fn(force: ?) -> !this","!doc":"<p>Synchronizes the \"first\", \"last\", and \"middle\" css classes when buttons are\nadded, removed, shown, or hidden</p>\n"},"beforeLayout":{"!type":"fn() -> !this","!doc":"<p>Occurs before componentLayout is run. In previous releases, this method could\nreturn <code>false</code> to prevent its layout but that is not supported in Ext JS 4.1 or\nhigher. This method is simply a notification of the impending layout to give the\ncomponent a chance to adjust the DOM. Ideally, DOM reads should be avoided at this\ntime to reduce expensive document reflows.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"getAllowDepress":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.button.Segmented-cfg-allowDepress\" rel=\"Ext.button.Segmented-cfg-allowDepress\" class=\"docClass\">allowDepress</a>.</p>\n"},"getAllowMultiple":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.button.Segmented-cfg-allowMultiple\" rel=\"Ext.button.Segmented-cfg-allowMultiple\" class=\"docClass\">allowMultiple</a>.</p>\n"},"getAllowToggle":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.button.Segmented-cfg-allowToggle\" rel=\"Ext.button.Segmented-cfg-allowToggle\" class=\"docClass\">allowToggle</a>.</p>\n"},"getDefaultUI":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.button.Segmented-cfg-defaultUI\" rel=\"Ext.button.Segmented-cfg-defaultUI\" class=\"docClass\">defaultUI</a>.</p>\n"},"getVertical":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.button.Segmented-cfg-vertical\" rel=\"Ext.button.Segmented-cfg-vertical\" class=\"docClass\">vertical</a>.</p>\n"},"onAdd":{"!type":"fn(item: ?) -> !this","!doc":"<p>This method is invoked after a new Component has been added. It\nis passed the Component which has been added. This method may\nbe used to update any internal structure which may depend upon\nthe state of the child items.</p>\n"},"onRemove":{"!type":"fn(item: ?) -> !this","!doc":"<p>This method is invoked after a new Component has been\nremoved. It is passed the Component which has been\nremoved. This method may be used to update any internal\nstructure which may depend upon the state of the child items.</p>\n"},"setAllowDepress":{"!type":"fn(allowDepress: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.button.Segmented-cfg-allowDepress\" rel=\"Ext.button.Segmented-cfg-allowDepress\" class=\"docClass\">allowDepress</a>.</p>\n"},"setAllowMultiple":{"!type":"fn(allowMultiple: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.button.Segmented-cfg-allowMultiple\" rel=\"Ext.button.Segmented-cfg-allowMultiple\" class=\"docClass\">allowMultiple</a>.</p>\n"},"setAllowToggle":{"!type":"fn(allowToggle: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.button.Segmented-cfg-allowToggle\" rel=\"Ext.button.Segmented-cfg-allowToggle\" class=\"docClass\">allowToggle</a>.</p>\n"},"setDefaultUI":{"!type":"fn(defaultUI: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.button.Segmented-cfg-defaultUI\" rel=\"Ext.button.Segmented-cfg-defaultUI\" class=\"docClass\">defaultUI</a>.</p>\n"},"setVertical":{"!type":"fn(vertical: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.button.Segmented-cfg-vertical\" rel=\"Ext.button.Segmented-cfg-vertical\" class=\"docClass\">vertical</a>.</p>\n"},"updateAllowDepress":{"!type":"fn(newAllowDepress: ?, oldAllowDepress: ?) -> !this","!doc":"<p><debug></debug></p>\n"},"updateAllowMultiple":{"!type":"fn(newAllowMultiple: ?, oldAllowMultiple: ?) -> !this"},"updateAllowToggle":{"!type":"fn(newAllowToggle: ?, oldAllowToggle: ?) -> !this"},"updateDefaultUI":{"!type":"fn(defaultUI: ?) -> !this"},"updateVertical":{"!type":"fn(newVertical: ?, oldVertical: ?) -> !this"},"toggle":{"!type":"fn(this: ?, button: ?, isPressed: ?, eOpts: ?)","!doc":"<p>Fires when any child button's pressed state has changed.</p>\n"}}}},"Component":{"prototype":{"alignTarget":{"!type":"string","!doc":"<p>A Component or Element by which to position this component according to the <a href=\"#!/api/Ext.Component-cfg-defaultAlign\" rel=\"Ext.Component-cfg-defaultAlign\" class=\"docClass\">defaultAlign</a>.\nDefaults to ths owning Container.</p>\n\n<p><em>Only applicable if this component is <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a></em></p>\n\n<p><em>Used upon first show</em>.</p>\n"},"autoRender":{"!doc":"<p>This config is intended mainly for non-<a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Components which may or may not be shown. Instead of using\n<a href=\"#!/api/Ext.Component-cfg-renderTo\" rel=\"Ext.Component-cfg-renderTo\" class=\"docClass\">renderTo</a> in the configuration, and rendering upon construction, this allows a Component to render itself\nupon first <em><a href=\"#!/api/Ext.Component-method-show\" rel=\"Ext.Component-method-show\" class=\"docClass\">show</a></em>. If <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> is <code>true</code>, the value of this config is omitted as if it is <code>true</code>.</p>\n\n<p>Specify as <code>true</code> to have this Component render to the document body upon first show.</p>\n\n<p>Specify as an element, or the ID of an element to have this Component render to a specific element upon first\nshow.</p>\n"},"autoScroll":{"!type":"bool","!doc":"<p><code>true</code> to use overflow:'auto' on the components layout element and show scroll bars automatically when necessary,\n<code>false</code> to clip any overflowing content.\nThis should not be combined with <a href=\"#!/api/Ext.Component-cfg-overflowX\" rel=\"Ext.Component-cfg-overflowX\" class=\"docClass\">overflowX</a> or  <a href=\"#!/api/Ext.Component-cfg-overflowY\" rel=\"Ext.Component-cfg-overflowY\" class=\"docClass\">overflowY</a>.</p>\n"},"autoShow":{"!type":"bool","!doc":"<p><code>true</code> to automatically show the component upon creation. This config option may only be used for\n<a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> components or components that use <a href=\"#!/api/Ext.Component-cfg-autoRender\" rel=\"Ext.Component-cfg-autoRender\" class=\"docClass\">autoRender</a>.</p>\n"},"border":{"!doc":"<p>Specifies the border size for this component. The border can be a single numeric value to apply to all sides or it can\nbe a CSS style specification for each style, for example: '10 5 3 10' (top, right, bottom, left).</p>\n\n<p>For components that have no border by default, setting this won't make the border appear by itself.\nYou also need to specify border color and style:</p>\n\n<pre><code>border: 5,\nstyle: {\n    borderColor: 'red',\n    borderStyle: 'solid'\n}\n</code></pre>\n\n<p>To turn off the border, use <code>border: false</code>.</p>\n"},"columnWidth":{"!doc":"<p>Defines the column width inside <a href=\"#!/api/Ext.layout.container.Column\" rel=\"Ext.layout.container.Column\" class=\"docClass\">column layout</a>.</p>\n\n<p>Can be specified as a number or as a percentage.</p>\n"},"componentCls":{"!type":"string","!doc":"<p>CSS Class to be added to a components root level element to give distinction to it via styling.</p>\n"},"componentLayout":{"!doc":"<p>The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout\nmanager which sizes a Component's internal structure in response to the Component being sized.</p>\n\n<p>Generally, developers will not use this configuration as all provided Components which need their internal\nelements sizing (Such as <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">input fields</a>) come with their own componentLayout managers.</p>\n\n<p>The <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used on instances of the base <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>\nclass which simply sizes the Component's encapsulating element to the height and width specified in the\n<a href=\"#!/api/Ext.Component-method-setSize\" rel=\"Ext.Component-method-setSize\" class=\"docClass\">setSize</a> method.</p>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"constrainTo":{"!doc":"<p>A Region (or an element from which a Region measurement will be read) which is used\nto constrain the component. Only applies when the component is floating.</p>\n"},"constraintInsets":{"!doc":"<p>An object or a string (in TRBL order) specifying insets from the configured <a href=\"#!/api/Ext.Component-cfg-constrainTo\" rel=\"Ext.Component-cfg-constrainTo\" class=\"docClass\">constrain region</a>\nwithin which this component must be constrained when positioning or sizing.\nexample:</p>\n\n<p>   constraintInsets: '10 10 10 10' // Constrain with 10px insets from parent</p>\n"},"contentEl":{"!type":"string","!doc":"<p>Specify an existing HTML element, or the <code>id</code> of an existing HTML element to use as the content for this component.</p>\n\n<p>This config option is used to take an existing HTML element and place it in the layout element of a new component\n(it simply moves the specified DOM element <em>after the Component is rendered</em> to use as the content.</p>\n\n<p><strong>Notes:</strong></p>\n\n<p>The specified HTML element is appended to the layout element of the component <em>after any configured\n<a href=\"#!/api/Ext.Component-cfg-html\" rel=\"Ext.Component-cfg-html\" class=\"docClass\">HTML</a> has been inserted</em>, and so the document will not contain this element at the time\nthe <a href=\"#!/api/Ext.Component-event-render\" rel=\"Ext.Component-event-render\" class=\"docClass\">render</a> event is fired.</p>\n\n<p>The specified HTML element used will not participate in any <strong><code><a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a></code></strong>\nscheme that the Component may use. It is just HTML. Layouts operate on child\n<strong><code><a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a></code></strong>.</p>\n\n<p>Add either the <code>x-hidden</code> or the <code>x-hidden-display</code> CSS class to prevent a brief flicker of the content before it\nis rendered to the panel.</p>\n"},"data":{"!doc":"<p>The initial set of data to apply to the <code><a href=\"#!/api/Ext.Component-cfg-tpl\" rel=\"Ext.Component-cfg-tpl\" class=\"docClass\">tpl</a></code> to update the content\narea of the Component.</p>\n"},"defaultAlign":{"!type":"string","!doc":"<p>The default Ext.dom.Element#getAlignToXY anchor position value for this component\nrelative to its <a href=\"#!/api/Ext.Component-cfg-alignTarget\" rel=\"Ext.Component-cfg-alignTarget\" class=\"docClass\">alignTarget</a> (which defaults to its owning Container).</p>\n\n<p><em>Only applicable if this component is <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a></em></p>\n\n<p><em>Used upon first show</em>.</p>\n"},"disabledCls":{"!type":"string","!doc":"<p>CSS class to add when the Component is disabled.</p>\n"},"draggable":{"!doc":"<p>Specify as true to make a <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Component draggable using the Component's encapsulating element as\nthe drag handle.</p>\n\n<p>This may also be specified as a config object for the <a href=\"#!/api/Ext.util.ComponentDragger\" rel=\"Ext.util.ComponentDragger\" class=\"docClass\">ComponentDragger</a> which is\ninstantiated to perform dragging.</p>\n\n<p>For example to create a Component which may only be dragged around using a certain internal element as the drag\nhandle, use the delegate option:</p>\n\n<pre><code>new <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>({\n    constrain: true,\n    floating: true,\n    style: {\n        backgroundColor: '#fff',\n        border: '1px solid black'\n    },\n    html: '&lt;h1 style=\"cursor:move\"&gt;The title&lt;/h1&gt;&lt;p&gt;The content&lt;/p&gt;',\n    draggable: {\n        delegate: 'h1'\n    }\n}).show();\n</code></pre>\n"},"floating":{"!type":"bool","!doc":"<p>Specify as true to float the Component outside of the document flow using CSS absolute positioning.</p>\n\n<p>Components such as <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Window</a>s and <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Menu</a>s are floating by default.</p>\n\n<p>Floating Components that are programatically <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">rendered</a> will register\nthemselves with the global <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a></p>\n\n<h3>Floating Components as child items of a Container</h3>\n\n<p>A floating Component may be used as a child item of a Container. This just allows the floating Component to seek\na ZIndexManager by examining the ownerCt chain.</p>\n\n<p>When configured as floating, Components acquire, at render time, a <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a> which\nmanages a stack of related floating Components. The ZIndexManager brings a single floating Component to the top\nof its stack when the Component's <a href=\"#!/api/Ext.Component-method-toFront\" rel=\"Ext.Component-method-toFront\" class=\"docClass\">toFront</a> method is called.</p>\n\n<p>The ZIndexManager is found by traversing up the <a href=\"#!/api/Ext.Component-property-ownerCt\" rel=\"Ext.Component-property-ownerCt\" class=\"docClass\">ownerCt</a> chain to find an ancestor which itself is\nfloating. This is so that descendant floating Components of floating <em>Containers</em> (Such as a ComboBox dropdown\nwithin a Window) can have its zIndex managed relative to any siblings, but always <strong>above</strong> that floating\nancestor Container.</p>\n\n<p>If no floating ancestor is found, a floating Component registers itself with the default <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a>.</p>\n\n<p>Floating components <em>do not participate in the Container's layout</em>. Because of this, they are not rendered until\nyou explicitly <a href=\"#!/api/Ext.Component-method-show\" rel=\"Ext.Component-method-show\" class=\"docClass\">show</a> them.</p>\n\n<p>After rendering, the ownerCt reference is deleted, and the <a href=\"#!/api/Ext.Component-property-floatParent\" rel=\"Ext.Component-property-floatParent\" class=\"docClass\">floatParent</a> property is set to the found\nfloating ancestor Container. If no floating ancestor Container was found the <a href=\"#!/api/Ext.Component-property-floatParent\" rel=\"Ext.Component-property-floatParent\" class=\"docClass\">floatParent</a> property will\nnot be set.</p>\n"},"formBind":{"!type":"bool","!doc":"<p>When inside FormPanel, any component configured with <code>formBind: true</code> will\nbe enabled/disabled depending on the validity state of the form.\nSee <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a> for more information and example.</p>\n"},"height":{"!type":"number","!doc":"<p>The height of this component in pixels.</p>\n"},"hideMode":{"!type":"string","!doc":"<p>A String which specifies how this Component's encapsulating DOM element will be hidden. Values may be:</p>\n\n<ul>\n<li><code>'display'</code> : The Component will be hidden using the <code>display: none</code> style.</li>\n<li><code>'visibility'</code> : The Component will be hidden using the <code>visibility: hidden</code> style.</li>\n<li><code>'offsets'</code> : The Component will be hidden by absolutely positioning it out of the visible area of the document.\nThis is useful when a hidden Component must maintain measurable dimensions. Hiding using <code>display</code> results in a\nComponent having zero dimensions.</li>\n</ul>\n\n"},"html":{"!doc":"<p>An HTML fragment, or a DomHelper specification to use as the layout element content.\nThe HTML content is added after the component is rendered, so the document will not contain this HTML at the time\nthe <a href=\"#!/api/Ext.Component-event-render\" rel=\"Ext.Component-event-render\" class=\"docClass\">render</a> event is fired. This content is inserted into the body <em>before</em> any configured <a href=\"#!/api/Ext.Component-cfg-contentEl\" rel=\"Ext.Component-cfg-contentEl\" class=\"docClass\">contentEl</a>\nis appended.</p>\n"},"id":{"!type":"string","!doc":"<p>The <strong>unique</strong> id of this component instance.</p>\n\n<p>Use of this config should be considered carefully as this value must be unique across\nall existing components. Components created with an <code>id</code> may be accessed globally\nusing Ext.getCmp.</p>\n\n<p>Instead of using assigned ids, consider a reference config and a ViewController\nto respond to events and perform processing upon this Component.</p>\n\n<p>Alternatively, <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">itemId</a> and ComponentQuery can be\nused to perform selector-based searching for Components analogous to DOM querying.\nThe <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a> class contains several helpful\nshortcut methods to query its descendant\nComponents by selector.</p>\n\n<p>Note that this <code>id</code> will also be used as the element id for the containing HTML\nelement that is rendered to the page for this component. This allows you to write\nid-based CSS rules to style the specific instance of this component uniquely, and\nalso to select sub-elements using this component's <code>id</code> as the parent.</p>\n\n<p>Defaults to an <a href=\"#!/api/Ext.Component-method-getId\" rel=\"Ext.Component-method-getId\" class=\"docClass\">auto-assigned id</a>.</p>\n\n<p><strong>Note</strong>: Valid identifiers start with a letter or underscore and are followed by\n(optional) additional letters, underscores, digits or hyphens.</p>\n"},"itemId":{"!type":"string","!doc":"<p>The <strong>unique</strong> id of this component instance within its container. See also the\nreference config.</p>\n\n<p>An <code>itemId</code> can be used as an alternative way to get a reference to a component when no object reference is\navailable. Instead of using an <code><a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">id</a></code> with getCmp, use\n<code>itemId</code> with <a href=\"#!/api/Ext.container.Container-method-getComponent\" rel=\"Ext.container.Container-method-getComponent\" class=\"docClass\">getComponent</a> which will\nretrieve <code>itemId</code>'s or <a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">id</a>'s. Since <code>itemId</code>'s are an index to the container's\ninternal collection, the <code>itemId</code> is scoped locally to the container -- avoiding\npotential conflicts with Ext.ComponentManager which requires a <strong>unique</strong>\n<code><a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">id</a></code> values.</p>\n\n<pre><code>var c = new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({ //\n    <a href=\"#!/api/Ext.Component-cfg-height\" rel=\"Ext.Component-cfg-height\" class=\"docClass\">height</a>: 300,\n    <a href=\"#!/api/Ext.Component-cfg-renderTo\" rel=\"Ext.Component-cfg-renderTo\" class=\"docClass\">renderTo</a>: document.body,\n    <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a>: 'auto',\n    <a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a>: [\n        {\n            itemId: 'p1',\n            <a href=\"#!/api/Ext.panel.Panel-cfg-title\" rel=\"Ext.panel.Panel-cfg-title\" class=\"docClass\">title</a>: 'Panel 1',\n            <a href=\"#!/api/Ext.Component-cfg-height\" rel=\"Ext.Component-cfg-height\" class=\"docClass\">height</a>: 150\n        },\n        {\n            itemId: 'p2',\n            <a href=\"#!/api/Ext.panel.Panel-cfg-title\" rel=\"Ext.panel.Panel-cfg-title\" class=\"docClass\">title</a>: 'Panel 2',\n            <a href=\"#!/api/Ext.Component-cfg-height\" rel=\"Ext.Component-cfg-height\" class=\"docClass\">height</a>: 150\n        }\n    ]\n})\np1 = c.<a href=\"#!/api/Ext.container.Container-method-getComponent\" rel=\"Ext.container.Container-method-getComponent\" class=\"docClass\">getComponent</a>('p1'); // not the same as Ext.getCmp()\np2 = p1.<a href=\"#!/api/Ext.Component-property-ownerCt\" rel=\"Ext.Component-property-ownerCt\" class=\"docClass\">ownerCt</a>.<a href=\"#!/api/Ext.container.Container-method-getComponent\" rel=\"Ext.container.Container-method-getComponent\" class=\"docClass\">getComponent</a>('p2'); // reference via a sibling\n</code></pre>\n\n<p>Also see <a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">id</a>, <code>Ext.container.Container.query</code>, <code>Ext.container.Container.down</code> and\n<code>Ext.container.Container.child</code>.</p>\n\n<p><strong>Note</strong>: Valid identifiers start with a letter or underscore and are followed by\n(optional) additional letters, underscores, digits or hyphens.</p>\n\n<p><strong>Note</strong>: to access the container of an item see <a href=\"#!/api/Ext.Component-property-ownerCt\" rel=\"Ext.Component-property-ownerCt\" class=\"docClass\">ownerCt</a>.</p>\n"},"loader":{"!doc":"<p>A configuration object or an instance of a <a href=\"#!/api/Ext.ComponentLoader\" rel=\"Ext.ComponentLoader\" class=\"docClass\">Ext.ComponentLoader</a> to load remote content\nfor this Component.</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    loader: {\n        url: 'content.html',\n        autoLoad: true\n    },\n    renderTo: Ext.getBody()\n});\n</code></pre>\n"},"margin":{"!doc":"<p>Specifies the margin for this component. The margin can be a single numeric value to apply to all sides or it can\nbe a CSS style specification for each style, for example: '10 5 3 10' (top, right, bottom, left).</p>\n"},"maskElement":{"!type":"string","!doc":"<p>Related to the <a href=\"#!/api/Ext.Component-cfg-childEls\" rel=\"Ext.Component-cfg-childEls\" class=\"docClass\">childEls</a> configuration which specifies named properties which correspond to component sub-elements.</p>\n\n<p>The name of the element property in this component to mask when masked by a LoadMask.</p>\n\n<p>Defaults to <code>null</code> to indicate that Components cannot by default contain a LoadMask, and that any LoadMask should be rendered into the document body.</p>\n\n<p>For example, Panels use <code>\"el\"</code> to indicate that the whole panel should be masked. This could be configured to be\n<code>\"body\"</code> so that only the body is masked and toolbars and the header are still mouse-accessible.</p>\n"},"maxHeight":{"!type":"number","!doc":"<p>The maximum value in pixels which this Component will set its height to.</p>\n\n<p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>\n"},"maxWidth":{"!type":"number","!doc":"<p>The maximum value in pixels which this Component will set its width to.</p>\n\n<p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>\n"},"minHeight":{"!type":"number","!doc":"<p>The minimum value in pixels which this Component will set its height to.</p>\n\n<p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>\n"},"modelValidation":{"!type":"bool","!doc":"<p>This config enables binding to your <code>Ext.data.Model.validators</code>. This\nis only processed by form fields (e.g., <code><a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a></code>) at present, but\nthis setting is inherited and so can be set on a parent container.</p>\n\n<p>When set to <code>true</code> by a component or not set by a component but inherited from\nan ancestor container, <code>Ext.data.Validation</code> records are used to automatically\nbind validation results for any form field to which a <code>value</code> is bound.</p>\n\n<p>While this config can be set arbitrarily high in the component hierarchy, doing\nso can create a lot overhead if most of your form fields do not actually rely on\n<code>validators</code> in your data model.</p>\n\n<p>Using this setting for a form that is bound to an <code>Ext.data.Model</code> might look\nlike this:</p>\n\n<pre><code> {\n     xtype: 'panel',\n     modelValidation: true,\n     items: [{\n         xtype: 'textfield',\n         bind: '{theUser.firstName}'\n     },{\n         xtype: 'textfield',\n         bind: '{theUser.lastName}'\n     },{\n         xtype: 'textfield',\n         bind: '{theUser.phoneNumber}'\n     },{\n         xtype: 'textfield',\n         bind: '{theUser.email}'\n     }]\n }\n</code></pre>\n\n<p>The above is equivalent to the following manual binding of validation:</p>\n\n<pre><code> {\n     xtype: 'panel',\n     items: [{\n         xtype: 'textfield',\n         bind: {\n             value:      '{theUser.firstName}'\n             validation: '{theUser.validation.firstName}'\n         }\n     },{\n         xtype: 'textfield',\n         bind: {\n             value:      '{theUser.lastName}'\n             validation: '{theUser.validation.lastName}'\n         }\n     },{\n         xtype: 'textfield',\n         bind: {\n             value:      '{theUser.phoneNumber}'\n             validation: '{theUser.validation.phoneNumber}'\n         }\n     },{\n         xtype: 'textfield',\n         bind: {\n             value:      '{theUser.email}'\n             validation: '{theUser.validation.email}'\n         }\n     }]\n }\n</code></pre>\n\n<p>Notice that \"validation\" is a pseudo-association defined for all entities. See\n<code>Ext.data.Model.getValidation</code> for further details.</p>\n"},"overflowX":{"!type":"string","!doc":"<p>Possible values are:</p>\n\n<ul>\n<li><code>'auto'</code> to enable automatic horizontal scrollbar (Style overflow-x: 'auto').</li>\n<li><code>'scroll'</code> to always enable horizontal scrollbar (Style overflow-x: 'scroll').</li>\n</ul>\n\n\n<p>The default is overflow-x: 'hidden'. This should not be combined with <a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a>.</p>\n"},"overflowY":{"!type":"string","!doc":"<p>Possible values are:</p>\n\n<ul>\n<li><code>'auto'</code> to enable automatic vertical scrollbar (Style overflow-y: 'auto').</li>\n<li><code>'scroll'</code> to always enable vertical scrollbar (Style overflow-y: 'scroll').</li>\n</ul>\n\n\n<p>The default is overflow-y: 'hidden'. This should not be combined with <a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a>.</p>\n"},"padding":{"!doc":"<p>Specifies the padding for this component. The padding can be a single numeric value to apply to all sides or it\ncan be a CSS style specification for each style, for example: '10 5 3 10' (top, right, bottom, left).</p>\n"},"plugins":{"!doc":"<p>An array of plugins to be added to this component. Can also be just a single plugin instead of array.</p>\n\n<p>Plugins provide custom functionality for a component. The only requirement for\na valid plugin is that it contain an <code>init</code> method that accepts a reference of type <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>. When a component\nis created, if any plugins are available, the component will call the init method on each plugin, passing a\nreference to itself. Each plugin can then call methods or respond to events on the component as needed to provide\nits functionality.</p>\n\n<p>Plugins can be added to component by either directly referencing the plugin instance:</p>\n\n<pre><code>plugins: [Ext.create('<a href=\"#!/api/Ext.grid.plugin.CellEditing\" rel=\"Ext.grid.plugin.CellEditing\" class=\"docClass\">Ext.grid.plugin.CellEditing</a>', {clicksToEdit: 1})],\n</code></pre>\n\n<p>By using config object with ptype:</p>\n\n<pre><code>plugins: [{ptype: 'cellediting', clicksToEdit: 1}],\n</code></pre>\n\n<p>Or with just a ptype:</p>\n\n<pre><code>plugins: ['cellediting', 'gridviewdragdrop'],\n</code></pre>\n\n<p>See <a href=\"#!/api/Ext.enums.Plugin\" rel=\"Ext.enums.Plugin\" class=\"docClass\">Ext.enums.Plugin</a> for list of all ptypes.</p>\n"},"region":{"!doc":"<p>Defines the region inside <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>.</p>\n\n<p>Possible values:</p>\n\n<ul>\n<li>north - Positions component at top.</li>\n<li>south - Positions component at bottom.</li>\n<li>east - Positions component at right.</li>\n<li>west - Positions component at left.</li>\n<li>center - Positions component at the remaining space.\nThere <strong>must</strong> be a component with <code>region: \"center\"</code> in every border layout.</li>\n</ul>\n\n"},"renderData":{"!doc":"<p>The data used by <a href=\"#!/api/Ext.Component-cfg-renderTpl\" rel=\"Ext.Component-cfg-renderTpl\" class=\"docClass\">renderTpl</a> in addition to the following property values of the component:</p>\n\n<ul>\n<li>id</li>\n<li>ui</li>\n<li>uiCls</li>\n<li>baseCls</li>\n<li>componentCls</li>\n<li>frame</li>\n</ul>\n\n\n<p>See <a href=\"#!/api/Ext.Component-cfg-renderSelectors\" rel=\"Ext.Component-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.Component-cfg-childEls\" rel=\"Ext.Component-cfg-childEls\" class=\"docClass\">childEls</a> for usage examples.</p>\n"},"renderSelectors":{"!doc":"<p>An object containing properties specifying CSS selectors which identify child elements\ncreated by the render process.</p>\n\n<p>After the Component's internal structure is rendered according to the <a href=\"#!/api/Ext.Component-cfg-renderTpl\" rel=\"Ext.Component-cfg-renderTpl\" class=\"docClass\">renderTpl</a>, this object is iterated through,\nand the found Elements are added as properties to the Component using the <code>renderSelector</code> property name.</p>\n\n<p>For example, a Component which renders a title and description into its element:</p>\n\n<pre><code> Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n     renderTo: Ext.getBody(),\n     renderTpl: [\n         '&lt;h1 class=\"title\"&gt;{title}&lt;/h1&gt;',\n         '&lt;p&gt;{desc}&lt;/p&gt;'\n     ],\n     renderData: {\n         title: \"Error\",\n         desc: \"Something went wrong\"\n     },\n     renderSelectors: {\n         titleEl: 'h1.title',\n         descEl: 'p'\n     },\n     listeners: {\n         afterrender: function(cmp){\n             // After rendering the component will have a titleEl and descEl properties\n             cmp.titleEl.setStyle({color: \"red\"});\n         }\n     }\n });\n</code></pre>\n\n<p>The use of <code>renderSelectors</code> is deprecated (for performance reasons). The above\ncode should be refactored into something like this:</p>\n\n<pre><code> Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n     renderTo: Ext.getBody(),\n     renderTpl: [\n         '&lt;h1 class=\"title\" id=\"{id}-titleEl\" data-ref=\"titleEl\"&gt;{title}&lt;/h1&gt;',\n         '&lt;p id=\"{id}-descEl\" data-ref=\"descEl\"&gt;{desc}&lt;/p&gt;'\n     ],\n     renderData: {\n         title: \"Error\",\n         desc: \"Something went wrong\"\n     },\n     childEls: [\n         'titleEl',\n         'descEl'\n     ]\n });\n</code></pre>\n\n<p>To use <code>childEls</code> yet retain the use of selectors (which remains as expensive as\n<code>renderSelectors</code>):</p>\n\n<pre><code> Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n     renderTo: Ext.getBody(),\n     renderTpl: [\n         '&lt;h1 class=\"title\"&gt;{title}&lt;/h1&gt;',\n         '&lt;p&gt;{desc}&lt;/p&gt;'\n     ],\n     renderData: {\n         title: \"Error\",\n         desc: \"Something went wrong\"\n     },\n     childEls: {\n         titleEl: { selectNode: 'h1.title' },\n         descEl: { selectNode: 'p' }\n     }\n });\n</code></pre>\n"},"renderTo":{"!doc":"<p>Specify the <code>id</code> of the element, a DOM element or an existing Element that this component will be rendered into.</p>\n\n<p><strong>Notes:</strong></p>\n\n<p>Do <em>not</em> use this option if the Component is to be a child item of a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a>.\nIt is the responsibility of the <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a>'s\n<a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout manager</a> to render and manage its child items.</p>\n\n<p>When using this config, a call to <code>render()</code> is not required.</p>\n\n<p>See also: <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">render</a>.</p>\n"},"resizable":{"!doc":"<p>Specify as <code>true</code> to apply a <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Resizer</a> to this Component after rendering.</p>\n\n<p>May also be specified as a config object to be passed to the constructor of <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Resizer</a>\nto override any defaults. By default the Component passes its minimum and maximum size, and uses\n<code><a href=\"#!/api/Ext.resizer.Resizer-cfg-dynamic\" rel=\"Ext.resizer.Resizer-cfg-dynamic\" class=\"docClass\">Ext.resizer.Resizer.dynamic</a>: false</code></p>\n"},"resizeHandles":{"!type":"string","!doc":"<p>A valid <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Ext.resizer.Resizer</a> handles config string. Only applies when resizable = true.</p>\n"},"rtl":{"!type":"bool","!doc":"<p>True to layout this component and its descendants in \"rtl\" (right-to-left) mode.\nCan be explicitly set to false to override a true value inherited from an ancestor.</p>\n\n<p><strong>Defined in override Ext.rtl.Component.</strong></p>\n"},"style":{"!doc":"<p>A custom style specification to be applied to this component's Element. Should be a valid argument to\nExt.dom.Element.applyStyles.</p>\n\n<pre><code>new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({\n    title: 'Some Title',\n    renderTo: Ext.getBody(),\n    width: 400, height: 300,\n    layout: 'form',\n    items: [{\n        xtype: 'textarea',\n        style: {\n            width: '95%',\n            marginBottom: '10px'\n        }\n    },\n    new <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>({\n        text: 'Send',\n        minWidth: '100',\n        style: {\n            marginBottom: '10px'\n        }\n    })\n    ]\n});\n</code></pre>\n"},"toFrontOnShow":{"!type":"bool","!doc":"<p>True to automatically call <a href=\"#!/api/Ext.Component-method-toFront\" rel=\"Ext.Component-method-toFront\" class=\"docClass\">toFront</a> when the <a href=\"#!/api/Ext.Component-method-show\" rel=\"Ext.Component-method-show\" class=\"docClass\">show</a> method is called on an already visible,\nfloating component.</p>\n"},"tpl":{"!doc":"<p>An Ext.Template, Ext.XTemplate or an array of strings to form an Ext.XTemplate. Used in\nconjunction with the <code><a href=\"#!/api/Ext.Component-cfg-data\" rel=\"Ext.Component-cfg-data\" class=\"docClass\">data</a></code> and <code><a href=\"#!/api/Ext.Component-cfg-tplWriteMode\" rel=\"Ext.Component-cfg-tplWriteMode\" class=\"docClass\">tplWriteMode</a></code> configurations.</p>\n"},"tplWriteMode":{"!type":"string","!doc":"<p>The Ext.(X)Template method to use when updating the content area of the Component.\nSee <code>Ext.XTemplate.overwrite</code> for information on default mode.</p>\n"},"ui":{"!type":"string","!doc":"<p>A UI style for a component.</p>\n"},"uiCls":{"!doc":"<p>An array of of <code>classNames</code> which are currently applied to this component.</p>\n"},"weight":{"!type":"number","!doc":"<p>A value to control how Components are laid out in a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Border</a> layout or as docked items.</p>\n\n<p>In a Border layout, this can control how the regions (not the center) region lay out if the west or east take full height\nor if the north or south region take full width. Also look at the <a href=\"#!/api/Ext.layout.container.Border-cfg-regionWeights\" rel=\"Ext.layout.container.Border-cfg-regionWeights\" class=\"docClass\">Ext.layout.container.Border.regionWeights</a> on the Border layout. An example to show how you can\ntake control of this is:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.container.Viewport\" rel=\"Ext.container.Viewport\" class=\"docClass\">Ext.container.Viewport</a>', {\n    layout      : 'border',\n    defaultType : 'panel',\n    items       : [\n        {\n            region : 'north',\n            title  : 'North',\n            height : 100\n        },\n        {\n            region : 'south',\n            title  : 'South',\n            height : 100,\n            weight : -25\n        },\n        {\n            region : 'west',\n            title  : 'West',\n            width  : 200,\n            weight : 15\n        },\n        {\n            region : 'east',\n            title  : 'East',\n            width  : 200\n        },\n        {\n            region : 'center',\n            title  : 'center'\n        }\n    ]\n});\n</code></pre>\n\n<p>If docked items, the weight will order how the items are laid out. Here is an example to put a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Ext.toolbar.Toolbar</a> above\na <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>'s header:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    renderTo    : document.body,\n    width       : 300,\n    height      : 300,\n    title       : 'Panel',\n    html        : 'Panel Body',\n    dockedItems : [\n        {\n            xtype : 'toolbar',\n            items : [\n                {\n                    text : 'Save'\n                }\n            ]\n        },\n        {\n            xtype  : 'toolbar',\n            weight : -10,\n            items  : [\n                {\n                    text : 'Remove'\n                }\n            ]\n        }\n    ]\n});\n</code></pre>\n"},"width":{"!type":"number","!doc":"<p>The width of this component in pixels.</p>\n"},"xtype":{"!doc":"<p>This property provides a shorter alternative to creating objects than using a full\nclass name. Using <code>xtype</code> is the most common way to define component instances,\nespecially in a container. For example, the items in a form containing text fields\ncould be created explicitly like so:</p>\n\n<pre><code> items: [\n     Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n         fieldLabel: 'Foo'\n     }),\n     Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n         fieldLabel: 'Bar'\n     }),\n     Ext.create('<a href=\"#!/api/Ext.form.field.Number\" rel=\"Ext.form.field.Number\" class=\"docClass\">Ext.form.field.Number</a>', {\n         fieldLabel: 'Num'\n     })\n ]\n</code></pre>\n\n<p>But by using <code>xtype</code>, the above becomes:</p>\n\n<pre><code> items: [\n     {\n         xtype: 'textfield',\n         fieldLabel: 'Foo'\n     },\n     {\n         xtype: 'textfield',\n         fieldLabel: 'Bar'\n     },\n     {\n         xtype: 'numberfield',\n         fieldLabel: 'Num'\n     }\n ]\n</code></pre>\n\n<p>When the <code>xtype</code> is common to many items, <a href=\"#!/api/Ext.container.Container-cfg-defaultType\" rel=\"Ext.container.Container-cfg-defaultType\" class=\"docClass\">Ext.container.Container.defaultType</a>\nis another way to specify the <code>xtype</code> for all items that don't have an explicit <code>xtype</code>:</p>\n\n<pre><code> defaultType: 'textfield',\n items: [\n     { fieldLabel: 'Foo' },\n     { fieldLabel: 'Bar' },\n     { fieldLabel: 'Num', xtype: 'numberfield' }\n ]\n</code></pre>\n\n<p>Each member of the <code>items</code> array is now just a \"configuration object\". These objects\nare used to create and configure component instances. A configuration object can be\nmanually used to instantiate a component using Ext.widget:</p>\n\n<pre><code> var text1 = Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n     fieldLabel: 'Foo'\n });\n\n // or alternatively:\n\n var text1 = Ext.widget({\n     xtype: 'textfield',\n     fieldLabel: 'Foo'\n });\n</code></pre>\n\n<p>This conversion of configuration objects into instantiated components is done when\na container is created as part of its {Ext.container.AbstractContainer#initComponent}\nprocess. As part of the same process, the <code>items</code> array is converted from its raw\narray form into a Ext.util.MixedCollection instance.</p>\n\n<p>You can define your own <code>xtype</code> on a custom <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">component</a> by specifying\nthe <code>xtype</code> property in Ext.define. For example:</p>\n\n<pre><code>Ext.define('MyApp.PressMeButton', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n    xtype: 'pressmebutton',\n    text: 'Press Me'\n});\n</code></pre>\n\n<p>Care should be taken when naming an <code>xtype</code> in a custom component because there is\na single, shared scope for all xtypes. Third part components should consider using\na prefix to avoid collisions.</p>\n\n<pre><code>Ext.define('Foo.form.CoolButton', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n    xtype: 'ux-coolbutton',\n    text: 'Cool!'\n});\n</code></pre>\n\n<p>See <a href=\"#!/api/Ext.enums.Widget\" rel=\"Ext.enums.Widget\" class=\"docClass\">Ext.enums.Widget</a> for list of all available xtypes.</p>\n"},"$configPrefixed":{"!type":"bool","!doc":"<p>We do not want \"_hidden\" style backing properties.</p>\n"},"$configStrict":{"!type":"bool","!doc":"<p>We also want non-config system properties to go to the instance.</p>\n"},"_asLayoutRoot":{},"_isLayoutRoot":{"!type":"bool","!doc":"<p>Setting this property to <code>true</code> causes the <a href=\"#!/api/Ext.Component-method-isLayoutRoot\" rel=\"Ext.Component-method-isLayoutRoot\" class=\"docClass\">isLayoutRoot</a> method to return\n<code>true</code> and stop the search for the top-most component for a layout.</p>\n"},"allowDomMove":{"!type":"bool","!doc":"<p>private</p>\n"},"autoGenId":{"!type":"bool","!doc":"<p><code>true</code> indicates an <code>id</code> was auto-generated rather than provided by configuration.</p>\n"},"borderBoxCls":{"!type":"string","!doc":"<p>private</p>\n"},"componentLayoutCounter":{"!type":"number","!doc":"<p>The number of component layout calls made on this object.</p>\n"},"contentPaddingProperty":{"!type":"string","!doc":"<p>The name of the padding property that is used by the layout to manage\npadding.  See <a href=\"#!/api/Ext.layout.container.Auto-property-managePadding\" rel=\"Ext.layout.container.Auto-property-managePadding\" class=\"docClass\">managePadding</a></p>\n"},"deferLayouts":{"!type":"bool","!doc":"<p>private</p>\n"},"deprecated":{"!doc":"<p>private</p>\n"},"disabledRe":{"!doc":"<p>http://www.w3.org/TR/html5/disabled-elements.html</p>\n"},"floatParent":{"!doc":"<p><strong>Only present for <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Components which were inserted as child items of Containers.</strong></p>\n\n<p>There are other similar relationships such as the <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">button</a> which activates a <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a>, or the\n<a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">menu item</a> which activated a <a href=\"#!/api/Ext.menu.Item-cfg-menu\" rel=\"Ext.menu.Item-cfg-menu\" class=\"docClass\">submenu</a>, or the\n<a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">column header</a> which activated the column menu.</p>\n\n<p>These differences are abstracted away by the <a href=\"#!/api/Ext.Component-method-up\" rel=\"Ext.Component-method-up\" class=\"docClass\">up</a> method.</p>\n\n<p>Floating Components that are programatically <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">rendered</a> will not have a <code>floatParent</code>\nproperty.</p>\n\n<p>See <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> and <a href=\"#!/api/Ext.Component-property-zIndexManager\" rel=\"Ext.Component-property-zIndexManager\" class=\"docClass\">zIndexManager</a></p>\n"},"frameSize":{"!doc":"<p>Indicates the width of any framing elements which were added within the encapsulating\nelement to provide graphical, rounded borders. See the <a href=\"#!/api/Ext.Component-cfg-frame\" rel=\"Ext.Component-cfg-frame\" class=\"docClass\">frame</a> config. This\nproperty is <code>null</code> if the component is not framed.</p>\n\n<p>This is an object containing the frame width in pixels for all four sides of the\nComponent containing the following properties:</p>\n"},"horizontalPosProp":{"!type":"string","!doc":"<p>private</p>\n"},"isComponent":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Component, or subclass thereof.</p>\n"},"layoutSuspendCount":{"!type":"number","!doc":"<p>private</p>\n"},"maskOnDisable":{"!type":"bool","!doc":"<p>This is an internal flag that you use when creating custom components. By default this is set to <code>true</code> which means\nthat every component gets a mask when it's disabled. Components like FieldContainer, FieldSet, Field, Button, Tab\noverride this property to <code>false</code> since they want to implement custom disable logic.</p>\n"},"nonMaskableRe":{},"offsetsCls":{"!type":"string","!doc":"<p>private</p>\n"},"ownerCt":{"!doc":"<p>This Component's owner <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a> (is set automatically\nwhen this Component is added to a Container).</p>\n\n<p><em>Important.</em> This is not a universal upwards navigation pointer. It indicates the Container which owns and manages\nthis Component if any. There are other similar relationships such as the <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">button</a> which activates a <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a>, or the\n<a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">menu item</a> which activated a <a href=\"#!/api/Ext.menu.Item-cfg-menu\" rel=\"Ext.menu.Item-cfg-menu\" class=\"docClass\">submenu</a>, or the\n<a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">column header</a> which activated the column menu.</p>\n\n<p>These differences are abstracted away by the <a href=\"#!/api/Ext.Component-method-up\" rel=\"Ext.Component-method-up\" class=\"docClass\">up</a> method.</p>\n\n<p><strong>Note</strong>: to access items within the Container see <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">itemId</a>.</p>\n"},"rendered":{"!type":"bool","!doc":"<p>Indicates whether or not the component has been rendered.</p>\n"},"rootCls":{"!type":"string","!doc":"<p>private</p>\n"},"scrollFlags":{"!doc":"<p>An object property which provides unified information as to which dimensions are scrollable based upon\nthe <a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a>, <a href=\"#!/api/Ext.Component-cfg-overflowX\" rel=\"Ext.Component-cfg-overflowX\" class=\"docClass\">overflowX</a> and <a href=\"#!/api/Ext.Component-cfg-overflowY\" rel=\"Ext.Component-cfg-overflowY\" class=\"docClass\">overflowY</a> settings (And for <em>views</em> of trees and grids, the owning panel's <a href=\"#!/api/Ext.panel.Table-cfg-scroll\" rel=\"Ext.panel.Table-cfg-scroll\" class=\"docClass\">scroll</a> setting).</p>\n\n<p>Note that if you set overflow styles using the <a href=\"#!/api/Ext.Component-cfg-style\" rel=\"Ext.Component-cfg-style\" class=\"docClass\">style</a> config or <a href=\"#!/api/Ext.panel.Panel-cfg-bodyStyle\" rel=\"Ext.panel.Panel-cfg-bodyStyle\" class=\"docClass\">bodyStyle</a> config, this object does not include that information;\nit is best to use <a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a>, <a href=\"#!/api/Ext.Component-cfg-overflowX\" rel=\"Ext.Component-cfg-overflowX\" class=\"docClass\">overflowX</a> and <a href=\"#!/api/Ext.Component-cfg-overflowY\" rel=\"Ext.Component-cfg-overflowY\" class=\"docClass\">overflowY</a> if you need to access these flags.</p>\n\n<p>This object has the following properties:</p>\n"},"scrollerCls":{"!type":"string","!doc":"<p>private</p>\n"},"scrollerSelector":{"!type":"string"},"synthetic":{"!type":"bool","!doc":"<p>This property is <code>true</code> if the component was created internally by the framework\nand is not explicitly user-defined. This is set for such things as <code>Splitter</code>\ninstances managed by <code>border</code> and <code>box</code> layouts.</p>\n"},"validIdRe":{},"zIndexManager":{"!doc":"<p>Only present for <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Components after they have been rendered.</p>\n\n<p>A reference to the ZIndexManager which is managing this Component's z-index.</p>\n\n<p>The <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a> maintains a stack of floating Component z-indices, and also provides\na single modal mask which is insert just beneath the topmost visible modal floating Component.</p>\n\n<p>Floating Components may be <a href=\"#!/api/Ext.Component-method-toFront\" rel=\"Ext.Component-method-toFront\" class=\"docClass\">brought to the front</a> or <a href=\"#!/api/Ext.Component-method-toBack\" rel=\"Ext.Component-method-toBack\" class=\"docClass\">sent to the back</a> of the\nz-index stack.</p>\n\n<p>This defaults to the global <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a> for floating Components that are\nprogramatically <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">rendered</a>.</p>\n\n<p>For <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Components which are added to a Container, the ZIndexManager is acquired from the first\nancestor Container found which is floating. If no floating ancestor is found, the global <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a> is\nused.</p>\n\n<p>See <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> and <a href=\"#!/api/Ext.Component-property-zIndexParent\" rel=\"Ext.Component-property-zIndexParent\" class=\"docClass\">zIndexParent</a></p>\n"},"zIndexParent":{"!doc":"<p>Only present for <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Components which were inserted as child items of Containers, and which have a floating\nContainer in their containment ancestry.</p>\n\n<p>For <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Components which are child items of a Container, the zIndexParent will be a floating\nancestor Container which is responsible for the base z-index value of all its floating descendants. It provides\na <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a> which provides z-indexing services for all its descendant floating\nComponents.</p>\n\n<p>Floating Components that are programatically <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">rendered</a> will not have a <code>zIndexParent</code>\nproperty.</p>\n\n<p>For example, the dropdown <a href=\"#!/api/Ext.view.BoundList\" rel=\"Ext.view.BoundList\" class=\"docClass\">BoundList</a> of a ComboBox which is in a Window will have the\nWindow as its <code>zIndexParent</code>, and will always show above that Window, wherever the Window is placed in the z-index stack.</p>\n\n<p>See <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> and <a href=\"#!/api/Ext.Component-property-zIndexManager\" rel=\"Ext.Component-property-zIndexManager\" class=\"docClass\">zIndexManager</a></p>\n"},"addClass":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Adds a CSS class to the top level element representing this component.</p>\n"},"addCls":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Adds a CSS class to the top level element representing this component.</p>\n"},"addClsWithUI":{"!type":"fn(classes: ?, skip?: ?) -> !this","!doc":"<p>Adds a <code>cls</code> to the <code>uiCls</code> array, which will also call <a href=\"#!/api/Ext.Component-method-addUIClsToElement\" rel=\"Ext.Component-method-addUIClsToElement\" class=\"docClass\">addUIClsToElement</a> and adds to all elements of this\ncomponent.</p>\n"},"addFocusListener":{"!type":"fn() -> !this","!doc":"<p>Sets up the focus listener on this Component's <a href=\"#!/api/Ext.Component-method-getFocusEl\" rel=\"Ext.Component-method-getFocusEl\" class=\"docClass\">focusEl</a> if it has one.</p>\n\n<p>Form Components which must implicitly participate in tabbing order usually have a naturally focusable\nelement as their <a href=\"#!/api/Ext.Component-method-getFocusEl\" rel=\"Ext.Component-method-getFocusEl\" class=\"docClass\">focusEl</a>, and it is the DOM event of that receiving focus which drives\nthe Component's <code>onFocus</code> handling, and the DOM event of it being blurred which drives the <code>onBlur</code> handling.</p>\n\n<p>If the <a href=\"#!/api/Ext.Component-method-getFocusEl\" rel=\"Ext.Component-method-getFocusEl\" class=\"docClass\">focusEl</a> is <strong>not</strong> naturally focusable, then the listeners are only added\nif the <a href=\"#!/api/Ext.FocusManager\" rel=\"Ext.FocusManager\" class=\"docClass\">FocusManager</a> is enabled.</p>\n"},"addListener":{"!type":"fn(element: ?, listeners: ?, scope: ?, options: ?) -> !this"},"addPlugin":{"!type":"fn(plugin: ?) -> !this","!doc":"<p>Adds a plugin. May be called at any time in the component's lifecycle.</p>\n"},"addPropertyToState":{"!type":"fn(state: ?, propName: ?, value?: ?) -> bool","!doc":"<p>Save a property to the given state object if it is not its default or configured\nvalue.</p>\n"},"addUIClsToElement":{"!type":"fn(uiCls: ?) -> !this","!doc":"<p>Method which adds a specified UI + <code>uiCls</code> to the components element. Can be overridden\nto add the UI to more than just the component's element.</p>\n"},"addUIToElement":{"!type":"fn() -> !this","!doc":"<p>Method which adds a specified UI to the components element.</p>\n"},"adjustPosition":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>private</p>\n"},"afterComponentLayout":{"!type":"fn(width: ?, height: ?, oldWidth: ?, oldHeight: ?) -> !this","!doc":"<p>Called by the layout system after the Component has been laid out.</p>\n"},"afterHide":{"!type":"fn(callback?: ?, scope?: ?) -> !this","!doc":"<p>Invoked after the Component has been hidden.</p>\n\n<p>Gets passed the same <code>callback</code> and <code>scope</code> parameters that <a href=\"#!/api/Ext.Component-method-onHide\" rel=\"Ext.Component-method-onHide\" class=\"docClass\">onHide</a> received.</p>\n"},"afterSetPosition":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Template method called after a Component has been positioned.</p>\n"},"afterShow":{"!type":"fn(animateTarget?: ?, callback?: ?, scope?: ?) -> !this","!doc":"<p>Invoked after the Component is shown (after <a href=\"#!/api/Ext.Component-method-onShow\" rel=\"Ext.Component-method-onShow\" class=\"docClass\">onShow</a> is called).</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.Component-event-show\" rel=\"Ext.Component-event-show\" class=\"docClass\">show</a>.</p>\n"},"animate":{"!type":"fn(animObj: ?) -> ?","!doc":"<p>Performs custom animation on this object.</p>\n\n<p>This method is applicable to both the <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a> class and the Sprite\nclass. It performs animated transitions of certain properties of this object over a specified timeline.</p>\n\n<h3>Animating a <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a></h3>\n\n<p>When animating a Component, the following properties may be specified in <code>from</code>, <code>to</code>, and <code>keyframe</code> objects:</p>\n\n<ul>\n<li><p><code>x</code> - The Component's page X position in pixels.</p></li>\n<li><p><code>y</code> - The Component's page Y position in pixels</p></li>\n<li><p><code>left</code> - The Component's <code>left</code> value in pixels.</p></li>\n<li><p><code>top</code> - The Component's <code>top</code> value in pixels.</p></li>\n<li><p><code>width</code> - The Component's <code>width</code> value in pixels.</p></li>\n<li><p><code>height</code> - The Component's <code>height</code> value in pixels.</p></li>\n<li><p><code>dynamic</code> - Specify as true to update the Component's layout (if it is a Container) at every frame of the animation.\n<em>Use sparingly as laying out on every intermediate size change is an expensive operation.</em></p></li>\n</ul>\n\n\n<p>For example, to animate a Window to a new size, ensuring that its internal layout and any shadow is correct:</p>\n\n<pre><code>myWindow = Ext.create('<a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>', {\n    title: 'Test Component animation',\n    width: 500,\n    height: 300,\n    layout: {\n        type: 'hbox',\n        align: 'stretch'\n    },\n    items: [{\n        title: 'Left: 33%',\n        margin: '5 0 5 5',\n        flex: 1\n    }, {\n        title: 'Left: 66%',\n        margin: '5 5 5 5',\n        flex: 2\n    }]\n});\nmyWindow.show();\nmyWindow.header.el.on('click', function() {\n    myWindow.animate({\n        to: {\n            width: (myWindow.getWidth() == 500) ? 700 : 500,\n            height: (myWindow.getHeight() == 300) ? 400 : 300\n        }\n    });\n});\n</code></pre>\n\n<p>For performance reasons, by default, the internal layout is only updated when the Window reaches its final <code>\"to\"</code>\nsize. If dynamic updating of the Window's child Components is required, then configure the animation with\n<code>dynamic: true</code> and the two child items will maintain their proportions during the animation.</p>\n"},"applyData":{"!type":"fn(data: ?) -> !this"},"beforeBlur":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method to do any pre-blur processing.</p>\n"},"beforeComponentLayout":{"!type":"fn(adjWidth: ?, adjHeight: ?) -> !this","!doc":"<p>Occurs before <code>componentLayout</code> is run. Returning <code>false</code> from this method will prevent the <code>componentLayout</code> from\nbeing executed.</p>\n"},"beforeFocus":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method to do any pre-focus processing.</p>\n"},"beforeInitConfig":{"!type":"fn() -> !this"},"beforeLayout":{"!type":"fn() -> !this","!doc":"<p>Occurs before componentLayout is run. In previous releases, this method could\nreturn <code>false</code> to prevent its layout but that is not supported in Ext JS 4.1 or\nhigher. This method is simply a notification of the impending layout to give the\ncomponent a chance to adjust the DOM. Ideally, DOM reads should be avoided at this\ntime to reduce expensive document reflows.</p>\n"},"beforeSetPosition":{"!type":"fn(x: ?, y: ?, animate: ?) -> !this","!doc":"<p>Template method called before a Component is positioned.</p>\n\n<p>Ensures that the position is adjusted so that the Component is constrained if so configured.</p>\n"},"beforeShow":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is shown.</p>\n"},"bubble":{"!type":"fn(fn: ?, scope?: ?, args?: ?) -> ?","!doc":"<p>Bubbles up the component/container hierarchy, calling the specified function with each component. The scope\n(<em>this</em>) of function call will be the scope provided or the current component. The arguments to the function will\nbe the args provided or the current component. If the function returns false at any point, the bubble is stopped.</p>\n"},"cancelFocus":{"!type":"fn() -> !this","!doc":"<p>Cancel any deferred focus on this component</p>\n"},"cloneConfig":{"!type":"fn(overrides: ?) -> ?","!doc":"<p>Clone the current component using the original config values passed into this instance by default.</p>\n"},"constructPlugin":{"!type":"fn(ptype: ?) -> !this"},"constructPlugins":{"!type":"fn() -> !this","!doc":"<p>Returns an array of fully constructed plugin instances. This converts any configs into their\nappropriate instances.</p>\n\n<p>It does not mutate the plugins array. It creates a new array.</p>\n"},"destroy":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component is <a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a>ed.</p>\n"},"doComponentLayout":{"!type":"fn() -> ?","!doc":"<p>This method needs to be called whenever you change something on this component that\nrequires the Component's layout to be recalculated.</p>\n"},"findComponentByElement":{"!type":"fn(node: ?) -> !this","!doc":"<p>Walk the DOM tree upwards and find the Component these elements belong to.</p>\n"},"findParentBy":{"!type":"fn(fn: ?) -> ?","!doc":"<p>Find a container above this component at any level by a custom function. If the passed function returns true, the\ncontainer will be returned.</p>\n\n<p>See also the <a href=\"#!/api/Ext.Component-method-up\" rel=\"Ext.Component-method-up\" class=\"docClass\">up</a> method.</p>\n"},"findParentByType":{"!type":"fn(xtype: ?) -> ?","!doc":"<p>Find a container above this component at any level by xtype or class</p>\n\n<p>See also the <a href=\"#!/api/Ext.Component-method-up\" rel=\"Ext.Component-method-up\" class=\"docClass\">up</a> method.</p>\n"},"findPlugin":{"!type":"fn(ptype: ?) -> ?","!doc":"<p>Retrieves plugin from this component's collection by its <code>ptype</code>.</p>\n"},"fireHierarchyEvent":{"!type":"fn(eventName: ?) -> !this","!doc":"<p>This method fires an event on <code>Ext.GlobalEvents</code> allowing interested parties to know\nof certain critical events for this component. This is done globally because the\n(few) listeners can immediately receive the event rather than bubbling the event\nonly to reach the top and have no listeners.</p>\n\n<p>The main usage for these events is to do with floating components. For example, the\nload mask is a floating component. The component it is masking may be inside several\ncontainers. As such, they need to know when component is hidden, either directly, or\nvia a parent container being hidden. To do this they subscribe to these events and\nfilter out the appropriate container.</p>\n\n<p>This functionality is contained in Component (as opposed to Container) because a\nComponent can be the ownerCt for a floating component (loadmask), and the loadmask\nneeds to know when its owner is shown/hidden so that its hidden state can be\nsynchronized.</p>\n"},"focus":{"!type":"fn(this: ?, The: ?, eOpts: ?)","!doc":"<p>Fires when this Component receives focus.</p>\n"},"forceComponentLayout":{"!type":"fn(options?: ?) -> !this","!doc":"<p>Updates this component's layout. If this update affects this components <a href=\"#!/api/Ext.Component-property-ownerCt\" rel=\"Ext.Component-property-ownerCt\" class=\"docClass\">ownerCt</a>,\nthat component's <code>updateLayout</code> method will be called to perform the layout instead.\nOtherwise, just this component (and its child items) will layout.</p>\n"},"getActiveComponent":{"!type":"fn() -> ?","!doc":"<p>Return the currently active (focused) Component</p>\n"},"getAnchorToXY":{"!type":"fn(el: ?, anchor: ?, local: ?, mySize: ?) -> !this","!doc":"<p><editor-fold desc=\"Positionable Methods\"></editor></p>\n\n<hr />\n\n<p> Begin Positionable methods</p>\n\n<hr />\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"getAnimateTarget":{"!type":"fn(target: ?) -> !this"},"getAutoId":{"!type":"fn() -> !this"},"getBorderPadding":{"!type":"fn() -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"getBubbleTarget":{"!type":"fn() -> !this","!doc":"<p>Implements an upward event bubbling policy. By default a Component bubbles events up to its reference owner.</p>\n\n<p>Component subclasses may implement a different bubbling strategy by overriding this method.</p>\n"},"getComponentByElement":{"!type":"fn(el: ?) -> ?","!doc":"<p>Find a Component that the given Element belongs to.</p>\n"},"getComponentLayout":{"!type":"fn() -> !this"},"getData":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.Component-cfg-data\" rel=\"Ext.Component-cfg-data\" class=\"docClass\">data</a>.</p>\n"},"getEl":{"!type":"fn() -> ?","!doc":"<p>Retrieves the top level element representing this component.</p>\n"},"getFocusEl":{"!type":"fn() -> ?","!doc":"<p>Returns the focus holder element associated with this Component. At the Component base class level, this function returns <code>undefined</code>.</p>\n\n<p>Subclasses which use embedded focusable elements (such as Window, Field and Button) should override this\nfor use by the <a href=\"#!/api/Ext.Component-method-focus\" rel=\"Ext.Component-method-focus\" class=\"docClass\">focus</a> method.</p>\n\n<p>Containers which need to participate in the <a href=\"#!/api/Ext.FocusManager\" rel=\"Ext.FocusManager\" class=\"docClass\">FocusManager</a>'s navigation and Container focusing scheme also\nneed to return a <code>focusEl</code>, although focus is only listened for in this case if the <a href=\"#!/api/Ext.FocusManager\" rel=\"Ext.FocusManager\" class=\"docClass\">FocusManager</a> is <a href=\"#!/api/Ext.FocusManager-method-enable\" rel=\"Ext.FocusManager-method-enable\" class=\"docClass\">enable</a>d.</p>\n"},"getHeight":{"!type":"fn() -> number","!doc":"<p>Gets the current height of the component's underlying element.</p>\n"},"getId":{"!type":"fn() -> string","!doc":"<p>Retrieves the <code>id</code> of this component. Will auto-generate an <code>id</code> if one has not already been set.</p>\n"},"getItemId":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">itemId</a> assigned to this component, or when that\nis not set, returns the value of <a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">id</a>.</p>\n"},"getLoader":{"!type":"fn() -> ?","!doc":"<p>Gets the <a href=\"#!/api/Ext.ComponentLoader\" rel=\"Ext.ComponentLoader\" class=\"docClass\">Ext.ComponentLoader</a> for this Component.</p>\n"},"getLocalX":{"!type":"fn() -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"getLocalXY":{"!type":"fn() -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"getLocalY":{"!type":"fn() -> !this"},"getMaskTarget":{"!type":"fn() -> !this","!doc":"<p>Returns the element which is masked by the <a href=\"#!/api/Ext.Component-method-mask\" rel=\"Ext.Component-method-mask\" class=\"docClass\">mask</a> method, or into which the <a href=\"#!/api/Ext.Component-method-setLoading\" rel=\"Ext.Component-method-setLoading\" class=\"docClass\">LoadMask</a> is rendered into.</p>\n\n<p>The default implementation uses the <a href=\"#!/api/Ext.Component-cfg-maskElement\" rel=\"Ext.Component-cfg-maskElement\" class=\"docClass\">maskElement</a> configuration to access the Component's child element by name. By default, <a href=\"#!/api/Ext.Component-cfg-maskElement\" rel=\"Ext.Component-cfg-maskElement\" class=\"docClass\">maskElement</a>\nis <code>null</code> which means that <code>null</code> is returned from this method indicating that the mask needs to be rendered into the document because\ncomponent structure should not be contaminated by mask elements.</p>\n\n<p>Some subclasses may override this method if they have knowledge about external structures where a mask could usefully be rendered.</p>\n\n<p>For example a <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">GridView</a> will request that its owning <a href=\"#!/api/Ext.panel.Table\" rel=\"Ext.panel.Table\" class=\"docClass\">GridPanel</a> be masked. The\nGridPanel will have its own implementation of <code>getMaskTarget</code> which will return the element dicated by its own <a href=\"#!/api/Ext.Component-cfg-maskElement\" rel=\"Ext.Component-cfg-maskElement\" class=\"docClass\">maskElement</a>\nPanels use <code>\"el\"</code> as their <a href=\"#!/api/Ext.Component-cfg-maskElement\" rel=\"Ext.Component-cfg-maskElement\" class=\"docClass\">maskElement</a> by default, but that could be overridden to be <code>\"body\"</code> to leave toolbars and the header\nmouse-accessible.</p>\n"},"getOverflowEl":{"!type":"fn() -> !this","!doc":"<p>Get an el for overflowing, defaults to the target el</p>\n"},"getOverflowStyle":{"!type":"fn() -> !this","!doc":"<p>Returns the CSS style object which will set the Component's scroll styles. This must be applied\nto the <a href=\"#!/api/Ext.Component-method-getTargetEl\" rel=\"Ext.Component-method-getTargetEl\" class=\"docClass\">target element</a>.</p>\n"},"getOwningBorderContainer":{"!type":"fn() -> ?","!doc":"<p>Returns the owning container if that container uses <code>border</code> layout. Otherwise\nthis method returns <code>null</code>.</p>\n\n<p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>\n"},"getOwningBorderLayout":{"!type":"fn() -> ?","!doc":"<p>Returns the owning <code>border</code> (<code><a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Ext.layout.container.Border</a></code>) instance if there is\none. Otherwise this method returns <code>null</code>.</p>\n\n<p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>\n"},"getPlugin":{"!type":"fn(pluginId: ?) -> ?","!doc":"<p>Retrieves a plugin from this component's collection by its <code>pluginId</code>.</p>\n"},"getPlugins":{"!type":"fn() -> !this","!doc":"<p>Returns an array of current fully constructed plugin instances.</p>\n"},"getPosition":{"!type":"fn(local?: ?) -> ?","!doc":"<p>Gets the current XY position of the component's underlying element.</p>\n"},"getScrollX":{"!type":"fn() -> number","!doc":"<p>Returns the \"x\" scroll position for this component.  Only applicable for components\nthat have been configured with <code><a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a></code> or <code><a href=\"#!/api/Ext.Component-cfg-overflowX\" rel=\"Ext.Component-cfg-overflowX\" class=\"docClass\">overflowX</a></code>.</p>\n"},"getScrollY":{"!type":"fn() -> number","!doc":"<p>Returns the \"y\" scroll position for this component.  Only applicable for components\nthat have been configured with <code><a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a></code> or <code><a href=\"#!/api/Ext.Component-cfg-overflowY\" rel=\"Ext.Component-cfg-overflowY\" class=\"docClass\">overflowY</a></code>.</p>\n"},"getSize":{"!type":"fn(contentSize?: ?) -> ?","!doc":"<p>Gets the current size of the component's underlying element.</p>\n"},"getSizeModel":{"!type":"fn(ownerCtSizeModel: ?) -> ?","!doc":"<p>Returns an object that describes how this component's width and height are managed.\nAll of these objects are shared and should not be modified.</p>\n"},"getState":{"!type":"fn() -> ?","!doc":"<p>The supplied default state gathering method for the Component class.</p>\n\n<p>This method returns dimension settings such as <code>flex</code>, <code>anchor</code>, <code>width</code> and <code>height</code> along with <code>collapsed</code>\nstate.</p>\n\n<p>Subclasses which implement more complex state should call the superclass's implementation, and apply their state\nto the result if this basic state is to be saved.</p>\n\n<p>Note that Component state will only be saved if the Component has a <a href=\"#!/api/Ext.Component-cfg-stateId\" rel=\"Ext.Component-cfg-stateId\" class=\"docClass\">stateId</a> and there as a StateProvider\nconfigured for the document.</p>\n"},"getTargetEl":{"!type":"fn() -> !this","!doc":"<p>This is used to determine where to insert the 'html', 'contentEl' and 'items' in this component.</p>\n"},"getTdType":{"!type":"fn() -> !this","!doc":"<p>Partner method to <a href=\"#!/api/Ext.Component-method-getTdCls\" rel=\"Ext.Component-method-getTdCls\" class=\"docClass\">getTdCls</a>.</p>\n\n<p>Returns the base type for the component. Defaults to return <code>this.xtype</code>, but\nAll derived classes of <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">TextField</a> can return the type 'textfield',\nand all derived classes of <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Button</a> can return the type 'button'</p>\n"},"getTpl":{"!type":"fn(name: ?) -> !this"},"getWidth":{"!type":"fn() -> number","!doc":"<p>Gets the current width of the component's underlying element.</p>\n"},"getX":{"!type":"fn() -> !this"},"getXType":{"!type":"fn() -> string","!doc":"<p>Gets the xtype for this component as registered with Ext.ComponentManager. For a list of all available\nxtypes, see the <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> header. Example usage:</p>\n\n<pre><code>var t = new <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>();\nalert(t.getXType());  // alerts 'textfield'\n</code></pre>\n"},"getXTypes":{"!type":"fn() -> string","!doc":"<p>Returns this Component's xtype hierarchy as a slash-delimited string. For a list of all available xtypes, see the\n<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> header.</p>\n\n<p><strong>If using your own subclasses, be aware that a Component must register its own xtype to participate in\ndetermination of inherited xtypes.</strong></p>\n\n<p>Example usage:</p>\n\n<pre class='inline-example '><code>var t = new <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>();\nalert(t.getXTypes());  // alerts 'component/field/textfield'\n</code></pre>\n"},"getXY":{"!type":"fn() -> !this"},"getY":{"!type":"fn() -> !this"},"hasCls":{"!type":"fn(className: ?) -> bool","!doc":"<p>Checks if the specified CSS class exists on this element's DOM node.</p>\n"},"hasUICls":{"!type":"fn(cls: ?) -> !this","!doc":"<p>Checks if there is currently a specified <code>uiCls</code>.</p>\n"},"hide":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component is hidden. Fires after the component is hidden when calling the <a href=\"#!/api/Ext.Component-method-hide\" rel=\"Ext.Component-method-hide\" class=\"docClass\">hide</a>\nmethod.</p>\n"},"initBorderRegion":{"!type":"fn() -> !this","!doc":"<p>This method is called by the <code><a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Ext.layout.container.Border</a></code> class when instances are\nadded as regions to the layout. Since it is valid to add any component to a border\nlayout as a region, this method must be added to <code><a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a></code> but is only ever\ncalled when that component is owned by a <code>border</code> layout.</p>\n\n<p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>Initialize any events on this component</p>\n"},"initInheritedState":{"!type":"fn(inheritedState: ?) -> !this","!doc":"<p>Called by <code>getInherited</code> to initialize the inheritedState the first time it is\nrequested.</p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"initPadding":{"!type":"fn(targetEl: ?) -> !this","!doc":"<p>Initializes padding by applying it to the target element, or if the layout manages\npadding ensures that the padding on the target element is \"0\".</p>\n"},"initStyles":{"!type":"fn(targetEl: ?) -> !this","!doc":"<p>Applies padding, margin, border, top, left, height, and width configs to the\nappropriate elements.</p>\n"},"is":{"!type":"fn(selector: ?) -> bool","!doc":"<p>Tests whether this Component matches a ComponentQuery\nselector string.</p>\n"},"isDescendantOf":{"!type":"fn(container: ?) -> bool","!doc":"<p>Determines whether this component is the descendant of a particular container.</p>\n"},"isDisabled":{"!type":"fn() -> bool","!doc":"<p>Method to determine whether this Component is currently disabled.</p>\n"},"isDraggable":{"!type":"fn() -> bool","!doc":"<p>Method to determine whether this Component is draggable.</p>\n"},"isDroppable":{"!type":"fn() -> bool","!doc":"<p>Method to determine whether this Component is droppable.</p>\n"},"isFloating":{"!type":"fn() -> bool","!doc":"<p>Method to determine whether this Component is floating.</p>\n"},"isFocusable":{"!type":"fn() -> !this"},"isHidden":{"!type":"fn() -> bool","!doc":"<p>Method to determine whether this Component is currently set to hidden.</p>\n"},"isHierarchicallyHidden":{"!type":"fn() -> !this"},"isLayoutRoot":{"!type":"fn() -> !this","!doc":"<p>Determines whether this Component is the root of a layout. This returns <code>true</code> if\nthis component can run its layout without assistance from or impact on its owner.\nIf this component cannot run its layout given these restrictions, <code>false</code> is returned\nand its owner will be considered as the next candidate for the layout root.</p>\n\n<p>Setting the <a href=\"#!/api/Ext.Component-property-_isLayoutRoot\" rel=\"Ext.Component-property-_isLayoutRoot\" class=\"docClass\">_isLayoutRoot</a> property to <code>true</code> causes this method to always\nreturn <code>true</code>. This may be useful when updating a layout of a Container which shrink\nwraps content, and you know that it will not change size, and so can safely be the\ntopmost participant in the layout run.</p>\n"},"isLayoutSuspended":{"!type":"fn() -> bool","!doc":"<p>Returns <code>true</code> if layout is suspended for this component. This can come from direct\nsuspension of this component's layout activity (<a href=\"#!/api/Ext.container.Container-cfg-suspendLayout\" rel=\"Ext.container.Container-cfg-suspendLayout\" class=\"docClass\">Ext.Container.suspendLayout</a>) or if one\nof this component's containers is suspended.</p>\n"},"isLocalRtl":{"!type":"fn() -> bool","!doc":"<p>Returns true if this component's local coordinate system is rtl. For normal\ncomponents this equates to the value of isParentRtl().  Floaters are a bit different\nbecause a floater's element can be a childNode of something other than its\nparent component's element.  For floaters we have to read the dom to see if the\ncomponent's element's parentNode has a css direction value of \"rtl\".</p>\n\n<p><strong>Defined in override Ext.rtl.Component.</strong></p>\n"},"isOppositeRootDirection":{"!type":"fn() -> !this","!doc":"<p><strong>Defined in override Ext.rtl.Component.</strong></p>\n"},"isParentRtl":{"!type":"fn() -> bool","!doc":"<p>Returns true if this component's parent container is rtl. Used by rtl positioning\nmethods to determine if the component should be positioned using a right-to-left\ncoordinate system.</p>\n\n<p><strong>Defined in override Ext.rtl.Component.</strong></p>\n"},"isVisible":{"!type":"fn(deep?: ?) -> bool","!doc":"<p>Returns <code>true</code> if this component is visible.</p>\n"},"isXType":{"!type":"fn(xtype: ?, shallow?: ?) -> bool","!doc":"<p>Tests whether or not this Component is of a specific xtype. This can test whether this Component is descended\nfrom the xtype (default) or whether it is directly of the xtype specified (<code>shallow = true</code>).</p>\n\n<p><strong>If using your own subclasses, be aware that a Component must register its own xtype to participate in\ndetermination of inherited xtypes.</strong></p>\n\n<p>For a list of all available xtypes, see the <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> header.</p>\n\n<p>Example usage:</p>\n\n<pre class='inline-example '><code>var t = new <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>();\nvar isText = t.isXType('textfield');        // true\nvar isBoxSubclass = t.isXType('field');       // true, descended from <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a>\nvar isBoxInstance = t.isXType('field', true); // false, not a direct <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a> instance\n</code></pre>\n"},"mask":{"!type":"fn(msg?: ?, msgCls?: ?) -> !this","!doc":"<p>Masks this component with a semi-opaque layer and makes the contents unavailable to clicks.</p>\n\n<p>See <a href=\"#!/api/Ext.Component-method-unmask\" rel=\"Ext.Component-method-unmask\" class=\"docClass\">unmask</a>.</p>\n"},"nextNode":{"!type":"fn(selector?: ?) -> ?","!doc":"<p>Returns the next node in the Component tree in tree traversal order.</p>\n\n<p>Note that this is not limited to siblings, and if invoked upon a node with no matching siblings, will walk the\ntree to attempt to find a match. Contrast with <a href=\"#!/api/Ext.Component-method-nextSibling\" rel=\"Ext.Component-method-nextSibling\" class=\"docClass\">nextSibling</a>.</p>\n"},"nextSibling":{"!type":"fn(selector?: ?) -> ?","!doc":"<p>Returns the next sibling of this Component.</p>\n\n<p>Optionally selects the next sibling which matches the passed ComponentQuery selector.</p>\n\n<p>May also be referred to as <strong><code>next()</code></strong></p>\n\n<p>Note that this is limited to siblings, and if no siblings of the item match, <code>null</code> is returned. Contrast with\n<a href=\"#!/api/Ext.Component-method-nextNode\" rel=\"Ext.Component-method-nextNode\" class=\"docClass\">nextNode</a></p>\n"},"onAdded":{"!type":"fn(container: ?, pos: ?, instanced: ?) -> !this","!doc":"<p>Method to manage awareness of when components are added to their\nrespective Container, firing an <a href=\"#!/api/Ext.Component-event-added\" rel=\"Ext.Component-event-added\" class=\"docClass\">added</a> event. References are\nestablished at add time rather than at render time.</p>\n\n<p>Allows addition of behavior when a Component is added to a\nContainer. At this stage, the Component is in the parent\nContainer's collection of child items. After calling the\nsuperclass's <code>onAdded</code>, the <code>ownerCt</code> reference will be present,\nand if configured with a ref, the <code>refOwner</code> will be set.</p>\n"},"onBlur":{"!type":"fn(e: ?) -> !this","!doc":"<p>private</p>\n"},"onBoxReady":{"!type":"fn(width: ?, height: ?) -> !this","!doc":"<p>Invoked when this component has first achieved size. Occurs after the\n<a href=\"#!/api/Ext.Component-cfg-componentLayout\" rel=\"Ext.Component-cfg-componentLayout\" class=\"docClass\">componentLayout</a> has completed its initial run.</p>\n"},"onDisable":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the disable operation.\nAfter calling the superclass's <code>onDisable</code>, the Component will be disabled.</p>\n"},"onEnable":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the enable operation.\nAfter calling the superclass's <code>onEnable</code>, the Component will be enabled.</p>\n"},"onFocus":{"!type":"fn(e: ?) -> !this","!doc":"<p>private</p>\n"},"onHide":{"!type":"fn(animateTarget?: ?, callback?: ?, scope?: ?) -> !this","!doc":"<p>Allows addition of behavior to the hide operation. After\ncalling the superclass's onHide, the Component will be hidden.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.Component-event-hide\" rel=\"Ext.Component-event-hide\" class=\"docClass\">hide</a>.</p>\n"},"onPosition":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Called after the component is moved, this method is empty by default but can be implemented by any\nsubclass that needs to perform custom logic after a move occurs.</p>\n"},"onRemoved":{"!type":"fn(destroying: ?) -> !this","!doc":"<p>Method to manage awareness of when components are removed from their\nrespective Container, firing a <a href=\"#!/api/Ext.Component-event-removed\" rel=\"Ext.Component-event-removed\" class=\"docClass\">removed</a> event. References are properly\ncleaned up after removing a component from its owning container.</p>\n\n<p>Allows addition of behavior when a Component is removed from\nits parent Container. At this stage, the Component has been\nremoved from its parent Container's collection of child items,\nbut has not been destroyed (It will be destroyed if the parent\nContainer's <code>autoDestroy</code> is <code>true</code>, or if the remove call was\npassed a truthy second parameter). After calling the\nsuperclass's <code>onRemoved</code>, the <code>ownerCt</code> and the <code>refOwner</code> will not\nbe present.</p>\n"},"onResize":{"!type":"fn(width: ?, height: ?, oldWidth: ?, oldHeight: ?) -> !this","!doc":"<p>Allows addition of behavior to the resize operation.</p>\n\n<p>Called when Ext.resizer.Resizer#drag event is fired.</p>\n"},"onShow":{"!type":"fn(animateTarget?: ?, callback?: ?, scope?: ?) -> !this","!doc":"<p>Allows addition of behavior to the show operation. After\ncalling the superclass's onShow, the Component will be visible.</p>\n\n<p>Override in subclasses where more complex behaviour is needed.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.Component-event-show\" rel=\"Ext.Component-event-show\" class=\"docClass\">show</a>.</p>\n"},"onShowComplete":{"!type":"fn(callback?: ?, scope?: ?) -> !this","!doc":"<p>Invoked after the <a href=\"#!/api/Ext.Component-method-afterShow\" rel=\"Ext.Component-method-afterShow\" class=\"docClass\">afterShow</a> method is complete.</p>\n\n<p>Gets passed the same <code>callback</code> and <code>scope</code> parameters that <a href=\"#!/api/Ext.Component-method-afterShow\" rel=\"Ext.Component-method-afterShow\" class=\"docClass\">afterShow</a> received.</p>\n"},"onShowVeto":{"!type":"fn() -> !this"},"owns":{"!type":"fn(element: ?) -> !this","!doc":"<p>Returns <code>true</code> if the passed element is within the container tree of this component.</p>\n\n<p>For example if a menu's submenu contains an <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, that top level\nmenu owns the elements of the date picker. Using this method, you can tell if an event took place\nwithin a certain component tree.</p>\n"},"postBlur":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method to do any post-blur processing.</p>\n"},"previousNode":{"!type":"fn(selector?: ?) -> ?","!doc":"<p>Returns the previous node in the Component tree in tree traversal order.</p>\n\n<p>Note that this is not limited to siblings, and if invoked upon a node with no matching siblings, will walk the\ntree in reverse order to attempt to find a match. Contrast with <a href=\"#!/api/Ext.Component-method-previousSibling\" rel=\"Ext.Component-method-previousSibling\" class=\"docClass\">previousSibling</a>.</p>\n"},"previousSibling":{"!type":"fn(selector?: ?) -> ?","!doc":"<p>Returns the previous sibling of this Component.</p>\n\n<p>Optionally selects the previous sibling which matches the passed ComponentQuery\nselector.</p>\n\n<p>May also be referred to as <strong><code>prev()</code></strong></p>\n\n<p>Note that this is limited to siblings, and if no siblings of the item match, <code>null</code> is returned. Contrast with\n<a href=\"#!/api/Ext.Component-method-previousNode\" rel=\"Ext.Component-method-previousNode\" class=\"docClass\">previousNode</a></p>\n"},"refreshScroll":{"!type":"fn() -> !this","!doc":"<p>Implementation which updates the scroll range of a touch scroller.\nSubclasses may change implementation.</p>\n"},"registerFloatingItem":{"!type":"fn(cmp: ?) -> !this","!doc":"<p>Called by Component#doAutoRender</p>\n\n<p>Register a Container configured <code>floating: true</code> with this Component's <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a>.</p>\n\n<p>Components added in this way will not participate in any layout, but will be rendered\nupon first show in the way that <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Window</a>s are.</p>\n"},"removeClass":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Removes a CSS class from the top level element representing this component.</p>\n"},"removeCls":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Removes a CSS class from the top level element representing this component.</p>\n"},"removeClsWithUI":{"!type":"fn(cls: ?) -> !this","!doc":"<p>Removes a <code>cls</code> to the <code>uiCls</code> array, which will also call <a href=\"#!/api/Ext.Component-method-removeUIClsFromElement\" rel=\"Ext.Component-method-removeUIClsFromElement\" class=\"docClass\">removeUIClsFromElement</a> and removes it from all\nelements of this component.</p>\n"},"removeUIClsFromElement":{"!type":"fn(uiCls: ?) -> !this","!doc":"<p>Method which removes a specified UI + <code>uiCls</code> from the components element. The <code>cls</code>\nwhich is added to the element will be: <code>this.baseCls + '-' + ui + uiCls</code>.</p>\n"},"removeUIFromElement":{"!type":"fn() -> !this","!doc":"<p>Method which removes a specified UI from the components element.</p>\n"},"resumeLayouts":{"!type":"fn(flushOptions: ?) -> !this"},"scrollBy":{"!type":"fn(deltaX: ?, deltaY: ?, animate: ?) -> !this","!doc":"<p>Scrolls this Component by the passed delta values, optionally animating.</p>\n\n<p>All of the following are equivalent:</p>\n\n<pre><code> comp.scrollBy(10, 10, true);\n comp.scrollBy([10, 10], true);\n comp.scrollBy({ x: 10, y: 10 }, true);\n</code></pre>\n"},"scrollTo":{"!type":"fn(x: ?, y: ?, animate?: ?) -> !this","!doc":"<p>Scrolls this component to the specified <code>x</code> and <code>y</code> coordinates.  Only applicable\nfor components that have been configured with <code><a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a></code> or\n<code><a href=\"#!/api/Ext.Component-cfg-overflowX\" rel=\"Ext.Component-cfg-overflowX\" class=\"docClass\">overflowX</a></code> and <code><a href=\"#!/api/Ext.Component-cfg-overflowY\" rel=\"Ext.Component-cfg-overflowY\" class=\"docClass\">overflowY</a></code>.</p>\n"},"setAutoScroll":{"!type":"fn(scroll: ?) -> ?","!doc":"<p>Sets the overflow on the content element of the component.</p>\n"},"setBorder":{"!type":"fn(border: ?) -> !this"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.Component-cfg-data\" rel=\"Ext.Component-cfg-data\" class=\"docClass\">data</a>.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?) -> !this","!doc":"<p>Enable or disable the component.</p>\n"},"setDock":{"!type":"fn(dock: ?) -> ?","!doc":"<p>Sets the dock position of this component in its parent panel. Note that this only has effect if this item is part\nof the <code>dockedItems</code> collection of a parent that has a DockLayout (note that any Panel has a DockLayout by default)</p>\n"},"setDocked":{"!type":"fn(dock: ?) -> ?","!doc":"<p>Sets the dock position of this component in its parent panel. Note that this only has effect if this item is part\nof the <code>dockedItems</code> collection of a parent that has a DockLayout (note that any Panel has a DockLayout by default)</p>\n"},"setFlex":{"!type":"fn(flex: ?) -> !this","!doc":"<p>Sets the flex property of this component. Only applicable when this component is\nan item of a box layout</p>\n"},"setHeight":{"!type":"fn(height: ?) -> ?","!doc":"<p>Sets the height of the component. This method fires the <a href=\"#!/api/Ext.Component-event-resize\" rel=\"Ext.Component-event-resize\" class=\"docClass\">resize</a> event.</p>\n"},"setHidden":{"!type":"fn(hidden: ?) -> ?","!doc":"<p>Sets the hidden state of this component. This is basically the same as\n<code><a href=\"#!/api/Ext.Component-method-setVisible\" rel=\"Ext.Component-method-setVisible\" class=\"docClass\">setVisible</a></code> but the boolean parameter has the opposite meaning.</p>\n"},"setHtml":{"!type":"fn(html: ?) -> !this"},"setLoading":{"!type":"fn(load: ?) -> ?","!doc":"<p>This method allows you to show or hide a LoadMask on top of this component.</p>\n\n<p>The mask will be rendered into the element returned by <a href=\"#!/api/Ext.Component-method-getMaskTarget\" rel=\"Ext.Component-method-getMaskTarget\" class=\"docClass\">getMaskTarget</a> which for most Components is the Component's\nelement. See <a href=\"#!/api/Ext.Component-method-getMaskTarget\" rel=\"Ext.Component-method-getMaskTarget\" class=\"docClass\">getMaskTarget</a> and <a href=\"#!/api/Ext.Component-cfg-maskElement\" rel=\"Ext.Component-cfg-maskElement\" class=\"docClass\">maskElement</a>.</p>\n\n<p>Most Components will return <code>null</code> indicating that their LoadMask cannot reside inside their element, but must\nbe rendered into the document body.</p>\n\n<p><a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">Grid Views</a> however will direct a LoadMask to be rendered into the owning <a href=\"#!/api/Ext.panel.Table\" rel=\"Ext.panel.Table\" class=\"docClass\">GridPanel</a>.</p>\n"},"setLocalX":{"!type":"fn(x: ?) -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"setLocalXY":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"setLocalY":{"!type":"fn(y: ?) -> !this"},"setMargin":{"!type":"fn(margin: ?) -> !this","!doc":"<p>Sets the margin on the target element.</p>\n"},"setOverflowXY":{"!type":"fn(overflowX: ?, overflowY: ?) -> ?","!doc":"<p>Sets the overflow x/y on the content element of the component. The x/y overflow\nvalues can be any valid CSS overflow (e.g., 'auto' or 'scroll'). By default, the\nvalue is 'hidden'. Passing null for one of the values will erase the inline style.\nPassing <code>undefined</code> will preserve the current value.</p>\n"},"setPagePosition":{"!type":"fn(x: ?, y?: ?, animate?: ?) -> ?","!doc":"<p>Sets the page XY position of the component. To set the left and top instead, use <a href=\"#!/api/Ext.Component-method-setPosition\" rel=\"Ext.Component-method-setPosition\" class=\"docClass\">setPosition</a>.\nThis method fires the <a href=\"#!/api/Ext.Component-event-move\" rel=\"Ext.Component-event-move\" class=\"docClass\">move</a> event.</p>\n"},"setPosition":{"!type":"fn(x: ?, y?: ?, animate?: ?) -> ?","!doc":"<p>Sets the left and top of the component. To set the page XY position instead, use <a href=\"#!/api/Ext.Component-method-setPagePosition\" rel=\"Ext.Component-method-setPagePosition\" class=\"docClass\">setPagePosition</a>. This\nmethod fires the <a href=\"#!/api/Ext.Component-event-move\" rel=\"Ext.Component-event-move\" class=\"docClass\">move</a> event.</p>\n"},"setRegion":{"!type":"fn(region: ?) -> string","!doc":"<p>This method changes the <code>region</code> config property for this border region. This is\nonly valid if this component is in a <code>border</code> layout (<code><a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Ext.layout.container.Border</a></code>).</p>\n\n<p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>\n"},"setScrollX":{"!type":"fn(x: ?) -> !this","!doc":"<p>Sets the \"x\" scroll position for this component.  Only applicable for components\nthat have been configured with <code><a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a></code> or <code><a href=\"#!/api/Ext.Component-cfg-overflowX\" rel=\"Ext.Component-cfg-overflowX\" class=\"docClass\">overflowX</a></code>.</p>\n"},"setScrollY":{"!type":"fn(y: ?) -> !this","!doc":"<p>Sets the \"y\" scroll position for this component.  Only applicable for components\nthat have been configured with <code><a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a></code> or <code><a href=\"#!/api/Ext.Component-cfg-overflowY\" rel=\"Ext.Component-cfg-overflowY\" class=\"docClass\">overflowY</a></code>.</p>\n"},"setSize":{"!type":"fn(width: ?, height: ?) -> ?","!doc":"<p>Sets the width and height of this Component. This method fires the <a href=\"#!/api/Ext.Component-event-resize\" rel=\"Ext.Component-event-resize\" class=\"docClass\">resize</a> event. This method can accept\neither width and height as separate arguments, or you can pass a size object like <code>{width:10, height:20}</code>.</p>\n"},"setStyle":{"!type":"fn(style: ?) -> !this","!doc":"<p>Sets the stlye for this Component's primary element.</p>\n"},"setVisible":{"!type":"fn(visible: ?) -> ?","!doc":"<p>Convenience function to hide or show this component by Boolean.</p>\n"},"setWeight":{"!type":"fn(weight: ?) -> number","!doc":"<p>Sets the <code>weight</code> config property for this component. This is only valid if this\ncomponent is in a <code>border</code> layout (<code><a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Ext.layout.container.Border</a></code>).</p>\n\n<p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>\n"},"setWidth":{"!type":"fn(width: ?) -> ?","!doc":"<p>Sets the width of the component. This method fires the <a href=\"#!/api/Ext.Component-event-resize\" rel=\"Ext.Component-event-resize\" class=\"docClass\">resize</a> event.</p>\n"},"setX":{"!type":"fn(x: ?, animate: ?) -> !this"},"setXY":{"!type":"fn(xy: ?, animate: ?) -> !this"},"setY":{"!type":"fn(y: ?, animate: ?) -> !this"},"show":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component is shown when calling the <a href=\"#!/api/Ext.Component-method-show\" rel=\"Ext.Component-method-show\" class=\"docClass\">show</a> method.</p>\n"},"showAt":{"!type":"fn(x: ?, y?: ?, animate?: ?) -> ?","!doc":"<p>Displays component at specific xy position.\nA floating component (like a menu) is positioned relative to its ownerCt if any.\nUseful for popping up a context menu:</p>\n\n<pre><code>listeners: {\n    itemcontextmenu: function(view, record, item, index, event, options) {\n        Ext.create('<a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a>', {\n            width: 100,\n            height: 100,\n            margin: '0 0 10 0',\n            items: [{\n                text: 'regular item 1'\n            },{\n                text: 'regular item 2'\n            },{\n                text: 'regular item 3'\n            }]\n        }).showAt(event.getXY());\n    }\n}\n</code></pre>\n"},"showBy":{"!type":"fn(component: ?, position?: ?, offsets?: ?) -> ?","!doc":"<p>Shows this component by the specified <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a> or Element.\nUsed when this component is <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a>.</p>\n"},"suspendLayouts":{"!type":"fn() -> !this"},"unitizeBox":{"!type":"fn(box: ?) -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"unmask":{"!type":"fn() -> !this","!doc":"<p>Removes the mask applied by <a href=\"#!/api/Ext.Component-method-mask\" rel=\"Ext.Component-method-mask\" class=\"docClass\">mask</a></p>\n"},"unregisterFloatingItem":{"!type":"fn(cmp: ?) -> !this"},"up":{"!type":"fn(selector?: ?, limit?: ?) -> ?","!doc":"<p>Navigates up the ownership hierarchy searching for an ancestor Container which matches any passed selector or component.</p>\n\n<p><em>Important.</em> There is not a universal upwards navigation pointer. There are several upwards relationships\nsuch as the <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">button</a> which activates a <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a>, or the\n<a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">menu item</a> which activated a <a href=\"#!/api/Ext.menu.Item-cfg-menu\" rel=\"Ext.menu.Item-cfg-menu\" class=\"docClass\">submenu</a>, or the\n<a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">column header</a> which activated the column menu.</p>\n\n<p>These differences are abstracted away by this method.</p>\n\n<p>Example:</p>\n\n<pre><code>var owningTabPanel = grid.up('tabpanel');\n</code></pre>\n"},"update":{"!type":"fn(htmlOrData: ?, loadScripts?: ?, callback?: ?) -> !this","!doc":"<p>Update the content area of a component.</p>\n"},"updateBox":{"!type":"fn(box: ?) -> ?","!doc":"<p>Sets the current box measurements of the component's underlying element.</p>\n"},"updateLayout":{"!type":"fn(options?: ?) -> !this","!doc":"<p>Updates this component's layout. If this update affects this components <a href=\"#!/api/Ext.Component-property-ownerCt\" rel=\"Ext.Component-property-ownerCt\" class=\"docClass\">ownerCt</a>,\nthat component's <code>updateLayout</code> method will be called to perform the layout instead.\nOtherwise, just this component (and its child items) will layout.</p>\n"},"activate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after a Component has been visually activated.</p>\n\n<p><strong>Note</strong> This event is only fired if this Component is a child of a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>\nthat uses <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> as it's layout or this Component is a floating Component.</p>\n"},"added":{"!type":"fn(this: ?, container: ?, pos: ?, eOpts: ?)","!doc":"<p>Fires after a Component had been added to a Container.</p>\n"},"afterrender":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component rendering is finished.</p>\n\n<p>The <code>afterrender</code> event is fired after this Component has been <a href=\"#!/api/Ext.Component-property-rendered\" rel=\"Ext.Component-property-rendered\" class=\"docClass\">rendered</a>, been postprocessed by any\n<code>afterRender</code> method defined for the Component.</p>\n"},"beforeactivate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before a Component has been visually activated. Returning <code>false</code> from an event listener can prevent\nthe activate from occurring.</p>\n\n<p><strong>Note</strong> This event is only fired if this Component is a child of a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>\nthat uses <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> as it's layout.</p>\n"},"beforedeactivate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before a Component has been visually deactivated. Returning <code>false</code> from an event listener can\nprevent the deactivate from occurring.</p>\n\n<p><strong>Note</strong> This event is only fired if this Component is a child of a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>\nthat uses <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> as it's layout.</p>\n"},"beforedestroy":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before the component is <a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a>ed. Return <code>false</code> from an event handler to stop the\n<a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a>.</p>\n"},"beforehide":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before the component is hidden when calling the <a href=\"#!/api/Ext.Component-method-hide\" rel=\"Ext.Component-method-hide\" class=\"docClass\">hide</a> method. Return <code>false</code> from an event\nhandler to stop the hide.</p>\n"},"beforerender":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before the component is <a href=\"#!/api/Ext.Component-property-rendered\" rel=\"Ext.Component-property-rendered\" class=\"docClass\">rendered</a>. Return <code>false</code> from an event handler to stop the\n<a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">render</a>.</p>\n"},"beforeshow":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before the component is shown when calling the <a href=\"#!/api/Ext.Component-method-show\" rel=\"Ext.Component-method-show\" class=\"docClass\">show</a> method. Return <code>false</code> from an event\nhandler to stop the show.</p>\n"},"blur":{"!type":"fn(this: ?, The: ?, eOpts: ?)","!doc":"<p>Fires when this Component loses focus.</p>\n"},"boxready":{"!type":"fn(this: ?, width: ?, height: ?, eOpts: ?)","!doc":"<p>Fires <em>one time</em> - after the component has been laid out for the first time at its initial size.</p>\n"},"deactivate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after a Component has been visually deactivated.</p>\n\n<p><strong>Note</strong> This event is only fired if this Component is a child of a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>\nthat uses <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> as it's layout or this Component is a floating Component.</p>\n"},"disable":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component is disabled.</p>\n"},"enable":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component is enabled.</p>\n"},"move":{"!type":"fn(this: ?, x: ?, y: ?, eOpts: ?)","!doc":"<p>Fires after the component is moved.</p>\n"},"removed":{"!type":"fn(this: ?, ownerCt: ?, eOpts: ?)","!doc":"<p>Fires when a component is removed from an <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a></p>\n"},"render":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component markup is <a href=\"#!/api/Ext.Component-property-rendered\" rel=\"Ext.Component-property-rendered\" class=\"docClass\">rendered</a>.</p>\n"},"resize":{"!type":"fn(this: ?, width: ?, height: ?, oldWidth: ?, oldHeight: ?, eOpts: ?)","!doc":"<p>Fires after the component is resized. Note that this does <em>not</em> fire when the component is first laid out at its initial\nsize. To hook that point in the life cycle, use the <a href=\"#!/api/Ext.Component-event-boxready\" rel=\"Ext.Component-event-boxready\" class=\"docClass\">boxready</a> event.</p>\n"},"autoEl":{"!doc":"<p>A tag name or DomHelper spec used to create the <a href=\"#!/api/Ext.Component-method-getEl\" rel=\"Ext.Component-method-getEl\" class=\"docClass\">Element</a> which will\nencapsulate this Component.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong>'div'</strong>. The more complex Sencha classes use a more\ncomplex DOM structure specified by their own <a href=\"#!/api/Ext.Component-cfg-renderTpl\" rel=\"Ext.Component-cfg-renderTpl\" class=\"docClass\">renderTpl</a>s.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components encapsulated by\ndifferent DOM elements. Example usage:</p>\n\n<pre><code>{\n    xtype: 'component',\n    autoEl: {\n        tag: 'img',\n        src: 'http://www.example.com/example.jpg'\n    }\n}, {\n    xtype: 'component',\n    autoEl: {\n        tag: 'blockquote',\n        html: 'autoEl is cool!'\n    }\n}, {\n    xtype: 'container',\n    autoEl: 'ul',\n    cls: 'ux-unordered-list',\n    items: {\n        xtype: 'component',\n        autoEl: 'li',\n        html: 'First list item'\n    }\n}\n</code></pre>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"cls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element. This can be useful\nfor adding customized styles to the component or any of its children using standard CSS rules.</p>\n"},"disabled":{"!type":"bool","!doc":"<p><code>true</code> to disable the component.</p>\n"},"frame":{"!type":"bool","!doc":"<p>Specify as <code>true</code> to have the Component inject framing elements within the Component at render time to provide a\ngraphical rounded frame around the Component content.</p>\n\n<p>This is only necessary when running on outdated, or non standard-compliant browsers such as Microsoft's Internet\nExplorer prior to version 9 which do not support rounded corners natively.</p>\n\n<p>The extra space taken up by this framing is available from the read only property <a href=\"#!/api/Ext.Component-property-frameSize\" rel=\"Ext.Component-property-frameSize\" class=\"docClass\">frameSize</a>.</p>\n"},"hidden":{"!type":"bool","!doc":"<p><code>true</code> to hide the component.</p>\n"},"minWidth":{"!type":"number","!doc":"<p>The minimum value in pixels which this Component will set its width to.</p>\n\n<p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>\n"},"overCls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element when the mouse moves over the Element,\nand removed when the mouse moves out. This can be useful for adding customized 'active' or 'hover' styles to the\ncomponent or any of its children using standard CSS rules.</p>\n"},"shrinkWrap":{"!doc":"<p>If this property is a number, it is interpreted as follows:</p>\n\n<ul>\n<li>0 (or <code>false</code>): Neither width nor height depend on content.</li>\n<li>1: Width depends on content (shrink wraps), but height does not.</li>\n<li>2 (default): Height depends on content (shrink wraps), but width does not.</li>\n<li>3 (or <code>true</code>): Both width and height depend on content (shrink wrap).</li>\n</ul>\n\n\n<p>In CSS terms, shrink-wrap width is analogous to an inline-block element as opposed\nto a block-level element. Some container layouts always shrink-wrap their children,\neffectively ignoring this property (e.g., <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">Ext.layout.container.HBox</a>,\n<a href=\"#!/api/Ext.layout.container.VBox\" rel=\"Ext.layout.container.VBox\" class=\"docClass\">Ext.layout.container.VBox</a>, <a href=\"#!/api/Ext.layout.component.Dock\" rel=\"Ext.layout.component.Dock\" class=\"docClass\">Ext.layout.component.Dock</a>).</p>\n"},"defaultBindProperty":{"!type":"string"},"liquidLayout":{"!type":"bool","!doc":"<p>Components that achieve their internal layout results using solely CSS with no JS\nintervention must set this to true.  Failure to set this property to true may result\nin setSize failing to work if the component opted out of the layout run.</p>\n"},"getTdCls":{"!type":"fn() -> !this","!doc":"<p>Needed for when widget is rendered into a grid cell. The class to add to the cell element.</p>\n"},"setUI":{"!type":"fn(ui: ?) -> !this","!doc":"<p>Sets the UI for the component. This will remove any existing UIs on the component. It will also loop through any\n<code>uiCls</code> set on the component and rename them so they include the new UI.</p>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.Component-method-getEl\" rel=\"Ext.Component-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.Component-method-getEl\" rel=\"Ext.Component-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.Component-cfg-renderSelectors\" rel=\"Ext.Component-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.Component-cfg-childEls\" rel=\"Ext.Component-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"AUTO_ID":{"!type":"number"},"DIRECTION_BOTTOM":{"!type":"string"},"DIRECTION_LEFT":{"!type":"string"},"DIRECTION_RIGHT":{"!type":"string"},"DIRECTION_TOP":{"!type":"string","!doc":"<p>Collapse/expand directions</p>\n"},"INVALID_ID_CHARS_Re":{"!doc":"<p>RegExp whih specifies characters in an xtype which must be translated to '-' when generating auto IDs.\nThis includes dot, comma and whitespace</p>\n"},"VERTICAL_DIRECTION_Re":{},"componentIdAttribute":{"!type":"string","!doc":"<p>Name of the element attribute containing its Component id. Used to look up Components\nby their Elements.</p>\n"},"pendingLayouts":{},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"cancelLayout":{"!type":"fn(comp: ?) -> !this","!doc":"<p>Cancels layout of a component.</p>\n"},"flushLayouts":{"!type":"fn() -> !this","!doc":"<p>Performs all pending layouts that were scheduled while\n<a href=\"#!/api/Ext.Component-method-suspendLayouts\" rel=\"Ext.Component-method-suspendLayouts\" class=\"docClass\">suspendLayouts</a> was in effect.</p>\n"}},"!doc":"<p>Base class for all Ext components.</p>\n\n<p>The Component base class has built-in support for basic hide/show and enable/disable\nand size control behavior.</p>\n\n<h2>xtypes</h2>\n\n<p>Every component has a specific xtype, which is its Ext-specific type name, along with\nmethods for checking the xtype like <a href=\"#!/api/Ext.Component-method-getXType\" rel=\"Ext.Component-method-getXType\" class=\"docClass\">getXType</a> and <a href=\"#!/api/Ext.Component-method-isXType\" rel=\"Ext.Component-method-isXType\" class=\"docClass\">isXType</a>. See the\n<a href=\"#!/guide/components\">Component Guide</a> for more information on xtypes and the Component hierarchy.</p>\n\n<h2>Finding components</h2>\n\n<p>All Components are registered with the Ext.ComponentManager on construction so\nthat they can be referenced at any time via Ext.getCmp, passing the\n<a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">id</a>.</p>\n\n<p>Additionally the Ext.ComponentQuery provides a CSS-selectors-like way to look\nup components by their xtype and many other attributes.  For example the following code\nwill find all textfield components inside component with <code>id: 'myform'</code>:</p>\n\n<pre><code>Ext.ComponentQuery.query('#myform textfield');\n</code></pre>\n\n<h2>Extending <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a></h2>\n\n<p>All subclasses of Component may participate in the automated Ext component\nlifecycle of creation, rendering and destruction which is provided by the\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a> class. Components may be added to a Container\nthrough the <a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a> config option at the time\nthe Container is created, or they may be added dynamically via the\n<a href=\"#!/api/Ext.container.Container-method-add\" rel=\"Ext.container.Container-method-add\" class=\"docClass\">add</a> method.</p>\n\n<p>All user-developed visual widgets that are required to participate in automated\nlifecycle and size management should subclass Component.</p>\n\n<p>See the Creating new UI controls chapter in <a href=\"#!/guide/components\">Component Guide</a> for details on how and\nto either extend or augment Ext JS base classes to create custom Components.</p>\n\n<h2>The <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> class by itself</h2>\n\n<p>Usually one doesn't need to instantiate the <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> class. There are subclasses\nwhich implement specialized use cases, covering most application needs. However it is\npossible to instantiate a base Component, and it can be rendered to document, or handled\nby layouts as the child item of a Container:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    html: 'Hello world!',\n    width: 300,\n    height: 200,\n    padding: 20,\n    style: {\n        color: '#FFFFFF',\n        backgroundColor:'#000000'\n    },\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>The Component above creates its encapsulating <code>div</code> upon render, and use the configured\nHTML as content. More complex internal structure may be created using the\n<a href=\"#!/api/Ext.Component-cfg-renderTpl\" rel=\"Ext.Component-cfg-renderTpl\" class=\"docClass\">renderTpl</a> configuration, although to display database-derived mass data, it is\nrecommended that an ExtJS data-backed Component such as a <a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">View</a>,\n<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">GridPanel</a>, or <a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">TreePanel</a> be used.</p>\n\n<p><strong>From override Ext.layout.container.border.Region:</strong> This override provides extra, border layout specific methods for <code><a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a></code>. The\n<code><a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Ext.layout.container.Border</a></code> class requires this override so that the added functions\nare only included in a build when <code>border</code> layout is used.</p>\n\n<p><strong>From override Ext.rtl.Component:</strong> This override adds RTL support and the <code>rtl</code> config option to AbstactComponent.</p>\n","!type":"fn(config: ?)","alignTarget":{"!type":"string","!doc":"<p>A Component or Element by which to position this component according to the <a href=\"#!/api/Ext.Component-cfg-defaultAlign\" rel=\"Ext.Component-cfg-defaultAlign\" class=\"docClass\">defaultAlign</a>.\nDefaults to ths owning Container.</p>\n\n<p><em>Only applicable if this component is <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a></em></p>\n\n<p><em>Used upon first show</em>.</p>\n"},"autoEl":{"!doc":"<p>A tag name or DomHelper spec used to create the <a href=\"#!/api/Ext.Component-method-getEl\" rel=\"Ext.Component-method-getEl\" class=\"docClass\">Element</a> which will\nencapsulate this Component.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong>'div'</strong>. The more complex Sencha classes use a more\ncomplex DOM structure specified by their own <a href=\"#!/api/Ext.Component-cfg-renderTpl\" rel=\"Ext.Component-cfg-renderTpl\" class=\"docClass\">renderTpl</a>s.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components encapsulated by\ndifferent DOM elements. Example usage:</p>\n\n<pre><code>{\n    xtype: 'component',\n    autoEl: {\n        tag: 'img',\n        src: 'http://www.example.com/example.jpg'\n    }\n}, {\n    xtype: 'component',\n    autoEl: {\n        tag: 'blockquote',\n        html: 'autoEl is cool!'\n    }\n}, {\n    xtype: 'container',\n    autoEl: 'ul',\n    cls: 'ux-unordered-list',\n    items: {\n        xtype: 'component',\n        autoEl: 'li',\n        html: 'First list item'\n    }\n}\n</code></pre>\n"},"autoShow":{"!type":"bool","!doc":"<p><code>true</code> to automatically show the component upon creation. This config option may only be used for\n<a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> components or components that use <a href=\"#!/api/Ext.Component-cfg-autoRender\" rel=\"Ext.Component-cfg-autoRender\" class=\"docClass\">autoRender</a>.</p>\n"},"columnWidth":{"!doc":"<p>Defines the column width inside <a href=\"#!/api/Ext.layout.container.Column\" rel=\"Ext.layout.container.Column\" class=\"docClass\">column layout</a>.</p>\n\n<p>Can be specified as a number or as a percentage.</p>\n"},"componentCls":{"!type":"string","!doc":"<p>CSS Class to be added to a components root level element to give distinction to it via styling.</p>\n"},"constrainTo":{"!doc":"<p>A Region (or an element from which a Region measurement will be read) which is used\nto constrain the component. Only applies when the component is floating.</p>\n"},"constraintInsets":{"!doc":"<p>An object or a string (in TRBL order) specifying insets from the configured <a href=\"#!/api/Ext.Component-cfg-constrainTo\" rel=\"Ext.Component-cfg-constrainTo\" class=\"docClass\">constrain region</a>\nwithin which this component must be constrained when positioning or sizing.\nexample:</p>\n\n<p>   constraintInsets: '10 10 10 10' // Constrain with 10px insets from parent</p>\n"},"contentEl":{"!type":"string","!doc":"<p>Specify an existing HTML element, or the <code>id</code> of an existing HTML element to use as the content for this component.</p>\n\n<p>This config option is used to take an existing HTML element and place it in the layout element of a new component\n(it simply moves the specified DOM element <em>after the Component is rendered</em> to use as the content.</p>\n\n<p><strong>Notes:</strong></p>\n\n<p>The specified HTML element is appended to the layout element of the component <em>after any configured\n<a href=\"#!/api/Ext.Component-cfg-html\" rel=\"Ext.Component-cfg-html\" class=\"docClass\">HTML</a> has been inserted</em>, and so the document will not contain this element at the time\nthe <a href=\"#!/api/Ext.Component-event-render\" rel=\"Ext.Component-event-render\" class=\"docClass\">render</a> event is fired.</p>\n\n<p>The specified HTML element used will not participate in any <strong><code><a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a></code></strong>\nscheme that the Component may use. It is just HTML. Layouts operate on child\n<strong><code><a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a></code></strong>.</p>\n\n<p>Add either the <code>x-hidden</code> or the <code>x-hidden-display</code> CSS class to prevent a brief flicker of the content before it\nis rendered to the panel.</p>\n"},"data":{"!doc":"<p>The initial set of data to apply to the <code><a href=\"#!/api/Ext.Component-cfg-tpl\" rel=\"Ext.Component-cfg-tpl\" class=\"docClass\">tpl</a></code> to update the content\narea of the Component.</p>\n"},"defaultAlign":{"!type":"string","!doc":"<p>The default Ext.dom.Element#getAlignToXY anchor position value for this component\nrelative to its <a href=\"#!/api/Ext.Component-cfg-alignTarget\" rel=\"Ext.Component-cfg-alignTarget\" class=\"docClass\">alignTarget</a> (which defaults to its owning Container).</p>\n\n<p><em>Only applicable if this component is <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a></em></p>\n\n<p><em>Used upon first show</em>.</p>\n"},"disabled":{"!type":"bool","!doc":"<p><code>true</code> to disable the component.</p>\n"},"disabledCls":{"!type":"string","!doc":"<p>CSS class to add when the Component is disabled.</p>\n"},"floating":{"!type":"bool","!doc":"<p>Specify as true to float the Component outside of the document flow using CSS absolute positioning.</p>\n\n<p>Components such as <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Window</a>s and <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Menu</a>s are floating by default.</p>\n\n<p>Floating Components that are programatically <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">rendered</a> will register\nthemselves with the global <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a></p>\n\n<h3>Floating Components as child items of a Container</h3>\n\n<p>A floating Component may be used as a child item of a Container. This just allows the floating Component to seek\na ZIndexManager by examining the ownerCt chain.</p>\n\n<p>When configured as floating, Components acquire, at render time, a <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a> which\nmanages a stack of related floating Components. The ZIndexManager brings a single floating Component to the top\nof its stack when the Component's <a href=\"#!/api/Ext.Component-method-toFront\" rel=\"Ext.Component-method-toFront\" class=\"docClass\">toFront</a> method is called.</p>\n\n<p>The ZIndexManager is found by traversing up the <a href=\"#!/api/Ext.Component-property-ownerCt\" rel=\"Ext.Component-property-ownerCt\" class=\"docClass\">ownerCt</a> chain to find an ancestor which itself is\nfloating. This is so that descendant floating Components of floating <em>Containers</em> (Such as a ComboBox dropdown\nwithin a Window) can have its zIndex managed relative to any siblings, but always <strong>above</strong> that floating\nancestor Container.</p>\n\n<p>If no floating ancestor is found, a floating Component registers itself with the default <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a>.</p>\n\n<p>Floating components <em>do not participate in the Container's layout</em>. Because of this, they are not rendered until\nyou explicitly <a href=\"#!/api/Ext.Component-method-show\" rel=\"Ext.Component-method-show\" class=\"docClass\">show</a> them.</p>\n\n<p>After rendering, the ownerCt reference is deleted, and the <a href=\"#!/api/Ext.Component-property-floatParent\" rel=\"Ext.Component-property-floatParent\" class=\"docClass\">floatParent</a> property is set to the found\nfloating ancestor Container. If no floating ancestor Container was found the <a href=\"#!/api/Ext.Component-property-floatParent\" rel=\"Ext.Component-property-floatParent\" class=\"docClass\">floatParent</a> property will\nnot be set.</p>\n"},"formBind":{"!type":"bool","!doc":"<p>When inside FormPanel, any component configured with <code>formBind: true</code> will\nbe enabled/disabled depending on the validity state of the form.\nSee <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a> for more information and example.</p>\n"},"height":{"!type":"number","!doc":"<p>The height of this component in pixels.</p>\n"},"html":{"!doc":"<p>An HTML fragment, or a DomHelper specification to use as the layout element content.\nThe HTML content is added after the component is rendered, so the document will not contain this HTML at the time\nthe <a href=\"#!/api/Ext.Component-event-render\" rel=\"Ext.Component-event-render\" class=\"docClass\">render</a> event is fired. This content is inserted into the body <em>before</em> any configured <a href=\"#!/api/Ext.Component-cfg-contentEl\" rel=\"Ext.Component-cfg-contentEl\" class=\"docClass\">contentEl</a>\nis appended.</p>\n"},"id":{"!type":"string","!doc":"<p>The <strong>unique</strong> id of this component instance.</p>\n\n<p>Use of this config should be considered carefully as this value must be unique across\nall existing components. Components created with an <code>id</code> may be accessed globally\nusing Ext.getCmp.</p>\n\n<p>Instead of using assigned ids, consider a reference config and a ViewController\nto respond to events and perform processing upon this Component.</p>\n\n<p>Alternatively, <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">itemId</a> and ComponentQuery can be\nused to perform selector-based searching for Components analogous to DOM querying.\nThe <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a> class contains several helpful\nshortcut methods to query its descendant\nComponents by selector.</p>\n\n<p>Note that this <code>id</code> will also be used as the element id for the containing HTML\nelement that is rendered to the page for this component. This allows you to write\nid-based CSS rules to style the specific instance of this component uniquely, and\nalso to select sub-elements using this component's <code>id</code> as the parent.</p>\n\n<p>Defaults to an <a href=\"#!/api/Ext.Component-method-getId\" rel=\"Ext.Component-method-getId\" class=\"docClass\">auto-assigned id</a>.</p>\n\n<p><strong>Note</strong>: Valid identifiers start with a letter or underscore and are followed by\n(optional) additional letters, underscores, digits or hyphens.</p>\n"},"itemId":{"!type":"string","!doc":"<p>The <strong>unique</strong> id of this component instance within its container. See also the\nreference config.</p>\n\n<p>An <code>itemId</code> can be used as an alternative way to get a reference to a component when no object reference is\navailable. Instead of using an <code><a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">id</a></code> with getCmp, use\n<code>itemId</code> with <a href=\"#!/api/Ext.container.Container-method-getComponent\" rel=\"Ext.container.Container-method-getComponent\" class=\"docClass\">getComponent</a> which will\nretrieve <code>itemId</code>'s or <a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">id</a>'s. Since <code>itemId</code>'s are an index to the container's\ninternal collection, the <code>itemId</code> is scoped locally to the container -- avoiding\npotential conflicts with Ext.ComponentManager which requires a <strong>unique</strong>\n<code><a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">id</a></code> values.</p>\n\n<pre><code>var c = new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({ //\n    <a href=\"#!/api/Ext.Component-cfg-height\" rel=\"Ext.Component-cfg-height\" class=\"docClass\">height</a>: 300,\n    <a href=\"#!/api/Ext.Component-cfg-renderTo\" rel=\"Ext.Component-cfg-renderTo\" class=\"docClass\">renderTo</a>: document.body,\n    <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a>: 'auto',\n    <a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a>: [\n        {\n            itemId: 'p1',\n            <a href=\"#!/api/Ext.panel.Panel-cfg-title\" rel=\"Ext.panel.Panel-cfg-title\" class=\"docClass\">title</a>: 'Panel 1',\n            <a href=\"#!/api/Ext.Component-cfg-height\" rel=\"Ext.Component-cfg-height\" class=\"docClass\">height</a>: 150\n        },\n        {\n            itemId: 'p2',\n            <a href=\"#!/api/Ext.panel.Panel-cfg-title\" rel=\"Ext.panel.Panel-cfg-title\" class=\"docClass\">title</a>: 'Panel 2',\n            <a href=\"#!/api/Ext.Component-cfg-height\" rel=\"Ext.Component-cfg-height\" class=\"docClass\">height</a>: 150\n        }\n    ]\n})\np1 = c.<a href=\"#!/api/Ext.container.Container-method-getComponent\" rel=\"Ext.container.Container-method-getComponent\" class=\"docClass\">getComponent</a>('p1'); // not the same as Ext.getCmp()\np2 = p1.<a href=\"#!/api/Ext.Component-property-ownerCt\" rel=\"Ext.Component-property-ownerCt\" class=\"docClass\">ownerCt</a>.<a href=\"#!/api/Ext.container.Container-method-getComponent\" rel=\"Ext.container.Container-method-getComponent\" class=\"docClass\">getComponent</a>('p2'); // reference via a sibling\n</code></pre>\n\n<p>Also see <a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">id</a>, <code>Ext.container.Container.query</code>, <code>Ext.container.Container.down</code> and\n<code>Ext.container.Container.child</code>.</p>\n\n<p><strong>Note</strong>: Valid identifiers start with a letter or underscore and are followed by\n(optional) additional letters, underscores, digits or hyphens.</p>\n\n<p><strong>Note</strong>: to access the container of an item see <a href=\"#!/api/Ext.Component-property-ownerCt\" rel=\"Ext.Component-property-ownerCt\" class=\"docClass\">ownerCt</a>.</p>\n"},"loader":{"!doc":"<p>A configuration object or an instance of a <a href=\"#!/api/Ext.ComponentLoader\" rel=\"Ext.ComponentLoader\" class=\"docClass\">Ext.ComponentLoader</a> to load remote content\nfor this Component.</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    loader: {\n        url: 'content.html',\n        autoLoad: true\n    },\n    renderTo: Ext.getBody()\n});\n</code></pre>\n"},"margin":{"!doc":"<p>Specifies the margin for this component. The margin can be a single numeric value to apply to all sides or it can\nbe a CSS style specification for each style, for example: '10 5 3 10' (top, right, bottom, left).</p>\n"},"modelValidation":{"!type":"bool","!doc":"<p>This config enables binding to your <code>Ext.data.Model.validators</code>. This\nis only processed by form fields (e.g., <code><a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a></code>) at present, but\nthis setting is inherited and so can be set on a parent container.</p>\n\n<p>When set to <code>true</code> by a component or not set by a component but inherited from\nan ancestor container, <code>Ext.data.Validation</code> records are used to automatically\nbind validation results for any form field to which a <code>value</code> is bound.</p>\n\n<p>While this config can be set arbitrarily high in the component hierarchy, doing\nso can create a lot overhead if most of your form fields do not actually rely on\n<code>validators</code> in your data model.</p>\n\n<p>Using this setting for a form that is bound to an <code>Ext.data.Model</code> might look\nlike this:</p>\n\n<pre><code> {\n     xtype: 'panel',\n     modelValidation: true,\n     items: [{\n         xtype: 'textfield',\n         bind: '{theUser.firstName}'\n     },{\n         xtype: 'textfield',\n         bind: '{theUser.lastName}'\n     },{\n         xtype: 'textfield',\n         bind: '{theUser.phoneNumber}'\n     },{\n         xtype: 'textfield',\n         bind: '{theUser.email}'\n     }]\n }\n</code></pre>\n\n<p>The above is equivalent to the following manual binding of validation:</p>\n\n<pre><code> {\n     xtype: 'panel',\n     items: [{\n         xtype: 'textfield',\n         bind: {\n             value:      '{theUser.firstName}'\n             validation: '{theUser.validation.firstName}'\n         }\n     },{\n         xtype: 'textfield',\n         bind: {\n             value:      '{theUser.lastName}'\n             validation: '{theUser.validation.lastName}'\n         }\n     },{\n         xtype: 'textfield',\n         bind: {\n             value:      '{theUser.phoneNumber}'\n             validation: '{theUser.validation.phoneNumber}'\n         }\n     },{\n         xtype: 'textfield',\n         bind: {\n             value:      '{theUser.email}'\n             validation: '{theUser.validation.email}'\n         }\n     }]\n }\n</code></pre>\n\n<p>Notice that \"validation\" is a pseudo-association defined for all entities. See\n<code>Ext.data.Model.getValidation</code> for further details.</p>\n"},"overCls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element when the mouse moves over the Element,\nand removed when the mouse moves out. This can be useful for adding customized 'active' or 'hover' styles to the\ncomponent or any of its children using standard CSS rules.</p>\n"},"overflowX":{"!type":"string","!doc":"<p>Possible values are:</p>\n\n<ul>\n<li><code>'auto'</code> to enable automatic horizontal scrollbar (Style overflow-x: 'auto').</li>\n<li><code>'scroll'</code> to always enable horizontal scrollbar (Style overflow-x: 'scroll').</li>\n</ul>\n\n\n<p>The default is overflow-x: 'hidden'. This should not be combined with <a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a>.</p>\n"},"overflowY":{"!type":"string","!doc":"<p>Possible values are:</p>\n\n<ul>\n<li><code>'auto'</code> to enable automatic vertical scrollbar (Style overflow-y: 'auto').</li>\n<li><code>'scroll'</code> to always enable vertical scrollbar (Style overflow-y: 'scroll').</li>\n</ul>\n\n\n<p>The default is overflow-y: 'hidden'. This should not be combined with <a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a>.</p>\n"},"padding":{"!doc":"<p>Specifies the padding for this component. The padding can be a single numeric value to apply to all sides or it\ncan be a CSS style specification for each style, for example: '10 5 3 10' (top, right, bottom, left).</p>\n"},"plugins":{"!doc":"<p>An array of plugins to be added to this component. Can also be just a single plugin instead of array.</p>\n\n<p>Plugins provide custom functionality for a component. The only requirement for\na valid plugin is that it contain an <code>init</code> method that accepts a reference of type <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>. When a component\nis created, if any plugins are available, the component will call the init method on each plugin, passing a\nreference to itself. Each plugin can then call methods or respond to events on the component as needed to provide\nits functionality.</p>\n\n<p>Plugins can be added to component by either directly referencing the plugin instance:</p>\n\n<pre><code>plugins: [Ext.create('<a href=\"#!/api/Ext.grid.plugin.CellEditing\" rel=\"Ext.grid.plugin.CellEditing\" class=\"docClass\">Ext.grid.plugin.CellEditing</a>', {clicksToEdit: 1})],\n</code></pre>\n\n<p>By using config object with ptype:</p>\n\n<pre><code>plugins: [{ptype: 'cellediting', clicksToEdit: 1}],\n</code></pre>\n\n<p>Or with just a ptype:</p>\n\n<pre><code>plugins: ['cellediting', 'gridviewdragdrop'],\n</code></pre>\n\n<p>See <a href=\"#!/api/Ext.enums.Plugin\" rel=\"Ext.enums.Plugin\" class=\"docClass\">Ext.enums.Plugin</a> for list of all ptypes.</p>\n"},"region":{"!doc":"<p>Defines the region inside <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>.</p>\n\n<p>Possible values:</p>\n\n<ul>\n<li>north - Positions component at top.</li>\n<li>south - Positions component at bottom.</li>\n<li>east - Positions component at right.</li>\n<li>west - Positions component at left.</li>\n<li>center - Positions component at the remaining space.\nThere <strong>must</strong> be a component with <code>region: \"center\"</code> in every border layout.</li>\n</ul>\n\n"},"renderData":{"!doc":"<p>The data used by <a href=\"#!/api/Ext.Component-cfg-renderTpl\" rel=\"Ext.Component-cfg-renderTpl\" class=\"docClass\">renderTpl</a> in addition to the following property values of the component:</p>\n\n<ul>\n<li>id</li>\n<li>ui</li>\n<li>uiCls</li>\n<li>baseCls</li>\n<li>componentCls</li>\n<li>frame</li>\n</ul>\n\n\n<p>See <a href=\"#!/api/Ext.Component-cfg-renderSelectors\" rel=\"Ext.Component-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.Component-cfg-childEls\" rel=\"Ext.Component-cfg-childEls\" class=\"docClass\">childEls</a> for usage examples.</p>\n"},"renderSelectors":{"!doc":"<p>An object containing properties specifying CSS selectors which identify child elements\ncreated by the render process.</p>\n\n<p>After the Component's internal structure is rendered according to the <a href=\"#!/api/Ext.Component-cfg-renderTpl\" rel=\"Ext.Component-cfg-renderTpl\" class=\"docClass\">renderTpl</a>, this object is iterated through,\nand the found Elements are added as properties to the Component using the <code>renderSelector</code> property name.</p>\n\n<p>For example, a Component which renders a title and description into its element:</p>\n\n<pre><code> Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n     renderTo: Ext.getBody(),\n     renderTpl: [\n         '&lt;h1 class=\"title\"&gt;{title}&lt;/h1&gt;',\n         '&lt;p&gt;{desc}&lt;/p&gt;'\n     ],\n     renderData: {\n         title: \"Error\",\n         desc: \"Something went wrong\"\n     },\n     renderSelectors: {\n         titleEl: 'h1.title',\n         descEl: 'p'\n     },\n     listeners: {\n         afterrender: function(cmp){\n             // After rendering the component will have a titleEl and descEl properties\n             cmp.titleEl.setStyle({color: \"red\"});\n         }\n     }\n });\n</code></pre>\n\n<p>The use of <code>renderSelectors</code> is deprecated (for performance reasons). The above\ncode should be refactored into something like this:</p>\n\n<pre><code> Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n     renderTo: Ext.getBody(),\n     renderTpl: [\n         '&lt;h1 class=\"title\" id=\"{id}-titleEl\" data-ref=\"titleEl\"&gt;{title}&lt;/h1&gt;',\n         '&lt;p id=\"{id}-descEl\" data-ref=\"descEl\"&gt;{desc}&lt;/p&gt;'\n     ],\n     renderData: {\n         title: \"Error\",\n         desc: \"Something went wrong\"\n     },\n     childEls: [\n         'titleEl',\n         'descEl'\n     ]\n });\n</code></pre>\n\n<p>To use <code>childEls</code> yet retain the use of selectors (which remains as expensive as\n<code>renderSelectors</code>):</p>\n\n<pre><code> Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n     renderTo: Ext.getBody(),\n     renderTpl: [\n         '&lt;h1 class=\"title\"&gt;{title}&lt;/h1&gt;',\n         '&lt;p&gt;{desc}&lt;/p&gt;'\n     ],\n     renderData: {\n         title: \"Error\",\n         desc: \"Something went wrong\"\n     },\n     childEls: {\n         titleEl: { selectNode: 'h1.title' },\n         descEl: { selectNode: 'p' }\n     }\n });\n</code></pre>\n"},"renderTo":{"!doc":"<p>Specify the <code>id</code> of the element, a DOM element or an existing Element that this component will be rendered into.</p>\n\n<p><strong>Notes:</strong></p>\n\n<p>Do <em>not</em> use this option if the Component is to be a child item of a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a>.\nIt is the responsibility of the <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a>'s\n<a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout manager</a> to render and manage its child items.</p>\n\n<p>When using this config, a call to <code>render()</code> is not required.</p>\n\n<p>See also: <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">render</a>.</p>\n"},"resizeHandles":{"!type":"string","!doc":"<p>A valid <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Ext.resizer.Resizer</a> handles config string. Only applies when resizable = true.</p>\n"},"rtl":{"!type":"bool","!doc":"<p>True to layout this component and its descendants in \"rtl\" (right-to-left) mode.\nCan be explicitly set to false to override a true value inherited from an ancestor.</p>\n\n<p><strong>Defined in override Ext.rtl.Component.</strong></p>\n"},"shrinkWrap":{"!doc":"<p>If this property is a number, it is interpreted as follows:</p>\n\n<ul>\n<li>0 (or <code>false</code>): Neither width nor height depend on content.</li>\n<li>1: Width depends on content (shrink wraps), but height does not.</li>\n<li>2 (default): Height depends on content (shrink wraps), but width does not.</li>\n<li>3 (or <code>true</code>): Both width and height depend on content (shrink wrap).</li>\n</ul>\n\n\n<p>In CSS terms, shrink-wrap width is analogous to an inline-block element as opposed\nto a block-level element. Some container layouts always shrink-wrap their children,\neffectively ignoring this property (e.g., <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">Ext.layout.container.HBox</a>,\n<a href=\"#!/api/Ext.layout.container.VBox\" rel=\"Ext.layout.container.VBox\" class=\"docClass\">Ext.layout.container.VBox</a>, <a href=\"#!/api/Ext.layout.component.Dock\" rel=\"Ext.layout.component.Dock\" class=\"docClass\">Ext.layout.component.Dock</a>).</p>\n"},"style":{"!doc":"<p>A custom style specification to be applied to this component's Element. Should be a valid argument to\nExt.dom.Element.applyStyles.</p>\n\n<pre><code>new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({\n    title: 'Some Title',\n    renderTo: Ext.getBody(),\n    width: 400, height: 300,\n    layout: 'form',\n    items: [{\n        xtype: 'textarea',\n        style: {\n            width: '95%',\n            marginBottom: '10px'\n        }\n    },\n    new <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>({\n        text: 'Send',\n        minWidth: '100',\n        style: {\n            marginBottom: '10px'\n        }\n    })\n    ]\n});\n</code></pre>\n"},"toFrontOnShow":{"!type":"bool","!doc":"<p>True to automatically call <a href=\"#!/api/Ext.Component-method-toFront\" rel=\"Ext.Component-method-toFront\" class=\"docClass\">toFront</a> when the <a href=\"#!/api/Ext.Component-method-show\" rel=\"Ext.Component-method-show\" class=\"docClass\">show</a> method is called on an already visible,\nfloating component.</p>\n"},"tpl":{"!doc":"<p>An Ext.Template, Ext.XTemplate or an array of strings to form an Ext.XTemplate. Used in\nconjunction with the <code><a href=\"#!/api/Ext.Component-cfg-data\" rel=\"Ext.Component-cfg-data\" class=\"docClass\">data</a></code> and <code><a href=\"#!/api/Ext.Component-cfg-tplWriteMode\" rel=\"Ext.Component-cfg-tplWriteMode\" class=\"docClass\">tplWriteMode</a></code> configurations.</p>\n"},"tplWriteMode":{"!type":"string","!doc":"<p>The Ext.(X)Template method to use when updating the content area of the Component.\nSee <code>Ext.XTemplate.overwrite</code> for information on default mode.</p>\n"},"ui":{"!type":"string","!doc":"<p>A UI style for a component.</p>\n"},"uiCls":{"!doc":"<p>An array of of <code>classNames</code> which are currently applied to this component.</p>\n"},"weight":{"!type":"number","!doc":"<p>A value to control how Components are laid out in a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Border</a> layout or as docked items.</p>\n\n<p>In a Border layout, this can control how the regions (not the center) region lay out if the west or east take full height\nor if the north or south region take full width. Also look at the <a href=\"#!/api/Ext.layout.container.Border-cfg-regionWeights\" rel=\"Ext.layout.container.Border-cfg-regionWeights\" class=\"docClass\">Ext.layout.container.Border.regionWeights</a> on the Border layout. An example to show how you can\ntake control of this is:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.container.Viewport\" rel=\"Ext.container.Viewport\" class=\"docClass\">Ext.container.Viewport</a>', {\n    layout      : 'border',\n    defaultType : 'panel',\n    items       : [\n        {\n            region : 'north',\n            title  : 'North',\n            height : 100\n        },\n        {\n            region : 'south',\n            title  : 'South',\n            height : 100,\n            weight : -25\n        },\n        {\n            region : 'west',\n            title  : 'West',\n            width  : 200,\n            weight : 15\n        },\n        {\n            region : 'east',\n            title  : 'East',\n            width  : 200\n        },\n        {\n            region : 'center',\n            title  : 'center'\n        }\n    ]\n});\n</code></pre>\n\n<p>If docked items, the weight will order how the items are laid out. Here is an example to put a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Ext.toolbar.Toolbar</a> above\na <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>'s header:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    renderTo    : document.body,\n    width       : 300,\n    height      : 300,\n    title       : 'Panel',\n    html        : 'Panel Body',\n    dockedItems : [\n        {\n            xtype : 'toolbar',\n            items : [\n                {\n                    text : 'Save'\n                }\n            ]\n        },\n        {\n            xtype  : 'toolbar',\n            weight : -10,\n            items  : [\n                {\n                    text : 'Remove'\n                }\n            ]\n        }\n    ]\n});\n</code></pre>\n"},"width":{"!type":"number","!doc":"<p>The width of this component in pixels.</p>\n"},"xtype":{"!doc":"<p>This property provides a shorter alternative to creating objects than using a full\nclass name. Using <code>xtype</code> is the most common way to define component instances,\nespecially in a container. For example, the items in a form containing text fields\ncould be created explicitly like so:</p>\n\n<pre><code> items: [\n     Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n         fieldLabel: 'Foo'\n     }),\n     Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n         fieldLabel: 'Bar'\n     }),\n     Ext.create('<a href=\"#!/api/Ext.form.field.Number\" rel=\"Ext.form.field.Number\" class=\"docClass\">Ext.form.field.Number</a>', {\n         fieldLabel: 'Num'\n     })\n ]\n</code></pre>\n\n<p>But by using <code>xtype</code>, the above becomes:</p>\n\n<pre><code> items: [\n     {\n         xtype: 'textfield',\n         fieldLabel: 'Foo'\n     },\n     {\n         xtype: 'textfield',\n         fieldLabel: 'Bar'\n     },\n     {\n         xtype: 'numberfield',\n         fieldLabel: 'Num'\n     }\n ]\n</code></pre>\n\n<p>When the <code>xtype</code> is common to many items, <a href=\"#!/api/Ext.container.Container-cfg-defaultType\" rel=\"Ext.container.Container-cfg-defaultType\" class=\"docClass\">Ext.container.Container.defaultType</a>\nis another way to specify the <code>xtype</code> for all items that don't have an explicit <code>xtype</code>:</p>\n\n<pre><code> defaultType: 'textfield',\n items: [\n     { fieldLabel: 'Foo' },\n     { fieldLabel: 'Bar' },\n     { fieldLabel: 'Num', xtype: 'numberfield' }\n ]\n</code></pre>\n\n<p>Each member of the <code>items</code> array is now just a \"configuration object\". These objects\nare used to create and configure component instances. A configuration object can be\nmanually used to instantiate a component using Ext.widget:</p>\n\n<pre><code> var text1 = Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n     fieldLabel: 'Foo'\n });\n\n // or alternatively:\n\n var text1 = Ext.widget({\n     xtype: 'textfield',\n     fieldLabel: 'Foo'\n });\n</code></pre>\n\n<p>This conversion of configuration objects into instantiated components is done when\na container is created as part of its {Ext.container.AbstractContainer#initComponent}\nprocess. As part of the same process, the <code>items</code> array is converted from its raw\narray form into a Ext.util.MixedCollection instance.</p>\n\n<p>You can define your own <code>xtype</code> on a custom <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">component</a> by specifying\nthe <code>xtype</code> property in Ext.define. For example:</p>\n\n<pre><code>Ext.define('MyApp.PressMeButton', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n    xtype: 'pressmebutton',\n    text: 'Press Me'\n});\n</code></pre>\n\n<p>Care should be taken when naming an <code>xtype</code> in a custom component because there is\na single, shared scope for all xtypes. Third part components should consider using\na prefix to avoid collisions.</p>\n\n<pre><code>Ext.define('Foo.form.CoolButton', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n    xtype: 'ux-coolbutton',\n    text: 'Cool!'\n});\n</code></pre>\n\n<p>See <a href=\"#!/api/Ext.enums.Widget\" rel=\"Ext.enums.Widget\" class=\"docClass\">Ext.enums.Widget</a> for list of all available xtypes.</p>\n"},"$configPrefixed":{"!type":"bool","!doc":"<p>We do not want \"_hidden\" style backing properties.</p>\n"},"$configStrict":{"!type":"bool","!doc":"<p>We also want non-config system properties to go to the instance.</p>\n"},"_asLayoutRoot":{},"_isLayoutRoot":{"!type":"bool","!doc":"<p>Setting this property to <code>true</code> causes the <a href=\"#!/api/Ext.Component-method-isLayoutRoot\" rel=\"Ext.Component-method-isLayoutRoot\" class=\"docClass\">isLayoutRoot</a> method to return\n<code>true</code> and stop the search for the top-most component for a layout.</p>\n"},"allowDomMove":{"!type":"bool","!doc":"<p>private</p>\n"},"autoGenId":{"!type":"bool","!doc":"<p><code>true</code> indicates an <code>id</code> was auto-generated rather than provided by configuration.</p>\n"},"borderBoxCls":{"!type":"string","!doc":"<p>private</p>\n"},"componentLayoutCounter":{"!type":"number","!doc":"<p>The number of component layout calls made on this object.</p>\n"},"deferLayouts":{"!type":"bool","!doc":"<p>private</p>\n"},"deprecated":{"!doc":"<p>private</p>\n"},"disabledRe":{"!doc":"<p>http://www.w3.org/TR/html5/disabled-elements.html</p>\n"},"floatParent":{"!doc":"<p><strong>Only present for <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Components which were inserted as child items of Containers.</strong></p>\n\n<p>There are other similar relationships such as the <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">button</a> which activates a <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a>, or the\n<a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">menu item</a> which activated a <a href=\"#!/api/Ext.menu.Item-cfg-menu\" rel=\"Ext.menu.Item-cfg-menu\" class=\"docClass\">submenu</a>, or the\n<a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">column header</a> which activated the column menu.</p>\n\n<p>These differences are abstracted away by the <a href=\"#!/api/Ext.Component-method-up\" rel=\"Ext.Component-method-up\" class=\"docClass\">up</a> method.</p>\n\n<p>Floating Components that are programatically <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">rendered</a> will not have a <code>floatParent</code>\nproperty.</p>\n\n<p>See <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> and <a href=\"#!/api/Ext.Component-property-zIndexManager\" rel=\"Ext.Component-property-zIndexManager\" class=\"docClass\">zIndexManager</a></p>\n"},"frameSize":{"!doc":"<p>Indicates the width of any framing elements which were added within the encapsulating\nelement to provide graphical, rounded borders. See the <a href=\"#!/api/Ext.Component-cfg-frame\" rel=\"Ext.Component-cfg-frame\" class=\"docClass\">frame</a> config. This\nproperty is <code>null</code> if the component is not framed.</p>\n\n<p>This is an object containing the frame width in pixels for all four sides of the\nComponent containing the following properties:</p>\n"},"horizontalPosProp":{"!type":"string","!doc":"<p>private</p>\n"},"isComponent":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Component, or subclass thereof.</p>\n"},"layoutSuspendCount":{"!type":"number","!doc":"<p>private</p>\n"},"liquidLayout":{"!type":"bool","!doc":"<p>Components that achieve their internal layout results using solely CSS with no JS\nintervention must set this to true.  Failure to set this property to true may result\nin setSize failing to work if the component opted out of the layout run.</p>\n"},"maskOnDisable":{"!type":"bool","!doc":"<p>This is an internal flag that you use when creating custom components. By default this is set to <code>true</code> which means\nthat every component gets a mask when it's disabled. Components like FieldContainer, FieldSet, Field, Button, Tab\noverride this property to <code>false</code> since they want to implement custom disable logic.</p>\n"},"nonMaskableRe":{},"offsetsCls":{"!type":"string","!doc":"<p>private</p>\n"},"ownerCt":{"!doc":"<p>This Component's owner <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a> (is set automatically\nwhen this Component is added to a Container).</p>\n\n<p><em>Important.</em> This is not a universal upwards navigation pointer. It indicates the Container which owns and manages\nthis Component if any. There are other similar relationships such as the <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">button</a> which activates a <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a>, or the\n<a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">menu item</a> which activated a <a href=\"#!/api/Ext.menu.Item-cfg-menu\" rel=\"Ext.menu.Item-cfg-menu\" class=\"docClass\">submenu</a>, or the\n<a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">column header</a> which activated the column menu.</p>\n\n<p>These differences are abstracted away by the <a href=\"#!/api/Ext.Component-method-up\" rel=\"Ext.Component-method-up\" class=\"docClass\">up</a> method.</p>\n\n<p><strong>Note</strong>: to access items within the Container see <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">itemId</a>.</p>\n"},"rendered":{"!type":"bool","!doc":"<p>Indicates whether or not the component has been rendered.</p>\n"},"rootCls":{"!type":"string","!doc":"<p>private</p>\n"},"scrollFlags":{"!doc":"<p>An object property which provides unified information as to which dimensions are scrollable based upon\nthe <a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a>, <a href=\"#!/api/Ext.Component-cfg-overflowX\" rel=\"Ext.Component-cfg-overflowX\" class=\"docClass\">overflowX</a> and <a href=\"#!/api/Ext.Component-cfg-overflowY\" rel=\"Ext.Component-cfg-overflowY\" class=\"docClass\">overflowY</a> settings (And for <em>views</em> of trees and grids, the owning panel's <a href=\"#!/api/Ext.panel.Table-cfg-scroll\" rel=\"Ext.panel.Table-cfg-scroll\" class=\"docClass\">scroll</a> setting).</p>\n\n<p>Note that if you set overflow styles using the <a href=\"#!/api/Ext.Component-cfg-style\" rel=\"Ext.Component-cfg-style\" class=\"docClass\">style</a> config or <a href=\"#!/api/Ext.panel.Panel-cfg-bodyStyle\" rel=\"Ext.panel.Panel-cfg-bodyStyle\" class=\"docClass\">bodyStyle</a> config, this object does not include that information;\nit is best to use <a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a>, <a href=\"#!/api/Ext.Component-cfg-overflowX\" rel=\"Ext.Component-cfg-overflowX\" class=\"docClass\">overflowX</a> and <a href=\"#!/api/Ext.Component-cfg-overflowY\" rel=\"Ext.Component-cfg-overflowY\" class=\"docClass\">overflowY</a> if you need to access these flags.</p>\n\n<p>This object has the following properties:</p>\n"},"scrollerCls":{"!type":"string","!doc":"<p>private</p>\n"},"scrollerSelector":{"!type":"string"},"synthetic":{"!type":"bool","!doc":"<p>This property is <code>true</code> if the component was created internally by the framework\nand is not explicitly user-defined. This is set for such things as <code>Splitter</code>\ninstances managed by <code>border</code> and <code>box</code> layouts.</p>\n"},"validIdRe":{},"zIndexManager":{"!doc":"<p>Only present for <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Components after they have been rendered.</p>\n\n<p>A reference to the ZIndexManager which is managing this Component's z-index.</p>\n\n<p>The <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a> maintains a stack of floating Component z-indices, and also provides\na single modal mask which is insert just beneath the topmost visible modal floating Component.</p>\n\n<p>Floating Components may be <a href=\"#!/api/Ext.Component-method-toFront\" rel=\"Ext.Component-method-toFront\" class=\"docClass\">brought to the front</a> or <a href=\"#!/api/Ext.Component-method-toBack\" rel=\"Ext.Component-method-toBack\" class=\"docClass\">sent to the back</a> of the\nz-index stack.</p>\n\n<p>This defaults to the global <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a> for floating Components that are\nprogramatically <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">rendered</a>.</p>\n\n<p>For <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Components which are added to a Container, the ZIndexManager is acquired from the first\nancestor Container found which is floating. If no floating ancestor is found, the global <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a> is\nused.</p>\n\n<p>See <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> and <a href=\"#!/api/Ext.Component-property-zIndexParent\" rel=\"Ext.Component-property-zIndexParent\" class=\"docClass\">zIndexParent</a></p>\n"},"zIndexParent":{"!doc":"<p>Only present for <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Components which were inserted as child items of Containers, and which have a floating\nContainer in their containment ancestry.</p>\n\n<p>For <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> Components which are child items of a Container, the zIndexParent will be a floating\nancestor Container which is responsible for the base z-index value of all its floating descendants. It provides\na <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a> which provides z-indexing services for all its descendant floating\nComponents.</p>\n\n<p>Floating Components that are programatically <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">rendered</a> will not have a <code>zIndexParent</code>\nproperty.</p>\n\n<p>For example, the dropdown <a href=\"#!/api/Ext.view.BoundList\" rel=\"Ext.view.BoundList\" class=\"docClass\">BoundList</a> of a ComboBox which is in a Window will have the\nWindow as its <code>zIndexParent</code>, and will always show above that Window, wherever the Window is placed in the z-index stack.</p>\n\n<p>See <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> and <a href=\"#!/api/Ext.Component-property-zIndexManager\" rel=\"Ext.Component-property-zIndexManager\" class=\"docClass\">zIndexManager</a></p>\n"},"addClass":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Adds a CSS class to the top level element representing this component.</p>\n"},"addCls":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Adds a CSS class to the top level element representing this component.</p>\n"},"addClsWithUI":{"!type":"fn(classes: ?, skip?: ?) -> !this","!doc":"<p>Adds a <code>cls</code> to the <code>uiCls</code> array, which will also call <a href=\"#!/api/Ext.Component-method-addUIClsToElement\" rel=\"Ext.Component-method-addUIClsToElement\" class=\"docClass\">addUIClsToElement</a> and adds to all elements of this\ncomponent.</p>\n"},"addFocusListener":{"!type":"fn() -> !this","!doc":"<p>Sets up the focus listener on this Component's <a href=\"#!/api/Ext.Component-method-getFocusEl\" rel=\"Ext.Component-method-getFocusEl\" class=\"docClass\">focusEl</a> if it has one.</p>\n\n<p>Form Components which must implicitly participate in tabbing order usually have a naturally focusable\nelement as their <a href=\"#!/api/Ext.Component-method-getFocusEl\" rel=\"Ext.Component-method-getFocusEl\" class=\"docClass\">focusEl</a>, and it is the DOM event of that receiving focus which drives\nthe Component's <code>onFocus</code> handling, and the DOM event of it being blurred which drives the <code>onBlur</code> handling.</p>\n\n<p>If the <a href=\"#!/api/Ext.Component-method-getFocusEl\" rel=\"Ext.Component-method-getFocusEl\" class=\"docClass\">focusEl</a> is <strong>not</strong> naturally focusable, then the listeners are only added\nif the <a href=\"#!/api/Ext.FocusManager\" rel=\"Ext.FocusManager\" class=\"docClass\">FocusManager</a> is enabled.</p>\n"},"addListener":{"!type":"fn(element: ?, listeners: ?, scope: ?, options: ?) -> !this"},"addPlugin":{"!type":"fn(plugin: ?) -> !this","!doc":"<p>Adds a plugin. May be called at any time in the component's lifecycle.</p>\n"},"addPropertyToState":{"!type":"fn(state: ?, propName: ?, value?: ?) -> bool","!doc":"<p>Save a property to the given state object if it is not its default or configured\nvalue.</p>\n"},"addUIToElement":{"!type":"fn() -> !this","!doc":"<p>Method which adds a specified UI to the components element.</p>\n"},"adjustPosition":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>private</p>\n"},"afterSetPosition":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Template method called after a Component has been positioned.</p>\n"},"afterShow":{"!type":"fn(animateTarget?: ?, callback?: ?, scope?: ?) -> !this","!doc":"<p>Invoked after the Component is shown (after <a href=\"#!/api/Ext.Component-method-onShow\" rel=\"Ext.Component-method-onShow\" class=\"docClass\">onShow</a> is called).</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.Component-event-show\" rel=\"Ext.Component-event-show\" class=\"docClass\">show</a>.</p>\n"},"animate":{"!type":"fn(animObj: ?) -> ?","!doc":"<p>Performs custom animation on this object.</p>\n\n<p>This method is applicable to both the <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a> class and the Sprite\nclass. It performs animated transitions of certain properties of this object over a specified timeline.</p>\n\n<h3>Animating a <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a></h3>\n\n<p>When animating a Component, the following properties may be specified in <code>from</code>, <code>to</code>, and <code>keyframe</code> objects:</p>\n\n<ul>\n<li><p><code>x</code> - The Component's page X position in pixels.</p></li>\n<li><p><code>y</code> - The Component's page Y position in pixels</p></li>\n<li><p><code>left</code> - The Component's <code>left</code> value in pixels.</p></li>\n<li><p><code>top</code> - The Component's <code>top</code> value in pixels.</p></li>\n<li><p><code>width</code> - The Component's <code>width</code> value in pixels.</p></li>\n<li><p><code>height</code> - The Component's <code>height</code> value in pixels.</p></li>\n<li><p><code>dynamic</code> - Specify as true to update the Component's layout (if it is a Container) at every frame of the animation.\n<em>Use sparingly as laying out on every intermediate size change is an expensive operation.</em></p></li>\n</ul>\n\n\n<p>For example, to animate a Window to a new size, ensuring that its internal layout and any shadow is correct:</p>\n\n<pre><code>myWindow = Ext.create('<a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>', {\n    title: 'Test Component animation',\n    width: 500,\n    height: 300,\n    layout: {\n        type: 'hbox',\n        align: 'stretch'\n    },\n    items: [{\n        title: 'Left: 33%',\n        margin: '5 0 5 5',\n        flex: 1\n    }, {\n        title: 'Left: 66%',\n        margin: '5 5 5 5',\n        flex: 2\n    }]\n});\nmyWindow.show();\nmyWindow.header.el.on('click', function() {\n    myWindow.animate({\n        to: {\n            width: (myWindow.getWidth() == 500) ? 700 : 500,\n            height: (myWindow.getHeight() == 300) ? 400 : 300\n        }\n    });\n});\n</code></pre>\n\n<p>For performance reasons, by default, the internal layout is only updated when the Window reaches its final <code>\"to\"</code>\nsize. If dynamic updating of the Window's child Components is required, then configure the animation with\n<code>dynamic: true</code> and the two child items will maintain their proportions during the animation.</p>\n"},"applyData":{"!type":"fn(data: ?) -> !this"},"beforeBlur":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method to do any pre-blur processing.</p>\n"},"beforeComponentLayout":{"!type":"fn(adjWidth: ?, adjHeight: ?) -> !this","!doc":"<p>Occurs before <code>componentLayout</code> is run. Returning <code>false</code> from this method will prevent the <code>componentLayout</code> from\nbeing executed.</p>\n"},"beforeFocus":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method to do any pre-focus processing.</p>\n"},"beforeInitConfig":{"!type":"fn() -> !this"},"beforeLayout":{"!type":"fn() -> !this","!doc":"<p>Occurs before componentLayout is run. In previous releases, this method could\nreturn <code>false</code> to prevent its layout but that is not supported in Ext JS 4.1 or\nhigher. This method is simply a notification of the impending layout to give the\ncomponent a chance to adjust the DOM. Ideally, DOM reads should be avoided at this\ntime to reduce expensive document reflows.</p>\n"},"beforeSetPosition":{"!type":"fn(x: ?, y: ?, animate: ?) -> !this","!doc":"<p>Template method called before a Component is positioned.</p>\n\n<p>Ensures that the position is adjusted so that the Component is constrained if so configured.</p>\n"},"beforeShow":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is shown.</p>\n"},"bubble":{"!type":"fn(fn: ?, scope?: ?, args?: ?) -> ?","!doc":"<p>Bubbles up the component/container hierarchy, calling the specified function with each component. The scope\n(<em>this</em>) of function call will be the scope provided or the current component. The arguments to the function will\nbe the args provided or the current component. If the function returns false at any point, the bubble is stopped.</p>\n"},"cancelFocus":{"!type":"fn() -> !this","!doc":"<p>Cancel any deferred focus on this component</p>\n"},"cloneConfig":{"!type":"fn(overrides: ?) -> ?","!doc":"<p>Clone the current component using the original config values passed into this instance by default.</p>\n"},"constructPlugin":{"!type":"fn(ptype: ?) -> !this"},"constructPlugins":{"!type":"fn() -> !this","!doc":"<p>Returns an array of fully constructed plugin instances. This converts any configs into their\nappropriate instances.</p>\n\n<p>It does not mutate the plugins array. It creates a new array.</p>\n"},"destroy":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component is <a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a>ed.</p>\n"},"doComponentLayout":{"!type":"fn() -> ?","!doc":"<p>This method needs to be called whenever you change something on this component that\nrequires the Component's layout to be recalculated.</p>\n"},"findComponentByElement":{"!type":"fn(node: ?) -> !this","!doc":"<p>Walk the DOM tree upwards and find the Component these elements belong to.</p>\n"},"findParentBy":{"!type":"fn(fn: ?) -> ?","!doc":"<p>Find a container above this component at any level by a custom function. If the passed function returns true, the\ncontainer will be returned.</p>\n\n<p>See also the <a href=\"#!/api/Ext.Component-method-up\" rel=\"Ext.Component-method-up\" class=\"docClass\">up</a> method.</p>\n"},"findParentByType":{"!type":"fn(xtype: ?) -> ?","!doc":"<p>Find a container above this component at any level by xtype or class</p>\n\n<p>See also the <a href=\"#!/api/Ext.Component-method-up\" rel=\"Ext.Component-method-up\" class=\"docClass\">up</a> method.</p>\n"},"findPlugin":{"!type":"fn(ptype: ?) -> ?","!doc":"<p>Retrieves plugin from this component's collection by its <code>ptype</code>.</p>\n"},"fireHierarchyEvent":{"!type":"fn(eventName: ?) -> !this","!doc":"<p>This method fires an event on <code>Ext.GlobalEvents</code> allowing interested parties to know\nof certain critical events for this component. This is done globally because the\n(few) listeners can immediately receive the event rather than bubbling the event\nonly to reach the top and have no listeners.</p>\n\n<p>The main usage for these events is to do with floating components. For example, the\nload mask is a floating component. The component it is masking may be inside several\ncontainers. As such, they need to know when component is hidden, either directly, or\nvia a parent container being hidden. To do this they subscribe to these events and\nfilter out the appropriate container.</p>\n\n<p>This functionality is contained in Component (as opposed to Container) because a\nComponent can be the ownerCt for a floating component (loadmask), and the loadmask\nneeds to know when its owner is shown/hidden so that its hidden state can be\nsynchronized.</p>\n"},"focus":{"!type":"fn(this: ?, The: ?, eOpts: ?)","!doc":"<p>Fires when this Component receives focus.</p>\n"},"forceComponentLayout":{"!type":"fn(options?: ?) -> !this","!doc":"<p>Updates this component's layout. If this update affects this components <a href=\"#!/api/Ext.Component-property-ownerCt\" rel=\"Ext.Component-property-ownerCt\" class=\"docClass\">ownerCt</a>,\nthat component's <code>updateLayout</code> method will be called to perform the layout instead.\nOtherwise, just this component (and its child items) will layout.</p>\n"},"getActiveComponent":{"!type":"fn() -> ?","!doc":"<p>Return the currently active (focused) Component</p>\n"},"getAnchorToXY":{"!type":"fn(el: ?, anchor: ?, local: ?, mySize: ?) -> !this","!doc":"<p><editor-fold desc=\"Positionable Methods\"></editor></p>\n\n<hr />\n\n<p> Begin Positionable methods</p>\n\n<hr />\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"getAnimateTarget":{"!type":"fn(target: ?) -> !this"},"getAutoId":{"!type":"fn() -> !this"},"getBorderPadding":{"!type":"fn() -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"getBubbleTarget":{"!type":"fn() -> !this","!doc":"<p>Implements an upward event bubbling policy. By default a Component bubbles events up to its reference owner.</p>\n\n<p>Component subclasses may implement a different bubbling strategy by overriding this method.</p>\n"},"getComponentByElement":{"!type":"fn(el: ?) -> ?","!doc":"<p>Find a Component that the given Element belongs to.</p>\n"},"getComponentLayout":{"!type":"fn() -> !this"},"getData":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.Component-cfg-data\" rel=\"Ext.Component-cfg-data\" class=\"docClass\">data</a>.</p>\n"},"getEl":{"!type":"fn() -> ?","!doc":"<p>Retrieves the top level element representing this component.</p>\n"},"getHeight":{"!type":"fn() -> number","!doc":"<p>Gets the current height of the component's underlying element.</p>\n"},"getId":{"!type":"fn() -> string","!doc":"<p>Retrieves the <code>id</code> of this component. Will auto-generate an <code>id</code> if one has not already been set.</p>\n"},"getItemId":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">itemId</a> assigned to this component, or when that\nis not set, returns the value of <a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">id</a>.</p>\n"},"getLoader":{"!type":"fn() -> ?","!doc":"<p>Gets the <a href=\"#!/api/Ext.ComponentLoader\" rel=\"Ext.ComponentLoader\" class=\"docClass\">Ext.ComponentLoader</a> for this Component.</p>\n"},"getLocalX":{"!type":"fn() -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"getLocalXY":{"!type":"fn() -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"getLocalY":{"!type":"fn() -> !this"},"getMaskTarget":{"!type":"fn() -> !this","!doc":"<p>Returns the element which is masked by the <a href=\"#!/api/Ext.Component-method-mask\" rel=\"Ext.Component-method-mask\" class=\"docClass\">mask</a> method, or into which the <a href=\"#!/api/Ext.Component-method-setLoading\" rel=\"Ext.Component-method-setLoading\" class=\"docClass\">LoadMask</a> is rendered into.</p>\n\n<p>The default implementation uses the <a href=\"#!/api/Ext.Component-cfg-maskElement\" rel=\"Ext.Component-cfg-maskElement\" class=\"docClass\">maskElement</a> configuration to access the Component's child element by name. By default, <a href=\"#!/api/Ext.Component-cfg-maskElement\" rel=\"Ext.Component-cfg-maskElement\" class=\"docClass\">maskElement</a>\nis <code>null</code> which means that <code>null</code> is returned from this method indicating that the mask needs to be rendered into the document because\ncomponent structure should not be contaminated by mask elements.</p>\n\n<p>Some subclasses may override this method if they have knowledge about external structures where a mask could usefully be rendered.</p>\n\n<p>For example a <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">GridView</a> will request that its owning <a href=\"#!/api/Ext.panel.Table\" rel=\"Ext.panel.Table\" class=\"docClass\">GridPanel</a> be masked. The\nGridPanel will have its own implementation of <code>getMaskTarget</code> which will return the element dicated by its own <a href=\"#!/api/Ext.Component-cfg-maskElement\" rel=\"Ext.Component-cfg-maskElement\" class=\"docClass\">maskElement</a>\nPanels use <code>\"el\"</code> as their <a href=\"#!/api/Ext.Component-cfg-maskElement\" rel=\"Ext.Component-cfg-maskElement\" class=\"docClass\">maskElement</a> by default, but that could be overridden to be <code>\"body\"</code> to leave toolbars and the header\nmouse-accessible.</p>\n"},"getOverflowEl":{"!type":"fn() -> !this","!doc":"<p>Get an el for overflowing, defaults to the target el</p>\n"},"getOverflowStyle":{"!type":"fn() -> !this","!doc":"<p>Returns the CSS style object which will set the Component's scroll styles. This must be applied\nto the <a href=\"#!/api/Ext.Component-method-getTargetEl\" rel=\"Ext.Component-method-getTargetEl\" class=\"docClass\">target element</a>.</p>\n"},"getOwningBorderContainer":{"!type":"fn() -> ?","!doc":"<p>Returns the owning container if that container uses <code>border</code> layout. Otherwise\nthis method returns <code>null</code>.</p>\n\n<p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>\n"},"getOwningBorderLayout":{"!type":"fn() -> ?","!doc":"<p>Returns the owning <code>border</code> (<code><a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Ext.layout.container.Border</a></code>) instance if there is\none. Otherwise this method returns <code>null</code>.</p>\n\n<p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>\n"},"getPlugin":{"!type":"fn(pluginId: ?) -> ?","!doc":"<p>Retrieves a plugin from this component's collection by its <code>pluginId</code>.</p>\n"},"getPlugins":{"!type":"fn() -> !this","!doc":"<p>Returns an array of current fully constructed plugin instances.</p>\n"},"getPosition":{"!type":"fn(local?: ?) -> ?","!doc":"<p>Gets the current XY position of the component's underlying element.</p>\n"},"getScrollX":{"!type":"fn() -> number","!doc":"<p>Returns the \"x\" scroll position for this component.  Only applicable for components\nthat have been configured with <code><a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a></code> or <code><a href=\"#!/api/Ext.Component-cfg-overflowX\" rel=\"Ext.Component-cfg-overflowX\" class=\"docClass\">overflowX</a></code>.</p>\n"},"getScrollY":{"!type":"fn() -> number","!doc":"<p>Returns the \"y\" scroll position for this component.  Only applicable for components\nthat have been configured with <code><a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a></code> or <code><a href=\"#!/api/Ext.Component-cfg-overflowY\" rel=\"Ext.Component-cfg-overflowY\" class=\"docClass\">overflowY</a></code>.</p>\n"},"getSize":{"!type":"fn(contentSize?: ?) -> ?","!doc":"<p>Gets the current size of the component's underlying element.</p>\n"},"getSizeModel":{"!type":"fn(ownerCtSizeModel: ?) -> ?","!doc":"<p>Returns an object that describes how this component's width and height are managed.\nAll of these objects are shared and should not be modified.</p>\n"},"getTargetEl":{"!type":"fn() -> !this","!doc":"<p>This is used to determine where to insert the 'html', 'contentEl' and 'items' in this component.</p>\n"},"getTdCls":{"!type":"fn() -> !this","!doc":"<p>Needed for when widget is rendered into a grid cell. The class to add to the cell element.</p>\n"},"getTdType":{"!type":"fn() -> !this","!doc":"<p>Partner method to <a href=\"#!/api/Ext.Component-method-getTdCls\" rel=\"Ext.Component-method-getTdCls\" class=\"docClass\">getTdCls</a>.</p>\n\n<p>Returns the base type for the component. Defaults to return <code>this.xtype</code>, but\nAll derived classes of <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">TextField</a> can return the type 'textfield',\nand all derived classes of <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Button</a> can return the type 'button'</p>\n"},"getTpl":{"!type":"fn(name: ?) -> !this"},"getWidth":{"!type":"fn() -> number","!doc":"<p>Gets the current width of the component's underlying element.</p>\n"},"getX":{"!type":"fn() -> !this"},"getXType":{"!type":"fn() -> string","!doc":"<p>Gets the xtype for this component as registered with Ext.ComponentManager. For a list of all available\nxtypes, see the <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> header. Example usage:</p>\n\n<pre><code>var t = new <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>();\nalert(t.getXType());  // alerts 'textfield'\n</code></pre>\n"},"getXTypes":{"!type":"fn() -> string","!doc":"<p>Returns this Component's xtype hierarchy as a slash-delimited string. For a list of all available xtypes, see the\n<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> header.</p>\n\n<p><strong>If using your own subclasses, be aware that a Component must register its own xtype to participate in\ndetermination of inherited xtypes.</strong></p>\n\n<p>Example usage:</p>\n\n<pre class='inline-example '><code>var t = new <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>();\nalert(t.getXTypes());  // alerts 'component/field/textfield'\n</code></pre>\n"},"getXY":{"!type":"fn() -> !this"},"getY":{"!type":"fn() -> !this"},"hasCls":{"!type":"fn(className: ?) -> bool","!doc":"<p>Checks if the specified CSS class exists on this element's DOM node.</p>\n"},"hasUICls":{"!type":"fn(cls: ?) -> !this","!doc":"<p>Checks if there is currently a specified <code>uiCls</code>.</p>\n"},"initBorderRegion":{"!type":"fn() -> !this","!doc":"<p>This method is called by the <code><a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Ext.layout.container.Border</a></code> class when instances are\nadded as regions to the layout. Since it is valid to add any component to a border\nlayout as a region, this method must be added to <code><a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a></code> but is only ever\ncalled when that component is owned by a <code>border</code> layout.</p>\n\n<p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>Initialize any events on this component</p>\n"},"initPadding":{"!type":"fn(targetEl: ?) -> !this","!doc":"<p>Initializes padding by applying it to the target element, or if the layout manages\npadding ensures that the padding on the target element is \"0\".</p>\n"},"initStyles":{"!type":"fn(targetEl: ?) -> !this","!doc":"<p>Applies padding, margin, border, top, left, height, and width configs to the\nappropriate elements.</p>\n"},"is":{"!type":"fn(selector: ?) -> bool","!doc":"<p>Tests whether this Component matches a ComponentQuery\nselector string.</p>\n"},"isDescendantOf":{"!type":"fn(container: ?) -> bool","!doc":"<p>Determines whether this component is the descendant of a particular container.</p>\n"},"isDisabled":{"!type":"fn() -> bool","!doc":"<p>Method to determine whether this Component is currently disabled.</p>\n"},"isDraggable":{"!type":"fn() -> bool","!doc":"<p>Method to determine whether this Component is draggable.</p>\n"},"isDroppable":{"!type":"fn() -> bool","!doc":"<p>Method to determine whether this Component is droppable.</p>\n"},"isFloating":{"!type":"fn() -> bool","!doc":"<p>Method to determine whether this Component is floating.</p>\n"},"isFocusable":{"!type":"fn() -> !this"},"isHidden":{"!type":"fn() -> bool","!doc":"<p>Method to determine whether this Component is currently set to hidden.</p>\n"},"isHierarchicallyHidden":{"!type":"fn() -> !this"},"isLayoutSuspended":{"!type":"fn() -> bool","!doc":"<p>Returns <code>true</code> if layout is suspended for this component. This can come from direct\nsuspension of this component's layout activity (<a href=\"#!/api/Ext.container.Container-cfg-suspendLayout\" rel=\"Ext.container.Container-cfg-suspendLayout\" class=\"docClass\">Ext.Container.suspendLayout</a>) or if one\nof this component's containers is suspended.</p>\n"},"isLocalRtl":{"!type":"fn() -> bool","!doc":"<p>Returns true if this component's local coordinate system is rtl. For normal\ncomponents this equates to the value of isParentRtl().  Floaters are a bit different\nbecause a floater's element can be a childNode of something other than its\nparent component's element.  For floaters we have to read the dom to see if the\ncomponent's element's parentNode has a css direction value of \"rtl\".</p>\n\n<p><strong>Defined in override Ext.rtl.Component.</strong></p>\n"},"isOppositeRootDirection":{"!type":"fn() -> !this","!doc":"<p><strong>Defined in override Ext.rtl.Component.</strong></p>\n"},"isParentRtl":{"!type":"fn() -> bool","!doc":"<p>Returns true if this component's parent container is rtl. Used by rtl positioning\nmethods to determine if the component should be positioned using a right-to-left\ncoordinate system.</p>\n\n<p><strong>Defined in override Ext.rtl.Component.</strong></p>\n"},"isXType":{"!type":"fn(xtype: ?, shallow?: ?) -> bool","!doc":"<p>Tests whether or not this Component is of a specific xtype. This can test whether this Component is descended\nfrom the xtype (default) or whether it is directly of the xtype specified (<code>shallow = true</code>).</p>\n\n<p><strong>If using your own subclasses, be aware that a Component must register its own xtype to participate in\ndetermination of inherited xtypes.</strong></p>\n\n<p>For a list of all available xtypes, see the <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> header.</p>\n\n<p>Example usage:</p>\n\n<pre class='inline-example '><code>var t = new <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>();\nvar isText = t.isXType('textfield');        // true\nvar isBoxSubclass = t.isXType('field');       // true, descended from <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a>\nvar isBoxInstance = t.isXType('field', true); // false, not a direct <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a> instance\n</code></pre>\n"},"mask":{"!type":"fn(msg?: ?, msgCls?: ?) -> !this","!doc":"<p>Masks this component with a semi-opaque layer and makes the contents unavailable to clicks.</p>\n\n<p>See <a href=\"#!/api/Ext.Component-method-unmask\" rel=\"Ext.Component-method-unmask\" class=\"docClass\">unmask</a>.</p>\n"},"nextNode":{"!type":"fn(selector?: ?) -> ?","!doc":"<p>Returns the next node in the Component tree in tree traversal order.</p>\n\n<p>Note that this is not limited to siblings, and if invoked upon a node with no matching siblings, will walk the\ntree to attempt to find a match. Contrast with <a href=\"#!/api/Ext.Component-method-nextSibling\" rel=\"Ext.Component-method-nextSibling\" class=\"docClass\">nextSibling</a>.</p>\n"},"nextSibling":{"!type":"fn(selector?: ?) -> ?","!doc":"<p>Returns the next sibling of this Component.</p>\n\n<p>Optionally selects the next sibling which matches the passed ComponentQuery selector.</p>\n\n<p>May also be referred to as <strong><code>next()</code></strong></p>\n\n<p>Note that this is limited to siblings, and if no siblings of the item match, <code>null</code> is returned. Contrast with\n<a href=\"#!/api/Ext.Component-method-nextNode\" rel=\"Ext.Component-method-nextNode\" class=\"docClass\">nextNode</a></p>\n"},"onBlur":{"!type":"fn(e: ?) -> !this","!doc":"<p>private</p>\n"},"onDisable":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the disable operation.\nAfter calling the superclass's <code>onDisable</code>, the Component will be disabled.</p>\n"},"onEnable":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the enable operation.\nAfter calling the superclass's <code>onEnable</code>, the Component will be enabled.</p>\n"},"onShowComplete":{"!type":"fn(callback?: ?, scope?: ?) -> !this","!doc":"<p>Invoked after the <a href=\"#!/api/Ext.Component-method-afterShow\" rel=\"Ext.Component-method-afterShow\" class=\"docClass\">afterShow</a> method is complete.</p>\n\n<p>Gets passed the same <code>callback</code> and <code>scope</code> parameters that <a href=\"#!/api/Ext.Component-method-afterShow\" rel=\"Ext.Component-method-afterShow\" class=\"docClass\">afterShow</a> received.</p>\n"},"onShowVeto":{"!type":"fn() -> !this"},"owns":{"!type":"fn(element: ?) -> !this","!doc":"<p>Returns <code>true</code> if the passed element is within the container tree of this component.</p>\n\n<p>For example if a menu's submenu contains an <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, that top level\nmenu owns the elements of the date picker. Using this method, you can tell if an event took place\nwithin a certain component tree.</p>\n"},"postBlur":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method to do any post-blur processing.</p>\n"},"previousNode":{"!type":"fn(selector?: ?) -> ?","!doc":"<p>Returns the previous node in the Component tree in tree traversal order.</p>\n\n<p>Note that this is not limited to siblings, and if invoked upon a node with no matching siblings, will walk the\ntree in reverse order to attempt to find a match. Contrast with <a href=\"#!/api/Ext.Component-method-previousSibling\" rel=\"Ext.Component-method-previousSibling\" class=\"docClass\">previousSibling</a>.</p>\n"},"previousSibling":{"!type":"fn(selector?: ?) -> ?","!doc":"<p>Returns the previous sibling of this Component.</p>\n\n<p>Optionally selects the previous sibling which matches the passed ComponentQuery\nselector.</p>\n\n<p>May also be referred to as <strong><code>prev()</code></strong></p>\n\n<p>Note that this is limited to siblings, and if no siblings of the item match, <code>null</code> is returned. Contrast with\n<a href=\"#!/api/Ext.Component-method-previousNode\" rel=\"Ext.Component-method-previousNode\" class=\"docClass\">previousNode</a></p>\n"},"refreshScroll":{"!type":"fn() -> !this","!doc":"<p>Implementation which updates the scroll range of a touch scroller.\nSubclasses may change implementation.</p>\n"},"registerFloatingItem":{"!type":"fn(cmp: ?) -> !this","!doc":"<p>Called by Component#doAutoRender</p>\n\n<p>Register a Container configured <code>floating: true</code> with this Component's <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a>.</p>\n\n<p>Components added in this way will not participate in any layout, but will be rendered\nupon first show in the way that <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Window</a>s are.</p>\n"},"removeClass":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Removes a CSS class from the top level element representing this component.</p>\n"},"removeCls":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Removes a CSS class from the top level element representing this component.</p>\n"},"removeClsWithUI":{"!type":"fn(cls: ?) -> !this","!doc":"<p>Removes a <code>cls</code> to the <code>uiCls</code> array, which will also call <a href=\"#!/api/Ext.Component-method-removeUIClsFromElement\" rel=\"Ext.Component-method-removeUIClsFromElement\" class=\"docClass\">removeUIClsFromElement</a> and removes it from all\nelements of this component.</p>\n"},"removeUIFromElement":{"!type":"fn() -> !this","!doc":"<p>Method which removes a specified UI from the components element.</p>\n"},"resumeLayouts":{"!type":"fn(flushOptions: ?) -> !this"},"scrollBy":{"!type":"fn(deltaX: ?, deltaY: ?, animate: ?) -> !this","!doc":"<p>Scrolls this Component by the passed delta values, optionally animating.</p>\n\n<p>All of the following are equivalent:</p>\n\n<pre><code> comp.scrollBy(10, 10, true);\n comp.scrollBy([10, 10], true);\n comp.scrollBy({ x: 10, y: 10 }, true);\n</code></pre>\n"},"scrollTo":{"!type":"fn(x: ?, y: ?, animate?: ?) -> !this","!doc":"<p>Scrolls this component to the specified <code>x</code> and <code>y</code> coordinates.  Only applicable\nfor components that have been configured with <code><a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a></code> or\n<code><a href=\"#!/api/Ext.Component-cfg-overflowX\" rel=\"Ext.Component-cfg-overflowX\" class=\"docClass\">overflowX</a></code> and <code><a href=\"#!/api/Ext.Component-cfg-overflowY\" rel=\"Ext.Component-cfg-overflowY\" class=\"docClass\">overflowY</a></code>.</p>\n"},"setAutoScroll":{"!type":"fn(scroll: ?) -> ?","!doc":"<p>Sets the overflow on the content element of the component.</p>\n"},"setData":{"!type":"fn(data: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.Component-cfg-data\" rel=\"Ext.Component-cfg-data\" class=\"docClass\">data</a>.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?) -> !this","!doc":"<p>Enable or disable the component.</p>\n"},"setDock":{"!type":"fn(dock: ?) -> ?","!doc":"<p>Sets the dock position of this component in its parent panel. Note that this only has effect if this item is part\nof the <code>dockedItems</code> collection of a parent that has a DockLayout (note that any Panel has a DockLayout by default)</p>\n"},"setDocked":{"!type":"fn(dock: ?) -> ?","!doc":"<p>Sets the dock position of this component in its parent panel. Note that this only has effect if this item is part\nof the <code>dockedItems</code> collection of a parent that has a DockLayout (note that any Panel has a DockLayout by default)</p>\n"},"setFlex":{"!type":"fn(flex: ?) -> !this","!doc":"<p>Sets the flex property of this component. Only applicable when this component is\nan item of a box layout</p>\n"},"setHeight":{"!type":"fn(height: ?) -> ?","!doc":"<p>Sets the height of the component. This method fires the <a href=\"#!/api/Ext.Component-event-resize\" rel=\"Ext.Component-event-resize\" class=\"docClass\">resize</a> event.</p>\n"},"setHidden":{"!type":"fn(hidden: ?) -> ?","!doc":"<p>Sets the hidden state of this component. This is basically the same as\n<code><a href=\"#!/api/Ext.Component-method-setVisible\" rel=\"Ext.Component-method-setVisible\" class=\"docClass\">setVisible</a></code> but the boolean parameter has the opposite meaning.</p>\n"},"setHtml":{"!type":"fn(html: ?) -> !this"},"setLoading":{"!type":"fn(load: ?) -> ?","!doc":"<p>This method allows you to show or hide a LoadMask on top of this component.</p>\n\n<p>The mask will be rendered into the element returned by <a href=\"#!/api/Ext.Component-method-getMaskTarget\" rel=\"Ext.Component-method-getMaskTarget\" class=\"docClass\">getMaskTarget</a> which for most Components is the Component's\nelement. See <a href=\"#!/api/Ext.Component-method-getMaskTarget\" rel=\"Ext.Component-method-getMaskTarget\" class=\"docClass\">getMaskTarget</a> and <a href=\"#!/api/Ext.Component-cfg-maskElement\" rel=\"Ext.Component-cfg-maskElement\" class=\"docClass\">maskElement</a>.</p>\n\n<p>Most Components will return <code>null</code> indicating that their LoadMask cannot reside inside their element, but must\nbe rendered into the document body.</p>\n\n<p><a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">Grid Views</a> however will direct a LoadMask to be rendered into the owning <a href=\"#!/api/Ext.panel.Table\" rel=\"Ext.panel.Table\" class=\"docClass\">GridPanel</a>.</p>\n"},"setLocalX":{"!type":"fn(x: ?) -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"setLocalXY":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"setLocalY":{"!type":"fn(y: ?) -> !this"},"setMargin":{"!type":"fn(margin: ?) -> !this","!doc":"<p>Sets the margin on the target element.</p>\n"},"setOverflowXY":{"!type":"fn(overflowX: ?, overflowY: ?) -> ?","!doc":"<p>Sets the overflow x/y on the content element of the component. The x/y overflow\nvalues can be any valid CSS overflow (e.g., 'auto' or 'scroll'). By default, the\nvalue is 'hidden'. Passing null for one of the values will erase the inline style.\nPassing <code>undefined</code> will preserve the current value.</p>\n"},"setPagePosition":{"!type":"fn(x: ?, y?: ?, animate?: ?) -> ?","!doc":"<p>Sets the page XY position of the component. To set the left and top instead, use <a href=\"#!/api/Ext.Component-method-setPosition\" rel=\"Ext.Component-method-setPosition\" class=\"docClass\">setPosition</a>.\nThis method fires the <a href=\"#!/api/Ext.Component-event-move\" rel=\"Ext.Component-event-move\" class=\"docClass\">move</a> event.</p>\n"},"setPosition":{"!type":"fn(x: ?, y?: ?, animate?: ?) -> ?","!doc":"<p>Sets the left and top of the component. To set the page XY position instead, use <a href=\"#!/api/Ext.Component-method-setPagePosition\" rel=\"Ext.Component-method-setPagePosition\" class=\"docClass\">setPagePosition</a>. This\nmethod fires the <a href=\"#!/api/Ext.Component-event-move\" rel=\"Ext.Component-event-move\" class=\"docClass\">move</a> event.</p>\n"},"setRegion":{"!type":"fn(region: ?) -> string","!doc":"<p>This method changes the <code>region</code> config property for this border region. This is\nonly valid if this component is in a <code>border</code> layout (<code><a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Ext.layout.container.Border</a></code>).</p>\n\n<p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>\n"},"setScrollX":{"!type":"fn(x: ?) -> !this","!doc":"<p>Sets the \"x\" scroll position for this component.  Only applicable for components\nthat have been configured with <code><a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a></code> or <code><a href=\"#!/api/Ext.Component-cfg-overflowX\" rel=\"Ext.Component-cfg-overflowX\" class=\"docClass\">overflowX</a></code>.</p>\n"},"setScrollY":{"!type":"fn(y: ?) -> !this","!doc":"<p>Sets the \"y\" scroll position for this component.  Only applicable for components\nthat have been configured with <code><a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a></code> or <code><a href=\"#!/api/Ext.Component-cfg-overflowY\" rel=\"Ext.Component-cfg-overflowY\" class=\"docClass\">overflowY</a></code>.</p>\n"},"setSize":{"!type":"fn(width: ?, height: ?) -> ?","!doc":"<p>Sets the width and height of this Component. This method fires the <a href=\"#!/api/Ext.Component-event-resize\" rel=\"Ext.Component-event-resize\" class=\"docClass\">resize</a> event. This method can accept\neither width and height as separate arguments, or you can pass a size object like <code>{width:10, height:20}</code>.</p>\n"},"setStyle":{"!type":"fn(style: ?) -> !this","!doc":"<p>Sets the stlye for this Component's primary element.</p>\n"},"setVisible":{"!type":"fn(visible: ?) -> ?","!doc":"<p>Convenience function to hide or show this component by Boolean.</p>\n"},"setWeight":{"!type":"fn(weight: ?) -> number","!doc":"<p>Sets the <code>weight</code> config property for this component. This is only valid if this\ncomponent is in a <code>border</code> layout (<code><a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Ext.layout.container.Border</a></code>).</p>\n\n<p><strong>Defined in override Ext.layout.container.border.Region.</strong></p>\n"},"setWidth":{"!type":"fn(width: ?) -> ?","!doc":"<p>Sets the width of the component. This method fires the <a href=\"#!/api/Ext.Component-event-resize\" rel=\"Ext.Component-event-resize\" class=\"docClass\">resize</a> event.</p>\n"},"setX":{"!type":"fn(x: ?, animate: ?) -> !this"},"setXY":{"!type":"fn(xy: ?, animate: ?) -> !this"},"setY":{"!type":"fn(y: ?, animate: ?) -> !this"},"showAt":{"!type":"fn(x: ?, y?: ?, animate?: ?) -> ?","!doc":"<p>Displays component at specific xy position.\nA floating component (like a menu) is positioned relative to its ownerCt if any.\nUseful for popping up a context menu:</p>\n\n<pre><code>listeners: {\n    itemcontextmenu: function(view, record, item, index, event, options) {\n        Ext.create('<a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a>', {\n            width: 100,\n            height: 100,\n            margin: '0 0 10 0',\n            items: [{\n                text: 'regular item 1'\n            },{\n                text: 'regular item 2'\n            },{\n                text: 'regular item 3'\n            }]\n        }).showAt(event.getXY());\n    }\n}\n</code></pre>\n"},"showBy":{"!type":"fn(component: ?, position?: ?, offsets?: ?) -> ?","!doc":"<p>Shows this component by the specified <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a> or Element.\nUsed when this component is <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a>.</p>\n"},"suspendLayouts":{"!type":"fn() -> !this"},"unitizeBox":{"!type":"fn(box: ?) -> !this","!doc":"<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"unmask":{"!type":"fn() -> !this","!doc":"<p>Removes the mask applied by <a href=\"#!/api/Ext.Component-method-mask\" rel=\"Ext.Component-method-mask\" class=\"docClass\">mask</a></p>\n"},"unregisterFloatingItem":{"!type":"fn(cmp: ?) -> !this"},"up":{"!type":"fn(selector?: ?, limit?: ?) -> ?","!doc":"<p>Navigates up the ownership hierarchy searching for an ancestor Container which matches any passed selector or component.</p>\n\n<p><em>Important.</em> There is not a universal upwards navigation pointer. There are several upwards relationships\nsuch as the <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">button</a> which activates a <a href=\"#!/api/Ext.button.Button-cfg-menu\" rel=\"Ext.button.Button-cfg-menu\" class=\"docClass\">menu</a>, or the\n<a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">menu item</a> which activated a <a href=\"#!/api/Ext.menu.Item-cfg-menu\" rel=\"Ext.menu.Item-cfg-menu\" class=\"docClass\">submenu</a>, or the\n<a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">column header</a> which activated the column menu.</p>\n\n<p>These differences are abstracted away by this method.</p>\n\n<p>Example:</p>\n\n<pre><code>var owningTabPanel = grid.up('tabpanel');\n</code></pre>\n"},"update":{"!type":"fn(htmlOrData: ?, loadScripts?: ?, callback?: ?) -> !this","!doc":"<p>Update the content area of a component.</p>\n"},"updateBox":{"!type":"fn(box: ?) -> ?","!doc":"<p>Sets the current box measurements of the component's underlying element.</p>\n"},"updateLayout":{"!type":"fn(options?: ?) -> !this","!doc":"<p>Updates this component's layout. If this update affects this components <a href=\"#!/api/Ext.Component-property-ownerCt\" rel=\"Ext.Component-property-ownerCt\" class=\"docClass\">ownerCt</a>,\nthat component's <code>updateLayout</code> method will be called to perform the layout instead.\nOtherwise, just this component (and its child items) will layout.</p>\n"},"added":{"!type":"fn(this: ?, container: ?, pos: ?, eOpts: ?)","!doc":"<p>Fires after a Component had been added to a Container.</p>\n"},"afterrender":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component rendering is finished.</p>\n\n<p>The <code>afterrender</code> event is fired after this Component has been <a href=\"#!/api/Ext.Component-property-rendered\" rel=\"Ext.Component-property-rendered\" class=\"docClass\">rendered</a>, been postprocessed by any\n<code>afterRender</code> method defined for the Component.</p>\n"},"beforeactivate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before a Component has been visually activated. Returning <code>false</code> from an event listener can prevent\nthe activate from occurring.</p>\n\n<p><strong>Note</strong> This event is only fired if this Component is a child of a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>\nthat uses <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> as it's layout.</p>\n"},"beforedeactivate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before a Component has been visually deactivated. Returning <code>false</code> from an event listener can\nprevent the deactivate from occurring.</p>\n\n<p><strong>Note</strong> This event is only fired if this Component is a child of a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>\nthat uses <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> as it's layout.</p>\n"},"beforedestroy":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before the component is <a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a>ed. Return <code>false</code> from an event handler to stop the\n<a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a>.</p>\n"},"beforehide":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before the component is hidden when calling the <a href=\"#!/api/Ext.Component-method-hide\" rel=\"Ext.Component-method-hide\" class=\"docClass\">hide</a> method. Return <code>false</code> from an event\nhandler to stop the hide.</p>\n"},"beforerender":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before the component is <a href=\"#!/api/Ext.Component-property-rendered\" rel=\"Ext.Component-property-rendered\" class=\"docClass\">rendered</a>. Return <code>false</code> from an event handler to stop the\n<a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">render</a>.</p>\n"},"beforeshow":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before the component is shown when calling the <a href=\"#!/api/Ext.Component-method-show\" rel=\"Ext.Component-method-show\" class=\"docClass\">show</a> method. Return <code>false</code> from an event\nhandler to stop the show.</p>\n"},"blur":{"!type":"fn(this: ?, The: ?, eOpts: ?)","!doc":"<p>Fires when this Component loses focus.</p>\n"},"boxready":{"!type":"fn(this: ?, width: ?, height: ?, eOpts: ?)","!doc":"<p>Fires <em>one time</em> - after the component has been laid out for the first time at its initial size.</p>\n"},"disable":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component is disabled.</p>\n"},"enable":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component is enabled.</p>\n"},"hide":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component is hidden. Fires after the component is hidden when calling the <a href=\"#!/api/Ext.Component-method-hide\" rel=\"Ext.Component-method-hide\" class=\"docClass\">hide</a>\nmethod.</p>\n"},"removed":{"!type":"fn(this: ?, ownerCt: ?, eOpts: ?)","!doc":"<p>Fires when a component is removed from an <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a></p>\n"},"render":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component markup is <a href=\"#!/api/Ext.Component-property-rendered\" rel=\"Ext.Component-property-rendered\" class=\"docClass\">rendered</a>.</p>\n"},"show":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the component is shown when calling the <a href=\"#!/api/Ext.Component-method-show\" rel=\"Ext.Component-method-show\" class=\"docClass\">show</a> method.</p>\n"}},"util":{"Floating":{"prototype":{"constrain":{"!type":"bool","!doc":"<p>True to constrain this Components within its containing element, false to allow it to fall outside of its containing\nelement. By default this Component will be rendered to <code>document.body</code>. To render and constrain this Component within\nanother element specify <a href=\"#!/api/Ext.Component-cfg-renderTo\" rel=\"Ext.Component-cfg-renderTo\" class=\"docClass\">renderTo</a>.</p>\n"},"fixed":{"!type":"bool","!doc":"<p>Configure as <code>true</code> to have this Component fixed at its <code>X, Y</code> coordinates in the browser viewport, immune\nto scrolling the document.</p>\n"},"focusOnToFront":{"!type":"bool","!doc":"<p>Specifies whether the floated component should be automatically <a href=\"#!/api/Ext.Component-method-focus\" rel=\"Ext.Component-method-focus\" class=\"docClass\">focused</a> when\nit is <a href=\"#!/api/Ext.util.Floating-method-toFront\" rel=\"Ext.util.Floating-method-toFront\" class=\"docClass\">brought to the front</a>.</p>\n"},"shadow":{"!doc":"<p>Specifies whether the floating component should be given a shadow. Set to true to automatically create an\n<a href=\"#!/api/Ext.Shadow\" rel=\"Ext.Shadow\" class=\"docClass\">Ext.Shadow</a>, or a string indicating the shadow's display <a href=\"#!/api/Ext.Shadow-cfg-mode\" rel=\"Ext.Shadow-cfg-mode\" class=\"docClass\">Ext.Shadow.mode</a>. Set to false to\ndisable the shadow.</p>\n"},"shadowOffset":{"!type":"number","!doc":"<p>Number of pixels to offset the shadow.</p>\n"},"mixinId":{"!type":"string"},"center":{"!type":"fn() -> ?","!doc":"<p>Center this Component in its container.</p>\n"},"doConstrain":{"!type":"fn(constrainTo?: ?) -> !this","!doc":"<p>Moves this floating Component into a constrain region.</p>\n\n<p>By default, this Component is constrained to be within the container it was added to, or the element it was\nrendered to.</p>\n\n<p>An alternative constraint may be passed.</p>\n"},"fitContainer":{"!type":"fn(animate: ?) -> !this"},"initHierarchyEvents":{"!type":"fn() -> !this"},"onAfterFloatLayout":{"!type":"fn() -> !this"},"onBeforeFloatLayout":{"!type":"fn() -> !this"},"onFloatShow":{"!type":"fn() -> !this"},"onKeyDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Listen for TAB events and wrap round if tabbing of either end of the Floater</p>\n"},"registerWithOwnerCt":{"!type":"fn() -> !this"},"setActive":{"!type":"fn(active?: ?, newActive?: ?) -> !this","!doc":"<p>This method is called internally by <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">Ext.ZIndexManager</a> to signal that a floating Component has either been\nmoved to the top of its zIndex stack, or pushed from the top of its zIndex stack.</p>\n\n<p>If a <em>Window</em> is superceded by another Window, deactivating it hides its shadow.</p>\n\n<p>This method also fires the <a href=\"#!/api/Ext.Component-event-activate\" rel=\"Ext.Component-event-activate\" class=\"docClass\">activate</a> or\n<a href=\"#!/api/Ext.Component-event-deactivate\" rel=\"Ext.Component-event-deactivate\" class=\"docClass\">deactivate</a> event depending on which action occurred.</p>\n"},"setFloatParent":{"!type":"fn(floatParent: ?) -> !this"},"setZIndex":{"!type":"fn(index: ?) -> !this","!doc":"<p>z-index is managed by the zIndexManager and may be overwritten at any time.\nReturns the next z-index to be used.\nIf this is a Container, then it will have rebased any managed floating Components,\nand so the next available z-index will be approximately 10000 above that.</p>\n"},"syncHidden":{"!type":"fn() -> !this","!doc":"<p>synchronizes the hidden state of this component with the state of its hierarchy</p>\n"},"syncShadow":{"!type":"fn() -> !this"},"toBack":{"!type":"fn() -> ?","!doc":"<p>Sends this Component to the back of (lower z-index than) any other visible windows</p>\n"},"toFront":{"!type":"fn(preventFocus?: ?) -> ?","!doc":"<p>Brings this floating Component to the front of any other visible, floating Components managed by the same\n<a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a></p>\n\n<p>If this Component is modal, inserts the modal mask just below this Component in the z-index stack.</p>\n"},"onMouseDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Mousedown brings to front, and programatically grabs focus\nunless the mousedown was on a focusable element*</p>\n"}},"fixed":{"!type":"bool","!doc":"<p>Configure as <code>true</code> to have this Component fixed at its <code>X, Y</code> coordinates in the browser viewport, immune\nto scrolling the document.</p>\n"},"focusOnToFront":{"!type":"bool","!doc":"<p>Specifies whether the floated component should be automatically <a href=\"#!/api/Ext.Component-method-focus\" rel=\"Ext.Component-method-focus\" class=\"docClass\">focused</a> when\nit is <a href=\"#!/api/Ext.util.Floating-method-toFront\" rel=\"Ext.util.Floating-method-toFront\" class=\"docClass\">brought to the front</a>.</p>\n"},"shadow":{"!doc":"<p>Specifies whether the floating component should be given a shadow. Set to true to automatically create an\n<a href=\"#!/api/Ext.Shadow\" rel=\"Ext.Shadow\" class=\"docClass\">Ext.Shadow</a>, or a string indicating the shadow's display <a href=\"#!/api/Ext.Shadow-cfg-mode\" rel=\"Ext.Shadow-cfg-mode\" class=\"docClass\">Ext.Shadow.mode</a>. Set to false to\ndisable the shadow.</p>\n"},"shadowOffset":{"!type":"number","!doc":"<p>Number of pixels to offset the shadow.</p>\n"},"mixinId":{"!type":"string"},"center":{"!type":"fn() -> ?","!doc":"<p>Center this Component in its container.</p>\n"},"doConstrain":{"!type":"fn(constrainTo?: ?) -> !this","!doc":"<p>Moves this floating Component into a constrain region.</p>\n\n<p>By default, this Component is constrained to be within the container it was added to, or the element it was\nrendered to.</p>\n\n<p>An alternative constraint may be passed.</p>\n"},"fitContainer":{"!type":"fn(animate: ?) -> !this"},"initHierarchyEvents":{"!type":"fn() -> !this"},"onAfterFloatLayout":{"!type":"fn() -> !this"},"onBeforeFloatLayout":{"!type":"fn() -> !this"},"onFloatShow":{"!type":"fn() -> !this"},"onKeyDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Listen for TAB events and wrap round if tabbing of either end of the Floater</p>\n"},"onMouseDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Mousedown brings to front, and programatically grabs focus\nunless the mousedown was on a focusable element*</p>\n"},"registerWithOwnerCt":{"!type":"fn() -> !this"},"setActive":{"!type":"fn(active?: ?, newActive?: ?) -> !this","!doc":"<p>This method is called internally by <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">Ext.ZIndexManager</a> to signal that a floating Component has either been\nmoved to the top of its zIndex stack, or pushed from the top of its zIndex stack.</p>\n\n<p>If a <em>Window</em> is superceded by another Window, deactivating it hides its shadow.</p>\n\n<p>This method also fires the <a href=\"#!/api/Ext.Component-event-activate\" rel=\"Ext.Component-event-activate\" class=\"docClass\">activate</a> or\n<a href=\"#!/api/Ext.Component-event-deactivate\" rel=\"Ext.Component-event-deactivate\" class=\"docClass\">deactivate</a> event depending on which action occurred.</p>\n"},"setFloatParent":{"!type":"fn(floatParent: ?) -> !this"},"setZIndex":{"!type":"fn(index: ?) -> !this","!doc":"<p>z-index is managed by the zIndexManager and may be overwritten at any time.\nReturns the next z-index to be used.\nIf this is a Container, then it will have rebased any managed floating Components,\nand so the next available z-index will be approximately 10000 above that.</p>\n"},"syncHidden":{"!type":"fn() -> !this","!doc":"<p>synchronizes the hidden state of this component with the state of its hierarchy</p>\n"},"syncShadow":{"!type":"fn() -> !this"},"toBack":{"!type":"fn() -> ?","!doc":"<p>Sends this Component to the back of (lower z-index than) any other visible windows</p>\n"},"toFront":{"!type":"fn(preventFocus?: ?) -> ?","!doc":"<p>Brings this floating Component to the front of any other visible, floating Components managed by the same\n<a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a></p>\n\n<p>If this Component is modal, inserts the modal mask just below this Component in the z-index stack.</p>\n"},"!doc":"<p>A mixin to add floating capability to a Component.</p>\n"},"Renderable":{"prototype":{"_renderState":{"!type":"number","!doc":"<p>This property holds one of the following values during the render process:</p>\n\n<ul>\n<li><strong>0</strong> - The component is not rendered.</li>\n<li><strong>1</strong> - The component has fired beforerender and is about to call beforeRender.\nThe component has just started rendering.</li>\n<li><strong>2</strong> - The component has finished the <code>beforeRender</code> process and is about to\ncall <code>onRender</code>. This is when <code>rendering</code> is set to <code>true</code>.</li>\n<li><strong>3</strong> - The component has started <code>onRender</code>. This is when <code>rendered</code> is set\nto <code>true</code>.</li>\n<li><strong>4</strong> - The component has finished its afterrender process.</li>\n</ul>\n\n"},"frameCls":{"!type":"string"},"frameElNames":{},"frameIdRegex":{},"frameTableTpl":{},"frameTpl":{},"afterFirstLayout":{"!type":"fn(width: ?, height: ?) -> !this"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"applyRenderSelectors":{"!type":"fn() -> !this","!doc":"<p>Sets references to elements inside the component. This applies <a href=\"#!/api/Ext.Component-cfg-renderSelectors\" rel=\"Ext.Component-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a>\nas well as <a href=\"#!/api/Ext.Component-cfg-childEls\" rel=\"Ext.Component-cfg-childEls\" class=\"docClass\">childEls</a>.</p>\n"},"cacheRefEls":{"!type":"fn(el: ?) -> !this","!doc":"<p>Ensures that all elements with \"data-ref\" attributes get loaded into the cache.\nThis really helps on IE8 where <code>getElementById</code> is a search not a lookup. By\npopulating our cache with one search of the DOM we then have random access to\nthe elements as we do our <code>childEls</code> wire up.</p>\n"},"doApplyRenderTpl":{"!type":"fn(out: ?, values: ?) -> !this","!doc":"<p>Called from the selected frame generation template to insert this Component's inner structure inside the framing structure.</p>\n\n<p>When framing is used, a selected frame generation template is used as the primary template of the <a href=\"#!/api/Ext.util.Renderable-method-getElConfig\" rel=\"Ext.util.Renderable-method-getElConfig\" class=\"docClass\">getElConfig</a> instead\nof the configured <a href=\"#!/api/Ext.Component-cfg-renderTpl\" rel=\"Ext.Component-cfg-renderTpl\" class=\"docClass\">renderTpl</a>. The renderTpl is invoked by this method which is injected into the framing template.</p>\n"},"doAutoRender":{"!type":"fn() -> !this","!doc":"<p>Handles autoRender.\nFloating Components may have an ownerCt. If they are asking to be constrained, constrain them within that\nownerCt, and have their z-index managed locally. Floating Components are always rendered to document.body</p>\n"},"ensureAttachedToBody":{"!type":"fn(runLayout?: ?) -> !this","!doc":"<p>Ensures that this component is attached to <code>document.body</code>. If the component was\nrendered to Ext.getDetachedBody, then it will be appended to <code>document.body</code>.\nAny configured position is also restored.</p>\n"},"finishRender":{"!type":"fn(containerIdx: ?) -> !this","!doc":"<p>This method visits the rendered component tree in a \"top-down\" order. That is, this\ncode runs on a parent component before running on a child. This method calls the\n<a href=\"#!/api/Ext.util.Renderable-method-onRender\" rel=\"Ext.util.Renderable-method-onRender\" class=\"docClass\">onRender</a> method of each component.</p>\n"},"getFrameInfo":{"!type":"fn() -> !this","!doc":"<p>On render, reads an encoded style attribute, \"filter\" from the style of this Component's element.\nThis information is memoized based upon the CSS class name of this Component's element.\nBecause child Components are rendered as textual HTML as part of the topmost Container, a dummy div is inserted\ninto the document to receive the document element's CSS class name, and therefore style attributes.</p>\n"},"getFrameTpl":{"!type":"fn(table: ?) -> !this"},"getInsertPosition":{"!type":"fn(position: ?) -> ?","!doc":"<p>This function takes the position argument passed to onRender and returns a\nDOM element that you can use in the insertBefore.</p>\n"},"getRenderTree":{"!type":"fn() -> !this"},"getStyleProxy":{"!type":"fn(cls: ?) -> !this","!doc":"<p>Returns an offscreen div with the same class name as the element this is being rendered.\nThis is because child item rendering takes place in a detached div which, being not\npart of the document, has no styling.</p>\n"},"initRenderTpl":{"!type":"fn() -> ?","!doc":"<p>Initializes the renderTpl.</p>\n"},"onClassMixedIn":{"!type":"fn(targetClass: ?) -> !this"},"render":{"!type":"fn(container?: ?, position?: ?) -> !this","!doc":"<p>Renders the Component into the passed HTML element.</p>\n\n<p><strong>If you are using a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a> object to house this\nComponent, then do not use the render method.</strong></p>\n\n<p>A Container's child Components are rendered by that Container's\n<a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a> manager when the Container is first rendered.</p>\n\n<p>If the Container is already rendered when a new child Component is added, you may need to call\nthe Container's <a href=\"#!/api/Ext.container.Container-method-doLayout\" rel=\"Ext.container.Container-method-doLayout\" class=\"docClass\">doLayout</a> to refresh the view which\ncauses any unrendered child Components to be rendered. This is required so that you can add\nmultiple child components if needed while only refreshing the layout once.</p>\n\n<p>When creating complex UIs, it is important to remember that sizing and positioning\nof child items is the responsibility of the Container's <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a>\nmanager.  If you expect child items to be sized in response to user interactions, you must\nconfigure the Container with a layout manager which creates and manages the type of layout you\nhave in mind.</p>\n\n<p><strong>Omitting the Container's <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a> config means that a basic\nlayout manager is used which does nothing but render child components sequentially into the\nContainer. No sizing or positioning will be performed in this situation.</strong></p>\n"},"setupFramingTpl":{"!type":"fn(frameTpl: ?) -> !this","!doc":"<p>Inject a reference to the function which applies the render template into the framing template. The framing template\nwraps the content.</p>\n"},"updateFrame":{"!type":"fn() -> !this","!doc":"<p>Updates the frame elements to match new framing. The current <code>frameBody</code> is\npreserved by transplanting it into the new frame. All other frame <code>childEls</code>\nare destroyed and recreated if needed by the new frame. This method cannot\ntransition from framed to non-framed or vise-versa or between table and div\nbased framing.</p>\n"},"getElConfig":{"!type":"fn() -> !this"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"onRender":{"!type":"fn(parentNode: ?, containerIdx: ?) -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"mixinId":{"!type":"string"},"privates":{"!doc":"<hr />\n"},"makeRenderSetter":{"!type":"fn(cfg: ?, renderState: ?) -> !this"},"processRenderConfig":{"!type":"fn(source: ?, configName: ?, state: ?) -> !this"}},"_renderState":{"!type":"number","!doc":"<p>This property holds one of the following values during the render process:</p>\n\n<ul>\n<li><strong>0</strong> - The component is not rendered.</li>\n<li><strong>1</strong> - The component has fired beforerender and is about to call beforeRender.\nThe component has just started rendering.</li>\n<li><strong>2</strong> - The component has finished the <code>beforeRender</code> process and is about to\ncall <code>onRender</code>. This is when <code>rendering</code> is set to <code>true</code>.</li>\n<li><strong>3</strong> - The component has started <code>onRender</code>. This is when <code>rendered</code> is set\nto <code>true</code>.</li>\n<li><strong>4</strong> - The component has finished its afterrender process.</li>\n</ul>\n\n"},"frameCls":{"!type":"string"},"frameElNames":{},"frameIdRegex":{},"frameTableTpl":{},"frameTpl":{},"afterFirstLayout":{"!type":"fn(width: ?, height: ?) -> !this"},"applyRenderSelectors":{"!type":"fn() -> !this","!doc":"<p>Sets references to elements inside the component. This applies <a href=\"#!/api/Ext.Component-cfg-renderSelectors\" rel=\"Ext.Component-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a>\nas well as <a href=\"#!/api/Ext.Component-cfg-childEls\" rel=\"Ext.Component-cfg-childEls\" class=\"docClass\">childEls</a>.</p>\n"},"cacheRefEls":{"!type":"fn(el: ?) -> !this","!doc":"<p>Ensures that all elements with \"data-ref\" attributes get loaded into the cache.\nThis really helps on IE8 where <code>getElementById</code> is a search not a lookup. By\npopulating our cache with one search of the DOM we then have random access to\nthe elements as we do our <code>childEls</code> wire up.</p>\n"},"doApplyRenderTpl":{"!type":"fn(out: ?, values: ?) -> !this","!doc":"<p>Called from the selected frame generation template to insert this Component's inner structure inside the framing structure.</p>\n\n<p>When framing is used, a selected frame generation template is used as the primary template of the <a href=\"#!/api/Ext.util.Renderable-method-getElConfig\" rel=\"Ext.util.Renderable-method-getElConfig\" class=\"docClass\">getElConfig</a> instead\nof the configured <a href=\"#!/api/Ext.Component-cfg-renderTpl\" rel=\"Ext.Component-cfg-renderTpl\" class=\"docClass\">renderTpl</a>. The renderTpl is invoked by this method which is injected into the framing template.</p>\n"},"doAutoRender":{"!type":"fn() -> !this","!doc":"<p>Handles autoRender.\nFloating Components may have an ownerCt. If they are asking to be constrained, constrain them within that\nownerCt, and have their z-index managed locally. Floating Components are always rendered to document.body</p>\n"},"ensureAttachedToBody":{"!type":"fn(runLayout?: ?) -> !this","!doc":"<p>Ensures that this component is attached to <code>document.body</code>. If the component was\nrendered to Ext.getDetachedBody, then it will be appended to <code>document.body</code>.\nAny configured position is also restored.</p>\n"},"finishRender":{"!type":"fn(containerIdx: ?) -> !this","!doc":"<p>This method visits the rendered component tree in a \"top-down\" order. That is, this\ncode runs on a parent component before running on a child. This method calls the\n<a href=\"#!/api/Ext.util.Renderable-method-onRender\" rel=\"Ext.util.Renderable-method-onRender\" class=\"docClass\">onRender</a> method of each component.</p>\n"},"getFrameInfo":{"!type":"fn() -> !this","!doc":"<p>On render, reads an encoded style attribute, \"filter\" from the style of this Component's element.\nThis information is memoized based upon the CSS class name of this Component's element.\nBecause child Components are rendered as textual HTML as part of the topmost Container, a dummy div is inserted\ninto the document to receive the document element's CSS class name, and therefore style attributes.</p>\n"},"getFrameTpl":{"!type":"fn(table: ?) -> !this"},"getInsertPosition":{"!type":"fn(position: ?) -> ?","!doc":"<p>This function takes the position argument passed to onRender and returns a\nDOM element that you can use in the insertBefore.</p>\n"},"getRenderTree":{"!type":"fn() -> !this"},"getStyleProxy":{"!type":"fn(cls: ?) -> !this","!doc":"<p>Returns an offscreen div with the same class name as the element this is being rendered.\nThis is because child item rendering takes place in a detached div which, being not\npart of the document, has no styling.</p>\n"},"initRenderTpl":{"!type":"fn() -> ?","!doc":"<p>Initializes the renderTpl.</p>\n"},"onClassMixedIn":{"!type":"fn(targetClass: ?) -> !this"},"render":{"!type":"fn(container?: ?, position?: ?) -> !this","!doc":"<p>Renders the Component into the passed HTML element.</p>\n\n<p><strong>If you are using a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a> object to house this\nComponent, then do not use the render method.</strong></p>\n\n<p>A Container's child Components are rendered by that Container's\n<a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a> manager when the Container is first rendered.</p>\n\n<p>If the Container is already rendered when a new child Component is added, you may need to call\nthe Container's <a href=\"#!/api/Ext.container.Container-method-doLayout\" rel=\"Ext.container.Container-method-doLayout\" class=\"docClass\">doLayout</a> to refresh the view which\ncauses any unrendered child Components to be rendered. This is required so that you can add\nmultiple child components if needed while only refreshing the layout once.</p>\n\n<p>When creating complex UIs, it is important to remember that sizing and positioning\nof child items is the responsibility of the Container's <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a>\nmanager.  If you expect child items to be sized in response to user interactions, you must\nconfigure the Container with a layout manager which creates and manages the type of layout you\nhave in mind.</p>\n\n<p><strong>Omitting the Container's <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a> config means that a basic\nlayout manager is used which does nothing but render child components sequentially into the\nContainer. No sizing or positioning will be performed in this situation.</strong></p>\n"},"setupFramingTpl":{"!type":"fn(frameTpl: ?) -> !this","!doc":"<p>Inject a reference to the function which applies the render template into the framing template. The framing template\nwraps the content.</p>\n"},"updateFrame":{"!type":"fn() -> !this","!doc":"<p>Updates the frame elements to match new framing. The current <code>frameBody</code> is\npreserved by transplanting it into the new frame. All other frame <code>childEls</code>\nare destroyed and recreated if needed by the new frame. This method cannot\ntransition from framed to non-framed or vise-versa or between table and div\nbased framing.</p>\n"},"!doc":"<p>Given a component hierarchy of this:</p>\n\n<pre><code> {\n     xtype: 'panel',\n     id: 'ContainerA',\n     layout: 'hbox',\n     renderTo: Ext.getBody(),\n     items: [\n         {\n             id: 'ContainerB',\n             xtype: 'container',\n             items: [\n                 { id: 'ComponentA' }\n             ]\n         }\n     ]\n }\n</code></pre>\n\n<p>The rendering of the above proceeds roughly like this:</p>\n\n<ul>\n<li>ContainerA's initComponent calls <a href=\"#!/api/Ext.util.Renderable-method-render\" rel=\"Ext.util.Renderable-method-render\" class=\"docClass\">render</a> passing the <code>renderTo</code> property as the\ncontainer argument.</li>\n<li><code>render</code> calls the <code>getRenderTree</code> method to get a complete Ext.dom.Helper spec.</li>\n<li><code>getRenderTree</code> fires the \"beforerender\" event and calls the <a href=\"#!/api/Ext.util.Renderable-method-beforeRender\" rel=\"Ext.util.Renderable-method-beforeRender\" class=\"docClass\">beforeRender</a>\nmethod. Its result is obtained by calling <a href=\"#!/api/Ext.util.Renderable-method-getElConfig\" rel=\"Ext.util.Renderable-method-getElConfig\" class=\"docClass\">getElConfig</a>.</li>\n<li>The <a href=\"#!/api/Ext.util.Renderable-method-getElConfig\" rel=\"Ext.util.Renderable-method-getElConfig\" class=\"docClass\">getElConfig</a> method uses the <code>renderTpl</code> and its render data as the content\nof the <code>autoEl</code> described element.</li>\n<li>The result of <code>getRenderTree</code> is passed to Ext.dom.Helper.append.</li>\n<li>The <code>renderTpl</code> contains calls to render things like docked items, container items\nand raw markup (such as the <code>html</code> or <code>tpl</code> config properties). These calls are to\nmethods added to the Ext.XTemplate instance by #setupRenderTpl.</li>\n<li>The #setupRenderTpl method adds methods such as <code>renderItems</code>, <code>renderContent</code>, etc.\nto the template. These are directed to \"doRenderItems\", \"doRenderContent\" etc..</li>\n<li>The #setupRenderTpl calls traverse from components to their <a href=\"#!/api/Ext.layout.Layout\" rel=\"Ext.layout.Layout\" class=\"docClass\">Ext.layout.Layout</a>\nobject.</li>\n<li>When a container is rendered, it also has a <code>renderTpl</code>. This is processed when the\n<code>renderContainer</code> method is called in the component's <code>renderTpl</code>. This call goes to\n<a href=\"#!/api/Ext.layout.container.Container-method-doRenderContainer\" rel=\"Ext.layout.container.Container-method-doRenderContainer\" class=\"docClass\">Ext.layout.container.Container.doRenderContainer</a>. This method repeats this\nprocess for all components in the container.</li>\n<li>After the top-most component's markup is generated and placed in to the DOM, the next\nstep is to link elements to their components and finish calling the component methods\n<code>onRender</code> and <code>afterRender</code> as well as fire the corresponding events.</li>\n<li>The first step in this is to call <a href=\"#!/api/Ext.util.Renderable-method-finishRender\" rel=\"Ext.util.Renderable-method-finishRender\" class=\"docClass\">finishRender</a>. This method descends the\ncomponent hierarchy and calls <code>onRender</code> and fires the <code>render</code> event. These calls\nare delivered top-down to approximate the timing of these calls/events from previous\nversions.</li>\n<li>During the pass, the component's <code>el</code> is set. Likewise, the <code>renderSelectors</code> and\n<code>childEls</code> are applied to capture references to the component's elements.</li>\n<li>These calls are also made on the <a href=\"#!/api/Ext.layout.container.Container\" rel=\"Ext.layout.container.Container\" class=\"docClass\">Ext.layout.container.Container</a> layout to\ncapture its elements. Both of these classes use <a href=\"#!/api/Ext.util.ElementContainer\" rel=\"Ext.util.ElementContainer\" class=\"docClass\">Ext.util.ElementContainer</a> to\nhandle <code>childEls</code> processing.</li>\n</ul>\n\n"},"Animate":{"prototype":{"isAnimate":{"!type":"bool"},"anim":{"!type":"fn(config: ?) -> !this","!doc":"<ul>\n<li>process the passed fx configuration.</li>\n</ul>\n\n"},"getActiveAnimation":{"!type":"fn() -> ?","!doc":"<p>Returns the current animation if this object has any effects actively running or queued, else returns false.</p>\n"},"getAnimationProps":{"!type":"fn() -> !this","!doc":"<ul>\n<li>get animation properties</li>\n</ul>\n\n"},"hasActiveFx":{"!type":"fn() -> ?","!doc":"<p>Returns the current animation if this object has any effects actively running or queued, else returns false.</p>\n"},"sequenceFx":{"!type":"fn() -> ?","!doc":"<p>Ensures that all effects queued after sequenceFx is called on this object are run in sequence. This is the\nopposite of <a href=\"#!/api/Ext.util.Animate-method-syncFx\" rel=\"Ext.util.Animate-method-syncFx\" class=\"docClass\">syncFx</a>.</p>\n"},"stopAnimation":{"!type":"fn() -> ?","!doc":"<p>Stops any running effects and clears this object's internal effects queue if it contains any additional effects\nthat haven't started yet.</p>\n"},"stopFx":{"!type":"fn() -> ?","!doc":"<p>Stops any running effects and clears this object's internal effects queue if it contains any additional effects\nthat haven't started yet.</p>\n"},"syncFx":{"!type":"fn() -> ?","!doc":"<p>Ensures that all effects queued after syncFx is called on this object are run concurrently. This is the opposite\nof <a href=\"#!/api/Ext.util.Animate-method-sequenceFx\" rel=\"Ext.util.Animate-method-sequenceFx\" class=\"docClass\">sequenceFx</a>.</p>\n"},"mixinId":{"!type":"string"},"animate":{"!type":"fn(config: ?) -> ?","!doc":"<p>Performs custom animation on this object.</p>\n\n<p>This method is applicable to both the <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a> class and the Sprite\nclass. It performs animated transitions of certain properties of this object over a specified timeline.</p>\n\n<h3>Animating a <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a></h3>\n\n<p>When animating a Component, the following properties may be specified in <code>from</code>, <code>to</code>, and <code>keyframe</code> objects:</p>\n\n<ul>\n<li><p><code>x</code> - The Component's page X position in pixels.</p></li>\n<li><p><code>y</code> - The Component's page Y position in pixels</p></li>\n<li><p><code>left</code> - The Component's <code>left</code> value in pixels.</p></li>\n<li><p><code>top</code> - The Component's <code>top</code> value in pixels.</p></li>\n<li><p><code>width</code> - The Component's <code>width</code> value in pixels.</p></li>\n<li><p><code>height</code> - The Component's <code>height</code> value in pixels.</p></li>\n<li><p><code>dynamic</code> - Specify as true to update the Component's layout (if it is a Container) at every frame of the animation.\n<em>Use sparingly as laying out on every intermediate size change is an expensive operation.</em></p></li>\n</ul>\n\n\n<p>For example, to animate a Window to a new size, ensuring that its internal layout and any shadow is correct:</p>\n\n<pre><code>myWindow = Ext.create('<a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>', {\n    title: 'Test Component animation',\n    width: 500,\n    height: 300,\n    layout: {\n        type: 'hbox',\n        align: 'stretch'\n    },\n    items: [{\n        title: 'Left: 33%',\n        margin: '5 0 5 5',\n        flex: 1\n    }, {\n        title: 'Left: 66%',\n        margin: '5 5 5 5',\n        flex: 2\n    }]\n});\nmyWindow.show();\nmyWindow.header.el.on('click', function() {\n    myWindow.animate({\n        to: {\n            width: (myWindow.getWidth() == 500) ? 700 : 500,\n            height: (myWindow.getHeight() == 300) ? 400 : 300\n        }\n    });\n});\n</code></pre>\n\n<p>For performance reasons, by default, the internal layout is only updated when the Window reaches its final <code>\"to\"</code>\nsize. If dynamic updating of the Window's child Components is required, then configure the animation with\n<code>dynamic: true</code> and the two child items will maintain their proportions during the animation.</p>\n"}},"isAnimate":{"!type":"bool"},"anim":{"!type":"fn(config: ?) -> !this","!doc":"<ul>\n<li>process the passed fx configuration.</li>\n</ul>\n\n"},"getActiveAnimation":{"!type":"fn() -> ?","!doc":"<p>Returns the current animation if this object has any effects actively running or queued, else returns false.</p>\n"},"hasActiveFx":{"!type":"fn() -> ?","!doc":"<p>Returns the current animation if this object has any effects actively running or queued, else returns false.</p>\n"},"sequenceFx":{"!type":"fn() -> ?","!doc":"<p>Ensures that all effects queued after sequenceFx is called on this object are run in sequence. This is the\nopposite of <a href=\"#!/api/Ext.util.Animate-method-syncFx\" rel=\"Ext.util.Animate-method-syncFx\" class=\"docClass\">syncFx</a>.</p>\n"},"stopAnimation":{"!type":"fn() -> ?","!doc":"<p>Stops any running effects and clears this object's internal effects queue if it contains any additional effects\nthat haven't started yet.</p>\n"},"stopFx":{"!type":"fn() -> ?","!doc":"<p>Stops any running effects and clears this object's internal effects queue if it contains any additional effects\nthat haven't started yet.</p>\n"},"syncFx":{"!type":"fn() -> ?","!doc":"<p>Ensures that all effects queued after syncFx is called on this object are run concurrently. This is the opposite\nof <a href=\"#!/api/Ext.util.Animate-method-sequenceFx\" rel=\"Ext.util.Animate-method-sequenceFx\" class=\"docClass\">sequenceFx</a>.</p>\n"},"!doc":"<p>This animation class is a mixin.</p>\n\n<p><a href=\"#!/api/Ext.util.Animate\" rel=\"Ext.util.Animate\" class=\"docClass\">Ext.util.Animate</a> provides an API for the creation of animated transitions of properties and styles.\nThis class is used as a mixin and currently applied to Ext.dom.Element, Ext.CompositeElement,\nExt.draw.sprite.Sprite, Ext.draw.sprite.Composite, and <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.  Note that Components\nhave a limited subset of what attributes can be animated such as top, left, x, y, height, width, and\nopacity (color, paddings, and margins can not be animated).</p>\n\n<h2>Animation Basics</h2>\n\n<p>All animations require three things - <code>easing</code>, <code>duration</code>, and <code>to</code> (the final end value for each property)\nyou wish to animate. Easing and duration are defaulted values specified below.\nEasing describes how the intermediate values used during a transition will be calculated.\n<a href=\"#!/api/Ext.fx.Anim-cfg-easing\" rel=\"Ext.fx.Anim-cfg-easing\" class=\"docClass\">Easing</a> allows for a transition to change speed over its duration.\nYou may use the defaults for easing and duration, but you must always set a\n<a href=\"#!/api/Ext.fx.Anim-cfg-to\" rel=\"Ext.fx.Anim-cfg-to\" class=\"docClass\">to</a> property which is the end value for all animations.</p>\n\n<p>Popular element 'to' configurations are:</p>\n\n<ul>\n<li>opacity</li>\n<li>x</li>\n<li>y</li>\n<li>color</li>\n<li>height</li>\n<li>width</li>\n</ul>\n\n\n<p>Popular sprite 'to' configurations are:</p>\n\n<ul>\n<li>translation</li>\n<li>path</li>\n<li>scale</li>\n<li>stroke</li>\n<li>rotation</li>\n</ul>\n\n\n<p>The default duration for animations is 250 (which is a 1/4 of a second).  Duration is denoted in\nmilliseconds.  Therefore 1 second is 1000, 1 minute would be 60000, and so on. The default easing curve\nused for all animations is 'ease'.  Popular easing functions are included and can be found in <a href=\"#!/api/Ext.fx.Anim-cfg-easing\" rel=\"Ext.fx.Anim-cfg-easing\" class=\"docClass\">Easing</a>.</p>\n\n<p>For example, a simple animation to fade out an element with a default easing and duration:</p>\n\n<pre><code>var p1 = Ext.get('myElementId');\n\np1.animate({\n    to: {\n        opacity: 0\n    }\n});\n</code></pre>\n\n<p>To make this animation fade out in a tenth of a second:</p>\n\n<pre><code>var p1 = Ext.get('myElementId');\n\np1.animate({\n   duration: 100,\n    to: {\n        opacity: 0\n    }\n});\n</code></pre>\n\n<h2>Animation Queues</h2>\n\n<p>By default all animations are added to a queue which allows for animation via a chain-style API.\nFor example, the following code will queue 4 animations which occur sequentially (one right after the other):</p>\n\n<pre><code>p1.animate({\n    to: {\n        x: 500\n    }\n}).animate({\n    to: {\n        y: 150\n    }\n}).animate({\n    to: {\n        backgroundColor: '#f00'  //red\n    }\n}).animate({\n    to: {\n        opacity: 0\n    }\n});\n</code></pre>\n\n<p>You can change this behavior by calling the <a href=\"#!/api/Ext.util.Animate-method-syncFx\" rel=\"Ext.util.Animate-method-syncFx\" class=\"docClass\">syncFx</a> method and all\nsubsequent animations for the specified target will be run concurrently (at the same time).</p>\n\n<pre><code>p1.syncFx();  //this will make all animations run at the same time\n\np1.animate({\n    to: {\n        x: 500\n    }\n}).animate({\n    to: {\n        y: 150\n    }\n}).animate({\n    to: {\n        backgroundColor: '#f00'  //red\n    }\n}).animate({\n    to: {\n        opacity: 0\n    }\n});\n</code></pre>\n\n<p>This works the same as:</p>\n\n<pre><code>p1.animate({\n    to: {\n        x: 500,\n        y: 150,\n        backgroundColor: '#f00'  //red\n        opacity: 0\n    }\n});\n</code></pre>\n\n<p>The <a href=\"#!/api/Ext.util.Animate-method-stopAnimation\" rel=\"Ext.util.Animate-method-stopAnimation\" class=\"docClass\">stopAnimation</a> method can be used to stop any\ncurrently running animations and clear any queued animations.</p>\n\n<h2>Animation Keyframes</h2>\n\n<p>You can also set up complex animations with <a href=\"#!/api/Ext.fx.Anim-cfg-keyframes\" rel=\"Ext.fx.Anim-cfg-keyframes\" class=\"docClass\">keyframes</a> which follow the\nCSS3 Animation configuration pattern. Note rotation, translation, and scaling can only be done for sprites.\nThe previous example can be written with the following syntax:</p>\n\n<pre><code>p1.animate({\n    duration: 1000,  //one second total\n    keyframes: {\n        25: {     //from 0 to 250ms (25%)\n            x: 0\n        },\n        50: {   //from 250ms to 500ms (50%)\n            y: 0\n        },\n        75: {  //from 500ms to 750ms (75%)\n            backgroundColor: '#f00'  //red\n        },\n        100: {  //from 750ms to 1sec\n            opacity: 0\n        }\n    }\n});\n</code></pre>\n\n<h2>Animation Events</h2>\n\n<p>Each animation you create has events for <a href=\"#!/api/Ext.fx.Anim-event-beforeanimate\" rel=\"Ext.fx.Anim-event-beforeanimate\" class=\"docClass\">beforeanimate</a>,\n<a href=\"#!/api/Ext.fx.Anim-event-afteranimate\" rel=\"Ext.fx.Anim-event-afteranimate\" class=\"docClass\">afteranimate</a>, and <a href=\"#!/api/Ext.fx.Anim-event-lastframe\" rel=\"Ext.fx.Anim-event-lastframe\" class=\"docClass\">lastframe</a>.\nKeyframed animations adds an additional <a href=\"#!/api/Ext.fx.Animator-event-keyframe\" rel=\"Ext.fx.Animator-event-keyframe\" class=\"docClass\">keyframe</a> event which\nfires for each keyframe in your animation.</p>\n\n<p>All animations support the listeners configuration to attact functions to these events.</p>\n\n<pre><code>startAnimate: function() {\n    var p1 = Ext.get('myElementId');\n    p1.animate({\n       duration: 100,\n        to: {\n            opacity: 0\n        },\n        listeners: {\n            beforeanimate:  function() {\n                // Execute my custom method before the animation\n                this.myBeforeAnimateFn();\n            },\n            afteranimate: function() {\n                // Execute my custom method after the animation\n                this.myAfterAnimateFn();\n            },\n            scope: this\n    });\n},\nmyBeforeAnimateFn: function() {\n  // My custom logic\n},\nmyAfterAnimateFn: function() {\n  // My custom logic\n}\n</code></pre>\n\n<p>Due to the fact that animations run asynchronously, you can determine if an animation is currently\nrunning on any target by using the <a href=\"#!/api/Ext.util.Animate-method-getActiveAnimation\" rel=\"Ext.util.Animate-method-getActiveAnimation\" class=\"docClass\">getActiveAnimation</a>\nmethod.  This method will return false if there are no active animations or return the currently\nrunning <a href=\"#!/api/Ext.fx.Anim\" rel=\"Ext.fx.Anim\" class=\"docClass\">Ext.fx.Anim</a> instance.</p>\n\n<p>In this example, we're going to wait for the current animation to finish, then stop any other\nqueued animations before we fade our element's opacity to 0:</p>\n\n<pre><code>var curAnim = p1.getActiveAnimation();\nif (curAnim) {\n    curAnim.on('afteranimate', function() {\n        p1.stopAnimation();\n        p1.animate({\n            to: {\n                opacity: 0\n            }\n        });\n    });\n}\n</code></pre>\n"},"ElementContainer":{"prototype":{"afterClassMixedIn":{"!type":"fn(targetClass: ?) -> !this","!doc":"<p>Called after the mixin is applied. We need to see if <code>childEls</code> were used by\nthe <code>targetClass</code> and apply them to the config.</p>\n"},"attachChildEls":{"!type":"fn(el: ?, owner: ?) -> !this","!doc":"<p>Sets references to elements inside the component.</p>\n"},"getChildEls":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.util.ElementContainer-cfg-childEls\" rel=\"Ext.util.ElementContainer-cfg-childEls\" class=\"docClass\">childEls</a>.</p>\n"},"setChildEls":{"!type":"fn(childEls: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.util.ElementContainer-cfg-childEls\" rel=\"Ext.util.ElementContainer-cfg-childEls\" class=\"docClass\">childEls</a>.</p>\n"},"mixinId":{"!type":"string"},"privates":{},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"destroy":{"!type":"fn() -> !this"}},"afterClassMixedIn":{"!type":"fn(targetClass: ?) -> !this","!doc":"<p>Called after the mixin is applied. We need to see if <code>childEls</code> were used by\nthe <code>targetClass</code> and apply them to the config.</p>\n"},"attachChildEls":{"!type":"fn(el: ?, owner: ?) -> !this","!doc":"<p>Sets references to elements inside the component.</p>\n"},"getChildEls":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.util.ElementContainer-cfg-childEls\" rel=\"Ext.util.ElementContainer-cfg-childEls\" class=\"docClass\">childEls</a>.</p>\n"},"setChildEls":{"!type":"fn(childEls: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.util.ElementContainer-cfg-childEls\" rel=\"Ext.util.ElementContainer-cfg-childEls\" class=\"docClass\">childEls</a>.</p>\n"},"!doc":"<p>This mixin enables classes to declare relationships to child elements and provides the\nmechanics for acquiring the elements and storing them on an object\ninstance as properties.</p>\n\n<p>This class is used by <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">components</a> and <a href=\"#!/api/Ext.layout.container.Container\" rel=\"Ext.layout.container.Container\" class=\"docClass\">container layouts</a> to\nmanage their child elements.</p>\n\n<p>A typical component that uses these features might look something like this:</p>\n\n<pre><code> Ext.define('Ext.ux.SomeComponent', {\n     extend: '<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>',\n\n     childEls: [\n         'bodyEl'\n     ],\n\n     renderTpl: [\n         '&lt;div id=\"{id}-bodyEl\" data-ref=\"bodyEl\"&gt;&lt;/div&gt;'\n     ],\n\n     // ...\n });\n</code></pre>\n\n<p>The <code><a href=\"#!/api/Ext.util.ElementContainer-cfg-childEls\" rel=\"Ext.util.ElementContainer-cfg-childEls\" class=\"docClass\">childEls</a></code> config lists one or more relationships to child elements managed\nby the component. The items in this array can be objects that more fully specify the\nchild. For example, the above could have used this instead to achieve the same result:</p>\n\n<pre><code> childEls: [\n     { name: 'bodyEl', itemId: 'bodyEl' }\n ]\n</code></pre>\n\n<p>Unlike a <code>renderTpl</code> where there is a single value for an instance, <code>childEls</code> are aggregated\nup the class hierarchy so that they are effectively inherited. In other words, if a\nclass where to derive from <code>Ext.ux.SomeComponent</code> in the example above, it could also\nhave a <code>childEls</code> property in the same way as <code>Ext.ux.SomeComponent</code>.</p>\n\n<pre><code> Ext.define('Ext.ux.AnotherComponent', {\n     extend: 'Ext.ux.SomeComponent',\n\n     childEls: [\n         // 'bodyEl' is inherited\n         'innerEl'\n     ],\n\n     renderTpl: [\n         '&lt;div id=\"{id}-bodyEl\" data-ref=\"bodyEl\"&gt;'\n             '&lt;div id=\"{id}-innerEl\" data-ref=\"innerEl\"&gt;&lt;/div&gt;'\n         '&lt;/div&gt;'\n     ],\n\n     // ...\n });\n</code></pre>\n\n<p><strong>IMPORTANT</strong>\nThe <code>renderTpl</code> contains both child elements and unites them in the desired markup, but\nthe <code>childEls</code> only contains the new child element. The <code>data-ref</code> attribute must be\nrendered on to child elements that do not use <code>select</code> or <code>selectNode</code> options. This\nis done for performance reasons on IE8 where element lookup (even by id) is not very\nefficient.</p>\n"},"StoreHolder":{"prototype":{"mixinId":{"!type":"string"},"bindStoreListeners":{"!type":"fn(store: ?) -> !this","!doc":"<p>Binds listeners for this component to the store. By default it will add\nanything bound by the getStoreListeners method, however it can be overridden\nin a subclass to provide any more complicated handling.</p>\n"},"setStore":{"!type":"fn(store: ?) -> !this","!doc":"<p>Sets the store to the specified store.</p>\n"},"unbindStoreListeners":{"!type":"fn(store: ?) -> !this","!doc":"<p>Unbinds listeners from this component to the store. By default it will remove\nanything bound by the bindStoreListeners method, however it can be overridden\nin a subclass to provide any more complicated handling.</p>\n"},"onBindStore":{"!type":"fn(store: ?, initial: ?) -> !this","!doc":"<p>Template method, it is called when a new store is bound\nto the current instance.</p>\n"},"onUnbindStore":{"!type":"fn(store: ?, initial: ?) -> !this","!doc":"<p>Template method, it is called when an existing store is unbound\nfrom the current instance.</p>\n"},"getStore":{"!type":"fn() -> ?","!doc":"<p>Gets the current store instance.</p>\n"},"bindStore":{"!type":"fn(store?: ?) -> !this","!doc":"<p>Binds a store to this instance.</p>\n"},"getStoreListeners":{"!type":"fn(store: ?) -> ?","!doc":"<p>Gets the listeners to bind to a new store.</p>\n"}},"!doc":"<p>This class is used as a mixin.</p>\n\n<p>This class is to be used to provide basic methods for binding/unbinding stores to other\nclasses.</p>\n\n<p>This class is not intended for direct use but rather internally by those classes that\nmanage a Store.</p>\n"},"KeyNav":{"prototype":{"defaultEventAction":{"!type":"string","!doc":"<p>The method to call on the Ext.event.Event after this KeyNav intercepts a key. Valid values are Ext.event.Event.stopEvent, Ext.event.Event.preventDefault and Ext.event.Event.stopPropagation.</p>\n\n<p>If a falsy value is specified, no method is called on the key event.</p>\n"},"disabled":{"!type":"bool","!doc":"<p>True to disable this KeyNav instance.</p>\n"},"eventName":{"!type":"string","!doc":"<p>The event to listen for to pick up key events.</p>\n"},"forceKeyDown":{"!type":"bool","!doc":"<p>Handle the keydown event instead of keypress. KeyNav automatically does this for IE since IE does not propagate\nspecial keys on keypress, but setting this to true will force other browsers to also handle keydown instead of\nkeypress.</p>\n"},"ignoreInputFields":{"!type":"bool","!doc":"<p>Configure this as <code>true</code> if there are any input fields within the <a href=\"#!/api/Ext.util.KeyNav-cfg-target\" rel=\"Ext.util.KeyNav-cfg-target\" class=\"docClass\">target</a>, and this KeyNav\nshould not process events from input fields, (<code>&amp;lt;input&gt;, &amp;lt;textarea&gt; and elements with</code>contentEditable=\"true\"`)</p>\n"},"keyMap":{"!doc":"<p>An optional pre-existing <a href=\"#!/api/Ext.util.KeyMap\" rel=\"Ext.util.KeyMap\" class=\"docClass\">KeyMap</a> to use to listen for key events. If not specified,\none is created.</p>\n"},"processEvent":{"!doc":"<p>An optional event processor function which accepts the argument list provided by the <a href=\"#!/api/Ext.util.KeyNav-cfg-eventName\" rel=\"Ext.util.KeyNav-cfg-eventName\" class=\"docClass\">configured\nevent</a> of the <a href=\"#!/api/Ext.util.KeyNav-cfg-target\" rel=\"Ext.util.KeyNav-cfg-target\" class=\"docClass\">target</a>, and returns a keyEvent for processing by the KeyMap.</p>\n\n<p>This may be useful when the <a href=\"#!/api/Ext.util.KeyNav-cfg-target\" rel=\"Ext.util.KeyNav-cfg-target\" class=\"docClass\">target</a> is a Component with s complex event signature. Extra information from\nthe event arguments may be injected into the event for use by the handler functions before returning it.</p>\n"},"processEventScope":{"!doc":"<p>The scope (<code>this</code> context) in which the <a href=\"#!/api/Ext.util.KeyNav-cfg-processEvent\" rel=\"Ext.util.KeyNav-cfg-processEvent\" class=\"docClass\">processEvent</a> method is executed.</p>\n"},"target":{"!doc":"<p>The object on which to listen for the event specified by the <a href=\"#!/api/Ext.util.KeyNav-cfg-eventName\" rel=\"Ext.util.KeyNav-cfg-eventName\" class=\"docClass\">eventName</a> config option.</p>\n"},"destroy":{"!type":"fn(removeEl: ?) -> !this","!doc":"<p>Destroy this KeyNav.</p>\n"},"disable":{"!type":"fn() -> !this","!doc":"<p>Disables this KeyNav.</p>\n"},"enable":{"!type":"fn() -> !this","!doc":"<p>Enables this KeyNav.</p>\n"},"getKeyEvent":{"!type":"fn(forceKeyDown: ?, configuredEventName: ?) -> string","!doc":"<p>Determines the event to bind to listen for keys. Defaults to the <a href=\"#!/api/Ext.util.KeyNav-cfg-eventName\" rel=\"Ext.util.KeyNav-cfg-eventName\" class=\"docClass\">eventName</a> value, but\nmay be overridden the <a href=\"#!/api/Ext.util.KeyNav-cfg-forceKeyDown\" rel=\"Ext.util.KeyNav-cfg-forceKeyDown\" class=\"docClass\">forceKeyDown</a> setting.</p>\n"},"handleEvent":{"!type":"fn(keyCode: ?, event: ?, options: ?) -> !this","!doc":"<p>Method for filtering out the map argument</p>\n"},"legacyConstructor":{"!type":"fn(el: ?, config: ?) -> !this","!doc":"<p>Old constructor signature.</p>\n"},"setConfig":{"!type":"fn(config: ?) -> !this","!doc":"<p>Sets up a configuration for the KeyNav.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?) -> !this","!doc":"<p>Convenience function for setting disabled/enabled by boolean.</p>\n"},"keyOptions":{}},"!doc":"<p>Provides a convenient wrapper for normalized keyboard navigation. KeyNav allows you to bind navigation keys to\nfunction calls that will get called when the keys are pressed, providing an easy way to implement custom navigation\nschemes for any UI component.</p>\n\n<p>The following are all of the possible keys that can be implemented: enter, space, left, right, up, down, tab, esc,\npageUp, pageDown, del, backspace, home, end.</p>\n\n<p>Usage:</p>\n\n<pre><code>var nav = new <a href=\"#!/api/Ext.util.KeyNav\" rel=\"Ext.util.KeyNav\" class=\"docClass\">Ext.util.KeyNav</a>({\n    target : \"my-element\",\n    left   : function(e){\n        this.moveLeft(e.ctrlKey);\n    },\n    right  : function(e){\n        this.moveRight(e.ctrlKey);\n    },\n    enter  : function(e){\n        this.save();\n    },\n\n    // Binding may be a function specifiying fn, scope and defaultAction\n    esc: {\n        fn: this.onEsc,\n        defaultEventAction: false\n    },\n\n    // Binding may be keyed by a single character\n    A: {\n        ctrl: true,\n        fn: selectAll\n    },\n\n    // Binding may be keyed by a key code (45 = INSERT)\n    45: {\n        fn: doInsert\n    }\n    scope : this\n});\n</code></pre>\n","!type":"fn(config: ?)"},"ClickRepeater":{"!doc":"<p>A wrapper class which can be applied to any element. Fires a \"click\" event while the\nmouse is pressed. The interval between firings may be specified in the config but\ndefaults to 20 milliseconds.</p>\n\n<p>Optionally, a CSS class may be applied to the element during the time it is pressed.</p>\n","!type":"fn(el: ?, config?: ?)","prototype":{"accelerate":{"!type":"bool","!doc":"<p>True if autorepeating should start slowly and accelerate.\n\"interval\" and \"delay\" are ignored.</p>\n"},"delay":{"!type":"number","!doc":"<p>The initial delay before the repeating event begins firing.\nSimilar to an autorepeat key delay.</p>\n"},"el":{"!doc":"<p>The element to act as a button.</p>\n"},"interval":{"!type":"number","!doc":"<p>The interval between firings of the \"click\" event (in milliseconds).</p>\n"},"pressedCls":{"!type":"string","!doc":"<p>A CSS class name to be applied to the element while pressed.</p>\n"},"preventDefault":{"!type":"bool","!doc":"<p>True to prevent the default click event</p>\n"},"stopDefault":{"!type":"bool","!doc":"<p>True to stop the default click event</p>\n"},"timer":{"!type":"number"},"click":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires on a specified interval during the time the element is pressed.</p>\n"},"destroy":{"!type":"fn() -> !this"},"disable":{"!type":"fn(force: ?) -> !this","!doc":"<p>Disables the repeater and stops events from firing.</p>\n"},"easeOutExpo":{"!type":"fn(t: ?, b: ?, c: ?, d: ?) -> !this"},"enable":{"!type":"fn() -> !this","!doc":"<p>Enables the repeater and allows events to fire.</p>\n"},"eventOptions":{"!type":"fn(e: ?) -> !this"},"handleDblClick":{"!type":"fn(e: ?) -> !this"},"handleMouseDown":{"!type":"fn(e: ?) -> !this"},"handleMouseOut":{"!type":"fn() -> !this"},"handleMouseReturn":{"!type":"fn() -> !this"},"handleMouseUp":{"!type":"fn(e: ?) -> !this"},"setDisabled":{"!type":"fn(disabled: ?) -> !this","!doc":"<p>Convenience function for setting disabled/enabled by boolean.</p>\n"},"mousedown":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse button is depressed.</p>\n"},"mouseup":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse key is released.</p>\n"}}},"ComponentDragger":{"!doc":"<p>A subclass of <a href=\"#!/api/Ext.dd.DragTracker\" rel=\"Ext.dd.DragTracker\" class=\"docClass\">Ext.dd.DragTracker</a> which handles dragging any Component.</p>\n\n<p>This is configured with a Component to be made draggable, and a config object for the <a href=\"#!/api/Ext.dd.DragTracker\" rel=\"Ext.dd.DragTracker\" class=\"docClass\">Ext.dd.DragTracker</a>\nclass.</p>\n\n<p>A <a href=\"#!/api/Ext.util.ComponentDragger-cfg-delegate\" rel=\"Ext.util.ComponentDragger-cfg-delegate\" class=\"docClass\">delegate</a> may be provided which may be either the element to use as the mousedown target or a\nCSS selector to activate multiple mousedown targets.</p>\n\n<p>When the Component begins to be dragged, its <code>beginDrag</code> method will be called if implemented.</p>\n\n<p>When the drag ends, its <code>endDrag</code> method will be called if implemented.</p>\n","!type":"fn(comp: ?, config?: ?)","prototype":{"constrain":{"!type":"bool","!doc":"<p>Specify as <code>true</code> to constrain the Component to within the bounds of the <a href=\"#!/api/Ext.util.ComponentDragger-cfg-constrainTo\" rel=\"Ext.util.ComponentDragger-cfg-constrainTo\" class=\"docClass\">constrainTo</a> region.</p>\n"},"constrainDelegate":{"!type":"bool","!doc":"<p>Specify as <code>true</code> to constrain the drag handles within the <a href=\"#!/api/Ext.util.ComponentDragger-cfg-constrainTo\" rel=\"Ext.util.ComponentDragger-cfg-constrainTo\" class=\"docClass\">constrainTo</a> region.</p>\n"},"delegate":{"!doc":"<p>A CSS selector which identifies child elements within the Component's encapsulating\nElement which are the drag handles. This limits dragging to only begin when the matching elements are\nmousedowned.</p>\n\n<p>This may also be a specific child element within the Component's encapsulating element to use as the drag handle.</p>\n"},"calculateConstrainRegion":{"!type":"fn() -> !this"},"onDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>Move either the ghost Component or the target Component to its new position on drag</p>\n"},"onEnd":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method which should be overridden by each DragTracker instance. Called when a drag operation has been completed\n(e.g. the user clicked and held the mouse down, dragged the element and then released the mouse button)</p>\n"},"onStart":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method which should be overridden by each DragTracker instance. Called when a drag operation starts\n(e.g. the user has moved the tracked element beyond the specified tolerance)</p>\n"}}},"Cookies":{"!doc":"<p>Utility class for setting/reading values from browser cookies.\nValues can be written using the <a href=\"#!/api/Ext.util.Cookies-method-set\" rel=\"Ext.util.Cookies-method-set\" class=\"docClass\">set</a> method.\nValues can be read using the <a href=\"#!/api/Ext.util.Cookies-method-get\" rel=\"Ext.util.Cookies-method-get\" class=\"docClass\">get</a> method.\nA cookie can be invalidated on the client machine using the <a href=\"#!/api/Ext.util.Cookies-method-clear\" rel=\"Ext.util.Cookies-method-clear\" class=\"docClass\">clear</a> method.</p>\n","clear":{"!type":"fn(name: ?, path?: ?) -> !this","!doc":"<p>Removes a cookie with the provided name from the browser\nif found by setting its expiration date to sometime in the past.</p>\n"},"get":{"!type":"fn(name: ?) -> ?","!doc":"<p>Retrieves cookies that are accessible by the current page. If a cookie does not exist, <code>get()</code> returns null. The\nfollowing example retrieves the cookie called \"valid\" and stores the String value in the variable validStatus.</p>\n\n<pre><code>var validStatus = <a href=\"#!/api/Ext.util.Cookies-method-get\" rel=\"Ext.util.Cookies-method-get\" class=\"docClass\">Ext.util.Cookies.get</a>(\"valid\");\n</code></pre>\n"},"getCookieVal":{"!type":"fn(offset: ?) -> !this"},"set":{"!type":"fn(name: ?, value: ?, expires?: ?, path?: ?, domain?: ?, secure?: ?) -> !this","!doc":"<p>Creates a cookie with the specified name and value. Additional settings for the cookie may be optionally specified\n(for example: expiration, access restriction, SSL).</p>\n"}},"CSS":{"!doc":"<p>Utility class for manipulating CSS rules</p>\n","createRule":{"!type":"fn(styleSheet: ?, selector: ?, property: ?) -> ?","!doc":"<p>Creates a rule.</p>\n"},"createStyleSheet":{"!type":"fn(cssText: ?, id: ?) -> ?","!doc":"<p>Creates a stylesheet from a text blob of rules.\nThese rules will be wrapped in a STYLE tag and appended to the HEAD of the document.</p>\n"},"getRule":{"!type":"fn(selector: ?, refreshCache: ?) -> ?","!doc":"<p>Gets an an individual CSS rule by selector(s)</p>\n"},"getRules":{"!type":"fn(refreshCache: ?) -> ?","!doc":"<p>Gets all css rules for the document</p>\n"},"refreshCache":{"!type":"fn() -> ?","!doc":"<p>Refresh the rule cache if you have dynamically added stylesheets</p>\n"},"removeStyleSheet":{"!type":"fn(id: ?) -> !this","!doc":"<p>Removes a style or link tag by id</p>\n"},"swapStyleSheet":{"!type":"fn(id: ?, url: ?) -> !this","!doc":"<p>Dynamically swaps an existing stylesheet reference for a new one</p>\n"},"updateRule":{"!type":"fn(selector: ?, property: ?, value: ?) -> bool","!doc":"<p>Updates a rule property</p>\n"}},"History":{"!doc":"<p>History management component that allows you to register arbitrary tokens that signify application\nhistory state on navigation actions.  You can then handle the history <a href=\"#!/api/Ext.util.History-event-change\" rel=\"Ext.util.History-event-change\" class=\"docClass\">change</a> event in order\nto reset your application UI to the appropriate state when the user navigates forward or backward through\nthe browser history stack.</p>\n\n<h2>Initializing</h2>\n\n<p>The <a href=\"#!/api/Ext.util.History-method-init\" rel=\"Ext.util.History-method-init\" class=\"docClass\">init</a> method of the History object must be called before using History. This sets up the internal\nstate and must be the first thing called before using History.</p>\n","currentToken":{"!type":"string","!doc":"<p>The current token.</p>\n"},"useTopWindow":{"!type":"bool","!doc":"<p>True to use <code>window.top.location.hash</code> or false to use <code>window.location.hash</code>. Must be set before <a href=\"#!/api/Ext.util.History-method-init\" rel=\"Ext.util.History-method-init\" class=\"docClass\">init</a> is called\nbecause the <code>hashchange</code> event listener is added to the window at initialization time.</p>\n"},"add":{"!type":"fn(token: ?, preventDuplicates?: ?) -> !this","!doc":"<p>Add a new token to the history stack. This can be any arbitrary value, although it would\ncommonly be the concatenation of a component id and another id marking the specific history\nstate of that component. Example usage:</p>\n\n<pre><code>// Handle tab changes on a TabPanel\ntabPanel.on('tabchange', function(tabPanel, tab){\n     <a href=\"#!/api/Ext.util.History-method-add\" rel=\"Ext.util.History-method-add\" class=\"docClass\">Ext.History.add</a>(tabPanel.id + ':' + tab.id);\n});\n</code></pre>\n"},"back":{"!type":"fn() -> !this","!doc":"<p>Programmatically steps back one step in browser history (equivalent to the user pressing the Back button).</p>\n"},"forward":{"!type":"fn() -> !this","!doc":"<p>Programmatically steps forward one step in browser history (equivalent to the user pressing the Forward button).</p>\n"},"getHash":{"!type":"fn() -> string","!doc":"<p>Gets the actual hash from the url. This shouldn't need to be used directly but use the\n<a href=\"#!/api/Ext.util.History-method-getToken\" rel=\"Ext.util.History-method-getToken\" class=\"docClass\">getToken</a> method instead.</p>\n"},"getToken":{"!type":"fn() -> string","!doc":"<p>Retrieves the currently-active history token.</p>\n"},"handleStateChange":{"!type":"fn(token: ?) -> !this","!doc":"<p>Handles when the hash in the URL has been updated. Will also fired the change event.</p>\n"},"init":{"!type":"fn(onReady?: ?, scope?: ?) -> !this","!doc":"<p>Initializes the global History instance.</p>\n"},"setHash":{"!type":"fn(hash: ?) -> !this","!doc":"<p>Updates the hash on the window. This shouldn't need to be used directly but use the\n<a href=\"#!/api/Ext.util.History-method-add\" rel=\"Ext.util.History-method-add\" class=\"docClass\">add</a> method instead.</p>\n"},"startUp":{"!type":"fn() -> !this","!doc":"<p>Bootstraps the initialization the location.hash.\nThis will setup the Ext.TaskManager to poll for hash changes every 50ms.</p>\n"},"change":{"!type":"fn(token: ?, eOpts: ?)","!doc":"<p>Fires when navigation back or forwards within the local page's history occurs.</p>\n"},"ready":{"!type":"fn(The: ?, eOpts: ?)","!doc":"<p>Fires when the <a href=\"#!/api/Ext.util.History\" rel=\"Ext.util.History\" class=\"docClass\">Ext.util.History</a> singleton has been initialized and is ready for use.</p>\n"}},"KeyMap":{"!doc":"<p>Handles mapping key events to handling functions for an element or a Component. One KeyMap can be used for multiple\nactions.</p>\n\n<p>A KeyMap must be configured with a <a href=\"#!/api/Ext.util.KeyMap-cfg-target\" rel=\"Ext.util.KeyMap-cfg-target\" class=\"docClass\">target</a> as an event source which may be an Element or a Component.</p>\n\n<p>If the target is an element, then the <code>keydown</code> event will trigger the invocation of <a href=\"#!/api/Ext.util.KeyMap-cfg-binding\" rel=\"Ext.util.KeyMap-cfg-binding\" class=\"docClass\">binding</a>s.</p>\n\n<p>It is possible to configure the KeyMap with a custom <a href=\"#!/api/Ext.util.KeyMap-cfg-eventName\" rel=\"Ext.util.KeyMap-cfg-eventName\" class=\"docClass\">eventName</a> to listen for. This may be useful when the\n<a href=\"#!/api/Ext.util.KeyMap-cfg-target\" rel=\"Ext.util.KeyMap-cfg-target\" class=\"docClass\">target</a> is a Component.</p>\n\n<p>The KeyMap's event handling requires that the first parameter passed is a key event. So if the Component's event\nsignature is different, specify a <a href=\"#!/api/Ext.util.KeyMap-cfg-processEvent\" rel=\"Ext.util.KeyMap-cfg-processEvent\" class=\"docClass\">processEvent</a> configuration which accepts the event's parameters and\nreturns a key event.</p>\n\n<p>Functions specified in <a href=\"#!/api/Ext.util.KeyMap-cfg-binding\" rel=\"Ext.util.KeyMap-cfg-binding\" class=\"docClass\">binding</a>s are called with this signature : <code>(String key, Ext.event.Event e)</code> (if the\nmatch is a multi-key combination the callback will still be called only once). A KeyMap can also handle a string\nrepresentation of keys. By default KeyMap starts enabled.</p>\n\n<p>Usage:</p>\n\n<pre><code>// map one key by key code\nvar map = new <a href=\"#!/api/Ext.util.KeyMap\" rel=\"Ext.util.KeyMap\" class=\"docClass\">Ext.util.KeyMap</a>({\n    target: \"my-element\",\n    key: 13, // or Ext.event.Event.ENTER\n    fn: myHandler,\n    scope: myObject\n});\n\n// map multiple keys to one action by string\nvar map = new <a href=\"#!/api/Ext.util.KeyMap\" rel=\"Ext.util.KeyMap\" class=\"docClass\">Ext.util.KeyMap</a>({\n    target: \"my-element\",\n    key: \"a\\r\\n\\t\",\n    fn: myHandler,\n    scope: myObject\n});\n\n// map multiple keys to multiple actions by strings and array of codes\nvar map = new <a href=\"#!/api/Ext.util.KeyMap\" rel=\"Ext.util.KeyMap\" class=\"docClass\">Ext.util.KeyMap</a>({\n    target: \"my-element\",\n    binding: [{\n        key: [10,13],\n        fn: function(){ alert(\"Return was pressed\"); }\n    }, {\n        key: \"abc\",\n        fn: function(){ alert('a, b or c was pressed'); }\n    }, {\n        key: \"\\t\",\n        ctrl:true,\n        shift:true,\n        fn: function(){ alert('Control + shift + tab was pressed.'); }\n    }]\n});\n</code></pre>\n\n<p>Since 4.1.0, KeyMaps can bind to Components and process key-based events fired by Components.</p>\n\n<p>To bind to a Component, use the single parameter form of constructor and include the Component event name\nto listen for, and a <code>processEvent</code> implementation which returns the key event for further processing by\nthe KeyMap:</p>\n\n<pre><code>var map = new <a href=\"#!/api/Ext.util.KeyMap\" rel=\"Ext.util.KeyMap\" class=\"docClass\">Ext.util.KeyMap</a>({\n    target: myGridView,\n    eventName: 'itemkeydown',\n    processEvent: function(view, record, node, index, event) {\n\n        // Load the event with the extra information needed by the mappings\n        event.view = view;\n        event.store = view.getStore();\n        event.record = record;\n        event.index = index;\n        return event;\n    },\n    binding: {\n        key: Ext.event.Event.DELETE,\n        fn: function(keyCode, e) {\n            e.store.remove(e.record);\n\n            // Attempt to select the record that's now in its place\n            e.view.getSelectionModel().select(e.index);\n            e.view.el.focus();\n        }\n    }\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"binding":{"!doc":"<p>Either a single object describing a handling function for s specified key (or set of keys), or\nan array of such objects.</p>\n"},"eventName":{"!type":"string","!doc":"<p>The event to listen for to pick up key events.</p>\n"},"ignoreInputFields":{"!type":"bool","!doc":"<p>Configure this as <code>true</code> if there are any input fields within the <a href=\"#!/api/Ext.util.KeyMap-cfg-target\" rel=\"Ext.util.KeyMap-cfg-target\" class=\"docClass\">target</a>, and this KeyNav\nshould not process events from input fields, (<code>&amp;lt;input&gt;, &amp;lt;textarea&gt; and elements with</code>contentEditable=\"true\"`)</p>\n"},"processEvent":{"!doc":"<p>An optional event processor function which accepts the argument list provided by the\n<a href=\"#!/api/Ext.util.KeyMap-cfg-eventName\" rel=\"Ext.util.KeyMap-cfg-eventName\" class=\"docClass\">configured event</a> of the <a href=\"#!/api/Ext.util.KeyMap-cfg-target\" rel=\"Ext.util.KeyMap-cfg-target\" class=\"docClass\">target</a>, and returns a keyEvent for processing by the KeyMap.</p>\n\n<p>This may be useful when the <a href=\"#!/api/Ext.util.KeyMap-cfg-target\" rel=\"Ext.util.KeyMap-cfg-target\" class=\"docClass\">target</a> is a Component with a complex event signature, where the event is not\nthe first parameter. Extra information from the event arguments may be injected into the event for use by the handler\nfunctions before returning it.</p>\n"},"processEventScope":{"!doc":"<p>The scope (<code>this</code> context) in which the <a href=\"#!/api/Ext.util.KeyMap-cfg-processEvent\" rel=\"Ext.util.KeyMap-cfg-processEvent\" class=\"docClass\">processEvent</a> method is executed.</p>\n"},"target":{"!doc":"<p>The object on which to listen for the event specified by the <a href=\"#!/api/Ext.util.KeyMap-cfg-eventName\" rel=\"Ext.util.KeyMap-cfg-eventName\" class=\"docClass\">eventName</a> config option.</p>\n"},"addBinding":{"!type":"fn(binding: ?) -> !this","!doc":"<p>Add a new binding to this KeyMap.</p>\n\n<p>Usage:</p>\n\n<pre><code>// Create a KeyMap\nvar map = new <a href=\"#!/api/Ext.util.KeyMap\" rel=\"Ext.util.KeyMap\" class=\"docClass\">Ext.util.KeyMap</a>(document, {\n    key: Ext.event.Event.ENTER,\n    fn: handleKey,\n    scope: this\n});\n\n//Add a new binding to the existing KeyMap later\nmap.addBinding({\n    key: 'abc',\n    shift: true,\n    fn: handleKey,\n    scope: this\n});\n</code></pre>\n"},"checkModifiers":{"!type":"fn(binding: ?, event: ?) -> bool","!doc":"<p>Check if the modifiers on the event match those on the binding</p>\n"},"destroy":{"!type":"fn(removeTarget: ?) -> !this","!doc":"<p>Destroys the KeyMap instance and removes all handlers.</p>\n"},"disable":{"!type":"fn() -> !this","!doc":"<p>Disable this KeyMap</p>\n"},"enable":{"!type":"fn() -> !this","!doc":"<p>Enables this KeyMap</p>\n"},"handleTargetEvent":{"!type":"fn(event: ?) -> !this","!doc":"<p>Process the <a href=\"#!/api/Ext.util.KeyMap-cfg-eventName\" rel=\"Ext.util.KeyMap-cfg-eventName\" class=\"docClass\">event</a> from the <a href=\"#!/api/Ext.util.KeyMap-cfg-target\" rel=\"Ext.util.KeyMap-cfg-target\" class=\"docClass\">target</a>.</p>\n"},"isEnabled":{"!type":"fn() -> bool","!doc":"<p>Returns true if this KeyMap is enabled</p>\n"},"legacyConstructor":{"!type":"fn(el: ?, binding: ?, eventName?: ?) -> !this","!doc":"<p>Old constructor signature</p>\n"},"on":{"!type":"fn(key: ?, fn: ?, scope?: ?) -> !this","!doc":"<p>Shorthand for adding a single key listener.</p>\n"},"processBinding":{"!type":"fn(binding: ?, event: ?) -> !this","!doc":"<p>Process a particular binding and fire the handler if necessary.</p>\n"},"processKeys":{"!type":"fn(keyCode: ?) -> !this"},"removeBinding":{"!type":"fn(binding: ?) -> !this","!doc":"<p>Remove a binding from this KeyMap.</p>\n"},"setDisabled":{"!type":"fn(disabled: ?) -> !this","!doc":"<p>Convenience function for setting disabled/enabled by boolean.</p>\n"},"un":{"!type":"fn(key: ?, fn: ?, scope?: ?) -> !this","!doc":"<p>Shorthand for removing a single key listener.</p>\n"}}},"Memento":{"!doc":"<p>This class manages a set of captured properties from an object. These captured properties\ncan later be restored to an object.</p>\n","!type":"fn(target: ?, props: ?)","prototype":{"data":{"!doc":"<p>The collection of captured properties.</p>\n"},"target":{"!doc":"<p>The default target object for capture/restore (passed to the constructor).</p>\n"},"capture":{"!type":"fn(props: ?, target: ?) -> !this","!doc":"<p>Captures the specified properties from the target object in this memento.</p>\n"},"remove":{"!type":"fn(props: ?) -> !this","!doc":"<p>Removes the specified properties from this memento. These properties will not be\nrestored later without re-capturing their values.</p>\n"},"restore":{"!type":"fn(props: ?, clear: ?, target: ?) -> !this","!doc":"<p>Restores the specified properties from this memento to the target object.</p>\n"},"restoreAll":{"!type":"fn(clear: ?, target: ?) -> !this","!doc":"<p>Restores all captured properties in this memento to the target object.</p>\n"}}},"ProtoElement":{"!doc":"<p>Manages certain element-like data prior to rendering. These values are passed\non to the render process. This is currently used to manage the \"class\" and \"style\" attributes\nof a component's primary el as well as the bodyEl of panels. This allows things like\naddBodyCls in Panel to share logic with addCls in Component.</p>\n","prototype":{"clsProp":{"!type":"string","!doc":"<p>The property name for the className on the data object passed to <a href=\"#!/api/Ext.util.ProtoElement-method-writeTo\" rel=\"Ext.util.ProtoElement-method-writeTo\" class=\"docClass\">writeTo</a>.</p>\n"},"removedProp":{"!type":"string","!doc":"<p>The property name for the removed classes on the data object passed to <a href=\"#!/api/Ext.util.ProtoElement-method-writeTo\" rel=\"Ext.util.ProtoElement-method-writeTo\" class=\"docClass\">writeTo</a>.</p>\n"},"styleIsText":{"!type":"bool","!doc":"<p>True if the style must be converted to text during <a href=\"#!/api/Ext.util.ProtoElement-method-writeTo\" rel=\"Ext.util.ProtoElement-method-writeTo\" class=\"docClass\">writeTo</a>. When used to\npopulate tpl data, this will be true. When used to populate Ext.dom.Helper\nspecs, this will be false (the default).</p>\n"},"styleProp":{"!type":"string","!doc":"<p>The property name for the style on the data object passed to <a href=\"#!/api/Ext.util.ProtoElement-method-writeTo\" rel=\"Ext.util.ProtoElement-method-writeTo\" class=\"docClass\">writeTo</a>.</p>\n"},"addCls":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Adds class to the element.</p>\n"},"flush":{"!type":"fn() -> !this","!doc":"<p>Indicates that the current state of the object has been flushed to the DOM, so we need\nto track any subsequent changes</p>\n"},"hasCls":{"!type":"fn(cls: ?) -> bool","!doc":"<p>True if the element has given class.</p>\n"},"removeCls":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Removes class from the element.</p>\n"},"setStyle":{"!type":"fn(prop: ?, value?: ?) -> ?","!doc":"<p>Adds styles to the element.</p>\n"},"writeTo":{"!type":"fn(to: ?) -> ?","!doc":"<p>Writes style and class properties to given object.\nStyles will be written to <a href=\"#!/api/Ext.util.ProtoElement-property-styleProp\" rel=\"Ext.util.ProtoElement-property-styleProp\" class=\"docClass\">styleProp</a> and class names to <a href=\"#!/api/Ext.util.ProtoElement-property-clsProp\" rel=\"Ext.util.ProtoElement-property-clsProp\" class=\"docClass\">clsProp</a>.</p>\n"}}},"Queue":{"!doc":"<p>An internal Queue class.</p>\n","!type":"fn()","prototype":{"add":{"!type":"fn(obj: ?) -> !this"},"clear":{"!type":"fn() -> !this","!doc":"<p>Removes all items from the collection.</p>\n"},"contains":{"!type":"fn(obj: ?) -> !this"},"getCount":{"!type":"fn() -> number","!doc":"<p>Returns the number of items in the collection.</p>\n"},"getKey":{"!type":"fn(obj: ?) -> !this"},"remove":{"!type":"fn(obj: ?) -> ?","!doc":"<p>Remove an item from the collection.</p>\n"}}}},"state":{"Stateful":{"prototype":{"saveDelay":{"!type":"number","!doc":"<p>A buffer to be applied if many state events are fired within a short period.</p>\n"},"stateEvents":{"!doc":"<p>&lt;An array of events that, when fired, should trigger this object to\nsave its state. Defaults to none. <code>stateEvents</code> may be any type\nof event supported by this object, including browser or custom events\n(e.g., <code>['click', 'customerchange']</code>).</p>\n\n<p>See <code><a href=\"#!/api/Ext.state.Stateful-cfg-stateful\" rel=\"Ext.state.Stateful-cfg-stateful\" class=\"docClass\">stateful</a></code> for an explanation of saving and\nrestoring object state.</p>\n"},"stateId":{"!type":"string","!doc":"<p>The unique id for this object to use for state management purposes.</p>\n\n<p>See <a href=\"#!/api/Ext.state.Stateful-cfg-stateful\" rel=\"Ext.state.Stateful-cfg-stateful\" class=\"docClass\">stateful</a> for an explanation of saving and restoring state.</p>\n"},"stateful":{"!type":"bool","!doc":"<p>A flag which causes the object to attempt to restore the state of\ninternal properties from a saved state on startup. The object must have\na <a href=\"#!/api/Ext.state.Stateful-cfg-stateId\" rel=\"Ext.state.Stateful-cfg-stateId\" class=\"docClass\">stateId</a> for state to be managed.</p>\n\n<p>Auto-generated ids are not guaranteed to be stable across page loads and\ncannot be relied upon to save and restore the same state for a object.</p>\n\n<p>For state saving to work, the state manager's provider must have been\nset to an implementation of <a href=\"#!/api/Ext.state.Provider\" rel=\"Ext.state.Provider\" class=\"docClass\">Ext.state.Provider</a> which overrides the\n<a href=\"#!/api/Ext.state.Provider-method-set\" rel=\"Ext.state.Provider-method-set\" class=\"docClass\">set</a> and <a href=\"#!/api/Ext.state.Provider-method-get\" rel=\"Ext.state.Provider-method-get\" class=\"docClass\">get</a>\nmethods to save and recall name/value pairs. A built-in implementation,\n<a href=\"#!/api/Ext.state.CookieProvider\" rel=\"Ext.state.CookieProvider\" class=\"docClass\">Ext.state.CookieProvider</a> is available.</p>\n\n<p>To set the state provider for the current page:</p>\n\n<p>   <a href=\"#!/api/Ext.state.Manager-method-setProvider\" rel=\"Ext.state.Manager-method-setProvider\" class=\"docClass\">Ext.state.Manager.setProvider</a>(new <a href=\"#!/api/Ext.state.CookieProvider\" rel=\"Ext.state.CookieProvider\" class=\"docClass\">Ext.state.CookieProvider</a>({\n       expires: new Date(new Date().getTime()+(1000<em>60</em>60<em>24</em>7)), //7 days from now\n   }));</p>\n\n<p>A stateful object attempts to save state when one of the events\nlisted in the <a href=\"#!/api/Ext.state.Stateful-cfg-stateEvents\" rel=\"Ext.state.Stateful-cfg-stateEvents\" class=\"docClass\">stateEvents</a> configuration fires.</p>\n\n<p>To save state, a stateful object first serializes its state by\ncalling <em><a href=\"#!/api/Ext.state.Stateful-method-getState\" rel=\"Ext.state.Stateful-method-getState\" class=\"docClass\">getState</a></em>.</p>\n\n<p>The Component base class implements <a href=\"#!/api/Ext.state.Stateful-method-getState\" rel=\"Ext.state.Stateful-method-getState\" class=\"docClass\">getState</a> to save its width and height within the state\nonly if they were initially configured, and have changed from the configured value.</p>\n\n<p>The Panel class saves its collapsed state in addition to that.</p>\n\n<p>The Grid class saves its column state and store state (sorters and filters and grouper) in addition to its superclass state.</p>\n\n<p>If there is more application state to be save, the developer must provide an implementation which\nfirst calls the superclass method to inherit the above behaviour, and then injects new properties\ninto the returned object.</p>\n\n<p>The value yielded by getState is passed to <a href=\"#!/api/Ext.state.Manager-method-set\" rel=\"Ext.state.Manager-method-set\" class=\"docClass\">Ext.state.Manager.set</a>\nwhich uses the configured <a href=\"#!/api/Ext.state.Provider\" rel=\"Ext.state.Provider\" class=\"docClass\">Ext.state.Provider</a> to save the object\nkeyed by the <a href=\"#!/api/Ext.state.Stateful-cfg-stateId\" rel=\"Ext.state.Stateful-cfg-stateId\" class=\"docClass\">stateId</a>.</p>\n\n<p>During construction, a stateful object attempts to <em>restore</em> its state by calling\n<a href=\"#!/api/Ext.state.Manager-method-get\" rel=\"Ext.state.Manager-method-get\" class=\"docClass\">Ext.state.Manager.get</a> passing the <a href=\"#!/api/Ext.state.Stateful-cfg-stateId\" rel=\"Ext.state.Stateful-cfg-stateId\" class=\"docClass\">stateId</a></p>\n\n<p>The resulting object is passed to <a href=\"#!/api/Ext.state.Stateful-method-applyState\" rel=\"Ext.state.Stateful-method-applyState\" class=\"docClass\">applyState</a>*. The default implementation of\n<a href=\"#!/api/Ext.state.Stateful-method-applyState\" rel=\"Ext.state.Stateful-method-applyState\" class=\"docClass\">applyState</a> simply copies properties into the object, but a developer may\noverride this to support restoration of more complex application state.</p>\n\n<p>You can perform extra processing on state save and restore by attaching\nhandlers to the <a href=\"#!/api/Ext.state.Stateful-event-beforestaterestore\" rel=\"Ext.state.Stateful-event-beforestaterestore\" class=\"docClass\">beforestaterestore</a>, <a href=\"#!/api/Ext.state.Stateful-event-staterestore\" rel=\"Ext.state.Stateful-event-staterestore\" class=\"docClass\">staterestore</a>,\n<a href=\"#!/api/Ext.state.Stateful-event-beforestatesave\" rel=\"Ext.state.Stateful-event-beforestatesave\" class=\"docClass\">beforestatesave</a> and <a href=\"#!/api/Ext.state.Stateful-event-statesave\" rel=\"Ext.state.Stateful-event-statesave\" class=\"docClass\">statesave</a> events.</p>\n"},"addStateEvents":{"!type":"fn(events: ?) -> !this","!doc":"<p>Add events that will trigger the state to be saved. If the first argument is an\narray, each element of that array is the name of a state event. Otherwise, each\nargument passed to this method is the name of a state event.</p>\n"},"applyState":{"!type":"fn(state: ?) -> !this","!doc":"<p>Applies the state to the object. This should be overridden in subclasses to do\nmore complex state operations. By default it applies the state properties onto\nthe current object.</p>\n"},"getStateId":{"!type":"fn() -> string","!doc":"<p>Gets the state id for this object.</p>\n"},"initState":{"!type":"fn() -> !this","!doc":"<p>Initializes the state of the object upon construction.</p>\n"},"onStateChange":{"!type":"fn() -> !this","!doc":"<p>This method is called when any of the <a href=\"#!/api/Ext.state.Stateful-cfg-stateEvents\" rel=\"Ext.state.Stateful-cfg-stateEvents\" class=\"docClass\">stateEvents</a> are fired.</p>\n"},"savePropToState":{"!type":"fn(propName: ?, state: ?, stateName?: ?) -> bool","!doc":"<p>Conditionally saves a single property from this object to the given state object.\nThe idea is to only save state which has changed from the initial state so that\ncurrent software settings do not override future software settings. Only those\nvalues that are user-changed state should be saved.</p>\n"},"savePropsToState":{"!type":"fn(propNames: ?, state: ?) -> ?","!doc":"<p>Gathers additional named properties of the instance and adds their current values\nto the passed state object.</p>\n"},"saveState":{"!type":"fn() -> !this","!doc":"<p>Saves the state of the object to the persistence store.</p>\n"},"beforestaterestore":{"!type":"fn(this: ?, state: ?, eOpts: ?)","!doc":"<p>Fires before the state of the object is restored. Return false from an event handler to stop the restore.</p>\n"},"beforestatesave":{"!type":"fn(this: ?, state: ?, eOpts: ?)","!doc":"<p>Fires before the state of the object is saved to the configured state provider. Return false to stop the save.</p>\n"},"staterestore":{"!type":"fn(this: ?, state: ?, eOpts: ?)","!doc":"<p>Fires after the state of the object is restored.</p>\n"},"statesave":{"!type":"fn(this: ?, state: ?, eOpts: ?)","!doc":"<p>Fires after the state of the object is saved to the configured state provider.</p>\n"},"mixinId":{"!type":"string"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroys this stateful object.</p>\n"},"getState":{"!type":"fn() -> ?","!doc":"<p>Gets the current state of the object. By default this function returns null,\nit should be overridden in subclasses to implement methods for getting the state.</p>\n"}},"saveDelay":{"!type":"number","!doc":"<p>A buffer to be applied if many state events are fired within a short period.</p>\n"},"stateEvents":{"!doc":"<p>&lt;An array of events that, when fired, should trigger this object to\nsave its state. Defaults to none. <code>stateEvents</code> may be any type\nof event supported by this object, including browser or custom events\n(e.g., <code>['click', 'customerchange']</code>).</p>\n\n<p>See <code><a href=\"#!/api/Ext.state.Stateful-cfg-stateful\" rel=\"Ext.state.Stateful-cfg-stateful\" class=\"docClass\">stateful</a></code> for an explanation of saving and\nrestoring object state.</p>\n"},"stateId":{"!type":"string","!doc":"<p>The unique id for this object to use for state management purposes.</p>\n\n<p>See <a href=\"#!/api/Ext.state.Stateful-cfg-stateful\" rel=\"Ext.state.Stateful-cfg-stateful\" class=\"docClass\">stateful</a> for an explanation of saving and restoring state.</p>\n"},"stateful":{"!type":"bool","!doc":"<p>A flag which causes the object to attempt to restore the state of\ninternal properties from a saved state on startup. The object must have\na <a href=\"#!/api/Ext.state.Stateful-cfg-stateId\" rel=\"Ext.state.Stateful-cfg-stateId\" class=\"docClass\">stateId</a> for state to be managed.</p>\n\n<p>Auto-generated ids are not guaranteed to be stable across page loads and\ncannot be relied upon to save and restore the same state for a object.</p>\n\n<p>For state saving to work, the state manager's provider must have been\nset to an implementation of <a href=\"#!/api/Ext.state.Provider\" rel=\"Ext.state.Provider\" class=\"docClass\">Ext.state.Provider</a> which overrides the\n<a href=\"#!/api/Ext.state.Provider-method-set\" rel=\"Ext.state.Provider-method-set\" class=\"docClass\">set</a> and <a href=\"#!/api/Ext.state.Provider-method-get\" rel=\"Ext.state.Provider-method-get\" class=\"docClass\">get</a>\nmethods to save and recall name/value pairs. A built-in implementation,\n<a href=\"#!/api/Ext.state.CookieProvider\" rel=\"Ext.state.CookieProvider\" class=\"docClass\">Ext.state.CookieProvider</a> is available.</p>\n\n<p>To set the state provider for the current page:</p>\n\n<p>   <a href=\"#!/api/Ext.state.Manager-method-setProvider\" rel=\"Ext.state.Manager-method-setProvider\" class=\"docClass\">Ext.state.Manager.setProvider</a>(new <a href=\"#!/api/Ext.state.CookieProvider\" rel=\"Ext.state.CookieProvider\" class=\"docClass\">Ext.state.CookieProvider</a>({\n       expires: new Date(new Date().getTime()+(1000<em>60</em>60<em>24</em>7)), //7 days from now\n   }));</p>\n\n<p>A stateful object attempts to save state when one of the events\nlisted in the <a href=\"#!/api/Ext.state.Stateful-cfg-stateEvents\" rel=\"Ext.state.Stateful-cfg-stateEvents\" class=\"docClass\">stateEvents</a> configuration fires.</p>\n\n<p>To save state, a stateful object first serializes its state by\ncalling <em><a href=\"#!/api/Ext.state.Stateful-method-getState\" rel=\"Ext.state.Stateful-method-getState\" class=\"docClass\">getState</a></em>.</p>\n\n<p>The Component base class implements <a href=\"#!/api/Ext.state.Stateful-method-getState\" rel=\"Ext.state.Stateful-method-getState\" class=\"docClass\">getState</a> to save its width and height within the state\nonly if they were initially configured, and have changed from the configured value.</p>\n\n<p>The Panel class saves its collapsed state in addition to that.</p>\n\n<p>The Grid class saves its column state and store state (sorters and filters and grouper) in addition to its superclass state.</p>\n\n<p>If there is more application state to be save, the developer must provide an implementation which\nfirst calls the superclass method to inherit the above behaviour, and then injects new properties\ninto the returned object.</p>\n\n<p>The value yielded by getState is passed to <a href=\"#!/api/Ext.state.Manager-method-set\" rel=\"Ext.state.Manager-method-set\" class=\"docClass\">Ext.state.Manager.set</a>\nwhich uses the configured <a href=\"#!/api/Ext.state.Provider\" rel=\"Ext.state.Provider\" class=\"docClass\">Ext.state.Provider</a> to save the object\nkeyed by the <a href=\"#!/api/Ext.state.Stateful-cfg-stateId\" rel=\"Ext.state.Stateful-cfg-stateId\" class=\"docClass\">stateId</a>.</p>\n\n<p>During construction, a stateful object attempts to <em>restore</em> its state by calling\n<a href=\"#!/api/Ext.state.Manager-method-get\" rel=\"Ext.state.Manager-method-get\" class=\"docClass\">Ext.state.Manager.get</a> passing the <a href=\"#!/api/Ext.state.Stateful-cfg-stateId\" rel=\"Ext.state.Stateful-cfg-stateId\" class=\"docClass\">stateId</a></p>\n\n<p>The resulting object is passed to <a href=\"#!/api/Ext.state.Stateful-method-applyState\" rel=\"Ext.state.Stateful-method-applyState\" class=\"docClass\">applyState</a>*. The default implementation of\n<a href=\"#!/api/Ext.state.Stateful-method-applyState\" rel=\"Ext.state.Stateful-method-applyState\" class=\"docClass\">applyState</a> simply copies properties into the object, but a developer may\noverride this to support restoration of more complex application state.</p>\n\n<p>You can perform extra processing on state save and restore by attaching\nhandlers to the <a href=\"#!/api/Ext.state.Stateful-event-beforestaterestore\" rel=\"Ext.state.Stateful-event-beforestaterestore\" class=\"docClass\">beforestaterestore</a>, <a href=\"#!/api/Ext.state.Stateful-event-staterestore\" rel=\"Ext.state.Stateful-event-staterestore\" class=\"docClass\">staterestore</a>,\n<a href=\"#!/api/Ext.state.Stateful-event-beforestatesave\" rel=\"Ext.state.Stateful-event-beforestatesave\" class=\"docClass\">beforestatesave</a> and <a href=\"#!/api/Ext.state.Stateful-event-statesave\" rel=\"Ext.state.Stateful-event-statesave\" class=\"docClass\">statesave</a> events.</p>\n"},"addStateEvents":{"!type":"fn(events: ?) -> !this","!doc":"<p>Add events that will trigger the state to be saved. If the first argument is an\narray, each element of that array is the name of a state event. Otherwise, each\nargument passed to this method is the name of a state event.</p>\n"},"getStateId":{"!type":"fn() -> string","!doc":"<p>Gets the state id for this object.</p>\n"},"initState":{"!type":"fn() -> !this","!doc":"<p>Initializes the state of the object upon construction.</p>\n"},"onStateChange":{"!type":"fn() -> !this","!doc":"<p>This method is called when any of the <a href=\"#!/api/Ext.state.Stateful-cfg-stateEvents\" rel=\"Ext.state.Stateful-cfg-stateEvents\" class=\"docClass\">stateEvents</a> are fired.</p>\n"},"savePropToState":{"!type":"fn(propName: ?, state: ?, stateName?: ?) -> bool","!doc":"<p>Conditionally saves a single property from this object to the given state object.\nThe idea is to only save state which has changed from the initial state so that\ncurrent software settings do not override future software settings. Only those\nvalues that are user-changed state should be saved.</p>\n"},"savePropsToState":{"!type":"fn(propNames: ?, state: ?) -> ?","!doc":"<p>Gathers additional named properties of the instance and adds their current values\nto the passed state object.</p>\n"},"saveState":{"!type":"fn() -> !this","!doc":"<p>Saves the state of the object to the persistence store.</p>\n"},"beforestaterestore":{"!type":"fn(this: ?, state: ?, eOpts: ?)","!doc":"<p>Fires before the state of the object is restored. Return false from an event handler to stop the restore.</p>\n"},"beforestatesave":{"!type":"fn(this: ?, state: ?, eOpts: ?)","!doc":"<p>Fires before the state of the object is saved to the configured state provider. Return false to stop the save.</p>\n"},"staterestore":{"!type":"fn(this: ?, state: ?, eOpts: ?)","!doc":"<p>Fires after the state of the object is restored.</p>\n"},"statesave":{"!type":"fn(this: ?, state: ?, eOpts: ?)","!doc":"<p>Fires after the state of the object is saved to the configured state provider.</p>\n"},"!doc":"<p>A mixin for being able to save the state of an object to an underlying\n<a href=\"#!/api/Ext.state.Provider\" rel=\"Ext.state.Provider\" class=\"docClass\">Ext.state.Provider</a>.</p>\n"},"CookieProvider":{"!doc":"<p>A Provider implementation which saves and retrieves state via cookies. The CookieProvider supports the usual cookie\noptions, such as:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.state.CookieProvider-cfg-path\" rel=\"Ext.state.CookieProvider-cfg-path\" class=\"docClass\">path</a></li>\n<li><a href=\"#!/api/Ext.state.CookieProvider-cfg-expires\" rel=\"Ext.state.CookieProvider-cfg-expires\" class=\"docClass\">expires</a></li>\n<li><a href=\"#!/api/Ext.state.CookieProvider-cfg-domain\" rel=\"Ext.state.CookieProvider-cfg-domain\" class=\"docClass\">domain</a></li>\n<li><a href=\"#!/api/Ext.state.CookieProvider-cfg-secure\" rel=\"Ext.state.CookieProvider-cfg-secure\" class=\"docClass\">secure</a></li>\n</ul>\n\n\n<p>Example:</p>\n\n<pre><code>var cp = Ext.create('<a href=\"#!/api/Ext.state.CookieProvider\" rel=\"Ext.state.CookieProvider\" class=\"docClass\">Ext.state.CookieProvider</a>', {\n    path: \"/cgi-bin/\",\n    expires: new Date(new Date().getTime()+(1000*60*60*24*30)), //30 days\n    domain: \"sencha.com\"\n});\n\n<a href=\"#!/api/Ext.state.Manager-method-setProvider\" rel=\"Ext.state.Manager-method-setProvider\" class=\"docClass\">Ext.state.Manager.setProvider</a>(cp);\n</code></pre>\n","!type":"fn(config?: ?)","prototype":{"domain":{"!type":"string","!doc":"<p>The domain to save the cookie for. Note that you cannot specify a different domain than your page is on, but you can\nspecify a sub-domain, or simply the domain itself like 'sencha.com' to include all sub-domains if you need to access\ncookies across different sub-domains. Defaults to null which uses the same domain the page is running on including\nthe 'www' like 'www.sencha.com'.</p>\n"},"expires":{"!doc":"<p>The cookie expiration date. Defaults to 7 days from now.</p>\n"},"path":{"!type":"string","!doc":"<p>The path for which the cookie is active. Defaults to root '/' which makes it active for all pages in the site.</p>\n"},"secure":{"!type":"bool","!doc":"<p>True if the site is using SSL</p>\n"},"clear":{"!type":"fn(name: ?) -> !this","!doc":"<p>private</p>\n"},"clearCookie":{"!type":"fn(name: ?) -> !this","!doc":"<p>private</p>\n"},"readCookies":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> !this","!doc":"<p>private</p>\n"},"setCookie":{"!type":"fn(name: ?, value: ?) -> !this","!doc":"<p>private</p>\n"}}},"Provider":{"prototype":{"prefix":{"!type":"string","!doc":"<p>A string to prefix to items stored in the underlying state store.\nDefaults to <tt>'ext-'</tt></p>\n"},"decodeValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Decodes a string previously encoded with <a href=\"#!/api/Ext.state.Provider-method-encodeValue\" rel=\"Ext.state.Provider-method-encodeValue\" class=\"docClass\">encodeValue</a>.</p>\n"},"encodeValue":{"!type":"fn(value: ?) -> string","!doc":"<p>Encodes a value including type information.  Decode with <a href=\"#!/api/Ext.state.Provider-method-decodeValue\" rel=\"Ext.state.Provider-method-decodeValue\" class=\"docClass\">decodeValue</a>.</p>\n"},"get":{"!type":"fn(name: ?, defaultValue: ?) -> ?","!doc":"<p>Returns the current value for a key</p>\n"},"statechange":{"!type":"fn(this: ?, key: ?, value: ?, eOpts: ?)","!doc":"<p>Fires when a state change occurs.</p>\n"},"clear":{"!type":"fn(name: ?) -> !this","!doc":"<p>Clears a value from the state</p>\n"},"set":{"!type":"fn(name: ?, value: ?) -> !this","!doc":"<p>Sets the value for a key</p>\n"}},"!doc":"<p>Abstract base class for state provider implementations. The provider is responsible\nfor setting values  and extracting values to/from the underlying storage source. The \nstorage source can vary and the details should be implemented in a subclass. For example\na provider could use a server side database or the browser localstorage where supported.</p>\n\n\n\n\n<p>This class provides methods for encoding and decoding <b>typed</b> variables including \ndates and defines the Provider interface. By default these methods put the value and the\ntype information into a delimited string that can be stored. These should be overridden in \na subclass if you want to change the format of the encoded value and subsequent decoding.</p>\n\n"},"LocalStorageProvider":{"!doc":"<p>A Provider implementation which saves and retrieves state via the HTML5 localStorage API\nor IE <code>userData</code> storage. For details see <code>Ext.util.LocalStorage</code>.</p>\n\n<p>If the browser does not support local storage, there will be no attempt to read the state.\nBefore creating this class, check Ext.util.LocalStorage.supported.</p>\n","!type":"fn()","prototype":{"clear":{"!type":"fn(name: ?) -> !this","!doc":"<p>private</p>\n"},"getStorageObject":{"!type":"fn() -> !this"},"readLocalStorage":{"!type":"fn() -> !this"},"set":{"!type":"fn(name: ?, value: ?) -> !this","!doc":"<p>Sets the value for a key</p>\n"}}},"Manager":{"!doc":"<p>This is the global state manager. By default all components that are \"state aware\" check this class\nfor state information if you don't pass them a custom state provider. In order for this class\nto be useful, it must be initialized with a provider when your application initializes. Example usage:</p>\n\n<pre><code> // in your initialization function\n init: function() {\n     <a href=\"#!/api/Ext.state.Manager-method-setProvider\" rel=\"Ext.state.Manager-method-setProvider\" class=\"docClass\">Ext.state.Manager.setProvider</a>(new <a href=\"#!/api/Ext.state.CookieProvider\" rel=\"Ext.state.CookieProvider\" class=\"docClass\">Ext.state.CookieProvider</a>());\n }\n</code></pre>\n\n<p>This class passes on calls from components to the underlying <a href=\"#!/api/Ext.state.Provider\" rel=\"Ext.state.Provider\" class=\"docClass\">Ext.state.Provider</a> so that\nthere is a common interface that can be used without needing to refer to a specific provider instance\nin every component.</p>\n","!type":"fn()","clear":{"!type":"fn(key: ?) -> !this","!doc":"<p>Clears a value from the state</p>\n"},"get":{"!type":"fn(key: ?, defaultValue: ?) -> ?","!doc":"<p>Returns the current value for a key</p>\n"},"getProvider":{"!type":"fn() -> ?","!doc":"<p>Gets the currently configured state provider</p>\n"},"set":{"!type":"fn(key: ?, value: ?) -> !this","!doc":"<p>Sets the value for a key</p>\n"},"setProvider":{"!type":"fn(stateProvider: ?) -> !this","!doc":"<p>Configures the default state provider for your application</p>\n"}}},"container":{"Container":{"prototype":{"activeItem":{"!doc":"<p>A string component id or the numeric index of the component that should be\ninitially activated within the container's layout on render.  For example,\nactiveItem: 'item-1' or activeItem: 0 (index 0 = the first item in the\ncontainer's collection).  activeItem only applies to layout styles that can\ndisplay items one at a time (like <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> and\n<a href=\"#!/api/Ext.layout.container.Fit\" rel=\"Ext.layout.container.Fit\" class=\"docClass\">Ext.layout.container.Fit</a>).</p>\n"},"anchorSize":{"!doc":"<p>Defines the anchoring size of container.\nEither a number to define the width of the container or an object with <code>width</code> and <code>height</code> fields.</p>\n"},"autoDestroy":{"!type":"bool","!doc":"<p>If true the container will automatically destroy any contained component that is removed\nfrom it, else destruction must be handled manually.</p>\n"},"bubbleEvents":{"!doc":"<p>An array of events that, when fired, should be bubbled to any parent container.\nSee Ext.util.Observable.enableBubble.</p>\n"},"defaults":{"!doc":"<p>This option is a means of applying default settings to all added items whether added\nthrough the <a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a> config or via the <a href=\"#!/api/Ext.container.Container-method-add\" rel=\"Ext.container.Container-method-add\" class=\"docClass\">add</a> or <a href=\"#!/api/Ext.container.Container-method-insert\" rel=\"Ext.container.Container-method-insert\" class=\"docClass\">insert</a> methods.</p>\n\n<p>Defaults are applied to both config objects and instantiated components conditionally\nso as not to override existing properties in the item (see Ext.applyIf).</p>\n\n<p>If the defaults option is specified as a function, then the function will be called\nusing this Container as the scope (<code>this</code> reference) and passing the added item as\nthe first parameter. Any resulting object from that call is then applied to the item\nas default properties.</p>\n\n<p>For example, to automatically apply padding to the body of each of a set of\ncontained <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a> items, you could pass:\n<code>defaults: {bodyStyle:'padding:15px'}</code>.</p>\n\n<p>Usage:</p>\n\n<pre><code>defaults: { // defaults are applied to items, not the container\n    autoScroll: true\n},\nitems: [\n    // default will not be applied here, panel1 will be autoScroll: false\n    {\n        xtype: 'panel',\n        id: 'panel1',\n        autoScroll: false\n    },\n    // this component will have autoScroll: true\n    new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({\n        id: 'panel2'\n    })\n]\n</code></pre>\n"},"detachOnRemove":{"!type":"bool","!doc":"<p>True to move any component to the detachedBody when the component is\nremoved from this container. This option is only applicable when the component is not destroyed while\nbeing removed, see <a href=\"#!/api/Ext.container.Container-cfg-autoDestroy\" rel=\"Ext.container.Container-cfg-autoDestroy\" class=\"docClass\">autoDestroy</a> and <a href=\"#!/api/Ext.container.Container-method-remove\" rel=\"Ext.container.Container-method-remove\" class=\"docClass\">remove</a>. If this option is set to false, the DOM\nof the component will remain in the current place until it is explicitly moved.</p>\n"},"items":{"!doc":"<p>The MixedCollection containing all the child items of this container.</p>\n"},"referenceHolder":{"!type":"bool","!doc":"<p>If <code>true</code>, this container will be marked as being a point in the hierarchy where\nreferences to items with a specified <code>reference</code> config will be held. The container\nwill automatically become a referenceHolder if a controller is specified.</p>\n\n<p>See the introductory docs for <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a> for more information\nabout references &amp; reference holders.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.container.Container-method-getEl\" rel=\"Ext.container.Container-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.container.Container-method-getEl\" rel=\"Ext.container.Container-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.container.Container-cfg-renderSelectors\" rel=\"Ext.container.Container-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.container.Container-cfg-childEls\" rel=\"Ext.container.Container-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"suspendLayout":{"!type":"bool","!doc":"<p>If true, suspend calls to doLayout. Useful when batching multiple adds to a container\nand not passing them as multiple arguments or an array.</p>\n"},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"isContainer":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Container, or subclass thereof.</p>\n"},"layoutCounter":{"!type":"number","!doc":"<p>The number of container layout calls made on this object.</p>\n"},"add":{"!type":"fn(this: ?, component: ?, index: ?, eOpts: ?)","!doc":"<p>Fires after any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is added or inserted into the container.</p>\n\n<p><strong>This event bubbles:</strong> 'add' will also be fired when Component is added to any of\nthe child containers or their childern or ...</p>\n"},"afterComponentLayout":{"!type":"fn() -> !this","!doc":"<p>Called by the layout system after the Component has been laid out.</p>\n"},"afterLayout":{"!type":"fn(layout: ?) -> !this","!doc":"<p>Invoked after the Container has laid out (and rendered if necessary)\nits child Components.</p>\n"},"attachReference":{"!type":"fn(component: ?) -> !this","!doc":"<p>Sets up a component reference.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"cascade":{"!type":"fn(fn: ?, scope?: ?, args?: ?) -> ?","!doc":"<p>Cascades down the component/container heirarchy from this component (passed in\nthe first call), calling the specified function with each component. The scope\n(this reference) of the function call will be the scope provided or the current\ncomponent. The arguments to the function will be the args provided or the current\ncomponent. If the function returns false at any point, the cascade is stopped on\nthat branch.</p>\n"},"clearReference":{"!type":"fn(component: ?) -> !this","!doc":"<p>Clear a component reference.</p>\n"},"clearReferences":{"!type":"fn() -> !this","!doc":"<p>Invalidates the references collection. Typically called when\nremoving a container from this container, since it's difficult\nto know what references got removed.</p>\n"},"contains":{"!type":"fn(comp: ?, deep?: ?) -> bool","!doc":"<p>Determines whether the passed Component is either an immediate child of this Container,\nor whether it is a descendant.</p>\n"},"disable":{"!type":"fn() -> ?","!doc":"<p>Disables all child input fields and buttons.</p>\n"},"doLayout":{"!type":"fn() -> ?","!doc":"<p>Manually force this container's layout to be recalculated.</p>\n"},"enable":{"!type":"fn() -> ?","!doc":"<p>Enables all child input fields and buttons.</p>\n"},"getChildByElement":{"!type":"fn(el: ?, deep: ?) -> ?","!doc":"<p>Return the immediate child Component in which the passed element is located.</p>\n"},"getChildItemsToDisable":{"!type":"fn() -> ?","!doc":"<p>Gets a list of child components to enable/disable when the container is\nenabled/disabled</p>\n"},"getComponent":{"!type":"fn(comp: ?) -> ?","!doc":"<p>Examines this container's <a href=\"#!/api/Ext.container.Container-property-items\" rel=\"Ext.container.Container-property-items\" class=\"docClass\">items</a> <strong>property</strong> and gets a direct child\ncomponent of this container.</p>\n"},"getFocusEl":{"!type":"fn() -> ?","!doc":"<p>Returns the focus holder element associated with this Container. By default, this is the Container's target\nelement. Subclasses which use embedded focusable elements (such as Window and Button) should override this for use\nby the <a href=\"#!/api/Ext.container.Container-method-focus\" rel=\"Ext.container.Container-method-focus\" class=\"docClass\">focus</a> method.</p>\n"},"getLayout":{"!type":"fn() -> ?","!doc":"<p>Returns the <a href=\"#!/api/Ext.layout.container.Container\" rel=\"Ext.layout.container.Container\" class=\"docClass\">layout</a> instance currently associated with this Container.\nIf a layout has not been instantiated yet, that is done first</p>\n"},"getRefItems":{"!type":"fn(deep: ?) -> !this","!doc":"<p>Used by ComponentQuery, child and down to retrieve all of the items\nwhich can potentially be considered a child of this Container.</p>\n\n<p>This may be overriden by Components which have ownership of Components\nthat are not contained in the <a href=\"#!/api/Ext.container.Container-property-items\" rel=\"Ext.container.Container-property-items\" class=\"docClass\">items</a> collection.</p>\n\n<p>NOTE: IMPORTANT note for maintainers:\nItems are returned in tree traversal order. Each item is appended to the result array\nfollowed by the results of that child's getRefItems call.\nFloating child items are appended after internal child items.</p>\n"},"getReferences":{"!type":"fn() -> ?","!doc":"<p>Returns an object holding the descendants of this view keyed by their\n<code>reference</code>. This object should not be held\npast the scope of the function calling this method. It will not be valid if items\nare added or removed from this or any sub-container.</p>\n\n<p>The intended usage is shown here (assume there are 3 components with reference\nvalues of \"foo\", \"bar\" and \"baz\" at some level below this container):</p>\n\n<pre><code> onClick: function () {\n     var refs = this.getReferences();\n\n     // using \"refs\" we can access any descendant by its \"reference\"\n\n     refs.foo.getValue() + refs.bar.getValue() + refs.baz.getValue();\n }\n</code></pre>\n\n<p>If <code>this</code> component has a <code>reference</code> assigned\nto it, that is <strong>not</strong> included in this object. That reference is understood to\nbelong to the ancestor container configured as the <code>referenceHolder</code>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initInheritedState":{"!type":"fn(inheritedState: ?, inheritedStateInner: ?) -> !this","!doc":"<p>Called by <code>getInherited</code> to initialize the inheritedState the first time it is\nrequested.</p>\n"},"initItems":{"!type":"fn() -> !this","!doc":"<p>This method is called to initialize the <code>items</code> collection. A derived class can\noverride this method to do any last minute manipulation of <code>items</code> and then call\nthis method using <code>callParent</code>. Upon return, the <code>items</code> will no longer be a simple\narray.</p>\n"},"insert":{"!type":"fn(index: ?, component: ?) -> ?","!doc":"<p>Inserts a Component into this Container at a specified index. Fires the\n<a href=\"#!/api/Ext.container.Container-event-beforeadd\" rel=\"Ext.container.Container-event-beforeadd\" class=\"docClass\">beforeadd</a> event before inserting, then fires the <a href=\"#!/api/Ext.container.Container-event-add\" rel=\"Ext.container.Container-event-add\" class=\"docClass\">add</a>\nevent after the Component has been inserted.</p>\n"},"isAncestor":{"!type":"fn(possibleDescendant: ?) -> !this","!doc":"<p>Determines whether <strong>this Container</strong> is an ancestor of the passed Component.\nThis will return <code>true</code> if the passed Component is anywhere within the subtree\nbeneath this Container.</p>\n"},"lookupComponent":{"!type":"fn(comp: ?) -> !this"},"lookupReference":{"!type":"fn(key: ?) -> ?","!doc":"<p>Gets a reference to a child specified using the reference configuration.</p>\n"},"move":{"!type":"fn(this: ?, component: ?, prevIndex: ?, newIndex: ?, eOpts: ?)","!doc":"<p>Fires after any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> has changed its ordinal position within the container.</p>\n\n<p><strong>This event bubbles:</strong> 'move' will also be fired when Component is removed from any of\nthe child containers or their children or ...</p>\n"},"moveAfter":{"!type":"fn(item: ?, after: ?) -> !this","!doc":"<p>Moves the given <code>item</code> into this container following <code>after</code>. This method will\naccount for layout-generated components like splitters and should be used instead\nof index based <code><a href=\"#!/api/Ext.container.Container-event-move\" rel=\"Ext.container.Container-event-move\" class=\"docClass\">move</a></code>. If <code>after</code> is <code>null</code> then the <code>item</code> will be the\nfirst item in this container.</p>\n"},"moveBefore":{"!type":"fn(item: ?, before: ?) -> !this","!doc":"<p>Moves the given <code>item</code> into this container in front of <code>before</code>. This method will\naccount for layout-generated components like splitters and should be used instead\nof index based <code><a href=\"#!/api/Ext.container.Container-event-move\" rel=\"Ext.container.Container-event-move\" class=\"docClass\">move</a></code>. If <code>before</code> is <code>null</code> then the <code>item</code> will be the\nlast item in this container.</p>\n"},"nextChild":{"!type":"fn(child: ?, selector?: ?) -> ?","!doc":"<p>A method to find a child component after the passed child parameter. If a selector is also provided,\nthe first child component matching the selector will be returned.</p>\n"},"onAdded":{"!type":"fn(container: ?, pos: ?, instanced: ?) -> !this","!doc":"<p>Method to manage awareness of when components are added to their\nrespective Container, firing an <a href=\"#!/api/Ext.container.Container-event-added\" rel=\"Ext.container.Container-event-added\" class=\"docClass\">added</a> event. References are\nestablished at add time rather than at render time.</p>\n\n<p>Allows addition of behavior when a Component is added to a\nContainer. At this stage, the Component is in the parent\nContainer's collection of child items. After calling the\nsuperclass's <code>onAdded</code>, the <code>ownerCt</code> reference will be present,\nand if configured with a ref, the <code>refOwner</code> will be set.</p>\n"},"onBeforeAdd":{"!type":"fn(item: ?) -> !this","!doc":"<p>This method is invoked before adding a new child Component. It\nis passed the new Component, and may be used to modify the\nComponent, or prepare the Container in some way. Returning\nfalse aborts the add operation.</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onMove":{"!type":"fn() -> !this"},"onPosition":{"!type":"fn() -> !this","!doc":"<p>Called after the component is moved, this method is empty by default but can be implemented by any\nsubclass that needs to perform custom logic after a move occurs.</p>\n"},"onRemoved":{"!type":"fn(destroying: ?) -> !this","!doc":"<p>Method to manage awareness of when components are removed from their\nrespective Container, firing a <a href=\"#!/api/Ext.container.Container-event-removed\" rel=\"Ext.container.Container-event-removed\" class=\"docClass\">removed</a> event. References are properly\ncleaned up after removing a component from its owning container.</p>\n\n<p>Allows addition of behavior when a Component is removed from\nits parent Container. At this stage, the Component has been\nremoved from its parent Container's collection of child items,\nbut has not been destroyed (It will be destroyed if the parent\nContainer's <code>autoDestroy</code> is <code>true</code>, or if the remove call was\npassed a truthy second parameter). After calling the\nsuperclass's <code>onRemoved</code>, the <code>ownerCt</code> and the <code>refOwner</code> will not\nbe present.</p>\n"},"onResize":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the resize operation.</p>\n\n<p>Called when Ext.resizer.Resizer#drag event is fired.</p>\n"},"prevChild":{"!type":"fn(child: ?, selector?: ?) -> ?","!doc":"<p>A method to find a child component before the passed child parameter. If a selector is also provided,\nthe first child component matching the selector will be returned.</p>\n"},"remove":{"!type":"fn(this: ?, component: ?, eOpts: ?)","!doc":"<p>Fires after any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is removed from the container.</p>\n\n<p><strong>This event bubbles:</strong> 'remove' will also be fired when Component is removed from any of\nthe child containers or their children or ...</p>\n"},"removeAll":{"!type":"fn(autoDestroy?: ?) -> ?","!doc":"<p>Removes all components from this container.</p>\n"},"setActiveItem":{"!type":"fn(item: ?) -> ?","!doc":"<p>Sets a component as the active layout item. This only applies when using\na <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> layout.</p>\n\n<pre><code>var card1 = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {itemId: 'card-1'});\nvar card2 = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {itemId: 'card-2'});\nvar panel = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    layout: 'card',\n    items: [card1, card2]\n});\n// These are all equivalent\npanel.getLayout().setActiveItem(card2);\npanel.getLayout().setActiveItem('card-2');\npanel.getLayout().setActiveItem(1);\n</code></pre>\n"},"setLayout":{"!type":"fn(configuration: ?) -> !this","!doc":"<p>Reconfigures the initially configured <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a>.</p>\n\n<p>NOTE: this method cannot be used to change the \"type\" of layout after the component\nhas been rendered to the DOM. After rendering, this method can only modify the\nexisting layout's configuration properties. The reason for this restriction is that\nmany container layouts insert special wrapping elements into the dom, and the\nframework does not currently support dynamically changing these elements once\nrendered.</p>\n"},"afterlayout":{"!type":"fn(this: ?, layout: ?, eOpts: ?)","!doc":"<p>Fires when the components in this container are arranged by the associated layout manager.</p>\n"},"beforeadd":{"!type":"fn(this: ?, component: ?, index: ?, eOpts: ?)","!doc":"<p>Fires before any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is added or inserted into the container.\nA handler can return false to cancel the add.</p>\n"},"beforeremove":{"!type":"fn(this: ?, component: ?, eOpts: ?)","!doc":"<p>Fires before any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is removed from the container.  A handler can return\nfalse to cancel the remove.</p>\n"},"onAdd":{"!type":"fn(component: ?, position: ?) -> !this","!doc":"<p>This method is invoked after a new Component has been added. It\nis passed the Component which has been added. This method may\nbe used to update any internal structure which may depend upon\nthe state of the child items.</p>\n"},"onRemove":{"!type":"fn(component: ?, autoDestroy: ?) -> !this","!doc":"<p>This method is invoked after a new Component has been\nremoved. It is passed the Component which has been\nremoved. This method may be used to update any internal\nstructure which may depend upon the state of the child items.</p>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"defaultType":{"!type":"string","!doc":"<p>The default <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">xtype</a> of child Components to create in this Container when\na child item is specified as a raw configuration object, rather than as an instantiated Component.</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"beforeRender":{"!type":"fn() -> !this"}},"!doc":"<p>Base class for any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> that may contain other Components. Containers handle the basic behavior of\ncontaining items, namely adding, inserting and removing items.</p>\n\n<p>The most commonly used Container classes are <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>, <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a> and\n<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>. If you do not need the capabilities offered by the aforementioned classes you can create a\nlightweight Container to be encapsulated by an HTML element to your specifications by using the\n<a href=\"#!/api/Ext.Component-cfg-autoEl\" rel=\"Ext.Component-cfg-autoEl\" class=\"docClass\">autoEl</a> config option.</p>\n\n<p>The code below illustrates how to explicitly create a Container:</p>\n\n<pre class='inline-example '><code>// Explicitly create a Container\nExt.create('<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>', {\n    layout: {\n        type: 'hbox'\n    },\n    width: 400,\n    renderTo: Ext.getBody(),\n    border: 1,\n    style: {borderColor:'#000000', borderStyle:'solid', borderWidth:'1px'},\n    defaults: {\n        labelWidth: 80,\n        // implicitly create Container by specifying xtype\n        xtype: 'datefield',\n        flex: 1,\n        style: {\n            padding: '10px'\n        }\n    },\n    items: [{\n        xtype: 'datefield',\n        name: 'startDate',\n        fieldLabel: 'Start date'\n    },{\n        xtype: 'datefield',\n        name: 'endDate',\n        fieldLabel: 'End date'\n    }]\n});\n</code></pre>\n\n<h2>Layout</h2>\n\n<p>Container classes delegate the rendering of child Components to a layout manager class which must be configured into\nthe Container using the <code><a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a></code> configuration property.</p>\n\n<p>When either specifying child <code><a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a></code> of a Container, or dynamically <a href=\"#!/api/Ext.container.Container-method-add\" rel=\"Ext.container.Container-method-add\" class=\"docClass\">adding</a> Components to a\nContainer, remember to consider how you wish the Container to arrange those child elements, and whether those child\nelements need to be sized using one of Ext's built-in <code><a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a></code> schemes. By default, Containers use the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Auto</a> scheme which only renders child components, appending them one after the other\ninside the Container, and <strong>does not apply any sizing</strong> at all.</p>\n\n<p>A common mistake is when a developer neglects to specify a <code><a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a></code> (e.g. GridPanels or\nTreePanels are added to Containers for which no <code><a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a></code> has been specified). If a Container is left to\nuse the default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Auto</a> scheme, none of its child components will be resized, or changed in\nany way when the Container is resized.</p>\n\n<p>Certain layout managers allow dynamic addition of child components. Those that do include\n<a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a>, <a href=\"#!/api/Ext.layout.container.Anchor\" rel=\"Ext.layout.container.Anchor\" class=\"docClass\">Ext.layout.container.Anchor</a>, <a href=\"#!/api/Ext.layout.container.VBox\" rel=\"Ext.layout.container.VBox\" class=\"docClass\">Ext.layout.container.VBox</a>,\n<a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">Ext.layout.container.HBox</a>, and <a href=\"#!/api/Ext.layout.container.Table\" rel=\"Ext.layout.container.Table\" class=\"docClass\">Ext.layout.container.Table</a>. For example:</p>\n\n<pre><code>//  Create the GridPanel.\nvar myNewGrid = Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    store: myStore,\n    headers: myHeaders,\n    title: 'Results', // the title becomes the title of the tab\n});\n\nmyTabPanel.add(myNewGrid); // <a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a> implicitly uses <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Card</a>\nmyTabPanel.<a href=\"#!/api/Ext.tab.Panel-method-setActiveTab\" rel=\"Ext.tab.Panel-method-setActiveTab\" class=\"docClass\">setActiveTab</a>(myNewGrid);\n</code></pre>\n\n<p>The example above adds a newly created GridPanel to a TabPanel. Note that a TabPanel uses <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> as its layout manager which means all its child items are sized to <a href=\"#!/api/Ext.layout.container.Fit\" rel=\"Ext.layout.container.Fit\" class=\"docClass\">fit</a> exactly into its client area.</p>\n\n<p><strong><em>Overnesting is a common problem</em></strong>. An example of overnesting occurs when a GridPanel is added to a TabPanel by\nwrapping the GridPanel <em>inside</em> a wrapping Panel (that has no <code><a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a></code> specified) and then add that\nwrapping Panel to the TabPanel. The point to realize is that a GridPanel <strong>is</strong> a Component which can be added\ndirectly to a Container. If the wrapping Panel has no <code><a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a></code> configuration, then the overnested\nGridPanel will not be sized as expected.</p>\n\n<h2>References and <a href=\"#!/api/Ext.container.Container-cfg-referenceHolder\" rel=\"Ext.container.Container-cfg-referenceHolder\" class=\"docClass\">Reference Holders</a></h2>\n\n<p>Reference holders are used to keep references to child components inside a hierarchy.</p>\n\n<p>This functionality allows the connection of encapsulated references between containers\nand their child components declaratively. Simple usage:</p>\n\n<pre><code>Ext.define('Login', {\n    extend: '<a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>',\n\n    // This config is not compatible with the more common \"controller\" config\n    // used to specify a ViewController for the view. When a ViewController is\n    // specified it effectively acts as the \"reference holder\" for the view. In\n    // this example we simply mark this container as the reference holder for\n    // demonstration purposes.\n    referenceHolder: true,\n\n    title: 'Login',\n    items: [{\n        xtype: 'form',\n        items: [{\n            xtype: 'textfield',\n            reference: 'username', // A named reference to be held on the referenceHolder\n            name: 'username',\n            fieldLabel: 'Username'\n        }, {\n            xtype: 'textfield',\n            reference: 'password', // A named reference to be held on the referenceHolder\n            name: 'password',\n            fieldLabel: 'Password'\n        }] \n    }] \n});\nvar w = new Login();\nconsole.log(w.lookupReference('password')); // The password field\n</code></pre>\n\n<p>Reference holders are also encapsulated, so a reference will only be put on the closest\nreference holder above it in the component hierarchy:</p>\n\n<pre><code>var ct = new <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>({\n    referenceHolder: true,\n    items: [{\n        xtype: 'container',\n        referenceHolder: true,\n        reference: 'innerCt1',\n        items: [{\n            xtype: 'component',\n            reference: 'a',\n            id: 'a1'\n        }, {\n            xtype: 'component',\n            reference: 'b',\n            id: 'b1'\n        }]\n    }, {\n        xtype: 'container',\n        referenceHolder: true,\n        reference: 'innerCt2',\n        items: [{\n            xtype: 'component',\n            reference: 'a',\n            id: 'a2'\n        }, {\n            xtype: 'component',\n            reference: 'b',\n            id: 'b2'\n        }]\n    }]\n});\n// The main container will not have references to a/b, each innerCt will\nconsole.log(ct.lookupReference('a'), ct.lookupReference('b'));\nvar inner1 = ct.lookupReference('innerCt1');\nvar inner2 = ct.lookupReference('innerCt2');\n\nconsole.log(inner1.lookupReference('a').id, inner1.lookupReference('b').id);\nconsole.log(inner2.lookupReference('a').id, inner2.lookupReference('b').id);\n</code></pre>\n\n<p>If the view has a controller attached, it will automatically become a <a href=\"#!/api/Ext.container.Container-cfg-referenceHolder\" rel=\"Ext.container.Container-cfg-referenceHolder\" class=\"docClass\">referenceHolder</a>.\nReferences will be available in both the view and the controller:</p>\n\n<pre><code>Ext.define('ProfileController', {\n    extend: '<a href=\"#!/api/Ext.app.ViewController\" rel=\"Ext.app.ViewController\" class=\"docClass\">Ext.app.ViewController</a>',\n    alias: 'controller.profile',\n\n    init: function() {\n        console.log(this.lookupReference('firstName'));\n    }\n});\n\nExt.define('Profile', {\n    extend: '<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>',\n    controller: 'profile',\n    items: [{\n        xtype: 'textfield',\n        reference: 'firstName',\n        fieldLabel: 'First Name'\n    }]\n});\n\nnew Profile(); \n</code></pre>\n\n<h2>Events &amp; defaultListenerScope</h2>\n\n<p>Events can use the default listener scope to determine at runtime the appropriate place\nto fire. This allows for declarative binding of events in a useful way:</p>\n\n<pre><code>Ext.define('MyView', {\n    extend: '<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>',\n    defaultListenerScope: true,\n    referenceHolder: true,\n    items: [{\n        xtype: 'textfield',\n        reference: 'myfield'\n    }, {\n        xtype: 'button',\n        text: 'Set to A',\n        listeners: {\n            click: 'onButtonAClick'\n        }\n    }, {\n        xtype: 'button',\n        text: 'Set to B',\n        listeners: {\n            click: 'onButtonBClick'\n        }\n    }],\n\n    onButtonAClick: function() {\n        this.lookupReference('myfield').setValue('A');\n    },\n\n    onButtonBClick: function() {\n        this.lookupReference('myfield').setValue('B');\n    }\n});\n</code></pre>\n\n<p>Like <a href=\"#!/api/Ext.container.Container-cfg-referenceHolder\" rel=\"Ext.container.Container-cfg-referenceHolder\" class=\"docClass\">referenceHolder</a>, the defaultListenerScope is encapsulated, the scope will\nbe resolved at the closest defaultListenerScope above it in the component hierarchy:</p>\n\n<pre><code>var ct = new <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>({\n    defaultListenerScope: true,\n    onCustomEvent: function() {\n        console.log('Outer called'); // Will NOT be called\n    },\n    items: [{\n        xtype: 'container',\n        defaultListenerScope: true,\n        onCustomEvent: function() {\n            console.log('Inner called'); // Will be called\n        },\n        items: [{\n            xtype: 'component',\n            itemId: 'child',\n            listeners: {\n                customevent: 'onCustomEvent'\n            }\n        }]\n    }]\n});\n// The main container will not have references to a/b, each innerCt will\nconsole.log(ct.lookupReference('a'), ct.lookupReference('b'));\nvar inner1 = ct.lookupReference('innerCt1');\nvar inner2 = ct.lookupReference('innerCt2');\n\nconsole.log(inner1.lookupReference('a').id, inner1.lookupReference('b').id);\nconsole.log(inner2.lookupReference('a').id, inner2.lookupReference('b').id);\n</code></pre>\n\n<p>Similar to references, if a <a href=\"#!/api/Ext.app.ViewController\" rel=\"Ext.app.ViewController\" class=\"docClass\">Ext.app.ViewController</a> is attached to this view, it becomes\nthe defaultListenerScope, which means un-scoped, late bound events will be directed to the\ncontroller. This is powerful as it allows views to be totally declarative:</p>\n\n<pre><code>Ext.define('MyApp.controller.Login', {\n    extend : '<a href=\"#!/api/Ext.app.ViewController\" rel=\"Ext.app.ViewController\" class=\"docClass\">Ext.app.ViewController</a>',\n    alias : 'controller.login',\n\n    init: function() {\n        this.sendCount = 0;    \n    },\n\n    onLoginClick : function(btn) {\n        this.login();\n    },\n\n    onFieldSpecialKey : function(field, e) {\n        if (e.getKey() === e.ENTER) {\n           this.login();\n        }\n    },\n\n    login : function() {\n       var form = this.lookupReference('form');\n        this.lookupReference('error').hide();\n        if (form.isValid()) {\n            console.log('Do the login!');\n            // Server responded...\n            if (++this.sendCount % 2 === 0) {\n                this.onServerSuccess();\n            } else {\n                this.onServerFailure();\n            }\n        }\n    },\n\n    onServerSuccess: function() {\n        // Proceed   \n        console.log('All good');\n    },\n\n    onServerFailure: function() {\n        var error = this.lookupReference('error');\n        error.update('Invalid username/password');\n        error.show();\n    }\n});\n\nExt.define('MyApp.view.Login', {\n    extend : '<a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>',\n    controller : 'login',\n    referenceHolder: true,\n\n    title : 'Login',\n    width : 400,\n\n    items : [{\n        xtype : 'form',\n        reference : 'form',\n        border : false,\n        bodyPadding : 10,\n        defaultType : 'textfield',\n        defaults : {\n            anchor : '90%',\n            allowBlank : false,\n            enableKeyEvents : true\n        },\n        items : [{\n            xtype: 'component',\n            reference: 'error',\n            hidden: true,\n            margin: '0 0 10 0',\n            style: 'color: red;'\n        }, {\n            name : 'username',\n            fieldLabel : 'Username',\n            reference : 'username',\n            listeners : {\n                specialkey : 'onFieldSpecialKey'\n            }\n        }, {\n            name : 'password',\n            fieldLabel : 'Password',\n            reference : 'password',\n            inputType : 'password',\n            listeners : {\n                specialkey : 'onFieldSpecialKey'\n            }\n        }]\n    }],\n    buttons : ['-&gt;', {\n       text : 'Login',\n        listeners : {\n            click : 'onLoginClick'\n        }\n    }]\n});\n</code></pre>\n\n<h2>Adding via remote configuration</h2>\n\n<p>A server side script can be used to add Components which are generated dynamically on the server. An example of\nadding a GridPanel to a TabPanel where the GridPanel is generated by the server based on certain parameters:</p>\n\n<pre><code>// execute an Ajax request to invoke server side script:\nExt.Ajax.request({\n    url: 'gen-invoice-grid.php',\n    // send additional parameters to instruct server script\n    params: {\n        startDate: Ext.getCmp('start-date').getValue(),\n        endDate: Ext.getCmp('end-date').getValue()\n    },\n    // process the response object to add it to the TabPanel:\n    success: function(xhr) {\n        var newComponent = eval(xhr.responseText); // see discussion below\n        myTabPanel.add(newComponent); // add the component to the TabPanel\n        myTabPanel.setActiveTab(newComponent);\n    },\n    failure: function() {\n        <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>(\"Grid create failed\", \"Server communication failure\");\n    }\n});\n</code></pre>\n\n<p>The server script needs to return a JSON representation of a configuration object, which, when decoded will return a\nconfig object with an <a href=\"#!/api/Ext.Component-cfg-xtype\" rel=\"Ext.Component-cfg-xtype\" class=\"docClass\">xtype</a>. The server might return the following JSON:</p>\n\n<pre><code>{\n    \"xtype\": 'grid',\n    \"title\": 'Invoice Report',\n    \"store\": {\n        \"model\": 'Invoice',\n        \"proxy\": {\n            \"type\": 'ajax',\n            \"url\": 'get-invoice-data.php',\n            \"reader\": {\n                \"type\": 'json'\n                \"record\": 'transaction',\n                \"idProperty\": 'id',\n                \"totalRecords\": 'total'\n            })\n        },\n        \"autoLoad\": {\n            \"params\": {\n                \"startDate\": '01/01/2008',\n                \"endDate\": '01/31/2008'\n            }\n        }\n    },\n    \"headers\": [\n        {\"header\": \"Customer\", \"width\": 250, \"dataIndex\": 'customer', \"sortable\": true},\n        {\"header\": \"Invoice Number\", \"width\": 120, \"dataIndex\": 'invNo', \"sortable\": true},\n        {\"header\": \"Invoice Date\", \"width\": 100, \"dataIndex\": 'date', \"renderer\": Ext.util.Format.dateRenderer('M d, y'), \"sortable\": true},\n        {\"header\": \"Value\", \"width\": 120, \"dataIndex\": 'value', \"renderer\": 'usMoney', \"sortable\": true}\n    ]\n}\n</code></pre>\n\n<p>When the above code fragment is passed through the <code>eval</code> function in the success handler of the Ajax request, the\nresult will be a config object which, when added to a Container, will cause instantiation of a GridPanel. <strong>Be sure\nthat the Container is configured with a layout which sizes and positions the child items to your requirements.</strong></p>\n\n<p><strong>Note:</strong> since the code above is <em>generated</em> by a server script, the <code>autoLoad</code> params for the Store, the user's\npreferred date format, the metadata to allow generation of the Model layout, and the ColumnModel can all be generated\ninto the code since these are all known on the server.</p>\n","!type":"fn(config: ?)","activeItem":{"!doc":"<p>A string component id or the numeric index of the component that should be\ninitially activated within the container's layout on render.  For example,\nactiveItem: 'item-1' or activeItem: 0 (index 0 = the first item in the\ncontainer's collection).  activeItem only applies to layout styles that can\ndisplay items one at a time (like <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> and\n<a href=\"#!/api/Ext.layout.container.Fit\" rel=\"Ext.layout.container.Fit\" class=\"docClass\">Ext.layout.container.Fit</a>).</p>\n"},"anchorSize":{"!doc":"<p>Defines the anchoring size of container.\nEither a number to define the width of the container or an object with <code>width</code> and <code>height</code> fields.</p>\n"},"autoDestroy":{"!type":"bool","!doc":"<p>If true the container will automatically destroy any contained component that is removed\nfrom it, else destruction must be handled manually.</p>\n"},"bubbleEvents":{"!doc":"<p>An array of events that, when fired, should be bubbled to any parent container.\nSee Ext.util.Observable.enableBubble.</p>\n"},"defaultType":{"!type":"string","!doc":"<p>The default <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">xtype</a> of child Components to create in this Container when\na child item is specified as a raw configuration object, rather than as an instantiated Component.</p>\n"},"defaults":{"!doc":"<p>This option is a means of applying default settings to all added items whether added\nthrough the <a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a> config or via the <a href=\"#!/api/Ext.container.Container-method-add\" rel=\"Ext.container.Container-method-add\" class=\"docClass\">add</a> or <a href=\"#!/api/Ext.container.Container-method-insert\" rel=\"Ext.container.Container-method-insert\" class=\"docClass\">insert</a> methods.</p>\n\n<p>Defaults are applied to both config objects and instantiated components conditionally\nso as not to override existing properties in the item (see Ext.applyIf).</p>\n\n<p>If the defaults option is specified as a function, then the function will be called\nusing this Container as the scope (<code>this</code> reference) and passing the added item as\nthe first parameter. Any resulting object from that call is then applied to the item\nas default properties.</p>\n\n<p>For example, to automatically apply padding to the body of each of a set of\ncontained <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a> items, you could pass:\n<code>defaults: {bodyStyle:'padding:15px'}</code>.</p>\n\n<p>Usage:</p>\n\n<pre><code>defaults: { // defaults are applied to items, not the container\n    autoScroll: true\n},\nitems: [\n    // default will not be applied here, panel1 will be autoScroll: false\n    {\n        xtype: 'panel',\n        id: 'panel1',\n        autoScroll: false\n    },\n    // this component will have autoScroll: true\n    new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({\n        id: 'panel2'\n    })\n]\n</code></pre>\n"},"detachOnRemove":{"!type":"bool","!doc":"<p>True to move any component to the detachedBody when the component is\nremoved from this container. This option is only applicable when the component is not destroyed while\nbeing removed, see <a href=\"#!/api/Ext.container.Container-cfg-autoDestroy\" rel=\"Ext.container.Container-cfg-autoDestroy\" class=\"docClass\">autoDestroy</a> and <a href=\"#!/api/Ext.container.Container-method-remove\" rel=\"Ext.container.Container-method-remove\" class=\"docClass\">remove</a>. If this option is set to false, the DOM\nof the component will remain in the current place until it is explicitly moved.</p>\n"},"items":{"!doc":"<p>The MixedCollection containing all the child items of this container.</p>\n"},"referenceHolder":{"!type":"bool","!doc":"<p>If <code>true</code>, this container will be marked as being a point in the hierarchy where\nreferences to items with a specified <code>reference</code> config will be held. The container\nwill automatically become a referenceHolder if a controller is specified.</p>\n\n<p>See the introductory docs for <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a> for more information\nabout references &amp; reference holders.</p>\n"},"suspendLayout":{"!type":"bool","!doc":"<p>If true, suspend calls to doLayout. Useful when batching multiple adds to a container\nand not passing them as multiple arguments or an array.</p>\n"},"isContainer":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Container, or subclass thereof.</p>\n"},"layoutCounter":{"!type":"number","!doc":"<p>The number of container layout calls made on this object.</p>\n"},"add":{"!type":"fn(this: ?, component: ?, index: ?, eOpts: ?)","!doc":"<p>Fires after any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is added or inserted into the container.</p>\n\n<p><strong>This event bubbles:</strong> 'add' will also be fired when Component is added to any of\nthe child containers or their childern or ...</p>\n"},"afterComponentLayout":{"!type":"fn() -> !this","!doc":"<p>Called by the layout system after the Component has been laid out.</p>\n"},"afterLayout":{"!type":"fn(layout: ?) -> !this","!doc":"<p>Invoked after the Container has laid out (and rendered if necessary)\nits child Components.</p>\n"},"attachReference":{"!type":"fn(component: ?) -> !this","!doc":"<p>Sets up a component reference.</p>\n"},"cascade":{"!type":"fn(fn: ?, scope?: ?, args?: ?) -> ?","!doc":"<p>Cascades down the component/container heirarchy from this component (passed in\nthe first call), calling the specified function with each component. The scope\n(this reference) of the function call will be the scope provided or the current\ncomponent. The arguments to the function will be the args provided or the current\ncomponent. If the function returns false at any point, the cascade is stopped on\nthat branch.</p>\n"},"clearReference":{"!type":"fn(component: ?) -> !this","!doc":"<p>Clear a component reference.</p>\n"},"clearReferences":{"!type":"fn() -> !this","!doc":"<p>Invalidates the references collection. Typically called when\nremoving a container from this container, since it's difficult\nto know what references got removed.</p>\n"},"contains":{"!type":"fn(comp: ?, deep?: ?) -> bool","!doc":"<p>Determines whether the passed Component is either an immediate child of this Container,\nor whether it is a descendant.</p>\n"},"disable":{"!type":"fn() -> ?","!doc":"<p>Disables all child input fields and buttons.</p>\n"},"doLayout":{"!type":"fn() -> ?","!doc":"<p>Manually force this container's layout to be recalculated.</p>\n"},"enable":{"!type":"fn() -> ?","!doc":"<p>Enables all child input fields and buttons.</p>\n"},"getChildByElement":{"!type":"fn(el: ?, deep: ?) -> ?","!doc":"<p>Return the immediate child Component in which the passed element is located.</p>\n"},"getChildItemsToDisable":{"!type":"fn() -> ?","!doc":"<p>Gets a list of child components to enable/disable when the container is\nenabled/disabled</p>\n"},"getLayout":{"!type":"fn() -> ?","!doc":"<p>Returns the <a href=\"#!/api/Ext.layout.container.Container\" rel=\"Ext.layout.container.Container\" class=\"docClass\">layout</a> instance currently associated with this Container.\nIf a layout has not been instantiated yet, that is done first</p>\n"},"getReferences":{"!type":"fn() -> ?","!doc":"<p>Returns an object holding the descendants of this view keyed by their\n<code>reference</code>. This object should not be held\npast the scope of the function calling this method. It will not be valid if items\nare added or removed from this or any sub-container.</p>\n\n<p>The intended usage is shown here (assume there are 3 components with reference\nvalues of \"foo\", \"bar\" and \"baz\" at some level below this container):</p>\n\n<pre><code> onClick: function () {\n     var refs = this.getReferences();\n\n     // using \"refs\" we can access any descendant by its \"reference\"\n\n     refs.foo.getValue() + refs.bar.getValue() + refs.baz.getValue();\n }\n</code></pre>\n\n<p>If <code>this</code> component has a <code>reference</code> assigned\nto it, that is <strong>not</strong> included in this object. That reference is understood to\nbelong to the ancestor container configured as the <code>referenceHolder</code>.</p>\n"},"initInheritedState":{"!type":"fn(inheritedState: ?, inheritedStateInner: ?) -> !this","!doc":"<p>Called by <code>getInherited</code> to initialize the inheritedState the first time it is\nrequested.</p>\n"},"insert":{"!type":"fn(index: ?, component: ?) -> ?","!doc":"<p>Inserts a Component into this Container at a specified index. Fires the\n<a href=\"#!/api/Ext.container.Container-event-beforeadd\" rel=\"Ext.container.Container-event-beforeadd\" class=\"docClass\">beforeadd</a> event before inserting, then fires the <a href=\"#!/api/Ext.container.Container-event-add\" rel=\"Ext.container.Container-event-add\" class=\"docClass\">add</a>\nevent after the Component has been inserted.</p>\n"},"isAncestor":{"!type":"fn(possibleDescendant: ?) -> !this","!doc":"<p>Determines whether <strong>this Container</strong> is an ancestor of the passed Component.\nThis will return <code>true</code> if the passed Component is anywhere within the subtree\nbeneath this Container.</p>\n"},"lookupComponent":{"!type":"fn(comp: ?) -> !this"},"lookupReference":{"!type":"fn(key: ?) -> ?","!doc":"<p>Gets a reference to a child specified using the reference configuration.</p>\n"},"move":{"!type":"fn(this: ?, component: ?, prevIndex: ?, newIndex: ?, eOpts: ?)","!doc":"<p>Fires after any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> has changed its ordinal position within the container.</p>\n\n<p><strong>This event bubbles:</strong> 'move' will also be fired when Component is removed from any of\nthe child containers or their children or ...</p>\n"},"moveAfter":{"!type":"fn(item: ?, after: ?) -> !this","!doc":"<p>Moves the given <code>item</code> into this container following <code>after</code>. This method will\naccount for layout-generated components like splitters and should be used instead\nof index based <code><a href=\"#!/api/Ext.container.Container-event-move\" rel=\"Ext.container.Container-event-move\" class=\"docClass\">move</a></code>. If <code>after</code> is <code>null</code> then the <code>item</code> will be the\nfirst item in this container.</p>\n"},"moveBefore":{"!type":"fn(item: ?, before: ?) -> !this","!doc":"<p>Moves the given <code>item</code> into this container in front of <code>before</code>. This method will\naccount for layout-generated components like splitters and should be used instead\nof index based <code><a href=\"#!/api/Ext.container.Container-event-move\" rel=\"Ext.container.Container-event-move\" class=\"docClass\">move</a></code>. If <code>before</code> is <code>null</code> then the <code>item</code> will be the\nlast item in this container.</p>\n"},"nextChild":{"!type":"fn(child: ?, selector?: ?) -> ?","!doc":"<p>A method to find a child component after the passed child parameter. If a selector is also provided,\nthe first child component matching the selector will be returned.</p>\n"},"onAdd":{"!type":"fn(component: ?, position: ?) -> !this","!doc":"<p>This method is invoked after a new Component has been added. It\nis passed the Component which has been added. This method may\nbe used to update any internal structure which may depend upon\nthe state of the child items.</p>\n"},"onAdded":{"!type":"fn(container: ?, pos: ?, instanced: ?) -> !this","!doc":"<p>Method to manage awareness of when components are added to their\nrespective Container, firing an <a href=\"#!/api/Ext.container.Container-event-added\" rel=\"Ext.container.Container-event-added\" class=\"docClass\">added</a> event. References are\nestablished at add time rather than at render time.</p>\n\n<p>Allows addition of behavior when a Component is added to a\nContainer. At this stage, the Component is in the parent\nContainer's collection of child items. After calling the\nsuperclass's <code>onAdded</code>, the <code>ownerCt</code> reference will be present,\nand if configured with a ref, the <code>refOwner</code> will be set.</p>\n"},"onBeforeAdd":{"!type":"fn(item: ?) -> !this","!doc":"<p>This method is invoked before adding a new child Component. It\nis passed the new Component, and may be used to modify the\nComponent, or prepare the Container in some way. Returning\nfalse aborts the add operation.</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onMove":{"!type":"fn() -> !this"},"onPosition":{"!type":"fn() -> !this","!doc":"<p>Called after the component is moved, this method is empty by default but can be implemented by any\nsubclass that needs to perform custom logic after a move occurs.</p>\n"},"onRemove":{"!type":"fn(component: ?, autoDestroy: ?) -> !this","!doc":"<p>This method is invoked after a new Component has been\nremoved. It is passed the Component which has been\nremoved. This method may be used to update any internal\nstructure which may depend upon the state of the child items.</p>\n"},"onResize":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the resize operation.</p>\n\n<p>Called when Ext.resizer.Resizer#drag event is fired.</p>\n"},"prevChild":{"!type":"fn(child: ?, selector?: ?) -> ?","!doc":"<p>A method to find a child component before the passed child parameter. If a selector is also provided,\nthe first child component matching the selector will be returned.</p>\n"},"remove":{"!type":"fn(this: ?, component: ?, eOpts: ?)","!doc":"<p>Fires after any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is removed from the container.</p>\n\n<p><strong>This event bubbles:</strong> 'remove' will also be fired when Component is removed from any of\nthe child containers or their children or ...</p>\n"},"removeAll":{"!type":"fn(autoDestroy?: ?) -> ?","!doc":"<p>Removes all components from this container.</p>\n"},"setActiveItem":{"!type":"fn(item: ?) -> ?","!doc":"<p>Sets a component as the active layout item. This only applies when using\na <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> layout.</p>\n\n<pre><code>var card1 = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {itemId: 'card-1'});\nvar card2 = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {itemId: 'card-2'});\nvar panel = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    layout: 'card',\n    items: [card1, card2]\n});\n// These are all equivalent\npanel.getLayout().setActiveItem(card2);\npanel.getLayout().setActiveItem('card-2');\npanel.getLayout().setActiveItem(1);\n</code></pre>\n"},"setLayout":{"!type":"fn(configuration: ?) -> !this","!doc":"<p>Reconfigures the initially configured <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a>.</p>\n\n<p>NOTE: this method cannot be used to change the \"type\" of layout after the component\nhas been rendered to the DOM. After rendering, this method can only modify the\nexisting layout's configuration properties. The reason for this restriction is that\nmany container layouts insert special wrapping elements into the dom, and the\nframework does not currently support dynamically changing these elements once\nrendered.</p>\n"},"afterlayout":{"!type":"fn(this: ?, layout: ?, eOpts: ?)","!doc":"<p>Fires when the components in this container are arranged by the associated layout manager.</p>\n"},"beforeadd":{"!type":"fn(this: ?, component: ?, index: ?, eOpts: ?)","!doc":"<p>Fires before any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is added or inserted into the container.\nA handler can return false to cancel the add.</p>\n"},"beforeremove":{"!type":"fn(this: ?, component: ?, eOpts: ?)","!doc":"<p>Fires before any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is removed from the container.  A handler can return\nfalse to cancel the remove.</p>\n"}},"ButtonGroup":{"!doc":"<p>Provides a container for arranging a group of related Buttons in a tabular manner.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Panel with ButtonGroup',\n    width: 300,\n    height:200,\n    renderTo: document.body,\n    bodyPadding: 10,\n    html: 'HTML Panel Content',\n    tbar: [{\n        xtype: 'buttongroup',\n        columns: 3,\n        title: 'Clipboard',\n        items: [{\n            text: 'Paste',\n            scale: 'large',\n            rowspan: 3,\n            iconCls: 'add',\n            iconAlign: 'top',\n            cls: 'btn-as-arrow'\n        },{\n            xtype:'splitbutton',\n            text: 'Menu Button',\n            scale: 'large',\n            rowspan: 3,\n            iconCls: 'add',\n            iconAlign: 'top',\n            arrowAlign:'bottom',\n            menu: [{ text: 'Menu Item 1' }]\n        },{\n            xtype:'splitbutton', text: 'Cut', iconCls: 'add16', menu: [{text: 'Cut Menu Item'}]\n        },{\n            text: 'Copy', iconCls: 'add16'\n        },{\n            text: 'Format', iconCls: 'add16'\n        }]\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"animCollapse":{"!type":"bool"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"closable":{"!type":"bool"},"collapseMode":{"!type":"bool"},"collapsible":{"!type":"bool"},"columns":{"!type":"number","!doc":"<p>The <code>columns</code> configuration property passed to the <a href=\"#!/api/Ext.container.ButtonGroup-cfg-layout\" rel=\"Ext.container.ButtonGroup-cfg-layout\" class=\"docClass\">configured layout manager</a>.\nSee <a href=\"#!/api/Ext.layout.container.Table-cfg-columns\" rel=\"Ext.layout.container.Table-cfg-columns\" class=\"docClass\">Ext.layout.container.Table.columns</a>.</p>\n"},"defaultButtonUI":{"!type":"string","!doc":"<p>A default <a href=\"#!/api/Ext.Component-cfg-ui\" rel=\"Ext.Component-cfg-ui\" class=\"docClass\">ui</a> to use for <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Button</a> items</p>\n"},"defaultType":{"!type":"string","!doc":"<p>The default <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">xtype</a> of child Components to create in this Container when\na child item is specified as a raw configuration object, rather than as an instantiated Component.</p>\n"},"frame":{"!type":"bool","!doc":"<p>True to apply a frame to the panel.</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.container.ButtonGroup-cfg-items\" rel=\"Ext.container.ButtonGroup-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"tools":{},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"noTitleCls":{"!type":"string"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"titleAlign":{"!type":"string"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"onBeforeAdd":{"!type":"fn(component: ?) -> !this","!doc":"<p>private</p>\n"}}},"DockingContainer":{"prototype":{"defaultDockWeights":{"!doc":"<p>This object holds the default weights applied to dockedItems that have no weight. These start with a\nweight of 1, to allow negative weights to insert before top items and are odd numbers\nso that even weights can be used to get between different dock orders.</p>\n\n<p>To make default docking order match border layout, do this:</p>\n\n<pre><code> Ext.panel.Panel.prototype.defaultDockWeights = { top: 1, bottom: 3, left: 5, right: 7 };\n</code></pre>\n\n<p>Changing these defaults as above or individually on this object will effect all Panels.\nTo change the defaults on a single panel, you should replace the entire object:</p>\n\n<pre><code> initComponent: function () {\n     // NOTE: Don't change members of defaultDockWeights since the object is shared.\n     this.defaultDockWeights = { top: 1, bottom: 3, left: 5, right: 7 };\n\n     this.callParent();\n }\n</code></pre>\n\n<p>To change only one of the default values, you do this:</p>\n\n<pre><code> initComponent: function () {\n     // NOTE: Don't change members of defaultDockWeights since the object is shared.\n     this.defaultDockWeights = Ext.applyIf({ top: 10 }, this.defaultDockWeights);\n\n     this.callParent();\n }\n</code></pre>\n"},"dockOrder":{"!doc":"<p>Values to decide which side of the body element docked items must go\nThis overides any weight. A left/top will <em>always</em> sort before a right/bottom\nregardless of any weight value. Weights sort at either side of the \"body\" dividing point.</p>\n"},"horizontalDocks":{"!type":"number","!doc":"<p>Number of dock 'left' and 'right' items.</p>\n"},"isDockingContainer":{"!type":"bool","!doc":"<p>End Definitions</p>\n"},"addDocked":{"!type":"fn(items: ?, pos?: ?) -> ?","!doc":"<p>Adds docked item(s) to the container.</p>\n"},"destroyDockedItems":{"!type":"fn() -> !this"},"doRenderDockedItems":{"!type":"fn(out: ?, renderData: ?, after: ?) -> !this"},"getDockedComponent":{"!type":"fn(comp: ?) -> ?","!doc":"<p>Finds a docked component by id, itemId or position. Also see <a href=\"#!/api/Ext.container.DockingContainer-method-getDockedItems\" rel=\"Ext.container.DockingContainer-method-getDockedItems\" class=\"docClass\">getDockedItems</a></p>\n"},"getDockedItems":{"!type":"fn(selector: ?, beforeBody: ?) -> ?","!doc":"<p>Retrieves an array of all currently docked Components.</p>\n\n<p>For example to find a toolbar that has been docked at top:</p>\n\n<pre><code>panel.getDockedItems('toolbar[dock=\"top\"]');\n</code></pre>\n"},"getDockingRefItems":{"!type":"fn(deep: ?, containerItems: ?) -> !this"},"initDockingItems":{"!type":"fn() -> !this"},"insertDocked":{"!type":"fn(pos: ?, items: ?) -> !this","!doc":"<p>Inserts docked item(s) to the panel at the indicated position.</p>\n"},"moveDocked":{"!type":"fn(item: ?, side: ?) -> !this","!doc":"<p>Moves a docked item to a different side.</p>\n"},"onDockedAdd":{"!type":"fn(component: ?) -> !this","!doc":"<p>Invoked after a docked item is added to the Panel.</p>\n"},"onDockedRemove":{"!type":"fn(component: ?) -> !this","!doc":"<p>Invoked after a docked item is removed from the Panel.</p>\n"},"removeDocked":{"!type":"fn(item: ?, autoDestroy?: ?) -> !this","!doc":"<p>Removes the docked item from the panel.</p>\n"},"setupDockingRenderTpl":{"!type":"fn(renderTpl: ?) -> !this"},"dockedadd":{"!type":"fn(this: ?, component: ?, index: ?, eOpts: ?)","!doc":"<p>Fires when any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is added or inserted as a docked item.</p>\n"},"dockedremove":{"!type":"fn(this: ?, component: ?, eOpts: ?)","!doc":"<p>Fires when any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is removed from the docked items.</p>\n"}},"defaultDockWeights":{"!doc":"<p>This object holds the default weights applied to dockedItems that have no weight. These start with a\nweight of 1, to allow negative weights to insert before top items and are odd numbers\nso that even weights can be used to get between different dock orders.</p>\n\n<p>To make default docking order match border layout, do this:</p>\n\n<pre><code> Ext.panel.Panel.prototype.defaultDockWeights = { top: 1, bottom: 3, left: 5, right: 7 };\n</code></pre>\n\n<p>Changing these defaults as above or individually on this object will effect all Panels.\nTo change the defaults on a single panel, you should replace the entire object:</p>\n\n<pre><code> initComponent: function () {\n     // NOTE: Don't change members of defaultDockWeights since the object is shared.\n     this.defaultDockWeights = { top: 1, bottom: 3, left: 5, right: 7 };\n\n     this.callParent();\n }\n</code></pre>\n\n<p>To change only one of the default values, you do this:</p>\n\n<pre><code> initComponent: function () {\n     // NOTE: Don't change members of defaultDockWeights since the object is shared.\n     this.defaultDockWeights = Ext.applyIf({ top: 10 }, this.defaultDockWeights);\n\n     this.callParent();\n }\n</code></pre>\n"},"dockOrder":{"!doc":"<p>Values to decide which side of the body element docked items must go\nThis overides any weight. A left/top will <em>always</em> sort before a right/bottom\nregardless of any weight value. Weights sort at either side of the \"body\" dividing point.</p>\n"},"horizontalDocks":{"!type":"number","!doc":"<p>Number of dock 'left' and 'right' items.</p>\n"},"isDockingContainer":{"!type":"bool","!doc":"<p>End Definitions</p>\n"},"addDocked":{"!type":"fn(items: ?, pos?: ?) -> ?","!doc":"<p>Adds docked item(s) to the container.</p>\n"},"destroyDockedItems":{"!type":"fn() -> !this"},"doRenderDockedItems":{"!type":"fn(out: ?, renderData: ?, after: ?) -> !this"},"getDockedComponent":{"!type":"fn(comp: ?) -> ?","!doc":"<p>Finds a docked component by id, itemId or position. Also see <a href=\"#!/api/Ext.container.DockingContainer-method-getDockedItems\" rel=\"Ext.container.DockingContainer-method-getDockedItems\" class=\"docClass\">getDockedItems</a></p>\n"},"getDockedItems":{"!type":"fn(selector: ?, beforeBody: ?) -> ?","!doc":"<p>Retrieves an array of all currently docked Components.</p>\n\n<p>For example to find a toolbar that has been docked at top:</p>\n\n<pre><code>panel.getDockedItems('toolbar[dock=\"top\"]');\n</code></pre>\n"},"getDockingRefItems":{"!type":"fn(deep: ?, containerItems: ?) -> !this"},"initDockingItems":{"!type":"fn() -> !this"},"insertDocked":{"!type":"fn(pos: ?, items: ?) -> !this","!doc":"<p>Inserts docked item(s) to the panel at the indicated position.</p>\n"},"moveDocked":{"!type":"fn(item: ?, side: ?) -> !this","!doc":"<p>Moves a docked item to a different side.</p>\n"},"onDockedAdd":{"!type":"fn(component: ?) -> !this","!doc":"<p>Invoked after a docked item is added to the Panel.</p>\n"},"onDockedRemove":{"!type":"fn(component: ?) -> !this","!doc":"<p>Invoked after a docked item is removed from the Panel.</p>\n"},"removeDocked":{"!type":"fn(item: ?, autoDestroy?: ?) -> !this","!doc":"<p>Removes the docked item from the panel.</p>\n"},"setupDockingRenderTpl":{"!type":"fn(renderTpl: ?) -> !this"},"dockedadd":{"!type":"fn(this: ?, component: ?, index: ?, eOpts: ?)","!doc":"<p>Fires when any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is added or inserted as a docked item.</p>\n"},"dockedremove":{"!type":"fn(this: ?, component: ?, eOpts: ?)","!doc":"<p>Fires when any <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> is removed from the docked items.</p>\n"}},"Monitor":{"!doc":"<p>This is a utility class for being able to track all items of a particular type\ninside any level at a container. This can be used in favour of bubbling add/remove events\nwhich can add a large perf cost when implemented globally</p>\n","!type":"fn(config: ?)","prototype":{"addHandler":{},"disabled":{"!type":"number"},"invalidateHandler":{},"removeHandler":{},"scope":{},"selector":{"!type":"string"},"target":{},"bind":{"!type":"fn(target: ?) -> !this"},"disable":{"!type":"fn() -> !this"},"enable":{"!type":"fn() -> !this"},"getItems":{"!type":"fn() -> !this"},"handleAdd":{"!type":"fn(ct: ?, comp: ?) -> !this"},"handleRemove":{"!type":"fn(ct: ?, comp: ?) -> !this"},"invalidateItems":{"!type":"fn(triggerHandler: ?) -> !this"},"onContainerAdd":{"!type":"fn(ct: ?, preventChildren: ?) -> !this"},"onContainerRemove":{"!type":"fn(ct: ?, comp: ?) -> !this"},"onItemAdd":{"!type":"fn(ct: ?, comp: ?) -> !this"},"onItemRemove":{"!type":"fn(ct: ?, comp: ?) -> !this"},"removeCtListeners":{"!type":"fn(comp: ?) -> !this"},"unbind":{"!type":"fn() -> !this"}}},"Viewport":{"!doc":"<p>A specialized container representing the viewable application area (the browser viewport).</p>\n\n<p>The Viewport renders itself to the document body, and automatically sizes itself to the size of\nthe browser viewport and manages window resizing. There may only be one Viewport created\nin a page.</p>\n\n<p>Like any <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a>, a Viewport will only perform sizing and positioning\non its child Components if you configure it with a <a href=\"#!/api/Ext.container.Viewport-cfg-layout\" rel=\"Ext.container.Viewport-cfg-layout\" class=\"docClass\">layout</a>.</p>\n\n<p>A Common layout used with Viewports is <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>, but if the\nrequired layout is simpler, a different layout should be chosen.</p>\n\n<p>For example, to simply make a single child item occupy all available space, use\n<a href=\"#!/api/Ext.layout.container.Fit\" rel=\"Ext.layout.container.Fit\" class=\"docClass\">fit layout</a>.</p>\n\n<p>To display one \"active\" item at full size from a choice of several child items, use\n<a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">card layout</a>.</p>\n\n<p>Inner layouts are available because all <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Panel</a>s\nadded to the Viewport, either through its <a href=\"#!/api/Ext.container.Viewport-cfg-items\" rel=\"Ext.container.Viewport-cfg-items\" class=\"docClass\">items</a>, or the <a href=\"#!/api/Ext.container.Viewport-method-add\" rel=\"Ext.container.Viewport-method-add\" class=\"docClass\">add</a>\nmethod of any of its child Panels may themselves have a layout.</p>\n\n<p>The Viewport does not provide scrolling, so child Panels within the Viewport should provide\nfor scrolling if needed using the <a href=\"#!/api/Ext.container.Viewport-cfg-autoScroll\" rel=\"Ext.container.Viewport-cfg-autoScroll\" class=\"docClass\">autoScroll</a> config.</p>\n\n<p>An example showing a classic application border layout:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.container.Viewport\" rel=\"Ext.container.Viewport\" class=\"docClass\">Ext.container.Viewport</a>', {\n    layout: 'border',\n    items: [{\n        region: 'north',\n        html: '&lt;h1 class=\"x-panel-header\"&gt;Page Title&lt;/h1&gt;',\n        border: false,\n        margin: '0 0 5 0'\n    }, {\n        region: 'west',\n        collapsible: true,\n        title: 'Navigation',\n        width: 150\n        // could use a TreePanel or AccordionLayout for navigational items\n    }, {\n        region: 'south',\n        title: 'South Panel',\n        collapsible: true,\n        html: 'Information goes here',\n        split: true,\n        height: 100,\n        minHeight: 100\n    }, {\n        region: 'east',\n        title: 'East Panel',\n        collapsible: true,\n        split: true,\n        width: 150\n    }, {\n        region: 'center',\n        xtype: 'tabpanel', // TabPanel itself has no title\n        activeTab: 0,      // First tab active by default\n        items: {\n            title: 'Default Tab',\n            html: 'The first tab\\'s content. Others may be added dynamically'\n        }\n    }]\n});\n</code></pre>\n\n<p>Alternatively you can turn any normal Container (or Component) into a Viewport using\nthe <code><a href=\"#!/api/Ext.plugin.Viewport\" rel=\"Ext.plugin.Viewport\" class=\"docClass\">viewport plugin</a></code>.</p>\n","!type":"fn(config: ?)","prototype":{"allowDomMove":{"!type":"bool"},"height":{"!type":"number","!doc":"<p>Sets itself to viewport width.</p>\n"},"maxUserScale":{"!type":"number","!doc":"<p>The maximum zoom scale. Only applicable for touch devices. Set this to 1 to\ndisable zooming.</p>\n"},"renderTo":{"!doc":"<p>Always renders to document body.</p>\n"},"width":{"!type":"number","!doc":"<p>Sets itself to viewport height.</p>\n"},"isViewport":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Viewport, or subclass thereof.</p>\n"}}}},"ComponentLoader":{"!doc":"<p>This class is used to load content via Ajax into a <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>. In general\nthis class will not be instanced directly, rather a loader configuration will be passed to the\nconstructor of the <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.</p>\n\n<h2>HTML Renderer</h2>\n\n<p>By default, the content loaded will be processed as raw html. The response text\nfrom the request is taken and added to the component. This can be used in\nconjunction with the <a href=\"#!/api/Ext.ComponentLoader-cfg-scripts\" rel=\"Ext.ComponentLoader-cfg-scripts\" class=\"docClass\">scripts</a> option to execute any inline scripts in\nthe resulting content. Using this renderer has the same effect as passing the\n<a href=\"#!/api/Ext.Component-cfg-html\" rel=\"Ext.Component-cfg-html\" class=\"docClass\">Ext.Component.html</a> configuration option.</p>\n\n<h2>Data Renderer</h2>\n\n<p>This renderer allows content to be added by using JSON data and a Ext.XTemplate.\nThe content received from the response is passed to the <a href=\"#!/api/Ext.Component-method-update\" rel=\"Ext.Component-method-update\" class=\"docClass\">Ext.Component.update</a> method.\nThis content is run through the attached <a href=\"#!/api/Ext.Component-cfg-tpl\" rel=\"Ext.Component-cfg-tpl\" class=\"docClass\">Ext.Component.tpl</a> and the data is added to\nthe Component. Using this renderer has the same effect as using the <a href=\"#!/api/Ext.Component-cfg-data\" rel=\"Ext.Component-cfg-data\" class=\"docClass\">Ext.Component.data</a>\nconfiguration in conjunction with a <a href=\"#!/api/Ext.Component-cfg-tpl\" rel=\"Ext.Component-cfg-tpl\" class=\"docClass\">Ext.Component.tpl</a>.</p>\n\n<h2>Component Renderer</h2>\n\n<p>This renderer can only be used with a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a> and subclasses. It allows for\nComponents to be loaded remotely into a Container. The response is expected to be a single/series of\n<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> configuration objects. When the response is received, the data is decoded\nand then passed to <a href=\"#!/api/Ext.container.Container-method-add\" rel=\"Ext.container.Container-method-add\" class=\"docClass\">Ext.container.Container.add</a>. Using this renderer has the same effect as specifying\nthe <a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">Ext.container.Container.items</a> configuration on a Container.</p>\n\n<h2>Custom Renderer</h2>\n\n<p>A custom function can be passed to handle any other special case, see the <a href=\"#!/api/Ext.ComponentLoader-cfg-renderer\" rel=\"Ext.ComponentLoader-cfg-renderer\" class=\"docClass\">renderer</a> option.</p>\n\n<h2>Example Usage</h2>\n\n<pre><code>var cmp = Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    tpl: '{firstName} - {lastName}',\n    loader: {\n        url: 'myPage.php',\n        renderer: 'data',\n        params: {\n            userId: 1\n        }\n    }\n});\n\n// call the loader manually (or use autoLoad:true instead)\ncmp.getLoader().load();\n</code></pre>\n","prototype":{"loadMask":{"!doc":"<p>True or a <a href=\"#!/api/Ext.LoadMask\" rel=\"Ext.LoadMask\" class=\"docClass\">Ext.LoadMask</a> configuration to enable masking during loading.</p>\n"},"renderer":{"!doc":"<p>The type of content that is to be loaded into, which can be one of 3 types:</p>\n\n<ul>\n<li><strong>html</strong> : Loads raw html content, see <a href=\"#!/api/Ext.Component-cfg-html\" rel=\"Ext.Component-cfg-html\" class=\"docClass\">Ext.Component.html</a></li>\n<li><strong>data</strong> : Loads raw html content, see <a href=\"#!/api/Ext.Component-cfg-data\" rel=\"Ext.Component-cfg-data\" class=\"docClass\">Ext.Component.data</a></li>\n<li><strong>component</strong> : Loads child {<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>} instances. This option is only valid when used with a Container.</li>\n</ul>\n\n\n<p>Alternatively, you can pass a function which is called with the following parameters.</p>\n\n<ul>\n<li>loader - Loader instance</li>\n<li>response - The server response</li>\n<li>active - The active request</li>\n</ul>\n\n\n<p>The function must return false is loading is not successful. Below is a sample of using a custom renderer:</p>\n\n<pre><code>new <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>({\n    loader: {\n        url: 'myPage.php',\n        renderer: function(loader, response, active) {\n            var text = response.responseText;\n            loader.getTarget().setHtml('The response is ' + text);\n            return true;\n        }\n    }\n});\n</code></pre>\n"},"scripts":{"!type":"bool","!doc":"<p>True to parse any inline script tags in the response. This only used when using the html\n<a href=\"#!/api/Ext.ComponentLoader-cfg-renderer\" rel=\"Ext.ComponentLoader-cfg-renderer\" class=\"docClass\">renderer</a>.</p>\n"},"target":{"!doc":"<p>The target <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> for the loader.\nIf a string is passed it will be looked up via the id.</p>\n"},"Renderer":{},"addMask":{"!type":"fn(mask: ?) -> !this","!doc":"<p>Add the mask on the target</p>\n"},"getRenderer":{"!type":"fn(renderer: ?) -> ?","!doc":"<p>Gets the renderer to use</p>\n"},"removeMask":{"!type":"fn() -> !this","!doc":"<p>inherit docs</p>\n"},"setOptions":{"!type":"fn(active: ?, options: ?) -> !this","!doc":"<p>Sets any additional options on the active request</p>\n"},"setTarget":{"!type":"fn(target: ?) -> !this","!doc":"<p>Set a {<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>} as the target of this loader. Note that if the target is changed,\nany active requests will be aborted.</p>\n"}}},"ElementLoader":{"prototype":{"ajaxOptions":{"!doc":"<p>Any additional options to be passed to the request, for example timeout or headers.</p>\n"},"autoLoad":{"!doc":"<p>True to have the loader make a request as soon as it is created.\nThis argument can also be a set of options that will be passed to <a href=\"#!/api/Ext.ElementLoader-method-load\" rel=\"Ext.ElementLoader-method-load\" class=\"docClass\">load</a> is called.</p>\n"},"baseParams":{"!doc":"<p>Params that will be attached to every request. These parameters\nwill not be overridden by any params in the load options.</p>\n"},"callback":{"!doc":"<p>A function to be called when a load request finishes.\nWill be called with the following config parameters:</p>\n\n<ul>\n<li>this - The ElementLoader instance.</li>\n<li>success - True if successful request.</li>\n<li>response - The response object.</li>\n<li>options - Ajax options.</li>\n</ul>\n\n"},"failure":{"!doc":"<p>A function to be called when a load request fails.\nWill be called with the following config parameters:</p>\n\n<ul>\n<li>this - The ElementLoader instance.</li>\n<li>response - The response object.</li>\n<li>options - Ajax options.</li>\n</ul>\n\n"},"params":{"!doc":"<p>Any params to be attached to the Ajax request. These parameters will\nbe overridden by any params in the load options.</p>\n"},"rendererScope":{"!doc":"<p>The scope to execute the <a href=\"#!/api/Ext.ElementLoader-cfg-renderer\" rel=\"Ext.ElementLoader-cfg-renderer\" class=\"docClass\">renderer</a> function in.</p>\n"},"scope":{"!doc":"<p>The scope to execute the <a href=\"#!/api/Ext.ElementLoader-cfg-success\" rel=\"Ext.ElementLoader-cfg-success\" class=\"docClass\">success</a> and <a href=\"#!/api/Ext.ElementLoader-cfg-failure\" rel=\"Ext.ElementLoader-cfg-failure\" class=\"docClass\">failure</a> functions in.</p>\n"},"success":{"!doc":"<p>A function to be called when a load request is successful.\nWill be called with the following config parameters:</p>\n\n<ul>\n<li>this - The ElementLoader instance.</li>\n<li>response - The response object.</li>\n<li>options - Ajax options.</li>\n</ul>\n\n"},"url":{"!type":"string","!doc":"<p>The url to retrieve the content from.</p>\n"},"isLoader":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated ElementLoader, or subclass thereof.</p>\n"},"abort":{"!type":"fn() -> !this","!doc":"<p>Aborts the active load request</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroys the loader. Any active requests will be aborted.</p>\n"},"getTarget":{"!type":"fn() -> ?","!doc":"<p>Returns the target of this loader.</p>\n"},"isAutoRefreshing":{"!type":"fn() -> bool","!doc":"<p>Checks whether the loader is automatically refreshing. See <a href=\"#!/api/Ext.ElementLoader-method-startAutoRefresh\" rel=\"Ext.ElementLoader-method-startAutoRefresh\" class=\"docClass\">startAutoRefresh</a>.</p>\n"},"load":{"!type":"fn(this: ?, response: ?, options: ?, eOpts: ?)","!doc":"<p>Fires after a successful load.</p>\n"},"onComplete":{"!type":"fn(options: ?, success: ?, response: ?) -> !this","!doc":"<p>Parses the response after the request completes</p>\n"},"startAutoRefresh":{"!type":"fn(interval: ?, options?: ?) -> !this","!doc":"<p>Automatically refreshes the content over a specified period.</p>\n"},"stopAutoRefresh":{"!type":"fn() -> !this","!doc":"<p>Clears any auto refresh. See <a href=\"#!/api/Ext.ElementLoader-method-startAutoRefresh\" rel=\"Ext.ElementLoader-method-startAutoRefresh\" class=\"docClass\">startAutoRefresh</a>.</p>\n"},"beforeload":{"!type":"fn(this: ?, options: ?, eOpts: ?)","!doc":"<p>Fires before a load request is made to the server.\nReturning false from an event listener can prevent the load\nfrom occurring.</p>\n"},"exception":{"!type":"fn(this: ?, response: ?, options: ?, eOpts: ?)","!doc":"<p>Fires after an unsuccessful load.</p>\n"},"loadMask":{"!doc":"<p>True or a string to show when the element is loading.</p>\n"},"renderer":{"!doc":"<p>A custom function to render the content to the element. The function should\nreturn false if the renderer could not be applied. The passed parameters are:</p>\n\n<ul>\n<li>The loader</li>\n<li>The response</li>\n<li>The active request</li>\n</ul>\n\n"},"scripts":{"!type":"bool","!doc":"<p>True to parse any inline script tags in the response.</p>\n"},"target":{"!doc":"<p>The target element for the loader. It can be the DOM element, the id or an Ext.dom.Element.</p>\n"},"Renderer":{},"addMask":{"!type":"fn(mask: ?) -> !this","!doc":"<p>Adds the mask on the target</p>\n"},"getRenderer":{"!type":"fn(renderer: ?) -> ?","!doc":"<p>Gets the renderer to use</p>\n"},"removeMask":{"!type":"fn() -> !this","!doc":"<p>Removes the mask on the target</p>\n"},"setOptions":{"!type":"fn(active: ?, options: ?) -> !this","!doc":"<p>Sets any additional options on the active request</p>\n"},"setTarget":{"!type":"fn(target: ?) -> !this","!doc":"<p>Sets an Ext.dom.Element as the target of this loader.\nNote that if the target is changed, any active requests will be aborted.</p>\n"}},"!doc":"<p>A class used to load remote content to an Element. Sample usage:</p>\n\n<pre><code>Ext.get('el').load({\n    url: 'myPage.php',\n    scripts: true,\n    params: {\n        id: 1\n    }\n});\n</code></pre>\n\n<p>In general this class will not be instanced directly, rather the Ext.dom.Element.load method\nwill be used.</p>\n"},"panel":{"Panel":{"prototype":{"bbar":{"!doc":"<p>Convenience config. Short for 'Bottom Bar'.</p>\n\n<pre><code>bbar: [\n  { xtype: 'button', text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'bottom',\n    items: [\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n"},"bodyBorder":{"!type":"bool","!doc":"<p>A shortcut to add or remove the border on the body of a panel. In the classic theme\nthis only applies to a panel which has the <a href=\"#!/api/Ext.panel.Panel-cfg-frame\" rel=\"Ext.panel.Panel-cfg-frame\" class=\"docClass\">frame</a> configuration set to <code>true</code>.</p>\n"},"bodyCls":{"!doc":"<p>A CSS class, space-delimited string of classes, or array of classes to be applied to the panel's body element.\nThe following examples are all valid:</p>\n\n<pre><code>bodyCls: 'foo'\nbodyCls: 'foo bar'\nbodyCls: ['foo', 'bar']\n</code></pre>\n"},"bodyPadding":{"!doc":"<p>A shortcut for setting a padding style on the body element. The value can either be\na number to be applied to all sides, or a normal css string describing padding.</p>\n"},"bodyStyle":{"!doc":"<p>Custom CSS styles to be applied to the panel's body element, which can be supplied as a valid CSS style string,\nan object containing style property name/value pairs or a function that returns such a string or object.\nFor example, these two formats are interpreted to be equivalent:</p>\n\n<pre><code>bodyStyle: 'background:#ffc; padding:10px;'\n\nbodyStyle: {\n    background: '#ffc',\n    padding: '10px'\n}\n</code></pre>\n"},"border":{"!type":"bool","!doc":"<p>Specify as <code>false</code> to render the Panel with zero width borders.</p>\n\n<p>Leaving the value as <code>true</code> uses the selected theme's $panel-border-width</p>\n\n<p>Defaults to <code>false</code> when using or extending Neptune.</p>\n"},"buttonAlign":{"!type":"string","!doc":"<p>The alignment of any buttons added to this panel. Valid values are 'right', 'left' and 'center' (defaults to\n'right' for buttons/fbar, 'left' for other toolbar types).</p>\n\n<p><strong>NOTE:</strong> The prefered way to specify toolbars is to use the dockedItems config. Instead of buttonAlign you\nwould add the layout: { pack: 'start' | 'center' | 'end' } option to the dockedItem config.</p>\n"},"buttons":{"!doc":"<p>Convenience config used for adding buttons docked to the bottom of the panel. This is a\nsynonym for the <a href=\"#!/api/Ext.panel.Panel-cfg-fbar\" rel=\"Ext.panel.Panel-cfg-fbar\" class=\"docClass\">fbar</a> config.</p>\n\n<pre><code>buttons: [\n  { text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'bottom',\n    ui: 'footer',\n    defaults: {minWidth: <a href=\"#!/api/Ext.panel.Panel-cfg-minButtonWidth\" rel=\"Ext.panel.Panel-cfg-minButtonWidth\" class=\"docClass\">minButtonWidth</a>},\n    items: [\n        { xtype: 'component', flex: 1 },\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n\n<p>The <a href=\"#!/api/Ext.panel.Panel-cfg-minButtonWidth\" rel=\"Ext.panel.Panel-cfg-minButtonWidth\" class=\"docClass\">minButtonWidth</a> is used as the default <a href=\"#!/api/Ext.button.Button-cfg-minWidth\" rel=\"Ext.button.Button-cfg-minWidth\" class=\"docClass\">minWidth</a> for\neach of the buttons in the buttons toolbar.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"closeAction":{"!type":"string","!doc":"<p>The action to take when the close header tool is clicked:</p>\n\n<ul>\n<li><p><strong><code>'<a href=\"#!/api/Ext.panel.Panel-method-destroy\" rel=\"Ext.panel.Panel-method-destroy\" class=\"docClass\">destroy</a>'</code></strong> :</p>\n\n<p><a href=\"#!/api/Ext.panel.Panel-method-remove\" rel=\"Ext.panel.Panel-method-remove\" class=\"docClass\">remove</a> the window from the DOM and <a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a> it and all descendant\nComponents. The window will <strong>not</strong> be available to be redisplayed via the <a href=\"#!/api/Ext.panel.Panel-method-show\" rel=\"Ext.panel.Panel-method-show\" class=\"docClass\">show</a> method.</p></li>\n<li><p><strong><code>'<a href=\"#!/api/Ext.panel.Panel-method-hide\" rel=\"Ext.panel.Panel-method-hide\" class=\"docClass\">hide</a>'</code></strong> :</p>\n\n<p><a href=\"#!/api/Ext.panel.Panel-method-hide\" rel=\"Ext.panel.Panel-method-hide\" class=\"docClass\">hide</a> the window by setting visibility to hidden and applying negative offsets. The window will be\navailable to be redisplayed via the <a href=\"#!/api/Ext.panel.Panel-method-show\" rel=\"Ext.panel.Panel-method-show\" class=\"docClass\">show</a> method.</p></li>\n</ul>\n\n\n<p><strong>Note:</strong> This behavior has changed! setting <em>does</em> affect the <a href=\"#!/api/Ext.panel.Panel-method-close\" rel=\"Ext.panel.Panel-method-close\" class=\"docClass\">close</a> method which will invoke the\napproriate closeAction.</p>\n"},"collapseDirection":{"!type":"string","!doc":"<p>The direction to collapse the Panel when the toggle button is clicked.</p>\n\n<p>Defaults to the <a href=\"#!/api/Ext.panel.Panel-property-headerPosition\" rel=\"Ext.panel.Panel-property-headerPosition\" class=\"docClass\">headerPosition</a></p>\n\n<p><strong>Important: This config is <em>ignored</em> for <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> Panels which are direct child items of a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>.</strong></p>\n\n<p>Specify as <code>'top'</code>, <code>'bottom'</code>, <code>'left'</code> or <code>'right'</code>.</p>\n"},"collapseFirst":{"!type":"bool","!doc":"<p><code>true</code> to make sure the collapse/expand toggle button always renders first (to the left of) any other tools in\nthe panel's title bar, <code>false</code> to render it last.</p>\n"},"collapsed":{"!type":"bool","!doc":"<p><code>true</code> to render the panel collapsed, <code>false</code> to render it expanded.</p>\n"},"collapsedCls":{"!type":"string","!doc":"<p>A CSS class to add to the panel's element after it has been collapsed.</p>\n"},"componentLayout":{"!doc":"<p>The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout\nmanager which sizes a Component's internal structure in response to the Component being sized.</p>\n\n<p>Generally, developers will not use this configuration as all provided Components which need their internal\nelements sizing (Such as <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">input fields</a>) come with their own componentLayout managers.</p>\n\n<p>The <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used on instances of the base <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>\nclass which simply sizes the Component's encapsulating element to the height and width specified in the\n<a href=\"#!/api/Ext.panel.Panel-method-setSize\" rel=\"Ext.panel.Panel-method-setSize\" class=\"docClass\">setSize</a> method.</p>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"constrain":{"!type":"bool","!doc":"<p>True to constrain the panel within its containing element, false to allow it to fall outside of its containing\nelement. By default floating components such as Windows will be rendered to <code>document.body</code>. To render and constrain the window within\nanother element specify <a href=\"#!/api/Ext.panel.Panel-cfg-renderTo\" rel=\"Ext.panel.Panel-cfg-renderTo\" class=\"docClass\">renderTo</a>. Optionally the header only can be constrained\nusing <a href=\"#!/api/Ext.panel.Panel-cfg-constrainHeader\" rel=\"Ext.panel.Panel-cfg-constrainHeader\" class=\"docClass\">constrainHeader</a>.</p>\n"},"constrainHeader":{"!type":"bool","!doc":"<p>True to constrain the panel header within its containing element (allowing the panel body to fall outside of\nits containing element) or false to allow the header to fall outside its containing element.\nOptionally the entire panel can be constrained using <a href=\"#!/api/Ext.panel.Panel-cfg-constrain\" rel=\"Ext.panel.Panel-cfg-constrain\" class=\"docClass\">constrain</a>.</p>\n"},"dockedItems":{"!doc":"<p>A component or series of components to be added as docked items to this panel. The docked items can be docked to\neither the top, right, left or bottom of a panel. This is typically used for things like toolbars or tab bars:</p>\n\n<pre><code>var panel = new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({\n    dockedItems: [{\n        xtype: 'toolbar',\n        dock: 'top',\n        items: [{\n            text: 'Docked to the top'\n        }]\n    }]\n});\n</code></pre>\n"},"fbar":{"!doc":"<p>Convenience config used for adding items to the bottom of the panel. Short for Footer Bar.</p>\n\n<pre><code>fbar: [\n  { type: 'button', text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'bottom',\n    ui: 'footer',\n    defaults: {minWidth: <a href=\"#!/api/Ext.panel.Panel-cfg-minButtonWidth\" rel=\"Ext.panel.Panel-cfg-minButtonWidth\" class=\"docClass\">minButtonWidth</a>},\n    items: [\n        { xtype: 'component', flex: 1 },\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n\n<p>The <a href=\"#!/api/Ext.panel.Panel-cfg-minButtonWidth\" rel=\"Ext.panel.Panel-cfg-minButtonWidth\" class=\"docClass\">minButtonWidth</a> is used as the default <a href=\"#!/api/Ext.button.Button-cfg-minWidth\" rel=\"Ext.button.Button-cfg-minWidth\" class=\"docClass\">minWidth</a> for\neach of the buttons in the fbar.</p>\n"},"floatable":{"!type":"bool","!doc":"<p><strong>Important: This config is only effective for <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> Panels which are direct child items of a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>.</strong></p>\n\n<p>true to allow clicking a collapsed Panel's <a href=\"#!/api/Ext.panel.Panel-cfg-placeholder\" rel=\"Ext.panel.Panel-cfg-placeholder\" class=\"docClass\">placeholder</a> to display the Panel floated above the layout,\nfalse to force the user to fully expand a collapsed region by clicking the expand button to see it again.</p>\n"},"frameHeader":{"!type":"bool","!doc":"<p>True to apply a frame to the panel panels header (if 'frame' is true).</p>\n"},"glyph":{"!doc":"<p>A numeric unicode character code to use as the icon for the panel header. The\ndefault font-family for glyphs can be set globally using\n<a href=\"#!/api/Ext-method-setGlyphFontFamily\" rel=\"Ext-method-setGlyphFontFamily\" class=\"docClass\">Ext.setGlyphFontFamily()</a>. Alternatively, this\nconfig option accepts a string with the charCode and font-family separated by the\n<code>@</code> symbol. For example '65@My Font Family'.</p>\n"},"header":{"!doc":"<p>Pass as <code>false</code> to prevent a Header from being created and shown.</p>\n\n<p>Pass as a config object (optionally containing an <code>xtype</code>) to custom-configure this Panel's header.</p>\n\n<p>See <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Ext.panel.Header</a> for all the options that may be specified here.</p>\n\n<p>A <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">panel header</a> is a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a> which contains the Panel's <a href=\"#!/api/Ext.panel.Panel-cfg-title\" rel=\"Ext.panel.Panel-cfg-title\" class=\"docClass\">title</a> and <a href=\"#!/api/Ext.panel.Panel-cfg-tools\" rel=\"Ext.panel.Panel-cfg-tools\" class=\"docClass\">tools</a>.\nYou may also configure the Panel's <code>header</code> option with its own child items which go <em>before</em> the <a href=\"#!/api/Ext.panel.Panel-cfg-tools\" rel=\"Ext.panel.Panel-cfg-tools\" class=\"docClass\">tools</a></p>\n\n<p>By default the panel <a href=\"#!/api/Ext.panel.Panel-cfg-title\" rel=\"Ext.panel.Panel-cfg-title\" class=\"docClass\">title</a> is inserted after items configured in this config, but before any tools.\nTo insert the title at any point in the full array, specify the <a href=\"#!/api/Ext.panel.Header-property-titlePosition\" rel=\"Ext.panel.Header-property-titlePosition\" class=\"docClass\">titlePosition</a> config:</p>\n\n<p>   new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({\n       title: 'Test',\n       tools: [{\n           type: 'refresh\n       }, {\n           type: 'help'\n       }],\n       titlePosition: 2 // Title will come AFTER the two tools\n       ...\n   });</p>\n"},"headerOverCls":{"!type":"string","!doc":"<p>Optional CSS class to apply to the header element on mouseover</p>\n"},"headerPosition":{"!type":"string","!doc":"<p>For performance reasons we give the following configs their default values on\nthe class body.  This prevents the updaters from running on initialization in the\ndefault configuration scenario</p>\n"},"hideCollapseTool":{"!type":"bool","!doc":"<p><code>true</code> to hide the expand/collapse toggle button when <code><a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> == true</code>, <code>false</code> to display it.</p>\n"},"icon":{"!type":"string","!doc":"<p>Path to image for an icon.</p>\n\n<p>There are no default icons that come with Ext JS.</p>\n"},"iconAlign":{"!type":"string"},"iconCls":{"!type":"string","!doc":"<p>CSS class for an icon.</p>\n\n<p>There are no default icon classes that come with Ext JS.</p>\n"},"lbar":{"!doc":"<p>Convenience config. Short for 'Left Bar' (left-docked, vertical toolbar).</p>\n\n<pre><code>lbar: [\n  { xtype: 'button', text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'left',\n    items: [\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n"},"manageHeight":{"!type":"bool","!doc":"<p>When true, the dock component layout writes\nheight information to the panel's DOM elements based on its shrink wrap height\ncalculation. This ensures that the browser respects the calculated height.\nWhen false, the dock component layout will not write heights on the panel or its\nbody element. In some simple layout cases, not writing the heights to the DOM may\nbe desired because this allows the browser to respond to direct DOM manipulations\n(like animations).</p>\n"},"maskElement":{"!type":"string","!doc":"<p>The name of the element property in this Panel to mask when masked by a LoadMask.</p>\n\n<p>Defaults to <code>\"el\"</code> to indicate that any LoadMask should be rendered into this Panel's encapsulating element.</p>\n\n<p>This could be configured to be <code>\"body\"</code> so that only the body is masked and toolbars and the header are still mouse-accessible.</p>\n"},"minButtonWidth":{"!type":"number","!doc":"<p>Minimum width of all footer toolbar buttons in pixels. If set, this will be used as the default\nvalue for the <a href=\"#!/api/Ext.button.Button-cfg-minWidth\" rel=\"Ext.button.Button-cfg-minWidth\" class=\"docClass\">Ext.button.Button.minWidth</a> config of each Button added to the <strong>footer toolbar</strong> via the\n<a href=\"#!/api/Ext.panel.Panel-cfg-fbar\" rel=\"Ext.panel.Panel-cfg-fbar\" class=\"docClass\">fbar</a> or <a href=\"#!/api/Ext.panel.Panel-cfg-buttons\" rel=\"Ext.panel.Panel-cfg-buttons\" class=\"docClass\">buttons</a> configurations. It will be ignored for buttons that have a minWidth configured\nsome other way, e.g. in their own config object or via the <a href=\"#!/api/Ext.container.Container-cfg-defaults\" rel=\"Ext.container.Container-cfg-defaults\" class=\"docClass\">defaults</a> of\ntheir parent container.</p>\n"},"overlapHeader":{"!type":"bool","!doc":"<p>True to overlap the header in a panel over the framing of the panel itself. This is needed when frame:true (and\nis done automatically for you). Otherwise it is undefined. If you manually add rounded corners to a panel header\nwhich does not have frame:true, this will need to be set to true.</p>\n"},"placeholder":{"!doc":"<p><strong>Important: This config is only effective for <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> Panels which are direct child items of a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a> when not using the <code>'header'</code> <a href=\"#!/api/Ext.panel.Panel-cfg-collapseMode\" rel=\"Ext.panel.Panel-cfg-collapseMode\" class=\"docClass\">collapseMode</a>.</strong></p>\n\n<p><strong>Optional.</strong> A Component (or config object for a Component) to show in place of this Panel when this Panel is\ncollapsed by a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>. Defaults to a generated <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Header</a> containing a <a href=\"#!/api/Ext.panel.Tool\" rel=\"Ext.panel.Tool\" class=\"docClass\">Tool</a> to re-expand the Panel.</p>\n"},"placeholderCollapseHideMode":{"!type":"number","!doc":"<p>The mode for hiding collapsed panels when\nusing <a href=\"#!/api/Ext.panel.Panel-cfg-collapseMode\" rel=\"Ext.panel.Panel-cfg-collapseMode\" class=\"docClass\">collapseMode</a> \"placeholder\".</p>\n"},"preventHeader":{"!type":"bool"},"rbar":{"!doc":"<p>Convenience config. Short for 'Right Bar' (right-docked, vertical toolbar).</p>\n\n<pre><code>rbar: [\n  { xtype: 'button', text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'right',\n    items: [\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.panel.Panel-method-getEl\" rel=\"Ext.panel.Panel-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.panel.Panel-method-getEl\" rel=\"Ext.panel.Panel-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.panel.Panel-cfg-renderSelectors\" rel=\"Ext.panel.Panel-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.panel.Panel-cfg-childEls\" rel=\"Ext.panel.Panel-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"shrinkWrapDock":{"!doc":"<p>Allows for this panel to include the <a href=\"#!/api/Ext.panel.Panel-cfg-dockedItems\" rel=\"Ext.panel.Panel-cfg-dockedItems\" class=\"docClass\">dockedItems</a> when trying to determine the overall\nsize of the panel. This option is only applicable when this panel is also shrink wrapping in the\nsame dimensions. See <a href=\"#!/api/Ext.Component-cfg-shrinkWrap\" rel=\"Ext.Component-cfg-shrinkWrap\" class=\"docClass\">Ext.Component.shrinkWrap</a> for an explanation of the configuration options.</p>\n"},"simpleDrag":{"!type":"bool","!doc":"<p>When <a href=\"#!/api/Ext.panel.Panel-cfg-draggable\" rel=\"Ext.panel.Panel-cfg-draggable\" class=\"docClass\">draggable</a> is <code>true</code>, Specify this as <code>true</code> to  cause the <code>draggable</code> config\nto work the same as it does in <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Window</a>. This Panel\njust becomes movable. No DragDrop instances receive any notifications.\nFor example:</p>\n\n<pre class='inline-example '><code>var win = Ext.create('widget.window', {\n    height: 300,\n    width: 300,\n    title: 'Constraining Window',\n    closable: false,\n    items: {\n        title: \"Floating Panel\",\n        width: 100,\n        height: 100,\n        floating: true,\n        draggable: true,\n        constrain: true,\n        simpleDrag: true\n    }\n});\nwin.show();\n// Floating components begin life hidden\nwin.child('[title=Floating Panel]').show();\n</code></pre>\n"},"tbar":{"!doc":"<p>Convenience config. Short for 'Top Bar'.</p>\n\n<pre><code>tbar: [\n  { xtype: 'button', text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'top',\n    items: [\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n"},"title":{"!type":"string","!doc":"<p>The title text to be used to display in the <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Panel Header</a>.\nOr a config object for a <a href=\"#!/api/Ext.panel.Title\" rel=\"Ext.panel.Title\" class=\"docClass\">Panel Title</a>. When a <code>title</code> is\nspecified the <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Ext.panel.Header</a> will automatically be created and\ndisplayed unless <a href=\"#!/api/Ext.panel.Panel-cfg-header\" rel=\"Ext.panel.Panel-cfg-header\" class=\"docClass\">header</a> is set to <code>false</code>.</p>\n"},"titleAlign":{"!type":"string"},"titleCollapse":{"!type":"bool","!doc":"<p><code>true</code> to allow expanding and collapsing the panel (when <code><a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> = true</code>) by clicking anywhere in\nthe header bar, <code>false</code>) to allow it only by clicking to tool button). When a panel is used in a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>, the <a href=\"#!/api/Ext.panel.Panel-cfg-floatable\" rel=\"Ext.panel.Panel-cfg-floatable\" class=\"docClass\">floatable</a> option can influence the behavior of collapsing.</p>\n"},"titleRotation":{"!type":"string"},"beforeRenderConfig":{},"body":{"!doc":"<p>The Panel's body Element which may be used to contain HTML content.\nThe content may be specified in the <a href=\"#!/api/Ext.panel.Panel-cfg-html\" rel=\"Ext.panel.Panel-cfg-html\" class=\"docClass\">html</a> config, or it may be loaded using the\n<a href=\"#!/api/Ext.panel.Panel-cfg-loader\" rel=\"Ext.panel.Panel-cfg-loader\" class=\"docClass\">loader</a> config. Read-only.</p>\n\n<p>If this is used to load visible HTML elements in either way, then\nthe Panel may not be used as a Layout for hosting nested Panels.</p>\n\n<p>If this Panel is intended to be used as the host of a Layout (See <a href=\"#!/api/Ext.panel.Panel-cfg-layout\" rel=\"Ext.panel.Panel-cfg-layout\" class=\"docClass\">layout</a>\nthen the body Element must not be loaded or changed - it is under the control\nof the Panel's Layout.</p>\n"},"contentPaddingProperty":{"!type":"string","!doc":"<p>The name of the padding property that is used by the layout to manage\npadding.  See <a href=\"#!/api/Ext.layout.container.Auto-property-managePadding\" rel=\"Ext.layout.container.Auto-property-managePadding\" class=\"docClass\">managePadding</a></p>\n"},"dd":{"!doc":"<p>Only present if this Panel has been configured with <a href=\"#!/api/Ext.panel.Panel-cfg-draggable\" rel=\"Ext.panel.Panel-cfg-draggable\" class=\"docClass\">draggable</a> <code>true</code>.</p>\n\n<h2>Simple dragging</h2>\n\n<p>If this Panel is configured <a href=\"#!/api/Ext.panel.Panel-cfg-simpleDrag\" rel=\"Ext.panel.Panel-cfg-simpleDrag\" class=\"docClass\">simpleDrag</a> <code>true</code> (the default is <code>false</code>), this property\nwill reference an instance of <a href=\"#!/api/Ext.util.ComponentDragger\" rel=\"Ext.util.ComponentDragger\" class=\"docClass\">Ext.util.ComponentDragger</a> (A subclass of\n<a href=\"#!/api/Ext.dd.DragTracker\" rel=\"Ext.dd.DragTracker\" class=\"docClass\">DragTracker</a>) which handles moving the Panel's DOM Element,\nand constraining according to the <a href=\"#!/api/Ext.panel.Panel-cfg-constrain\" rel=\"Ext.panel.Panel-cfg-constrain\" class=\"docClass\">constrain</a> and <a href=\"#!/api/Ext.panel.Panel-cfg-constrainHeader\" rel=\"Ext.panel.Panel-cfg-constrainHeader\" class=\"docClass\">constrainHeader</a> .</p>\n\n<p>This object fires various events during its lifecycle and during a drag operation.</p>\n\n<h2>Complex dragging interacting with other DragDrop instances</h2>\n\n<p>By default, this property in a <a href=\"#!/api/Ext.panel.Panel-cfg-draggable\" rel=\"Ext.panel.Panel-cfg-draggable\" class=\"docClass\">draggable</a> Panel will contain an instance of <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> which handles dragging the Panel.</p>\n\n<p>The developer must provide implementations of the abstract methods of <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> in order to\nsupply behaviour for each stage of the drag/drop process. See <a href=\"#!/api/Ext.panel.Panel-cfg-draggable\" rel=\"Ext.panel.Panel-cfg-draggable\" class=\"docClass\">draggable</a>.</p>\n"},"defaultBindProperty":{"!type":"string"},"emptyArray":{},"isPanel":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Panel, or subclass thereof.</p>\n"},"addBodyCls":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Adds a CSS class to the body element. If not rendered, the class will\nbe added when the panel is rendered.</p>\n"},"addTool":{"!type":"fn(tools: ?) -> !this","!doc":"<p>Add tools to this panel</p>\n"},"addTools":{"!type":"fn() -> !this","!doc":"<p>Template method to be implemented in subclasses to add their tools after the collapsible tool.</p>\n"},"addUIClsToElement":{"!type":"fn(cls: ?) -> !this","!doc":"<p>inherit docs</p>\n"},"afterCollapse":{"!type":"fn(animated: ?) -> !this","!doc":"<p>Invoked after the Panel is Collapsed.</p>\n"},"afterExpand":{"!type":"fn(animated: ?) -> !this","!doc":"<p>Invoked after the Panel is Expanded.</p>\n"},"applyState":{"!type":"fn(state: ?) -> !this","!doc":"<p>Applies the state to the object. This should be overridden in subclasses to do\nmore complex state operations. By default it applies the state properties onto\nthe current object.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"beginCollapse":{"!type":"fn() -> !this","!doc":"<p>Called before the change from default, configured state into the collapsed state.\nThis method may be called at render time to enable rendering in an initially collapsed state,\nor at runtime when an existing, fully layed out Panel may be collapsed.\nIt basically saves configs which need to be clobbered for the duration of the collapsed state.</p>\n"},"beginDrag":{"!type":"fn() -> !this"},"beginExpand":{"!type":"fn() -> !this"},"bridgeToolbars":{"!type":"fn() -> !this"},"calculateConstrainedPosition":{"!type":"fn(constrainTo: ?, proposedPosition: ?, local: ?, proposedSize: ?) -> !this","!doc":"<p>Override of Positionable method to calculate constrained position based upon possibly only\nconstraining our header.</p>\n"},"close":{"!type":"fn(panel: ?, eOpts: ?)","!doc":"<p>Fires when the user closes the panel.</p>\n"},"collapse":{"!type":"fn(p: ?, eOpts: ?)","!doc":"<p>Fires after this Panel has collapsed.</p>\n"},"collapsedHorizontal":{"!type":"fn() -> !this"},"collapsedVertical":{"!type":"fn() -> !this"},"convertCollapseDir":{"!type":"fn(collapseDir: ?) -> !this","!doc":"<p>converts a collapsdDir into an anchor argument for Element.slideIn\noverridden in rtl mode to switch \"l\" and \"r\"</p>\n"},"createGhost":{"!type":"fn(cls: ?) -> !this"},"createReExpander":{"!type":"fn(direction: ?, defaults: ?) -> !this"},"doClose":{"!type":"fn() -> !this"},"doCollapseExpand":{"!type":"fn(flags: ?, animate: ?) -> !this"},"endDrag":{"!type":"fn() -> !this"},"expand":{"!type":"fn(p: ?, eOpts: ?)","!doc":"<p>Fires after this Panel has expanded.</p>\n"},"findReExpander":{"!type":"fn(direction: ?) -> !this"},"floatCollapsedPanel":{"!type":"fn() -> !this"},"getAnimationProps":{"!type":"fn() -> !this","!doc":"<ul>\n<li>get animation properties</li>\n</ul>\n\n"},"getCollapsed":{"!type":"fn() -> ?","!doc":"<p>Returns the current collapsed state of the panel.</p>\n"},"getCollapsedDockedItems":{"!type":"fn() -> !this"},"getComponent":{"!type":"fn(comp: ?) -> ?","!doc":"<p>Attempts a default component lookup (see <a href=\"#!/api/Ext.container.Container-method-getComponent\" rel=\"Ext.container.Container-method-getComponent\" class=\"docClass\">Ext.container.Container.getComponent</a>). If the component is not found in the normal\nitems, the dockedItems are searched and the matched component (if any) returned (see <a href=\"#!/api/Ext.panel.Panel-method-getDockedComponent\" rel=\"Ext.panel.Panel-method-getDockedComponent\" class=\"docClass\">getDockedComponent</a>). Note that docked\nitems will only be matched by component id or itemId -- if you pass a numeric index only non-docked child components will be searched.</p>\n"},"getHeader":{"!type":"fn() -> !this","!doc":"<p>Gets the <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Header</a> for this panel.</p>\n"},"getHeaderCollapsedClasses":{"!type":"fn(header: ?) -> !this","!doc":"<p>Create the class array to add to the Header when collpsed.</p>\n"},"getHeightAuthority":{"!type":"fn() -> !this"},"getKeyMap":{"!type":"fn() -> !this"},"getMemento":{"!type":"fn(name: ?) -> !this","!doc":"<p>Memento Factory method</p>\n"},"getOppositeDirection":{"!type":"fn(d: ?) -> !this"},"getPlaceholder":{"!type":"fn(direction: ?) -> !this"},"getProtoBody":{"!type":"fn() -> !this"},"getReExpander":{"!type":"fn(direction: ?) -> !this"},"getRefItems":{"!type":"fn(deep: ?) -> !this","!doc":"<p>Used by ComponentQuery, child and down to retrieve all of the items\nwhich can potentially be considered a child of this Container.</p>\n\n<p>This may be overriden by Components which have ownership of Components\nthat are not contained in the <a href=\"#!/api/Ext.panel.Panel-property-items\" rel=\"Ext.panel.Panel-property-items\" class=\"docClass\">items</a> collection.</p>\n\n<p>NOTE: IMPORTANT note for maintainers:\nItems are returned in tree traversal order. Each item is appended to the result array\nfollowed by the results of that child's getRefItems call.\nFloating child items are appended after internal child items.</p>\n"},"getState":{"!type":"fn() -> ?","!doc":"<p>The supplied default state gathering method for the Component class.</p>\n\n<p>This method returns dimension settings such as <code>flex</code>, <code>anchor</code>, <code>width</code> and <code>height</code> along with <code>collapsed</code>\nstate.</p>\n\n<p>Subclasses which implement more complex state should call the superclass's implementation, and apply their state\nto the result if this basic state is to be saved.</p>\n\n<p>Note that Component state will only be saved if the Component has a <a href=\"#!/api/Ext.panel.Panel-cfg-stateId\" rel=\"Ext.panel.Panel-cfg-stateId\" class=\"docClass\">stateId</a> and there as a StateProvider\nconfigured for the document.</p>\n"},"ghost":{"!type":"fn(cls: ?) -> !this","!doc":"<p>used for dragging</p>\n"},"ghostTools":{"!type":"fn() -> !this","!doc":"<p>helper function for ghost</p>\n"},"initBodyBorder":{"!type":"fn() -> !this"},"initBodyStyles":{"!type":"fn() -> string","!doc":"<p>Parses the <a href=\"#!/api/Ext.panel.Panel-cfg-bodyStyle\" rel=\"Ext.panel.Panel-cfg-bodyStyle\" class=\"docClass\">bodyStyle</a> config if available to create a style string that will be applied to the body element.\nThis also includes <a href=\"#!/api/Ext.panel.Panel-cfg-bodyPadding\" rel=\"Ext.panel.Panel-cfg-bodyPadding\" class=\"docClass\">bodyPadding</a> and <a href=\"#!/api/Ext.panel.Panel-cfg-bodyBorder\" rel=\"Ext.panel.Panel-cfg-bodyBorder\" class=\"docClass\">bodyBorder</a> if available.</p>\n"},"initBorderProps":{"!type":"fn() -> !this"},"initItems":{"!type":"fn() -> !this","!doc":"<p>This method is called to initialize the <code>items</code> collection. A derived class can\noverride this method to do any last minute manipulation of <code>items</code> and then call\nthis method using <code>callParent</code>. Upon return, the <code>items</code> will no longer be a simple\narray.</p>\n"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"initSimpleDraggable":{"!type":"fn() -> !this","!doc":"<p>Override Component.initDraggable.\nPanel (and subclasses) use the header element as the delegate.</p>\n"},"initTools":{"!type":"fn() -> !this","!doc":"<p>Tools are a Panel-specific capabilty.\nPanel uses initTools. Subclasses may contribute tools by implementing addTools.</p>\n"},"isLayoutRoot":{"!type":"fn() -> !this","!doc":"<p>Determines whether this Component is the root of a layout. This returns <code>true</code> if\nthis component can run its layout without assistance from or impact on its owner.\nIf this component cannot run its layout given these restrictions, <code>false</code> is returned\nand its owner will be considered as the next candidate for the layout root.</p>\n\n<p>Setting the <a href=\"#!/api/Ext.panel.Panel-property-_isLayoutRoot\" rel=\"Ext.panel.Panel-property-_isLayoutRoot\" class=\"docClass\">_isLayoutRoot</a> property to <code>true</code> causes this method to always\nreturn <code>true</code>. This may be useful when updating a layout of a Container which shrink\nwraps content, and you know that it will not change size, and so can safely be the\ntopmost participant in the layout run.</p>\n"},"isPlaceHolderCollapse":{"!type":"fn() -> !this"},"isVisible":{"!type":"fn(deep: ?) -> bool","!doc":"<p>Returns <code>true</code> if this component is visible.</p>\n"},"onBoxReady":{"!type":"fn() -> !this","!doc":"<p>Invoked when this component has first achieved size. Occurs after the\n<a href=\"#!/api/Ext.panel.Panel-cfg-componentLayout\" rel=\"Ext.panel.Panel-cfg-componentLayout\" class=\"docClass\">componentLayout</a> has completed its initial run.</p>\n"},"onHide":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the hide operation. After\ncalling the superclass's onHide, the Component will be hidden.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.panel.Panel-event-hide\" rel=\"Ext.panel.Panel-event-hide\" class=\"docClass\">hide</a>.</p>\n"},"onMouseEnterFloated":{"!type":"fn(e: ?) -> !this"},"onMouseLeaveFloated":{"!type":"fn(e: ?) -> !this"},"onPlaceholderResize":{"!type":"fn(ph: ?, newWidth: ?, newHeight: ?) -> !this"},"onRemoved":{"!type":"fn(destroying: ?) -> !this","!doc":"<p>Method to manage awareness of when components are removed from their\nrespective Container, firing a <a href=\"#!/api/Ext.panel.Panel-event-removed\" rel=\"Ext.panel.Panel-event-removed\" class=\"docClass\">removed</a> event. References are properly\ncleaned up after removing a component from its owning container.</p>\n\n<p>Allows addition of behavior when a Component is removed from\nits parent Container. At this stage, the Component has been\nremoved from its parent Container's collection of child items,\nbut has not been destroyed (It will be destroyed if the parent\nContainer's <code>autoDestroy</code> is <code>true</code>, or if the remove call was\npassed a truthy second parameter). After calling the\nsuperclass's <code>onRemoved</code>, the <code>ownerCt</code> and the <code>refOwner</code> will not\nbe present.</p>\n"},"onShow":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the show operation. After\ncalling the superclass's onShow, the Component will be visible.</p>\n\n<p>Override in subclasses where more complex behaviour is needed.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.panel.Panel-event-show\" rel=\"Ext.panel.Panel-event-show\" class=\"docClass\">show</a>.</p>\n"},"placeholderCollapse":{"!type":"fn(direction: ?, animate: ?) -> !this"},"placeholderExpand":{"!type":"fn(animate: ?) -> !this"},"removeBodyCls":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Removes a CSS class from the body element.</p>\n"},"removeUIClsFromElement":{"!type":"fn(cls: ?) -> !this","!doc":"<p>Method which removes a specified UI + <code>uiCls</code> from the components element. The <code>cls</code>\nwhich is added to the element will be: <code>this.baseCls + '-' + ui + uiCls</code>.</p>\n"},"restoreDimension":{"!type":"fn() -> !this"},"restoreHiddenDocked":{"!type":"fn() -> !this"},"setBodyStyle":{"!type":"fn(style: ?, value: ?) -> ?","!doc":"<p>Sets the body style according to the passed parameters.</p>\n"},"setBorder":{"!type":"fn(border: ?, targetEl: ?) -> !this"},"setCollapsible":{"!type":"fn(collapsible: ?) -> !this"},"setHiddenDocked":{"!type":"fn() -> !this"},"setUI":{"!type":"fn(ui: ?) -> !this","!doc":"<p>Sets the UI for the component. This will remove any existing UIs on the component. It will also loop through any\n<code>uiCls</code> set on the component and rename them so they include the new UI.</p>\n"},"slideOutFloatedPanelBegin":{"!type":"fn() -> !this","!doc":"<p>This method begins the slide out of the floated panel.</p>\n"},"slideOutFloatedPanelEnd":{"!type":"fn(suppressEvents: ?) -> !this","!doc":"<p>This method cleans up after the slide out of the floated panel.</p>\n"},"toggleCollapse":{"!type":"fn() -> ?","!doc":"<p>Shortcut for performing an <a href=\"#!/api/Ext.panel.Panel-method-expand\" rel=\"Ext.panel.Panel-method-expand\" class=\"docClass\">expand</a> or <a href=\"#!/api/Ext.panel.Panel-method-collapse\" rel=\"Ext.panel.Panel-method-collapse\" class=\"docClass\">collapse</a> based on the current state of the panel.</p>\n"},"unghost":{"!type":"fn(show: ?, matchPosition: ?, focus: ?) -> !this"},"updateCollapseTool":{"!type":"fn() -> !this"},"updateGlyph":{"!type":"fn(glyph: ?, oldGlyph: ?) -> !this"},"updateHeader":{"!type":"fn(force: ?) -> !this","!doc":"<p>Create, hide, or show the header component as appropriate based on the current config.</p>\n"},"updateHeaderPosition":{"!type":"fn(position: ?) -> !this"},"updateIcon":{"!type":"fn(icon: ?, oldIcon: ?) -> !this"},"updateIconAlign":{"!type":"fn(align: ?) -> !this"},"updateIconCls":{"!type":"fn(iconCls: ?) -> !this","!doc":"<p>Set the iconCls for the panel's header. See <a href=\"#!/api/Ext.panel.Header-cfg-iconCls\" rel=\"Ext.panel.Header-cfg-iconCls\" class=\"docClass\">Ext.panel.Header.iconCls</a>. It will fire the\n<a href=\"#!/api/Ext.panel.Panel-event-iconclschange\" rel=\"Ext.panel.Panel-event-iconclschange\" class=\"docClass\">iconclschange</a> event after completion.</p>\n"},"updateTitle":{"!type":"fn(title: ?, oldTitle: ?) -> !this"},"updateTitleAlign":{"!type":"fn(align: ?) -> !this"},"updateTitleRotation":{"!type":"fn(rotation: ?) -> !this"},"beforeclose":{"!type":"fn(panel: ?, eOpts: ?)","!doc":"<p>Fires before the user closes the panel. Return false from any listener to stop the close event being\nfired</p>\n"},"beforecollapse":{"!type":"fn(p: ?, direction: ?, animate: ?, eOpts: ?)","!doc":"<p>Fires before this panel is collapsed. Return false to prevent the collapse.</p>\n"},"beforeexpand":{"!type":"fn(p: ?, animate: ?, eOpts: ?)","!doc":"<p>Fires before this panel is expanded. Return false to prevent the expand.</p>\n"},"float":{"!type":"fn(eOpts: ?)","!doc":"<p>Fires after a collapsed Panel has been \"floated\" by clicking on\nit's header. Only applicable when the Panel is an item in a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Border Layout</a>.</p>\n"},"glyphchange":{"!type":"fn(this: ?, newGlyph: ?, oldGlyph: ?, eOpts: ?)","!doc":"<p>Fired when the Panel glyph has been changed by the setGlyph method.</p>\n"},"iconchange":{"!type":"fn(p: ?, newIcon: ?, oldIcon: ?, eOpts: ?)","!doc":"<p>Fires after the Panel icon has been set or changed.</p>\n"},"iconclschange":{"!type":"fn(p: ?, newIconCls: ?, oldIconCls: ?, eOpts: ?)","!doc":"<p>Fires after the Panel iconCls has been set or changed.</p>\n"},"titlechange":{"!type":"fn(p: ?, newTitle: ?, oldTitle: ?, eOpts: ?)","!doc":"<p>Fires after the Panel title has been set or changed.</p>\n"},"unfloat":{"!type":"fn(eOpts: ?)","!doc":"<p>Fires after a \"floated\" Panel has returned to it's collapsed state\nas a result of the mouse leaving the Panel. Only applicable when\nthe Panel is an item in a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Border Layout</a>.</p>\n"},"animCollapse":{"!type":"bool","!doc":"<p><code>true</code> to animate the transition when the panel is collapsed, <code>false</code> to skip the animation (defaults to <code>true</code>\nif the <a href=\"#!/api/Ext.fx.Anim\" rel=\"Ext.fx.Anim\" class=\"docClass\">Ext.fx.Anim</a> class is available, otherwise <code>false</code>). May also be specified as the animation\nduration in milliseconds.</p>\n"},"baseCls":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"closable":{"!type":"bool","!doc":"<p>True to display the 'close' tool button and allow the user to close the window, false to hide the button and\ndisallow closing the window.</p>\n\n<p>By default, when close is requested by clicking the close button in the header, the <a href=\"#!/api/Ext.panel.Panel-method-close\" rel=\"Ext.panel.Panel-method-close\" class=\"docClass\">close</a> method will be\ncalled. This will <em><a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a></em> the Panel and its content meaning that it may not be\nreused.</p>\n\n<p>To make closing a Panel <em>hide</em> the Panel so that it may be reused, set <a href=\"#!/api/Ext.panel.Panel-cfg-closeAction\" rel=\"Ext.panel.Panel-cfg-closeAction\" class=\"docClass\">closeAction</a> to 'hide'.</p>\n"},"collapseMode":{"!type":"string","!doc":"<p><strong>Important: this config is only effective for <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> Panels which are direct child items of a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>.</strong></p>\n\n<p>When <em>not</em> a direct child item of a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>, then the Panel's header\nremains visible, and the body is collapsed to zero dimensions. If the Panel has no header, then a new header\n(orientated correctly depending on the <a href=\"#!/api/Ext.panel.Panel-cfg-collapseDirection\" rel=\"Ext.panel.Panel-cfg-collapseDirection\" class=\"docClass\">collapseDirection</a>) will be inserted to show a the title and a re-\nexpand tool.</p>\n\n<p>When a child item of a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>, this config has three possible values:</p>\n\n<ul>\n<li><p><code>undefined</code> - When collapsed, a placeholder <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Header</a> is injected into the layout to\nrepresent the Panel and to provide a UI with a Tool to allow the user to re-expand the Panel.</p></li>\n<li><p><code>\"header\"</code> - The Panel collapses to leave its header visible as when not inside a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>.</p></li>\n<li><p><code>\"mini\"</code> - The Panel collapses without a visible header.</p></li>\n</ul>\n\n"},"collapsible":{"!type":"bool","!doc":"<p>True to make the panel collapsible and have an expand/collapse toggle Tool added into the header tool button\narea. False to keep the panel sized either statically, or by an owning layout manager, with no toggle Tool.\nWhen a panel is used in a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>, the <a href=\"#!/api/Ext.panel.Panel-cfg-floatable\" rel=\"Ext.panel.Panel-cfg-floatable\" class=\"docClass\">floatable</a> option\ncan influence the behavior of collapsing.\nSee <a href=\"#!/api/Ext.panel.Panel-cfg-collapseMode\" rel=\"Ext.panel.Panel-cfg-collapseMode\" class=\"docClass\">collapseMode</a> and <a href=\"#!/api/Ext.panel.Panel-cfg-collapseDirection\" rel=\"Ext.panel.Panel-cfg-collapseDirection\" class=\"docClass\">collapseDirection</a></p>\n"},"frame":{"!type":"bool","!doc":"<p>True to apply a frame to the panel.</p>\n"},"tools":{"!doc":"<p>An array of <a href=\"#!/api/Ext.panel.Tool\" rel=\"Ext.panel.Tool\" class=\"docClass\">Ext.panel.Tool</a> configs/instances to be added to the header tool area. The tools are stored as\nchild components of the header container. They can be accessed using down and {#query}, as well as the\nother component methods. The toggle tool is automatically created if <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> is set to true.</p>\n\n<p>Note that, apart from the toggle tool which is provided when a panel is collapsible, these tools only provide the\nvisual button. Any required functionality must be provided by adding handlers that implement the necessary\nbehavior.</p>\n\n<p>Example usage:</p>\n\n<pre><code>tools:[{\n    type:'refresh',\n    tooltip: 'Refresh form Data',\n    // hidden:true,\n    handler: function(event, toolEl, panelHeader) {\n        // refresh logic\n    }\n},\n{\n    type:'help',\n    tooltip: 'Get Help',\n    callback: function(panel, tool, event) {\n        // show help here\n    }\n}]\n</code></pre>\n\n<p>The difference between <code>handler</code> and <code>callback</code> is the signature. For details on\nthe distinction, see <a href=\"#!/api/Ext.panel.Tool\" rel=\"Ext.panel.Tool\" class=\"docClass\">Ext.panel.Tool</a>.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"}},"animCollapse":{"!type":"bool","!doc":"<p><code>true</code> to animate the transition when the panel is collapsed, <code>false</code> to skip the animation (defaults to <code>true</code>\nif the <a href=\"#!/api/Ext.fx.Anim\" rel=\"Ext.fx.Anim\" class=\"docClass\">Ext.fx.Anim</a> class is available, otherwise <code>false</code>). May also be specified as the animation\nduration in milliseconds.</p>\n"},"bbar":{"!doc":"<p>Convenience config. Short for 'Bottom Bar'.</p>\n\n<pre><code>bbar: [\n  { xtype: 'button', text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'bottom',\n    items: [\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n"},"bodyBorder":{"!type":"bool","!doc":"<p>A shortcut to add or remove the border on the body of a panel. In the classic theme\nthis only applies to a panel which has the <a href=\"#!/api/Ext.panel.Panel-cfg-frame\" rel=\"Ext.panel.Panel-cfg-frame\" class=\"docClass\">frame</a> configuration set to <code>true</code>.</p>\n"},"bodyCls":{"!doc":"<p>A CSS class, space-delimited string of classes, or array of classes to be applied to the panel's body element.\nThe following examples are all valid:</p>\n\n<pre><code>bodyCls: 'foo'\nbodyCls: 'foo bar'\nbodyCls: ['foo', 'bar']\n</code></pre>\n"},"bodyPadding":{"!doc":"<p>A shortcut for setting a padding style on the body element. The value can either be\na number to be applied to all sides, or a normal css string describing padding.</p>\n"},"bodyStyle":{"!doc":"<p>Custom CSS styles to be applied to the panel's body element, which can be supplied as a valid CSS style string,\nan object containing style property name/value pairs or a function that returns such a string or object.\nFor example, these two formats are interpreted to be equivalent:</p>\n\n<pre><code>bodyStyle: 'background:#ffc; padding:10px;'\n\nbodyStyle: {\n    background: '#ffc',\n    padding: '10px'\n}\n</code></pre>\n"},"border":{"!type":"bool","!doc":"<p>Specify as <code>false</code> to render the Panel with zero width borders.</p>\n\n<p>Leaving the value as <code>true</code> uses the selected theme's $panel-border-width</p>\n\n<p>Defaults to <code>false</code> when using or extending Neptune.</p>\n"},"buttonAlign":{"!type":"string","!doc":"<p>The alignment of any buttons added to this panel. Valid values are 'right', 'left' and 'center' (defaults to\n'right' for buttons/fbar, 'left' for other toolbar types).</p>\n\n<p><strong>NOTE:</strong> The prefered way to specify toolbars is to use the dockedItems config. Instead of buttonAlign you\nwould add the layout: { pack: 'start' | 'center' | 'end' } option to the dockedItem config.</p>\n"},"buttons":{"!doc":"<p>Convenience config used for adding buttons docked to the bottom of the panel. This is a\nsynonym for the <a href=\"#!/api/Ext.panel.Panel-cfg-fbar\" rel=\"Ext.panel.Panel-cfg-fbar\" class=\"docClass\">fbar</a> config.</p>\n\n<pre><code>buttons: [\n  { text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'bottom',\n    ui: 'footer',\n    defaults: {minWidth: <a href=\"#!/api/Ext.panel.Panel-cfg-minButtonWidth\" rel=\"Ext.panel.Panel-cfg-minButtonWidth\" class=\"docClass\">minButtonWidth</a>},\n    items: [\n        { xtype: 'component', flex: 1 },\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n\n<p>The <a href=\"#!/api/Ext.panel.Panel-cfg-minButtonWidth\" rel=\"Ext.panel.Panel-cfg-minButtonWidth\" class=\"docClass\">minButtonWidth</a> is used as the default <a href=\"#!/api/Ext.button.Button-cfg-minWidth\" rel=\"Ext.button.Button-cfg-minWidth\" class=\"docClass\">minWidth</a> for\neach of the buttons in the buttons toolbar.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"collapseDirection":{"!type":"string","!doc":"<p>The direction to collapse the Panel when the toggle button is clicked.</p>\n\n<p>Defaults to the <a href=\"#!/api/Ext.panel.Panel-property-headerPosition\" rel=\"Ext.panel.Panel-property-headerPosition\" class=\"docClass\">headerPosition</a></p>\n\n<p><strong>Important: This config is <em>ignored</em> for <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> Panels which are direct child items of a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>.</strong></p>\n\n<p>Specify as <code>'top'</code>, <code>'bottom'</code>, <code>'left'</code> or <code>'right'</code>.</p>\n"},"collapseFirst":{"!type":"bool","!doc":"<p><code>true</code> to make sure the collapse/expand toggle button always renders first (to the left of) any other tools in\nthe panel's title bar, <code>false</code> to render it last.</p>\n"},"collapseMode":{"!type":"string","!doc":"<p><strong>Important: this config is only effective for <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> Panels which are direct child items of a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>.</strong></p>\n\n<p>When <em>not</em> a direct child item of a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>, then the Panel's header\nremains visible, and the body is collapsed to zero dimensions. If the Panel has no header, then a new header\n(orientated correctly depending on the <a href=\"#!/api/Ext.panel.Panel-cfg-collapseDirection\" rel=\"Ext.panel.Panel-cfg-collapseDirection\" class=\"docClass\">collapseDirection</a>) will be inserted to show a the title and a re-\nexpand tool.</p>\n\n<p>When a child item of a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>, this config has three possible values:</p>\n\n<ul>\n<li><p><code>undefined</code> - When collapsed, a placeholder <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Header</a> is injected into the layout to\nrepresent the Panel and to provide a UI with a Tool to allow the user to re-expand the Panel.</p></li>\n<li><p><code>\"header\"</code> - The Panel collapses to leave its header visible as when not inside a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>.</p></li>\n<li><p><code>\"mini\"</code> - The Panel collapses without a visible header.</p></li>\n</ul>\n\n"},"collapsedCls":{"!type":"string","!doc":"<p>A CSS class to add to the panel's element after it has been collapsed.</p>\n"},"componentLayout":{"!doc":"<p>The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout\nmanager which sizes a Component's internal structure in response to the Component being sized.</p>\n\n<p>Generally, developers will not use this configuration as all provided Components which need their internal\nelements sizing (Such as <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">input fields</a>) come with their own componentLayout managers.</p>\n\n<p>The <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used on instances of the base <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>\nclass which simply sizes the Component's encapsulating element to the height and width specified in the\n<a href=\"#!/api/Ext.panel.Panel-method-setSize\" rel=\"Ext.panel.Panel-method-setSize\" class=\"docClass\">setSize</a> method.</p>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"dockedItems":{"!doc":"<p>A component or series of components to be added as docked items to this panel. The docked items can be docked to\neither the top, right, left or bottom of a panel. This is typically used for things like toolbars or tab bars:</p>\n\n<pre><code>var panel = new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({\n    dockedItems: [{\n        xtype: 'toolbar',\n        dock: 'top',\n        items: [{\n            text: 'Docked to the top'\n        }]\n    }]\n});\n</code></pre>\n"},"fbar":{"!doc":"<p>Convenience config used for adding items to the bottom of the panel. Short for Footer Bar.</p>\n\n<pre><code>fbar: [\n  { type: 'button', text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'bottom',\n    ui: 'footer',\n    defaults: {minWidth: <a href=\"#!/api/Ext.panel.Panel-cfg-minButtonWidth\" rel=\"Ext.panel.Panel-cfg-minButtonWidth\" class=\"docClass\">minButtonWidth</a>},\n    items: [\n        { xtype: 'component', flex: 1 },\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n\n<p>The <a href=\"#!/api/Ext.panel.Panel-cfg-minButtonWidth\" rel=\"Ext.panel.Panel-cfg-minButtonWidth\" class=\"docClass\">minButtonWidth</a> is used as the default <a href=\"#!/api/Ext.button.Button-cfg-minWidth\" rel=\"Ext.button.Button-cfg-minWidth\" class=\"docClass\">minWidth</a> for\neach of the buttons in the fbar.</p>\n"},"floatable":{"!type":"bool","!doc":"<p><strong>Important: This config is only effective for <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> Panels which are direct child items of a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>.</strong></p>\n\n<p>true to allow clicking a collapsed Panel's <a href=\"#!/api/Ext.panel.Panel-cfg-placeholder\" rel=\"Ext.panel.Panel-cfg-placeholder\" class=\"docClass\">placeholder</a> to display the Panel floated above the layout,\nfalse to force the user to fully expand a collapsed region by clicking the expand button to see it again.</p>\n"},"frame":{"!type":"bool","!doc":"<p>True to apply a frame to the panel.</p>\n"},"frameHeader":{"!type":"bool","!doc":"<p>True to apply a frame to the panel panels header (if 'frame' is true).</p>\n"},"glyph":{"!doc":"<p>A numeric unicode character code to use as the icon for the panel header. The\ndefault font-family for glyphs can be set globally using\n<a href=\"#!/api/Ext-method-setGlyphFontFamily\" rel=\"Ext-method-setGlyphFontFamily\" class=\"docClass\">Ext.setGlyphFontFamily()</a>. Alternatively, this\nconfig option accepts a string with the charCode and font-family separated by the\n<code>@</code> symbol. For example '65@My Font Family'.</p>\n"},"header":{"!doc":"<p>Pass as <code>false</code> to prevent a Header from being created and shown.</p>\n\n<p>Pass as a config object (optionally containing an <code>xtype</code>) to custom-configure this Panel's header.</p>\n\n<p>See <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Ext.panel.Header</a> for all the options that may be specified here.</p>\n\n<p>A <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">panel header</a> is a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a> which contains the Panel's <a href=\"#!/api/Ext.panel.Panel-cfg-title\" rel=\"Ext.panel.Panel-cfg-title\" class=\"docClass\">title</a> and <a href=\"#!/api/Ext.panel.Panel-cfg-tools\" rel=\"Ext.panel.Panel-cfg-tools\" class=\"docClass\">tools</a>.\nYou may also configure the Panel's <code>header</code> option with its own child items which go <em>before</em> the <a href=\"#!/api/Ext.panel.Panel-cfg-tools\" rel=\"Ext.panel.Panel-cfg-tools\" class=\"docClass\">tools</a></p>\n\n<p>By default the panel <a href=\"#!/api/Ext.panel.Panel-cfg-title\" rel=\"Ext.panel.Panel-cfg-title\" class=\"docClass\">title</a> is inserted after items configured in this config, but before any tools.\nTo insert the title at any point in the full array, specify the <a href=\"#!/api/Ext.panel.Header-property-titlePosition\" rel=\"Ext.panel.Header-property-titlePosition\" class=\"docClass\">titlePosition</a> config:</p>\n\n<p>   new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({\n       title: 'Test',\n       tools: [{\n           type: 'refresh\n       }, {\n           type: 'help'\n       }],\n       titlePosition: 2 // Title will come AFTER the two tools\n       ...\n   });</p>\n"},"headerOverCls":{"!type":"string","!doc":"<p>Optional CSS class to apply to the header element on mouseover</p>\n"},"headerPosition":{"!type":"string","!doc":"<p>For performance reasons we give the following configs their default values on\nthe class body.  This prevents the updaters from running on initialization in the\ndefault configuration scenario</p>\n"},"hideCollapseTool":{"!type":"bool","!doc":"<p><code>true</code> to hide the expand/collapse toggle button when <code><a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> == true</code>, <code>false</code> to display it.</p>\n"},"icon":{"!type":"string","!doc":"<p>Path to image for an icon.</p>\n\n<p>There are no default icons that come with Ext JS.</p>\n"},"iconAlign":{"!type":"string"},"iconCls":{"!type":"string","!doc":"<p>CSS class for an icon.</p>\n\n<p>There are no default icon classes that come with Ext JS.</p>\n"},"lbar":{"!doc":"<p>Convenience config. Short for 'Left Bar' (left-docked, vertical toolbar).</p>\n\n<pre><code>lbar: [\n  { xtype: 'button', text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'left',\n    items: [\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n"},"manageHeight":{"!type":"bool","!doc":"<p>When true, the dock component layout writes\nheight information to the panel's DOM elements based on its shrink wrap height\ncalculation. This ensures that the browser respects the calculated height.\nWhen false, the dock component layout will not write heights on the panel or its\nbody element. In some simple layout cases, not writing the heights to the DOM may\nbe desired because this allows the browser to respond to direct DOM manipulations\n(like animations).</p>\n"},"maskElement":{"!type":"string","!doc":"<p>The name of the element property in this Panel to mask when masked by a LoadMask.</p>\n\n<p>Defaults to <code>\"el\"</code> to indicate that any LoadMask should be rendered into this Panel's encapsulating element.</p>\n\n<p>This could be configured to be <code>\"body\"</code> so that only the body is masked and toolbars and the header are still mouse-accessible.</p>\n"},"minButtonWidth":{"!type":"number","!doc":"<p>Minimum width of all footer toolbar buttons in pixels. If set, this will be used as the default\nvalue for the <a href=\"#!/api/Ext.button.Button-cfg-minWidth\" rel=\"Ext.button.Button-cfg-minWidth\" class=\"docClass\">Ext.button.Button.minWidth</a> config of each Button added to the <strong>footer toolbar</strong> via the\n<a href=\"#!/api/Ext.panel.Panel-cfg-fbar\" rel=\"Ext.panel.Panel-cfg-fbar\" class=\"docClass\">fbar</a> or <a href=\"#!/api/Ext.panel.Panel-cfg-buttons\" rel=\"Ext.panel.Panel-cfg-buttons\" class=\"docClass\">buttons</a> configurations. It will be ignored for buttons that have a minWidth configured\nsome other way, e.g. in their own config object or via the <a href=\"#!/api/Ext.container.Container-cfg-defaults\" rel=\"Ext.container.Container-cfg-defaults\" class=\"docClass\">defaults</a> of\ntheir parent container.</p>\n"},"placeholder":{"!doc":"<p><strong>Important: This config is only effective for <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> Panels which are direct child items of a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a> when not using the <code>'header'</code> <a href=\"#!/api/Ext.panel.Panel-cfg-collapseMode\" rel=\"Ext.panel.Panel-cfg-collapseMode\" class=\"docClass\">collapseMode</a>.</strong></p>\n\n<p><strong>Optional.</strong> A Component (or config object for a Component) to show in place of this Panel when this Panel is\ncollapsed by a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>. Defaults to a generated <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Header</a> containing a <a href=\"#!/api/Ext.panel.Tool\" rel=\"Ext.panel.Tool\" class=\"docClass\">Tool</a> to re-expand the Panel.</p>\n"},"placeholderCollapseHideMode":{"!type":"number","!doc":"<p>The mode for hiding collapsed panels when\nusing <a href=\"#!/api/Ext.panel.Panel-cfg-collapseMode\" rel=\"Ext.panel.Panel-cfg-collapseMode\" class=\"docClass\">collapseMode</a> \"placeholder\".</p>\n"},"preventHeader":{"!type":"bool"},"rbar":{"!doc":"<p>Convenience config. Short for 'Right Bar' (right-docked, vertical toolbar).</p>\n\n<pre><code>rbar: [\n  { xtype: 'button', text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'right',\n    items: [\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.panel.Panel-method-getEl\" rel=\"Ext.panel.Panel-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.panel.Panel-method-getEl\" rel=\"Ext.panel.Panel-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.panel.Panel-cfg-renderSelectors\" rel=\"Ext.panel.Panel-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.panel.Panel-cfg-childEls\" rel=\"Ext.panel.Panel-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"tbar":{"!doc":"<p>Convenience config. Short for 'Top Bar'.</p>\n\n<pre><code>tbar: [\n  { xtype: 'button', text: 'Button 1' }\n]\n</code></pre>\n\n<p>is equivalent to</p>\n\n<pre><code>dockedItems: [{\n    xtype: 'toolbar',\n    dock: 'top',\n    items: [\n        { xtype: 'button', text: 'Button 1' }\n    ]\n}]\n</code></pre>\n"},"titleAlign":{"!type":"string"},"titleCollapse":{"!type":"bool","!doc":"<p><code>true</code> to allow expanding and collapsing the panel (when <code><a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> = true</code>) by clicking anywhere in\nthe header bar, <code>false</code>) to allow it only by clicking to tool button). When a panel is used in a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>, the <a href=\"#!/api/Ext.panel.Panel-cfg-floatable\" rel=\"Ext.panel.Panel-cfg-floatable\" class=\"docClass\">floatable</a> option can influence the behavior of collapsing.</p>\n"},"titleRotation":{"!type":"string"},"tools":{"!doc":"<p>An array of <a href=\"#!/api/Ext.panel.Tool\" rel=\"Ext.panel.Tool\" class=\"docClass\">Ext.panel.Tool</a> configs/instances to be added to the header tool area. The tools are stored as\nchild components of the header container. They can be accessed using down and {#query}, as well as the\nother component methods. The toggle tool is automatically created if <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> is set to true.</p>\n\n<p>Note that, apart from the toggle tool which is provided when a panel is collapsible, these tools only provide the\nvisual button. Any required functionality must be provided by adding handlers that implement the necessary\nbehavior.</p>\n\n<p>Example usage:</p>\n\n<pre><code>tools:[{\n    type:'refresh',\n    tooltip: 'Refresh form Data',\n    // hidden:true,\n    handler: function(event, toolEl, panelHeader) {\n        // refresh logic\n    }\n},\n{\n    type:'help',\n    tooltip: 'Get Help',\n    callback: function(panel, tool, event) {\n        // show help here\n    }\n}]\n</code></pre>\n\n<p>The difference between <code>handler</code> and <code>callback</code> is the signature. For details on\nthe distinction, see <a href=\"#!/api/Ext.panel.Tool\" rel=\"Ext.panel.Tool\" class=\"docClass\">Ext.panel.Tool</a>.</p>\n"},"beforeRenderConfig":{},"body":{"!doc":"<p>The Panel's body Element which may be used to contain HTML content.\nThe content may be specified in the <a href=\"#!/api/Ext.panel.Panel-cfg-html\" rel=\"Ext.panel.Panel-cfg-html\" class=\"docClass\">html</a> config, or it may be loaded using the\n<a href=\"#!/api/Ext.panel.Panel-cfg-loader\" rel=\"Ext.panel.Panel-cfg-loader\" class=\"docClass\">loader</a> config. Read-only.</p>\n\n<p>If this is used to load visible HTML elements in either way, then\nthe Panel may not be used as a Layout for hosting nested Panels.</p>\n\n<p>If this Panel is intended to be used as the host of a Layout (See <a href=\"#!/api/Ext.panel.Panel-cfg-layout\" rel=\"Ext.panel.Panel-cfg-layout\" class=\"docClass\">layout</a>\nthen the body Element must not be loaded or changed - it is under the control\nof the Panel's Layout.</p>\n"},"contentPaddingProperty":{"!type":"string","!doc":"<p>The name of the padding property that is used by the layout to manage\npadding.  See <a href=\"#!/api/Ext.layout.container.Auto-property-managePadding\" rel=\"Ext.layout.container.Auto-property-managePadding\" class=\"docClass\">managePadding</a></p>\n"},"defaultBindProperty":{"!type":"string"},"emptyArray":{},"isPanel":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Panel, or subclass thereof.</p>\n"},"addBodyCls":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Adds a CSS class to the body element. If not rendered, the class will\nbe added when the panel is rendered.</p>\n"},"addTool":{"!type":"fn(tools: ?) -> !this","!doc":"<p>Add tools to this panel</p>\n"},"addUIClsToElement":{"!type":"fn(cls: ?) -> !this","!doc":"<p>inherit docs</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"beginCollapse":{"!type":"fn() -> !this","!doc":"<p>Called before the change from default, configured state into the collapsed state.\nThis method may be called at render time to enable rendering in an initially collapsed state,\nor at runtime when an existing, fully layed out Panel may be collapsed.\nIt basically saves configs which need to be clobbered for the duration of the collapsed state.</p>\n"},"beginDrag":{"!type":"fn() -> !this"},"beginExpand":{"!type":"fn() -> !this"},"bridgeToolbars":{"!type":"fn() -> !this"},"calculateConstrainedPosition":{"!type":"fn(constrainTo: ?, proposedPosition: ?, local: ?, proposedSize: ?) -> !this","!doc":"<p>Override of Positionable method to calculate constrained position based upon possibly only\nconstraining our header.</p>\n"},"close":{"!type":"fn(panel: ?, eOpts: ?)","!doc":"<p>Fires when the user closes the panel.</p>\n"},"collapse":{"!type":"fn(p: ?, eOpts: ?)","!doc":"<p>Fires after this Panel has collapsed.</p>\n"},"collapsedHorizontal":{"!type":"fn() -> !this"},"collapsedVertical":{"!type":"fn() -> !this"},"convertCollapseDir":{"!type":"fn(collapseDir: ?) -> !this","!doc":"<p>converts a collapsdDir into an anchor argument for Element.slideIn\noverridden in rtl mode to switch \"l\" and \"r\"</p>\n"},"createReExpander":{"!type":"fn(direction: ?, defaults: ?) -> !this"},"doCollapseExpand":{"!type":"fn(flags: ?, animate: ?) -> !this"},"endDrag":{"!type":"fn() -> !this"},"expand":{"!type":"fn(p: ?, eOpts: ?)","!doc":"<p>Fires after this Panel has expanded.</p>\n"},"findReExpander":{"!type":"fn(direction: ?) -> !this"},"floatCollapsedPanel":{"!type":"fn() -> !this"},"getAnimationProps":{"!type":"fn() -> !this","!doc":"<ul>\n<li>get animation properties</li>\n</ul>\n\n"},"getCollapsed":{"!type":"fn() -> ?","!doc":"<p>Returns the current collapsed state of the panel.</p>\n"},"getCollapsedDockedItems":{"!type":"fn() -> !this"},"getComponent":{"!type":"fn(comp: ?) -> ?","!doc":"<p>Attempts a default component lookup (see <a href=\"#!/api/Ext.container.Container-method-getComponent\" rel=\"Ext.container.Container-method-getComponent\" class=\"docClass\">Ext.container.Container.getComponent</a>). If the component is not found in the normal\nitems, the dockedItems are searched and the matched component (if any) returned (see <a href=\"#!/api/Ext.panel.Panel-method-getDockedComponent\" rel=\"Ext.panel.Panel-method-getDockedComponent\" class=\"docClass\">getDockedComponent</a>). Note that docked\nitems will only be matched by component id or itemId -- if you pass a numeric index only non-docked child components will be searched.</p>\n"},"getHeader":{"!type":"fn() -> !this","!doc":"<p>Gets the <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Header</a> for this panel.</p>\n"},"getHeaderCollapsedClasses":{"!type":"fn(header: ?) -> !this","!doc":"<p>Create the class array to add to the Header when collpsed.</p>\n"},"getHeightAuthority":{"!type":"fn() -> !this"},"getKeyMap":{"!type":"fn() -> !this"},"getMemento":{"!type":"fn(name: ?) -> !this","!doc":"<p>Memento Factory method</p>\n"},"getOppositeDirection":{"!type":"fn(d: ?) -> !this"},"getPlaceholder":{"!type":"fn(direction: ?) -> !this"},"getProtoBody":{"!type":"fn() -> !this"},"getReExpander":{"!type":"fn(direction: ?) -> !this"},"getRefItems":{"!type":"fn(deep: ?) -> !this","!doc":"<p>Used by ComponentQuery, child and down to retrieve all of the items\nwhich can potentially be considered a child of this Container.</p>\n\n<p>This may be overriden by Components which have ownership of Components\nthat are not contained in the <a href=\"#!/api/Ext.panel.Panel-property-items\" rel=\"Ext.panel.Panel-property-items\" class=\"docClass\">items</a> collection.</p>\n\n<p>NOTE: IMPORTANT note for maintainers:\nItems are returned in tree traversal order. Each item is appended to the result array\nfollowed by the results of that child's getRefItems call.\nFloating child items are appended after internal child items.</p>\n"},"ghost":{"!type":"fn(cls: ?) -> !this","!doc":"<p>used for dragging</p>\n"},"ghostTools":{"!type":"fn() -> !this","!doc":"<p>helper function for ghost</p>\n"},"initBodyBorder":{"!type":"fn() -> !this"},"initBodyStyles":{"!type":"fn() -> string","!doc":"<p>Parses the <a href=\"#!/api/Ext.panel.Panel-cfg-bodyStyle\" rel=\"Ext.panel.Panel-cfg-bodyStyle\" class=\"docClass\">bodyStyle</a> config if available to create a style string that will be applied to the body element.\nThis also includes <a href=\"#!/api/Ext.panel.Panel-cfg-bodyPadding\" rel=\"Ext.panel.Panel-cfg-bodyPadding\" class=\"docClass\">bodyPadding</a> and <a href=\"#!/api/Ext.panel.Panel-cfg-bodyBorder\" rel=\"Ext.panel.Panel-cfg-bodyBorder\" class=\"docClass\">bodyBorder</a> if available.</p>\n"},"initBorderProps":{"!type":"fn() -> !this"},"initItems":{"!type":"fn() -> !this","!doc":"<p>This method is called to initialize the <code>items</code> collection. A derived class can\noverride this method to do any last minute manipulation of <code>items</code> and then call\nthis method using <code>callParent</code>. Upon return, the <code>items</code> will no longer be a simple\narray.</p>\n"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"initSimpleDraggable":{"!type":"fn() -> !this","!doc":"<p>Override Component.initDraggable.\nPanel (and subclasses) use the header element as the delegate.</p>\n"},"initTools":{"!type":"fn() -> !this","!doc":"<p>Tools are a Panel-specific capabilty.\nPanel uses initTools. Subclasses may contribute tools by implementing addTools.</p>\n"},"isLayoutRoot":{"!type":"fn() -> !this","!doc":"<p>Determines whether this Component is the root of a layout. This returns <code>true</code> if\nthis component can run its layout without assistance from or impact on its owner.\nIf this component cannot run its layout given these restrictions, <code>false</code> is returned\nand its owner will be considered as the next candidate for the layout root.</p>\n\n<p>Setting the <a href=\"#!/api/Ext.panel.Panel-property-_isLayoutRoot\" rel=\"Ext.panel.Panel-property-_isLayoutRoot\" class=\"docClass\">_isLayoutRoot</a> property to <code>true</code> causes this method to always\nreturn <code>true</code>. This may be useful when updating a layout of a Container which shrink\nwraps content, and you know that it will not change size, and so can safely be the\ntopmost participant in the layout run.</p>\n"},"isPlaceHolderCollapse":{"!type":"fn() -> !this"},"isVisible":{"!type":"fn(deep: ?) -> bool","!doc":"<p>Returns <code>true</code> if this component is visible.</p>\n"},"onBoxReady":{"!type":"fn() -> !this","!doc":"<p>Invoked when this component has first achieved size. Occurs after the\n<a href=\"#!/api/Ext.panel.Panel-cfg-componentLayout\" rel=\"Ext.panel.Panel-cfg-componentLayout\" class=\"docClass\">componentLayout</a> has completed its initial run.</p>\n"},"onHide":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the hide operation. After\ncalling the superclass's onHide, the Component will be hidden.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.panel.Panel-event-hide\" rel=\"Ext.panel.Panel-event-hide\" class=\"docClass\">hide</a>.</p>\n"},"onMouseEnterFloated":{"!type":"fn(e: ?) -> !this"},"onMouseLeaveFloated":{"!type":"fn(e: ?) -> !this"},"onPlaceholderResize":{"!type":"fn(ph: ?, newWidth: ?, newHeight: ?) -> !this"},"onRemoved":{"!type":"fn(destroying: ?) -> !this","!doc":"<p>Method to manage awareness of when components are removed from their\nrespective Container, firing a <a href=\"#!/api/Ext.panel.Panel-event-removed\" rel=\"Ext.panel.Panel-event-removed\" class=\"docClass\">removed</a> event. References are properly\ncleaned up after removing a component from its owning container.</p>\n\n<p>Allows addition of behavior when a Component is removed from\nits parent Container. At this stage, the Component has been\nremoved from its parent Container's collection of child items,\nbut has not been destroyed (It will be destroyed if the parent\nContainer's <code>autoDestroy</code> is <code>true</code>, or if the remove call was\npassed a truthy second parameter). After calling the\nsuperclass's <code>onRemoved</code>, the <code>ownerCt</code> and the <code>refOwner</code> will not\nbe present.</p>\n"},"placeholderCollapse":{"!type":"fn(direction: ?, animate: ?) -> !this"},"placeholderExpand":{"!type":"fn(animate: ?) -> !this"},"removeBodyCls":{"!type":"fn(cls: ?) -> ?","!doc":"<p>Removes a CSS class from the body element.</p>\n"},"removeUIClsFromElement":{"!type":"fn(cls: ?) -> !this","!doc":"<p>Method which removes a specified UI + <code>uiCls</code> from the components element. The <code>cls</code>\nwhich is added to the element will be: <code>this.baseCls + '-' + ui + uiCls</code>.</p>\n"},"restoreDimension":{"!type":"fn() -> !this"},"restoreHiddenDocked":{"!type":"fn() -> !this"},"setBodyStyle":{"!type":"fn(style: ?, value: ?) -> ?","!doc":"<p>Sets the body style according to the passed parameters.</p>\n"},"setBorder":{"!type":"fn(border: ?, targetEl: ?) -> !this"},"setCollapsible":{"!type":"fn(collapsible: ?) -> !this"},"setHiddenDocked":{"!type":"fn() -> !this"},"setUI":{"!type":"fn(ui: ?) -> !this","!doc":"<p>Sets the UI for the component. This will remove any existing UIs on the component. It will also loop through any\n<code>uiCls</code> set on the component and rename them so they include the new UI.</p>\n"},"slideOutFloatedPanelBegin":{"!type":"fn() -> !this","!doc":"<p>This method begins the slide out of the floated panel.</p>\n"},"slideOutFloatedPanelEnd":{"!type":"fn(suppressEvents: ?) -> !this","!doc":"<p>This method cleans up after the slide out of the floated panel.</p>\n"},"toggleCollapse":{"!type":"fn() -> ?","!doc":"<p>Shortcut for performing an <a href=\"#!/api/Ext.panel.Panel-method-expand\" rel=\"Ext.panel.Panel-method-expand\" class=\"docClass\">expand</a> or <a href=\"#!/api/Ext.panel.Panel-method-collapse\" rel=\"Ext.panel.Panel-method-collapse\" class=\"docClass\">collapse</a> based on the current state of the panel.</p>\n"},"unghost":{"!type":"fn(show: ?, matchPosition: ?, focus: ?) -> !this"},"updateCollapseTool":{"!type":"fn() -> !this"},"updateGlyph":{"!type":"fn(glyph: ?, oldGlyph: ?) -> !this"},"updateHeader":{"!type":"fn(force: ?) -> !this","!doc":"<p>Create, hide, or show the header component as appropriate based on the current config.</p>\n"},"updateHeaderPosition":{"!type":"fn(position: ?) -> !this"},"updateIcon":{"!type":"fn(icon: ?, oldIcon: ?) -> !this"},"updateIconAlign":{"!type":"fn(align: ?) -> !this"},"updateIconCls":{"!type":"fn(iconCls: ?) -> !this","!doc":"<p>Set the iconCls for the panel's header. See <a href=\"#!/api/Ext.panel.Header-cfg-iconCls\" rel=\"Ext.panel.Header-cfg-iconCls\" class=\"docClass\">Ext.panel.Header.iconCls</a>. It will fire the\n<a href=\"#!/api/Ext.panel.Panel-event-iconclschange\" rel=\"Ext.panel.Panel-event-iconclschange\" class=\"docClass\">iconclschange</a> event after completion.</p>\n"},"updateTitle":{"!type":"fn(title: ?, oldTitle: ?) -> !this"},"updateTitleAlign":{"!type":"fn(align: ?) -> !this"},"updateTitleRotation":{"!type":"fn(rotation: ?) -> !this"},"beforeclose":{"!type":"fn(panel: ?, eOpts: ?)","!doc":"<p>Fires before the user closes the panel. Return false from any listener to stop the close event being\nfired</p>\n"},"beforecollapse":{"!type":"fn(p: ?, direction: ?, animate: ?, eOpts: ?)","!doc":"<p>Fires before this panel is collapsed. Return false to prevent the collapse.</p>\n"},"beforeexpand":{"!type":"fn(p: ?, animate: ?, eOpts: ?)","!doc":"<p>Fires before this panel is expanded. Return false to prevent the expand.</p>\n"},"float":{"!type":"fn(eOpts: ?)","!doc":"<p>Fires after a collapsed Panel has been \"floated\" by clicking on\nit's header. Only applicable when the Panel is an item in a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Border Layout</a>.</p>\n"},"glyphchange":{"!type":"fn(this: ?, newGlyph: ?, oldGlyph: ?, eOpts: ?)","!doc":"<p>Fired when the Panel glyph has been changed by the setGlyph method.</p>\n"},"iconchange":{"!type":"fn(p: ?, newIcon: ?, oldIcon: ?, eOpts: ?)","!doc":"<p>Fires after the Panel icon has been set or changed.</p>\n"},"iconclschange":{"!type":"fn(p: ?, newIconCls: ?, oldIconCls: ?, eOpts: ?)","!doc":"<p>Fires after the Panel iconCls has been set or changed.</p>\n"},"titlechange":{"!type":"fn(p: ?, newTitle: ?, oldTitle: ?, eOpts: ?)","!doc":"<p>Fires after the Panel title has been set or changed.</p>\n"},"unfloat":{"!type":"fn(eOpts: ?)","!doc":"<p>Fires after a \"floated\" Panel has returned to it's collapsed state\nas a result of the mouse leaving the Panel. Only applicable when\nthe Panel is an item in a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Border Layout</a>.</p>\n"},"!doc":"<p>Panel is a container that has specific functionality and structural components that make it the perfect building\nblock for application-oriented user interfaces.</p>\n\n<p>Panels are, by virtue of their inheritance from <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, capable of being configured with a\n<a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a>, and containing child Components.</p>\n\n<p>When either specifying child <a href=\"#!/api/Ext.panel.Panel-cfg-items\" rel=\"Ext.panel.Panel-cfg-items\" class=\"docClass\">items</a> of a Panel, or dynamically <a href=\"#!/api/Ext.container.Container-method-add\" rel=\"Ext.container.Container-method-add\" class=\"docClass\">adding</a>\nComponents to a Panel, remember to consider how you wish the Panel to arrange those child elements, and whether those\nchild elements need to be sized using one of Ext's built-in <code><a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a></code>\nschemes. By default, Panels use the <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Auto</a> scheme. This simply renders child\ncomponents, appending them one after the other inside the Container, and <strong>does not apply any sizing</strong> at all.</p>\n\n<p><p><img src=\"\" alt=\"Panel components\" width=\"\" height=\"\"></p></p>\n\n<p>A Panel may also contain <a href=\"#!/api/Ext.panel.Panel-cfg-bbar\" rel=\"Ext.panel.Panel-cfg-bbar\" class=\"docClass\">bottom</a> and <a href=\"#!/api/Ext.panel.Panel-cfg-tbar\" rel=\"Ext.panel.Panel-cfg-tbar\" class=\"docClass\">top</a> toolbars, along with separate <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">header</a>, <a href=\"#!/api/Ext.panel.Panel-cfg-fbar\" rel=\"Ext.panel.Panel-cfg-fbar\" class=\"docClass\">footer</a> and body sections.</p>\n\n<p>Panel also provides built-in <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible, expandable</a> and <a href=\"#!/api/Ext.panel.Panel-cfg-closable\" rel=\"Ext.panel.Panel-cfg-closable\" class=\"docClass\">closable</a> behavior. Panels can\nbe easily dropped into any <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a> or layout, and the layout and rendering pipeline\nis <a href=\"#!/api/Ext.container.Container-method-add\" rel=\"Ext.container.Container-method-add\" class=\"docClass\">completely managed by the framework</a>.</p>\n\n<p><strong>Note:</strong> By default, the <code><a href=\"#!/api/Ext.panel.Panel-cfg-closable\" rel=\"Ext.panel.Panel-cfg-closable\" class=\"docClass\">close</a></code> header tool <em>destroys</em> the Panel resulting in removal of the\nPanel and the destruction of any descendant Components. This makes the Panel object, and all its descendants\n<strong>unusable</strong>. To enable the close tool to simply <em>hide</em> a Panel for later re-use, configure the Panel with\n<code><a href=\"#!/api/Ext.panel.Panel-cfg-closeAction\" rel=\"Ext.panel.Panel-cfg-closeAction\" class=\"docClass\">closeAction</a>: 'hide'</code>.</p>\n\n<p>Usually, Panels are used as constituents within an application, in which case, they would be used as child items of\nContainers, and would themselves use Ext.Components as child <a href=\"#!/api/Ext.panel.Panel-cfg-items\" rel=\"Ext.panel.Panel-cfg-items\" class=\"docClass\">items</a>. However to illustrate simply rendering a\nPanel into the document, here's how to do it:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Hello',\n    width: 200,\n    html: '&lt;p&gt;World!&lt;/p&gt;',\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>A more realistic scenario is a Panel created to house input fields which will not be rendered, but used as a\nconstituent part of a Container:</p>\n\n<pre class='inline-example '><code>var filterPanel = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    bodyPadding: 5,  // Don't want content to crunch against the borders\n    width: 300,\n    title: 'Filters',\n    items: [{\n        xtype: 'datefield',\n        fieldLabel: 'Start date'\n    }, {\n        xtype: 'datefield',\n        fieldLabel: 'End date'\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>Note that the Panel above is configured to render into the document and assigned a size. In a real world scenario,\nthe Panel will often be added inside a Container which will use a <a href=\"#!/api/Ext.panel.Panel-cfg-layout\" rel=\"Ext.panel.Panel-cfg-layout\" class=\"docClass\">layout</a> to render, size and position its\nchild Components.</p>\n\n<p>Panels will often use specific <a href=\"#!/api/Ext.panel.Panel-cfg-layout\" rel=\"Ext.panel.Panel-cfg-layout\" class=\"docClass\">layout</a>s to provide an application with shape and structure by containing and\narranging child Components:</p>\n\n<pre class='inline-example '><code>var resultsPanel = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Results',\n    width: 600,\n    height: 400,\n    renderTo: Ext.getBody(),\n    layout: {\n        type: 'vbox',       // Arrange child items vertically\n        align: 'stretch',    // Each takes up full width\n        padding: 5\n    },\n    items: [{               // Results grid specified as a config object with an xtype of 'grid'\n        xtype: 'grid',\n        columns: [{header: 'Column One'}],            // One header just for show. There's no data,\n        store: Ext.create('Ext.data.ArrayStore', {}), // A dummy empty data store\n        flex: 1                                       // Use 1/3 of Container's height (hint to Box layout)\n    }, {\n        xtype: 'splitter'   // A splitter between the two child items\n    }, {                    // Details Panel specified as a config object (no xtype defaults to 'panel').\n        title: 'Details',\n        bodyPadding: 5,\n        items: [{\n            fieldLabel: 'Data item',\n            xtype: 'textfield'\n        }], // An array of form fields\n        flex: 2             // Use 2/3 of Container's height (hint to Box layout)\n    }]\n});\n</code></pre>\n\n<p>The example illustrates one possible method of displaying search results. The Panel contains a grid with the\nresulting data arranged in rows. Each selected row may be displayed in detail in the Panel below. The <a href=\"#!/api/Ext.layout.container.VBox\" rel=\"Ext.layout.container.VBox\" class=\"docClass\">vbox</a> layout is used to arrange the two vertically. It is configured to stretch child items\nhorizontally to full width. Child items may either be configured with a numeric height, or with a <code>flex</code> value to\ndistribute available space proportionately.</p>\n\n<p>This Panel itself may be a child item of, for exaple, a <a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a> which will size its child items to fit\nwithin its content area.</p>\n\n<p>Using these techniques, as long as the <strong>layout</strong> is chosen and configured correctly, an application may have any\nlevel of nested containment, all dynamically sized according to configuration, the user's preference and available\nbrowser size.</p>\n","!type":"fn(config: ?)"},"Table":{"prototype":{"allowDeselect":{"!type":"bool","!doc":"<p>True to allow deselecting a record. This config is forwarded to <a href=\"#!/api/Ext.selection.Model-cfg-allowDeselect\" rel=\"Ext.selection.Model-cfg-allowDeselect\" class=\"docClass\">Ext.selection.Model.allowDeselect</a>.</p>\n"},"autoLoad":{"!type":"bool","!doc":"<p>Use <code>true</code> to load the store as soon as this component is fully constructed. It is\nbest to initiate the store load this way to allow this component and potentially\nits plugins (such as <code><a href=\"#!/api/Ext.grid.filters.Filters\" rel=\"Ext.grid.filters.Filters\" class=\"docClass\">Ext.grid.filters.Filters</a></code> to be ready to load.</p>\n"},"bufferedRenderer":{"!type":"bool","!doc":"<p><code>false</code> to disable buffered rendering. See Ext.grid.plugin.BufferedRenderer.</p>\n"},"columnLines":{"!type":"bool","!doc":"<p>Adds column line styling</p>\n"},"deferRowRender":{"!type":"bool","!doc":"<p>Configure as <code>true</code> to enable deferred row rendering.</p>\n\n<p>This allows the View to execute a refresh quickly, with the update of the row structure deferred so\nthat layouts with GridPanels appear, and lay out more quickly.</p>\n"},"disableSelection":{"!type":"bool","!doc":"<p>True to disable selection model.</p>\n"},"emptyText":{"!type":"string","!doc":"<p>Default text (html tags are accepted) to display in the Panel body when the Store\nis empty. When specified, and the Store is empty, the text will be rendered inside a DIV with the CSS class \"x-grid-empty\".</p>\n"},"enableColumnHide":{"!type":"bool","!doc":"<p>False to disable column hiding within this grid.</p>\n"},"enableColumnMove":{"!type":"bool","!doc":"<p>False to disable column dragging within this grid.</p>\n"},"enableColumnResize":{"!type":"bool","!doc":"<p>False to disable column resizing within this grid.</p>\n"},"enableLocking":{"!type":"bool","!doc":"<p>Configure as <code>true</code> to enable locking support for this grid. Alternatively, locking will also be automatically\nenabled if any of the columns in the <a href=\"#!/api/Ext.panel.Table-cfg-columns\" rel=\"Ext.panel.Table-cfg-columns\" class=\"docClass\">columns</a> configuration contain a <a href=\"#!/api/Ext.grid.column.Column-cfg-locked\" rel=\"Ext.grid.column.Column-cfg-locked\" class=\"docClass\">locked</a> config option.</p>\n\n<p>A locking grid is processed in a special way. The configuration options are cloned and <em>two</em> grids are created to be the locked (left) side\nand the normal (right) side. This Panel becomes merely a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">container</a> which arranges both in an <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">HBox</a> layout.</p>\n\n<p><a href=\"#!/api/Ext.panel.Table-cfg-plugins\" rel=\"Ext.panel.Table-cfg-plugins\" class=\"docClass\">Plugins</a> may be targeted at either locked, or unlocked grid, or, both, in which case the plugin is cloned and used on both sides.</p>\n\n<p>Plugins may also be targeted at the containing locking Panel.</p>\n\n<p>This is configured by specifying a <code>lockableScope</code> property in your plugin which may have the following values:</p>\n\n<ul>\n<li><code>\"both\"</code> (the default) - The plugin is added to both grids</li>\n<li><code>\"top\"</code> - The plugin is added to the containing Panel</li>\n<li><code>\"locked\"</code> - The plugin is added to the locked (left) grid</li>\n<li><code>\"normal\"</code> - The plugin is added to the normal (right) grid</li>\n</ul>\n\n\n<p>If <code>both</code> is specified, then each copy of the plugin gains a property <code>lockingPartner</code> which references its sibling on the other side so that they\ncan synchronize operations is necessary.</p>\n\n<p><a href=\"#!/api/Ext.panel.Table-cfg-features\" rel=\"Ext.panel.Table-cfg-features\" class=\"docClass\">Features</a> may also be configured with <code>lockableScope</code> and may target the locked grid, the normal grid or both grids. Features\nalso get a <code>lockingPartner</code> reference injected.</p>\n"},"features":{"!doc":"<p>An array of grid Features to be added to this grid. Can also be just a single feature instead of array.</p>\n\n<p>Features config behaves much like <a href=\"#!/api/Ext.panel.Table-cfg-plugins\" rel=\"Ext.panel.Table-cfg-plugins\" class=\"docClass\">plugins</a>.\nA feature can be added by either directly referencing the instance:</p>\n\n<pre><code>features: [Ext.create('<a href=\"#!/api/Ext.grid.feature.GroupingSummary\" rel=\"Ext.grid.feature.GroupingSummary\" class=\"docClass\">Ext.grid.feature.GroupingSummary</a>', {groupHeaderTpl: 'Subject: {name}'})],\n</code></pre>\n\n<p>By using config object with ftype:</p>\n\n<pre><code>features: [{ftype: 'groupingsummary', groupHeaderTpl: 'Subject: {name}'}],\n</code></pre>\n\n<p>Or with just a ftype:</p>\n\n<pre><code>features: ['grouping', 'groupingsummary'],\n</code></pre>\n\n<p>See <a href=\"#!/api/Ext.enums.Feature\" rel=\"Ext.enums.Feature\" class=\"docClass\">Ext.enums.Feature</a> for list of all ftypes.</p>\n"},"forceFit":{"!type":"bool","!doc":"<p>True to force the columns to fit into the available width. Headers are first sized according to configuration,\nwhether that be a specific width, or flex. Then they are all proportionally changed in width so that the entire\ncontent width is used. For more accurate control, it is more optimal to specify a flex setting on the columns\nthat are to be stretched &amp; explicit widths on columns that are not.</p>\n"},"hideHeaders":{"!type":"bool","!doc":"<p>True to hide column headers.</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.panel.Table-cfg-items\" rel=\"Ext.panel.Table-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"leadingBufferZone":{"!type":"number","!doc":"<p>TableViews are buffer rendered in 5.x which means that only the visible subset of data rows\nare rendered into the DOM. These are removed and added as scrolling demands.</p>\n\n<p>This configures the number of extra rows to render on the leading side of scrolling\n<strong>outside the <a href=\"#!/api/Ext.panel.Table-cfg-numFromEdge\" rel=\"Ext.panel.Table-cfg-numFromEdge\" class=\"docClass\">numFromEdge</a></strong> buffer as scrolling proceeds.</p>\n"},"multiColumnSort":{"!type":"bool","!doc":"<p>Configure as <code>true</code> to have columns remember their sorted state after other columns have been clicked upon to sort.</p>\n\n<p>As subsequent columns are clicked upon, they become the new primary sort key.</p>\n\n<p>The maximum number of sorters allowed in a Store is configurable via its underlying data collection. See Ext.util.Collection.multiSortLimit</p>\n"},"multiSelect":{"!type":"bool","!doc":"<p>True to enable 'MULTI' selection mode on selection model.</p>\n"},"numFromEdge":{"!type":"number","!doc":"<p>This configures the zone which causes new rows to be appended to the view. As soon as the edge\nof the rendered grid is this number of rows from the edge of the viewport, the view is moved.</p>\n"},"reserveScrollbar":{"!type":"bool","!doc":"<p>Set this to true to <strong>always</strong> leave a scrollbar sized space at the end of the grid content when\nfitting content into the width of the grid.</p>\n\n<p>If the grid's record count fluctuates enough to hide and show the scrollbar regularly, this setting\navoids the multiple layouts associated with switching from scrollbar present to scrollbar not present.</p>\n"},"scroll":{"!doc":"<p>Scrollers configuration. Valid values are 'both', 'horizontal' or 'vertical'.\nTrue implies 'both'. False implies 'none'.</p>\n"},"sealedColumns":{"!type":"bool","!doc":"<p>True to constrain column dragging so that a column cannot be dragged in or out of it's\ncurrent group. Only relevant while <a href=\"#!/api/Ext.panel.Table-cfg-enableColumnMove\" rel=\"Ext.panel.Table-cfg-enableColumnMove\" class=\"docClass\">enableColumnMove</a> is enabled.</p>\n"},"selModel":{"!doc":"<p>A <a href=\"#!/api/Ext.selection.Model\" rel=\"Ext.selection.Model\" class=\"docClass\">selection model</a> instance or config object.  In latter case the <a href=\"#!/api/Ext.panel.Table-cfg-selType\" rel=\"Ext.panel.Table-cfg-selType\" class=\"docClass\">selType</a>\nconfig option determines to which type of selection model this config is applied.</p>\n"},"selType":{"!type":"string","!doc":"<p>An xtype of selection model to use. This is used to create selection model if just\na config object or nothing at all given in <a href=\"#!/api/Ext.panel.Table-cfg-selModel\" rel=\"Ext.panel.Table-cfg-selModel\" class=\"docClass\">selModel</a> config.</p>\n"},"simpleSelect":{"!type":"bool","!doc":"<p>True to enable 'SIMPLE' selection mode on selection model.</p>\n"},"sortableColumns":{"!type":"bool","!doc":"<p>False to disable column sorting via clicking the header and via the Sorting menu items.</p>\n"},"store":{"!doc":"<p>The Store the grid should use as its data source.</p>\n"},"trailingBufferZone":{"!type":"number","!doc":"<p>TableViews are buffer rendered in 5.x which means that only the visible subset of data rows\nare rendered into the DOM. These are removed and added as scrolling demands.</p>\n\n<p>This configures the number of extra rows to render on the trailing side of scrolling\n<strong>outside the <a href=\"#!/api/Ext.panel.Table-cfg-numFromEdge\" rel=\"Ext.panel.Table-cfg-numFromEdge\" class=\"docClass\">numFromEdge</a></strong> buffer as scrolling proceeds.</p>\n"},"variableRowHeight":{"!type":"bool"},"view":{"!doc":"<p>The <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">Ext.view.Table</a> used by the grid. Use <a href=\"#!/api/Ext.panel.Table-cfg-viewConfig\" rel=\"Ext.panel.Table-cfg-viewConfig\" class=\"docClass\">viewConfig</a> to just supply some config options to\nview (instead of creating an entire View instance).</p>\n"},"viewConfig":{"!doc":"<p>A config object that will be applied to the grid's UI view. Any of the config options available for\n<a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">Ext.view.Table</a> can be specified here. This option is ignored if <a href=\"#!/api/Ext.panel.Table-cfg-view\" rel=\"Ext.panel.Table-cfg-view\" class=\"docClass\">view</a> is specified.</p>\n"},"colLinesCls":{"!type":"string"},"defaultBindProperty":{"!type":"string"},"emptyCls":{"!type":"string"},"extraBaseCls":{"!type":"string"},"extraBodyCls":{"!type":"string"},"hasView":{"!type":"bool","!doc":"<p>True to indicate that a view has been injected into the panel.</p>\n"},"hiddenHeaderCls":{"!type":"string"},"hiddenHeaderCtCls":{"!type":"string"},"noRowLinesCls":{"!type":"string"},"optimizedColumnMove":{"!type":"bool","!doc":"<p>If you are writing a grid plugin or a {<a href=\"#!/api/Ext.grid.feature.Feature\" rel=\"Ext.grid.feature.Feature\" class=\"docClass\">Ext.grid.feature.Feature</a> Feature} which creates a column-based structure which\nneeds a view refresh when columns are moved, then set this property in the grid.</p>\n\n<p>An example is the built in <a href=\"#!/api/Ext.grid.feature.AbstractSummary\" rel=\"Ext.grid.feature.AbstractSummary\" class=\"docClass\">Summary</a> Feature. This creates summary rows, and the\nsummary columns must be in the same order as the data columns. This plugin sets the <code>optimizedColumnMove</code> to `false.</p>\n"},"ownerGrid":{"!doc":"<p>A reference to the top-level owning grid component. This is <code>null</code> if this grid is\nnot part of a locked grid arrangement.</p>\n"},"resizeMarkerCls":{"!type":"string"},"rowLinesCls":{"!type":"string"},"scrollerOwner":{"!type":"bool","!doc":"<p>private property used to determine where to go down to find views\nthis is here to support locking.</p>\n"},"afterCollapse":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Panel is Collapsed.</p>\n"},"afterExpand":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Panel is Expanded.</p>\n"},"applyState":{"!type":"fn(state: ?) -> !this","!doc":"<p>Applies the state to the object. This should be overridden in subclasses to do\nmore complex state operations. By default it applies the state properties onto\nthe current object.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"bindStore":{"!type":"fn(store: ?, initial: ?) -> !this"},"delayScroll":{"!type":"fn() -> !this"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroys the Component.</p>\n"},"getColumnManager":{"!type":"fn() -> !this"},"getEditorParent":{"!type":"fn() -> !this"},"getLhsMarker":{"!type":"fn() -> !this","!doc":"<p>Gets left hand side marker for header resizing.</p>\n"},"getRhsMarker":{"!type":"fn() -> !this","!doc":"<p>Gets right hand side marker for header resizing.</p>\n"},"getScrollTarget":{"!type":"fn() -> !this"},"getScrollerOwner":{"!type":"fn() -> !this"},"getSelection":{"!type":"fn() -> ?","!doc":"<p>Returns the grid's selection. See <code><a href=\"#!/api/Ext.selection.Model-method-getSelection\" rel=\"Ext.selection.Model-method-getSelection\" class=\"docClass\">Ext.selection.Model.getSelection</a></code>.</p>\n"},"getSelectionModel":{"!type":"fn() -> ?","!doc":"<p>Returns the selection model being used and creates it via the configuration if it has not been created already.</p>\n"},"getState":{"!type":"fn() -> ?","!doc":"<p>The supplied default state gathering method for the Component class.</p>\n\n<p>This method returns dimension settings such as <code>flex</code>, <code>anchor</code>, <code>width</code> and <code>height</code> along with <code>collapsed</code>\nstate.</p>\n\n<p>Subclasses which implement more complex state should call the superclass's implementation, and apply their state\nto the result if this basic state is to be saved.</p>\n\n<p>Note that Component state will only be saved if the Component has a <a href=\"#!/api/Ext.panel.Table-cfg-stateId\" rel=\"Ext.panel.Table-cfg-stateId\" class=\"docClass\">stateId</a> and there as a StateProvider\nconfigured for the document.</p>\n"},"getStore":{"!type":"fn() -> ?","!doc":"<p>Returns the store associated with this Panel.</p>\n"},"getTopLevelColumnManager":{"!type":"fn() -> !this"},"getTopLevelVisibleColumnManager":{"!type":"fn() -> !this"},"getView":{"!type":"fn() -> ?","!doc":"<p>Gets the view for this panel.</p>\n"},"getVisibleColumnManager":{"!type":"fn() -> !this"},"hasLockedColumns":{"!type":"fn(columns: ?) -> !this","!doc":"<p>Private. Determine if there are any columns with a locked configuration option</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onHeaderHide":{"!type":"fn(headerCt: ?, header: ?) -> !this","!doc":"<p>Section onHeaderHide is invoked after view.</p>\n"},"onHeaderMove":{"!type":"fn(headerCt: ?, header: ?, colsToMove: ?, fromIdx: ?, toIdx: ?) -> !this","!doc":"<p>Update the view when a header moves</p>\n"},"onHeaderResize":{"!type":"fn() -> !this"},"onHeaderShow":{"!type":"fn(headerCt: ?, header: ?) -> !this"},"onHeadersChanged":{"!type":"fn(headerCt: ?, header: ?) -> !this","!doc":"<p>To be triggered on add/remove/move for a leaf header</p>\n"},"onHorizontalScroll":{"!type":"fn(view: ?) -> !this"},"onRestoreHorzScroll":{"!type":"fn() -> !this","!doc":"<p>Tracks when things happen to the view and preserves the horizontal scroll position.</p>\n"},"onStoreLoad":{"!type":"fn() -> !this","!doc":"<p>template method meant to be overriden</p>\n"},"onViewReady":{"!type":"fn() -> !this","!doc":"<p>Fires the TablePanel's viewready event when the view declares that its internal DOM is ready</p>\n"},"processEvent":{"!type":"fn(type: ?, view: ?, cell: ?, recordIndex: ?, cellIndex: ?, e: ?) -> !this","!doc":"<p>Processes UI events from the view. Propagates them to whatever internal Components need to process them.</p>\n"},"relayHeaderCtEvents":{"!type":"fn(headerCt: ?) -> !this"},"restoreScrollPos":{"!type":"fn() -> !this"},"saveScrollPos":{"!type":"fn() -> !this"},"scrollByDeltaX":{"!type":"fn(xDelta: ?, animate: ?) -> !this"},"scrollByDeltaY":{"!type":"fn(yDelta: ?, animate: ?) -> !this"},"setAutoScroll":{"!type":"fn() -> !this","!doc":"<p>autoScroll is never valid for all classes which extend TablePanel.</p>\n"},"setColumns":{"!type":"fn(columns: ?) -> !this"},"setSelection":{"!type":"fn(records: ?, keepExisting?: ?, suppressEvent?: ?) -> !this","!doc":"<p>Sets the grid's selection. See <code><a href=\"#!/api/Ext.selection.Model-method-select\" rel=\"Ext.selection.Model-method-select\" class=\"docClass\">Ext.selection.Model.select</a></code>.</p>\n"},"setStore":{"!type":"fn(store: ?) -> !this"},"syncHorizontalScroll":{"!type":"fn(target: ?, setBody: ?) -> !this"},"unbindStore":{"!type":"fn() -> !this"},"beforecellclick":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell click is processed. Return false to cancel the default action.</p>\n"},"beforecellcontextmenu":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell right click is processed. Return false to cancel the default action.</p>\n"},"beforecelldblclick":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell double click is processed. Return false to cancel the default action.</p>\n"},"beforecellkeydown":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell key down is processed. Return false to cancel the default action.</p>\n"},"beforecellmousedown":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell mouse down is processed. Return false to cancel the default action.</p>\n"},"beforecellmouseup":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell mouse up is processed. Return false to cancel the default action.</p>\n"},"beforecontainerclick":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the click event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainercontextmenu":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the contextmenu event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainerdblclick":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the dblclick event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainerkeydown":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the keydown event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainermousedown":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mousedown event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainermouseout":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseout event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainermouseover":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseover event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainermouseup":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseup event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforedeselect":{"!type":"fn(this: ?, record: ?, index: ?, eOpts: ?)","!doc":"<p>Fired before a record is deselected. If any listener returns false, the\ndeselection is cancelled.</p>\n"},"beforeitemclick":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the click event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemcontextmenu":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the contextmenu event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemdblclick":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the dblclick event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemkeydown":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the keydown event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemmousedown":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mousedown event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemmouseenter":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseenter event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemmouseleave":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseleave event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemmouseup":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseup event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeselect":{"!type":"fn(this: ?, record: ?, index: ?, eOpts: ?)","!doc":"<p>Fired before a record is selected. If any listener returns false, the\nselection is cancelled.</p>\n"},"cellclick":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is clicked.</p>\n"},"cellcontextmenu":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is right clicked.</p>\n"},"celldblclick":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is double clicked.</p>\n"},"cellkeydown":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the keydown event is captured on the cell.</p>\n"},"cellmousedown":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the mousedown event is captured on the cell.</p>\n"},"cellmouseup":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the mouseup event is captured on the cell.</p>\n"},"columnhide":{"!type":"fn(ct: ?, column: ?, eOpts: ?)"},"columnmove":{"!type":"fn(ct: ?, column: ?, fromIdx: ?, toIdx: ?, eOpts: ?)"},"columnresize":{"!type":"fn(ct: ?, column: ?, width: ?, eOpts: ?)"},"columnschanged":{"!type":"fn(ct: ?, eOpts: ?)","!doc":"<p>Fired after the columns change in any way, when a column has been hidden or shown, or when a column\nis added to or removed from this header container.</p>\n"},"columnshow":{"!type":"fn(ct: ?, column: ?, eOpts: ?)"},"containerclick":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the container is clicked.</p>\n"},"containercontextmenu":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the container is right clicked.</p>\n"},"containerdblclick":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the container is double clicked.</p>\n"},"containerkeydown":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when a key is pressed while the container is focused, and no item is currently selected.</p>\n"},"containermousedown":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when there is a mousedown on the container</p>\n"},"containermouseout":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when you move the mouse out of the container.</p>\n"},"containermouseover":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when you move the mouse over the container.</p>\n"},"containermouseup":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when there is a mouseup on the container</p>\n"},"deselect":{"!type":"fn(this: ?, record: ?, index: ?, eOpts: ?)","!doc":"<p>Fired after a record is deselected</p>\n"},"filterchange":{"!type":"fn(eOpts: ?)"},"groupchange":{"!type":"fn(eOpts: ?)"},"headerclick":{"!type":"fn(ct: ?, column: ?, e: ?, t: ?, eOpts: ?)"},"headercontextmenu":{"!type":"fn(ct: ?, column: ?, e: ?, t: ?, eOpts: ?)"},"headertriggerclick":{"!type":"fn(ct: ?, column: ?, e: ?, t: ?, eOpts: ?)"},"itemclick":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when an item is clicked.</p>\n"},"itemcontextmenu":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when an item is right clicked.</p>\n"},"itemdblclick":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when an item is double clicked.</p>\n"},"itemkeydown":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when a key is pressed while an item is currently selected.</p>\n"},"itemmousedown":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when there is a mouse down on an item</p>\n"},"itemmouseenter":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse enters an item.</p>\n"},"itemmouseleave":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse leaves an item.</p>\n"},"itemmouseup":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when there is a mouse up on an item</p>\n"},"rowclick":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is clicked.</p>\n"},"rowcontextmenu":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is right clicked.</p>\n"},"rowdblclick":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is double clicked.</p>\n"},"rowkeydown":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the keydown event is captured on the cell.</p>\n"},"rowmousedown":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the mousedown event is captured on the cell.</p>\n"},"rowmouseup":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the mouseup event is captured on the cell.</p>\n"},"select":{"!type":"fn(this: ?, record: ?, index: ?, eOpts: ?)","!doc":"<p>Fired after a record is selected</p>\n"},"selectionchange":{"!type":"fn(this: ?, selected: ?, eOpts: ?)","!doc":"<p>Fired after a selection change has occurred</p>\n"},"sortchange":{"!type":"fn(ct: ?, column: ?, direction: ?, eOpts: ?)"},"viewready":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when the grid view is available (use this for selecting a default row).</p>\n"},"columns":{"!doc":"<p>An array of <a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">column</a> definition objects which define all columns that appear in this\ngrid. Each column definition provides the header text for the column, and a definition of where the data for that\ncolumn comes from.</p>\n\n<p>This can also be a configuration object for a <a href=\"#!/api/Ext.grid.header.Container\" rel=\"Ext.grid.header.Container\" class=\"docClass\">HeaderContainer</a> which may override\ncertain default configurations if necessary. For example, the special layout may be overridden to use a simpler\nlayout, or one can set default values shared by all columns:</p>\n\n<pre><code>columns: {\n    items: [\n        {\n            text: \"Column A\",\n            dataIndex: \"field_A\"\n        },{\n            text: \"Column B\",\n            dataIndex: \"field_B\"\n        }, \n        ...\n    ],\n    defaults: {\n        flex: 1\n    }\n}\n</code></pre>\n"},"rowLines":{"!type":"bool","!doc":"<p>Adds row line styling</p>\n"},"viewType":{"!type":"string","!doc":"<p>An xtype of view to use. This is automatically set to 'gridview' by <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Grid</a>\nand to 'treeview' by <a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">Tree</a>.</p>\n"},"reconfigure":{"!type":"fn(store: ?, columns: ?) -> !this","!doc":"<p>documented on GridPanel</p>\n"}},"!doc":"<p>This class is the base class for both <a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">TreePanel</a> and\n<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">GridPanel</a>.</p>\n\n<p>TablePanel aggregates:</p>\n\n<ul>\n<li>a Selection Model</li>\n<li>a View</li>\n<li>a Store</li>\n<li><a href=\"#!/api/Ext.grid.header.Container\" rel=\"Ext.grid.header.Container\" class=\"docClass\">Ext.grid.header.Container</a></li>\n</ul>\n\n","!type":"fn(config: ?)"},"Bar":{"!doc":"<p>Abstract base class for common functionality shared between <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Ext.panel.Header</a>\nand <a href=\"#!/api/Ext.tab.Bar\" rel=\"Ext.tab.Bar\" class=\"docClass\">Ext.tab.Bar</a></p>\n","!type":"fn(config: ?)","prototype":{"_verticalSides":{},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"vertical":{"!type":"bool"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"onAdded":{"!type":"fn(container: ?, pos: ?, instanced: ?) -> !this","!doc":"<p>Method to manage awareness of when components are added to their\nrespective Container, firing an <a href=\"#!/api/Ext.panel.Bar-event-added\" rel=\"Ext.panel.Bar-event-added\" class=\"docClass\">added</a> event. References are\nestablished at add time rather than at render time.</p>\n\n<p>Allows addition of behavior when a Component is added to a\nContainer. At this stage, the Component is in the parent\nContainer's collection of child items. After calling the\nsuperclass's <code>onAdded</code>, the <code>ownerCt</code> reference will be present,\nand if configured with a ref, the <code>refOwner</code> will be set.</p>\n"},"onRemoved":{"!type":"fn(destroying: ?) -> !this","!doc":"<p>Method to manage awareness of when components are removed from their\nrespective Container, firing a <a href=\"#!/api/Ext.panel.Bar-event-removed\" rel=\"Ext.panel.Bar-event-removed\" class=\"docClass\">removed</a> event. References are properly\ncleaned up after removing a component from its owning container.</p>\n\n<p>Allows addition of behavior when a Component is removed from\nits parent Container. At this stage, the Component has been\nremoved from its parent Container's collection of child items,\nbut has not been destroyed (It will be destroyed if the parent\nContainer's <code>autoDestroy</code> is <code>true</code>, or if the remove call was\npassed a truthy second parameter). After calling the\nsuperclass's <code>onRemoved</code>, the <code>ownerCt</code> and the <code>refOwner</code> will not\nbe present.</p>\n"},"setDock":{"!type":"fn(dock: ?) -> ?","!doc":"<p>Sets the dock position of this component in its parent panel. Note that this only has effect if this item is part\nof the <code>dockedItems</code> collection of a parent that has a DockLayout (note that any Panel has a DockLayout by default)</p>\n"}}},"DD":{"!doc":"<p>DD implementation for Panels.</p>\n","!type":"fn(panel: ?, cfg: ?)","prototype":{"afterInvalidDrop":{"!type":"fn(e: ?, id: ?)","!doc":"<p>An empty function by default, but provided so that you can perform a custom action\nafter an invalid drop has occurred by providing an implementation.</p>\n"},"autoOffset":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Sets the pointer offset to the distance between the linked element's top\nleft corner and the location the element was clicked.</p>\n"},"b4MouseDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>overrides <a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a></p>\n"},"b4StartDrag":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>overrides <a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a></p>\n"},"createFrame":{"!type":"fn() -> !this","!doc":"<p>Creates the proxy element if it does not yet exist</p>\n"},"endDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>overrides <a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a>\nBy default we try to move the element to the last location of the frame.\nThis is so that the default behavior mirrors that of <a href=\"#!/api/Ext.dd.DD\" rel=\"Ext.dd.DD\" class=\"docClass\">Ext.dd.DD</a>.</p>\n"},"getDragEl":{"!type":"fn(e: ?) -> ?","!doc":"<p>Returns a reference to the actual element to drag.  By default this is\nthe same as the html element, but it can be assigned to another\nelement. An example of this can be found in <a href=\"#!/api/Ext.dd.DDProxy\" rel=\"Ext.dd.DDProxy\" class=\"docClass\">Ext.dd.DDProxy</a></p>\n"},"onInitDrag":{"!type":"fn(x: ?, y: ?) -> !this"},"onInvalidDrop":{"!type":"fn(target: ?, e: ?, id: ?) -> !this","!doc":"<p>Override this, we don't want to repair on an \"invalid\" drop, the panel\nshould main it's position</p>\n"},"setupEl":{"!type":"fn(panel: ?) -> !this"},"showFrame":{"!type":"fn(iPageX: ?, iPageY: ?) -> !this","!doc":"<p>Resizes the drag frame to the dimensions of the clicked object, positions\nit over the object, and finally displays it</p>\n"},"startDrag":{"!type":"fn() -> !this","!doc":"<p>Abstract method called after a drag/drop object is clicked\nand the drag or mousedown time thresholds have beeen met.</p>\n"}}},"Header":{"!doc":"<p>Simple header class which is used for on <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a> and <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>.</p>\n","!type":"fn(config: ?)","prototype":{"defaultType":{"!type":"string","!doc":"<p>The default <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">xtype</a> of child Components to create in this Container when\na child item is specified as a raw configuration object, rather than as an instantiated Component.</p>\n"},"glyph":{"!doc":"<p>A numeric unicode character code to use as the icon for the panel header. The\ndefault font-family for glyphs can be set globally using\n<a href=\"#!/api/Ext-method-setGlyphFontFamily\" rel=\"Ext-method-setGlyphFontFamily\" class=\"docClass\">Ext.setGlyphFontFamily()</a>. Alternatively, this\nconfig option accepts a string with the charCode and font-family separated by the\n<code>@</code> symbol. For example '65@My Font Family'.</p>\n"},"icon":{"!type":"string","!doc":"<p>Path to image for an icon.</p>\n\n<p>There are no default icons that come with Ext JS.</p>\n"},"iconAlign":{"!type":"string","!doc":"<p>For performance reasons we give the following configs their default values on\nthe class body.  This prevents the updaters from running on initialization in the\ndefault configuration scenario</p>\n"},"iconCls":{"!type":"string","!doc":"<p>CSS class for an icon.</p>\n\n<p>There are no default icon classes that come with Ext JS.</p>\n"},"shrinkWrap":{"!doc":"<p>If this property is a number, it is interpreted as follows:</p>\n\n<ul>\n<li>0 (or <code>false</code>): Neither width nor height depend on content.</li>\n<li>1: Width depends on content (shrink wraps), but height does not.</li>\n<li>2 (default): Height depends on content (shrink wraps), but width does not.</li>\n<li>3 (or <code>true</code>): Both width and height depend on content (shrink wrap).</li>\n</ul>\n\n\n<p>In CSS terms, shrink-wrap width is analogous to an inline-block element as opposed\nto a block-level element. Some container layouts always shrink-wrap their children,\neffectively ignoring this property (e.g., <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">Ext.layout.container.HBox</a>,\n<a href=\"#!/api/Ext.layout.container.VBox\" rel=\"Ext.layout.container.VBox\" class=\"docClass\">Ext.layout.container.VBox</a>, <a href=\"#!/api/Ext.layout.component.Dock\" rel=\"Ext.layout.component.Dock\" class=\"docClass\">Ext.layout.component.Dock</a>).</p>\n"},"title":{"!doc":"<p>The title text or config object for the <a href=\"#!/api/Ext.panel.Title\" rel=\"Ext.panel.Title\" class=\"docClass\">Title</a> component.</p>\n"},"titleAlign":{"!type":"string"},"titlePosition":{"!type":"number"},"titleRotation":{"!type":"string"},"weight":{"!type":"number","!doc":"<p>A value to control how Components are laid out in a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Border</a> layout or as docked items.</p>\n\n<p>In a Border layout, this can control how the regions (not the center) region lay out if the west or east take full height\nor if the north or south region take full width. Also look at the <a href=\"#!/api/Ext.layout.container.Border-cfg-regionWeights\" rel=\"Ext.layout.container.Border-cfg-regionWeights\" class=\"docClass\">Ext.layout.container.Border.regionWeights</a> on the Border layout. An example to show how you can\ntake control of this is:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.container.Viewport\" rel=\"Ext.container.Viewport\" class=\"docClass\">Ext.container.Viewport</a>', {\n    layout      : 'border',\n    defaultType : 'panel',\n    items       : [\n        {\n            region : 'north',\n            title  : 'North',\n            height : 100\n        },\n        {\n            region : 'south',\n            title  : 'South',\n            height : 100,\n            weight : -25\n        },\n        {\n            region : 'west',\n            title  : 'West',\n            width  : 200,\n            weight : 15\n        },\n        {\n            region : 'east',\n            title  : 'East',\n            width  : 200\n        },\n        {\n            region : 'center',\n            title  : 'center'\n        }\n    ]\n});\n</code></pre>\n\n<p>If docked items, the weight will order how the items are laid out. Here is an example to put a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Ext.toolbar.Toolbar</a> above\na <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>'s header:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    renderTo    : document.body,\n    width       : 300,\n    height      : 300,\n    title       : 'Panel',\n    html        : 'Panel Body',\n    dockedItems : [\n        {\n            xtype : 'toolbar',\n            items : [\n                {\n                    text : 'Save'\n                }\n            ]\n        },\n        {\n            xtype  : 'toolbar',\n            weight : -10,\n            items  : [\n                {\n                    text : 'Remove'\n                }\n            ]\n        }\n    ]\n});\n</code></pre>\n"},"beforeRenderConfig":{},"headerCls":{"!type":"string","!doc":"<p>a class for styling that is shared between panel and window headers</p>\n"},"indicateDrag":{"!type":"bool"},"isHeader":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Header, or subclass thereof.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"addTool":{"!type":"fn(tool: ?) -> !this","!doc":"<p>Add a tool to the header</p>\n"},"afterLayout":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Container has laid out (and rendered if necessary)\nits child Components.</p>\n"},"applyTitle":{"!type":"fn(title: ?, oldTitle: ?) -> !this"},"applyTitlePosition":{"!type":"fn(position: ?) -> !this"},"beforeLayout":{"!type":"fn() -> !this","!doc":"<p>Occurs before componentLayout is run. In previous releases, this method could\nreturn <code>false</code> to prevent its layout but that is not supported in Ext JS 4.1 or\nhigher. This method is simply a notification of the impending layout to give the\ncomponent a chance to adjust the DOM. Ideally, DOM reads should be avoided at this\ntime to reduce expensive document reflows.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"getTools":{"!type":"fn() -> ?","!doc":"<p>Gets the tools for this header.</p>\n"},"onAdd":{"!type":"fn(component: ?, index: ?) -> !this","!doc":"<p>This method is invoked after a new Component has been added. It\nis passed the Component which has been added. This method may\nbe used to update any internal structure which may depend upon\nthe state of the child items.</p>\n"},"onAdded":{"!type":"fn(container: ?, pos: ?, instanced: ?) -> !this","!doc":"<p>Method to manage awareness of when components are added to their\nrespective Container, firing an <a href=\"#!/api/Ext.panel.Header-event-added\" rel=\"Ext.panel.Header-event-added\" class=\"docClass\">added</a> event. References are\nestablished at add time rather than at render time.</p>\n\n<p>Allows addition of behavior when a Component is added to a\nContainer. At this stage, the Component is in the parent\nContainer's collection of child items. After calling the\nsuperclass's <code>onAdded</code>, the <code>ownerCt</code> reference will be present,\nand if configured with a ref, the <code>refOwner</code> will be set.</p>\n"},"onRemoved":{"!type":"fn(container: ?, pos: ?, instanced: ?) -> !this","!doc":"<p>Method to manage awareness of when components are removed from their\nrespective Container, firing a <a href=\"#!/api/Ext.panel.Header-event-removed\" rel=\"Ext.panel.Header-event-removed\" class=\"docClass\">removed</a> event. References are properly\ncleaned up after removing a component from its owning container.</p>\n\n<p>Allows addition of behavior when a Component is removed from\nits parent Container. At this stage, the Component has been\nremoved from its parent Container's collection of child items,\nbut has not been destroyed (It will be destroyed if the parent\nContainer's <code>autoDestroy</code> is <code>true</code>, or if the remove call was\npassed a truthy second parameter). After calling the\nsuperclass's <code>onRemoved</code>, the <code>ownerCt</code> and the <code>refOwner</code> will not\nbe present.</p>\n"},"setDock":{"!type":"fn(dock: ?) -> ?","!doc":"<p>Sets the dock position of this component in its parent panel. Note that this only has effect if this item is part\nof the <code>dockedItems</code> collection of a parent that has a DockLayout (note that any Panel has a DockLayout by default)</p>\n"},"updateGlyph":{"!type":"fn(glyph: ?) -> !this"},"updateIcon":{"!type":"fn(icon: ?) -> !this"},"updateIconAlign":{"!type":"fn(align: ?, oldAlign: ?) -> !this"},"updateIconCls":{"!type":"fn(cls: ?) -> !this"},"updateTitle":{"!type":"fn(title: ?, oldTitle: ?) -> !this"},"updateTitleAlign":{"!type":"fn(align: ?, oldAlign: ?) -> !this"},"updateTitlePosition":{"!type":"fn(position: ?) -> !this"},"updateTitleRotation":{"!type":"fn(rotation: ?) -> !this"},"click":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the header is clicked. This event will not be fired\nif the click was on a <a href=\"#!/api/Ext.panel.Tool\" rel=\"Ext.panel.Tool\" class=\"docClass\">Ext.panel.Tool</a></p>\n"},"dblclick":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the header is double clicked. This event will not\nbe fired if the click was on a <a href=\"#!/api/Ext.panel.Tool\" rel=\"Ext.panel.Tool\" class=\"docClass\">Ext.panel.Tool</a></p>\n"}}},"Pinnable":{"!doc":"<p>This mixin is applied to panels that want to manage a Pin state and corresponding tool.</p>\n","prototype":{"mixinId":{"!type":"string"},"pinnable":{"!type":"bool"},"pinnedTip":{"!type":"string"},"unpinnedTip":{"!type":"string"},"initPinnable":{"!type":"fn() -> !this"},"isPinned":{"!type":"fn() -> !this"},"setPinned":{"!type":"fn(pinned: ?) -> !this"},"togglePin":{"!type":"fn() -> !this"},"updatePinned":{"!type":"fn(pinned: ?) -> !this"}}},"Proxy":{"!doc":"<p>A custom drag proxy implementation specific to <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>s. This class\nis primarily used internally for the Panel's drag drop implementation, and\nshould never need to be created directly.</p>\n","!type":"fn(panel: ?, config?: ?)","prototype":{"insertProxy":{"!type":"bool","!doc":"<p>True to insert a placeholder proxy element while dragging the panel, false to drag with no proxy.\nMost Panels are not absolute positioned and therefore we need to reserve this space.</p>\n"},"moveOnDrag":{"!type":"bool","!doc":"<p>True to move the panel to the dragged position when dropped</p>\n"},"panel":{},"getEl":{"!type":"fn() -> ?","!doc":"<p>Gets the proxy's element</p>\n"},"getGhost":{"!type":"fn() -> ?","!doc":"<p>Gets the proxy's ghost Panel</p>\n"},"getProxy":{"!type":"fn() -> ?","!doc":"<p>Gets the proxy element. This is the element that represents where the\nPanel was before we started the drag operation.</p>\n"},"hide":{"!type":"fn() -> !this","!doc":"<p>Hides the proxy</p>\n"},"moveProxy":{"!type":"fn(parentNode: ?, before?: ?) -> !this","!doc":"<p>Moves the proxy to a different position in the DOM.  This is typically\ncalled while dragging the Panel to keep the proxy sync'd to the Panel's\nlocation.</p>\n"},"repair":{"!type":"fn(xy: ?, callback: ?, scope: ?) -> !this","!doc":"<p>private</p>\n"},"reset":{"!type":"fn() -> !this"},"setStatus":{"!type":"fn() -> !this","!doc":"<p>private overrides</p>\n"},"show":{"!type":"fn() -> !this","!doc":"<p>Shows the proxy</p>\n"},"stop":{"!type":"fn() -> !this"},"sync":{"!type":"fn() -> !this"},"update":{"!type":"fn() -> !this"}}},"Title":{"!doc":"<p>A basic title component for a Panel Header</p>\n","!type":"fn(config: ?)","prototype":{"autoEl":{"!doc":"<p>A tag name or DomHelper spec used to create the <a href=\"#!/api/Ext.panel.Title-method-getEl\" rel=\"Ext.panel.Title-method-getEl\" class=\"docClass\">Element</a> which will\nencapsulate this Component.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong>'div'</strong>. The more complex Sencha classes use a more\ncomplex DOM structure specified by their own <a href=\"#!/api/Ext.panel.Title-cfg-renderTpl\" rel=\"Ext.panel.Title-cfg-renderTpl\" class=\"docClass\">renderTpl</a>s.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components encapsulated by\ndifferent DOM elements. Example usage:</p>\n\n<pre><code>{\n    xtype: 'component',\n    autoEl: {\n        tag: 'img',\n        src: 'http://www.example.com/example.jpg'\n    }\n}, {\n    xtype: 'component',\n    autoEl: {\n        tag: 'blockquote',\n        html: 'autoEl is cool!'\n    }\n}, {\n    xtype: 'container',\n    autoEl: 'ul',\n    cls: 'ux-unordered-list',\n    items: {\n        xtype: 'component',\n        autoEl: 'li',\n        html: 'First list item'\n    }\n}\n</code></pre>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"glyph":{"!doc":"<p>A numeric unicode character code to use as the icon. The\ndefault font-family for glyphs can be set globally using\n<a href=\"#!/api/Ext-method-setGlyphFontFamily\" rel=\"Ext-method-setGlyphFontFamily\" class=\"docClass\">Ext.setGlyphFontFamily()</a>. Alternatively, this\nconfig option accepts a string with the charCode and font-family separated by the\n<code>@</code> symbol. For example '65@My Font Family'.</p>\n"},"icon":{"!type":"string","!doc":"<p>Path to image for an icon.</p>\n"},"iconAlign":{"!type":"string"},"iconCls":{"!type":"string","!doc":"<p>CSS class for an icon.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.panel.Title-method-getEl\" rel=\"Ext.panel.Title-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.panel.Title-method-getEl\" rel=\"Ext.panel.Title-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.panel.Title-cfg-renderSelectors\" rel=\"Ext.panel.Title-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.panel.Title-cfg-childEls\" rel=\"Ext.panel.Title-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"rotation":{"!type":"number"},"text":{"!type":"string"},"textAlign":{"!type":"string","!doc":"<p>For performance reasons we give the following configs their default values on\nthe class body.  This prevents the updaters from running on initialization in the\ndefault configuration scenario</p>\n"},"_baseIconCls":{"!type":"string"},"_glyphCls":{"!type":"string"},"_iconAlignClasses":{},"_iconWrapCls":{"!type":"string"},"_itemCls":{"!type":"string"},"_rotationAngles":{},"_rotationClasses":{},"_textAlignClasses":{},"_textCls":{"!type":"string"},"_titleSuffix":{"!type":"string"},"beforeRenderConfig":{},"iconTpl":{"!type":"string"},"isTitle":{"!type":"bool"},"noWrap":{"!type":"bool","!doc":"<p>layout system optimization.  Allows autocomponent layout to measure height without\nhaving to first know the width.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"afterComponentLayout":{"!type":"fn() -> !this","!doc":"<p>Called by the layout system after the Component has been laid out.</p>\n"},"applyText":{"!type":"fn(text: ?) -> !this"},"beforeRender":{"!type":"fn() -> !this"},"getIconMarkup":{"!type":"fn() -> !this"},"getIconRenderData":{"!type":"fn() -> !this"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"onAdded":{"!type":"fn(container: ?, pos: ?, instanced: ?) -> !this","!doc":"<p>Method to manage awareness of when components are added to their\nrespective Container, firing an <a href=\"#!/api/Ext.panel.Title-event-added\" rel=\"Ext.panel.Title-event-added\" class=\"docClass\">added</a> event. References are\nestablished at add time rather than at render time.</p>\n\n<p>Allows addition of behavior when a Component is added to a\nContainer. At this stage, the Component is in the parent\nContainer's collection of child items. After calling the\nsuperclass's <code>onAdded</code>, the <code>ownerCt</code> reference will be present,\nand if configured with a ref, the <code>refOwner</code> will be set.</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"updateGlyph":{"!type":"fn(glyph: ?, oldGlyph: ?) -> !this"},"updateIcon":{"!type":"fn(icon: ?, oldIcon: ?) -> !this"},"updateIconAlign":{"!type":"fn(align: ?, oldAlign: ?) -> !this"},"updateIconCls":{"!type":"fn(cls: ?, oldCls: ?) -> !this"},"updateRotation":{"!type":"fn(rotation: ?, oldRotation: ?) -> !this"},"updateText":{"!type":"fn(text: ?) -> !this"},"updateTextAlign":{"!type":"fn(align: ?, oldAlign: ?) -> !this"}}},"Tool":{"!doc":"<p>This class is used to display small visual icons in the header of a panel. There are a set of\n25 icons that can be specified by using the <a href=\"#!/api/Ext.panel.Tool-cfg-type\" rel=\"Ext.panel.Tool-cfg-type\" class=\"docClass\">type</a> config. The <a href=\"#!/api/Ext.panel.Tool-cfg-callback\" rel=\"Ext.panel.Tool-cfg-callback\" class=\"docClass\">callback</a> config\ncan be used to provide a function that will respond to any click events. In general, this class\nwill not be instantiated directly, rather it will be created by specifying the <a href=\"#!/api/Ext.panel.Panel-cfg-tools\" rel=\"Ext.panel.Panel-cfg-tools\" class=\"docClass\">Ext.panel.Panel.tools</a>\nconfiguration on the Panel itself.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    width: 200,\n    height: 200,\n    renderTo: document.body,\n    title: 'A Panel',\n    tools: [{\n        type: 'help',\n        callback: function() {\n            // show help here\n        }\n    }, {\n        itemId: 'refresh',\n        type: 'refresh',\n        hidden: true,\n        callback: function() {\n            // do refresh\n        }\n    }, {\n        type: 'search',\n        callback: function (panel) {\n            // do search\n            panel.down('#refresh').show();\n        }\n    }]\n});\n</code></pre>\n\n<p>The <code>callback</code> config was added in Ext JS 4.2.1 as an alternative to <a href=\"#!/api/Ext.panel.Tool-cfg-handler\" rel=\"Ext.panel.Tool-cfg-handler\" class=\"docClass\">handler</a>\nto provide a more convenient list of arguments. In Ext JS 4.2.1 it is also possible to\npass a method name instead of a direct function:</p>\n\n<pre><code> tools: [{\n     type: 'help',\n     callback: 'onHelp',\n     scope: this\n },\n ...\n</code></pre>\n\n<p>The <code>callback</code> (or <code>handler</code>) name is looked up on the <code>scope</code> which will also be the\n<code>this</code> reference when the method is called.</p>\n","!type":"fn(config: ?)","prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"callback":{"!doc":"<p>A function to execute when the tool is clicked.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"disabledCls":{"!type":"string","!doc":"<p>CSS class to add when the Component is disabled.</p>\n"},"handler":{"!doc":"<p>A function to execute when the tool is clicked. Arguments passed are:</p>\n\n<ul>\n<li><strong>event</strong> : Ext.event.Event - The click event.</li>\n<li><strong>toolEl</strong> : Ext.Element - The tool Element.</li>\n<li><strong>owner</strong> : <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Ext.panel.Header</a> - The host panel header.</li>\n<li><strong>tool</strong> : <a href=\"#!/api/Ext.panel.Tool\" rel=\"Ext.panel.Tool\" class=\"docClass\">Ext.panel.Tool</a> - The tool object</li>\n</ul>\n\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.panel.Tool-method-getEl\" rel=\"Ext.panel.Tool-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.panel.Tool-method-getEl\" rel=\"Ext.panel.Tool-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.panel.Tool-cfg-renderSelectors\" rel=\"Ext.panel.Tool-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.panel.Tool-cfg-childEls\" rel=\"Ext.panel.Tool-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"scope":{"!doc":"<p>The scope to execute the <a href=\"#!/api/Ext.panel.Tool-cfg-callback\" rel=\"Ext.panel.Tool-cfg-callback\" class=\"docClass\">callback</a> or <a href=\"#!/api/Ext.panel.Tool-cfg-handler\" rel=\"Ext.panel.Tool-cfg-handler\" class=\"docClass\">handler</a> function. Defaults\nto the tool.</p>\n"},"stopEvent":{"!type":"bool","!doc":"<p>Specify as false to allow click event to propagate.</p>\n"},"toolOverCls":{"!type":"string"},"toolOwner":{"!doc":"<p>The owner to report to the <code>callback</code> method. Default is <code>null</code> for the <code>ownerCt</code>.\nThis is automatically set to the owning <code><a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a></code> when a tool is created as\na member of a panel's <code>tools</code>.</p>\n"},"toolPressedCls":{"!type":"string"},"tooltip":{"!doc":"<p>The tooltip for the tool - can be a string to be used as innerHTML (html tags are accepted) or QuickTips config\nobject</p>\n"},"tooltipType":{"!type":"string","!doc":"<p>The type of tooltip to use. Either 'qtip' (default) for QuickTips or 'title' for title attribute.</p>\n"},"type":{"!type":"string","!doc":"<p>The type of tool to render. The following types are available:</p>\n\n<ul>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-close\"></span> close</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-minimize\"></span> minimize</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-maximize\"></span> maximize</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-restore\"></span> restore</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-toggle\"></span> toggle</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-gear\"></span> gear</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-prev\"></span> prev</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-next\"></span> next</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-pin\"></span> pin</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-unpin\"></span> unpin</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-right\"></span> right</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-left\"></span> left</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-down\"></span> down</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-up\"></span> up</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-refresh\"></span> refresh</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-plus\"></span> plus</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-minus\"></span> minus</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-search\"></span> search</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-save\"></span> save</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-help\"></span> help</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-print\"></span> print</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-expand\"></span> expand</li>\n<li><span class=\"x-tool\"><img src=\"data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==\" class=\"x-tool-collapse\"></span> collapse</li>\n</ul>\n\n"},"_toolTypes":{"!doc":"<p><debug></debug></p>\n"},"ariaRole":{"!type":"string"},"cacheHeight":{"!type":"bool"},"cacheWidth":{"!type":"bool"},"isTool":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Tool, or subclass thereof.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"tipAttrs":{},"afterRender":{"!type":"fn() -> !this","!doc":"<p>inherit docs</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p></p>\n"},"onClick":{"!type":"fn(e: ?, target: ?) -> !this","!doc":"<p>Called when the tool element is clicked</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>inherit docs</p>\n"},"onMouseDown":{"!type":"fn() -> !this","!doc":"<p>Called when the user presses their mouse button down on a tool\nAdds the press class (<a href=\"#!/api/Ext.panel.Tool-cfg-toolPressedCls\" rel=\"Ext.panel.Tool-cfg-toolPressedCls\" class=\"docClass\">toolPressedCls</a>)</p>\n"},"onMouseOut":{"!type":"fn() -> !this","!doc":"<p>Called when the user rolls out from a tool.\nRemoves the over class (<a href=\"#!/api/Ext.panel.Tool-cfg-toolOverCls\" rel=\"Ext.panel.Tool-cfg-toolOverCls\" class=\"docClass\">toolOverCls</a>)</p>\n"},"onMouseOver":{"!type":"fn() -> !this","!doc":"<p>Called when the user rolls over a tool\nAdds the over class (<a href=\"#!/api/Ext.panel.Tool-cfg-toolOverCls\" rel=\"Ext.panel.Tool-cfg-toolOverCls\" class=\"docClass\">toolOverCls</a>)</p>\n"},"setTooltip":{"!type":"fn(tooltip: ?, type: ?) -> !this"},"setType":{"!type":"fn(type: ?) -> ?","!doc":"<p>Sets the type of the tool. Allows the icon to be changed.</p>\n"},"click":{"!type":"fn(this: ?, e: ?, owner: ?, eOpts: ?)","!doc":"<p>Fires when the tool is clicked</p>\n"}}}},"dashboard":{"Column":{"!doc":"<p>This class manages columns in a <code>Dashboard</code>. The primary role here is the <code>defaultType</code>\nconfig which points to <code><a href=\"#!/api/Ext.dashboard.Panel\" rel=\"Ext.dashboard.Panel\" class=\"docClass\">Ext.dashboard.Panel</a></code> and the self-destruct mechanism to get\nrid of empty columns.</p>\n","!type":"fn(config: ?)","prototype":{"cls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element. This can be useful\nfor adding customized styles to the component or any of its children using standard CSS rules.</p>\n"},"defaultType":{"!type":"string","!doc":"<p>The default <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">xtype</a> of child Components to create in this Container when\na child item is specified as a raw configuration object, rather than as an instantiated Component.</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.dashboard.Column-cfg-items\" rel=\"Ext.dashboard.Column-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"isDashboardColumn":{"!type":"bool"},"synthetic":{"!type":"bool","!doc":"<p>This property is <code>true</code> if the component was created internally by the framework\nand is not explicitly user-defined. This is set for such things as <code>Splitter</code>\ninstances managed by <code>border</code> and <code>box</code> layouts.</p>\n"},"onRemove":{"!type":"fn(comp: ?, isDestroying: ?) -> !this","!doc":"<p>not user-defined</p>\n"}}},"Dashboard":{"!doc":"<p>This class manages a drag-drop Dashboard similar to the legacy Ext JS Portal example.\nThe user-directed layout of the Dashboard is preserved the Ext JS <code>stateful</code> mechanism\nto preserve potentially dynamic user sizing and collapsed states as well as order of\nitems in their columns.</p>\n","!type":"fn(config: ?)","prototype":{"autoScroll":{"!type":"bool","!doc":"<p><code>true</code> to use overflow:'auto' on the components layout element and show scroll bars automatically when necessary,\n<code>false</code> to clip any overflowing content.\nThis should not be combined with <a href=\"#!/api/Ext.dashboard.Dashboard-cfg-overflowX\" rel=\"Ext.dashboard.Dashboard-cfg-overflowX\" class=\"docClass\">overflowX</a> or  <a href=\"#!/api/Ext.dashboard.Dashboard-cfg-overflowY\" rel=\"Ext.dashboard.Dashboard-cfg-overflowY\" class=\"docClass\">overflowY</a>.</p>\n"},"bodyCls":{"!doc":"<p>A CSS class, space-delimited string of classes, or array of classes to be applied to the panel's body element.\nThe following examples are all valid:</p>\n\n<pre><code>bodyCls: 'foo'\nbodyCls: 'foo bar'\nbodyCls: ['foo', 'bar']\n</code></pre>\n"},"cls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element. This can be useful\nfor adding customized styles to the component or any of its children using standard CSS rules.</p>\n"},"defaultType":{"!type":"string","!doc":"<p>The default <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">xtype</a> of child Components to create in this Container when\na child item is specified as a raw configuration object, rather than as an instantiated Component.</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.dashboard.Dashboard-cfg-items\" rel=\"Ext.dashboard.Dashboard-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"maxColumns":{"!doc":"<p>The maximum number of visible columns.</p>\n"},"parts":{"!doc":"<p>An object keyed by <code>id</code> for the parts that can be created for this <code>Dashboard</code>.</p>\n"},"stateful":{"!type":"bool","!doc":"<p>A flag which causes the object to attempt to restore the state of\ninternal properties from a saved state on startup. The object must have\na <a href=\"#!/api/Ext.dashboard.Dashboard-cfg-stateId\" rel=\"Ext.dashboard.Dashboard-cfg-stateId\" class=\"docClass\">stateId</a> for state to be managed.</p>\n\n<p>Auto-generated ids are not guaranteed to be stable across page loads and\ncannot be relied upon to save and restore the same state for a object.</p>\n\n<p>For state saving to work, the state manager's provider must have been\nset to an implementation of <a href=\"#!/api/Ext.state.Provider\" rel=\"Ext.state.Provider\" class=\"docClass\">Ext.state.Provider</a> which overrides the\n<a href=\"#!/api/Ext.state.Provider-method-set\" rel=\"Ext.state.Provider-method-set\" class=\"docClass\">set</a> and <a href=\"#!/api/Ext.state.Provider-method-get\" rel=\"Ext.state.Provider-method-get\" class=\"docClass\">get</a>\nmethods to save and recall name/value pairs. A built-in implementation,\n<a href=\"#!/api/Ext.state.CookieProvider\" rel=\"Ext.state.CookieProvider\" class=\"docClass\">Ext.state.CookieProvider</a> is available.</p>\n\n<p>To set the state provider for the current page:</p>\n\n<p>   <a href=\"#!/api/Ext.state.Manager-method-setProvider\" rel=\"Ext.state.Manager-method-setProvider\" class=\"docClass\">Ext.state.Manager.setProvider</a>(new <a href=\"#!/api/Ext.state.CookieProvider\" rel=\"Ext.state.CookieProvider\" class=\"docClass\">Ext.state.CookieProvider</a>({\n       expires: new Date(new Date().getTime()+(1000<em>60</em>60<em>24</em>7)), //7 days from now\n   }));</p>\n\n<p>A stateful object attempts to save state when one of the events\nlisted in the <a href=\"#!/api/Ext.dashboard.Dashboard-cfg-stateEvents\" rel=\"Ext.dashboard.Dashboard-cfg-stateEvents\" class=\"docClass\">stateEvents</a> configuration fires.</p>\n\n<p>To save state, a stateful object first serializes its state by\ncalling <em><a href=\"#!/api/Ext.dashboard.Dashboard-method-getState\" rel=\"Ext.dashboard.Dashboard-method-getState\" class=\"docClass\">getState</a></em>.</p>\n\n<p>The Component base class implements <a href=\"#!/api/Ext.dashboard.Dashboard-method-getState\" rel=\"Ext.dashboard.Dashboard-method-getState\" class=\"docClass\">getState</a> to save its width and height within the state\nonly if they were initially configured, and have changed from the configured value.</p>\n\n<p>The Panel class saves its collapsed state in addition to that.</p>\n\n<p>The Grid class saves its column state and store state (sorters and filters and grouper) in addition to its superclass state.</p>\n\n<p>If there is more application state to be save, the developer must provide an implementation which\nfirst calls the superclass method to inherit the above behaviour, and then injects new properties\ninto the returned object.</p>\n\n<p>The value yielded by getState is passed to <a href=\"#!/api/Ext.state.Manager-method-set\" rel=\"Ext.state.Manager-method-set\" class=\"docClass\">Ext.state.Manager.set</a>\nwhich uses the configured <a href=\"#!/api/Ext.state.Provider\" rel=\"Ext.state.Provider\" class=\"docClass\">Ext.state.Provider</a> to save the object\nkeyed by the <a href=\"#!/api/Ext.dashboard.Dashboard-cfg-stateId\" rel=\"Ext.dashboard.Dashboard-cfg-stateId\" class=\"docClass\">stateId</a>.</p>\n\n<p>During construction, a stateful object attempts to <em>restore</em> its state by calling\n<a href=\"#!/api/Ext.state.Manager-method-get\" rel=\"Ext.state.Manager-method-get\" class=\"docClass\">Ext.state.Manager.get</a> passing the <a href=\"#!/api/Ext.dashboard.Dashboard-cfg-stateId\" rel=\"Ext.dashboard.Dashboard-cfg-stateId\" class=\"docClass\">stateId</a></p>\n\n<p>The resulting object is passed to <a href=\"#!/api/Ext.dashboard.Dashboard-method-applyState\" rel=\"Ext.dashboard.Dashboard-method-applyState\" class=\"docClass\">applyState</a>*. The default implementation of\n<a href=\"#!/api/Ext.dashboard.Dashboard-method-applyState\" rel=\"Ext.dashboard.Dashboard-method-applyState\" class=\"docClass\">applyState</a> simply copies properties into the object, but a developer may\noverride this to support restoration of more complex application state.</p>\n\n<p>You can perform extra processing on state save and restore by attaching\nhandlers to the <a href=\"#!/api/Ext.dashboard.Dashboard-event-beforestaterestore\" rel=\"Ext.dashboard.Dashboard-event-beforestaterestore\" class=\"docClass\">beforestaterestore</a>, <a href=\"#!/api/Ext.dashboard.Dashboard-event-staterestore\" rel=\"Ext.dashboard.Dashboard-event-staterestore\" class=\"docClass\">staterestore</a>,\n<a href=\"#!/api/Ext.dashboard.Dashboard-event-beforestatesave\" rel=\"Ext.dashboard.Dashboard-event-beforestatesave\" class=\"docClass\">beforestatesave</a> and <a href=\"#!/api/Ext.dashboard.Dashboard-event-statesave\" rel=\"Ext.dashboard.Dashboard-event-statesave\" class=\"docClass\">statesave</a> events.</p>\n"},"firstColumnCls":{"!type":"string"},"idSeed":{"!type":"number"},"isDashboard":{"!type":"bool"},"lastColumnCls":{"!type":"string"},"addNew":{"!type":"fn(type: ?) -> !this"},"addView":{"!type":"fn(instance: ?, columnIndex: ?, beforeAfter: ?) -> !this"},"applyParts":{"!type":"fn(parts: ?, collection: ?) -> !this"},"applyState":{"!type":"fn(state: ?) -> !this","!doc":"<hr />\n\n<p> State and Item Persistence</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"createColumn":{"!type":"fn(config: ?) -> !this"},"createView":{"!type":"fn(config: ?) -> !this"},"deserializeItems":{"!type":"fn(serialized: ?) -> !this"},"getPart":{"!type":"fn(type: ?) -> !this"},"getParts":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.dashboard.Dashboard-cfg-parts\" rel=\"Ext.dashboard.Dashboard-cfg-parts\" class=\"docClass\">parts</a>.</p>\n"},"getState":{"!type":"fn() -> ?","!doc":"<p>The supplied default state gathering method for the Component class.</p>\n\n<p>This method returns dimension settings such as <code>flex</code>, <code>anchor</code>, <code>width</code> and <code>height</code> along with <code>collapsed</code>\nstate.</p>\n\n<p>Subclasses which implement more complex state should call the superclass's implementation, and apply their state\nto the result if this basic state is to be saved.</p>\n\n<p>Note that Component state will only be saved if the Component has a <a href=\"#!/api/Ext.dashboard.Dashboard-cfg-stateId\" rel=\"Ext.dashboard.Dashboard-cfg-stateId\" class=\"docClass\">stateId</a> and there as a StateProvider\nconfigured for the document.</p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>Initialize any events on this component</p>\n"},"initItems":{"!type":"fn() -> !this","!doc":"<p>This method is called to initialize the <code>items</code> collection. A derived class can\noverride this method to do any last minute manipulation of <code>items</code> and then call\nthis method using <code>callParent</code>. Upon return, the <code>items</code> will no longer be a simple\narray.</p>\n"},"onRemoveItem":{"!type":"fn(column: ?, item: ?) -> !this"},"serializeItem":{"!type":"fn(item: ?) -> !this"},"serializeItems":{"!type":"fn() -> !this"},"setParts":{"!type":"fn(parts: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.dashboard.Dashboard-cfg-parts\" rel=\"Ext.dashboard.Dashboard-cfg-parts\" class=\"docClass\">parts</a>.</p>\n"},"beforedragover":{"!type":"fn(eOpts: ?)"},"beforedrop":{"!type":"fn(eOpts: ?)"},"dragover":{"!type":"fn(eOpts: ?)"},"drop":{"!type":"fn(eOpts: ?)"},"validatedrop":{"!type":"fn(eOpts: ?)"}}},"DropZone":{"!doc":"<p>Internal class that manages drag/drop for the <code>Dashboard</code>.</p>\n","!type":"fn(dashboard: ?, cfg: ?)","prototype":{"overClass":{"!type":"string","!doc":"<p>This causes overflow to go hidden during the drag so that we don't cause panels to\nwrap by triggering overflow.</p>\n"},"ddScrollConfig":{},"getOverEvent":{"!type":"fn(dd: ?, e: ?, data: ?) -> !this"},"notifyDrop":{"!type":"fn(dd: ?, e: ?, data: ?) -> bool","!doc":"<p>The function a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> calls once to notify this drop target that the dragged item has\nbeen dropped on it. This method removes any <code>overClass</code> and returns false, so you must provide an\nimplementation that does something to process the drop event and returns true so that the drag source's\nrepair action does not run.</p>\n\n<p>You should <code>callParent</code> from an override of this method to ensure proper cleanup is\nperformed.</p>\n"},"notifyOver":{"!type":"fn(dd: ?, e: ?, data: ?) -> string","!doc":"<p>The function a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> calls continuously while it is being dragged over the target.\nThis method will be called on every mouse movement while the drag source is over the drop target.\nThis default implementation simply returns the dropAllowed config value.</p>\n"},"unreg":{"!type":"fn() -> !this","!doc":"<p>unregister the dropzone from ScrollManager</p>\n"}}},"Panel":{"!doc":"<p>This class is used to wrap content items in the <code>Dashboard</code>. It uses an\n<code>anchor</code> layout by default and provides resizing on the bottom edge only.</p>\n","!type":"fn(config: ?)","prototype":{"animCollapse":{"!type":"bool","!doc":"<p><code>true</code> to animate the transition when the panel is collapsed, <code>false</code> to skip the animation (defaults to <code>true</code>\nif the <a href=\"#!/api/Ext.fx.Anim\" rel=\"Ext.fx.Anim\" class=\"docClass\">Ext.fx.Anim</a> class is available, otherwise <code>false</code>). May also be specified as the animation\nduration in milliseconds.</p>\n"},"closable":{"!type":"bool","!doc":"<p>True to display the 'close' tool button and allow the user to close the window, false to hide the button and\ndisallow closing the window.</p>\n\n<p>By default, when close is requested by clicking the close button in the header, the <a href=\"#!/api/Ext.dashboard.Panel-method-close\" rel=\"Ext.dashboard.Panel-method-close\" class=\"docClass\">close</a> method will be\ncalled. This will <em><a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a></em> the Panel and its content meaning that it may not be\nreused.</p>\n\n<p>To make closing a Panel <em>hide</em> the Panel so that it may be reused, set <a href=\"#!/api/Ext.dashboard.Panel-cfg-closeAction\" rel=\"Ext.dashboard.Panel-cfg-closeAction\" class=\"docClass\">closeAction</a> to 'hide'.</p>\n"},"cls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element. This can be useful\nfor adding customized styles to the component or any of its children using standard CSS rules.</p>\n"},"collapsible":{"!type":"bool","!doc":"<p>True to make the panel collapsible and have an expand/collapse toggle Tool added into the header tool button\narea. False to keep the panel sized either statically, or by an owning layout manager, with no toggle Tool.\nWhen a panel is used in a <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>, the <a href=\"#!/api/Ext.dashboard.Panel-cfg-floatable\" rel=\"Ext.dashboard.Panel-cfg-floatable\" class=\"docClass\">floatable</a> option\ncan influence the behavior of collapsing.\nSee <a href=\"#!/api/Ext.dashboard.Panel-cfg-collapseMode\" rel=\"Ext.dashboard.Panel-cfg-collapseMode\" class=\"docClass\">collapseMode</a> and <a href=\"#!/api/Ext.dashboard.Panel-cfg-collapseDirection\" rel=\"Ext.dashboard.Panel-cfg-collapseDirection\" class=\"docClass\">collapseDirection</a></p>\n"},"draggable":{"!doc":"<p>Specify as true to make a <a href=\"#!/api/Ext.dashboard.Panel-cfg-floating\" rel=\"Ext.dashboard.Panel-cfg-floating\" class=\"docClass\">floating</a> Component draggable using the Component's encapsulating element as\nthe drag handle.</p>\n\n<p>This may also be specified as a config object for the <a href=\"#!/api/Ext.util.ComponentDragger\" rel=\"Ext.util.ComponentDragger\" class=\"docClass\">ComponentDragger</a> which is\ninstantiated to perform dragging.</p>\n\n<p>For example to create a Component which may only be dragged around using a certain internal element as the drag\nhandle, use the delegate option:</p>\n\n<pre><code>new <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>({\n    constrain: true,\n    floating: true,\n    style: {\n        backgroundColor: '#fff',\n        border: '1px solid black'\n    },\n    html: '&lt;h1 style=\"cursor:move\"&gt;The title&lt;/h1&gt;&lt;p&gt;The content&lt;/p&gt;',\n    draggable: {\n        delegate: 'h1'\n    }\n}).show();\n</code></pre>\n"},"frame":{"!type":"bool","!doc":"<p>True to apply a frame to the panel.</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.dashboard.Panel-cfg-items\" rel=\"Ext.dashboard.Panel-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"minHeight":{"!type":"number","!doc":"<p>The minimum value in pixels which this Component will set its height to.</p>\n\n<p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>\n"},"resizable":{"!doc":"<p>Specify as <code>true</code> to apply a <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Resizer</a> to this Component after rendering.</p>\n\n<p>May also be specified as a config object to be passed to the constructor of <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Resizer</a>\nto override any defaults. By default the Component passes its minimum and maximum size, and uses\n<code><a href=\"#!/api/Ext.resizer.Resizer-cfg-dynamic\" rel=\"Ext.resizer.Resizer-cfg-dynamic\" class=\"docClass\">Ext.resizer.Resizer.dynamic</a>: false</code></p>\n"},"resizeHandles":{"!type":"string","!doc":"<p>A valid <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Ext.resizer.Resizer</a> handles config string. Only applies when resizable = true.</p>\n"},"stateful":{"!type":"bool","!doc":"<p>A flag which causes the object to attempt to restore the state of\ninternal properties from a saved state on startup. The object must have\na <a href=\"#!/api/Ext.dashboard.Panel-cfg-stateId\" rel=\"Ext.dashboard.Panel-cfg-stateId\" class=\"docClass\">stateId</a> for state to be managed.</p>\n\n<p>Auto-generated ids are not guaranteed to be stable across page loads and\ncannot be relied upon to save and restore the same state for a object.</p>\n\n<p>For state saving to work, the state manager's provider must have been\nset to an implementation of <a href=\"#!/api/Ext.state.Provider\" rel=\"Ext.state.Provider\" class=\"docClass\">Ext.state.Provider</a> which overrides the\n<a href=\"#!/api/Ext.state.Provider-method-set\" rel=\"Ext.state.Provider-method-set\" class=\"docClass\">set</a> and <a href=\"#!/api/Ext.state.Provider-method-get\" rel=\"Ext.state.Provider-method-get\" class=\"docClass\">get</a>\nmethods to save and recall name/value pairs. A built-in implementation,\n<a href=\"#!/api/Ext.state.CookieProvider\" rel=\"Ext.state.CookieProvider\" class=\"docClass\">Ext.state.CookieProvider</a> is available.</p>\n\n<p>To set the state provider for the current page:</p>\n\n<p>   <a href=\"#!/api/Ext.state.Manager-method-setProvider\" rel=\"Ext.state.Manager-method-setProvider\" class=\"docClass\">Ext.state.Manager.setProvider</a>(new <a href=\"#!/api/Ext.state.CookieProvider\" rel=\"Ext.state.CookieProvider\" class=\"docClass\">Ext.state.CookieProvider</a>({\n       expires: new Date(new Date().getTime()+(1000<em>60</em>60<em>24</em>7)), //7 days from now\n   }));</p>\n\n<p>A stateful object attempts to save state when one of the events\nlisted in the <a href=\"#!/api/Ext.dashboard.Panel-cfg-stateEvents\" rel=\"Ext.dashboard.Panel-cfg-stateEvents\" class=\"docClass\">stateEvents</a> configuration fires.</p>\n\n<p>To save state, a stateful object first serializes its state by\ncalling <em><a href=\"#!/api/Ext.dashboard.Panel-method-getState\" rel=\"Ext.dashboard.Panel-method-getState\" class=\"docClass\">getState</a></em>.</p>\n\n<p>The Component base class implements <a href=\"#!/api/Ext.dashboard.Panel-method-getState\" rel=\"Ext.dashboard.Panel-method-getState\" class=\"docClass\">getState</a> to save its width and height within the state\nonly if they were initially configured, and have changed from the configured value.</p>\n\n<p>The Panel class saves its collapsed state in addition to that.</p>\n\n<p>The Grid class saves its column state and store state (sorters and filters and grouper) in addition to its superclass state.</p>\n\n<p>If there is more application state to be save, the developer must provide an implementation which\nfirst calls the superclass method to inherit the above behaviour, and then injects new properties\ninto the returned object.</p>\n\n<p>The value yielded by getState is passed to <a href=\"#!/api/Ext.state.Manager-method-set\" rel=\"Ext.state.Manager-method-set\" class=\"docClass\">Ext.state.Manager.set</a>\nwhich uses the configured <a href=\"#!/api/Ext.state.Provider\" rel=\"Ext.state.Provider\" class=\"docClass\">Ext.state.Provider</a> to save the object\nkeyed by the <a href=\"#!/api/Ext.dashboard.Panel-cfg-stateId\" rel=\"Ext.dashboard.Panel-cfg-stateId\" class=\"docClass\">stateId</a>.</p>\n\n<p>During construction, a stateful object attempts to <em>restore</em> its state by calling\n<a href=\"#!/api/Ext.state.Manager-method-get\" rel=\"Ext.state.Manager-method-get\" class=\"docClass\">Ext.state.Manager.get</a> passing the <a href=\"#!/api/Ext.dashboard.Panel-cfg-stateId\" rel=\"Ext.dashboard.Panel-cfg-stateId\" class=\"docClass\">stateId</a></p>\n\n<p>The resulting object is passed to <a href=\"#!/api/Ext.dashboard.Panel-method-applyState\" rel=\"Ext.dashboard.Panel-method-applyState\" class=\"docClass\">applyState</a>*. The default implementation of\n<a href=\"#!/api/Ext.dashboard.Panel-method-applyState\" rel=\"Ext.dashboard.Panel-method-applyState\" class=\"docClass\">applyState</a> simply copies properties into the object, but a developer may\noverride this to support restoration of more complex application state.</p>\n\n<p>You can perform extra processing on state save and restore by attaching\nhandlers to the <a href=\"#!/api/Ext.dashboard.Panel-event-beforestaterestore\" rel=\"Ext.dashboard.Panel-event-beforestaterestore\" class=\"docClass\">beforestaterestore</a>, <a href=\"#!/api/Ext.dashboard.Panel-event-staterestore\" rel=\"Ext.dashboard.Panel-event-staterestore\" class=\"docClass\">staterestore</a>,\n<a href=\"#!/api/Ext.dashboard.Panel-event-beforestatesave\" rel=\"Ext.dashboard.Panel-event-beforestatesave\" class=\"docClass\">beforestatesave</a> and <a href=\"#!/api/Ext.dashboard.Panel-event-statesave\" rel=\"Ext.dashboard.Panel-event-statesave\" class=\"docClass\">statesave</a> events.</p>\n"},"titleCollapse":{"!type":"bool","!doc":"<p><code>true</code> to allow expanding and collapsing the panel (when <code><a href=\"#!/api/Ext.dashboard.Panel-cfg-collapsible\" rel=\"Ext.dashboard.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> = true</code>) by clicking anywhere in\nthe header bar, <code>false</code>) to allow it only by clicking to tool button). When a panel is used in a\n<a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">border layout</a>, the <a href=\"#!/api/Ext.dashboard.Panel-cfg-floatable\" rel=\"Ext.dashboard.Panel-cfg-floatable\" class=\"docClass\">floatable</a> option can influence the behavior of collapsing.</p>\n"},"anchor":{"!type":"string"},"animateClose":{"!type":"bool"},"loadMask":{"!type":"bool"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"doClose":{"!type":"fn() -> !this","!doc":"<p>Override Panel's default doClose to provide a custom fade out effect\nwhen a portlet is removed from the portal</p>\n"},"finishClose":{"!type":"fn() -> !this"},"getLoadMask":{"!type":"fn() -> !this"},"getState":{"!type":"fn() -> !this"},"onAdd":{"!type":"fn(view: ?) -> !this","!doc":"<p>This method is invoked after a new Component has been added. It\nis passed the Component which has been added. This method may\nbe used to update any internal structure which may depend upon\nthe state of the child items.</p>\n"},"onViewBeforeLoad":{"!type":"fn() -> !this"},"onViewLoaded":{"!type":"fn() -> !this"},"setBox":{"!type":"fn(box: ?) -> !this"}}},"Part":{"!doc":"<p>This class encapsulates the creation of items for a <code>Dashboard</code>. Generally a <code>Part</code> is a\ncomponent factory that allows all parts of a common type to be easily coordinated as\nneeded for that type. For example, an RSS feed might need certain configuration data to\nproperly initialize. Perahps not all of this data can or should be supplied from the UI\nthat creates new instances for the <code>Dashboard</code>.</p>\n\n<h2>Part Configuration</h2>\n\n<p>The primary role of a <code>Part</code> is to provide an abstract way to define the configuration\nneeded to create views. For example, an RSS Part would at least need the URL for the\nfeed.</p>\n\n<p>To implement this a derived class provides a <code>displayForm</code> method:</p>\n\n<pre><code> Ext.define('App.parts.RSS', {\n     extend: '<a href=\"#!/api/Ext.dashboard.Part\" rel=\"Ext.dashboard.Part\" class=\"docClass\">Ext.dashboard.Part</a>',\n     alias: 'part.rss',\n\n     displayForm: function (instance, currentConfig, callback, scope) {\n         var me = this,\n             title = instance ? 'Edit RSS Feed' : 'Add RSS Feed';\n\n         // Display a prompt using current URL as default text.\n         //\n         <a href=\"#!/api/Ext.MessageBox-method-prompt\" rel=\"Ext.MessageBox-method-prompt\" class=\"docClass\">Ext.Msg.prompt</a>(title, 'RSS Feed URL', function (btn, text) {\n             if (btn === 'ok') {\n                 var config = {\n                     feedUrl: text\n                 };\n\n                 callback.call(scope || me, config);\n             }\n         }, me, false, currentConfig ? currentConfig.feedUrl : '');\n     }\n });\n</code></pre>\n\n<p>The returned configuration object is used to create views. It is also passed back to\n<code>displayForm</code> to allow the user to edit the configuration for an existing view.</p>\n\n<h2>Creating Views</h2>\n\n<p>The next step is to define the view (the components) appropriate for the part. To\ncontinue with the above example.</p>\n\n<pre><code> Ext.define('App.parts.RSS', {\n     extend: '<a href=\"#!/api/Ext.dashboard.Part\" rel=\"Ext.dashboard.Part\" class=\"docClass\">Ext.dashboard.Part</a>',\n     alias: 'part.rss',\n\n     // The viewTemplate is \"component template\" that is evaluated against the\n     // configuration (as returned by displayForm). The top-most component is\n     // a panel specific to the Dashboard so it can be configured but should\n     // not be replaced. Instead, fit an appropriate component to the panel.\n     //\n     viewTemplate: {\n         layout: 'fit',\n         items: [{\n             xtype: 'feedpanel',\n             feedUrl: '{feedUrl}'  // from the configuration object\n         }]\n     },\n\n     displayForm: ...\n });\n</code></pre>\n\n<p>You can instead choose to override the <code>createView</code> method if <code>viewTemplate</code> does not\nprovide enough flexibility. It is usually a better solution to create a class (like\nin the above example) and pass basic configurations to it rather than over-complicate\neither the <code>viewTemplate</code> or a custom <code>createView</code> method.</p>\n","!type":"fn(config: ?)","prototype":{"id":{},"viewTemplate":{"!doc":"<p>The configuration object used for creating instances of this <code>Part</code>. This is\nused by the <code>createView</code> method to create views.</p>\n"},"_lastId":{"!type":"number","!doc":"<p>The last assigned identifier for instances created by this <code>Part</code>.</p>\n"},"dashboard":{"!doc":"<p>The <code>Dashboard</code> instance that owns this <code>part</code>.</p>\n"},"factoryConfig":{},"isPart":{"!type":"bool"},"valueRe":{},"viewTemplateOptions":{},"applyViewTemplate":{"!type":"fn(template: ?) -> !this"},"createView":{"!type":"fn(config: ?) -> ?","!doc":"<p>This method is responsible for converting a configuration object from <code>displayForm</code>\ninto a \"view\" (an object that can be passed to <code>Ext.widget</code>).</p>\n\n<p>If you override this method it is recommended that you <code>callParent</code> to get the view\nproduced and then edit that result. This is because there are several private\nproperties placed on the returned configuration object.</p>\n\n<pre><code> createView: function (config) {\n     var view = this.callParent([config]);\n\n     // edit view\n\n     return view;\n }\n</code></pre>\n"},"displayForm":{"!type":"fn(instance: ?, currentConfig: ?, callback: ?, scope: ?) -> !this","!doc":"<p>This method should display an appropriate edit form (probably a modal <code><a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.Window</a></code>\nor <code><a href=\"#!/api/Ext.MessageBox-method-prompt\" rel=\"Ext.MessageBox-method-prompt\" class=\"docClass\">Ext.Msg.prompt</a></code>) to get or edit configuration for an instance of this part.</p>\n\n<p>See the class documentation for examples on implementing this method.</p>\n"},"getId":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.dashboard.Part-cfg-id\" rel=\"Ext.dashboard.Part-cfg-id\" class=\"docClass\">id</a>.</p>\n"},"getViewTemplate":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.dashboard.Part-cfg-viewTemplate\" rel=\"Ext.dashboard.Part-cfg-viewTemplate\" class=\"docClass\">viewTemplate</a>.</p>\n"},"setId":{"!type":"fn(id: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.dashboard.Part-cfg-id\" rel=\"Ext.dashboard.Part-cfg-id\" class=\"docClass\">id</a>.</p>\n"},"setViewTemplate":{"!type":"fn(viewTemplate: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.dashboard.Part-cfg-viewTemplate\" rel=\"Ext.dashboard.Part-cfg-viewTemplate\" class=\"docClass\">viewTemplate</a>.</p>\n"}}}},"dd":{"DropTarget":{"prototype":{"ddGroup":{"!type":"string","!doc":"<p>A named drag drop group to which this object belongs.  If a group is specified, then this object will only\ninteract with other drag drop objects in the same group.</p>\n"},"dropAllowed":{"!type":"string","!doc":"<p>The CSS class returned to the drag source when drop is allowed.</p>\n"},"dropNotAllowed":{"!type":"string","!doc":"<p>The CSS class returned to the drag source when drop is not allowed.</p>\n"},"isNotifyTarget":{"!type":"bool","!doc":"<p>private</p>\n"},"isTarget":{"!type":"bool","!doc":"<p>private</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroy this DragDrop instance</p>\n"},"notifyEnter":{"!type":"fn(source: ?, e: ?, data: ?) -> string","!doc":"<p>The function a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> calls once to notify this drop target that the source is now over the\ntarget.  This default implementation adds the CSS class specified by overClass (if any) to the drop element\nand returns the dropAllowed config value.  This method should be overridden if drop validation is required.</p>\n"},"notifyOut":{"!type":"fn(source: ?, e: ?, data: ?) -> !this","!doc":"<p>The function a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> calls once to notify this drop target that the source has been dragged\nout of the target without dropping. This default implementation simply removes the CSS class specified by\n<code>overClass</code> (if any) from the drop element.</p>\n"},"overClass":{"!type":"string","!doc":"<p>The CSS class applied to the drop target element while the drag source is over it.</p>\n"},"notifyDrop":{"!type":"fn(source: ?, e: ?, data: ?) -> bool","!doc":"<p>The function a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> calls once to notify this drop target that the dragged item has\nbeen dropped on it. This method removes any <code>overClass</code> and returns false, so you must provide an\nimplementation that does something to process the drop event and returns true so that the drag source's\nrepair action does not run.</p>\n\n<p>You should <code>callParent</code> from an override of this method to ensure proper cleanup is\nperformed.</p>\n"},"notifyOver":{"!type":"fn(source: ?, e: ?, data: ?) -> string","!doc":"<p>The function a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> calls continuously while it is being dragged over the target.\nThis method will be called on every mouse movement while the drag source is over the drop target.\nThis default implementation simply returns the dropAllowed config value.</p>\n"}},"!doc":"<p>A simple class that provides the basic implementation needed to make any element a drop target that can have\ndraggable items dropped onto it.  The drop has no effect until an implementation of notifyDrop is provided.</p>\n","!type":"fn(el: ?, config: ?)"},"DragDrop":{"prototype":{"__ygDragDrop":{"!type":"bool","!doc":"<p>Internal typeof flag</p>\n"},"_domRef":{"!doc":"<p>Cached reference to the linked element</p>\n"},"available":{"!type":"bool","!doc":"<p>The available property is false until the linked dom element is accessible.</p>\n"},"config":{"!doc":"<p>Configuration attributes passed into the constructor</p>\n"},"constrainX":{"!type":"bool","!doc":"<p>Set to true when horizontal contraints are applied</p>\n"},"constrainY":{"!type":"bool","!doc":"<p>Set to true when vertical contraints are applied</p>\n"},"defaultPadding":{"!doc":"<p>Provides default constraint padding to \"constrainTo\" elements.</p>\n"},"dragElId":{"!type":"string","!doc":"<p>The id of the element that will be dragged.  By default this is same\nas the linked element, but could be changed to another element. Ex:\n<a href=\"#!/api/Ext.dd.DDProxy\" rel=\"Ext.dd.DDProxy\" class=\"docClass\">Ext.dd.DDProxy</a></p>\n"},"groups":{"!doc":"<p>The group defines a logical collection of DragDrop objects that are\nrelated.  Instances only get events when interacting with other\nDragDrop object in the same group.  This lets us define multiple\ngroups using a single DragDrop subclass if we want.</p>\n\n<p>An object in the format {'group1':true, 'group2':true}</p>\n"},"handleElId":{"!type":"string","!doc":"<p>The ID of the element that initiates the drag operation.  By default\nthis is the linked element, but could be changed to be a child of this\nelement.  This lets us do things like only starting the drag when the\nheader element within the linked html element is clicked.</p>\n"},"hasOuterHandles":{"!type":"bool","!doc":"<p>By default, drags can only be initiated if the mousedown occurs in the\nregion the linked element is.  This is done in part to work around a\nbug in some browsers that mis-report the mousedown if the previous\nmouseup happened outside of the window.  This property is set to true\nif outer handles are defined. Defaults to false.</p>\n"},"id":{"!type":"string","!doc":"<p>The id of the element associated with this object.  This is what we\nrefer to as the \"linked element\" because the size and position of\nthis element is used to determine when the drag and drop objects have\ninteracted.</p>\n"},"ignoreSelf":{"!type":"bool","!doc":"<p>Set to false to enable a DragDrop object to fire drag events while dragging\nover its own Element. Defaults to true - DragDrop objects do not by default\nfire drag events to themselves.</p>\n"},"invalidHandleClasses":{"!doc":"<p>An Array of CSS class names for elements to be considered in valid as drag handles.</p>\n"},"invalidHandleIds":{"!doc":"<p>An object who's property names identify the IDs of elements to be considered invalid as drag handles.\nA non-null property value identifies the ID as invalid. For example, to prevent\ndragging from being initiated on element ID \"foo\", use:</p>\n\n<pre><code>{\n    foo: true\n}\n</code></pre>\n"},"invalidHandleTypes":{"!doc":"<p>An object who's property names identify HTML tags to be considered invalid as drag handles.\nA non-null property value identifies the tag as invalid. Defaults to the\nfollowing value which prevents drag operations from being initiated by <code>&lt;a&gt;</code> elements:</p>\n\n<pre><code>{\n    A: \"A\"\n}\n</code></pre>\n"},"locked":{"!type":"bool","!doc":"<p>Individual drag/drop instances can be locked.  This will prevent\nonmousedown start drag.</p>\n"},"maintainOffset":{"!type":"bool","!doc":"<p>Maintain offsets when we resetconstraints.  Set to true when you want\nthe position of the element relative to its parent to stay the same\nwhen the page changes</p>\n"},"maxX":{"!type":"number","!doc":"<p>The right constraint</p>\n"},"maxY":{"!type":"number","!doc":"<p>The down constraint</p>\n"},"minX":{"!type":"number","!doc":"<p>The left constraint</p>\n"},"minY":{"!type":"number","!doc":"<p>The up constraint</p>\n"},"moveOnly":{"!type":"bool","!doc":"<p>When set to true, other DD objects in cooperating DDGroups do not receive\nnotification events when this DD object is dragged over them.</p>\n"},"padding":{"!doc":"<p>The padding configured for this drag and drop object for calculating\nthe drop zone intersection with this object.\nAn array containing the 4 padding values: [top, right, bottom, left]</p>\n"},"primaryButtonOnly":{"!type":"bool","!doc":"<p>By default the drag and drop instance will only respond to the primary\nbutton click (left button for a right-handed mouse).  Set to true to\nallow drag and drop to start with any mouse click that is propogated\nby the browser</p>\n"},"startPageX":{"!type":"number","!doc":"<p>The linked element's absolute X position at the time the drag was\nstarted</p>\n"},"startPageY":{"!type":"number","!doc":"<p>The linked element's absolute X position at the time the drag was\nstarted</p>\n"},"triggerEvent":{"!type":"string"},"xTicks":{"!doc":"<p>Array of pixel locations the element will snap to if we specified a\nhorizontal graduation/interval.  This array is generated automatically\nwhen you define a tick interval.</p>\n"},"yTicks":{"!doc":"<p>Array of pixel locations the element will snap to if we specified a\nvertical graduation/interval.  This array is generated automatically\nwhen you define a tick interval.</p>\n"},"addToGroup":{"!type":"fn(sGroup: ?) -> !this","!doc":"<p>Adds this instance to a group of related drag/drop objects.  All\ninstances belong to at least one group, and can belong to as many\ngroups as needed.</p>\n"},"applyConfig":{"!type":"fn() -> !this","!doc":"<p>Applies the configuration parameters that were passed into the constructor.\nThis is supposed to happen at each level through the inheritance chain.  So\na DDProxy implentation will execute apply config on DDProxy, DD, and\nDragDrop in order to get all of the parameters that are available in\neach object.</p>\n"},"b4Drag":{"!type":"fn(e: ?) -> !this","!doc":"<p>Code that executes immediately before the onDrag event</p>\n"},"b4DragDrop":{"!type":"fn(e: ?) -> !this","!doc":"<p>Code that executes immediately before the onDragDrop event</p>\n"},"b4DragOut":{"!type":"fn(e: ?) -> !this","!doc":"<p>Code that executes immediately before the onDragOut event</p>\n"},"b4DragOver":{"!type":"fn(e: ?) -> !this","!doc":"<p>Code that executes immediately before the onDragOver event</p>\n"},"b4EndDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>Code that executes immediately before the endDrag event</p>\n"},"b4MouseDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Code executed immediately before the onMouseDown event</p>\n"},"b4StartDrag":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Code that executes immediately before the startDrag event</p>\n"},"clickValidator":{"!type":"fn(e: ?) -> !this"},"constrainTo":{"!type":"fn(constrainTo: ?, pad?: ?, inContent?: ?) -> !this","!doc":"<p>Initializes the drag drop object's constraints to restrict movement to a certain element.</p>\n\n<p>Usage:</p>\n\n<pre><code>var dd = new <a href=\"#!/api/Ext.dd.DDProxy\" rel=\"Ext.dd.DDProxy\" class=\"docClass\">Ext.dd.DDProxy</a>(\"dragDiv1\", \"proxytest\",\n               { dragElId: \"existingProxyDiv\" });\ndd.startDrag = function(){\n    this.constrainTo(\"parent-id\");\n};\n</code></pre>\n\n<p>Or you can initalize it using the Ext.dom.Element object:</p>\n\n<pre><code>Ext.get(\"dragDiv1\").initDDProxy(\"proxytest\", {dragElId: \"existingProxyDiv\"}, {\n    startDrag : function(){\n        this.constrainTo(\"parent-id\");\n    }\n});\n</code></pre>\n"},"getEl":{"!type":"fn() -> ?","!doc":"<p>Returns a reference to the linked element</p>\n"},"getTick":{"!type":"fn(val: ?, tickArray: ?) -> number","!doc":"<p>Normally the drag element is moved pixel by pixel, but we can specify\nthat it move a number of pixels at a time.  This method resolves the\nlocation when we have it set up like this.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?, oDD: ?) -> !this","!doc":"<p>Called when this object is clicked</p>\n"},"handleOnAvailable":{"!type":"fn() -> !this","!doc":"<p>Executed when the linked element is available</p>\n"},"init":{"!type":"fn(id: ?, sGroup: ?, config: ?) -> !this","!doc":"<p>Sets up the DragDrop object.  Must be called in the constructor of any\n<a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a> subclass</p>\n"},"initTarget":{"!type":"fn(id: ?, sGroup: ?, config: ?) -> !this","!doc":"<p>Initializes Targeting functionality only... the object does not\nget a mousedown handler.</p>\n"},"isLocked":{"!type":"fn() -> bool","!doc":"<p>Returns true if this instance is locked, or the drag drop mgr is locked\n(meaning that all drag/drop is disabled on the page.)</p>\n"},"lock":{"!type":"fn() -> !this","!doc":"<p>Locks this instance</p>\n"},"onAvailable":{"!type":"fn() -> !this","!doc":"<p>Override the onAvailable method to do what is needed after the initial\nposition was determined.</p>\n"},"removeFromGroup":{"!type":"fn(sGroup: ?) -> !this","!doc":"<p>Removes this instance from the supplied interaction group</p>\n"},"setPadding":{"!type":"fn(iTop: ?, iRight: ?, iBot: ?, iLeft: ?) -> !this","!doc":"<p>Configures the padding for the target zone in px.  Effectively expands\n(or reduces) the virtual object size for targeting calculations.\nSupports css-style shorthand; if only one parameter is passed, all sides\nwill have that padding, and if only two are passed, the top and bottom\nwill have the first param, the left and right the second.</p>\n"},"setStartPosition":{"!type":"fn(pos: ?) -> !this","!doc":"<p>Sets the start position of the element.  This is set when the obj\nis initialized, the reset when a drag is started.</p>\n"},"setXTicks":{"!type":"fn(iStartX: ?, iTickSize: ?) -> !this","!doc":"<p>Creates the array of horizontal tick marks if an interval was specified\nin setXConstraint().</p>\n"},"setYTicks":{"!type":"fn(iStartY: ?, iTickSize: ?) -> !this","!doc":"<p>Creates the array of vertical tick marks if an interval was specified in\nsetYConstraint().</p>\n"},"unlock":{"!type":"fn() -> !this","!doc":"<p>Unlocks this instace</p>\n"},"isTarget":{"!type":"bool","!doc":"<p>By default, all instances can be a drop target.  This can be disabled by\nsetting isTarget to false.</p>\n"},"addInvalidHandleClass":{"!type":"fn(cssClass: ?) -> !this","!doc":"<p>Lets you specify a css class of elements that will not initiate a drag</p>\n"},"addInvalidHandleId":{"!type":"fn(id: ?) -> !this","!doc":"<p>Lets you to specify an element id for a child of a drag handle\nthat should not initiate a drag</p>\n"},"addInvalidHandleType":{"!type":"fn(tagName: ?) -> !this","!doc":"<p>Allows you to specify a tag name that should not start a drag operation\nwhen clicked.  This is designed to facilitate embedding links within a\ndrag handle that do something other than start the drag.</p>\n"},"clearConstraints":{"!type":"fn() -> !this","!doc":"<p>Clears any constraints applied to this instance.  Also clears ticks\nsince they can't exist independent of a constraint at this time.</p>\n"},"clearTicks":{"!type":"fn() -> !this","!doc":"<p>Clears any tick interval defined for this instance</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroy this DragDrop instance</p>\n"},"endDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>Called when we are done dragging the object</p>\n"},"getDragEl":{"!type":"fn() -> ?","!doc":"<p>Returns a reference to the actual element to drag.  By default this is\nthe same as the html element, but it can be assigned to another\nelement. An example of this can be found in <a href=\"#!/api/Ext.dd.DDProxy\" rel=\"Ext.dd.DDProxy\" class=\"docClass\">Ext.dd.DDProxy</a></p>\n"},"isValidHandleChild":{"!type":"fn(node: ?) -> bool","!doc":"<p>Checks the tag exclusion list to see if this click should be ignored</p>\n"},"onDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>Abstract method called during the onMouseMove event while dragging an\nobject.</p>\n"},"onDragDrop":{"!type":"fn(e: ?, id: ?) -> !this","!doc":"<p>Abstract method called when this item is dropped on another DragDrop\nobj</p>\n"},"onDragEnter":{"!type":"fn(e: ?, id: ?) -> !this","!doc":"<p>Abstract method called when this element fist begins hovering over\nanother DragDrop obj</p>\n"},"onDragOut":{"!type":"fn(e: ?, id: ?) -> !this","!doc":"<p>Abstract method called when we are no longer hovering over an element</p>\n"},"onDragOver":{"!type":"fn(e: ?, id: ?) -> !this","!doc":"<p>Abstract method called when this element is hovering over another\nDragDrop obj</p>\n"},"onInvalidDrop":{"!type":"fn(e: ?) -> !this","!doc":"<p>Abstract method called when this item is dropped on an area with no\ndrop target</p>\n"},"onMouseDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Called when a drag/drop obj gets a mousedown</p>\n"},"onMouseUp":{"!type":"fn(e: ?) -> !this","!doc":"<p>Called when a drag/drop obj gets a mouseup</p>\n"},"removeInvalidHandleClass":{"!type":"fn(cssClass: ?) -> !this","!doc":"<p>Unsets an invalid css class</p>\n"},"removeInvalidHandleId":{"!type":"fn(id: ?) -> !this","!doc":"<p>Unsets an invalid handle id</p>\n"},"removeInvalidHandleType":{"!type":"fn(tagName: ?) -> !this","!doc":"<p>Unsets an excluded tag name set by addInvalidHandleType</p>\n"},"resetConstraints":{"!type":"fn(maintainOffset: ?) -> !this","!doc":"<p>Must be called if you manually reposition a dd element.</p>\n"},"setDragElId":{"!type":"fn(id: ?) -> !this","!doc":"<p>Allows you to specify that an element other than the linked element\nwill be moved with the cursor during a drag</p>\n"},"setHandleElId":{"!type":"fn(id: ?) -> !this","!doc":"<p>Allows you to specify a child of the linked element that should be\nused to initiate the drag operation.  An example of this would be if\nyou have a content div with text and links.  Clicking anywhere in the\ncontent area would normally start the drag operation.  Use this method\nto specify that an element inside of the content div is the element\nthat starts the drag operation.</p>\n"},"setInitPosition":{"!type":"fn(diffX: ?, diffY: ?) -> !this","!doc":"<p>Stores the initial placement of the linked element.</p>\n"},"setOuterHandleElId":{"!type":"fn(id: ?) -> !this","!doc":"<p>Allows you to set an element outside of the linked element as a drag\nhandle</p>\n"},"setXConstraint":{"!type":"fn(iLeft: ?, iRight: ?, iTickSize?: ?) -> !this","!doc":"<p>By default, the element can be dragged any place on the screen.  Use\nthis method to limit the horizontal travel of the element.  Pass in\n0,0 for the parameters if you want to lock the drag to the y axis.</p>\n"},"setYConstraint":{"!type":"fn(iUp: ?, iDown: ?, iTickSize?: ?) -> !this","!doc":"<p>By default, the element can be dragged any place on the screen.  Set\nthis to limit the vertical travel of the element.  Pass in 0,0 for the\nparameters if you want to lock the drag to the x axis.</p>\n"},"startDrag":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Abstract method called after a drag/drop object is clicked\nand the drag or mousedown time thresholds have beeen met.</p>\n"},"unreg":{"!type":"fn() -> !this","!doc":"<p>Removes all drag and drop hooks for this element</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>toString method</p>\n"}},"!doc":"<p>Defines the interface and base operation of items that that can be\ndragged or can be drop targets.  It was designed to be extended, overriding\nthe event handlers for startDrag, onDrag, onDragOver and onDragOut.\nUp to three html elements can be associated with a DragDrop instance:</p>\n\n<ul>\n<li><p>linked element: the element that is passed into the constructor.\nThis is the element which defines the boundaries for interaction with\nother DragDrop objects.</p></li>\n<li><p>handle element(s): The drag operation only occurs if the element that\nwas clicked matches a handle element.  By default this is the linked\nelement, but there are times that you will want only a portion of the\nlinked element to initiate the drag operation, and the setHandleElId()\nmethod provides a way to define this.</p></li>\n<li><p>drag element: this represents the element that would be moved along\nwith the cursor during a drag operation.  By default, this is the linked\nelement itself as in <a href=\"#!/api/Ext.dd.DD\" rel=\"Ext.dd.DD\" class=\"docClass\">Ext.dd.DD</a>.  setDragElId() lets you define\na separate element that would be moved, as in <a href=\"#!/api/Ext.dd.DDProxy\" rel=\"Ext.dd.DDProxy\" class=\"docClass\">Ext.dd.DDProxy</a>.</p></li>\n</ul>\n\n\n<p>This class should not be instantiated until the onload event to ensure that\nthe associated elements are available.\nThe following would define a DragDrop obj that would interact with any\nother DragDrop obj in the \"group1\" group:</p>\n\n<pre><code>dd = new <a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a>(\"div1\", \"group1\");\n</code></pre>\n\n<p>Since none of the event handlers have been implemented, nothing would\nactually happen if you were to run the code above.  Normally you would\noverride this class or one of the default implementations, but you can\nalso override the methods you want on an instance of the class...</p>\n\n<pre><code>dd.onDragDrop = function(e, id) {\n    alert(\"dd was dropped on \" + id);\n}\n</code></pre>\n","!type":"fn(id: ?, sGroup: ?, config: ?)"},"DDTarget":{"prototype":{"addInvalidHandleClass":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"addInvalidHandleId":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"addInvalidHandleType":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"clearConstraints":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"clearTicks":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"endDrag":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"getDragEl":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"isValidHandleChild":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"onDrag":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"onDragDrop":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"onDragEnter":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"onDragOut":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"onDragOver":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"onInvalidDrop":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"onMouseDown":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"onMouseUp":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"removeInvalidHandleClass":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"removeInvalidHandleId":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"removeInvalidHandleType":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"resetConstraints":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"setDragElId":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"setHandleElId":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"setInitPosition":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"setOuterHandleElId":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"setXConstraint":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"setYConstraint":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"startDrag":{"!type":"fn() -> !this","!doc":"<p>Overridden and disabled. A DDTarget does not support being dragged.</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>toString method</p>\n"}},"!doc":"<p>A DragDrop implementation that does not move, but can be a drop\ntarget.  You would get the same result by simply omitting implementation\nfor the event callbacks, but this way we reduce the processing cost of the\nevent listener and the callbacks.</p>\n","!type":"fn(id: ?, sGroup: ?, config: ?)"},"DD":{"!doc":"<p>A DragDrop implementation where the linked element follows the\nmouse cursor during a drag.</p>\n","!type":"fn(id: ?, sGroup: ?, config: ?)","prototype":{"scroll":{"!type":"bool","!doc":"<p>When set to true, the utility automatically tries to scroll the browser\nwindow when a drag and drop element is dragged near the viewport boundary.</p>\n"},"alignElWithMouse":{"!type":"fn(el: ?, iPageX: ?, iPageY: ?) -> !this","!doc":"<p>Sets the element to the location of the mousedown or click event,\nmaintaining the cursor location relative to the location on the element\nthat was clicked.  Override this if you want to place the element in a\nlocation other than where the cursor is.</p>\n"},"applyConfig":{"!type":"fn() -> !this","!doc":"<p>Sets up config options specific to this class. Overrides\n<a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a>, but all versions of this method through the\ninheritance chain are called</p>\n"},"autoOffset":{"!type":"fn(iPageX: ?, iPageY: ?) -> !this","!doc":"<p>Sets the pointer offset to the distance between the linked element's top\nleft corner and the location the element was clicked.</p>\n"},"autoScroll":{"!type":"fn(x: ?, y: ?, h: ?, w: ?) -> !this","!doc":"<p>Auto-scroll the window if the dragged object has been moved beyond the\nvisible window boundary.</p>\n"},"b4Drag":{"!type":"fn(e: ?) -> !this","!doc":"<p>Event that fires prior to the onDrag event.  Overrides\n<a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a>.</p>\n"},"b4MouseDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Event that fires prior to the onMouseDown event.  Overrides\n<a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a>.</p>\n"},"cachePosition":{"!type":"fn(iPageX?: ?, iPageY?: ?) -> !this","!doc":"<p>Saves the most recent position so that we can reset the constraints and\ntick marks on-demand.  We need to know this so that we can calculate the\nnumber of pixels the element is offset from its original position.</p>\n"},"getLocalX":{"!type":"fn(el: ?) -> !this"},"getTargetCoord":{"!type":"fn(iPageX: ?, iPageY: ?) -> ?","!doc":"<p>Finds the location the element should be placed if we want to move\nit to where the mouse location less the click offset would place us.</p>\n"},"setDelta":{"!type":"fn(iDeltaX: ?, iDeltaY: ?) -> !this","!doc":"<p>Sets the pointer offset.  You can call this directly to force the\noffset to be in a particular location (e.g., pass in 0,0 to set it\nto the center of the object)</p>\n"},"setDragElPos":{"!type":"fn(iPageX: ?, iPageY: ?) -> !this","!doc":"<p>Sets the drag element to the location of the mousedown or click event,\nmaintaining the cursor location relative to the location on the element\nthat was clicked.  Override this if you want to place the element in a\nlocation other than where the cursor is.</p>\n"},"setLocalXY":{"!type":"fn(el: ?, x: ?, y: ?) -> !this"},"toString":{"!type":"fn() -> string","!doc":"<p>toString method</p>\n"}}},"DDProxy":{"!doc":"<p>A DragDrop implementation that inserts an empty, bordered div into\nthe document that follows the cursor during drag operations.  At the time of\nthe click, the frame div is resized to the dimensions of the linked html\nelement, and moved to the exact location of the linked element.</p>\n\n<p>References to the \"frame\" element refer to the single proxy element that\nwas created to be dragged in place of all DDProxy elements on the\npage.</p>\n","!type":"fn(id: ?, sGroup: ?, config: ?)","prototype":{"centerFrame":{"!type":"bool","!doc":"<p>By default the frame is positioned exactly where the drag element is, so\nwe use the cursor offset provided by <a href=\"#!/api/Ext.dd.DD\" rel=\"Ext.dd.DD\" class=\"docClass\">Ext.dd.DD</a>.  Another option that works only if\nyou do not have constraints on the obj is to have the drag frame centered\naround the cursor.  Set centerFrame to true for this effect.</p>\n"},"resizeFrame":{"!type":"bool","!doc":"<p>By default we resize the drag frame to be the same size as the element\nwe want to drag (this is to get the frame effect).  We can turn it off\nif we want a different behavior.</p>\n"},"dragElId":{"!type":"string","!doc":"<p>The default drag frame div id</p>\n"},"_resizeProxy":{"!type":"fn() -> !this","!doc":"<p>The proxy is automatically resized to the dimensions of the linked\nelement when a drag is initiated, unless resizeFrame is set to false</p>\n"},"afterDrag":{"!type":"fn() -> !this"},"applyConfig":{"!type":"fn() -> !this","!doc":"<p>Sets up config options specific to this class. Overrides\n<a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a>, but all versions of this method through the\ninheritance chain are called</p>\n"},"b4EndDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>overrides <a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a></p>\n"},"b4MouseDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>overrides <a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a></p>\n"},"b4StartDrag":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>overrides <a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a></p>\n"},"beforeMove":{"!type":"fn() -> !this"},"createFrame":{"!type":"fn() -> !this","!doc":"<p>Creates the proxy element if it does not yet exist</p>\n"},"endDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>overrides <a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a>\nBy default we try to move the element to the last location of the frame.\nThis is so that the default behavior mirrors that of <a href=\"#!/api/Ext.dd.DD\" rel=\"Ext.dd.DD\" class=\"docClass\">Ext.dd.DD</a>.</p>\n"},"initFrame":{"!type":"fn() -> !this","!doc":"<p>Initialization for the drag frame element.  Must be called in the\nconstructor of all subclasses</p>\n"},"showFrame":{"!type":"fn(iPageX: ?, iPageY: ?) -> !this","!doc":"<p>Resizes the drag frame to the dimensions of the clicked object, positions\nit over the object, and finally displays it</p>\n"},"toString":{"!type":"fn() -> string","!doc":"<p>toString method</p>\n"}}},"DragDropElement":{"prototype":{"getPosX":{"!type":"fn(el: ?) -> number","!doc":"<p>Returns the X position of an html element</p>\n"},"getPosY":{"!type":"fn(el: ?) -> number","!doc":"<p>Returns the Y position of an html element</p>\n"},"getScroll":{"!type":"fn() -> !this","!doc":"<p>Returns the current scroll position</p>\n"},"getScrollLeft":{"!type":"fn() -> number","!doc":"<p>Gets the scrollLeft</p>\n"},"getScrollTop":{"!type":"fn() -> number","!doc":"<p>Gets the scrollTop</p>\n"},"getStyle":{"!type":"fn(el: ?, styleProp: ?) -> string","!doc":"<p>Returns the specified element style property</p>\n"},"handleWasClicked":{"!type":"fn(node: ?) -> !this","!doc":"<p>Recursively searches the immediate parent and all child nodes for\nthe handle element in order to determine wheter or not it was\nclicked.</p>\n"},"moveToEl":{"!type":"fn(moveEl: ?, targetEl: ?) -> !this","!doc":"<p>Sets the x/y position of an element to the location of the\ntarget element.</p>\n"},"numericSort":{"!type":"fn(a: ?, b: ?) -> number","!doc":"<p>Numeric array sort function</p>\n"},"swapNode":{"!type":"fn(n1: ?, n2: ?) -> !this","!doc":"<p>Swap two nodes.  In IE, we use the native method, for others we\nemulate the IE behavior</p>\n"}}},"DragDropManager":{"ElementWrapper":{"!doc":"<p>Deprecated inner class for cached elements.</p>\n","prototype":{"css":{"!doc":"<p>A reference to the style property</p>\n"},"el":{"!doc":"<p>The element</p>\n"},"id":{"!doc":"<p>The element id</p>\n"}}},"!doc":"<p>DragDropManager is a singleton that tracks the element interaction for\nall DragDrop items in the window.  Generally, you will not call\nthis class directly, but it does have helper methods that could\nbe useful in your DragDrop implementations.</p>\n","INTERSECT":{"!type":"number","!doc":"<p>In intersect mode, drag and drop interaction is defined by the\noverlap of two or more drag and drop objects.</p>\n"},"POINT":{"!type":"number","!doc":"<p>In point mode, drag and drop interaction is defined by the\nlocation of the cursor during the drag/drop</p>\n"},"clickPixelThresh":{"!type":"number","!doc":"<p>The number of pixels that the mouse needs to move after the\nmousedown before the drag is initiated.  Default=8;\ndefaults to the same value used in the LongPress gesture so that drag cannot be\ninitiated if there is a possible pending longpress</p>\n"},"clickTimeout":{"!doc":"<p>Timeout used for the click time threshold</p>\n"},"deltaX":{"!type":"number","!doc":"<p>the X distance between the cursor and the object being dragged</p>\n"},"deltaY":{"!type":"number","!doc":"<p>the Y distance between the cursor and the object being dragged</p>\n"},"dragCls":{"!type":"string","!doc":"<p>Class to add to the <a href=\"#!/api/Ext.dd.DragDrop-method-getDragEl\" rel=\"Ext.dd.DragDrop-method-getDragEl\" class=\"docClass\">dragged element</a> of a DragDrop instance.</p>\n"},"dragCurrent":{"!doc":"<p>the DragDrop object that is currently being dragged</p>\n"},"dragOvers":{"!doc":"<p>the DragDrop object(s) that are being hovered over</p>\n"},"dragThreshMet":{"!type":"bool","!doc":"<p>Flag that indicates that either the drag pixel threshold or the\nmousdown time threshold has been met</p>\n"},"elementCache":{"!doc":"<p>A cache of DOM elements</p>\n"},"handleIds":{"!doc":"<p>Array of element ids defined as drag handles.  Used to determine\nif the element that generated the mousedown event is actually the\nhandle and not the html element itself.</p>\n"},"ids":{"!doc":"<p>Two dimensional Array of registered DragDrop objects.  The first\ndimension is the DragDrop item group, the second the DragDrop\nobject.</p>\n"},"initialized":{"!type":"bool","!doc":"<p>Internal flag that is set to true when drag and drop has been\nintialized</p>\n"},"locationCache":{"!doc":"<p>Location cache that is set for all drag drop objects when a drag is\ninitiated, cleared when the drag is finished.</p>\n"},"locked":{"!type":"bool","!doc":"<p>All drag and drop can be disabled.</p>\n"},"mode":{"!type":"number","!doc":"<p>The current drag and drop mode.  Default: POINT</p>\n"},"notifyOccluded":{"!type":"bool","!doc":"<p>This config is only provided to provide old, usually unwanted drag/drop behaviour.</p>\n\n<p>From ExtJS 4.1.0 onwards, when drop targets are contained in floating, absolutely positioned elements\nsuch as in <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Windows</a>, which may overlap each other, <code>over</code> and <code>drop</code> events\nare only delivered to the topmost drop target at the mouse position.</p>\n\n<p>If all targets below that in zIndex order should also receive notifications, set\n<code>notifyOccluded</code> to <code>true</code>.</p>\n"},"preventDefault":{"!type":"bool","!doc":"<p>Flag to determine if we should prevent the default behavior of the\nevents we define. By default this is true, but this can be set to\nfalse if you need the default behavior (not recommended)</p>\n"},"startX":{"!type":"number","!doc":"<p>The X position of the mousedown event stored for later use when a\ndrag threshold is met.</p>\n"},"startY":{"!type":"number","!doc":"<p>The Y position of the mousedown event stored for later use when a\ndrag threshold is met.</p>\n"},"stopPropagation":{"!type":"bool","!doc":"<p>Flag to determine if we should stop the propagation of the events\nwe generate. This is true by default but you may want to set it to\nfalse if the html element contains other features that require the\nmouse click.</p>\n"},"useCache":{"!type":"bool","!doc":"<p>Set useCache to false if you want to force object the lookup of each\ndrag and drop linked element constantly during a drag.</p>\n"},"_execOnAll":{"!type":"fn(sMethod: ?, args: ?) -> !this","!doc":"<p>Runs method on all drag and drop objects</p>\n"},"_onResize":{"!type":"fn(e: ?) -> !this","!doc":"<p>Reset constraints on all drag and drop objs</p>\n"},"_onUnload":{"!type":"fn(e: ?, me: ?) -> !this","!doc":"<p>unload event handler</p>\n"},"_remove":{"!type":"fn(oDD: ?, clearGroup: ?) -> !this","!doc":"<p>Unregisters a drag and drop item.  This is executed in\nDragDrop.unreg, use that method instead of calling this directly.</p>\n"},"addListeners":{"!type":"fn() -> !this","!doc":"<p>Drag and drop initialization.  Sets up the global event handlers</p>\n"},"byZIndex":{"!type":"fn(d1: ?, d2: ?) -> !this","!doc":"<p>Utility method to pass to Ext.Array.sort when sorting potential drop targets by z-index.</p>\n"},"fireEvents":{"!type":"fn(e: ?, isDrop: ?) -> !this","!doc":"<p>Iterates over all of the DragDrop elements to find ones we are\nhovering over or dropping on</p>\n"},"getBestMatch":{"!type":"fn(dds: ?) -> ?","!doc":"<p>Helper function for getting the best match from the list of drag\nand drop objects returned by the drag and drop events when we are\nin INTERSECT mode.  It returns either the first object that the\ncursor is over, or the object that has the greatest overlap with\nthe dragged element.</p>\n"},"getCss":{"!type":"fn(id: ?) -> ?","!doc":"<p>Returns the style property for the DOM element (i.e.,\ndocument.getElById(id).style)</p>\n"},"getDDById":{"!type":"fn(id: ?) -> ?","!doc":"<p>Returns the DragDrop instance for a given id</p>\n"},"getElWrapper":{"!type":"fn(id: ?) -> ?","!doc":"<p>Get the wrapper for the DOM element specified</p>\n"},"getElement":{"!type":"fn(id: ?) -> ?","!doc":"<p>Returns the actual DOM element</p>\n"},"getLocation":{"!type":"fn(oDD: ?) -> ?","!doc":"<p>Returns a Region object containing the drag and drop element's position\nand size, including the padding configured for it</p>\n"},"getRelated":{"!type":"fn(p_oDD: ?, bTargetsOnly: ?) -> ?","!doc":"<p>Returns the drag and drop instances that are in all groups the\npassed in instance belongs to.</p>\n"},"getZIndex":{"!type":"fn(element: ?) -> number","!doc":"<p>Collects the z-index of the passed element, looking up the parentNode axis to find an absolutely positioned ancestor\nwhich is able to yield a z-index. If found to be not absolutely positionedm returns -1.</p>\n\n<p>This is used when sorting potential drop targets into z-index order so that only the topmost receives <code>over</code> and <code>drop</code> events.</p>\n"},"handleMouseDown":{"!type":"fn(e: ?, oDD: ?) -> !this","!doc":"<p>Fired after a registered DragDrop object gets the mousedown event.\nSets up the events required to track the object being dragged</p>\n"},"handleMouseMove":{"!type":"fn(e: ?) -> !this","!doc":"<p>Internal function to handle the mousemove event.  Will be invoked\nfrom the context of the html element.</p>\n\n<p>TODO: figure out what we can do about mouse events lost when the\nuser drags objects beyond the window boundary.  Currently we can\ndetect this in internet explorer by verifying that the mouse is\ndown during the mousemove event.  Firefox doesn't give us the\nbutton state on the mousemove event.</p>\n"},"handleMouseUp":{"!type":"fn(e: ?) -> !this","!doc":"<p>Internal function to handle the mouseup event.  Will be invoked\nfrom the context of the document.</p>\n"},"init":{"!type":"fn() -> !this","!doc":"<p>Called the first time an element is registered.</p>\n"},"isDragDrop":{"!type":"fn(id: ?) -> bool","!doc":"<p>Utility function to determine if a given element has been\nregistered as a drag drop item.</p>\n"},"isHandle":{"!type":"fn(id: ?) -> bool","!doc":"<p>Utility function to determine if a given element has been\nregistered as a drag drop handle for the given Drag Drop object.</p>\n"},"isLegalTarget":{"!type":"fn(oDD: ?, oTargetDD: ?) -> bool","!doc":"<p>Returns true if the specified dd target is a legal target for\nthe specifice drag obj</p>\n"},"isLocked":{"!type":"fn() -> bool","!doc":"<p>Is drag and drop locked?</p>\n"},"isOverTarget":{"!type":"fn(pt: ?, oTarget: ?) -> bool","!doc":"<p>Checks the cursor location to see if it over the target</p>\n"},"isTypeOfDD":{"!type":"fn(the: ?) -> bool","!doc":"<p>My goal is to be able to transparently determine if an object is\ntypeof DragDrop, and the exact subclass of DragDrop.  typeof\nreturns \"object\", oDD.constructor.toString() always returns\n\"DragDrop\" and not the name of the subclass.  So for now it just\nevaluates a well-known variable in DragDrop.</p>\n"},"lock":{"!type":"fn() -> !this","!doc":"<p>Lock all drag and drop functionality</p>\n"},"preventDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>if a drag/drop operation is currently underway, this method stops the dragstart,\ndrag, and dragend events from propagating down to any other listeners, e.g. scrollers</p>\n"},"refreshCache":{"!type":"fn(groups: ?) -> !this","!doc":"<p>Refreshes the cache of the top-left and bottom-right points of the\ndrag and drop objects in the specified group(s).  This is in the\nformat that is stored in the drag and drop instance, so typical\nusage is:</p>\n\n<pre><code><a href=\"#!/api/Ext.dd.DragDropManager-method-refreshCache\" rel=\"Ext.dd.DragDropManager-method-refreshCache\" class=\"docClass\">Ext.dd.DragDropManager.refreshCache</a>(ddinstance.groups);\n</code></pre>\n\n<p>Alternatively:</p>\n\n<pre><code><a href=\"#!/api/Ext.dd.DragDropManager-method-refreshCache\" rel=\"Ext.dd.DragDropManager-method-refreshCache\" class=\"docClass\">Ext.dd.DragDropManager.refreshCache</a>({group1:true, group2:true});\n</code></pre>\n\n<p>TODO: this really should be an indexed array.  Alternatively this\nmethod could accept both.</p>\n"},"regDragDrop":{"!type":"fn(oDD: ?, sGroup: ?) -> !this","!doc":"<p>Each DragDrop instance must be registered with the DragDropManager.\nThis is executed in DragDrop.init()</p>\n"},"regHandle":{"!type":"fn(sDDId: ?, sHandleId: ?) -> !this","!doc":"<p>Each DragDrop handle element must be registered.  This is done\nautomatically when executing DragDrop.setHandleElId()</p>\n"},"removeDDFromGroup":{"!type":"fn(oDD: ?, sGroup: ?) -> !this","!doc":"<p>Removes the supplied dd instance from the supplied group. Executed\nby DragDrop.removeFromGroup, so don't call this function directly.</p>\n"},"startDrag":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Fired when either the drag pixel threshold or the mousedown hold\ntime threshold has been met.</p>\n"},"stopDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>Internal function to clean up event handlers after the drag\noperation is complete</p>\n"},"stopEvent":{"!type":"fn(e: ?) -> !this","!doc":"<p>Utility to stop event propagation and event default, if these\nfeatures are turned on.</p>\n"},"unlock":{"!type":"fn() -> !this","!doc":"<p>Unlock all drag and drop functionality</p>\n"},"unregAll":{"!type":"fn() -> !this","!doc":"<p>Cleans up the drag and drop events and objects.</p>\n"},"verifyEl":{"!type":"fn(el: ?) -> bool","!doc":"<p>This checks to make sure an element exists and is in the DOM.  The\nmain purpose is to handle cases where innerHTML is used to remove\ndrag and drop objects from the DOM.  IE provides an 'unspecified\nerror' when trying to access the offsetParent of such an element</p>\n"}},"DragSource":{"!doc":"<p>A simple class that provides the basic implementation needed to make any element draggable.</p>\n","!type":"fn(el: ?, config?: ?)","prototype":{"animRepair":{"!type":"bool","!doc":"<p>If true, animates the proxy element back to the position of the handle element used to trigger the drag.</p>\n"},"ddGroup":{"!type":"string","!doc":"<p>A named drag drop group to which this object belongs.  If a group is specified, then this object will only\ninteract with other drag drop objects in the same group.</p>\n"},"dropAllowed":{"!type":"string","!doc":"<p>The CSS class returned to the drag source when drop is allowed.</p>\n"},"dropNotAllowed":{"!type":"string","!doc":"<p>The CSS class returned to the drag source when drop is not allowed.</p>\n"},"repairHighlightColor":{"!type":"string","!doc":"<p>The color to use when visually highlighting the drag source in the afterRepair\nmethod after a failed drop (defaults to light blue). The color must be a 6 digit hex value, without\na preceding '#'.</p>\n"},"dragData":{"!doc":"<p>This property contains the data representing the dragged object. This data is set up by the implementation of the\n<a href=\"#!/api/Ext.dd.DragSource-method-getDragData\" rel=\"Ext.dd.DragSource-method-getDragData\" class=\"docClass\">getDragData</a> method. It must contain a ddel property, but can contain any other data according to the\napplication's needs.</p>\n"},"afterDragDrop":{"!type":"fn(target: ?, e: ?, id: ?)","!doc":"<p>An empty function by default, but provided so that you can perform a custom action\nafter a valid drag drop has occurred by providing an implementation.</p>\n"},"afterDragEnter":{"!type":"fn(target: ?, e: ?, id: ?)","!doc":"<p>An empty function by default, but provided so that you can perform a custom action\nwhen the dragged item enters the drop target by providing an implementation.</p>\n"},"afterDragOut":{"!type":"fn(target: ?, e: ?, id: ?)","!doc":"<p>An empty function by default, but provided so that you can perform a custom action\nafter the dragged item is dragged out of the target without dropping.</p>\n"},"afterDragOver":{"!type":"fn(target: ?, e: ?, id: ?)","!doc":"<p>An empty function by default, but provided so that you can perform a custom action\nwhile the dragged item is over the drop target by providing an implementation.</p>\n"},"afterInvalidDrop":{"!type":"fn(e: ?, id: ?)","!doc":"<p>An empty function by default, but provided so that you can perform a custom action\nafter an invalid drop has occurred by providing an implementation.</p>\n"},"afterRepair":{"!type":"fn() -> !this"},"afterValidDrop":{"!type":"fn(target: ?, e: ?, id: ?)","!doc":"<p>An empty function by default, but provided so that you can perform a custom action\nafter a valid drop has occurred by providing an implementation.</p>\n"},"alignElWithMouse":{"!type":"fn() -> !this","!doc":"<p>Sets the element to the location of the mousedown or click event,\nmaintaining the cursor location relative to the location on the element\nthat was clicked.  Override this if you want to place the element in a\nlocation other than where the cursor is.</p>\n"},"autoOffset":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Sets the pointer offset to the distance between the linked element's top\nleft corner and the location the element was clicked.</p>\n"},"b4EndDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>overrides <a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a></p>\n"},"beforeDragDrop":{"!type":"fn(target: ?, e: ?, id: ?) -> bool","!doc":"<p>An empty function by default, but provided so that you can perform a custom action before the dragged\nitem is dropped onto the target and optionally cancel the onDragDrop.</p>\n"},"beforeDragEnter":{"!type":"fn(target: ?, e: ?, id: ?) -> bool","!doc":"<p>An empty function by default, but provided so that you can perform a custom action\nbefore the dragged item enters the drop target and optionally cancel the onDragEnter.</p>\n"},"beforeDragOut":{"!type":"fn(target: ?, e: ?, id: ?) -> bool","!doc":"<p>An empty function by default, but provided so that you can perform a custom action before the dragged\nitem is dragged out of the target without dropping, and optionally cancel the onDragOut.</p>\n"},"beforeDragOver":{"!type":"fn(target: ?, e: ?, id: ?) -> bool","!doc":"<p>An empty function by default, but provided so that you can perform a custom action\nwhile the dragged item is over the drop target and optionally cancel the onDragOver.</p>\n"},"beforeInvalidDrop":{"!type":"fn(target: ?, e: ?, id: ?) -> bool","!doc":"<p>An empty function by default, but provided so that you can perform a custom action after an invalid\ndrop has occurred.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroy this DragDrop instance</p>\n"},"endDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>overrides <a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a>\nBy default we try to move the element to the last location of the frame.\nThis is so that the default behavior mirrors that of <a href=\"#!/api/Ext.dd.DD\" rel=\"Ext.dd.DD\" class=\"docClass\">Ext.dd.DD</a>.</p>\n"},"getDragData":{"!type":"fn(e: ?) -> ?","!doc":"<p>Returns the data object associated with this drag source</p>\n"},"getProxy":{"!type":"fn() -> ?","!doc":"<p>Returns the drag source's underlying <a href=\"#!/api/Ext.dd.StatusProxy\" rel=\"Ext.dd.StatusProxy\" class=\"docClass\">Ext.dd.StatusProxy</a></p>\n"},"getRepairXY":{"!type":"fn(e: ?, data: ?) -> !this"},"handleMouseDown":{"!type":"fn(e: ?, oDD: ?) -> !this","!doc":"<p>Called when this object is clicked</p>\n"},"hideProxy":{"!type":"fn() -> !this","!doc":"<p>Hides the drag source's <a href=\"#!/api/Ext.dd.StatusProxy\" rel=\"Ext.dd.StatusProxy\" class=\"docClass\">Ext.dd.StatusProxy</a></p>\n"},"onBeforeDrag":{"!type":"fn(data: ?, e: ?) -> bool","!doc":"<p>An empty function by default, but provided so that you can perform a custom action before the initial\ndrag event begins and optionally cancel it.</p>\n"},"onDragDrop":{"!type":"fn(e: ?, id: ?) -> !this","!doc":"<p>Abstract method called when this item is dropped on another DragDrop\nobj</p>\n"},"onDragEnter":{"!type":"fn(e: ?, id: ?) -> !this","!doc":"<p>Abstract method called when this element fist begins hovering over\nanother DragDrop obj</p>\n"},"onDragOut":{"!type":"fn(e: ?, id: ?) -> !this","!doc":"<p>Abstract method called when we are no longer hovering over an element</p>\n"},"onDragOver":{"!type":"fn(e: ?, id: ?) -> !this","!doc":"<p>Abstract method called when this element is hovering over another\nDragDrop obj</p>\n"},"onEndDrag":{"!type":"fn(data: ?, e: ?) -> !this"},"onInitDrag":{"!type":"fn(x: ?, y: ?) -> !this"},"onInvalidDrop":{"!type":"fn(target: ?, e: ?, id: ?) -> !this","!doc":"<p>Abstract method called when this item is dropped on an area with no\ndrop target</p>\n"},"onStartDrag":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>An empty function by default, but provided so that you can perform a custom action once the initial\ndrag event has begun.  The drag cannot be canceled from this function.</p>\n"},"onValidDrop":{"!type":"fn(target: ?, e: ?, id: ?) -> !this"},"startDrag":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Abstract method called after a drag/drop object is clicked\nand the drag or mousedown time thresholds have beeen met.</p>\n"},"triggerCacheRefresh":{"!type":"fn() -> !this"}}},"DragTracker":{"!doc":"<p>A DragTracker listens for drag events on an Element and fires events at the start and end of the drag,\nas well as during the drag. This is useful for components such as <a href=\"#!/api/Ext.slider.Multi\" rel=\"Ext.slider.Multi\" class=\"docClass\">Ext.slider.Multi</a>, where there is\nan element that can be dragged around to change the Slider's value.</p>\n\n<p>DragTracker provides a series of template methods that should be overridden to provide functionality\nin response to detected drag operations. These are onBeforeStart, onStart, onDrag and onEnd.\nSee <a href=\"#!/api/Ext.slider.Multi\" rel=\"Ext.slider.Multi\" class=\"docClass\">Ext.slider.Multi</a>'s initEvents function for an example implementation.</p>\n","prototype":{"autoStart":{"!doc":"<p>Specify <code>true</code> to defer trigger start by 1000 ms.\nSpecify a Number for the number of milliseconds to defer trigger start.</p>\n"},"constrainTo":{"!doc":"<p>A Region (Or an element from which a Region measurement will be read)\nwhich is used to constrain the result of the <a href=\"#!/api/Ext.dd.DragTracker-method-getOffset\" rel=\"Ext.dd.DragTracker-method-getOffset\" class=\"docClass\">getOffset</a> call.</p>\n\n<p>This may be set any time during the DragTracker's lifecycle to set a dynamic constraining region.</p>\n"},"delegate":{"!type":"string","!doc":"<p>A CSS selector which identifies child elements within the DragTracker's encapsulating\nElement which are the tracked elements. This limits tracking to only begin when the matching elements are mousedowned.</p>\n\n<p>This may also be a specific child element within the DragTracker's encapsulating element to use as the tracked element.</p>\n"},"overCls":{"!type":"string","!doc":"<p>A CSS class to add to the DragTracker's target element when the element (or, if the <a href=\"#!/api/Ext.dd.DragTracker-cfg-delegate\" rel=\"Ext.dd.DragTracker-cfg-delegate\" class=\"docClass\">delegate</a>\noption is used, when a delegate element) is mouseovered.</p>\n\n<p>If the <a href=\"#!/api/Ext.dd.DragTracker-cfg-delegate\" rel=\"Ext.dd.DragTracker-cfg-delegate\" class=\"docClass\">delegate</a> option is used, these events fire only when a delegate element is entered of left.</p>\n"},"preventDefault":{"!type":"bool","!doc":"<p>Specify <code>false</code> to enable default actions on onMouseDown events.</p>\n"},"stopEvent":{"!type":"bool","!doc":"<p>Specify <code>true</code> to stop the <code>mousedown</code> event from bubbling to outer listeners from the target element (or its delegates).</p>\n"},"tolerance":{"!type":"number","!doc":"<p>Number of pixels the drag target must be moved before dragging is\nconsidered to have started.</p>\n"},"trackOver":{"!type":"bool","!doc":"<p>Set to true to fire mouseover and mouseout events when the mouse enters or leaves the target element.</p>\n\n<p>This is implicitly set when an <a href=\"#!/api/Ext.dd.DragTracker-cfg-overCls\" rel=\"Ext.dd.DragTracker-cfg-overCls\" class=\"docClass\">overCls</a> is specified.</p>\n\n<p>If the <a href=\"#!/api/Ext.dd.DragTracker-cfg-delegate\" rel=\"Ext.dd.DragTracker-cfg-delegate\" class=\"docClass\">delegate</a> option is used, these events fire only when a delegate element is entered of left.</p>\n"},"active":{"!type":"bool","!doc":"<p>Indicates whether the user is currently dragging this tracker.</p>\n"},"constrainModes":{},"dragTarget":{"!doc":"<p>The element being dragged.</p>\n\n<p>Only valid during drag operations.</p>\n\n<p>If the <a href=\"#!/api/Ext.dd.DragTracker-cfg-delegate\" rel=\"Ext.dd.DragTracker-cfg-delegate\" class=\"docClass\">delegate</a> option is used, this will be the delegate element which was mousedowned.</p>\n"},"clearStart":{"!type":"fn() -> !this"},"destroy":{"!type":"fn() -> !this"},"disable":{"!type":"fn() -> !this"},"enable":{"!type":"fn() -> !this"},"endDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>Stop the drag operation, and remove active mouse listeners.</p>\n"},"getConstrainRegion":{"!type":"fn() -> !this","!doc":"<p>Return the Region into which the drag operation is constrained.\nEither the XY pointer itself can be constrained, or the dragTarget element\nThe private property _constrainRegion is cached until onMouseUp</p>\n"},"getDragCt":{"!type":"fn() -> ?"},"getDragTarget":{"!type":"fn() -> ?","!doc":"<p>Returns the drag target. This is usually the DragTracker's encapsulating element.</p>\n\n<p>If the <a href=\"#!/api/Ext.dd.DragTracker-cfg-delegate\" rel=\"Ext.dd.DragTracker-cfg-delegate\" class=\"docClass\">delegate</a> option is being used, this may be a child element which matches the\n<a href=\"#!/api/Ext.dd.DragTracker-cfg-delegate\" rel=\"Ext.dd.DragTracker-cfg-delegate\" class=\"docClass\">delegate</a> selector.</p>\n"},"getOffset":{"!type":"fn(constrainMode?: ?) -> ?","!doc":"<p>Returns the X, Y offset of the current mouse position from the mousedown point.</p>\n\n<p>This method may optionally constrain the real offset values, and returns a point coerced in one\nof two modes:</p>\n\n<ul>\n<li><code>point</code>\nThe current mouse position is coerced into the constrainRegion and the resulting position is returned.</li>\n<li><code>dragTarget</code>\nThe new Region of the <a href=\"#!/api/Ext.dd.DragTracker-method-getDragTarget\" rel=\"Ext.dd.DragTracker-method-getDragTarget\" class=\"docClass\">dragTarget</a> is calculated\nbased upon the current mouse position, and then coerced into the constrainRegion. The returned\nmouse position is then adjusted by the same delta as was used to coerce the region.</li>\n</ul>\n\n"},"getXY":{"!type":"fn(constrain: ?) -> !this"},"initEl":{"!type":"fn(el: ?) -> !this","!doc":"<p>Initializes the DragTracker on a given element.</p>\n"},"onBeforeStart":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method which should be overridden by each DragTracker instance. Called when the user first clicks and\nholds the mouse button down. Return false to disallow the drag</p>\n"},"onDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method which should be overridden by each DragTracker instance. Called whenever a drag has been detected.</p>\n"},"onDragStart":{"!type":"fn(e: ?) -> !this"},"onEnd":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method which should be overridden by each DragTracker instance. Called when a drag operation has been completed\n(e.g. the user clicked and held the mouse down, dragged the element and then released the mouse button)</p>\n"},"onMouseDown":{"!type":"fn(e: ?, target: ?) -> !this"},"onMouseMove":{"!type":"fn(e: ?, target: ?) -> !this"},"onMouseOut":{"!type":"fn(e: ?) -> !this","!doc":"<p>When the pointer exits a tracking element, fire a mouseout.\nThis is mouseleave functionality, but we cannot use mouseleave because we are using \"delegate\" to filter mouse targets</p>\n"},"onMouseOver":{"!type":"fn(e: ?, target: ?) -> !this","!doc":"<p>When the pointer enters a tracking element, fire a mouseover if the mouse entered from outside.\nThis is mouseenter functionality, but we cannot use mouseenter because we are using \"delegate\" to filter mouse targets</p>\n"},"onMouseUp":{"!type":"fn(e: ?) -> !this"},"onStart":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method which should be overridden by each DragTracker instance. Called when a drag operation starts\n(e.g. the user has moved the tracked element beyond the specified tolerance)</p>\n"},"stopSelect":{"!type":"fn(e: ?) -> !this"},"triggerStart":{"!type":"fn(e: ?) -> !this"},"beforestart":{"!type":"fn(this: ?, e: ?, eOpts: ?)"},"drag":{"!type":"fn(this: ?, e: ?, eOpts: ?)"},"dragend":{"!type":"fn(this: ?, e: ?, eOpts: ?)"},"dragstart":{"!type":"fn(this: ?, e: ?, eOpts: ?)"},"mousedown":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse button is pressed down, but before a drag operation begins. The\ndrag operation begins after either the mouse has been moved by <a href=\"#!/api/Ext.dd.DragTracker-cfg-tolerance\" rel=\"Ext.dd.DragTracker-cfg-tolerance\" class=\"docClass\">tolerance</a> pixels,\nor after the <a href=\"#!/api/Ext.dd.DragTracker-cfg-autoStart\" rel=\"Ext.dd.DragTracker-cfg-autoStart\" class=\"docClass\">autoStart</a> timer fires.</p>\n\n<p>Return <code>false</code> to veto the drag operation.</p>\n"},"mousemove":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the mouse is moved. Returning false cancels the drag operation.</p>\n"},"mouseout":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse exits the DragTracker's target element (or if <a href=\"#!/api/Ext.dd.DragTracker-cfg-delegate\" rel=\"Ext.dd.DragTracker-cfg-delegate\" class=\"docClass\">delegate</a> is\nused, when the mouse exits a delegate element).</p>\n\n<p><strong>Only available when <a href=\"#!/api/Ext.dd.DragTracker-cfg-trackOver\" rel=\"Ext.dd.DragTracker-cfg-trackOver\" class=\"docClass\">trackOver</a> is <code>true</code></strong></p>\n"},"mouseover":{"!type":"fn(this: ?, e: ?, target: ?, eOpts: ?)","!doc":"<p>Fires when the mouse enters the DragTracker's target element (or if <a href=\"#!/api/Ext.dd.DragTracker-cfg-delegate\" rel=\"Ext.dd.DragTracker-cfg-delegate\" class=\"docClass\">delegate</a> is\nused, when the mouse enters a delegate element).</p>\n\n<p><strong>Only available when <a href=\"#!/api/Ext.dd.DragTracker-cfg-trackOver\" rel=\"Ext.dd.DragTracker-cfg-trackOver\" class=\"docClass\">trackOver</a> is <code>true</code></strong></p>\n"},"mouseup":{"!type":"fn(this: ?, e: ?, eOpts: ?)"}}},"DragZone":{"!doc":"<p>This class provides a container DD instance that allows dragging of multiple child source nodes.</p>\n\n<p>This class does not move the drag target nodes, but a proxy element which may contain any DOM structure you wish. The\nDOM element to show in the proxy is provided by either a provided implementation of <a href=\"#!/api/Ext.dd.DragZone-method-getDragData\" rel=\"Ext.dd.DragZone-method-getDragData\" class=\"docClass\">getDragData</a>, or by\nregistered draggables registered with <a href=\"#!/api/Ext.dd.Registry\" rel=\"Ext.dd.Registry\" class=\"docClass\">Ext.dd.Registry</a></p>\n\n<p>If you wish to provide draggability for an arbitrary number of DOM nodes, each of which represent some application\nobject (For example nodes in a <a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">DataView</a>) then use of this class is the most efficient way to\n\"activate\" those nodes.</p>\n\n<p>By default, this class requires that draggable child nodes are registered with <a href=\"#!/api/Ext.dd.Registry\" rel=\"Ext.dd.Registry\" class=\"docClass\">Ext.dd.Registry</a>. However a\nsimpler way to allow a DragZone to manage any number of draggable elements is to configure the DragZone with an\nimplementation of the <a href=\"#!/api/Ext.dd.DragZone-method-getDragData\" rel=\"Ext.dd.DragZone-method-getDragData\" class=\"docClass\">getDragData</a> method which interrogates the passed mouse event to see if it has taken\nplace within an element, or class of elements. This is easily done by using the event's getTarget method to identify a node based on a CSS selector. For example,\nto make the nodes of a DataView draggable, use the following technique. Knowledge of the use of the DataView is\nrequired:</p>\n\n<pre><code>myDataView.on('render', function(v) {\n    myDataView.dragZone = new <a href=\"#!/api/Ext.dd.DragZone\" rel=\"Ext.dd.DragZone\" class=\"docClass\">Ext.dd.DragZone</a>(v.getEl(), {\n\n//      On receipt of a mousedown event, see if it is within a DataView node.\n//      Return a drag data object if so.\n        getDragData: function(e) {\n\n//          Use the DataView's own itemSelector (a mandatory property) to\n//          test if the mousedown is within one of the DataView's nodes.\n            var sourceEl = e.getTarget(v.itemSelector, 10);\n\n//          If the mousedown is within a DataView node, clone the node to produce\n//          a ddel element for use by the drag proxy. Also add application data\n//          to the returned data object.\n            if (sourceEl) {\n                d = sourceEl.cloneNode(true);\n                d.id = Ext.id();\n                return {\n                    ddel: d,\n                    sourceEl: sourceEl,\n                    repairXY: Ext.fly(sourceEl).getXY(),\n                    sourceStore: v.store,\n                    draggedRecord: v.<a href=\"#!/api/Ext.view.View-method-getRecord\" rel=\"Ext.view.View-method-getRecord\" class=\"docClass\">getRecord</a>(sourceEl)\n                }\n            }\n        },\n\n//      Provide coordinates for the proxy to slide back to on failed drag.\n//      This is the original XY coordinates of the draggable element captured\n//      in the getDragData method.\n        getRepairXY: function() {\n            return this.dragData.repairXY;\n        }\n    });\n});\n</code></pre>\n\n<p>See the <a href=\"#!/api/Ext.dd.DropZone\" rel=\"Ext.dd.DropZone\" class=\"docClass\">DropZone</a> documentation for details about building a DropZone which cooperates with\nthis DragZone.</p>\n","!type":"fn(el: ?, config: ?)","prototype":{"containerScroll":{"!doc":"<p>True to register this container with the Scrollmanager for auto scrolling during drag operations.\nA <a href=\"#!/api/Ext.dd.ScrollManager\" rel=\"Ext.dd.ScrollManager\" class=\"docClass\">Ext.dd.ScrollManager</a> configuration may also be passed.</p>\n"},"scrollEl":{"!doc":"<p>An element to register with the ScrollManager if <a href=\"#!/api/Ext.dd.DragZone-cfg-containerScroll\" rel=\"Ext.dd.DragZone-cfg-containerScroll\" class=\"docClass\">containerScroll</a>\nis set. Defaults to the drag element.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroy this DragDrop instance</p>\n"},"getDragData":{"!type":"fn(e: ?) -> ?","!doc":"<p>Called when a mousedown occurs in this container. Looks in <a href=\"#!/api/Ext.dd.Registry\" rel=\"Ext.dd.Registry\" class=\"docClass\">Ext.dd.Registry</a> for a valid target to drag\nbased on the mouse down. Override this method to provide your own lookup logic (e.g. finding a child by class\nname). Make sure your returned object has a \"ddel\" attribute (with an HTML Element) for other functions to work.</p>\n"},"getRepairXY":{"!type":"fn(e: ?) -> ?","!doc":"<p>Called before a repair of an invalid drop to get the XY to animate to. By default returns the XY of\nthis.dragData.ddel</p>\n"},"onInitDrag":{"!type":"fn(x: ?, y: ?) -> bool","!doc":"<p>Called once drag threshold has been reached to initialize the proxy element. By default, it clones the\nthis.dragData.ddel</p>\n"}}},"DropZone":{"!doc":"<p>This class provides a container DD instance that allows dropping on multiple child target nodes.</p>\n\n<p>By default, this class requires that child nodes accepting drop are registered with <a href=\"#!/api/Ext.dd.Registry\" rel=\"Ext.dd.Registry\" class=\"docClass\">Ext.dd.Registry</a>.\nHowever a simpler way to allow a DropZone to manage any number of target elements is to configure the\nDropZone with an implementation of <a href=\"#!/api/Ext.dd.DropZone-method-getTargetFromEvent\" rel=\"Ext.dd.DropZone-method-getTargetFromEvent\" class=\"docClass\">getTargetFromEvent</a> which interrogates the passed\nmouse event to see if it has taken place within an element, or class of elements. This is easily done\nby using the event's getTarget method to identify a node based on a\nCSS selector.</p>\n\n<p>Once the DropZone has detected through calling getTargetFromEvent, that the mouse is over\na drop target, that target is passed as the first parameter to <a href=\"#!/api/Ext.dd.DropZone-method-onNodeEnter\" rel=\"Ext.dd.DropZone-method-onNodeEnter\" class=\"docClass\">onNodeEnter</a>, <a href=\"#!/api/Ext.dd.DropZone-method-onNodeOver\" rel=\"Ext.dd.DropZone-method-onNodeOver\" class=\"docClass\">onNodeOver</a>,\n<a href=\"#!/api/Ext.dd.DropZone-method-onNodeOut\" rel=\"Ext.dd.DropZone-method-onNodeOut\" class=\"docClass\">onNodeOut</a>, <a href=\"#!/api/Ext.dd.DropZone-method-onNodeDrop\" rel=\"Ext.dd.DropZone-method-onNodeDrop\" class=\"docClass\">onNodeDrop</a>. You may configure the instance of DropZone with implementations\nof these methods to provide application-specific behaviour for these events to update both\napplication state, and UI state.</p>\n\n<p>For example to make a GridPanel a cooperating target with the example illustrated in\n<a href=\"#!/api/Ext.dd.DragZone\" rel=\"Ext.dd.DragZone\" class=\"docClass\">DragZone</a>, the following technique might be used:</p>\n\n<pre><code>myGridPanel.on('render', function() {\n    myGridPanel.dropZone = new <a href=\"#!/api/Ext.dd.DropZone\" rel=\"Ext.dd.DropZone\" class=\"docClass\">Ext.dd.DropZone</a>(myGridPanel.getView().scroller, {\n\n        // If the mouse is over a grid row, return that node. This is\n        // provided as the \"target\" parameter in all \"onNodeXXXX\" node event handling functions\n        getTargetFromEvent: function(e) {\n            return e.getTarget(myGridPanel.getView().rowSelector);\n        },\n\n        // On entry into a target node, highlight that node.\n        onNodeEnter : function(target, dd, e, data){\n            Ext.fly(target).addCls('my-row-highlight-class');\n        },\n\n        // On exit from a target node, unhighlight that node.\n        onNodeOut : function(target, dd, e, data){\n            Ext.fly(target).removeCls('my-row-highlight-class');\n        },\n\n        // While over a target node, return the default drop allowed class which\n        // places a \"tick\" icon into the drag proxy.\n        onNodeOver : function(target, dd, e, data){\n            return Ext.dd.DropZone.prototype.dropAllowed;\n        },\n\n        // On node drop we can interrogate the target to find the underlying\n        // application object that is the real target of the dragged data.\n        // In this case, it is a Record in the GridPanel's Store.\n        // We can use the data set up by the DragZone's getDragData method to read\n        // any data we decided to attach in the DragZone's getDragData method.\n        onNodeDrop : function(target, dd, e, data){\n            var rowIndex = myGridPanel.getView().findRowIndex(target);\n            var r = myGridPanel.getStore().getAt(rowIndex);\n            <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Drop gesture', 'Dropped Record id ' + data.draggedRecord.id +\n                ' on Record id ' + r.id);\n            return true;\n        }\n    });\n}\n</code></pre>\n\n<p>See the <a href=\"#!/api/Ext.dd.DragZone\" rel=\"Ext.dd.DragZone\" class=\"docClass\">DragZone</a> documentation for details about building a DragZone which\ncooperates with this DropZone.</p>\n","!type":"fn(el: ?, config: ?)","prototype":{"getTargetFromEvent":{"!type":"fn(e: ?) -> ?","!doc":"<p>Returns a custom data object associated with the DOM node that is the target of the event.  By default\nthis looks up the event target in the <a href=\"#!/api/Ext.dd.Registry\" rel=\"Ext.dd.Registry\" class=\"docClass\">Ext.dd.Registry</a>, although you can override this method to\nprovide your own custom lookup.</p>\n"},"notifyDrop":{"!type":"fn(source: ?, e: ?, data: ?) -> bool","!doc":"<p>The function a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> calls once to notify this drop zone that the dragged item has\nbeen dropped on it.  The drag zone will look up the target node based on the event passed in, and if there\nis a node registered for that event, it will delegate to <a href=\"#!/api/Ext.dd.DropZone-method-onNodeDrop\" rel=\"Ext.dd.DropZone-method-onNodeDrop\" class=\"docClass\">onNodeDrop</a> for node-specific handling,\notherwise it will call <a href=\"#!/api/Ext.dd.DropZone-method-onContainerDrop\" rel=\"Ext.dd.DropZone-method-onContainerDrop\" class=\"docClass\">onContainerDrop</a>.</p>\n"},"notifyEnter":{"!type":"fn(source: ?, e: ?, data: ?) -> string","!doc":"<p>The function a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> calls once to notify this drop zone that the source is now over\nthe zone.  The default implementation returns this.dropNotAllowed and expects that only registered drop\nnodes can process drag drop operations, so if you need the drop zone itself to be able to process drops\nyou should override this method and provide a custom implementation.</p>\n"},"notifyOut":{"!type":"fn(source: ?, e: ?, data: ?) -> !this","!doc":"<p>The function a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> calls once to notify this drop zone that the source has been dragged\nout of the zone without dropping.  If the drag source is currently over a registered node, the notification\nwill be delegated to <a href=\"#!/api/Ext.dd.DropZone-method-onNodeOut\" rel=\"Ext.dd.DropZone-method-onNodeOut\" class=\"docClass\">onNodeOut</a> for node-specific handling, otherwise it will be ignored.</p>\n"},"notifyOver":{"!type":"fn(source: ?, e: ?, data: ?) -> string","!doc":"<p>The function a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> calls continuously while it is being dragged over the drop zone.\nThis method will be called on every mouse movement while the drag source is over the drop zone.\nIt will call <a href=\"#!/api/Ext.dd.DropZone-method-onNodeOver\" rel=\"Ext.dd.DropZone-method-onNodeOver\" class=\"docClass\">onNodeOver</a> while the drag source is over a registered node, and will also automatically\ndelegate to the appropriate node-specific methods as necessary when the drag source enters and exits\nregistered nodes (<a href=\"#!/api/Ext.dd.DropZone-method-onNodeEnter\" rel=\"Ext.dd.DropZone-method-onNodeEnter\" class=\"docClass\">onNodeEnter</a>, <a href=\"#!/api/Ext.dd.DropZone-method-onNodeOut\" rel=\"Ext.dd.DropZone-method-onNodeOut\" class=\"docClass\">onNodeOut</a>). If the drag source is not currently over a\nregistered node, it will call <a href=\"#!/api/Ext.dd.DropZone-method-onContainerOver\" rel=\"Ext.dd.DropZone-method-onContainerOver\" class=\"docClass\">onContainerOver</a>.</p>\n"},"onContainerDrop":{"!type":"fn(source: ?, e: ?, data: ?) -> bool","!doc":"<p>Called when the DropZone determines that a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> has been dropped on it,\nbut not on any of its registered drop nodes.  The default implementation returns false, so it should be\noverridden to provide the appropriate processing of the drop event if you need the drop zone itself to\nbe able to accept drops.  It should return true when valid so that the drag source's repair action does not run.</p>\n"},"onContainerOver":{"!type":"fn(source: ?, e: ?, data: ?) -> string","!doc":"<p>Called while the DropZone determines that a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> is being dragged over it,\nbut not over any of its registered drop nodes.  The default implementation returns this.dropNotAllowed, so\nit should be overridden to provide the proper feedback if necessary.</p>\n"},"onNodeDrop":{"!type":"fn(nodeData: ?, source: ?, e: ?, data: ?) -> bool","!doc":"<p>Called when the DropZone determines that a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> has been dropped onto\nthe drop node.  The default implementation returns false, so it should be overridden to provide the\nappropriate processing of the drop event and return true so that the drag source's repair action does not run.</p>\n"},"onNodeEnter":{"!type":"fn(nodeData: ?, source: ?, e: ?, data: ?) -> !this","!doc":"<p>Called when the DropZone determines that a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> has entered a drop node\nthat has either been registered or detected by a configured implementation of <a href=\"#!/api/Ext.dd.DropZone-method-getTargetFromEvent\" rel=\"Ext.dd.DropZone-method-getTargetFromEvent\" class=\"docClass\">getTargetFromEvent</a>.\nThis method has no default implementation and should be overridden to provide\nnode-specific processing if necessary.</p>\n"},"onNodeOut":{"!type":"fn(nodeData: ?, source: ?, e: ?, data: ?) -> !this","!doc":"<p>Called when the DropZone determines that a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> has been dragged out of\nthe drop node without dropping.  This method has no default implementation and should be overridden to provide\nnode-specific processing if necessary.</p>\n"},"onNodeOver":{"!type":"fn(nodeData: ?, source: ?, e: ?, data: ?) -> string","!doc":"<p>Called while the DropZone determines that a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> is over a drop node\nthat has either been registered or detected by a configured implementation of <a href=\"#!/api/Ext.dd.DropZone-method-getTargetFromEvent\" rel=\"Ext.dd.DropZone-method-getTargetFromEvent\" class=\"docClass\">getTargetFromEvent</a>.\nThe default implementation returns this.dropAllowed, so it should be\noverridden to provide the proper feedback.</p>\n"},"triggerCacheRefresh":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"}}},"Registry":{"!doc":"<p>Provides easy access to all drag drop components that are registered on a page. Items can be retrieved either\ndirectly by DOM node id, or by passing in the drag drop event that occurred and looking up the event target.</p>\n","!type":"fn()","getHandle":{"!type":"fn(id: ?) -> ?","!doc":"<p>Returns the handle registered for a DOM Node by id</p>\n"},"getHandleFromEvent":{"!type":"fn(e: ?) -> ?","!doc":"<p>Returns the handle that is registered for the DOM node that is the target of the event</p>\n"},"getId":{"!type":"fn(el: ?, autogen: ?) -> !this"},"getTarget":{"!type":"fn(id: ?) -> ?","!doc":"<p>Returns a custom data object that is registered for a DOM node by id</p>\n"},"getTargetFromEvent":{"!type":"fn(e: ?) -> ?","!doc":"<p>Returns a custom data object that is registered for the DOM node that is the target of the event</p>\n"},"register":{"!type":"fn(element: ?, data: ?) -> !this","!doc":"<p>Registers a drag drop element.</p>\n"},"unregister":{"!type":"fn(element: ?) -> !this","!doc":"<p>Unregister a drag drop element</p>\n"}},"ScrollManager":{"!doc":"<p>Provides automatic scrolling of overflow regions in the page during drag operations.</p>\n\n<p>The ScrollManager configs will be used as the defaults for any scroll container registered with it, but you can also\noverride most of the configs per scroll container by adding a ddScrollConfig object to the target element that\ncontains these properties: <a href=\"#!/api/Ext.dd.ScrollManager-property-hthresh\" rel=\"Ext.dd.ScrollManager-property-hthresh\" class=\"docClass\">hthresh</a>, <a href=\"#!/api/Ext.dd.ScrollManager-property-vthresh\" rel=\"Ext.dd.ScrollManager-property-vthresh\" class=\"docClass\">vthresh</a>, <a href=\"#!/api/Ext.dd.ScrollManager-property-increment\" rel=\"Ext.dd.ScrollManager-property-increment\" class=\"docClass\">increment</a> and <a href=\"#!/api/Ext.dd.ScrollManager-property-frequency\" rel=\"Ext.dd.ScrollManager-property-frequency\" class=\"docClass\">frequency</a>. Example\nusage:</p>\n\n<pre><code>var el = Ext.get('scroll-ct');\nel.ddScrollConfig = {\n    vthresh: 50,\n    hthresh: -1,\n    frequency: 100,\n    increment: 200\n};\n<a href=\"#!/api/Ext.dd.ScrollManager-method-register\" rel=\"Ext.dd.ScrollManager-method-register\" class=\"docClass\">Ext.dd.ScrollManager.register</a>(el);\n</code></pre>\n\n<p>Note: This class is designed to be used in \"Point Mode</p>\n","!type":"fn()","animDuration":{"!type":"number","!doc":"<p>The animation duration in seconds - MUST BE less than <a href=\"#!/api/Ext.dd.ScrollManager-property-frequency\" rel=\"Ext.dd.ScrollManager-property-frequency\" class=\"docClass\">Ext.dd.ScrollManager.frequency</a>!</p>\n"},"animate":{"!type":"bool","!doc":"<p>True to animate the scroll</p>\n"},"ddGroup":{"!type":"string","!doc":"<p>The named drag drop <a href=\"#!/api/Ext.dd.DragSource-cfg-ddGroup\" rel=\"Ext.dd.DragSource-cfg-ddGroup\" class=\"docClass\">group</a> to which this container belongs. If a ddGroup is\nspecified, then container scrolling will only occur when a dragged object is in the same ddGroup.</p>\n"},"dirTrans":{},"frequency":{"!type":"number","!doc":"<p>The frequency of scrolls in milliseconds</p>\n"},"hthresh":{"!doc":"<p>The number of pixels from the right or left edge of a container the pointer needs to be to trigger scrolling</p>\n"},"increment":{"!type":"number","!doc":"<p>The number of pixels to scroll in each scroll increment</p>\n"},"vthresh":{"!doc":"<p>The number of pixels from the top or bottom edge of a container the pointer needs to be to trigger scrolling</p>\n"},"clearProc":{"!type":"fn() -> !this"},"doScroll":{"!type":"fn() -> !this"},"onFire":{"!type":"fn(e: ?, isDrop: ?) -> !this"},"onStop":{"!type":"fn(e: ?) -> !this"},"refreshCache":{"!type":"fn() -> !this","!doc":"<p>Manually trigger a cache refresh.</p>\n"},"register":{"!type":"fn(el: ?) -> !this","!doc":"<p>Registers new overflow element(s) to auto scroll</p>\n"},"startProc":{"!type":"fn(el: ?, dir: ?) -> !this"},"triggerRefresh":{"!type":"fn() -> !this"},"unregister":{"!type":"fn(el: ?) -> !this","!doc":"<p>Unregisters overflow element(s) so they are no longer scrolled</p>\n"}},"StatusProxy":{"!doc":"<p>A specialized floating Component that supports a drop status icon, <a href=\"#!/api/Ext.dom.Layer\" rel=\"Ext.dom.Layer\" class=\"docClass\">Ext.Layer</a> styles\nand auto-repair.  This is the default drag proxy used by all Ext.dd components.</p>\n","!type":"fn(config?: ?)","prototype":{"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"dropAllowed":{"!type":"string","!doc":"<p>The CSS class to apply to the status element when drop is allowed.</p>\n"},"dropNotAllowed":{"!type":"string","!doc":"<p>The CSS class to apply to the status element when drop is not allowed.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.dd.StatusProxy-method-getEl\" rel=\"Ext.dd.StatusProxy-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.dd.StatusProxy-method-getEl\" rel=\"Ext.dd.StatusProxy-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.dd.StatusProxy-cfg-renderSelectors\" rel=\"Ext.dd.StatusProxy-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.dd.StatusProxy-cfg-childEls\" rel=\"Ext.dd.StatusProxy-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"animRepair":{"!type":"bool"},"ariaRole":{"!type":"string"},"repairCls":{"!type":"string"},"afterRepair":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"getGhost":{"!type":"fn() -> ?","!doc":"<p>Returns the ghost element</p>\n"},"hide":{"!type":"fn(clear: ?) -> !this","!doc":"<p>Hides the proxy</p>\n"},"repair":{"!type":"fn(xy: ?, callback: ?, scope: ?) -> !this","!doc":"<p>Causes the proxy to return to its position of origin via an animation.\nShould be called after an invalid drop operation by the item being dragged.</p>\n"},"reset":{"!type":"fn(clearGhost: ?) -> !this","!doc":"<p>Resets the status indicator to the default dropNotAllowed value</p>\n"},"setStatus":{"!type":"fn(cssClass: ?) -> !this","!doc":"<p>Updates the proxy's visual element to indicate the status of whether or not drop is allowed\nover the current target element.</p>\n"},"stop":{"!type":"fn() -> !this","!doc":"<p>Stops the repair animation if it's currently running</p>\n"},"sync":{"!type":"fn() -> !this","!doc":"<p>Force the Layer to sync its shadow and shim positions to the element</p>\n"},"update":{"!type":"fn(html: ?) -> !this","!doc":"<p>Updates the contents of the ghost element</p>\n"}}}},"dom":{"ButtonElement":{"!doc":"<p>A special Ext.dom.Element used by Buttons.  Since buttons use <code>display:table</code> elements\nfor their layout, some special adjustments are needed when the width switches from\nauto to fixed width and back.</p>\n\n<p>When the button has a width other than <code>auto</code>, and a right-aligned arrow, long button\ntext will cause the arrow to disappear off the right edge of the button if the btnWrap\nelement has table-layout:auto.  To prevent this, we need to set table-layout:fixed\non the btnWrap when the button has a width, however, when the button is shrinkwrap\nwidth the btnWrap cannot have table-layout:fixed because its width:100% style will\ncause the button to expand outward.</p>\n\n<p>Additionally, in shrinkWrap height mode, the button css sets a height on the btnEl\nelement, but if the height is being stretched, the btnEl's height will cause the contents\nto be incorrectly vertically centered, so we dynamically set the btnEl's height to\n\"auto\" in fixed-height mode.</p>\n","prototype":{"setHeight":{"!type":"fn(height: ?, animate: ?) -> ?"},"setSize":{"!type":"fn(width: ?, height: ?, animate: ?) -> !this"},"setStyle":{"!type":"fn(prop: ?, value: ?) -> !this"},"setWidth":{"!type":"fn(width: ?, animate: ?) -> ?"}}},"Layer":{"!doc":"<p>An extended Ext.dom.Element object that supports a shadow, constrain to viewport\nand automatic maintaining of shadow position.</p>\n","!type":"fn(config?: ?, existingEl?: ?)","prototype":{"cls":{"!type":"string","!doc":"<p>CSS class to add to the element</p>\n"},"constrain":{"!type":"bool","!doc":"<p>False to disable constrain to viewport.</p>\n"},"dh":{"!doc":"<p>DomHelper object config to create element with.</p>\n"},"hideMode":{"!type":"string","!doc":"<p>A String which specifies how this Layer will be hidden.\nValues may be:</p>\n\n<ul>\n<li><code>'display'</code> : The Component will be hidden using the <code>display: none</code> style.</li>\n<li><code>'visibility'</code> : The Component will be hidden using the <code>visibility: hidden</code> style.</li>\n<li><code>'offsets'</code> : The Component will be hidden by absolutely positioning it out of the visible area\nof the document. This is useful when a hidden Component must maintain measurable dimensions.\nHiding using <code>display</code> results in a Component having zero dimensions.</li>\n</ul>\n\n"},"shadow":{"!doc":"<p>True to automatically create an <a href=\"#!/api/Ext.Shadow\" rel=\"Ext.Shadow\" class=\"docClass\">Ext.Shadow</a>, or a string indicating the\nshadow's display <a href=\"#!/api/Ext.Shadow-cfg-mode\" rel=\"Ext.Shadow-cfg-mode\" class=\"docClass\">Ext.Shadow.mode</a>. False to disable the shadow.</p>\n"},"shadowOffset":{"!type":"number","!doc":"<p>Number of pixels to offset the shadow</p>\n"},"useDisplay":{"!type":"bool","!doc":"<p>Defaults to use css offsets to hide the Layer. Specify <tt>true</tt>\nto use css style <tt>'display:none;'</tt> to hide the Layer.</p>\n"},"visibilityCls":{"!type":"string","!doc":"<p>The CSS class name to add in order to hide this Layer if this layer\nis configured with <code><a href=\"#!/api/Ext.dom.Layer-cfg-hideMode\" rel=\"Ext.dom.Layer-cfg-hideMode\" class=\"docClass\">hideMode</a>: 'asclass'</code></p>\n"},"zindex":{"!type":"number","!doc":"<p>Starting z-index.</p>\n"},"deprecated":{},"localXYNames":{},"afterFx":{"!type":"fn() -> !this"},"beforeAction":{"!type":"fn() -> !this"},"beforeFx":{"!type":"fn() -> !this"},"beginUpdate":{"!type":"fn() -> !this"},"constrainXY":{"!type":"fn() -> ?"},"createCB":{"!type":"fn(callback: ?) -> !this"},"destroy":{"!type":"fn() -> !this"},"disableShadow":{"!type":"fn() -> !this"},"enableShadow":{"!type":"fn(show: ?) -> !this"},"endUpdate":{"!type":"fn() -> !this"},"getConstrainOffset":{"!type":"fn() -> !this"},"getZIndex":{"!type":"fn() -> !this"},"hideUnders":{"!type":"fn() -> !this"},"onOpacitySet":{"!type":"fn(opacity: ?) -> !this"},"setHeight":{"!type":"fn(h: ?, animate: ?, duration: ?, callback: ?, easing: ?) -> !this","!doc":"<p>overridden Element method</p>\n"},"setLeft":{"!type":"fn(left: ?) -> !this","!doc":"<p>overridden Element method</p>\n"},"setLeftTop":{"!type":"fn(left: ?, top: ?) -> !this"},"setLocalX":{"!type":"fn() -> !this"},"setLocalXY":{"!type":"fn() -> !this"},"setLocalY":{"!type":"fn() -> !this"},"setSize":{"!type":"fn(w: ?, h: ?, animate: ?, duration: ?, callback: ?, easing: ?) -> !this","!doc":"<p>overridden Element method</p>\n"},"setTop":{"!type":"fn(top: ?) -> !this"},"setVisible":{"!type":"fn(visible: ?, animate: ?, duration: ?, callback: ?, easing: ?) -> !this","!doc":"<p>overridden Element method</p>\n"},"setWidth":{"!type":"fn(w: ?, animate: ?, duration: ?, callback: ?, easing: ?) -> !this","!doc":"<p>overridden Element method</p>\n"},"setX":{"!type":"fn(x: ?, animate: ?, duration: ?, callback: ?, easing: ?) -> ?","!doc":"<p>overridden Element method</p>\n"},"setXY":{"!type":"fn(xy: ?, animate: ?, duration: ?, callback: ?, easing: ?) -> !this"},"setY":{"!type":"fn(y: ?, animate: ?, duration: ?, callback: ?, easing: ?) -> ?","!doc":"<p>overridden Element method</p>\n"},"setZIndex":{"!type":"fn(zindex: ?) -> ?","!doc":"<p>Sets the z-index of this layer and adjusts shadow z-index. The layer\nz-index is automatically incremented depending upon the presence of a\nshadow in so that it always shows above the shadow.</p>\n"},"sync":{"!type":"fn(doShow: ?) -> !this","!doc":"<p>Synchronize this Layer's associated elements and shadow.</p>\n\n<p>This code can execute repeatedly in milliseconds,\neg: dragging a Component configured liveDrag: true, or which has no ghost method\nso code size was sacrificed for efficiency (e.g. no getBox/setBox, no XY calls)</p>\n"}}}},"Editor":{"!doc":"<p>The Editor class is used to provide inline editing for elements on the page. The editor\nis backed by a <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Ext.form.field.Field</a> that will be displayed to edit the underlying content.\nThe editor is a floating Component, when the editor is shown it is automatically aligned to\ndisplay over the top of the bound element it is editing. The Editor contains several options\nfor how to handle key presses:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.Editor-cfg-completeOnEnter\" rel=\"Ext.Editor-cfg-completeOnEnter\" class=\"docClass\">completeOnEnter</a></li>\n<li><a href=\"#!/api/Ext.Editor-cfg-cancelOnEsc\" rel=\"Ext.Editor-cfg-cancelOnEsc\" class=\"docClass\">cancelOnEsc</a></li>\n<li><a href=\"#!/api/Ext.Editor-cfg-swallowKeys\" rel=\"Ext.Editor-cfg-swallowKeys\" class=\"docClass\">swallowKeys</a></li>\n</ul>\n\n\n<p>It also has options for how to use the value once the editor has been activated:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.Editor-cfg-revertInvalid\" rel=\"Ext.Editor-cfg-revertInvalid\" class=\"docClass\">revertInvalid</a></li>\n<li><a href=\"#!/api/Ext.Editor-cfg-ignoreNoChange\" rel=\"Ext.Editor-cfg-ignoreNoChange\" class=\"docClass\">ignoreNoChange</a></li>\n<li><a href=\"#!/api/Ext.Editor-cfg-updateEl\" rel=\"Ext.Editor-cfg-updateEl\" class=\"docClass\">updateEl</a></li>\n</ul>\n\n\n<p>Sample usage:</p>\n\n<pre><code>var editor = new <a href=\"#!/api/Ext.Editor\" rel=\"Ext.Editor\" class=\"docClass\">Ext.Editor</a>({\n    updateEl: true, // update the innerHTML of the bound element when editing completes\n    field: {\n        xtype: 'textfield'\n    }\n});\nvar el = Ext.get('my-text'); // The element to 'edit'\neditor.startEdit(el); // The value of the field will be taken as the innerHTML of the element.\n</code></pre>\n\n<p><p><img src=\"\" alt=\"Ext.Editor component\" width=\"\" height=\"\"></p></p>\n","!type":"fn(config: ?)","prototype":{"alignment":{"!type":"string","!doc":"<p>The position to align to (see Ext.util.Positionable.alignTo for more details).</p>\n"},"allowBlur":{"!type":"bool","!doc":"<p>True to <a href=\"#!/api/Ext.Editor-method-completeEdit\" rel=\"Ext.Editor-method-completeEdit\" class=\"docClass\">complete the editing process</a> if in edit mode when the\nfield is blurred.</p>\n"},"autoSize":{"!doc":"<p>True for the editor to automatically adopt the size of the underlying field. Otherwise, an object\ncan be passed to indicate where to get each dimension. The available properties are 'boundEl' and\n'field'. If a dimension is not specified, it will use the underlying height/width specified on\nthe editor object.\nExamples:</p>\n\n<pre><code>autoSize: true // The editor will be sized to the height/width of the field\n\nheight: 21,\nautoSize: {\n    width: 'boundEl' // The width will be determined by the width of the boundEl, the height from the editor (21)\n}\n\nautoSize: {\n    width: 'field', // Width from the field\n    height: 'boundEl' // Height from the boundEl\n}\n</code></pre>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"cancelOnEsc":{"!type":"bool","!doc":"<p>True to cancel the edit when the escape key is pressed.</p>\n"},"completeOnEnter":{"!type":"bool","!doc":"<p>True to complete the edit when the enter key is pressed.</p>\n"},"constrain":{"!type":"bool","!doc":"<p>True to constrain the editor to the viewport</p>\n"},"field":{"!doc":"<p>The Field object (or descendant) or config object for field</p>\n"},"focusOnToFront":{"!type":"bool","!doc":"<p>Do not participate in the ZIndexManager's focus switching operations.\nWhen an editor is hidden, the ZIndexManager will not automatically activate\nthe last visible floater on the stack.</p>\n"},"hidden":{"!type":"bool","!doc":"<p>private overrides</p>\n"},"hideEl":{"!type":"bool","!doc":"<p>False to keep the bound element visible while the editor is displayed</p>\n"},"ignoreNoChange":{"!type":"bool","!doc":"<p>True to skip the edit completion process (no save, no events fired) if the user completes an edit and\nthe value has not changed.  Applies only to string values - edits for other data types\nwill never be ignored.</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.Editor-cfg-items\" rel=\"Ext.Editor-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"offsets":{"!doc":"<p>The offsets to use when aligning (see Ext.util.Positionable.alignTo for more details.</p>\n"},"parentEl":{"!doc":"<p>An element to render to.</p>\n"},"revertInvalid":{"!type":"bool","!doc":"<p>True to automatically revert the field value and cancel the edit when the user completes an edit and the field\nvalidation fails</p>\n"},"shadow":{"!doc":"<p>\"sides\" for sides/bottom only, \"frame\" for 4-way shadow, and \"drop\" for bottom-right shadow.</p>\n"},"swallowKeys":{"!type":"bool","!doc":"<p>Handle the keydown/keypress events so they don't propagate</p>\n"},"updateEl":{"!type":"bool","!doc":"<p>True to update the innerHTML of the bound element when the update completes</p>\n"},"value":{"!doc":"<p>The data value of the underlying field</p>\n"},"afterRender":{"!type":"fn(ct: ?, position: ?) -> !this","!doc":"<p>private</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"cancelEdit":{"!type":"fn(remainVisible?: ?) -> !this","!doc":"<p>Cancels the editing process and hides the editor without persisting any changes.  The field value will be\nreverted to the original starting value.</p>\n"},"completeEdit":{"!type":"fn(remainVisible?: ?) -> !this","!doc":"<p>Ends the editing process, persists the changed value to the underlying field, and hides the editor.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Gets the data value of the editor</p>\n"},"onEditComplete":{"!type":"fn(remainVisible: ?) -> !this","!doc":"<p>private</p>\n"},"onFieldAutosize":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onFieldBlur":{"!type":"fn(field: ?, e: ?) -> !this","!doc":"<p>private</p>\n"},"onHide":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onShow":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onSpecialKey":{"!type":"fn(field: ?, event: ?) -> !this","!doc":"<p>private</p>\n"},"realign":{"!type":"fn(autoSize?: ?) -> !this","!doc":"<p>Realigns the editor to the bound field based on the current alignment config value.</p>\n"},"setValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Sets the data value of the editor</p>\n"},"startEdit":{"!type":"fn(el: ?, value?: ?) -> !this","!doc":"<p>Starts the editing process and shows the editor.</p>\n"},"beforecomplete":{"!type":"fn(this: ?, value: ?, startValue: ?, eOpts: ?)","!doc":"<p>Fires after a change has been made to the field, but before the change is reflected in the underlying\nfield.  Saving the change to the field can be canceled by returning false from the handler of this event.\nNote that if the value has not changed and ignoreNoChange = true, the editing will still end but this\nevent will not fire since no edit actually occurred.</p>\n"},"beforestartedit":{"!type":"fn(this: ?, boundEl: ?, value: ?, eOpts: ?)","!doc":"<p>Fires when editing is initiated, but before the value changes.  Editing can be canceled by returning\nfalse from the handler of this event.</p>\n"},"canceledit":{"!type":"fn(this: ?, value: ?, startValue: ?, eOpts: ?)","!doc":"<p>Fires after editing has been canceled and the editor's value has been reset.</p>\n"},"complete":{"!type":"fn(this: ?, value: ?, startValue: ?, eOpts: ?)","!doc":"<p>Fires after editing is complete and any changed value has been written to the underlying field.</p>\n"},"specialkey":{"!type":"fn(this: ?, field: ?, event: ?, eOpts: ?)","!doc":"<p>Fires when any key related to navigation (arrows, tab, enter, esc, etc.) is pressed.  You can check\nExt.event.Event.getKey to determine which key was pressed.</p>\n"},"startedit":{"!type":"fn(this: ?, boundEl: ?, value: ?, eOpts: ?)","!doc":"<p>Fires when this editor is displayed</p>\n"}}},"enums":{"Feature":{"!doc":"<p>Enumeration of all ftypes.</p>\n","prototype":{"abstractsummary":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.feature.AbstractSummary\" rel=\"Ext.grid.feature.AbstractSummary\" class=\"docClass\">Ext.grid.feature.AbstractSummary</a>.</p>\n"},"feature":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.feature.Feature\" rel=\"Ext.grid.feature.Feature\" class=\"docClass\">Ext.grid.feature.Feature</a>.</p>\n"},"grouping":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.feature.Grouping\" rel=\"Ext.grid.feature.Grouping\" class=\"docClass\">Ext.grid.feature.Grouping</a>.</p>\n"},"groupingsummary":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.feature.GroupingSummary\" rel=\"Ext.grid.feature.GroupingSummary\" class=\"docClass\">Ext.grid.feature.GroupingSummary</a>.</p>\n"},"rowbody":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.feature.RowBody\" rel=\"Ext.grid.feature.RowBody\" class=\"docClass\">Ext.grid.feature.RowBody</a>.</p>\n"},"summary":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.feature.Summary\" rel=\"Ext.grid.feature.Summary\" class=\"docClass\">Ext.grid.feature.Summary</a>.</p>\n"}}},"Layout":{"!doc":"<p>Enumeration of all layout types.</p>\n","prototype":{"absolute":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Absolute\" rel=\"Ext.layout.container.Absolute\" class=\"docClass\">Ext.layout.container.Absolute</a>.</p>\n"},"accordion":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Accordion\" rel=\"Ext.layout.container.Accordion\" class=\"docClass\">Ext.layout.container.Accordion</a>.</p>\n"},"anchor":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Anchor\" rel=\"Ext.layout.container.Anchor\" class=\"docClass\">Ext.layout.container.Anchor</a>.</p>\n"},"auto":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a>.</p>\n"},"autocomponent":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.component.Auto\" rel=\"Ext.layout.component.Auto\" class=\"docClass\">Ext.layout.component.Auto</a>.</p>\n"},"autocontainer":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a>.</p>\n"},"body":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.component.Body\" rel=\"Ext.layout.component.Body\" class=\"docClass\">Ext.layout.component.Body</a>.</p>\n"},"border":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Ext.layout.container.Border</a>.</p>\n"},"boundlist":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.component.BoundList\" rel=\"Ext.layout.component.BoundList\" class=\"docClass\">Ext.layout.component.BoundList</a>.</p>\n"},"box":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Box\" rel=\"Ext.layout.container.Box\" class=\"docClass\">Ext.layout.container.Box</a>.</p>\n"},"card":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a>.</p>\n"},"center":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Center\" rel=\"Ext.layout.container.Center\" class=\"docClass\">Ext.layout.container.Center</a>.</p>\n"},"checkboxgroup":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.CheckboxGroup\" rel=\"Ext.layout.container.CheckboxGroup\" class=\"docClass\">Ext.layout.container.CheckboxGroup</a>.</p>\n"},"column":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Column\" rel=\"Ext.layout.container.Column\" class=\"docClass\">Ext.layout.container.Column</a>.</p>\n"},"columncomponent":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.ColumnComponentLayout\" rel=\"Ext.grid.ColumnComponentLayout\" class=\"docClass\">Ext.grid.ColumnComponentLayout</a>.</p>\n"},"container":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Container\" rel=\"Ext.layout.container.Container\" class=\"docClass\">Ext.layout.container.Container</a>.</p>\n"},"dock":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.component.Dock\" rel=\"Ext.layout.component.Dock\" class=\"docClass\">Ext.layout.component.Dock</a>.</p>\n"},"editor":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Editor\" rel=\"Ext.layout.container.Editor\" class=\"docClass\">Ext.layout.container.Editor</a>.</p>\n"},"fieldcontainer":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.component.field.FieldContainer\" rel=\"Ext.layout.component.field.FieldContainer\" class=\"docClass\">Ext.layout.component.field.FieldContainer</a>.</p>\n"},"fieldset":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.component.FieldSet\" rel=\"Ext.layout.component.FieldSet\" class=\"docClass\">Ext.layout.component.FieldSet</a>.</p>\n"},"fit":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Fit\" rel=\"Ext.layout.container.Fit\" class=\"docClass\">Ext.layout.container.Fit</a>.</p>\n"},"form":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Form\" rel=\"Ext.layout.container.Form\" class=\"docClass\">Ext.layout.container.Form</a>.</p>\n"},"gridcolumn":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.ColumnLayout\" rel=\"Ext.grid.ColumnLayout\" class=\"docClass\">Ext.grid.ColumnLayout</a>.</p>\n"},"hbox":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">Ext.layout.container.HBox</a>.</p>\n"},"htmleditor":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.component.field.HtmlEditor\" rel=\"Ext.layout.component.field.HtmlEditor\" class=\"docClass\">Ext.layout.component.field.HtmlEditor</a>.</p>\n"},"progressbar":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.component.ProgressBar\" rel=\"Ext.layout.component.ProgressBar\" class=\"docClass\">Ext.layout.component.ProgressBar</a>.</p>\n"},"split-column":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.SplitColumn\" rel=\"Ext.layout.container.SplitColumn\" class=\"docClass\">Ext.layout.container.SplitColumn</a>.</p>\n"},"table":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Table\" rel=\"Ext.layout.container.Table\" class=\"docClass\">Ext.layout.container.Table</a>.</p>\n"},"tableview":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.view.TableLayout\" rel=\"Ext.view.TableLayout\" class=\"docClass\">Ext.view.TableLayout</a>.</p>\n"},"ux.center":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.Center\" rel=\"Ext.layout.container.Center\" class=\"docClass\">Ext.layout.container.Center</a>.</p>\n"},"vbox":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.VBox\" rel=\"Ext.layout.container.VBox\" class=\"docClass\">Ext.layout.container.VBox</a>.</p>\n"}}},"Plugin":{"!doc":"<p>Enumeration of all ptypes.</p>\n","prototype":{"bufferedrenderer":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.plugin.BufferedRenderer\" rel=\"Ext.grid.plugin.BufferedRenderer\" class=\"docClass\">Ext.grid.plugin.BufferedRenderer</a>.</p>\n"},"cellediting":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.plugin.CellEditing\" rel=\"Ext.grid.plugin.CellEditing\" class=\"docClass\">Ext.grid.plugin.CellEditing</a>.</p>\n"},"gridfilters":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.filters.Filters\" rel=\"Ext.grid.filters.Filters\" class=\"docClass\">Ext.grid.filters.Filters</a>.</p>\n"},"gridheaderreorderer":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.plugin.HeaderReorderer\" rel=\"Ext.grid.plugin.HeaderReorderer\" class=\"docClass\">Ext.grid.plugin.HeaderReorderer</a>.</p>\n"},"gridheaderresizer":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.plugin.HeaderResizer\" rel=\"Ext.grid.plugin.HeaderResizer\" class=\"docClass\">Ext.grid.plugin.HeaderResizer</a>.</p>\n"},"gridviewdragdrop":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.plugin.DragDrop\" rel=\"Ext.grid.plugin.DragDrop\" class=\"docClass\">Ext.grid.plugin.DragDrop</a>.</p>\n"},"responsive":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.plugin.Responsive\" rel=\"Ext.plugin.Responsive\" class=\"docClass\">Ext.plugin.Responsive</a>.</p>\n"},"rowediting":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.plugin.RowEditing\" rel=\"Ext.grid.plugin.RowEditing\" class=\"docClass\">Ext.grid.plugin.RowEditing</a>.</p>\n"},"rowexpander":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.plugin.RowExpander\" rel=\"Ext.grid.plugin.RowExpander\" class=\"docClass\">Ext.grid.plugin.RowExpander</a>.</p>\n"},"treeviewdragdrop":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop\" rel=\"Ext.tree.plugin.TreeViewDragDrop\" class=\"docClass\">Ext.tree.plugin.TreeViewDragDrop</a>.</p>\n"},"viewport":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.plugin.Viewport\" rel=\"Ext.plugin.Viewport\" class=\"docClass\">Ext.plugin.Viewport</a>.</p>\n"}}},"Widget":{"!doc":"<p>Enumeration of all xtypes.</p>\n","prototype":{"actioncolumn":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.column.Action\" rel=\"Ext.grid.column.Action\" class=\"docClass\">Ext.grid.column.Action</a>.</p>\n"},"booleancolumn":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.column.Boolean\" rel=\"Ext.grid.column.Boolean\" class=\"docClass\">Ext.grid.column.Boolean</a>.</p>\n"},"bordersplitter":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.resizer.BorderSplitter\" rel=\"Ext.resizer.BorderSplitter\" class=\"docClass\">Ext.resizer.BorderSplitter</a>.</p>\n"},"boundlist":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.view.BoundList\" rel=\"Ext.view.BoundList\" class=\"docClass\">Ext.view.BoundList</a>.</p>\n"},"box":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.</p>\n"},"breadcrumb":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.toolbar.Breadcrumb\" rel=\"Ext.toolbar.Breadcrumb\" class=\"docClass\">Ext.toolbar.Breadcrumb</a>.</p>\n"},"button":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>.</p>\n"},"buttongroup":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.container.ButtonGroup\" rel=\"Ext.container.ButtonGroup\" class=\"docClass\">Ext.container.ButtonGroup</a>.</p>\n"},"checkbox":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Checkbox\" rel=\"Ext.form.field.Checkbox\" class=\"docClass\">Ext.form.field.Checkbox</a>.</p>\n"},"checkboxfield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Checkbox\" rel=\"Ext.form.field.Checkbox\" class=\"docClass\">Ext.form.field.Checkbox</a>.</p>\n"},"checkboxgroup":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.CheckboxGroup\" rel=\"Ext.form.CheckboxGroup\" class=\"docClass\">Ext.form.CheckboxGroup</a>.</p>\n"},"checkcolumn":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.column.Check\" rel=\"Ext.grid.column.Check\" class=\"docClass\">Ext.grid.column.Check</a>.</p>\n"},"colormenu":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.menu.ColorPicker\" rel=\"Ext.menu.ColorPicker\" class=\"docClass\">Ext.menu.ColorPicker</a>.</p>\n"},"colorpicker":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.picker.Color\" rel=\"Ext.picker.Color\" class=\"docClass\">Ext.picker.Color</a>.</p>\n"},"columnsplitter":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.layout.container.ColumnSplitter\" rel=\"Ext.layout.container.ColumnSplitter\" class=\"docClass\">Ext.layout.container.ColumnSplitter</a>.</p>\n"},"combo":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.field.ComboBox</a>.</p>\n"},"combobox":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.field.ComboBox</a>.</p>\n"},"component":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.</p>\n"},"container":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>.</p>\n"},"cycle":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.button.Cycle\" rel=\"Ext.button.Cycle\" class=\"docClass\">Ext.button.Cycle</a>.</p>\n"},"dashboard":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.dashboard.Dashboard\" rel=\"Ext.dashboard.Dashboard\" class=\"docClass\">Ext.dashboard.Dashboard</a>.</p>\n"},"dashboard-column":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.dashboard.Column\" rel=\"Ext.dashboard.Column\" class=\"docClass\">Ext.dashboard.Column</a>.</p>\n"},"dashboard-panel":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.dashboard.Panel\" rel=\"Ext.dashboard.Panel\" class=\"docClass\">Ext.dashboard.Panel</a>.</p>\n"},"dataview":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a>.</p>\n"},"datecolumn":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.column.Date\" rel=\"Ext.grid.column.Date\" class=\"docClass\">Ext.grid.column.Date</a>.</p>\n"},"datefield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>.</p>\n"},"datemenu":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.menu.DatePicker\" rel=\"Ext.menu.DatePicker\" class=\"docClass\">Ext.menu.DatePicker</a>.</p>\n"},"datepicker":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.picker.Date\" rel=\"Ext.picker.Date\" class=\"docClass\">Ext.picker.Date</a>.</p>\n"},"displayfield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Display\" rel=\"Ext.form.field.Display\" class=\"docClass\">Ext.form.field.Display</a>.</p>\n"},"editor":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.Editor\" rel=\"Ext.Editor\" class=\"docClass\">Ext.Editor</a>.</p>\n"},"field":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a>.</p>\n"},"fieldcontainer":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.FieldContainer\" rel=\"Ext.form.FieldContainer\" class=\"docClass\">Ext.form.FieldContainer</a>.</p>\n"},"fieldset":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.FieldSet\" rel=\"Ext.form.FieldSet\" class=\"docClass\">Ext.form.FieldSet</a>.</p>\n"},"filebutton":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.FileButton\" rel=\"Ext.form.field.FileButton\" class=\"docClass\">Ext.form.field.FileButton</a>.</p>\n"},"filefield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.File\" rel=\"Ext.form.field.File\" class=\"docClass\">Ext.form.field.File</a>.</p>\n"},"fileuploadfield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.File\" rel=\"Ext.form.field.File\" class=\"docClass\">Ext.form.field.File</a>.</p>\n"},"flash":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.flash.Component\" rel=\"Ext.flash.Component\" class=\"docClass\">Ext.flash.Component</a>.</p>\n"},"form":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>.</p>\n"},"grid":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>.</p>\n"},"gridcolumn":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">Ext.grid.column.Column</a>.</p>\n"},"gridpanel":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>.</p>\n"},"gridview":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.View\" rel=\"Ext.grid.View\" class=\"docClass\">Ext.grid.View</a>.</p>\n"},"header":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Ext.panel.Header</a>.</p>\n"},"headercontainer":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.header.Container\" rel=\"Ext.grid.header.Container\" class=\"docClass\">Ext.grid.header.Container</a>.</p>\n"},"hidden":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Hidden\" rel=\"Ext.form.field.Hidden\" class=\"docClass\">Ext.form.field.Hidden</a>.</p>\n"},"hiddenfield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Hidden\" rel=\"Ext.form.field.Hidden\" class=\"docClass\">Ext.form.field.Hidden</a>.</p>\n"},"htmleditor":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.HtmlEditor\" rel=\"Ext.form.field.HtmlEditor\" class=\"docClass\">Ext.form.field.HtmlEditor</a>.</p>\n"},"image":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.Img\" rel=\"Ext.Img\" class=\"docClass\">Ext.Img</a>.</p>\n"},"imagecomponent":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.Img\" rel=\"Ext.Img\" class=\"docClass\">Ext.Img</a>.</p>\n"},"label":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.Label\" rel=\"Ext.form.Label\" class=\"docClass\">Ext.form.Label</a>.</p>\n"},"loadmask":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.LoadMask\" rel=\"Ext.LoadMask\" class=\"docClass\">Ext.LoadMask</a>.</p>\n"},"menu":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a>.</p>\n"},"menucheckitem":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.menu.CheckItem\" rel=\"Ext.menu.CheckItem\" class=\"docClass\">Ext.menu.CheckItem</a>.</p>\n"},"menuitem":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">Ext.menu.Item</a>.</p>\n"},"menuseparator":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.menu.Separator\" rel=\"Ext.menu.Separator\" class=\"docClass\">Ext.menu.Separator</a>.</p>\n"},"messagebox":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.window.MessageBox\" rel=\"Ext.window.MessageBox\" class=\"docClass\">Ext.window.MessageBox</a>.</p>\n"},"monthpicker":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.picker.Month\" rel=\"Ext.picker.Month\" class=\"docClass\">Ext.picker.Month</a>.</p>\n"},"multiselector":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.view.MultiSelector\" rel=\"Ext.view.MultiSelector\" class=\"docClass\">Ext.view.MultiSelector</a>.</p>\n"},"multiselector-search":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.view.MultiSelectorSearch\" rel=\"Ext.view.MultiSelectorSearch\" class=\"docClass\">Ext.view.MultiSelectorSearch</a>.</p>\n"},"multislider":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.slider.Multi\" rel=\"Ext.slider.Multi\" class=\"docClass\">Ext.slider.Multi</a>.</p>\n"},"numbercolumn":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.column.Number\" rel=\"Ext.grid.column.Number\" class=\"docClass\">Ext.grid.column.Number</a>.</p>\n"},"numberfield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Number\" rel=\"Ext.form.field.Number\" class=\"docClass\">Ext.form.field.Number</a>.</p>\n"},"pagingtoolbar":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.toolbar.Paging\" rel=\"Ext.toolbar.Paging\" class=\"docClass\">Ext.toolbar.Paging</a>.</p>\n"},"panel":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>.</p>\n"},"pickerfield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Picker\" rel=\"Ext.form.field.Picker\" class=\"docClass\">Ext.form.field.Picker</a>.</p>\n"},"progressbar":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.ProgressBar\" rel=\"Ext.ProgressBar\" class=\"docClass\">Ext.ProgressBar</a>.</p>\n"},"progressbarwidget":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.ProgressBarWidget\" rel=\"Ext.ProgressBarWidget\" class=\"docClass\">Ext.ProgressBarWidget</a>.</p>\n"},"propertygrid":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.property.Grid\" rel=\"Ext.grid.property.Grid\" class=\"docClass\">Ext.grid.property.Grid</a>.</p>\n"},"quicktip":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.tip.QuickTip\" rel=\"Ext.tip.QuickTip\" class=\"docClass\">Ext.tip.QuickTip</a>.</p>\n"},"radio":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Radio\" rel=\"Ext.form.field.Radio\" class=\"docClass\">Ext.form.field.Radio</a>.</p>\n"},"radiofield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Radio\" rel=\"Ext.form.field.Radio\" class=\"docClass\">Ext.form.field.Radio</a>.</p>\n"},"radiogroup":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.RadioGroup\" rel=\"Ext.form.RadioGroup\" class=\"docClass\">Ext.form.RadioGroup</a>.</p>\n"},"roweditor":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.RowEditor\" rel=\"Ext.grid.RowEditor\" class=\"docClass\">Ext.grid.RowEditor</a>.</p>\n"},"roweditorbuttons":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.RowEditorButtons\" rel=\"Ext.grid.RowEditorButtons\" class=\"docClass\">Ext.grid.RowEditorButtons</a>.</p>\n"},"rownumberer":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.column.RowNumberer\" rel=\"Ext.grid.column.RowNumberer\" class=\"docClass\">Ext.grid.column.RowNumberer</a>.</p>\n"},"segmentedbutton":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.button.Segmented\" rel=\"Ext.button.Segmented\" class=\"docClass\">Ext.button.Segmented</a>.</p>\n"},"slider":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.slider.Single\" rel=\"Ext.slider.Single\" class=\"docClass\">Ext.slider.Single</a>.</p>\n"},"sliderfield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.slider.Single\" rel=\"Ext.slider.Single\" class=\"docClass\">Ext.slider.Single</a>.</p>\n"},"slidertip":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.slider.Tip\" rel=\"Ext.slider.Tip\" class=\"docClass\">Ext.slider.Tip</a>.</p>\n"},"sliderwidget":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.slider.Widget\" rel=\"Ext.slider.Widget\" class=\"docClass\">Ext.slider.Widget</a>.</p>\n"},"sparklinebar":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.sparkline.Bar\" rel=\"Ext.sparkline.Bar\" class=\"docClass\">Ext.sparkline.Bar</a>.</p>\n"},"sparklinebox":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.sparkline.Box\" rel=\"Ext.sparkline.Box\" class=\"docClass\">Ext.sparkline.Box</a>.</p>\n"},"sparklinebullet":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.sparkline.Bullet\" rel=\"Ext.sparkline.Bullet\" class=\"docClass\">Ext.sparkline.Bullet</a>.</p>\n"},"sparklinediscrete":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.sparkline.Discrete\" rel=\"Ext.sparkline.Discrete\" class=\"docClass\">Ext.sparkline.Discrete</a>.</p>\n"},"sparklineline":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.sparkline.Line\" rel=\"Ext.sparkline.Line\" class=\"docClass\">Ext.sparkline.Line</a>.</p>\n"},"sparklinepie":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.sparkline.Pie\" rel=\"Ext.sparkline.Pie\" class=\"docClass\">Ext.sparkline.Pie</a>.</p>\n"},"sparklinetristate":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.sparkline.TriState\" rel=\"Ext.sparkline.TriState\" class=\"docClass\">Ext.sparkline.TriState</a>.</p>\n"},"spinnerfield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Spinner\" rel=\"Ext.form.field.Spinner\" class=\"docClass\">Ext.form.field.Spinner</a>.</p>\n"},"splitbutton":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.button.Split\" rel=\"Ext.button.Split\" class=\"docClass\">Ext.button.Split</a>.</p>\n"},"splitter":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.resizer.Splitter\" rel=\"Ext.resizer.Splitter\" class=\"docClass\">Ext.resizer.Splitter</a>.</p>\n"},"tab":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.tab.Tab\" rel=\"Ext.tab.Tab\" class=\"docClass\">Ext.tab.Tab</a>.</p>\n"},"tabbar":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.tab.Bar\" rel=\"Ext.tab.Bar\" class=\"docClass\">Ext.tab.Bar</a>.</p>\n"},"tablepanel":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.panel.Table\" rel=\"Ext.panel.Table\" class=\"docClass\">Ext.panel.Table</a>.</p>\n"},"tableview":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">Ext.view.Table</a>.</p>\n"},"tabpanel":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>.</p>\n"},"tagfield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Tag\" rel=\"Ext.form.field.Tag\" class=\"docClass\">Ext.form.field.Tag</a>.</p>\n"},"tbfill":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.toolbar.Fill\" rel=\"Ext.toolbar.Fill\" class=\"docClass\">Ext.toolbar.Fill</a>.</p>\n"},"tbitem":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.toolbar.Item\" rel=\"Ext.toolbar.Item\" class=\"docClass\">Ext.toolbar.Item</a>.</p>\n"},"tbseparator":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.toolbar.Separator\" rel=\"Ext.toolbar.Separator\" class=\"docClass\">Ext.toolbar.Separator</a>.</p>\n"},"tbspacer":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.toolbar.Spacer\" rel=\"Ext.toolbar.Spacer\" class=\"docClass\">Ext.toolbar.Spacer</a>.</p>\n"},"tbtext":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.toolbar.TextItem\" rel=\"Ext.toolbar.TextItem\" class=\"docClass\">Ext.toolbar.TextItem</a>.</p>\n"},"templatecolumn":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.column.Template\" rel=\"Ext.grid.column.Template\" class=\"docClass\">Ext.grid.column.Template</a>.</p>\n"},"textarea":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.TextArea\" rel=\"Ext.form.field.TextArea\" class=\"docClass\">Ext.form.field.TextArea</a>.</p>\n"},"textareafield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.TextArea\" rel=\"Ext.form.field.TextArea\" class=\"docClass\">Ext.form.field.TextArea</a>.</p>\n"},"textfield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>.</p>\n"},"timefield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Time\" rel=\"Ext.form.field.Time\" class=\"docClass\">Ext.form.field.Time</a>.</p>\n"},"timepicker":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.picker.Time\" rel=\"Ext.picker.Time\" class=\"docClass\">Ext.picker.Time</a>.</p>\n"},"tip":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.tip.Tip\" rel=\"Ext.tip.Tip\" class=\"docClass\">Ext.tip.Tip</a>.</p>\n"},"title":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.panel.Title\" rel=\"Ext.panel.Title\" class=\"docClass\">Ext.panel.Title</a>.</p>\n"},"toast":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.window.Toast\" rel=\"Ext.window.Toast\" class=\"docClass\">Ext.window.Toast</a>.</p>\n"},"tool":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.panel.Tool\" rel=\"Ext.panel.Tool\" class=\"docClass\">Ext.panel.Tool</a>.</p>\n"},"toolbar":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Ext.toolbar.Toolbar</a>.</p>\n"},"tooltip":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.tip.ToolTip\" rel=\"Ext.tip.ToolTip\" class=\"docClass\">Ext.tip.ToolTip</a>.</p>\n"},"treecolumn":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.tree.Column\" rel=\"Ext.tree.Column\" class=\"docClass\">Ext.tree.Column</a>.</p>\n"},"treepanel":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">Ext.tree.Panel</a>.</p>\n"},"treeview":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.tree.View\" rel=\"Ext.tree.View\" class=\"docClass\">Ext.tree.View</a>.</p>\n"},"trigger":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Trigger\" rel=\"Ext.form.field.Trigger\" class=\"docClass\">Ext.form.field.Trigger</a>.</p>\n"},"triggerfield":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.form.field.Trigger\" rel=\"Ext.form.field.Trigger\" class=\"docClass\">Ext.form.field.Trigger</a>.</p>\n"},"viewport":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.container.Viewport\" rel=\"Ext.container.Viewport\" class=\"docClass\">Ext.container.Viewport</a>.</p>\n"},"widgetcolumn":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.column.Widget\" rel=\"Ext.grid.column.Widget\" class=\"docClass\">Ext.grid.column.Widget</a>.</p>\n"},"window":{"!type":"string","!doc":"<p>Alias for <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>.</p>\n"}}}},"EventManager":{"!doc":"<p>Registers event handlers on DOM elements.</p>\n\n<p>This class is deprecated.  Please use the Ext.dom.Element api to atach listeners to\nDOM Elements.  For example:</p>\n\n<pre><code>var element = Ext.get('myId');\n\nelement.on('click', function(e) {\n    // event handling logic here\n});\n</code></pre>\n","mouseEnterRe":{},"mouseLeaveRe":{},"addListener":{"!type":"fn(el: ?, eventName: ?, handler?: ?, scope?: ?, options?: ?) -> !this","!doc":"<p>Appends an event handler to an element.  The shorthand version <a href=\"#!/api/Ext.EventManager-method-on\" rel=\"Ext.EventManager-method-on\" class=\"docClass\">on</a> is equivalent.\nTypically you will use Ext.dom.Element.addListener directly on an Element in favor of\ncalling this version.</p>\n\n<p><a href=\"#!/api/Ext.EventManager-method-on\" rel=\"Ext.EventManager-method-on\" class=\"docClass\">on</a> is an alias for <a href=\"#!/api/Ext.EventManager-method-addListener\" rel=\"Ext.EventManager-method-addListener\" class=\"docClass\">addListener</a>.</p>\n"},"getId":{"!type":"fn(element: ?) -> string","!doc":"<p>Get the id of the element. If one has not been assigned, automatically assign it.</p>\n"},"getPageX":{"!type":"fn(event: ?) -> number","!doc":"<p>Gets the x coordinate from the event</p>\n"},"getPageXY":{"!type":"fn(event: ?) -> ?","!doc":"<p>Gets the x &amp; y coordinate from the event</p>\n"},"getPageY":{"!type":"fn(event: ?) -> number","!doc":"<p>Gets the y coordinate from the event</p>\n"},"getRelatedTarget":{"!type":"fn(event: ?) -> ?","!doc":"<p>Gets the related target from the event.</p>\n"},"getTarget":{"!type":"fn(event: ?) -> ?","!doc":"<p>Gets the target of the event.</p>\n"},"on":{"!type":"fn(el: ?, eventName: ?, handler?: ?, scope?: ?, options?: ?) -> !this","!doc":"<p>Appends an event handler to an element.  The shorthand version <a href=\"#!/api/Ext.EventManager-method-on\" rel=\"Ext.EventManager-method-on\" class=\"docClass\">on</a> is equivalent.\nTypically you will use Ext.dom.Element.addListener directly on an Element in favor of\ncalling this version.</p>\n\n<p><a href=\"#!/api/Ext.EventManager-method-on\" rel=\"Ext.EventManager-method-on\" class=\"docClass\">on</a> is an alias for <a href=\"#!/api/Ext.EventManager-method-addListener\" rel=\"Ext.EventManager-method-addListener\" class=\"docClass\">addListener</a>.</p>\n"},"onWindowResize":{"!type":"fn(fn: ?, scope: ?, options?: ?) -> !this","!doc":"<p>Adds a listener to be notified when the browser window is resized and provides resize event buffering (100 milliseconds),\npasses new viewport width and height to handlers.</p>\n"},"onWindowUnload":{"!type":"fn(fn: ?, scope: ?, options: ?) -> !this","!doc":"<p>Adds a listener to be notified when the browser window is unloaded.</p>\n"},"preventDefault":{"!type":"fn(event: ?) -> !this","!doc":"<p>Prevents the browsers default handling of the event.</p>\n"},"purgeElement":{"!type":"fn(el: ?, eventName?: ?) -> !this","!doc":"<p>Recursively removes all previous added listeners from an element and its children.\nTypically you will use Ext.dom.Element.clearListeners directly on an Element\nin favor of calling this method.</p>\n"},"removeAll":{"!type":"fn(el: ?) -> !this","!doc":"<p>Removes all event handers from an element.  Typically you will use Ext.dom.Element.clearListeners directly on an Element in favor of calling this method.</p>\n"},"removeListener":{"!type":"fn(el: ?, eventName: ?, fn: ?, scope: ?) -> !this","!doc":"<p>Removes an event handler from an element.  The shorthand version <a href=\"#!/api/Ext.EventManager-method-un\" rel=\"Ext.EventManager-method-un\" class=\"docClass\">un</a> is equivalent.  Typically\nyou will use Ext.dom.Element.removeListener directly on an Element in favor of calling this version.</p>\n\n<p><a href=\"#!/api/Ext.EventManager-method-on\" rel=\"Ext.EventManager-method-on\" class=\"docClass\">on</a> is an alias for <a href=\"#!/api/Ext.EventManager-method-addListener\" rel=\"Ext.EventManager-method-addListener\" class=\"docClass\">addListener</a>.</p>\n"},"removeResizeListener":{"!type":"fn(fn: ?, scope: ?) -> !this","!doc":"<p>Removes the passed window resize listener.</p>\n"},"removeUnloadListener":{"!type":"fn(fn: ?, scope: ?) -> !this","!doc":"<p>Removes the passed window unload listener.</p>\n"},"resolveTextNode":{"!type":"fn(node: ?) -> ?","!doc":"<p>Resolve any text nodes accounting for browser differences.</p>\n"},"stopEvent":{"!type":"fn(event: ?) -> !this","!doc":"<p>Stop the event (preventDefault and stopPropagation)</p>\n"},"stopPropagation":{"!type":"fn(event: ?) -> !this","!doc":"<p>Cancels bubbling of the event.</p>\n"},"un":{"!type":"fn(el: ?, eventName: ?, fn: ?, scope: ?) -> !this","!doc":"<p>Removes an event handler from an element.  The shorthand version <a href=\"#!/api/Ext.EventManager-method-un\" rel=\"Ext.EventManager-method-un\" class=\"docClass\">un</a> is equivalent.  Typically\nyou will use Ext.dom.Element.removeListener directly on an Element in favor of calling this version.</p>\n\n<p><a href=\"#!/api/Ext.EventManager-method-on\" rel=\"Ext.EventManager-method-on\" class=\"docClass\">on</a> is an alias for <a href=\"#!/api/Ext.EventManager-method-addListener\" rel=\"Ext.EventManager-method-addListener\" class=\"docClass\">addListener</a>.</p>\n"}},"flash":{"Component":{"!doc":"<p>A simple Component for displaying an Adobe Flash SWF movie. The movie will be sized and can participate\nin layout like any other Component.</p>\n\n<p>This component requires the third-party SWFObject library version 2.2 or above. It is not included within\nthe ExtJS distribution, so you will have to include it into your page manually in order to use this component.\nThe SWFObject library can be downloaded from the <a href=\"http://code.google.com/p/swfobject\">SWFObject project page</a>\nand then simply import it into the head of your HTML document:</p>\n\n<pre><code>&lt;script type=\"text/javascript\" src=\"path/to/local/swfobject.js\"&gt;&lt;/script&gt;\n</code></pre>\n\n<h2>Configuration</h2>\n\n<p>This component allows several options for configuring how the target Flash movie is embedded. The most\nimportant is the required <a href=\"#!/api/Ext.flash.Component-cfg-url\" rel=\"Ext.flash.Component-cfg-url\" class=\"docClass\">url</a> which points to the location of the Flash movie to load. Other\nconfigurations include:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.flash.Component-cfg-backgroundColor\" rel=\"Ext.flash.Component-cfg-backgroundColor\" class=\"docClass\">backgroundColor</a></li>\n<li><a href=\"#!/api/Ext.flash.Component-cfg-wmode\" rel=\"Ext.flash.Component-cfg-wmode\" class=\"docClass\">wmode</a></li>\n<li><a href=\"#!/api/Ext.flash.Component-cfg-flashVars\" rel=\"Ext.flash.Component-cfg-flashVars\" class=\"docClass\">flashVars</a></li>\n<li><a href=\"#!/api/Ext.flash.Component-cfg-flashParams\" rel=\"Ext.flash.Component-cfg-flashParams\" class=\"docClass\">flashParams</a></li>\n<li><a href=\"#!/api/Ext.flash.Component-cfg-flashAttributes\" rel=\"Ext.flash.Component-cfg-flashAttributes\" class=\"docClass\">flashAttributes</a></li>\n</ul>\n\n\n<h2>Example usage:</h2>\n\n<pre><code>var win = Ext.widget('window', {\n    title: \"It's a tiger!\",\n    layout: 'fit',\n    width: 300,\n    height: 300,\n    x: 20,\n    y: 20,\n    resizable: true,\n    items: {\n        xtype: 'flash',\n        url: 'tiger.swf'\n    }\n});\nwin.show();\n</code></pre>\n\n<h2>Express Install</h2>\n\n<p>Adobe provides a tool called <a href=\"http://www.adobe.com/devnet/flashplayer/articles/express_install.html\">Express Install</a>\nthat offers users an easy way to upgrade their Flash player. If you wish to make use of this, you should set\nthe static EXPRESS_INSTALL_URL property to the location of your Express Install SWF file:</p>\n\n<pre><code><a href=\"#!/api/Ext.flash.Component-static-property-EXPRESS_INSTALL_URL\" rel=\"Ext.flash.Component-static-property-EXPRESS_INSTALL_URL\" class=\"docClass\">Ext.flash.Component.EXPRESS_INSTALL_URL</a> = 'path/to/local/expressInstall.swf';\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"backgroundColor":{"!type":"string","!doc":"<p>The background color of the SWF movie.</p>\n"},"expressInstall":{"!type":"bool","!doc":"<p>True to prompt the user to install flash if not installed. Note that this uses\n<a href=\"#!/api/Ext.flash.Component-static-property-EXPRESS_INSTALL_URL\" rel=\"Ext.flash.Component-static-property-EXPRESS_INSTALL_URL\" class=\"docClass\">Ext.FlashComponent.EXPRESS_INSTALL_URL</a>, which should be set to the local resource.</p>\n"},"flashAttributes":{"!doc":"<p>A set of key value pairs to be passed to the flash object as attributes.</p>\n"},"flashParams":{"!doc":"<p>A set of key value pairs to be passed to the flash object as parameters. Possible parameters can be found here:\nhttp://kb2.adobe.com/cps/127/tn_12701.html</p>\n"},"flashVars":{"!doc":"<p>A set of key value pairs to be passed to the flash object as flash variables.</p>\n"},"flashVersion":{"!type":"string","!doc":"<p>Indicates the version the flash content was published for.</p>\n"},"renderTpl":{"!doc":"<p>Have to create a placeholder div with the swfId, which SWFObject will replace with the object/embed element.</p>\n"},"swfHeight":{"!doc":"<p>The height of the embedded SWF movie inside the component.</p>\n\n<p>Defaults to \"100%\" so that the movie matches the height of the component.</p>\n"},"swfWidth":{"!doc":"<p>The width of the embedded SWF movie inside the component.</p>\n\n<p>Defaults to \"100%\" so that the movie matches the width of the component.</p>\n"},"url":{"!type":"string","!doc":"<p>The URL of the SWF file to include.</p>\n"},"wmode":{"!type":"string","!doc":"<p>The wmode of the flash object. This can be used to control layering.\nSet to 'transparent' to ignore the <a href=\"#!/api/Ext.flash.Component-cfg-backgroundColor\" rel=\"Ext.flash.Component-cfg-backgroundColor\" class=\"docClass\">backgroundColor</a> and make the background of the Flash\nmovie transparent.</p>\n"},"swf":{"!doc":"<p>A reference to the object or embed element into which the SWF file is loaded. Only\npopulated after the component is rendered and the SWF has been successfully embedded.</p>\n"},"EXPRESS_INSTALL_URL":{"!type":"string","!doc":"<p>The url for installing flash if it doesn't exist. This should be set to a local resource.\nSee <a href=\"http://get.adobe.com/flashplayer/\">http://get.adobe.com/flashplayer/</a> for details.</p>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"getSwfId":{"!type":"fn() -> !this","!doc":"<p>Retrieves the id of the SWF object/embed element.</p>\n"},"onFailure":{"!type":"fn() -> !this"},"onSuccess":{"!type":"fn() -> !this"},"swfCallback":{"!type":"fn(e: ?) -> !this","!doc":"<p>The callback method for handling an embedding success or failure by SWFObject</p>\n"},"failure":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fired when the Flash movie embedding fails</p>\n"},"success":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fired when the Flash movie has been successfully embedded</p>\n"}}}},"FocusManager":{"!doc":"<p>The FocusManager is responsible for globally:</p>\n\n<ol>\n<li>Managing component focus</li>\n<li>Providing basic keyboard navigation</li>\n<li>(optional) Provide a visual cue for focused components, in the form of a focus ring/frame.</li>\n</ol>\n\n\n<p>To activate the FocusManager, simply call <code><a href=\"#!/api/Ext.FocusManager-event-enable\" rel=\"Ext.FocusManager-event-enable\" class=\"docClass\">Ext.FocusManager.enable</a>();</code>. In turn, you may\ndeactivate the FocusManager by subsequently calling <code><a href=\"#!/api/Ext.FocusManager-event-disable\" rel=\"Ext.FocusManager-event-disable\" class=\"docClass\">Ext.FocusManager.disable</a>();</code>.  The\nFocusManager is disabled by default.</p>\n\n<p>To enable the optional focus frame, pass <code>true</code> or <code>{focusFrame: true}</code> to <a href=\"#!/api/Ext.FocusManager-method-enable\" rel=\"Ext.FocusManager-method-enable\" class=\"docClass\">enable</a>.</p>\n\n<p>Another feature of the FocusManager is to provide basic keyboard focus navigation scoped to any <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>\nthat would like to have navigation between its child <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>'s.</p>\n","enabled":{"!type":"bool","!doc":"<p>Whether or not the FocusManager is currently enabled</p>\n"},"focusFrameCls":{"!type":"string"},"focusedCmp":{"!doc":"<p>The currently focused component.</p>\n"},"whitelist":{"!doc":"<p>A list of xtypes that should ignore certain navigation input keys and\nallow for the default browser event/behavior. These input keys include:</p>\n\n<ol>\n<li>Backspace</li>\n<li>Delete</li>\n<li>Left</li>\n<li>Right</li>\n<li>Up</li>\n<li>Down</li>\n</ol>\n\n\n<p>The FocusManager will not attempt to navigate when a component is an xtype (or descendents thereof)\nthat belongs to this whitelist. E.g., an <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a> should allow\nthe user to move the input cursor left and right, and to delete characters, etc.</p>\n"},"addXTypeToWhitelist":{"!type":"fn(xtype: ?) -> !this","!doc":"<p>Adds the specified xtype to the <a href=\"#!/api/Ext.FocusManager-property-whitelist\" rel=\"Ext.FocusManager-property-whitelist\" class=\"docClass\">whitelist</a>.</p>\n"},"clearComponent":{"!type":"fn(cmp: ?) -> !this"},"disable":{"!type":"fn(fm: ?, eOpts: ?)","!doc":"<p>Fires when the FocusManager is disabled</p>\n"},"enable":{"!type":"fn(fm: ?, eOpts: ?)","!doc":"<p>Fires when the FocusManager is enabled</p>\n"},"focusLast":{"!type":"fn(e: ?) -> !this"},"getKeyNav":{"!type":"fn() -> !this"},"getRootComponents":{"!type":"fn() -> !this"},"handleComponentFocus":{"!type":"fn(cmp: ?, focusEl: ?) -> !this"},"initDOM":{"!type":"fn(options: ?) -> !this"},"isWhitelisted":{"!type":"fn(cmp: ?) -> !this"},"navigateIn":{"!type":"fn(e: ?) -> !this"},"navigateOut":{"!type":"fn(e: ?) -> !this"},"navigateSiblings":{"!type":"fn(e: ?, source: ?, parent: ?) -> !this"},"onComponentBlur":{"!type":"fn(cmp: ?, e: ?) -> !this"},"onComponentDestroy":{"!type":"fn() -> !this"},"onComponentFocus":{"!type":"fn(cmp: ?, e: ?) -> !this"},"onComponentHide":{"!type":"fn(cmp: ?) -> !this"},"removeDOM":{"!type":"fn() -> !this"},"removeXTypeFromWhitelist":{"!type":"fn(xtype: ?) -> !this","!doc":"<p>Removes the specified xtype from the <a href=\"#!/api/Ext.FocusManager-property-whitelist\" rel=\"Ext.FocusManager-property-whitelist\" class=\"docClass\">whitelist</a>.</p>\n"},"setupSubscriberKeys":{"!type":"fn(container: ?, keys: ?) -> !this"},"shouldShowFocusFrame":{"!type":"fn(cmp: ?) -> !this"},"beforecomponentfocus":{"!type":"fn(fm: ?, cmp: ?, previousCmp: ?, eOpts: ?)","!doc":"<p>Fires before a component becomes focused. Return <code>false</code> to prevent\nthe component from gaining focus.</p>\n"},"componentfocus":{"!type":"fn(fm: ?, cmp: ?, previousCmp: ?, eOpts: ?)","!doc":"<p>Fires after a component becomes focused.</p>\n"}},"form":{"action":{"Action":{"!doc":"<p>The subclasses of this class provide actions to perform upon <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Form</a>s.</p>\n\n<p>Instances of this class are only created by a <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Form</a> when the Form needs to perform an action\nsuch as submit or load. The Configuration options listed for this class are set through the Form's action methods:\n<a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submit</a>, <a href=\"#!/api/Ext.form.Basic-method-load\" rel=\"Ext.form.Basic-method-load\" class=\"docClass\">load</a> and <a href=\"#!/api/Ext.form.Basic-method-doAction\" rel=\"Ext.form.Basic-method-doAction\" class=\"docClass\">doAction</a></p>\n\n<p>The instance of Action which performed the action is passed to the success and failure callbacks of the Form's action\nmethods (<a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submit</a>, <a href=\"#!/api/Ext.form.Basic-method-load\" rel=\"Ext.form.Basic-method-load\" class=\"docClass\">load</a> and\n<a href=\"#!/api/Ext.form.Basic-method-doAction\" rel=\"Ext.form.Basic-method-doAction\" class=\"docClass\">doAction</a>), and to the <a href=\"#!/api/Ext.form.Basic-event-actioncomplete\" rel=\"Ext.form.Basic-event-actioncomplete\" class=\"docClass\">actioncomplete</a> and\n<a href=\"#!/api/Ext.form.Basic-event-actionfailed\" rel=\"Ext.form.Basic-event-actionfailed\" class=\"docClass\">actionfailed</a> event handlers.</p>\n","!type":"fn(config?: ?)","prototype":{"failure":{"!doc":"<p>The function to call when a failure packet was received, or when an error ocurred in the Ajax communication.</p>\n"},"form":{"!doc":"<p>The <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">BasicForm</a> instance that is invoking this Action. Required.</p>\n"},"headers":{"!doc":"<p>Extra headers to be sent in the AJAX request for submit and load actions.\nSee Ext.data.proxy.Ajax.headers.</p>\n\n<p><strong>Note:</strong> Headers are not sent during file upload.</p>\n"},"method":{"!type":"string","!doc":"<p>The HTTP method to use to access the requested URL.\nDefaults to the <a href=\"#!/api/Ext.form.Basic-cfg-method\" rel=\"Ext.form.Basic-cfg-method\" class=\"docClass\">BasicForm's method</a>, or 'POST' if not specified.</p>\n"},"params":{"!doc":"<p>Extra parameter values to pass. These are added to the Form's <a href=\"#!/api/Ext.form.Basic-cfg-baseParams\" rel=\"Ext.form.Basic-cfg-baseParams\" class=\"docClass\">Ext.form.Basic.baseParams</a> and passed to the\nspecified URL along with the Form's input fields.</p>\n\n<p>Parameters are encoded as standard HTTP parameters using Ext.Object.toQueryString.</p>\n"},"reset":{"!type":"bool","!doc":"<p>When set to <strong>true</strong>, causes the Form to be <a href=\"#!/api/Ext.form.Basic-method-reset\" rel=\"Ext.form.Basic-method-reset\" class=\"docClass\">reset</a> on Action success. If specified,\nthis happens before the <a href=\"#!/api/Ext.form.action.Action-cfg-success\" rel=\"Ext.form.action.Action-cfg-success\" class=\"docClass\">success</a> callback is called and before the Form's\n<a href=\"#!/api/Ext.form.Basic-event-actioncomplete\" rel=\"Ext.form.Basic-event-actioncomplete\" class=\"docClass\">actioncomplete</a> event fires.</p>\n"},"scope":{"!doc":"<p>The scope in which to call the configured <a href=\"#!/api/Ext.form.action.Action-cfg-success\" rel=\"Ext.form.action.Action-cfg-success\" class=\"docClass\">success</a> and <a href=\"#!/api/Ext.form.action.Action-cfg-failure\" rel=\"Ext.form.action.Action-cfg-failure\" class=\"docClass\">failure</a> callback functions\n(the <code>this</code> reference for the callback functions).</p>\n"},"submitEmptyText":{"!type":"bool","!doc":"<p>If set to true, the emptyText value will be sent with the form when it is submitted.</p>\n"},"success":{"!doc":"<p>The function to call when a valid success return packet is received.</p>\n"},"timeout":{"!type":"number","!doc":"<p>The number of seconds to wait for a server response before failing with the <a href=\"#!/api/Ext.form.action.Action-property-failureType\" rel=\"Ext.form.action.Action-property-failureType\" class=\"docClass\">failureType</a> as\n<a href=\"#!/api/Ext.form.action.Action-static-property-CONNECT_FAILURE\" rel=\"Ext.form.action.Action-static-property-CONNECT_FAILURE\" class=\"docClass\">CONNECT_FAILURE</a>. If not specified, defaults to the configured\n<a href=\"#!/api/Ext.form.Basic-cfg-timeout\" rel=\"Ext.form.Basic-cfg-timeout\" class=\"docClass\">timeout</a> of the <a href=\"#!/api/Ext.form.action.Action-cfg-form\" rel=\"Ext.form.action.Action-cfg-form\" class=\"docClass\">form</a>.</p>\n"},"url":{"!type":"string","!doc":"<p>The URL that the Action is to invoke. Will default to the <a href=\"#!/api/Ext.form.Basic-cfg-url\" rel=\"Ext.form.Basic-cfg-url\" class=\"docClass\">url</a> configured on the\n<a href=\"#!/api/Ext.form.action.Action-cfg-form\" rel=\"Ext.form.action.Action-cfg-form\" class=\"docClass\">form</a>.</p>\n"},"waitMsg":{"!type":"string","!doc":"<p>The message to be displayed by a call to <a href=\"#!/api/Ext.window.MessageBox-method-wait\" rel=\"Ext.window.MessageBox-method-wait\" class=\"docClass\">Ext.window.MessageBox.wait</a> during the time the action is being\nprocessed.</p>\n"},"waitTitle":{"!type":"string","!doc":"<p>The title to be displayed by a call to <a href=\"#!/api/Ext.window.MessageBox-method-wait\" rel=\"Ext.window.MessageBox-method-wait\" class=\"docClass\">Ext.window.MessageBox.wait</a> during the time the action is being\nprocessed.</p>\n"},"failureType":{"!type":"string","!doc":"<p>The type of failure detected will be one of these:\n<a href=\"#!/api/Ext.form.action.Action-static-property-CLIENT_INVALID\" rel=\"Ext.form.action.Action-static-property-CLIENT_INVALID\" class=\"docClass\">CLIENT_INVALID</a>, <a href=\"#!/api/Ext.form.action.Action-static-property-SERVER_INVALID\" rel=\"Ext.form.action.Action-static-property-SERVER_INVALID\" class=\"docClass\">SERVER_INVALID</a>, <a href=\"#!/api/Ext.form.action.Action-static-property-CONNECT_FAILURE\" rel=\"Ext.form.action.Action-static-property-CONNECT_FAILURE\" class=\"docClass\">CONNECT_FAILURE</a>, or <a href=\"#!/api/Ext.form.action.Action-static-property-LOAD_FAILURE\" rel=\"Ext.form.action.Action-static-property-LOAD_FAILURE\" class=\"docClass\">LOAD_FAILURE</a>.</p>\n\n<p>Usage:</p>\n\n<pre><code>var fp = new <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>({\n...\nbuttons: [{\n    text: 'Save',\n    formBind: true,\n    handler: function(){\n        if(fp.getForm().isValid()){\n            fp.getForm().submit({\n                url: 'form-submit.php',\n                waitMsg: 'Submitting your data...',\n                success: function(form, action){\n                    // server responded with success = true\n                    var result = action.<a href=\"#!/api/Ext.form.action.Action-property-result\" rel=\"Ext.form.action.Action-property-result\" class=\"docClass\">result</a>;\n                },\n                failure: function(form, action){\n                    if (action.<a href=\"#!/api/Ext.form.action.Action-property-failureType\" rel=\"Ext.form.action.Action-property-failureType\" class=\"docClass\">failureType</a> === <a href=\"#!/api/Ext.form.action.Action-static-property-CONNECT_FAILURE\" rel=\"Ext.form.action.Action-static-property-CONNECT_FAILURE\" class=\"docClass\">Ext.form.action.Action.CONNECT_FAILURE</a>) {\n                        <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Error',\n                            'Status:'+action.<a href=\"#!/api/Ext.form.action.Action-property-response\" rel=\"Ext.form.action.Action-property-response\" class=\"docClass\">response</a>.status+': '+\n                            action.<a href=\"#!/api/Ext.form.action.Action-property-response\" rel=\"Ext.form.action.Action-property-response\" class=\"docClass\">response</a>.statusText);\n                    }\n                    if (action.failureType === <a href=\"#!/api/Ext.form.action.Action-static-property-SERVER_INVALID\" rel=\"Ext.form.action.Action-static-property-SERVER_INVALID\" class=\"docClass\">Ext.form.action.Action.SERVER_INVALID</a>){\n                        // server responded with success = false\n                        <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Invalid', action.<a href=\"#!/api/Ext.form.action.Action-property-result\" rel=\"Ext.form.action.Action-property-result\" class=\"docClass\">result</a>.errormsg);\n                    }\n                }\n            });\n        }\n    }\n},{\n    text: 'Reset',\n    handler: function(){\n        fp.getForm().reset();\n    }\n}]\n</code></pre>\n"},"response":{"!doc":"<p>The raw XMLHttpRequest object used to perform the action.</p>\n"},"result":{"!doc":"<p>The decoded response object containing a boolean <code>success</code> property and other, action-specific properties.</p>\n"},"type":{"!type":"string","!doc":"<p>The type of action this Action instance performs. Currently only \"submit\" and \"load\" are supported.</p>\n"},"CLIENT_INVALID":{"!type":"string","!doc":"<p>Failure type returned when client side validation of the Form fails thus aborting a submit action. Client\nside validation is performed unless <a href=\"#!/api/Ext.form.action.Submit-cfg-clientValidation\" rel=\"Ext.form.action.Submit-cfg-clientValidation\" class=\"docClass\">Ext.form.action.Submit.clientValidation</a> is explicitly set to\nfalse.</p>\n"},"CONNECT_FAILURE":{"!type":"string","!doc":"<p>Failure type returned when a communication error happens when attempting to send a request to the remote\nserver. The <a href=\"#!/api/Ext.form.action.Action-property-response\" rel=\"Ext.form.action.Action-property-response\" class=\"docClass\">response</a> may be examined to provide further information.</p>\n"},"LOAD_FAILURE":{"!type":"string","!doc":"<p>Failure type returned when the response's <code>success</code> property is set to false, or no field values are returned\nin the response's data property.</p>\n"},"SERVER_INVALID":{"!type":"string","!doc":"<p>Failure type returned when server side processing fails and the <a href=\"#!/api/Ext.form.action.Action-property-result\" rel=\"Ext.form.action.Action-property-result\" class=\"docClass\">result</a>'s <code>success</code> property is set to\nfalse.</p>\n\n<p>In the case of a form submission, field-specific error messages may be returned in the <a href=\"#!/api/Ext.form.action.Action-property-result\" rel=\"Ext.form.action.Action-property-result\" class=\"docClass\">result</a>'s\nerrors property.</p>\n"},"createCallback":{"!type":"fn() -> !this","!doc":"<p>Creates a callback object.</p>\n"},"getMethod":{"!type":"fn() -> string","!doc":"<p>Determine the HTTP method to be used for the request.</p>\n"},"getParams":{"!type":"fn() -> ?","!doc":"<p>Get the set of parameters specified in the BasicForm's baseParams and/or the params option.\nItems in params override items of the same name in baseParams.</p>\n"},"getUrl":{"!type":"fn() -> string","!doc":"<p>Build the URL for the AJAX request. Used by the standard AJAX submit and load actions.</p>\n"},"handleResponse":{"!type":"fn(response: ?)","!doc":"<p>Handles the raw response and builds a result object from it. Must be implemented by subclasses.</p>\n"},"onFailure":{"!type":"fn(response: ?) -> !this","!doc":"<p>Handles a failure response.</p>\n"},"onSuccess":{"!type":"fn(response: ?)","!doc":"<p>Callback method that gets invoked when the action completes successfully. Must be implemented by subclasses.</p>\n"},"processResponse":{"!type":"fn(response: ?) -> ?","!doc":"<p>Validates that a response contains either responseText or responseXML and invokes\n<a href=\"#!/api/Ext.form.action.Action-method-handleResponse\" rel=\"Ext.form.action.Action-method-handleResponse\" class=\"docClass\">handleResponse</a> to build the result object.</p>\n"},"run":{"!type":"fn() -> !this","!doc":"<p>Invokes this action using the current configuration.</p>\n"}}},"DirectLoad":{"!doc":"<p>Provides Ext.direct.Manager support for loading form data.</p>\n\n<p>This example illustrates usage of Ext.direct.Direct to <strong>load</strong> a form through Ext.Direct.</p>\n\n<pre><code>var myFormPanel = new <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>({\n    // configs for FormPanel\n    title: 'Basic Information',\n    renderTo: document.body,\n    width: 300, height: 160,\n    padding: 10,\n\n    // configs apply to child items\n    defaults: {anchor: '100%'},\n    defaultType: 'textfield',\n    items: [{\n        fieldLabel: 'Name',\n        name: 'name'\n    },{\n        fieldLabel: 'Email',\n        name: 'email'\n    },{\n        fieldLabel: 'Company',\n        name: 'company'\n    }],\n\n    // configs for BasicForm\n    api: {\n        // The server-side method to call for load() requests\n        load: Profile.getBasicInfo,\n        // The server-side must mark the submit handler as a 'formHandler'\n        submit: Profile.updateBasicInfo\n    },\n    // specify the order for the passed params\n    paramOrder: ['uid', 'foo']\n});\n\n// load the form\nmyFormPanel.getForm().load({\n    // pass 2 arguments to server side getBasicInfo method (len=2)\n    params: {\n        foo: 'bar',\n        uid: 34\n    }\n});\n</code></pre>\n\n<p>The data packet sent to the server will resemble something like:</p>\n\n<pre><code>[\n    {\n        \"action\":\"Profile\",\"method\":\"getBasicInfo\",\"type\":\"rpc\",\"tid\":2,\n        \"data\":[34,\"bar\"] // note the order of the params\n    }\n]\n</code></pre>\n\n<p>The form will process a data packet returned by the server that is similar to the following format:</p>\n\n<pre><code>[\n    {\n        \"action\":\"Profile\",\"method\":\"getBasicInfo\",\"type\":\"rpc\",\"tid\":2,\n        \"result\":{\n            \"success\":true,\n            \"data\":{\n                \"name\":\"Fred Flintstone\",\n                \"company\":\"Slate Rock and Gravel\",\n                \"email\":\"fred.flintstone@slaterg.com\"\n            }\n        }\n    }\n]\n</code></pre>\n","!type":"fn(config?: ?)","prototype":{"type":{"!type":"string","!doc":"<p>The type of action this Action instance performs. Currently only \"submit\" and \"load\" are supported.</p>\n"},"onComplete":{"!type":"fn(data: ?) -> !this"},"processResponse":{"!type":"fn(response: ?) -> ?","!doc":"<p>Direct actions have already been processed and therefore\nwe can directly set the result; Direct Actions do not have\na this.response property.</p>\n"},"run":{"!type":"fn() -> !this"}}},"Load":{"prototype":{"handleResponse":{"!type":"fn(response: ?) -> !this"},"onSuccess":{"!type":"fn(response: ?) -> !this"},"type":{"!type":"string","!doc":"<p>The type of action this Action instance performs. Currently only \"submit\" and \"load\" are supported.</p>\n"},"run":{"!type":"fn() -> !this"}},"!doc":"<p>A class which handles loading of data from a server into the Fields of an <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a>.</p>\n\n<p>Instances of this class are only created by a <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Form</a> when <a href=\"#!/api/Ext.form.Basic-method-load\" rel=\"Ext.form.Basic-method-load\" class=\"docClass\">load</a>ing.</p>\n\n<h2>Response Packet Criteria</h2>\n\n<p>A response packet <strong>must</strong> contain:</p>\n\n<ul>\n<li><strong><code>success</code></strong> property : Boolean</li>\n<li><strong><code>data</code></strong> property : Object</li>\n</ul>\n\n\n<p>The <code>data</code> property contains the values of Fields to load. The individual value object for each Field is passed to\nthe Field's <a href=\"#!/api/Ext.form.field.Field-method-setValue\" rel=\"Ext.form.field.Field-method-setValue\" class=\"docClass\">setValue</a> method.</p>\n\n<h2>JSON Packets</h2>\n\n<p>By default, response packets are assumed to be JSON, so for the following form load call:</p>\n\n<pre><code>var myFormPanel = new <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>({\n    title: 'Client and routing info',\n    renderTo: Ext.getBody(),\n    defaults: {\n        xtype: 'textfield'\n    },\n    items: [{\n        fieldLabel: 'Client',\n        name: 'clientName'\n    }, {\n        fieldLabel: 'Port of loading',\n        name: 'portOfLoading'\n    }, {\n        fieldLabel: 'Port of discharge',\n        name: 'portOfDischarge'\n    }]\n});\nmyFormPanel.<a href=\"#!/api/Ext.form.Panel-method-getForm\" rel=\"Ext.form.Panel-method-getForm\" class=\"docClass\">getForm</a>().<a href=\"#!/api/Ext.form.Basic-method-load\" rel=\"Ext.form.Basic-method-load\" class=\"docClass\">load</a>({\n    url: '/getRoutingInfo.php',\n    params: {\n        consignmentRef: myConsignmentRef\n    },\n    failure: function(form, action) {\n        <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>(\"Load failed\", action.result.errorMessage);\n    }\n});\n</code></pre>\n\n<p>a <strong>success response</strong> packet may look like this:</p>\n\n<pre><code>{\n    success: true,\n    data: {\n        clientName: \"Fred. Olsen Lines\",\n        portOfLoading: \"FXT\",\n        portOfDischarge: \"OSL\"\n    }\n}\n</code></pre>\n\n<p>while a <strong>failure response</strong> packet may look like this:</p>\n\n<pre><code>{\n    success: false,\n    errorMessage: \"Consignment reference not found\"\n}\n</code></pre>\n\n<p>Other data may be placed into the response for processing the <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Form</a>'s callback or event handler\nmethods. The object decoded from this JSON is available in the <a href=\"#!/api/Ext.form.action.Action-property-result\" rel=\"Ext.form.action.Action-property-result\" class=\"docClass\">result</a> property.</p>\n","!type":"fn(config?: ?)"},"DirectSubmit":{"!doc":"<p>Provides Ext.direct support for submitting form data.</p>\n\n<p>This example illustrates usage of Ext.direct.Direct to <strong>submit</strong> a form through Ext.Direct.</p>\n\n<pre><code>var myFormPanel = new <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>({\n    // configs for FormPanel\n    title: 'Basic Information',\n    renderTo: document.body,\n    width: 300, height: 160,\n    padding: 10,\n    buttons:[{\n        text: 'Submit',\n        handler: function(){\n            myFormPanel.getForm().submit({\n                params: {\n                    foo: 'bar',\n                    uid: 34\n                }\n            });\n        }\n    }],\n\n    // configs apply to child items\n    defaults: {anchor: '100%'},\n    defaultType: 'textfield',\n    items: [{\n        fieldLabel: 'Name',\n        name: 'name'\n    },{\n        fieldLabel: 'Email',\n        name: 'email'\n    },{\n        fieldLabel: 'Company',\n        name: 'company'\n    }],\n\n    // configs for BasicForm\n    api: {\n        // The server-side method to call for load() requests\n        load: Profile.getBasicInfo,\n        // The server-side must mark the submit handler as a 'formHandler'\n        submit: Profile.updateBasicInfo\n    },\n    // specify the order for the passed params\n    paramOrder: ['uid', 'foo']\n});\n</code></pre>\n\n<p>The data packet sent to the server will resemble something like:</p>\n\n<pre><code>{\n    \"action\":\"Profile\",\"method\":\"updateBasicInfo\",\"type\":\"rpc\",\"tid\":\"6\",\n    \"result\":{\n        \"success\":true,\n        \"id\":{\n            \"extAction\":\"Profile\",\"extMethod\":\"updateBasicInfo\",\n            \"extType\":\"rpc\",\"extTID\":\"6\",\"extUpload\":\"false\",\n            \"name\":\"Aaron Conran\",\"email\":\"aaron@sencha.com\",\"company\":\"Sencha Inc.\"\n        }\n    }\n}\n</code></pre>\n\n<p>The form will process a data packet returned by the server that is similar to the following:</p>\n\n<pre><code>// sample success packet (batched requests)\n[\n    {\n        \"action\":\"Profile\",\"method\":\"updateBasicInfo\",\"type\":\"rpc\",\"tid\":3,\n        \"result\":{\n            \"success\":true\n        }\n    }\n]\n\n// sample failure packet (one request)\n{\n        \"action\":\"Profile\",\"method\":\"updateBasicInfo\",\"type\":\"rpc\",\"tid\":\"6\",\n        \"result\":{\n            \"errors\":{\n                \"email\":\"already taken\"\n            },\n            \"success\":false,\n            \"foo\":\"bar\"\n        }\n}\n</code></pre>\n\n<p>Also see the discussion in <a href=\"#!/api/Ext.form.action.DirectLoad\" rel=\"Ext.form.action.DirectLoad\" class=\"docClass\">Ext.form.action.DirectLoad</a>.</p>\n","!type":"fn(config?: ?)","prototype":{"type":{"!type":"string","!doc":"<p>The type of action this Action instance performs. Currently only \"submit\" and \"load\" are supported.</p>\n"},"doSubmit":{"!type":"fn() -> !this","!doc":"<p>Performs the submit of the form data.</p>\n"},"onComplete":{"!type":"fn(data: ?) -> !this"},"processResponse":{"!type":"fn(response: ?) -> ?","!doc":"<p>Direct actions have already been processed and therefore\nwe can directly set the result; Direct Actions do not have\na this.response property.</p>\n"}}},"Submit":{"prototype":{"clientValidation":{"!type":"bool","!doc":"<p>Determines whether a Form's fields are validated in a final call to <a href=\"#!/api/Ext.form.Basic-method-isValid\" rel=\"Ext.form.Basic-method-isValid\" class=\"docClass\">isValid</a> prior\nto submission. Pass false in the Form's submit options to prevent this.</p>\n"},"buildForm":{"!type":"fn() -> ?","!doc":"<p>Builds a form element containing fields corresponding to all the parameters to be\nsubmitted (everything returned by <a href=\"#!/api/Ext.form.action.Submit-method-getParams\" rel=\"Ext.form.action.Submit-method-getParams\" class=\"docClass\">getParams</a>.</p>\n\n<p>NOTE: the form element is automatically added to the DOM, so any code that uses\nit must remove it from the DOM after finishing with it.</p>\n"},"cleanup":{"!type":"fn(formInfo: ?) -> !this"},"getFieldConfig":{"!type":"fn(name: ?, value: ?) -> !this"},"getParams":{"!type":"fn(useModelValues: ?) -> !this","!doc":"<p>Builds the full set of parameters from the field values plus any additional configured params.</p>\n"},"handleResponse":{"!type":"fn(response: ?) -> !this"},"onSuccess":{"!type":"fn(response: ?) -> !this"},"run":{"!type":"fn() -> !this","!doc":"<p>inherit docs</p>\n"},"type":{"!type":"string","!doc":"<p>The type of action this Action instance performs. Currently only \"submit\" and \"load\" are supported.</p>\n"},"doSubmit":{"!type":"fn() -> !this","!doc":"<p>Performs the submit of the form data.</p>\n"}},"!doc":"<p>A class which handles submission of data from <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Form</a>s and processes the returned response.</p>\n\n<p>Instances of this class are only created by a <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Form</a> when\n<a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submit</a>ting.</p>\n\n<h1>Response Packet Criteria</h1>\n\n<p>A response packet may contain:</p>\n\n<ul>\n<li><p><strong><code>success</code></strong> property : Boolean - required.</p></li>\n<li><p><strong><code>errors</code></strong> property : Object - optional, contains error messages for invalid fields.</p></li>\n</ul>\n\n\n<h1>JSON Packets</h1>\n\n<p>By default, response packets are assumed to be JSON, so a typical response packet may look like this:</p>\n\n<pre><code>{\n    success: false,\n    errors: {\n        clientCode: \"Client not found\",\n        portOfLoading: \"This field must not be null\"\n    }\n}\n</code></pre>\n\n<p>Other data may be placed into the response for processing by the <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a>'s callback or event handler\nmethods. The object decoded from this JSON is available in the <a href=\"#!/api/Ext.form.action.Action-property-result\" rel=\"Ext.form.action.Action-property-result\" class=\"docClass\">result</a> property.</p>\n\n<p>Alternatively, if an <a href=\"#!/api/Ext.form.Basic-cfg-errorReader\" rel=\"Ext.form.Basic-cfg-errorReader\" class=\"docClass\">errorReader</a> is specified as an\nXmlReader:</p>\n\n<pre><code>errorReader: new Ext.data.reader.Xml({\n        record : 'field',\n        success: '@success'\n    }, [\n        'id', 'msg'\n    ]\n)\n</code></pre>\n\n<p>then the results may be sent back in XML format:</p>\n\n<pre><code>&lt;?xml version=\"1.0\" encoding=\"UTF-8\"?&gt;\n&lt;message success=\"false\"&gt;\n&lt;errors&gt;\n    &lt;field&gt;\n        &lt;id&gt;clientCode&lt;/id&gt;\n        &lt;msg&gt;&lt;![CDATA[Code not found. &lt;br /&gt;&lt;i&gt;This is a test validation message from the server &lt;/i&gt;]]&gt;&lt;/msg&gt;\n    &lt;/field&gt;\n    &lt;field&gt;\n        &lt;id&gt;portOfLoading&lt;/id&gt;\n        &lt;msg&gt;&lt;![CDATA[Port not found. &lt;br /&gt;&lt;i&gt;This is a test validation message from the server &lt;/i&gt;]]&gt;&lt;/msg&gt;\n    &lt;/field&gt;\n&lt;/errors&gt;\n&lt;/message&gt;\n</code></pre>\n\n<p>Other elements may be placed into the response XML for processing by the <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a>'s callback or event\nhandler methods. The XML document is available in the <a href=\"#!/api/Ext.form.Basic-cfg-errorReader\" rel=\"Ext.form.Basic-cfg-errorReader\" class=\"docClass\">errorReader</a>'s\nxmlData property.</p>\n","!type":"fn(config?: ?)"},"StandardSubmit":{"!doc":"<p>A class which handles submission of data from <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Form</a>s using a standard <code>&lt;form&gt;</code> element submit.\nIt does not handle the response from the submit.</p>\n\n<p>If validation of the form fields fails, the Form's afterAction method will be called. Otherwise, afterAction will not\nbe called.</p>\n\n<p>Instances of this class are only created by a <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Form</a> when\n<a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submit</a>ting, when the form's <a href=\"#!/api/Ext.form.Basic-cfg-standardSubmit\" rel=\"Ext.form.Basic-cfg-standardSubmit\" class=\"docClass\">Ext.form.Basic.standardSubmit</a> config option is true.</p>\n","!type":"fn(config?: ?)","prototype":{"target":{"!type":"string","!doc":"<p>Optional target attribute to be used for the form when submitting.</p>\n\n<p>Defaults to the current window/frame.</p>\n"},"doSubmit":{"!type":"fn() -> !this","!doc":"<p>Perform the form submit. Creates and submits a temporary form element containing an input element for each\nfield value returned by <a href=\"#!/api/Ext.form.Basic-method-getValues\" rel=\"Ext.form.Basic-method-getValues\" class=\"docClass\">Ext.form.Basic.getValues</a>, plus any configured <a href=\"#!/api/Ext.form.action.StandardSubmit-cfg-params\" rel=\"Ext.form.action.StandardSubmit-cfg-params\" class=\"docClass\">params</a> or\n<a href=\"#!/api/Ext.form.Basic-cfg-baseParams\" rel=\"Ext.form.Basic-cfg-baseParams\" class=\"docClass\">baseParams</a>.</p>\n"}}}},"Basic":{"!doc":"<p>Provides input field management, validation, submission, and form loading services for the collection\nof <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Field</a> instances within a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>. It is recommended\nthat you use a <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a> as the form container, as that has logic to automatically\nhook up an instance of <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a> (plus other conveniences related to field configuration.)</p>\n\n<h2>Form Actions</h2>\n\n<p>The Basic class delegates the handling of form loads and submits to instances of <a href=\"#!/api/Ext.form.action.Action\" rel=\"Ext.form.action.Action\" class=\"docClass\">Ext.form.action.Action</a>.\nSee the various Action implementations for specific details of each one's functionality, as well as the\ndocumentation for <a href=\"#!/api/Ext.form.Basic-method-doAction\" rel=\"Ext.form.Basic-method-doAction\" class=\"docClass\">doAction</a> which details the configuration options that can be specified in\neach action call.</p>\n\n<p>The default submit Action is <a href=\"#!/api/Ext.form.action.Submit\" rel=\"Ext.form.action.Submit\" class=\"docClass\">Ext.form.action.Submit</a>, which uses an Ajax request to submit the\nform's values to a configured URL. To enable normal browser submission of an Ext form, use the\n<a href=\"#!/api/Ext.form.Basic-cfg-standardSubmit\" rel=\"Ext.form.Basic-cfg-standardSubmit\" class=\"docClass\">standardSubmit</a> config option.</p>\n\n<h2>File uploads</h2>\n\n<p>File uploads are not performed using normal 'Ajax' techniques; see the description for\n<a href=\"#!/api/Ext.form.Basic-method-hasUpload\" rel=\"Ext.form.Basic-method-hasUpload\" class=\"docClass\">hasUpload</a> for details. If you're using file uploads you should read the method description.</p>\n\n<h2>Example usage:</h2>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'Basic Form',\n    renderTo: Ext.getBody(),\n    bodyPadding: 5,\n    width: 350,\n\n    // Any configuration items here will be automatically passed along to\n    // the <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a> instance when it gets created.\n\n    // The form will submit an AJAX request to this URL when submitted\n    url: 'save-form.php',\n\n    items: [{\n        xtype: 'textfield',\n        fieldLabel: 'Field',\n        name: 'theField'\n    }],\n\n    buttons: [{\n        text: 'Submit',\n        handler: function() {\n            // The getForm() method returns the <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a> instance:\n            var form = this.up('form').getForm();\n            if (form.isValid()) {\n                // Submit the Ajax request and handle the response\n                form.submit({\n                    success: function(form, action) {\n                       <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Success', action.result.message);\n                    },\n                    failure: function(form, action) {\n                        <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Failed', action.result ? action.result.message : 'No response');\n                    }\n                });\n            }\n        }\n    }]\n});\n</code></pre>\n","!type":"fn(owner: ?, config: ?)","prototype":{"api":{"!doc":"<p>If specified, load and submit actions will be handled with <a href=\"#!/api/Ext.form.action.DirectLoad\" rel=\"Ext.form.action.DirectLoad\" class=\"docClass\">DirectLoad</a>\nand <a href=\"#!/api/Ext.form.action.DirectSubmit\" rel=\"Ext.form.action.DirectSubmit\" class=\"docClass\">DirectSubmit</a>.  Methods which have been imported by\nExt.direct.Manager can be specified here to load and submit forms. API methods may also be\nspecified as strings. See Ext.data.proxy.Direct.directFn.  Such as the following:</p>\n\n<pre><code>api: {\n    load: App.ss.MyProfile.load,\n    submit: App.ss.MyProfile.submit\n}\n</code></pre>\n\n<p>Load actions can use <a href=\"#!/api/Ext.form.Basic-cfg-paramOrder\" rel=\"Ext.form.Basic-cfg-paramOrder\" class=\"docClass\">paramOrder</a> or <a href=\"#!/api/Ext.form.Basic-cfg-paramsAsHash\" rel=\"Ext.form.Basic-cfg-paramsAsHash\" class=\"docClass\">paramsAsHash</a> to customize how the load method\nis invoked.  Submit actions will always use a standard form submit. The <code>formHandler</code> configuration\n(see Ext.direct.RemotingProvider#action) must be set on the associated server-side method which has\nbeen imported by Ext.direct.Manager.</p>\n"},"baseParams":{"!doc":"<p>Parameters to pass with all requests. e.g. baseParams: <code>{id: '123', foo: 'bar'}</code>.</p>\n\n<p>Parameters are encoded as standard HTTP parameters using Ext.Object.toQueryString.</p>\n"},"errorReader":{"!doc":"<p>An Ext.data.reader.Reader (e.g. Ext.data.reader.Xml) instance or\nconfiguration to be used to read field error messages returned from 'submit' actions.\nThis is optional as there is built-in support for processing JSON responses.</p>\n\n<p>The Records which provide messages for the invalid Fields must use the\nField name (or id) as the Record ID, and must contain a field called 'msg'\nwhich contains the error message.</p>\n\n<p>The errorReader does not have to be a full-blown implementation of a\nReader. It simply needs to implement a <code>read(xhr)</code> function\nwhich returns an Array of Records in an object with the following\nstructure:</p>\n\n<pre><code>{\n    records: recordArray\n}\n</code></pre>\n"},"jsonSubmit":{"!type":"bool","!doc":"<p>If set to true, the field values are sent as JSON in the request body.\nAll of the field values, plus any additional params configured via <a href=\"#!/api/Ext.form.Basic-cfg-baseParams\" rel=\"Ext.form.Basic-cfg-baseParams\" class=\"docClass\">baseParams</a>\nand/or the <code>options</code> to <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submit</a>, will be included in the values POSTed in the body of the request.</p>\n"},"method":{"!type":"string","!doc":"<p>The request method to use (GET or POST) for form actions if one isn't supplied in the action options.</p>\n"},"paramOrder":{"!doc":"<p>A list of params to be executed server side. Only used for the <a href=\"#!/api/Ext.form.Basic-cfg-api\" rel=\"Ext.form.Basic-cfg-api\" class=\"docClass\">api</a> <code>load</code>\nconfiguration.</p>\n\n<p>Specify the params in the order in which they must be executed on the\nserver-side as either (1) an Array of String values, or (2) a String of params\ndelimited by either whitespace, comma, or pipe. For example,\nany of the following would be acceptable:</p>\n\n<pre><code>paramOrder: ['param1','param2','param3']\nparamOrder: 'param1 param2 param3'\nparamOrder: 'param1,param2,param3'\nparamOrder: 'param1|param2|param'\n</code></pre>\n"},"paramsAsHash":{"!type":"bool","!doc":"<p>Only used for the <a href=\"#!/api/Ext.form.Basic-cfg-api\" rel=\"Ext.form.Basic-cfg-api\" class=\"docClass\">api</a> <code>load</code> configuration. If true, parameters will be sent as a\nsingle hash collection of named arguments. Providing a <a href=\"#!/api/Ext.form.Basic-cfg-paramOrder\" rel=\"Ext.form.Basic-cfg-paramOrder\" class=\"docClass\">paramOrder</a> nullifies this\nconfiguration.</p>\n"},"reader":{"!doc":"<p>An Ext.data.reader.Reader (e.g. Ext.data.reader.Xml) instance or\nconfiguration to be used to read data when executing 'load' actions. This\nis optional as there is built-in support for processing JSON responses.</p>\n"},"standardSubmit":{"!type":"bool","!doc":"<p>If set to true, a standard HTML form submit is used instead of a XHR (Ajax) style form submission.\nAll of the field values, plus any additional params configured via <a href=\"#!/api/Ext.form.Basic-cfg-baseParams\" rel=\"Ext.form.Basic-cfg-baseParams\" class=\"docClass\">baseParams</a>\nand/or the <code>options</code> to <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submit</a>, will be included in the values submitted in the form.</p>\n"},"timeout":{"!type":"number","!doc":"<p>Timeout for form actions in seconds.</p>\n"},"trackResetOnLoad":{"!type":"bool","!doc":"<p>If set to true, <a href=\"#!/api/Ext.form.Basic-method-reset\" rel=\"Ext.form.Basic-method-reset\" class=\"docClass\">reset</a>() resets to the last loaded or <a href=\"#!/api/Ext.form.Basic-method-setValues\" rel=\"Ext.form.Basic-method-setValues\" class=\"docClass\">setValues</a>() data instead of\nwhen the form was first created.</p>\n"},"url":{"!type":"string","!doc":"<p>The URL to use for form actions if one isn't supplied in the\n<a href=\"#!/api/Ext.form.Basic-method-doAction\" rel=\"Ext.form.Basic-method-doAction\" class=\"docClass\">doAction</a> options.</p>\n"},"waitMsgTarget":{"!doc":"<p>By default wait messages are displayed with <a href=\"#!/api/Ext.MessageBox-method-wait\" rel=\"Ext.MessageBox-method-wait\" class=\"docClass\">Ext.MessageBox.wait</a>. You can target a specific\nelement by passing it or its id or mask the form itself by passing in true.</p>\n"},"waitTitle":{"!type":"string","!doc":"<p>The default title to show for the waiting message box</p>\n"},"owner":{"!doc":"<p>The container component to which this BasicForm is attached.</p>\n"},"taskDelay":{"!type":"number","!doc":"<p>Not a public API config, this is useful when we're unit testing so we can\nturn off the delayed tasks so they fire immediately.</p>\n"},"wasDirty":{"!type":"bool","!doc":"<p>Private</p>\n"},"afterAction":{"!type":"fn(action: ?, success: ?) -> !this","!doc":"<p>Called after an action is performed via <a href=\"#!/api/Ext.form.Basic-method-doAction\" rel=\"Ext.form.Basic-method-doAction\" class=\"docClass\">doAction</a>.</p>\n"},"applyIfToFields":{"!type":"fn(obj: ?) -> ?","!doc":"<p>Calls Ext.applyIf for all field in this form with the passed object.</p>\n"},"applyToFields":{"!type":"fn(obj: ?) -> ?","!doc":"<p>Calls Ext.apply for all fields in this form with the passed object.</p>\n"},"beforeAction":{"!type":"fn(action: ?) -> !this","!doc":"<p>Called before an action is performed via <a href=\"#!/api/Ext.form.Basic-method-doAction\" rel=\"Ext.form.Basic-method-doAction\" class=\"docClass\">doAction</a>.</p>\n"},"checkDirty":{"!type":"fn() -> !this","!doc":"<p>Check whether the dirty state of the entire form has changed since it was last checked, and\nif so fire the <a href=\"#!/api/Ext.form.Basic-event-dirtychange\" rel=\"Ext.form.Basic-event-dirtychange\" class=\"docClass\">dirtychange</a> event. This is automatically invoked\nwhen an individual field's <code>dirty</code> state changes.</p>\n"},"checkDirtyDelay":{"!type":"fn() -> !this"},"checkValidity":{"!type":"fn() -> !this","!doc":"<p>Check whether the validity of the entire form has changed since it was last checked, and\nif so fire the <a href=\"#!/api/Ext.form.Basic-event-validitychange\" rel=\"Ext.form.Basic-event-validitychange\" class=\"docClass\">validitychange</a> event. This is automatically invoked\nwhen an individual field's validity changes.</p>\n"},"checkValidityDelay":{"!type":"fn() -> !this"},"clearInvalid":{"!type":"fn() -> ?","!doc":"<p>Clears all invalid field messages in this form.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroys this object.</p>\n"},"doAction":{"!type":"fn(action: ?, options?: ?) -> ?","!doc":"<p>Performs a predefined action (an implementation of <a href=\"#!/api/Ext.form.action.Action\" rel=\"Ext.form.action.Action\" class=\"docClass\">Ext.form.action.Action</a>) to perform application-\nspecific processing.</p>\n"},"findField":{"!type":"fn(id: ?) -> ?","!doc":"<p>Find a specific <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Ext.form.field.Field</a> in this form by id or name.</p>\n"},"getBoundItems":{"!type":"fn() -> ?","!doc":"<p>Finds and returns the set of all items bound to fields inside this form</p>\n"},"getFieldValues":{"!type":"fn(dirtyOnly?: ?) -> ?","!doc":"<p>Retrieves the fields in the form as a set of key/value pairs, using their\n<a href=\"#!/api/Ext.form.field.Field-method-getModelData\" rel=\"Ext.form.field.Field-method-getModelData\" class=\"docClass\">getModelData()</a> method to collect the values.\nIf multiple fields return values under the same name those values will be combined into an Array.\nThis is similar to <a href=\"#!/api/Ext.form.Basic-method-getValues\" rel=\"Ext.form.Basic-method-getValues\" class=\"docClass\">getValues</a> except that this method collects type-specific data values\n(e.g. Date objects for date fields) while getValues returns only String values for submission.</p>\n"},"getFields":{"!type":"fn() -> ?","!doc":"<p>Return all the <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Ext.form.field.Field</a> components in the owner container.</p>\n"},"getRecord":{"!type":"fn() -> ?","!doc":"<p>Returns the last Ext.data.Model instance that was loaded via <a href=\"#!/api/Ext.form.Basic-method-loadRecord\" rel=\"Ext.form.Basic-method-loadRecord\" class=\"docClass\">loadRecord</a></p>\n"},"getValues":{"!type":"fn(asString?: ?, dirtyOnly?: ?, includeEmptyText?: ?, useDataValues?: ?) -> ?","!doc":"<p>Retrieves the fields in the form as a set of key/value pairs, using their\n<a href=\"#!/api/Ext.form.field.Field-method-getSubmitData\" rel=\"Ext.form.field.Field-method-getSubmitData\" class=\"docClass\">getSubmitData()</a> method to collect the values.\nIf multiple fields return values under the same name those values will be combined into an Array.\nThis is similar to <a href=\"#!/api/Ext.form.Basic-method-getFieldValues\" rel=\"Ext.form.Basic-method-getFieldValues\" class=\"docClass\">getFieldValues</a> except that this method\ncollects only String values for submission, while getFieldValues collects type-specific data\nvalues (e.g. Date objects for date fields.)</p>\n"},"hasInvalidField":{"!type":"fn() -> !this","!doc":"<p>Returns true if the form contains any invalid fields. No fields will be marked as invalid\nas a result of calling this; to trigger marking of fields use <a href=\"#!/api/Ext.form.Basic-method-isValid\" rel=\"Ext.form.Basic-method-isValid\" class=\"docClass\">isValid</a> instead.</p>\n"},"hasUpload":{"!type":"fn() -> bool","!doc":"<p>Returns <code>true</code> if the form contains a file upload field. This is used to determine the method for submitting the\nform: File uploads are not performed using normal 'Ajax' techniques, that is they are <strong>not</strong> performed using\nXMLHttpRequests. Instead a hidden <code>&lt;form&gt;</code> element containing all the fields is created temporarily and submitted\nwith its <a href=\"http://www.w3.org/TR/REC-html40/present/frames.html#adef-target\">target</a> set to refer to a dynamically generated, hidden <code>&lt;iframe&gt;</code> which is inserted into the document\nbut removed after the return data has been gathered.</p>\n\n<p>The server response is parsed by the browser to create the document for the IFRAME. If the server is using JSON\nto send the return object, then the <a href=\"http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.17\">Content-Type</a> header should be set to \"text/plain\" in order to tell the\nbrowser to insert the text unchanged into a '&lt;pre>' element in the document body from which it can be retrieved.</p>\n\n<p>If the <a href=\"http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.17\">Content-Type</a> header is sent as the default, \"text/html\", then characters which are significant to an HTML\nparser must be sent as HTML entities, so encode <code>\"&lt;\"</code> as <code>\"&amp;lt;\"</code>, <code>\"&amp;\"</code> as <code>\"&amp;amp;\"</code> etc.</p>\n\n<p>The response text is retrieved from the document, and a fake XMLHttpRequest object is created containing a\nresponseText property in order to conform to the requirements of event handlers and callbacks.</p>\n\n<p>Be aware that file upload packets are sent with the content type <a href=\"http://www.faqs.org/rfcs/rfc2388.html\">multipart/form</a> and some server technologies\n(notably JEE) may require some custom processing in order to retrieve parameter names and parameter values from\nthe packet content.</p>\n"},"initialize":{"!type":"fn() -> !this","!doc":"<p>Do any post layout initialization</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns <code>true</code> if any fields in this form have changed from their original values.</p>\n\n<p>Note that if this BasicForm was configured with <a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a> then the Fields' <em>original values</em> are updated when the values are\nloaded by <a href=\"#!/api/Ext.form.Basic-method-setValues\" rel=\"Ext.form.Basic-method-setValues\" class=\"docClass\">setValues</a> or <a href=\"#!/api/Ext.form.Basic-method-loadRecord\" rel=\"Ext.form.Basic-method-loadRecord\" class=\"docClass\">loadRecord</a>. This means\nthat:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a>: <code>false</code> -> Will return <code>true</code> after calling this method.</li>\n<li><a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a>: <code>true</code> -> Will return <code>false</code> after calling this method.</li>\n</ul>\n\n"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Returns true if client-side validation on the form is successful. Any invalid fields will be\nmarked as invalid. If you only want to determine overall form validity without marking anything,\nuse <a href=\"#!/api/Ext.form.Basic-method-hasInvalidField\" rel=\"Ext.form.Basic-method-hasInvalidField\" class=\"docClass\">hasInvalidField</a> instead.</p>\n"},"load":{"!type":"fn(options: ?) -> ?","!doc":"<p>Shortcut to <a href=\"#!/api/Ext.form.Basic-method-doAction\" rel=\"Ext.form.Basic-method-doAction\" class=\"docClass\">do</a> a <a href=\"#!/api/Ext.form.action.Load\" rel=\"Ext.form.action.Load\" class=\"docClass\">load action</a>.</p>\n"},"loadRecord":{"!type":"fn(record: ?) -> ?","!doc":"<p>Loads an Ext.data.Model into this form by calling <a href=\"#!/api/Ext.form.Basic-method-setValues\" rel=\"Ext.form.Basic-method-setValues\" class=\"docClass\">setValues</a> with the\nrecord data. The fields in the model are mapped to\nfields in the form by matching either the <a href=\"#!/api/Ext.form.field.Base-cfg-name\" rel=\"Ext.form.field.Base-cfg-name\" class=\"docClass\">Ext.form.field.Base.name</a> or <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">Ext.Component.itemId</a>.<br/>\nSee also <a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a>.</p>\n"},"markInvalid":{"!type":"fn(errors: ?) -> ?","!doc":"<p>Mark fields in this form invalid in bulk.</p>\n"},"onFieldAdd":{"!type":"fn(field: ?) -> !this"},"onFieldRemove":{"!type":"fn(field: ?) -> !this"},"onMonitorInvalidate":{"!type":"fn() -> !this"},"onValidityChange":{"!type":"fn(valid: ?) -> !this","!doc":"<p>Handle changes in the form's validity. If there are any sub components with\n<code>formBind=true</code> then they are enabled/disabled based on the new validity.</p>\n"},"reset":{"!type":"fn(resetRecord?: ?) -> ?","!doc":"<p>Resets all fields in this form. By default, any record bound by <a href=\"#!/api/Ext.form.Basic-method-loadRecord\" rel=\"Ext.form.Basic-method-loadRecord\" class=\"docClass\">loadRecord</a>\nwill be retained.</p>\n"},"setValues":{"!type":"fn(values: ?) -> ?","!doc":"<p>Set values for fields in this form in bulk.</p>\n"},"submit":{"!type":"fn(options: ?) -> ?","!doc":"<p>Shortcut to <a href=\"#!/api/Ext.form.Basic-method-doAction\" rel=\"Ext.form.Basic-method-doAction\" class=\"docClass\">do</a> a <a href=\"#!/api/Ext.form.action.Submit\" rel=\"Ext.form.action.Submit\" class=\"docClass\">submit action</a>. This will use the\n<a href=\"#!/api/Ext.form.action.Submit\" rel=\"Ext.form.action.Submit\" class=\"docClass\">AJAX submit action</a> by default. If the <a href=\"#!/api/Ext.form.Basic-cfg-standardSubmit\" rel=\"Ext.form.Basic-cfg-standardSubmit\" class=\"docClass\">standardSubmit</a> config\nis enabled it will use a standard form element to submit, or if the <a href=\"#!/api/Ext.form.Basic-cfg-api\" rel=\"Ext.form.Basic-cfg-api\" class=\"docClass\">api</a> config is present\nit will use the <a href=\"#!/api/Ext.form.action.DirectLoad\" rel=\"Ext.form.action.DirectLoad\" class=\"docClass\">Ext.direct.Direct submit action</a>.</p>\n\n<p>The following code:</p>\n\n<pre><code>myFormPanel.getForm().submit({\n    clientValidation: true,\n    url: 'updateConsignment.php',\n    params: {\n        newStatus: 'delivered'\n    },\n    success: function(form, action) {\n       <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Success', action.result.msg);\n    },\n    failure: function(form, action) {\n        switch (action.failureType) {\n            case <a href=\"#!/api/Ext.form.action.Action-static-property-CLIENT_INVALID\" rel=\"Ext.form.action.Action-static-property-CLIENT_INVALID\" class=\"docClass\">Ext.form.action.Action.CLIENT_INVALID</a>:\n                <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Failure', 'Form fields may not be submitted with invalid values');\n                break;\n            case <a href=\"#!/api/Ext.form.action.Action-static-property-CONNECT_FAILURE\" rel=\"Ext.form.action.Action-static-property-CONNECT_FAILURE\" class=\"docClass\">Ext.form.action.Action.CONNECT_FAILURE</a>:\n                <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Failure', 'Ajax communication failed');\n                break;\n            case <a href=\"#!/api/Ext.form.action.Action-static-property-SERVER_INVALID\" rel=\"Ext.form.action.Action-static-property-SERVER_INVALID\" class=\"docClass\">Ext.form.action.Action.SERVER_INVALID</a>:\n               <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Failure', action.result.msg);\n       }\n    }\n});\n</code></pre>\n\n<p>would process the following server response for a successful submission:</p>\n\n<pre><code>{\n    \"success\":true, // note this is Boolean, not string\n    \"msg\":\"Consignment updated\"\n}\n</code></pre>\n\n<p>and the following server response for a failed submission:</p>\n\n<pre><code>{\n    \"success\":false, // note this is Boolean, not string\n    \"msg\":\"You do not have permission to perform this operation\"\n}\n</code></pre>\n"},"updateRecord":{"!type":"fn(record?: ?) -> ?","!doc":"<p>Persists the values in this form into the passed Ext.data.Model object in a beginEdit/endEdit block.\nIf the record is not specified, it will attempt to update (if it exists) the record provided to loadRecord.</p>\n"},"actioncomplete":{"!type":"fn(this: ?, action: ?, eOpts: ?)","!doc":"<p>Fires when an action is completed.</p>\n"},"actionfailed":{"!type":"fn(this: ?, action: ?, eOpts: ?)","!doc":"<p>Fires when an action fails.</p>\n"},"beforeaction":{"!type":"fn(this: ?, action: ?, eOpts: ?)","!doc":"<p>Fires before any action is performed. Return false to cancel the action.</p>\n"},"dirtychange":{"!type":"fn(this: ?, dirty: ?, eOpts: ?)","!doc":"<p>Fires when the dirty state of the entire form changes.</p>\n"},"validitychange":{"!type":"fn(this: ?, valid: ?, eOpts: ?)","!doc":"<p>Fires when the validity of the entire form changes.</p>\n"}}},"CheckboxGroup":{"!doc":"<p>A <a href=\"#!/api/Ext.form.FieldContainer\" rel=\"Ext.form.FieldContainer\" class=\"docClass\">field container</a> which has a specialized layout for arranging\n<a href=\"#!/api/Ext.form.field.Checkbox\" rel=\"Ext.form.field.Checkbox\" class=\"docClass\">Ext.form.field.Checkbox</a> controls into columns, and provides convenience\n<a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Ext.form.field.Field</a> methods for <a href=\"#!/api/Ext.form.CheckboxGroup-method-getValue\" rel=\"Ext.form.CheckboxGroup-method-getValue\" class=\"docClass\">getting</a>, <a href=\"#!/api/Ext.form.CheckboxGroup-method-setValue\" rel=\"Ext.form.CheckboxGroup-method-setValue\" class=\"docClass\">setting</a>,\nand <a href=\"#!/api/Ext.form.CheckboxGroup-method-validate\" rel=\"Ext.form.CheckboxGroup-method-validate\" class=\"docClass\">validating</a> the group of checkboxes as a whole.</p>\n\n<h1>Validation</h1>\n\n<p>Individual checkbox fields themselves have no default validation behavior, but\nsometimes you want to require a user to select at least one of a group of checkboxes. CheckboxGroup\nallows this by setting the config <code><a href=\"#!/api/Ext.form.CheckboxGroup-cfg-allowBlank\" rel=\"Ext.form.CheckboxGroup-cfg-allowBlank\" class=\"docClass\">allowBlank</a>:false</code>; when the user does not check at\nleast one of the checkboxes, the entire group will be highlighted as invalid and the\n<a href=\"#!/api/Ext.form.CheckboxGroup-cfg-blankText\" rel=\"Ext.form.CheckboxGroup-cfg-blankText\" class=\"docClass\">error message</a> will be displayed according to the <a href=\"#!/api/Ext.form.CheckboxGroup-cfg-msgTarget\" rel=\"Ext.form.CheckboxGroup-cfg-msgTarget\" class=\"docClass\">msgTarget</a> config.</p>\n\n<h1>Layout</h1>\n\n<p>The default layout for CheckboxGroup makes it easy to arrange the checkboxes into\ncolumns; see the <a href=\"#!/api/Ext.form.CheckboxGroup-cfg-columns\" rel=\"Ext.form.CheckboxGroup-cfg-columns\" class=\"docClass\">columns</a> and <a href=\"#!/api/Ext.form.CheckboxGroup-cfg-vertical\" rel=\"Ext.form.CheckboxGroup-cfg-vertical\" class=\"docClass\">vertical</a> config documentation for details. You may also\nuse a completely different layout by setting the <a href=\"#!/api/Ext.form.CheckboxGroup-cfg-layout\" rel=\"Ext.form.CheckboxGroup-cfg-layout\" class=\"docClass\">layout</a> to one of the other supported layout\ntypes; for instance you may wish to use a custom arrangement of hbox and vbox containers. In that case\nthe checkbox components at any depth will still be managed by the CheckboxGroup's validation.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'Checkbox Group',\n    width: 300,\n    height: 125,\n    bodyPadding: 10,\n    renderTo: Ext.getBody(),\n    items:[{\n        xtype: 'checkboxgroup',\n        fieldLabel: 'Two Columns',\n        // Arrange checkboxes into two columns, distributed vertically\n        columns: 2,\n        vertical: true,\n        items: [\n            { boxLabel: 'Item 1', name: 'rb', inputValue: '1' },\n            { boxLabel: 'Item 2', name: 'rb', inputValue: '2', checked: true },\n            { boxLabel: 'Item 3', name: 'rb', inputValue: '3' },\n            { boxLabel: 'Item 4', name: 'rb', inputValue: '4' },\n            { boxLabel: 'Item 5', name: 'rb', inputValue: '5' },\n            { boxLabel: 'Item 6', name: 'rb', inputValue: '6' }\n        ]\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"allowBlank":{"!type":"bool","!doc":"<p>False to validate that at least one item in the group is checked. If no items are selected at\nvalidation time, <a href=\"#!/api/Ext.form.CheckboxGroup-cfg-blankText\" rel=\"Ext.form.CheckboxGroup-cfg-blankText\" class=\"docClass\">blankText</a> will be used as the error text.</p>\n"},"blankText":{"!type":"string","!doc":"<p>Error text to display if the <a href=\"#!/api/Ext.form.CheckboxGroup-cfg-allowBlank\" rel=\"Ext.form.CheckboxGroup-cfg-allowBlank\" class=\"docClass\">allowBlank</a> validation fails</p>\n"},"columns":{"!doc":"<p>Specifies the number of columns to use when displaying grouped checkbox/radio controls using automatic layout.\nThis config can take several types of values:</p>\n\n<ul>\n<li>'auto' - The controls will be rendered one per column on one row and the width of each column will be evenly\ndistributed based on the width of the overall field container. This is the default.</li>\n<li>Number - If you specific a number (e.g., 3) that number of columns will be created and the contained controls\nwill be automatically distributed based on the value of <a href=\"#!/api/Ext.form.CheckboxGroup-cfg-vertical\" rel=\"Ext.form.CheckboxGroup-cfg-vertical\" class=\"docClass\">vertical</a>.</li>\n<li>Array - You can also specify an array of column widths, mixing integer (fixed width) and float (percentage\nwidth) values as needed (e.g., [100, .25, .75]). Any integer values will be rendered first, then any float\nvalues will be calculated as a percentage of the remaining space. Float values do not have to add up to 1\n(100%) although if you want the controls to take up the entire field container you should do so.</li>\n</ul>\n\n"},"componentCls":{"!type":"string","!doc":"<p>CSS Class to be added to a components root level element to give distinction to it via styling.</p>\n"},"defaultType":{"!type":"string","!doc":"<p>private</p>\n"},"items":{"!doc":"<p>An Array of <a href=\"#!/api/Ext.form.field.Checkbox\" rel=\"Ext.form.field.Checkbox\" class=\"docClass\">Checkbox</a>es or Checkbox config objects to arrange in the group.</p>\n"},"layout":{"!doc":"<p>private</p>\n"},"name":{"!type":"string"},"vertical":{"!type":"bool","!doc":"<p>True to distribute contained controls across columns, completely filling each column top to bottom before\nstarting on the next column. The number of controls in each column will be automatically calculated to keep\ncolumns as even as possible. The default value is false, so that controls will be added to columns one at a time,\ncompletely filling each row left to right before starting on the next row.</p>\n"},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"extraFieldBodyCls":{"!type":"string","!doc":"<p>private</p>\n"},"groupCls":{"!type":"string","!doc":"<p>private</p>\n"},"eachBox":{"!type":"fn(fn: ?, scope?: ?) -> !this","!doc":"<p>Convenience function which calls the given function for every checkbox in the group</p>\n"},"getBoxes":{"!type":"fn(query?: ?) -> !this","!doc":"<p>Returns all checkbox components within the container</p>\n"},"getChecked":{"!type":"fn() -> ?","!doc":"<p>Returns an Array of all checkboxes in the container which are currently checked</p>\n"},"getErrors":{"!type":"fn() -> ?","!doc":"<p>Runs CheckboxGroup's validations and returns an array of any errors. The only error by default is if allowBlank\nis set to true and no items are checked.</p>\n"},"getModelData":{"!type":"fn() -> ?","!doc":"<p>Don't return any data for the model; the form will get the info from the individual checkboxes themselves.</p>\n"},"getSubmitData":{"!type":"fn() -> ?","!doc":"<p>Don't return any data for submit; the form will get the info from the individual checkboxes themselves.</p>\n"},"getValue":{"!type":"fn() -> !this","!doc":"<p>Returns an object containing the values of all checked checkboxes within the group. Each key-value pair in the\nobject corresponds to a checkbox <a href=\"#!/api/Ext.form.field.Checkbox-cfg-name\" rel=\"Ext.form.field.Checkbox-cfg-name\" class=\"docClass\">name</a>. If there is only one checked checkbox\nwith a particular name, the value of that pair will be the String <a href=\"#!/api/Ext.form.field.Checkbox-cfg-inputValue\" rel=\"Ext.form.field.Checkbox-cfg-inputValue\" class=\"docClass\">inputValue</a> of that checkbox. If there are multiple checked checkboxes with that name, the value of that pair\nwill be an Array of the selected inputValues.</p>\n\n<p>The object format returned from this method can also be passed directly to the <a href=\"#!/api/Ext.form.CheckboxGroup-method-setValue\" rel=\"Ext.form.CheckboxGroup-method-setValue\" class=\"docClass\">setValue</a> method.</p>\n\n<p>NOTE: In Ext 3, this method returned an array of Checkbox components; this was changed to make it more consistent\nwith other field components and with the <a href=\"#!/api/Ext.form.CheckboxGroup-method-setValue\" rel=\"Ext.form.CheckboxGroup-method-setValue\" class=\"docClass\">setValue</a> argument signature. If you need the old behavior in\nExt 4+, use the <a href=\"#!/api/Ext.form.CheckboxGroup-method-getChecked\" rel=\"Ext.form.CheckboxGroup-method-getChecked\" class=\"docClass\">getChecked</a> method instead.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initValue":{"!type":"fn() -> !this","!doc":"<p>Initializes the field's value based on the initial config. If the <a href=\"#!/api/Ext.form.CheckboxGroup-cfg-value\" rel=\"Ext.form.CheckboxGroup-cfg-value\" class=\"docClass\">value</a> config is specified then we use\nthat to set the value; otherwise we initialize the originalValue by querying the values of all sub-checkboxes\nafter they have been initialized.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>private override</p>\n"},"isEqual":{"!type":"fn(value1: ?, value2: ?) -> bool","!doc":"<p>private override - the group value is a complex object, compare using object serialization</p>\n"},"onAdd":{"!type":"fn(field: ?) -> !this","!doc":"<p>When a checkbox is added to the group, monitor it for changes</p>\n"},"onRemove":{"!type":"fn(item: ?) -> !this","!doc":"<p>Called when a <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">Ext.form.Labelable</a> instance is removed from the container's subtree.</p>\n"},"reset":{"!type":"fn() -> !this","!doc":"<p>Resets the checked state of all <a href=\"#!/api/Ext.form.field.Checkbox\" rel=\"Ext.form.field.Checkbox\" class=\"docClass\">checkboxes</a> in the group to their originally\nloaded values and clears any validation messages.\nSee <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a>.<a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a></p>\n"},"resetOriginalValue":{"!type":"fn() -> !this","!doc":"<p>Resets the field's <a href=\"#!/api/Ext.form.CheckboxGroup-property-originalValue\" rel=\"Ext.form.CheckboxGroup-property-originalValue\" class=\"docClass\">originalValue</a> property so it matches the current <a href=\"#!/api/Ext.form.CheckboxGroup-method-getValue\" rel=\"Ext.form.CheckboxGroup-method-getValue\" class=\"docClass\">value</a>. This is\ncalled by <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a>.<a href=\"#!/api/Ext.form.Basic-method-setValues\" rel=\"Ext.form.Basic-method-setValues\" class=\"docClass\">setValues</a> if the form's\n<a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a> property is set to true.</p>\n"},"setReadOnly":{"!type":"fn(readOnly: ?) -> !this","!doc":"<p>private override</p>\n"},"setValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Sets the value(s) of all checkboxes in the group. The expected format is an Object of name-value pairs\ncorresponding to the names of the checkboxes in the group. Each pair can have either a single or multiple values:</p>\n\n<ul>\n<li>A single Boolean or String value will be passed to the <code>setValue</code> method of the checkbox with that name.\nSee the rules in <a href=\"#!/api/Ext.form.field.Checkbox-method-setValue\" rel=\"Ext.form.field.Checkbox-method-setValue\" class=\"docClass\">Ext.form.field.Checkbox.setValue</a> for accepted values.</li>\n<li>An Array of String values will be matched against the <a href=\"#!/api/Ext.form.field.Checkbox-cfg-inputValue\" rel=\"Ext.form.field.Checkbox-cfg-inputValue\" class=\"docClass\">inputValue</a>\nof checkboxes in the group with that name; those checkboxes whose inputValue exists in the array will be\nchecked and others will be unchecked.</li>\n</ul>\n\n\n<p>If a checkbox's name is not in the mapping at all, it will be unchecked.</p>\n\n<p>An example:</p>\n\n<pre><code>var myCheckboxGroup = new <a href=\"#!/api/Ext.form.CheckboxGroup\" rel=\"Ext.form.CheckboxGroup\" class=\"docClass\">Ext.form.CheckboxGroup</a>({\n    columns: 3,\n    items: [{\n        name: 'cb1',\n        boxLabel: 'Single 1'\n    }, {\n        name: 'cb2',\n        boxLabel: 'Single 2'\n    }, {\n        name: 'cb3',\n        boxLabel: 'Single 3'\n    }, {\n        name: 'cbGroup',\n        boxLabel: 'Grouped 1'\n        inputValue: 'value1'\n    }, {\n        name: 'cbGroup',\n        boxLabel: 'Grouped 2'\n        inputValue: 'value2'\n    }, {\n        name: 'cbGroup',\n        boxLabel: 'Grouped 3'\n        inputValue: 'value3'\n    }]\n});\n\nmyCheckboxGroup.setValue({\n    cb1: true,\n    cb3: false,\n    cbGroup: ['value1', 'value3']\n});\n</code></pre>\n\n<p>The above code will cause the checkbox named 'cb1' to be checked, as well as the first and third checkboxes named\n'cbGroup'. The other three checkboxes will be unchecked.</p>\n"},"validate":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not the field value is currently valid by <a href=\"#!/api/Ext.form.CheckboxGroup-method-getErrors\" rel=\"Ext.form.CheckboxGroup-method-getErrors\" class=\"docClass\">validating</a> the field's current\nvalue, and fires the <a href=\"#!/api/Ext.form.CheckboxGroup-event-validitychange\" rel=\"Ext.form.CheckboxGroup-event-validitychange\" class=\"docClass\">validitychange</a> event if the field's validity has changed since the last validation.\n<strong>Note</strong>: <a href=\"#!/api/Ext.form.CheckboxGroup-cfg-disabled\" rel=\"Ext.form.CheckboxGroup-cfg-disabled\" class=\"docClass\">disabled</a> fields are always treated as valid.</p>\n\n<p>Custom implementations of this method are allowed to have side-effects such as triggering error message display.\nTo validate without side-effects, use <a href=\"#!/api/Ext.form.CheckboxGroup-method-isValid\" rel=\"Ext.form.CheckboxGroup-method-isValid\" class=\"docClass\">isValid</a>.</p>\n"}}},"Labelable":{"prototype":{"activeError":{"!type":"string","!doc":"<p>If specified, then the component will be displayed with this value as its active error when first rendered. Use\n<a href=\"#!/api/Ext.form.Labelable-method-setActiveError\" rel=\"Ext.form.Labelable-method-setActiveError\" class=\"docClass\">setActiveError</a> or <a href=\"#!/api/Ext.form.Labelable-method-unsetActiveError\" rel=\"Ext.form.Labelable-method-unsetActiveError\" class=\"docClass\">unsetActiveError</a> to change it after component creation.</p>\n"},"activeErrorsTpl":{"!doc":"<p>The template used to format the Array of error messages passed to <a href=\"#!/api/Ext.form.Labelable-method-setActiveErrors\" rel=\"Ext.form.Labelable-method-setActiveErrors\" class=\"docClass\">setActiveErrors</a> into a single HTML\nstring. if the <a href=\"#!/api/Ext.form.Labelable-cfg-msgTarget\" rel=\"Ext.form.Labelable-cfg-msgTarget\" class=\"docClass\">msgTarget</a> is title, it defaults to a list separated by new lines. Otherwise, it\nrenders each message as an item in an unordered list.</p>\n"},"afterBodyEl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nat the end of the input containing element. If an <code>XTemplate</code> is used, the component's <a href=\"#!/api/Ext.Component-cfg-renderData\" rel=\"Ext.Component-cfg-renderData\" class=\"docClass\">render data</a>\nserves as the context.</p>\n"},"afterLabelTextTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nafter the label text. If an <code>XTemplate</code> is used, the component's <a href=\"#!/api/Ext.Component-cfg-renderData\" rel=\"Ext.Component-cfg-renderData\" class=\"docClass\">render data</a>\nserves as the context.</p>\n"},"afterLabelTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nafter the label element. If an <code>XTemplate</code> is used, the component's <a href=\"#!/api/Ext.Component-cfg-renderData\" rel=\"Ext.Component-cfg-renderData\" class=\"docClass\">render data</a>\nserves as the context.</p>\n"},"afterSubTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nafter the <a href=\"#!/api/Ext.form.Labelable-method-getSubTplMarkup\" rel=\"Ext.form.Labelable-method-getSubTplMarkup\" class=\"docClass\">subTpl markup</a>. If an <code>XTemplate</code> is used, the\ncomponent's <a href=\"#!/api/Ext.Component-cfg-renderData\" rel=\"Ext.Component-cfg-renderData\" class=\"docClass\">render data</a> serves as the context.</p>\n"},"autoFitErrors":{"!type":"bool","!doc":"<p>Whether to adjust the component's body width to make room for 'side'\n<a href=\"#!/api/Ext.form.Labelable-cfg-msgTarget\" rel=\"Ext.form.Labelable-cfg-msgTarget\" class=\"docClass\">error messages</a>.</p>\n"},"baseBodyCls":{"!type":"string","!doc":"<p>The CSS class to be applied to the body content element.</p>\n"},"beforeBodyEl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nat the beginning of the input containing element. If an <code>XTemplate</code> is used, the component's <a href=\"#!/api/Ext.Component-cfg-renderData\" rel=\"Ext.Component-cfg-renderData\" class=\"docClass\">render data</a>\nserves as the context.</p>\n"},"beforeLabelTextTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nbefore the label text. If an <code>XTemplate</code> is used, the component's <a href=\"#!/api/Ext.Component-cfg-renderData\" rel=\"Ext.Component-cfg-renderData\" class=\"docClass\">render data</a>\nserves as the context.</p>\n"},"beforeLabelTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nbefore the label element. If an <code>XTemplate</code> is used, the component's <a href=\"#!/api/Ext.Component-cfg-renderData\" rel=\"Ext.Component-cfg-renderData\" class=\"docClass\">render data</a>\nserves as the context.</p>\n"},"beforeSubTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nbefore the <a href=\"#!/api/Ext.form.Labelable-method-getSubTplMarkup\" rel=\"Ext.form.Labelable-method-getSubTplMarkup\" class=\"docClass\">subTpl markup</a>. If an <code>XTemplate</code> is used, the\ncomponent's <a href=\"#!/api/Ext.Component-cfg-renderData\" rel=\"Ext.Component-cfg-renderData\" class=\"docClass\">render data</a> serves as the context.</p>\n"},"errorMsgCls":{"!type":"string","!doc":"<p>The CSS class to be applied to the error message element.</p>\n"},"fieldBodyCls":{"!type":"string","!doc":"<p>An extra CSS class to be applied to the body content element in addition to <a href=\"#!/api/Ext.form.Labelable-cfg-baseBodyCls\" rel=\"Ext.form.Labelable-cfg-baseBodyCls\" class=\"docClass\">baseBodyCls</a>.</p>\n"},"fieldLabel":{"!type":"string","!doc":"<p>The label for the field. It gets appended with the <a href=\"#!/api/Ext.form.Labelable-cfg-labelSeparator\" rel=\"Ext.form.Labelable-cfg-labelSeparator\" class=\"docClass\">labelSeparator</a>, and its position and sizing is\ndetermined by the <a href=\"#!/api/Ext.form.Labelable-cfg-labelAlign\" rel=\"Ext.form.Labelable-cfg-labelAlign\" class=\"docClass\">labelAlign</a> and <a href=\"#!/api/Ext.form.Labelable-cfg-labelWidth\" rel=\"Ext.form.Labelable-cfg-labelWidth\" class=\"docClass\">labelWidth</a> configs.</p>\n"},"formItemCls":{"!type":"string","!doc":"<p>A CSS class to be applied to the outermost element to denote that it is participating in the form field layout.</p>\n"},"hideEmptyLabel":{"!type":"bool","!doc":"<p>When set to true, the label element (<a href=\"#!/api/Ext.form.Labelable-cfg-fieldLabel\" rel=\"Ext.form.Labelable-cfg-fieldLabel\" class=\"docClass\">fieldLabel</a> and <a href=\"#!/api/Ext.form.Labelable-cfg-labelSeparator\" rel=\"Ext.form.Labelable-cfg-labelSeparator\" class=\"docClass\">labelSeparator</a>) will be automatically\nhidden if the <a href=\"#!/api/Ext.form.Labelable-cfg-fieldLabel\" rel=\"Ext.form.Labelable-cfg-fieldLabel\" class=\"docClass\">fieldLabel</a> is empty. Setting this to false will cause the empty label element to be\nrendered and space to be reserved for it; this is useful if you want a field without a label to line up with\nother labeled fields in the same form.</p>\n\n<p>If you wish to unconditionall hide the label even if a non-empty fieldLabel is configured, then set the\n<a href=\"#!/api/Ext.form.Labelable-cfg-hideLabel\" rel=\"Ext.form.Labelable-cfg-hideLabel\" class=\"docClass\">hideLabel</a> config to true.</p>\n"},"hideLabel":{"!type":"bool","!doc":"<p>Set to true to completely hide the label element (<a href=\"#!/api/Ext.form.Labelable-cfg-fieldLabel\" rel=\"Ext.form.Labelable-cfg-fieldLabel\" class=\"docClass\">fieldLabel</a> and <a href=\"#!/api/Ext.form.Labelable-cfg-labelSeparator\" rel=\"Ext.form.Labelable-cfg-labelSeparator\" class=\"docClass\">labelSeparator</a>). Also see\n<a href=\"#!/api/Ext.form.Labelable-cfg-hideEmptyLabel\" rel=\"Ext.form.Labelable-cfg-hideEmptyLabel\" class=\"docClass\">hideEmptyLabel</a>, which controls whether space will be reserved for an empty fieldLabel.</p>\n"},"labelAlign":{"!type":"string","!doc":"<p>Controls the position and alignment of the <a href=\"#!/api/Ext.form.Labelable-cfg-fieldLabel\" rel=\"Ext.form.Labelable-cfg-fieldLabel\" class=\"docClass\">fieldLabel</a>. Valid values are:</p>\n\n<ul>\n<li>\"left\" (the default) - The label is positioned to the left of the field, with its text aligned to the left.\nIts width is determined by the <a href=\"#!/api/Ext.form.Labelable-cfg-labelWidth\" rel=\"Ext.form.Labelable-cfg-labelWidth\" class=\"docClass\">labelWidth</a> config.</li>\n<li>\"top\" - The label is positioned above the field.</li>\n<li>\"right\" - The label is positioned to the left of the field, with its text aligned to the right.\nIts width is determined by the <a href=\"#!/api/Ext.form.Labelable-cfg-labelWidth\" rel=\"Ext.form.Labelable-cfg-labelWidth\" class=\"docClass\">labelWidth</a> config.</li>\n</ul>\n\n"},"labelAttrTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\ninside the label element (as attributes). If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.Component-cfg-renderData\" rel=\"Ext.Component-cfg-renderData\" class=\"docClass\">render data</a> serves as the context.</p>\n"},"labelCls":{"!type":"string","!doc":"<p>The CSS class to be applied to the label element. This (single) CSS class is used to formulate the renderSelector\nand drives the field layout where it is concatenated with a hyphen ('-') and <a href=\"#!/api/Ext.form.Labelable-cfg-labelAlign\" rel=\"Ext.form.Labelable-cfg-labelAlign\" class=\"docClass\">labelAlign</a>. To add\nadditional classes, use <a href=\"#!/api/Ext.form.Labelable-cfg-labelClsExtra\" rel=\"Ext.form.Labelable-cfg-labelClsExtra\" class=\"docClass\">labelClsExtra</a>.</p>\n"},"labelClsExtra":{"!type":"string","!doc":"<p>An optional string of one or more additional CSS classes to add to the label element. Defaults to empty.</p>\n"},"labelPad":{"!type":"number","!doc":"<p>The amount of space in pixels between the <a href=\"#!/api/Ext.form.Labelable-cfg-fieldLabel\" rel=\"Ext.form.Labelable-cfg-fieldLabel\" class=\"docClass\">fieldLabel</a> and the field body.\nThis defaults to <code>5</code> for compatibility with Ext JS 4, however, as of Ext JS 5\nthe space between the label and the body can optionally be determined by the theme\nusing the $form-label-horizontal-spacing (for side-aligned labels) and\n$form-label-vertical-spacing (for top-aligned labels) SASS variables.\nIn order for the stylesheet values as to take effect, you must use a labelPad value\nof <code>null</code>.</p>\n"},"labelSeparator":{"!type":"string","!doc":"<p>Character(s) to be inserted at the end of the <a href=\"#!/api/Ext.form.Labelable-cfg-fieldLabel\" rel=\"Ext.form.Labelable-cfg-fieldLabel\" class=\"docClass\">label text</a>.</p>\n\n<p>Set to empty string to hide the separator completely.</p>\n"},"labelStyle":{"!type":"string","!doc":"<p>A CSS style specification string to apply directly to this field's label.</p>\n"},"labelWidth":{"!type":"number","!doc":"<p>The width of the <a href=\"#!/api/Ext.form.Labelable-cfg-fieldLabel\" rel=\"Ext.form.Labelable-cfg-fieldLabel\" class=\"docClass\">fieldLabel</a> in pixels. Only applicable if <a href=\"#!/api/Ext.form.Labelable-cfg-labelAlign\" rel=\"Ext.form.Labelable-cfg-labelAlign\" class=\"docClass\">labelAlign</a>\nis set to \"left\" or \"right\".</p>\n"},"labelableRenderTpl":{"!doc":"<p>The rendering template for the field decorations. Component classes using this mixin\nshould include logic to use this as their <a href=\"#!/api/Ext.Component-cfg-renderTpl\" rel=\"Ext.Component-cfg-renderTpl\" class=\"docClass\">renderTpl</a>,\nand implement the <a href=\"#!/api/Ext.form.Labelable-method-getSubTplMarkup\" rel=\"Ext.form.Labelable-method-getSubTplMarkup\" class=\"docClass\">getSubTplMarkup</a> method to generate the field body content.</p>\n"},"msgTarget":{"!type":"string","!doc":"<p>The location where the error message text should display. Must be one of the following values:</p>\n\n<ul>\n<li><p><code>qtip</code> Display a quick tip containing the message when the user hovers over the field.\nThis is the default.</p>\n\n<p><strong><a href=\"#!/api/Ext.tip.QuickTipManager-method-init\" rel=\"Ext.tip.QuickTipManager-method-init\" class=\"docClass\">Ext.tip.QuickTipManager.init</a> must have been called for this setting to work.</strong></p></li>\n<li><p><code>title</code> Display the message in a default browser title attribute popup.</p></li>\n<li><code>under</code> Add a block div beneath the field containing the error message.</li>\n<li><code>side</code> Add an error icon to the right of the field, displaying the message in a popup on hover.</li>\n<li><code>none</code> Don't display any error message. This might be useful if you are implementing custom error display.</li>\n<li><code>[element id]</code> Add the error message directly to the innerHTML of the specified element.</li>\n</ul>\n\n"},"preventMark":{"!type":"bool","!doc":"<p>true to disable displaying any <a href=\"#!/api/Ext.form.Labelable-method-setActiveError\" rel=\"Ext.form.Labelable-method-setActiveError\" class=\"docClass\">error message</a> set on this object.</p>\n"},"bodyEl":{"!doc":"<p>The div Element wrapping the component's contents. Only available after the component has been rendered.</p>\n"},"errorEl":{"!doc":"<p>The div Element that will contain the component's error message(s). Note that depending on the configured\n<a href=\"#!/api/Ext.form.Labelable-cfg-msgTarget\" rel=\"Ext.form.Labelable-cfg-msgTarget\" class=\"docClass\">msgTarget</a>, this element may be hidden in favor of some other form of presentation, but will always\nbe present in the DOM for use by assistive technologies.</p>\n"},"errorWrapCls":{"!type":"string"},"errorWrapSideCls":{"!type":"string"},"errorWrapUnderCls":{"!type":"string"},"errorWrapUnderSideLabelCls":{"!type":"string"},"htmlActiveErrorsTpl":{},"invalidIconCls":{"!type":"string"},"invalidUnderCls":{"!type":"string"},"isFieldLabelable":{"!type":"bool","!doc":"<p>Flag denoting that this object is labelable as a field. Always true.</p>\n"},"isLabelable":{"!type":"bool"},"labelEl":{"!doc":"<p>The label Element for this component. Only available after the component has been rendered.</p>\n"},"labelInnerCls":{"!type":"string"},"labelableInsertions":{},"msgTargets":{"!doc":"<p>private map for msg target lookup, if target is not in this map it is assumed\nto be an element id</p>\n"},"noWrap":{"!type":"bool","!doc":"<p>Tells the layout system that the height can be measured immediately because the width does not need setting.</p>\n"},"plaintextActiveErrorsTpl":{},"rightLabelCls":{"!type":"string"},"topLabelCls":{"!type":"string","!doc":"<p>private</p>\n"},"topLabelSideErrorCls":{"!type":"string"},"beforeLabelRender":{"!type":"fn() -> !this"},"getActiveError":{"!type":"fn() -> string","!doc":"<p>Gets the active error message for this component, if any. This does not trigger validation on its own, it merely\nreturns any message that the component may already hold.</p>\n"},"getActiveErrors":{"!type":"fn() -> ?","!doc":"<p>Gets an Array of any active error messages currently applied to the field. This does not trigger validation on\nits own, it merely returns any messages that the component may already hold.</p>\n"},"getChildEls":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.form.Labelable-cfg-childEls\" rel=\"Ext.form.Labelable-cfg-childEls\" class=\"docClass\">childEls</a>.</p>\n"},"getHorizontalPaddingStyle":{"!type":"fn() -> !this","!doc":"<p>hook for rtl</p>\n"},"getInputId":{"!type":"fn() -> string","!doc":"<p>Get the input id, if any, for this component. This is used as the \"for\" attribute on the label element.\nImplementing subclasses may also use this as e.g. the id for their own input element.</p>\n"},"getInsertionRenderData":{"!type":"fn(data: ?, names: ?) -> !this"},"getLabelableRenderData":{"!type":"fn(data: ?) -> ?","!doc":"<p>Generates the arguments for the field decorations <a href=\"#!/api/Ext.form.Labelable-cfg-labelableRenderTpl\" rel=\"Ext.form.Labelable-cfg-labelableRenderTpl\" class=\"docClass\">rendering template</a>.</p>\n"},"hasActiveError":{"!type":"fn() -> bool","!doc":"<p>Tells whether the field currently has an active error message. This does not trigger validation on its own, it\nmerely looks for any message that the component may already hold.</p>\n"},"hasVisibleLabel":{"!type":"fn() -> bool","!doc":"<p>Checks if the field has a visible label</p>\n"},"initLabelable":{"!type":"fn() -> !this","!doc":"<p>Performs initialization of this mixin. Component classes using this mixin should call this method during their\nown initialization.</p>\n"},"onLabelRender":{"!type":"fn() -> !this"},"renderActiveError":{"!type":"fn() -> !this","!doc":"<p>Updates the rendered DOM to match the current activeError. This only updates the content and\nattributes, you'll have to call doComponentLayout to actually update the display.</p>\n"},"setActiveError":{"!type":"fn(msg: ?) -> !this","!doc":"<p>Sets the active error message to the given string. This replaces the entire error message contents with the given\nstring. Also see <a href=\"#!/api/Ext.form.Labelable-method-setActiveErrors\" rel=\"Ext.form.Labelable-method-setActiveErrors\" class=\"docClass\">setActiveErrors</a> which accepts an Array of messages and formats them according to the\n<a href=\"#!/api/Ext.form.Labelable-cfg-activeErrorsTpl\" rel=\"Ext.form.Labelable-cfg-activeErrorsTpl\" class=\"docClass\">activeErrorsTpl</a>. Note that this only updates the error message element's text and attributes, you'll\nhave to call doComponentLayout to actually update the field's layout to match. If the field extends <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a> you should call <a href=\"#!/api/Ext.form.field.Base-method-markInvalid\" rel=\"Ext.form.field.Base-method-markInvalid\" class=\"docClass\">markInvalid</a> instead.</p>\n"},"setActiveErrors":{"!type":"fn(errors: ?) -> !this","!doc":"<p>Set the active error message to an Array of error messages. The messages are formatted into a single message\nstring using the <a href=\"#!/api/Ext.form.Labelable-cfg-activeErrorsTpl\" rel=\"Ext.form.Labelable-cfg-activeErrorsTpl\" class=\"docClass\">activeErrorsTpl</a>. Also see <a href=\"#!/api/Ext.form.Labelable-method-setActiveError\" rel=\"Ext.form.Labelable-method-setActiveError\" class=\"docClass\">setActiveError</a> which allows setting the entire error\ncontents with a single string. Note that this only updates the error message element's text and attributes,\nyou'll have to call doComponentLayout to actually update the field's layout to match. If the field extends\n<a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a> you should call <a href=\"#!/api/Ext.form.field.Base-method-markInvalid\" rel=\"Ext.form.field.Base-method-markInvalid\" class=\"docClass\">markInvalid</a> instead.</p>\n"},"setChildEls":{"!type":"fn(childEls: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.form.Labelable-cfg-childEls\" rel=\"Ext.form.Labelable-cfg-childEls\" class=\"docClass\">childEls</a>.</p>\n"},"setFieldDefaults":{"!type":"fn(defaults: ?) -> !this","!doc":"<p>Applies a set of default configuration values to this Labelable instance. For each of the properties in the given\nobject, check if this component hasOwnProperty that config; if not then it's inheriting a default value from its\nprototype and we should apply the default value.</p>\n"},"setFieldLabel":{"!type":"fn(label: ?) -> !this","!doc":"<p>Set the label of this field.</p>\n"},"toggleInvalidCls":{"!type":"fn(hasError: ?) -> !this","!doc":"<p>Add/remove invalid class(es)</p>\n"},"trimLabelSeparator":{"!type":"fn() -> string","!doc":"<p>Returns the trimmed label by slicing off the label separator character. Can be overridden.</p>\n"},"unsetActiveError":{"!type":"fn() -> !this","!doc":"<p>Clears the active error message(s). Note that this only clears the error message element's text and attributes,\nyou'll have to call doComponentLayout to actually update the field's layout to match. If the field extends <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a> you should call <a href=\"#!/api/Ext.form.field.Base-method-clearInvalid\" rel=\"Ext.form.field.Base-method-clearInvalid\" class=\"docClass\">clearInvalid</a> instead.</p>\n"},"errorchange":{"!type":"fn(this: ?, error: ?, eOpts: ?)","!doc":"<p>Fires when the active error message is changed via <a href=\"#!/api/Ext.form.Labelable-method-setActiveError\" rel=\"Ext.form.Labelable-method-setActiveError\" class=\"docClass\">setActiveError</a>.</p>\n"},"invalidCls":{"!type":"string","!doc":"<p>The CSS class to use when marking the component invalid.</p>\n"},"mixinConfig":{},"getFieldLabel":{"!type":"fn() -> string","!doc":"<p>Returns the label for the field. Defaults to simply returning the <a href=\"#!/api/Ext.form.Labelable-cfg-fieldLabel\" rel=\"Ext.form.Labelable-cfg-fieldLabel\" class=\"docClass\">fieldLabel</a> config. Can be overridden\nto provide a custom generated label.</p>\n"},"childEls":{},"getSubTplMarkup":{"!type":"fn() -> string","!doc":"<p>Gets the markup to be inserted into the outer template's bodyEl. Defaults to empty string, should be implemented\nby classes including this mixin as needed.</p>\n"},"initTip":{"!type":"fn() -> !this","!doc":"<p>Use a custom QuickTip instance separate from the main QuickTips singleton, so that we\ncan give it a custom frame style. Responds to errorqtip rather than the qtip property.</p>\n"}},"!doc":"<p>A mixin which allows a component to be configured and decorated with a label and/or error message as is\ncommon for form fields. This is used by e.g. <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a> and <a href=\"#!/api/Ext.form.FieldContainer\" rel=\"Ext.form.FieldContainer\" class=\"docClass\">Ext.form.FieldContainer</a>\nto let them be managed by the Field layout.</p>\n\n<p>NOTE: This mixin is mainly for internal library use and most users should not need to use it directly. It\nis more likely you will want to use one of the component classes that import this mixin, such as\n<a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a> or <a href=\"#!/api/Ext.form.FieldContainer\" rel=\"Ext.form.FieldContainer\" class=\"docClass\">Ext.form.FieldContainer</a>.</p>\n\n<p>Use of this mixin does not make a component a field in the logical sense, meaning it does not provide any\nlogic or state related to values or validation; that is handled by the related <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Ext.form.field.Field</a>\nmixin. These two mixins may be used separately (for example <a href=\"#!/api/Ext.form.FieldContainer\" rel=\"Ext.form.FieldContainer\" class=\"docClass\">Ext.form.FieldContainer</a> is Labelable but not a\nField), or in combination (for example <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a> implements both and has logic for connecting the\ntwo.)</p>\n\n<p>Component classes which use this mixin should use the Field layout\nor a derivation thereof to properly size and position the label and message according to the component config.\nThey must also call the <a href=\"#!/api/Ext.form.Labelable-method-initLabelable\" rel=\"Ext.form.Labelable-method-initLabelable\" class=\"docClass\">initLabelable</a> method during component initialization to ensure the mixin gets\nset up correctly.</p>\n"},"FieldContainer":{"prototype":{"childEls":{},"combineErrors":{"!type":"bool","!doc":"<p>If set to true, the field container will automatically combine and display the validation errors from\nall the fields it contains as a single error on the container, according to the configured\n<a href=\"#!/api/Ext.form.FieldContainer-cfg-msgTarget\" rel=\"Ext.form.FieldContainer-cfg-msgTarget\" class=\"docClass\">msgTarget</a>. Defaults to false.</p>\n"},"combineLabels":{"!type":"bool","!doc":"<p>If set to true, and there is no defined <a href=\"#!/api/Ext.form.FieldContainer-cfg-fieldLabel\" rel=\"Ext.form.FieldContainer-cfg-fieldLabel\" class=\"docClass\">fieldLabel</a>, the field container will automatically\ngenerate its label by combining the labels of all the fields it contains. Defaults to false.</p>\n"},"componentLayout":{"!doc":"<p>The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout\nmanager which sizes a Component's internal structure in response to the Component being sized.</p>\n\n<p>Generally, developers will not use this configuration as all provided Components which need their internal\nelements sizing (Such as <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">input fields</a>) come with their own componentLayout managers.</p>\n\n<p>The <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used on instances of the base <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>\nclass which simply sizes the Component's encapsulating element to the height and width specified in the\n<a href=\"#!/api/Ext.form.FieldContainer-method-setSize\" rel=\"Ext.form.FieldContainer-method-setSize\" class=\"docClass\">setSize</a> method.</p>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"invalidCls":{"!type":"string","!doc":"<p>If we allow this to mark with the invalidCls it will cascade to all\nchild fields, let them handle themselves</p>\n"},"labelConnector":{"!type":"string","!doc":"<p>The string to use when joining the labels of individual sub-fields, when <a href=\"#!/api/Ext.form.FieldContainer-cfg-combineLabels\" rel=\"Ext.form.FieldContainer-cfg-combineLabels\" class=\"docClass\">combineLabels</a> is\nset to true. Defaults to ', '.</p>\n"},"shrinkWrap":{"!doc":"<p>If this property is a number, it is interpreted as follows:</p>\n\n<ul>\n<li>0 (or <code>false</code>): Neither width nor height depend on content.</li>\n<li>1: Width depends on content (shrink wraps), but height does not.</li>\n<li>2 (default): Height depends on content (shrink wraps), but width does not.</li>\n<li>3 (or <code>true</code>): Both width and height depend on content (shrink wrap).</li>\n</ul>\n\n\n<p>In CSS terms, shrink-wrap width is analogous to an inline-block element as opposed\nto a block-level element. Some container layouts always shrink-wrap their children,\neffectively ignoring this property (e.g., <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">Ext.layout.container.HBox</a>,\n<a href=\"#!/api/Ext.layout.container.VBox\" rel=\"Ext.layout.container.VBox\" class=\"docClass\">Ext.layout.container.VBox</a>, <a href=\"#!/api/Ext.layout.component.Dock\" rel=\"Ext.layout.component.Dock\" class=\"docClass\">Ext.layout.component.Dock</a>).</p>\n"},"customOverflowEl":{"!type":"string","!doc":"<p>Used by the layout system, typically the scrolling el is the targetEl, however we need\nto let it know we're using something different</p>\n"},"fieldSubTpl":{},"maskOnDisable":{"!type":"bool","!doc":"<p>This is an internal flag that you use when creating custom components. By default this is set to <code>true</code> which means\nthat every component gets a mask when it's disabled. Components like FieldContainer, FieldSet, Field, Button, Tab\noverride this property to <code>false</code> since they want to implement custom disable logic.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"getCombinedErrors":{"!type":"fn(invalidFields: ?) -> ?","!doc":"<p>Takes an Array of invalid <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Ext.form.field.Field</a> objects and builds a combined list of error\nmessages from them. Defaults to prepending each message by the field name and a colon. This\ncan be overridden to provide custom combined error message handling, for instance changing\nthe format of each message or sorting the array (it is sorted in order of appearance by default).</p>\n"},"getFieldLabel":{"!type":"fn() -> string","!doc":"<p>Returns the combined field label if <a href=\"#!/api/Ext.form.FieldContainer-cfg-combineLabels\" rel=\"Ext.form.FieldContainer-cfg-combineLabels\" class=\"docClass\">combineLabels</a> is set to true and if there is no\nset <a href=\"#!/api/Ext.form.FieldContainer-cfg-fieldLabel\" rel=\"Ext.form.FieldContainer-cfg-fieldLabel\" class=\"docClass\">fieldLabel</a>. Otherwise returns the fieldLabel like normal. You can also override\nthis method to provide a custom generated label.</p>\n"},"getSubTplData":{"!type":"fn() -> !this"},"getSubTplMarkup":{"!type":"fn() -> string","!doc":"<p>Gets the markup to be inserted into the outer template's bodyEl. Defaults to empty string, should be implemented\nby classes including this mixin as needed.</p>\n"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"onFieldErrorChange":{"!type":"fn() -> !this","!doc":"<p>Fired when the error message of any field within the container changes, and updates the\ncombined error message to match.</p>\n"},"updateLabel":{"!type":"fn() -> !this","!doc":"<p>Updates the content of the labelEl if it is rendered</p>\n"},"componentCls":{"!type":"string","!doc":"<p>CSS Class to be added to a components root level element to give distinction to it via styling.</p>\n"},"onAdd":{"!type":"fn(labelItem: ?) -> !this","!doc":"<p>Called when a <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">Ext.form.Labelable</a> instance is added to the container's subtree.</p>\n"},"onRemove":{"!type":"fn(labelItem: ?) -> !this","!doc":"<p>Called when a <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">Ext.form.Labelable</a> instance is removed from the container's subtree.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"}},"!doc":"<p>FieldContainer is a derivation of <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a> that implements the\n<a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">Labelable</a> mixin. This allows it to be configured so that it is rendered with\na <a href=\"#!/api/Ext.form.FieldContainer-cfg-fieldLabel\" rel=\"Ext.form.FieldContainer-cfg-fieldLabel\" class=\"docClass\">field label</a> and optional <a href=\"#!/api/Ext.form.FieldContainer-cfg-msgTarget\" rel=\"Ext.form.FieldContainer-cfg-msgTarget\" class=\"docClass\">error message</a> around its sub-items.\nThis is useful for arranging a group of fields or other components within a single item in a form, so\nthat it lines up nicely with other fields. A common use is for grouping a set of related fields under\na single label in a form.</p>\n\n<p>The container's configured <a href=\"#!/api/Ext.form.FieldContainer-cfg-items\" rel=\"Ext.form.FieldContainer-cfg-items\" class=\"docClass\">items</a> will be layed out within the field body area according to the\nconfigured <a href=\"#!/api/Ext.form.FieldContainer-cfg-layout\" rel=\"Ext.form.FieldContainer-cfg-layout\" class=\"docClass\">layout</a> type. The default layout is <code>'autocontainer'</code>.</p>\n\n<p>Like regular fields, FieldContainer can inherit its decoration configuration from the\n<a href=\"#!/api/Ext.form.Panel-cfg-fieldDefaults\" rel=\"Ext.form.Panel-cfg-fieldDefaults\" class=\"docClass\">fieldDefaults</a> of an enclosing FormPanel. In addition,\nFieldContainer itself can pass <a href=\"#!/api/Ext.form.FieldContainer-cfg-fieldDefaults\" rel=\"Ext.form.FieldContainer-cfg-fieldDefaults\" class=\"docClass\">fieldDefaults</a> to any <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">fields</a>\nit may itself contain.</p>\n\n<p>If you are grouping a set of <a href=\"#!/api/Ext.form.field.Checkbox\" rel=\"Ext.form.field.Checkbox\" class=\"docClass\">Checkbox</a> or <a href=\"#!/api/Ext.form.field.Radio\" rel=\"Ext.form.field.Radio\" class=\"docClass\">Radio</a>\nfields in a single labeled container, consider using a <a href=\"#!/api/Ext.form.CheckboxGroup\" rel=\"Ext.form.CheckboxGroup\" class=\"docClass\">Ext.form.CheckboxGroup</a>\nor <a href=\"#!/api/Ext.form.RadioGroup\" rel=\"Ext.form.RadioGroup\" class=\"docClass\">Ext.form.RadioGroup</a> instead as they are specialized for handling those types.</p>\n\n<h1>Example</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'FieldContainer Example',\n    width: 550,\n    bodyPadding: 10,\n\n    items: [{\n        xtype: 'fieldcontainer',\n        fieldLabel: 'Last Three Jobs',\n        labelWidth: 100,\n\n        // The body area will contain three text fields, arranged\n        // horizontally, separated by draggable splitters.\n        layout: 'hbox',\n        items: [{\n            xtype: 'textfield',\n            flex: 1\n        }, {\n            xtype: 'splitter'\n        }, {\n            xtype: 'textfield',\n            flex: 1\n        }, {\n            xtype: 'splitter'\n        }, {\n            xtype: 'textfield',\n            flex: 1\n        }]\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<h1>Usage of fieldDefaults</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'FieldContainer Example',\n    width: 350,\n    bodyPadding: 10,\n\n    items: [{\n        xtype: 'fieldcontainer',\n        fieldLabel: 'Your Name',\n        labelWidth: 75,\n        defaultType: 'textfield',\n\n        // Arrange fields vertically, stretched to full width\n        layout: 'anchor',\n        defaults: {\n            layout: '100%'\n        },\n\n        // These config values will be applied to both sub-fields, except\n        // for Last Name which will use its own msgTarget.\n        fieldDefaults: {\n            msgTarget: 'under',\n            labelAlign: 'top'\n        },\n\n        items: [{\n            fieldLabel: 'First Name',\n            name: 'firstName'\n        }, {\n            fieldLabel: 'Last Name',\n            name: 'lastName',\n            msgTarget: 'under'\n        }]\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)"},"field":{"Field":{"prototype":{"disabled":{"!type":"bool","!doc":"<p>True to disable the field. Disabled Fields will not be <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submitted</a>.</p>\n"},"submitValue":{"!type":"bool","!doc":"<p>Setting this to false will prevent the field from being <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submitted</a> even when it is\nnot disabled.</p>\n"},"validateOnChange":{"!type":"bool","!doc":"<p>Specifies whether this field should be validated immediately whenever a change in its value is detected.\nIf the validation results in a change in the field's validity, a <a href=\"#!/api/Ext.form.field.Field-event-validitychange\" rel=\"Ext.form.field.Field-event-validitychange\" class=\"docClass\">validitychange</a> event will be\nfired. This allows the field to show feedback about the validity of its contents immediately as the user is\ntyping.</p>\n\n<p>When set to false, feedback will not be immediate. However the form will still be validated before submitting if\nthe clientValidation option to <a href=\"#!/api/Ext.form.Basic-method-doAction\" rel=\"Ext.form.Basic-method-doAction\" class=\"docClass\">Ext.form.Basic.doAction</a> is enabled, or if the field or form are validated\nmanually.</p>\n\n<p>See also <a href=\"#!/api/Ext.form.field.Base-cfg-checkChangeEvents\" rel=\"Ext.form.field.Base-cfg-checkChangeEvents\" class=\"docClass\">Ext.form.field.Base.checkChangeEvents</a> for controlling how changes to the field's value are\ndetected.</p>\n"},"validation":{"!doc":"<p>This property, when a <code>String</code>, contributes its value to the error state of this\ninstance as reported by <code>getErrors</code>. This property is implicitly bound when the\n<code>value</code> of this field is bound based on <code><a href=\"#!/api/Ext.Component-cfg-modelValidation\" rel=\"Ext.Component-cfg-modelValidation\" class=\"docClass\">Ext.Component.modelValidation</a></code>.</p>\n"},"value":{"!doc":"<p>A value to initialize this field with.</p>\n"},"dirty":{"!type":"bool","!doc":"<p>The dirty state of the field.</p>\n"},"isFormField":{"!type":"bool","!doc":"<p>Flag denoting that this component is a Field. Always true.</p>\n"},"mixinId":{"!type":"string"},"originalValue":{"!doc":"<p>The original value of the field as configured in the <a href=\"#!/api/Ext.form.field.Field-cfg-value\" rel=\"Ext.form.field.Field-cfg-value\" class=\"docClass\">value</a> configuration, or as loaded by the last\nform load operation if the form's <a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a> setting is <code>true</code>.</p>\n"},"suspendCheckChange":{"!type":"number"},"batchChanges":{"!type":"fn(fn: ?) -> !this","!doc":"<p>A utility for grouping a set of modifications which may trigger value changes into a single transaction, to\nprevent excessive firing of <a href=\"#!/api/Ext.form.field.Field-event-change\" rel=\"Ext.form.field.Field-event-change\" class=\"docClass\">change</a> events. This is useful for instance if the field has sub-fields which\nare being updated as a group; you don't want the container field to check its own changed state for each subfield\nchange.</p>\n"},"beforeReset":{"!type":"fn() -> !this","!doc":"<p>Template method before a field is reset.</p>\n"},"checkChange":{"!type":"fn() -> !this","!doc":"<p>Checks whether the value of the field has changed since the last time it was checked.\nIf the value has changed, it:</p>\n\n<ol>\n<li>Fires the <a href=\"#!/api/Ext.form.field.Field-event-change\" rel=\"Ext.form.field.Field-event-change\" class=\"docClass\">change event</a>,</li>\n<li>Performs validation if the <a href=\"#!/api/Ext.form.field.Field-cfg-validateOnChange\" rel=\"Ext.form.field.Field-cfg-validateOnChange\" class=\"docClass\">validateOnChange</a> config is enabled, firing the\n<a href=\"#!/api/Ext.form.field.Field-event-validitychange\" rel=\"Ext.form.field.Field-event-validitychange\" class=\"docClass\">validitychange event</a> if the validity has changed, and</li>\n<li>Checks the <a href=\"#!/api/Ext.form.field.Field-method-isDirty\" rel=\"Ext.form.field.Field-method-isDirty\" class=\"docClass\">dirty state</a> of the field and fires the <a href=\"#!/api/Ext.form.field.Field-event-dirtychange\" rel=\"Ext.form.field.Field-event-dirtychange\" class=\"docClass\">dirtychange event</a>\nif it has changed.</li>\n</ol>\n\n"},"checkDirty":{"!type":"fn() -> !this","!doc":"<p>Checks the <a href=\"#!/api/Ext.form.field.Field-method-isDirty\" rel=\"Ext.form.field.Field-method-isDirty\" class=\"docClass\">isDirty</a> state of the field and if it has changed since the last time it was checked,\nfires the <a href=\"#!/api/Ext.form.field.Field-event-dirtychange\" rel=\"Ext.form.field.Field-event-dirtychange\" class=\"docClass\">dirtychange</a> event.</p>\n"},"clearInvalid":{"!type":"fn() -> !this","!doc":"<p>Clear any invalid styles/messages for this field. Components using this mixin should implement this method to\nupdate the components rendering to clear any existing messages.</p>\n\n<p><strong>Note</strong>: this method does not cause the Field's <a href=\"#!/api/Ext.form.field.Field-method-validate\" rel=\"Ext.form.field.Field-method-validate\" class=\"docClass\">validate</a> or <a href=\"#!/api/Ext.form.field.Field-method-isValid\" rel=\"Ext.form.field.Field-method-isValid\" class=\"docClass\">isValid</a> methods to return <code>true</code>\nif the value does not <em>pass</em> validation. So simply clearing a field's errors will not necessarily allow\nsubmission of forms submitted with the <a href=\"#!/api/Ext.form.action.Submit-cfg-clientValidation\" rel=\"Ext.form.action.Submit-cfg-clientValidation\" class=\"docClass\">Ext.form.action.Submit.clientValidation</a> option set.</p>\n"},"didValueChange":{"!type":"fn(newVal: ?, oldVal: ?) -> !this","!doc":"<p>Checks if the value has changed. Allows subclasses to override for\nany more complex logic.</p>\n"},"extractFileInput":{"!type":"fn() -> ?","!doc":"<p>Only relevant if the instance's <a href=\"#!/api/Ext.form.field.Field-method-isFileUpload\" rel=\"Ext.form.field.Field-method-isFileUpload\" class=\"docClass\">isFileUpload</a> method returns true. Returns a reference to the file input\nDOM element holding the user's selected file. The input will be appended into the submission form and will not be\nreturned, so this method should also create a replacement.</p>\n"},"getFieldIdentifier":{"!type":"fn() -> !this","!doc":"<p>Fields can be editors, and some editors may not have a name property that maps\nto its data index, so it's necessary in these cases to look it up by its dataIndex\nproperty.  See EXTJSIV-11650.</p>\n"},"getName":{"!type":"fn() -> string","!doc":"<p>Returns the <a href=\"#!/api/Ext.form.field.Field-cfg-name\" rel=\"Ext.form.field.Field-cfg-name\" class=\"docClass\">name</a> attribute of the field. This is used as the parameter name\nwhen including the field value in a <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">form submit()</a>.</p>\n"},"getValidation":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.form.field.Field-cfg-validation\" rel=\"Ext.form.field.Field-cfg-validation\" class=\"docClass\">validation</a>.</p>\n"},"initField":{"!type":"fn() -> !this","!doc":"<p>Initializes this Field mixin on the current instance. Components using this mixin should call this method during\ntheir own initialization process.</p>\n"},"isEqualAsString":{"!type":"fn(value1: ?, value2: ?) -> bool","!doc":"<p>Returns whether two values are logically equal.\nSimilar to <a href=\"#!/api/Ext.form.field.Field-method-isEqual\" rel=\"Ext.form.field.Field-method-isEqual\" class=\"docClass\">isEqual</a>, however null or undefined values will be treated as empty strings.</p>\n"},"isFileUpload":{"!type":"fn() -> bool","!doc":"<p>Returns whether this Field is a file upload field; if it returns true, forms will use special techniques for\n<a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submitting the form</a> via AJAX. See <a href=\"#!/api/Ext.form.Basic-method-hasUpload\" rel=\"Ext.form.Basic-method-hasUpload\" class=\"docClass\">Ext.form.Basic.hasUpload</a> for details. If\nthis returns true, the <a href=\"#!/api/Ext.form.field.Field-method-extractFileInput\" rel=\"Ext.form.field.Field-method-extractFileInput\" class=\"docClass\">extractFileInput</a> method must also be implemented to return the corresponding file\ninput element.</p>\n"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not the field value is currently valid by <a href=\"#!/api/Ext.form.field.Field-method-getErrors\" rel=\"Ext.form.field.Field-method-getErrors\" class=\"docClass\">validating</a> the field's current\nvalue. The <a href=\"#!/api/Ext.form.field.Field-event-validitychange\" rel=\"Ext.form.field.Field-event-validitychange\" class=\"docClass\">validitychange</a> event will not be fired; use <a href=\"#!/api/Ext.form.field.Field-method-validate\" rel=\"Ext.form.field.Field-method-validate\" class=\"docClass\">validate</a> instead if you want the event\nto fire. <strong>Note</strong>: <a href=\"#!/api/Ext.form.field.Field-cfg-disabled\" rel=\"Ext.form.field.Field-cfg-disabled\" class=\"docClass\">disabled</a> fields are always treated as valid.</p>\n\n<p>Implementations are encouraged to ensure that this method does not have side-effects such as triggering error\nmessage display.</p>\n"},"markInvalid":{"!type":"fn(errors: ?) -> !this","!doc":"<p>Associate one or more error messages with this field. Components using this mixin should implement this method to\nupdate the component's rendering to display the messages.</p>\n\n<p><strong>Note</strong>: this method does not cause the Field's <a href=\"#!/api/Ext.form.field.Field-method-validate\" rel=\"Ext.form.field.Field-method-validate\" class=\"docClass\">validate</a> or <a href=\"#!/api/Ext.form.field.Field-method-isValid\" rel=\"Ext.form.field.Field-method-isValid\" class=\"docClass\">isValid</a> methods to return <code>false</code>\nif the value does <em>pass</em> validation. So simply marking a Field as invalid will not prevent submission of forms\nsubmitted with the <a href=\"#!/api/Ext.form.action.Submit-cfg-clientValidation\" rel=\"Ext.form.action.Submit-cfg-clientValidation\" class=\"docClass\">Ext.form.action.Submit.clientValidation</a> option set.</p>\n"},"onChange":{"!type":"fn(newVal: ?) -> !this","!doc":"<p>Called when the field's value changes. Performs validation if the <a href=\"#!/api/Ext.form.field.Field-cfg-validateOnChange\" rel=\"Ext.form.field.Field-cfg-validateOnChange\" class=\"docClass\">validateOnChange</a>\nconfig is enabled, and invokes the dirty check.</p>\n"},"onDirtyChange":{"!type":"fn(isDirty: ?) -> !this","!doc":"<p>Called when the field's dirty state changes.</p>\n"},"publishValue":{"!type":"fn(value: ?) -> !this"},"setValidation":{"!type":"fn(validation: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.form.field.Field-cfg-validation\" rel=\"Ext.form.field.Field-cfg-validation\" class=\"docClass\">validation</a>.</p>\n"},"updateValidation":{"!type":"fn() -> !this"},"change":{"!type":"fn(this: ?, newValue: ?, oldValue: ?, eOpts: ?)","!doc":"<p>Fires when the value of a field is changed via the <a href=\"#!/api/Ext.form.field.Field-method-setValue\" rel=\"Ext.form.field.Field-method-setValue\" class=\"docClass\">setValue</a> method.</p>\n"},"dirtychange":{"!type":"fn(this: ?, isDirty: ?, eOpts: ?)","!doc":"<p>Fires when a change in the field's <a href=\"#!/api/Ext.form.field.Field-method-isDirty\" rel=\"Ext.form.field.Field-method-isDirty\" class=\"docClass\">isDirty</a> state is detected.</p>\n"},"validitychange":{"!type":"fn(this: ?, isValid: ?, eOpts: ?)","!doc":"<p>Fires when a change in the field's validity is detected.</p>\n"},"getErrors":{"!type":"fn(value: ?) -> ?","!doc":"<p>Runs this field's validators and returns an array of error messages for any validation failures. This is called\ninternally during validation and would not usually need to be used manually.</p>\n\n<p>Each subclass should override or augment the return value to provide their own errors.</p>\n"},"getModelData":{"!type":"fn(includeEmptyText: ?, isSubmitting: ?) -> ?","!doc":"<p>Returns the value(s) that should be saved to the Ext.data.Model instance for this field, when <a href=\"#!/api/Ext.form.Basic-method-updateRecord\" rel=\"Ext.form.Basic-method-updateRecord\" class=\"docClass\">Ext.form.Basic.updateRecord</a> is called. Typically this will be an object with a single name-value pair, the name\nbeing this field's <a href=\"#!/api/Ext.form.field.Field-method-getName\" rel=\"Ext.form.field.Field-method-getName\" class=\"docClass\">name</a> and the value being its current data value. More advanced field\nimplementations may return more than one name-value pair. The returned values will be saved to the corresponding\nfield names in the Model.</p>\n\n<p>Note that the values returned from this method are not guaranteed to have been successfully <a href=\"#!/api/Ext.form.field.Field-method-validate\" rel=\"Ext.form.field.Field-method-validate\" class=\"docClass\">validated</a>.</p>\n"},"initValue":{"!type":"fn() -> !this","!doc":"<p>Initializes the field's value based on the initial config.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns true if the value of this Field has been changed from its <a href=\"#!/api/Ext.form.field.Field-property-originalValue\" rel=\"Ext.form.field.Field-property-originalValue\" class=\"docClass\">originalValue</a>.\nWill always return false if the field is disabled.</p>\n\n<p>Note that if the owning <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">form</a> was configured with\n<a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a> then the <a href=\"#!/api/Ext.form.field.Field-property-originalValue\" rel=\"Ext.form.field.Field-property-originalValue\" class=\"docClass\">originalValue</a> is updated when\nthe values are loaded by <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a>.<a href=\"#!/api/Ext.form.Basic-method-setValues\" rel=\"Ext.form.Basic-method-setValues\" class=\"docClass\">setValues</a>.</p>\n"},"isEqual":{"!type":"fn(value1: ?, value2: ?) -> bool","!doc":"<p>Returns whether two field <a href=\"#!/api/Ext.form.field.Field-method-getValue\" rel=\"Ext.form.field.Field-method-getValue\" class=\"docClass\">values</a> are logically equal. Field implementations may override this\nto provide custom comparison logic appropriate for the particular field's data type.</p>\n"},"reset":{"!type":"fn() -> !this","!doc":"<p>Resets the current field value to the originally loaded value and clears any validation messages. See <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a>.<a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a></p>\n"},"resetOriginalValue":{"!type":"fn() -> !this","!doc":"<p>Resets the field's <a href=\"#!/api/Ext.form.field.Field-property-originalValue\" rel=\"Ext.form.field.Field-property-originalValue\" class=\"docClass\">originalValue</a> property so it matches the current <a href=\"#!/api/Ext.form.field.Field-method-getValue\" rel=\"Ext.form.field.Field-method-getValue\" class=\"docClass\">value</a>. This is\ncalled by <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a>.<a href=\"#!/api/Ext.form.Basic-method-setValues\" rel=\"Ext.form.Basic-method-setValues\" class=\"docClass\">setValues</a> if the form's\n<a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a> property is set to true.</p>\n"},"validate":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not the field value is currently valid by <a href=\"#!/api/Ext.form.field.Field-method-getErrors\" rel=\"Ext.form.field.Field-method-getErrors\" class=\"docClass\">validating</a> the field's current\nvalue, and fires the <a href=\"#!/api/Ext.form.field.Field-event-validitychange\" rel=\"Ext.form.field.Field-event-validitychange\" class=\"docClass\">validitychange</a> event if the field's validity has changed since the last validation.\n<strong>Note</strong>: <a href=\"#!/api/Ext.form.field.Field-cfg-disabled\" rel=\"Ext.form.field.Field-cfg-disabled\" class=\"docClass\">disabled</a> fields are always treated as valid.</p>\n\n<p>Custom implementations of this method are allowed to have side-effects such as triggering error message display.\nTo validate without side-effects, use <a href=\"#!/api/Ext.form.field.Field-method-isValid\" rel=\"Ext.form.field.Field-method-isValid\" class=\"docClass\">isValid</a>.</p>\n"},"name":{"!type":"string","!doc":"<p>The name of the field. By default this is used as the parameter name when including the\n<a href=\"#!/api/Ext.form.field.Field-method-getSubmitData\" rel=\"Ext.form.field.Field-method-getSubmitData\" class=\"docClass\">field value</a> in a <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">form submit()</a>. To prevent the field from\nbeing included in the form submit, set <a href=\"#!/api/Ext.form.field.Field-cfg-submitValue\" rel=\"Ext.form.field.Field-cfg-submitValue\" class=\"docClass\">submitValue</a> to false.</p>\n"},"getSubmitData":{"!type":"fn() -> ?","!doc":"<p>Returns the parameter(s) that would be included in a standard form submit for this field. Typically this will be\nan object with a single name-value pair, the name being this field's <a href=\"#!/api/Ext.form.field.Field-method-getName\" rel=\"Ext.form.field.Field-method-getName\" class=\"docClass\">name</a> and the value being\nits current stringified value. More advanced field implementations may return more than one name-value pair.</p>\n\n<p>Note that the values returned from this method are not guaranteed to have been successfully <a href=\"#!/api/Ext.form.field.Field-method-validate\" rel=\"Ext.form.field.Field-method-validate\" class=\"docClass\">validated</a>.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Returns the current data value of the field. The type of value returned is particular to the type of the\nparticular field (e.g. a Date object for <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>).</p>\n"},"setValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Sets a data value into the field and runs the change detection and validation.</p>\n"}},"!doc":"<p>This mixin provides a common interface for the logical behavior and state of form fields, including:</p>\n\n<ul>\n<li>Getter and setter methods for field values</li>\n<li>Events and methods for tracking value and validity changes</li>\n<li>Methods for triggering validation</li>\n</ul>\n\n\n<p><strong>NOTE</strong>: When implementing custom fields, it is most likely that you will want to extend the <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a>\ncomponent class rather than using this mixin directly, as BaseField contains additional logic for generating an\nactual DOM complete with <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">label and error message</a> display and a form input field,\nplus methods that bind the Field value getters and setters to the input field's value.</p>\n\n<p>If you do want to implement this mixin directly and don't want to extend <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a>, then\nyou will most likely want to override the following methods with custom implementations: <a href=\"#!/api/Ext.form.field.Field-method-getValue\" rel=\"Ext.form.field.Field-method-getValue\" class=\"docClass\">getValue</a>,\n<a href=\"#!/api/Ext.form.field.Field-method-setValue\" rel=\"Ext.form.field.Field-method-setValue\" class=\"docClass\">setValue</a>, and <a href=\"#!/api/Ext.form.field.Field-method-getErrors\" rel=\"Ext.form.field.Field-method-getErrors\" class=\"docClass\">getErrors</a>. Other methods may be overridden as needed but their base\nimplementations should be sufficient for common cases. You will also need to make sure that <a href=\"#!/api/Ext.form.field.Field-method-initField\" rel=\"Ext.form.field.Field-method-initField\" class=\"docClass\">initField</a>\nis called during the component's initialization.</p>\n"},"Base":{"!doc":"<p>Base class for form fields that provides default event handling, rendering, and other common functionality\nneeded by all form field types. Utilizes the <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Ext.form.field.Field</a> mixin for value handling and validation,\nand the <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">Ext.form.Labelable</a> mixin to provide label and error message display.</p>\n\n<p>In most cases you will want to use a subclass, such as <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a> or <a href=\"#!/api/Ext.form.field.Checkbox\" rel=\"Ext.form.field.Checkbox\" class=\"docClass\">Ext.form.field.Checkbox</a>,\nrather than creating instances of this class directly. However if you are implementing a custom form field,\nusing this as the parent class is recommended.</p>\n\n<h1>Values and Conversions</h1>\n\n<p>Because Base implements the Field mixin, it has a main value that can be initialized with the\n<a href=\"#!/api/Ext.form.field.Base-cfg-value\" rel=\"Ext.form.field.Base-cfg-value\" class=\"docClass\">value</a> config and manipulated via the <a href=\"#!/api/Ext.form.field.Base-method-getValue\" rel=\"Ext.form.field.Base-method-getValue\" class=\"docClass\">getValue</a> and <a href=\"#!/api/Ext.form.field.Base-method-setValue\" rel=\"Ext.form.field.Base-method-setValue\" class=\"docClass\">setValue</a> methods. This main\nvalue can be one of many data types appropriate to the current field, for instance a <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Date</a>\nfield would use a JavaScript Date object as its value type. However, because the field is rendered as a HTML\ninput, this value data type can not always be directly used in the rendered field.</p>\n\n<p>Therefore Base introduces the concept of a \"raw value\". This is the value of the rendered HTML input field,\nand is normally a String. The <a href=\"#!/api/Ext.form.field.Base-method-getRawValue\" rel=\"Ext.form.field.Base-method-getRawValue\" class=\"docClass\">getRawValue</a> and <a href=\"#!/api/Ext.form.field.Base-method-setRawValue\" rel=\"Ext.form.field.Base-method-setRawValue\" class=\"docClass\">setRawValue</a> methods can be used to directly\nwork with the raw value, though it is recommended to use getValue and setValue in most cases.</p>\n\n<p>Conversion back and forth between the main value and the raw value is handled by the <a href=\"#!/api/Ext.form.field.Base-method-valueToRaw\" rel=\"Ext.form.field.Base-method-valueToRaw\" class=\"docClass\">valueToRaw</a> and\n<a href=\"#!/api/Ext.form.field.Base-method-rawToValue\" rel=\"Ext.form.field.Base-method-rawToValue\" class=\"docClass\">rawToValue</a> methods. If you are implementing a subclass that uses a non-String value data type, you\nshould override these methods to handle the conversion.</p>\n\n<h1>Rendering</h1>\n\n<p>The content of the field body is defined by the <a href=\"#!/api/Ext.form.field.Base-cfg-fieldSubTpl\" rel=\"Ext.form.field.Base-cfg-fieldSubTpl\" class=\"docClass\">fieldSubTpl</a> XTemplate, with its argument data\ncreated by the <a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">getSubTplData</a> method. Override this template and/or method to create custom\nfield renderings.</p>\n\n<h1>Example usage:</h1>\n\n<pre class='inline-example '><code>// A simple subclass of Base that creates a HTML5 search field. Redirects to the\n// searchUrl when the Enter key is pressed.222\nExt.define('Ext.form.SearchField', {\n    extend: '<a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a>',\n    alias: 'widget.searchfield',\n\n    inputType: 'search',\n\n    // Config defining the search URL\n    searchUrl: 'http://www.google.com/search?q={0}',\n\n    // Add specialkey listener\n    initComponent: function() {\n        this.callParent();\n        this.on('specialkey', this.checkEnterKey, this);\n    },\n\n    // Handle enter key presses, execute the search if the field has a value\n    checkEnterKey: function(field, e) {\n        var value = this.getValue();\n        if (e.getKey() === e.ENTER &amp;&amp; !Ext.isEmpty(value)) {\n            location.href = Ext.String.format(this.searchUrl, value);\n        }\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'Base Example',\n    bodyPadding: 5,\n    width: 250,\n\n    // Fields will be arranged vertically, stretched to full width\n    layout: 'anchor',\n    defaults: {\n        anchor: '100%'\n    },\n    items: [{\n        xtype: 'searchfield',\n        fieldLabel: 'Search',\n        name: 'query'\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"checkChangeBuffer":{"!type":"number","!doc":"<p>Defines a timeout in milliseconds for buffering <a href=\"#!/api/Ext.form.field.Base-cfg-checkChangeEvents\" rel=\"Ext.form.field.Base-cfg-checkChangeEvents\" class=\"docClass\">checkChangeEvents</a> that fire in rapid succession.\nDefaults to 50 milliseconds.</p>\n"},"checkChangeEvents":{"!doc":"<p>A list of event names that will be listened for on the field's <a href=\"#!/api/Ext.form.field.Base-property-inputEl\" rel=\"Ext.form.field.Base-property-inputEl\" class=\"docClass\">input element</a>, which will cause\nthe field's value to be checked for changes. If a change is detected, the <a href=\"#!/api/Ext.form.field.Base-event-change\" rel=\"Ext.form.field.Base-event-change\" class=\"docClass\">change event</a> will be\nfired, followed by validation if the <a href=\"#!/api/Ext.form.field.Base-cfg-validateOnChange\" rel=\"Ext.form.field.Base-cfg-validateOnChange\" class=\"docClass\">validateOnChange</a> option is enabled.</p>\n\n<p>Defaults to ['change', 'propertychange', 'keyup'] in Internet Explorer, and ['change', 'input', 'textInput', 'keyup',\n'dragdrop'] in other browsers. This catches all the ways that field values can be changed in most supported\nbrowsers; the only known exceptions at the time of writing are:</p>\n\n<ul>\n<li>Safari 3.2 and older: cut/paste in textareas via the context menu, and dragging text into textareas</li>\n<li>Opera 10 and 11: dragging text into text fields and textareas, and cut via the context menu in text fields\nand textareas</li>\n<li>Opera 9: Same as Opera 10 and 11, plus paste from context menu in text fields and textareas</li>\n</ul>\n\n\n<p>If you need to guarantee on-the-fly change notifications including these edge cases, you can call the\n<a href=\"#!/api/Ext.form.field.Base-method-checkChange\" rel=\"Ext.form.field.Base-method-checkChange\" class=\"docClass\">checkChange</a> method on a repeating interval, e.g. using Ext.TaskManager, or if the field is within\na <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>, you can use the FormPanel's <a href=\"#!/api/Ext.form.Panel-cfg-pollForChanges\" rel=\"Ext.form.Panel-cfg-pollForChanges\" class=\"docClass\">Ext.form.Panel.pollForChanges</a> configuration to set up\nsuch a task automatically.</p>\n"},"childEls":{},"dirtyCls":{"!type":"string","!doc":"<p>The CSS class to use when the field value <a href=\"#!/api/Ext.form.field.Base-method-isDirty\" rel=\"Ext.form.field.Base-method-isDirty\" class=\"docClass\">is dirty</a>.</p>\n"},"fieldBodyCls":{"!type":"string","!doc":"<p>An extra CSS class to be applied to the body content element in addition to <a href=\"#!/api/Ext.form.field.Base-cfg-baseBodyCls\" rel=\"Ext.form.field.Base-cfg-baseBodyCls\" class=\"docClass\">baseBodyCls</a>.</p>\n"},"fieldCls":{"!type":"string","!doc":"<p>The default CSS class for the field input</p>\n"},"fieldStyle":{"!type":"string","!doc":"<p>Optional CSS style(s) to be applied to the <a href=\"#!/api/Ext.form.field.Base-property-inputEl\" rel=\"Ext.form.field.Base-property-inputEl\" class=\"docClass\">field input element</a>. Should be a valid argument to\nExt.dom.Element.applyStyles. Defaults to undefined. See also the <a href=\"#!/api/Ext.form.field.Base-method-setFieldStyle\" rel=\"Ext.form.field.Base-method-setFieldStyle\" class=\"docClass\">setFieldStyle</a> method for changing\nthe style after initialization.</p>\n"},"fieldSubTpl":{"!doc":"<p>The content of the field body is defined by this config option.</p>\n"},"focusCls":{"!type":"string","!doc":"<p>The CSS class to use when the field receives focus</p>\n"},"inputAttrTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\ninside the input element (as attributes). If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">subTpl data</a> serves as the context.</p>\n"},"inputId":{"!type":"string","!doc":"<p>The id that will be given to the generated input DOM element. Defaults to an automatically generated id. If you\nconfigure this manually, you must make sure it is unique in the document.</p>\n"},"inputType":{"!type":"string","!doc":"<p>The type attribute for input fields -- e.g. radio, text, password, file. The extended types\nsupported by HTML5 inputs (url, email, etc.) may also be used, though using them will cause older browsers to\nfall back to 'text'.</p>\n\n<p>The type 'password' must be used to render that field type currently -- there is no separate Ext component for\nthat. You can use <a href=\"#!/api/Ext.form.field.File\" rel=\"Ext.form.field.File\" class=\"docClass\">Ext.form.field.File</a> which creates a custom-rendered file upload field, but if you want\na plain unstyled file input you can use a Base with inputType:'file'.</p>\n"},"invalidText":{"!type":"string","!doc":"<p>The error text to use when marking a field invalid and no message is provided</p>\n"},"name":{"!type":"string","!doc":"<p>The name of the field. This is used as the parameter name when including the field value\nin a <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">form submit()</a>. If no name is configured, it falls back to the <a href=\"#!/api/Ext.form.field.Base-cfg-inputId\" rel=\"Ext.form.field.Base-cfg-inputId\" class=\"docClass\">inputId</a>.\nTo prevent the field from being included in the form submit, set <a href=\"#!/api/Ext.form.field.Base-cfg-submitValue\" rel=\"Ext.form.field.Base-cfg-submitValue\" class=\"docClass\">submitValue</a> to false.</p>\n"},"readOnly":{"!type":"bool","!doc":"<p>true to mark the field as readOnly in HTML.</p>\n"},"readOnlyCls":{"!type":"string","!doc":"<p>The CSS class applied to the component's main element when it is <a href=\"#!/api/Ext.form.field.Base-cfg-readOnly\" rel=\"Ext.form.field.Base-cfg-readOnly\" class=\"docClass\">readOnly</a>.</p>\n"},"shrinkWrap":{"!doc":"<p>If this property is a number, it is interpreted as follows:</p>\n\n<ul>\n<li>0 (or <code>false</code>): Neither width nor height depend on content.</li>\n<li>1: Width depends on content (shrink wraps), but height does not.</li>\n<li>2 (default): Height depends on content (shrink wraps), but width does not.</li>\n<li>3 (or <code>true</code>): Both width and height depend on content (shrink wrap).</li>\n</ul>\n\n\n<p>In CSS terms, shrink-wrap width is analogous to an inline-block element as opposed\nto a block-level element. Some container layouts always shrink-wrap their children,\neffectively ignoring this property (e.g., <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">Ext.layout.container.HBox</a>,\n<a href=\"#!/api/Ext.layout.container.VBox\" rel=\"Ext.layout.container.VBox\" class=\"docClass\">Ext.layout.container.VBox</a>, <a href=\"#!/api/Ext.layout.component.Dock\" rel=\"Ext.layout.component.Dock\" class=\"docClass\">Ext.layout.component.Dock</a>).</p>\n"},"tabIndex":{"!type":"number","!doc":"<p>The tabIndex for this field. Note this only applies to fields that are rendered, not those which are built via\napplyTo</p>\n"},"validateOnBlur":{"!type":"bool","!doc":"<p>Whether the field should validate when it loses focus. This will cause fields to be validated\nas the user steps through the fields in the form regardless of whether they are making changes to those fields\nalong the way. See also <a href=\"#!/api/Ext.form.field.Base-cfg-validateOnChange\" rel=\"Ext.form.field.Base-cfg-validateOnChange\" class=\"docClass\">validateOnChange</a>.</p>\n"},"defaultBindProperty":{"!type":"string"},"hasFocus":{"!type":"bool","!doc":"<p>private</p>\n"},"ignoreChangeRe":{"!doc":"<p>While input is supported in IE9, we use attachEvent for events, so we need to fall back here</p>\n"},"inputEl":{"!doc":"<p>The input Element for this Field. Only available after the field has been rendered.</p>\n"},"liquidLayout":{"!type":"bool","!doc":"<p>Components that achieve their internal layout results using solely CSS with no JS\nintervention must set this to true.  Failure to set this property to true may result\nin setSize failing to work if the component opted out of the layout run.</p>\n"},"maskOnDisable":{"!type":"bool","!doc":"<p>This is an internal flag that you use when creating custom components. By default this is set to <code>true</code> which means\nthat every component gets a mask when it's disabled. Components like FieldContainer, FieldSet, Field, Button, Tab\noverride this property to <code>false</code> since they want to implement custom disable logic.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"stretchInputElFixed":{"!type":"bool","!doc":"<p>Instructs the layout to stretch the inputEl to 100% width when laying\nout under fixed conditions. Defaults to true for all fields except check/radio\nDoesn't seem worth it to introduce a whole new layout class just for this flag</p>\n"},"subTplInsertions":{},"bindPropertyChange":{"!type":"fn(active: ?) -> !this"},"clearInvalid":{"!type":"fn() -> !this","!doc":"<p>Clear any invalid styles/messages for this field.</p>\n\n<p><strong>Note</strong>: this method does not cause the Field's <a href=\"#!/api/Ext.form.field.Base-method-validate\" rel=\"Ext.form.field.Base-method-validate\" class=\"docClass\">validate</a> or <a href=\"#!/api/Ext.form.field.Base-method-isValid\" rel=\"Ext.form.field.Base-method-isValid\" class=\"docClass\">isValid</a> methods to return <code>true</code>\nif the value does not <em>pass</em> validation. So simply clearing a field's errors will not necessarily allow\nsubmission of forms submitted with the <a href=\"#!/api/Ext.form.action.Submit-cfg-clientValidation\" rel=\"Ext.form.action.Submit-cfg-clientValidation\" class=\"docClass\">Ext.form.action.Submit.clientValidation</a> option set.</p>\n"},"doComponentLayout":{"!type":"fn() -> ?","!doc":"<p>This method needs to be called whenever you change something on this component that\nrequires the Component's layout to be recalculated.</p>\n"},"fireKey":{"!type":"fn(e: ?) -> !this","!doc":"<p>private</p>\n"},"getFieldStyle":{"!type":"fn() -> !this"},"getInputId":{"!type":"fn() -> !this","!doc":"<p>Returns the input id for this field. If none was specified via the <a href=\"#!/api/Ext.form.field.Base-cfg-inputId\" rel=\"Ext.form.field.Base-cfg-inputId\" class=\"docClass\">inputId</a> config, then an id will be\nautomatically generated.</p>\n"},"getRawValue":{"!type":"fn() -> string","!doc":"<p>Returns the raw value of the field, without performing any normalization, conversion, or validation. To get a\nnormalized and converted value see <a href=\"#!/api/Ext.form.field.Base-method-getValue\" rel=\"Ext.form.field.Base-method-getValue\" class=\"docClass\">getValue</a>.</p>\n"},"getSubTplData":{"!type":"fn() -> ?","!doc":"<p>Creates and returns the data object to be used when rendering the <a href=\"#!/api/Ext.form.field.Base-cfg-fieldSubTpl\" rel=\"Ext.form.field.Base-cfg-fieldSubTpl\" class=\"docClass\">fieldSubTpl</a>.</p>\n"},"getSubTplMarkup":{"!type":"fn() -> !this","!doc":"<p>Gets the markup to be inserted into the outer template's bodyEl. For fields this is the actual input element.</p>\n"},"getSubmitData":{"!type":"fn() -> ?","!doc":"<p>private override to use getSubmitValue() as a convenience</p>\n"},"getSubmitValue":{"!type":"fn() -> string","!doc":"<p>Returns the value that would be included in a standard form submit for this field. This will be combined with the\nfield's name to form a name=value pair in the <a href=\"#!/api/Ext.form.field.Base-method-getSubmitData\" rel=\"Ext.form.field.Base-method-getSubmitData\" class=\"docClass\">submitted parameters</a>. If an empty string is\nreturned then just the name= will be submitted; if null is returned then nothing will be submitted.</p>\n\n<p>Note that the value returned will have been <a href=\"#!/api/Ext.form.field.Base-method-processRawValue\" rel=\"Ext.form.field.Base-method-processRawValue\" class=\"docClass\">processed</a> but may or may not have been\nsuccessfully <a href=\"#!/api/Ext.form.field.Base-method-validate\" rel=\"Ext.form.field.Base-method-validate\" class=\"docClass\">validated</a>.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Returns the current data value of the field. The type of value returned is particular to the type of the\nparticular field (e.g. a Date object for <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>), as the result of calling <a href=\"#!/api/Ext.form.field.Base-method-rawToValue\" rel=\"Ext.form.field.Base-method-rawToValue\" class=\"docClass\">rawToValue</a> on\nthe field's <a href=\"#!/api/Ext.form.field.Base-method-processRawValue\" rel=\"Ext.form.field.Base-method-processRawValue\" class=\"docClass\">processed</a> String value. To return the raw String value, see <a href=\"#!/api/Ext.form.field.Base-method-getRawValue\" rel=\"Ext.form.field.Base-method-getRawValue\" class=\"docClass\">getRawValue</a>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"isFileUpload":{"!type":"fn() -> bool","!doc":"<p>Returns whether this Field is a file upload field; if it returns true, forms will use special techniques for\n<a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submitting the form</a> via AJAX. See <a href=\"#!/api/Ext.form.Basic-method-hasUpload\" rel=\"Ext.form.Basic-method-hasUpload\" class=\"docClass\">Ext.form.Basic.hasUpload</a> for details. If\nthis returns true, the <a href=\"#!/api/Ext.form.field.Base-method-extractFileInput\" rel=\"Ext.form.field.Base-method-extractFileInput\" class=\"docClass\">extractFileInput</a> method must also be implemented to return the corresponding file\ninput element.</p>\n"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Returns whether or not the field value is currently valid by <a href=\"#!/api/Ext.form.field.Base-method-getErrors\" rel=\"Ext.form.field.Base-method-getErrors\" class=\"docClass\">validating</a> the\n<a href=\"#!/api/Ext.form.field.Base-method-processRawValue\" rel=\"Ext.form.field.Base-method-processRawValue\" class=\"docClass\">processed raw value</a> of the field. <strong>Note</strong>: <a href=\"#!/api/Ext.form.field.Base-cfg-disabled\" rel=\"Ext.form.field.Base-cfg-disabled\" class=\"docClass\">disabled</a> fields are\nalways treated as valid.</p>\n"},"markInvalid":{"!type":"fn(errors: ?) -> !this","!doc":"<p>Display one or more error messages associated with this field, using <a href=\"#!/api/Ext.form.field.Base-cfg-msgTarget\" rel=\"Ext.form.field.Base-cfg-msgTarget\" class=\"docClass\">msgTarget</a> to determine how to\ndisplay the messages and applying <a href=\"#!/api/Ext.form.field.Base-cfg-invalidCls\" rel=\"Ext.form.field.Base-cfg-invalidCls\" class=\"docClass\">invalidCls</a> to the field's UI element.</p>\n\n<p><strong>Note</strong>: this method does not cause the Field's <a href=\"#!/api/Ext.form.field.Base-method-validate\" rel=\"Ext.form.field.Base-method-validate\" class=\"docClass\">validate</a> or <a href=\"#!/api/Ext.form.field.Base-method-isValid\" rel=\"Ext.form.field.Base-method-isValid\" class=\"docClass\">isValid</a> methods to return <code>false</code>\nif the value does <em>pass</em> validation. So simply marking a Field as invalid will not prevent submission of forms\nsubmitted with the <a href=\"#!/api/Ext.form.action.Submit-cfg-clientValidation\" rel=\"Ext.form.action.Submit-cfg-clientValidation\" class=\"docClass\">Ext.form.action.Submit.clientValidation</a> option set.</p>\n"},"onBoxReady":{"!type":"fn() -> !this","!doc":"<p>Invoked when this component has first achieved size. Occurs after the\n<a href=\"#!/api/Ext.form.field.Base-cfg-componentLayout\" rel=\"Ext.form.field.Base-cfg-componentLayout\" class=\"docClass\">componentLayout</a> has completed its initial run.</p>\n"},"onDirtyChange":{"!type":"fn(isDirty: ?) -> !this","!doc":"<p>Called when the field's dirty state changes. Adds/removes the <a href=\"#!/api/Ext.form.field.Base-cfg-dirtyCls\" rel=\"Ext.form.field.Base-cfg-dirtyCls\" class=\"docClass\">dirtyCls</a> on the main element.</p>\n"},"onDisable":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onEnable":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"processRawValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Performs any necessary manipulation of a raw field value to prepare it for <a href=\"#!/api/Ext.form.field.Base-method-rawToValue\" rel=\"Ext.form.field.Base-method-rawToValue\" class=\"docClass\">conversion</a> and/or\n<a href=\"#!/api/Ext.form.field.Base-method-validate\" rel=\"Ext.form.field.Base-method-validate\" class=\"docClass\">validation</a>, for instance stripping out ignored characters. In the base implementation it does\nnothing; individual subclasses may override this as needed.</p>\n"},"rawToValue":{"!type":"fn(rawValue: ?) -> ?","!doc":"<p>Converts a raw input field value into a mixed-type value that is suitable for this particular field type. This\nallows controlling the normalization and conversion of user-entered values into field-type-appropriate values,\ne.g. a Date object for <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, and is invoked by <a href=\"#!/api/Ext.form.field.Base-method-getValue\" rel=\"Ext.form.field.Base-method-getValue\" class=\"docClass\">getValue</a>.</p>\n\n<p>It is up to individual implementations to decide how to handle raw values that cannot be successfully converted\nto the desired object type.</p>\n\n<p>See <a href=\"#!/api/Ext.form.field.Base-method-valueToRaw\" rel=\"Ext.form.field.Base-method-valueToRaw\" class=\"docClass\">valueToRaw</a> for the opposite conversion.</p>\n\n<p>The base implementation does no conversion, returning the raw value untouched.</p>\n"},"renderActiveError":{"!type":"fn() -> !this","!doc":"<p>Overrides the method from the <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">Ext.form.Labelable</a> mixin to also add the invalidCls to the inputEl,\nas that is required for proper styling in IE with nested fields (due to lack of child selector)</p>\n"},"setError":{"!type":"fn(error: ?) -> !this","!doc":"<p>Set the current error state</p>\n"},"setFieldStyle":{"!type":"fn(style: ?) -> !this","!doc":"<p>Set the <a href=\"#!/api/Ext.form.field.Base-cfg-fieldStyle\" rel=\"Ext.form.field.Base-cfg-fieldStyle\" class=\"docClass\">CSS style</a> of the <a href=\"#!/api/Ext.form.field.Base-property-inputEl\" rel=\"Ext.form.field.Base-property-inputEl\" class=\"docClass\">field input element</a>.</p>\n"},"setRawValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Sets the field's raw value directly, bypassing <a href=\"#!/api/Ext.form.field.Base-method-valueToRaw\" rel=\"Ext.form.field.Base-method-valueToRaw\" class=\"docClass\">value conversion</a>, change detection, and\nvalidation. To set the value with these additional inspections see <a href=\"#!/api/Ext.form.field.Base-method-setValue\" rel=\"Ext.form.field.Base-method-setValue\" class=\"docClass\">setValue</a>.</p>\n"},"setReadOnly":{"!type":"fn(readOnly: ?) -> !this","!doc":"<p>Sets the read only state of this field.</p>\n"},"setValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Sets a data value into the field and runs the change detection and validation. To set the value directly\nwithout these inspections see <a href=\"#!/api/Ext.form.field.Base-method-setRawValue\" rel=\"Ext.form.field.Base-method-setRawValue\" class=\"docClass\">setRawValue</a>.</p>\n"},"transformRawValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Transform the raw value before it is set</p>\n"},"validateValue":{"!type":"fn(value: ?) -> bool","!doc":"<p>Uses <a href=\"#!/api/Ext.form.field.Base-method-getErrors\" rel=\"Ext.form.field.Base-method-getErrors\" class=\"docClass\">getErrors</a> to build an array of validation errors. If any errors are found, they are passed to\n<a href=\"#!/api/Ext.form.field.Base-method-markInvalid\" rel=\"Ext.form.field.Base-method-markInvalid\" class=\"docClass\">markInvalid</a> and false is returned, otherwise true is returned.</p>\n\n<p>Previously, subclasses were invited to provide an implementation of this to process validations - from 3.2\nonwards <a href=\"#!/api/Ext.form.field.Base-method-getErrors\" rel=\"Ext.form.field.Base-method-getErrors\" class=\"docClass\">getErrors</a> should be overridden instead.</p>\n"},"valueToRaw":{"!type":"fn(value: ?) -> ?","!doc":"<p>Converts a mixed-type value to a raw representation suitable for displaying in the field. This allows controlling\nhow value objects passed to <a href=\"#!/api/Ext.form.field.Base-method-setValue\" rel=\"Ext.form.field.Base-method-setValue\" class=\"docClass\">setValue</a> are shown to the user, including localization. For instance, for a\n<a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, this would control how a Date object passed to <a href=\"#!/api/Ext.form.field.Base-method-setValue\" rel=\"Ext.form.field.Base-method-setValue\" class=\"docClass\">setValue</a> would be converted\nto a String for display in the field.</p>\n\n<p>See <a href=\"#!/api/Ext.form.field.Base-method-rawToValue\" rel=\"Ext.form.field.Base-method-rawToValue\" class=\"docClass\">rawToValue</a> for the opposite conversion.</p>\n\n<p>The base implementation simply does a standard toString conversion, and converts empty values\nto an empty string.</p>\n"},"destroyTip":{"!type":"fn() -> !this","!doc":"<p>Destroy the error tip instance.</p>\n"},"initTip":{"!type":"fn() -> !this","!doc":"<p>Use a custom QuickTip instance separate from the main QuickTips singleton, so that we\ncan give it a custom frame style. Responds to errorqtip rather than the qtip property.</p>\n"},"specialkey":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when any key related to navigation (arrows, tab, enter, esc, etc.) is pressed. To handle other keys\nsee <a href=\"#!/api/Ext.util.KeyMap\" rel=\"Ext.util.KeyMap\" class=\"docClass\">Ext.util.KeyMap</a>. You can check Ext.event.Event.getKey to determine which key was\npressed. For example:</p>\n\n<pre><code>var form = new <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>({\n    ...\n    items: [{\n            fieldLabel: 'Field 1',\n            name: 'field1',\n            allowBlank: false\n        },{\n            fieldLabel: 'Field 2',\n            name: 'field2',\n            listeners: {\n                specialkey: function(field, e){\n                    // e.HOME, e.END, e.PAGE_UP, e.PAGE_DOWN,\n                    // e.TAB, e.ESC, arrow keys: e.LEFT, e.RIGHT, e.UP, e.DOWN\n                    if (e.getKey() == e.ENTER) {\n                        var form = field.up('form').getForm();\n                        form.submit();\n                    }\n                }\n            }\n        }\n    ],\n    ...\n});\n</code></pre>\n"},"writeablechange":{"!type":"fn(this: ?, Read: ?, eOpts: ?)","!doc":"<p>Fires when this field changes its read-only status.</p>\n"}}},"Checkbox":{"!doc":"<p>Single checkbox field. Can be used as a direct replacement for traditional checkbox fields. Also serves as a\nparent class for <a href=\"#!/api/Ext.form.field.Radio\" rel=\"Ext.form.field.Radio\" class=\"docClass\">radio buttons</a>.</p>\n\n<h1>Labeling</h1>\n\n<p>In addition to the <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">standard field labeling options</a>, checkboxes\nmay be given an optional <a href=\"#!/api/Ext.form.field.Checkbox-cfg-boxLabel\" rel=\"Ext.form.field.Checkbox-cfg-boxLabel\" class=\"docClass\">boxLabel</a> which will be displayed immediately after checkbox. Also see\n<a href=\"#!/api/Ext.form.CheckboxGroup\" rel=\"Ext.form.CheckboxGroup\" class=\"docClass\">Ext.form.CheckboxGroup</a> for a convenient method of grouping related checkboxes.</p>\n\n<h1>Values</h1>\n\n<p>The main value of a checkbox is a boolean, indicating whether or not the checkbox is checked.\nThe following values will check the checkbox:</p>\n\n<ul>\n<li><code>true</code></li>\n<li><code>'true'</code></li>\n<li><code>'1'</code></li>\n<li><code>'on'</code></li>\n</ul>\n\n\n<p>Any other value will uncheck the checkbox.</p>\n\n<p>In addition to the main boolean value, you may also specify a separate <a href=\"#!/api/Ext.form.field.Checkbox-cfg-inputValue\" rel=\"Ext.form.field.Checkbox-cfg-inputValue\" class=\"docClass\">inputValue</a>. This will be\nsent as the parameter value when the form is <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submitted</a>. You will want to set\nthis value if you have multiple checkboxes with the same <a href=\"#!/api/Ext.form.field.Checkbox-cfg-name\" rel=\"Ext.form.field.Checkbox-cfg-name\" class=\"docClass\">name</a>. If not specified, the value <code>on</code>\nwill be used.</p>\n\n<h1>Example usage</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    bodyPadding: 10,\n    width: 300,\n    title: 'Pizza Order',\n    items: [\n        {\n            xtype: 'fieldcontainer',\n            fieldLabel: 'Toppings',\n            defaultType: 'checkboxfield',\n            items: [\n                {\n                    boxLabel  : 'Anchovies',\n                    name      : 'topping',\n                    inputValue: '1',\n                    id        : 'checkbox1'\n                }, {\n                    boxLabel  : 'Artichoke Hearts',\n                    name      : 'topping',\n                    inputValue: '2',\n                    checked   : true,\n                    id        : 'checkbox2'\n                }, {\n                    boxLabel  : 'Bacon',\n                    name      : 'topping',\n                    inputValue: '3',\n                    id        : 'checkbox3'\n                }\n            ]\n        }\n    ],\n    bbar: [\n        {\n            text: 'Select Bacon',\n            handler: function() {\n                Ext.getCmp('checkbox3').setValue(true);\n            }\n        },\n        '-',\n        {\n            text: 'Select All',\n            handler: function() {\n                Ext.getCmp('checkbox1').setValue(true);\n                Ext.getCmp('checkbox2').setValue(true);\n                Ext.getCmp('checkbox3').setValue(true);\n            }\n        },\n        {\n            text: 'Deselect All',\n            handler: function() {\n                Ext.getCmp('checkbox1').setValue(false);\n                Ext.getCmp('checkbox2').setValue(false);\n                Ext.getCmp('checkbox3').setValue(false);\n            }\n        }\n    ],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"afterBoxLabelTextTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nafter the box label text. If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">subTpl data</a> serves as the context.</p>\n"},"afterBoxLabelTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nafter the box label element. If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">subTpl data</a> serves as the context.</p>\n"},"beforeBoxLabelTextTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nbefore the box label text. If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">subTpl data</a> serves as the context.</p>\n"},"beforeBoxLabelTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nbefore the box label element. If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">subTpl data</a> serves as the context.</p>\n"},"boxLabel":{"!type":"string","!doc":"<p>An optional text label that will appear next to the checkbox. Whether it appears before or after the checkbox is\ndetermined by the <a href=\"#!/api/Ext.form.field.Checkbox-cfg-boxLabelAlign\" rel=\"Ext.form.field.Checkbox-cfg-boxLabelAlign\" class=\"docClass\">boxLabelAlign</a> config.</p>\n"},"boxLabelAlign":{"!type":"string","!doc":"<p>The position relative to the checkbox where the <a href=\"#!/api/Ext.form.field.Checkbox-cfg-boxLabel\" rel=\"Ext.form.field.Checkbox-cfg-boxLabel\" class=\"docClass\">boxLabel</a> should appear. Recognized values are 'before'\nand 'after'.</p>\n"},"boxLabelAttrTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\ninside the box label element (as attributes). If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">subTpl data</a> serves as the context.</p>\n"},"boxLabelCls":{"!type":"string","!doc":"<p>The CSS class to be applied to the <a href=\"#!/api/Ext.form.field.Checkbox-cfg-boxLabel\" rel=\"Ext.form.field.Checkbox-cfg-boxLabel\" class=\"docClass\">boxLabel</a> element</p>\n"},"checkChangeEvents":{"!doc":"<p>private overrides</p>\n"},"checked":{"!type":"bool","!doc":"<p>true if the checkbox should render initially checked</p>\n"},"checkedCls":{"!type":"string","!doc":"<p>The CSS class(es) added to the component's main element when it is in the checked state.\nYou can add your own class (checkedCls='myClass x-form-cb-checked') or replace the default\nclass altogether (checkedCls='myClass').</p>\n"},"childEls":{},"fieldBodyCls":{"!type":"string","!doc":"<p>private</p>\n"},"fieldCls":{"!type":"string","!doc":"<p>The default CSS class for the checkbox</p>\n"},"fieldSubTpl":{"!doc":"<p>note: {id} here is really {inputId}, but {cmpId} is available</p>\n"},"focusCls":{"!type":"string","!doc":"<p>The CSS class to use when the checkbox receives focus</p>\n"},"handler":{"!doc":"<p>A function called when the <a href=\"#!/api/Ext.form.field.Checkbox-cfg-checked\" rel=\"Ext.form.field.Checkbox-cfg-checked\" class=\"docClass\">checked</a> value changes (can be used instead of handling the <a href=\"#!/api/Ext.form.field.Checkbox-event-change\" rel=\"Ext.form.field.Checkbox-event-change\" class=\"docClass\">change event</a>).</p>\n"},"inputType":{"!type":"string","!doc":"<p>The type attribute for input fields -- e.g. radio, text, password, file. The extended types\nsupported by HTML5 inputs (url, email, etc.) may also be used, though using them will cause older browsers to\nfall back to 'text'.</p>\n\n<p>The type 'password' must be used to render that field type currently -- there is no separate Ext component for\nthat. You can use <a href=\"#!/api/Ext.form.field.File\" rel=\"Ext.form.field.File\" class=\"docClass\">Ext.form.field.File</a> which creates a custom-rendered file upload field, but if you want\na plain unstyled file input you can use a Base with inputType:'file'.</p>\n"},"inputValue":{"!type":"string","!doc":"<p>The value that should go into the generated input element's value attribute and should be used as the parameter\nvalue when submitting as part of a form.</p>\n"},"scope":{"!doc":"<p>An object to use as the scope ('this' reference) of the <a href=\"#!/api/Ext.form.field.Checkbox-cfg-handler\" rel=\"Ext.form.field.Checkbox-cfg-handler\" class=\"docClass\">handler</a> function.</p>\n\n<p>Defaults to this Checkbox.</p>\n"},"uncheckedValue":{"!type":"string","!doc":"<p>If configured, this will be submitted as the checkbox's value during form submit if the checkbox is unchecked. By\ndefault this is undefined, which results in nothing being submitted for the checkbox field when the form is\nsubmitted (the default behavior of HTML checkboxes).</p>\n"},"afterLabelCls":{"!type":"string"},"ariaRole":{"!type":"string"},"boxLabelEl":{"!doc":"<p>A reference to the label element created for the <a href=\"#!/api/Ext.form.field.Checkbox-cfg-boxLabel\" rel=\"Ext.form.field.Checkbox-cfg-boxLabel\" class=\"docClass\">boxLabel</a>. Only present if the component has been\nrendered and has a boxLabel configured.</p>\n"},"inputCls":{"!type":"string","!doc":"<p>the form-cb css class is for styling shared between checkbox and subclasses (radio)</p>\n"},"isCheckbox":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Checkbox, or subclass thereof.</p>\n"},"noBoxLabelCls":{"!type":"string","!doc":"<p>This is to work around lack of min-width support in older IE browsers.\nIf it's determined that there is no box label, apply the following class to the\nwrapper around the inputEl and all browsers will get width from its theme's CSS rule.\nSee EXTJSIV-10302 and EXTJSIV-10977.</p>\n"},"onRe":{"!doc":"<p>private</p>\n"},"originalValue":{"!doc":"<p>The original value of the field as configured in the <a href=\"#!/api/Ext.form.field.Checkbox-cfg-checked\" rel=\"Ext.form.field.Checkbox-cfg-checked\" class=\"docClass\">checked</a> configuration, or as loaded by the last\nform load operation if the form's <a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a> setting is <code>true</code>.</p>\n"},"publishes":{},"stretchInputElFixed":{"!type":"bool","!doc":"<p>inputEl should always retain the same size, never stretch</p>\n"},"subTplInsertions":{},"valueToRaw":{"!doc":"<p>private</p>\n"},"wrapInnerCls":{"!type":"string"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>inherit docs</p>\n"},"getElConfig":{"!type":"fn() -> !this"},"getFormId":{"!type":"fn() -> !this"},"getManager":{"!type":"fn() -> !this","!doc":"<p>inherit docs</p>\n"},"getRawValue":{"!type":"fn() -> bool","!doc":"<p>Returns the checked state of the checkbox.</p>\n"},"getSubTplData":{"!type":"fn() -> ?","!doc":"<p>Creates and returns the data object to be used when rendering the <a href=\"#!/api/Ext.form.field.Checkbox-cfg-fieldSubTpl\" rel=\"Ext.form.field.Checkbox-cfg-fieldSubTpl\" class=\"docClass\">fieldSubTpl</a>.</p>\n"},"getSubmitValue":{"!type":"fn() -> string","!doc":"<p>Returns the submit value for the checkbox which can be used when submitting forms.</p>\n"},"getValue":{"!type":"fn() -> bool","!doc":"<p>Returns the checked state of the checkbox.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"initValue":{"!type":"fn() -> !this","!doc":"<p>Initializes the field's value based on the initial config.</p>\n"},"isChecked":{"!type":"fn(rawValue: ?, inputValue: ?) -> !this"},"onBoxClick":{"!type":"fn() -> !this","!doc":"<p>Handle click on the checkbox button</p>\n"},"onChange":{"!type":"fn(newVal: ?, oldVal: ?) -> !this","!doc":"<p>Called when the checkbox's checked state changes. Invokes the <a href=\"#!/api/Ext.form.field.Checkbox-cfg-handler\" rel=\"Ext.form.field.Checkbox-cfg-handler\" class=\"docClass\">handler</a> callback\nfunction if specified.</p>\n"},"onEnable":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"resetOriginalValue":{"!type":"fn(fromBoxInGroup: ?) -> !this","!doc":"<p>Resets the field's <a href=\"#!/api/Ext.form.field.Checkbox-property-originalValue\" rel=\"Ext.form.field.Checkbox-property-originalValue\" class=\"docClass\">originalValue</a> property so it matches the current <a href=\"#!/api/Ext.form.field.Checkbox-method-getValue\" rel=\"Ext.form.field.Checkbox-method-getValue\" class=\"docClass\">value</a>. This is\ncalled by <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a>.<a href=\"#!/api/Ext.form.Basic-method-setValues\" rel=\"Ext.form.Basic-method-setValues\" class=\"docClass\">setValues</a> if the form's\n<a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a> property is set to true.</p>\n"},"setBoxLabel":{"!type":"fn(boxLabel: ?) -> !this","!doc":"<p>Sets the <a href=\"#!/api/Ext.form.field.Checkbox-cfg-boxLabel\" rel=\"Ext.form.field.Checkbox-cfg-boxLabel\" class=\"docClass\">boxLabel</a> for this checkbox.</p>\n"},"setRawValue":{"!type":"fn(value: ?) -> bool","!doc":"<p>Sets the checked state of the checkbox.</p>\n"},"setReadOnly":{"!type":"fn(readOnly: ?) -> !this","!doc":"<p>Sets the read only state of this field.</p>\n"},"setValue":{"!type":"fn(checked: ?) -> ?","!doc":"<p>Sets the checked state of the checkbox, and invokes change detection.</p>\n"}}},"ComboBox":{"!doc":"<p>A combobox control with support for autocomplete, remote loading, and many other features.</p>\n\n<p>A ComboBox is like a combination of a traditional HTML text <code>&lt;input&gt;</code> field and a <code>&lt;select&gt;</code>\nfield; the user is able to type freely into the field, and/or pick values from a dropdown selection\nlist. The user can input any value by default, even if it does not appear in the selection list;\nto prevent free-form values and restrict them to items in the list, set <a href=\"#!/api/Ext.form.field.ComboBox-cfg-forceSelection\" rel=\"Ext.form.field.ComboBox-cfg-forceSelection\" class=\"docClass\">forceSelection</a> to <code>true</code>.</p>\n\n<p>The selection list's options are populated from any Ext.data.Store, including remote\nstores. The data items in the store are mapped to each option's displayed text and backing value via\nthe <a href=\"#!/api/Ext.form.field.ComboBox-cfg-valueField\" rel=\"Ext.form.field.ComboBox-cfg-valueField\" class=\"docClass\">valueField</a> and <a href=\"#!/api/Ext.form.field.ComboBox-cfg-displayField\" rel=\"Ext.form.field.ComboBox-cfg-displayField\" class=\"docClass\">displayField</a> configurations, respectively.</p>\n\n<p>If your store is not remote, i.e. it depends only on local data and is loaded up front, you should be\nsure to set the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a> to <code>'local'</code>, as this will improve responsiveness for the user.</p>\n\n<h1>Example usage:</h1>\n\n<pre class='inline-example '><code>// The data store containing the list of states\nvar states = Ext.create('Ext.data.Store', {\n    fields: ['abbr', 'name'],\n    data : [\n        {\"abbr\":\"AL\", \"name\":\"Alabama\"},\n        {\"abbr\":\"AK\", \"name\":\"Alaska\"},\n        {\"abbr\":\"AZ\", \"name\":\"Arizona\"}\n        //...\n    ]\n});\n\n// Create the combo box, attached to the states data store\nExt.create('<a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.ComboBox</a>', {\n    fieldLabel: 'Choose State',\n    store: states,\n    queryMode: 'local',\n    displayField: 'name',\n    valueField: 'abbr',\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<h1>Events</h1>\n\n<p>To do something when something in ComboBox is selected, configure the select event:</p>\n\n<pre><code>var cb = Ext.create('<a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.ComboBox</a>', {\n    // all of your config options\n    listeners:{\n         scope: yourScope,\n         'select': yourFunction\n    }\n});\n\n// Alternatively, you can assign events after the object is created:\nvar cb = new <a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.field.ComboBox</a>(yourOptions);\ncb.on('select', yourFunction, yourScope);\n</code></pre>\n\n<h1>Multiple Selection</h1>\n\n<p>ComboBox also allows selection of multiple items from the list; to enable multi-selection set the\n<a href=\"#!/api/Ext.form.field.ComboBox-cfg-multiSelect\" rel=\"Ext.form.field.ComboBox-cfg-multiSelect\" class=\"docClass\">multiSelect</a> config to <code>true</code>.</p>\n\n<h1>Filtered Stores</h1>\n\n<p>If you have a local store that is already filtered, you can use the <a href=\"#!/api/Ext.form.field.ComboBox-property-lastQuery\" rel=\"Ext.form.field.ComboBox-property-lastQuery\" class=\"docClass\">lastQuery</a> config option\nto prevent the store from having the filter being cleared on first expand.</p>\n\n<h2>Customized combobox</h2>\n\n<p>Both the text shown in dropdown menu and text field can be easily customized:</p>\n\n<pre class='inline-example '><code>var states = Ext.create('Ext.data.Store', {\n    fields: ['abbr', 'name'],\n    data : [\n        {\"abbr\":\"AL\", \"name\":\"Alabama\"},\n        {\"abbr\":\"AK\", \"name\":\"Alaska\"},\n        {\"abbr\":\"AZ\", \"name\":\"Arizona\"}\n    ]\n});\n\nExt.create('<a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.ComboBox</a>', {\n    fieldLabel: 'Choose State',\n    store: states,\n    queryMode: 'local',\n    valueField: 'abbr',\n    renderTo: Ext.getBody(),\n    // Template for the dropdown menu.\n    // Note the use of \"x-boundlist-item\" class,\n    // this is required to make the items selectable.\n    tpl: Ext.create('Ext.XTemplate',\n        '&lt;tpl for=\".\"&gt;',\n            '&lt;div class=\"x-boundlist-item\"&gt;{abbr} - {name}&lt;/div&gt;',\n        '&lt;/tpl&gt;'\n    ),\n    // template for the content inside text field\n    displayTpl: Ext.create('Ext.XTemplate',\n        '&lt;tpl for=\".\"&gt;',\n            '{abbr} - {name}',\n        '&lt;/tpl&gt;'\n    )\n});\n</code></pre>\n\n<p>See also the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-listConfig\" rel=\"Ext.form.field.ComboBox-cfg-listConfig\" class=\"docClass\">listConfig</a> option for additional configuration of the dropdown.</p>\n","!type":"fn(config: ?)","prototype":{"allQuery":{"!type":"string","!doc":"<p>The text query to send to the server to return all records for the list with no filtering</p>\n"},"anyMatch":{"!type":"bool","!doc":"<p>Configure as <code>true</code> to allow matching of the typed characters at any position in the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-valueField\" rel=\"Ext.form.field.ComboBox-cfg-valueField\" class=\"docClass\">valueField</a>'s value.</p>\n"},"autoLoadOnValue":{"!type":"bool","!doc":"<p>This option controls whether to <em>initially</em> load the store when a value is set so that\nthe display value can be determined from the appropriate record.\nThe store will only be loaded in a limited set of circumstances:\n- The store is not currently loading.\n- The store does not have a pending Ext.data.Store.autoLoad.\n- The store has not been loaded before.</p>\n"},"autoSelect":{"!type":"bool","!doc":"<p><code>true</code> to automatically highlight the first result gathered by the data store in the dropdown list when it is\nopened. A false value would cause nothing in the list to be highlighted automatically, so\nthe user would have to manually highlight an item before pressing the enter or <a href=\"#!/api/Ext.form.field.ComboBox-cfg-selectOnTab\" rel=\"Ext.form.field.ComboBox-cfg-selectOnTab\" class=\"docClass\">tab</a> key to\nselect it (unless the value of (<a href=\"#!/api/Ext.form.field.ComboBox-cfg-typeAhead\" rel=\"Ext.form.field.ComboBox-cfg-typeAhead\" class=\"docClass\">typeAhead</a>) were true), or use the mouse to select a value.</p>\n"},"caseSensitive":{"!type":"bool","!doc":"<p>Configure as <code>true</code> to make the filtering match with exact case matching</p>\n"},"clearFilterOnBlur":{"!type":"bool","!doc":"<p><em>When <a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a> is <code>'local'</code> only</em></p>\n\n<p>As text is entered, the underlying store is filtered to match the value. When this option is <code>true</code>,\nany filtering applied by this field will be cleared when focus is removed &amp; reinstated on focus.\nIf <code>false</code>, the filters will be left in place.</p>\n"},"defaultListConfig":{"!doc":"<p>Set of options that will be used as defaults for the user-configured <a href=\"#!/api/Ext.form.field.ComboBox-cfg-listConfig\" rel=\"Ext.form.field.ComboBox-cfg-listConfig\" class=\"docClass\">listConfig</a> object.</p>\n"},"delimiter":{"!type":"string","!doc":"<p>The character(s) used to separate the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-displayField\" rel=\"Ext.form.field.ComboBox-cfg-displayField\" class=\"docClass\">display values</a> of multiple selected items when\n<code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-multiSelect\" rel=\"Ext.form.field.ComboBox-cfg-multiSelect\" class=\"docClass\">multiSelect</a> = true</code>.</p>\n"},"displayField":{"!type":"string","!doc":"<p>The underlying data field name to bind to this ComboBox.</p>\n\n<p>See also <code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-valueField\" rel=\"Ext.form.field.ComboBox-cfg-valueField\" class=\"docClass\">valueField</a></code>.</p>\n"},"displayTpl":{"!doc":"<p>The template to be used to display selected records inside the text field. An array of the selected records' data\nwill be passed to the template. Defaults to:</p>\n\n<pre><code>'&lt;tpl for=\".\"&gt;' +\n    '{[typeof values === \"string\" ? values : values[\"' + me.displayField + '\"]]}' +\n    '&lt;tpl if=\"xindex &lt; xcount\"&gt;' + me.delimiter + '&lt;/tpl&gt;' +\n'&lt;/tpl&gt;'\n</code></pre>\n"},"enableRegEx":{"!type":"bool","!doc":"<p><em>When <a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a> is <code>'local'</code> only</em></p>\n\n<p>Set to <code>true</code> to have the ComboBox use the typed value as a RegExp source to filter the store to get possible matches.</p>\n"},"filters":{},"forceSelection":{"!type":"bool","!doc":"<p><code>true</code> to restrict the selected value to one of the values in the list, <code>false</code> to allow the user to set\narbitrary text into the field.</p>\n"},"growToLongestValue":{"!type":"bool","!doc":"<p><code>false</code> to not allow the component to resize itself when its data changes\n(and its <a href=\"#!/api/Ext.form.field.ComboBox-cfg-grow\" rel=\"Ext.form.field.ComboBox-cfg-grow\" class=\"docClass\">grow</a> property is <code>true</code>)</p>\n"},"hiddenDataCls":{"!type":"string","!doc":"<p>CSS class used to find the <a href=\"#!/api/Ext.form.field.ComboBox-property-hiddenDataEl\" rel=\"Ext.form.field.ComboBox-property-hiddenDataEl\" class=\"docClass\">hiddenDataEl</a></p>\n"},"hiddenName":{"!type":"string","!doc":"<p>The name of an underlying hidden field which will be synchronized with the underlying value of the combo.\nThis option is useful if the combo is part of a form element doing a regular form post. The hidden field\nwill not be created unless a hiddenName is specified.</p>\n"},"listConfig":{"!doc":"<p>An optional set of configuration properties that will be passed to the <a href=\"#!/api/Ext.view.BoundList\" rel=\"Ext.view.BoundList\" class=\"docClass\">Ext.view.BoundList</a>'s constructor.\nAny configuration that is valid for BoundList can be included. Some of the more useful ones are:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.view.BoundList-cfg-cls\" rel=\"Ext.view.BoundList-cfg-cls\" class=\"docClass\">cls</a> - defaults to empty</li>\n<li><a href=\"#!/api/Ext.view.BoundList-cfg-emptyText\" rel=\"Ext.view.BoundList-cfg-emptyText\" class=\"docClass\">emptyText</a> - defaults to empty string</li>\n<li><a href=\"#!/api/Ext.view.BoundList-cfg-itemSelector\" rel=\"Ext.view.BoundList-cfg-itemSelector\" class=\"docClass\">itemSelector</a> - defaults to the value defined in BoundList</li>\n<li><a href=\"#!/api/Ext.view.BoundList-cfg-loadingText\" rel=\"Ext.view.BoundList-cfg-loadingText\" class=\"docClass\">loadingText</a> - defaults to <code>'Loading...'</code></li>\n<li><a href=\"#!/api/Ext.view.BoundList-cfg-minWidth\" rel=\"Ext.view.BoundList-cfg-minWidth\" class=\"docClass\">minWidth</a> - defaults to <code>70</code></li>\n<li><a href=\"#!/api/Ext.view.BoundList-cfg-maxWidth\" rel=\"Ext.view.BoundList-cfg-maxWidth\" class=\"docClass\">maxWidth</a> - defaults to <code>undefined</code></li>\n<li><a href=\"#!/api/Ext.view.BoundList-cfg-maxHeight\" rel=\"Ext.view.BoundList-cfg-maxHeight\" class=\"docClass\">maxHeight</a> - defaults to <code>300</code></li>\n<li><a href=\"#!/api/Ext.view.BoundList-cfg-resizable\" rel=\"Ext.view.BoundList-cfg-resizable\" class=\"docClass\">resizable</a> - defaults to <code>false</code></li>\n<li><a href=\"#!/api/Ext.view.BoundList-cfg-shadow\" rel=\"Ext.view.BoundList-cfg-shadow\" class=\"docClass\">shadow</a> - defaults to <code>'sides'</code></li>\n<li><p><a href=\"#!/api/Ext.view.BoundList-cfg-width\" rel=\"Ext.view.BoundList-cfg-width\" class=\"docClass\">width</a> - defaults to <code>undefined</code> (automatically set to the width of the ComboBox\nfield if <a href=\"#!/api/Ext.form.field.ComboBox-cfg-matchFieldWidth\" rel=\"Ext.form.field.ComboBox-cfg-matchFieldWidth\" class=\"docClass\">matchFieldWidth</a> is true)\n<a href=\"#!/api/Ext.view.BoundList-method-getInnerTpl\" rel=\"Ext.view.BoundList-method-getInnerTpl\" class=\"docClass\">getInnerTpl</a> A function which returns a template string which renders\nthe ComboBox's <a href=\"#!/api/Ext.form.field.ComboBox-cfg-displayField\" rel=\"Ext.form.field.ComboBox-cfg-displayField\" class=\"docClass\">displayField</a> value in the dropdown. This defaults to just outputting the raw value,\nbut may use any XTemplate methods to produce output.</p>\n\n<p>The running template is configured with some extra properties that provide some context:</p>\n\n<pre><code>- field <a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">ComboBox</a> This combobox\n- store Store This combobox's data store\n</code></pre></li>\n</ul>\n\n"},"minChars":{"!type":"number","!doc":"<p>The minimum number of characters the user must type before autocomplete and <a href=\"#!/api/Ext.form.field.ComboBox-cfg-typeAhead\" rel=\"Ext.form.field.ComboBox-cfg-typeAhead\" class=\"docClass\">typeAhead</a> activate.</p>\n\n<p>Defaults to <code>4</code> if <code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a> = 'remote'</code> or <code>0</code> if <code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a> = 'local'</code>,\ndoes not apply if <code><a href=\"#!/api/Ext.form.field.Trigger-cfg-editable\" rel=\"Ext.form.field.Trigger-cfg-editable\" class=\"docClass\">editable</a> = false</code>.</p>\n"},"multiSelect":{"!type":"bool","!doc":"<p>If set to <code>true</code>, allows the combo field to hold more than one value at a time, and allows selecting multiple\nitems from the dropdown list. The combo's text field will show all selected values separated by the\n<a href=\"#!/api/Ext.form.field.ComboBox-cfg-delimiter\" rel=\"Ext.form.field.ComboBox-cfg-delimiter\" class=\"docClass\">delimiter</a>.</p>\n"},"pageSize":{"!type":"number","!doc":"<p>If greater than <code>0</code>, a <a href=\"#!/api/Ext.toolbar.Paging\" rel=\"Ext.toolbar.Paging\" class=\"docClass\">Ext.toolbar.Paging</a> is displayed in the footer of the dropdown list and the\n<a href=\"#!/api/Ext.form.field.ComboBox-method-doQuery\" rel=\"Ext.form.field.ComboBox-method-doQuery\" class=\"docClass\">filter queries</a> will execute with page start and <a href=\"#!/api/Ext.view.BoundList-cfg-pageSize\" rel=\"Ext.view.BoundList-cfg-pageSize\" class=\"docClass\">limit</a>\nparameters. Only applies when <code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a> = 'remote'</code>.</p>\n"},"queryCaching":{"!type":"bool","!doc":"<p>When true, this prevents the combo from re-querying (either locally or remotely) when the current query\nis the same as the previous query.</p>\n"},"queryDelay":{"!type":"number","!doc":"<p>The length of time in milliseconds to delay between the start of typing and sending the query to filter the\ndropdown list.</p>\n\n<p>Defaults to <code>500</code> if <code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a> = 'remote'</code> or <code>10</code> if <code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a> = 'local'</code></p>\n"},"queryMode":{"!type":"string","!doc":"<p>The mode in which the ComboBox uses the configured Store. Acceptable values are:</p>\n\n<ul>\n<li><p><strong><code>'remote'</code></strong> :</p>\n\n<p>In <code>queryMode: 'remote'</code>, the ComboBox loads its Store dynamically based upon user interaction.</p>\n\n<p>This is typically used for \"autocomplete\" type inputs, and after the user finishes typing, the Store is loaded.</p>\n\n<p>A parameter containing the typed string is sent in the load request. The default parameter name for the input\nstring is <code>query</code>, but this can be configured using the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryParam\" rel=\"Ext.form.field.ComboBox-cfg-queryParam\" class=\"docClass\">queryParam</a> config.</p>\n\n<p>In <code>queryMode: 'remote'</code>, the Store may be configured with <code>remoteFilter:\ntrue</code>, and further filters may be <em>programatically</em> added to the Store which are then passed with every load\nrequest which allows the server to further refine the returned dataset.</p>\n\n<p>Typically, in an autocomplete situation, <a href=\"#!/api/Ext.form.field.ComboBox-cfg-hideTrigger\" rel=\"Ext.form.field.ComboBox-cfg-hideTrigger\" class=\"docClass\">hideTrigger</a> is configured <code>true</code> because it has no meaning for\nautocomplete.</p></li>\n<li><p><strong><code>'local'</code></strong> :</p>\n\n<p>ComboBox loads local data</p>\n\n<pre><code>var combo = new <a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.field.ComboBox</a>({\n    renderTo: document.body,\n    queryMode: 'local',\n    store: new Ext.data.ArrayStore({\n        id: 0,\n        fields: [\n            'myId',  // numeric value is the key\n            'displayText'\n        ],\n        data: [[1, 'item1'], [2, 'item2']]  // data is local\n    }),\n    valueField: 'myId',\n    displayField: 'displayText',\n    triggerAction: 'all'\n});\n</code></pre></li>\n</ul>\n\n"},"queryParam":{"!type":"string","!doc":"<p>Name of the parameter used by the Store to pass the typed string when the ComboBox is configured with\n<code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a>: 'remote'</code>. If explicitly set to a falsy value it will not be sent.</p>\n"},"selectOnTab":{"!type":"bool","!doc":"<p>Whether the Tab key should select the currently highlighted item.</p>\n"},"store":{"!doc":"<p>The data source to which this combo is bound. Acceptable values for this property are:</p>\n\n<ul>\n<li><strong>any Store subclass</strong></li>\n<li><strong>an ID of a store</strong></li>\n<li><p><strong>an Array</strong> : Arrays will be converted to a Ext.data.Store internally, automatically generating\nfield names to work with all data components.</p>\n\n<ul>\n<li><p><strong>1-dimensional array</strong> : (e.g., <code>['Foo','Bar']</code>)</p>\n\n<p>A 1-dimensional array will automatically be expanded (each array item will be used for both the combo\n<a href=\"#!/api/Ext.form.field.ComboBox-cfg-valueField\" rel=\"Ext.form.field.ComboBox-cfg-valueField\" class=\"docClass\">valueField</a> and <a href=\"#!/api/Ext.form.field.ComboBox-cfg-displayField\" rel=\"Ext.form.field.ComboBox-cfg-displayField\" class=\"docClass\">displayField</a>)</p></li>\n<li><p><strong>2-dimensional array</strong> : (e.g., <code>[['f','Foo'],['b','Bar']]</code>)</p>\n\n<p>For a multi-dimensional array, the value in index 0 of each item will be assumed to be the combo\n<a href=\"#!/api/Ext.form.field.ComboBox-cfg-valueField\" rel=\"Ext.form.field.ComboBox-cfg-valueField\" class=\"docClass\">valueField</a>, while the value at index 1 is assumed to be the combo <a href=\"#!/api/Ext.form.field.ComboBox-cfg-displayField\" rel=\"Ext.form.field.ComboBox-cfg-displayField\" class=\"docClass\">displayField</a>.</p></li>\n</ul>\n</li>\n</ul>\n\n\n<p>See also <a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a>.</p>\n"},"transform":{"!doc":"<p>The id, DOM node or Ext.dom.Element of an existing HTML <code>&lt;select&gt;</code> element to convert into a ComboBox. The\ntarget select's options will be used to build the options in the ComboBox dropdown; a configured <a href=\"#!/api/Ext.form.field.ComboBox-cfg-store\" rel=\"Ext.form.field.ComboBox-cfg-store\" class=\"docClass\">store</a>\nwill take precedence over this.</p>\n"},"transformInPlace":{"!type":"bool","!doc":"<p><code>true</code> to automatically render this combo box in place of the select element that is being\n<a href=\"#!/api/Ext.form.field.ComboBox-cfg-transform\" rel=\"Ext.form.field.ComboBox-cfg-transform\" class=\"docClass\">transformed</a>. If <code>false</code>, this combo will be rendered using the normal rendering,\neither as part of a layout, or using <a href=\"#!/api/Ext.form.field.ComboBox-cfg-renderTo\" rel=\"Ext.form.field.ComboBox-cfg-renderTo\" class=\"docClass\">renderTo</a> or <a href=\"#!/api/Ext.form.field.ComboBox-method-render\" rel=\"Ext.form.field.ComboBox-method-render\" class=\"docClass\">render</a>.</p>\n"},"triggerAction":{"!type":"string","!doc":"<p>The action to execute when the trigger is clicked.</p>\n\n<ul>\n<li><p><strong><code>'all'</code></strong> :</p>\n\n<p><a href=\"#!/api/Ext.form.field.ComboBox-method-doQuery\" rel=\"Ext.form.field.ComboBox-method-doQuery\" class=\"docClass\">run the query</a> specified by the <code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-allQuery\" rel=\"Ext.form.field.ComboBox-cfg-allQuery\" class=\"docClass\">allQuery</a></code> config option</p></li>\n<li><p><strong><code>'last'</code></strong> :</p>\n\n<p><a href=\"#!/api/Ext.form.field.ComboBox-method-doQuery\" rel=\"Ext.form.field.ComboBox-method-doQuery\" class=\"docClass\">run the query</a> using the <code><a href=\"#!/api/Ext.form.field.ComboBox-property-lastQuery\" rel=\"Ext.form.field.ComboBox-property-lastQuery\" class=\"docClass\">last query value</a></code>.</p></li>\n<li><p><strong><code>'query'</code></strong> :</p>\n\n<p><a href=\"#!/api/Ext.form.field.ComboBox-method-doQuery\" rel=\"Ext.form.field.ComboBox-method-doQuery\" class=\"docClass\">run the query</a> using the <a href=\"#!/api/Ext.form.field.Base-method-getRawValue\" rel=\"Ext.form.field.Base-method-getRawValue\" class=\"docClass\">raw value</a>.</p></li>\n</ul>\n\n\n<p>See also <code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryParam\" rel=\"Ext.form.field.ComboBox-cfg-queryParam\" class=\"docClass\">queryParam</a></code>.</p>\n"},"triggerCls":{"!type":"string","!doc":"<p>An additional CSS class used to style the trigger button. The trigger will always get the <a href=\"#!/api/Ext.form.trigger.Trigger-property-baseCls\" rel=\"Ext.form.trigger.Trigger-property-baseCls\" class=\"docClass\">Ext.form.trigger.Trigger.baseCls</a>\nby default and <code>triggerCls</code> will be <strong>appended</strong> if specified.</p>\n"},"typeAhead":{"!type":"bool","!doc":"<p><code>true</code> to populate and autoselect the remainder of the text being typed after a configurable delay\n(<a href=\"#!/api/Ext.form.field.ComboBox-cfg-typeAheadDelay\" rel=\"Ext.form.field.ComboBox-cfg-typeAheadDelay\" class=\"docClass\">typeAheadDelay</a>) if it matches a known value.</p>\n"},"typeAheadDelay":{"!type":"number","!doc":"<p>The length of time in milliseconds to wait until the typeahead text is displayed if <code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-typeAhead\" rel=\"Ext.form.field.ComboBox-cfg-typeAhead\" class=\"docClass\">typeAhead</a> = true</code></p>\n"},"valueField":{"!type":"string","!doc":"<p>The underlying data value name to bind to this ComboBox.</p>\n\n<p><strong>Note</strong>: use of a <code>valueField</code> requires the user to make a selection in order for a value to be mapped. See also\n<code><a href=\"#!/api/Ext.form.field.ComboBox-cfg-displayField\" rel=\"Ext.form.field.ComboBox-cfg-displayField\" class=\"docClass\">displayField</a></code>.</p>\n\n<p>Defaults to match the value of the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-displayField\" rel=\"Ext.form.field.ComboBox-cfg-displayField\" class=\"docClass\">displayField</a> config.</p>\n"},"valueNotFoundText":{"!type":"string","!doc":"<p>When using a name/value combo, if the value passed to setValue is not found in the store, valueNotFoundText will\nbe displayed as the field text if defined. If this default text is used, it means there\nis no value set and no validation will occur on this field.</p>\n"},"ariaRole":{"!type":"string"},"filtered":{"!type":"bool","!doc":"<p>True if there are extra <code>filters</code> appllied to this component.</p>\n"},"hiddenDataEl":{},"ignoreSelection":{"!type":"number","!doc":"<p>private</p>\n"},"lastQuery":{"!type":"string","!doc":"<p>The value of the match string used to filter the store. Delete this property to force a requery. Example use:</p>\n\n<pre><code>var combo = new <a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.field.ComboBox</a>({\n    ...\n    queryMode: 'remote',\n    listeners: {\n        // delete the previous query in the beforequery event or set\n        // combo.lastQuery = null (this will reload the store the next time it expands)\n        beforequery: function(qe){\n            delete qe.combo.lastQuery;\n        }\n    }\n});\n</code></pre>\n\n<p>To make sure the filter in the store is not cleared the first time the ComboBox trigger is used configure the\ncombo with <code>lastQuery=''</code>. Example use:</p>\n\n<pre><code>var combo = new <a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.field.ComboBox</a>({\n    ...\n    queryMode: 'local',\n    triggerAction: 'all',\n    lastQuery: ''\n});\n</code></pre>\n"},"removingRecords":{"!doc":"<p>private, tells the layout to recalculate its startingWidth when a record is removed from its bound store</p>\n"},"afterQuery":{"!type":"fn(queryPlan: ?) -> !this","!doc":"<p>A method called when the filtering caused by the <a href=\"#!/api/Ext.form.field.ComboBox-method-doQuery\" rel=\"Ext.form.field.ComboBox-method-doQuery\" class=\"docClass\">doQuery</a> call is complete and the store has been\neither filtered locally (if <a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a> is <code>\"local\"</code>), or has been loaded using the specified filtering.</p>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"alignPicker":{"!type":"fn() -> !this","!doc":"<p>Aligns the picker to the input element</p>\n"},"applyFilters":{"!type":"fn(filters: ?, collection: ?) -> !this"},"assertValue":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"beforeBlur":{"!type":"fn() -> !this","!doc":"<p>Template method to do any pre-blur processing.</p>\n"},"beforeQuery":{"!type":"fn(queryPlan: ?) -> !this","!doc":"<p>A method which may modify aspects of how the store is to be filtered (if <a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a> is <code>\"local\"</code>)\nof loaded (if <a href=\"#!/api/Ext.form.field.ComboBox-cfg-queryMode\" rel=\"Ext.form.field.ComboBox-cfg-queryMode\" class=\"docClass\">queryMode</a> is <code>\"remote\"</code>).</p>\n\n<p>This is called by the {@link <a href=\"#!/api/Ext.form.field.ComboBox-method-doQuery\" rel=\"Ext.form.field.ComboBox-method-doQuery\" class=\"docClass\">doQuery</a> method, and may be overridden in subclasses to modify\nthe default behaviour.</p>\n\n<p>This method is passed an object containing information about the upcoming query operation which it may modify\nbefore returning.</p>\n"},"beforeReset":{"!type":"fn() -> !this","!doc":"<p>Template method before a field is reset.</p>\n"},"bindStore":{"!type":"fn(store?: ?, preventFilter?: ?) -> !this","!doc":"<p>Binds a store to this instance.</p>\n"},"clearValue":{"!type":"fn() -> !this","!doc":"<p>Clears any value currently set in the ComboBox.</p>\n"},"createPicker":{"!type":"fn() -> !this","!doc":"<p>Creates and returns the component to be used as this field's picker. Must be implemented by subclasses of Picker.\nThe current field should also be passed as a configuration option to the picker component as the pickerField\nproperty.</p>\n"},"doAutoSelect":{"!type":"fn() -> !this","!doc":"<p>If the autoSelect config is true, and the picker is open, highlights the first item.</p>\n"},"doLocalQuery":{"!type":"fn(queryPlan: ?) -> !this"},"doQuery":{"!type":"fn(queryString: ?, forceAll?: ?, rawQuery?: ?) -> bool","!doc":"<p>Executes a query to filter the dropdown list. Fires the <a href=\"#!/api/Ext.form.field.ComboBox-event-beforequery\" rel=\"Ext.form.field.ComboBox-event-beforequery\" class=\"docClass\">beforequery</a> event prior to performing the query\nallowing the query action to be canceled if needed.</p>\n"},"doRawQuery":{"!type":"fn() -> !this","!doc":"<p>Execute the query with the raw contents within the textfield.</p>\n"},"doRemoteQuery":{"!type":"fn(queryPlan: ?) -> !this"},"doTypeAhead":{"!type":"fn() -> !this"},"findRecord":{"!type":"fn(field: ?, value: ?) -> ?","!doc":"<p>Finds the record by searching for a specific field/value combination.</p>\n"},"findRecordByDisplay":{"!type":"fn(value: ?) -> ?","!doc":"<p>Finds the record by searching values in the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-displayField\" rel=\"Ext.form.field.ComboBox-cfg-displayField\" class=\"docClass\">displayField</a>.</p>\n"},"findRecordByValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Finds the record by searching values in the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-valueField\" rel=\"Ext.form.field.ComboBox-cfg-valueField\" class=\"docClass\">valueField</a>.</p>\n"},"getDisplayValue":{"!type":"fn() -> !this","!doc":"<p>Generates the string value to be displayed in the text field for the currently stored value</p>\n"},"getFilters":{"!type":"fn(autoCreate?: ?) -> ?","!doc":"<p>Returns the <code>Ext.util.FilterCollection</code>. Unless <code>autoCreate</code> is explicitly passed\nas <code>false</code> this collection will be automatically created if it does not yet exist.</p>\n"},"getParams":{"!type":"fn(queryString: ?) -> !this","!doc":"<p>private</p>\n"},"getSelectedRecord":{"!type":"fn() -> !this"},"getStore":{"!type":"fn() -> ?","!doc":"<p>Returns the store associated with this ComboBox.</p>\n"},"getStoreListeners":{"!type":"fn() -> ?","!doc":"<p>Gets the listeners to bind to a new store.</p>\n"},"getSubTplMarkup":{"!type":"fn() -> !this","!doc":"<p>Gets the markup to be inserted into the outer template's bodyEl. For fields this is the actual input element.</p>\n"},"getSubmitValue":{"!type":"fn() -> string","!doc":"<p>Returns the value that would be included in a standard form submit for this field. This will be combined with the\nfield's name to form a name=value pair in the <a href=\"#!/api/Ext.form.field.ComboBox-method-getSubmitData\" rel=\"Ext.form.field.ComboBox-method-getSubmitData\" class=\"docClass\">submitted parameters</a>. If an empty string is\nreturned then just the name= will be submitted; if null is returned then nothing will be submitted.</p>\n\n<p>Note that the value returned will have been <a href=\"#!/api/Ext.form.field.ComboBox-method-processRawValue\" rel=\"Ext.form.field.ComboBox-method-processRawValue\" class=\"docClass\">processed</a> but may or may not have been\nsuccessfully <a href=\"#!/api/Ext.form.field.ComboBox-method-validate\" rel=\"Ext.form.field.ComboBox-method-validate\" class=\"docClass\">validated</a>.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Returns the current data value of the field. The type of value returned is particular to the type of the\nparticular field (e.g. a Date object for <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>), as the result of calling <a href=\"#!/api/Ext.form.field.ComboBox-method-rawToValue\" rel=\"Ext.form.field.ComboBox-method-rawToValue\" class=\"docClass\">rawToValue</a> on\nthe field's <a href=\"#!/api/Ext.form.field.ComboBox-method-processRawValue\" rel=\"Ext.form.field.ComboBox-method-processRawValue\" class=\"docClass\">processed</a> String value. To return the raw String value, see <a href=\"#!/api/Ext.form.field.ComboBox-method-getRawValue\" rel=\"Ext.form.field.ComboBox-method-getRawValue\" class=\"docClass\">getRawValue</a>.</p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"isEqual":{"!type":"fn(v1: ?, v2: ?) -> !this","!doc":"<p>Override. Treat undefined and null values as equal to an empty string value.</p>\n"},"loadPage":{"!type":"fn(pageNum: ?, options: ?) -> !this"},"onAdded":{"!type":"fn() -> !this","!doc":"<p>The picker (the dropdown) must have its zIndex managed by the same ZIndexManager which is\nproviding the zIndex of our Container.</p>\n"},"onBeforeDeselect":{"!type":"fn(list: ?, record: ?) -> !this"},"onBeforeLoad":{"!type":"fn() -> !this"},"onBeforeSelect":{"!type":"fn(list: ?, record: ?) -> !this"},"onBindStore":{"!type":"fn(store: ?, initial: ?) -> !this","!doc":"<p>Template method, it is called when a new store is bound\nto the current instance.</p>\n"},"onClear":{"!type":"fn() -> !this"},"onCollapse":{"!type":"fn() -> !this","!doc":"<p>Disables the key nav for the BoundList when it is collapsed.</p>\n"},"onDataChanged":{"!type":"fn() -> !this"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onEditorTab":{"!type":"fn(e: ?) -> !this"},"onEndUpdateFilters":{"!type":"fn(filters: ?) -> !this"},"onException":{"!type":"fn() -> !this"},"onExpand":{"!type":"fn() -> !this","!doc":"<p>Enables the key nav for the BoundList when it is expanded.</p>\n"},"onFocus":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onItemClick":{"!type":"fn(picker: ?, record: ?) -> !this"},"onKeyUp":{"!type":"fn(e: ?, t: ?) -> !this","!doc":"<p>store the last key and doQuery if relevant</p>\n"},"onListRefresh":{"!type":"fn() -> !this"},"onListSelectionChange":{"!type":"fn(list: ?, selectedRecords: ?) -> !this"},"onLoad":{"!type":"fn(store: ?, records: ?, success: ?) -> !this"},"onPageChange":{"!type":"fn(toolbar: ?, newPage: ?) -> !this"},"onPaste":{"!type":"fn() -> !this"},"onRemove":{"!type":"fn() -> !this"},"onTriggerClick":{"!type":"fn() -> !this","!doc":"<p>Handles the trigger click; by default toggles between expanding and collapsing the picker component.</p>\n"},"onTypeAhead":{"!type":"fn() -> !this"},"onUnbindStore":{"!type":"fn() -> !this","!doc":"<p>Template method, it is called when an existing store is unbound\nfrom the current instance.</p>\n"},"resetToDefault":{"!type":"fn() -> !this","!doc":"<p>invoked when a different store is bound to this combo\nthan the original</p>\n"},"resizeComboToGrow":{"!type":"fn() -> !this","!doc":"<p>private helper</p>\n"},"select":{"!type":"fn(combo: ?, records: ?, eOpts: ?)","!doc":"<p>Fires when at least one list item is selected.</p>\n"},"setFilters":{"!type":"fn(filters: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.form.field.ComboBox-cfg-filters\" rel=\"Ext.form.field.ComboBox-cfg-filters\" class=\"docClass\">filters</a>.</p>\n"},"setHiddenValue":{"!type":"fn(values: ?) -> !this","!doc":"<p>Set the value of <a href=\"#!/api/Ext.form.field.ComboBox-property-hiddenDataEl\" rel=\"Ext.form.field.ComboBox-property-hiddenDataEl\" class=\"docClass\">hiddenDataEl</a>\nDynamically adds and removes input[type=hidden] elements</p>\n"},"setValue":{"!type":"fn(value: ?, doSelect?: ?) -> ?","!doc":"<p>Sets the specified value(s) into the field. For each value, if a record is found in the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-store\" rel=\"Ext.form.field.ComboBox-cfg-store\" class=\"docClass\">store</a> that\nmatches based on the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-valueField\" rel=\"Ext.form.field.ComboBox-cfg-valueField\" class=\"docClass\">valueField</a>, then that record's <a href=\"#!/api/Ext.form.field.ComboBox-cfg-displayField\" rel=\"Ext.form.field.ComboBox-cfg-displayField\" class=\"docClass\">displayField</a> will be displayed in the\nfield. If no match is found, and the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-valueNotFoundText\" rel=\"Ext.form.field.ComboBox-cfg-valueNotFoundText\" class=\"docClass\">valueNotFoundText</a> config option is defined, then that will be\ndisplayed as the default field text. Otherwise a blank value will be shown, although the value will still be set.</p>\n"},"setValueOnData":{"!type":"fn() -> !this"},"syncSelection":{"!type":"fn() -> !this","!doc":"<p>Synchronizes the selection in the picker to match the current value of the combobox.</p>\n"},"updateFilters":{"!type":"fn(newFilters: ?, oldFilters: ?) -> !this"},"beforedeselect":{"!type":"fn(combo: ?, record: ?, index: ?, eOpts: ?)","!doc":"<p>Fires before the deselected item is removed from the collection</p>\n"},"beforequery":{"!type":"fn(queryPlan: ?, eOpts: ?)","!doc":"<p>Fires before all queries are processed. Return false to cancel the query or set the queryPlan's cancel\nproperty to true.</p>\n"},"beforeselect":{"!type":"fn(combo: ?, record: ?, index: ?, eOpts: ?)","!doc":"<p>Fires before the selected item is added to the collection</p>\n"}}},"Text":{"prototype":{"allowBlank":{"!type":"bool","!doc":"<p>Specify false to validate that the value's length must be > 0. If <code>true</code>, then a blank value is <strong>always</strong> taken to be valid regardless of any <a href=\"#!/api/Ext.form.field.Text-cfg-vtype\" rel=\"Ext.form.field.Text-cfg-vtype\" class=\"docClass\">vtype</a>\nvalidation that may be applied.</p>\n\n<p>If <a href=\"#!/api/Ext.form.field.Text-cfg-vtype\" rel=\"Ext.form.field.Text-cfg-vtype\" class=\"docClass\">vtype</a> validation must still be applied to blank values, configure <a href=\"#!/api/Ext.form.field.Text-cfg-validateBlank\" rel=\"Ext.form.field.Text-cfg-validateBlank\" class=\"docClass\">validateBlank</a> as <code>true</code>;</p>\n"},"allowOnlyWhitespace":{"!type":"bool","!doc":"<p>Specify false to automatically trim the value before validating\nthe whether the value is blank. Setting this to false automatically\nsets <a href=\"#!/api/Ext.form.field.Text-cfg-allowBlank\" rel=\"Ext.form.field.Text-cfg-allowBlank\" class=\"docClass\">allowBlank</a> to false.</p>\n"},"blankText":{"!type":"string","!doc":"<p>The error text to display if the <strong><a href=\"#!/api/Ext.form.field.Text-cfg-allowBlank\" rel=\"Ext.form.field.Text-cfg-allowBlank\" class=\"docClass\">allowBlank</a></strong> validation fails</p>\n"},"childEls":{},"disableKeyFilter":{"!type":"bool","!doc":"<p>Specify true to disable input keystroke filtering</p>\n"},"emptyCls":{"!type":"string","!doc":"<p>The CSS class to apply to an empty field to style the <strong><a href=\"#!/api/Ext.form.field.Text-cfg-emptyText\" rel=\"Ext.form.field.Text-cfg-emptyText\" class=\"docClass\">emptyText</a></strong>.\nThis class is automatically added and removed as needed depending on the current field value.</p>\n"},"emptyText":{"!type":"string","!doc":"<p>The default text to place into an empty field.</p>\n\n<p>Note that normally this value will be submitted to the server if this field is enabled; to prevent this you can\nset the <a href=\"#!/api/Ext.form.action.Action-cfg-submitEmptyText\" rel=\"Ext.form.action.Action-cfg-submitEmptyText\" class=\"docClass\">submitEmptyText</a> option of <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">Ext.form.Basic.submit</a> to\nfalse.</p>\n\n<p>Also note that if you use <a href=\"#!/api/Ext.form.field.Text-cfg-inputType\" rel=\"Ext.form.field.Text-cfg-inputType\" class=\"docClass\">inputType</a>:'file', <a href=\"#!/api/Ext.form.field.Text-cfg-emptyText\" rel=\"Ext.form.field.Text-cfg-emptyText\" class=\"docClass\">emptyText</a> is not supported and should be\navoided.</p>\n\n<p>Note that for browsers that support it, setting this property will use the HTML 5 placeholder attribute, and for\nolder browsers that don't support the HTML 5 placeholder attribute the value will be placed directly into the input\nelement itself as the raw value. This means that older browsers will obfuscate the <a href=\"#!/api/Ext.form.field.Text-cfg-emptyText\" rel=\"Ext.form.field.Text-cfg-emptyText\" class=\"docClass\">emptyText</a> value for\npassword input fields.</p>\n"},"enableKeyEvents":{"!type":"bool","!doc":"<p>true to enable the proxying of key events for the HTML input field</p>\n"},"enforceMaxLength":{"!type":"bool","!doc":"<p>True to set the maxLength property on the underlying input field. Defaults to false</p>\n"},"fieldBodyCls":{"!type":"string","!doc":"<p>An extra CSS class to be applied to the body content element in addition to <a href=\"#!/api/Ext.form.field.Text-cfg-baseBodyCls\" rel=\"Ext.form.field.Text-cfg-baseBodyCls\" class=\"docClass\">baseBodyCls</a>.</p>\n"},"grow":{"!type":"bool","!doc":"<p>true if this field should automatically grow and shrink to its content</p>\n"},"growAppend":{"!type":"string","!doc":"<p>A string that will be appended to the field's current value for the purposes of calculating the target field\nsize. Only used when the <a href=\"#!/api/Ext.form.field.Text-cfg-grow\" rel=\"Ext.form.field.Text-cfg-grow\" class=\"docClass\">grow</a> config is true. Defaults to a single capital \"W\" (the widest character in\ncommon fonts) to leave enough space for the next typed character and avoid the field value shifting before the\nwidth is adjusted.</p>\n"},"growMax":{"!type":"number","!doc":"<p>The maximum width to allow when <code><a href=\"#!/api/Ext.form.field.Text-cfg-grow\" rel=\"Ext.form.field.Text-cfg-grow\" class=\"docClass\">grow</a> = true</code></p>\n"},"growMin":{"!type":"number","!doc":"<p>The minimum width to allow when <code><a href=\"#!/api/Ext.form.field.Text-cfg-grow\" rel=\"Ext.form.field.Text-cfg-grow\" class=\"docClass\">grow</a> = true</code></p>\n"},"hideTrigger":{"!type":"bool","!doc":"<p><code>true</code> to hide all triggers</p>\n"},"inputWrapCls":{"!type":"string","!doc":"<p>The CSS class that is added to the element wrapping the input element</p>\n"},"maskRe":{"!doc":"<p>An input mask regular expression that will be used to filter keystrokes (character being\ntyped) that do not match.\nNote: It does not filter characters already in the input.</p>\n"},"maxLength":{"!type":"number","!doc":"<p>Maximum input field length allowed by validation. This behavior is intended to\nprovide instant feedback to the user by improving usability to allow pasting and editing or overtyping and back\ntracking. To restrict the maximum number of characters that can be entered into the field use the\n<strong><a href=\"#!/api/Ext.form.field.Text-cfg-enforceMaxLength\" rel=\"Ext.form.field.Text-cfg-enforceMaxLength\" class=\"docClass\">enforceMaxLength</a></strong> option.</p>\n\n<p>Defaults to Number.MAX_VALUE.</p>\n"},"maxLengthText":{"!type":"string","!doc":"<p>Error text to display if the <strong><a href=\"#!/api/Ext.form.field.Text-cfg-maxLength\" rel=\"Ext.form.field.Text-cfg-maxLength\" class=\"docClass\">maximum length</a></strong> validation fails</p>\n"},"minLength":{"!type":"number","!doc":"<p>Minimum input field length required</p>\n"},"minLengthText":{"!type":"string","!doc":"<p>Error text to display if the <strong><a href=\"#!/api/Ext.form.field.Text-cfg-minLength\" rel=\"Ext.form.field.Text-cfg-minLength\" class=\"docClass\">minimum length</a></strong> validation fails.</p>\n"},"readOnly":{"!type":"bool","!doc":"<p><code>true</code> to prevent the user from changing the field, and hide all triggers.</p>\n"},"regex":{"!doc":"<p>A JavaScript RegExp object to be tested against the field value during validation.\nIf the test fails, the field will be marked invalid using\neither <strong><a href=\"#!/api/Ext.form.field.Text-cfg-regexText\" rel=\"Ext.form.field.Text-cfg-regexText\" class=\"docClass\">regexText</a></strong> or <strong><a href=\"#!/api/Ext.form.field.Text-cfg-invalidText\" rel=\"Ext.form.field.Text-cfg-invalidText\" class=\"docClass\">invalidText</a></strong>.</p>\n"},"regexText":{"!type":"string","!doc":"<p>The error text to display if <strong><a href=\"#!/api/Ext.form.field.Text-cfg-regex\" rel=\"Ext.form.field.Text-cfg-regex\" class=\"docClass\">regex</a></strong> is used and the test fails during validation</p>\n"},"repeatTriggerClick":{"!type":"bool","!doc":"<p><code>true</code> to attach a <a href=\"#!/api/Ext.util.ClickRepeater\" rel=\"Ext.util.ClickRepeater\" class=\"docClass\">click repeater</a> to the trigger(s).\nClick repeating behavior can also be configured on the individual <a href=\"#!/api/Ext.form.field.Text-cfg-triggers\" rel=\"Ext.form.field.Text-cfg-triggers\" class=\"docClass\">trigger instances using the trigger's {@link {Ext.form.trigger.Trigger#repeatClick\nrepeatClick</a> config.</p>\n"},"requiredCls":{"!type":"string","!doc":"<p>The CSS class to apply to a required field, i.e. a field where <strong><a href=\"#!/api/Ext.form.field.Text-cfg-allowBlank\" rel=\"Ext.form.field.Text-cfg-allowBlank\" class=\"docClass\">allowBlank</a></strong> is false.</p>\n"},"selectOnFocus":{"!type":"bool","!doc":"<p><code>true</code> to automatically select any existing field text when the field receives input\nfocus. Only applies when <a href=\"#!/api/Ext.form.field.Text-cfg-editable\" rel=\"Ext.form.field.Text-cfg-editable\" class=\"docClass\">editable</a> = true</p>\n"},"size":{"!type":"number","!doc":"<p>An initial value for the 'size' attribute on the text input element. This is only\nused if the field has no configured <a href=\"#!/api/Ext.form.field.Text-cfg-width\" rel=\"Ext.form.field.Text-cfg-width\" class=\"docClass\">width</a> and is not given a width by its\ncontainer's layout. Defaults to 20.</p>\n"},"stripCharsRe":{"!doc":"<p>A JavaScript RegExp object used to strip unwanted content from the value\nduring input. If <code>stripCharsRe</code> is specified,\nevery <em>character sequence</em> matching <code>stripCharsRe</code> will be removed.</p>\n"},"triggerWrapCls":{"!type":"string","!doc":"<p>The CSS class that is added to the div wrapping the input element and trigger button(s).</p>\n"},"validateBlank":{"!type":"bool","!doc":"<p>Specify as <code>true</code> to modify the behaviour of <a href=\"#!/api/Ext.form.field.Text-cfg-allowBlank\" rel=\"Ext.form.field.Text-cfg-allowBlank\" class=\"docClass\">allowBlank</a> so that blank values are not passed as valid, but are subject to any configure <a href=\"#!/api/Ext.form.field.Text-cfg-vtype\" rel=\"Ext.form.field.Text-cfg-vtype\" class=\"docClass\">vtype</a> validation.</p>\n"},"validator":{"!doc":"<p>A custom validation function to be called during field validation (<a href=\"#!/api/Ext.form.field.Text-method-getErrors\" rel=\"Ext.form.field.Text-method-getErrors\" class=\"docClass\">getErrors</a>).\nIf specified, this function will be called first, allowing the developer to override the default validation\nprocess.</p>\n\n<p>This function will be passed the following parameters:</p>\n"},"vtype":{"!type":"string","!doc":"<p>A validation type name as defined in <a href=\"#!/api/Ext.form.field.VTypes\" rel=\"Ext.form.field.VTypes\" class=\"docClass\">Ext.form.field.VTypes</a></p>\n"},"vtypeText":{"!type":"string","!doc":"<p>A custom error message to display in place of the default message provided for the <strong><code><a href=\"#!/api/Ext.form.field.Text-cfg-vtype\" rel=\"Ext.form.field.Text-cfg-vtype\" class=\"docClass\">vtype</a></code></strong> currently\nset for this field. <strong>Note</strong>: only applies if <strong><code><a href=\"#!/api/Ext.form.field.Text-cfg-vtype\" rel=\"Ext.form.field.Text-cfg-vtype\" class=\"docClass\">vtype</a></code></strong> is set, else ignored.</p>\n"},"deprecated":{"!doc":"<p>private</p>\n"},"growCls":{"!type":"string"},"inputCell":{"!doc":"<p>A reference to the element that wraps the input element. Only set after the\nfield has been rendered.</p>\n"},"inputWrap":{"!doc":"<p>A reference to the element that wraps the input element. Only set after the\nfield has been rendered.</p>\n"},"inputWrapFocusCls":{"!type":"string"},"inputWrapInvalidCls":{"!type":"string"},"mimicing":{"!type":"bool","!doc":"<p>private</p>\n"},"monitorTab":{"!type":"bool","!doc":"<p>private</p>\n"},"postSubTpl":{},"preSubTpl":{},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"triggerEl":{},"triggerWrap":{"!doc":"<p>A reference to the element which encapsulates the input field and all\ntrigger button(s). Only set after the field has been rendered.</p>\n"},"triggerWrapFocusCls":{"!type":"string"},"triggerWrapInvalidCls":{"!type":"string"},"valueContainsPlaceholder":{"!type":"bool","!doc":"<p>private</p>\n"},"afterFirstLayout":{"!type":"fn() -> !this"},"applyEmptyText":{"!type":"fn() -> !this"},"applyState":{"!type":"fn(state: ?) -> !this","!doc":"<p>Applies the state to the object. This should be overridden in subclasses to do\nmore complex state operations. By default it applies the state properties onto\nthe current object.</p>\n"},"applyTriggers":{"!type":"fn(triggers: ?) -> !this"},"autoSize":{"!type":"fn() -> !this","!doc":"<p>Automatically grows the field to accommodate the width of the text up to the maximum\nfield width allowed. This only takes effect if <a href=\"#!/api/Ext.form.field.Text-cfg-grow\" rel=\"Ext.form.field.Text-cfg-grow\" class=\"docClass\">grow</a> = true, and fires the\n<a href=\"#!/api/Ext.form.field.Text-event-autosize\" rel=\"Ext.form.field.Text-event-autosize\" class=\"docClass\">autosize</a> event if the width changes.</p>\n"},"beforeFocus":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"checkTab":{"!type":"fn(me: ?, e: ?) -> !this"},"filterKeys":{"!type":"fn(e: ?) -> !this","!doc":"<p>private</p>\n"},"focusInput":{"!type":"fn() -> !this"},"getErrors":{"!type":"fn(value: ?) -> ?","!doc":"<p>Validates a value according to the field's validation rules and returns an array of errors\nfor any failing validations. Validation rules are processed in the following order:</p>\n\n<ol>\n<li><p><strong>Field specific validator</strong></p>\n\n<p> A validator offers a way to customize and reuse a validation specification.\n If a field is configured with a <code><a href=\"#!/api/Ext.form.field.Text-cfg-validator\" rel=\"Ext.form.field.Text-cfg-validator\" class=\"docClass\">validator</a></code>\n function, it will be passed the current field value.  The <code><a href=\"#!/api/Ext.form.field.Text-cfg-validator\" rel=\"Ext.form.field.Text-cfg-validator\" class=\"docClass\">validator</a></code>\n function is expected to return either:</p>\n\n<ul>\n<li>Boolean <code>true</code>  if the value is valid (validation continues).</li>\n<li>a String to represent the invalid message if invalid (validation halts).</li>\n</ul>\n</li>\n<li><p><strong>Basic Validation</strong></p>\n\n<p> If the <code><a href=\"#!/api/Ext.form.field.Text-cfg-validator\" rel=\"Ext.form.field.Text-cfg-validator\" class=\"docClass\">validator</a></code> has not halted validation,\n basic validation proceeds as follows:</p>\n\n<ul>\n<li><p><code><a href=\"#!/api/Ext.form.field.Text-cfg-allowBlank\" rel=\"Ext.form.field.Text-cfg-allowBlank\" class=\"docClass\">allowBlank</a></code> : (Invalid message = <code><a href=\"#!/api/Ext.form.field.Text-cfg-blankText\" rel=\"Ext.form.field.Text-cfg-blankText\" class=\"docClass\">blankText</a></code>)</p>\n\n<p>  Depending on the configuration of <code><a href=\"#!/api/Ext.form.field.Text-cfg-allowBlank\" rel=\"Ext.form.field.Text-cfg-allowBlank\" class=\"docClass\">allowBlank</a></code>, a\n  blank field will cause validation to halt at this step and return\n  Boolean true or false accordingly.</p></li>\n<li><p><code><a href=\"#!/api/Ext.form.field.Text-cfg-minLength\" rel=\"Ext.form.field.Text-cfg-minLength\" class=\"docClass\">minLength</a></code> : (Invalid message = <code><a href=\"#!/api/Ext.form.field.Text-cfg-minLengthText\" rel=\"Ext.form.field.Text-cfg-minLengthText\" class=\"docClass\">minLengthText</a></code>)</p>\n\n<p>  If the passed value does not satisfy the <code><a href=\"#!/api/Ext.form.field.Text-cfg-minLength\" rel=\"Ext.form.field.Text-cfg-minLength\" class=\"docClass\">minLength</a></code>\n  specified, validation halts.</p></li>\n<li><p><code><a href=\"#!/api/Ext.form.field.Text-cfg-maxLength\" rel=\"Ext.form.field.Text-cfg-maxLength\" class=\"docClass\">maxLength</a></code> : (Invalid message = <code><a href=\"#!/api/Ext.form.field.Text-cfg-maxLengthText\" rel=\"Ext.form.field.Text-cfg-maxLengthText\" class=\"docClass\">maxLengthText</a></code>)</p>\n\n<p> If the passed value does not satisfy the <code><a href=\"#!/api/Ext.form.field.Text-cfg-maxLength\" rel=\"Ext.form.field.Text-cfg-maxLength\" class=\"docClass\">maxLength</a></code>\n specified, validation halts.</p></li>\n</ul>\n</li>\n<li><p><strong>Preconfigured Validation Types (VTypes)</strong></p>\n\n<p> If none of the prior validation steps halts validation, a field\n configured with a <code><a href=\"#!/api/Ext.form.field.Text-cfg-vtype\" rel=\"Ext.form.field.Text-cfg-vtype\" class=\"docClass\">vtype</a></code> will utilize the\n corresponding <a href=\"#!/api/Ext.form.field.VTypes\" rel=\"Ext.form.field.VTypes\" class=\"docClass\">VTypes</a> validation function.\n If invalid, either the field's <code><a href=\"#!/api/Ext.form.field.Text-cfg-vtypeText\" rel=\"Ext.form.field.Text-cfg-vtypeText\" class=\"docClass\">vtypeText</a></code> or\n the VTypes vtype Text property will be used for the invalid message.\n Keystrokes on the field will be filtered according to the VTypes\n vtype Mask property.</p></li>\n<li><p><strong>Field specific regex test</strong></p>\n\n<p> If none of the prior validation steps halts validation, a field's\n configured <code><a href=\"#!/api/Ext.form.field.Text-cfg-regex\" rel=\"Ext.form.field.Text-cfg-regex\" class=\"docClass\">regex</a></code> test will be processed.\n The invalid message for this test is configured with <code><a href=\"#!/api/Ext.form.field.Text-cfg-regexText\" rel=\"Ext.form.field.Text-cfg-regexText\" class=\"docClass\">regexText</a></code></p></li>\n</ol>\n\n"},"getHideTrigger":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.form.field.Text-cfg-hideTrigger\" rel=\"Ext.form.field.Text-cfg-hideTrigger\" class=\"docClass\">hideTrigger</a>.</p>\n"},"getRawValue":{"!type":"fn() -> string","!doc":"<p>Returns the raw String value of the field, without performing any normalization, conversion, or validation. Gets\nthe current value of the input element if the field has been rendered, ignoring the value if it is the\n<a href=\"#!/api/Ext.form.field.Text-cfg-emptyText\" rel=\"Ext.form.field.Text-cfg-emptyText\" class=\"docClass\">emptyText</a>. To get a normalized and converted value see <a href=\"#!/api/Ext.form.field.Text-method-getValue\" rel=\"Ext.form.field.Text-method-getValue\" class=\"docClass\">getValue</a>.</p>\n"},"getState":{"!type":"fn() -> ?","!doc":"<p>The supplied default state gathering method for the Component class.</p>\n\n<p>This method returns dimension settings such as <code>flex</code>, <code>anchor</code>, <code>width</code> and <code>height</code> along with <code>collapsed</code>\nstate.</p>\n\n<p>Subclasses which implement more complex state should call the superclass's implementation, and apply their state\nto the result if this basic state is to be saved.</p>\n\n<p>Note that Component state will only be saved if the Component has a <a href=\"#!/api/Ext.form.field.Text-cfg-stateId\" rel=\"Ext.form.field.Text-cfg-stateId\" class=\"docClass\">stateId</a> and there as a StateProvider\nconfigured for the document.</p>\n"},"getSubTplData":{"!type":"fn() -> ?","!doc":"<p>Creates and returns the data object to be used when rendering the <a href=\"#!/api/Ext.form.field.Text-cfg-fieldSubTpl\" rel=\"Ext.form.field.Text-cfg-fieldSubTpl\" class=\"docClass\">fieldSubTpl</a>.</p>\n"},"getTdType":{"!type":"fn() -> !this"},"getTrigger":{"!type":"fn(id: ?) -> ?","!doc":"<p>Returns the trigger with the given id</p>\n"},"getTriggerWidth":{"!type":"fn() -> number","!doc":"<p>Get the total width of the trigger button area.</p>\n"},"invokeTriggers":{"!type":"fn(methodName: ?) -> !this","!doc":"<p>Invokes a method on all triggers.</p>\n"},"onBlur":{"!type":"fn() -> !this","!doc":"<p>The default blur handling must not occur for a TriggerField, implementing this template method disables that.\nInstead the tab key is monitored, and the superclass's onBlur is called when tab is detected</p>\n"},"onChange":{"!type":"fn(newVal: ?, oldVal: ?) -> !this","!doc":"<p>If grow=true, invoke the autoSize method when the field's value is changed.</p>\n"},"onDisable":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onEnable":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onKeyDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Listen for TAB events and wrap round if tabbing of either end of the Floater</p>\n"},"onKeyPress":{"!type":"fn(e: ?) -> !this"},"onMouseDown":{"!type":"fn() -> !this","!doc":"<p>Mousedown brings to front, and programatically grabs focus\nunless the mousedown was on a focusable element*</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"postBlur":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"processRawValue":{"!type":"fn(value: ?) -> string","!doc":"<p>Performs any necessary manipulation of a raw String value to prepare it for conversion and/or\n<a href=\"#!/api/Ext.form.field.Text-method-validate\" rel=\"Ext.form.field.Text-method-validate\" class=\"docClass\">validation</a>. For text fields this applies the configured <a href=\"#!/api/Ext.form.field.Text-cfg-stripCharsRe\" rel=\"Ext.form.field.Text-cfg-stripCharsRe\" class=\"docClass\">stripCharsRe</a>\nto the raw value.</p>\n"},"reset":{"!type":"fn() -> !this","!doc":"<p>Resets the current field value to the originally-loaded value and clears any validation messages.\nAlso adds <strong><a href=\"#!/api/Ext.form.field.Text-cfg-emptyText\" rel=\"Ext.form.field.Text-cfg-emptyText\" class=\"docClass\">emptyText</a></strong> and <strong><a href=\"#!/api/Ext.form.field.Text-cfg-emptyCls\" rel=\"Ext.form.field.Text-cfg-emptyCls\" class=\"docClass\">emptyCls</a></strong> if the original value was blank.</p>\n"},"selectText":{"!type":"fn(start?: ?, end?: ?) -> !this","!doc":"<p>Selects text in this field</p>\n"},"setEditable":{"!type":"fn(editable: ?) -> !this","!doc":"<p>Sets the editable state of this field.</p>\n"},"setGrowSizePolicy":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"setHideTrigger":{"!type":"fn(hideTrigger: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.form.field.Text-cfg-hideTrigger\" rel=\"Ext.form.field.Text-cfg-hideTrigger\" class=\"docClass\">hideTrigger</a>.</p>\n"},"setReadOnly":{"!type":"fn(readOnly: ?) -> !this","!doc":"<p>Sets the read-only state of this field.</p>\n"},"setReadOnlyAttr":{"!type":"fn(readOnly: ?) -> !this","!doc":"<p>private. sets the readonly attribute of the input element</p>\n"},"toggleInvalidCls":{"!type":"fn(hasError: ?) -> !this","!doc":"<p>private</p>\n"},"updateHideTrigger":{"!type":"fn(hideTrigger: ?) -> !this"},"autosize":{"!type":"fn(this: ?, width: ?, eOpts: ?)","!doc":"<p>Fires when the <strong><a href=\"#!/api/Ext.form.field.Text-method-autoSize\" rel=\"Ext.form.field.Text-method-autoSize\" class=\"docClass\">autoSize</a></strong> function is triggered and the field is resized according to the\n<a href=\"#!/api/Ext.form.field.Text-cfg-grow\" rel=\"Ext.form.field.Text-cfg-grow\" class=\"docClass\">grow</a>/<a href=\"#!/api/Ext.form.field.Text-cfg-growMin\" rel=\"Ext.form.field.Text-cfg-growMin\" class=\"docClass\">growMin</a>/<a href=\"#!/api/Ext.form.field.Text-cfg-growMax\" rel=\"Ext.form.field.Text-cfg-growMax\" class=\"docClass\">growMax</a> configs as a result. This event provides a hook for the\ndeveloper to apply additional logic at runtime to resize the field if needed.</p>\n"},"keydown":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Keydown input field event. This event only fires if <strong><a href=\"#!/api/Ext.form.field.Text-cfg-enableKeyEvents\" rel=\"Ext.form.field.Text-cfg-enableKeyEvents\" class=\"docClass\">enableKeyEvents</a></strong> is set to true.</p>\n"},"keypress":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Keypress input field event. This event only fires if <strong><a href=\"#!/api/Ext.form.field.Text-cfg-enableKeyEvents\" rel=\"Ext.form.field.Text-cfg-enableKeyEvents\" class=\"docClass\">enableKeyEvents</a></strong> is set to true.</p>\n"},"keyup":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Keyup input field event. This event only fires if <strong><a href=\"#!/api/Ext.form.field.Text-cfg-enableKeyEvents\" rel=\"Ext.form.field.Text-cfg-enableKeyEvents\" class=\"docClass\">enableKeyEvents</a></strong> is set to true.</p>\n"},"ariaRole":{"!type":"string"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"isEqual":{"!type":"fn(value1: ?, value2: ?) -> !this","!doc":"<p>Override. Treat undefined and null values as equal to an empty string value.</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onFocus":{"!type":"fn(e: ?) -> !this","!doc":"<p>private</p>\n"},"onKeyUp":{"!type":"fn(e: ?) -> !this"},"getTriggers":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.form.field.Text-cfg-triggers\" rel=\"Ext.form.field.Text-cfg-triggers\" class=\"docClass\">triggers</a>.</p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"mimicBlur":{"!type":"fn(e: ?) -> !this"},"onOtherFocus":{"!type":"fn(dom: ?) -> !this","!doc":"<p>ensures we trigger a blur if some other component/element is programmatically\nfocused (see EXTJSIV-11712)</p>\n"},"onTriggerClick":{"!type":"fn() -> !this"},"setTriggers":{"!type":"fn(triggers: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.form.field.Text-cfg-triggers\" rel=\"Ext.form.field.Text-cfg-triggers\" class=\"docClass\">triggers</a>.</p>\n"},"triggerBlur":{"!type":"fn(e: ?) -> !this"},"editable":{"!type":"bool","!doc":"<p>false to prevent the user from typing text directly into the field; the field can\nonly have its value set programmatically or via an action invoked by a trigger.</p>\n"},"setValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Sets a data value into the field and runs the change detection and validation. Also applies any configured\n<a href=\"#!/api/Ext.form.field.Text-cfg-emptyText\" rel=\"Ext.form.field.Text-cfg-emptyText\" class=\"docClass\">emptyText</a> for text fields. To set the value directly without these inspections see <a href=\"#!/api/Ext.form.field.Text-method-setRawValue\" rel=\"Ext.form.field.Text-method-setRawValue\" class=\"docClass\">setRawValue</a>.</p>\n"},"triggers":{"!doc":"<p><a href=\"#!/api/Ext.form.trigger.Trigger\" rel=\"Ext.form.trigger.Trigger\" class=\"docClass\">Triggers</a> to use in this field.  The keys in\nthis object are unique identifiers for the triggers. The values in this object\nare <a href=\"#!/api/Ext.form.trigger.Trigger\" rel=\"Ext.form.trigger.Trigger\" class=\"docClass\">Trigger</a> configuration objects.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n    renderTo: Ext.getBody(),\n    fieldLabel: 'My Custom Field',\n    triggers: {\n        foo: {\n            cls: 'my-foo-trigger',\n            handler: function() {\n                console.log('foo trigger clicked');\n            }\n        },\n        bar: {\n            cls: 'my-bar-trigger',\n            handler: function() {\n                console.log('bar trigger clicked');\n            }\n        }\n    }\n});\n</code></pre>\n"}},"!doc":"<p>A basic text field.  Can be used as a direct replacement for traditional text inputs,\nor as the base class for more sophisticated input controls (like <a href=\"#!/api/Ext.form.field.TextArea\" rel=\"Ext.form.field.TextArea\" class=\"docClass\">Ext.form.field.TextArea</a>\nand <a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.field.ComboBox</a>). Has support for empty-field placeholder values (see <a href=\"#!/api/Ext.form.field.Text-cfg-emptyText\" rel=\"Ext.form.field.Text-cfg-emptyText\" class=\"docClass\">emptyText</a>).</p>\n\n<h1>Validation</h1>\n\n<p>The Text field has a useful set of validations built in:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.form.field.Text-cfg-allowBlank\" rel=\"Ext.form.field.Text-cfg-allowBlank\" class=\"docClass\">allowBlank</a> for making the field required</li>\n<li><a href=\"#!/api/Ext.form.field.Text-cfg-minLength\" rel=\"Ext.form.field.Text-cfg-minLength\" class=\"docClass\">minLength</a> for requiring a minimum value length</li>\n<li><a href=\"#!/api/Ext.form.field.Text-cfg-maxLength\" rel=\"Ext.form.field.Text-cfg-maxLength\" class=\"docClass\">maxLength</a> for setting a maximum value length (with <a href=\"#!/api/Ext.form.field.Text-cfg-enforceMaxLength\" rel=\"Ext.form.field.Text-cfg-enforceMaxLength\" class=\"docClass\">enforceMaxLength</a> to add it\nas the <code>maxlength</code> attribute on the input element)</li>\n<li><a href=\"#!/api/Ext.form.field.Text-cfg-regex\" rel=\"Ext.form.field.Text-cfg-regex\" class=\"docClass\">regex</a> to specify a custom regular expression for validation</li>\n</ul>\n\n\n<p>In addition, custom validations may be added:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.form.field.Text-cfg-vtype\" rel=\"Ext.form.field.Text-cfg-vtype\" class=\"docClass\">vtype</a> specifies a virtual type implementation from <a href=\"#!/api/Ext.form.field.VTypes\" rel=\"Ext.form.field.VTypes\" class=\"docClass\">Ext.form.field.VTypes</a> which can contain\ncustom validation logic</li>\n<li><a href=\"#!/api/Ext.form.field.Text-cfg-validator\" rel=\"Ext.form.field.Text-cfg-validator\" class=\"docClass\">validator</a> allows a custom arbitrary function to be called during validation</li>\n</ul>\n\n\n<p>The details around how and when each of these validation options get used are described in the\ndocumentation for <a href=\"#!/api/Ext.form.field.Text-method-getErrors\" rel=\"Ext.form.field.Text-method-getErrors\" class=\"docClass\">getErrors</a>.</p>\n\n<p>By default, the field value is checked for validity immediately while the user is typing in the\nfield. This can be controlled with the <a href=\"#!/api/Ext.form.field.Text-cfg-validateOnChange\" rel=\"Ext.form.field.Text-cfg-validateOnChange\" class=\"docClass\">validateOnChange</a>, <a href=\"#!/api/Ext.form.field.Text-cfg-checkChangeEvents\" rel=\"Ext.form.field.Text-cfg-checkChangeEvents\" class=\"docClass\">checkChangeEvents</a>, and\n<a href=\"#!/api/Ext.form.field.Text-cfg-checkChangeBuffer\" rel=\"Ext.form.field.Text-cfg-checkChangeBuffer\" class=\"docClass\">checkChangeBuffer</a> configurations. Also see the details on Form Validation in the\n<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a> class documentation.</p>\n\n<h1>Masking and Character Stripping</h1>\n\n<p>Text fields can be configured with custom regular expressions to be applied to entered values before\nvalidation: see <a href=\"#!/api/Ext.form.field.Text-cfg-maskRe\" rel=\"Ext.form.field.Text-cfg-maskRe\" class=\"docClass\">maskRe</a> and <a href=\"#!/api/Ext.form.field.Text-cfg-stripCharsRe\" rel=\"Ext.form.field.Text-cfg-stripCharsRe\" class=\"docClass\">stripCharsRe</a> for details.</p>\n\n<h1>Example usage</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'Contact Info',\n    width: 300,\n    bodyPadding: 10,\n    renderTo: Ext.getBody(),\n    items: [{\n        xtype: 'textfield',\n        name: 'name',\n        fieldLabel: 'Name',\n        allowBlank: false  // requires a non-empty value\n    }, {\n        xtype: 'textfield',\n        name: 'email',\n        fieldLabel: 'Email Address',\n        vtype: 'email'  // requires value to be a valid email address format\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)"},"Picker":{"prototype":{"editable":{"!type":"bool","!doc":"<p>False to prevent the user from typing text directly into the field; the field can only have its value set via\nselecting a value from the picker. In this state, the picker can also be opened by clicking directly on the input\nfield itself.</p>\n"},"matchFieldWidth":{"!type":"bool","!doc":"<p>Whether the picker dropdown's width should be explicitly set to match the width of the field. Defaults to true.</p>\n"},"openCls":{"!type":"string","!doc":"<p>A class to be added to the field's <a href=\"#!/api/Ext.form.field.Picker-property-bodyEl\" rel=\"Ext.form.field.Picker-property-bodyEl\" class=\"docClass\">bodyEl</a> element when the picker is opened.</p>\n"},"pickerAlign":{"!type":"string","!doc":"<p>The alignment position with which to align the picker. Defaults to \"tl-bl?\"</p>\n"},"pickerOffset":{"!doc":"<p>An offset [x,y] to use in addition to the <a href=\"#!/api/Ext.form.field.Picker-cfg-pickerAlign\" rel=\"Ext.form.field.Picker-cfg-pickerAlign\" class=\"docClass\">pickerAlign</a> when positioning the picker.\nDefaults to undefined.</p>\n"},"triggers":{"!doc":"<p><a href=\"#!/api/Ext.form.trigger.Trigger\" rel=\"Ext.form.trigger.Trigger\" class=\"docClass\">Triggers</a> to use in this field.  The keys in\nthis object are unique identifiers for the triggers. The values in this object\nare <a href=\"#!/api/Ext.form.trigger.Trigger\" rel=\"Ext.form.trigger.Trigger\" class=\"docClass\">Trigger</a> configuration objects.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n    renderTo: Ext.getBody(),\n    fieldLabel: 'My Custom Field',\n    triggers: {\n        foo: {\n            cls: 'my-foo-trigger',\n            handler: function() {\n                console.log('foo trigger clicked');\n            }\n        },\n        bar: {\n            cls: 'my-bar-trigger',\n            handler: function() {\n                console.log('bar trigger clicked');\n            }\n        }\n    }\n});\n</code></pre>\n"},"isExpanded":{"!type":"bool","!doc":"<p>True if the picker is currently expanded, false if not.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"collapse":{"!type":"fn(field: ?, eOpts: ?)","!doc":"<p>Fires when the field's picker is collapsed.</p>\n"},"collapseIf":{"!type":"fn(e: ?) -> !this","!doc":"<p>Runs on mousewheel and mousedown of doc to check to see if we should collapse the picker</p>\n"},"doAlign":{"!type":"fn() -> !this","!doc":"<p>Performs the alignment on the picker using the class defaults</p>\n"},"expand":{"!type":"fn(field: ?, eOpts: ?)","!doc":"<p>Fires when the field's picker is expanded.</p>\n"},"getPicker":{"!type":"fn() -> ?","!doc":"<p>Returns a reference to the picker component for this field, creating it if necessary by\ncalling <a href=\"#!/api/Ext.form.field.Picker-method-createPicker\" rel=\"Ext.form.field.Picker-method-createPicker\" class=\"docClass\">createPicker</a>.</p>\n"},"getRefItems":{"!type":"fn() -> !this","!doc":"<p>The CQ interface. Allow drilling down into the picker when it exists.\nImportant for determining whether an event took place in the bounds of some\nhigher level containing component. See AbstractComponent#owns</p>\n"},"getTriggers":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.form.field.Picker-cfg-triggers\" rel=\"Ext.form.field.Picker-cfg-triggers\" class=\"docClass\">triggers</a>.</p>\n"},"mimicBlur":{"!type":"fn(e: ?) -> !this"},"onDownArrow":{"!type":"fn() -> !this"},"onEsc":{"!type":"fn(e: ?) -> !this","!doc":"<p>private</p>\n"},"onOtherFocus":{"!type":"fn(dom: ?) -> !this","!doc":"<p>ensures we trigger a blur if some other component/element is programmatically\nfocused (see EXTJSIV-11712)</p>\n"},"setTriggers":{"!type":"fn(triggers: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.form.field.Picker-cfg-triggers\" rel=\"Ext.form.field.Picker-cfg-triggers\" class=\"docClass\">triggers</a>.</p>\n"},"triggerBlur":{"!type":"fn() -> !this"},"alignPicker":{"!type":"fn() -> !this","!doc":"<p>Aligns the picker to the input element</p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onCollapse":{"!type":"fn() -> !this"},"onTriggerClick":{"!type":"fn() -> !this","!doc":"<p>Handles the trigger click; by default toggles between expanding and collapsing the picker component.</p>\n"},"select":{"!type":"fn(field: ?, value: ?, eOpts: ?)","!doc":"<p>Fires when a value is selected via the picker.</p>\n"},"triggerCls":{"!type":"string","!doc":"<p>An additional CSS class used to style the trigger button. The trigger will always\nget the class 'x-form-trigger' and triggerCls will be appended if specified.</p>\n"},"createPicker":{"!type":"fn() -> !this","!doc":"<p>Creates and returns the component to be used as this field's picker. Must be implemented by subclasses of Picker.\nThe current field should also be passed as a configuration option to the picker component as the pickerField\nproperty.</p>\n"},"onExpand":{"!type":"fn() -> !this"}},"!doc":"<p>An abstract class for fields that have a single trigger which opens a \"picker\" popup below the field, e.g. a combobox\nmenu list or a date picker. It provides a base implementation for toggling the picker's visibility when the trigger\nis clicked, as well as keyboard navigation and some basic events. Sizing and alignment of the picker can be\ncontrolled via the <a href=\"#!/api/Ext.form.field.Picker-cfg-matchFieldWidth\" rel=\"Ext.form.field.Picker-cfg-matchFieldWidth\" class=\"docClass\">matchFieldWidth</a> and <a href=\"#!/api/Ext.form.field.Picker-cfg-pickerAlign\" rel=\"Ext.form.field.Picker-cfg-pickerAlign\" class=\"docClass\">pickerAlign</a>/<a href=\"#!/api/Ext.form.field.Picker-cfg-pickerOffset\" rel=\"Ext.form.field.Picker-cfg-pickerOffset\" class=\"docClass\">pickerOffset</a> config properties\nrespectively.</p>\n\n<p>You would not normally use this class directly, but instead use it as the parent class for a specific picker field\nimplementation. Subclasses must implement the <a href=\"#!/api/Ext.form.field.Picker-method-createPicker\" rel=\"Ext.form.field.Picker-method-createPicker\" class=\"docClass\">createPicker</a> method to create a picker component appropriate\nfor the field.</p>\n","!type":"fn(config: ?)"},"Date":{"!doc":"<p>Provides a date input field with a <a href=\"#!/api/Ext.picker.Date\" rel=\"Ext.picker.Date\" class=\"docClass\">date picker</a> dropdown and automatic date\nvalidation.</p>\n\n<p>This field recognizes and uses the JavaScript Date object as its main <a href=\"#!/api/Ext.form.field.Date-cfg-value\" rel=\"Ext.form.field.Date-cfg-value\" class=\"docClass\">value</a> type. In addition,\nit recognizes string values which are parsed according to the <a href=\"#!/api/Ext.form.field.Date-cfg-format\" rel=\"Ext.form.field.Date-cfg-format\" class=\"docClass\">format</a> and/or <a href=\"#!/api/Ext.form.field.Date-cfg-altFormats\" rel=\"Ext.form.field.Date-cfg-altFormats\" class=\"docClass\">altFormats</a>\nconfigs. These may be reconfigured to use date formats appropriate for the user's locale.</p>\n\n<p>The field may be limited to a certain range of dates by using the <a href=\"#!/api/Ext.form.field.Date-cfg-minValue\" rel=\"Ext.form.field.Date-cfg-minValue\" class=\"docClass\">minValue</a>, <a href=\"#!/api/Ext.form.field.Date-cfg-maxValue\" rel=\"Ext.form.field.Date-cfg-maxValue\" class=\"docClass\">maxValue</a>,\n<a href=\"#!/api/Ext.form.field.Date-cfg-disabledDays\" rel=\"Ext.form.field.Date-cfg-disabledDays\" class=\"docClass\">disabledDays</a>, and <a href=\"#!/api/Ext.form.field.Date-cfg-disabledDates\" rel=\"Ext.form.field.Date-cfg-disabledDates\" class=\"docClass\">disabledDates</a> config parameters. These configurations will be used both\nin the field's validation, and in the date picker dropdown by preventing invalid dates from being selected.</p>\n\n<h1>Example usage</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    renderTo: Ext.getBody(),\n    width: 300,\n    bodyPadding: 10,\n    title: 'Dates',\n    items: [{\n        xtype: 'datefield',\n        anchor: '100%',\n        fieldLabel: 'From',\n        name: 'from_date',\n        maxValue: new Date()  // limited to the current date or prior\n    }, {\n        xtype: 'datefield',\n        anchor: '100%',\n        fieldLabel: 'To',\n        name: 'to_date',\n        value: new Date()  // defaults to today\n    }]\n});\n</code></pre>\n\n<h1>Date Formats Examples</h1>\n\n<p>This example shows a couple of different date format parsing scenarios. Both use custom date format\nconfigurations; the first one matches the configured <code>format</code> while the second matches the <code>altFormats</code>.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    renderTo: Ext.getBody(),\n    width: 300,\n    bodyPadding: 10,\n    title: 'Dates',\n    items: [{\n        xtype: 'datefield',\n        anchor: '100%',\n        fieldLabel: 'Date',\n        name: 'date',\n        // The value matches the format; will be parsed and displayed using that format.\n        format: 'm d Y',\n        value: '2 4 1978'\n    }, {\n        xtype: 'datefield',\n        anchor: '100%',\n        fieldLabel: 'Date',\n        name: 'date',\n        // The value does not match the format, but does match an altFormat; will be parsed\n        // using the altFormat and displayed using the format.\n        format: 'm d Y',\n        altFormats: 'm,d,Y|m.d.Y',\n        value: '2.4.1978'\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"altFormats":{"!type":"string","!doc":"<p>Multiple date formats separated by \"|\" to try when parsing a user input value and it does not match the defined\nformat.</p>\n"},"disabledDates":{"!doc":"<p>An array of \"dates\" to disable, as strings. These strings will be used to build a dynamic regular expression so\nthey are very powerful. Some examples:</p>\n\n<pre><code>// disable these exact dates:\ndisabledDates: [\"03/08/2003\", \"09/16/2003\"]\n// disable these days for every year:\ndisabledDates: [\"03/08\", \"09/16\"]\n// only match the beginning (useful if you are using short years):\ndisabledDates: [\"^03/08\"]\n// disable every day in March 2006:\ndisabledDates: [\"03/../2006\"]\n// disable every day in every March:\ndisabledDates: [\"^03\"]\n</code></pre>\n\n<p>Note that the format of the dates included in the array should exactly match the <a href=\"#!/api/Ext.form.field.Date-cfg-format\" rel=\"Ext.form.field.Date-cfg-format\" class=\"docClass\">format</a> config. In order\nto support regular expressions, if you are using a <a href=\"#!/api/Ext.form.field.Date-cfg-format\" rel=\"Ext.form.field.Date-cfg-format\" class=\"docClass\">date format</a> that has \".\" in it, you will have\nto escape the dot when restricting dates. For example: <code>[\"03\\\\.08\\\\.03\"]</code>.</p>\n"},"disabledDatesText":{"!type":"string","!doc":"<p>The tooltip text to display when the date falls on a disabled date.</p>\n"},"disabledDays":{"!doc":"<p>An array of days to disable, 0 based. Some examples:</p>\n\n<pre><code>// disable Sunday and Saturday:\ndisabledDays:  [0, 6]\n// disable weekdays:\ndisabledDays: [1,2,3,4,5]\n</code></pre>\n"},"disabledDaysText":{"!type":"string","!doc":"<p>The tooltip to display when the date falls on a disabled day.</p>\n"},"format":{"!type":"string","!doc":"<p>The default date format string which can be overriden for localization support. The format must be valid\naccording to Ext.Date.parse.</p>\n"},"grow":{"!type":"bool"},"growMax":{"!type":"number"},"growMin":{"!type":"number"},"invalidText":{"!type":"string","!doc":"<p>The error text to display when the date in the field is invalid.</p>\n"},"matchFieldWidth":{"!type":"bool","!doc":"<p>Whether the picker dropdown's width should be explicitly set to match the width of the field. Defaults to true.</p>\n"},"maxText":{"!type":"string","!doc":"<p>The error text to display when the date in the cell is after <a href=\"#!/api/Ext.form.field.Date-cfg-maxValue\" rel=\"Ext.form.field.Date-cfg-maxValue\" class=\"docClass\">maxValue</a>.</p>\n"},"maxValue":{"!doc":"<p>The maximum allowed date. Can be either a Javascript date object or a string date in a valid format.</p>\n"},"minText":{"!type":"string","!doc":"<p>The error text to display when the date in the cell is before <a href=\"#!/api/Ext.form.field.Date-cfg-minValue\" rel=\"Ext.form.field.Date-cfg-minValue\" class=\"docClass\">minValue</a>.</p>\n"},"minValue":{"!doc":"<p>The minimum allowed date. Can be either a Javascript date object or a string date in a valid format.</p>\n"},"showToday":{"!type":"bool","!doc":"<p>false to hide the footer area of the Date picker containing the Today button and disable the keyboard handler for\nspacebar that selects the current date.</p>\n"},"startDay":{"!type":"number","!doc":"<p>Day index at which the week should begin, 0-based.</p>\n\n<p>Defaults to <code>0</code> (Sunday).</p>\n"},"submitFormat":{"!type":"string","!doc":"<p>The date format string which will be submitted to the server. The format must be valid according to\nExt.Date.parse.</p>\n\n<p>Defaults to <a href=\"#!/api/Ext.form.field.Date-cfg-format\" rel=\"Ext.form.field.Date-cfg-format\" class=\"docClass\">format</a>.</p>\n"},"triggerCls":{"!type":"string","!doc":"<p>An additional CSS class used to style the trigger button. The trigger will always get the class 'x-form-trigger'\nand triggerCls will be <strong>appended</strong> if specified (default class displays a calendar icon).</p>\n"},"useStrict":{"!type":"bool","!doc":"<p>True to enforce strict date parsing to prevent the default Javascript \"date rollover\".\nDefaults to the useStrict parameter set on Ext.Date\nSee Ext.Date.parse.</p>\n"},"initTime":{"!type":"string","!doc":"<p>in the absence of a time value, a default value of 12 noon will be used\n(note: 12 noon was chosen because it steers well clear of all DST timezone changes)</p>\n"},"initTimeFormat":{"!type":"string","!doc":"<p>24 hour format</p>\n"},"autoSize":{"!type":"fn()"},"beforeBlur":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"createPicker":{"!type":"fn() -> !this","!doc":"<p>Creates and returns the component to be used as this field's picker. Must be implemented by subclasses of Picker.\nThe current field should also be passed as a configuration option to the picker component as the pickerField\nproperty.</p>\n"},"formatDate":{"!type":"fn(date: ?) -> !this","!doc":"<p>private</p>\n"},"getErrors":{"!type":"fn(value?: ?) -> ?","!doc":"<p>Runs all of Date's validations and returns an array of any errors. Note that this first runs Text's validations,\nso the returned array is an amalgamation of all field errors. The additional validation checks are testing that\nthe date format is valid, that the chosen date is within the min and max date constraints set, that the date\nchosen is not in the disabledDates regex and that the day chosed is not one of the disabledDays.</p>\n"},"getSubmitValue":{"!type":"fn() -> string","!doc":"<p>Returns the value that would be included in a standard form submit for this field. This will be combined with the\nfield's name to form a name=value pair in the <a href=\"#!/api/Ext.form.field.Date-method-getSubmitData\" rel=\"Ext.form.field.Date-method-getSubmitData\" class=\"docClass\">submitted parameters</a>. If an empty string is\nreturned then just the name= will be submitted; if null is returned then nothing will be submitted.</p>\n\n<p>Note that the value returned will have been <a href=\"#!/api/Ext.form.field.Date-method-processRawValue\" rel=\"Ext.form.field.Date-method-processRawValue\" class=\"docClass\">processed</a> but may or may not have been\nsuccessfully <a href=\"#!/api/Ext.form.field.Date-method-validate\" rel=\"Ext.form.field.Date-method-validate\" class=\"docClass\">validated</a>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p></p>\n"},"initDisabledDays":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"initValue":{"!type":"fn() -> !this","!doc":"<p>Initializes the field's value based on the initial config.</p>\n"},"onDownArrow":{"!type":"fn(e: ?) -> !this"},"onExpand":{"!type":"fn() -> !this","!doc":"<p>Sets the Date picker's value to match the current field value when expanding.</p>\n"},"onSelect":{"!type":"fn(m: ?, d: ?) -> !this"},"parseDate":{"!type":"fn(value: ?) -> !this"},"rawToValue":{"!type":"fn(rawValue: ?) -> ?","!doc":"<p>Converts a raw input field value into a mixed-type value that is suitable for this particular field type. This\nallows controlling the normalization and conversion of user-entered values into field-type-appropriate values,\ne.g. a Date object for <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, and is invoked by <a href=\"#!/api/Ext.form.field.Date-method-getValue\" rel=\"Ext.form.field.Date-method-getValue\" class=\"docClass\">getValue</a>.</p>\n\n<p>It is up to individual implementations to decide how to handle raw values that cannot be successfully converted\nto the desired object type.</p>\n\n<p>See <a href=\"#!/api/Ext.form.field.Date-method-valueToRaw\" rel=\"Ext.form.field.Date-method-valueToRaw\" class=\"docClass\">valueToRaw</a> for the opposite conversion.</p>\n\n<p>The base implementation does no conversion, returning the raw value untouched.</p>\n"},"safeParse":{"!type":"fn(value: ?, format: ?) -> ?","!doc":"<p>Attempts to parse a given string value using a given date format.</p>\n"},"setDisabledDates":{"!type":"fn(disabledDates: ?) -> !this","!doc":"<p>Replaces any existing disabled dates with new values and refreshes the Date picker.</p>\n"},"setDisabledDays":{"!type":"fn(disabledDays: ?) -> !this","!doc":"<p>Replaces any existing disabled days (by index, 0-6) with new values and refreshes the Date picker.</p>\n"},"setMaxValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Replaces any existing <a href=\"#!/api/Ext.form.field.Date-cfg-maxValue\" rel=\"Ext.form.field.Date-cfg-maxValue\" class=\"docClass\">maxValue</a> with the new value and refreshes the Date picker.</p>\n"},"setMinValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Replaces any existing <a href=\"#!/api/Ext.form.field.Date-cfg-minValue\" rel=\"Ext.form.field.Date-cfg-minValue\" class=\"docClass\">minValue</a> with the new value and refreshes the Date picker.</p>\n"},"setValue":{"!type":"fn(date: ?) -> ?","!doc":"<p>Sets the value of the date field. You can pass a date object or any string that can be parsed into a valid date,\nusing <a href=\"#!/api/Ext.form.field.Date-cfg-format\" rel=\"Ext.form.field.Date-cfg-format\" class=\"docClass\">format</a> as the date format, according to the same rules as Ext.Date.parse (the default\nformat used is \"m/d/Y\").</p>\n\n<p>Usage:</p>\n\n<pre><code>//All of these calls set the same date value (May 4, 2006)\n\n//Pass a date object:\nvar dt = new Date('5/4/2006');\ndateField.setValue(dt);\n\n//Pass a date string (default format):\ndateField.setValue('05/04/2006');\n\n//Pass a date string (custom format):\ndateField.format = 'Y-m-d';\ndateField.setValue('2006-05-04');\n</code></pre>\n"},"valueToRaw":{"!type":"fn(value: ?) -> ?","!doc":"<p>Converts a mixed-type value to a raw representation suitable for displaying in the field. This allows controlling\nhow value objects passed to <a href=\"#!/api/Ext.form.field.Date-method-setValue\" rel=\"Ext.form.field.Date-method-setValue\" class=\"docClass\">setValue</a> are shown to the user, including localization. For instance, for a\n<a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, this would control how a Date object passed to <a href=\"#!/api/Ext.form.field.Date-method-setValue\" rel=\"Ext.form.field.Date-method-setValue\" class=\"docClass\">setValue</a> would be converted\nto a String for display in the field.</p>\n\n<p>See <a href=\"#!/api/Ext.form.field.Date-method-rawToValue\" rel=\"Ext.form.field.Date-method-rawToValue\" class=\"docClass\">rawToValue</a> for the opposite conversion.</p>\n\n<p>The base implementation simply does a standard toString conversion, and converts empty values\nto an empty string.</p>\n"}}},"Display":{"!doc":"<p>A display-only text field which is not validated and not submitted. This is useful for when you want to display a\nvalue from a form's <a href=\"#!/api/Ext.form.Basic-method-load\" rel=\"Ext.form.Basic-method-load\" class=\"docClass\">loaded data</a> but do not want to allow the user to edit or submit that\nvalue. The value can be optionally <a href=\"#!/api/Ext.form.field.Display-cfg-htmlEncode\" rel=\"Ext.form.field.Display-cfg-htmlEncode\" class=\"docClass\">HTML encoded</a> if it contains HTML markup that you do not want\nto be rendered.</p>\n\n<p>If you have more complex content, or need to include components within the displayed content, also consider using a\n<a href=\"#!/api/Ext.form.FieldContainer\" rel=\"Ext.form.FieldContainer\" class=\"docClass\">Ext.form.FieldContainer</a> instead.</p>\n\n<p>Example:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    renderTo: Ext.getBody(),\n    width: 175,\n    height: 120,\n    bodyPadding: 10,\n    title: 'Final Score',\n    items: [{\n        xtype: 'displayfield',\n        fieldLabel: 'Home',\n        name: 'home_score',\n        value: '10'\n    }, {\n        xtype: 'displayfield',\n        fieldLabel: 'Visitor',\n        name: 'visitor_score',\n        value: '11'\n    }],\n    buttons: [{\n        text: 'Update'\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"checkChangeBuffer":{"!type":"number"},"checkChangeEvents":{"!type":"number"},"disabled":{"!type":"bool"},"fieldBodyCls":{"!type":"string","!doc":"<p>An extra CSS class to be applied to the body content element in addition to <a href=\"#!/api/Ext.form.field.Display-cfg-baseBodyCls\" rel=\"Ext.form.field.Display-cfg-baseBodyCls\" class=\"docClass\">baseBodyCls</a>.</p>\n"},"fieldCls":{"!type":"string","!doc":"<p>The default CSS class for the field.</p>\n"},"fieldSubTpl":{"!doc":"<p>The content of the field body is defined by this config option.</p>\n"},"htmlEncode":{"!type":"bool","!doc":"<p>True to escape HTML in text when rendering it.</p>\n"},"inputType":{"!type":"string"},"readOnly":{"!type":"bool"},"renderer":{"!doc":"<p>A function to transform the raw value for display in the field. The function will receive 2 arguments, the raw value\nand the <a href=\"#!/api/Ext.form.field.Display\" rel=\"Ext.form.field.Display\" class=\"docClass\">Ext.form.field.Display</a> object.</p>\n"},"scope":{"!doc":"<p>The scope to execute the <a href=\"#!/api/Ext.form.field.Display-cfg-renderer\" rel=\"Ext.form.field.Display-cfg-renderer\" class=\"docClass\">renderer</a> function. Defaults to this.</p>\n"},"submitValue":{"!type":"bool","!doc":"<p>Setting this to false will prevent the field from being <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submitted</a> even when it is\nnot disabled.</p>\n"},"validateOnChange":{"!type":"bool"},"ariaRole":{"!type":"string"},"isValid":{},"validate":{},"getDisplayValue":{"!type":"fn() -> !this","!doc":"<p>Format the value to display.</p>\n"},"getRawValue":{"!type":"fn() -> string","!doc":"<p>Returns the raw value of the field, without performing any normalization, conversion, or validation. To get a\nnormalized and converted value see <a href=\"#!/api/Ext.form.field.Display-method-getValue\" rel=\"Ext.form.field.Display-method-getValue\" class=\"docClass\">getValue</a>.</p>\n"},"getSubTplData":{"!type":"fn() -> ?","!doc":"<p>Creates and returns the data object to be used when rendering the <a href=\"#!/api/Ext.form.field.Display-cfg-fieldSubTpl\" rel=\"Ext.form.field.Display-cfg-fieldSubTpl\" class=\"docClass\">fieldSubTpl</a>.</p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Returns true if the value of this Field has been changed from its <a href=\"#!/api/Ext.form.field.Display-property-originalValue\" rel=\"Ext.form.field.Display-property-originalValue\" class=\"docClass\">originalValue</a>.\nWill always return false if the field is disabled.</p>\n\n<p>Note that if the owning <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">form</a> was configured with\n<a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a> then the <a href=\"#!/api/Ext.form.field.Display-property-originalValue\" rel=\"Ext.form.field.Display-property-originalValue\" class=\"docClass\">originalValue</a> is updated when\nthe values are loaded by <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a>.<a href=\"#!/api/Ext.form.Basic-method-setValues\" rel=\"Ext.form.Basic-method-setValues\" class=\"docClass\">setValues</a>.</p>\n"},"setRawValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Sets the field's raw value directly, bypassing <a href=\"#!/api/Ext.form.field.Display-method-valueToRaw\" rel=\"Ext.form.field.Display-method-valueToRaw\" class=\"docClass\">value conversion</a>, change detection, and\nvalidation. To set the value with these additional inspections see <a href=\"#!/api/Ext.form.field.Display-method-setValue\" rel=\"Ext.form.field.Display-method-setValue\" class=\"docClass\">setValue</a>.</p>\n"},"valueToRaw":{"!type":"fn(value: ?) -> ?","!doc":"<p>Converts a mixed-type value to a raw representation suitable for displaying in the field. This allows controlling\nhow value objects passed to <a href=\"#!/api/Ext.form.field.Display-method-setValue\" rel=\"Ext.form.field.Display-method-setValue\" class=\"docClass\">setValue</a> are shown to the user, including localization. For instance, for a\n<a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, this would control how a Date object passed to <a href=\"#!/api/Ext.form.field.Display-method-setValue\" rel=\"Ext.form.field.Display-method-setValue\" class=\"docClass\">setValue</a> would be converted\nto a String for display in the field.</p>\n\n<p>See <a href=\"#!/api/Ext.form.field.Display-method-rawToValue\" rel=\"Ext.form.field.Display-method-rawToValue\" class=\"docClass\">rawToValue</a> for the opposite conversion.</p>\n\n<p>The base implementation simply does a standard toString conversion, and converts empty values\nto an empty string.</p>\n"}}},"File":{"!doc":"<p>A file upload field which has custom styling and allows control over the button text and other\nfeatures of <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">text fields</a> like <a href=\"#!/api/Ext.form.field.Text-cfg-emptyText\" rel=\"Ext.form.field.Text-cfg-emptyText\" class=\"docClass\">empty text</a>.\nIt uses a hidden file input element behind the scenes to allow user selection of a file and to\nperform the actual upload during <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">form submit</a>.</p>\n\n<p>Because there is no secure cross-browser way to programmatically set the value of a file input,\nthe standard Field <code>setValue</code> method is not implemented. The <code><a href=\"#!/api/Ext.form.field.File-method-getValue\" rel=\"Ext.form.field.File-method-getValue\" class=\"docClass\">getValue</a></code> method will return\na value that is browser-dependent; some have just the file name, some have a full path, some use\na fake path.</p>\n\n<p><strong>IMPORTANT:</strong> File uploads are not performed using normal 'Ajax' techniques; see the description for\n<a href=\"#!/api/Ext.form.Basic-method-hasUpload\" rel=\"Ext.form.Basic-method-hasUpload\" class=\"docClass\">Ext.form.Basic.hasUpload</a> for details.</p>\n\n<h1>Example Usage</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'Upload a Photo',\n    width: 400,\n    bodyPadding: 10,\n    frame: true,\n    renderTo: Ext.getBody(),\n    items: [{\n        xtype: 'filefield',\n        name: 'photo',\n        fieldLabel: 'Photo',\n        labelWidth: 50,\n        msgTarget: 'side',\n        allowBlank: false,\n        anchor: '100%',\n        buttonText: 'Select Photo...'\n    }],\n\n    buttons: [{\n        text: 'Upload',\n        handler: function() {\n            var form = this.up('form').getForm();\n            if(form.isValid()){\n                form.submit({\n                    url: 'photo-upload.php',\n                    waitMsg: 'Uploading your photo...',\n                    success: function(fp, o) {\n                        <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Success', 'Your photo \"' + o.result.file + '\" has been uploaded.');\n                    }\n                });\n            }\n        }\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"buttonConfig":{"!doc":"<p>Specify optional custom button <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a> config (eg. iconCls, text) for the upload button</p>\n"},"buttonMargin":{"!type":"number","!doc":"<p>The number of pixels of space reserved between the button and the text field. Note that this only\napplies if <a href=\"#!/api/Ext.form.field.File-cfg-buttonOnly\" rel=\"Ext.form.field.File-cfg-buttonOnly\" class=\"docClass\">buttonOnly</a> = false.</p>\n"},"buttonOnly":{"!type":"bool","!doc":"<p>True to display the file upload field as a button with no visible text field. If true, all\ninherited Text members will still be available.</p>\n"},"buttonText":{"!type":"string","!doc":"<p>The button text to display on the upload button. Note that if you supply a value for\n<a href=\"#!/api/Ext.form.field.File-cfg-buttonConfig\" rel=\"Ext.form.field.File-cfg-buttonConfig\" class=\"docClass\">buttonConfig</a>, the buttonConfig.text value will be used instead if available.</p>\n"},"childEls":{"!doc":"<p>Extract the file element, button outer element, and button active element.</p>\n"},"clearOnSubmit":{"!type":"bool","!doc":"<p>True to clear the selected file value when the form this field belongs to\nis submitted to the server.</p>\n"},"editable":{"!type":"bool","!doc":"<p>false to prevent the user from typing text directly into the field; the field can\nonly have its value set programmatically or via an action invoked by a trigger.</p>\n"},"readOnly":{"!type":"bool","!doc":"<p>Unlike with other form fields, the readOnly config defaults to true in File field.</p>\n"},"submitValue":{"!type":"bool","!doc":"<p>Setting this to false will prevent the field from being <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submitted</a> even when it is\nnot disabled.</p>\n"},"triggers":{"!doc":"<p><a href=\"#!/api/Ext.form.trigger.Trigger\" rel=\"Ext.form.trigger.Trigger\" class=\"docClass\">Triggers</a> to use in this field.  The keys in\nthis object are unique identifiers for the triggers. The values in this object\nare <a href=\"#!/api/Ext.form.trigger.Trigger\" rel=\"Ext.form.trigger.Trigger\" class=\"docClass\">Trigger</a> configuration objects.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n    renderTo: Ext.getBody(),\n    fieldLabel: 'My Custom Field',\n    triggers: {\n        foo: {\n            cls: 'my-foo-trigger',\n            handler: function() {\n                console.log('foo trigger clicked');\n            }\n        },\n        bar: {\n            cls: 'my-bar-trigger',\n            handler: function() {\n                console.log('bar trigger clicked');\n            }\n        }\n    }\n});\n</code></pre>\n"},"button":{"!doc":"<p>A reference to the trigger Button component created for this upload field. Only populated after this component is\nrendered.</p>\n"},"extraFieldBodyCls":{"!type":"string","!doc":"<p>private</p>\n"},"fileInputEl":{"!doc":"<p>A reference to the invisible file input element created for this upload field. Only populated after this\ncomponent is rendered.</p>\n"},"inputCls":{"!type":"string","!doc":"<p>private</p>\n"},"triggerNoEditCls":{"!type":"string","!doc":"<p>Do not show hand pointer over text field since file choose dialog is only shown when clicking in the button</p>\n"},"applyTriggers":{"!type":"fn(triggers: ?) -> !this"},"didValueChange":{"!type":"fn() -> !this","!doc":"<p>Checks if the value has changed. Allows subclasses to override for\nany more complex logic.</p>\n"},"extractFileInput":{"!type":"fn() -> ?","!doc":"<p>Only relevant if the instance's <a href=\"#!/api/Ext.form.field.File-method-isFileUpload\" rel=\"Ext.form.field.File-method-isFileUpload\" class=\"docClass\">isFileUpload</a> method returns true. Returns a reference to the file input\nDOM element holding the user's selected file. The input will be appended into the submission form and will not be\nreturned, so this method should also create a replacement.</p>\n"},"getButtonMarginProp":{"!type":"fn() -> !this"},"getTriggerMarkup":{"!type":"fn() -> !this","!doc":"<p>Gets the markup to be inserted into the subTplMarkup.</p>\n"},"isFileUpload":{"!type":"fn() -> bool","!doc":"<p>Returns whether this Field is a file upload field; if it returns true, forms will use special techniques for\n<a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submitting the form</a> via AJAX. See <a href=\"#!/api/Ext.form.Basic-method-hasUpload\" rel=\"Ext.form.Basic-method-hasUpload\" class=\"docClass\">Ext.form.Basic.hasUpload</a> for details. If\nthis returns true, the <a href=\"#!/api/Ext.form.field.File-method-extractFileInput\" rel=\"Ext.form.field.File-method-extractFileInput\" class=\"docClass\">extractFileInput</a> method must also be implemented to return the corresponding file\ninput element.</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onDisable":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onEnable":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onFileChange":{"!type":"fn(button: ?, e: ?, value: ?) -> !this","!doc":"<p>Event handler fired when the user selects a file.</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onShow":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the show operation. After\ncalling the superclass's onShow, the Component will be visible.</p>\n\n<p>Override in subclasses where more complex behaviour is needed.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.form.field.File-event-show\" rel=\"Ext.form.field.File-event-show\" class=\"docClass\">show</a>.</p>\n"},"reset":{"!type":"fn() -> !this","!doc":"<p>Resets the current field value to the originally-loaded value and clears any validation messages.\nAlso adds <strong><a href=\"#!/api/Ext.form.field.File-cfg-emptyText\" rel=\"Ext.form.field.File-cfg-emptyText\" class=\"docClass\">emptyText</a></strong> and <strong><a href=\"#!/api/Ext.form.field.File-cfg-emptyCls\" rel=\"Ext.form.field.File-cfg-emptyCls\" class=\"docClass\">emptyCls</a></strong> if the original value was blank.</p>\n"},"restoreInput":{"!type":"fn(el: ?) -> !this"},"setValue":{"!type":"fn() -> !this","!doc":"<p>Overridden to do nothing</p>\n"},"change":{"!type":"fn(this: ?, value: ?, eOpts: ?)","!doc":"<p>Fires when the underlying file input field's value has changed from the user selecting a new file from the system\nfile selection dialog.</p>\n"}}},"FileButton":{"!type":"fn(config: ?)","prototype":{"autoEl":{"!doc":"<p>A tag name or DomHelper spec used to create the <a href=\"#!/api/Ext.form.field.FileButton-method-getEl\" rel=\"Ext.form.field.FileButton-method-getEl\" class=\"docClass\">Element</a> which will\nencapsulate this Component.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong>'div'</strong>. The more complex Sencha classes use a more\ncomplex DOM structure specified by their own <a href=\"#!/api/Ext.form.field.FileButton-cfg-renderTpl\" rel=\"Ext.form.field.FileButton-cfg-renderTpl\" class=\"docClass\">renderTpl</a>s.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components encapsulated by\ndifferent DOM elements. Example usage:</p>\n\n<pre><code>{\n    xtype: 'component',\n    autoEl: {\n        tag: 'img',\n        src: 'http://www.example.com/example.jpg'\n    }\n}, {\n    xtype: 'component',\n    autoEl: {\n        tag: 'blockquote',\n        html: 'autoEl is cool!'\n    }\n}, {\n    xtype: 'container',\n    autoEl: 'ul',\n    cls: 'ux-unordered-list',\n    items: {\n        xtype: 'component',\n        autoEl: 'li',\n        html: 'First list item'\n    }\n}\n</code></pre>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"cls":{"!type":"string","!doc":"<p>A CSS class string to apply to the button's main element.</p>\n"},"preventDefault":{"!type":"bool","!doc":"<p>True to prevent the default action when the <a href=\"#!/api/Ext.form.field.FileButton-cfg-clickEvent\" rel=\"Ext.form.field.FileButton-cfg-clickEvent\" class=\"docClass\">clickEvent</a> is processed.</p>\n"},"afterTpl":{"!type":"string"},"inputCls":{"!type":"string"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"createFileInput":{"!type":"fn(isTemporary: ?) -> !this","!doc":"<p>Creates the file input element. It is inserted into the trigger button component, made\ninvisible, and floated on top of the button's other content so that it will receive the\nbutton's clicks.</p>\n"},"fireChange":{"!type":"fn(e: ?) -> !this"},"getAfterMarkup":{"!type":"fn(values: ?) -> !this","!doc":"<p>private</p>\n"},"getTemplateArgs":{"!type":"fn() -> ?","!doc":"<p>This method returns an object which provides substitution parameters for the <a href=\"#!/api/Ext.form.field.FileButton-cfg-renderTpl\" rel=\"Ext.form.field.FileButton-cfg-renderTpl\" class=\"docClass\">XTemplate</a> used to\ncreate this Button's DOM structure.</p>\n\n<p>Instances or subclasses which use a different Template to create a different DOM structure may need to provide\ntheir own implementation of this method.</p>\n"},"onDisable":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the disable operation.\nAfter calling the superclass's <code>onDisable</code>, the Component will be disabled.</p>\n"},"onEnable":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the enable operation.\nAfter calling the superclass's <code>onEnable</code>, the Component will be enabled.</p>\n"},"reset":{"!type":"fn(remove: ?) -> !this"},"restoreInput":{"!type":"fn(el: ?) -> !this"}}},"Hidden":{"!doc":"<p>A basic hidden field for storing hidden values in forms that need to be passed in the form submit.</p>\n\n<p>This creates an actual input element with type=\"hidden\" in the DOM. While its label is\n<a href=\"#!/api/Ext.form.field.Hidden-cfg-hideLabel\" rel=\"Ext.form.field.Hidden-cfg-hideLabel\" class=\"docClass\">not rendered</a> by default, it is still a real component and may be sized according\nto its owner container's layout.</p>\n\n<p>Because of this, in most cases it is more convenient and less problematic to simply\n<a href=\"#!/api/Ext.form.action.Action-cfg-params\" rel=\"Ext.form.action.Action-cfg-params\" class=\"docClass\">pass hidden parameters</a> directly when\n<a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submitting the form</a>.</p>\n\n<p>Example:</p>\n\n<pre class='inline-example '><code>new <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>({\n    title: 'My Form',\n    items: [{\n        xtype: 'textfield',\n        fieldLabel: 'Text Field',\n        name: 'text_field',\n        value: 'value from text field'\n    }, {\n        xtype: 'hiddenfield',\n        name: 'hidden_field_1',\n        value: 'value from hidden field'\n    }],\n\n    buttons: [{\n        text: 'Submit',\n        handler: function() {\n            this.up('form').getForm().submit({\n                params: {\n                    hidden_field_2: 'value from submit call'\n                }\n            });\n        }\n    }]\n});\n</code></pre>\n\n<p>Submitting the above form will result in three values sent to the server:</p>\n\n<pre><code>text_field=value+from+text+field&amp;hidden;_field_1=value+from+hidden+field&amp;hidden_field_2=value+from+submit+call\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"hidden":{"!type":"bool","!doc":"<p><code>true</code> to hide the component.</p>\n"},"hideLabel":{"!type":"bool","!doc":"<p>Set to true to completely hide the label element (<a href=\"#!/api/Ext.form.field.Hidden-cfg-fieldLabel\" rel=\"Ext.form.field.Hidden-cfg-fieldLabel\" class=\"docClass\">fieldLabel</a> and <a href=\"#!/api/Ext.form.field.Hidden-cfg-labelSeparator\" rel=\"Ext.form.field.Hidden-cfg-labelSeparator\" class=\"docClass\">labelSeparator</a>). Also see\n<a href=\"#!/api/Ext.form.field.Hidden-cfg-hideEmptyLabel\" rel=\"Ext.form.field.Hidden-cfg-hideEmptyLabel\" class=\"docClass\">hideEmptyLabel</a>, which controls whether space will be reserved for an empty fieldLabel.</p>\n"},"inputType":{"!type":"string","!doc":"<p>private</p>\n"},"ariaRole":{"!type":"string"},"clearInvalid":{"!type":"fn() -> !this","!doc":"<p>Clear any invalid styles/messages for this field.</p>\n\n<p><strong>Note</strong>: this method does not cause the Field's <a href=\"#!/api/Ext.form.field.Hidden-method-validate\" rel=\"Ext.form.field.Hidden-method-validate\" class=\"docClass\">validate</a> or <a href=\"#!/api/Ext.form.field.Hidden-method-isValid\" rel=\"Ext.form.field.Hidden-method-isValid\" class=\"docClass\">isValid</a> methods to return <code>true</code>\nif the value does not <em>pass</em> validation. So simply clearing a field's errors will not necessarily allow\nsubmission of forms submitted with the <a href=\"#!/api/Ext.form.action.Submit-cfg-clientValidation\" rel=\"Ext.form.action.Submit-cfg-clientValidation\" class=\"docClass\">Ext.form.action.Submit.clientValidation</a> option set.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>These are all private overrides</p>\n"},"isEqual":{"!type":"fn(value1: ?, value2: ?) -> !this","!doc":"<p>Override. Treat undefined and null values as equal to an empty string value.</p>\n"},"markInvalid":{"!type":"fn() -> !this","!doc":"<p>Display one or more error messages associated with this field, using <a href=\"#!/api/Ext.form.field.Hidden-cfg-msgTarget\" rel=\"Ext.form.field.Hidden-cfg-msgTarget\" class=\"docClass\">msgTarget</a> to determine how to\ndisplay the messages and applying <a href=\"#!/api/Ext.form.field.Hidden-cfg-invalidCls\" rel=\"Ext.form.field.Hidden-cfg-invalidCls\" class=\"docClass\">invalidCls</a> to the field's UI element.</p>\n\n<p><strong>Note</strong>: this method does not cause the Field's <a href=\"#!/api/Ext.form.field.Hidden-method-validate\" rel=\"Ext.form.field.Hidden-method-validate\" class=\"docClass\">validate</a> or <a href=\"#!/api/Ext.form.field.Hidden-method-isValid\" rel=\"Ext.form.field.Hidden-method-isValid\" class=\"docClass\">isValid</a> methods to return <code>false</code>\nif the value does <em>pass</em> validation. So simply marking a Field as invalid will not prevent submission of forms\nsubmitted with the <a href=\"#!/api/Ext.form.action.Submit-cfg-clientValidation\" rel=\"Ext.form.action.Submit-cfg-clientValidation\" class=\"docClass\">Ext.form.action.Submit.clientValidation</a> option set.</p>\n"},"setHeight":{"!type":"fn() -> ?","!doc":"<p>Sets the height of the component. This method fires the <a href=\"#!/api/Ext.form.field.Hidden-event-resize\" rel=\"Ext.form.field.Hidden-event-resize\" class=\"docClass\">resize</a> event.</p>\n"},"setPagePosition":{"!type":"fn() -> ?","!doc":"<p>Sets the page XY position of the component. To set the left and top instead, use <a href=\"#!/api/Ext.form.field.Hidden-method-setPosition\" rel=\"Ext.form.field.Hidden-method-setPosition\" class=\"docClass\">setPosition</a>.\nThis method fires the <a href=\"#!/api/Ext.form.field.Hidden-event-move\" rel=\"Ext.form.field.Hidden-event-move\" class=\"docClass\">move</a> event.</p>\n"},"setPosition":{"!type":"fn() -> ?","!doc":"<p>Sets the left and top of the component. To set the page XY position instead, use <a href=\"#!/api/Ext.Component-method-setPagePosition\" rel=\"Ext.Component-method-setPagePosition\" class=\"docClass\">setPagePosition</a>. This\nmethod fires the <a href=\"#!/api/Ext.form.field.Hidden-event-move\" rel=\"Ext.form.field.Hidden-event-move\" class=\"docClass\">move</a> event.</p>\n"},"setSize":{"!type":"fn() -> ?","!doc":"<p>Sets the width and height of this Component. This method fires the <a href=\"#!/api/Ext.form.field.Hidden-event-resize\" rel=\"Ext.form.field.Hidden-event-resize\" class=\"docClass\">resize</a> event. This method can accept\neither width and height as separate arguments, or you can pass a size object like <code>{width:10, height:20}</code>.</p>\n"},"setWidth":{"!type":"fn() -> ?","!doc":"<p>Sets the width of the component. This method fires the <a href=\"#!/api/Ext.form.field.Hidden-event-resize\" rel=\"Ext.form.field.Hidden-event-resize\" class=\"docClass\">resize</a> event.</p>\n"}}},"HtmlEditor":{"!doc":"<p>Provides a lightweight HTML Editor component. Some toolbar features are not supported by Safari and will be\nautomatically hidden when needed. These are noted in the config options where appropriate.</p>\n\n<p>The editor's toolbar buttons have tooltips defined in the <a href=\"#!/api/Ext.form.field.HtmlEditor-property-buttonTips\" rel=\"Ext.form.field.HtmlEditor-property-buttonTips\" class=\"docClass\">buttonTips</a> property, but they are not\nenabled by default unless the global <a href=\"#!/api/Ext.tip.QuickTipManager\" rel=\"Ext.tip.QuickTipManager\" class=\"docClass\">Ext.tip.QuickTipManager</a> singleton is\n<a href=\"#!/api/Ext.tip.QuickTipManager-method-init\" rel=\"Ext.tip.QuickTipManager-method-init\" class=\"docClass\">initialized</a>.</p>\n\n<p>An Editor is a sensitive component that can't be used in all spots standard fields can be used. Putting an\nEditor within any element that has display set to 'none' can cause problems in Safari and Firefox due to their\ndefault iframe reloading bugs.</p>\n\n<h1>Example usage</h1>\n\n<p>Simple example rendered with default options:</p>\n\n<pre class='inline-example '><code><a href=\"#!/api/Ext.tip.QuickTipManager-method-init\" rel=\"Ext.tip.QuickTipManager-method-init\" class=\"docClass\">Ext.tip.QuickTipManager.init</a>();  // enable tooltips\nExt.create('<a href=\"#!/api/Ext.form.field.HtmlEditor\" rel=\"Ext.form.field.HtmlEditor\" class=\"docClass\">Ext.form.HtmlEditor</a>', {\n    width: 580,\n    height: 250,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>Passed via xtype into a container and with custom options:</p>\n\n<pre class='inline-example '><code><a href=\"#!/api/Ext.tip.QuickTipManager-method-init\" rel=\"Ext.tip.QuickTipManager-method-init\" class=\"docClass\">Ext.tip.QuickTipManager.init</a>();  // enable tooltips\nnew <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({\n    title: 'HTML Editor',\n    renderTo: Ext.getBody(),\n    width: 550,\n    height: 250,\n    frame: true,\n    layout: 'fit',\n    items: {\n        xtype: 'htmleditor',\n        enableColors: false,\n        enableAlignments: false\n    }\n});\n</code></pre>\n\n<h1>Reflow issues</h1>\n\n<p>In some browsers, a layout reflow will cause the underlying editor iframe to be reset. This\nis most commonly seen when using the editor in collapsed panels with animation. In these cases\nit is best to avoid animation. More information can be found here: https://bugzilla.mozilla.org/show_bug.cgi?id=90268</p>\n","!type":"fn(config: ?)","prototype":{"afterIFrameTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nafter the iframe element. If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">subTpl data</a> serves as the context.</p>\n"},"afterTextAreaTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nafter the textarea element. If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">subTpl data</a> serves as the context.</p>\n"},"allowDomMove":{"!type":"bool"},"autoCreate":{"!type":"string"},"beforeIFrameTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nbefore the iframe element. If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">subTpl data</a> serves as the context.</p>\n"},"beforeTextAreaTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\nbefore the textarea element. If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">subTpl data</a> serves as the context.</p>\n"},"componentLayout":{"!doc":"<p>The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout\nmanager which sizes a Component's internal structure in response to the Component being sized.</p>\n\n<p>Generally, developers will not use this configuration as all provided Components which need their internal\nelements sizing (Such as <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">input fields</a>) come with their own componentLayout managers.</p>\n\n<p>The <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used on instances of the base <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>\nclass which simply sizes the Component's encapsulating element to the height and width specified in the\n<a href=\"#!/api/Ext.form.field.HtmlEditor-method-setSize\" rel=\"Ext.form.field.HtmlEditor-method-setSize\" class=\"docClass\">setSize</a> method.</p>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"createLinkText":{"!type":"string","!doc":"<p>The default text for the create link prompt</p>\n"},"defaultButtonUI":{"!type":"string","!doc":"<p>A default <a href=\"#!/api/Ext.Component-cfg-ui\" rel=\"Ext.Component-cfg-ui\" class=\"docClass\">ui</a> to use for the HtmlEditor's toolbar\n<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Buttons</a></p>\n"},"defaultLinkValue":{"!type":"string","!doc":"<p>The default value for the create link prompt</p>\n"},"defaultValue":{"!type":"string","!doc":"<p>A default value to be put into the editor to resolve focus issues.</p>\n\n<p>Defaults to (Non-breaking space) in Opera,\n(Zero-width space) in all other browsers.</p>\n"},"enableAlignments":{"!type":"bool","!doc":"<p>Enable the left, center, right alignment buttons</p>\n"},"enableColors":{"!type":"bool","!doc":"<p>Enable the fore/highlight color buttons</p>\n"},"enableFont":{"!type":"bool","!doc":"<p>Enable font selection. Not available in Safari 2.</p>\n"},"enableFontSize":{"!type":"bool","!doc":"<p>Enable the increase/decrease font size buttons</p>\n"},"enableFormat":{"!type":"bool","!doc":"<p>Enable the bold, italic and underline buttons</p>\n"},"enableLinks":{"!type":"bool","!doc":"<p>Enable the create link button. Not available in Safari 2.</p>\n"},"enableLists":{"!type":"bool","!doc":"<p>Enable the bullet and numbered list buttons. Not available in Safari 2.</p>\n"},"enableSourceEdit":{"!type":"bool","!doc":"<p>Enable the switch to source edit button. Not available in Safari 2.</p>\n"},"fieldCls":{"!type":"string"},"focusCls":{"!type":"string"},"fontFamilies":{"!doc":"<p>An array of available font families</p>\n"},"hideMode":{"!type":"string","!doc":"<p>A String which specifies how this Component's encapsulating DOM element will be hidden. Values may be:</p>\n\n<ul>\n<li><code>'display'</code> : The Component will be hidden using the <code>display: none</code> style.</li>\n<li><code>'visibility'</code> : The Component will be hidden using the <code>visibility: hidden</code> style.</li>\n<li><code>'offsets'</code> : The Component will be hidden by absolutely positioning it out of the visible area of the document.\nThis is useful when a hidden Component must maintain measurable dimensions. Hiding using <code>display</code> results in a\nComponent having zero dimensions.</li>\n</ul>\n\n"},"iframeAttrTpl":{"!doc":"<p>An optional string or <code>XTemplate</code> configuration to insert in the field markup\ninside the iframe element (as attributes). If an <code>XTemplate</code> is used, the component's\n<a href=\"#!/api/Ext.form.field.Base-method-getSubTplData\" rel=\"Ext.form.field.Base-method-getSubTplData\" class=\"docClass\">subTpl data</a> serves as the context.</p>\n"},"inputType":{"!type":"string"},"invalidCls":{"!type":"string"},"invalidText":{"!type":"string"},"readOnly":{"!type":"string"},"tabIndex":{"!type":"string"},"activated":{"!type":"bool"},"buttonTips":{"!doc":"<p>Object collection of toolbar tooltips for the buttons in the editor. The key is the command id associated with\nthat button and the value is a valid QuickTips object. For example:</p>\n\n<pre><code>{\n    bold: {\n        title: 'Bold (Ctrl+B)',\n        text: 'Make the selected text bold.',\n        cls: 'x-html-editor-tip'\n    },\n    italic: {\n        title: 'Italic (Ctrl+I)',\n        text: 'Make the selected text italic.',\n        cls: 'x-html-editor-tip'\n    }\n    // ...\n}\n</code></pre>\n"},"componentTpl":{},"containerElCls":{"!type":"string"},"extraFieldBodyCls":{"!type":"string","!doc":"<p>private</p>\n"},"fixKeys":{},"fixKeysAfter":{},"iframePad":{"!type":"number"},"initialized":{"!type":"bool"},"maskOnDisable":{"!type":"bool","!doc":"<p>This is an internal flag that you use when creating custom components. By default this is set to <code>true</code> which means\nthat every component gets a mask when it's disabled. Components like FieldContainer, FieldSet, Field, Button, Tab\noverride this property to <code>false</code> since they want to implement custom disable logic.</p>\n"},"reStripQuotes":{"!doc":"<p>This will strip any number of single or double quotes (in any order) from a string at the anchors.</p>\n"},"sourceEditMode":{"!type":"bool"},"stretchInputElFixed":{"!type":"bool"},"subTplInsertions":{},"adjustFont":{"!type":"fn(btn: ?) -> !this"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"applyCommand":{"!type":"fn(e: ?) -> !this"},"beforeDestroy":{"!type":"fn() -> !this"},"checkDesignMode":{"!type":"fn() -> !this"},"cleanHtml":{"!type":"fn(html: ?) -> string","!doc":"<p>If you need/want custom HTML cleanup, this is the method you should override.</p>\n"},"clearInvalid":{"!type":"fn() -> !this","!doc":"<p>Clear any invalid styles/messages for this field. Components using this mixin should implement this method to\nupdate the components rendering to clear any existing messages.</p>\n\n<p><strong>Note</strong>: this method does not cause the Field's <a href=\"#!/api/Ext.form.field.HtmlEditor-method-validate\" rel=\"Ext.form.field.HtmlEditor-method-validate\" class=\"docClass\">validate</a> or <a href=\"#!/api/Ext.form.field.HtmlEditor-method-isValid\" rel=\"Ext.form.field.HtmlEditor-method-isValid\" class=\"docClass\">isValid</a> methods to return <code>true</code>\nif the value does not <em>pass</em> validation. So simply clearing a field's errors will not necessarily allow\nsubmission of forms submitted with the <a href=\"#!/api/Ext.form.action.Submit-cfg-clientValidation\" rel=\"Ext.form.action.Submit-cfg-clientValidation\" class=\"docClass\">Ext.form.action.Submit.clientValidation</a> option set.</p>\n"},"createInputCmp":{"!type":"fn() -> !this"},"createLink":{"!type":"fn() -> !this","!doc":"<p>used internally</p>\n"},"createToolbar":{"!type":"fn(editor: ?) -> !this","!doc":"<p>Called when the editor creates its toolbar. Override this method if you need to\nadd custom toolbar buttons.</p>\n"},"disableItems":{"!type":"fn(disabled: ?) -> !this"},"execCmd":{"!type":"fn(cmd: ?, value?: ?) -> !this","!doc":"<p>Executes a Midas editor command directly on the editor document. For visual commands, you should use\n<a href=\"#!/api/Ext.form.field.HtmlEditor-method-relayCmd\" rel=\"Ext.form.field.HtmlEditor-method-relayCmd\" class=\"docClass\">relayCmd</a> instead. <strong>This should only be called after the editor is initialized.</strong></p>\n"},"focus":{"!type":"fn(eOpts: ?)"},"getDesignMode":{"!type":"fn() -> !this"},"getDoc":{"!type":"fn() -> !this"},"getDocMarkup":{"!type":"fn() -> !this","!doc":"<p>Called when the editor initializes the iframe with HTML contents. Override this method if you\nwant to change the initialization markup of the iframe (e.g. to add stylesheets).</p>\n\n<p><strong>Note:</strong> IE8-Standards has unwanted scroller behavior, so the default meta tag forces IE7 compatibility.\nAlso note that forcing IE7 mode works when the page is loaded normally, but if you are using IE's Web\nDeveloper Tools to manually set the document mode, that will take precedence and override what this\ncode sets by default. This can be confusing when developing, but is not a user-facing issue.</p>\n"},"getEditorBody":{"!type":"fn() -> !this"},"getInputCmpCfg":{"!type":"fn() -> !this"},"getMaskTarget":{"!type":"fn() -> !this","!doc":"<p>Returns the element which is masked by the <a href=\"#!/api/Ext.form.field.HtmlEditor-method-mask\" rel=\"Ext.form.field.HtmlEditor-method-mask\" class=\"docClass\">mask</a> method, or into which the <a href=\"#!/api/Ext.form.field.HtmlEditor-method-setLoading\" rel=\"Ext.form.field.HtmlEditor-method-setLoading\" class=\"docClass\">LoadMask</a> is rendered into.</p>\n\n<p>The default implementation uses the <a href=\"#!/api/Ext.form.field.HtmlEditor-cfg-maskElement\" rel=\"Ext.form.field.HtmlEditor-cfg-maskElement\" class=\"docClass\">maskElement</a> configuration to access the Component's child element by name. By default, <a href=\"#!/api/Ext.form.field.HtmlEditor-cfg-maskElement\" rel=\"Ext.form.field.HtmlEditor-cfg-maskElement\" class=\"docClass\">maskElement</a>\nis <code>null</code> which means that <code>null</code> is returned from this method indicating that the mask needs to be rendered into the document because\ncomponent structure should not be contaminated by mask elements.</p>\n\n<p>Some subclasses may override this method if they have knowledge about external structures where a mask could usefully be rendered.</p>\n\n<p>For example a <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">GridView</a> will request that its owning <a href=\"#!/api/Ext.panel.Table\" rel=\"Ext.panel.Table\" class=\"docClass\">GridPanel</a> be masked. The\nGridPanel will have its own implementation of <code>getMaskTarget</code> which will return the element dicated by its own <a href=\"#!/api/Ext.form.field.HtmlEditor-cfg-maskElement\" rel=\"Ext.form.field.HtmlEditor-cfg-maskElement\" class=\"docClass\">maskElement</a>\nPanels use <code>\"el\"</code> as their <a href=\"#!/api/Ext.form.field.HtmlEditor-cfg-maskElement\" rel=\"Ext.form.field.HtmlEditor-cfg-maskElement\" class=\"docClass\">maskElement</a> by default, but that could be overridden to be <code>\"body\"</code> to leave toolbars and the header\nmouse-accessible.</p>\n"},"getToolbar":{"!type":"fn() -> ?","!doc":"<p>Returns the editor's toolbar. <strong>This is only available after the editor has been rendered.</strong></p>\n"},"getToolbarCfg":{"!type":"fn() -> !this"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Returns the current data value of the field. The type of value returned is particular to the type of the\nparticular field (e.g. a Date object for <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>).</p>\n"},"getWin":{"!type":"fn() -> !this"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initDefaultFont":{"!type":"fn() -> !this"},"initEditor":{"!type":"fn() -> !this"},"initFrameDoc":{"!type":"fn() -> !this"},"insertAtCursor":{"!type":"fn(text: ?) -> !this","!doc":"<p>Inserts the passed text at the current cursor position.\n<strong>Note:</strong> the editor must be initialized and activated to insert text.</p>\n"},"isEqual":{"!type":"fn(value1: ?, value2: ?) -> bool","!doc":"<p>Returns whether two field <a href=\"#!/api/Ext.form.field.HtmlEditor-method-getValue\" rel=\"Ext.form.field.HtmlEditor-method-getValue\" class=\"docClass\">values</a> are logically equal. Field implementations may override this\nto provide custom comparison logic appropriate for the particular field's data type.</p>\n"},"onEditorEvent":{"!type":"fn() -> !this"},"onFirstFocus":{"!type":"fn() -> !this"},"onRelayedEvent":{"!type":"fn(event: ?) -> !this"},"pushValue":{"!type":"fn() -> !this","!doc":"<p>Pushes the value of the textarea into the iframe editor.</p>\n"},"relayBtnCmd":{"!type":"fn(btn: ?) -> !this"},"relayCmd":{"!type":"fn(cmd: ?, value?: ?) -> !this","!doc":"<p>Executes a Midas editor command on the editor document and performs necessary focus and toolbar updates.\n<strong>This should only be called after the editor is initialized.</strong></p>\n"},"setDesignMode":{"!type":"fn(mode: ?) -> !this","!doc":"<p>Sets current design mode. To enable, mode can be true or 'on', off otherwise</p>\n"},"setReadOnly":{"!type":"fn(readOnly: ?) -> !this","!doc":"<p>Sets the read only state of this field.</p>\n"},"setValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Sets a data value into the field and runs the change detection and validation.</p>\n"},"syncValue":{"!type":"fn() -> !this","!doc":"<p>Syncs the contents of the editor iframe with the textarea.</p>\n"},"toggleSourceEdit":{"!type":"fn(sourceEditMode?: ?) -> !this","!doc":"<p>Toggles the editor between standard and source edit mode.</p>\n"},"updateToolbar":{"!type":"fn() -> !this","!doc":"<p>Triggers a toolbar update by reading the markup state of the current selection in the editor.</p>\n"},"validate":{"!type":"fn()"},"activate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when the editor is first receives the focus. Any insertion must wait until after this event.</p>\n"},"beforepush":{"!type":"fn(this: ?, html: ?, eOpts: ?)","!doc":"<p>Fires before the iframe editor is updated with content from the textarea. Return false to cancel the\npush.</p>\n"},"beforesync":{"!type":"fn(this: ?, html: ?, eOpts: ?)","!doc":"<p>Fires before the textarea is updated with content from the editor iframe. Return false to cancel the\nsync.</p>\n"},"blur":{"!type":"fn(eOpts: ?)"},"editmodechange":{"!type":"fn(this: ?, sourceEdit: ?, eOpts: ?)","!doc":"<p>Fires when the editor switches edit modes</p>\n"},"initialize":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when the editor is fully initialized (including the iframe)</p>\n"},"push":{"!type":"fn(this: ?, html: ?, eOpts: ?)","!doc":"<p>Fires when the iframe editor is updated with content from the textarea.</p>\n"},"specialkey":{"!type":"fn(eOpts: ?)"},"sync":{"!type":"fn(this: ?, html: ?, eOpts: ?)","!doc":"<p>Fires when the textarea is updated with content from the editor iframe.</p>\n"}}},"Number":{"!doc":"<p>A numeric text field that provides automatic keystroke filtering to disallow non-numeric characters,\nand numeric validation to limit the value to a range of valid numbers. The range of acceptable number\nvalues can be controlled by setting the <a href=\"#!/api/Ext.form.field.Number-cfg-minValue\" rel=\"Ext.form.field.Number-cfg-minValue\" class=\"docClass\">minValue</a> and <a href=\"#!/api/Ext.form.field.Number-cfg-maxValue\" rel=\"Ext.form.field.Number-cfg-maxValue\" class=\"docClass\">maxValue</a> configs, and fractional\ndecimals can be disallowed by setting <a href=\"#!/api/Ext.form.field.Number-cfg-allowDecimals\" rel=\"Ext.form.field.Number-cfg-allowDecimals\" class=\"docClass\">allowDecimals</a> to <code>false</code>.</p>\n\n<p>By default, the number field is also rendered with a set of up/down spinner buttons and has\nup/down arrow key and mouse wheel event listeners attached for incrementing/decrementing the value by the\n<a href=\"#!/api/Ext.form.field.Number-cfg-step\" rel=\"Ext.form.field.Number-cfg-step\" class=\"docClass\">step</a> value. To hide the spinner buttons set <code><a href=\"#!/api/Ext.form.field.Number-cfg-hideTrigger\" rel=\"Ext.form.field.Number-cfg-hideTrigger\" class=\"docClass\">hideTrigger</a>:true</code>; to disable\nthe arrow key and mouse wheel handlers set <code><a href=\"#!/api/Ext.form.field.Number-cfg-keyNavEnabled\" rel=\"Ext.form.field.Number-cfg-keyNavEnabled\" class=\"docClass\">keyNavEnabled</a>:false</code> and\n<code><a href=\"#!/api/Ext.form.field.Number-cfg-mouseWheelEnabled\" rel=\"Ext.form.field.Number-cfg-mouseWheelEnabled\" class=\"docClass\">mouseWheelEnabled</a>:false</code>. See the example below.</p>\n\n<h1>Example usage</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'On The Wall',\n    width: 300,\n    bodyPadding: 10,\n    renderTo: Ext.getBody(),\n    items: [{\n        xtype: 'numberfield',\n        anchor: '100%',\n        name: 'bottles',\n        fieldLabel: 'Bottles of Beer',\n        value: 99,\n        maxValue: 99,\n        minValue: 0\n    }],\n    buttons: [{\n        text: 'Take one down, pass it around',\n        handler: function() {\n            this.up('form').down('[name=bottles]').spinDown();\n        }\n    }]\n});\n</code></pre>\n\n<h1>Removing UI Enhancements</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'Personal Info',\n    width: 300,\n    bodyPadding: 10,\n    renderTo: Ext.getBody(),\n    items: [{\n        xtype: 'numberfield',\n        anchor: '100%',\n        name: 'age',\n        fieldLabel: 'Age',\n        minValue: 0, //prevents negative numbers\n\n        // Remove spinner buttons, and arrow key and mouse wheel listeners\n        hideTrigger: true,\n        keyNavEnabled: false,\n        mouseWheelEnabled: false\n    }]\n});\n</code></pre>\n\n<h1>Using Step</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    renderTo: Ext.getBody(),\n    title: 'Step',\n    width: 300,\n    bodyPadding: 10,\n    items: [{\n        xtype: 'numberfield',\n        anchor: '100%',\n        name: 'evens',\n        fieldLabel: 'Even Numbers',\n\n        // Set step so it skips every other number\n        step: 2,\n        value: 0,\n\n        // Add change handler to force user-entered numbers to evens\n        listeners: {\n            change: function(field, value) {\n                value = parseInt(value, 10);\n                field.setValue(value + value % 2);\n            }\n        }\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"allowDecimals":{"!type":"bool","!doc":"<p>False to disallow decimal values</p>\n"},"allowExponential":{"!type":"bool","!doc":"<p>Set to <code>false</code> to disallow Exponential number notation</p>\n"},"autoStripChars":{"!type":"bool","!doc":"<p>True to automatically strip not allowed characters from the field.</p>\n"},"baseChars":{"!type":"string","!doc":"<p>The base set of characters to evaluate as valid numbers.</p>\n"},"decimalPrecision":{"!type":"number","!doc":"<p>The maximum precision to display after the decimal separator</p>\n"},"decimalSeparator":{"!type":"string","!doc":"<p>Character(s) to allow as the decimal separator. Defaults to decimalSeparator.</p>\n"},"maskRe":{},"maxText":{"!type":"string","!doc":"<p>Error text to display if the maximum value validation fails.</p>\n"},"maxValue":{"!type":"number","!doc":"<p>The maximum allowed value. Will be used by the field's validation logic, and for\n<a href=\"#!/api/Ext.form.field.Spinner-method-setSpinUpEnabled\" rel=\"Ext.form.field.Spinner-method-setSpinUpEnabled\" class=\"docClass\">enabling/disabling the up spinner button</a>.</p>\n\n<p>Defaults to Number.MAX_VALUE.</p>\n"},"minText":{"!type":"string","!doc":"<p>Error text to display if the minimum value validation fails.</p>\n"},"minValue":{"!type":"number","!doc":"<p>The minimum allowed value. Will be used by the field's validation logic,\nand for <a href=\"#!/api/Ext.form.field.Spinner-method-setSpinUpEnabled\" rel=\"Ext.form.field.Spinner-method-setSpinUpEnabled\" class=\"docClass\">enabling/disabling the down spinner button</a>.</p>\n\n<p>Defaults to Number.NEGATIVE_INFINITY.</p>\n"},"nanText":{"!type":"string","!doc":"<p>Error text to display if the value is not a valid number. For example, this can happen if a valid character like\n'.' or '-' is left in the field with no number.</p>\n"},"negativeText":{"!type":"string","!doc":"<p>Error text to display if the value is negative and <a href=\"#!/api/Ext.form.field.Number-cfg-minValue\" rel=\"Ext.form.field.Number-cfg-minValue\" class=\"docClass\">minValue</a> is set to 0. This is used instead of the\n<a href=\"#!/api/Ext.form.field.Number-cfg-minText\" rel=\"Ext.form.field.Number-cfg-minText\" class=\"docClass\">minText</a> in that circumstance only.</p>\n"},"step":{"!type":"number","!doc":"<p>Specifies a numeric interval by which the field's value will be incremented or decremented when the user invokes\nthe spinner.</p>\n"},"stripCharsRe":{},"submitLocaleSeparator":{"!type":"bool","!doc":"<p>False to ensure that the <a href=\"#!/api/Ext.form.field.Number-method-getSubmitValue\" rel=\"Ext.form.field.Number-method-getSubmitValue\" class=\"docClass\">getSubmitValue</a> method strips\nalways uses <code>.</code> as the separator, regardless of the <a href=\"#!/api/Ext.form.field.Number-cfg-decimalSeparator\" rel=\"Ext.form.field.Number-cfg-decimalSeparator\" class=\"docClass\">decimalSeparator</a>\nconfiguration.</p>\n"},"beforeBlur":{"!type":"fn() -> !this","!doc":"<p>Template method to do any pre-blur processing.</p>\n"},"fixPrecision":{"!type":"fn(value: ?) -> !this"},"getErrors":{"!type":"fn(value?: ?) -> ?","!doc":"<p>Runs all of Number's validations and returns an array of any errors. Note that this first runs Text's\nvalidations, so the returned array is an amalgamation of all field errors. The additional validations run test\nthat the value is a number, and that it is within the configured min and max values.</p>\n"},"getSubmitValue":{"!type":"fn() -> string","!doc":"<p>Returns the value that would be included in a standard form submit for this field. This will be combined with the\nfield's name to form a name=value pair in the <a href=\"#!/api/Ext.form.field.Number-method-getSubmitData\" rel=\"Ext.form.field.Number-method-getSubmitData\" class=\"docClass\">submitted parameters</a>. If an empty string is\nreturned then just the name= will be submitted; if null is returned then nothing will be submitted.</p>\n\n<p>Note that the value returned will have been <a href=\"#!/api/Ext.form.field.Number-method-processRawValue\" rel=\"Ext.form.field.Number-method-processRawValue\" class=\"docClass\">processed</a> but may or may not have been\nsuccessfully <a href=\"#!/api/Ext.form.field.Number-method-validate\" rel=\"Ext.form.field.Number-method-validate\" class=\"docClass\">validated</a>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onChange":{"!type":"fn() -> !this","!doc":"<p>If grow=true, invoke the autoSize method when the field's value is changed.</p>\n"},"onSpinDown":{"!type":"fn() -> !this","!doc":"<p>This method is called when the spinner down button is clicked, or when the down arrow key is pressed if\n<a href=\"#!/api/Ext.form.field.Number-cfg-keyNavEnabled\" rel=\"Ext.form.field.Number-cfg-keyNavEnabled\" class=\"docClass\">keyNavEnabled</a> is true. Must be implemented by subclasses.</p>\n"},"onSpinUp":{"!type":"fn() -> !this","!doc":"<p>This method is called when the spinner up button is clicked, or when the up arrow key is pressed if\n<a href=\"#!/api/Ext.form.field.Number-cfg-keyNavEnabled\" rel=\"Ext.form.field.Number-cfg-keyNavEnabled\" class=\"docClass\">keyNavEnabled</a> is true. Must be implemented by subclasses.</p>\n"},"parseValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>private</p>\n"},"rawToValue":{"!type":"fn(rawValue: ?) -> ?","!doc":"<p>Converts a raw input field value into a mixed-type value that is suitable for this particular field type. This\nallows controlling the normalization and conversion of user-entered values into field-type-appropriate values,\ne.g. a Date object for <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, and is invoked by <a href=\"#!/api/Ext.form.field.Number-method-getValue\" rel=\"Ext.form.field.Number-method-getValue\" class=\"docClass\">getValue</a>.</p>\n\n<p>It is up to individual implementations to decide how to handle raw values that cannot be successfully converted\nto the desired object type.</p>\n\n<p>See <a href=\"#!/api/Ext.form.field.Number-method-valueToRaw\" rel=\"Ext.form.field.Number-method-valueToRaw\" class=\"docClass\">valueToRaw</a> for the opposite conversion.</p>\n\n<p>The base implementation does no conversion, returning the raw value untouched.</p>\n"},"setMaxValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Replaces any existing <a href=\"#!/api/Ext.form.field.Number-cfg-maxValue\" rel=\"Ext.form.field.Number-cfg-maxValue\" class=\"docClass\">maxValue</a> with the new value.</p>\n"},"setMinValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Replaces any existing <a href=\"#!/api/Ext.form.field.Number-cfg-minValue\" rel=\"Ext.form.field.Number-cfg-minValue\" class=\"docClass\">minValue</a> with the new value.</p>\n"},"setSpinDownEnabled":{"!type":"fn(enabled: ?, internal: ?) -> !this","!doc":"<p>Sets whether the spinner down button is enabled.</p>\n"},"setSpinUpEnabled":{"!type":"fn(enabled: ?, internal: ?) -> !this","!doc":"<p>Sets whether the spinner up button is enabled.</p>\n"},"setSpinValue":{"!type":"fn(value: ?) -> !this"},"toggleSpinners":{"!type":"fn() -> !this"},"valueToRaw":{"!type":"fn(value: ?) -> ?","!doc":"<p>Converts a mixed-type value to a raw representation suitable for displaying in the field. This allows controlling\nhow value objects passed to <a href=\"#!/api/Ext.form.field.Number-method-setValue\" rel=\"Ext.form.field.Number-method-setValue\" class=\"docClass\">setValue</a> are shown to the user, including localization. For instance, for a\n<a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, this would control how a Date object passed to <a href=\"#!/api/Ext.form.field.Number-method-setValue\" rel=\"Ext.form.field.Number-method-setValue\" class=\"docClass\">setValue</a> would be converted\nto a String for display in the field.</p>\n\n<p>See <a href=\"#!/api/Ext.form.field.Number-method-rawToValue\" rel=\"Ext.form.field.Number-method-rawToValue\" class=\"docClass\">rawToValue</a> for the opposite conversion.</p>\n\n<p>The base implementation simply does a standard toString conversion, and converts empty values\nto an empty string.</p>\n"}}},"Spinner":{"prototype":{"keyNavEnabled":{"!type":"bool","!doc":"<p>Specifies whether the up and down arrow keys should trigger spinning up and down. Defaults to true.</p>\n"},"mouseWheelEnabled":{"!type":"bool","!doc":"<p>Specifies whether the mouse wheel should trigger spinning up and down while the field has focus.\nDefaults to true.</p>\n"},"repeatTriggerClick":{"!type":"bool","!doc":"<p>Whether a <a href=\"#!/api/Ext.util.ClickRepeater\" rel=\"Ext.util.ClickRepeater\" class=\"docClass\">click repeater</a> should be attached to the spinner buttons.\nDefaults to true.</p>\n"},"spinDownEnabled":{"!type":"bool","!doc":"<p>Specifies whether the down spinner button is enabled. Defaults to true. To change this after the component is\ncreated, use the <a href=\"#!/api/Ext.form.field.Spinner-method-setSpinDownEnabled\" rel=\"Ext.form.field.Spinner-method-setSpinDownEnabled\" class=\"docClass\">setSpinDownEnabled</a> method.</p>\n"},"spinUpEnabled":{"!type":"bool","!doc":"<p>Specifies whether the up spinner button is enabled. Defaults to true. To change this after the component is\ncreated, use the <a href=\"#!/api/Ext.form.field.Spinner-method-setSpinUpEnabled\" rel=\"Ext.form.field.Spinner-method-setSpinUpEnabled\" class=\"docClass\">setSpinUpEnabled</a> method.</p>\n"},"triggers":{"!doc":"<p><a href=\"#!/api/Ext.form.trigger.Trigger\" rel=\"Ext.form.trigger.Trigger\" class=\"docClass\">Triggers</a> to use in this field.  The keys in\nthis object are unique identifiers for the triggers. The values in this object\nare <a href=\"#!/api/Ext.form.trigger.Trigger\" rel=\"Ext.form.trigger.Trigger\" class=\"docClass\">Trigger</a> configuration objects.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n    renderTo: Ext.getBody(),\n    fieldLabel: 'My Custom Field',\n    triggers: {\n        foo: {\n            cls: 'my-foo-trigger',\n            handler: function() {\n                console.log('foo trigger clicked');\n            }\n        },\n        bar: {\n            cls: 'my-bar-trigger',\n            handler: function() {\n                console.log('bar trigger clicked');\n            }\n        }\n    }\n});\n</code></pre>\n"},"ariaRole":{"!type":"string"},"getTriggers":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.form.field.Spinner-cfg-triggers\" rel=\"Ext.form.field.Spinner-cfg-triggers\" class=\"docClass\">triggers</a>.</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onMouseWheel":{"!type":"fn(e: ?) -> !this","!doc":"<p>Handles mousewheel events on the field</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Override.</p>\n"},"onSpinnerDownClick":{"!type":"fn() -> !this","!doc":"<p>Handles the spinner down button clicks.</p>\n"},"onSpinnerUpClick":{"!type":"fn() -> !this","!doc":"<p>Handles the spinner up button clicks.</p>\n"},"setTriggers":{"!type":"fn(triggers: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.form.field.Spinner-cfg-triggers\" rel=\"Ext.form.field.Spinner-cfg-triggers\" class=\"docClass\">triggers</a>.</p>\n"},"spinDown":{"!type":"fn() -> !this","!doc":"<p>Triggers the spinner to step down; fires the <a href=\"#!/api/Ext.form.field.Spinner-event-spin\" rel=\"Ext.form.field.Spinner-event-spin\" class=\"docClass\">spin</a> and <a href=\"#!/api/Ext.form.field.Spinner-event-spindown\" rel=\"Ext.form.field.Spinner-event-spindown\" class=\"docClass\">spindown</a> events and calls the\n<a href=\"#!/api/Ext.form.field.Spinner-method-onSpinDown\" rel=\"Ext.form.field.Spinner-method-onSpinDown\" class=\"docClass\">onSpinDown</a> method. Does nothing if the field is <a href=\"#!/api/Ext.form.field.Spinner-cfg-disabled\" rel=\"Ext.form.field.Spinner-cfg-disabled\" class=\"docClass\">disabled</a> or if <a href=\"#!/api/Ext.form.field.Spinner-cfg-spinDownEnabled\" rel=\"Ext.form.field.Spinner-cfg-spinDownEnabled\" class=\"docClass\">spinDownEnabled</a>\nis false.</p>\n"},"spinUp":{"!type":"fn() -> !this","!doc":"<p>Triggers the spinner to step up; fires the <a href=\"#!/api/Ext.form.field.Spinner-event-spin\" rel=\"Ext.form.field.Spinner-event-spin\" class=\"docClass\">spin</a> and <a href=\"#!/api/Ext.form.field.Spinner-event-spinup\" rel=\"Ext.form.field.Spinner-event-spinup\" class=\"docClass\">spinup</a> events and calls the\n<a href=\"#!/api/Ext.form.field.Spinner-method-onSpinUp\" rel=\"Ext.form.field.Spinner-method-onSpinUp\" class=\"docClass\">onSpinUp</a> method. Does nothing if the field is <a href=\"#!/api/Ext.form.field.Spinner-cfg-disabled\" rel=\"Ext.form.field.Spinner-cfg-disabled\" class=\"docClass\">disabled</a> or if <a href=\"#!/api/Ext.form.field.Spinner-cfg-spinUpEnabled\" rel=\"Ext.form.field.Spinner-cfg-spinUpEnabled\" class=\"docClass\">spinUpEnabled</a>\nis false.</p>\n"},"spin":{"!type":"fn(this: ?, direction: ?, eOpts: ?)","!doc":"<p>Fires when the spinner is made to spin up or down.</p>\n"},"spindown":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when the spinner is made to spin down.</p>\n"},"spinup":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when the spinner is made to spin up.</p>\n"},"onSpinDown":{"!type":"fn() -> !this","!doc":"<p>This method is called when the spinner down button is clicked, or when the down arrow key is pressed if\n<a href=\"#!/api/Ext.form.field.Spinner-cfg-keyNavEnabled\" rel=\"Ext.form.field.Spinner-cfg-keyNavEnabled\" class=\"docClass\">keyNavEnabled</a> is true. Must be implemented by subclasses.</p>\n"},"onSpinUp":{"!type":"fn() -> !this","!doc":"<p>This method is called when the spinner up button is clicked, or when the up arrow key is pressed if\n<a href=\"#!/api/Ext.form.field.Spinner-cfg-keyNavEnabled\" rel=\"Ext.form.field.Spinner-cfg-keyNavEnabled\" class=\"docClass\">keyNavEnabled</a> is true. Must be implemented by subclasses.</p>\n"},"setSpinDownEnabled":{"!type":"fn(enabled: ?) -> !this","!doc":"<p>Sets whether the spinner down button is enabled.</p>\n"},"setSpinUpEnabled":{"!type":"fn(enabled: ?) -> !this","!doc":"<p>Sets whether the spinner up button is enabled.</p>\n"}},"!doc":"<p>A field with a pair of up/down spinner buttons. This class is not normally instantiated directly,\ninstead it is subclassed and the <a href=\"#!/api/Ext.form.field.Spinner-method-onSpinUp\" rel=\"Ext.form.field.Spinner-method-onSpinUp\" class=\"docClass\">onSpinUp</a> and <a href=\"#!/api/Ext.form.field.Spinner-method-onSpinDown\" rel=\"Ext.form.field.Spinner-method-onSpinDown\" class=\"docClass\">onSpinDown</a> methods are implemented\nto handle when the buttons are clicked. A good example of this is the <a href=\"#!/api/Ext.form.field.Number\" rel=\"Ext.form.field.Number\" class=\"docClass\">Ext.form.field.Number</a>\nfield which uses the spinner to increment and decrement the field's value by its\n<a href=\"#!/api/Ext.form.field.Number-cfg-step\" rel=\"Ext.form.field.Number-cfg-step\" class=\"docClass\">step</a> config value.</p>\n\n<p>For example:</p>\n\n<pre class='inline-example '><code>Ext.define('Ext.ux.CustomSpinner', {\n    extend: '<a href=\"#!/api/Ext.form.field.Spinner\" rel=\"Ext.form.field.Spinner\" class=\"docClass\">Ext.form.field.Spinner</a>',\n    alias: 'widget.customspinner',\n\n    // override onSpinUp (using step isn't neccessary)\n    onSpinUp: function() {\n        var me = this;\n        if (!me.readOnly) {\n            var val = parseInt(me.getValue().split(' '), 10)||0; // gets rid of \" Pack\", defaults to zero on parse failure\n            me.setValue((val + me.step) + ' Pack');\n        }\n    },\n\n    // override onSpinDown\n    onSpinDown: function() {\n        var me = this;\n        if (!me.readOnly) {\n           var val = parseInt(me.getValue().split(' '), 10)||0; // gets rid of \" Pack\", defaults to zero on parse failure\n           if (val &lt;= me.step) {\n               me.setValue('Dry!');\n           } else {\n               me.setValue((val - me.step) + ' Pack');\n           }\n        }\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.FormPanel</a>', {\n    title: 'Form with SpinnerField',\n    bodyPadding: 5,\n    width: 350,\n    renderTo: Ext.getBody(),\n    items:[{\n        xtype: 'customspinner',\n        fieldLabel: 'How Much Beer?',\n        step: 6\n    }]\n});\n</code></pre>\n\n<p>By default, pressing the up and down arrow keys will also trigger the onSpinUp and onSpinDown methods;\nto prevent this, set <code><a href=\"#!/api/Ext.form.field.Spinner-cfg-keyNavEnabled\" rel=\"Ext.form.field.Spinner-cfg-keyNavEnabled\" class=\"docClass\">keyNavEnabled</a> = false</code>.</p>\n","!type":"fn(config: ?)"},"Radio":{"!doc":"<p>Single radio field. Similar to checkbox, but automatically handles making sure only one radio is checked\nat a time within a group of radios with the same name.</p>\n\n<h1>Labeling</h1>\n\n<p>In addition to the <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">standard field labeling options</a>, radio buttons\nmay be given an optional <a href=\"#!/api/Ext.form.field.Radio-cfg-boxLabel\" rel=\"Ext.form.field.Radio-cfg-boxLabel\" class=\"docClass\">boxLabel</a> which will be displayed immediately to the right of the input. Also\nsee <a href=\"#!/api/Ext.form.RadioGroup\" rel=\"Ext.form.RadioGroup\" class=\"docClass\">Ext.form.RadioGroup</a> for a convenient method of grouping related radio buttons.</p>\n\n<h1>Values</h1>\n\n<p>The main value of a Radio field is a boolean, indicating whether or not the radio is checked.</p>\n\n<p>The following values will check the radio:</p>\n\n<ul>\n<li><code>true</code></li>\n<li><code>'true'</code></li>\n<li><code>'1'</code></li>\n<li><code>'on'</code></li>\n</ul>\n\n\n<p>Any other value will uncheck it.</p>\n\n<p>In addition to the main boolean value, you may also specify a separate <a href=\"#!/api/Ext.form.field.Radio-cfg-inputValue\" rel=\"Ext.form.field.Radio-cfg-inputValue\" class=\"docClass\">inputValue</a>. This will be sent\nas the parameter value when the form is <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">submitted</a>. You will want to set this\nvalue if you have multiple radio buttons with the same <a href=\"#!/api/Ext.form.field.Radio-cfg-name\" rel=\"Ext.form.field.Radio-cfg-name\" class=\"docClass\">name</a>, as is almost always the case.</p>\n\n<h1>Example usage</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title      : 'Order Form',\n    width      : 300,\n    bodyPadding: 10,\n    renderTo   : Ext.getBody(),\n    items: [\n        {\n            xtype      : 'fieldcontainer',\n            fieldLabel : 'Size',\n            defaultType: 'radiofield',\n            defaults: {\n                flex: 1\n            },\n            layout: 'hbox',\n            items: [\n                {\n                    boxLabel  : 'M',\n                    name      : 'size',\n                    inputValue: 'm',\n                    id        : 'radio1'\n                }, {\n                    boxLabel  : 'L',\n                    name      : 'size',\n                    inputValue: 'l',\n                    id        : 'radio2'\n                }, {\n                    boxLabel  : 'XL',\n                    name      : 'size',\n                    inputValue: 'xl',\n                    id        : 'radio3'\n                }\n            ]\n        },\n        {\n            xtype      : 'fieldcontainer',\n            fieldLabel : 'Color',\n            defaultType: 'radiofield',\n            defaults: {\n                flex: 1\n            },\n            layout: 'hbox',\n            items: [\n                {\n                    boxLabel  : 'Blue',\n                    name      : 'color',\n                    inputValue: 'blue',\n                    id        : 'radio4'\n                }, {\n                    boxLabel  : 'Grey',\n                    name      : 'color',\n                    inputValue: 'grey',\n                    id        : 'radio5'\n                }, {\n                    boxLabel  : 'Black',\n                    name      : 'color',\n                    inputValue: 'black',\n                    id        : 'radio6'\n                }\n            ]\n        }\n    ],\n    bbar: [\n        {\n            text: 'Smaller Size',\n            handler: function() {\n                var radio1 = Ext.getCmp('radio1'),\n                    radio2 = Ext.getCmp('radio2'),\n                    radio3 = Ext.getCmp('radio3');\n\n                //if L is selected, change to M\n                if (radio2.getValue()) {\n                    radio1.setValue(true);\n                    return;\n                }\n\n                //if XL is selected, change to L\n                if (radio3.getValue()) {\n                    radio2.setValue(true);\n                    return;\n                }\n\n                //if nothing is set, set size to S\n                radio1.setValue(true);\n            }\n        },\n        {\n            text: 'Larger Size',\n            handler: function() {\n                var radio1 = Ext.getCmp('radio1'),\n                    radio2 = Ext.getCmp('radio2'),\n                    radio3 = Ext.getCmp('radio3');\n\n                //if M is selected, change to L\n                if (radio1.getValue()) {\n                    radio2.setValue(true);\n                    return;\n                }\n\n                //if L is selected, change to XL\n                if (radio2.getValue()) {\n                    radio3.setValue(true);\n                    return;\n                }\n\n                //if nothing is set, set size to XL\n                radio3.setValue(true);\n            }\n        },\n        '-',\n        {\n            text: 'Select color',\n            menu: {\n                indent: false,\n                items: [\n                    {\n                        text: 'Blue',\n                        handler: function() {\n                            var radio = Ext.getCmp('radio4');\n                            radio.setValue(true);\n                        }\n                    },\n                    {\n                        text: 'Grey',\n                        handler: function() {\n                            var radio = Ext.getCmp('radio5');\n                            radio.setValue(true);\n                        }\n                    },\n                    {\n                        text: 'Black',\n                        handler: function() {\n                            var radio = Ext.getCmp('radio6');\n                            radio.setValue(true);\n                        }\n                    }\n                ]\n            }\n        }\n    ]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"inputType":{"!type":"string","!doc":"<p>private</p>\n"},"uncheckedValue":{"!type":"string"},"ariaRole":{"!type":"string"},"formId":{},"isRadio":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Radio, or subclass thereof.</p>\n"},"getGroupValue":{"!type":"fn() -> string","!doc":"<p>If this radio is part of a group, it will return the selected value</p>\n"},"getManager":{"!type":"fn() -> !this","!doc":"<p>inherit docs</p>\n"},"getModelData":{"!type":"fn() -> ?","!doc":"<p>Returns the value(s) that should be saved to the Ext.data.Model instance for this field, when <a href=\"#!/api/Ext.form.Basic-method-updateRecord\" rel=\"Ext.form.Basic-method-updateRecord\" class=\"docClass\">Ext.form.Basic.updateRecord</a> is called. Typically this will be an object with a single name-value pair, the name\nbeing this field's <a href=\"#!/api/Ext.form.field.Radio-method-getName\" rel=\"Ext.form.field.Radio-method-getName\" class=\"docClass\">name</a> and the value being its current data value. More advanced field\nimplementations may return more than one name-value pair. The returned values will be saved to the corresponding\nfield names in the Model.</p>\n\n<p>Note that the values returned from this method are not guaranteed to have been successfully <a href=\"#!/api/Ext.form.field.Radio-method-validate\" rel=\"Ext.form.field.Radio-method-validate\" class=\"docClass\">validated</a>.</p>\n"},"getSubmitValue":{"!type":"fn() -> ?","!doc":"<p>Returns the submit value for the checkbox which can be used when submitting forms.</p>\n"},"onBoxClick":{"!type":"fn() -> !this","!doc":"<p>Handle click on the radio button</p>\n"},"onChange":{"!type":"fn(newVal: ?, oldVal: ?) -> !this","!doc":"<p>inherit docs</p>\n"},"onRemoved":{"!type":"fn() -> !this","!doc":"<p>Method to manage awareness of when components are removed from their\nrespective Container, firing a <a href=\"#!/api/Ext.form.field.Radio-event-removed\" rel=\"Ext.form.field.Radio-event-removed\" class=\"docClass\">removed</a> event. References are properly\ncleaned up after removing a component from its owning container.</p>\n\n<p>Allows addition of behavior when a Component is removed from\nits parent Container. At this stage, the Component has been\nremoved from its parent Container's collection of child items,\nbut has not been destroyed (It will be destroyed if the parent\nContainer's <code>autoDestroy</code> is <code>true</code>, or if the remove call was\npassed a truthy second parameter). After calling the\nsuperclass's <code>onRemoved</code>, the <code>ownerCt</code> and the <code>refOwner</code> will not\nbe present.</p>\n"},"setValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Sets either the checked/unchecked status of this Radio, or, if a string value is passed, checks a sibling Radio\nof the same name whose value is the value specified.</p>\n"}}},"Tag":{"!doc":"<p>The <code>tagfield</code> is a combo box improved for multiple value editing, selection and easy\nmanagement.</p>\n\n<h3>Histroy</h3>\n\n<p>Inspired by the <a href=\"http://technomedia.co.uk/SuperBoxSelect/examples3.html\">SuperBoxSelect component for ExtJS 3</a>,\nwhich in turn was inspired by the <a href=\"http://efattal.fr/en/extjs/extuxboxselect/\">BoxSelect component for ExtJS 2</a>.</p>\n\n<p>Various contributions and suggestions made by many members of the ExtJS community which can be seen\nin the <a href=\"http://www.sencha.com/forum/showthread.php?134751-Ext.ux.form.field.BoxSelect\">user extension forum post</a>.</p>\n\n<p>By: kvee_iv http://www.sencha.com/forum/member.php?29437-kveeiv</p>\n","!type":"fn(config: ?)","prototype":{"createNewOnBlur":{"!type":"bool","!doc":"<p>Has no effect if <a href=\"#!/api/Ext.form.field.Tag-cfg-forceSelection\" rel=\"Ext.form.field.Tag-cfg-forceSelection\" class=\"docClass\">forceSelection</a> is <code>true</code>.</p>\n\n<p>With <a href=\"#!/api/Ext.form.field.Tag-cfg-createNewOnBlur\" rel=\"Ext.form.field.Tag-cfg-createNewOnBlur\" class=\"docClass\">createNewOnBlur</a> set to <code>true</code>, the creation described in\n<a href=\"#!/api/Ext.form.field.Tag-cfg-forceSelection\" rel=\"Ext.form.field.Tag-cfg-forceSelection\" class=\"docClass\">forceSelection</a> will also be triggered when the field loses focus.</p>\n\n<p>Please note that this behavior is also affected by the configuration options\n<a href=\"#!/api/Ext.form.field.Tag-cfg-autoSelect\" rel=\"Ext.form.field.Tag-cfg-autoSelect\" class=\"docClass\">autoSelect</a> and <a href=\"#!/api/Ext.form.field.Tag-cfg-selectOnTab\" rel=\"Ext.form.field.Tag-cfg-selectOnTab\" class=\"docClass\">selectOnTab</a>. If those are true and an existing\nitem would have been selected as a result, the partial text the user has entered will\nbe discarded and the existing item will be added to the selection.</p>\n"},"createNewOnEnter":{"!type":"bool","!doc":"<p>Has no effect if <a href=\"#!/api/Ext.form.field.Tag-cfg-forceSelection\" rel=\"Ext.form.field.Tag-cfg-forceSelection\" class=\"docClass\">forceSelection</a> is <code>true</code>.</p>\n\n<p>With <a href=\"#!/api/Ext.form.field.Tag-cfg-createNewOnEnter\" rel=\"Ext.form.field.Tag-cfg-createNewOnEnter\" class=\"docClass\">createNewOnEnter</a> set to <code>true</code>, the creation described in\n<a href=\"#!/api/Ext.form.field.Tag-cfg-forceSelection\" rel=\"Ext.form.field.Tag-cfg-forceSelection\" class=\"docClass\">forceSelection</a> will also be triggered by the 'enter' key.</p>\n"},"encodeSubmitValue":{"!type":"bool","!doc":"<p>Has no effect if <a href=\"#!/api/Ext.form.field.Tag-cfg-multiSelect\" rel=\"Ext.form.field.Tag-cfg-multiSelect\" class=\"docClass\">multiSelect</a> is <code>false</code>.</p>\n\n<p>Controls the formatting of the form submit value of the field as returned by <a href=\"#!/api/Ext.form.field.Tag-method-getSubmitValue\" rel=\"Ext.form.field.Tag-method-getSubmitValue\" class=\"docClass\">getSubmitValue</a></p>\n\n<ul>\n<li><code>true</code> for the field value to submit as a json encoded array in a single GET/POST variable</li>\n<li><code>false</code> for the field to submit as an array of GET/POST variables</li>\n</ul>\n\n"},"filterPickList":{"!type":"bool","!doc":"<p>True to hide the currently selected values from the drop down list. These items are hidden via\ncss to maintain simplicity in store and filter management.</p>\n\n<ul>\n<li><code>true</code> to hide currently selected values from the drop down pick list</li>\n<li><code>false</code> to keep the item in the pick list as a selected item</li>\n</ul>\n\n"},"forceSelection":{"!type":"bool","!doc":"<p>When <a href=\"#!/api/Ext.form.field.Tag-cfg-forceSelection\" rel=\"Ext.form.field.Tag-cfg-forceSelection\" class=\"docClass\">forceSelection</a> is <code>false</code>, new records can be created by the user as they\nare typed. These records are <strong>not</strong> added to the combo's store. This creation\nis triggered by typing the configured 'delimiter', and can be further configured using the\n<a href=\"#!/api/Ext.form.field.Tag-cfg-createNewOnEnter\" rel=\"Ext.form.field.Tag-cfg-createNewOnEnter\" class=\"docClass\">createNewOnEnter</a> and <a href=\"#!/api/Ext.form.field.Tag-cfg-createNewOnBlur\" rel=\"Ext.form.field.Tag-cfg-createNewOnBlur\" class=\"docClass\">createNewOnBlur</a> configuration options.</p>\n\n<p>This functionality is primarily useful for things such as an email address.</p>\n"},"grow":{"!type":"bool","!doc":"<p><code>true</code> if this field should automatically grow and shrink vertically to its content.\nNote that this overrides the natural trigger grow functionality, which is used to size\nthe field horizontally.</p>\n"},"growMax":{"!doc":"<p>Has no effect if <a href=\"#!/api/Ext.form.field.Tag-cfg-grow\" rel=\"Ext.form.field.Tag-cfg-grow\" class=\"docClass\">grow</a> is <code>false</code></p>\n\n<p>The maximum height to allow when <a href=\"#!/api/Ext.form.field.Tag-cfg-grow\" rel=\"Ext.form.field.Tag-cfg-grow\" class=\"docClass\">grow</a> is <code>true</code>, or <code>false</code> to allow for\nnatural vertical growth based on the current selected values. See also <a href=\"#!/api/Ext.form.field.Tag-cfg-growMin\" rel=\"Ext.form.field.Tag-cfg-growMin\" class=\"docClass\">growMin</a>.</p>\n"},"growMin":{"!doc":"<p>Has no effect if <a href=\"#!/api/Ext.form.field.Tag-cfg-grow\" rel=\"Ext.form.field.Tag-cfg-grow\" class=\"docClass\">grow</a> is <code>false</code></p>\n\n<p>The minimum height to allow when <a href=\"#!/api/Ext.form.field.Tag-cfg-grow\" rel=\"Ext.form.field.Tag-cfg-grow\" class=\"docClass\">grow</a> is <code>true</code>, or <code>false</code> to allow for\nnatural vertical growth based on the current selected values. See also <a href=\"#!/api/Ext.form.field.Tag-cfg-growMax\" rel=\"Ext.form.field.Tag-cfg-growMax\" class=\"docClass\">growMax</a>.</p>\n"},"labelTpl":{"!doc":"<p>The <a href=\"http://docs.sencha.com/ext-js/4-1/#!/api/Ext.XTemplate\">XTemplate</a> to use for the inner\nmarkup of the labelled items. Defaults to the configured <a href=\"#!/api/Ext.form.field.Tag-cfg-displayField\" rel=\"Ext.form.field.Tag-cfg-displayField\" class=\"docClass\">displayField</a></p>\n"},"multiSelect":{"!type":"bool","!doc":"<p>If set to <code>true</code>, allows the combo field to hold more than one value at a time, and allows selecting multiple\nitems from the dropdown list. The combo's text field will show all selected values using the template\ndefined by <a href=\"#!/api/Ext.form.field.Tag-cfg-labelTpl\" rel=\"Ext.form.field.Tag-cfg-labelTpl\" class=\"docClass\">labelTpl</a>.</p>\n"},"pinList":{"!type":"bool","!doc":"<p>Has no effect if <a href=\"#!/api/Ext.form.field.Tag-cfg-multiSelect\" rel=\"Ext.form.field.Tag-cfg-multiSelect\" class=\"docClass\">multiSelect</a> is <code>false</code></p>\n\n<p><code>true</code> to keep the pick list expanded after each selection from the pick list\n<code>false</code> to automatically collapse the pick list after a selection is made</p>\n"},"selectOnFocus":{"!type":"bool","!doc":"<p><code>true</code> to automatically select any existing field text when the field receives input\nfocus. Only applies when <a href=\"#!/api/Ext.form.field.Tag-cfg-editable\" rel=\"Ext.form.field.Tag-cfg-editable\" class=\"docClass\">editable</a> = true</p>\n"},"stacked":{"!type":"bool","!doc":"<ul>\n<li><code>true</code> to have each selected value fill to the width of the form field</li>\n<li>`false to have each selected value size to its displayed contents</li>\n</ul>\n\n"},"triggerOnClick":{"!type":"bool","!doc":"<p><code>true</code> to activate the trigger when clicking in empty space in the field. Note that the\nsubsequent behavior of this is controlled by the field's <a href=\"#!/api/Ext.form.field.Tag-cfg-triggerAction\" rel=\"Ext.form.field.Tag-cfg-triggerAction\" class=\"docClass\">triggerAction</a>.\nThis behavior is similar to that of a basic ComboBox with <a href=\"#!/api/Ext.form.field.Tag-cfg-editable\" rel=\"Ext.form.field.Tag-cfg-editable\" class=\"docClass\">editable</a> <code>false</code>.</p>\n"},"valueParam":{"!type":"string","!doc":"<p>The name of the parameter used to load unknown records into the store. If left unspecified, <a href=\"#!/api/Ext.form.field.Tag-cfg-valueField\" rel=\"Ext.form.field.Tag-cfg-valueField\" class=\"docClass\">valueField</a>\nwill be used.</p>\n"},"childEls":{},"emptyInputCls":{"!type":"string"},"fieldSubTpl":{},"addValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Adds a value or values to the current value of the field</p>\n"},"afterComponentLayout":{"!type":"fn() -> !this","!doc":"<p>Track height change to fire <a href=\"#!/api/Ext.form.field.Tag-event-autosize\" rel=\"Ext.form.field.Tag-event-autosize\" class=\"docClass\">autosize</a> event, when applicable.</p>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Overridden to avoid use of placeholder, as our main input field is often empty</p>\n"},"applyEmptyText":{"!type":"fn() -> !this","!doc":"<p>Overridden to use value (selection) instead of raw value and to avoid the use of placeholder</p>\n"},"applyMultiselectItemMarkup":{"!type":"fn() -> !this","!doc":"<p>Update the labelled items rendering</p>\n"},"assertValue":{"!type":"fn() -> !this","!doc":"<p>Overridden to handle partial-input selections more directly</p>\n"},"autoSize":{"!type":"fn() -> !this","!doc":"<p>Initiate auto-sizing for height based on <a href=\"#!/api/Ext.form.field.Tag-cfg-grow\" rel=\"Ext.form.field.Tag-cfg-grow\" class=\"docClass\">grow</a>, if applicable.</p>\n"},"checkChange":{"!type":"fn() -> !this","!doc":"<p>Expand record values for evaluating change and fire change events for UI to respond to</p>\n"},"createPicker":{"!type":"fn() -> !this","!doc":"<p>Add refresh tracking to the picker for selection management</p>\n"},"doRawQuery":{"!type":"fn() -> !this","!doc":"<p>Overridden to allow for continued querying with multiSelect selections already made</p>\n"},"findRecord":{"!type":"fn(field: ?, value: ?) -> !this","!doc":"<p>Overridden to search entire unfiltered store since already selected values\ncan span across multiple store page loads and other filtering. Overlaps\nsome with <a href=\"#!/api/Ext.form.field.Tag-method-isFilteredRecord\" rel=\"Ext.form.field.Tag-method-isFilteredRecord\" class=\"docClass\">isFilteredRecord</a>, but findRecord is used by the base component\nfor various logic so this logic is applied here as well.</p>\n"},"getCursorPosition":{"!type":"fn() -> !this","!doc":"<p>Get the current cursor position in the input field, for key-based navigation</p>\n"},"getMultiSelectItemMarkup":{"!type":"fn() -> !this","!doc":"<p>Build the markup for the labelled items. Template must be built on demand due to ComboBox initComponent\nlifecycle for the creation of on-demand stores (to account for automatic valueField/displayField setting)</p>\n"},"getRawValue":{"!type":"fn() -> string","!doc":"<p>Intercept calls to getRawValue to pretend there is no inputEl for rawValue handling,\nso that we can use inputEl for user input of just the current value.</p>\n"},"getRecordByListItemNode":{"!type":"fn(itemEl: ?) -> !this","!doc":"<p>Returns the record from valueStore for the labelled item node</p>\n"},"getSubTplData":{"!type":"fn() -> !this","!doc":"<p>Add empty text support to initial render.</p>\n"},"getSubmitData":{"!type":"fn() -> ?","!doc":"<p>Overridden to optionally allow for submitting the field as a json encoded array.</p>\n"},"getValueRecords":{"!type":"fn() -> ?","!doc":"<p>Returns the records for the field's current value</p>\n"},"hasSelectedText":{"!type":"fn() -> !this","!doc":"<p>Check to see if the input field has selected text, for key-based navigation</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>Initialize additional settings and enable simultaneous typeAhead and multiSelect support</p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>Register events for management controls of labelled items</p>\n"},"isEqual":{"!type":"fn(v1: ?, v2: ?) -> !this","!doc":"<p>Overridden to be more accepting of varied value types</p>\n"},"isFilteredRecord":{"!type":"fn(record: ?) -> !this","!doc":"<p>Used to determine if a record is filtered out of the current store's data set,\nfor determining if a currently selected value should be retained.</p>\n\n<p>Slightly complicated logic. A record is considered filtered and should be retained if:</p>\n\n<ul>\n<li>It is not in the combo store and the store has no filter or it is in the filtered data set\n(Happens when our selected value is just part of a different load, page or query)</li>\n<li>It is not in the combo store and forceSelection is false and it is in the value store\n(Happens when our selected value was created manually)</li>\n</ul>\n\n"},"mimicBlur":{"!type":"fn() -> !this","!doc":"<p>Overridden to clear the input field if we are auto-setting a value as we blur.</p>\n"},"onBeforeListRefresh":{"!type":"fn() -> !this","!doc":"<p>When the picker is refreshing, we should ignore selection changes. Otherwise\nthe value of our field will be changing just because our view of the choices is.</p>\n"},"onBindStore":{"!type":"fn(store: ?) -> !this","!doc":"<p>Create a store for the records of our current value based on the main store's model</p>\n"},"onBlur":{"!type":"fn() -> !this","!doc":"<p>Intercept calls to onBlur to remove focusCls, because the base field\nclasses assume this should be applied to inputEl</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Clean up selected values management controls</p>\n"},"onFocus":{"!type":"fn() -> !this","!doc":"<p>Intercept calls to onFocus to add focusCls, because the base field\nclasses assume this should be applied to inputEl</p>\n"},"onFocusChange":{"!type":"fn(selectionModel: ?, oldFocused: ?, newFocused: ?) -> !this"},"onItemListClick":{"!type":"fn(evt: ?) -> !this","!doc":"<p>Delegation control for selecting and removing labelled items or triggering list collapse/expansion</p>\n"},"onKeyDown":{"!type":"fn(e: ?, t: ?) -> !this","!doc":"<p>Handles keyDown processing of key-based selection of labelled items.\nSupported keyboard controls:</p>\n\n<ul>\n<li><p>If pick list is expanded</p>\n\n<ul>\n<li><code>CTRL-A</code> will select all the items in the pick list</li>\n</ul>\n</li>\n<li><p>If the cursor is at the beginning of the input field and there are values present</p>\n\n<ul>\n<li><code>CTRL-A</code> will highlight all the currently selected values</li>\n<li><code>BACKSPACE</code> and <code>DELETE</code> will remove any currently highlighted selected values</li>\n<li><code>RIGHT</code> and <code>LEFT</code> will move the current highlight in the appropriate direction</li>\n<li><code>SHIFT-RIGHT</code> and <code>SHIFT-LEFT</code> will add to the current highlight in the appropriate direction</li>\n</ul>\n</li>\n</ul>\n\n"},"onKeyUp":{"!type":"fn(e: ?, t: ?) -> !this","!doc":"<p>Handles auto-selection and creation of labelled items based on this field's\ndelimiter, as well as the keyUp processing of key-based selection of labelled items.</p>\n"},"onListRefresh":{"!type":"fn() -> !this","!doc":"<p>When the picker is refreshing, we should ignore selection changes. Otherwise\nthe value of our field will be changing just because our view of the choices is.</p>\n"},"onListSelectionChange":{"!type":"fn(list: ?, selectedRecords: ?) -> !this","!doc":"<p>Overridden to preserve current labelled items when list is filtered/paged/loaded\nand does not include our current value. See <a href=\"#!/api/Ext.form.field.Tag-method-isFilteredRecord\" rel=\"Ext.form.field.Tag-method-isFilteredRecord\" class=\"docClass\">isFilteredRecord</a></p>\n"},"onLoad":{"!type":"fn() -> !this","!doc":"<p>Overridden to map previously selected records to the \"new\" versions of the records\nbased on value field, if they are part of the new store load</p>\n"},"onPaste":{"!type":"fn(e: ?) -> !this","!doc":"<p>Handles auto-selection of labelled items based on this field's delimiter when pasting\na list of values in to the field (e.g., for email addresses)</p>\n"},"onSelectionChange":{"!type":"fn(selModel: ?, selectedRecs: ?) -> !this"},"onTypeAhead":{"!type":"fn() -> !this","!doc":"<p>Overridden to get and set the DOM value directly for type-ahead suggestion (bypassing get/setRawValue)</p>\n"},"onUnbindStore":{"!type":"fn(store: ?) -> !this","!doc":"<p>Remove the selected value store and associated listeners</p>\n"},"onValueStoreChange":{"!type":"fn() -> !this"},"onValueStoreRemove":{"!type":"fn() -> !this"},"preFocus":{"!type":"fn() -> !this","!doc":"<p>Overridden to use inputEl instead of raw value and to avoid the use of placeholder</p>\n"},"removeByListItemNode":{"!type":"fn(itemEl: ?) -> !this","!doc":"<p>Removal of labelled item by node reference</p>\n"},"removeValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Removes a value or values from the current value of the field</p>\n"},"renderActiveError":{"!type":"fn() -> !this","!doc":"<p>Intercept calls to renderActiveError to add invalidCls, because the base\nfield classes assume this should be applied to inputEl</p>\n"},"setRawValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Intercept calls to setRawValue to pretend there is no inputEl for rawValue handling,\nso that we can use inputEl for user input of just the current value.</p>\n"},"setValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Sets the specified value(s) into the field. The following value formats are recognised:</p>\n\n<ul>\n<li><p>Single Values</p>\n\n<ul>\n<li>A string associated to this field's configured <a href=\"#!/api/Ext.form.field.Tag-cfg-valueField\" rel=\"Ext.form.field.Tag-cfg-valueField\" class=\"docClass\">valueField</a></li>\n<li>A record containing at least this field's configured <a href=\"#!/api/Ext.form.field.Tag-cfg-valueField\" rel=\"Ext.form.field.Tag-cfg-valueField\" class=\"docClass\">valueField</a> and <a href=\"#!/api/Ext.form.field.Tag-cfg-displayField\" rel=\"Ext.form.field.Tag-cfg-displayField\" class=\"docClass\">displayField</a></li>\n</ul>\n</li>\n<li><p>Multiple Values</p>\n\n<ul>\n<li>If <a href=\"#!/api/Ext.form.field.Tag-cfg-multiSelect\" rel=\"Ext.form.field.Tag-cfg-multiSelect\" class=\"docClass\">multiSelect</a> is <code>true</code>, a string containing multiple strings as\nspecified in the Single Values section above, concatenated in to one string\nwith each entry separated by this field's configured <a href=\"#!/api/Ext.form.field.Tag-cfg-delimiter\" rel=\"Ext.form.field.Tag-cfg-delimiter\" class=\"docClass\">delimiter</a></li>\n<li>An array of strings as specified in the Single Values section above</li>\n<li>An array of records as specified in the Single Values section above</li>\n</ul>\n</li>\n</ul>\n\n\n<p>In any of the string formats above, the following occurs if an associated record cannot be found:</p>\n\n<ol>\n<li>If <a href=\"#!/api/Ext.form.field.Tag-cfg-forceSelection\" rel=\"Ext.form.field.Tag-cfg-forceSelection\" class=\"docClass\">forceSelection</a> is <code>false</code>, a new record of the <a href=\"#!/api/Ext.form.field.Tag-cfg-store\" rel=\"Ext.form.field.Tag-cfg-store\" class=\"docClass\">store</a>'s configured model type\nwill be created using the given value as the <a href=\"#!/api/Ext.form.field.Tag-cfg-displayField\" rel=\"Ext.form.field.Tag-cfg-displayField\" class=\"docClass\">displayField</a> and <a href=\"#!/api/Ext.form.field.Tag-cfg-valueField\" rel=\"Ext.form.field.Tag-cfg-valueField\" class=\"docClass\">valueField</a>.\nThis record will be added to the current value, but it will <strong>not</strong> be added to the store.</li>\n<li>If <a href=\"#!/api/Ext.form.field.Tag-cfg-forceSelection\" rel=\"Ext.form.field.Tag-cfg-forceSelection\" class=\"docClass\">forceSelection</a> is <code>true</code> and <a href=\"#!/api/Ext.form.field.Tag-cfg-queryMode\" rel=\"Ext.form.field.Tag-cfg-queryMode\" class=\"docClass\">queryMode</a> is <code>remote</code>, the list of unknown\nvalues will be submitted as a call to the <a href=\"#!/api/Ext.form.field.Tag-cfg-store\" rel=\"Ext.form.field.Tag-cfg-store\" class=\"docClass\">store</a>'s load as a parameter named by\nthe <a href=\"#!/api/Ext.form.field.Tag-cfg-valueParam\" rel=\"Ext.form.field.Tag-cfg-valueParam\" class=\"docClass\">valueParam</a> with values separated by the configured <a href=\"#!/api/Ext.form.field.Tag-cfg-delimiter\" rel=\"Ext.form.field.Tag-cfg-delimiter\" class=\"docClass\">delimiter</a>.\n<strong> This process will cause setValue to asynchronously process. </strong> This will only be attempted\nonce. Any unknown values that the server does not return records for will be removed.</li>\n<li>Otherwise, unknown values will be removed.</li>\n</ol>\n\n"},"syncSelection":{"!type":"fn() -> !this","!doc":"<p>Overridden to use valueStore instead of valueModels, for inclusion of\nfiltered records. See <a href=\"#!/api/Ext.form.field.Tag-method-isFilteredRecord\" rel=\"Ext.form.field.Tag-method-isFilteredRecord\" class=\"docClass\">isFilteredRecord</a></p>\n"},"toggleSelectionByListItemNode":{"!type":"fn(itemEl: ?, keepExisting: ?) -> !this","!doc":"<p>Toggle of labelled item selection by node reference</p>\n"},"autosize":{"!type":"fn(this: ?, height: ?, eOpts: ?)","!doc":"<p>Fires when the <strong><a href=\"#!/api/Ext.form.field.Tag-method-autoSize\" rel=\"Ext.form.field.Tag-method-autoSize\" class=\"docClass\">autoSize</a></strong> function is triggered and the field is resized according to the\n<a href=\"#!/api/Ext.form.field.Tag-cfg-grow\" rel=\"Ext.form.field.Tag-cfg-grow\" class=\"docClass\">grow</a>/<a href=\"#!/api/Ext.form.field.Tag-cfg-growMin\" rel=\"Ext.form.field.Tag-cfg-growMin\" class=\"docClass\">growMin</a>/<a href=\"#!/api/Ext.form.field.Tag-cfg-growMax\" rel=\"Ext.form.field.Tag-cfg-growMax\" class=\"docClass\">growMax</a> configs as a result. This event provides a hook for the\ndeveloper to apply additional logic at runtime to resize the field if needed.</p>\n"}}},"TextArea":{"!doc":"<p>This class creates a multiline text field, which can be used as a direct replacement for traditional\ntextarea fields. In addition, it supports automatically <a href=\"#!/api/Ext.form.field.TextArea-cfg-grow\" rel=\"Ext.form.field.TextArea-cfg-grow\" class=\"docClass\">growing</a> the height of the textarea to\nfit its content.</p>\n\n<p>All of the configuration options from <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a> can be used on TextArea.</p>\n\n<p>Example usage:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.FormPanel</a>', {\n    title      : 'Sample TextArea',\n    width      : 400,\n    bodyPadding: 10,\n    renderTo   : Ext.getBody(),\n    items: [{\n        xtype     : 'textareafield',\n        grow      : true,\n        name      : 'message',\n        fieldLabel: 'Message',\n        anchor    : '100%'\n    }]\n});\n</code></pre>\n\n<p>Some other useful configuration options when using <a href=\"#!/api/Ext.form.field.TextArea-cfg-grow\" rel=\"Ext.form.field.TextArea-cfg-grow\" class=\"docClass\">grow</a> are <a href=\"#!/api/Ext.form.field.TextArea-cfg-growMin\" rel=\"Ext.form.field.TextArea-cfg-growMin\" class=\"docClass\">growMin</a> and <a href=\"#!/api/Ext.form.field.TextArea-cfg-growMax\" rel=\"Ext.form.field.TextArea-cfg-growMax\" class=\"docClass\">growMax</a>.\nThese allow you to set the minimum and maximum grow heights for the textarea.</p>\n\n<p><strong>NOTE:</strong> In some browsers, carriage returns ('\\r', not to be confused with new lines)\nwill be automatically stripped out the value is set to the textarea. Since we cannot\nuse any reasonable method to attempt to re-insert these, they will automatically be\nstripped out to ensure the behaviour is consistent across browser.</p>\n","!type":"fn(config: ?)","prototype":{"enterIsSpecial":{"!type":"bool","!doc":"<p>True if you want the ENTER key to be classed as a special key and the <a href=\"#!/api/Ext.form.field.TextArea-event-specialkey\" rel=\"Ext.form.field.TextArea-event-specialkey\" class=\"docClass\">specialkey</a> event to be fired\nwhen ENTER is pressed.</p>\n"},"fieldSubTpl":{"!doc":"<p>This template includes a <code>\\n</code> after <code>&lt;textarea&gt;</code> opening tag so that an\ninitial value starting with <code>\\n</code> does not lose its first character when\nthe markup is parsed. Both textareas below have the same value:</p>\n\n<pre><code>&lt;textarea&gt;initial value&lt;/textarea&gt;\n\n&lt;textarea&gt;\ninitial value\n&lt;/textarea&gt;\n</code></pre>\n"},"growAppend":{"!type":"string","!doc":"<p>A string that will be appended to the field's current value for the purposes of calculating the target field\nsize. Only used when the <a href=\"#!/api/Ext.form.field.TextArea-cfg-grow\" rel=\"Ext.form.field.TextArea-cfg-grow\" class=\"docClass\">grow</a> config is true. Defaults to a newline for TextArea to ensure there is\nalways a space below the current line.</p>\n"},"growMax":{"!type":"number","!doc":"<p>The maximum height to allow when <a href=\"#!/api/Ext.form.field.TextArea-cfg-grow\" rel=\"Ext.form.field.TextArea-cfg-grow\" class=\"docClass\">grow</a>=true</p>\n"},"growMin":{"!type":"number","!doc":"<p>The minimum height to allow when <a href=\"#!/api/Ext.form.field.TextArea-cfg-grow\" rel=\"Ext.form.field.TextArea-cfg-grow\" class=\"docClass\">grow</a>=true</p>\n"},"preventScrollbars":{"!type":"bool","!doc":"<p>true to prevent scrollbars from appearing regardless of how much text is in the field. This option is only\nrelevant when <a href=\"#!/api/Ext.form.field.TextArea-cfg-grow\" rel=\"Ext.form.field.TextArea-cfg-grow\" class=\"docClass\">grow</a> is true. Equivalent to setting overflow: hidden.</p>\n"},"extraFieldBodyCls":{"!type":"string"},"inputCls":{"!type":"string"},"returnRe":{},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"autoSize":{"!type":"fn() -> !this","!doc":"<p>Automatically grows the field to accomodate the height of the text up to the maximum\nfield height allowed. This only takes effect if <a href=\"#!/api/Ext.form.field.TextArea-cfg-grow\" rel=\"Ext.form.field.TextArea-cfg-grow\" class=\"docClass\">grow</a> = true, and fires the\n<a href=\"#!/api/Ext.form.field.TextArea-event-autosize\" rel=\"Ext.form.field.TextArea-event-autosize\" class=\"docClass\">autosize</a> event if the height changes.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"fireKey":{"!type":"fn(e: ?) -> !this","!doc":"<p>private</p>\n"},"getSubTplData":{"!type":"fn() -> ?","!doc":"<p>private</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Returns the current data value of the field. The type of value returned is particular to the type of the\nparticular field (e.g. a Date object for <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>), as the result of calling <a href=\"#!/api/Ext.form.field.TextArea-method-rawToValue\" rel=\"Ext.form.field.TextArea-method-rawToValue\" class=\"docClass\">rawToValue</a> on\nthe field's <a href=\"#!/api/Ext.form.field.TextArea-method-processRawValue\" rel=\"Ext.form.field.TextArea-method-processRawValue\" class=\"docClass\">processed</a> String value. To return the raw String value, see <a href=\"#!/api/Ext.form.field.TextArea-method-getRawValue\" rel=\"Ext.form.field.TextArea-method-getRawValue\" class=\"docClass\">getRawValue</a>.</p>\n"},"isCutCopyPasteSelectAll":{"!type":"fn(e: ?, key: ?) -> !this"},"onPaste":{"!type":"fn() -> !this"},"pasteCheck":{"!type":"fn() -> !this"},"setGrowSizePolicy":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"stripReturns":{"!type":"fn(value: ?) -> !this"},"transformRawValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>The following overrides deal with an issue whereby some browsers\nwill strip carriage returns from the textarea input, while others\nwill not. Since there's no way to be sure where to insert returns,\nthe best solution is to strip them out in all cases to ensure that\nthe behaviour is consistent in a cross browser fashion. As such,\nwe override in all cases when setting the value to control this.</p>\n"},"valueToRaw":{"!type":"fn(value: ?) -> ?","!doc":"<p>Converts a mixed-type value to a raw representation suitable for displaying in the field. This allows controlling\nhow value objects passed to <a href=\"#!/api/Ext.form.field.TextArea-method-setValue\" rel=\"Ext.form.field.TextArea-method-setValue\" class=\"docClass\">setValue</a> are shown to the user, including localization. For instance, for a\n<a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, this would control how a Date object passed to <a href=\"#!/api/Ext.form.field.TextArea-method-setValue\" rel=\"Ext.form.field.TextArea-method-setValue\" class=\"docClass\">setValue</a> would be converted\nto a String for display in the field.</p>\n\n<p>See <a href=\"#!/api/Ext.form.field.TextArea-method-rawToValue\" rel=\"Ext.form.field.TextArea-method-rawToValue\" class=\"docClass\">rawToValue</a> for the opposite conversion.</p>\n\n<p>The base implementation simply does a standard toString conversion, and converts empty values\nto an empty string.</p>\n"}}},"Time":{"!doc":"<p>Provides a time input field with a time dropdown and automatic time validation.</p>\n\n<p>This field recognizes and uses JavaScript Date objects as its main <a href=\"#!/api/Ext.form.field.Time-cfg-value\" rel=\"Ext.form.field.Time-cfg-value\" class=\"docClass\">value</a> type (only the time portion of the\ndate is used; the month/day/year are ignored). In addition, it recognizes string values which are parsed according to\nthe <a href=\"#!/api/Ext.form.field.Time-cfg-format\" rel=\"Ext.form.field.Time-cfg-format\" class=\"docClass\">format</a> and/or <a href=\"#!/api/Ext.form.field.Time-cfg-altFormats\" rel=\"Ext.form.field.Time-cfg-altFormats\" class=\"docClass\">altFormats</a> configs. These may be reconfigured to use time formats appropriate for\nthe user's locale.</p>\n\n<p>The field may be limited to a certain range of times by using the <a href=\"#!/api/Ext.form.field.Time-cfg-minValue\" rel=\"Ext.form.field.Time-cfg-minValue\" class=\"docClass\">minValue</a> and <a href=\"#!/api/Ext.form.field.Time-cfg-maxValue\" rel=\"Ext.form.field.Time-cfg-maxValue\" class=\"docClass\">maxValue</a> configs,\nand the interval between time options in the dropdown can be changed with the <a href=\"#!/api/Ext.form.field.Time-cfg-increment\" rel=\"Ext.form.field.Time-cfg-increment\" class=\"docClass\">increment</a> config.</p>\n\n<p>Example usage:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'Time Card',\n    width: 300,\n    bodyPadding: 10,\n    renderTo: Ext.getBody(),\n    items: [{\n        xtype: 'timefield',\n        name: 'in',\n        fieldLabel: 'Time In',\n        minValue: '6:00 AM',\n        maxValue: '8:00 PM',\n        increment: 30,\n        anchor: '100%'\n    }, {\n        xtype: 'timefield',\n        name: 'out',\n        fieldLabel: 'Time Out',\n        minValue: '6:00 AM',\n        maxValue: '8:00 PM',\n        increment: 30,\n        anchor: '100%'\n   }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"altFormats":{"!type":"string","!doc":"<p>Multiple date formats separated by \"|\" to try when parsing a user input value and it doesn't match the defined\nformat.</p>\n"},"displayField":{"!type":"string","!doc":"<p>The underlying data field name to bind to this ComboBox.</p>\n\n<p>See also <code><a href=\"#!/api/Ext.form.field.Time-cfg-valueField\" rel=\"Ext.form.field.Time-cfg-valueField\" class=\"docClass\">valueField</a></code>.</p>\n"},"format":{"!type":"string","!doc":"<p>The default time format string which can be overriden for localization support. The format must be valid\naccording to Ext.Date.parse.</p>\n\n<p>Defaults to <code>'g:i A'</code>, e.g., <code>'3:15 PM'</code>. For 24-hour time format try <code>'H:i'</code> instead.</p>\n"},"increment":{"!type":"number","!doc":"<p>The number of minutes between each time value in the list.</p>\n\n<p>Note that this only affects the <em>list of suggested times.</em></p>\n\n<p>To enforce that only times on the list are valid, use <a href=\"#!/api/Ext.form.field.Time-cfg-snapToIncrement\" rel=\"Ext.form.field.Time-cfg-snapToIncrement\" class=\"docClass\">snapToIncrement</a>. That will coerce\nany typed values to the nearest increment point upon blur.</p>\n"},"invalidText":{"!type":"string","!doc":"<p>The error text to display when the time in the field is invalid.</p>\n"},"maxText":{"!type":"string","!doc":"<p>The error text to display when the entered time is after <a href=\"#!/api/Ext.form.field.Time-cfg-maxValue\" rel=\"Ext.form.field.Time-cfg-maxValue\" class=\"docClass\">maxValue</a>.</p>\n"},"maxValue":{"!doc":"<p>The maximum allowed time. Can be either a Javascript date object with a valid time value or a string time in a\nvalid format -- see <a href=\"#!/api/Ext.form.field.Time-cfg-format\" rel=\"Ext.form.field.Time-cfg-format\" class=\"docClass\">format</a> and <a href=\"#!/api/Ext.form.field.Time-cfg-altFormats\" rel=\"Ext.form.field.Time-cfg-altFormats\" class=\"docClass\">altFormats</a>.</p>\n"},"minText":{"!type":"string","!doc":"<p>The error text to display when the entered time is before <a href=\"#!/api/Ext.form.field.Time-cfg-minValue\" rel=\"Ext.form.field.Time-cfg-minValue\" class=\"docClass\">minValue</a>.</p>\n"},"minValue":{"!doc":"<p>The minimum allowed time. Can be either a Javascript date object with a valid time value or a string time in a\nvalid format -- see <a href=\"#!/api/Ext.form.field.Time-cfg-format\" rel=\"Ext.form.field.Time-cfg-format\" class=\"docClass\">format</a> and <a href=\"#!/api/Ext.form.field.Time-cfg-altFormats\" rel=\"Ext.form.field.Time-cfg-altFormats\" class=\"docClass\">altFormats</a>.</p>\n"},"pickerMaxHeight":{"!type":"number","!doc":"<p>The maximum height of the <a href=\"#!/api/Ext.picker.Time\" rel=\"Ext.picker.Time\" class=\"docClass\">Ext.picker.Time</a> dropdown.</p>\n"},"queryMode":{"!type":"string","!doc":"<p>The mode in which the ComboBox uses the configured Store. Acceptable values are:</p>\n\n<ul>\n<li><p><strong><code>'remote'</code></strong> :</p>\n\n<p>In <code>queryMode: 'remote'</code>, the ComboBox loads its Store dynamically based upon user interaction.</p>\n\n<p>This is typically used for \"autocomplete\" type inputs, and after the user finishes typing, the Store is loaded.</p>\n\n<p>A parameter containing the typed string is sent in the load request. The default parameter name for the input\nstring is <code>query</code>, but this can be configured using the <a href=\"#!/api/Ext.form.field.Time-cfg-queryParam\" rel=\"Ext.form.field.Time-cfg-queryParam\" class=\"docClass\">queryParam</a> config.</p>\n\n<p>In <code>queryMode: 'remote'</code>, the Store may be configured with <code>remoteFilter:\ntrue</code>, and further filters may be <em>programatically</em> added to the Store which are then passed with every load\nrequest which allows the server to further refine the returned dataset.</p>\n\n<p>Typically, in an autocomplete situation, <a href=\"#!/api/Ext.form.field.Time-cfg-hideTrigger\" rel=\"Ext.form.field.Time-cfg-hideTrigger\" class=\"docClass\">hideTrigger</a> is configured <code>true</code> because it has no meaning for\nautocomplete.</p></li>\n<li><p><strong><code>'local'</code></strong> :</p>\n\n<p>ComboBox loads local data</p>\n\n<pre><code>var combo = new <a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.field.ComboBox</a>({\n    renderTo: document.body,\n    queryMode: 'local',\n    store: new Ext.data.ArrayStore({\n        id: 0,\n        fields: [\n            'myId',  // numeric value is the key\n            'displayText'\n        ],\n        data: [[1, 'item1'], [2, 'item2']]  // data is local\n    }),\n    valueField: 'myId',\n    displayField: 'displayText',\n    triggerAction: 'all'\n});\n</code></pre></li>\n</ul>\n\n"},"selectOnTab":{"!type":"bool","!doc":"<p>Whether the Tab key should select the currently highlighted item.</p>\n"},"snapToIncrement":{"!type":"bool","!doc":"<p>Specify as <code>true</code> to enforce that only values on the <a href=\"#!/api/Ext.form.field.Time-cfg-increment\" rel=\"Ext.form.field.Time-cfg-increment\" class=\"docClass\">increment</a> boundary are accepted.</p>\n\n<p>Typed values will be coerced to the nearest <a href=\"#!/api/Ext.form.field.Time-cfg-increment\" rel=\"Ext.form.field.Time-cfg-increment\" class=\"docClass\">increment</a> point on blur.</p>\n"},"submitFormat":{"!type":"string","!doc":"<p>The date format string which will be submitted to the server. The format must be valid according to\nExt.Date.parse.</p>\n\n<p>Defaults to <a href=\"#!/api/Ext.form.field.Time-cfg-format\" rel=\"Ext.form.field.Time-cfg-format\" class=\"docClass\">format</a>.</p>\n"},"triggerCls":{"!type":"string","!doc":"<p>An additional CSS class used to style the trigger button. The trigger will always get the <a href=\"#!/api/Ext.form.trigger.Trigger-property-baseCls\" rel=\"Ext.form.trigger.Trigger-property-baseCls\" class=\"docClass\">Ext.form.trigger.Trigger.baseCls</a>\nby default and triggerCls will be <strong>appended</strong> if specified.</p>\n"},"valueField":{"!type":"string","!doc":"<p>The underlying data value name to bind to this ComboBox.</p>\n\n<p><strong>Note</strong>: use of a <code>valueField</code> requires the user to make a selection in order for a value to be mapped. See also\n<code><a href=\"#!/api/Ext.form.field.Time-cfg-displayField\" rel=\"Ext.form.field.Time-cfg-displayField\" class=\"docClass\">displayField</a></code>.</p>\n\n<p>Defaults to match the value of the <a href=\"#!/api/Ext.form.field.Time-cfg-displayField\" rel=\"Ext.form.field.Time-cfg-displayField\" class=\"docClass\">displayField</a> config.</p>\n"},"ignoreSelection":{"!type":"number","!doc":"<p>private</p>\n"},"initDate":{"!type":"string","!doc":"<p>This is the date to use when generating time values in the absence of either minValue\nor maxValue.  Using the current date causes DST issues on DST boundary dates, so this is an\narbitrary \"safe\" date that can be any date aside from DST boundary dates.</p>\n"},"initDateFormat":{"!type":"string"},"initDateParts":{},"createPicker":{"!type":"fn() -> !this","!doc":"<p>Creates the <a href=\"#!/api/Ext.picker.Time\" rel=\"Ext.picker.Time\" class=\"docClass\">Ext.picker.Time</a></p>\n"},"findRecordByValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Finds the record by searching values in the <a href=\"#!/api/Ext.form.field.Time-cfg-valueField\" rel=\"Ext.form.field.Time-cfg-valueField\" class=\"docClass\">valueField</a>.</p>\n"},"formatDate":{"!type":"fn(items: ?) -> !this"},"getErrors":{"!type":"fn(value?: ?) -> ?","!doc":"<p>Runs all of Time's validations and returns an array of any errors. Note that this first runs Text's validations,\nso the returned array is an amalgamation of all field errors. The additional validation checks are testing that\nthe time format is valid, that the chosen time is within the <a href=\"#!/api/Ext.form.field.Time-cfg-minValue\" rel=\"Ext.form.field.Time-cfg-minValue\" class=\"docClass\">minValue</a> and <a href=\"#!/api/Ext.form.field.Time-cfg-maxValue\" rel=\"Ext.form.field.Time-cfg-maxValue\" class=\"docClass\">maxValue</a> constraints\nset.</p>\n"},"getInitDate":{"!type":"fn(hours: ?, minutes: ?, seconds: ?) -> !this"},"getSubmitValue":{"!type":"fn() -> string","!doc":"<p>Returns the value that would be included in a standard form submit for this field. This will be combined with the\nfield's name to form a name=value pair in the <a href=\"#!/api/Ext.form.field.Time-method-getSubmitData\" rel=\"Ext.form.field.Time-method-getSubmitData\" class=\"docClass\">submitted parameters</a>. If an empty string is\nreturned then just the name= will be submitted; if null is returned then nothing will be submitted.</p>\n\n<p>Note that the value returned will have been <a href=\"#!/api/Ext.form.field.Time-method-processRawValue\" rel=\"Ext.form.field.Time-method-processRawValue\" class=\"docClass\">processed</a> but may or may not have been\nsuccessfully <a href=\"#!/api/Ext.form.field.Time-method-validate\" rel=\"Ext.form.field.Time-method-validate\" class=\"docClass\">validated</a>.</p>\n"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Returns the current data value of the field. The type of value returned is particular to the type of the\nparticular field (e.g. a Date object for <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>), as the result of calling <a href=\"#!/api/Ext.form.field.Time-method-rawToValue\" rel=\"Ext.form.field.Time-method-rawToValue\" class=\"docClass\">rawToValue</a> on\nthe field's <a href=\"#!/api/Ext.form.field.Time-method-processRawValue\" rel=\"Ext.form.field.Time-method-processRawValue\" class=\"docClass\">processed</a> String value. To return the raw String value, see <a href=\"#!/api/Ext.form.field.Time-method-getRawValue\" rel=\"Ext.form.field.Time-method-getRawValue\" class=\"docClass\">getRawValue</a>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"isEqual":{"!type":"fn(v1: ?, v2: ?) -> !this"},"onItemClick":{"!type":"fn(picker: ?, record: ?) -> !this"},"parseDate":{"!type":"fn(value: ?) -> !this","!doc":"<p>Parses an input value into a valid Date object.</p>\n"},"postBlur":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"rawToValue":{"!type":"fn(item: ?) -> ?","!doc":"<p>Converts a raw input field value into a mixed-type value that is suitable for this particular field type. This\nallows controlling the normalization and conversion of user-entered values into field-type-appropriate values,\ne.g. a Date object for <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, and is invoked by <a href=\"#!/api/Ext.form.field.Time-method-getValue\" rel=\"Ext.form.field.Time-method-getValue\" class=\"docClass\">getValue</a>.</p>\n\n<p>It is up to individual implementations to decide how to handle raw values that cannot be successfully converted\nto the desired object type.</p>\n\n<p>See <a href=\"#!/api/Ext.form.field.Time-method-valueToRaw\" rel=\"Ext.form.field.Time-method-valueToRaw\" class=\"docClass\">valueToRaw</a> for the opposite conversion.</p>\n\n<p>The base implementation does no conversion, returning the raw value untouched.</p>\n"},"safeParse":{"!type":"fn(value: ?, format: ?) -> !this"},"setLimit":{"!type":"fn(value: ?, isMin: ?) -> !this","!doc":"<p>Updates either the min or max value. Converts the user's value into a Date object whose\nyear/month/day is set to the <a href=\"#!/api/Ext.form.field.Time-property-initDate\" rel=\"Ext.form.field.Time-property-initDate\" class=\"docClass\">initDate</a> so that only the time fields are significant.</p>\n"},"setMaxValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Replaces any existing <a href=\"#!/api/Ext.form.field.Time-cfg-maxValue\" rel=\"Ext.form.field.Time-cfg-maxValue\" class=\"docClass\">maxValue</a> with the new time and refreshes the picker's range.</p>\n"},"setMinValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Replaces any existing <a href=\"#!/api/Ext.form.field.Time-cfg-minValue\" rel=\"Ext.form.field.Time-cfg-minValue\" class=\"docClass\">minValue</a> with the new time and refreshes the picker's range.</p>\n"},"setValue":{"!type":"fn(v: ?) -> ?","!doc":"<p>Sets the specified value(s) into the field. For each value, if a record is found in the <a href=\"#!/api/Ext.form.field.Time-cfg-store\" rel=\"Ext.form.field.Time-cfg-store\" class=\"docClass\">store</a> that\nmatches based on the <a href=\"#!/api/Ext.form.field.Time-cfg-valueField\" rel=\"Ext.form.field.Time-cfg-valueField\" class=\"docClass\">valueField</a>, then that record's <a href=\"#!/api/Ext.form.field.Time-cfg-displayField\" rel=\"Ext.form.field.Time-cfg-displayField\" class=\"docClass\">displayField</a> will be displayed in the\nfield. If no match is found, and the <a href=\"#!/api/Ext.form.field.Time-cfg-valueNotFoundText\" rel=\"Ext.form.field.Time-cfg-valueNotFoundText\" class=\"docClass\">valueNotFoundText</a> config option is defined, then that will be\ndisplayed as the default field text. Otherwise a blank value will be shown, although the value will still be set.</p>\n"},"syncSelection":{"!type":"fn() -> !this","!doc":"<p>Synchronizes the selection in the picker to match the current value</p>\n"},"valueToRaw":{"!type":"fn(value: ?) -> ?","!doc":"<p>Converts a mixed-type value to a raw representation suitable for displaying in the field. This allows controlling\nhow value objects passed to <a href=\"#!/api/Ext.form.field.Time-method-setValue\" rel=\"Ext.form.field.Time-method-setValue\" class=\"docClass\">setValue</a> are shown to the user, including localization. For instance, for a\n<a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a>, this would control how a Date object passed to <a href=\"#!/api/Ext.form.field.Time-method-setValue\" rel=\"Ext.form.field.Time-method-setValue\" class=\"docClass\">setValue</a> would be converted\nto a String for display in the field.</p>\n\n<p>See <a href=\"#!/api/Ext.form.field.Time-method-rawToValue\" rel=\"Ext.form.field.Time-method-rawToValue\" class=\"docClass\">rawToValue</a> for the opposite conversion.</p>\n\n<p>The base implementation simply does a standard toString conversion, and converts empty values\nto an empty string.</p>\n"}}},"Trigger":{"!type":"fn(config: ?)","prototype":{"triggerCls":{"!type":"string","!doc":"<p>An additional CSS class used to style the trigger button. The trigger will always get the <a href=\"#!/api/Ext.form.trigger.Trigger-property-baseCls\" rel=\"Ext.form.trigger.Trigger-property-baseCls\" class=\"docClass\">Ext.form.trigger.Trigger.baseCls</a>\nby default and triggerCls will be <strong>appended</strong> if specified.</p>\n"},"onClassExtended":{"!type":"fn() -> !this"},"warnDeprecated":{"!type":"fn() -> !this"}}},"VTypes":{"!doc":"<p>This is a singleton object which contains a set of commonly used field validation functions\nand provides a mechanism for creating reusable custom field validations.\nThe following field validation functions are provided out of the box:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.form.field.VTypes-method-alpha\" rel=\"Ext.form.field.VTypes-method-alpha\" class=\"docClass\">alpha</a></li>\n<li><a href=\"#!/api/Ext.form.field.VTypes-method-alphanum\" rel=\"Ext.form.field.VTypes-method-alphanum\" class=\"docClass\">alphanum</a></li>\n<li><a href=\"#!/api/Ext.form.field.VTypes-method-email\" rel=\"Ext.form.field.VTypes-method-email\" class=\"docClass\">email</a></li>\n<li><a href=\"#!/api/Ext.form.field.VTypes-method-url\" rel=\"Ext.form.field.VTypes-method-url\" class=\"docClass\">url</a></li>\n</ul>\n\n\n<p>VTypes can be applied to a <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Text Field</a> using the <code><a href=\"#!/api/Ext.form.field.Text-cfg-vtype\" rel=\"Ext.form.field.Text-cfg-vtype\" class=\"docClass\">vtype</a></code> configuration:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n    fieldLabel: 'Email Address',\n    name: 'email',\n    vtype: 'email' // applies email validation rules to this field\n});\n</code></pre>\n\n<p>To create custom VTypes:</p>\n\n<pre><code>// custom Vtype for vtype:'time'\nvar timeTest = /^([1-9]|1[0-9]):([0-5][0-9])(\\s[a|p]m)$/i;\nExt.apply(<a href=\"#!/api/Ext.form.field.VTypes\" rel=\"Ext.form.field.VTypes\" class=\"docClass\">Ext.form.field.VTypes</a>, {\n    //  vtype validation function\n    time: function(val, field) {\n        return timeTest.test(val);\n    },\n    // vtype Text property: The error text to display when the validation function returns false\n    timeText: 'Not a valid time.  Must be in the format \"12:34 PM\".',\n    // vtype Mask property: The keystroke filter mask\n    timeMask: /[\\d\\s:amp]/i\n});\n</code></pre>\n\n<p>In the above example the <code>time</code> function is the validator that will run when field validation occurs,\n<code>timeText</code> is the error message, and <code>timeMask</code> limits what characters can be typed into the field.\nNote that the <code>Text</code> and <code>Mask</code> functions must begin with the same name as the validator function.</p>\n\n<p>Using a custom validator is the same as using one of the build-in validators - just use the name of the validator function\nas the <code><a href=\"#!/api/Ext.form.field.Text-cfg-vtype\" rel=\"Ext.form.field.Text-cfg-vtype\" class=\"docClass\">vtype</a></code> configuration on a <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Text Field</a>:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a>', {\n    fieldLabel: 'Departure Time',\n    name: 'departureTime',\n    vtype: 'time' // applies custom time validation rules to this field\n});\n</code></pre>\n\n<p>Another example of a custom validator:</p>\n\n<pre><code>// custom Vtype for vtype:'IPAddress'\nExt.apply(<a href=\"#!/api/Ext.form.field.VTypes\" rel=\"Ext.form.field.VTypes\" class=\"docClass\">Ext.form.field.VTypes</a>, {\n    IPAddress:  function(v) {\n        return /^\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}$/.test(v);\n    },\n    IPAddressText: 'Must be a numeric IP address',\n    IPAddressMask: /[\\d\\.]/i\n});\n</code></pre>\n\n<p>It's important to note that using Ext.apply() means that the custom validator function\nas well as <code>Text</code> and <code>Mask</code> fields are added as properties of the <code><a href=\"#!/api/Ext.form.field.VTypes\" rel=\"Ext.form.field.VTypes\" class=\"docClass\">Ext.form.field.VTypes</a></code> singleton.</p>\n","alphaMask":{"!doc":"<p>The keystroke filter mask to be applied on alpha input. Defaults to: /[a-z_]/i</p>\n"},"alphaText":{"!type":"string","!doc":"<p>The error text to display when the alpha validation function returns false.\nDefaults to: 'This field should only contain letters and _'</p>\n"},"alphanumMask":{"!doc":"<p>The keystroke filter mask to be applied on alphanumeric input. Defaults to: /[a-z0-9_]/i</p>\n"},"alphanumText":{"!type":"string","!doc":"<p>The error text to display when the alphanumeric validation function returns false.\nDefaults to: 'This field should only contain letters, numbers and _'</p>\n"},"emailMask":{"!doc":"<p>The keystroke filter mask to be applied on email input. See the <a href=\"#!/api/Ext.form.field.VTypes-method-email\" rel=\"Ext.form.field.VTypes-method-email\" class=\"docClass\">email</a> method for information about\nmore complex email validation. Defaults to: /[a-z0-9_.-@]/i</p>\n"},"emailText":{"!type":"string","!doc":"<p>The error text to display when the email validation function returns false.\nDefaults to: 'This field should be an e-mail address in the format \"user@example.com\"'</p>\n"},"urlText":{"!type":"string","!doc":"<p>The error text to display when the url validation function returns false.\nDefaults to: 'This field should be a URL in the format \"http:/'+'/www.example.com\"'</p>\n"},"alpha":{"!type":"fn(value: ?) -> bool","!doc":"<p>The function used to validate alpha values</p>\n"},"alphanum":{"!type":"fn(value: ?) -> bool","!doc":"<p>The function used to validate alphanumeric values</p>\n"},"email":{"!type":"fn(value: ?) -> bool","!doc":"<p>The function used to validate email addresses. Note that complete validation per the email RFC\nspecifications is very complex and beyond the scope of this class, although this function can be\noverridden if a more comprehensive validation scheme is desired. See the validation section\nof the <a href=\"http://en.wikipedia.org/wiki/E-mail_address\">Wikipedia article on email addresses</a> for additional information. This implementation is\nintended to validate the following types of emails:</p>\n\n<ul>\n<li><code>barney@example.de</code></li>\n<li><code>barney.rubble@example.com</code></li>\n<li><code>barney-rubble@example.coop</code></li>\n<li><code>barney+rubble@example.com</code></li>\n<li><code>barney'rubble@example.com</code></li>\n<li><code>b.arne.y_r.ubbl.e@example.com</code></li>\n<li><code>barney4rubble@example.com</code></li>\n<li><code>barney4rubble!@example.com</code></li>\n<li><code>_barney+rubble@example.com</code></li>\n<li><code>\"barney+rubble\"@example.com</code></li>\n</ul>\n\n"},"url":{"!type":"fn(value: ?) -> bool","!doc":"<p>The function used to validate URLs</p>\n"}}},"FieldAncestor":{"prototype":{"fieldDefaults":{"!doc":"<p>If specified, the properties in this object are used as default config values for each <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">Ext.form.Labelable</a>\ninstance (e.g. <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a> or <a href=\"#!/api/Ext.form.FieldContainer\" rel=\"Ext.form.FieldContainer\" class=\"docClass\">Ext.form.FieldContainer</a>) that is added as a descendant of\nthis container. Corresponding values specified in an individual field's own configuration, or from the <a href=\"#!/api/Ext.container.Container-cfg-defaults\" rel=\"Ext.container.Container-cfg-defaults\" class=\"docClass\">defaults config</a> of its parent container, will take precedence. See the\ndocumentation for <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">Ext.form.Labelable</a> to see what config options may be specified in the fieldDefaults.</p>\n\n<p>Example:</p>\n\n<pre><code>new <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>({\n    fieldDefaults: {\n        labelAlign: 'left',\n        labelWidth: 100\n    },\n    items: [{\n        xtype: 'fieldset',\n        defaults: {\n            labelAlign: 'top'\n        },\n        items: [{\n            name: 'field1'\n        }, {\n            name: 'field2'\n        }]\n    }, {\n        xtype: 'fieldset',\n        items: [{\n            name: 'field3',\n            labelWidth: 150\n        }, {\n            name: 'field4'\n        }]\n    }]\n});\n</code></pre>\n\n<p>In this example, field1 and field2 will get labelAlign:'top' (from the fieldset's defaults) and labelWidth:100\n(from fieldDefaults), field3 and field4 will both get labelAlign:'left' (from fieldDefaults and field3 will use\nthe labelWidth:150 from its own config.</p>\n"},"mixinConfig":{},"beforeDestroy":{"!type":"fn() -> !this"},"handleFieldErrorChange":{"!type":"fn(labelable: ?, activeError: ?) -> !this","!doc":"<p>Handle bubbled errorchange events from descendants; invoke the aggregated event and method</p>\n"},"handleFieldValidityChange":{"!type":"fn(field: ?, isValid: ?) -> !this","!doc":"<p>Handle bubbled validitychange events from descendants; invoke the aggregated event and method</p>\n"},"initFieldAncestor":{"!type":"fn() -> !this","!doc":"<p>Initializes the FieldAncestor's state; this must be called from the initComponent method of any components\nimporting this mixin.</p>\n"},"initFieldDefaults":{"!type":"fn() -> !this","!doc":"<p>Initialize the <a href=\"#!/api/Ext.form.FieldAncestor-cfg-fieldDefaults\" rel=\"Ext.form.FieldAncestor-cfg-fieldDefaults\" class=\"docClass\">fieldDefaults</a> object</p>\n"},"initFieldInheritedState":{"!type":"fn(inheritedState: ?) -> !this"},"initMonitor":{"!type":"fn() -> !this"},"onChildFieldAdd":{"!type":"fn(field: ?) -> !this"},"onChildFieldRemove":{"!type":"fn(field: ?) -> !this"},"onFieldValidityChange":{"!type":"fn(field: ?, valid: ?) -> !this","!doc":"<p>Fired when the validity of any field within the container changes.</p>\n"},"fielderrorchange":{"!type":"fn(this: ?, The: ?, error: ?, eOpts: ?)","!doc":"<p>Fires when the active error message is changed for any one of the <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">Ext.form.Labelable</a> instances\nwithin this container.</p>\n"},"fieldvaliditychange":{"!type":"fn(this: ?, The: ?, isValid: ?, eOpts: ?)","!doc":"<p>Fires when the validity state of any one of the <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Ext.form.field.Field</a> instances within this\ncontainer changes.</p>\n"},"onFieldErrorChange":{"!type":"fn(field: ?, error: ?) -> !this","!doc":"<p>Fired when the error message of any field within the container changes.</p>\n"}},"!doc":"<p>A mixin for <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a> components that are likely to have form fields in their\nitems subtree. Adds the following capabilities:</p>\n\n<ul>\n<li>Methods for handling the addition and removal of <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">Ext.form.Labelable</a> and <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Ext.form.field.Field</a>\ninstances at any depth within the container.</li>\n<li>Events (<a href=\"#!/api/Ext.form.FieldAncestor-event-fieldvaliditychange\" rel=\"Ext.form.FieldAncestor-event-fieldvaliditychange\" class=\"docClass\">fieldvaliditychange</a> and <a href=\"#!/api/Ext.form.FieldAncestor-event-fielderrorchange\" rel=\"Ext.form.FieldAncestor-event-fielderrorchange\" class=\"docClass\">fielderrorchange</a>) for handling changes to the state\nof individual fields at the container level.</li>\n<li>Automatic application of <a href=\"#!/api/Ext.form.FieldAncestor-cfg-fieldDefaults\" rel=\"Ext.form.FieldAncestor-cfg-fieldDefaults\" class=\"docClass\">fieldDefaults</a> config properties to each field added within the\ncontainer, to facilitate uniform configuration of all fields.</li>\n</ul>\n\n\n<p>This mixin is primarily for internal use by <a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a> and <a href=\"#!/api/Ext.form.FieldContainer\" rel=\"Ext.form.FieldContainer\" class=\"docClass\">Ext.form.FieldContainer</a>,\nand should not normally need to be used directly.</p>\n"},"CheckboxManager":{"!doc":"<p>Private utility class for managing all <a href=\"#!/api/Ext.form.field.Checkbox\" rel=\"Ext.form.field.Checkbox\" class=\"docClass\">Ext.form.field.Checkbox</a> fields grouped by name.</p>\n","getByName":{"!type":"fn(name: ?, formId: ?) -> !this"}},"FieldSet":{"!doc":"<p>A container for grouping sets of fields, rendered as a HTML <code>fieldset</code> element. The <a href=\"#!/api/Ext.form.FieldSet-cfg-title\" rel=\"Ext.form.FieldSet-cfg-title\" class=\"docClass\">title</a>\nconfig will be rendered as the fieldset's <code>legend</code>.</p>\n\n<p>While FieldSets commonly contain simple groups of fields, they are general <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Containers</a>\nand may therefore contain any type of components in their <a href=\"#!/api/Ext.form.FieldSet-cfg-items\" rel=\"Ext.form.FieldSet-cfg-items\" class=\"docClass\">items</a>, including other nested containers.\nThe default <a href=\"#!/api/Ext.form.FieldSet-cfg-layout\" rel=\"Ext.form.FieldSet-cfg-layout\" class=\"docClass\">layout</a> for the FieldSet's items is <code>'anchor'</code>, but it can be configured to use any other\nlayout type.</p>\n\n<p>FieldSets may also be collapsed if configured to do so; this can be done in two ways:</p>\n\n<ol>\n<li>Set the <a href=\"#!/api/Ext.form.FieldSet-cfg-collapsible\" rel=\"Ext.form.FieldSet-cfg-collapsible\" class=\"docClass\">collapsible</a> config to true; this will result in a collapse button being rendered next to\nthe <a href=\"#!/api/Ext.form.FieldSet-cfg-title\" rel=\"Ext.form.FieldSet-cfg-title\" class=\"docClass\">legend title</a>, or:</li>\n<li>Set the <a href=\"#!/api/Ext.form.FieldSet-cfg-checkboxToggle\" rel=\"Ext.form.FieldSet-cfg-checkboxToggle\" class=\"docClass\">checkboxToggle</a> config to true; this is similar to using <a href=\"#!/api/Ext.form.FieldSet-cfg-collapsible\" rel=\"Ext.form.FieldSet-cfg-collapsible\" class=\"docClass\">collapsible</a> but renders\na <a href=\"#!/api/Ext.form.field.Checkbox\" rel=\"Ext.form.field.Checkbox\" class=\"docClass\">checkbox</a> in place of the toggle button. The fieldset will be expanded when the\ncheckbox is checked and collapsed when it is unchecked. The checkbox will also be included in the\n<a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">form submit parameters</a> using the <a href=\"#!/api/Ext.form.FieldSet-cfg-checkboxName\" rel=\"Ext.form.FieldSet-cfg-checkboxName\" class=\"docClass\">checkboxName</a> as its parameter name.</li>\n</ol>\n\n\n<h1>Example usage</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'Simple Form with FieldSets',\n    labelWidth: 75, // label settings here cascade unless overridden\n    url: 'save-form.php',\n    frame: true,\n    bodyStyle: 'padding:5px 5px 0',\n    width: 550,\n    renderTo: Ext.getBody(),\n    layout: 'column', // arrange fieldsets side by side\n    items: [{\n        // Fieldset in Column 1 - collapsible via toggle button\n        xtype:'fieldset',\n        columnWidth: 0.5,\n        title: 'Fieldset 1',\n        collapsible: true,\n        defaultType: 'textfield',\n        defaults: {anchor: '100%'},\n        layout: 'anchor',\n        items :[{\n            fieldLabel: 'Field 1',\n            name: 'field1'\n        }, {\n            fieldLabel: 'Field 2',\n            name: 'field2'\n        }]\n    }, {\n        // Fieldset in Column 2 - collapsible via checkbox, collapsed by default, contains a panel\n        xtype:'fieldset',\n        title: 'Show Panel', // title or checkboxToggle creates fieldset header\n        columnWidth: 0.5,\n        checkboxToggle: true,\n        collapsed: true, // fieldset initially collapsed\n        layout:'anchor',\n        items :[{\n            xtype: 'panel',\n            anchor: '100%',\n            title: 'Panel inside a fieldset',\n            frame: true,\n            height: 52\n        }]\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"autoEl":{"!doc":"<p>A tag name or DomHelper spec used to create the <a href=\"#!/api/Ext.form.FieldSet-method-getEl\" rel=\"Ext.form.FieldSet-method-getEl\" class=\"docClass\">Element</a> which will\nencapsulate this Component.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong>'div'</strong>. The more complex Sencha classes use a more\ncomplex DOM structure specified by their own <a href=\"#!/api/Ext.form.FieldSet-cfg-renderTpl\" rel=\"Ext.form.FieldSet-cfg-renderTpl\" class=\"docClass\">renderTpl</a>s.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components encapsulated by\ndifferent DOM elements. Example usage:</p>\n\n<pre><code>{\n    xtype: 'component',\n    autoEl: {\n        tag: 'img',\n        src: 'http://www.example.com/example.jpg'\n    }\n}, {\n    xtype: 'component',\n    autoEl: {\n        tag: 'blockquote',\n        html: 'autoEl is cool!'\n    }\n}, {\n    xtype: 'container',\n    autoEl: 'ul',\n    cls: 'ux-unordered-list',\n    items: {\n        xtype: 'component',\n        autoEl: 'li',\n        html: 'First list item'\n    }\n}\n</code></pre>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class applied to the fieldset.</p>\n"},"checkboxName":{"!type":"string","!doc":"<p>The name to assign to the fieldset's checkbox if <a href=\"#!/api/Ext.form.FieldSet-cfg-checkboxToggle\" rel=\"Ext.form.FieldSet-cfg-checkboxToggle\" class=\"docClass\">checkboxToggle</a> = true\n(defaults to '[fieldset id]-checkbox').</p>\n"},"checkboxToggle":{"!type":"bool","!doc":"<p>Set to true to render a checkbox into the fieldset frame just in front of the legend to expand/collapse the\nfieldset when the checkbox is toggled.. This checkbox will be included in form submits using\nthe <a href=\"#!/api/Ext.form.FieldSet-cfg-checkboxName\" rel=\"Ext.form.FieldSet-cfg-checkboxName\" class=\"docClass\">checkboxName</a>.</p>\n"},"checkboxUI":{"!type":"string","!doc":"<p>The ui to use for the fieldset's checkbox.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"collapsed":{"!type":"bool","!doc":"<p>Set to true to render the fieldset as collapsed by default. If <a href=\"#!/api/Ext.form.FieldSet-cfg-checkboxToggle\" rel=\"Ext.form.FieldSet-cfg-checkboxToggle\" class=\"docClass\">checkboxToggle</a> is specified, the checkbox\nwill also be unchecked by default.</p>\n"},"collapsible":{"!type":"bool","!doc":"<p>Set to true to make the fieldset collapsible and have the expand/collapse toggle button automatically rendered\ninto the legend element, false to keep the fieldset statically sized with no collapse button.\nAnother option is to configure <a href=\"#!/api/Ext.form.FieldSet-cfg-checkboxToggle\" rel=\"Ext.form.FieldSet-cfg-checkboxToggle\" class=\"docClass\">checkboxToggle</a>. Use the <a href=\"#!/api/Ext.form.FieldSet-cfg-collapsed\" rel=\"Ext.form.FieldSet-cfg-collapsed\" class=\"docClass\">collapsed</a> config to collapse the\nfieldset by default.</p>\n"},"componentLayout":{"!doc":"<p>The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout\nmanager which sizes a Component's internal structure in response to the Component being sized.</p>\n\n<p>Generally, developers will not use this configuration as all provided Components which need their internal\nelements sizing (Such as <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">input fields</a>) come with their own componentLayout managers.</p>\n\n<p>The <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used on instances of the base <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>\nclass which simply sizes the Component's encapsulating element to the height and width specified in the\n<a href=\"#!/api/Ext.form.FieldSet-method-setSize\" rel=\"Ext.form.FieldSet-method-setSize\" class=\"docClass\">setSize</a> method.</p>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"layout":{"!doc":"<p>The <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">Ext.container.Container.layout</a> for the fieldset's immediate child items.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.form.FieldSet-method-getEl\" rel=\"Ext.form.FieldSet-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.form.FieldSet-method-getEl\" rel=\"Ext.form.FieldSet-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.form.FieldSet-cfg-renderSelectors\" rel=\"Ext.form.FieldSet-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.form.FieldSet-cfg-childEls\" rel=\"Ext.form.FieldSet-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"stateEvents":{"!doc":"<p>&lt;An array of events that, when fired, should trigger this object to\nsave its state. Defaults to none. <code>stateEvents</code> may be any type\nof event supported by this object, including browser or custom events\n(e.g., <code>['click', 'customerchange']</code>).</p>\n\n<p>See <code><a href=\"#!/api/Ext.form.FieldSet-cfg-stateful\" rel=\"Ext.form.FieldSet-cfg-stateful\" class=\"docClass\">stateful</a></code> for an explanation of saving and\nrestoring object state.</p>\n"},"title":{"!type":"string","!doc":"<p>A title to be displayed in the fieldset's legend. May contain HTML markup.</p>\n"},"toggleOnTitleClick":{"!type":"bool","!doc":"<p>Set to true will add a listener to the titleCmp property for the click event which will execute the\n<a href=\"#!/api/Ext.form.FieldSet-method-toggle\" rel=\"Ext.form.FieldSet-method-toggle\" class=\"docClass\">toggle</a> method. This option is only used when the <a href=\"#!/api/Ext.form.FieldSet-cfg-collapsible\" rel=\"Ext.form.FieldSet-cfg-collapsible\" class=\"docClass\">collapsible</a> property is set to true.</p>\n"},"ariaRole":{"!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"checkboxCmp":{"!doc":"<p>Refers to the <a href=\"#!/api/Ext.form.field.Checkbox\" rel=\"Ext.form.field.Checkbox\" class=\"docClass\">Ext.form.field.Checkbox</a> component that is added next to the title in the legend. Only\npopulated if the fieldset is configured with <a href=\"#!/api/Ext.form.FieldSet-cfg-checkboxToggle\" rel=\"Ext.form.FieldSet-cfg-checkboxToggle\" class=\"docClass\">checkboxToggle</a>:true.</p>\n"},"legend":{"!doc":"<p>The component for the fieldset's legend. Will only be defined if the configuration requires a legend to be\ncreated, by setting the <a href=\"#!/api/Ext.form.FieldSet-cfg-title\" rel=\"Ext.form.FieldSet-cfg-title\" class=\"docClass\">title</a> or <a href=\"#!/api/Ext.form.FieldSet-cfg-checkboxToggle\" rel=\"Ext.form.FieldSet-cfg-checkboxToggle\" class=\"docClass\">checkboxToggle</a> options.</p>\n"},"maskOnDisable":{"!type":"bool","!doc":"<p>This is an internal flag that you use when creating custom components. By default this is set to <code>true</code> which means\nthat every component gets a mask when it's disabled. Components like FieldContainer, FieldSet, Field, Button, Tab\noverride this property to <code>false</code> since they want to implement custom disable logic.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"toggleCmp":{"!doc":"<p>Refers to the <a href=\"#!/api/Ext.panel.Tool\" rel=\"Ext.panel.Tool\" class=\"docClass\">Ext.panel.Tool</a> component that is added as the collapse/expand button next to the title in\nthe legend. Only populated if the fieldset is configured with <a href=\"#!/api/Ext.form.FieldSet-cfg-collapsible\" rel=\"Ext.form.FieldSet-cfg-collapsible\" class=\"docClass\">collapsible</a>:true.</p>\n"},"addTitleClasses":{"!type":"fn() -> !this"},"afterCollapse":{"!type":"fn() -> !this"},"afterExpand":{"!type":"fn() -> !this"},"collapse":{"!type":"fn(fieldset: ?, eOpts: ?)","!doc":"<p>Fires after this FieldSet has collapsed.</p>\n"},"collapsedHorizontal":{"!type":"fn() -> !this"},"collapsedVertical":{"!type":"fn() -> !this"},"createCheckboxCmp":{"!type":"fn() -> ?","!doc":"<p>Creates the checkbox component. This is only called internally, but could be overridden in subclasses to\ncustomize the checkbox's configuration or even return an entirely different component type.</p>\n"},"createLegendCt":{"!type":"fn() -> !this"},"createTitleCmp":{"!type":"fn() -> ?","!doc":"<p>Creates the legend title component. This is only called internally, but could be overridden in subclasses to\ncustomize the title component. If <a href=\"#!/api/Ext.form.FieldSet-cfg-toggleOnTitleClick\" rel=\"Ext.form.FieldSet-cfg-toggleOnTitleClick\" class=\"docClass\">toggleOnTitleClick</a> is set to true, a listener for the click event\nwill toggle the collapsed state of the FieldSet.</p>\n"},"createToggleCmp":{"!type":"fn() -> ?","!doc":"<p>Creates the toggle button component. This is only called internally, but could be overridden in subclasses to\ncustomize the toggle component.</p>\n"},"doRenderLegend":{"!type":"fn(out: ?, renderData: ?) -> !this"},"expand":{"!type":"fn(fieldset: ?, eOpts: ?)","!doc":"<p>Fires after this FieldSet has expanded.</p>\n"},"getCollapsed":{"!type":"fn() -> !this"},"getCollapsedDockedItems":{"!type":"fn() -> !this"},"getRefItems":{"!type":"fn(deep: ?) -> !this","!doc":"<p>Used by ComponentQuery, child and down to retrieve all of the items\nwhich can potentially be considered a child of this Container.</p>\n\n<p>This may be overriden by Components which have ownership of Components\nthat are not contained in the <a href=\"#!/api/Ext.form.FieldSet-property-items\" rel=\"Ext.form.FieldSet-property-items\" class=\"docClass\">items</a> collection.</p>\n\n<p>NOTE: IMPORTANT note for maintainers:\nItems are returned in tree traversal order. Each item is appended to the result array\nfollowed by the results of that child's getRefItems call.\nFloating child items are appended after internal child items.</p>\n"},"getState":{"!type":"fn() -> ?","!doc":"<p>The supplied default state gathering method for the Component class.</p>\n\n<p>This method returns dimension settings such as <code>flex</code>, <code>anchor</code>, <code>width</code> and <code>height</code> along with <code>collapsed</code>\nstate.</p>\n\n<p>Subclasses which implement more complex state should call the superclass's implementation, and apply their state\nto the result if this basic state is to be saved.</p>\n\n<p>Note that Component state will only be saved if the Component has a <a href=\"#!/api/Ext.form.FieldSet-cfg-stateId\" rel=\"Ext.form.FieldSet-cfg-stateId\" class=\"docClass\">stateId</a> and there as a StateProvider\nconfigured for the document.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"onCheckChange":{"!type":"fn(cmp: ?, checked: ?) -> !this","!doc":"<p>Handle changes in the checkbox checked state.</p>\n"},"setExpanded":{"!type":"fn(expanded: ?) -> !this","!doc":"<p>Collapse or expand the fieldset.</p>\n"},"setTitle":{"!type":"fn(title: ?) -> ?","!doc":"<p>Sets the title of this fieldset.</p>\n"},"toggle":{"!type":"fn() -> !this","!doc":"<p>Toggle the fieldset's collapsed state to the opposite of what it is currently.</p>\n"},"beforecollapse":{"!type":"fn(fieldset: ?, eOpts: ?)","!doc":"<p>Fires before this FieldSet is collapsed. Return false to prevent the collapse.</p>\n"},"beforeexpand":{"!type":"fn(fieldset: ?, eOpts: ?)","!doc":"<p>Fires before this FieldSet is expanded. Return false to prevent the expand.</p>\n"}}},"Label":{"!doc":"<p>Produces a standalone <code>&lt;label /&gt;</code> element which can be inserted into a form and be associated with a field\nin that form using the <a href=\"#!/api/Ext.form.Label-cfg-forId\" rel=\"Ext.form.Label-cfg-forId\" class=\"docClass\">forId</a> property.</p>\n\n<p><strong>NOTE:</strong> in most cases it will be more appropriate to use the <a href=\"#!/api/Ext.form.Labelable-cfg-fieldLabel\" rel=\"Ext.form.Labelable-cfg-fieldLabel\" class=\"docClass\">fieldLabel</a>\nand associated config properties (<a href=\"#!/api/Ext.form.Labelable-cfg-labelAlign\" rel=\"Ext.form.Labelable-cfg-labelAlign\" class=\"docClass\">Ext.form.Labelable.labelAlign</a>, <a href=\"#!/api/Ext.form.Labelable-cfg-labelWidth\" rel=\"Ext.form.Labelable-cfg-labelWidth\" class=\"docClass\">Ext.form.Labelable.labelWidth</a>,\netc.) in field components themselves, as that allows labels to be uniformly sized throughout the form.\n<a href=\"#!/api/Ext.form.Label\" rel=\"Ext.form.Label\" class=\"docClass\">Ext.form.Label</a> should only be used when your layout can not be achieved with the standard\n<a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">field layout</a>.</p>\n\n<p>You will likely be associating the label with a field component that extends <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a>, so\nyou should make sure the <a href=\"#!/api/Ext.form.Label-cfg-forId\" rel=\"Ext.form.Label-cfg-forId\" class=\"docClass\">forId</a> is set to the same value as the <a href=\"#!/api/Ext.form.field.Base-cfg-inputId\" rel=\"Ext.form.field.Base-cfg-inputId\" class=\"docClass\">inputId</a>\nof that field.</p>\n\n<p>The label's text can be set using either the <a href=\"#!/api/Ext.form.Label-cfg-text\" rel=\"Ext.form.Label-cfg-text\" class=\"docClass\">text</a> or <a href=\"#!/api/Ext.form.Label-cfg-html\" rel=\"Ext.form.Label-cfg-html\" class=\"docClass\">html</a> configuration properties; the\ndifference between the two is that the former will automatically escape HTML characters when rendering, while\nthe latter will not.</p>\n\n<h1>Example</h1>\n\n<p>This example creates a Label after its associated Text field, an arrangement that cannot currently\nbe achieved using the standard Field layout's labelAlign.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'Field with Label',\n    width: 400,\n    bodyPadding: 10,\n    renderTo: Ext.getBody(),\n    layout: {\n        type: 'hbox',\n        align: 'middle'\n    },\n    items: [{\n        xtype: 'textfield',\n        hideLabel: true,\n        flex: 1\n    }, {\n        xtype: 'label',\n        forId: 'myFieldId',\n        text: 'My Awesome Field',\n        margin: '0 0 0 10'\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"autoEl":{"!doc":"<p>A tag name or DomHelper spec used to create the <a href=\"#!/api/Ext.form.Label-method-getEl\" rel=\"Ext.form.Label-method-getEl\" class=\"docClass\">Element</a> which will\nencapsulate this Component.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong>'div'</strong>. The more complex Sencha classes use a more\ncomplex DOM structure specified by their own <a href=\"#!/api/Ext.form.Label-cfg-renderTpl\" rel=\"Ext.form.Label-cfg-renderTpl\" class=\"docClass\">renderTpl</a>s.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components encapsulated by\ndifferent DOM elements. Example usage:</p>\n\n<pre><code>{\n    xtype: 'component',\n    autoEl: {\n        tag: 'img',\n        src: 'http://www.example.com/example.jpg'\n    }\n}, {\n    xtype: 'component',\n    autoEl: {\n        tag: 'blockquote',\n        html: 'autoEl is cool!'\n    }\n}, {\n    xtype: 'container',\n    autoEl: 'ul',\n    cls: 'ux-unordered-list',\n    items: {\n        xtype: 'component',\n        autoEl: 'li',\n        html: 'First list item'\n    }\n}\n</code></pre>\n"},"forId":{"!type":"string","!doc":"<p>The id of the input element to which this label will be bound via the standard HTML 'for'\nattribute. If not specified, the attribute will not be added to the label. In most cases you will be\nassociating the label with a <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">Ext.form.field.Base</a> component, so you should make sure this matches\nthe <a href=\"#!/api/Ext.form.field.Base-cfg-inputId\" rel=\"Ext.form.field.Base-cfg-inputId\" class=\"docClass\">inputId</a> of that field.</p>\n"},"html":{"!type":"string","!doc":"<p>An HTML fragment that will be used as the label's innerHTML.\nNote that if <a href=\"#!/api/Ext.form.Label-cfg-text\" rel=\"Ext.form.Label-cfg-text\" class=\"docClass\">text</a> is specified it will take precedence and this value will be ignored.</p>\n"},"text":{"!type":"string","!doc":"<p>The plain text to display within the label. If you need to include HTML\ntags within the label's innerHTML, use the <a href=\"#!/api/Ext.form.Label-cfg-html\" rel=\"Ext.form.Label-cfg-html\" class=\"docClass\">html</a> config instead.</p>\n"},"getElConfig":{"!type":"fn() -> !this"},"setText":{"!type":"fn(text: ?, encode?: ?) -> ?","!doc":"<p>Updates the label's innerHTML with the specified string.</p>\n"}}},"Panel":{"!doc":"<p>FormPanel provides a standard container for forms. It is essentially a standard <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a> which\nautomatically creates a <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">BasicForm</a> for managing any <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Ext.form.field.Field</a>\nobjects that are added as descendants of the panel. It also includes conveniences for configuring and\nworking with the BasicForm and the collection of Fields.</p>\n\n<h1>Layout</h1>\n\n<p>By default, FormPanel is configured with <code><a href=\"#!/api/Ext.layout.container.Anchor\" rel=\"Ext.layout.container.Anchor\" class=\"docClass\">layout:'anchor'</a></code> for\nthe layout of its immediate child items. This can be changed to any of the supported container layouts.\nThe layout of sub-containers is configured in <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">the standard way</a>.</p>\n\n<h1>BasicForm</h1>\n\n<p>FormPanel class accepts all\nof the config options supported by the <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a> class, and will pass them along to\nthe internal BasicForm when it is created.</p>\n\n<p>The following events fired by the BasicForm will be re-fired by the FormPanel and can therefore be\nlistened for on the FormPanel itself:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.form.Basic-event-beforeaction\" rel=\"Ext.form.Basic-event-beforeaction\" class=\"docClass\">beforeaction</a></li>\n<li><a href=\"#!/api/Ext.form.Basic-event-actionfailed\" rel=\"Ext.form.Basic-event-actionfailed\" class=\"docClass\">actionfailed</a></li>\n<li><a href=\"#!/api/Ext.form.Basic-event-actioncomplete\" rel=\"Ext.form.Basic-event-actioncomplete\" class=\"docClass\">actioncomplete</a></li>\n<li><a href=\"#!/api/Ext.form.Basic-event-validitychange\" rel=\"Ext.form.Basic-event-validitychange\" class=\"docClass\">validitychange</a></li>\n<li><a href=\"#!/api/Ext.form.Basic-event-dirtychange\" rel=\"Ext.form.Basic-event-dirtychange\" class=\"docClass\">dirtychange</a></li>\n</ul>\n\n\n<h1>Field Defaults</h1>\n\n<p>The <a href=\"#!/api/Ext.form.Panel-cfg-fieldDefaults\" rel=\"Ext.form.Panel-cfg-fieldDefaults\" class=\"docClass\">fieldDefaults</a> config option conveniently allows centralized configuration of default values\nfor all fields added as descendants of the FormPanel. Any config option recognized by implementations\nof <a href=\"#!/api/Ext.form.Labelable\" rel=\"Ext.form.Labelable\" class=\"docClass\">Ext.form.Labelable</a> may be included in this object. See the <a href=\"#!/api/Ext.form.Panel-cfg-fieldDefaults\" rel=\"Ext.form.Panel-cfg-fieldDefaults\" class=\"docClass\">fieldDefaults</a> documentation\nfor details of how the defaults are applied.</p>\n\n<h1>Form Validation</h1>\n\n<p>With the default configuration, form fields are validated on-the-fly while the user edits their values.\nThis can be controlled on a per-field basis (or via the <a href=\"#!/api/Ext.form.Panel-cfg-fieldDefaults\" rel=\"Ext.form.Panel-cfg-fieldDefaults\" class=\"docClass\">fieldDefaults</a> config) with the field\nconfig properties <a href=\"#!/api/Ext.form.field.Field-cfg-validateOnChange\" rel=\"Ext.form.field.Field-cfg-validateOnChange\" class=\"docClass\">Ext.form.field.Field.validateOnChange</a> and <a href=\"#!/api/Ext.form.field.Base-cfg-checkChangeEvents\" rel=\"Ext.form.field.Base-cfg-checkChangeEvents\" class=\"docClass\">Ext.form.field.Base.checkChangeEvents</a>,\nand the FormPanel's config properties <a href=\"#!/api/Ext.form.Panel-cfg-pollForChanges\" rel=\"Ext.form.Panel-cfg-pollForChanges\" class=\"docClass\">pollForChanges</a> and <a href=\"#!/api/Ext.form.Panel-cfg-pollInterval\" rel=\"Ext.form.Panel-cfg-pollInterval\" class=\"docClass\">pollInterval</a>.</p>\n\n<p>Any component within the FormPanel can be configured with <code>formBind: true</code>. This will cause that\ncomponent to be automatically disabled when the form is invalid, and enabled when it is valid. This is most\ncommonly used for Button components to prevent submitting the form in an invalid state, but can be used on\nany component type.</p>\n\n<p>For more information on form validation see the following:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.form.field.Field-cfg-validateOnChange\" rel=\"Ext.form.field.Field-cfg-validateOnChange\" class=\"docClass\">Ext.form.field.Field.validateOnChange</a></li>\n<li><a href=\"#!/api/Ext.form.Panel-cfg-pollForChanges\" rel=\"Ext.form.Panel-cfg-pollForChanges\" class=\"docClass\">pollForChanges</a> and <a href=\"#!/api/Ext.form.Panel-cfg-pollInterval\" rel=\"Ext.form.Panel-cfg-pollInterval\" class=\"docClass\">pollInterval</a></li>\n<li><a href=\"#!/api/Ext.form.field.VTypes\" rel=\"Ext.form.field.VTypes\" class=\"docClass\">Ext.form.field.VTypes</a></li>\n<li><a href=\"#!/api/Ext.form.Basic-method-doAction\" rel=\"Ext.form.Basic-method-doAction\" class=\"docClass\">BasicForm.doAction clientValidation notes</a></li>\n</ul>\n\n\n<h1>Form Submission</h1>\n\n<p>By default, Ext Forms are submitted through Ajax, using <a href=\"#!/api/Ext.form.action.Action\" rel=\"Ext.form.action.Action\" class=\"docClass\">Ext.form.action.Action</a>. See the documentation for\n<a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a> for details.</p>\n\n<h1>Example usage</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'Simple Form',\n    bodyPadding: 5,\n    width: 350,\n\n    // The form will submit an AJAX request to this URL when submitted\n    url: 'save-form.php',\n\n    // Fields will be arranged vertically, stretched to full width\n    layout: 'anchor',\n    defaults: {\n        anchor: '100%'\n    },\n\n    // The fields\n    defaultType: 'textfield',\n    items: [{\n        fieldLabel: 'First Name',\n        name: 'first',\n        allowBlank: false\n    },{\n        fieldLabel: 'Last Name',\n        name: 'last',\n        allowBlank: false\n    }],\n\n    // Reset and Submit buttons\n    buttons: [{\n        text: 'Reset',\n        handler: function() {\n            this.up('form').getForm().reset();\n        }\n    }, {\n        text: 'Submit',\n        formBind: true, //only enabled once the form is valid\n        disabled: true,\n        handler: function() {\n            var form = this.up('form').getForm();\n            if (form.isValid()) {\n                form.submit({\n                    success: function(form, action) {\n                       <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Success', action.result.msg);\n                    },\n                    failure: function(form, action) {\n                        <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Failed', action.result.msg);\n                    }\n                });\n            }\n        }\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"api":{"!doc":"<p>If specified, load and submit actions will be handled with <a href=\"#!/api/Ext.form.action.DirectLoad\" rel=\"Ext.form.action.DirectLoad\" class=\"docClass\">DirectLoad</a>\nand <a href=\"#!/api/Ext.form.action.DirectSubmit\" rel=\"Ext.form.action.DirectSubmit\" class=\"docClass\">DirectSubmit</a>.  Methods which have been imported by\nExt.direct.Manager can be specified here to load and submit forms. API methods may also be\nspecified as strings. See Ext.data.proxy.Direct.directFn.  Such as the following:</p>\n\n<pre><code>api: {\n    load: App.ss.MyProfile.load,\n    submit: App.ss.MyProfile.submit\n}\n</code></pre>\n\n<p>Load actions can use <a href=\"#!/api/Ext.form.Panel-cfg-paramOrder\" rel=\"Ext.form.Panel-cfg-paramOrder\" class=\"docClass\">paramOrder</a> or <a href=\"#!/api/Ext.form.Panel-cfg-paramsAsHash\" rel=\"Ext.form.Panel-cfg-paramsAsHash\" class=\"docClass\">paramsAsHash</a> to customize how the load method\nis invoked.  Submit actions will always use a standard form submit. The <code>formHandler</code> configuration\n(see Ext.direct.RemotingProvider#action) must be set on the associated server-side method which has\nbeen imported by Ext.direct.Manager.</p>\n"},"baseParams":{"!doc":"<p>Parameters to pass with all requests. e.g. baseParams: <code>{id: '123', foo: 'bar'}</code>.</p>\n\n<p>Parameters are encoded as standard HTTP parameters using Ext.Object.toQueryString.</p>\n"},"errorReader":{"!doc":"<p>An Ext.data.reader.Reader (e.g. Ext.data.reader.Xml) instance or\nconfiguration to be used to read field error messages returned from 'submit' actions.\nThis is optional as there is built-in support for processing JSON responses.</p>\n\n<p>The Records which provide messages for the invalid Fields must use the\nField name (or id) as the Record ID, and must contain a field called 'msg'\nwhich contains the error message.</p>\n\n<p>The errorReader does not have to be a full-blown implementation of a\nReader. It simply needs to implement a <code>read(xhr)</code> function\nwhich returns an Array of Records in an object with the following\nstructure:</p>\n\n<pre><code>{\n    records: recordArray\n}\n</code></pre>\n"},"jsonSubmit":{"!type":"bool","!doc":"<p>If set to true, the field values are sent as JSON in the request body.\nAll of the field values, plus any additional params configured via <a href=\"#!/api/Ext.form.Panel-cfg-baseParams\" rel=\"Ext.form.Panel-cfg-baseParams\" class=\"docClass\">baseParams</a>\nand/or the <code>options</code> to <a href=\"#!/api/Ext.form.Panel-method-submit\" rel=\"Ext.form.Panel-method-submit\" class=\"docClass\">submit</a>, will be included in the values POSTed in the body of the request.</p>\n"},"layout":{"!doc":"<p>The <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">Ext.container.Container.layout</a> for the form panel's immediate child items.</p>\n"},"method":{"!type":"string","!doc":"<p>The request method to use (GET or POST) for form actions if one isn't supplied in the action options.</p>\n"},"paramOrder":{"!doc":"<p>A list of params to be executed server side. Only used for the <a href=\"#!/api/Ext.form.Panel-cfg-api\" rel=\"Ext.form.Panel-cfg-api\" class=\"docClass\">api</a> <code>load</code>\nconfiguration.</p>\n\n<p>Specify the params in the order in which they must be executed on the\nserver-side as either (1) an Array of String values, or (2) a String of params\ndelimited by either whitespace, comma, or pipe. For example,\nany of the following would be acceptable:</p>\n\n<pre><code>paramOrder: ['param1','param2','param3']\nparamOrder: 'param1 param2 param3'\nparamOrder: 'param1,param2,param3'\nparamOrder: 'param1|param2|param'\n</code></pre>\n"},"paramsAsHash":{"!type":"bool","!doc":"<p>Only used for the <a href=\"#!/api/Ext.form.Panel-cfg-api\" rel=\"Ext.form.Panel-cfg-api\" class=\"docClass\">api</a> <code>load</code> configuration. If true, parameters will be sent as a\nsingle hash collection of named arguments. Providing a <a href=\"#!/api/Ext.form.Panel-cfg-paramOrder\" rel=\"Ext.form.Panel-cfg-paramOrder\" class=\"docClass\">paramOrder</a> nullifies this\nconfiguration.</p>\n"},"pollForChanges":{"!type":"bool","!doc":"<p>If set to <code>true</code>, sets up an interval task (using the <a href=\"#!/api/Ext.form.Panel-cfg-pollInterval\" rel=\"Ext.form.Panel-cfg-pollInterval\" class=\"docClass\">pollInterval</a>) in which the\npanel's fields are repeatedly checked for changes in their values. This is in addition to the normal detection\neach field does on its own input element, and is not needed in most cases. It does, however, provide a\nmeans to absolutely guarantee detection of all changes including some edge cases in some browsers which\ndo not fire native events. Defaults to <code>false</code>.</p>\n"},"pollInterval":{"!type":"number","!doc":"<p>Interval in milliseconds at which the form's fields are checked for value changes. Only used if\nthe <a href=\"#!/api/Ext.form.Panel-cfg-pollForChanges\" rel=\"Ext.form.Panel-cfg-pollForChanges\" class=\"docClass\">pollForChanges</a> option is set to <code>true</code>. Defaults to 500 milliseconds.</p>\n"},"reader":{"!doc":"<p>An Ext.data.reader.Reader (e.g. Ext.data.reader.Xml) instance or\nconfiguration to be used to read data when executing 'load' actions. This\nis optional as there is built-in support for processing JSON responses.</p>\n"},"standardSubmit":{"!type":"bool","!doc":"<p>If set to true, a standard HTML form submit is used instead of a XHR (Ajax) style form submission.\nAll of the field values, plus any additional params configured via <a href=\"#!/api/Ext.form.Panel-cfg-baseParams\" rel=\"Ext.form.Panel-cfg-baseParams\" class=\"docClass\">baseParams</a>\nand/or the <code>options</code> to <a href=\"#!/api/Ext.form.Panel-method-submit\" rel=\"Ext.form.Panel-method-submit\" class=\"docClass\">submit</a>, will be included in the values submitted in the form.</p>\n"},"timeout":{"!type":"number","!doc":"<p>Timeout for form actions in seconds.</p>\n"},"trackResetOnLoad":{"!type":"bool","!doc":"<p>If set to true, <a href=\"#!/api/Ext.form.Panel-method-reset\" rel=\"Ext.form.Panel-method-reset\" class=\"docClass\">reset</a>() resets to the last loaded or setValues() data instead of\nwhen the form was first created.</p>\n"},"url":{"!type":"string","!doc":"<p>The URL to use for form actions if one isn't supplied in the\ndoAction options.</p>\n"},"waitMsgTarget":{"!doc":"<p>By default wait messages are displayed with <a href=\"#!/api/Ext.MessageBox-method-wait\" rel=\"Ext.MessageBox-method-wait\" class=\"docClass\">Ext.MessageBox.wait</a>. You can target a specific\nelement by passing it or its id or mask the form itself by passing in true.</p>\n"},"waitTitle":{"!type":"string","!doc":"<p>The default title to show for the waiting message box</p>\n"},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"basicFormConfigs":{},"afterFirstLayout":{"!type":"fn() -> !this","!doc":"<p>Initialize the BasicForm after all layouts have been completed.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this"},"checkChange":{"!type":"fn() -> !this","!doc":"<p>Forces each field within the form panel to\n<a href=\"#!/api/Ext.form.field.Field-method-checkChange\" rel=\"Ext.form.field.Field-method-checkChange\" class=\"docClass\">check if its value has changed</a>.</p>\n"},"createForm":{"!type":"fn() -> !this"},"getForm":{"!type":"fn() -> ?","!doc":"<p>Provides access to the <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Form</a> which this Panel contains.</p>\n"},"getRecord":{"!type":"fn() -> ?","!doc":"<p>Returns the currently loaded Ext.data.Model instance if one was loaded via <a href=\"#!/api/Ext.form.Panel-method-loadRecord\" rel=\"Ext.form.Panel-method-loadRecord\" class=\"docClass\">loadRecord</a>.</p>\n"},"getValues":{"!type":"fn(asString?: ?, dirtyOnly?: ?, includeEmptyText?: ?, useDataValues?: ?) -> ?","!doc":"<p>Convenience function for fetching the current value of each field in the form. This is the same as calling\n<a href=\"#!/api/Ext.form.Basic-method-getValues\" rel=\"Ext.form.Basic-method-getValues\" class=\"docClass\">this.getForm().getValues()</a>.</p>\n"},"hasInvalidField":{"!type":"fn() -> !this","!doc":"<p>Convenience function to check if the form has any invalid fields. This is the same as calling\n<a href=\"#!/api/Ext.form.Basic-method-hasInvalidField\" rel=\"Ext.form.Basic-method-hasInvalidField\" class=\"docClass\">this.getForm().hasInvalidField()</a>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initItems":{"!type":"fn() -> !this","!doc":"<p>This method is called to initialize the <code>items</code> collection. A derived class can\noverride this method to do any last minute manipulation of <code>items</code> and then call\nthis method using <code>callParent</code>. Upon return, the <code>items</code> will no longer be a simple\narray.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Convenience function to check if the form has any dirty fields. This is the same as calling\n<a href=\"#!/api/Ext.form.Basic-method-isDirty\" rel=\"Ext.form.Basic-method-isDirty\" class=\"docClass\">this.getForm().isDirty()</a>.</p>\n"},"isValid":{"!type":"fn() -> bool","!doc":"<p>Convenience function to check if the form has all valid fields. This is the same as calling\n<a href=\"#!/api/Ext.form.Basic-method-isValid\" rel=\"Ext.form.Basic-method-isValid\" class=\"docClass\">this.getForm().isValid()</a>.</p>\n"},"load":{"!type":"fn(options: ?) -> !this","!doc":"<p>This is a proxy for the underlying BasicForm's <a href=\"#!/api/Ext.form.Basic-method-load\" rel=\"Ext.form.Basic-method-load\" class=\"docClass\">Ext.form.Basic.load</a> call.</p>\n"},"loadRecord":{"!type":"fn(record: ?) -> ?","!doc":"<p>Loads an Ext.data.Model into this form (internally just calls <a href=\"#!/api/Ext.form.Basic-method-loadRecord\" rel=\"Ext.form.Basic-method-loadRecord\" class=\"docClass\">Ext.form.Basic.loadRecord</a>)\nSee also <a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a>. The fields in the model are mapped to\nfields in the form by matching either the <a href=\"#!/api/Ext.form.field.Base-cfg-name\" rel=\"Ext.form.field.Base-cfg-name\" class=\"docClass\">Ext.form.field.Base.name</a> or <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">Ext.Component.itemId</a>.</p>\n"},"reset":{"!type":"fn(resetRecord?: ?) -> ?","!doc":"<p>Convenience function reset the form. This is the same as calling\n<a href=\"#!/api/Ext.form.Basic-method-reset\" rel=\"Ext.form.Basic-method-reset\" class=\"docClass\">this.getForm().reset()</a>.</p>\n"},"startPolling":{"!type":"fn(interval: ?) -> !this","!doc":"<p>Start an interval task to continuously poll all the fields in the form for changes in their\nvalues. This is normally started automatically by setting the <a href=\"#!/api/Ext.form.Panel-cfg-pollForChanges\" rel=\"Ext.form.Panel-cfg-pollForChanges\" class=\"docClass\">pollForChanges</a> config.</p>\n"},"stopPolling":{"!type":"fn() -> !this","!doc":"<p>Stop a running interval task that was started by <a href=\"#!/api/Ext.form.Panel-method-startPolling\" rel=\"Ext.form.Panel-method-startPolling\" class=\"docClass\">startPolling</a>.</p>\n"},"submit":{"!type":"fn(options: ?) -> !this","!doc":"<p>This is a proxy for the underlying BasicForm's <a href=\"#!/api/Ext.form.Basic-method-submit\" rel=\"Ext.form.Basic-method-submit\" class=\"docClass\">Ext.form.Basic.submit</a> call.</p>\n"},"updateRecord":{"!type":"fn(record?: ?) -> ?","!doc":"<p>Persists the values in this form into the passed Ext.data.Model object in a beginEdit/endEdit block.\nIf the record is not specified, it will attempt to update (if it exists) the record provided to <a href=\"#!/api/Ext.form.Panel-method-loadRecord\" rel=\"Ext.form.Panel-method-loadRecord\" class=\"docClass\">loadRecord</a>.</p>\n"},"actioncomplete":{"!type":"fn(this: ?, action: ?, eOpts: ?)","!doc":"<p>Fires when an action is completed.</p>\n"},"actionfailed":{"!type":"fn(this: ?, action: ?, eOpts: ?)","!doc":"<p>Fires when an action fails.</p>\n"},"beforeaction":{"!type":"fn(this: ?, action: ?, eOpts: ?)","!doc":"<p>Fires before any action is performed. Return false to cancel the action.</p>\n"},"dirtychange":{"!type":"fn(this: ?, dirty: ?, eOpts: ?)","!doc":"<p>Fires when the dirty state of the entire form changes.</p>\n"},"validitychange":{"!type":"fn(this: ?, valid: ?, eOpts: ?)","!doc":"<p>Fires when the validity of the entire form changes.</p>\n"}}},"RadioGroup":{"!doc":"<p>A <a href=\"#!/api/Ext.form.FieldContainer\" rel=\"Ext.form.FieldContainer\" class=\"docClass\">field container</a> which has a specialized layout for arranging\n<a href=\"#!/api/Ext.form.field.Radio\" rel=\"Ext.form.field.Radio\" class=\"docClass\">Ext.form.field.Radio</a> controls into columns, and provides convenience <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Ext.form.field.Field</a>\nmethods for <a href=\"#!/api/Ext.form.RadioGroup-method-getValue\" rel=\"Ext.form.RadioGroup-method-getValue\" class=\"docClass\">getting</a>, <a href=\"#!/api/Ext.form.RadioGroup-method-setValue\" rel=\"Ext.form.RadioGroup-method-setValue\" class=\"docClass\">setting</a>, and <a href=\"#!/api/Ext.form.RadioGroup-method-validate\" rel=\"Ext.form.RadioGroup-method-validate\" class=\"docClass\">validating</a> the\ngroup of radio buttons as a whole.</p>\n\n<h1>Validation</h1>\n\n<p>Individual radio buttons themselves have no default validation behavior, but\nsometimes you want to require a user to select one of a group of radios. RadioGroup\nallows this by setting the config <code><a href=\"#!/api/Ext.form.RadioGroup-cfg-allowBlank\" rel=\"Ext.form.RadioGroup-cfg-allowBlank\" class=\"docClass\">allowBlank</a>:false</code>; when the user does not check at\none of the radio buttons, the entire group will be highlighted as invalid and the\n<a href=\"#!/api/Ext.form.RadioGroup-cfg-blankText\" rel=\"Ext.form.RadioGroup-cfg-blankText\" class=\"docClass\">error message</a> will be displayed according to the <a href=\"#!/api/Ext.form.RadioGroup-cfg-msgTarget\" rel=\"Ext.form.RadioGroup-cfg-msgTarget\" class=\"docClass\">msgTarget</a> config.</p>\n\n<h1>Layout</h1>\n\n<p>The default layout for RadioGroup makes it easy to arrange the radio buttons into\ncolumns; see the <a href=\"#!/api/Ext.form.RadioGroup-cfg-columns\" rel=\"Ext.form.RadioGroup-cfg-columns\" class=\"docClass\">columns</a> and <a href=\"#!/api/Ext.form.RadioGroup-cfg-vertical\" rel=\"Ext.form.RadioGroup-cfg-vertical\" class=\"docClass\">vertical</a> config documentation for details. You may also\nuse a completely different layout by setting the <a href=\"#!/api/Ext.form.RadioGroup-cfg-layout\" rel=\"Ext.form.RadioGroup-cfg-layout\" class=\"docClass\">layout</a> to one of the other supported layout\ntypes; for instance you may wish to use a custom arrangement of hbox and vbox containers. In that case\nthe Radio components at any depth will still be managed by the RadioGroup's validation.</p>\n\n<h1>Example usage</h1>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'RadioGroup Example',\n    width: 300,\n    height: 125,\n    bodyPadding: 10,\n    renderTo: Ext.getBody(),\n    items:[{\n        xtype: 'radiogroup',\n        fieldLabel: 'Two Columns',\n        // Arrange radio buttons into two columns, distributed vertically\n        columns: 2,\n        vertical: true,\n        items: [\n            { boxLabel: 'Item 1', name: 'rb', inputValue: '1' },\n            { boxLabel: 'Item 2', name: 'rb', inputValue: '2', checked: true},\n            { boxLabel: 'Item 3', name: 'rb', inputValue: '3' },\n            { boxLabel: 'Item 4', name: 'rb', inputValue: '4' },\n            { boxLabel: 'Item 5', name: 'rb', inputValue: '5' },\n            { boxLabel: 'Item 6', name: 'rb', inputValue: '6' }\n        ]\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"allowBlank":{"!type":"bool","!doc":"<p>True to allow every item in the group to be blank.\nIf allowBlank = false and no items are selected at validation time, <a href=\"#!/api/Ext.form.RadioGroup-cfg-blankText\" rel=\"Ext.form.RadioGroup-cfg-blankText\" class=\"docClass\">blankText</a> will\nbe used as the error text.</p>\n"},"blankText":{"!type":"string","!doc":"<p>Error text to display if the <a href=\"#!/api/Ext.form.RadioGroup-cfg-allowBlank\" rel=\"Ext.form.RadioGroup-cfg-allowBlank\" class=\"docClass\">allowBlank</a> validation fails</p>\n"},"defaultType":{"!type":"string","!doc":"<p>private</p>\n"},"items":{"!doc":"<p>An Array of <a href=\"#!/api/Ext.form.field.Radio\" rel=\"Ext.form.field.Radio\" class=\"docClass\">Radio</a>s or Radio config objects to arrange in the group.</p>\n"},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"groupCls":{"!type":"string","!doc":"<p>private</p>\n"},"checkChange":{"!type":"fn() -> !this","!doc":"<p>Checks whether the value of the field has changed since the last time it was checked.\nIf the value has changed, it:</p>\n\n<ol>\n<li>Fires the <a href=\"#!/api/Ext.form.RadioGroup-event-change\" rel=\"Ext.form.RadioGroup-event-change\" class=\"docClass\">change event</a>,</li>\n<li>Performs validation if the <a href=\"#!/api/Ext.form.RadioGroup-cfg-validateOnChange\" rel=\"Ext.form.RadioGroup-cfg-validateOnChange\" class=\"docClass\">validateOnChange</a> config is enabled, firing the\n<a href=\"#!/api/Ext.form.RadioGroup-event-validitychange\" rel=\"Ext.form.RadioGroup-event-validitychange\" class=\"docClass\">validitychange event</a> if the validity has changed, and</li>\n<li>Checks the <a href=\"#!/api/Ext.form.RadioGroup-method-isDirty\" rel=\"Ext.form.RadioGroup-method-isDirty\" class=\"docClass\">dirty state</a> of the field and fires the <a href=\"#!/api/Ext.form.RadioGroup-event-dirtychange\" rel=\"Ext.form.RadioGroup-event-dirtychange\" class=\"docClass\">dirtychange event</a>\nif it has changed.</li>\n</ol>\n\n"},"getBoxes":{"!type":"fn(query?: ?) -> !this","!doc":"<p>Returns all checkbox components within the container</p>\n"},"setValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Sets the value of the radio group. The radio with corresponding name and value will be set.\nThis method is simpler than <a href=\"#!/api/Ext.form.CheckboxGroup-method-setValue\" rel=\"Ext.form.CheckboxGroup-method-setValue\" class=\"docClass\">Ext.form.CheckboxGroup.setValue</a> because only 1 value is allowed\nfor each name. You can use the setValue method as:</p>\n\n<pre><code>var form = Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title       : 'RadioGroup Example',\n    width       : 300,\n    bodyPadding : 10,\n    renderTo    : Ext.getBody(),\n    items       : [\n        {\n            xtype      : 'radiogroup',\n            fieldLabel : 'Group',\n            items      : [\n                { boxLabel : 'Item 1', name : 'rb', inputValue : 1 },\n                { boxLabel : 'Item 2', name : 'rb', inputValue : 2 }\n            ]\n        }\n    ],\n    tbar        : [\n        {\n            text    : 'setValue on RadioGroup',\n            handler : function () {\n                form.child('radiogroup').setValue({\n                    rb : 2\n                });\n            }\n        }\n    ]\n});\n</code></pre>\n"}}},"RadioManager":{"!doc":"<p>Private utility class for managing all <a href=\"#!/api/Ext.form.field.Radio\" rel=\"Ext.form.field.Radio\" class=\"docClass\">Ext.form.field.Radio</a> fields grouped by name.</p>\n","getByName":{"!type":"fn(name: ?, formId: ?) -> !this"},"getChecked":{"!type":"fn(name: ?, formId: ?) -> !this"},"getWithValue":{"!type":"fn(name: ?, value: ?, formId: ?) -> !this"}},"trigger":{"Component":{"!doc":"<p>A Text Field Trigger that contains a <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a> or Widget.</p>\n","!type":"fn(config: ?)","prototype":{"A":{"!doc":"<p>config object for a Component or Widget,\nor an already instantiated Component or Widget.</p>\n"},"cls":{"!type":"string","!doc":"<p>CSS class to add to the trigger element.</p>\n"},"component":{"!doc":"<p>The component or widget</p>\n"},"destroy":{"!type":"fn() -> !this"}}},"Trigger":{"prototype":{"extraCls":{"!type":"string","!doc":"<p>An additional CSS class (or classes) to be added to the trigger's element. Can\nbe a single class name (e.g. 'foo') or a space-separated list of class names\n(e.g. 'foo bar').</p>\n"},"focusFieldOnClick":{"!type":"bool","!doc":"<p><code>true</code> to focus the field's input element after the trigger is clicked.</p>\n"},"handler":{"!doc":"<p>Function to run when trigger is clicked or tapped.</p>\n"},"hidden":{"!type":"bool","!doc":"<p><code>true</code> to initially render the trigger hidden.</p>\n"},"hideOnReadOnly":{"!type":"bool","!doc":"<p>Set 'false' to prevent trigger from being hidden even though the related field is set <a href=\"#!/api/Ext.form.field.Text-cfg-readOnly\" rel=\"Ext.form.field.Text-cfg-readOnly\" class=\"docClass\">readOnly</a></p>\n"},"repeatClick":{"!type":"bool","!doc":"<p><code>true</code> to attach a <a href=\"#!/api/Ext.util.ClickRepeater\" rel=\"Ext.util.ClickRepeater\" class=\"docClass\">click repeater</a> to the trigger</p>\n"},"scope":{"!doc":"<p>Execution context for the handler function.</p>\n"},"weight":{"!type":"number","!doc":"<p>An optional weighting to change the ordering of the items. The default weight is\n<code>0</code>.  Triggers are sorted by weight in ascending order before being rendered.</p>\n"},"width":{"!type":"number","!doc":"<p>The trigger's width, in pixels. Typically this is not needed\nas the trigger width is normally determined by the stylesheet, (see extjs-text-field or Ext.form.field.Text#$extjs-text-field-ui).</p>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class that is always added to the trigger element.</p>\n"},"bodyTpl":{"!doc":"<p>An optional template for rendering child elements inside the trigger element.\nUseful for creating more complex triggers such as <a href=\"#!/api/Ext.form.trigger.Spinner\" rel=\"Ext.form.trigger.Spinner\" class=\"docClass\">Ext.form.trigger.Spinner</a>.</p>\n"},"clickCls":{"!type":"string","!doc":"<p>The CSS class that is added to the trigger element it is clicked.</p>\n"},"el":{"!doc":"<p>The trigger's main element</p>\n"},"factoryConfig":{},"focusCls":{"!type":"string","!doc":"<p>The CSS class that is added to the trigger element when the field is focused.</p>\n"},"overCls":{"!type":"string","!doc":"<p>The CSS class that is added to the trigger element when it is hovered.</p>\n"},"validIdRe":{"!doc":"<p>private</p>\n"},"afterFieldRender":{"!type":"fn() -> !this","!doc":"<p>Called when this trigger's field is rendered</p>\n"},"getBodyRenderData":{"!type":"fn() -> ?","!doc":"<p>Allows addition of data to the render data object for the <a href=\"#!/api/Ext.form.trigger.Trigger-property-bodyTpl\" rel=\"Ext.form.trigger.Trigger-property-bodyTpl\" class=\"docClass\">bodyTpl</a>.</p>\n"},"getEl":{"!type":"fn() -> ?","!doc":"<p>Get the element for this trigger.</p>\n"},"getStateEl":{"!type":"fn() -> !this","!doc":"<p>Returns the element that should receive the \"state\" classes - <a href=\"#!/api/Ext.form.trigger.Trigger-property-focusCls\" rel=\"Ext.form.trigger.Trigger-property-focusCls\" class=\"docClass\">focusCls</a>,\n#overCls, and <a href=\"#!/api/Ext.form.trigger.Trigger-property-clickCls\" rel=\"Ext.form.trigger.Trigger-property-clickCls\" class=\"docClass\">clickCls</a>.</p>\n"},"hide":{"!type":"fn() -> !this","!doc":"<p>Hides the trigger</p>\n"},"initEvents":{"!type":"fn() -> !this"},"isFieldEnabled":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"isVisible":{"!type":"fn() -> bool","!doc":"<p>Returns <code>true</code> if this Trigger is visible.</p>\n"},"onClick":{"!type":"fn() -> !this","!doc":"<p>Handles a click on the trigger's element</p>\n"},"onFieldBlur":{"!type":"fn() -> !this","!doc":"<p>Called when this trigger's field is blurred</p>\n"},"onFieldFocus":{"!type":"fn() -> !this","!doc":"<p>Called when this trigger's field is focused</p>\n"},"onFieldRender":{"!type":"fn() -> !this","!doc":"<p>Called when this trigger's field is rendered</p>\n"},"onMouseUp":{"!type":"fn() -> !this"},"renderBody":{"!type":"fn(renderData: ?) -> string","!doc":"<p>Renders the bodyTpl</p>\n"},"renderTrigger":{"!type":"fn(fieldData: ?) -> string","!doc":"<p>Generates the trigger markup. Called during rendering of the field the trigger\nbelongs to.</p>\n"},"resolveListenerScope":{"!type":"fn(scope: ?) -> !this","!doc":"<p>\"this\" refers to our owning input field.</p>\n"},"setHidden":{"!type":"fn(hidden: ?) -> !this"},"setVisible":{"!type":"fn(visible: ?) -> !this"},"show":{"!type":"fn() -> !this","!doc":"<p>Shows the trigger</p>\n"},"destroy":{"!type":"fn() -> !this"},"cls":{"!type":"string","!doc":"<p>CSS class to add to the trigger element.</p>\n"},"weightComparator":{"!type":"fn(triggerA: ?, triggerB: ?) -> number","!doc":"<p>Comparison function for sorting an array of triggers in ascending order</p>\n"}},"!doc":"<p>Base class for <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Text Field</a> triggers</p>\n","!type":"fn(config: ?)"},"Spinner":{"!doc":"<p>A Trigger that contains 2 clickable elements inside in the form of a \"up\" and a \"down\"\ntrigger.</p>\n","!type":"fn(config: ?)","prototype":{"cls":{"!type":"string","!doc":"<p>CSS class to add to the trigger element.</p>\n"},"downHandler":{"!doc":"<p>The handler for the 'down' button</p>\n"},"focusFieldOnClick":{"!type":"bool","!doc":"<p>restore focus to the input element on spin end.</p>\n"},"upHandler":{"!doc":"<p>The handler for the 'up' button</p>\n"},"bodyTpl":{"!doc":"<p>An optional template for rendering child elements inside the trigger element.\nUseful for creating more complex triggers such as <a href=\"#!/api/Ext.form.trigger.Spinner\" rel=\"Ext.form.trigger.Spinner\" class=\"docClass\">Ext.form.trigger.Spinner</a>.</p>\n"},"clickCls":{"!type":"string","!doc":"<p>The CSS class that is added to the trigger element it is clicked.</p>\n"},"focusCls":{"!type":"string","!doc":"<p>The CSS class that is added to the trigger element when the field is focused.</p>\n"},"overCls":{"!type":"string","!doc":"<p>The CSS class that is added to the trigger element when it is hovered.</p>\n"},"spinnerCls":{"!type":"string"},"spinnerDownCls":{"!type":"string"},"spinnerEl":{"!doc":"<p>The \"up\" spinner element</p>\n"},"spinnerUpCls":{"!type":"string"},"vertical":{"!type":"bool","!doc":"<p>private</p>\n"},"getBodyRenderData":{"!type":"fn() -> ?","!doc":"<p>Allows addition of data to the render data object for the <a href=\"#!/api/Ext.form.trigger.Spinner-property-bodyTpl\" rel=\"Ext.form.trigger.Spinner-property-bodyTpl\" class=\"docClass\">bodyTpl</a>.</p>\n"},"getStateEl":{"!type":"fn() -> !this","!doc":"<p>Returns the element that should receive the \"state\" classes - <a href=\"#!/api/Ext.form.trigger.Spinner-property-focusCls\" rel=\"Ext.form.trigger.Spinner-property-focusCls\" class=\"docClass\">focusCls</a>,\n#overCls, and <a href=\"#!/api/Ext.form.trigger.Spinner-property-clickCls\" rel=\"Ext.form.trigger.Spinner-property-clickCls\" class=\"docClass\">clickCls</a>.</p>\n"},"onClick":{"!type":"fn(e: ?) -> !this","!doc":"<p>Handles a click on the trigger's element</p>\n"},"onFieldRender":{"!type":"fn() -> !this","!doc":"<p>Called when this trigger's field is rendered</p>\n"},"setDownEnabled":{"!type":"fn(enabled: ?) -> !this","!doc":"<p>private</p>\n"},"setUpEnabled":{"!type":"fn(enabled: ?) -> !this","!doc":"<p>private</p>\n"}}}}},"fx":{"Anim":{"!doc":"<p>This class manages animation for a specific <a href=\"#!/api/Ext.fx.Anim-cfg-target\" rel=\"Ext.fx.Anim-cfg-target\" class=\"docClass\">target</a>. The animation allows\nanimation of various properties on the target, such as size, position, color and others.</p>\n\n<h2>Starting Conditions</h2>\n\n<p>The starting conditions for the animation are provided by the <a href=\"#!/api/Ext.fx.Anim-cfg-from\" rel=\"Ext.fx.Anim-cfg-from\" class=\"docClass\">from</a> configuration.\nAny/all of the properties in the <a href=\"#!/api/Ext.fx.Anim-cfg-from\" rel=\"Ext.fx.Anim-cfg-from\" class=\"docClass\">from</a> configuration can be specified. If a particular\nproperty is not defined, the starting value for that property will be read directly from the target.</p>\n\n<h2>End Conditions</h2>\n\n<p>The ending conditions for the animation are provided by the <a href=\"#!/api/Ext.fx.Anim-cfg-to\" rel=\"Ext.fx.Anim-cfg-to\" class=\"docClass\">to</a> configuration. These mark\nthe final values once the animations has finished. The values in the <a href=\"#!/api/Ext.fx.Anim-cfg-from\" rel=\"Ext.fx.Anim-cfg-from\" class=\"docClass\">from</a> can mirror\nthose in the <a href=\"#!/api/Ext.fx.Anim-cfg-to\" rel=\"Ext.fx.Anim-cfg-to\" class=\"docClass\">to</a> configuration to provide a starting point.</p>\n\n<h2>Other Options</h2>\n\n<ul>\n<li><a href=\"#!/api/Ext.fx.Anim-cfg-duration\" rel=\"Ext.fx.Anim-cfg-duration\" class=\"docClass\">duration</a>: Specifies the time period of the animation.</li>\n<li><a href=\"#!/api/Ext.fx.Anim-cfg-easing\" rel=\"Ext.fx.Anim-cfg-easing\" class=\"docClass\">easing</a>: Specifies the easing of the animation.</li>\n<li><a href=\"#!/api/Ext.fx.Anim-cfg-iterations\" rel=\"Ext.fx.Anim-cfg-iterations\" class=\"docClass\">iterations</a>: Allows the animation to repeat a number of times.</li>\n<li><a href=\"#!/api/Ext.fx.Anim-cfg-alternate\" rel=\"Ext.fx.Anim-cfg-alternate\" class=\"docClass\">alternate</a>: Used in conjunction with <a href=\"#!/api/Ext.fx.Anim-cfg-iterations\" rel=\"Ext.fx.Anim-cfg-iterations\" class=\"docClass\">iterations</a>, reverses the direction every second iteration.</li>\n</ul>\n\n\n<h2>Example Code</h2>\n\n<pre class='inline-example '><code>var myComponent = Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: document.body,\n    width: 200,\n    height: 200,\n    style: 'border: 1px solid red;'\n});\n\nExt.create('<a href=\"#!/api/Ext.fx.Anim\" rel=\"Ext.fx.Anim\" class=\"docClass\">Ext.fx.Anim</a>', {\n    target: myComponent,\n    duration: 1000,\n    from: {\n        width: 400 //starting width 400\n    },\n    to: {\n        width: 300, //end width 300\n        height: 300 // end height 300\n    }\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"alternate":{"!type":"bool","!doc":"<p>Used in conjunction with iterations to reverse the animation each time an iteration completes.</p>\n"},"autoEnd":{"!type":"bool","!doc":"<p><code>true</code> to immediately force this animation to its final state. This can be useful\nin cases where you want the final effect of an animation, but need to the actual\nanimation dynamically. Also see the <a href=\"#!/api/Ext.fx.Anim-method-jumpToEnd\" rel=\"Ext.fx.Anim-method-jumpToEnd\" class=\"docClass\">jumpToEnd</a> method.</p>\n"},"callback":{"!doc":"<p>A function to be run after the animation has completed.</p>\n"},"delay":{"!type":"number","!doc":"<p>Time to delay before starting the animation.</p>\n"},"duration":{"!type":"number","!doc":"<p>Time in milliseconds for a single animation to last. If the <a href=\"#!/api/Ext.fx.Anim-cfg-iterations\" rel=\"Ext.fx.Anim-cfg-iterations\" class=\"docClass\">iterations</a> property is\nspecified, then each animate will take the same duration for each iteration.</p>\n"},"dynamic":{"!type":"bool","!doc":"<p>Currently only for Component Animation: Only set a component's outer element size bypassing layouts.\nSet to true to do full layouts for every frame of the animation.</p>\n"},"easing":{"!type":"string","!doc":"<p>This describes how the intermediate values used during a transition will be calculated.\nIt allows for a transition to change speed over its duration.</p>\n\n<ul>\n<li>backIn</li>\n<li>backOut</li>\n<li>bounceIn</li>\n<li>bounceOut</li>\n<li>ease</li>\n<li>easeIn</li>\n<li>easeOut</li>\n<li>easeInOut</li>\n<li>elasticIn</li>\n<li>elasticOut</li>\n<li>cubic-bezier(x1, y1, x2, y2)</li>\n</ul>\n\n\n<p>Note that cubic-bezier will create a custom easing curve following the CSS3 <a href=\"http://www.w3.org/TR/css3-transitions/#transition-timing-function_tag\">transition-timing-function</a>\nspecification.  The four values specify points P1 and P2 of the curve as (x1, y1, x2, y2). All values must\nbe in the range [0, 1] or the definition is invalid.</p>\n"},"from":{"!doc":"<p>An object containing property/value pairs for the beginning of the animation.  If not specified, the current state of the\nExt.fx.target will be used. For example:</p>\n\n<pre><code>from: {\n    opacity: 0,       // Transparent\n    color: '#ffffff', // White\n    left: 0\n}\n</code></pre>\n"},"iterations":{"!type":"number","!doc":"<p>Number of times to execute the animation.</p>\n"},"keyframes":{"!doc":"<p>Animation keyframes follow the CSS3 Animation configuration pattern. 'from' is always considered '0%' and 'to'\nis considered '100%'. <strong>Every keyframe declaration must have a keyframe rule for 0% and 100%, possibly defined using\n\"from\" or \"to\".</strong>  A keyframe declaration without these keyframe selectors is invalid and will not be available for\nanimation.  The keyframe declaration for a keyframe rule consists of properties and values. Properties that are unable to\nbe animated are ignored in these rules, with the exception of 'easing' which can be changed at each keyframe. For example:</p>\n\n<pre><code>keyframes : {\n    '0%': {\n        left: 100\n    },\n    '40%': {\n        left: 150\n    },\n    '60%': {\n        left: 75\n    },\n    '100%': {\n        left: 100\n    }\n}\n</code></pre>\n"},"remove":{"!type":"bool","!doc":"<p><code>true</code> to remove the target when the animation is complete, using the appropriate removal\nmethod for the target. For example, a component will be destroyed, elements will be removed.</p>\n"},"reverse":{"!type":"bool","!doc":"<p>Run the animation from the end to the beginning\nDefaults to false.</p>\n"},"scope":{"!doc":"<p>The scope that the <a href=\"#!/api/Ext.fx.Anim-cfg-callback\" rel=\"Ext.fx.Anim-cfg-callback\" class=\"docClass\">callback</a> function will be called with</p>\n"},"target":{"!doc":"<p>The <a href=\"#!/api/Ext.fx.target.Target\" rel=\"Ext.fx.target.Target\" class=\"docClass\">Ext.fx.target.Target</a> to apply the animation to.  This should only be specified when creating an <a href=\"#!/api/Ext.fx.Anim\" rel=\"Ext.fx.Anim\" class=\"docClass\">Ext.fx.Anim</a> directly.\nThe target does not need to be a <a href=\"#!/api/Ext.fx.target.Target\" rel=\"Ext.fx.target.Target\" class=\"docClass\">Ext.fx.target.Target</a> instance, it can be the underlying object. For example, you can\npass a Component, Element or Sprite as the target and the Anim will create the appropriate <a href=\"#!/api/Ext.fx.target.Target\" rel=\"Ext.fx.target.Target\" class=\"docClass\">Ext.fx.target.Target</a> object\nautomatically.</p>\n"},"to":{"!doc":"<p>An object containing property/value pairs for the end of the animation. For example:</p>\n\n<pre><code>to: {\n    opacity: 1,       // Opaque\n    color: '#00ff00', // Green\n    left: 500\n}\n</code></pre>\n"},"bezierRE":{},"currentIteration":{"!type":"number","!doc":"<p>Current iteration the animation is running.</p>\n"},"damper":{"!type":"number"},"delayStart":{"!type":"number","!doc":"<p>used to track a delayed starting time</p>\n"},"endWasCalled":{"!type":"number"},"frameCount":{"!type":"number"},"isAnimation":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Anim, or subclass thereof.</p>\n"},"paused":{"!type":"bool","!doc":"<p>Flag to determine if the animation is paused. Only set this to true if you need to\nkeep the Anim instance around to be unpaused later; otherwise call <a href=\"#!/api/Ext.fx.Anim-method-end\" rel=\"Ext.fx.Anim-method-end\" class=\"docClass\">end</a>.</p>\n"},"propHandlers":{"!doc":"<p>Contains a cache of the interpolators to be used.</p>\n"},"running":{"!type":"bool","!doc":"<p>Flag to determine if the animation has started</p>\n"},"startTime":{"!doc":"<p>Starting time of the animation.</p>\n"},"end":{"!type":"fn() -> !this","!doc":"<p>Fire afteranimate event and end the animation. Usually called automatically when the\nanimation reaches its final frame, but can also be called manually to pre-emptively\nstop and destroy the running animation.</p>\n"},"initAttrs":{"!type":"fn() -> !this","!doc":"<p>Set up the initial currentAttrs hash.</p>\n"},"isReady":{"!type":"fn() -> !this"},"isRunning":{"!type":"fn() -> !this"},"jumpToEnd":{"!type":"fn() -> !this","!doc":"<p>Immediately force this animation to its final state.</p>\n"},"lastFrame":{"!type":"fn() -> ?","!doc":"<p>Perform lastFrame cleanup and handle iterations</p>\n"},"runAnim":{"!type":"fn(elapsedTime: ?) -> ?","!doc":"<p>Calculate attribute value at the passed timestamp.</p>\n"},"setAttr":{"!type":"fn(attr: ?, value: ?) -> !this","!doc":"<p>Helper to the target</p>\n"},"start":{"!type":"fn(startTime: ?) -> !this","!doc":"<p>Fires beforeanimate and sets the running flag.</p>\n"},"afteranimate":{"!type":"fn(this: ?, startTime: ?, eOpts: ?)","!doc":"<p>Fires when the animation is complete.</p>\n"},"beforeanimate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before the animation starts. A handler can return false to cancel the animation.</p>\n"},"lastframe":{"!type":"fn(this: ?, startTime: ?, eOpts: ?)","!doc":"<p>Fires when the animation's last frame has been set.</p>\n"}}},"Animator":{"!doc":"<p>This class is used to run keyframe based animations, which follows the CSS3 based animation structure.\nKeyframe animations differ from typical from/to animations in that they offer the ability to specify values\nat various points throughout the animation.</p>\n\n<h2>Using Keyframes</h2>\n\n<p>The <a href=\"#!/api/Ext.fx.Animator-cfg-keyframes\" rel=\"Ext.fx.Animator-cfg-keyframes\" class=\"docClass\">keyframes</a> option is the most important part of specifying an animation when using this\nclass. A key frame is a point in a particular animation. We represent this as a percentage of the\ntotal animation duration. At each key frame, we can specify the target values at that time. Note that\nyou <em>must</em> specify the values at 0% and 100%, the start and ending values. There is also a <a href=\"#!/api/Ext.fx.Animator-event-keyframe\" rel=\"Ext.fx.Animator-event-keyframe\" class=\"docClass\">keyframe</a>\nevent that fires after each key frame is reached.</p>\n\n<h2>Example</h2>\n\n<p>In the example below, we modify the values of the element at each fifth throughout the animation.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.fx.Animator\" rel=\"Ext.fx.Animator\" class=\"docClass\">Ext.fx.Animator</a>', {\n    target: Ext.getBody().createChild({\n        style: {\n            width: '100px',\n            height: '100px',\n            'background-color': 'red'\n        }\n    }),\n    duration: 10000, // 10 seconds\n    keyframes: {\n        0: {\n            opacity: 1,\n            backgroundColor: 'FF0000'\n        },\n        20: {\n            x: 30,\n            opacity: 0.5\n        },\n        40: {\n            x: 130,\n            backgroundColor: '0000FF'\n        },\n        60: {\n            y: 80,\n            opacity: 0.3\n        },\n        80: {\n            width: 200,\n            y: 200\n        },\n        100: {\n            opacity: 1,\n            backgroundColor: '00FF00'\n        }\n    }\n});\n</code></pre>\n","prototype":{"delay":{"!type":"number","!doc":"<p>Time to delay before starting the animation. Defaults to 0.</p>\n"},"duration":{"!type":"number","!doc":"<p>Time in milliseconds for the animation to last. Defaults to 250.</p>\n"},"dynamic":{"!type":"bool","!doc":"<p>Currently only for Component Animation: Only set a component's outer element size bypassing layouts.  Set to true to do full layouts for every frame of the animation.  Defaults to false.</p>\n"},"easing":{"!type":"string","!doc":"<p>This describes how the intermediate values used during a transition will be calculated. It allows for a transition to change\nspeed over its duration.</p>\n\n<ul>\n<li>backIn</li>\n<li>backOut</li>\n<li>bounceIn</li>\n<li>bounceOut</li>\n<li>ease</li>\n<li>easeIn</li>\n<li>easeOut</li>\n<li>easeInOut</li>\n<li>elasticIn</li>\n<li>elasticOut</li>\n<li>cubic-bezier(x1, y1, x2, y2)</li>\n</ul>\n\n\n<p>Note that cubic-bezier will create a custom easing curve following the CSS3 <a href=\"http://www.w3.org/TR/css3-transitions/#transition-timing-function_tag\">transition-timing-function</a>\nspecification.  The four values specify points P1 and P2 of the curve as (x1, y1, x2, y2). All values must\nbe in the range [0, 1] or the definition is invalid.</p>\n"},"iterations":{"!type":"number","!doc":"<p>Number of times to execute the animation. Defaults to 1.</p>\n"},"keyframes":{"!doc":"<p>Animation keyframes follow the CSS3 Animation configuration pattern. 'from' is always considered '0%' and 'to'\nis considered '100%'.<b>Every keyframe declaration must have a keyframe rule for 0% and 100%, possibly defined using\n\"from\" or \"to\"</b>.  A keyframe declaration without these keyframe selectors is invalid and will not be available for\nanimation.  The keyframe declaration for a keyframe rule consists of properties and values. Properties that are unable to\nbe animated are ignored in these rules, with the exception of 'easing' which can be changed at each keyframe. For example:</p>\n\n<pre><code>keyframes : {\n    '0%': {\n        left: 100\n    },\n    '40%': {\n        left: 150\n    },\n    '60%': {\n        left: 75\n    },\n    '100%': {\n        left: 100\n    }\n}\n </code></pre>\n\n"},"target":{"!doc":"<p>The Ext.fx.target to apply the animation to.  If not specified during initialization, this can be passed to the applyAnimator\nmethod to apply the same animation to many targets.</p>\n"},"animKeyFramesRE":{},"applyAnimator":{"!doc":"<p>Applies animation to the Ext.fx.target</p>\n"},"currentIteration":{"!type":"number","!doc":"<p>Current iteration the animation is running.</p>\n"},"damper":{"!type":"number"},"delayStart":{"!type":"number","!doc":"<p>private used to track a delayed starting time</p>\n"},"isAnimator":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Animator, or subclass thereof.</p>\n"},"keyframeStep":{"!type":"number","!doc":"<p>Current keyframe step of the animation.</p>\n"},"paused":{"!type":"bool","!doc":"<p>Flag to determine if the animation is paused. Only set this to true if you need to\nkeep the Anim instance around to be unpaused later; otherwise call <a href=\"#!/api/Ext.fx.Animator-method-end\" rel=\"Ext.fx.Animator-method-end\" class=\"docClass\">end</a>.</p>\n"},"running":{"!type":"bool","!doc":"<p>Flag to determine if the animation has started</p>\n"},"createTimeline":{"!type":"fn(keyframes: ?) -> !this","!doc":"<p>Takes the given keyframe configuration object and converts it into an ordered array with the passed attributes per keyframe\nor applying the 'to' configuration to all keyframes.  Also calculates the proper animation duration per keyframe.</p>\n"},"end":{"!type":"fn() -> !this","!doc":"<p>Fire afteranimate event and end the animation. Usually called automatically when the\nanimation reaches its final frame, but can also be called manually to pre-emptively\nstop and destroy the running animation.</p>\n"},"isReady":{"!type":"fn() -> !this"},"isRunning":{"!type":"fn() -> !this"},"lastFrame":{"!type":"fn() -> ?","!doc":"<p>Perform lastFrame cleanup and handle iterations</p>\n"},"sorter":{"!type":"fn(a: ?, b: ?) -> !this"},"start":{"!type":"fn(startTime: ?) -> !this","!doc":"<p>Fires beforeanimate and sets the running flag.</p>\n"},"afteranimate":{"!type":"fn(this: ?, startTime: ?, eOpts: ?)","!doc":"<p>Fires when the animation is complete.</p>\n"},"beforeanimate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before the animation starts. A handler can return false to cancel the animation.</p>\n"},"keyframe":{"!type":"fn(this: ?, keyframe: ?, eOpts: ?)","!doc":"<p>Fires at each keyframe.</p>\n"}}},"CubicBezier":{"cubicBezier":{"!type":"fn(x1: ?, y1: ?, x2: ?, y2: ?) -> !this"},"cubicBezierAtTime":{"!type":"fn(t: ?, p1x: ?, p1y: ?, p2x: ?, p2y: ?, duration: ?) -> !this","!doc":"<p>End Definitions</p>\n"}},"DrawPath":{"!doc":"<p>Provides SVG Paths handling functions. Copied from Ext.draw.Draw in ExtJs 4.2 in order\nto break the dependencies on parsePathString() and interpolatePaths() in PropertyHandler.js</p>\n","pathCommandRE":{},"pathToStringRE":{"!doc":"<p>End Definitions</p>\n"},"pathValuesRE":{},"radian":{},"stopsRE":{},"arc2curve":{"!type":"fn(x1: ?, y1: ?, rx: ?, ry: ?, angle: ?, large_arc_flag: ?, sweep_flag: ?, x2: ?, y2: ?, recursive: ?) -> !this"},"command2curve":{"!type":"fn(pathCommand: ?, d: ?) -> !this","!doc":"<p>Returns any path command as a curveto command based on the attrs passed</p>\n"},"interpolatePaths":{"!type":"fn(path: ?, path2: ?) -> !this"},"is":{"!type":"fn(o: ?, type: ?) -> !this"},"parsePathString":{"!type":"fn(pathString: ?) -> !this"},"path2string":{"!type":"fn() -> !this","!doc":"<p>To be deprecated, converts itself (an arrayPath) to a proper SVG path string</p>\n"},"pathClone":{"!type":"fn(pathArray: ?) -> !this"},"pathToAbsolute":{"!type":"fn(pathArray: ?) -> !this"},"pathToString":{"!type":"fn(arrayPath: ?) -> !this","!doc":"<p>Convert the passed arrayPath to a proper SVG path string (d attribute)</p>\n"},"quadratic2curve":{"!type":"fn(x1: ?, y1: ?, ax: ?, ay: ?, x2: ?, y2: ?) -> !this"},"rotate":{"!type":"fn(x: ?, y: ?, rad: ?) -> !this"}},"Easing":{"!doc":"<p>This class contains a series of function definitions used to modify values during an animation.\nThey describe how the intermediate values used during a transition will be calculated. It allows for a transition to change\nspeed over its duration. The following options are available:</p>\n\n<ul>\n<li>linear The default easing type</li>\n<li>backIn</li>\n<li>backOut</li>\n<li>bounceIn</li>\n<li>bounceOut</li>\n<li>ease</li>\n<li>easeIn</li>\n<li>easeOut</li>\n<li>easeInOut</li>\n<li>elasticIn</li>\n<li>elasticOut</li>\n<li>cubic-bezier(x1, y1, x2, y2)</li>\n</ul>\n\n\n<p>Note that cubic-bezier will create a custom easing curve following the CSS3 <a href=\"http://www.w3.org/TR/css3-transitions/#transition-timing-function_tag\">transition-timing-function</a>\nspecification.  The four values specify points P1 and P2 of the curve as (x1, y1, x2, y2). All values must\nbe in the range [0, 1] or the definition is invalid.</p>\n"},"Manager":{"!doc":"<p>Animation Manager which keeps track of all current animations and manages them on a frame by frame basis.</p>\n","!type":"fn()","forceJS":{"!type":"bool","!doc":"<p>Force the use of JavaScript-based animation instead of CSS3 animation, even when CSS3\nanimation is supported by the browser. This defaults to true currently, as CSS3 animation support is still\nconsidered experimental at this time, and if used should be thouroughly tested across all targeted browsers.</p>\n"},"interval":{"!type":"number","!doc":"<p>Default interval in miliseconds to calculate each frame.  Defaults to 16ms (~60fps)</p>\n"},"addAnim":{"!type":"fn(anim: ?) -> !this","!doc":"<p>Add an Anim to the manager. This is done automatically when an Anim instance is created.</p>\n"},"applyAnimAttrs":{"!type":"fn(target: ?, animWrap: ?) -> !this","!doc":"<p>Duplicating this code for performance reasons. We only want to apply the anims\nto a single animation because we're hitting the end. It may be out of sequence from\nthe runner timer.</p>\n"},"applyPendingAttrs":{"!type":"fn() -> !this","!doc":"<p>Apply all pending attribute changes to their targets</p>\n"},"collectTargetData":{"!type":"fn(anim: ?, timestamp: ?, useCSS3?: ?, isLastFrame?: ?) -> ?","!doc":"<p>Collect target attributes for the given Anim object at the given timestamp</p>\n"},"createTarget":{"!type":"fn(target: ?) -> !this","!doc":"<p>Target factory</p>\n"},"jumpToEnd":{"!type":"fn(anim: ?) -> !this"},"removeAnim":{"!type":"fn(anim: ?) -> !this","!doc":"<p>Remove an Anim from the manager. This is done automatically when an Anim ends.</p>\n"},"runAnim":{"!type":"fn(anim: ?, forceEnd: ?) -> !this","!doc":"<p>Run the individual animation for this frame</p>\n"},"runner":{"!type":"fn() -> !this","!doc":"<p>Runner function being called each frame</p>\n"},"startAnim":{"!type":"fn(anim: ?) -> !this","!doc":"<p>Start the individual animation (initialization)</p>\n"}},"Queue":{"getActiveAnimation":{"!type":"fn(targetId: ?) -> !this","!doc":"<p>Returns current animation object if the element has any effects actively running or queued, else returns false.</p>\n"},"getFxDefaults":{"!type":"fn(targetId: ?) -> !this"},"getFxQueue":{"!type":"fn(targetId: ?) -> !this","!doc":"<p>get fx queue for passed target, create if needed.</p>\n"},"hasFxBlock":{"!type":"fn(targetId: ?) -> !this"},"queueFx":{"!type":"fn(anim: ?) -> !this"},"setFxDefaults":{"!type":"fn(targetId: ?, obj: ?) -> !this"},"stopAnimation":{"!type":"fn(targetId: ?) -> !this"},"!doc":"<p>Animation Queue mixin to handle chaining and queueing by target.</p>\n","!type":"fn()","prototype":{"getActiveAnimation":{"!type":"fn(targetId: ?) -> !this","!doc":"<p>Returns current animation object if the element has any effects actively running or queued, else returns false.</p>\n"},"getFxDefaults":{"!type":"fn(targetId: ?) -> !this"},"getFxQueue":{"!type":"fn(targetId: ?) -> !this","!doc":"<p>get fx queue for passed target, create if needed.</p>\n"},"hasFxBlock":{"!type":"fn(targetId: ?) -> !this"},"queueFx":{"!type":"fn(anim: ?) -> !this"},"setFxDefaults":{"!type":"fn(targetId: ?, obj: ?) -> !this"},"stopAnimation":{"!type":"fn(targetId: ?) -> !this"}}},"PropertyHandler":{"prototype":{"color":{},"defaultHandler":{},"object":{},"path":{},"stringHandler":{}}},"target":{"Component":{"!doc":"<p>This class represents a animation target for a <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>. In general this class will not be\ncreated directly, the <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> will be passed to the animation and\nand the appropriate target will be created.</p>\n","!type":"fn(target: ?)","prototype":{"getPropMethod":{"!doc":"<p>Methods to call to retrieve unspecified \"from\" values from a target Component</p>\n"},"setMethods":{},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"getAttr":{"!type":"fn(attr: ?, val: ?) -> !this","!doc":"<p>Read the named attribute from the target Component. Use the defined getter for the attribute</p>\n"},"setAttr":{"!type":"fn(targetData: ?, isFirstFrame: ?, isLastFrame: ?) -> !this"}}},"Target":{"prototype":{"isAnimTarget":{"!type":"bool"},"getId":{"!type":"fn() -> !this"},"remove":{"!type":"fn() -> !this"}},"!doc":"<p>This class specifies a generic target for an animation. It provides a wrapper around a\nseries of different types of objects to allow for a generic animation API.\nA target can be a single object or a Composite object containing other objects that are\nto be animated. This class and it's subclasses are generally not created directly, the\nunderlying animation will create the appropriate <a href=\"#!/api/Ext.fx.target.Target\" rel=\"Ext.fx.target.Target\" class=\"docClass\">Ext.fx.target.Target</a> object by passing\nthe instance to be animated.</p>\n\n<p>The following types of objects can be animated:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.fx.target.Component\" rel=\"Ext.fx.target.Component\" class=\"docClass\">Components</a></li>\n<li><a href=\"#!/api/Ext.fx.target.Element\" rel=\"Ext.fx.target.Element\" class=\"docClass\">Elements</a></li>\n<li><a href=\"#!/api/Ext.fx.target.Sprite\" rel=\"Ext.fx.target.Sprite\" class=\"docClass\">Sprites</a></li>\n</ul>\n\n","!type":"fn(target: ?)"},"CompositeElement":{"!doc":"<p>This class represents a animation target for a Ext.CompositeElement. It allows\neach Ext.dom.Element in the group to be animated as a whole. In general this class will not be\ncreated directly, the Ext.CompositeElement will be passed to the animation and\nand the appropriate target will be created.</p>\n","!type":"fn(target: ?)","prototype":{"isComposite":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated CompositeElement, or subclass thereof.</p>\n"},"getAttr":{"!type":"fn(attr: ?, val: ?) -> !this"},"remove":{"!type":"fn() -> !this"},"setAttr":{"!type":"fn(targetData: ?) -> !this"}}},"Element":{"prototype":{"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"getElVal":{"!type":"fn(el: ?, attr: ?, val: ?) -> !this"},"setElVal":{"!type":"fn(element: ?, attr: ?, value: ?) -> !this"},"getAttr":{"!type":"fn(attr: ?, val: ?) -> !this"},"setAttr":{"!type":"fn(targetData: ?) -> !this"}},"!doc":"<p>This class represents a animation target for an Ext.dom.Element. In general this class will not be\ncreated directly, the Ext.dom.Element will be passed to the animation and\nand the appropriate target will be created.</p>\n","!type":"fn(target: ?)"},"CompositeElementCSS":{"!doc":"<p>This class represents a animation target for a Ext.CompositeElement, where the\nconstituent elements support CSS based animation. It allows each Ext.dom.Element in\nthe group to be animated as a whole. In general this class will not be created directly,\nthe Ext.CompositeElement will be passed to the animation and the appropriate target\nwill be created.</p>\n","!type":"fn(target: ?)","prototype":{"setAttr":{"!type":"fn() -> !this","!doc":"<p>End Definitions</p>\n"}}},"CompositeSprite":{"!doc":"<p>This class represents a animation target for a Ext.draw.sprite.Composite. It allows\neach Ext.draw.sprite.Sprite in the group to be animated as a whole. In general this class will not be\ncreated directly, the Ext.draw.sprite.Composite will be passed to the animation and\nand the appropriate target will be created.</p>\n","!type":"fn(target: ?)","prototype":{"getAttr":{"!type":"fn(attr: ?, val: ?) -> !this","!doc":"<p>End Definitions</p>\n"}}},"Sprite":{"prototype":{"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"getFromPrim":{"!type":"fn(sprite: ?, attr: ?) -> !this"},"setAttr":{"!type":"fn(targetData: ?) -> !this"},"getAttr":{"!type":"fn(attr: ?, val: ?) -> !this"}},"!doc":"<p>This class represents an animation target for a Ext.draw.sprite.Sprite. In general this class will not be\n created directly, the Ext.draw.sprite.Sprite will be passed to the animation and\n and the appropriate target will be created.</p>\n","!type":"fn(target: ?)"},"ElementCSS":{"!doc":"<p>This class represents a animation target for an Ext.dom.Element that supports CSS\nbased animation. In general this class will not be created directly, the Ext.dom.Element\nwill be passed to the animation and the appropriate target will be created.</p>\n","!type":"fn(target: ?)","prototype":{"setAttr":{"!type":"fn(targetData: ?, isFirstFrame: ?) -> !this","!doc":"<p>End Definitions</p>\n"}}}}},"grid":{"CellContext":{"!doc":"<p>Internal utility class that provides a unique cell context.</p>\n","!type":"fn(view: ?)","prototype":{"isCellContext":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated CellContext, or subclass thereof.</p>\n"},"equal":{"!type":"fn(other: ?) -> !this"},"isEqual":{"!type":"fn(other: ?) -> !this"},"setColumn":{"!type":"fn(col: ?) -> !this"},"setPosition":{"!type":"fn(row: ?, col: ?) -> !this","!doc":"<p>Selection row/record &amp; column/columnHeader</p>\n"},"setRow":{"!type":"fn(row: ?) -> !this"}}},"CellEditor":{"!doc":"<p>Internal utility class that provides default configuration for cell editing.</p>\n","!type":"fn(config: ?)","prototype":{"alignment":{"!type":"string","!doc":"<p>The position to align to (see Ext.util.Positionable.alignTo for more details).</p>\n"},"cls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element. This can be useful\nfor adding customized styles to the component or any of its children using standard CSS rules.</p>\n"},"hideEl":{"!type":"bool","!doc":"<p>False to keep the bound element visible while the editor is displayed</p>\n"},"shadow":{"!doc":"<p>\"sides\" for sides/bottom only, \"frame\" for 4-way shadow, and \"drop\" for bottom-right shadow.</p>\n"},"shim":{"!type":"bool"},"treeNodeSelector":{"!type":"string"},"y":{"!type":"number","!doc":"<p>Editor must appear at the top so that it does not contribute to scrollbars</p>\n"},"adjustPosition":{"!type":"fn() -> !this","!doc":"<p>Final position is decided upon by the Editor's realign() call which syncs position over the edited element.</p>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Fix checkbox blur when it is clicked.</p>\n"},"beforeViewRefresh":{"!type":"fn() -> !this"},"getTreeNodeOffset":{"!type":"fn(innerCell: ?) -> !this","!doc":"<p>private</p>\n"},"onCheckBoxClick":{"!type":"fn() -> !this","!doc":"<p>Restore checkbox focus and completeEdit method.</p>\n"},"onCheckBoxMouseDown":{"!type":"fn() -> !this","!doc":"<p>Because when checkbox is clicked it loses focus  completeEdit is bypassed.</p>\n"},"onDocMouseDown":{"!type":"fn(e: ?) -> !this"},"onEditComplete":{"!type":"fn(remainVisible: ?) -> !this","!doc":"<p>private</p>\n"},"onEditorTab":{"!type":"fn(e: ?) -> !this"},"onFieldBlur":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onHide":{"!type":"fn() -> !this","!doc":"<p>Shows the grid cell inner element when a cell editor is hidden</p>\n"},"onShow":{"!type":"fn() -> !this","!doc":"<p>Shows the editor, end ensures that it is rendered into the correct view\nHides the grid cell inner element when a cell editor is shown.</p>\n"},"onViewRefresh":{"!type":"fn() -> !this"},"realign":{"!type":"fn(autoSize: ?) -> !this","!doc":"<p>Realigns the Editor to the grid cell, or to the text node in the grid inner cell\nif the inner cell contains multiple child nodes.</p>\n"},"restoreCell":{"!type":"fn() -> !this"},"setGrid":{"!type":"fn(grid: ?) -> !this","!doc":"<p>Set the grid that owns this editor.\nUsually this will only <em>change</em> once, and the renderTo will cause\nrendering into the owning grid.\nHowever in a Lockable assembly the editor has to swap sides if the column is moved across.\nCalled by CellEditing#getEditor</p>\n"}}},"column":{"Action":{"!doc":"<p>A Grid header type which renders an icon, or a series of icons in a grid cell, and offers a scoped click\nhandler for each icon.</p>\n\n<pre class='inline-example '><code>Ext.create('Ext.data.Store', {\n    storeId:'employeeStore',\n    fields:['firstname', 'lastname', 'seniority', 'dep', 'hired'],\n    data:[\n        {firstname:\"Michael\", lastname:\"Scott\"},\n        {firstname:\"Dwight\", lastname:\"Schrute\"},\n        {firstname:\"Jim\", lastname:\"Halpert\"},\n        {firstname:\"Kevin\", lastname:\"Malone\"},\n        {firstname:\"Angela\", lastname:\"Martin\"}\n    ]\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Action Column Demo',\n    store: Ext.data.StoreManager.lookup('employeeStore'),\n    columns: [\n        {text: 'First Name',  dataIndex:'firstname'},\n        {text: 'Last Name',  dataIndex:'lastname'},\n        {\n            xtype:'actioncolumn',\n            width:50,\n            items: [{\n                icon: 'extjs/examples/shared/icons/fam/cog_edit.png',  // Use a URL in the icon config\n                tooltip: 'Edit',\n                handler: function(grid, rowIndex, colIndex) {\n                    var rec = grid.getStore().getAt(rowIndex);\n                    alert(\"Edit \" + rec.get('firstname'));\n                }\n            },{\n                icon: 'extjs/examples/restful/images/delete.png',\n                tooltip: 'Delete',\n                handler: function(grid, rowIndex, colIndex) {\n                    var rec = grid.getStore().getAt(rowIndex);\n                    alert(\"Terminate \" + rec.get('firstname'));\n                }\n            }]\n        }\n    ],\n    width: 250,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>The action column can be at any index in the columns array, and a grid can have any number of\naction columns.</p>\n","!type":"fn(config: ?)","prototype":{"altText":{"!type":"string","!doc":"<p>The alt text to use for the image element.</p>\n"},"disabled":{"!type":"bool","!doc":"<p>If true, the action will not respond to click events, and will be displayed semi-opaque.</p>\n\n<p>This Column may also be disabled on a row by row basis by configuring a <a href=\"#!/api/Ext.grid.column.Action-cfg-isDisabled\" rel=\"Ext.grid.column.Action-cfg-isDisabled\" class=\"docClass\">isDisabled</a> method.</p>\n"},"getClass":{"!doc":"<p>A function which returns the CSS class to apply to the icon image.</p>\n"},"getTip":{"!doc":"<p>A function which returns the tooltip string for any row.</p>\n"},"handler":{"!doc":"<p>A function called when the icon is clicked.</p>\n"},"icon":{"!type":"string","!doc":"<p>The URL of an image to display as the clickable element in the column.</p>\n\n<p>There are no default icons that come with Ext JS.</p>\n\n<p>Defaults to <code>Ext.BLANK_IMAGE_URL</code>.</p>\n"},"iconCls":{"!type":"string","!doc":"<p>A CSS class to apply to the icon image. To determine the class dynamically, configure the Column with\na <code><a href=\"#!/api/Ext.grid.column.Action-cfg-getClass\" rel=\"Ext.grid.column.Action-cfg-getClass\" class=\"docClass\">getClass</a></code> function.</p>\n\n<p>There are no default icon classes that come with Ext JS.</p>\n"},"isDisabled":{"!doc":"<p>A function which determines whether the action item for any row is disabled and returns <code>true</code> or <code>false</code>.</p>\n"},"items":{"!doc":"<p>An array of action items copied from the configured <a href=\"#!/api/Ext.grid.column.Action-cfg-items\" rel=\"Ext.grid.column.Action-cfg-items\" class=\"docClass\">items</a> configuration. Each will have\nan <code>enable</code> and <code>disable</code> method added which will enable and disable the associated action, and\nupdate the displayed icon accordingly.</p>\n"},"menuText":{"!type":"string","!doc":"<p>Text to display in this column's menu item if no <a href=\"#!/api/Ext.grid.column.Action-cfg-text\" rel=\"Ext.grid.column.Action-cfg-text\" class=\"docClass\">text</a> was specified as a header.</p>\n"},"scope":{"!doc":"<p>The scope (<code>this</code> reference) in which the <code><a href=\"#!/api/Ext.grid.column.Action-cfg-handler\" rel=\"Ext.grid.column.Action-cfg-handler\" class=\"docClass\">handler</a></code>, <code><a href=\"#!/api/Ext.grid.column.Action-cfg-getClass\" rel=\"Ext.grid.column.Action-cfg-getClass\" class=\"docClass\">getClass</a></code>, <code><a href=\"#!/api/Ext.grid.column.Action-cfg-isDisabled\" rel=\"Ext.grid.column.Action-cfg-isDisabled\" class=\"docClass\">isDisabled</a></code> and <code><a href=\"#!/api/Ext.grid.column.Action-cfg-getTip\" rel=\"Ext.grid.column.Action-cfg-getTip\" class=\"docClass\">getTip</a></code> fuctions are executed.\nDefaults to this Column.</p>\n"},"sortable":{"!type":"bool","!doc":"<p>False to disable sorting of this column. Whether local/remote sorting is used is specified in\n<code>Ext.data.Store.remoteSort</code>.</p>\n"},"stopSelection":{"!type":"bool","!doc":"<p>Prevent grid selection upon click.\nBeware that if you allow for the selection to happen then the selection model will steal focus from\nany possible floating window (like a message box) raised in the handler. This will prevent closing the\nwindow when pressing the Escape button since it will no longer contain a focused component.</p>\n"},"tooltip":{"!type":"string","!doc":"<p>A tooltip message to be displayed on hover. <a href=\"#!/api/Ext.tip.QuickTipManager-method-init\" rel=\"Ext.tip.QuickTipManager-method-init\" class=\"docClass\">Ext.tip.QuickTipManager</a> must\nhave been initialized.</p>\n\n<p>The tooltip may also be determined on a row by row basis by configuring a <a href=\"#!/api/Ext.grid.column.Action-cfg-getTip\" rel=\"Ext.grid.column.Action-cfg-getTip\" class=\"docClass\">getTip</a> method.</p>\n"},"actionIconCls":{"!type":"string"},"innerCls":{"!type":"string"},"cascade":{"!type":"fn(fn: ?, scope: ?) -> ?","!doc":"<p>Cascades down the component/container heirarchy from this component (passed in\nthe first call), calling the specified function with each component. The scope\n(this reference) of the function call will be the scope provided or the current\ncomponent. The arguments to the function will be the args provided or the current\ncomponent. If the function returns false at any point, the cascade is stopped on\nthat branch.</p>\n"},"defaultRenderer":{"!type":"fn(v: ?, cellValues: ?, record: ?, rowIdx: ?, colIdx: ?, store: ?, view: ?) -> !this","!doc":"<p>Renderer closure iterates through items creating an <img> element for each and tagging with an identifying\nclass name x-action-col-{n}</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroys the Component.</p>\n"},"disableAction":{"!type":"fn(index: ?, silent?: ?) -> !this","!doc":"<p>Disables this ActionColumn's action at the specified index.</p>\n"},"enableAction":{"!type":"fn(index: ?, silent?: ?) -> !this","!doc":"<p>Enables this ActionColumn's action at the specified index.</p>\n"},"getRefItems":{"!type":"fn() -> !this","!doc":"<p>Private override because this cannot function as a Container, and it has an items property which is an Array, NOT a MixedCollection.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"processEvent":{"!type":"fn(type: ?, view: ?, cell: ?, recordIndex: ?, cellIndex: ?, e: ?, record: ?, row: ?) -> !this","!doc":"<p>Process and refire events routed from the GridView's processEvent method.\nAlso fires any configured click handlers. By default, cancels the mousedown event to prevent selection.\nReturns the event handler's status to allow canceling of GridView's bubbling process.</p>\n"},"updater":{"!type":"fn(cell: ?, value: ?) -> !this"}}},"Column":{"prototype":{"align":{"!type":"string","!doc":"<p>Sets the alignment of the header and rendered columns.\nPossible values are: <code>'left'</code>, <code>'center'</code>, and <code>'right'</code>.</p>\n"},"baseCls":{"!type":"string","!doc":"<p>TODO: Implement visible triggers for touch.\nStyling will need tweaking - looks a bit ugly with all triggers always visible.\n   platformConfig: [{\n       platform: 'tablet',\n       triggerVisible: 'true'\n   }],</p>\n"},"cellWrap":{"!type":"bool","!doc":"<p>True to allow whitespace in this column's cells to wrap, and cause taller column height where\nnecessary.</p>\n\n<p>This implicitly sets the <a href=\"#!/api/Ext.grid.column.Column-cfg-variableRowHeight\" rel=\"Ext.grid.column.Column-cfg-variableRowHeight\" class=\"docClass\">variableRowHeight</a> config to <code>true</code></p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"columns":{"!doc":"<p>An optional array of sub-column definitions. This column becomes a group, and houses the columns defined in the\n<code>columns</code> config.</p>\n\n<p>Group columns may not be sortable. But they may be hideable and moveable. And you may move headers into and out\nof a group. Note that if all sub columns are dragged out of a group, the group is destroyed.</p>\n"},"componentLayout":{"!doc":"<p>The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout\nmanager which sizes a Component's internal structure in response to the Component being sized.</p>\n\n<p>Generally, developers will not use this configuration as all provided Components which need their internal\nelements sizing (Such as <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">input fields</a>) come with their own componentLayout managers.</p>\n\n<p>The <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used on instances of the base <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>\nclass which simply sizes the Component's encapsulating element to the height and width specified in the\n<a href=\"#!/api/Ext.grid.column.Column-method-setSize\" rel=\"Ext.grid.column.Column-method-setSize\" class=\"docClass\">setSize</a> method.</p>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"dataIndex":{"!type":"string","!doc":"<p>The name of the field in the grid's Ext.data.Store's Ext.data.Model definition from\nwhich to draw the column's value. <strong>Required.</strong></p>\n"},"detachOnRemove":{"!type":"bool","!doc":"<p>So that when removing from group headers which are then empty and then get destroyed, there's no child DOM left</p>\n"},"draggable":{"!type":"bool","!doc":"<p>False to disable drag-drop reordering of this column.</p>\n"},"editRenderer":{"!doc":"<p>A renderer to be used in conjunction with <a href=\"#!/api/Ext.grid.plugin.RowEditing\" rel=\"Ext.grid.plugin.RowEditing\" class=\"docClass\">RowEditing</a>. This renderer is used to\ndisplay a custom value for non-editable fields.</p>\n\n<p>If a string is passed it is assumed to be the name of a method defined by the\n<code><a href=\"#!/api/Ext.app.ViewController\" rel=\"Ext.app.ViewController\" class=\"docClass\">Ext.app.ViewController</a></code> or ancestor component identified as <code>defaultListenerScope</code>.</p>\n"},"editor":{"!doc":"<p>An optional xtype or config object for a <a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Field</a> to use for editing.\nOnly applicable if the grid is using an <a href=\"#!/api/Ext.grid.plugin.Editing\" rel=\"Ext.grid.plugin.Editing\" class=\"docClass\">Editing</a> plugin.</p>\n"},"emptyCellText":{"!type":"string","!doc":"<p>The text to diplay in empty cells (cells with a value of <code>undefined</code>, <code>null</code>, or <code>''</code>).</p>\n\n<p>Defaults to <code>&amp;#160;</code> aka <code>&amp;nbsp;</code>.</p>\n"},"field":{"!doc":"<p>Alias for <a href=\"#!/api/Ext.grid.column.Column-cfg-editor\" rel=\"Ext.grid.column.Column-cfg-editor\" class=\"docClass\">editor</a>.</p>\n"},"fixed":{"!type":"bool","!doc":"<p>True to prevent the column from being resizable.</p>\n"},"formatter":{"!type":"string","!doc":"<p>This config accepts a format specification as would be used in a <code>Ext.Template</code>\nformatted token. For example <code>'round(2)'</code> to round numbers to 2 decimal places\nor <code>'date(\"Y-m-d\")'</code> to format a Date.</p>\n\n<p>In previous releases the <code>renderer</code> config had limited abilities to use one\nof the <code>Ext.util.Format</code> methods but <code>formatter</code> now replaces that usage and\ncan also handle formatting parameters.</p>\n\n<p>When the value begins with <code>\"this.\"</code> (for example, <code>\"this.foo(2)\"</code>), the\nimplied scope on which \"foo\" is found is the <code>scope</code> config for the column.</p>\n\n<p>If the <code>scope</code> is not given, or implied using a prefix of <code>\"this\"</code>, then either the\nViewController or the closest ancestor component configured\nas defaultListenerScope is assumed to be the object with the method.</p>\n"},"groupable":{"!type":"bool","!doc":"<p>If the grid uses a <a href=\"#!/api/Ext.grid.feature.Grouping\" rel=\"Ext.grid.feature.Grouping\" class=\"docClass\">Ext.grid.feature.Grouping</a>, this option may be used to disable the header menu\nitem to group by the column selected. By default, the header menu group option is enabled. Set to false to\ndisable (but still show) the group option in the header menu for the column.</p>\n"},"header":{"!type":"string","!doc":"<p>The header text.</p>\n"},"headerWrap":{"!type":"bool","!doc":"<p>The default setting indicates that external CSS rules dictate that the title is <code>white-space: nowrap</code> and\ntherefore, width cannot affect the measured height by causing text wrapping. This is what the Sencha-supplied\nstyles set. If you change those styles to allow text wrapping, you must set this to <code>true</code>.</p>\n"},"hideable":{"!type":"bool","!doc":"<p>False to prevent the user from hiding this column.</p>\n"},"lockable":{"!type":"bool","!doc":"<p>If the grid is configured with <a href=\"#!/api/Ext.panel.Table-cfg-enableLocking\" rel=\"Ext.panel.Table-cfg-enableLocking\" class=\"docClass\">enableLocking</a>, or has columns which are\nconfigured with a <a href=\"#!/api/Ext.grid.column.Column-cfg-locked\" rel=\"Ext.grid.column.Column-cfg-locked\" class=\"docClass\">locked</a> value, this option may be used to disable user-driven locking or unlocking\nof this column. This column will remain in the side into which its own <a href=\"#!/api/Ext.grid.column.Column-cfg-locked\" rel=\"Ext.grid.column.Column-cfg-locked\" class=\"docClass\">locked</a> configuration placed it.</p>\n"},"locked":{"!type":"bool","!doc":"<p>True to lock this column in place.  Implicitly enables locking on the grid.\nSee also <a href=\"#!/api/Ext.grid.Panel-cfg-enableLocking\" rel=\"Ext.grid.Panel-cfg-enableLocking\" class=\"docClass\">Ext.grid.Panel.enableLocking</a>.</p>\n"},"menuDisabled":{"!type":"bool","!doc":"<p>True to disable the column header menu containing sort/hide options.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.grid.column.Column-method-getEl\" rel=\"Ext.grid.column.Column-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.grid.column.Column-method-getEl\" rel=\"Ext.grid.column.Column-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.grid.column.Column-cfg-renderSelectors\" rel=\"Ext.grid.column.Column-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.grid.column.Column-cfg-childEls\" rel=\"Ext.grid.column.Column-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"renderer":{"!doc":"<p>A renderer is an 'interceptor' method which can be used to transform data (value, appearance, etc.)\nbefore it is rendered. Example:</p>\n\n<pre><code>{\n    renderer: function(value){\n        if (value === 1) {\n            return '1 person';\n        }\n        return value + ' people';\n    }\n}\n</code></pre>\n\n<p>If a string is passed it is assumed to be the name of a method defined by the\nViewController or an ancestor component configured as defaultListenerScope.\nNote, in previous releases a string was treated as a method on <code>Ext.util.Format</code>\nbut that is now handled by the <a href=\"#!/api/Ext.grid.column.Column-cfg-formatter\" rel=\"Ext.grid.column.Column-cfg-formatter\" class=\"docClass\">formatter</a> config.</p>\n"},"resizable":{"!type":"bool","!doc":"<p>False to prevent the column from being resizable.</p>\n"},"stateId":{"!type":"string","!doc":"<p>An identifier which identifies this column uniquely within the owning grid's <a href=\"#!/api/Ext.grid.column.Column-cfg-stateful\" rel=\"Ext.grid.column.Column-cfg-stateful\" class=\"docClass\">state</a>.</p>\n\n<p>This does not have to be <em>globally</em> unique. A column's state is not saved standalone. It is encapsulated within\nthe owning grid's state.</p>\n"},"summaryRenderer":{"!doc":"<p>A renderer to be used in conjunction with the <a href=\"#!/api/Ext.grid.feature.Summary\" rel=\"Ext.grid.feature.Summary\" class=\"docClass\">Summary</a> or\n<a href=\"#!/api/Ext.grid.feature.GroupingSummary\" rel=\"Ext.grid.feature.GroupingSummary\" class=\"docClass\">GroupingSummary</a> features. This renderer is used to\ndisplay a summary value for this column.</p>\n\n<p>If a string is passed it is assumed to be the name of a method defined by the\n<code><a href=\"#!/api/Ext.app.ViewController\" rel=\"Ext.app.ViewController\" class=\"docClass\">Ext.app.ViewController</a></code> or ancestor component identified as <code>defaultListenerScope</code>.</p>\n"},"tdCls":{"!type":"string","!doc":"<p>A CSS class names to apply to the table cells for this column.</p>\n"},"text":{"!type":"string","!doc":"<p>The header text to be used as innerHTML (html tags are accepted) to display in the Grid.\n<strong>Note</strong>: to have a clickable header with no text displayed you can use the default of <code>&amp;#160;</code> aka <code>&amp;nbsp;</code>.</p>\n"},"tooltipType":{"!type":"string","!doc":"<p>The type of <a href=\"#!/api/Ext.grid.column.Column-cfg-tooltip\" rel=\"Ext.grid.column.Column-cfg-tooltip\" class=\"docClass\">tooltip</a> to use. Either 'qtip' for QuickTips or 'title' for title attribute.</p>\n"},"triggerVisible":{"!type":"bool"},"updater":{"!doc":"<p>An updater is a method which is used when records are updated, and an <em>existing</em> grid row needs updating.\nThe method is passed the cell element and may manipulate it in any way.</p>\n\n<p>If a string is passed it is assumed to be the name of a method defined by the\nViewController or an ancestor component configured as defaultListenerScope.</p>\n"},"variableRowHeight":{"!type":"bool","!doc":"<p>True to indicate that data in this column may take on an unpredictable height, possibly differing from row to row.</p>\n\n<p>If this is set, then View refreshes, and removal and addition of new rows will result in an ExtJS layout of the grid\nin order to adjust for possible addition/removal of scrollbars in the case of data changing height.</p>\n\n<p>This config also tells the View's buffered renderer that row heights are unpredictable, and must be remeasured as the view is refreshed.</p>\n"},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"ascSortCls":{"!type":"string"},"clickTargetName":{"!type":"string"},"deprecated":{"!doc":"<p>private</p>\n"},"descSortCls":{"!type":"string"},"formatterNames":{},"groupHeaderCls":{"!type":"string"},"groupSubHeaderCls":{"!type":"string"},"handleWidth":{},"hoverCls":{"!type":"string","!doc":"<p>Not the standard, automatically applied overCls because we must filter out overs of child headers.</p>\n"},"isColumn":{"!type":"bool","!doc":"<p>Set in this class to identify, at runtime, instances which are not instances of the\nHeaderContainer base class, but are in fact simple column headers.</p>\n"},"isHeader":{"!type":"bool"},"possibleSortStates":{},"producesHTML":{"!type":"bool","!doc":"<p>This flag indicates that the renderer produces HTML.</p>\n\n<p>If this column is going to be updated rapidly, and the <a href=\"#!/api/Ext.grid.column.Column-cfg-renderer\" rel=\"Ext.grid.column.Column-cfg-renderer\" class=\"docClass\">renderer</a> or\n<a href=\"#!/api/Ext.grid.column.Column-cfg-updater\" rel=\"Ext.grid.column.Column-cfg-updater\" class=\"docClass\">updater</a> only produces text, then to avoid the expense of HTML parsing\nand element production during the update, this property may be configured as <code>false</code>.</p>\n"},"rendererNames":{"!doc":"<p>Property names to reference the different types of renderers and fornatters that we can use.</p>\n"},"sortState":{},"textEl":{"!doc":"<p>Element that contains the text in column header.</p>\n"},"triggerEl":{"!doc":"<p>Element that acts as button for column header dropdown menu.</p>\n"},"afterComponentLayout":{"!type":"fn(width: ?, height: ?, oldWidth: ?, oldHeight: ?) -> !this","!doc":"<p>private\nInform the header container about the resize</p>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"applyColumnState":{"!type":"fn(state: ?) -> !this"},"autoSize":{"!type":"fn() -> !this","!doc":"<p>Sizes this Column to fit the max content width.\n<em>Note that group columns shrinkwrap around the size of leaf columns. Auto sizing a group column\nautosizes descendant leaf columns.</em></p>\n"},"beforeRender":{"!type":"fn() -> !this"},"bindFormatter":{"!type":"fn(format: ?) -> !this"},"bindRenderer":{"!type":"fn(renderer: ?) -> !this"},"getCellId":{"!type":"fn() -> !this"},"getCellInnerSelector":{"!type":"fn() -> !this"},"getCellSelector":{"!type":"fn() -> !this"},"getCellWidth":{"!type":"fn() -> !this"},"getColumnState":{"!type":"fn() -> !this"},"getEditor":{"!type":"fn(record: ?, defaultField: ?) -> ?","!doc":"<p>Retrieves the editing field for editing associated with this header. Returns false if there is no field\nassociated with the Header the method will return false. If the field has not been instantiated it will be\ncreated. Note: These methods only have an implementation if an Editing plugin has been enabled on the grid.</p>\n"},"getIndex":{"!type":"fn() -> number","!doc":"<p>Returns the index of this column only if this column is a base level Column. If it\nis a group column, it returns <code>false</code>.</p>\n"},"getSortParam":{"!type":"fn() -> string","!doc":"<p>Returns the parameter to sort upon when sorting this header. By default this returns the dataIndex and will not\nneed to be overriden in most cases.</p>\n"},"getStateId":{"!type":"fn() -> string","!doc":"<p>Gets the state id for this object.</p>\n"},"getTriggerVisible":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.grid.column.Column-cfg-triggerVisible\" rel=\"Ext.grid.column.Column-cfg-triggerVisible\" class=\"docClass\">triggerVisible</a>.</p>\n"},"getView":{"!type":"fn() -> !this"},"getVisibleIndex":{"!type":"fn() -> number","!doc":"<p>Returns the index of this column in the list of <em>visible</em> columns only if this column is a base level Column. If it\nis a group column, it returns <code>false</code>.</p>\n"},"hasMultipleVisibleChildren":{"!type":"fn(result: ?) -> !this","!doc":"<p>Private bubble function used in determining whether this column is lockable.\nExecutes in the scope of each component in the bubble sequence</p>\n"},"hasOtherMenuEnabledChildren":{"!type":"fn(result: ?) -> !this","!doc":"<p>Private bubble function used in determining whether this column is hideable.\nExecutes in the scope of each component in the bubble sequence</p>\n"},"hasVisibleChildColumns":{"!type":"fn() -> !this"},"hide":{"!type":"fn(fromOwner: ?) -> ?","!doc":"<p>Hides this Component, setting it to invisible using the configured <a href=\"#!/api/Ext.grid.column.Column-cfg-hideMode\" rel=\"Ext.grid.column.Column-cfg-hideMode\" class=\"docClass\">hideMode</a>.</p>\n"},"initDraggable":{"!type":"fn() -> !this","!doc":"<p>Header does not use the typical ComponentDraggable class and therefore we\noverride this with an emptyFn. It is controlled at the HeaderDragZone.</p>\n"},"initItems":{"!type":"fn() -> !this","!doc":"<p>This method is called to initialize the <code>items</code> collection. A derived class can\noverride this method to do any last minute manipulation of <code>items</code> and then call\nthis method using <code>callParent</code>. Upon return, the <code>items</code> will no longer be a simple\narray.</p>\n"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"initResizable":{"!type":"fn() -> !this","!doc":"<p>We need to override the default component resizable behaviour here</p>\n"},"isHideable":{"!type":"fn() -> !this","!doc":"<p>Determines whether the UI should be allowed to offer an option to hide this column.</p>\n\n<p>A column may <em>not</em> be hidden if to do so would leave the grid with no visible columns.</p>\n\n<p>This is used to determine the enabled/disabled state of header hide menu items.</p>\n"},"isLockable":{"!type":"fn() -> !this","!doc":"<p>Determines whether the UI should be allowed to offer an option to lock or unlock this column. Note\nthat this includes dragging a column into the opposite side of a <a href=\"#!/api/Ext.panel.Table-cfg-enableLocking\" rel=\"Ext.panel.Table-cfg-enableLocking\" class=\"docClass\">lockable</a> grid.</p>\n\n<p>A column may <em>not</em> be moved from one side to the other of a <a href=\"#!/api/Ext.panel.Table-cfg-enableLocking\" rel=\"Ext.panel.Table-cfg-enableLocking\" class=\"docClass\">lockable</a> grid\nif to do so would leave one side with no visible columns.</p>\n\n<p>This is used to determine the enabled/disabled state of the lock/unlock\nmenu item used in <a href=\"#!/api/Ext.panel.Table-cfg-enableLocking\" rel=\"Ext.panel.Table-cfg-enableLocking\" class=\"docClass\">lockable</a> grids, and to determine dropppabilty when dragging a header.</p>\n"},"isLocked":{"!type":"fn() -> !this","!doc":"<p>Determines whether this column is in the locked side of a grid. It may be a descendant node of a locked column\nand as such will <em>not</em> have the <a href=\"#!/api/Ext.grid.column.Column-cfg-locked\" rel=\"Ext.grid.column.Column-cfg-locked\" class=\"docClass\">locked</a> flag set.</p>\n"},"isOnLeftEdge":{"!type":"fn(e: ?) -> !this"},"isOnRightEdge":{"!type":"fn(e: ?, margin: ?) -> !this"},"onAdd":{"!type":"fn(child: ?) -> !this","!doc":"<p>Invalidate column cache on add\nWe cannot refresh the View on every add because this method is called\nwhen the HeaderDropZone moves Headers around, that will also refresh the view</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onDownKey":{"!type":"fn(e: ?) -> !this"},"onEnterKey":{"!type":"fn(e: ?) -> !this"},"onRemove":{"!type":"fn(child: ?) -> !this","!doc":"<p>Invalidate column cache on remove\nWe cannot refresh the View on every remove because this method is called\nwhen the HeaderDropZone moves Headers around, that will also refresh the view</p>\n"},"onResize":{"!type":"fn(width: ?, height: ?, oldWidth: ?, oldHeight: ?) -> !this","!doc":"<p>Allows addition of behavior to the resize operation.</p>\n\n<p>Called when Ext.resizer.Resizer#drag event is fired.</p>\n"},"onTitleElClick":{"!type":"fn(e: ?, t: ?) -> !this"},"onTitleElDblClick":{"!type":"fn(e: ?) -> !this","!doc":"<p>Double click handler which, if on left or right edges, auto-sizes the column to the left.</p>\n"},"onTitleMouseOut":{"!type":"fn() -> !this"},"onTitleMouseOver":{"!type":"fn() -> !this"},"setEditor":{"!type":"fn(field: ?)","!doc":"<p>Sets the form field to be used for editing. Note: This method only has an implementation if an Editing plugin has\nbeen enabled on the grid.</p>\n"},"setMenuActive":{"!type":"fn(isMenuOpen: ?) -> !this","!doc":"<p>Called when the column menu is activated/deactivated.\nChange the UI to indicate active/inactive menu</p>\n"},"setSortState":{"!type":"fn(sorter: ?) -> !this","!doc":"<p>Private\nSet the UI state to reflect the state of any passed Sorter\nCalled by the grid's HeaderContainer on view refresh</p>\n"},"setText":{"!type":"fn(text: ?) -> !this","!doc":"<p>Sets the header text for this Column.</p>\n"},"setTriggerVisible":{"!type":"fn(triggerVisible: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.grid.column.Column-cfg-triggerVisible\" rel=\"Ext.grid.column.Column-cfg-triggerVisible\" class=\"docClass\">triggerVisible</a>.</p>\n"},"setupRenderer":{"!type":"fn(type: ?) -> !this","!doc":"<p>type can be null or 'edit', or 'summary'</p>\n"},"show":{"!type":"fn(fromOwner: ?, fromChild: ?) -> ?","!doc":"<p>Shows this Component, rendering it first if <a href=\"#!/api/Ext.grid.column.Column-cfg-autoRender\" rel=\"Ext.grid.column.Column-cfg-autoRender\" class=\"docClass\">autoRender</a> or <a href=\"#!/api/Ext.grid.column.Column-cfg-floating\" rel=\"Ext.grid.column.Column-cfg-floating\" class=\"docClass\">floating</a> are <code>true</code>.</p>\n\n<p>After being shown, a <a href=\"#!/api/Ext.grid.column.Column-cfg-floating\" rel=\"Ext.grid.column.Column-cfg-floating\" class=\"docClass\">floating</a> Component (such as a <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>), is activated it and\nbrought to the front of its <a href=\"#!/api/Ext.grid.column.Column-property-zIndexManager\" rel=\"Ext.grid.column.Column-property-zIndexManager\" class=\"docClass\">z-index stack</a>.</p>\n"},"sort":{"!type":"fn(direction: ?) -> !this"},"toggleSortState":{"!type":"fn() -> !this"},"menuText":{"!type":"string","!doc":"<p>The text to render in the column visibility selection menu for this column.  If not\nspecified, will default to the text value.</p>\n"},"sortable":{"!type":"bool","!doc":"<p>False to disable sorting of this column. Whether local/remote sorting is used is specified in\n<code>Ext.data.Store.remoteSort</code>.</p>\n"},"tooltip":{"!type":"string","!doc":"<p>A tooltip to display for this column header</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"processEvent":{"!type":"fn(type: ?, view: ?, cell: ?, recordIndex: ?, cellIndex: ?, e: ?) -> !this","!doc":"<p>Process UI events from the view. The owning TablePanel calls this method, relaying events from the TableView</p>\n"},"scope":{"!doc":"<p>The scope to use when calling the <a href=\"#!/api/Ext.grid.column.Column-cfg-renderer\" rel=\"Ext.grid.column.Column-cfg-renderer\" class=\"docClass\">renderer</a> function.</p>\n"},"defaultRenderer":{"!type":"fn()","!doc":"<p>When defined this will take precedence over the <a href=\"#!/api/Ext.grid.column.Column-cfg-renderer\" rel=\"Ext.grid.column.Column-cfg-renderer\" class=\"docClass\">renderer</a> config.\nThis is meant to be defined in subclasses that wish to supply their own renderer.</p>\n"}},"!doc":"<p>This class specifies the definition for a column inside a <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>. It encompasses\nboth the grid header configuration as well as displaying data within the grid itself. If the\n<a href=\"#!/api/Ext.grid.column.Column-cfg-columns\" rel=\"Ext.grid.column.Column-cfg-columns\" class=\"docClass\">columns</a> configuration is specified, this column will become a column group and can\ncontain other columns inside. In general, this class will not be created directly, rather\nan array of column configurations will be passed to the grid:</p>\n\n<pre class='inline-example '><code>Ext.create('Ext.data.Store', {\n    storeId:'employeeStore',\n    fields:['firstname', 'lastname', 'seniority', 'dep', 'hired'],\n    data:[\n        {firstname:\"Michael\", lastname:\"Scott\", seniority:7, dep:\"Management\", hired:\"01/10/2004\"},\n        {firstname:\"Dwight\", lastname:\"Schrute\", seniority:2, dep:\"Sales\", hired:\"04/01/2004\"},\n        {firstname:\"Jim\", lastname:\"Halpert\", seniority:3, dep:\"Sales\", hired:\"02/22/2006\"},\n        {firstname:\"Kevin\", lastname:\"Malone\", seniority:4, dep:\"Accounting\", hired:\"06/10/2007\"},\n        {firstname:\"Angela\", lastname:\"Martin\", seniority:5, dep:\"Accounting\", hired:\"10/21/2008\"}\n    ]\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Column Demo',\n    store: Ext.data.StoreManager.lookup('employeeStore'),\n    columns: [\n        {text: 'First Name',  dataIndex:'firstname'},\n        {text: 'Last Name',  dataIndex:'lastname'},\n        {text: 'Hired Month',  dataIndex:'hired', xtype:'datecolumn', format:'M'},\n        {text: 'Department (Yrs)', xtype:'templatecolumn', tpl:'{dep} ({seniority})'}\n    ],\n    width: 400,\n    forceFit: true,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<h1>Convenience Subclasses</h1>\n\n<p>There are several column subclasses that provide default rendering for various data types</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.grid.column.Action\" rel=\"Ext.grid.column.Action\" class=\"docClass\">Ext.grid.column.Action</a>: Renders icons that can respond to click events inline</li>\n<li><a href=\"#!/api/Ext.grid.column.Boolean\" rel=\"Ext.grid.column.Boolean\" class=\"docClass\">Ext.grid.column.Boolean</a>: Renders for boolean values</li>\n<li><a href=\"#!/api/Ext.grid.column.Date\" rel=\"Ext.grid.column.Date\" class=\"docClass\">Ext.grid.column.Date</a>: Renders for date values</li>\n<li><a href=\"#!/api/Ext.grid.column.Number\" rel=\"Ext.grid.column.Number\" class=\"docClass\">Ext.grid.column.Number</a>: Renders for numeric values</li>\n<li><a href=\"#!/api/Ext.grid.column.Template\" rel=\"Ext.grid.column.Template\" class=\"docClass\">Ext.grid.column.Template</a>: Renders a value using an Ext.XTemplate using the record data</li>\n</ul>\n\n\n<h1>Setting Sizes</h1>\n\n<p>The columns are laid out by a <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">Ext.layout.container.HBox</a> layout, so a column can either\nbe given an explicit width value or a flex configuration. If no width is specified the grid will\nautomatically the size the column to 100px. For column groups, the size is calculated by measuring\nthe width of the child columns, so a width option should not be specified in that case.</p>\n\n<h1>Header Options</h1>\n\n<ul>\n<li><a href=\"#!/api/Ext.grid.column.Column-cfg-text\" rel=\"Ext.grid.column.Column-cfg-text\" class=\"docClass\">text</a>: Sets the header text for the column</li>\n<li><a href=\"#!/api/Ext.grid.column.Column-cfg-sortable\" rel=\"Ext.grid.column.Column-cfg-sortable\" class=\"docClass\">sortable</a>: Specifies whether the column can be sorted by clicking the header or using the column menu</li>\n<li><a href=\"#!/api/Ext.grid.column.Column-cfg-hideable\" rel=\"Ext.grid.column.Column-cfg-hideable\" class=\"docClass\">hideable</a>: Specifies whether the column can be hidden using the column menu</li>\n<li><a href=\"#!/api/Ext.grid.column.Column-cfg-menuDisabled\" rel=\"Ext.grid.column.Column-cfg-menuDisabled\" class=\"docClass\">menuDisabled</a>: Disables the column header menu</li>\n<li><a href=\"#!/api/Ext.grid.column.Column-cfg-draggable\" rel=\"Ext.grid.column.Column-cfg-draggable\" class=\"docClass\">draggable</a>: Specifies whether the column header can be reordered by dragging</li>\n<li><a href=\"#!/api/Ext.grid.column.Column-cfg-groupable\" rel=\"Ext.grid.column.Column-cfg-groupable\" class=\"docClass\">groupable</a>: Specifies whether the grid can be grouped by the column dataIndex. See also <a href=\"#!/api/Ext.grid.feature.Grouping\" rel=\"Ext.grid.feature.Grouping\" class=\"docClass\">Ext.grid.feature.Grouping</a></li>\n</ul>\n\n\n<h1>Data Options</h1>\n\n<ul>\n<li><a href=\"#!/api/Ext.grid.column.Column-cfg-dataIndex\" rel=\"Ext.grid.column.Column-cfg-dataIndex\" class=\"docClass\">dataIndex</a>: The dataIndex is the field in the underlying Ext.data.Store to use as the value for the column.</li>\n<li><a href=\"#!/api/Ext.grid.column.Column-cfg-renderer\" rel=\"Ext.grid.column.Column-cfg-renderer\" class=\"docClass\">renderer</a>: Allows the underlying store value to be transformed before being displayed in the grid</li>\n</ul>\n\n","!type":"fn(config: ?)"},"Boolean":{"!doc":"<p>A Column definition class which renders boolean data fields.  See the <a href=\"#!/api/Ext.grid.column.Column-cfg-xtype\" rel=\"Ext.grid.column.Column-cfg-xtype\" class=\"docClass\">xtype</a>\nconfig option of <a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">Ext.grid.column.Column</a> for more details.</p>\n\n<pre class='inline-example '><code>Ext.create('Ext.data.Store', {\n   storeId:'sampleStore',\n   fields:[\n       {name: 'framework', type: 'string'},\n       {name: 'rocks', type: 'boolean'}\n   ],\n   data:{'items':[\n       { 'framework': \"Ext JS 4\",     'rocks': true  },\n       { 'framework': \"Sencha Touch\", 'rocks': true  },\n       { 'framework': \"Ext GWT\",      'rocks': true  },\n       { 'framework': \"Other Guys\",   'rocks': false }\n   ]},\n   proxy: {\n       type: 'memory',\n       reader: {\n           type: 'json',\n           rootProperty: 'items'\n       }\n   }\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Boolean Column Demo',\n    store: Ext.data.StoreManager.lookup('sampleStore'),\n    columns: [\n        { text: 'Framework',  dataIndex: 'framework', flex: 1 },\n        {\n            xtype: 'booleancolumn',\n            text: 'Rocks',\n            trueText: 'Yes',\n            falseText: 'No',\n            dataIndex: 'rocks'\n        }\n    ],\n    height: 200,\n    width: 400,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"falseText":{"!type":"string","!doc":"<p>The string returned by the renderer when the column value is falsey (but not undefined).</p>\n"},"trueText":{"!type":"string","!doc":"<p>The string returned by the renderer when the column value is not falsey.</p>\n"},"undefinedText":{"!type":"string","!doc":"<p>The string returned by the renderer when the column value is undefined.</p>\n"},"defaultFilterType":{"!type":"string"},"defaultRenderer":{"!type":"fn(value: ?) -> !this","!doc":"<p>When defined this will take precedence over the <a href=\"#!/api/Ext.grid.column.Column-cfg-renderer\" rel=\"Ext.grid.column.Column-cfg-renderer\" class=\"docClass\">renderer</a> config.\nThis is meant to be defined in subclasses that wish to supply their own renderer.</p>\n"},"updater":{"!type":"fn(cell: ?, value: ?) -> !this"}}},"Check":{"!doc":"<p>A Column subclass which renders a checkbox in each column cell which toggles the truthiness of the associated data field on click.</p>\n\n<p>Example usage:</p>\n\n<pre class='inline-example '><code>var store = Ext.create('Ext.data.Store', {\n    fields : ['name', 'email', 'phone', 'active'],\n    data   : {\n        items : [\n            { name : 'Lisa',  email : 'lisa@simpsons.com',  phone : '555-111-1224', active : true  },\n            { name : 'Bart',  email : 'bart@simpsons.com',  phone : '555-222-1234', active : true  },\n            { name : 'Homer', email : 'homer@simpsons.com',  phone : '555-222-1244', active : false },\n            { name : 'Marge', email : 'marge@simpsons.com', phone : '555-222-1254', active : true  }\n        ]\n    },\n    proxy  : {\n        type   : 'memory',\n        reader : {\n            type : 'json',\n            root : 'items'\n        }\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title    : 'Simpsons',\n    height   : 200,\n    width    : 400,\n    renderTo : Ext.getBody(),\n    store    : store,\n    columns  : [\n        { text : 'Name', dataIndex : 'name' },\n        { text : 'Email', dataIndex : 'email', flex : 1 },\n        { text : 'Phone', dataIndex : 'phone' },\n        { xtype : 'checkcolumn', text : 'Active', dataIndex : 'active' }\n    ]\n});\n</code></pre>\n\n<p>The check column can be at any index in the columns array.</p>\n","!type":"fn(config: ?)","prototype":{"stopSelection":{"!type":"bool","!doc":"<p>Prevent grid selection upon mousedown.</p>\n"},"tdCls":{"!type":"string","!doc":"<p>A CSS class names to apply to the table cells for this column.</p>\n"},"clickTargetName":{"!type":"string"},"defaultFilterType":{"!type":"string"},"innerCls":{"!type":"string"},"_setDisabled":{"!type":"fn(disabled: ?) -> !this","!doc":"<p>Don't want to conflict with the Component method</p>\n"},"defaultRenderer":{"!type":"fn(value: ?, cellValues: ?) -> !this","!doc":"<p>Note: class names are not placed on the prototype bc renderer scope\nis not in the header.</p>\n"},"isRecordChecked":{"!type":"fn(record: ?) -> !this"},"onDisable":{"!type":"fn() -> !this","!doc":"<p>Disables this CheckColumn.</p>\n"},"onEnable":{"!type":"fn() -> !this","!doc":"<p>Enables this CheckColumn.</p>\n"},"processEvent":{"!type":"fn(type: ?, view: ?, cell: ?, recordIndex: ?, cellIndex: ?, e: ?, record: ?, row: ?) -> !this","!doc":"<p>Process and refire events routed from the GridView's processEvent method.</p>\n"},"setRecordCheck":{"!type":"fn(record: ?, checked: ?, cell: ?, row: ?, e: ?) -> !this"},"updater":{"!type":"fn(cell: ?, value: ?) -> !this"},"beforecheckchange":{"!type":"fn(this: ?, rowIndex: ?, checked: ?, eOpts: ?)","!doc":"<p>Fires when before checked state of a row changes.\nThe change may be vetoed by returning <code>false</code> from a listener.</p>\n"},"checkchange":{"!type":"fn(this: ?, rowIndex: ?, checked: ?, eOpts: ?)","!doc":"<p>Fires when the checked state of a row changes</p>\n"}}},"Date":{"!doc":"<p>A Column definition class which renders a passed date according to the default locale, or a configured\n<a href=\"#!/api/Ext.grid.column.Date-cfg-format\" rel=\"Ext.grid.column.Date-cfg-format\" class=\"docClass\">format</a>.</p>\n\n<pre class='inline-example '><code>Ext.create('Ext.data.Store', {\n    storeId:'sampleStore',\n    fields:[\n        { name: 'symbol', type: 'string' },\n        { name: 'date',   type: 'date' },\n        { name: 'change', type: 'number' },\n        { name: 'volume', type: 'number' },\n        { name: 'topday', type: 'date' }\n    ],\n    data:[\n        { symbol: \"msft\",   date: '2011/04/22', change: 2.43, volume: 61606325, topday: '04/01/2010' },\n        { symbol: \"goog\",   date: '2011/04/22', change: 0.81, volume: 3053782,  topday: '04/11/2010' },\n        { symbol: \"apple\",  date: '2011/04/22', change: 1.35, volume: 24484858, topday: '04/28/2010' },\n        { symbol: \"sencha\", date: '2011/04/22', change: 8.85, volume: 5556351,  topday: '04/22/2010' }\n    ]\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Date Column Demo',\n    store: Ext.data.StoreManager.lookup('sampleStore'),\n    columns: [\n        { text: 'Symbol',   dataIndex: 'symbol', flex: 1 },\n        { text: 'Date',     dataIndex: 'date',   xtype: 'datecolumn',   format:'Y-m-d' },\n        { text: 'Change',   dataIndex: 'change', xtype: 'numbercolumn', format:'0.00' },\n        { text: 'Volume',   dataIndex: 'volume', xtype: 'numbercolumn', format:'0,000' },\n        { text: 'Top Day',  dataIndex: 'topday', xtype: 'datecolumn',   format:'l' }\n    ],\n    height: 200,\n    width: 450,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"format":{"!type":"string","!doc":"<p>A formatting string as used by Ext.Date.format to format a Date for this Column.</p>\n\n<p>Defaults to the default date from Ext.Date.defaultFormat which itself my be overridden\nin a locale file.</p>\n"},"defaultFilterType":{"!type":"string"},"isDateColumn":{"!type":"bool"},"defaultRenderer":{"!type":"fn(value: ?) -> !this","!doc":"<p>When defined this will take precedence over the <a href=\"#!/api/Ext.grid.column.Column-cfg-renderer\" rel=\"Ext.grid.column.Column-cfg-renderer\" class=\"docClass\">renderer</a> config.\nThis is meant to be defined in subclasses that wish to supply their own renderer.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"updater":{"!type":"fn(cell: ?, value: ?) -> !this"}}},"Number":{"!doc":"<p>A Column definition class which renders a numeric data field according to a <a href=\"#!/api/Ext.grid.column.Number-cfg-format\" rel=\"Ext.grid.column.Number-cfg-format\" class=\"docClass\">format</a> string.</p>\n\n<pre class='inline-example '><code>Ext.create('Ext.data.Store', {\n   storeId:'sampleStore',\n   fields:[\n       { name: 'symbol', type: 'string' },\n       { name: 'price',  type: 'number' },\n       { name: 'change', type: 'number' },\n       { name: 'volume', type: 'number' }\n   ],\n   data:[\n       { symbol: \"msft\",   price: 25.76,  change: 2.43, volume: 61606325 },\n       { symbol: \"goog\",   price: 525.73, change: 0.81, volume: 3053782  },\n       { symbol: \"apple\",  price: 342.41, change: 1.35, volume: 24484858 },\n       { symbol: \"sencha\", price: 142.08, change: 8.85, volume: 5556351  }\n   ]\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Number Column Demo',\n    store: Ext.data.StoreManager.lookup('sampleStore'),\n    columns: [\n        { text: 'Symbol',         dataIndex: 'symbol', flex: 1 },\n        { text: 'Current Price',  dataIndex: 'price',  renderer: Ext.util.Format.usMoney },\n        { text: 'Change',         dataIndex: 'change', xtype: 'numbercolumn', format:'0.00' },\n        { text: 'Volume',         dataIndex: 'volume', xtype: 'numbercolumn', format:'0,000' }\n    ],\n    height: 200,\n    width: 400,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"format":{"!type":"string","!doc":"<p>A formatting string as used by Ext.util.Format.number to format a numeric value for this Column.</p>\n"},"defaultFilterType":{"!type":"string"},"defaultRenderer":{"!type":"fn(value: ?) -> !this","!doc":"<p>When defined this will take precedence over the <a href=\"#!/api/Ext.grid.column.Column-cfg-renderer\" rel=\"Ext.grid.column.Column-cfg-renderer\" class=\"docClass\">renderer</a> config.\nThis is meant to be defined in subclasses that wish to supply their own renderer.</p>\n"},"updater":{"!type":"fn(cell: ?, value: ?) -> !this"}}},"RowNumberer":{"!doc":"<p>A special type of Grid <a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">Ext.grid.column.Column</a> that provides automatic\nrow numbering.</p>\n\n<p>Usage:</p>\n\n<pre><code>columns: [\n    {xtype: 'rownumberer'},\n    {text: \"Company\", flex: 1, sortable: true, dataIndex: 'company'},\n    {text: \"Price\", width: 120, sortable: true, renderer: Ext.util.Format.usMoney, dataIndex: 'price'},\n    {text: \"Change\", width: 120, sortable: true, dataIndex: 'change'},\n    {text: \"% Change\", width: 120, sortable: true, dataIndex: 'pctChange'},\n    {text: \"Last Updated\", width: 120, sortable: true, renderer: Ext.util.Format.dateRenderer('m/d/Y'), dataIndex: 'lastChange'}\n]\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"align":{"!type":"string","!doc":"<p>Sets the alignment of the header and rendered columns.\nPossible values are: <code>'left'</code>, <code>'center'</code>, and <code>'right'</code>.</p>\n"},"cls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element. This can be useful\nfor adding customized styles to the component or any of its children using standard CSS rules.</p>\n"},"dataIndex":{"!type":"string","!doc":"<p>The name of the field in the grid's Ext.data.Store's Ext.data.Model definition from\nwhich to draw the column's value. <strong>Required.</strong></p>\n"},"draggable":{"!type":"bool","!doc":"<p>False to disable drag-drop reordering of this column.</p>\n"},"hideable":{"!type":"bool","!doc":"<p>False to prevent the user from hiding this column.</p>\n"},"lockable":{"!type":"bool","!doc":"<p>May not be moved from its preferred locked side when grid is enableLocking:true</p>\n"},"menuDisabled":{"!type":"bool","!doc":"<p>True to disable the column header menu containing sort/hide options.</p>\n"},"resizable":{"!type":"bool","!doc":"<p>private</p>\n"},"tdCls":{"!type":"string","!doc":"<p>A CSS class names to apply to the table cells for this column.</p>\n"},"text":{"!type":"string","!doc":"<p>Any valid text or HTML fragment to display in the header cell for the row number column.</p>\n"},"width":{"!type":"number","!doc":"<p>The default width in pixels of the row number column.</p>\n"},"autoLock":{"!type":"bool","!doc":"<p>Flag to Lockable to move instances of this column to the locked side.</p>\n"},"innerCls":{"!type":"string"},"producesHTML":{"!type":"bool","!doc":"<p>This flag indicates that the renderer produces HTML.</p>\n\n<p>If this column is going to be updated rapidly, and the <a href=\"#!/api/Ext.grid.column.RowNumberer-cfg-renderer\" rel=\"Ext.grid.column.RowNumberer-cfg-renderer\" class=\"docClass\">renderer</a> or\n<a href=\"#!/api/Ext.grid.column.RowNumberer-cfg-updater\" rel=\"Ext.grid.column.RowNumberer-cfg-updater\" class=\"docClass\">updater</a> only produces text, then to avoid the expense of HTML parsing\nand element production during the update, this property may be configured as <code>false</code>.</p>\n"},"rowspan":{},"defaultRenderer":{"!type":"fn(value: ?, metaData: ?, record: ?, rowIdx: ?, colIdx: ?, dataSource: ?, view: ?) -> !this","!doc":"<p>private</p>\n"},"updater":{"!type":"fn(cell: ?, value: ?) -> !this"}}},"Template":{"!doc":"<p>A Column definition class which renders a value by processing a Model's\ndata using a <a href=\"#!/api/Ext.grid.column.Template-cfg-tpl\" rel=\"Ext.grid.column.Template-cfg-tpl\" class=\"docClass\">configured</a>\nXTemplate.</p>\n\n<pre class='inline-example '><code>Ext.create('Ext.data.Store', {\n    storeId:'employeeStore',\n    fields:['firstname', 'lastname', 'seniority', 'department'],\n    groupField: 'department',\n    data:[\n        { firstname: \"Michael\", lastname: \"Scott\",   seniority: 7, department: \"Management\" },\n        { firstname: \"Dwight\",  lastname: \"Schrute\", seniority: 2, department: \"Sales\" },\n        { firstname: \"Jim\",     lastname: \"Halpert\", seniority: 3, department: \"Sales\" },\n        { firstname: \"Kevin\",   lastname: \"Malone\",  seniority: 4, department: \"Accounting\" },\n        { firstname: \"Angela\",  lastname: \"Martin\",  seniority: 5, department: \"Accounting\" }\n    ]\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Column Template Demo',\n    store: Ext.data.StoreManager.lookup('employeeStore'),\n    columns: [\n        { text: 'Full Name',       xtype: 'templatecolumn', tpl: '{firstname} {lastname}', flex:1 },\n        { text: 'Department (Yrs)', xtype: 'templatecolumn', tpl: '{department} ({seniority})' }\n    ],\n    height: 200,\n    width: 300,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"tpl":{"!doc":"<p>An XTemplate, or an XTemplate <em>definition string</em> to use to process a\nModel's data object to produce a cell's rendered value.</p>\n"},"defaultRenderer":{"!type":"fn(value: ?, meta: ?, record: ?) -> !this","!doc":"<p>When defined this will take precedence over the <a href=\"#!/api/Ext.grid.column.Column-cfg-renderer\" rel=\"Ext.grid.column.Column-cfg-renderer\" class=\"docClass\">renderer</a> config.\nThis is meant to be defined in subclasses that wish to supply their own renderer.</p>\n"},"updater":{"!type":"fn(cell: ?, value: ?) -> !this"}}},"Widget":{"!doc":"<p>A widget column is configured with a <a href=\"#!/api/Ext.grid.column.Widget-cfg-widget\" rel=\"Ext.grid.column.Widget-cfg-widget\" class=\"docClass\">widget</a> config object which specifies an\n<a href=\"#!/api/Ext.Component-cfg-xtype\" rel=\"Ext.Component-cfg-xtype\" class=\"docClass\">xtype</a> to indicate which type of Widget or Component belongs\nin the cells of this column.</p>\n\n<p>When a widget cell is rendered, a Widget or <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a> of the specified type\nis rendered into that cell. Its <a href=\"#!/api/Ext.Component-property-defaultBindProperty\" rel=\"Ext.Component-property-defaultBindProperty\" class=\"docClass\">defaultBindProperty</a> is set using this\ncolumn's <a href=\"#!/api/Ext.grid.column.Widget-cfg-dataIndex\" rel=\"Ext.grid.column.Widget-cfg-dataIndex\" class=\"docClass\">dataIndex</a> field from the associated record.</p>\n\n<p>In the example below we are monitoring the throughput of electricity substations. The capacity being\nused as a proportion of the maximum rated capacity is displayed as a progress bar. As new data arrives and the\ninstananeous usage value is updated, the <code>capacityUsed</code> field updates itself, and that is used as the <a href=\"#!/api/Ext.grid.column.Widget-cfg-dataIndex\" rel=\"Ext.grid.column.Widget-cfg-dataIndex\" class=\"docClass\">dataIndex</a>\nfor the <code>WidgetColumn</code> which contains the progress bar. The progess Bar's\n<a href=\"#!/api/Ext.ProgressBarWidget-property-defaultBindProperty\" rel=\"Ext.ProgressBarWidget-property-defaultBindProperty\" class=\"docClass\">defaultBindProperty</a> (which is \"value\") is set to the calculated <code>capacityUsed</code>.</p>\n\n<pre class='inline-example '><code>var grid = new <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>({\n    title: 'Substation power monitor',\n    width: 600,\n    columns: [{\n        text: 'Id',\n        dataIndex: 'id',\n        width: 120\n    }, {\n        text: 'Rating',\n        dataIndex: 'maxCapacity',\n        width: 80\n    }, {\n        text: 'Avg.',\n        dataIndex: 'avg',\n        width: 85,\n        formatter: 'number(\"0.00\")'\n    }, {\n        text: 'Max',\n        dataIndex: 'max',\n        width: 80\n    }, {\n        text: 'Instant',\n        dataIndex: 'instant',\n        width: 80\n    }, {\n        text: '%Capacity',\n        width: 150,\n\n        // This is our Widget column\n        xtype: 'widgetcolumn',\n        dataIndex: 'capacityUsed',\n\n        // This is the widget definition for each cell.\n        // Its \"value\" setting is taken from the column's dataIndex\n        widget: {\n        xtype: 'progressbarwidget',\n            textTpl: [\n                '{percent:number(\"0\")}% capacity'\n            ]\n        }\n    }],\n    renderTo: document.body,\n    disableSelection: true,\n    store: {\n       fields: [{\n           name: 'id',\n           type: 'string'\n       }, {\n           name: 'maxCapacity',\n           type: 'int'\n       }, {\n           name: 'avg',\n           type: 'int',\n           calculate: function(data) {\n               // Make this depend upon the instant field being set which sets the sampleCount and total.\n               // Use subscript format to access the other psuedo fields which are set by the instant field's converter\n               return data.instant &amp;&amp; data['total'] / data['sampleCount'];\n           }\n       }, {\n           name: 'max',\n           type: 'int',\n           calculate: function(data) {\n               // This will be seen to depend on the \"instant\" field.\n               // Use subscript format to access this field's current value to avoid circular dependency error.\n               return (data['max'] || 0) &lt; data.instant ? data.instant : data['max'];\n           }\n       }, {\n           name: 'instant',\n           type: 'int',\n\n           // Upon every update of instananeous power throughput,\n           // update the sample count and total so that the max field can calculate itself\n           convert: function(value, rec) {\n               rec.data.sampleCount = (rec.data.sampleCount || 0) + 1;\n               rec.data.total = (rec.data.total || 0) + value;\n               return value;\n           },\n          depends: []\n       }, {\n           name: 'capacityUsed',\n           calculate: function(data) {\n               return data.instant / data.maxCapacity;\n           }\n       }],\n       data: [{\n           id: 'Substation A',\n           maxCapacity: 1000,\n           avg: 770,\n           max: 950,\n           instant: 685\n       }, {\n           id: 'Substation B',\n           maxCapacity: 1000,\n           avg: 819,\n           max: 992,\n           instant: 749\n       }, {\n           id: 'Substation C',\n           maxCapacity: 1000,\n           avg: 588,\n             max: 936,\n           instant: 833\n       }, {\n           id: 'Substation D',\n           maxCapacity: 1000,\n           avg: 639,\n           max: 917,\n           instant: 825\n       }]\n   }\n});\n\n// Fake data updating...\n// Change one record per second to a random power value\nsetInterval(function() {\n    var recIdx = Ext.Number.randomInt(0, 3),\n        newPowerReading = Ext.Number.randomInt(500, 1000);\n\n    grid.store.getAt(recIdx).set('instant', newPowerReading);\n}, 1000);\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"defaultCellUI":{"!doc":"<p>{Object}\nA map of xtype to <a href=\"#!/api/Ext.Component-cfg-ui\" rel=\"Ext.Component-cfg-ui\" class=\"docClass\">Ext.Component.ui</a> names to use when using Components in this column.</p>\n\n<p>Currently <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Button</a> and all subclasses of <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">TextField</a> default\nto using <code>ui: \"default\"</code> when in a WidgetColumn except for in the \"classic\" theme, when they use ui \"grid-cell\".</p>\n"},"stopSelection":{"!type":"bool","!doc":"<p>Prevent grid selection upon click on the widget.</p>\n"},"widget":{"!doc":"<p>A config object containing an <a href=\"#!/api/Ext.Component-cfg-xtype\" rel=\"Ext.Component-cfg-xtype\" class=\"docClass\">xtype</a>.</p>\n\n<p>This is used to create the widgets or components which are rendered into the cells of this column.</p>\n\n<p>This column's <a href=\"#!/api/Ext.grid.column.Widget-cfg-dataIndex\" rel=\"Ext.grid.column.Widget-cfg-dataIndex\" class=\"docClass\">dataIndex</a> is used to update the widget/component's <a href=\"#!/api/Ext.Component-property-defaultBindProperty\" rel=\"Ext.Component-property-defaultBindProperty\" class=\"docClass\">defaultBindProperty</a>.</p>\n"},"preventUpdate":{"!type":"bool"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"sortable":{"!type":"bool"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"defaultRenderer":{"!type":"fn() -> !this","!doc":"<p>Cell must be left blank</p>\n"},"getWidget":{"!type":"fn(record: ?) -> !this"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"onAdded":{"!type":"fn() -> !this","!doc":"<p>Method to manage awareness of when components are added to their\nrespective Container, firing an <a href=\"#!/api/Ext.grid.column.Widget-event-added\" rel=\"Ext.grid.column.Widget-event-added\" class=\"docClass\">added</a> event. References are\nestablished at add time rather than at render time.</p>\n\n<p>Allows addition of behavior when a Component is added to a\nContainer. At this stage, the Component is in the parent\nContainer's collection of child items. After calling the\nsuperclass's <code>onAdded</code>, the <code>ownerCt</code> reference will be present,\nand if configured with a ref, the <code>refOwner</code> will be set.</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onRemoved":{"!type":"fn(isDestroying: ?) -> !this","!doc":"<p>Method to manage awareness of when components are removed from their\nrespective Container, firing a <a href=\"#!/api/Ext.grid.column.Widget-event-removed\" rel=\"Ext.grid.column.Widget-event-removed\" class=\"docClass\">removed</a> event. References are properly\ncleaned up after removing a component from its owning container.</p>\n\n<p>Allows addition of behavior when a Component is removed from\nits parent Container. At this stage, the Component has been\nremoved from its parent Container's collection of child items,\nbut has not been destroyed (It will be destroyed if the parent\nContainer's <code>autoDestroy</code> is <code>true</code>, or if the remove call was\npassed a truthy second parameter). After calling the\nsuperclass's <code>onRemoved</code>, the <code>ownerCt</code> and the <code>refOwner</code> will not\nbe present.</p>\n"},"onResize":{"!type":"fn(newWidth: ?) -> !this","!doc":"<p>Allows addition of behavior to the resize operation.</p>\n\n<p>Called when Ext.resizer.Resizer#drag event is fired.</p>\n"},"processEvent":{"!type":"fn(type: ?, view: ?, cell: ?, recordIndex: ?, cellIndex: ?, e: ?) -> !this","!doc":"<p>Process UI events from the view. The owning TablePanel calls this method, relaying events from the TableView</p>\n"},"updater":{"!type":"fn(cell: ?, value: ?, record: ?) -> !this"}}}},"header":{"Container":{"prototype":{"border":{"!doc":"<p>Specifies the border size for this component. The border can be a single numeric value to apply to all sides or it can\nbe a CSS style specification for each style, for example: '10 5 3 10' (top, right, bottom, left).</p>\n\n<p>For components that have no border by default, setting this won't make the border appear by itself.\nYou also need to specify border color and style:</p>\n\n<pre><code>border: 5,\nstyle: {\n    borderColor: 'red',\n    borderStyle: 'solid'\n}\n</code></pre>\n\n<p>To turn off the border, use <code>border: false</code>.</p>\n"},"defaultType":{"!type":"string","!doc":"<p>The default <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">xtype</a> of child Components to create in this Container when\na child item is specified as a raw configuration object, rather than as an instantiated Component.</p>\n"},"defaultWidth":{"!type":"number","!doc":"<p>Width of the header if no width or flex is specified.</p>\n"},"enableColumnHide":{"!type":"bool","!doc":"<p>False to disable column hiding within this grid.</p>\n"},"sealed":{"!type":"bool","!doc":"<p>Specify as <code>true</code> to constrain column dragging so that a column cannot be dragged into or out of this column.</p>\n\n<p><strong>Note that this config is only valid for column headers which contain child column headers, eg:</strong>\n    {\n        sealed: true\n        text: 'ExtJS',\n        columns: [{\n            text: '3.0.4',\n            dataIndex: 'ext304'\n        }, {\n            text: '4.1.0',\n            dataIndex: 'ext410'\n        }\n    }</p>\n"},"weight":{"!type":"number","!doc":"<p>HeaderContainer overrides the default weight of 0 for all docked items to 100.\nThis is so that it has more priority over things like toolbars.</p>\n"},"columnsText":{"!type":"string","!doc":"<p>\n<locale></locale></p>\n"},"ddLock":{"!type":"bool"},"dock":{"!type":"string"},"dragging":{"!type":"bool"},"headerOpenCls":{"!type":"string","!doc":"<p></p>\n"},"isGroupHeader":{"!type":"bool","!doc":"<p>True if this HeaderContainer is in fact a group header which contains sub headers.</p>\n"},"menuColsIcon":{"!type":"string"},"menuSortAscCls":{"!type":"string"},"menuSortDescCls":{"!type":"string"},"sortAscText":{"!type":"string","!doc":"<p><locale></locale></p>\n"},"sortClearText":{"!type":"string","!doc":"<p>\n<locale></locale></p>\n"},"sortDescText":{"!type":"string","!doc":"<p>\n<locale></locale></p>\n"},"applyColumnsState":{"!type":"fn(columns: ?) -> !this"},"applyForceFit":{"!type":"fn(header: ?) -> !this"},"autoSizeColumn":{"!type":"fn(header: ?) -> !this"},"beforeMenuShow":{"!type":"fn(menu: ?) -> !this","!doc":"<p>Render our menus to the first enclosing scrolling element so that they scroll with the grid</p>\n"},"getColumnCount":{"!type":"fn() -> !this","!doc":"<p>Returns the number of <b>grid columns</b> descended from this HeaderContainer.\nGroup Columns are HeaderContainers. All grid columns are returned, including hidden ones.</p>\n"},"getColumnMenu":{"!type":"fn(headerContainer: ?) -> !this","!doc":"<p>Returns an array of menu CheckItems corresponding to all immediate children\nof the passed Container which have been configured as hideable.</p>\n"},"getColumnsState":{"!type":"fn() -> !this"},"getGridColumns":{"!type":"fn(inResult: ?, hiddenAncestor: ?) -> ?","!doc":"<p>Returns an array of all columns which appear in the grid's View. This goes down to the leaf column header\nlevel, and does not return <strong>grouped</strong> headers which contain sub headers.</p>\n\n<p>It includes hidden headers even though they are not rendered. This is for collection of menu items for the column hide/show menu.</p>\n\n<p>Headers which have a hidden ancestor have a <code>hiddenAncestor: true</code> property injected so that descendants are known to be hidden without interrogating\nthat header's ownerCt axis for a hidden ancestor.</p>\n"},"getHeaderAtIndex":{"!type":"fn(index: ?) -> !this","!doc":"<p>Get a leaf level header by index regardless of what the nesting\nstructure is.</p>\n"},"getHeaderElByEvent":{"!type":"fn(e: ?) -> !this"},"getHeaderIndex":{"!type":"fn(header: ?) -> number","!doc":"<p>Returns the index of a leaf level header regardless of what the nesting\nstructure is.</p>\n\n<p>If a group header is passed, the index of the first leaf level header within it is returned.</p>\n"},"getHeaderMenu":{"!type":"fn() -> !this"},"getHideableColumns":{"!type":"fn() -> !this","!doc":"<p>For use by column headers in determining whether there are any hideable columns when deciding whether or not\nthe header menu should be disabled.</p>\n"},"getMenu":{"!type":"fn() -> !this","!doc":"<p>Gets the menu (and will create it if it doesn't already exist)</p>\n"},"getMenuItemForHeader":{"!type":"fn(menu: ?, header: ?) -> !this"},"getMenuItems":{"!type":"fn() -> ?","!doc":"<p>Returns an array of menu items to be placed into the shared menu\nacross all headers in this header container.</p>\n"},"getOwnerHeaderCt":{"!type":"fn() -> !this","!doc":"<p>Find the topmost HeaderContainer</p>\n"},"getTableWidth":{"!type":"fn() -> !this","!doc":"<p>Gets the full width of all columns that are visible for setting width of tables.</p>\n"},"getVisibleGridColumns":{"!type":"fn() -> ?","!doc":"<p>Returns an array of the <strong>visible</strong> columns in the grid. This goes down to the lowest column header\nlevel, and does not return <strong>grouped</strong> headers which contain sub headers.</p>\n"},"getVisibleHeaderClosestToIndex":{"!type":"fn(index: ?) -> !this","!doc":"<p>When passed a column index, returns the closet <em>visible</em> column to that. If the column at the passed index is visible,\nthat is returned. If it is hidden, either the next visible, or the previous visible column is returned.</p>\n"},"hideMenu":{"!type":"fn() -> !this"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>Initialize any events on this component</p>\n"},"isLayoutRoot":{"!type":"fn() -> !this","!doc":"<p>Determines whether this Component is the root of a layout. This returns <code>true</code> if\nthis component can run its layout without assistance from or impact on its owner.\nIf this component cannot run its layout given these restrictions, <code>false</code> is returned\nand its owner will be considered as the next candidate for the layout root.</p>\n\n<p>Setting the <a href=\"#!/api/Ext.grid.header.Container-property-_isLayoutRoot\" rel=\"Ext.grid.header.Container-property-_isLayoutRoot\" class=\"docClass\">_isLayoutRoot</a> property to <code>true</code> causes this method to always\nreturn <code>true</code>. This may be useful when updating a layout of a Container which shrink\nwraps content, and you know that it will not change size, and so can safely be the\ntopmost participant in the layout run.</p>\n"},"lookupComponent":{"!type":"fn(comp: ?) -> !this"},"move":{"!type":"fn(fromIdx: ?, toIdx: ?) -> ?","!doc":"<p>Moves a Component within the Container. This method does <strong>not</strong> account for things\nlike splitter components added by a layout. To better handle these situations, it\nis recommended to use <code><a href=\"#!/api/Ext.grid.header.Container-method-moveBefore\" rel=\"Ext.grid.header.Container-method-moveBefore\" class=\"docClass\">moveBefore</a></code> or <code>moveAfter</code> instead.</p>\n"},"moveHeader":{"!type":"fn(fromIdx: ?, toIdx: ?) -> !this"},"onColumnCheckChange":{"!type":"fn(checkItem: ?, checked: ?) -> !this"},"onColumnsChanged":{"!type":"fn() -> !this","!doc":"<p>Private\nOnly called on the grid's headerCt.\nCalled whenever a column is added or removed or moved at any level below.\nEnsures that the gridColumns caches are cleared.</p>\n"},"onHeaderClick":{"!type":"fn(header: ?, e: ?, t: ?) -> !this"},"onHeaderContextMenu":{"!type":"fn(header: ?, e: ?, t: ?) -> !this"},"onHeaderCtEvent":{"!type":"fn(e: ?, t: ?) -> !this"},"onHeaderCtLongPress":{"!type":"fn(e: ?) -> !this"},"onHeaderCtMouseOut":{"!type":"fn(e: ?, t: ?) -> !this"},"onHeaderCtMouseOver":{"!type":"fn(e: ?, t: ?) -> !this"},"onHeaderHide":{"!type":"fn(header: ?) -> !this"},"onHeaderMoved":{"!type":"fn(header: ?, colsToMove: ?, fromIdx: ?, toIdx: ?) -> !this","!doc":"<p>Private</p>\n"},"onHeaderResize":{"!type":"fn(header: ?, w: ?) -> !this"},"onHeaderShow":{"!type":"fn(header: ?) -> !this"},"onHeaderTriggerClick":{"!type":"fn(header: ?, e: ?, t: ?) -> !this"},"onHeaderVisibilityChange":{"!type":"fn(header: ?, visible: ?) -> !this"},"onHeadersChanged":{"!type":"fn(c: ?, isMove: ?) -> !this","!doc":"<p>Private\nCalled to clear all caches of columns whenever columns are added, removed to just moved.\nWe need to be informed if it's just a move operation so that we don't call the heavier\ngrid.onHeadersChanged which refreshes the view.\nThe onMove handler ensures that grid.inHeaderMove is called which just swaps cells.</p>\n"},"onMenuHide":{"!type":"fn(menu: ?) -> !this","!doc":"<p>remove the trigger open class when the menu is hidden</p>\n"},"onMove":{"!type":"fn(headerToMove: ?, fromIdx: ?, toIdx: ?) -> !this"},"onSortAscClick":{"!type":"fn() -> !this","!doc":"<p>sort asc when clicking on item in menu</p>\n"},"onSortDescClick":{"!type":"fn() -> !this","!doc":"<p>sort desc when clicking on item in menu</p>\n"},"purgeCache":{"!type":"fn() -> !this"},"purgeHeaderCtCache":{"!type":"fn(headerCt: ?) -> !this"},"showMenuBy":{"!type":"fn(t: ?, header: ?) -> !this","!doc":"<p>Shows the column menu under the target element passed. This method is used when the trigger element on the column\nheader is clicked on and rarely should be used otherwise.</p>\n"},"tempLock":{"!type":"fn() -> !this","!doc":"<p>Temporarily lock the headerCt. This makes it so that clicking on headers\ndon't trigger actions like sorting or opening of the header menu. This is\ndone because extraneous events may be fired on the headers after interacting\nwith a drag drop operation.</p>\n"},"updateMenuDisabledState":{"!type":"fn(menu: ?) -> !this"},"columnhide":{"!type":"fn(ct: ?, column: ?, eOpts: ?)"},"columnmove":{"!type":"fn(ct: ?, column: ?, fromIdx: ?, toIdx: ?, eOpts: ?)"},"columnresize":{"!type":"fn(ct: ?, column: ?, width: ?, eOpts: ?)"},"columnschanged":{"!type":"fn(ct: ?, eOpts: ?)","!doc":"<p>Fired after the columns change in any way, when a column has been hidden or shown, or when a column\nis added to or removed from this header container.</p>\n"},"columnshow":{"!type":"fn(ct: ?, column: ?, eOpts: ?)"},"headerclick":{"!type":"fn(ct: ?, column: ?, e: ?, t: ?, eOpts: ?)"},"headercontextmenu":{"!type":"fn(ct: ?, column: ?, e: ?, t: ?, eOpts: ?)"},"headertriggerclick":{"!type":"fn(ct: ?, column: ?, e: ?, t: ?, eOpts: ?)"},"menucreate":{"!type":"fn(ct: ?, menu: ?, eOpts: ?)","!doc":"<p>Fired immediately after the column header menu is created.</p>\n"},"sortchange":{"!type":"fn(ct: ?, column: ?, direction: ?, eOpts: ?)"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"detachOnRemove":{"!type":"bool","!doc":"<p>True to move any component to the detachedBody when the component is\nremoved from this container. This option is only applicable when the component is not destroyed while\nbeing removed, see <a href=\"#!/api/Ext.grid.header.Container-cfg-autoDestroy\" rel=\"Ext.grid.header.Container-cfg-autoDestroy\" class=\"docClass\">autoDestroy</a> and <a href=\"#!/api/Ext.grid.header.Container-method-remove\" rel=\"Ext.grid.header.Container-method-remove\" class=\"docClass\">remove</a>. If this option is set to false, the DOM\nof the component will remain in the current place until it is explicitly moved.</p>\n"},"sortable":{"!type":"bool","!doc":"<p>Provides the default sortable state for all Headers within this HeaderContainer.\nAlso turns on or off the menus in the HeaderContainer. Note that the menu is\nshared across every header and therefore turning it off will remove the menu\nitems for every header.</p>\n"},"onAdd":{"!type":"fn(c: ?) -> !this","!doc":"<p>Invalidate column cache on add\nWe cannot refresh the View on every add because this method is called\nwhen the HeaderDropZone moves Headers around, that will also refresh the view</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onRemove":{"!type":"fn(c: ?) -> !this","!doc":"<p>Invalidate column cache on remove\nWe cannot refresh the View on every remove because this method is called\nwhen the HeaderDropZone moves Headers around, that will also refresh the view</p>\n"},"setSortState":{"!type":"fn() -> !this","!doc":"<p>Private\nSynchronize column UI visible sort state with Store's sorters.</p>\n"}},"!doc":"<p>Container which holds headers and is docked at the top or bottom of a TablePanel.\nThe HeaderContainer drives resizing/moving/hiding of columns within the TableView.\nAs headers are hidden, moved or resized the headercontainer is responsible for\ntriggering changes within the view.</p>\n","!type":"fn(config: ?)"},"DragZone":{"!type":"fn(headerCt: ?)","prototype":{"colHeaderSelector":{"!type":"string"},"colInnerSelector":{"!type":"string"},"maxProxyWidth":{"!type":"number"},"afterRepair":{"!type":"fn() -> !this"},"disable":{"!type":"fn() -> !this"},"enable":{"!type":"fn() -> !this"},"getDDGroup":{"!type":"fn() -> !this"},"getDragData":{"!type":"fn(e: ?) -> ?","!doc":"<p>Called when a mousedown occurs in this container. Looks in <a href=\"#!/api/Ext.dd.Registry\" rel=\"Ext.dd.Registry\" class=\"docClass\">Ext.dd.Registry</a> for a valid target to drag\nbased on the mouse down. Override this method to provide your own lookup logic (e.g. finding a child by class\nname). Make sure your returned object has a \"ddel\" attribute (with an HTML Element) for other functions to work.</p>\n"},"getRepairXY":{"!type":"fn() -> ?","!doc":"<p>Called before a repair of an invalid drop to get the XY to animate to. By default returns the XY of\nthis.dragData.ddel</p>\n"},"onBeforeDrag":{"!type":"fn() -> bool","!doc":"<p>An empty function by default, but provided so that you can perform a custom action before the initial\ndrag event begins and optionally cancel it.</p>\n"},"onDragDrop":{"!type":"fn() -> !this","!doc":"<p>Abstract method called when this item is dropped on another DragDrop\nobj</p>\n"},"onInitDrag":{"!type":"fn() -> bool","!doc":"<p>Called once drag threshold has been reached to initialize the proxy element. By default, it clones the\nthis.dragData.ddel</p>\n"}}},"DropZone":{"!type":"fn(headerCt: ?)","prototype":{"colHeaderCls":{"!type":"string"},"proxyOffsets":{},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroy this DragDrop instance</p>\n"},"getBottomIndicator":{"!type":"fn() -> !this"},"getDDGroup":{"!type":"fn() -> !this"},"getLocation":{"!type":"fn(e: ?, t: ?) -> !this"},"getTargetFromEvent":{"!type":"fn(e: ?) -> ?","!doc":"<p>Returns a custom data object associated with the DOM node that is the target of the event.  By default\nthis looks up the event target in the <a href=\"#!/api/Ext.dd.Registry\" rel=\"Ext.dd.Registry\" class=\"docClass\">Ext.dd.Registry</a>, although you can override this method to\nprovide your own custom lookup.</p>\n"},"getTopIndicator":{"!type":"fn() -> !this"},"hideIndicators":{"!type":"fn() -> !this"},"invalidateDrop":{"!type":"fn() -> !this"},"onNodeDrop":{"!type":"fn(node: ?, dragZone: ?, e: ?, data: ?) -> bool","!doc":"<p>Called when the DropZone determines that a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> has been dropped onto\nthe drop node.  The default implementation returns false, so it should be overridden to provide the\nappropriate processing of the drop event and return true so that the drag source's repair action does not run.</p>\n"},"onNodeOut":{"!type":"fn() -> !this","!doc":"<p>Called when the DropZone determines that a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> has been dragged out of\nthe drop node without dropping.  This method has no default implementation and should be overridden to provide\nnode-specific processing if necessary.</p>\n"},"onNodeOver":{"!type":"fn(node: ?, dragZone: ?, e: ?, data: ?) -> string","!doc":"<p>Called while the DropZone determines that a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> is over a drop node\nthat has either been registered or detected by a configured implementation of <a href=\"#!/api/Ext.grid.header.DropZone-method-getTargetFromEvent\" rel=\"Ext.grid.header.DropZone-method-getTargetFromEvent\" class=\"docClass\">getTargetFromEvent</a>.\nThe default implementation returns this.dropAllowed, so it should be\noverridden to provide the proper feedback.</p>\n"},"positionIndicator":{"!type":"fn(data: ?, node: ?, e: ?) -> !this"}}}},"ColumnComponentLayout":{"!doc":"<p>Component layout for grid column headers which have a title element at the top followed by content.</p>\n","!type":"fn(config: ?)","prototype":{"setWidthInDom":{"!type":"bool","!doc":"<p>When publishing width of an auto Component, it is usually not written to the DOM.\nSetting this to <code>true</code> overrides this behaviour.</p>\n"},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to prepare for layout.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"calculateOwnerHeightFromContentHeight":{"!type":"fn(ownerContext: ?, contentHeight: ?) -> !this","!doc":"<p>Push content height outwards when we are shrinkwrapping</p>\n"},"calculateOwnerWidthFromContentWidth":{"!type":"fn(ownerContext: ?, contentWidth: ?) -> !this","!doc":"<p>Push content width outwards when we are shrinkwrapping</p>\n"},"getTriggerOffset":{"!type":"fn(owner: ?, ownerContext: ?) -> !this"},"measureContentHeight":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>We do not need the Direct2D sub pixel measurement here. Just the offsetHeight will do.\nTODO: When https://sencha.jira.com/browse/EXTJSIV-7734 is fixed to not do subpixel adjustment on height,\nremove this override.</p>\n"},"publishInnerHeight":{"!type":"fn(ownerContext: ?, outerHeight: ?) -> !this","!doc":"<p>If not shrink wrapping, push height info down into child items</p>\n"},"publishInnerWidth":{"!type":"fn(ownerContext: ?, outerWidth: ?) -> !this","!doc":"<p>If not shrink wrapping, push width info down into child items</p>\n"}}},"ColumnLayout":{"!doc":"<p>This class is used only by the grid's HeaderContainer docked child.</p>\n\n<p>It adds the ability to shrink the vertical size of the inner container element back if a grouped\ncolumn header has all its child columns dragged out, and the whole HeaderContainer needs to shrink back down.</p>\n\n<p>Also, after every layout, after all headers have attained their 'stretchmax' height, it goes through and calls\n<code>setPadding</code> on the columns so that they lay out correctly.</p>\n","!type":"fn(config: ?)","prototype":{"firstHeaderCls":{"!type":"string"},"lastHeaderCls":{"!type":"string"},"reserveOffset":{"!type":"bool","!doc":"<p>whether or not to reserve the availableSpaceOffset in layout calculations</p>\n"},"type":{"!type":"string"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Collect the height of the table of data upon layout begin</p>\n"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.grid.ColumnLayout-property-done\" rel=\"Ext.grid.ColumnLayout-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.grid.ColumnLayout-property-done\" rel=\"Ext.grid.ColumnLayout-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"},"completeLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method (if implemented) is called at the end of the cycle in which this layout\ncompletes (by not setting <a href=\"#!/api/Ext.grid.ColumnLayout-property-done\" rel=\"Ext.grid.ColumnLayout-property-done\" class=\"docClass\">done</a> to <code>false</code> in <a href=\"#!/api/Ext.grid.ColumnLayout-method-calculate\" rel=\"Ext.grid.ColumnLayout-method-calculate\" class=\"docClass\">calculate</a>). It is\npossible for the layout to complete and yet become invalid before the end of the cycle,\nin which case, this method will not be called. It is also possible for this method to\nbe called and then later the layout becomes invalidated. This will result in\n<a href=\"#!/api/Ext.grid.ColumnLayout-method-calculate\" rel=\"Ext.grid.ColumnLayout-method-calculate\" class=\"docClass\">calculate</a> being called again, followed by another call to this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\nbe flushed at the next opportunity.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"convertWidthsToFlexes":{"!type":"fn(ownerContext: ?) -> !this"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method is called after all layouts are complete and their calculations flushed\nto the DOM. No further layouts will be run and this method is only called once per\nlayout run. The base component layout caches <code>lastComponentSize</code>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"getColumnContainerSize":{"!type":"fn(ownerContext: ?) -> !this"},"getContainerSize":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Local getContainerSize implementation accounts for vertical scrollbar in the view.</p>\n"},"initLayout":{"!type":"fn() -> !this","!doc":"<p>A one-time initialization method called just before rendering.</p>\n"},"injectViewContext":{"!type":"fn(ownerContext: ?, view: ?) -> !this","!doc":"<p>TableView performs its non-deferred initial refresh during its render layout run.\nBecause the ColumnLayout has already been through its beginLayout phase at that point,\na viewContext will not have been captured. This method allows the TableView\nto inject a ViewContext at the time of initial refresh.</p>\n"},"publishInnerCtSize":{"!type":"fn(ownerContext: ?) -> !this"},"roundFlex":{"!type":"fn(width: ?) -> !this"}}},"ColumnManager":{"!doc":"<p>Manages and provides information about a TablePanel's <em>visible leaf</em> columns.</p>\n","!type":"fn(visibleOnly: ?, headerCt: ?, secondHeaderCt: ?)","prototype":{"columns":{},"cacheColumns":{"!type":"fn() -> !this"},"destroy":{"!type":"fn() -> !this"},"getColumns":{"!type":"fn() -> !this"},"getFirst":{"!type":"fn() -> ?","!doc":"<p>Get the first column.</p>\n"},"getHeaderAtIndex":{"!type":"fn(index: ?) -> ?","!doc":"<p>If called from a root header, gets a leaf level header by index regardless of what the nesting\nstructure is.</p>\n\n<p>If called from a group header, returns the index of a leaf level header relative to the group header.</p>\n"},"getHeaderByDataIndex":{"!type":"fn(dataIndex: ?) -> ?","!doc":"<p>Get a leaf level header by data index regardless of what the nesting\nstructure is.</p>\n"},"getHeaderById":{"!type":"fn(id: ?) -> ?","!doc":"<p>Get a leaf level header by index regardless of what the nesting\nstructure is.</p>\n"},"getHeaderColumns":{"!type":"fn(header: ?) -> !this"},"getHeaderIndex":{"!type":"fn(header: ?) -> number","!doc":"<p>If called from a root header, returns the index of a leaf level header regardless of what the nesting\nstructure is.</p>\n\n<p>If called from a group header, returns the index of a leaf level header relative to the group header.</p>\n\n<p>If a group header is passed, the index of the first leaf level header within it is returned.</p>\n"},"getLast":{"!type":"fn() -> ?","!doc":"<p>Get the last column.</p>\n"},"getNextSibling":{"!type":"fn(header: ?) -> !this"},"getPreviousSibling":{"!type":"fn(header: ?) -> !this"},"getVisibleHeaderClosestToIndex":{"!type":"fn(index: ?) -> !this","!doc":"<p>When passed a column index, returns the closet <em>visible</em> column to that. If the column at the passed index is visible,\nthat is returned. If it is hidden, either the next visible, or the previous visible column is returned.</p>\n\n<p>If called from a group header, returns the visible index of a leaf level header relative to the group header with the\nsame stipulations as outlined above.</p>\n"},"hasVariableRowHeight":{"!type":"fn() -> !this"},"invalidate":{"!type":"fn() -> !this"}}},"feature":{"AbstractSummary":{"!doc":"<p>A small abstract class that contains the shared behaviour for any summary\ncalculations to be used in the grid.</p>\n","!type":"fn(config: ?)","prototype":{"remoteRoot":{"!type":"string","!doc":"<p>The name of the property which contains the Array of summary objects.\nIt allows to use server-side calculated summaries.</p>\n"},"showSummaryRow":{"!type":"bool","!doc":"<p>True to show the summary row.</p>\n"},"readDataOptions":{},"summaryRowCls":{"!type":"string"},"summaryRowSelector":{"!type":"string"},"summaryRowTpl":{"!doc":"<p>High priority rowTpl interceptor which sees summary rows early, and renders them correctly and then aborts the row rendering chain.\nThis will only see action when summary rows are being updated and Table.onUpdate->Table.bufferRender renders the individual updated sumary row.</p>\n"},"createRenderer":{"!type":"fn(column: ?, record: ?) -> !this"},"generateSummaryData":{"!type":"fn() -> ?","!doc":"<p>Used by the Grouping Feature when <a href=\"#!/api/Ext.grid.feature.AbstractSummary-cfg-showSummaryRow\" rel=\"Ext.grid.feature.AbstractSummary-cfg-showSummaryRow\" class=\"docClass\">showSummaryRow</a> is <code>true</code>.</p>\n\n<p>Generates group summary data for the whole store.</p>\n"},"getAggregateRecord":{"!type":"fn(group: ?, forceNew: ?) -> !this"},"getGroupInfo":{"!type":"fn(group: ?) -> !this"},"getSummary":{"!type":"fn(store: ?, type: ?, field: ?, group: ?) -> ?","!doc":"<p>Get the summary data for a field.</p>\n"},"init":{"!type":"fn() -> !this","!doc":"<p>Listen for store updates. Eg, from an Editor.</p>\n"},"outputSummaryRecord":{"!type":"fn(summaryRecord: ?, contextValues: ?, out: ?) -> !this"},"populateRecord":{"!type":"fn(group: ?, groupInfo: ?, remoteData: ?) -> !this"},"setSummaryData":{"!type":"fn(record: ?, colId: ?, summaryValue: ?, groupName: ?) -> !this"},"toggleSummaryRow":{"!type":"fn(visible: ?) -> !this","!doc":"<p>Toggle whether or not to show the summary row.</p>\n"}}},"Feature":{"prototype":{"disabled":{"!type":"bool","!doc":"<p>True when feature is disabled.</p>\n"},"eventPrefix":{"!type":"string","!doc":"<p>Prefix to use when firing events on the view.\nFor example a prefix of group would expose \"groupclick\", \"groupcontextmenu\", \"groupdblclick\".</p>\n"},"eventSelector":{"!type":"string","!doc":"<p>Selector used to determine when to fire the event with the eventPrefix.</p>\n"},"grid":{"!doc":"<p>Reference to the grid panel</p>\n"},"hasFeatureEvent":{"!type":"bool","!doc":"<p>Most features will expose additional events, some may not and will\nneed to change this to false.</p>\n"},"isFeature":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Feature, or subclass thereof.</p>\n"},"view":{"!doc":"<p>Reference to the TableView.</p>\n"},"wrapsItem":{"!type":"bool"},"clone":{"!type":"fn() -> !this"},"destroy":{"!type":"fn() -> !this"},"disable":{"!type":"fn() -> !this","!doc":"<p>Disables the feature.</p>\n"},"enable":{"!type":"fn() -> !this","!doc":"<p>Enables the feature.</p>\n"},"getFireEventArgs":{"!type":"fn(eventName: ?, view: ?, featureTarget: ?, e: ?) -> !this","!doc":"<p>Abstract method to be overriden when a feature should add additional\narguments to its event signature. By default the event will fire:</p>\n\n<ul>\n<li>view - The underlying <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">Ext.view.Table</a></li>\n<li>featureTarget - The matched element by the defined <a href=\"#!/api/Ext.grid.feature.Feature-property-eventSelector\" rel=\"Ext.grid.feature.Feature-property-eventSelector\" class=\"docClass\">eventSelector</a></li>\n</ul>\n\n\n<p>The method must also return the eventName as the first index of the array\nto be passed to fireEvent.</p>\n"},"vetoEvent":{"!type":"fn() -> !this"},"init":{"!type":"fn() -> !this"}},"!doc":"<p>A feature is a type of plugin that is specific to the <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>. It provides several\nhooks that allows the developer to inject additional functionality at certain points throughout the\ngrid creation cycle. This class provides the base template methods that are available to the developer,\nit should be extended.</p>\n\n<p>There are several built in features that extend this class, for example:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.grid.feature.Grouping\" rel=\"Ext.grid.feature.Grouping\" class=\"docClass\">Ext.grid.feature.Grouping</a> - Shows grid rows in groups as specified by the Ext.data.Store</li>\n<li><a href=\"#!/api/Ext.grid.feature.RowBody\" rel=\"Ext.grid.feature.RowBody\" class=\"docClass\">Ext.grid.feature.RowBody</a> - Adds a body section for each grid row that can contain markup.</li>\n<li><a href=\"#!/api/Ext.grid.feature.Summary\" rel=\"Ext.grid.feature.Summary\" class=\"docClass\">Ext.grid.feature.Summary</a> - Adds a summary row at the bottom of the grid with aggregate totals for a column.</li>\n</ul>\n\n\n<h2>Using Features</h2>\n\n<p>A feature is added to the grid by specifying it an array of features in the configuration:</p>\n\n<pre><code>var groupingFeature = Ext.create('<a href=\"#!/api/Ext.grid.feature.Grouping\" rel=\"Ext.grid.feature.Grouping\" class=\"docClass\">Ext.grid.feature.Grouping</a>');\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    // other options\n    features: [groupingFeature]\n});\n</code></pre>\n\n<h2>Writing Features</h2>\n\n<p>A Feature may add new DOM structure within the structure of a grid.</p>\n\n<p>A grid is essentially a <code>&lt;table&gt;</code> element. A <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">TableView</a> instance uses four XTemplates\nto render the grid, <code>tpl</code>, <code>itemTpl</code>, <code>rowTpl</code>, <code>cellTpl</code>.</p>\n\n<p>A <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">TableView</a> uses its <code>tpl</code> to emit the item encapsulating HTML tags into its output stream.\nTo render the rows, it invokes <a href=\"#!/api/Ext.view.Table-method-renderRows\" rel=\"Ext.view.Table-method-renderRows\" class=\"docClass\">Ext.view.Table.renderRows</a> passing the <code>rows</code> member of its data object and the <code>columns</code> member of its data object.</p>\n\n<p>The <code>tpl</code>'s data object Looks like this:\n    {\n        view: owningTableView,\n        rows: recordsToRender,\n        viewStartIndex: indexOfFirstRecordInStore,\n        tableStyle: styleString\n    }</p>\n\n<ul>\n<li>A <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">TableView</a> uses its <code>rowTpl</code> to emit a <code>&lt;tr&gt;</code> HTML tag to its output stream. To render cells,\nit invokes <a href=\"#!/api/Ext.view.Table-method-renderCell\" rel=\"Ext.view.Table-method-renderCell\" class=\"docClass\">Ext.view.Table.renderCell</a> passing the <code>rows</code> member of its data object.</li>\n</ul>\n\n\n<p>The <code>itemTpl</code> and <code>rowTpl</code>'s data object looks like this:</p>\n\n<pre><code>{\n    view:        owningTableView,\n    record:      recordToRender,\n    recordIndex: indexOfRecordInStore,\n    rowIndex:    indexOfRowInView,\n    columns:     arrayOfColumnDefinitions,\n    itemClasses: arrayOfClassNames, // For outermost row in case of wrapping\n    rowClasses:  arrayOfClassNames,  // For internal data bearing row in case of wrapping\n    rowStyle:    styleString\n}\n</code></pre>\n\n<ul>\n<li>A <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">TableView</a> uses its <code>cellTpl</code> to emit a <code>&lt;td&gt;</code> HTML tag to its output stream.</li>\n</ul>\n\n\n<p>The <code>cellTpl's</code> data object looks like this:</p>\n\n<pre><code>{\n    record: recordToRender\n    column: columnToRender;\n    recordIndex: indexOfRecordInStore,\n    rowIndex:    indexOfRowInView,\n    columnIndex: columnIndex,\n    align: columnAlign,\n    tdCls: classForCell\n}\n</code></pre>\n\n<p>A Feature may inject its own tpl or rowTpl or cellTpl into the <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">TableView</a>'s rendering by\ncalling <a href=\"#!/api/Ext.view.Table-method-addTpl\" rel=\"Ext.view.Table-method-addTpl\" class=\"docClass\">Ext.view.Table.addTpl</a> or <a href=\"#!/api/Ext.view.Table-method-addRowTpl\" rel=\"Ext.view.Table-method-addRowTpl\" class=\"docClass\">Ext.view.Table.addRowTpl</a> or <a href=\"#!/api/Ext.view.Table-method-addCellTpl\" rel=\"Ext.view.Table-method-addCellTpl\" class=\"docClass\">Ext.view.Table.addCellTpl</a>.</p>\n\n<p>The passed XTemplate is added <em>upstream</em> of the default template for the table element in a link list of XTemplates which contribute\nto the element's HTML. It may emit appropriate HTML strings into the output stream <em>around</em> a call to</p>\n\n<pre><code>this.nextTpl.apply(values, out, parent);\n</code></pre>\n\n<p>This passes the current value context, output stream and the parent value context to the next XTemplate in the list.</p>\n","!type":"fn(config: ?)"},"Grouping":{"!doc":"<p>This feature allows to display the grid rows aggregated into groups as specified by the grouper\nspecified on the Store. The group will show the title for the group name and then the appropriate records for the group\nunderneath. The groups can also be expanded and collapsed.</p>\n\n<h2>Extra Events</h2>\n\n<p>This feature adds several extra events that will be fired on the grid to interact with the groups:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.grid.feature.Grouping-event-groupclick\" rel=\"Ext.grid.feature.Grouping-event-groupclick\" class=\"docClass\">groupclick</a></li>\n<li><a href=\"#!/api/Ext.grid.feature.Grouping-event-groupdblclick\" rel=\"Ext.grid.feature.Grouping-event-groupdblclick\" class=\"docClass\">groupdblclick</a></li>\n<li><a href=\"#!/api/Ext.grid.feature.Grouping-event-groupcontextmenu\" rel=\"Ext.grid.feature.Grouping-event-groupcontextmenu\" class=\"docClass\">groupcontextmenu</a></li>\n<li><a href=\"#!/api/Ext.grid.feature.Grouping-event-groupexpand\" rel=\"Ext.grid.feature.Grouping-event-groupexpand\" class=\"docClass\">groupexpand</a></li>\n<li><a href=\"#!/api/Ext.grid.feature.Grouping-event-groupcollapse\" rel=\"Ext.grid.feature.Grouping-event-groupcollapse\" class=\"docClass\">groupcollapse</a></li>\n</ul>\n\n\n<h2>Menu Augmentation</h2>\n\n<p>This feature adds extra options to the grid column menu to provide the user with functionality to modify the grouping.\nThis can be disabled by setting the <a href=\"#!/api/Ext.grid.feature.Grouping-cfg-enableGroupingMenu\" rel=\"Ext.grid.feature.Grouping-cfg-enableGroupingMenu\" class=\"docClass\">enableGroupingMenu</a> option. The option to disallow grouping from being turned off\nby the user is <a href=\"#!/api/Ext.grid.feature.Grouping-cfg-enableNoGroups\" rel=\"Ext.grid.feature.Grouping-cfg-enableNoGroups\" class=\"docClass\">enableNoGroups</a>.</p>\n\n<h2>Controlling Group Text</h2>\n\n<p>The <a href=\"#!/api/Ext.grid.feature.Grouping-cfg-groupHeaderTpl\" rel=\"Ext.grid.feature.Grouping-cfg-groupHeaderTpl\" class=\"docClass\">groupHeaderTpl</a> is used to control the rendered title for each group. It can modified to customized\nthe default display.</p>\n\n<h2>Example Usage</h2>\n\n<pre class='inline-example '><code>var store = Ext.create('Ext.data.Store', {\n    storeId:'employeeStore',\n    fields:['name', 'seniority', 'department'],\n    groupField: 'department',\n    data: {'employees':[\n        { \"name\": \"Michael Scott\",  \"seniority\": 7, \"department\": \"Management\" },\n        { \"name\": \"Dwight Schrute\", \"seniority\": 2, \"department\": \"Sales\" },\n        { \"name\": \"Jim Halpert\",    \"seniority\": 3, \"department\": \"Sales\" },\n        { \"name\": \"Kevin Malone\",   \"seniority\": 4, \"department\": \"Accounting\" },\n        { \"name\": \"Angela Martin\",  \"seniority\": 5, \"department\": \"Accounting\" }\n    ]},\n    proxy: {\n        type: 'memory',\n        reader: {\n            type: 'json',\n            rootProperty: 'employees'\n        }\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Employees',\n    store: Ext.data.StoreManager.lookup('employeeStore'),\n    columns: [\n        { text: 'Name',     dataIndex: 'name' },\n        { text: 'Seniority', dataIndex: 'seniority' }\n    ],\n    features: [{ftype:'grouping'}],\n    width: 200,\n    height: 275,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p><strong>Note:</strong> To use grouping with a grid that has <a href=\"#!/api/Ext.grid.column.Column-cfg-locked\" rel=\"Ext.grid.column.Column-cfg-locked\" class=\"docClass\">locked columns</a>, you need to supply\nthe grouping feature as a config object - so the grid can create two instances of the grouping feature.</p>\n","!type":"fn()","prototype":{"collapsible":{"!type":"bool","!doc":"<p>Set to <code>false</code> to disable collapsing groups from the UI.</p>\n\n<p>This is set to <code>false</code> when the associated store is\na BufferedStore.</p>\n"},"depthToIndent":{"!type":"number","!doc":"<p>Number of pixels to indent per grouping level</p>\n"},"enableGroupingMenu":{"!type":"bool","!doc":"<p>True to enable the grouping control in the header menu.</p>\n"},"enableNoGroups":{"!type":"bool","!doc":"<p>True to allow the user to turn off grouping.</p>\n"},"groupByText":{"!type":"string","!doc":"<p>Text displayed in the grid header menu for grouping by header.</p>\n"},"groupHeaderTpl":{"!doc":"<p>A string Template snippet, an array of strings (optionally followed by an object containing Template methods) to be used to construct a Template, or a Template instance.</p>\n\n<ul>\n<li><p>Example 1 (Template snippet):</p>\n\n<pre><code>groupHeaderTpl: 'Group: {name}'\n</code></pre></li>\n<li><p>Example 2 (Array):</p>\n\n<pre><code>groupHeaderTpl: [\n    'Group: ',\n    '&lt;div&gt;{name:this.formatName}&lt;/div&gt;',\n    {\n        formatName: function(name) {\n            return Ext.String.trim(name);\n        }\n    }\n]\n</code></pre></li>\n<li><p>Example 3 (Template Instance):</p>\n\n<pre><code>groupHeaderTpl: Ext.create('Ext.XTemplate',\n    'Group: ',\n    '&lt;div&gt;{name:this.formatName}&lt;/div&gt;',\n    {\n        formatName: function(name) {\n            return Ext.String.trim(name);\n        }\n    }\n)\n</code></pre></li>\n</ul>\n\n"},"hideGroupedHeader":{"!type":"bool","!doc":"<p>True to hide the header that is currently grouped.</p>\n"},"showGroupsText":{"!type":"string","!doc":"<p>Text displayed in the grid header for enabling/disabling grouping.</p>\n"},"showSummaryRow":{"!type":"bool","!doc":"<p></p>\n"},"startCollapsed":{"!type":"bool","!doc":"<p>True to start all groups collapsed.</p>\n"},"collapseTip":{"!type":"string","!doc":"<p><locale></locale></p>\n"},"collapsedCls":{"!type":"string"},"collapsibleCls":{"!type":"string"},"ctCls":{"!type":"string"},"eventPrefix":{"!type":"string","!doc":"<p>Prefix to use when firing events on the view.\nFor example a prefix of group would expose \"groupclick\", \"groupcontextmenu\", \"groupdblclick\".</p>\n"},"eventSelector":{"!type":"string","!doc":"<p>Selector used to determine when to fire the event with the eventPrefix.</p>\n"},"expandTip":{"!type":"string","!doc":"<p><locale></locale></p>\n"},"groupCls":{"!type":"string"},"groupInfo":{},"groupRowTpl":{},"hdCollapsedCls":{"!type":"string"},"hdNotCollapsibleCls":{"!type":"string"},"outerTpl":{},"refreshData":{},"wrapsItem":{"!type":"bool"},"afterCollapseExpand":{"!type":"fn(collapsed: ?, groupName: ?, focus: ?) -> !this"},"afterViewRender":{"!type":"fn() -> !this","!doc":"<p>Attach events to view</p>\n"},"block":{"!type":"fn(fromPartner: ?) -> !this"},"cleanup":{"!type":"fn(rows: ?, rowValues: ?) -> !this"},"clearGroupCache":{"!type":"fn() -> !this"},"collapse":{"!type":"fn(groupName: ?, focus: ?) -> !this","!doc":"<p>Collapse a group</p>\n"},"collapseAll":{"!type":"fn() -> !this","!doc":"<p>Collapse all groups</p>\n"},"createGroupCls":{"!type":"fn(group: ?) -> !this"},"createGroupId":{"!type":"fn(group: ?) -> !this"},"destroy":{"!type":"fn() -> !this"},"disable":{"!type":"fn() -> !this","!doc":"<p>Disables the feature.</p>\n"},"doCollapseExpand":{"!type":"fn(collapsed: ?, groupName: ?, focus: ?) -> !this"},"enable":{"!type":"fn() -> !this","!doc":"<p>Enables the feature.</p>\n"},"expand":{"!type":"fn(groupName: ?, focus: ?) -> !this","!doc":"<p>Expand a group</p>\n"},"expandAll":{"!type":"fn() -> !this","!doc":"<p>Expand all groups</p>\n"},"getFireEventArgs":{"!type":"fn(type: ?, view: ?, targetEl: ?, e: ?) -> !this","!doc":"<p>Abstract method to be overriden when a feature should add additional\narguments to its event signature. By default the event will fire:</p>\n\n<ul>\n<li>view - The underlying <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">Ext.view.Table</a></li>\n<li>featureTarget - The matched element by the defined <a href=\"#!/api/Ext.grid.feature.Grouping-property-eventSelector\" rel=\"Ext.grid.feature.Grouping-property-eventSelector\" class=\"docClass\">eventSelector</a></li>\n</ul>\n\n\n<p>The method must also return the eventName as the first index of the array\nto be passed to fireEvent.</p>\n"},"getGroup":{"!type":"fn(name: ?) -> !this"},"getGroupField":{"!type":"fn() -> !this"},"getGroupName":{"!type":"fn(element: ?) -> !this"},"getGroupedHeader":{"!type":"fn(groupField: ?) -> !this"},"getHeaderNode":{"!type":"fn(groupName: ?) -> !this"},"getMenuItem":{"!type":"fn(dataIndex: ?) -> ?","!doc":"<p>Gets the related menu item for a dataIndex</p>\n"},"getMenuItems":{"!type":"fn() -> !this"},"getRecordGroup":{"!type":"fn(record: ?) -> ?","!doc":"<p>Returns the group data object for the group to which the passed record belongs <strong>if the Store is grouped</strong>.</p>\n"},"indexOf":{"!type":"fn(record: ?) -> !this"},"init":{"!type":"fn(grid: ?) -> !this","!doc":"<p>Listen for store updates. Eg, from an Editor.</p>\n"},"injectGroupingMenu":{"!type":"fn() -> !this"},"isAllCollapsed":{"!type":"fn() -> !this","!doc":"<p>private\nReturns true if all groups are collapsed</p>\n"},"isAllExpanded":{"!type":"fn() -> !this","!doc":"<p>private\nReturns true if all groups are expanded</p>\n"},"isExpanded":{"!type":"fn(groupName: ?) -> bool","!doc":"<p>Returns <code>true</code> if the named group is expanded.</p>\n"},"isInCollapsedGroup":{"!type":"fn(record: ?) -> !this"},"onColumnHideShow":{"!type":"fn(headerOwnerCt: ?, header: ?) -> !this"},"onColumnMove":{"!type":"fn() -> !this","!doc":"<p>Update first and last records in groups when column moves\nBecause of the RowWrap template, this will update the groups' headers and footers</p>\n"},"onGroupChange":{"!type":"fn(store: ?, grouper: ?) -> !this"},"onGroupClick":{"!type":"fn(view: ?, rowElement: ?, groupName: ?, e: ?) -> !this","!doc":"<p>Toggle between expanded/collapsed state when clicking on\nthe group.</p>\n"},"onGroupKey":{"!type":"fn(keyCode: ?, event: ?) -> !this"},"onGroupMenuItemClick":{"!type":"fn(menuItem: ?, e: ?) -> !this","!doc":"<p>Group by the header the user has clicked on.</p>\n"},"onGroupToggleMenuItemClick":{"!type":"fn(menuItem: ?, checked: ?) -> !this","!doc":"<p>Turn on and off grouping via the menu</p>\n"},"onReconfigure":{"!type":"fn(grid: ?, store: ?, columns: ?, oldStore: ?, oldColumns: ?) -> !this"},"pruneGroupedHeader":{"!type":"fn() -> !this","!doc":"<p>Prunes the grouped header from the header container</p>\n"},"setup":{"!type":"fn(rows: ?, rowValues: ?) -> !this"},"setupRowData":{"!type":"fn(record: ?, idx: ?, rowValues: ?) -> !this"},"showMenuBy":{"!type":"fn(t: ?, header: ?) -> !this"},"unblock":{"!type":"fn(fromPartner: ?) -> !this"},"vetoEvent":{"!type":"fn(record: ?, row: ?, rowIndex: ?, e: ?) -> !this"},"groupclick":{"!type":"fn(view: ?, node: ?, group: ?, e: ?, eOpts: ?)"},"groupcollapse":{"!type":"fn(view: ?, node: ?, group: ?, eOpts: ?)"},"groupcontextmenu":{"!type":"fn(view: ?, node: ?, group: ?, e: ?, eOpts: ?)"},"groupdblclick":{"!type":"fn(view: ?, node: ?, group: ?, e: ?, eOpts: ?)"},"groupexpand":{"!type":"fn(view: ?, node: ?, group: ?, eOpts: ?)"}}},"GroupingSummary":{"!doc":"<p>This feature adds an aggregate summary row at the bottom of each group that is provided\nby the <a href=\"#!/api/Ext.grid.feature.Grouping\" rel=\"Ext.grid.feature.Grouping\" class=\"docClass\">Ext.grid.feature.Grouping</a> feature. There are two aspects to the summary:</p>\n\n<h2>Calculation</h2>\n\n<p>The summary value needs to be calculated for each column in the grid. This is controlled\nby the summaryType option specified on the column. There are several built in summary types,\nwhich can be specified as a string on the column configuration. These call underlying methods\non the store:</p>\n\n<ul>\n<li>count</li>\n<li>sum</li>\n<li>min</li>\n<li>max</li>\n<li>average</li>\n</ul>\n\n\n<p>Alternatively, the summaryType can be a function definition. If this is the case,\nthe function is called with two parameters: an array of records, and an array of field values\nto calculate the summary value.</p>\n\n<h2>Rendering</h2>\n\n<p>Similar to a column, the summary also supports a summaryRenderer function. This\nsummaryRenderer is called before displaying a value. The function is optional, if\nnot specified the default calculated value is shown. The summaryRenderer is called with:</p>\n\n<ul>\n<li>value {Object} - The calculated value.</li>\n<li>summaryData {Object} - Contains all raw summary values for the row.</li>\n<li>field {String} - The name of the field we are calculating</li>\n</ul>\n\n\n<h2>Example Usage</h2>\n\n<pre class='inline-example '><code>Ext.define('TestResult', {\n    extend: 'Ext.data.Model',\n    fields: ['student', 'subject', {\n        name: 'mark',\n        type: 'int'\n    }]\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    width: 200,\n    height: 240,\n    renderTo: document.body,\n    features: [{\n        groupHeaderTpl: 'Subject: {name}',\n        ftype: 'groupingsummary'\n    }],\n    store: {\n        model: 'TestResult',\n        groupField: 'subject',\n        data: [{\n            student: 'Student 1',\n            subject: 'Math',\n            mark: 84\n        },{\n            student: 'Student 1',\n            subject: 'Science',\n            mark: 72\n        },{\n            student: 'Student 2',\n            subject: 'Math',\n            mark: 96\n        },{\n            student: 'Student 2',\n            subject: 'Science',\n            mark: 68\n        }]\n    },\n    columns: [{\n        dataIndex: 'student',\n        text: 'Name',\n        summaryType: 'count',\n        summaryRenderer: function(value){\n            return Ext.String.format('{0} student{1}', value, value !== 1 ? 's' : '');\n        }\n    }, {\n        dataIndex: 'mark',\n        text: 'Mark',\n        summaryType: 'average'\n    }]\n});\n</code></pre>\n","!type":"fn()","prototype":{"showSummaryRow":{"!type":"bool","!doc":"<p>True to show the summary row.</p>\n"},"vetoEvent":{"!type":"fn(record: ?, row: ?, rowIndex: ?, e: ?) -> !this"}}},"GroupStore":{"!doc":"<p>Private record store class which takes the place of the view's data store to provide a grouped\nview of the data when the Grouping feature is used.</p>\n\n<p>Relays granular mutation events from the underlying store as refresh events to the view.</p>\n\n<p>On mutation events from the underlying store, updates the summary rows by firing update events on the corresponding\nsummary records.</p>\n","!type":"fn(groupingFeature: ?, store: ?)","prototype":{"defaultViewSize":{"!type":"number","!doc":"<p>Number of records to load into a buffered grid before it has been bound to a view of known size</p>\n"},"isFeatureStore":{"!type":"bool","!doc":"<p>Use this property moving forward for all feature stores. It will be used to ensure\nthat the correct object is used to call various APIs. See EXTJSIV-10022.</p>\n"},"isStore":{"!type":"bool"},"bindStore":{"!type":"fn(store: ?) -> !this"},"collapseGroup":{"!type":"fn(group: ?) -> !this"},"expandGroup":{"!type":"fn(group: ?) -> !this"},"getAt":{"!type":"fn(index: ?) -> !this"},"getById":{"!type":"fn(id: ?) -> !this"},"getByInternalId":{"!type":"fn(internalId: ?) -> !this"},"getCount":{"!type":"fn() -> !this"},"getGroupPlaceholder":{"!type":"fn(group: ?) -> !this"},"getRange":{"!type":"fn(start: ?, end: ?, options: ?) -> !this"},"getTotalCount":{"!type":"fn() -> !this"},"indexOf":{"!type":"fn(record: ?) -> !this","!doc":"<p>Find index of record in group store.\nIf it's in a collapsed group, then it's -1, not present</p>\n"},"indexOfTotal":{"!type":"fn(record: ?) -> number","!doc":"<p>Get the index within the entire dataset. From 0 to the totalCount.</p>\n\n<p>Like <a href=\"#!/api/Ext.grid.feature.GroupStore-method-indexOf\" rel=\"Ext.grid.feature.GroupStore-method-indexOf\" class=\"docClass\">indexOf</a>, this method is effected by filtering.</p>\n"},"isCollapsed":{"!type":"fn(name: ?) -> !this"},"isInCollapsedGroup":{"!type":"fn(record: ?) -> !this"},"onAdd":{"!type":"fn(store: ?, records: ?, startIndex: ?) -> !this"},"onClear":{"!type":"fn(store: ?, records: ?, startIndex: ?) -> !this"},"onGroupChange":{"!type":"fn(store: ?) -> !this","!doc":"<p>Relay the groupchange event</p>\n"},"onRefresh":{"!type":"fn(store: ?) -> !this"},"onRemove":{"!type":"fn(store: ?, records: ?, index: ?, isMove: ?) -> !this"},"onUpdate":{"!type":"fn(store: ?, record: ?, operation: ?, modifiedFieldNames: ?) -> !this"},"processStore":{"!type":"fn(store: ?) -> !this"},"rangeCached":{"!type":"fn(start: ?, end: ?) -> !this","!doc":"<p>This class is only created for fully loaded, non-buffered stores</p>\n"}}},"RowBody":{"!doc":"<p>The rowbody feature enhances the grid's markup to have an additional\ntr -> td -> div which spans the entire width of the original row.</p>\n\n<p>This is useful to to associate additional information with a particular\nrecord in a grid.</p>\n\n<p>Rowbodies are initially hidden unless you override setupRowData.</p>\n\n<p>Will expose additional events on the gridview with the prefix of 'rowbody'.\nFor example: 'rowbodyclick', 'rowbodydblclick', 'rowbodycontextmenu'.</p>\n\n<h1>Example</h1>\n\n<pre class='inline-example '><code>Ext.define('Animal', {\n    extend: 'Ext.data.Model',\n    fields: ['name', 'latin', 'desc']\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    width: 400,\n    height: 300,\n    renderTo: Ext.getBody(),\n    store: {\n        model: 'Animal',\n        data: [\n            {name: 'Tiger', latin: 'Panthera tigris',\n             desc: 'The largest cat species, weighing up to 306 kg (670 lb).'},\n            {name: 'Roman snail', latin: 'Helix pomatia',\n             desc: 'A species of large, edible, air-breathing land snail.'},\n            {name: 'Yellow-winged darter', latin: 'Sympetrum flaveolum',\n             desc: 'A dragonfly found in Europe and mid and Northern China.'},\n            {name: 'Superb Fairy-wren', latin: 'Malurus cyaneus',\n             desc: 'Common and familiar across south-eastern Australia.'}\n        ]\n    },\n    columns: [{\n        dataIndex: 'name',\n        text: 'Common name',\n        width: 125\n    }, {\n        dataIndex: 'latin',\n        text: 'Scientific name',\n        flex: 1\n    }],\n    features: [{\n        ftype: 'rowbody',\n        setupRowData: function(record, rowIndex, rowValues) {\n            var headerCt = this.view.headerCt,\n                colspan = headerCt.getColumnCount();\n\n            // Usually you would style the my-body-class in CSS file\n            Ext.apply(rowValues, {\n                rowBody: '&lt;div style=\"padding: 1em\"&gt;'+record.get(\"desc\")+'&lt;/div&gt;',\n                rowBodyCls: \"my-body-class\",\n                rowBodyColspan: colspan\n            });\n        }\n    }]\n});\n</code></pre>\n\n<p> # Cell Editing and Cell Selection Model</p>\n\n<p>Note that if <a href=\"#!/api/Ext.grid.plugin.CellEditing\" rel=\"Ext.grid.plugin.CellEditing\" class=\"docClass\">cell editing</a> or the <a href=\"#!/api/Ext.selection.CellModel\" rel=\"Ext.selection.CellModel\" class=\"docClass\">cell selection model</a> are going\nto be used, then the RowWrap feature, or <a href=\"#!/api/Ext.grid.plugin.RowExpander\" rel=\"Ext.grid.plugin.RowExpander\" class=\"docClass\">RowExpander</a> plugin MUST\nbe used for intra-cell navigation to be correct.</p>\n","!type":"fn(config: ?)","prototype":{"colSpanDecrement":{"!type":"number"},"eventPrefix":{"!type":"string","!doc":"<p>Prefix to use when firing events on the view.\nFor example a prefix of group would expose \"groupclick\", \"groupcontextmenu\", \"groupdblclick\".</p>\n"},"eventSelector":{"!type":"string","!doc":"<p>Selector used to determine when to fire the event with the eventPrefix.</p>\n"},"extraRowTpl":{},"outerTpl":{},"rowBodyCls":{"!type":"string"},"rowBodyHiddenCls":{"!type":"string"},"rowBodyTdSelector":{"!type":"string"},"cleanup":{"!type":"fn(rows: ?, rowValues: ?) -> !this"},"getAdditionalData":{"!type":"fn(data: ?, idx: ?, record: ?, orig: ?)","!doc":"<p>Provides additional data to the prepareData call within the grid view.\nThe rowbody feature adds 3 additional variables into the grid view's template.\nThese are rowBodyCls, rowBodyColspan, and rowBody.</p>\n"},"getSelectedRow":{"!type":"fn(view: ?, rowIndex: ?) -> !this"},"init":{"!type":"fn(grid: ?) -> !this"},"onClick":{"!type":"fn(e: ?) -> !this","!doc":"<p>Needed to select the data row when clicked on the body row.</p>\n"},"onColumnsChanged":{"!type":"fn(headerCt: ?) -> !this","!doc":"<p>When columns added/removed, keep row body colspan in sync with number of columns.</p>\n"},"setup":{"!type":"fn(rows: ?, rowValues: ?) -> !this"}}},"Summary":{"!doc":"<p>This feature is used to place a summary row at the bottom of the grid. If using a grouping,\nsee <a href=\"#!/api/Ext.grid.feature.GroupingSummary\" rel=\"Ext.grid.feature.GroupingSummary\" class=\"docClass\">Ext.grid.feature.GroupingSummary</a>. There are 2 aspects to calculating the summaries,\ncalculation and rendering.</p>\n\n<h2>Calculation</h2>\n\n<p>The summary value needs to be calculated for each column in the grid. This is controlled\nby the summaryType option specified on the column. There are several built in summary types,\nwhich can be specified as a string on the column configuration. These call underlying methods\non the store:</p>\n\n<ul>\n<li>count</li>\n<li>sum</li>\n<li>min</li>\n<li>max</li>\n<li>average</li>\n</ul>\n\n\n<p>Alternatively, the summaryType can be a function definition. If this is the case,\nthe function is called with an array of records to calculate the summary value.</p>\n\n<h2>Rendering</h2>\n\n<p>Similar to a column, the summary also supports a summaryRenderer function. This\nsummaryRenderer is called before displaying a value. The function is optional, if\nnot specified the default calculated value is shown. The summaryRenderer is called with:</p>\n\n<ul>\n<li>value {Object} - The calculated value.</li>\n<li>summaryData {Object} - Contains all raw summary values for the row.</li>\n<li>field {String} - The name of the field we are calculating</li>\n</ul>\n\n\n<h2>Example Usage</h2>\n\n<pre class='inline-example '><code>Ext.define('TestResult', {\n    extend: 'Ext.data.Model',\n    fields: ['student', {\n        name: 'mark',\n        type: 'int'\n    }]\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    width: 400,\n    height: 200,\n    title: 'Summary Test',\n    style: 'padding: 20px',\n    renderTo: document.body,\n    features: [{\n        ftype: 'summary'\n    }],\n    store: {\n        model: 'TestResult',\n        data: [{\n            student: 'Student 1',\n            mark: 84\n        },{\n            student: 'Student 2',\n            mark: 72\n        },{\n            student: 'Student 3',\n            mark: 96\n        },{\n            student: 'Student 4',\n            mark: 68\n        }]\n    },\n    columns: [{\n        dataIndex: 'student',\n        text: 'Name',\n        summaryType: 'count',\n        summaryRenderer: function(value, summaryData, dataIndex) {\n            return Ext.String.format('{0} student{1}', value, value !== 1 ? 's' : '');\n        }\n    }, {\n        dataIndex: 'mark',\n        text: 'Mark',\n        summaryType: 'average'\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"dock":{"!type":"string","!doc":"<p>Configure <code>'top'</code> or <code>'bottom'</code> top create a fixed summary row either above or below the scrollable table.</p>\n"},"dockedSummaryCls":{"!type":"string"},"hasFeatureEvent":{"!type":"bool","!doc":"<p>turn off feature events.</p>\n"},"panelBodyCls":{"!type":"string"},"scrollPadProperty":{"!type":"string"},"summaryItemCls":{"!type":"string"},"createSummaryRecord":{"!type":"fn(view: ?) -> !this"},"init":{"!type":"fn(grid: ?) -> !this","!doc":"<p>Listen for store updates. Eg, from an Editor.</p>\n"},"onColumnHeaderLayout":{"!type":"fn() -> !this","!doc":"<p>Synchronize column widths in the docked summary Component</p>\n"},"onStoreUpdate":{"!type":"fn() -> !this"},"onViewScroll":{"!type":"fn() -> !this"},"renderSummaryRow":{"!type":"fn(values: ?, out: ?, parent: ?) -> !this"},"toggleSummaryRow":{"!type":"fn(visible: ?) -> !this","!doc":"<p>Toggle whether or not to show the summary row.</p>\n"},"vetoEvent":{"!type":"fn(record: ?, row: ?, rowIndex: ?, e: ?) -> !this"}}}},"filters":{"filter":{"Base":{"!doc":"<p>Abstract base class for filter implementations.</p>\n","!type":"fn(config: ?)","prototype":{"active":{"!type":"bool","!doc":"<p>True if this filter is active.  Use setActive() to alter after configuration.</p>\n"},"dataIndex":{"!type":"string","!doc":"<p>The Ext.data.Store dataIndex of the field this filter represents.\nThe dataIndex does not actually have to exist in the store.</p>\n"},"itemDefaults":{"!doc":"<p>The default configuration options for any menu items created by this filter.</p>\n\n<p>Example usage:</p>\n\n<pre><code> itemDefaults: {\n     width: 150\n },\n</code></pre>\n"},"menuDefaults":{},"updateBuffer":{"!type":"number","!doc":"<p>Number of milliseconds to wait after user interaction to fire an update. Only supported\nby filters: 'list', 'numeric', and 'string'.</p>\n"},"$configPrefixed":{"!type":"bool"},"$configStrict":{"!type":"bool"},"defaultRoot":{"!type":"string"},"factoryConfig":{},"filterIdPrefix":{"!type":"string","!doc":"<p>The prefix for id's used to track stateful Store filters.</p>\n"},"isGridFilter":{"!type":"bool"},"menu":{"!doc":"<p>The filter configuration menu that will be installed into the filter submenu of a column menu.</p>\n"},"type":{"!type":"string","!doc":"<p>The filter type. Used by the filters.Feature class when adding filters and applying state.</p>\n"},"":{"!type":"fn() -> bool","!doc":"<p>Template method to be implemented by all subclasses that is to\nreturn true if the filter has enough configuration information to be activated.\nDefaults to always returning true.</p>\n"},"addStoreFilter":{"!type":"fn(filter: ?) -> !this"},"createFilter":{"!type":"fn(config: ?, key: ?) -> !this"},"createMenu":{"!type":"fn(config: ?) -> ?","!doc":"<p>Creates the Menu for this filter.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroys this filter by purging any event listeners, and removing any menus.</p>\n"},"getBaseIdPrefix":{"!type":"fn() -> !this"},"getItemDefaults":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.grid.filters.filter.Base-cfg-itemDefaults\" rel=\"Ext.grid.filters.filter.Base-cfg-itemDefaults\" class=\"docClass\">itemDefaults</a>.</p>\n"},"getMenuConfig":{"!type":"fn() -> !this"},"getMenuDefaults":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.grid.filters.filter.Base-cfg-menuDefaults\" rel=\"Ext.grid.filters.filter.Base-cfg-menuDefaults\" class=\"docClass\">menuDefaults</a>.</p>\n"},"getStoreFilter":{"!type":"fn(key: ?) -> !this"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Template method to be implemented by all subclasses that is to\nget and return the value of the filter.</p>\n"},"removeStoreFilter":{"!type":"fn(filter: ?) -> !this"},"setActive":{"!type":"fn(active: ?, key: ?) -> !this","!doc":"<p>Sets the status of the filter and fires the appropriate events.</p>\n"},"setItemDefaults":{"!type":"fn(itemDefaults: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.grid.filters.filter.Base-cfg-itemDefaults\" rel=\"Ext.grid.filters.filter.Base-cfg-itemDefaults\" class=\"docClass\">itemDefaults</a>.</p>\n"},"setMenuDefaults":{"!type":"fn(menuDefaults: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.grid.filters.filter.Base-cfg-menuDefaults\" rel=\"Ext.grid.filters.filter.Base-cfg-menuDefaults\" class=\"docClass\">menuDefaults</a>.</p>\n"},"setValue":{"!type":"fn(data: ?)","!doc":"<p>Template method to be implemented by all subclasses that is to\nset the value of the filter and fire the 'update' event.</p>\n"},"showMenu":{"!type":"fn(menuItem: ?) -> !this"},"updateStoreFilter":{"!type":"fn(filter: ?) -> !this"},"activate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when an inactive filter becomes active</p>\n"},"deactivate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when an active filter becomes inactive</p>\n"},"update":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when a filter configuration has changed</p>\n"}}},"Boolean":{"!doc":"<p>Boolean filters use unique radio group IDs (so you can have more than one!)</p>\n","!type":"fn(config: ?)","prototype":{"defaultValue":{"!type":"bool","!doc":"<p>Set this to null if you do not want either option to be checked by default. Defaults to false.</p>\n"},"noText":{"!type":"string","!doc":"<p>Defaults to 'No'.</p>\n"},"yesText":{"!type":"string","!doc":"<p>Defaults to 'Yes'.</p>\n"},"operator":{"!type":"string"},"type":{"!type":"string","!doc":"<p>The filter type. Used by the filters.Feature class when adding filters and applying state.</p>\n"},"activateMenu":{"!type":"fn() -> !this","!doc":"<p>This is supposed to be just a stub.</p>\n"},"createMenu":{"!type":"fn(config: ?) -> !this","!doc":"<p>Template method that is to initialize the filter and install required menu items.</p>\n"},"onClick":{"!type":"fn(field: ?) -> !this"},"setValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Template method that is to set the value of the filter.</p>\n"}}},"SingleFilter":{"prototype":{"activate":{"!type":"fn(showingMenu: ?) -> !this"},"deactivate":{"!type":"fn() -> !this"},"onFilterRemove":{"!type":"fn() -> !this"}},"!doc":"<p>This abstract base class is used by grid filters that have a single\nstore filter.</p>\n","!type":"fn(config: ?)"},"Date":{"!doc":"<p>Filter by a configurable Ext.picker.DatePicker menu</p>\n\n<p>Example Usage:</p>\n\n<pre><code>var grid = Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    ...\n    columns: [{\n        // required configs\n        text: 'Date Added',\n        dataIndex: 'dateAdded',\n\n        filter: {\n            type: 'date',\n\n            // optional configs\n            dateFormat: 'm/d/Y',  // default\n            beforeText: 'Before', // default\n            afterText: 'After',   // default\n            onText: 'On',         // default\n            pickerOpts: {\n                // any DatePicker configs\n            },\n\n            active: true // default is false\n        }\n    }],\n    ...\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"dateFormat":{"!type":"string","!doc":"<p>The date format to return when using getValue.\nDefaults to 'm/d/Y'.</p>\n"},"fields":{"!doc":"<p>Configures field items individually. These properties override those defined\nby <code><a href=\"#!/api/Ext.grid.filters.filter.Date-cfg-itemDefaults\" rel=\"Ext.grid.filters.filter.Date-cfg-itemDefaults\" class=\"docClass\">itemDefaults</a></code>.</p>\n\n<p>Example usage:\n     fields: {\n         gt: { // override fieldCfg options\n             width: 200\n         }\n     },</p>\n"},"itemDefaults":{"!doc":"<p>The default configuration options for any menu items created by this filter.</p>\n\n<p>Example usage:</p>\n\n<pre><code> itemDefaults: {\n     width: 150\n },\n</code></pre>\n"},"maxDate":{"!doc":"<p>Allowable date as passed to the <a href=\"#!/api/Ext.picker.Date\" rel=\"Ext.picker.Date\" class=\"docClass\">Ext.DatePicker</a>\nDefaults to undefined.</p>\n"},"minDate":{"!doc":"<p>Allowable date as passed to the <a href=\"#!/api/Ext.picker.Date\" rel=\"Ext.picker.Date\" class=\"docClass\">Ext.DatePicker</a>\nDefaults to undefined.</p>\n"},"pickerOpts":{"!doc":"<p>Configuration options for the date picker associated with each field.</p>\n"},"":{"!doc":"<p>Handler method called when there is a keyup event on an input\nitem of this menu.</p>\n"},"type":{"!type":"string","!doc":"<p>The filter type. Used by the filters.Feature class when adding filters and applying state.</p>\n"},"convertValue":{"!type":"fn(value: ?, convertToDate?: ?) -> ?","!doc":"<p>Will convert a timestamp to a Date object or vice-versa.</p>\n"},"createMenu":{"!type":"fn(config: ?) -> !this","!doc":"<p>Template method that is to initialize the filter and install required menu items.</p>\n"},"getFields":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.grid.filters.filter.Date-cfg-fields\" rel=\"Ext.grid.filters.filter.Date-cfg-fields\" class=\"docClass\">fields</a>.</p>\n"},"getPicker":{"!type":"fn(item: ?) -> ?","!doc":"<p>Gets the menu picker associated with the passed field</p>\n"},"onCheckChange":{"!type":"fn(field: ?, checked: ?) -> !this","!doc":"<p>Remove the filter from the store but don't update its value or the field UI.</p>\n"},"onFilterRemove":{"!type":"fn(operator: ?) -> !this"},"onMenuSelect":{"!type":"fn(picker: ?, date: ?) -> !this","!doc":"<p>Handler for when the DatePicker for a field fires the 'select' event</p>\n"},"setFields":{"!type":"fn(fields: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.grid.filters.filter.Date-cfg-fields\" rel=\"Ext.grid.filters.filter.Date-cfg-fields\" class=\"docClass\">fields</a>.</p>\n"}}},"TriFilter":{"prototype":{"menuItems":{"!doc":"<p>The items to be shown in this menu.  Items are added to the menu\naccording to their position within this array.\nDefaults to:\n     menuItems : ['lt', 'gt', '-', 'eq']</p>\n"},"activate":{"!type":"fn(showingMenu: ?) -> !this","!doc":"<p>This method will be called when a column's menu trigger is clicked as well as when a filter is\nactivated. Depending on the caller, the UI and the store will be synced.</p>\n"},"deactivate":{"!type":"fn() -> !this","!doc":"<p>This method will be called when a filter is deactivated. The UI and the store will be synced.</p>\n"},"hasActiveFilter":{"!type":"fn() -> !this"},"setValue":{"!type":"fn(data: ?) -> !this","!doc":"<p>Template method to be implemented by all subclasses that is to\nset the value of the filter and fire the 'update' event.</p>\n"},"onFilterRemove":{"!type":"fn(operator: ?) -> !this"}},"!doc":"<p>This abstract base class is used by grid filters that have a three\nstore filter.</p>\n","!type":"fn(config: ?)"},"List":{"!doc":"<p>List filters are able to be preloaded/backed by an Ext.data.Store to load\ntheir options the first time they are shown.</p>\n\n<p>List filters are also able to create their own list of values from  all unique values of\nthe specified <a href=\"#!/api/Ext.grid.filters.filter.List-cfg-dataIndex\" rel=\"Ext.grid.filters.filter.List-cfg-dataIndex\" class=\"docClass\">dataIndex</a> field in the store at first time of filter invocation.</p>\n\n<p>Example Usage:</p>\n\n<pre><code>var filters = Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    ...\n    columns: [{\n        text: 'Size',\n        dataIndex: 'size',\n\n        filter: {\n            type: 'list',\n            // options will be used as data to implicitly creates an ArrayStore\n            options: ['extra small', 'small', 'medium', 'large', 'extra large']\n        }\n    }],\n    ...\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"idField":{"!type":"string","!doc":"<p>Defaults to 'id'.</p>\n"},"itemDefaults":{"!doc":"<p>The default configuration options for any menu items created by this filter.</p>\n\n<p>Example usage:</p>\n\n<pre><code> itemDefaults: {\n     width: 150\n },\n</code></pre>\n"},"labelField":{"!type":"string","!doc":"<p>Defaults to 'text'.</p>\n"},"loadOnShow":{"!type":"bool","!doc":"<p>Defaults to true.</p>\n"},"options":{"!doc":"<p><code>data</code> to be used to implicitly create a data store\nto back this list when the data source is <strong>local</strong>. If the\ndata for the list is remote, use the <a href=\"#!/api/Ext.grid.filters.filter.List-cfg-store\" rel=\"Ext.grid.filters.filter.List-cfg-store\" class=\"docClass\">store</a>\nconfig instead.</p>\n\n<p>If neither store nor <a href=\"#!/api/Ext.grid.filters.filter.List-cfg-options\" rel=\"Ext.grid.filters.filter.List-cfg-options\" class=\"docClass\">options</a> is specified, then the choices list is automatically\npopulated from all unique values of the specified <a href=\"#!/api/Ext.grid.filters.filter.List-cfg-dataIndex\" rel=\"Ext.grid.filters.filter.List-cfg-dataIndex\" class=\"docClass\">dataIndex</a> field in the store at first\ntime of filter invocation.</p>\n\n<p>Each item within the provided array may be in one of the\nfollowing formats:</p>\n\n<ul>\n<li><p><strong>Array</strong> :</p>\n\n<pre><code>options: [\n    [11, 'extra small'],\n    [18, 'small'],\n    [22, 'medium'],\n    [35, 'large'],\n    [44, 'extra large']\n]\n</code></pre></li>\n<li><p><strong>Object</strong> :</p>\n\n<pre><code>labelField: 'name', // override default of 'text'\noptions: [\n    {id: 11, name:'extra small'},\n    {id: 18, name:'small'},\n    {id: 22, name:'medium'},\n    {id: 35, name:'large'},\n    {id: 44, name:'extra large'}\n]\n</code></pre></li>\n<li><p><strong>String</strong> :</p>\n\n<pre><code>options: ['extra small', 'small', 'medium', 'large', 'extra large']\n</code></pre></li>\n</ul>\n\n"},"paramPrefix":{"!type":"string","!doc":"<p>Defaults to 'Loading...'.</p>\n"},"single":{"!type":"bool","!doc":"<p>Specify true to group all items in this list into a single-select\nradio button group. Defaults to false.</p>\n"},"store":{"!doc":"<p>The Ext.data.Store this list should use as its data source\nwhen the data source is <strong>remote</strong>. If the data for the list\nis local, use the <a href=\"#!/api/Ext.grid.filters.filter.List-cfg-options\" rel=\"Ext.grid.filters.filter.List-cfg-options\" class=\"docClass\">options</a> config instead.</p>\n\n<p>If neither store nor <a href=\"#!/api/Ext.grid.filters.filter.List-cfg-options\" rel=\"Ext.grid.filters.filter.List-cfg-options\" class=\"docClass\">options</a> is specified, then the choices list is automatically\npopulated from all unique values of the specified <a href=\"#!/api/Ext.grid.filters.filter.List-cfg-dataIndex\" rel=\"Ext.grid.filters.filter.List-cfg-dataIndex\" class=\"docClass\">dataIndex</a> field in the store at first\ntime of filter invocation.</p>\n"},"operator":{"!type":"string"},"plain":{"!type":"bool"},"type":{"!type":"string","!doc":"<p>The filter type. Used by the filters.Feature class when adding filters and applying state.</p>\n"},"activateMenu":{"!type":"fn() -> !this"},"createMenu":{"!type":"fn(config: ?) -> ?","!doc":"<p>Creates the Menu for this filter.</p>\n"},"createMenuItems":{"!type":"fn(store: ?) -> !this"},"createMenuStore":{"!type":"fn(options: ?) -> !this"},"setValue":{"!type":"fn() -> !this","!doc":"<p>Template method that is to set the value of the filter.</p>\n"},"show":{"!type":"fn() -> !this","!doc":"<p>Lists will initially show a 'loading' item while the data is retrieved from the store.\nIn some cases the loaded data will result in a list that goes off the screen to the\nright (as placement calculations were done with the loading item). This adapter will\nallow show to be called with no arguments to show with the previous arguments and\nthus recalculate the width and potentially hang the menu from the left.</p>\n"}}},"Number":{"!doc":"<p>Filter type for <a href=\"#!/api/Ext.grid.column.Number\" rel=\"Ext.grid.column.Number\" class=\"docClass\">number columns</a>.</p>\n","!type":"fn(config: ?)","prototype":{"fields":{"!doc":"<p>Configures field items individually. These properties override those defined\nby <code><a href=\"#!/api/Ext.grid.filters.filter.Number-cfg-itemDefaults\" rel=\"Ext.grid.filters.filter.Number-cfg-itemDefaults\" class=\"docClass\">itemDefaults</a></code>.</p>\n\n<p>Example usage:</p>\n\n<pre><code> fields: {\n     // Override itemDefaults for one field:\n     gt: {\n         width: 200\n     }\n\n     // \"lt\" and \"eq\" fields retain all itemDefaults\n },\n</code></pre>\n"},"itemDefaults":{"!doc":"<p>The default configuration options for any menu items created by this filter.</p>\n\n<p>Example usage:</p>\n\n<pre><code> itemDefaults: {\n     width: 150\n },\n</code></pre>\n"},"menuDefaults":{},"convertValue":{"!doc":"<p>See the Date type for a full implementation.</p>\n"},"type":{"!type":"string","!doc":"<p>The filter type. Used by the filters.Feature class when adding filters and applying state.</p>\n"},"createMenu":{"!type":"fn(config: ?) -> ?","!doc":"<p>Creates the Menu for this filter.</p>\n"},"getFields":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.grid.filters.filter.Number-cfg-fields\" rel=\"Ext.grid.filters.filter.Number-cfg-fields\" class=\"docClass\">fields</a>.</p>\n"},"onInputKeyUp":{"!type":"fn(field: ?, e: ?) -> !this","!doc":"<p>Handler method called when there is a keyup event on an input\nitem of this menu.</p>\n"},"onInputSpin":{"!type":"fn(field: ?, direction: ?) -> !this","!doc":"<p>Handler method called when there is a spin event on a NumberField\nitem of this menu.</p>\n"},"setFields":{"!type":"fn(fields: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.grid.filters.filter.Number-cfg-fields\" rel=\"Ext.grid.filters.filter.Number-cfg-fields\" class=\"docClass\">fields</a>.</p>\n"},"stopFn":{"!type":"fn(e: ?) -> !this"}}},"String":{"!doc":"<p>Filter by a configurable <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a></p>\n\n<p>Example Usage:</p>\n\n<pre><code>var grid = Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    ...\n    columns: [{\n        text: 'Name',\n        dataIndex: 'name',\n\n        filter: {\n            // required configs\n            type: 'string',\n\n            // optional configs\n            value: 'foo',\n            active: true, // default is false\n            itemDefaults: {\n                // any <a href=\"#!/api/Ext.form.field.Text\" rel=\"Ext.form.field.Text\" class=\"docClass\">Ext.form.field.Text</a> configs accepted\n            }\n        }\n    }],\n    ...\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"itemDefaults":{"!doc":"<p>The default configuration options for any menu items created by this filter.</p>\n\n<p>Example usage:</p>\n\n<pre><code> itemDefaults: {\n     width: 150\n },\n</code></pre>\n"},"menuDefaults":{},"operator":{"!type":"string"},"type":{"!type":"string","!doc":"<p>The filter type. Used by the filters.Feature class when adding filters and applying state.</p>\n"},"activateMenu":{"!type":"fn() -> !this"},"createMenu":{"!type":"fn() -> !this","!doc":"<p>Template method that is to initialize the filter and install required menu items.</p>\n"},"onInputKeyUp":{"!type":"fn(field: ?, e: ?) -> !this","!doc":"<p>Handler method called when there is a keyup event on this.inputItem</p>\n"},"setValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Template method that is to set the value of the filter.</p>\n"}}}},"Filters":{"!doc":"<p>This class is a grid <a href=\"#!/api/Ext.plugin.Abstract\" rel=\"Ext.plugin.Abstract\" class=\"docClass\">plugin</a> that adds a simple and flexible\npresentation for store filters.</p>\n\n<p>Filters can be modified by the end-user using the grid's column header menu. Through\nthis menu users can configure, enable, and disable filters for each column.</p>\n\n<h1>Example Usage</h1>\n\n<pre><code>var grid = Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n     store: {\n         url: 'path/to/data'\n     },\n\n     plugins: 'gridfilters',\n\n     columns: [{\n         dataIndex: 'id',\n         text: 'Id',\n\n         filter: 'number'\n     }, {\n         dataIndex: 'name'\n         text: 'Name',\n\n         filter: {\n             type: 'string',\n             value: 'Ben'\n         }\n     }, {\n         ...\n     }]\n});\n\n// A filters property is added to the grid:\n\nvar plugin = grid.filters;\n</code></pre>\n\n<h1>Features</h1>\n\n<h2>Filtering implementations</h2>\n\n<p>Currently provided filter types are:</p>\n\n<ul>\n<li><code><a href=\"#!/api/Ext.grid.filters.filter.Boolean\" rel=\"Ext.grid.filters.filter.Boolean\" class=\"docClass\">boolean</a></code></li>\n<li><code><a href=\"#!/api/Ext.grid.filters.filter.Date\" rel=\"Ext.grid.filters.filter.Date\" class=\"docClass\">date</a></code></li>\n<li><code><a href=\"#!/api/Ext.grid.filters.filter.List\" rel=\"Ext.grid.filters.filter.List\" class=\"docClass\">list</a></code></li>\n<li><code><a href=\"#!/api/Ext.grid.filters.filter.Number\" rel=\"Ext.grid.filters.filter.Number\" class=\"docClass\">number</a></code></li>\n<li><code><a href=\"#!/api/Ext.grid.filters.filter.String\" rel=\"Ext.grid.filters.filter.String\" class=\"docClass\">string</a></code></li>\n</ul>\n\n\n<h2>Graphical Indicators</h2>\n\n<p>Columns that are filtered have <a href=\"#!/api/Ext.grid.filters.Filters-property-filterCls\" rel=\"Ext.grid.filters.Filters-property-filterCls\" class=\"docClass\">CSS class</a> applied to their column\nheaders. This style can be managed using that CSS class or by setting these Sass\nvariables in your theme or application:</p>\n\n<pre><code> $grid-filters-column-filtered-font-style: italic !default;\n\n $grid-filters-column-filtered-font-weight: bold !default;\n</code></pre>\n\n<h2>Stateful</h2>\n\n<p>Filter information will be persisted across page loads by specifying a <code>stateId</code>\nin the Grid configuration. In actuality this state is saved by the <code>store</code>, but this\nplugin ensures that saved filters are properly identified and reclaimed on subsequent\nvisits to the page.</p>\n\n<h2>Grid Changes</h2>\n\n<ul>\n<li>A <code>filters</code> property is added to the Grid using this plugin.</li>\n</ul>\n\n\n<h1>Upgrading From Ext.ux.grid.FilterFeature</h1>\n\n<p>The biggest change for developers converting from the user extension is most likely the\nconversion to standard store filters. In the\nprocess, the \"like\" and \"in\" operators are now supported by <code>Ext.util.Filter</code>.\nThese filters and all other filters added to the store will be sent in the standard\nway (using the \"filters\" parameter by default).</p>\n\n<p>Since this plugin now uses actual store filters, the <code>onBeforeLoad</code> listener and all\nhelper methods that were used to clean and build the params have been removed. The store\nwill send the filters managed by this plugin along in its normal request.</p>\n","!type":"fn(config?: ?)","prototype":{"menuFilterText":{"!type":"string","!doc":"<p>The text for the filters menu.</p>\n"},"pluginId":{"!type":"string","!doc":"<p>A name for the plugin that can be set at creation time to then retrieve the plugin\nthrough <a href=\"#!/api/Ext.Component-method-getPlugin\" rel=\"Ext.Component-method-getPlugin\" class=\"docClass\">getPlugin</a> method.  For example:</p>\n\n<pre><code>var grid = Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    plugins: [{\n        ptype: 'cellediting',\n        clicksToEdit: 2,\n        pluginId: 'cellplugin'\n    }]\n});\n\n// later on:\nvar plugin = grid.getPlugin('cellplugin');\n</code></pre>\n"},"showMenu":{"!type":"bool","!doc":"<p>Defaults to true, including a filter submenu in the default header menu.</p>\n"},"stateId":{"!type":"string","!doc":"<p>Name of the value to be used to store state information.</p>\n"},"defaultFilterTypes":{"!doc":"<p>This property maps field type to the appropriate\ngrid filter type.</p>\n"},"filterCls":{"!type":"string","!doc":"<p>The CSS applied to column headers with active filters.</p>\n"},"addFilter":{"!type":"fn(filters: ?) -> ?","!doc":"<p>Adds a filter to the collection and creates a store filter if has a <code>value</code> property.</p>\n"},"addFilters":{"!type":"fn(filters: ?) -> ?","!doc":"<p>Adds filters to the collection.</p>\n"},"bindStore":{"!type":"fn(store: ?) -> !this","!doc":"<p>Changes the data store bound to this view and refreshes it.</p>\n"},"clearFilters":{"!type":"fn(autoFilter: ?) -> !this","!doc":"<p>Turns all filters off. This does not clear the configuration information.</p>\n"},"createColumnFilter":{"!type":"fn(column: ?) -> !this"},"createMenuItem":{"!type":"fn(menu: ?, ownerGridId: ?) -> !this"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Handler called by the grid 'beforedestroy' event</p>\n"},"getFilter":{"!type":"fn(dataIndex: ?) -> ?","!doc":"<p>Returns a filter for the given dataIndex, if one exists.</p>\n"},"getHeaders":{"!type":"fn() -> !this"},"getMenuFilter":{"!type":"fn(headerCt: ?) -> !this","!doc":"<p>Get the filter menu from the filters MixedCollection based on the clicked header.</p>\n"},"init":{"!type":"fn(grid: ?) -> !this","!doc":"<p>The init method is invoked after initComponent method has been run for the client Component.</p>\n\n<p>The supplied implementation is empty. Subclasses should perform plugin initialization, and set up bidirectional\nlinks between the plugin and its client Component in their own implementation of this method.</p>\n"},"initColumns":{"!type":"fn() -> !this","!doc":"<p>Creates the Filter objects for the current configuration.\nReconfigure and on add handlers.</p>\n"},"isStateful":{"!type":"fn() -> bool","!doc":"<p>Checks the plugin's grid for statefulness.</p>\n"},"onAdd":{"!type":"fn(headerCt: ?, column: ?, index: ?) -> !this"},"onBeforeReconfigure":{"!type":"fn(grid: ?, store: ?, columns: ?) -> !this"},"onCheckChange":{"!type":"fn(item: ?, value: ?) -> !this"},"onFilterRemove":{"!type":"fn(filterCollection: ?, list: ?) -> !this"},"onMenuBeforeShow":{"!type":"fn(menu: ?) -> !this","!doc":"<p>Handle showing of the grid's header menu. Sets up the filter item and menu\nappropriate for the target column.</p>\n"},"onMenuCreate":{"!type":"fn(headerCt: ?, menu: ?) -> !this","!doc":"<p>Handle creation of the grid's header menu.</p>\n"},"onReconfigure":{"!type":"fn(grid: ?, store: ?, columns: ?, oldStore: ?) -> !this"}}}},"locking":{"HeaderContainer":{"!doc":"<p>Private class which acts as a HeaderContainer for the Lockable which aggregates all columns\nfrom both sides of the Lockable. It is never rendered, it's just used to interrogate the\ncolumn collection.</p>\n","!type":"fn(lockable: ?)","prototype":{"headerCtRelayEvents":{},"applyColumnsState":{"!type":"fn(columns: ?) -> !this","!doc":"<p>Lockable uses its headerCt to apply column state</p>\n"},"getColumnsState":{"!type":"fn() -> !this","!doc":"<p>Lockable uses its headerCt to gather column state</p>\n"},"getGridColumns":{"!type":"fn() -> ?","!doc":"<p>This is the function which all other column access methods are based upon\nReturn the full column set for the whole Lockable assembly</p>\n"},"getRefItems":{"!type":"fn() -> !this","!doc":"<p>Used by ComponentQuery, child and down to retrieve all of the items\nwhich can potentially be considered a child of this Container.</p>\n\n<p>This may be overriden by Components which have ownership of Components\nthat are not contained in the <a href=\"#!/api/Ext.grid.locking.HeaderContainer-property-items\" rel=\"Ext.grid.locking.HeaderContainer-property-items\" class=\"docClass\">items</a> collection.</p>\n\n<p>NOTE: IMPORTANT note for maintainers:\nItems are returned in tree traversal order. Each item is appended to the result array\nfollowed by the results of that child's getRefItems call.\nFloating child items are appended after internal child items.</p>\n"}}},"Lockable":{"!doc":"<p>Lockable is a private mixin which injects lockable behavior into any\nTablePanel subclass such as GridPanel or TreePanel. TablePanel will\nautomatically inject the <a href=\"#!/api/Ext.grid.locking.Lockable\" rel=\"Ext.grid.locking.Lockable\" class=\"docClass\">Ext.grid.locking.Lockable</a> mixin in when one of the\nthese conditions are met:</p>\n\n<ul>\n<li>The TablePanel has the lockable configuration set to true</li>\n<li>One of the columns in the TablePanel has locked set to true/false</li>\n</ul>\n\n\n<p>Each TablePanel subclass must register an alias. It should have an array\nof configurations to copy to the 2 separate tablepanels that will be generated\nto note what configurations should be copied. These are named normalCfgCopy and\nlockedCfgCopy respectively.</p>\n\n<p>Configurations which are specified in this class will be available on any grid or\ntree which is using the lockable functionality.</p>\n\n<p>By default the two grids, \"locked\" and \"normal\" will be arranged using an <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">hbox</a>\nlayout. If the lockable grid is configured with <code><a href=\"#!/api/Ext.grid.locking.Lockable-cfg-split\" rel=\"Ext.grid.locking.Lockable-cfg-split\" class=\"docClass\">split:true</a></code>, a vertical splitter\nwill be placed between the two grids to resize them.</p>\n\n<p>It is possible to override the layout of the lockable grid, or example, you may wish to\nuse a border layout and have one of the grids collapsible.</p>\n","prototype":{"layout":{"!doc":"<p>By default, a lockable grid uses an <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">HBox</a> layout to arrange\nthe two grids (possibly separated by a splitter).</p>\n\n<p>Using this config it is possible to specify a different layout to arrange the two grids.</p>\n"},"lockedGridConfig":{"!doc":"<p>Any special configuration options for the locked part of the grid</p>\n"},"lockedViewConfig":{"!doc":"<p>A view configuration to be applied to the\nlocked side of the grid. Any conflicting configurations between lockedViewConfig\nand viewConfig will be overwritten by the lockedViewConfig.</p>\n"},"normalGridConfig":{"!doc":"<p>Any special configuration options for the normal part of the grid</p>\n"},"normalViewConfig":{"!doc":"<p>A view configuration to be applied to the\nnormal/unlocked side of the grid. Any conflicting configurations between normalViewConfig\nand viewConfig will be overwritten by the normalViewConfig.</p>\n"},"scrollDelta":{"!type":"number","!doc":"<p>Number of pixels to scroll when scrolling the locked section with mousewheel.</p>\n"},"split":{"!type":"bool","!doc":"<p>Configure as true to place a resizing <a href=\"#!/api/Ext.resizer.Splitter\" rel=\"Ext.resizer.Splitter\" class=\"docClass\">splitter</a> between the locked\nand unlocked columns.</p>\n"},"subGridXType":{"!type":"string","!doc":"<p>The xtype of the subgrid to specify. If this is\nnot specified lockable will determine the subgrid xtype to create by the\nfollowing rule. Use the superclasses xtype if the superclass is NOT\ntablepanel, otherwise use the xtype itself.</p>\n"},"syncRowHeight":{"!type":"bool","!doc":"<p>Synchronize rowHeight between the normal and\nlocked grid view. This is turned on by default. If your grid is guaranteed\nto have rows of all the same height, you should set this to false to\noptimize performance.</p>\n"},"bothCfgCopy":{"!doc":"<p>Required for the Lockable Mixin. These are the configurations which will be copied to the\nnormal and locked sub tablepanels</p>\n"},"lockText":{"!type":"string","!doc":"<p>\n<locale></locale></p>\n"},"lockedCfgCopy":{},"normalCfgCopy":{},"normalGridCls":{"!type":"string"},"supportsOverflowX":{},"unlockText":{"!type":"string","!doc":"<p>i8n text\n<locale></locale></p>\n"},"afterLockedViewLayout":{"!type":"fn() -> !this","!doc":"<p>Due to automatic component border setting using inline style, to create the scrollbar-replacing\nbottom border, we have to postprocess the locked view <em>after</em> render.\nA tall bottom border takes the place of a horiz scrollbar if the opposite side has a  horiz scrollbar.\nWhen we can use overflow-x: scroll to create a matching scrollbar, we do this instead.</p>\n"},"constructLockableFeatures":{"!type":"fn() -> !this"},"constructLockablePlugins":{"!type":"fn() -> !this"},"destroyLockable":{"!type":"fn() -> !this"},"getColumnWidth":{"!type":"fn(column: ?) -> !this","!doc":"<p>Used when calculating total locked column width in processColumns\nUse shrinkwrapping of child columns if no top level width.</p>\n"},"getLockingViewConfig":{"!type":"fn() -> !this"},"getMenuItems":{"!type":"fn(getMenuItems: ?, locked: ?) -> !this"},"injectLockable":{"!type":"fn() -> !this","!doc":"<p>injectLockable will be invoked before initComponent's parent class implementation\nis called, so throughout this method this. are configurations</p>\n"},"lock":{"!type":"fn(header?: ?, toIdx?: ?) -> !this","!doc":"<p>Locks the activeHeader as determined by which menu is open OR a header\nas specified.</p>\n"},"modifyHeaderCt":{"!type":"fn() -> !this","!doc":"<p>inject Lock and Unlock text\nHide/show Lock/Unlock options</p>\n"},"onLockMenuClick":{"!type":"fn() -> !this"},"onLockedHeaderAdd":{"!type":"fn() -> !this"},"onLockedHeaderHide":{"!type":"fn() -> !this"},"onLockedHeaderResize":{"!type":"fn() -> !this"},"onLockedHeaderShow":{"!type":"fn() -> !this"},"onLockedHeaderSortChange":{"!type":"fn() -> !this"},"onLockedViewMouseWheel":{"!type":"fn(e: ?) -> !this","!doc":"<p>Listen for mousewheel events on the locked section which does not scroll.\nScroll it in response, and the other section will automatically sync.</p>\n"},"onLockedViewScroll":{"!type":"fn() -> !this"},"onNormalHeaderSortChange":{"!type":"fn() -> !this"},"onNormalViewScroll":{"!type":"fn() -> !this"},"onUnlockMenuClick":{"!type":"fn() -> !this"},"processColumns":{"!type":"fn(columns: ?, lockedGrid: ?) -> !this"},"reconfigureLockable":{"!type":"fn(store: ?, columns: ?) -> !this","!doc":"<p>we want to totally override the reconfigure behaviour here, since we're creating 2 sub-grids</p>\n"},"showMenuBy":{"!type":"fn(t: ?, header: ?) -> !this"},"syncLockedWidth":{"!type":"fn() -> bool","!doc":"<p>Updates the overall view after columns have been resized, or moved from\nthe locked to unlocked side or vice-versa.</p>\n\n<p>If all columns are removed from either side, that side must be hidden, and the\nsole remaining column owning grid then becomes <em>the</em> grid. It must flex to occupy the\nwhole of the locking view. And it must also allow scrolling.</p>\n\n<p>If columns are shared between the two sides, the <em>locked</em> grid shrinkwraps the\nwidth of the visible locked columns while the normal grid flexes in what space remains.</p>\n"},"syncRowHeights":{"!type":"fn() -> !this","!doc":"<p>Synchronizes the row heights between the locked and non locked portion of the grid for each\nrow. If one row is smaller than the other, the height will be increased to match the larger one.</p>\n"},"unlock":{"!type":"fn(header?: ?, toIdx?: ?) -> !this","!doc":"<p>Unlocks the activeHeader as determined by which menu is open OR a header\nas specified.</p>\n"},"filterchange":{"!type":"fn(eOpts: ?)"},"groupchange":{"!type":"fn(eOpts: ?)"},"lockcolumn":{"!type":"fn(this: ?, column: ?, eOpts: ?)","!doc":"<p>Fires when a column is locked.</p>\n"},"processcolumns":{"!type":"fn(lockedColumns: ?, normalColumns: ?, eOpts: ?)","!doc":"<p>Fires when the configured (or <strong>reconfigured</strong>) column set is split into two depending on the <a href=\"#!/api/Ext.grid.column.Column-cfg-locked\" rel=\"Ext.grid.column.Column-cfg-locked\" class=\"docClass\">locked</a> flag.</p>\n"},"unlockcolumn":{"!type":"fn(this: ?, column: ?, eOpts: ?)","!doc":"<p>Fires when a column is unlocked.</p>\n"},"viewready":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when the grid view is available (use this for selecting a default row).</p>\n"}}},"View":{"!doc":"<p>This class is used internally to provide a single interface when using\na locking grid. Internally, the locking grid creates two separate grids,\nso this class is used to map calls appropriately.</p>\n","!type":"fn(config: ?)","prototype":{"eventRelayRe":{},"isLockingView":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated LockingView, or subclass thereof.</p>\n"},"loadMask":{"!type":"bool"},"addElListener":{"!type":"fn(eventName: ?, fn: ?, scope: ?) -> !this"},"addRowCls":{"!type":"fn() -> !this"},"beforeLayout":{"!type":"fn() -> !this","!doc":"<p>Called in the context of a child view when the first child view begins its layout run</p>\n"},"bindStore":{"!type":"fn(store: ?) -> !this","!doc":"<p>Changes the data store bound to this view and refreshes it.</p>\n"},"destroy":{"!type":"fn() -> !this"},"focus":{"!type":"fn() -> !this"},"focusCell":{"!type":"fn(position: ?) -> !this"},"focusRow":{"!type":"fn(row: ?) -> !this"},"getCell":{"!type":"fn(record: ?, column: ?) -> !this"},"getCellByPosition":{"!type":"fn(pos: ?, returnDom: ?) -> !this"},"getEl":{"!type":"fn(column: ?) -> !this"},"getGridColumns":{"!type":"fn() -> !this"},"getNode":{"!type":"fn(nodeInfo: ?) -> !this"},"getRecord":{"!type":"fn(node: ?) -> !this"},"getRefOwner":{"!type":"fn() -> !this"},"getRow":{"!type":"fn(nodeInfo: ?) -> !this"},"getSelectionModel":{"!type":"fn() -> !this"},"getStore":{"!type":"fn() -> ?","!doc":"<p>Gets the current store instance.</p>\n"},"getStoreListeners":{"!type":"fn() -> ?","!doc":"<p>Gets the listeners to bind to a new store.</p>\n"},"getViewForColumn":{"!type":"fn(column: ?) -> !this"},"indexOf":{"!type":"fn(record: ?) -> !this"},"isVisible":{"!type":"fn(deep: ?) -> !this"},"onAdd":{"!type":"fn(eventName: ?, fn: ?, scope: ?) -> !this"},"onDataRefresh":{"!type":"fn(eventName: ?, fn: ?, scope: ?) -> !this"},"onItemMouseEnter":{"!type":"fn(view: ?, record: ?) -> !this"},"onItemMouseLeave":{"!type":"fn(view: ?, record: ?) -> !this"},"onPanelRender":{"!type":"fn() -> !this"},"onRemove":{"!type":"fn(eventName: ?, fn: ?, scope: ?) -> !this"},"onReplace":{"!type":"fn(eventName: ?, fn: ?, scope: ?) -> !this"},"onUpdate":{"!type":"fn(eventName: ?, fn: ?, scope: ?) -> !this"},"refresh":{"!type":"fn(eventName: ?, fn: ?, scope: ?) -> !this"},"refreshNode":{"!type":"fn() -> !this"},"relayFn":{"!type":"fn(name: ?, args: ?) -> !this"},"removeRowCls":{"!type":"fn() -> !this"},"scrollBy":{"!type":"fn() -> !this"}}}},"Panel":{"!doc":"<p>Grids are an excellent way of showing large amounts of tabular data on the client side. Essentially a supercharged\n<code>&lt;table&gt;</code>, GridPanel makes it easy to fetch, sort and filter large amounts of data.</p>\n\n<p>Grids are composed of two main pieces - a Store full of data and a set of columns to render.</p>\n\n<h2>Basic GridPanel</h2>\n\n<pre class='inline-example '><code>Ext.create('Ext.data.Store', {\n    storeId:'simpsonsStore',\n    fields:['name', 'email', 'phone'],\n    data:{'items':[\n        { 'name': 'Lisa',  \"email\":\"lisa@simpsons.com\",  \"phone\":\"555-111-1224\"  },\n        { 'name': 'Bart',  \"email\":\"bart@simpsons.com\",  \"phone\":\"555-222-1234\" },\n        { 'name': 'Homer', \"email\":\"homer@simpsons.com\",  \"phone\":\"555-222-1244\"  },\n        { 'name': 'Marge', \"email\":\"marge@simpsons.com\", \"phone\":\"555-222-1254\"  }\n    ]},\n    proxy: {\n        type: 'memory',\n        reader: {\n            type: 'json',\n            rootProperty: 'items'\n        }\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Simpsons',\n    store: Ext.data.StoreManager.lookup('simpsonsStore'),\n    columns: [\n        { text: 'Name',  dataIndex: 'name' },\n        { text: 'Email', dataIndex: 'email', flex: 1 },\n        { text: 'Phone', dataIndex: 'phone' }\n    ],\n    height: 200,\n    width: 400,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>The code above produces a simple grid with three columns. We specified a Store which will load JSON data inline.\nIn most apps we would be placing the grid inside another container and wouldn't need to use the\n<a href=\"#!/api/Ext.grid.Panel-cfg-height\" rel=\"Ext.grid.Panel-cfg-height\" class=\"docClass\">height</a>, <a href=\"#!/api/Ext.grid.Panel-cfg-width\" rel=\"Ext.grid.Panel-cfg-width\" class=\"docClass\">width</a> and <a href=\"#!/api/Ext.grid.Panel-cfg-renderTo\" rel=\"Ext.grid.Panel-cfg-renderTo\" class=\"docClass\">renderTo</a> configurations but they are included here to make it easy to get\nup and running.</p>\n\n<p>The grid we created above will contain a header bar with a title ('Simpsons'), a row of column headers directly underneath\nand finally the grid rows under the headers.</p>\n\n<h2>Configuring columns</h2>\n\n<p>By default, each column is sortable and will toggle between ASC and DESC sorting when you click on its header. Each\ncolumn header is also reorderable by default, and each gains a drop-down menu with options to hide and show columns.\nIt's easy to configure each column - here we use the same example as above and just modify the columns config:</p>\n\n<pre><code>columns: [\n    {\n        text: 'Name',\n        dataIndex: 'name',\n        sortable: false,\n        hideable: false,\n        flex: 1\n    },\n    {\n        text: 'Email',\n        dataIndex: 'email',\n        hidden: true\n    },\n    {\n        text: 'Phone',\n        dataIndex: 'phone',\n        width: 100\n    }\n]\n</code></pre>\n\n<p>We turned off sorting and hiding on the 'Name' column so clicking its header now has no effect. We also made the Email\ncolumn hidden by default (it can be shown again by using the menu on any other column). We also set the Phone column to\na fixed with of 100px and flexed the Name column, which means it takes up all remaining width after the other columns\nhave been accounted for. See the <a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">column docs</a> for more details.</p>\n\n<h2>Renderers</h2>\n\n<p>As well as customizing columns, it's easy to alter the rendering of individual cells using renderers. A renderer is\ntied to a particular column and is passed the value that would be rendered into each cell in that column. For example,\nwe could define a renderer function for the email column to turn each email address into a mailto link:</p>\n\n<pre><code>columns: [\n    {\n        text: 'Email',\n        dataIndex: 'email',\n        renderer: function(value) {\n            return Ext.String.format('&lt;a href=\"mailto:{0}\"&gt;{1}&lt;/a&gt;', value, value);\n        }\n    }\n]\n</code></pre>\n\n<p>See the <a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">column docs</a> for more information on renderers.</p>\n\n<h2>Selection Models</h2>\n\n<p>Sometimes all you want is to render data onto the screen for viewing, but usually it's necessary to interact with or\nupdate that data. Grids use a concept called a Selection Model, which is simply a mechanism for selecting some part of\nthe data in the grid. The two main types of Selection Model are RowSelectionModel, where entire rows are selected, and\nCellSelectionModel, where individual cells are selected.</p>\n\n<p>Grids use a Row Selection Model by default, but this is easy to customise like so:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    selType: 'cellmodel',\n    store: ...\n});\n</code></pre>\n\n<p>Specifying the <code>cellmodel</code> changes a couple of things. Firstly, clicking on a cell now\nselects just that cell (using a <a href=\"#!/api/Ext.selection.RowModel\" rel=\"Ext.selection.RowModel\" class=\"docClass\">rowmodel</a> will select the entire row), and secondly the\nkeyboard navigation will walk from cell to cell instead of row to row. Cell-based selection models are usually used in\nconjunction with editing.</p>\n\n<h2>Sorting &amp; Filtering</h2>\n\n<p>Every grid is attached to a Store, which provides multi-sort and filtering capabilities. It's\neasy to set up a grid to be sorted from the start:</p>\n\n<pre><code>var myGrid = Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    store: {\n        fields: ['name', 'email', 'phone'],\n        sorters: ['name', 'phone']\n    },\n    columns: [\n        { text: 'Name',  dataIndex: 'name' },\n        { text: 'Email', dataIndex: 'email' }\n    ]\n});\n</code></pre>\n\n<p>Sorting at run time is easily accomplished by simply clicking each column header. If you need to perform sorting on\nmore than one field at run time it's easy to do so by adding new sorters to the store:</p>\n\n<pre><code>myGrid.store.sort([\n    { property: 'name',  direction: 'ASC' },\n    { property: 'email', direction: 'DESC' }\n]);\n</code></pre>\n\n<p>See Ext.data.Store for examples of filtering.</p>\n\n<h2>State saving</h2>\n\n<p>When configured <a href=\"#!/api/Ext.grid.Panel-cfg-stateful\" rel=\"Ext.grid.Panel-cfg-stateful\" class=\"docClass\">stateful</a>, grids save their column state (order and width) encapsulated within the default\nPanel state of changed width and height and collapsed/expanded state.</p>\n\n<p>Each <a href=\"#!/api/Ext.grid.Panel-cfg-columns\" rel=\"Ext.grid.Panel-cfg-columns\" class=\"docClass\">column</a> of the grid may be configured with a <a href=\"#!/api/Ext.grid.column.Column-cfg-stateId\" rel=\"Ext.grid.column.Column-cfg-stateId\" class=\"docClass\">stateId</a> which\nidentifies that column locally within the grid.</p>\n\n<h2>Plugins and Features</h2>\n\n<p>Grid supports addition of extra functionality through features and plugins:</p>\n\n<ul>\n<li><p><a href=\"#!/api/Ext.grid.plugin.CellEditing\" rel=\"Ext.grid.plugin.CellEditing\" class=\"docClass\">CellEditing</a> - editing grid contents one cell at a time.</p></li>\n<li><p><a href=\"#!/api/Ext.grid.plugin.RowEditing\" rel=\"Ext.grid.plugin.RowEditing\" class=\"docClass\">RowEditing</a> - editing grid contents an entire row at a time.</p></li>\n<li><p><a href=\"#!/api/Ext.grid.plugin.DragDrop\" rel=\"Ext.grid.plugin.DragDrop\" class=\"docClass\">DragDrop</a> - drag-drop reordering of grid rows.</p></li>\n<li><p><a href=\"#!/api/Ext.toolbar.Paging\" rel=\"Ext.toolbar.Paging\" class=\"docClass\">Paging toolbar</a> - paging through large sets of data.</p></li>\n<li><p><a href=\"#!/api/Ext.grid.plugin.BufferedRenderer\" rel=\"Ext.grid.plugin.BufferedRenderer\" class=\"docClass\">Infinite scrolling</a> - another way to handle large sets of data.</p></li>\n<li><p><a href=\"#!/api/Ext.grid.column.RowNumberer\" rel=\"Ext.grid.column.RowNumberer\" class=\"docClass\">RowNumberer</a> - automatically numbered rows.</p></li>\n<li><p><a href=\"#!/api/Ext.grid.feature.Grouping\" rel=\"Ext.grid.feature.Grouping\" class=\"docClass\">Grouping</a> - grouping together rows having the same value in a particular field.</p></li>\n<li><p><a href=\"#!/api/Ext.grid.feature.Summary\" rel=\"Ext.grid.feature.Summary\" class=\"docClass\">Summary</a> - a summary row at the bottom of a grid.</p></li>\n<li><p><a href=\"#!/api/Ext.grid.feature.GroupingSummary\" rel=\"Ext.grid.feature.GroupingSummary\" class=\"docClass\">GroupingSummary</a> - a summary row at the bottom of each group.</p></li>\n</ul>\n\n","!type":"fn(config: ?)","prototype":{"columns":{"!doc":"<p>An array of <a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">column</a> definition objects which define all columns that appear in this\ngrid. Each column definition provides the header text for the column, and a definition of where the data for that\ncolumn comes from.</p>\n\n<p>This can also be a configuration object for a <a href=\"#!/api/Ext.grid.header.Container\" rel=\"Ext.grid.header.Container\" class=\"docClass\">HeaderContainer</a> which may override\ncertain default configurations if necessary. For example, the special layout may be overridden to use a simpler\nlayout, or one can set default values shared by all columns:</p>\n\n<pre><code>columns: {\n    items: [\n        {\n            text: \"Column A\",\n            dataIndex: \"field_A\"\n        },{\n            text: \"Column B\",\n            dataIndex: \"field_B\"\n        }, \n        ...\n    ],\n    defaults: {\n        flex: 1\n    }\n}\n</code></pre>\n"},"rowLines":{"!type":"bool","!doc":"<p>False to remove row line styling</p>\n"},"viewType":{"!type":"string","!doc":"<p>An xtype of view to use. This is automatically set to 'gridview' by <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Grid</a>\nand to 'treeview' by <a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">Tree</a>.</p>\n"},"lockable":{"!type":"bool"},"reconfigure":{"!type":"fn(this: ?, store: ?, columns: ?, oldStore: ?, oldColumns: ?, eOpts: ?)","!doc":"<p>Fires after a reconfigure.</p>\n"},"beforereconfigure":{"!type":"fn(this: ?, store: ?, columns: ?, oldStore: ?, oldColumns: ?, eOpts: ?)","!doc":"<p>Fires before a reconfigure to enable modification of incoming Store and columns.</p>\n"}}},"plugin":{"BufferedRenderer":{"!doc":"<p>Implements buffered rendering of a grid, allowing users to scroll\nthrough thousands of records without the performance penalties of\nrenderering all the records into the DOM at once.</p>\n\n<p>The number of rows rendered outside the visible area can be controlled by configuring the plugin.</p>\n\n<p>Users should not instantiate this class. It is instantiated automatically\nand applied to all grids.</p>\n\n<h2>Implementation notes</h2>\n\n<p>This class monitors scrolling of the <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">TableView</a> within a <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">GridPanel</a> to render a small section of\nthe dataset.</p>\n","!type":"fn(config?: ?)","prototype":{"leadingBufferZone":{"!type":"number","!doc":"<p>The number of extra rows to render on the leading side of scrolling\n<strong>outside the <a href=\"#!/api/Ext.grid.plugin.BufferedRenderer-cfg-numFromEdge\" rel=\"Ext.grid.plugin.BufferedRenderer-cfg-numFromEdge\" class=\"docClass\">numFromEdge</a></strong> buffer as scrolling proceeds.</p>\n"},"numFromEdge":{"!type":"number","!doc":"<p>The zone which causes new rows to be appended to the view. As soon as the edge\nof the rendered grid is this number of rows from the edge of the viewport, the view is moved.</p>\n"},"scrollToLoadBuffer":{"!type":"number","!doc":"<p>This is the time in milliseconds to buffer load requests when the store is a buffered store\nand a page required for rendering is not present in the store's cache and needs loading.</p>\n"},"synchronousRender":{"!type":"bool","!doc":"<p>By default, on detection of a scroll event which brings the end of the rendered table within\n<code><a href=\"#!/api/Ext.grid.plugin.BufferedRenderer-cfg-numFromEdge\" rel=\"Ext.grid.plugin.BufferedRenderer-cfg-numFromEdge\" class=\"docClass\">numFromEdge</a></code> rows of the grid viewport, if the required rows are available in the Store,\nthe BufferedRenderer will render rows from the Store <em>immediately</em> before returning from the event handler.\nThis setting helps avoid the impression of whitespace appearing during scrolling.</p>\n\n<p>Set this to <code>false</code> to defer the render until the scroll event handler exits. This allows for faster\nscrolling, but also allows whitespace to be more easily scrolled into view.</p>\n"},"trailingBufferZone":{"!type":"number","!doc":"<p>The number of extra rows to render on the trailing side of scrolling\n<strong>outside the <a href=\"#!/api/Ext.grid.plugin.BufferedRenderer-cfg-numFromEdge\" rel=\"Ext.grid.plugin.BufferedRenderer-cfg-numFromEdge\" class=\"docClass\">numFromEdge</a></strong> buffer as scrolling proceeds.</p>\n"},"bodyTop":{"!type":"number"},"isBufferedRenderer":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated BufferedRenderer, or subclass thereof.</p>\n"},"lastScrollDirection":{"!type":"number"},"lockableScope":{"!type":"string"},"position":{"!type":"number","!doc":"<p>Current pixel scroll position of the associated <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">View</a>.</p>\n"},"rowHeight":{"!type":"number","!doc":"<p>private. Start at default value</p>\n"},"scrollHeight":{"!type":"number"},"viewSize":{"!type":"number","!doc":"<p>private. Initial value of 100.</p>\n"},"attemptLoad":{"!type":"fn(start: ?, end: ?) -> !this"},"bindStore":{"!type":"fn(store: ?) -> !this"},"bottomOfViewCloseToEdge":{"!type":"fn() -> !this"},"cancelLoad":{"!type":"fn() -> !this"},"checkVariableRowHeight":{"!type":"fn() -> !this","!doc":"<p>Keep the variableRowHeight and any Lockable's syncRowHeight property correct WRT variable row heights being possible.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>The destroy method is invoked by the owning Component at the time the Component is being destroyed.</p>\n\n<p>The supplied implementation is empty. Subclasses should perform plugin cleanup in their own implementation of\nthis method.</p>\n"},"doAttemptLoad":{"!type":"fn(start: ?, end: ?) -> !this"},"doRefreshView":{"!type":"fn(range: ?, startIndex: ?, endIndex: ?, options: ?) -> !this"},"getFirstVisibleRowIndex":{"!type":"fn(startRow: ?, endRow: ?, viewportTop: ?, viewportBottom: ?) -> !this"},"getLastVisibleRowIndex":{"!type":"fn(startRow: ?, endRow: ?, viewportTop: ?, viewportBottom: ?) -> !this"},"getScrollHeight":{"!type":"fn(calculatedOnly: ?) -> !this"},"getViewRange":{"!type":"fn() -> !this","!doc":"<p>TableView's getViewRange delegates the operation to this method if buffered rendering is present.</p>\n"},"handleViewScroll":{"!type":"fn(direction: ?) -> !this"},"init":{"!type":"fn(grid: ?) -> !this","!doc":"<p>Initialize this as a plugin</p>\n"},"onRangeFetched":{"!type":"fn(range: ?, start: ?, end: ?, options: ?, fromLockingPartner: ?) -> !this"},"onReconfigure":{"!type":"fn(grid: ?, store: ?) -> !this"},"onReplace":{"!type":"fn(store: ?, startIndex: ?, oldRecords: ?, newRecords: ?) -> !this","!doc":"<p>Handles the Store replace event, producing a correct buffered view after the replace operation.</p>\n"},"onStoreClear":{"!type":"fn() -> !this"},"onStoreGroupChange":{"!type":"fn(store: ?) -> !this","!doc":"<p>If the store is not grouped, we can switch to fixed row height mode</p>\n"},"onViewRefresh":{"!type":"fn() -> !this"},"onViewResize":{"!type":"fn(view: ?, width: ?, height: ?, oldWidth: ?, oldHeight: ?) -> !this"},"onViewScroll":{"!type":"fn() -> !this"},"refreshView":{"!type":"fn(startIndex: ?) -> !this","!doc":"<p>Refreshes the current rendered range if possible.\nOptinally refreshes starting at the specified index.</p>\n"},"renderRange":{"!type":"fn(start: ?, end: ?, forceSynchronous: ?) -> !this"},"scrollTo":{"!type":"fn(recordIdx: ?, doSelect: ?, callback: ?, scope: ?) -> !this","!doc":"<p>Scrolls to and optionlly selects the specified row index <strong>in the total dataset</strong>.</p>\n"},"setBodyTop":{"!type":"fn(bodyTop: ?) -> !this"},"setViewSize":{"!type":"fn(viewSize: ?) -> !this"},"stretchView":{"!type":"fn(view: ?, scrollRange: ?) -> !this"},"topOfViewCloseToEdge":{"!type":"fn() -> !this"},"unbindStore":{"!type":"fn() -> !this"}}},"CellEditing":{"!doc":"<p>The <a href=\"#!/api/Ext.grid.plugin.CellEditing\" rel=\"Ext.grid.plugin.CellEditing\" class=\"docClass\">Ext.grid.plugin.CellEditing</a> plugin injects editing at a cell level for a Grid. Only a single\ncell will be editable at a time. The field that will be used for the editor is defined at the\n<a href=\"#!/api/Ext.grid.column.Column-cfg-editor\" rel=\"Ext.grid.column.Column-cfg-editor\" class=\"docClass\">editor</a>. The editor can be a field instance or a field configuration.</p>\n\n<p>If an editor is not specified for a particular column then that cell will not be editable and it will\nbe skipped when activated via the mouse or the keyboard.</p>\n\n<p>The editor may be shared for each column in the grid, or a different one may be specified for each column.\nAn appropriate field type should be chosen to match the data structure that it will be editing. For example,\nto edit a date, it would be useful to specify <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a> as the editor.</p>\n\n<h2>Example</h2>\n\n<p>A grid with editor for the name and the email columns:</p>\n\n<pre class='inline-example '><code>Ext.create('Ext.data.Store', {\n    storeId:'simpsonsStore',\n    fields:['name', 'email', 'phone'],\n    data:{'items':[\n        {\"name\":\"Lisa\", \"email\":\"lisa@simpsons.com\", \"phone\":\"555-111-1224\"},\n        {\"name\":\"Bart\", \"email\":\"bart@simpsons.com\", \"phone\":\"555-222-1234\"},\n        {\"name\":\"Homer\", \"email\":\"homer@simpsons.com\", \"phone\":\"555-222-1244\"},\n        {\"name\":\"Marge\", \"email\":\"marge@simpsons.com\", \"phone\":\"555-222-1254\"}\n    ]},\n    proxy: {\n        type: 'memory',\n        reader: {\n            type: 'json',\n            rootProperty: 'items'\n        }\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Simpsons',\n    store: Ext.data.StoreManager.lookup('simpsonsStore'),\n    columns: [\n        {header: 'Name',  dataIndex: 'name', editor: 'textfield'},\n        {header: 'Email', dataIndex: 'email', flex:1,\n            editor: {\n                xtype: 'textfield',\n                allowBlank: false\n            }\n        },\n        {header: 'Phone', dataIndex: 'phone'}\n    ],\n    selType: 'cellmodel',\n    plugins: [\n        Ext.create('<a href=\"#!/api/Ext.grid.plugin.CellEditing\" rel=\"Ext.grid.plugin.CellEditing\" class=\"docClass\">Ext.grid.plugin.CellEditing</a>', {\n            clicksToEdit: 1\n        })\n    ],\n    height: 200,\n    width: 400,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>This requires a little explanation. We're passing in <code>store</code> and <code>columns</code> as normal, but\nwe also specify a <a href=\"#!/api/Ext.grid.column.Column-cfg-field\" rel=\"Ext.grid.column.Column-cfg-field\" class=\"docClass\">field</a> on two of our columns. For the\nName column we just want a default textfield to edit the value, so we specify 'textfield'.\nFor the Email column we customized the editor slightly by passing allowBlank: false, which\nwill provide inline validation.</p>\n\n<p>To support cell editing, we also specified that the grid should use the 'cellmodel'\n<a href=\"#!/api/Ext.grid.Panel-cfg-selType\" rel=\"Ext.grid.Panel-cfg-selType\" class=\"docClass\">selType</a>, and created an instance of the CellEditing plugin,\nwhich we configured to activate each editor after a single click.</p>\n","!type":"fn(config?: ?)","prototype":{"lockableScope":{"!type":"string"},"beforeGridHeaderDestroy":{"!type":"fn(headerCt: ?) -> !this","!doc":"<p>Ensure editors are cleaned up.</p>\n"},"cancelEdit":{"!type":"fn() -> !this","!doc":"<p>Cancels any active editing.</p>\n"},"completeEdit":{"!type":"fn(remainVisible: ?) -> !this","!doc":"<p>Completes the edit if there is an active edit in progress.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Component calls destroy on all its plugins at destroy time.</p>\n"},"getActiveColumn":{"!type":"fn() -> !this"},"getActiveEditor":{"!type":"fn() -> !this"},"getActiveRecord":{"!type":"fn() -> !this"},"getCell":{"!type":"fn(record: ?, column: ?) -> !this","!doc":"<p>Gets the cell (td) for a particular record and column.</p>\n"},"getEditor":{"!type":"fn(record: ?, column: ?) -> !this"},"initCancelTriggers":{"!type":"fn() -> !this","!doc":"<p>Template method called from base class's initEvents</p>\n"},"isCellEditable":{"!type":"fn(record: ?, columnHeader: ?) -> !this"},"onColumnRemoved":{"!type":"fn(column: ?) -> !this"},"onEditComplete":{"!type":"fn(ed: ?, value: ?, startValue: ?) -> !this"},"onReconfigure":{"!type":"fn(grid: ?, store: ?, columns: ?) -> !this","!doc":"<p>Fires after the grid is reconfigured</p>\n"},"onSpecialKey":{"!type":"fn(ed: ?, field: ?, e: ?) -> !this"},"setActiveColumn":{"!type":"fn(column: ?) -> !this"},"setActiveEditor":{"!type":"fn(ed: ?) -> !this"},"setActiveRecord":{"!type":"fn(record: ?) -> !this"},"setColumnField":{"!type":"fn(column: ?, field: ?) -> !this","!doc":"<p>inherit docs</p>\n"},"setEditingContext":{"!type":"fn(context: ?) -> !this","!doc":"<p>internal getters/setters</p>\n"},"showEditor":{"!type":"fn(ed: ?, context: ?, value: ?) -> !this"},"startEdit":{"!type":"fn(record: ?, columnHeader: ?) -> bool","!doc":"<p>Starts editing the specified record, using the specified Column definition to define which field is being edited.</p>\n"},"startEditByPosition":{"!type":"fn(position: ?) -> !this","!doc":"<p>Starts editing by position (row/column)</p>\n"},"beforeedit":{"!type":"fn(editor: ?, context: ?, eOpts: ?)","!doc":"<p>Fires before cell editing is triggered. Return false from event handler to stop the editing.</p>\n"},"canceledit":{"!type":"fn(editor: ?, context: ?, eOpts: ?)","!doc":"<p>Fires when the user started editing a cell but then cancelled the edit.</p>\n"},"edit":{"!type":"fn(editor: ?, context: ?, eOpts: ?)","!doc":"<p>Fires after a cell is edited. Usage example:</p>\n\n<pre><code>grid.on('edit', function(editor, e) {\n    // commit the changes right after editing finished\n    e.record.commit();\n});\n</code></pre>\n"},"validateedit":{"!type":"fn(editor: ?, context: ?, eOpts: ?)","!doc":"<p>Fires after a cell is edited, but before the value is set in the record. Return false from event handler to\ncancel the change.</p>\n\n<p>Usage example showing how to remove the red triangle (dirty record indicator) from some records (not all). By\nobserving the grid's validateedit event, it can be cancelled if the edit occurs on a targeted row (for\nexample) and then setting the field's new value in the Record directly:</p>\n\n<pre><code>grid.on('validateedit', function(editor, e) {\n  var myTargetRow = 6;\n\n  if (e.row == myTargetRow) {\n    e.cancel = true;\n    e.record.data[e.field] = e.value;\n  }\n});\n</code></pre>\n"}}},"Editing":{"prototype":{"clicksToEdit":{"!type":"number","!doc":"<p>The number of clicks on a grid required to display the editor.\nThe only accepted values are <strong>1</strong> and <strong>2</strong>.</p>\n"},"default":{"!type":"string","!doc":"<p>UI for editor fields</p>\n"},"triggerEvent":{"!type":"string","!doc":"<p>The event which triggers editing. Supercedes the <a href=\"#!/api/Ext.grid.plugin.Editing-cfg-clicksToEdit\" rel=\"Ext.grid.plugin.Editing-cfg-clicksToEdit\" class=\"docClass\">clicksToEdit</a> configuration. Maybe one of:</p>\n\n<ul>\n<li>cellclick</li>\n<li>celldblclick</li>\n<li>cellfocus</li>\n<li>rowfocus</li>\n</ul>\n\n"},"defaultFieldXType":{"!type":"string"},"editStyle":{"!type":"string","!doc":"<p>cell, row, form</p>\n"},"editing":{"!type":"bool","!doc":"<p>Set to <code>true</code> while the editing plugin is active and an Editor is visible.</p>\n"},"beforeEdit":{"!type":"fn(context: ?) -> bool","!doc":"<p>Template method called before editing begins.</p>\n"},"beforeViewCellFocus":{"!type":"fn(position: ?) -> !this","!doc":"<p>Override of View's method so that we can pre-empt the View's processing if the view is being triggered by a mousedown</p>\n"},"createColumnField":{"!type":"fn(columnHeader: ?, defaultField: ?) -> !this"},"getColumnField":{"!type":"fn(columnHeader: ?, defaultField: ?) -> !this","!doc":"<p>remaps to the public API of <a href=\"#!/api/Ext.grid.column.Column-method-getEditor\" rel=\"Ext.grid.column.Column-method-getEditor\" class=\"docClass\">Ext.grid.column.Column.getEditor</a></p>\n"},"getEditStyle":{"!type":"fn() -> !this"},"getEditingContext":{"!type":"fn(record: ?, columnHeader: ?) -> ?","!doc":"<p>Collects all information necessary for any subclasses to perform their editing functions.</p>\n"},"hasColumnField":{"!type":"fn(columnHeader: ?) -> !this","!doc":"<p>remaps to the public API of Ext.grid.column.Column.hasEditor</p>\n"},"init":{"!type":"fn(grid: ?) -> !this","!doc":"<p>The init method is invoked after initComponent method has been run for the client Component.</p>\n\n<p>The supplied implementation is empty. Subclasses should perform plugin initialization, and set up bidirectional\nlinks between the plugin and its client Component in their own implementation of this method.</p>\n"},"initAddRemoveHeaderEvents":{"!type":"fn() -> !this"},"initEditTriggers":{"!type":"fn() -> !this"},"initEvents":{"!type":"fn() -> !this"},"initFieldAccessors":{"!type":"fn(columns: ?) -> !this"},"initKeyNavHeaderEvents":{"!type":"fn() -> !this"},"onBeforeReconfigure":{"!type":"fn() -> !this"},"onCellClick":{"!type":"fn(view: ?, cell: ?, colIdx: ?, record: ?, row: ?, rowIdx: ?, e: ?) -> !this","!doc":"<p>Used if we are triggered by a cellclick event\nIMPORTANT* Due to V4.0.0 history, the colIdx here is the index within ALL columns, including hidden.</p>\n"},"onCellFocus":{"!type":"fn(record: ?, cell: ?, position: ?) -> !this","!doc":"<p>Used if we are triggered by the cellfocus event</p>\n"},"onColumnAdd":{"!type":"fn(ct: ?, column: ?) -> !this"},"onColumnMove":{"!type":"fn() -> !this","!doc":"<p>Template method which may be implemented in subclasses (RowEditing and CellEditing)</p>\n"},"onEnterKey":{"!type":"fn(e: ?) -> !this"},"onEscKey":{"!type":"fn(e: ?) -> !this"},"onRowFocus":{"!type":"fn(record: ?, row: ?, rowIdx: ?) -> !this","!doc":"<p>Used if we are triggered by the rowfocus event</p>\n"},"removeFieldAccessors":{"!type":"fn(columns: ?) -> !this"},"resolveListenerScope":{"!type":"fn(defaultScope: ?) -> !this"},"shouldStartEdit":{"!type":"fn(editor: ?) -> !this"},"validateEdit":{"!type":"fn() -> !this"},"cancelEdit":{"!type":"fn() -> !this","!doc":"<p>Cancels any active edit that is in progress.</p>\n"},"completeEdit":{"!type":"fn() -> !this","!doc":"<p>Completes the edit if there is an active edit in progress.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Component calls destroy on all its plugins at destroy time.</p>\n"},"initCancelTriggers":{"!type":"fn() -> !this"},"onReconfigure":{"!type":"fn() -> !this","!doc":"<p>Fires after the grid is reconfigured</p>\n"},"setColumnField":{"!type":"fn(columnHeader: ?, field: ?) -> !this","!doc":"<p>remaps to the public API of <a href=\"#!/api/Ext.grid.column.Column-method-setEditor\" rel=\"Ext.grid.column.Column-method-setEditor\" class=\"docClass\">Ext.grid.column.Column.setEditor</a></p>\n"},"startEdit":{"!type":"fn(record: ?, columnHeader: ?) -> !this","!doc":"<p>Starts editing the specified record, using the specified Column definition to define which field is being edited.</p>\n"},"beforeedit":{"!type":"fn(editor: ?, context: ?, eOpts: ?)","!doc":"<p>Fires before editing is triggered. Return false from event handler to stop the editing.</p>\n"},"canceledit":{"!type":"fn(editor: ?, context: ?, eOpts: ?)","!doc":"<p>Fires when the user started editing but then cancelled the edit.</p>\n"},"edit":{"!type":"fn(editor: ?, context: ?, eOpts: ?)","!doc":"<p>Fires after editing. Usage example:</p>\n\n<pre><code>grid.on('edit', function(editor, e) {\n    // commit the changes right after editing finished\n    e.record.commit();\n});\n</code></pre>\n"},"validateedit":{"!type":"fn(editor: ?, context: ?, eOpts: ?)","!doc":"<p>Fires after editing, but before the value is set in the record. Return false from event handler to\ncancel the change.</p>\n\n<p>Usage example showing how to remove the red triangle (dirty record indicator) from some records (not all). By\nobserving the grid's validateedit event, it can be cancelled if the edit occurs on a targeted row (for example)\nand then setting the field's new value in the Record directly:</p>\n\n<pre><code>grid.on('validateedit', function (editor, context) {\n    var myTargetRow = 6;\n\n    if (context.rowIdx === myTargetRow) {\n        context.record.data[context.field] = context.value;\n    }\n});\n</code></pre>\n"}},"!doc":"<p>This class provides an abstract grid editing plugin on selected <a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">columns</a>.\nThe editable columns are specified by providing an <a href=\"#!/api/Ext.grid.column.Column-cfg-editor\" rel=\"Ext.grid.column.Column-cfg-editor\" class=\"docClass\">editor</a>\nin the <a href=\"#!/api/Ext.grid.column.Column\" rel=\"Ext.grid.column.Column\" class=\"docClass\">column configuration</a>.</p>\n\n<p><strong>Note:</strong> This class should not be used directly. See <a href=\"#!/api/Ext.grid.plugin.CellEditing\" rel=\"Ext.grid.plugin.CellEditing\" class=\"docClass\">Ext.grid.plugin.CellEditing</a> and\n<a href=\"#!/api/Ext.grid.plugin.RowEditing\" rel=\"Ext.grid.plugin.RowEditing\" class=\"docClass\">Ext.grid.plugin.RowEditing</a>.</p>\n","!type":"fn(config?: ?)"},"DragDrop":{"!doc":"<p>This plugin provides drag and/or drop functionality for a <a href=\"#!/api/Ext.grid.View\" rel=\"Ext.grid.View\" class=\"docClass\">GridView</a>.</p>\n\n<p>It creates a specialized instance of <a href=\"#!/api/Ext.dd.DragZone\" rel=\"Ext.dd.DragZone\" class=\"docClass\">DragZone</a> which knows how to drag out of a <a href=\"#!/api/Ext.grid.View\" rel=\"Ext.grid.View\" class=\"docClass\">GridView</a> and loads the data object which is passed to a cooperating <a href=\"#!/api/Ext.dd.DragZone\" rel=\"Ext.dd.DragZone\" class=\"docClass\">DragZone</a>'s\nmethods with the following properties:</p>\n\n<ul>\n<li><p><code>copy</code> : Boolean</p>\n\n<p>The value of the <a href=\"#!/api/Ext.grid.View\" rel=\"Ext.grid.View\" class=\"docClass\">GridView</a>'s <code>copy</code> property, or <code>true</code> if the GridView was configured with <code>allowCopy: true</code> <em>and</em>\nthe control key was pressed when the drag operation was begun.</p></li>\n<li><p><code>view</code> : <a href=\"#!/api/Ext.grid.View\" rel=\"Ext.grid.View\" class=\"docClass\">GridView</a></p>\n\n<p>The source GridView from which the drag originated.</p></li>\n<li><p><code>ddel</code> : HTMLElement</p>\n\n<p>The drag proxy element which moves with the mouse</p></li>\n<li><p><code>item</code> : HTMLElement</p>\n\n<p>The GridView node upon which the mousedown event was registered.</p></li>\n<li><p><code>records</code> : Array</p>\n\n<p>An Array of Models representing the selected data being dragged from the source <a href=\"#!/api/Ext.grid.View\" rel=\"Ext.grid.View\" class=\"docClass\">GridView</a>.</p></li>\n</ul>\n\n\n<p>It also creates a specialized instance of <a href=\"#!/api/Ext.dd.DropZone\" rel=\"Ext.dd.DropZone\" class=\"docClass\">Ext.dd.DropZone</a> which cooperates with other DropZones which are\nmembers of the same ddGroup which processes such data objects.</p>\n\n<p>Adding this plugin to a view means that two new events may be fired from the client GridView, <code><a href=\"#!/api/Ext.grid.plugin.DragDrop-event-beforedrop\" rel=\"Ext.grid.plugin.DragDrop-event-beforedrop\" class=\"docClass\">beforedrop</a></code> and <code><a href=\"#!/api/Ext.grid.plugin.DragDrop-event-drop\" rel=\"Ext.grid.plugin.DragDrop-event-drop\" class=\"docClass\">drop</a></code></p>\n\n<pre class='inline-example '><code>Ext.create('Ext.data.Store', {\n    storeId:'simpsonsStore',\n    fields:['name'],\n    data: [[\"Lisa\"], [\"Bart\"], [\"Homer\"], [\"Marge\"]],\n    proxy: {\n        type: 'memory',\n        reader: 'array'\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    store: 'simpsonsStore',\n    columns: [\n        {header: 'Name',  dataIndex: 'name', flex: true}\n    ],\n    viewConfig: {\n        plugins: {\n            ptype: 'gridviewdragdrop',\n            dragText: 'Drag and drop to reorganize'\n        }\n    },\n    height: 200,\n    width: 400,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config?: ?)","prototype":{"containerScroll":{"!doc":"<p><code>true</code> to register this container with the Scrollmanager for auto scrolling during drag operations.\nA <a href=\"#!/api/Ext.dd.ScrollManager\" rel=\"Ext.dd.ScrollManager\" class=\"docClass\">Ext.dd.ScrollManager</a> configuration may also be passed.</p>\n"},"ddGroup":{"!type":"string","!doc":"<p>A named drag drop group to which this object belongs. If a group is specified, then both the DragZones and\nDropZone used by this plugin will only interact with other drag drop objects in the same group.</p>\n"},"dragGroup":{"!type":"string","!doc":"<p>The <a href=\"#!/api/Ext.grid.plugin.DragDrop-cfg-ddGroup\" rel=\"Ext.grid.plugin.DragDrop-cfg-ddGroup\" class=\"docClass\">ddGroup</a> to which the DragZone will belong.</p>\n\n<p>This defines which other DropZones the DragZone will interact with. Drag/DropZones only interact with other\nDrag/DropZones which are members of the same <a href=\"#!/api/Ext.grid.plugin.DragDrop-cfg-ddGroup\" rel=\"Ext.grid.plugin.DragDrop-cfg-ddGroup\" class=\"docClass\">ddGroup</a>.</p>\n"},"dragText":{"!type":"string","!doc":"<p>The text to show while dragging.</p>\n\n<p>Two placeholders can be used in the text:</p>\n\n<ul>\n<li><code>{0}</code> The number of selected items.</li>\n<li><code>{1}</code> 's' when more than 1 items (only useful for English).</li>\n</ul>\n\n"},"dragZone":{"!doc":"<p>An <a href=\"#!/api/Ext.view.DragZone\" rel=\"Ext.view.DragZone\" class=\"docClass\">DragZone</a> which handles mousedown and dragging of records from the grid.</p>\n"},"dropGroup":{"!type":"string","!doc":"<p>The <a href=\"#!/api/Ext.grid.plugin.DragDrop-cfg-ddGroup\" rel=\"Ext.grid.plugin.DragDrop-cfg-ddGroup\" class=\"docClass\">ddGroup</a> to which the DropZone will belong.</p>\n\n<p>This defines which other DragZones the DropZone will interact with. Drag/DropZones only interact with other\nDrag/DropZones which are members of the same <a href=\"#!/api/Ext.grid.plugin.DragDrop-cfg-ddGroup\" rel=\"Ext.grid.plugin.DragDrop-cfg-ddGroup\" class=\"docClass\">ddGroup</a>.</p>\n"},"dropZone":{"!doc":"<p>An <a href=\"#!/api/Ext.grid.ViewDropZone\" rel=\"Ext.grid.ViewDropZone\" class=\"docClass\">DropZone</a> which handles mouseover and dropping records in any grid which shares the same <a href=\"#!/api/Ext.grid.plugin.DragDrop-cfg-dropGroup\" rel=\"Ext.grid.plugin.DragDrop-cfg-dropGroup\" class=\"docClass\">dropGroup</a>.</p>\n"},"enableDrag":{"!type":"bool","!doc":"<p><code>false</code> to disallow dragging items from the View.</p>\n"},"enableDrop":{"!type":"bool","!doc":"<p><code>false</code> to disallow the View from accepting drop gestures.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Component calls destroy on all its plugins at destroy time.</p>\n"},"disable":{"!type":"fn() -> !this","!doc":"<p>The base implementation just sets the plugin's <code>disabled</code> flag to <code>true</code></p>\n\n<p>Plugin subclasses which need more complex processing may implement an overriding implementation.</p>\n"},"enable":{"!type":"fn() -> !this","!doc":"<p>The base implementation just sets the plugin's <code>disabled</code> flag to <code>false</code></p>\n\n<p>Plugin subclasses which need more complex processing may implement an overriding implementation.</p>\n"},"onViewRender":{"!type":"fn(view: ?) -> !this"},"beforedrop":{"!type":"fn(node: ?, data: ?, overModel: ?, dropPosition: ?, dropHandlers: ?, eOpts: ?)","!doc":"<p><strong>This event is fired through the GridView. Add listeners to the <a href=\"#!/api/Ext.grid.View\" rel=\"Ext.grid.View\" class=\"docClass\">GridView</a> object</strong></p>\n\n<p>Fired when a drop gesture has been triggered by a mouseup event in a valid drop position in the GridView.</p>\n\n<p>Returning <code>false</code> to this event signals that the drop gesture was invalid, and if the drag proxy will animate\nback to the point from which the drag began.</p>\n\n<p>The dropHandlers parameter can be used to defer the processing of this event. For example to wait for the result of\na message box confirmation or an asynchronous server call. See the details of this property for more information.</p>\n\n<pre class='inline-example '><code>view.on('beforedrop', function(node, data, overModel, dropPosition, dropHandlers) {\n    // Defer the handling\n    dropHandlers.wait = true;\n    <a href=\"#!/api/Ext.MessageBox-method-confirm\" rel=\"Ext.MessageBox-method-confirm\" class=\"docClass\">Ext.MessageBox.confirm</a>('Drop', 'Are you sure', function(btn){\n        if (btn === 'yes') {\n            dropHandlers.processDrop();\n        } else {\n            dropHandlers.cancelDrop();\n        }\n    });\n});\n</code></pre>\n\n<p>Any other return value continues with the data transfer operation, unless the wait property is set.</p>\n"},"drop":{"!type":"fn(node: ?, data: ?, overModel: ?, dropPosition: ?, eOpts: ?)","!doc":"<p><strong>This event is fired through the GridView. Add listeners to the GridView object</strong> Fired when a drop operation\nhas been completed and the data has been moved or copied.</p>\n"}}},"HeaderReorderer":{"!type":"fn(config?: ?)","prototype":{"destroy":{"!type":"fn() -> !this","!doc":"<p>Component calls destroy on all its plugins at destroy time.</p>\n"},"disable":{"!type":"fn() -> !this","!doc":"<p>The base implementation just sets the plugin's <code>disabled</code> flag to <code>true</code></p>\n\n<p>Plugin subclasses which need more complex processing may implement an overriding implementation.</p>\n"},"enable":{"!type":"fn() -> !this","!doc":"<p>The base implementation just sets the plugin's <code>disabled</code> flag to <code>false</code></p>\n\n<p>Plugin subclasses which need more complex processing may implement an overriding implementation.</p>\n"},"init":{"!type":"fn(headerCt: ?) -> !this","!doc":"<p>The init method is invoked after initComponent method has been run for the client Component.</p>\n\n<p>The supplied implementation is empty. Subclasses should perform plugin initialization, and set up bidirectional\nlinks between the plugin and its client Component in their own implementation of this method.</p>\n"},"onHeaderCtRender":{"!type":"fn() -> !this"}}},"HeaderResizer":{"!doc":"<p>Plugin to add header resizing functionality to a HeaderContainer.\nAlways resizing header to the left of the splitter you are resizing.</p>\n","!type":"fn(config?: ?)","prototype":{"dynamic":{"!type":"bool","!doc":"<p>True to resize on the fly rather than using a proxy marker.</p>\n"},"colHeaderCls":{"!type":"string"},"disabled":{"!type":"bool"},"eResizeCursor":{"!type":"string"},"maxColWidth":{"!type":"number"},"minColWidth":{"!type":"number"},"adjustColumnWidth":{"!type":"fn(offsetX: ?) -> !this"},"adjustConstrainRegion":{"!type":"fn(region: ?, t: ?, r: ?, b: ?, l: ?) -> !this"},"afterHeaderRender":{"!type":"fn() -> !this"},"calculateDragX":{"!type":"fn(markerOwner: ?) -> !this"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Component calls destroy on all its plugins at destroy time.</p>\n"},"disable":{"!type":"fn() -> !this","!doc":"<p>The base implementation just sets the plugin's <code>disabled</code> flag to <code>true</code></p>\n\n<p>Plugin subclasses which need more complex processing may implement an overriding implementation.</p>\n"},"doResize":{"!type":"fn() -> !this"},"enable":{"!type":"fn() -> !this","!doc":"<p>The base implementation just sets the plugin's <code>disabled</code> flag to <code>false</code></p>\n\n<p>Plugin subclasses which need more complex processing may implement an overriding implementation.</p>\n"},"findActiveHeader":{"!type":"fn(e: ?) -> !this"},"getConstrainRegion":{"!type":"fn() -> !this","!doc":"<p>get the region to constrain to, takes into account max and min col widths</p>\n"},"getDynamic":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.grid.plugin.HeaderResizer-cfg-dynamic\" rel=\"Ext.grid.plugin.HeaderResizer-cfg-dynamic\" class=\"docClass\">dynamic</a>.</p>\n"},"getLeftMarkerX":{"!type":"fn(markerOwner: ?) -> !this"},"getMovingMarker":{"!type":"fn(markerOwner: ?) -> !this"},"headerInSameGrid":{"!type":"fn(header: ?) -> !this","!doc":"<p>nextNode can traverse out of this grid, possibly to others on the page, so limit it here</p>\n"},"init":{"!type":"fn(headerCt: ?) -> !this","!doc":"<p>The init method is invoked after initComponent method has been run for the client Component.</p>\n\n<p>The supplied implementation is empty. Subclasses should perform plugin initialization, and set up bidirectional\nlinks between the plugin and its client Component in their own implementation of this method.</p>\n"},"onBeforeStart":{"!type":"fn(e: ?) -> !this","!doc":"<p>only start when there is an activeHd</p>\n"},"onDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>synchronize the rhsMarker with the mouse movement</p>\n"},"onEnd":{"!type":"fn(e: ?) -> !this"},"onHeaderCtMouseMove":{"!type":"fn(e: ?) -> !this","!doc":"<p>As we mouse over individual headers, change the cursor to indicate\nthat resizing is available, and cache the resize target header for use\nif/when they mousedown.</p>\n"},"onStart":{"!type":"fn(e: ?) -> !this","!doc":"<p>initialize the left and right hand side markers around\nthe header that we are resizing</p>\n"},"setDynamic":{"!type":"fn(dynamic: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.grid.plugin.HeaderResizer-cfg-dynamic\" rel=\"Ext.grid.plugin.HeaderResizer-cfg-dynamic\" class=\"docClass\">dynamic</a>.</p>\n"},"setMarkerX":{"!type":"fn(marker: ?, x: ?) -> !this"}}},"RowEditing":{"!doc":"<p>The <a href=\"#!/api/Ext.grid.plugin.RowEditing\" rel=\"Ext.grid.plugin.RowEditing\" class=\"docClass\">Ext.grid.plugin.RowEditing</a> plugin injects editing at a row level for a Grid. When editing begins,\na small floating dialog will be shown for the appropriate row. Each editable column will show a field\nfor editing. There is a button to save or cancel all changes for the edit.</p>\n\n<p>The field that will be used for the editor is defined at the\n<a href=\"#!/api/Ext.grid.column.Column-cfg-editor\" rel=\"Ext.grid.column.Column-cfg-editor\" class=\"docClass\">editor</a>. The editor can be a field instance or a field configuration.\nIf an editor is not specified for a particular column then that column won't be editable and the value of\nthe column will be displayed. To provide a custom renderer for non-editable values, use the\n<a href=\"#!/api/Ext.grid.column.Column-cfg-editRenderer\" rel=\"Ext.grid.column.Column-cfg-editRenderer\" class=\"docClass\">editRenderer</a> configuration on the column.</p>\n\n<p>The editor may be shared for each column in the grid, or a different one may be specified for each column.\nAn appropriate field type should be chosen to match the data structure that it will be editing. For example,\nto edit a date, it would be useful to specify <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a> as the editor.</p>\n\n<pre class='inline-example '><code>Ext.create('Ext.data.Store', {\n    storeId:'simpsonsStore',\n    fields:['name', 'email', 'phone'],\n    data: [\n        {\"name\":\"Lisa\", \"email\":\"lisa@simpsons.com\", \"phone\":\"555-111-1224\"},\n        {\"name\":\"Bart\", \"email\":\"bart@simpsons.com\", \"phone\":\"555-222-1234\"},\n        {\"name\":\"Homer\", \"email\":\"homer@simpsons.com\", \"phone\":\"555-222-1244\"},\n        {\"name\":\"Marge\", \"email\":\"marge@simpsons.com\", \"phone\":\"555-222-1254\"}\n    ]\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Simpsons',\n    store: Ext.data.StoreManager.lookup('simpsonsStore'),\n    columns: [\n        {header: 'Name',  dataIndex: 'name', editor: 'textfield'},\n        {header: 'Email', dataIndex: 'email', flex:1,\n            editor: {\n                xtype: 'textfield',\n                allowBlank: false\n            }\n        },\n        {header: 'Phone', dataIndex: 'phone'}\n    ],\n    selType: 'rowmodel',\n    plugins: [\n        Ext.create('<a href=\"#!/api/Ext.grid.plugin.RowEditing\" rel=\"Ext.grid.plugin.RowEditing\" class=\"docClass\">Ext.grid.plugin.RowEditing</a>', {\n            clicksToEdit: 1\n        })\n    ],\n    height: 200,\n    width: 400,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn()","prototype":{"autoCancel":{"!type":"bool","!doc":"<p><code>true</code> to automatically cancel any pending changes when the row editor begins editing a new row.\n<code>false</code> to force the user to explicitly cancel the pending changes.</p>\n"},"clicksToMoveEditor":{"!type":"number","!doc":"<p>The number of clicks to move the row editor to a new row while it is visible and actively editing another row.\nThis will default to the same value as <a href=\"#!/api/Ext.grid.plugin.Editing-cfg-clicksToEdit\" rel=\"Ext.grid.plugin.Editing-cfg-clicksToEdit\" class=\"docClass\">clicksToEdit</a>.</p>\n"},"errorSummary":{"!type":"bool","!doc":"<p>True to show a <a href=\"#!/api/Ext.tip.ToolTip\" rel=\"Ext.tip.ToolTip\" class=\"docClass\">tooltip</a> that summarizes all validation errors present\nin the row editor. Set to false to prevent the tooltip from showing.</p>\n"},"editStyle":{"!type":"string","!doc":"<p>cell, row, form</p>\n"},"lockableScope":{"!type":"string"},"beforeGridHeaderDestroy":{"!type":"fn(headerCt: ?) -> !this","!doc":"<p>Ensure editors are cleaned up.</p>\n"},"cancelEdit":{"!type":"fn() -> !this","!doc":"<p>Cancels any active edit that is in progress.</p>\n"},"completeEdit":{"!type":"fn() -> !this","!doc":"<p>Completes the edit if there is an active edit in progress.</p>\n"},"createColumnField":{"!type":"fn(column: ?, defaultField: ?) -> !this"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Component calls destroy on all its plugins at destroy time.</p>\n"},"getContextFieldValues":{"!type":"fn() -> !this"},"getEditor":{"!type":"fn() -> !this"},"initEditTriggers":{"!type":"fn() -> !this"},"initEditor":{"!type":"fn() -> !this"},"initEditorConfig":{"!type":"fn() -> !this"},"moveEditorByClick":{"!type":"fn() -> !this"},"onBeforeReconfigure":{"!type":"fn() -> !this"},"onColumnAdd":{"!type":"fn(ct: ?, column: ?) -> !this"},"onColumnHide":{"!type":"fn(ct: ?, column: ?) -> !this"},"onColumnMove":{"!type":"fn(ct: ?, column: ?, fromIdx: ?, toIdx: ?) -> !this","!doc":"<p>Template method which may be implemented in subclasses (RowEditing and CellEditing)</p>\n"},"onColumnResize":{"!type":"fn(ct: ?, column: ?, width: ?) -> !this"},"onColumnShow":{"!type":"fn(ct: ?, column: ?) -> !this"},"onReconfigure":{"!type":"fn(grid: ?, store: ?, columns: ?) -> !this","!doc":"<p>Fires after the grid is reconfigured</p>\n"},"setColumnField":{"!type":"fn(column: ?, field: ?) -> !this","!doc":"<p>remaps to the public API of <a href=\"#!/api/Ext.grid.column.Column-method-setEditor\" rel=\"Ext.grid.column.Column-method-setEditor\" class=\"docClass\">Ext.grid.column.Column.setEditor</a></p>\n"},"shouldStartEdit":{"!type":"fn(editor: ?) -> !this"},"startEdit":{"!type":"fn(record: ?, columnHeader?: ?) -> bool","!doc":"<p>Starts editing the specified record, using the specified Column definition to define which field is being edited.</p>\n"},"startEditByClick":{"!type":"fn() -> !this"},"validateEdit":{"!type":"fn() -> !this"}}},"RowExpander":{"!doc":"<p>Plugin (ptype = 'rowexpander') that adds the ability to have a Column in a grid which enables\na second row body which expands/contracts.  The expand/contract behavior is configurable to react\non clicking of the column, double click of the row, and/or hitting enter while a row is selected.</p>\n","!type":"fn(config?: ?)","prototype":{"columnWidth":{"!type":"number","!doc":"<p>The width of the row expander column which contains the [+]/[-] icons to toggle row expansion.</p>\n"},"expandOnDblClick":{"!type":"bool","!doc":"<p><code>true</code> to toggle a row between expanded/collapsed when double clicked\n(defaults to <code>true</code>).</p>\n"},"expandOnEnter":{"!type":"bool","!doc":"<p><code>true</code> to toggle selected row(s) between expanded/collapsed when the enter\nkey is pressed (defaults to <code>true</code>).</p>\n"},"headerWidth":{"!type":"number","!doc":"<p>The width of the Row Expander column header</p>\n"},"lockedTpl":{"!doc":"<p>An XTemplate which, when passed a record data object, produces HTML for the expanded row content <em>on the locked side of a lockable grid</em>.</p>\n"},"rowBodyTpl":{"!doc":"<p>An XTemplate which, when passed a record data object, produces HTML for the expanded row content.</p>\n\n<p>Note that if this plugin is applied to a lockable grid, the rowBodyTpl applies to the normal (unlocked) side.\nSee <a href=\"#!/api/Ext.grid.plugin.RowExpander-cfg-lockedTpl\" rel=\"Ext.grid.plugin.RowExpander-cfg-lockedTpl\" class=\"docClass\">lockedTpl</a></p>\n"},"selectRowOnExpand":{"!type":"bool","!doc":"<p><code>true</code> to select a row when clicking on the expander icon\n(defaults to <code>false</code>).</p>\n"},"addCollapsedCls":{},"lockableScope":{"!type":"string"},"rowBodyHiddenCls":{"!type":"string"},"rowBodyTrSelector":{"!type":"string"},"rowCollapsedCls":{"!type":"string"},"addExpander":{"!type":"fn(expanderGrid: ?) -> !this","!doc":"<p>Inject the expander column into the correct grid.</p>\n\n<p>If we are expanding the normal side of a lockable grid, poke the column into the locked side if the locked side has columns</p>\n"},"beforeReconfigure":{"!type":"fn(grid: ?, store: ?, columns: ?, oldStore: ?, oldColumns: ?) -> !this"},"bindView":{"!type":"fn(view: ?) -> !this"},"getFeatureConfig":{"!type":"fn(grid: ?) -> ?"},"getHeaderConfig":{"!type":"fn() -> !this"},"getRowBodyContentsFn":{"!type":"fn(rowBodyTpl: ?) -> !this"},"getRowBodyFeatureData":{"!type":"fn(record: ?, idx: ?, rowValues: ?) -> !this"},"init":{"!type":"fn(grid: ?) -> !this","!doc":"<p>The init method is invoked after initComponent method has been run for the client Component.</p>\n\n<p>The supplied implementation is empty. Subclasses should perform plugin initialization, and set up bidirectional\nlinks between the plugin and its client Component in their own implementation of this method.</p>\n"},"onColumnLock":{"!type":"fn(lockable: ?, column: ?) -> !this"},"onColumnUnlock":{"!type":"fn(lockable: ?, column: ?) -> !this"},"onDblClick":{"!type":"fn(view: ?, record: ?, row: ?, rowIdx: ?, e: ?) -> !this"},"onKeyDown":{"!type":"fn(view: ?, record: ?, row: ?, rowIdx: ?, e: ?) -> !this"},"onLockableProcessColumns":{"!type":"fn(lockable: ?, lockedHeaders: ?, normalHeaders: ?) -> !this"},"setup":{"!type":"fn(rows: ?, rowValues: ?) -> !this"},"syncRowHeights":{"!type":"fn(lockedItem: ?, normalItem: ?) -> !this","!doc":"<p>Called from TableLayout.finishedLayout</p>\n"},"toggleRow":{"!type":"fn(rowIdx: ?, record: ?) -> !this"},"collapsebody":{"!type":"fn(rowNode: ?, record: ?, expandRow: ?, eOpts: ?)","!doc":"<p><strong>Fired through the grid's View.</strong></p>\n"},"expandbody":{"!type":"fn(rowNode: ?, record: ?, expandRow: ?, eOpts: ?)","!doc":"<p><strong>Fired through the grid's View</strong></p>\n"}}}},"property":{"Grid":{"!doc":"<p>A specialized grid implementation intended to mimic the traditional property grid as typically seen in\ndevelopment IDEs.  Each row in the grid represents a property of some object, and the data is stored\nas a set of name/value pairs in <a href=\"#!/api/Ext.grid.property.Property\" rel=\"Ext.grid.property.Property\" class=\"docClass\">Properties</a>. By default, the editors\nshown are inferred from the data in the cell. More control over this can be specified by using the\n<a href=\"#!/api/Ext.grid.property.Grid-cfg-sourceConfig\" rel=\"Ext.grid.property.Grid-cfg-sourceConfig\" class=\"docClass\">sourceConfig</a> option. Example usage:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.grid.property.Grid\" rel=\"Ext.grid.property.Grid\" class=\"docClass\">Ext.grid.property.Grid</a>', {\n    title: 'Properties Grid',\n    width: 300,\n    renderTo: Ext.getBody(),\n    source: {\n        \"(name)\": \"My Object\",\n        \"Created\": Ext.Date.parse('10/15/2006', 'm/d/Y'),\n        \"Available\": false,\n        \"Version\": 0.01,\n        \"Description\": \"A test object\"\n    }\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"columnLines":{"!type":"bool","!doc":"<p>Adds column line styling</p>\n"},"columns":{},"customEditors":{"!doc":"<p>An object containing name/value pairs of custom editor type definitions that allow\nthe grid to support additional types of editable fields.  By default, the grid supports strongly-typed editing\nof strings, dates, numbers and booleans using built-in form editors, but any custom type can be supported and\nassociated with a custom input control by specifying a custom editor.  The name of the editor\ntype should correspond with the name of the property that will use the editor.  Example usage:</p>\n\n<pre><code>var grid = new <a href=\"#!/api/Ext.grid.property.Grid\" rel=\"Ext.grid.property.Grid\" class=\"docClass\">Ext.grid.property.Grid</a>({\n\n    // Custom editors for certain property names\n    customEditors: {\n        evtStart: Ext.create('<a href=\"#!/api/Ext.form.field.Time\" rel=\"Ext.form.field.Time\" class=\"docClass\">Ext.form.TimeField</a>', {selectOnFocus: true})\n    },\n\n    // Displayed name for property names in the source\n    propertyNames: {\n        evtStart: 'Start Time'\n    },\n\n    // Data object containing properties to edit\n    source: {\n        evtStart: '10:00 AM'\n    }\n});\n</code></pre>\n"},"customRenderers":{"!doc":"<p>An object containing name/value pairs of custom renderer type definitions that allow\nthe grid to support custom rendering of fields.  By default, the grid supports strongly-typed rendering\nof strings, dates, numbers and booleans using built-in form editors, but any custom type can be supported and\nassociated with the type of the value.  The name of the renderer type should correspond with the name of the property\nthat it will render.  Example usage:</p>\n\n<pre><code>var grid = Ext.create('<a href=\"#!/api/Ext.grid.property.Grid\" rel=\"Ext.grid.property.Grid\" class=\"docClass\">Ext.grid.property.Grid</a>', {\n    customRenderers: {\n        Available: function(v){\n            if (v) {\n                return '&lt;span style=\"color: green;\"&gt;Yes&lt;/span&gt;';\n            } else {\n                return '&lt;span style=\"color: red;\"&gt;No&lt;/span&gt;';\n            }\n        }\n    },\n    source: {\n        Available: true\n    }\n});\n</code></pre>\n"},"enableColumnMove":{"!type":"bool","!doc":"<p>private config overrides</p>\n"},"inferTypes":{"!type":"bool","!doc":"<p>True to automatically infer the <a href=\"#!/api/Ext.grid.property.Grid-cfg-sourceConfig\" rel=\"Ext.grid.property.Grid-cfg-sourceConfig\" class=\"docClass\">type</a> based on the initial value passed\nfor each field. This ensures the editor remains the correct type even if the value is blanked\nand becomes empty.</p>\n"},"nameColumnWidth":{"!doc":"<p>Specify the width for the name column. The value column will take any remaining space.</p>\n"},"nameField":{"!type":"string","!doc":"<p>The name of the field from the property store to use as the property field name.\nThis may be useful if you do not configure the property Grid from an object, but use your own store configuration.</p>\n"},"propertyNames":{"!doc":"<p>An object containing custom property name/display name pairs.\nIf specified, the display name will be shown in the name column instead of the property name.</p>\n"},"source":{"!doc":"<p>A data object to use as the data source of the grid (see <a href=\"#!/api/Ext.grid.property.Grid-method-setSource\" rel=\"Ext.grid.property.Grid-method-setSource\" class=\"docClass\">setSource</a> for details).</p>\n"},"sourceConfig":{"!doc":"<p>This option allows various configurations to be set for each field in the property grid.\nNone of these configurations are required</p>\n\n<h4>displayName</h4>\n\n<p>A custom name to appear as label for this field. If specified, the display name will be shown\nin the name column instead of the property name. Example usage:</p>\n\n<pre><code>new <a href=\"#!/api/Ext.grid.property.Grid\" rel=\"Ext.grid.property.Grid\" class=\"docClass\">Ext.grid.property.Grid</a>({\n    source: {\n        clientIsAvailable: true\n    },\n    sourceConfig: {\n        clientIsAvailable: {\n            // Custom name different to the field\n            displayName: 'Available'\n        }\n    }\n});\n</code></pre>\n\n<h4>renderer</h4>\n\n<p>A function used to transform the underlying value before it is displayed in the grid.\nBy default, the grid supports strongly-typed rendering of strings, dates, numbers and booleans using built-in form editors,\nbut any custom type can be supported and associated with the type of the value. Example usage:</p>\n\n<pre><code>new <a href=\"#!/api/Ext.grid.property.Grid\" rel=\"Ext.grid.property.Grid\" class=\"docClass\">Ext.grid.property.Grid</a>({\n    source: {\n        clientIsAvailable: true\n    },\n    sourceConfig: {\n        clientIsAvailable: {\n            // Custom renderer to change the color based on the value\n            renderer: function(v){\n                var color = v ? 'green' : 'red';\n                return '&lt;span style=\"color: ' + color + ';\"&gt;' + v + '&lt;/span&gt;';\n            }\n        }\n    }\n});\n</code></pre>\n\n<h4>type</h4>\n\n<p>Used to explicitly specify the editor type for a particular value. By default, the type is\nautomatically inferred from the value. See <a href=\"#!/api/Ext.grid.property.Grid-cfg-inferTypes\" rel=\"Ext.grid.property.Grid-cfg-inferTypes\" class=\"docClass\">inferTypes</a>. Accepted values are:</p>\n\n<ul>\n<li>'date'</li>\n<li>'boolean'</li>\n<li>'number'</li>\n<li>'string'</li>\n</ul>\n\n\n<p>For more advanced control an editor configuration can be passed (see the next section).\nExample usage:</p>\n\n<pre><code>new <a href=\"#!/api/Ext.grid.property.Grid\" rel=\"Ext.grid.property.Grid\" class=\"docClass\">Ext.grid.property.Grid</a>({\n    source: {\n        attending: null\n    },\n    sourceConfig: {\n        attending: {\n            // Force the type to be a numberfield, a null value would otherwise default to a textfield\n            type: 'number'\n        }\n    }\n});\n</code></pre>\n\n<h4>editor</h4>\n\n<p>Allows the grid to support additional types of editable fields.  By default, the grid supports strongly-typed editing\nof strings, dates, numbers and booleans using built-in form editors, but any custom type can be supported and\nassociated with a custom input control by specifying a custom editor. Example usage</p>\n\n<pre><code>new <a href=\"#!/api/Ext.grid.property.Grid\" rel=\"Ext.grid.property.Grid\" class=\"docClass\">Ext.grid.property.Grid</a>({\n    // Data object containing properties to edit\n    source: {\n        evtStart: '10:00 AM'\n    },\n\n    sourceConfig: {\n        evtStart: {\n            editor: Ext.create('<a href=\"#!/api/Ext.form.field.Time\" rel=\"Ext.form.field.Time\" class=\"docClass\">Ext.form.field.Time</a>', {selectOnFocus: true}),\n            displayName: 'Start Time'\n        }\n    }\n});\n</code></pre>\n"},"store":{},"valueField":{"!type":"string","!doc":"<p>The name of the field from the property store to use as the value field name.\nThis may be useful if you do not configure the property Grid from an object, but use your own store configuration.</p>\n"},"clicksToEdit":{"!type":"number"},"enableHdMenu":{"!type":"bool"},"gridCls":{"!type":"string"},"stripeRows":{"!type":"bool"},"trackMouseOver":{"!type":"bool"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"configure":{"!type":"fn(config: ?) -> !this"},"configureLegacy":{"!type":"fn(config: ?) -> !this","!doc":"<p>to be deprecated in 4.2</p>\n"},"copyLegacyObject":{"!type":"fn(config: ?, o: ?, keyName: ?) -> !this"},"destroyEditors":{"!type":"fn(editors: ?) -> !this"},"getCellEditor":{"!type":"fn(record: ?, column: ?) -> !this","!doc":"<p>Returns the correct editor type for the property type, or a custom one keyed by the property name</p>\n"},"getConfig":{"!type":"fn(fieldName: ?, key: ?, defaultValue: ?) -> !this"},"getSource":{"!type":"fn() -> ?","!doc":"<p>Gets the source data object containing the property data.  See <a href=\"#!/api/Ext.grid.property.Grid-method-setSource\" rel=\"Ext.grid.property.Grid-method-setSource\" class=\"docClass\">setSource</a> for details regarding the\nformat of the data object.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onUpdate":{"!type":"fn(store: ?, record: ?, operation: ?) -> !this"},"removeProperty":{"!type":"fn(prop: ?) -> !this","!doc":"<p>Removes a property from the grid.</p>\n"},"setConfig":{"!type":"fn(fieldName: ?, key: ?, value: ?) -> !this"},"setProperty":{"!type":"fn(prop: ?, value: ?, create?: ?) -> !this","!doc":"<p>Sets the value of a property.</p>\n"},"setSource":{"!type":"fn(source: ?, sourceConfig?: ?) -> !this","!doc":"<p>Sets the source data object containing the property data.  The data object can contain one or more name/value\npairs representing all of the properties of an object to display in the grid, and this data will automatically\nbe loaded into the grid's <a href=\"#!/api/Ext.grid.property.Grid-cfg-store\" rel=\"Ext.grid.property.Grid-cfg-store\" class=\"docClass\">store</a>.  The values should be supplied in the proper data type if needed,\notherwise string type will be assumed.  If the grid already contains data, this method will replace any\nexisting data.  See also the <a href=\"#!/api/Ext.grid.property.Grid-cfg-source\" rel=\"Ext.grid.property.Grid-cfg-source\" class=\"docClass\">source</a> config value.  Example usage:</p>\n\n<pre><code>grid.setSource({\n    \"(name)\": \"My Object\",\n    \"Created\": Ext.Date.parse('10/15/2006', 'm/d/Y'),  // date type\n    \"Available\": false,  // boolean type\n    \"Version\": .01,      // decimal type\n    \"Description\": \"A test object\"\n});\n</code></pre>\n"},"walkCells":{"!type":"fn(pos: ?, direction: ?, e: ?, preventWrap: ?, verifierFn: ?, scope: ?) -> !this","!doc":"<p>Custom implementation of walkCells which only goes up and down.\nRuns in the scope of the TableView</p>\n"},"beforepropertychange":{"!type":"fn(source: ?, recordId: ?, value: ?, oldValue: ?, eOpts: ?)","!doc":"<p>Fires before a property value changes.  Handlers can return false to cancel the property change\n(this will internally call Ext.data.Model.reject on the property's record).</p>\n"},"propertychange":{"!type":"fn(source: ?, recordId: ?, value: ?, oldValue: ?, eOpts: ?)","!doc":"<p>Fires after a property value has changed.</p>\n"}}},"HeaderContainer":{"!doc":"<p>A custom HeaderContainer for the <a href=\"#!/api/Ext.grid.property.Grid\" rel=\"Ext.grid.property.Grid\" class=\"docClass\">Ext.grid.property.Grid</a>.\nGenerally it should not need to be used directly.</p>\n","!type":"fn(grid: ?, source: ?)","prototype":{"dateFormat":{"!type":"string","!doc":"<p>\n<locale></locale></p>\n"},"falseText":{"!type":"string","!doc":"<p>\n<locale></locale></p>\n"},"nameColumnCls":{"!type":"string"},"nameColumnInnerCls":{"!type":"string"},"nameText":{"!type":"string","!doc":"<p>strings used for locale support\n<locale></locale></p>\n"},"nameWidth":{"!type":"number"},"renderDate":{},"trueText":{"!type":"string","!doc":"<p>\n<locale></locale></p>\n"},"valueText":{"!type":"string","!doc":"<p>\n<locale></locale></p>\n"},"getCellEditor":{"!type":"fn(record: ?) -> !this"},"getPropertyName":{"!type":"fn(name: ?) -> !this","!doc":"<p>Renders custom property names instead of raw names if defined in the Grid</p>\n"},"renderBool":{"!type":"fn(bVal: ?) -> !this"},"renderCell":{"!type":"fn(val: ?, meta: ?, rec: ?) -> !this","!doc":"<p>Render a property value cell</p>\n"},"renderProp":{"!type":"fn(v: ?) -> !this","!doc":"<p>Render a property name cell</p>\n"}}},"Property":{"!doc":"<p>A specific Ext.data.Model type that represents a name/value pair and is made to work with the\n<a href=\"#!/api/Ext.grid.property.Grid\" rel=\"Ext.grid.property.Grid\" class=\"docClass\">Ext.grid.property.Grid</a>. Typically, Properties do not need to be created directly as they can be\ncreated implicitly by simply using the appropriate data configs either via the\n<a href=\"#!/api/Ext.grid.property.Grid-cfg-source\" rel=\"Ext.grid.property.Grid-cfg-source\" class=\"docClass\">Ext.grid.property.Grid.source</a> config property or by calling <a href=\"#!/api/Ext.grid.property.Grid-method-setSource\" rel=\"Ext.grid.property.Grid-method-setSource\" class=\"docClass\">Ext.grid.property.Grid.setSource</a>.\nHowever, if the need arises, these records can also be created explicitly as shown below. Example usage:</p>\n\n<pre><code>var rec = new <a href=\"#!/api/Ext.grid.property.Property\" rel=\"Ext.grid.property.Property\" class=\"docClass\">Ext.grid.property.Property</a>({\n    name: 'birthday',\n    value: Ext.Date.parse('17/06/1962', 'd/m/Y')\n});\n// Add record to an already populated grid\ngrid.store.addSorted(rec);\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"fields":{},"idProperty":{"!type":"string"}}},"Reader":{"!doc":"<p>Custom reader for property grid data</p>\n","prototype":{"messageProperty":{},"successProperty":{},"totalProperty":{},"isEditableValue":{"!type":"fn(val: ?) -> !this"},"read":{"!type":"fn(dataObject: ?) -> !this"},"readRecords":{"!type":"fn(dataObject: ?) -> !this"}}},"Store":{"!doc":"<p>A custom Ext.data.Store for the <a href=\"#!/api/Ext.grid.property.Grid\" rel=\"Ext.grid.property.Grid\" class=\"docClass\">Ext.grid.property.Grid</a>. This class handles the mapping\nbetween the custom data source objects supported by the grid and the <a href=\"#!/api/Ext.grid.property.Property\" rel=\"Ext.grid.property.Property\" class=\"docClass\">Ext.grid.property.Property</a> format\nused by the Ext.data.Store base class.</p>\n","!type":"fn(grid: ?, source: ?)","prototype":{"remoteSort":{"!type":"bool"},"getProperty":{"!type":"fn(row: ?) -> !this"},"getProxy":{"!type":"fn() -> !this","!doc":"<p>Return a singleton, customized Proxy object which configures itself with a custom Reader</p>\n"},"getReader":{"!type":"fn() -> !this","!doc":"<p>Return a singleton, customized Reader object which reads <a href=\"#!/api/Ext.grid.property.Property\" rel=\"Ext.grid.property.Property\" class=\"docClass\">Ext.grid.property.Property</a> records from an object.</p>\n"},"getRec":{"!type":"fn(prop: ?) -> !this"},"getSource":{"!type":"fn() -> !this","!doc":"<p>Should only be called by the grid.  Use grid.getSource instead.</p>\n"},"onDestroy":{"!type":"fn() -> !this"},"remove":{"!type":"fn(prop: ?) -> !this"},"setSource":{"!type":"fn(dataObject: ?) -> !this","!doc":"<p>Should only be called by the grid.  Use grid.setSource instead.</p>\n"},"setValue":{"!type":"fn(prop: ?, value: ?, create: ?) -> !this"}}}},"RowEditor":{"!doc":"<p>Internal utility class used to provide row editing functionality. For developers, they should use\nthe RowEditing plugin to use this functionality with a grid.</p>\n","!type":"fn(config: ?)","prototype":{"border":{"!type":"bool","!doc":"<p>Specify as <code>false</code> to render the Panel with zero width borders.</p>\n\n<p>Leaving the value as <code>true</code> uses the selected theme's Ext.panel.Panel.$panel-border-width</p>\n\n<p>Defaults to <code>false</code> when using or extending Neptune.</p>\n"},"hideMode":{"!type":"string","!doc":"<p>Change the hideMode to offsets so that we get accurate measurements when\nthe roweditor is hidden for laying out things like a TriggerField.</p>\n"},"buttonUI":{"!type":"string"},"cancelBtnText":{"!type":"string","!doc":"<p>\n<locale></locale></p>\n"},"dirtyText":{"!type":"string","!doc":"<p>\n<locale></locale></p>\n"},"errorCls":{"!type":"string"},"errorsText":{"!type":"string","!doc":"<p>\n<locale></locale></p>\n"},"lastScrollLeft":{"!type":"number","!doc":"<p></p>\n"},"lastScrollTop":{"!type":"number"},"saveBtnText":{"!type":"string","!doc":"<p><locale></locale></p>\n"},"addFieldsForColumn":{"!type":"fn(column: ?, initial: ?) -> !this"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this"},"beforeEdit":{"!type":"fn() -> !this"},"calculateEditorTop":{"!type":"fn(rowTop: ?) -> !this","!doc":"<p>Given the top pixel position of a row in the scroll space,\ncalculate the editor top position in the view's encapsulating element.\nThis will only ever be in the visible range of the view's element.</p>\n"},"calculateLocalRowTop":{"!type":"fn(row: ?) -> !this","!doc":"<p>Calculates the top pixel position of the passed row within the view's scroll space.\nSo in a large, scrolled grid, this could be several thousand pixels.</p>\n"},"cancelEdit":{"!type":"fn() -> !this"},"clearClip":{"!type":"fn(el: ?) -> !this"},"clipBottom":{"!type":"fn(value: ?) -> !this"},"clipTop":{"!type":"fn(value: ?) -> !this"},"completeEdit":{"!type":"fn() -> !this"},"createErrorListItem":{"!type":"fn(e: ?) -> !this"},"destroyColumnEditor":{"!type":"fn(column: ?) -> !this"},"focusContextCell":{"!type":"fn() -> !this","!doc":"<p>Focus the cell on start edit based upon the current context</p>\n"},"getClientWidth":{"!type":"fn() -> !this"},"getDefaultFieldCfg":{"!type":"fn() -> !this"},"getEditor":{"!type":"fn(fieldInfo: ?) -> !this"},"getErrors":{"!type":"fn() -> !this"},"getFieldScrollerScrollX":{"!type":"fn() -> !this"},"getFloatingButtons":{"!type":"fn() -> !this"},"getRefItems":{"!type":"fn(deep: ?) -> !this","!doc":"<p>Used by ComponentQuery, child and down to retrieve all of the items\nwhich can potentially be considered a child of this Container.</p>\n\n<p>This may be overriden by Components which have ownership of Components\nthat are not contained in the <a href=\"#!/api/Ext.grid.RowEditor-property-items\" rel=\"Ext.grid.RowEditor-property-items\" class=\"docClass\">items</a> collection.</p>\n\n<p>NOTE: IMPORTANT note for maintainers:\nItems are returned in tree traversal order. Each item is appended to the result array\nfollowed by the results of that child's getRefItems call.\nFloating child items are appended after internal child items.</p>\n"},"getRefOwner":{"!type":"fn() -> !this"},"getScrollDelta":{"!type":"fn() -> number","!doc":"<p>Returns the scroll delta required to scroll the context row into view in order to make\nthe whole of this editor visible.</p>\n"},"getToolTip":{"!type":"fn() -> !this"},"hideToolTip":{"!type":"fn() -> !this"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initKeyNav":{"!type":"fn() -> !this"},"insertColumnEditor":{"!type":"fn(column: ?) -> !this"},"isDescendantOf":{"!type":"fn() -> bool","!doc":"<p>Because we implement getRefOwner to return the grid, this makes this a descendant of the grid and the layout system\ndoes not queue an independent layout for this while it's a descendant of a component which itself is pending a layout.\nThis is floating and not a descendant of anything.</p>\n"},"isDirty":{"!type":"fn() -> bool","!doc":"<p>Convenience function to check if the form has any dirty fields. This is the same as calling\n<a href=\"#!/api/Ext.form.Basic-method-isDirty\" rel=\"Ext.form.Basic-method-isDirty\" class=\"docClass\">this.getForm().isDirty()</a>.</p>\n"},"loadRecord":{"!type":"fn(record: ?) -> ?","!doc":"<p>Loads an Ext.data.Model into this form (internally just calls <a href=\"#!/api/Ext.form.Basic-method-loadRecord\" rel=\"Ext.form.Basic-method-loadRecord\" class=\"docClass\">Ext.form.Basic.loadRecord</a>)\nSee also <a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a>. The fields in the model are mapped to\nfields in the form by matching either the <a href=\"#!/api/Ext.form.field.Base-cfg-name\" rel=\"Ext.form.field.Base-cfg-name\" class=\"docClass\">Ext.form.field.Base.name</a> or <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">Ext.Component.itemId</a>.</p>\n"},"onBeforeViewRefresh":{"!type":"fn(view: ?) -> !this"},"onColumnAdd":{"!type":"fn(column: ?) -> !this"},"onColumnHide":{"!type":"fn(column: ?) -> !this"},"onColumnMove":{"!type":"fn(column: ?, fromIdx: ?, toIdx: ?) -> !this"},"onColumnResize":{"!type":"fn(column: ?, width: ?) -> !this"},"onColumnShow":{"!type":"fn(column: ?) -> !this"},"onFieldContainerScroll":{"!type":"fn() -> !this","!doc":"<p>Synchronize the horizontal scroll position of the grid view with the fields.</p>\n"},"onGridResize":{"!type":"fn() -> !this","!doc":"<p>Grid listener added when this is rendered.\nKeep our containing element sized correctly</p>\n"},"onHide":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the hide operation. After\ncalling the superclass's onHide, the Component will be hidden.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.grid.RowEditor-event-hide\" rel=\"Ext.grid.RowEditor-event-hide\" class=\"docClass\">hide</a>.</p>\n"},"onShow":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the show operation. After\ncalling the superclass's onShow, the Component will be visible.</p>\n\n<p>Override in subclasses where more complex behaviour is needed.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.grid.RowEditor-event-show\" rel=\"Ext.grid.RowEditor-event-show\" class=\"docClass\">show</a>.</p>\n"},"onValidityChange":{"!type":"fn(form: ?, valid: ?) -> !this"},"onViewItemRemove":{"!type":"fn(record: ?, index: ?, item: ?, view: ?) -> !this"},"onViewRefresh":{"!type":"fn(view: ?) -> !this"},"onViewScroll":{"!type":"fn() -> !this"},"renderColumnData":{"!type":"fn(field: ?, record: ?, activeColumn: ?) -> !this"},"reposition":{"!type":"fn(animateConfig: ?, fromScrollHandler: ?) -> !this"},"repositionIfVisible":{"!type":"fn(c: ?) -> !this"},"repositionTip":{"!type":"fn() -> !this"},"setColumnEditor":{"!type":"fn(column: ?, idx: ?, fieldContainer: ?) -> !this"},"showToolTip":{"!type":"fn() -> !this"},"startEdit":{"!type":"fn(record: ?, columnHeader: ?) -> !this","!doc":"<p>Start editing the specified grid at the specified position.</p>\n"},"syncAllFieldWidths":{"!type":"fn() -> !this"},"syncButtonPosition":{"!type":"fn(scrollDelta: ?) -> !this","!doc":"<p>determines the amount by which the row editor will overflow, and flips the buttons\nto the top of the editor if the required scroll amount is greater than the available\nscroll space. Returns the scrollDelta required to scroll the editor into view after\nadjusting the button position.</p>\n"},"syncEditorClip":{"!type":"fn() -> !this","!doc":"<p>since the editor is rendered to the grid el, it must be clipped when scrolled\noutside of the grid view area so that it does not overlap the scrollbar or docked items</p>\n"},"syncFieldWidth":{"!type":"fn(column: ?) -> !this"},"syncFieldsHorizontalScroll":{"!type":"fn() -> !this","!doc":"<p>Synchronize the horizontal scroll position of the fields with the state of the grid view</p>\n"},"updateButton":{"!type":"fn(valid: ?) -> !this"}}},"RowEditorButtons":{"!doc":"<p>Private Container class used by the <a href=\"#!/api/Ext.grid.RowEditor\" rel=\"Ext.grid.RowEditor\" class=\"docClass\">Ext.grid.RowEditor</a> to hold its buttons.</p>\n","!type":"fn(config: ?)","prototype":{"frame":{"!type":"bool","!doc":"<p>Specify as <code>true</code> to have the Component inject framing elements within the Component at render time to provide a\ngraphical rounded frame around the Component content.</p>\n\n<p>This is only necessary when running on outdated, or non standard-compliant browsers such as Microsoft's Internet\nExplorer prior to version 9 which do not support rounded corners natively.</p>\n\n<p>The extra space taken up by this framing is available from the read only property <a href=\"#!/api/Ext.grid.RowEditorButtons-property-frameSize\" rel=\"Ext.grid.RowEditorButtons-property-frameSize\" class=\"docClass\">frameSize</a>.</p>\n"},"shrinkWrap":{"!doc":"<p>If this property is a number, it is interpreted as follows:</p>\n\n<ul>\n<li>0 (or <code>false</code>): Neither width nor height depend on content.</li>\n<li>1: Width depends on content (shrink wraps), but height does not.</li>\n<li>2 (default): Height depends on content (shrink wraps), but width does not.</li>\n<li>3 (or <code>true</code>): Both width and height depend on content (shrink wrap).</li>\n</ul>\n\n\n<p>In CSS terms, shrink-wrap width is analogous to an inline-block element as opposed\nto a block-level element. Some container layouts always shrink-wrap their children,\neffectively ignoring this property (e.g., <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">Ext.layout.container.HBox</a>,\n<a href=\"#!/api/Ext.layout.container.VBox\" rel=\"Ext.layout.container.VBox\" class=\"docClass\">Ext.layout.container.VBox</a>, <a href=\"#!/api/Ext.layout.component.Dock\" rel=\"Ext.layout.component.Dock\" class=\"docClass\">Ext.layout.component.Dock</a>).</p>\n"},"position":{"!type":"string"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"setButtonPosition":{"!type":"fn(position: ?) -> !this"}}},"View":{"!doc":"<p>The grid View class provides extra <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a> specific functionality to the\n<a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">Ext.view.Table</a>. In general, this class is not instanced directly, instead a viewConfig\noption is passed to the grid:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    // other options\n    viewConfig: {\n        stripeRows: false\n    }\n});\n</code></pre>\n\n<h2>Drag Drop</h2>\n\n<p>Drag and drop functionality can be achieved in the grid by attaching a <a href=\"#!/api/Ext.grid.plugin.DragDrop\" rel=\"Ext.grid.plugin.DragDrop\" class=\"docClass\">Ext.grid.plugin.DragDrop</a> plugin\nwhen creating the view.</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    // other options\n    viewConfig: {\n        plugins: {\n            ddGroup: 'people-group',\n            ptype: 'gridviewdragdrop',\n            enableDrop: false\n        }\n    }\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"autoScroll":{"!type":"bool","!doc":"<p><code>true</code> to use overflow:'auto' on the components layout element and show scroll bars automatically when necessary,\n<code>false</code> to clip any overflowing content.\nThis should not be combined with <a href=\"#!/api/Ext.grid.View-cfg-overflowX\" rel=\"Ext.grid.View-cfg-overflowX\" class=\"docClass\">overflowX</a> or  <a href=\"#!/api/Ext.grid.View-cfg-overflowY\" rel=\"Ext.grid.View-cfg-overflowY\" class=\"docClass\">overflowY</a>.</p>\n"}}},"ViewDropZone":{"!type":"fn(config: ?)","prototype":{"indicatorCls":{"!type":"string"},"indicatorHtml":{"!type":"string"},"handleNodeDrop":{"!type":"fn(data: ?, record: ?, position: ?) -> !this"}}}},"layout":{"component":{"Auto":{"prototype":{"setHeightInDom":{"!type":"bool","!doc":"<p>When publishing height of an auto Component, it is usually not written to the DOM.\nSetting this to <code>true</code> overrides this behaviour.</p>\n"},"waitForOuterHeightInDom":{"!type":"bool"},"waitForOuterWidthInDom":{"!type":"bool"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.layout.component.Auto-property-done\" rel=\"Ext.layout.component.Auto-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.layout.component.Auto-property-done\" rel=\"Ext.layout.component.Auto-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"},"publishOwnerHeight":{"!type":"fn(ownerContext: ?, contentHeight: ?) -> !this"},"publishOwnerWidth":{"!type":"fn(ownerContext: ?, contentWidth: ?) -> !this"},"setWidthInDom":{"!type":"bool","!doc":"<p>When publishing width of an auto Component, it is usually not written to the DOM.\nSetting this to <code>true</code> overrides this behaviour.</p>\n"},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?, firstCycle: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"calculateOwnerHeightFromContentHeight":{"!type":"fn(ownerContext: ?, contentHeight: ?) -> !this"},"calculateOwnerWidthFromContentWidth":{"!type":"fn(ownerContext: ?, contentWidth: ?) -> !this"}},"!doc":"<p>The class is the default component layout for <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> when no explicit\n<code><a href=\"#!/api/Ext.Component-cfg-componentLayout\" rel=\"Ext.Component-cfg-componentLayout\" class=\"docClass\">componentLayout</a></code> is configured.</p>\n\n<p>This class uses template methods to perform the individual aspects of measurement,\ncalculation and publication of results. The methods called depend on the component's\n<a href=\"#!/api/Ext.Component-method-getSizeModel\" rel=\"Ext.Component-method-getSizeModel\" class=\"docClass\">size model</a>.</p>\n\n<h2>configured / calculated</h2>\n\n<p>In either of these size models, the dimension of the outer element is of a known size.\nThe size is found in the <code>ownerContext</code> (the <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> for the owner\ncomponent) as either \"width\" or \"height\". This value, if available, is passed to the\n<code>publishInnerWidth</code> or <code>publishInnerHeight</code> method, respectively.</p>\n\n<h2>shrinkWrap</h2>\n\n<p>When a dimension uses the <code>shrinkWrap</code> size model, that means the content is measured,\nthen the outer (owner) size is calculated and published.</p>\n\n<p>For example, for a shrinkWrap width, the following sequence of calls are made:</p>\n\n<ul>\n<li><code><a href=\"#!/api/Ext.layout.component.Component-method-measureContentWidth\" rel=\"Ext.layout.component.Component-method-measureContentWidth\" class=\"docClass\">Ext.layout.component.Component.measureContentWidth</a></code></li>\n<li><code>publishOwnerWidth</code>\n\n<ul>\n<li><code>calculateOwnerWidthFromContentWidth</code></li>\n<li><code>publishInnerWidth</code> (in the event of hitting a min/maxWidth constraint)</li>\n</ul>\n</li>\n</ul>\n\n\n<h2>natural</h2>\n\n<p>When a dimension uses the <code>natural</code> size model, the measurement is made on the outer\n(owner) element. This size is then used to determine the content area in much the same\nway as if the outer element had a <code>configured</code> or <code>calculated</code> size model.</p>\n\n<ul>\n<li><code><a href=\"#!/api/Ext.layout.component.Component-method-measureOwnerWidth\" rel=\"Ext.layout.component.Component-method-measureOwnerWidth\" class=\"docClass\">Ext.layout.component.Component.measureOwnerWidth</a></code></li>\n<li><code>publishInnerWidth</code></li>\n</ul>\n\n","!type":"fn(config: ?)"},"Component":{"prototype":{"heightCache":{},"isComponentLayout":{"!type":"bool"},"nullBox":{},"usesContentHeight":{"!type":"bool"},"usesContentWidth":{"!type":"bool"},"usesHeight":{"!type":"bool"},"usesWidth":{"!type":"bool"},"widthCache":{},"cacheTargetInfo":{"!type":"fn(ownerContext: ?) -> !this"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method is called after all layouts are complete and their calculations flushed\nto the DOM. No further layouts will be run and this method is only called once per\nlayout run. The base component layout caches <code>lastComponentSize</code>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"getRenderTarget":{"!type":"fn() -> ?","!doc":"<p>Returns the element into which rendering must take place. Defaults to the owner Component's encapsulating element.</p>\n\n<p>May be overridden in Component layout managers which implement an inner element.</p>\n"},"getTarget":{"!type":"fn() -> ?","!doc":"<p>Returns the owner component's resize element.</p>\n"},"measureAutoDimensions":{"!type":"fn(ownerContext: ?, dimensions: ?) -> !this"},"measureContentWidth":{"!type":"fn(ownerContext: ?) -> !this"},"measureOwnerHeight":{"!type":"fn(ownerContext: ?) -> !this"},"measureOwnerWidth":{"!type":"fn(ownerContext: ?) -> !this"},"notifyOwner":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method (if implemented) is called after all layouts are finished, and all have\na <code>lastComponentSize</code> cached. No further layouts will be run and this method is only\ncalled once per layout run. It is the bookend to <a href=\"#!/api/Ext.layout.component.Component-method-beginLayout\" rel=\"Ext.layout.component.Component-method-beginLayout\" class=\"docClass\">beginLayout</a>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method. This is the catch-all tail method to a layout and so the rules are more\nrelaxed. Even so, for performance reasons, it is best to avoid reading the DOM. If\na read is necessary, consider implementing a <a href=\"#!/api/Ext.layout.component.Component-method-finalizeLayout\" rel=\"Ext.layout.component.Component-method-finalizeLayout\" class=\"docClass\">finalizeLayout</a> method to do the\nrequired reads.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"measureContentHeight":{"!type":"fn(ownerContext: ?) -> !this"},"type":{"!type":"string"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?, firstCycle: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"}},"!doc":"<p>This class is intended to be extended or created via the <a href=\"#!/api/Ext.Component-cfg-componentLayout\" rel=\"Ext.Component-cfg-componentLayout\" class=\"docClass\">layout</a>\nconfiguration property.  See <a href=\"#!/api/Ext.Component-cfg-componentLayout\" rel=\"Ext.Component-cfg-componentLayout\" class=\"docClass\">Ext.Component.componentLayout</a> for additional details.</p>\n","!type":"fn(config: ?)"},"Body":{"!doc":"<p>Component layout for components which maintain an inner body element which must be resized to synchronize with the\nComponent size.</p>\n","!type":"fn(config: ?)","prototype":{"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to prepare for layout.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?, firstCycle: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"calculateOwnerHeightFromContentHeight":{"!type":"fn(ownerContext: ?, contentHeight: ?) -> !this","!doc":"<p>Padding is exciting here because we have 2 el's: owner.el and owner.body. Content\nsize always includes the padding of the targetEl, which should be owner.body. But\nit is common to have padding on owner.el also (such as a panel header), so we need\nto do some more padding work if targetContext is not owner.el. The base class has\nalready handled the ownerContext's frameInfo (border+framing) so all that is left\nis padding.</p>\n"},"calculateOwnerWidthFromContentWidth":{"!type":"fn(ownerContext: ?, contentWidth: ?) -> !this"},"measureContentHeight":{"!type":"fn(ownerContext: ?) -> !this"},"measureContentWidth":{"!type":"fn(ownerContext: ?) -> !this"},"publishInnerHeight":{"!type":"fn(ownerContext: ?, height: ?) -> !this"},"publishInnerWidth":{"!type":"fn(ownerContext: ?, width: ?) -> !this"}}},"BoundList":{"!doc":"<p>Component layout for <a href=\"#!/api/Ext.view.BoundList\" rel=\"Ext.view.BoundList\" class=\"docClass\">Ext.view.BoundList</a>.</p>\n","!type":"fn(config: ?)","prototype":{"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to prepare for layout.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"calculateOwnerHeightFromContentHeight":{"!type":"fn(ownerContext: ?) -> !this"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method is called after all layouts are complete and their calculations flushed\nto the DOM. No further layouts will be run and this method is only called once per\nlayout run. The base component layout caches <code>lastComponentSize</code>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"getLayoutItems":{"!type":"fn() -> !this","!doc":"<p>Returns the set of items to layout (empty by default).</p>\n"},"isValidParent":{"!type":"fn() -> !this","!doc":"<p>Validates item is in the proper place in the dom.</p>\n"},"measureContentHeight":{"!type":"fn(ownerContext: ?) -> !this"},"measureContentWidth":{"!type":"fn(ownerContext: ?) -> !this"},"publishInnerHeight":{"!type":"fn(ownerContext: ?, height: ?) -> !this"}}},"Dock":{"!doc":"<p>This ComponentLayout handles docking for Panels. It takes care of panels that are\npart of a ContainerLayout that sets this Panel's size and Panels that are part of\nan AutoContainerLayout in which this panel get his height based of the CSS or\nits content.</p>\n","!type":"fn(config: ?)","prototype":{"_itemCls":{"!type":"string"},"borderCollapseMap":{"!doc":"<p>This object is indexed by a component's <code>baseCls</code> to yield another object which\nis then indexed by the component's <code>ui</code> to produce an array of CSS class names.\nThis array is indexed in the same manner as the <code>noBorderClassTable</code> and indicates\nthe a particular edge of a docked item or the body element is actually \"collapsed\"\nwith the component's outer border.</p>\n"},"borderWidthProps":{},"horizontalCollapsePolicy":{},"horzAxisProps":{},"initializedBorders":{},"noBorderClasses":{},"noBorderClassesSides":{},"sizePolicy":{},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"vertAxisProps":{},"verticalCollapsePolicy":{},"afterInvalidateShrinkWrapDock":{"!type":"fn(itemContext: ?, options: ?) -> !this"},"afterRemove":{"!type":"fn(item: ?) -> !this","!doc":"<p>Removes layout's itemCls and owning Container's itemCls.\nClears the managed dimensions flags</p>\n"},"beforeInvalidateShrinkWrapDock":{"!type":"fn(itemContext: ?, options: ?) -> !this"},"beforeLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to prepare for layout.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.layout.component.Dock-property-done\" rel=\"Ext.layout.component.Dock-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.layout.component.Dock-property-done\" rel=\"Ext.layout.component.Dock-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"},"configureItem":{"!type":"fn(item: ?) -> !this","!doc":"<p>We are overriding the <a href=\"#!/api/Ext.layout.Layout\" rel=\"Ext.layout.Layout\" class=\"docClass\">Ext.layout.Layout</a> configureItem method to also add a class that\nindicates the position of the docked item. We use the itemCls (x-docked) as a prefix.\nAn example of a class added to a dock: right item is x-docked-right</p>\n"},"createAxis":{"!type":"fn(ownerContext: ?, contentSize: ?, sizeModel: ?, axisProps: ?, collapsedAxis: ?) -> !this","!doc":"<p>Creates an axis object given the particulars. The process starts by placing the\ndockedItems in an idealized box where this method is called once for each side.\nThe ideal box is defined by the CSS3 border and padding values (which account for\nthe influence of border-radius). The origin (the (0,0) point) of the ideal box is\nthe top-left edge of the border or the border-box. Normal dockedItems are placed\ninside this box at an offset to clear the border and padding and sit properly in\nthe panel next to the body.</p>\n\n<p>The origin has to be started differently if the axis is in shrinkWrap mode. When\nshrink-wrapping an axis, the axis starts at the edge of the body and expands\noutwards as items are docked. This means the ideal (0,0) for shrinkWrap is on the\ntop-left corner of the body.</p>\n\n<p>The following diagram illustrates this using the vertical axis.</p>\n\n<pre><code> +---------------------------+ 10px (border)\n |                           |\n |  xxxxxxxxxxxxxxxxxxxxxxx  | 5px (padding)   shrinkWrap    other\n |  +=====================+  |                   -50         15\n |  |  Header             |  | 30px\n |  |                     |  |\n |  +=====================+  |\n |  +---------------------+  |                   -20         45\n |  |  tbar               |  | 20 px\n |  +---------------------+  |\n |  +---------------------+  |                   0           65\n |  |  Body               |  | 100px\n |  |                     |  |\n |  |                     |  |\n |  +---------------------+  |\n |  +---------------------+  |                   100         165\n |  |  bbar               |  | 15px\n |  +---------------------+  |\n |  xxxxxxxxxxxxxxxxxxxxxxx  | 5px\n |                           |\n +---------------------------+ 10px\n</code></pre>\n\n<p>These are sufficient to determine sizes of things, but to finalize this process\nand assign proper positions, the tentative coordinates have to be adjusted by an\namount appropriate for the item. Because dockedItems are position:absolute, they\nsit inside the border and so must be adjusted for padding. The body is different\nbecause it is position:relative and so it naturally sits inside the padding and\nthe padding must not be included in its position.</p>\n\n<p>Headers and footers that use <code>ignoreParentFrame</code> interact with this process by\nmoving themselves outside the border and padding. So in the above diagram, the\nHeader would move up by 15px and <em>everything else</em> would move up by 10px. When\nshrinkWrap is taking place, the 10px of border on the top is removed from the\nheight as well.</p>\n\n<p>The bbar behaves slightly different when it is <code>ignoreParentFrame</code>. In shrinkWrap\nmode, it alone would move down by the padding and the bottom border would not be\nincluded in the height. Otherwise, the bbar would be moved down 15px (since the\nedge is fixed) and the next dockedItem would be placed at, or the body would be\nstretched down to, 5px (padding) pixels above the bbar.</p>\n"},"dockChild":{"!type":"fn(ownerContext: ?, axis: ?, backward: ?, forward: ?) -> !this","!doc":"<p>Docks a child item on the specified axis. This boils down to determining if the item\nis docked at the \"beginning\" of the axis (\"left\" if horizontal, \"top\" if vertical),\nthe \"end\" of the axis (\"right\" if horizontal, \"bottom\" if vertical) or stretches\nalong the axis (\"top\" or \"bottom\" if horizontal, \"left\" or \"right\" if vertical). It\nalso has to differentiate between fixed and shrinkWrap sized dimensions.</p>\n"},"dockInwardBegin":{"!type":"fn(ownerContext: ?, itemContext: ?, item: ?, axis: ?) -> !this","!doc":"<p>Docks an item on a fixed-size axis at the \"beginning\". The \"beginning\" of the horizontal\naxis is \"left\" and the vertical is \"top\". For a fixed-size axis, the size works from\nthe outer element (the panel) towards the body.</p>\n"},"dockInwardEnd":{"!type":"fn(ownerContext: ?, itemContext: ?, item: ?, axis: ?) -> !this","!doc":"<p>Docks an item on a fixed-size axis at the \"end\". The \"end\" of the horizontal axis is\n\"right\" and the vertical is \"bottom\".</p>\n"},"dockOutwardBegin":{"!type":"fn(ownerContext: ?, itemContext: ?, item: ?, axis: ?) -> !this","!doc":"<p>Docks an item on a shrinkWrap axis at the \"beginning\". The \"beginning\" of the horizontal\naxis is \"left\" and the vertical is \"top\". For a shrinkWrap axis, the size works from\nthe body outward to the outermost element (the panel).</p>\n\n<p>During the docking process, coordinates are allowed to be negative. We start with the\nbody at (0,0) so items docked \"top\" or \"left\" will simply be assigned negative x/y. In\nthe <a href=\"#!/api/Ext.layout.component.Dock-method-finishPositions\" rel=\"Ext.layout.component.Dock-method-finishPositions\" class=\"docClass\">finishPositions</a> method these are corrected and framing is added. This way\nthe correction is applied as a simple translation of delta x/y on all coordinates to\nbring the origin back to (0,0).</p>\n"},"dockOutwardEnd":{"!type":"fn(ownerContext: ?, itemContext: ?, item: ?, axis: ?) -> !this","!doc":"<p>Docks an item on a shrinkWrap axis at the \"end\". The \"end\" of the horizontal axis is\n\"right\" and the vertical is \"bottom\".</p>\n"},"dockStretch":{"!type":"fn(ownerContext: ?, itemContext: ?, item: ?, axis: ?) -> !this","!doc":"<p>Docks an item that might stretch across an axis. This is done for dock \"top\" and\n\"bottom\" items on the horizontal axis and dock \"left\" and \"right\" on the vertical.</p>\n"},"finishAxis":{"!type":"fn(ownerContext: ?, axis: ?) -> !this","!doc":"<p>Finishes the calculation of an axis by determining its size. In non-shrink-wrap\ncases, this is also where we set the body size.</p>\n"},"finishConstraints":{"!type":"fn(ownerContext: ?, horz: ?, vert: ?) -> !this","!doc":"<p>Finishes processing of each axis by applying the min/max size constraints.</p>\n"},"finishPositions":{"!type":"fn(ownerContext: ?, horz: ?, vert: ?) -> !this","!doc":"<p>Finishes the calculation by setting positions on the body and all of the items.</p>\n"},"finishRender":{"!type":"fn() -> !this"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method is called after all layouts are complete and their calculations flushed\nto the DOM. No further layouts will be run and this method is only called once per\nlayout run. The base component layout caches <code>lastComponentSize</code>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"getAnimatePolicy":{"!type":"fn(ownerContext: ?) -> !this"},"getBorderCollapseTable":{"!type":"fn() -> !this","!doc":"<p>Returns the array of class names to add to a docked item or body element when for\nthe edges that should collapse with the outer component border. Basically, the\npanel's outer border must look visually like a contiguous border but may need to\nbe realized by using the border of docked items and/or the body. This class name\nallows the border color and width to be controlled accordingly and distinctly from\nthe border of the docked item or body element when it is not having its border\ncollapsed.</p>\n"},"getDockCls":{"!type":"fn(dock: ?) -> string","!doc":"<p>Get's the css class name for a given dock position.</p>\n"},"getDockedItems":{"!type":"fn(order?: ?, beforeBody?: ?) -> ?","!doc":"<p>Retrieve an ordered and/or filtered array of all docked Components.</p>\n"},"getItemSizePolicy":{"!type":"fn(item: ?, ownerSizeModel: ?) -> ?","!doc":"<p>Returns an object describing how this layout manages the size of the given component.\nThis method must be implemented by any layout that manages components.</p>\n"},"getItemWeight":{"!type":"fn(item: ?, order: ?) -> !this"},"getLayoutItems":{"!type":"fn() -> ?","!doc":"<p>Returns an array containing all the <strong>visible</strong> docked items inside this layout's owner Panel</p>\n"},"handleItemBorders":{"!type":"fn() -> !this"},"invalidateAxes":{"!type":"fn(ownerContext: ?, horz: ?, vert: ?) -> !this","!doc":"<p>The default weighting of docked items produces this arrangement:</p>\n\n<pre><code> +--------------------------------------------+\n |                    Top 1                   |\n +--------------------------------------------+\n |                    Top 2                   |\n +-----+-----+--------------------+-----+-----+\n |     |     |                    |     |     |\n |     |     |                    |     |     |\n |     |     |                    |  R  |  R  |\n |  L  |  L  |                    |  I  |  I  |\n |  E  |  E  |                    |  G  |  G  |\n |  F  |  F  |                    |  H  |  H  |\n |  T  |  T  |                    |  T  |  T  |\n |     |     |                    |     |     |\n |  2  |  1  |                    |  1  |  2  |\n |     |     |                    |     |     |\n |     |     |                    |     |     |\n +-----+-----+--------------------+-----+-----+\n |                  Bottom 1                  |\n +--------------------------------------------+\n |                  Bottom 2                  |\n +--------------------------------------------+\n</code></pre>\n\n<p>So when we are shrinkWrapDock on the horizontal, the stretch size for top/bottom\ndocked items is the final axis size. For the vertical axis, however, the stretch</p>\n"},"isItemBoxParent":{"!type":"fn(itemContext: ?) -> !this"},"isItemShrinkWrap":{"!type":"fn(item: ?) -> !this"},"measureContentHeight":{"!type":"fn(ownerContext: ?) -> !this"},"measureContentWidth":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Content size includes padding but not borders, so subtract them off</p>\n"},"redoLayout":{"!type":"fn(ownerContext: ?) -> !this"},"renderChildren":{"!type":"fn() -> !this","!doc":"<p>override inherited.\nWe need to render in the correct order, top/left before bottom/right</p>\n"},"renderItems":{"!type":"fn(items: ?, target: ?) -> !this","!doc":"<p>Render the top and left docked items before any existing DOM nodes in our render target,\nand then render the right and bottom docked items after. This is important, for such things\nas tab stops and ARIA readers, that the DOM nodes are in a meaningful order.\nOur collection of docked items will already be ordered via Panel.getDockedItems().</p>\n"},"undoLayout":{"!type":"fn(ownerContext: ?) -> !this"}}},"field":{"FieldContainer":{"!type":"fn(config: ?)","prototype":{"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"waitForOuterHeightInDom":{"!type":"bool"},"waitForOuterWidthInDom":{"!type":"bool"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to prepare for layout.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"measureContentHeight":{"!type":"fn(ownerContext: ?) -> !this"},"measureContentWidth":{"!type":"fn(ownerContext: ?) -> !this"},"publishInnerHeight":{"!type":"fn(ownerContext: ?, height: ?) -> !this"},"publishInnerWidth":{"!type":"fn(ownerContext: ?, width: ?) -> !this"}}},"HtmlEditor":{"!doc":"<p>Layout class for <a href=\"#!/api/Ext.form.field.HtmlEditor\" rel=\"Ext.form.field.HtmlEditor\" class=\"docClass\">Ext.form.field.HtmlEditor</a> fields. Sizes textarea and iframe elements.</p>\n","!type":"fn(config: ?)","prototype":{"naturalHeight":{"!type":"number"},"naturalWidth":{"!type":"number"},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to prepare for layout.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"finishedLayout":{"!type":"fn() -> !this","!doc":"<p>This method is called after all layouts are complete and their calculations flushed\nto the DOM. No further layouts will be run and this method is only called once per\nlayout run. The base component layout caches <code>lastComponentSize</code>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"publishInnerHeight":{"!type":"fn(ownerContext: ?, height: ?) -> !this"},"publishInnerWidth":{"!type":"fn(ownerContext: ?, width: ?) -> !this","!doc":"<p>The offsets for the text area are needed for bugs in sizing with IE.\nThe main issue behind it is that the iframe requires an initial height\nto be set while the component is auto sizing, however we need to switch\nit when using a configured size. A more permanent solution might be to\nhave the iframe and text area be child components of the container\nas opposed to being directly inserted into the DOM.</p>\n"},"publishOwnerWidth":{"!type":"fn(ownerContext: ?, width: ?) -> !this"}}}},"FieldSet":{"!doc":"<p>Component layout for <a href=\"#!/api/Ext.form.FieldSet\" rel=\"Ext.form.FieldSet\" class=\"docClass\">Ext.form.FieldSet</a> components</p>\n","!type":"fn(config: ?)","prototype":{"defaultCollapsedWidth":{"!type":"number"},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"beforeLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"calculateOwnerHeightFromContentHeight":{"!type":"fn(ownerContext: ?, contentHeight: ?) -> !this","!doc":"<p>Padding is exciting here because we have 2 el's: owner.el and owner.body. Content\nsize always includes the padding of the targetEl, which should be owner.body. But\nit is common to have padding on owner.el also (such as a panel header), so we need\nto do some more padding work if targetContext is not owner.el. The base class has\nalready handled the ownerContext's frameInfo (border+framing) so all that is left\nis padding.</p>\n"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method is called after all layouts are complete and their calculations flushed\nto the DOM. No further layouts will be run and this method is only called once per\nlayout run. The base component layout caches <code>lastComponentSize</code>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"getLayoutItems":{"!type":"fn() -> !this","!doc":"<p>Returns the set of items to layout (empty by default).</p>\n"},"publishInnerHeight":{"!type":"fn(ownerContext: ?, height: ?) -> !this"}}},"ProgressBar":{"!type":"fn(config: ?)","prototype":{"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to prepare for layout.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.layout.component.ProgressBar-property-done\" rel=\"Ext.layout.component.ProgressBar-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.layout.component.ProgressBar-property-done\" rel=\"Ext.layout.component.ProgressBar-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"}}}},"Layout":{"prototype":{"$configPrefixed":{"!type":"bool"},"$configStrict":{"!type":"bool"},"autoSizePolicy":{},"done":{"!type":"bool","!doc":"<p>Used only during a layout run, this value indicates that a\nlayout has finished its calculations. This flag is set to true prior to the call to\n<a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a> and should be set to false if this layout has more work to do.</p>\n"},"factoryConfig":{},"initialized":{"!type":"bool"},"isLayout":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Layout, or subclass thereof.</p>\n"},"needsItemSize":{"!type":"bool","!doc":"<p><code>true</code> if this layout may need to incorporate the dimensions of individual child\nitems into its layout calculations.  Layouts that handle the size of their children\nas a group (autocontainer, form) can set this to false for an additional performance\noptimization.  When <code>false</code> the layout system will not recurse into the child\nitems if <a href=\"#!/api/Ext.layout.container.Container-property-activeItemCount\" rel=\"Ext.layout.container.Container-property-activeItemCount\" class=\"docClass\">Ext.layout.container.Container.activeItemCount</a> is <code>0</code>, which will be the case if all child items\nuse \"liquid\" CSS layout, e.g. form fields.  (See <a href=\"#!/api/Ext.Component-property-liquidLayout\" rel=\"Ext.Component-property-liquidLayout\" class=\"docClass\">Ext.Component.liquidLayout</a>)</p>\n"},"running":{"!type":"bool"},"setsItemSize":{"!type":"bool","!doc":"<p><code>true</code> if this layout may set the size of its child items.  Layouts that do not\nset the size of their child items (autocontainer, form) can set this to false\nfor an additional performance optimization.  When <code>true</code> the layout system will\nnot create a context item for children that use liquid layout, because there is\nno need for a context item if item size is neither read nor set by the owning layout.</p>\n"},"afterRemove":{"!type":"fn(item: ?) -> !this","!doc":"<p>Removes layout's itemCls and owning Container's itemCls.\nClears the managed dimensions flags</p>\n"},"completeLayout":{"!type":"fn(ownerContext: ?)","!doc":"<p>This method (if implemented) is called at the end of the cycle in which this layout\ncompletes (by not setting <a href=\"#!/api/Ext.layout.Layout-property-done\" rel=\"Ext.layout.Layout-property-done\" class=\"docClass\">done</a> to <code>false</code> in <a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a>). It is\npossible for the layout to complete and yet become invalid before the end of the cycle,\nin which case, this method will not be called. It is also possible for this method to\nbe called and then later the layout becomes invalidated. This will result in\n<a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a> being called again, followed by another call to this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\nbe flushed at the next opportunity.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"configureItem":{"!type":"fn(item: ?) -> !this","!doc":"<p>Called before an item is rendered to allow the layout to configure the item.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroys this layout. This method removes a <code>targetCls</code> from the <code>target</code>\nelement and calls <code>onDestroy</code>.</p>\n\n<p>A derived class can override either this method or <code>onDestroy</code> but in all\ncases must call the base class versions of these methods to allow the base class to\nperform its cleanup.</p>\n\n<p>This method (or <code>onDestroy</code>) are overridden by subclasses most often to purge\nevent handlers or remove unmanged DOM nodes.</p>\n"},"finalizeLayout":{"!type":"fn(ownerContext: ?)","!doc":"<p>This method (if implemented) is called after all layouts have completed. In most\nways this is similar to <a href=\"#!/api/Ext.layout.Layout-method-completeLayout\" rel=\"Ext.layout.Layout-method-completeLayout\" class=\"docClass\">completeLayout</a>. This call can cause this (or any\nlayout) to be become invalid (see <a href=\"#!/api/Ext.layout.Context-method-invalidate\" rel=\"Ext.layout.Context-method-invalidate\" class=\"docClass\">Ext.layout.Context.invalidate</a>), but this\nis best avoided. This method is intended to be where final reads are made and so it\nis best to avoid invalidating layouts at this point whenever possible. Even so, this\nmethod can be used to perform final checks that may require all other layouts to be\ncomplete and then invalidate some results.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\nbe flushed at the next opportunity.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"finishRender":{"!type":"fn() -> !this"},"finishRenderItems":{"!type":"fn(target: ?, items: ?) -> !this"},"getAnimatePolicy":{"!type":"fn() -> !this"},"getItemLayoutEl":{"!type":"fn(item: ?) -> ?","!doc":"<p>For a given item, returns the element that participates in the childNodes array\nof the layout's target element.  This is usually the component's \"el\", but can\nalso be a wrapper</p>\n"},"getItemSizePolicy":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns an object describing how this layout manages the size of the given component.\nThis method must be implemented by any layout that manages components.</p>\n"},"getItemsRenderTree":{"!type":"fn(items: ?, renderCfgs: ?) -> !this","!doc":"<hr />\n"},"getLayoutItems":{"!type":"fn() -> !this","!doc":"<p>Returns the set of items to layout (empty by default).</p>\n"},"getPositionOffset":{"!type":"fn(position: ?) -> !this"},"getScrollerEl":{"!type":"fn() -> !this","!doc":"<p>Returns the element that wraps the contents for the purposes of touch scrolling.\nOnly applicable when the layout adds the scroller element as part of its renderTpl\n(e.g. autocontainer and box)</p>\n"},"initLayout":{"!type":"fn() -> !this","!doc":"<p>A one-time initialization method called just before rendering.</p>\n"},"isItemBoxParent":{"!type":"fn(itemContext: ?) -> !this"},"isItemLayoutRoot":{"!type":"fn(item: ?) -> !this"},"isItemShrinkWrap":{"!type":"fn(item: ?) -> !this"},"isRunning":{"!type":"fn() -> !this"},"isValidParent":{"!type":"fn(item: ?, target: ?, position: ?) -> !this","!doc":"<p>Validates item is in the proper place in the dom.</p>\n"},"moveItem":{"!type":"fn(item: ?, target: ?, position: ?) -> !this","!doc":"<p>Moves Component to the provided target instead.</p>\n"},"onAdd":{"!type":"fn(item: ?) -> !this"},"onContentChange":{"!type":"fn(child: ?) -> bool","!doc":"<p>This method is called when a child item changes in some way. By default this calls\n<a href=\"#!/api/Ext.Component-method-updateLayout\" rel=\"Ext.Component-method-updateLayout\" class=\"docClass\">Ext.Component.updateLayout</a> on this layout's owner.</p>\n"},"onDestroy":{"!type":"fn() -> !this"},"onRemove":{"!type":"fn() -> !this"},"renderChildren":{"!type":"fn() -> !this"},"renderItem":{"!type":"fn(item: ?, target: ?, position: ?) -> !this","!doc":"<p>Renders the given Component into the target Element.</p>\n"},"renderItems":{"!type":"fn(items: ?, target: ?) -> !this","!doc":"<p>Iterates over all passed items, ensuring they are rendered.  If the items are already rendered,\nalso determines if the items are in the proper place in the dom.</p>\n"},"setOwner":{"!type":"fn(owner: ?) -> !this","!doc":"<p>Sets the layout owner</p>\n"},"sortWeightedItems":{"!type":"fn(items: ?, reverseProp: ?) -> !this"},"undoLayout":{"!type":"fn() -> !this"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to prepare for layout.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"calculate":{"!type":"fn(ownerContext: ?)","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.layout.Layout-property-done\" rel=\"Ext.layout.Layout-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.layout.Layout-property-done\" rel=\"Ext.layout.Layout-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method is called after all layouts are complete and their calculations flushed\nto the DOM. No further layouts will be run and this method is only called once per\nlayout run. The base component layout caches <code>lastComponentSize</code>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"notifyOwner":{"!type":"fn(ownerContext: ?)","!doc":"<p>This method (if implemented) is called after all layouts are finished, and all have\na <code>lastComponentSize</code> cached. No further layouts will be run and this method is only\ncalled once per layout run. It is the bookend to <a href=\"#!/api/Ext.layout.Layout-method-beginLayout\" rel=\"Ext.layout.Layout-method-beginLayout\" class=\"docClass\">beginLayout</a>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method. This is the catch-all tail method to a layout and so the rules are more\nrelaxed. Even so, for performance reasons, it is best to avoid reading the DOM. If\na read is necessary, consider implementing a <a href=\"#!/api/Ext.layout.Layout-method-finalizeLayout\" rel=\"Ext.layout.Layout-method-finalizeLayout\" class=\"docClass\">finalizeLayout</a> method to do the\nrequired reads.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"}},"!doc":"<p>This class is the base for all layout types: component and container.</p>\n","!type":"fn(config: ?)"},"container":{"Box":{"prototype":{"align":{"!type":"string","!doc":"<p>Controls how the child items of the container are aligned. The value is used to\nposition items \"perpendicularly\". That is, for horizontal boxes (where <code>vertical</code>\nis <code>false</code>), then this will position items vertically. Otherwise, this will position\nitems horizontally. The acceptable values for this property are best explained in\ncontext with the value of <code>vertical</code>.</p>\n\n<p>If <code>vertical</code> is <code>false</code> then this layout is behaving as an <code>hbox</code> and this config\noperates as follows:</p>\n\n<ul>\n<li><strong>begin</strong> : Child items are aligned vertically at the top of the container.</li>\n<li><strong>middle</strong> : Child items are vertically centered in the container.</li>\n<li><strong>end</strong> : Child items are aligned vertically at the bottom of the container.</li>\n<li><strong>stretch</strong> : Child items are stretched vertically to fill the height of the container.</li>\n<li><strong>stretchmax</strong> : Child items are stretched vertically to the height of the largest item.</li>\n</ul>\n\n\n<p>If <code>vertical</code> is <code>true</code> then this layout is behaving as an <code>vbox</code> and this config\noperates as follows:</p>\n\n<ul>\n<li><strong>begin</strong> : Child items are aligned horizontally at the left side of the container.</li>\n<li><strong>middle</strong> : Child items are horizontally centered in the container.</li>\n<li><strong>end</strong> : Child items are aligned horizontally at the right of the container.</li>\n<li><strong>stretch</strong> : Child items are stretched horizontally to fill the width of the container.</li>\n<li><strong>stretchmax</strong> : Child items are stretched horizontally to the size of the largest item.</li>\n</ul>\n\n\n<p>For backwards compatibility, the following values are also recognized:</p>\n\n<ul>\n<li><strong>left</strong> : Same as <strong>begin</strong>.</li>\n<li><strong>top</strong> : Same as <strong>begin</strong>.</li>\n<li><strong>center</strong> : Same as <strong>middle</strong>.</li>\n<li><strong>right</strong> : Samas as <strong>end</strong>.</li>\n<li><strong>bottom</strong> : Samas as <strong>end</strong>.</li>\n</ul>\n\n"},"alignRoundingMethod":{"!doc":"<p>The Math method to\nuse for rounding fractional pixels when <code><a href=\"#!/api/Ext.layout.container.Box-cfg-align\" rel=\"Ext.layout.container.Box-cfg-align\" class=\"docClass\">align</a>:middle</code> is used.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"constrainAlign":{"!type":"bool","!doc":"<p>Limits the size of <a href=\"#!/api/Ext.layout.container.Box-cfg-align\" rel=\"Ext.layout.container.Box-cfg-align\" class=\"docClass\">aligned</a> components to the size of the container\nunder certain circumstances. Firstly, the container's height (for <code>hbox</code>) or width\n(for <code>vbox</code>) must not be determined by the size of the child components. Secondly,\nthe child components must have <a href=\"#!/api/Ext.Component-cfg-shrinkWrap\" rel=\"Ext.Component-cfg-shrinkWrap\" class=\"docClass\">shrinkwrap</a>\nenabled for this dimension.</p>\n"},"enableSplitters":{"!type":"bool","!doc":"<p>This flag can be set to <code>false</code> to ignore the <code>split</code> config on box items. This is\nset to <code>false</code> by <code><a href=\"#!/api/Ext.layout.container.Accordion\" rel=\"Ext.layout.container.Accordion\" class=\"docClass\">Ext.layout.container.Accordion</a></code>.</p>\n"},"flex":{"!type":"number","!doc":"<p>This configuration option is to be applied to <strong>child items</strong> of the container\nmanaged by this layout. Each child item with a flex property will be flexed\n(horizontally in <code>hbox</code>, vertically in <code>vbox</code>) according to each item's\n<strong>relative</strong> flex value compared to the sum of all items with a flex value\nspecified. Any child items that have either a <code>flex = 0</code> or <code>flex = undefined</code>\nwill not be 'flexed' (the initial size will not be changed).</p>\n"},"itemCls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to the container. This can be useful for\nadding customized styles to the container or any of its children using standard CSS\nrules. See <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.<a href=\"#!/api/Ext.Component-cfg-componentCls\" rel=\"Ext.Component-cfg-componentCls\" class=\"docClass\">componentCls</a> also.</p>\n"},"overflowHandler":{"!doc":"<p>An overflow handler or config object for an overflow handler.  This is typically\nspecified as one of the following strings:</p>\n\n<ul>\n<li><code>scroller</code> - Scroller buttons are rendered before and after the content.</li>\n<li><code>menu</code> - Overflowing items are rendered into a menu, and a button is rendered\n after the items, which shows the menu when clicked.</li>\n</ul>\n\n\n<p>NOTE: This config is currently only supported when box layout is used by the\nfollowing components:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Ext.toolbar.Toolbar</a></li>\n<li><a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a></li>\n<li><a href=\"#!/api/Ext.toolbar.Breadcrumb\" rel=\"Ext.toolbar.Breadcrumb\" class=\"docClass\">Ext.toolbar.Breadcrumb</a></li>\n<li><a href=\"#!/api/Ext.tab.Bar\" rel=\"Ext.tab.Bar\" class=\"docClass\">Ext.tab.Bar</a></li>\n</ul>\n\n\n<p>Components where <code>overflowHandler</code> is not supported should use\n<code><a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">autoScroll</a>:true</code> if they have overflowing\ncontent.</p>\n"},"pack":{"!type":"string","!doc":"<p>Controls how the child items of the container are packed together. Acceptable\nconfiguration values for this property are:</p>\n\n<ul>\n<li><strong>start</strong> - child items are packed together at <strong>left</strong> (HBox) or <strong>top</strong>\n(VBox) side of container (*default**)</li>\n<li><strong>center</strong> - child items are packed together at <strong>mid-width</strong> (HBox) or\n<strong>mid-height</strong> (VBox) of container</li>\n<li><strong>end</strong> - child items are packed together at <strong>right</strong> (HBox) or <strong>bottom</strong>\n(VBox) side of container</li>\n</ul>\n\n"},"padding":{"!type":"string","!doc":"<p>Sets the padding to be applied to all child items managed by this layout.</p>\n\n<p>This property must be specified as a string containing space-separated, numeric\npadding values. The order of the sides associated with each value matches the\nway CSS processes padding values:</p>\n\n<ul>\n<li>If there is only one value, it applies to all sides.</li>\n<li>If there are two values, the top and bottom borders are set to the first\nvalue and the right and left are set to the second.</li>\n<li>If there are three values, the top is set to the first value, the left and\nright are set to the second, and the bottom is set to the third.</li>\n<li>If there are four values, they apply to the top, right, bottom, and left,\nrespectively.</li>\n</ul>\n\n"},"stretchMaxPartner":{"!doc":"<p>Allows stretchMax calculation to take into account the max perpendicular size\n(height for HBox layout and width for VBox layout) of another Box layout when\ncalculating its maximum perpendicular child size.</p>\n\n<p>If specified as a string, this may be either a known Container ID, or a\nComponentQuery selector which is rooted at this layout's Container (ie, to find\na sibling, use <code>\"^&gt;#siblingItemId</code>).</p>\n"},"_beginRe":{},"_centerRe":{},"_endRe":{},"_percentageRe":{"!doc":"<p>Matches: <code>&lt;spaces&gt;digits[.digits]&lt;spaces&gt;%&lt;spaces&gt;</code>\nCaptures: <code>digits[.digits]</code></p>\n"},"_props":{},"availableSpaceOffset":{"!type":"number","!doc":"<p>availableSpaceOffset is used to adjust the availableWidth, typically used\nto reserve space for a scrollbar</p>\n"},"calculateChildBox":{"!doc":"<p><debug></debug></p>\n"},"calculateChildBoxes":{},"createsInnerCt":{"!type":"bool"},"innerCls":{"!type":"string"},"manageMargins":{"!type":"bool"},"renderTpl":{},"targetCls":{"!type":"string"},"targetElCls":{"!type":"string"},"updateChildBoxes":{},"applyOverflowHandler":{"!type":"fn(overflowHandler: ?, oldOverflowHandler: ?) -> !this"},"beginCollapse":{"!type":"fn(child: ?) -> !this","!doc":"<p>Called by an owning Panel before the Panel begins its collapse process.\nMost layouts will not need to override the default Ext.emptyFn implementation.</p>\n"},"beginExpand":{"!type":"fn(child: ?) -> !this","!doc":"<p>Called by an owning Panel before the Panel begins its expand process.\nMost layouts will not need to override the default Ext.emptyFn implementation.</p>\n"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?, firstCycle: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"cacheFlexes":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method is called to (re)cache our understanding of flexes. This happens during beginLayout and may need to\nbe called again if the flexes are changed during the layout (e.g., like ColumnLayout).</p>\n"},"calculateParallel":{"!type":"fn(ownerContext: ?, names: ?, plan: ?) -> !this"},"calculatePerpendicular":{"!type":"fn(ownerContext: ?, names: ?, plan: ?) -> !this"},"calculateStretchMax":{"!type":"fn(ownerContext: ?, names: ?, plan: ?) -> !this"},"destroy":{"!type":"fn() -> !this"},"flexSort":{"!type":"fn(a: ?, b: ?) -> !this"},"getAlign":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-align\" rel=\"Ext.layout.container.Box-cfg-align\" class=\"docClass\">align</a>.</p>\n"},"getAlignRoundingMethod":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-alignRoundingMethod\" rel=\"Ext.layout.container.Box-cfg-alignRoundingMethod\" class=\"docClass\">alignRoundingMethod</a>.</p>\n"},"getConstrainAlign":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-constrainAlign\" rel=\"Ext.layout.container.Box-cfg-constrainAlign\" class=\"docClass\">constrainAlign</a>.</p>\n"},"getElementTarget":{"!type":"fn() -> ?","!doc":"<p>Overridden method from <a href=\"#!/api/Ext.layout.container.Container\" rel=\"Ext.layout.container.Container\" class=\"docClass\">Ext.layout.container.Container</a>.\nUsed by Container classes to insert special DOM elements which must exist in addition to the child components</p>\n"},"getEnableSplitters":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-enableSplitters\" rel=\"Ext.layout.container.Box-cfg-enableSplitters\" class=\"docClass\">enableSplitters</a>.</p>\n"},"getItemSizePolicy":{"!type":"fn(item: ?, ownerSizeModel: ?) -> ?","!doc":"<p>Returns an object describing how this layout manages the size of the given component.\nThis method must be implemented by any layout that manages components.</p>\n"},"getLayoutItems":{"!type":"fn() -> ?","!doc":"<p>Returns an array of child components either for a render phase (Performed in the beforeLayout\nmethod of the layout's base class), or the layout phase (onLayout).</p>\n"},"getOverflowHandler":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-overflowHandler\" rel=\"Ext.layout.container.Box-cfg-overflowHandler\" class=\"docClass\">overflowHandler</a>.</p>\n"},"getPack":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-pack\" rel=\"Ext.layout.container.Box-cfg-pack\" class=\"docClass\">pack</a>.</p>\n"},"getPadding":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-padding\" rel=\"Ext.layout.container.Box-cfg-padding\" class=\"docClass\">padding</a>.</p>\n"},"getRenderData":{"!type":"fn() -> !this"},"getRenderTarget":{"!type":"fn() -> ?","!doc":"<p>Overridden method from <a href=\"#!/api/Ext.layout.container.Container\" rel=\"Ext.layout.container.Container\" class=\"docClass\">Ext.layout.container.Container</a>.\nUsed in the beforeLayout method to render all items into.</p>\n"},"getScrollerEl":{"!type":"fn() -> !this","!doc":"<p>Returns the element that wraps the contents for the purposes of touch scrolling.\nOnly applicable when the layout adds the scroller element as part of its renderTpl\n(e.g. autocontainer and box)</p>\n"},"getStretchMaxPartner":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-stretchMaxPartner\" rel=\"Ext.layout.container.Box-cfg-stretchMaxPartner\" class=\"docClass\">stretchMaxPartner</a>.</p>\n"},"getVertical":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-vertical\" rel=\"Ext.layout.container.Box-cfg-vertical\" class=\"docClass\">vertical</a>.</p>\n"},"insertSplitter":{"!type":"fn(item: ?, index: ?, hidden: ?, splitterCfg: ?) -> !this","!doc":"<p>Inserts the splitter for a given region. A reference to the splitter is also stored\non the component as \"splitter\".</p>\n"},"isItemBoxParent":{"!type":"fn(itemContext: ?) -> !this"},"isItemShrinkWrap":{"!type":"fn(item: ?) -> !this"},"onAdd":{"!type":"fn(item: ?, index: ?) -> !this"},"onAfterConstrainInvalidateChild":{"!type":"fn(childContext: ?, options: ?) -> !this"},"onAfterStretchMaxInvalidateChild":{"!type":"fn(childContext: ?, options: ?) -> !this"},"onBeforeConstrainInvalidateChild":{"!type":"fn(childContext: ?, options: ?) -> !this"},"onBeforeStretchMaxInvalidateChild":{"!type":"fn(childContext: ?, options: ?) -> !this"},"onRemove":{"!type":"fn(comp: ?, isDestroying: ?) -> !this"},"setAlign":{"!type":"fn(align: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-align\" rel=\"Ext.layout.container.Box-cfg-align\" class=\"docClass\">align</a>.</p>\n"},"setAlignRoundingMethod":{"!type":"fn(alignRoundingMethod: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-alignRoundingMethod\" rel=\"Ext.layout.container.Box-cfg-alignRoundingMethod\" class=\"docClass\">alignRoundingMethod</a>.</p>\n"},"setConstrainAlign":{"!type":"fn(constrainAlign: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-constrainAlign\" rel=\"Ext.layout.container.Box-cfg-constrainAlign\" class=\"docClass\">constrainAlign</a>.</p>\n"},"setEnableSplitters":{"!type":"fn(enableSplitters: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-enableSplitters\" rel=\"Ext.layout.container.Box-cfg-enableSplitters\" class=\"docClass\">enableSplitters</a>.</p>\n"},"setOverflowHandler":{"!type":"fn(overflowHandler: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-overflowHandler\" rel=\"Ext.layout.container.Box-cfg-overflowHandler\" class=\"docClass\">overflowHandler</a>.</p>\n"},"setPack":{"!type":"fn(pack: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-pack\" rel=\"Ext.layout.container.Box-cfg-pack\" class=\"docClass\">pack</a>.</p>\n"},"setPadding":{"!type":"fn(padding: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-padding\" rel=\"Ext.layout.container.Box-cfg-padding\" class=\"docClass\">padding</a>.</p>\n"},"setStretchMaxPartner":{"!type":"fn(stretchMaxPartner: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-stretchMaxPartner\" rel=\"Ext.layout.container.Box-cfg-stretchMaxPartner\" class=\"docClass\">stretchMaxPartner</a>.</p>\n"},"setVertical":{"!type":"fn(vertical: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.layout.container.Box-cfg-vertical\" rel=\"Ext.layout.container.Box-cfg-vertical\" class=\"docClass\">vertical</a>.</p>\n"},"updateVertical":{"!type":"fn(vertical: ?) -> !this"},"reserveOffset":{"!type":"bool","!doc":"<p>whether or not to reserve the availableSpaceOffset in layout calculations</p>\n"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.layout.container.Box-property-done\" rel=\"Ext.layout.container.Box-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.layout.container.Box-property-done\" rel=\"Ext.layout.container.Box-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"},"completeLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method (if implemented) is called at the end of the cycle in which this layout\ncompletes (by not setting <a href=\"#!/api/Ext.layout.container.Box-property-done\" rel=\"Ext.layout.container.Box-property-done\" class=\"docClass\">done</a> to <code>false</code> in <a href=\"#!/api/Ext.layout.container.Box-method-calculate\" rel=\"Ext.layout.container.Box-method-calculate\" class=\"docClass\">calculate</a>). It is\npossible for the layout to complete and yet become invalid before the end of the cycle,\nin which case, this method will not be called. It is also possible for this method to\nbe called and then later the layout becomes invalidated. This will result in\n<a href=\"#!/api/Ext.layout.container.Box-method-calculate\" rel=\"Ext.layout.container.Box-method-calculate\" class=\"docClass\">calculate</a> being called again, followed by another call to this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\nbe flushed at the next opportunity.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method is called after all layouts are complete and their calculations flushed\nto the DOM. No further layouts will be run and this method is only called once per\nlayout run. The base component layout caches <code>lastComponentSize</code>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"publishInnerCtSize":{"!type":"fn(ownerContext: ?, reservedSpace: ?) -> !this"},"roundFlex":{"!type":"fn(width: ?) -> !this"},"vertical":{"!type":"bool","!doc":"<p>Set to <code>true</code> to switch the layout to <code>vbox</code>.</p>\n"},"type":{"!type":"string"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>In addition to work done by our base classes, containers benefit from some extra\ncached data. The following properties are added to the ownerContext:</p>\n\n<ul>\n<li>visibleItems: the result of <a href=\"#!/api/Ext.layout.container.Box-method-getVisibleItems\" rel=\"Ext.layout.container.Box-method-getVisibleItems\" class=\"docClass\">getVisibleItems</a></li>\n<li>childItems: the ContextItem[] for each visible item</li>\n<li>targetContext: the ContextItem for the <a href=\"#!/api/Ext.layout.container.Box-method-getTarget\" rel=\"Ext.layout.container.Box-method-getTarget\" class=\"docClass\">getTarget</a> element</li>\n</ul>\n\n"}},"!doc":"<p>Base Class for HBoxLayout and VBoxLayout Classes. Generally it should not need to be used directly.</p>\n","!type":"fn(config: ?)"},"HBox":{"prototype":{"vertical":{"!type":"bool","!doc":"<p>Set to <code>true</code> to switch the layout to <code>vbox</code>.</p>\n"},"type":{"!type":"string"}},"!doc":"<p>A layout that arranges items horizontally across a Container. This layout optionally divides available horizontal\nspace between child items containing a numeric <code>flex</code> configuration.</p>\n\n<p>This layout may also be used to set the heights of child items by configuring it with the <a href=\"#!/api/Ext.layout.container.HBox-cfg-align\" rel=\"Ext.layout.container.HBox-cfg-align\" class=\"docClass\">align</a> option.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.Panel</a>', {\n    width: 500,\n    height: 300,\n    title: \"HBoxLayout Panel\",\n    layout: {\n        type: 'hbox',\n        align: 'stretch'\n    },\n    renderTo: document.body,\n    items: [{\n        xtype: 'panel',\n        title: 'Inner Panel One',\n        flex: 2\n    },{\n        xtype: 'panel',\n        title: 'Inner Panel Two',\n        flex: 1\n    },{\n        xtype: 'panel',\n        title: 'Inner Panel Three',\n        flex: 1\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)"},"Container":{"prototype":{"activeItemCount":{"!type":"number","!doc":"<p>tracks the number of child items that do not use \"liquid\" CSS layout</p>\n"},"animatePolicy":{"!doc":"<p>An object which contains boolean properties specifying which properties are to be\nanimated upon flush of child Component ContextItems. For example, Accordion would\nhave:</p>\n\n<pre><code> {\n     y: true,\n     height: true\n }\n</code></pre>\n"},"usesContainerHeight":{"!type":"bool"},"usesContainerWidth":{"!type":"bool"},"usesHeight":{"!type":"bool"},"usesWidth":{"!type":"bool"},"cacheChildItems":{"!type":"fn(ownerContext: ?) -> !this"},"cacheElements":{"!type":"fn() -> !this"},"configureItem":{"!type":"fn(item: ?) -> !this","!doc":"<p>Adds layout's itemCls and owning Container's itemCls</p>\n"},"doRenderBody":{"!type":"fn(out: ?, renderData: ?) -> !this"},"doRenderContainer":{"!type":"fn(out: ?, renderData: ?) -> !this"},"doRenderItems":{"!type":"fn(out: ?, renderData: ?) -> !this"},"finishRender":{"!type":"fn() -> !this"},"getContentTarget":{"!type":"fn() -> !this"},"getMoveAfterIndex":{"!type":"fn(after: ?) -> !this"},"getPositionOffset":{"!type":"fn(position: ?) -> !this","!doc":"<p>This method is used to offset the DOM position when checking\nwhether the element is a certain child of the target. This is\nrequired in cases where the extra elements prepended to the target\nbefore any of the items. An example of this is when using labelAlign: 'top'\non a field. The label appears first in the DOM before any child items are\ncreated, so when we check the position we need to add an extra offset.\nContainers that create an innerCt are exempt because this new element\npreserves the order</p>\n"},"getRenderTpl":{"!type":"fn() -> !this"},"getRenderTree":{"!type":"fn() -> !this"},"getRenderedItems":{"!type":"fn() -> ?","!doc":"<p>Returns all items that are rendered</p>\n"},"getScrollbarsNeeded":{"!type":"fn(width: ?, height: ?, contentWidth: ?, contentHeight: ?) -> !this"},"getTarget":{"!type":"fn() -> ?","!doc":"<p>Returns the owner component's resize element.</p>\n"},"getVisibleItems":{"!type":"fn() -> ?","!doc":"<p>Returns all items that are both rendered and visible</p>\n"},"moveItemBefore":{"!type":"fn(item: ?, before: ?) -> !this"},"notifyOwner":{"!type":"fn() -> !this","!doc":"<p>Called for every layout in the layout context after all the layouts have been finally flushed</p>\n"},"renderChildren":{"!type":"fn() -> !this"},"setupRenderTpl":{"!type":"fn(renderTpl: ?) -> !this"},"beginCollapse":{"!type":"fn() -> !this","!doc":"<p>Called by an owning Panel before the Panel begins its collapse process.\nMost layouts will not need to override the default Ext.emptyFn implementation.</p>\n"},"beginExpand":{"!type":"fn() -> !this","!doc":"<p>Called by an owning Panel before the Panel begins its expand process.\nMost layouts will not need to override the default Ext.emptyFn implementation.</p>\n"},"destroy":{"!type":"fn() -> !this"},"getLayoutItems":{"!type":"fn() -> ?","!doc":"<p>Returns an array of child components either for a render phase (Performed in the beforeLayout\nmethod of the layout's base class), or the layout phase (onLayout).</p>\n"},"onAdd":{"!type":"fn(item: ?) -> !this"},"onRemove":{"!type":"fn(item: ?) -> !this"},"getContainerSize":{"!type":"fn(ownerContext: ?, inDom?: ?) -> ?","!doc":"<p>Returns the container size (that of the target). Only the fixed-sized dimensions can\nbe returned because the shrinkWrap dimensions are based on the contentWidth/Height\nas determined by the container layout.</p>\n"},"itemCls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to the container. This can be useful for\nadding customized styles to the container or any of its children using standard CSS\nrules. See <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.<a href=\"#!/api/Ext.Component-cfg-componentCls\" rel=\"Ext.Component-cfg-componentCls\" class=\"docClass\">componentCls</a> also.</p>\n"},"renderTpl":{},"beginLayoutCycle":{"!type":"fn(ownerContext: ?, firstCycle: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"getElementTarget":{"!type":"fn() -> ?","!doc":"<p>Returns the element into which extra functional DOM elements can be inserted. Defaults to the owner Component's encapsulating element.</p>\n\n<p>May be overridden in Component layout managers which implement a <a href=\"#!/api/Ext.layout.container.Container-method-getRenderTarget\" rel=\"Ext.layout.container.Container-method-getRenderTarget\" class=\"docClass\">component render target</a> which must only\ncontain child components.</p>\n"},"getRenderData":{"!type":"fn() -> !this"},"getRenderTarget":{"!type":"fn() -> ?","!doc":"<p>Returns the element into which rendering must take place. Defaults to the owner Container's\ntarget element.</p>\n\n<p>May be overridden in layout managers which implement an inner element.</p>\n"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>In addition to work done by our base classes, containers benefit from some extra\ncached data. The following properties are added to the ownerContext:</p>\n\n<ul>\n<li>visibleItems: the result of <a href=\"#!/api/Ext.layout.container.Container-method-getVisibleItems\" rel=\"Ext.layout.container.Container-method-getVisibleItems\" class=\"docClass\">getVisibleItems</a></li>\n<li>childItems: the ContextItem[] for each visible item</li>\n<li>targetContext: the ContextItem for the <a href=\"#!/api/Ext.layout.container.Container-method-getTarget\" rel=\"Ext.layout.container.Container-method-getTarget\" class=\"docClass\">getTarget</a> element</li>\n</ul>\n\n"},"type":{"!type":"string"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.layout.container.Container-property-done\" rel=\"Ext.layout.container.Container-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.layout.container.Container-property-done\" rel=\"Ext.layout.container.Container-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"}},"!doc":"<p>This class is intended to be extended or created via the <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a>\nconfiguration property.  See <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">Ext.container.Container.layout</a> for additional details.</p>\n","!type":"fn()"},"Absolute":{"!doc":"<p>This is a layout that inherits the anchoring of <a href=\"#!/api/Ext.layout.container.Anchor\" rel=\"Ext.layout.container.Anchor\" class=\"docClass\">Ext.layout.container.Anchor</a> and adds the\nability for x/y positioning using the standard x and y component config options.</p>\n\n<p>This class is intended to be extended or created via the <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a>\nconfiguration property.  See <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">Ext.container.Container.layout</a> for additional details.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.form.Panel\" rel=\"Ext.form.Panel\" class=\"docClass\">Ext.form.Panel</a>', {\n    title: 'Absolute Layout',\n    width: 300,\n    height: 275,\n    layout: {\n        type: 'absolute'\n        // layout-specific configs go here\n        //itemCls: 'x-abs-layout-item',\n    },\n    url:'save-form.php',\n    defaultType: 'textfield',\n    items: [{\n        x: 10,\n        y: 10,\n        xtype:'label',\n        text: 'Send To:'\n    },{\n        x: 80,\n        y: 10,\n        name: 'to',\n        anchor:'90%'  // anchor width by percentage\n    },{\n        x: 10,\n        y: 40,\n        xtype:'label',\n        text: 'Subject:'\n    },{\n        x: 80,\n        y: 40,\n        name: 'subject',\n        anchor: '90%'  // anchor width by percentage\n    },{\n        x:0,\n        y: 80,\n        xtype: 'textareafield',\n        name: 'msg',\n        anchor: '100% 100%'  // anchor width and height\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn()","prototype":{"ignoreOnContentChange":{"!type":"bool","!doc":"<p>True indicates that changes to one item in this layout do not effect the layout in\ngeneral. This may need to be set to false if <a href=\"#!/api/Ext.Component-cfg-autoScroll\" rel=\"Ext.Component-cfg-autoScroll\" class=\"docClass\">Ext.Component.autoScroll</a>\nis enabled for the container.</p>\n"},"itemCls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to the container. This can be useful for\nadding customized styles to the container or any of its children using standard CSS\nrules. See <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.<a href=\"#!/api/Ext.Component-cfg-componentCls\" rel=\"Ext.Component-cfg-componentCls\" class=\"docClass\">componentCls</a> also.</p>\n"},"targetCls":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"adjustHeightAnchor":{"!type":"fn(value: ?, childContext: ?) -> !this","!doc":"<p>private</p>\n"},"adjustWidthAnchor":{"!type":"fn(value: ?, childContext: ?) -> !this","!doc":"<p>private</p>\n"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>In addition to work done by our base classes, containers benefit from some extra\ncached data. The following properties are added to the ownerContext:</p>\n\n<ul>\n<li>visibleItems: the result of <a href=\"#!/api/Ext.layout.container.Absolute-method-getVisibleItems\" rel=\"Ext.layout.container.Absolute-method-getVisibleItems\" class=\"docClass\">getVisibleItems</a></li>\n<li>childItems: the ContextItem[] for each visible item</li>\n<li>targetContext: the ContextItem for the <a href=\"#!/api/Ext.layout.container.Absolute-method-getTarget\" rel=\"Ext.layout.container.Absolute-method-getTarget\" class=\"docClass\">getTarget</a> element</li>\n</ul>\n\n"},"calculateContentSize":{"!type":"fn(ownerContext: ?, dimensions: ?) -> !this"},"isItemBoxParent":{"!type":"fn(itemContext: ?) -> !this"},"isItemLayoutRoot":{"!type":"fn(item: ?) -> !this"},"isItemShrinkWrap":{"!type":"fn(item: ?) -> !this"},"onContentChange":{"!type":"fn() -> bool","!doc":"<p>This method is called when a child item changes in some way. By default this calls\n<a href=\"#!/api/Ext.Component-method-updateLayout\" rel=\"Ext.Component-method-updateLayout\" class=\"docClass\">Ext.Component.updateLayout</a> on this layout's owner.</p>\n"}}},"Anchor":{"prototype":{"anchor":{"!type":"string","!doc":"<p>This configuation option is to be applied to <strong>child <code>items</code></strong> of a container managed by\nthis layout (ie. configured with <code>layout:'anchor'</code>).</p>\n\n<p>This value is what tells the layout how an item should be anchored to the container. <code>items</code>\nadded to an AnchorLayout accept an anchoring-specific config property of <strong>anchor</strong> which is a string\ncontaining two values: the horizontal anchor value and the vertical anchor value (for example, '100% 50%').\nThe following types of anchor values are supported:</p>\n\n<ul>\n<li><p><strong>Percentage</strong> : Any value between 1 and 100, expressed as a percentage.</p>\n\n<p>The first anchor is the percentage width that the item should take up within the container, and the\nsecond is the percentage height.  For example:</p>\n\n<pre><code>// two values specified\nanchor: '100% 50%' // render item complete width of the container and\n                   // 1/2 height of the container\n// one value specified\nanchor: '100%'     // the width value; the height will default to auto\n</code></pre></li>\n<li><p><strong>Offsets</strong> : Any positive or negative integer value.</p>\n\n<p>This is a raw adjustment where the first anchor is the offset from the right edge of the container,\nand the second is the offset from the bottom edge. For example:</p>\n\n<pre><code>// two values specified\nanchor: '-50 -100' // render item the complete width of the container\n                   // minus 50 pixels and\n                   // the complete height minus 100 pixels.\n// one value specified\nanchor: '-50'      // anchor value is assumed to be the right offset value\n                   // bottom offset will default to 0\n</code></pre></li>\n<li><p><strong>Sides</strong> : Valid values are <code>right</code> (or <code>r</code>) and <code>bottom</code> (or <code>b</code>).</p>\n\n<p>Either the container must have a fixed size or an anchorSize config value defined at render time in\norder for these to have any effect.</p></li>\n<li><p><strong>Mixed</strong> :</p>\n\n<p>Anchor values can also be mixed as needed.  For example, to render the width offset from the container\nright edge by 50 pixels and 75% of the container's height use:</p>\n\n<pre><code>anchor:   '-50 75%'\n</code></pre></li>\n</ul>\n\n"},"defaultAnchor":{"!type":"string","!doc":"<p>Default anchor for all child <strong>container</strong> items applied if no anchor or specific width is set on the child item.</p>\n"},"anchorFactory":{"!doc":"<p>private</p>\n"},"manageOverflow":{"!type":"bool","!doc":"<p>true to rerun the layout if scrollbars are needed.</p>\n"},"parseAnchorRE":{},"setsItemSize":{"!type":"bool","!doc":"<p>Anchor layout does not read the size of individual items in the shrink-wrapping\ndimension(s) because, as a subclass of autocontainer, it measures them as a whole\nusing an outer element.  However, anchor layout may set the size of its items in\nnon-shrink-wrapping dimension(s).</p>\n"},"sizePolicy":{},"beginLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"calculateItems":{"!type":"fn(ownerContext: ?, containerSize: ?) -> !this"},"configureItem":{"!type":"fn(item: ?) -> !this","!doc":"<p>Adds layout's itemCls and owning Container's itemCls</p>\n"},"getItemSizePolicy":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns an object describing how this layout manages the size of the given component.\nThis method must be implemented by any layout that manages components.</p>\n"},"parseAnchor":{"!type":"fn(a: ?, start: ?, cstart: ?) -> !this"},"sanityCheck":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p><debug></debug></p>\n"},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"adjustHeightAnchor":{"!type":"fn(value: ?, childContext: ?) -> !this","!doc":"<p>private</p>\n"},"adjustWidthAnchor":{"!type":"fn(value: ?, childContext: ?) -> !this","!doc":"<p>private</p>\n"}},"!doc":"<p>This is a layout that enables anchoring of contained elements relative to the container's dimensions.\nIf the container is resized, all anchored items are automatically rerendered according to their\n<code><a href=\"#!/api/Ext.layout.container.Anchor-cfg-anchor\" rel=\"Ext.layout.container.Anchor-cfg-anchor\" class=\"docClass\">anchor</a></code> rules.</p>\n\n<p>This class is intended to be extended or created via the <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a>: 'anchor'\nconfig, and should generally not need to be created directly via the new keyword.</p>\n\n<p>AnchorLayout does not have any direct config options (other than inherited ones). By default,\nAnchorLayout will calculate anchor measurements based on the size of the container itself. However, the\ncontainer using the AnchorLayout can supply an anchoring-specific config property of <code>anchorSize</code>.</p>\n\n<p>If anchorSize is specifed, the layout will use it as a virtual container for the purposes of calculating\nanchor measurements based on it instead, allowing the container to be sized independently of the anchoring\nlogic if necessary.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.Panel</a>', {\n    width: 500,\n    height: 400,\n    title: \"AnchorLayout Panel\",\n    layout: 'anchor',\n    renderTo: Ext.getBody(),\n    items: [\n        {\n            xtype: 'panel',\n            title: '75% Width and 20% Height',\n            anchor: '75% 20%'\n        },\n        {\n            xtype: 'panel',\n            title: 'Offset -300 Width &amp; -200 Height',\n            anchor: '-300 -200'   \n        },\n        {\n            xtype: 'panel',\n            title: 'Mixed Offset and Percent',\n            anchor: '-250 20%'\n        }\n    ]\n});\n</code></pre>\n","!type":"fn()"},"Auto":{"prototype":{"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"reserveScrollbar":{"!type":"bool","!doc":"<p>Set to <code>true</code> to leave space for a vertical scrollbar (if the OS shows space-consuming scrollbars) regardless\nof whether a scrollbar is needed.</p>\n\n<p>This is useful if content height changes during application usage, but you do not want the calculated width\nof child items to change when a scrollbar appears or disappears. The scrollbar will appear in the reserved space,\nand the calculated width of child Components will not change.</p>\n\n<pre class='inline-example '><code>Ext.define('Employee', {\n    extend: 'Ext.data.Model',\n    fields: [\n       {name: 'rating', type: 'int'},\n       {name: 'salary', type: 'float'},\n       {name: 'name'}\n    ]\n});\n\nfunction createFakeData(count) {\n    var firstNames   = ['Ed', 'Tommy', 'Aaron', 'Abe', 'Jamie', 'Adam', 'Dave', 'David', 'Jay', 'Nicolas', 'Nige'],\n        lastNames    = ['Spencer', 'Maintz', 'Conran', 'Elias', 'Avins', 'Mishcon', 'Kaneda', 'Davis', 'Robinson', 'Ferrero', 'White'],\n        ratings      = [1, 2, 3, 4, 5],\n        salaries     = [100, 400, 900, 1500, 1000000];\n\n    var data = [];\n    for (var i = 0; i &lt; (count || 25); i++) {\n        var ratingId    = Math.floor(Math.random() * ratings.length),\n            salaryId    = Math.floor(Math.random() * salaries.length),\n            firstNameId = Math.floor(Math.random() * firstNames.length),\n            lastNameId  = Math.floor(Math.random() * lastNames.length),\n\n            rating      = ratings[ratingId],\n            salary      = salaries[salaryId],\n            name        = Ext.String.format(\"{0} {1}\", firstNames[firstNameId], lastNames[lastNameId]);\n\n        data.push({\n            rating: rating,\n            salary: salary,\n            name: name\n        });\n    }\n    store.loadData(data);\n}\n\n// create the Data Store\nvar store = Ext.create('Ext.data.Store', {\n    id: 'store',\n    model: 'Employee',\n    proxy: {\n        type: 'memory'\n    }\n});\ncreateFakeData(10);\n\nvar grid = Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Grid loaded with varying number of records',\n    anchor: '100%',\n    store: store,\n    columns: [{\n        xtype: 'rownumberer',\n        width: 40,\n        sortable: false\n    },{\n        text: 'Name',\n        flex: 1,\n        sortable: true,\n        dataIndex: 'name'\n    },{\n        text: 'Rating',\n        width: 125,\n        sortable: true,\n        dataIndex: 'rating'\n    },{\n        text: 'Salary',\n        width: 125,\n        sortable: true,\n        dataIndex: 'salary',\n        align: 'right',\n        renderer: Ext.util.Format.usMoney\n    }]\n});\n\nExt.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    renderTo: document.body,\n    width: 800,\n    height: 600,\n    layout: {\n        type: 'anchor',\n        reserveScrollbar: true // There will be a gap even when there's no scrollbar\n    },\n    autoScroll: true,\n    items: grid,\n    tbar: {\n        defaults: {\n            handler: function(b) {\n                createFakeData(b.count);\n            }\n        },\n        items: [{\n             text: '10 Items',\n             count: 10\n        },{\n             text: '100 Items',\n             count: 100\n        },{\n             text: '300 Items',\n             count: 300\n        },{\n             text: '1000 Items',\n             count: 1000\n        },{\n             text: '5000 Items',\n             count: 5000\n        }]\n    }\n});\n</code></pre>\n"},"innerCtCls":{"!type":"string"},"lastOverflowAdjust":{"!doc":"<p>Begin with no previous adjustments</p>\n"},"managePadding":{"!type":"bool","!doc":"<p>indicates that this layout will correct cross browser padding differences when the\ncontainer has overflow.</p>\n\n<p>In some browsers the right and/or bottom padding of a container is lost when\nthe container has overflow.  If managePadding is true the layout will apply the\npadding to an inner wrapping element instead of the container element that has the\noverflow so that paddding will be included in the scrollable area.\nNote: padding will not be managed if it is configured on the container using\na style config or css class.  In order to be managed, padding must be added to the\ncontainer using the appropriate <a href=\"#!/api/Ext.Component-property-contentPaddingProperty\" rel=\"Ext.Component-property-contentPaddingProperty\" class=\"docClass\">contentPaddingProperty</a>.  For <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Panels</a> use\n<a href=\"#!/api/Ext.panel.Panel-cfg-bodyPadding\" rel=\"Ext.panel.Panel-cfg-bodyPadding\" class=\"docClass\">Ext.panel.Panel.bodyPadding</a>, and for\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Containers</a>, use\n<a href=\"#!/api/Ext.Component-cfg-padding\" rel=\"Ext.Component-cfg-padding\" class=\"docClass\">padding</a></p>\n"},"needsItemSize":{"!type":"bool","!doc":"<p>auto layout does not care about the dimensions of individual child items since\nit does not size them, and it measures them as a whole when in shrinkWrap mode.</p>\n"},"outerCtCls":{"!type":"string"},"renderTpl":{"!doc":"<p>Auto layout's renderTpl wraps the content in an outerCt which is used to accomplish\nthe following 3 goals:</p>\n\n<ol>\n<li>When the container has a shrink wrapped width and/or height, the outerCt is used\nto measure the size of the content.</li>\n<li>When the container has overflow some browsers lose the container's right and/or\nbottom padding.  To fix this, the padding is rendered to the outerCt instead of\nthe container target element.  This ensures that the padding is included in the\ncontainer's scrollWidth/scrollHeight. In Old IE when a table is used, the padding\nis rendered to the innerCt td element.</li>\n<li>The outerCt contains the margins of its children, that is to say, it prevents\nthem from collapsing.</li>\n</ol>\n\n"},"beforeLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.layout.container.Auto-property-done\" rel=\"Ext.layout.container.Auto-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.layout.container.Auto-property-done\" rel=\"Ext.layout.container.Auto-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"},"calculateOverflow":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Handles overflow processing for a container.  In addition to the ownerContext\npassed to the <a href=\"#!/api/Ext.layout.container.Auto-method-calculate\" rel=\"Ext.layout.container.Auto-method-calculate\" class=\"docClass\">calculate</a> method, this method also needs the containerSize\n(the object returned by <a href=\"#!/api/Ext.layout.container.Auto-method-getContainerSize\" rel=\"Ext.layout.container.Auto-method-getContainerSize\" class=\"docClass\">getContainerSize</a>).</p>\n"},"completeLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method (if implemented) is called at the end of the cycle in which this layout\ncompletes (by not setting <a href=\"#!/api/Ext.layout.container.Auto-property-done\" rel=\"Ext.layout.container.Auto-property-done\" class=\"docClass\">done</a> to <code>false</code> in <a href=\"#!/api/Ext.layout.container.Auto-method-calculate\" rel=\"Ext.layout.container.Auto-method-calculate\" class=\"docClass\">calculate</a>). It is\npossible for the layout to complete and yet become invalid before the end of the cycle,\nin which case, this method will not be called. It is also possible for this method to\nbe called and then later the layout becomes invalidated. This will result in\n<a href=\"#!/api/Ext.layout.container.Auto-method-calculate\" rel=\"Ext.layout.container.Auto-method-calculate\" class=\"docClass\">calculate</a> being called again, followed by another call to this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\nbe flushed at the next opportunity.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"doRenderBody":{"!type":"fn(out: ?, renderData: ?) -> !this"},"doRenderPadding":{"!type":"fn(out: ?, renderData: ?) -> !this"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method is called after all layouts are complete and their calculations flushed\nto the DOM. No further layouts will be run and this method is only called once per\nlayout run. The base component layout caches <code>lastComponentSize</code>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"getContainerSize":{"!type":"fn(ownerContext: ?, inDom?: ?) -> ?","!doc":"<p>Returns the container size (that of the target). Only the fixed-sized dimensions can\nbe returned because the shrinkWrap dimensions are based on the contentWidth/Height\nas determined by the container layout.</p>\n\n<p>If the <a href=\"#!/api/Ext.layout.container.Auto-method-calculateOverflow\" rel=\"Ext.layout.container.Auto-method-calculateOverflow\" class=\"docClass\">calculateOverflow</a> method is used and if <a href=\"#!/api/Ext.layout.container.Auto-property-manageOverflow\" rel=\"Ext.layout.container.Auto-property-manageOverflow\" class=\"docClass\">manageOverflow</a> is\ntrue, this will adjust the width/height by the size of scrollbars.</p>\n"},"getContentTarget":{"!type":"fn() -> !this"},"getElementTarget":{"!type":"fn() -> ?","!doc":"<p>Overridden method from <a href=\"#!/api/Ext.layout.container.Container\" rel=\"Ext.layout.container.Container\" class=\"docClass\">Ext.layout.container.Container</a>.\nUsed by Container classes to insert special DOM elements which must exist in addition to the child components</p>\n"},"getOverflowXStyle":{"!type":"fn(ownerContext: ?) -> string","!doc":"<p>Returns the overflow-x style of the render target.\nNote: If overflow is configured on a container using style or css class this method\nwill read the dom the first time it is called. It is therefore preferable for\nperformance reasons to use the autoScroll or overflowX config when horizontal\noverflow is desired.</p>\n"},"getOverflowYStyle":{"!type":"fn(ownerContext: ?) -> string","!doc":"<p>Returns the overflow-y style of the render target.\nNote: If overflow is configured on a container using style or css class this method\nwill read the dom the first time it is called. It is therefore preferable for\nperformance reasons to use the autoScroll or overflowY config when vertical\noverflow is desired.</p>\n"},"getRenderData":{"!type":"fn() -> !this"},"getRenderTarget":{"!type":"fn() -> ?","!doc":"<p>Overridden method from <a href=\"#!/api/Ext.layout.container.Container\" rel=\"Ext.layout.container.Container\" class=\"docClass\">Ext.layout.container.Container</a>.\nUsed in the beforeLayout method to render all items into.</p>\n"},"getScrollerEl":{"!type":"fn() -> !this","!doc":"<p>Returns the element that wraps the contents for the purposes of touch scrolling.\nOnly applicable when the layout adds the scroller element as part of its renderTpl\n(e.g. autocontainer and box)</p>\n"},"initContextItems":{"!type":"fn(ownerContext: ?) -> !this"},"initLayout":{"!type":"fn() -> !this","!doc":"<p>A one-time initialization method called just before rendering.</p>\n"},"measureContentHeight":{"!type":"fn(ownerContext: ?) -> !this"},"measureContentWidth":{"!type":"fn(ownerContext: ?) -> !this"},"setCtSizeIfNeeded":{"!type":"fn(ownerContext: ?, containerSize: ?) -> !this","!doc":"<p>This method sets the height and/or width of the outerCt/innerCt to adjust for the\nfollowing browser-specific issues:</p>\n\n<ol>\n<li><p>In some browsers a percentage-height element ignores the horizontal scrollbar\nof its parent (see Ext.supports.PercentageHeightOverflowBug).  If the browser is\naffected by this bug the outerCt needs a pixel height in order to support\npercentage-height children when not shrink-wrapping height. If the browser is not\naffected by this bug, a height of 100% is assigned to the outerCt (see\nbeginLayoutCycle).</p></li>\n<li><p>IE8 mode has a bug with percentage height children.  if the innerCt has\na height of 100%, has padding, and has a child item with a percentage height, that\nchild item will be sized as a percentage of the parent's height plus padding height.\nIn other words, a child with height:50% would have its height caclulated thusly:\n(parentHeight + parentPaddingHeight) * 0.5\nTo fix this, we have to give the innerCt a pixel height.</p></li>\n</ol>\n\n"},"setupRenderTpl":{"!type":"fn(renderTpl: ?) -> !this"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>In addition to work done by our base classes, containers benefit from some extra\ncached data. The following properties are added to the ownerContext:</p>\n\n<ul>\n<li>visibleItems: the result of <a href=\"#!/api/Ext.layout.container.Auto-method-getVisibleItems\" rel=\"Ext.layout.container.Auto-method-getVisibleItems\" class=\"docClass\">getVisibleItems</a></li>\n<li>childItems: the ContextItem[] for each visible item</li>\n<li>targetContext: the ContextItem for the <a href=\"#!/api/Ext.layout.container.Auto-method-getTarget\" rel=\"Ext.layout.container.Auto-method-getTarget\" class=\"docClass\">getTarget</a> element</li>\n</ul>\n\n"},"calculateContentSize":{"!type":"fn(ownerContext: ?) -> !this"},"manageOverflow":{"!type":"bool","!doc":"<p>true to rerun the layout if scrollbars are needed.</p>\n"},"setsItemSize":{"!type":"bool","!doc":"<p><code>true</code> if this layout may set the size of its child items.  Layouts that do not\nset the size of their child items (autocontainer, form) can set this to false\nfor an additional performance optimization.  When <code>true</code> the layout system will\nnot create a context item for children that use liquid layout, because there is\nno need for a context item if item size is neither read nor set by the owning layout.</p>\n"},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"}},"!doc":"<p>The AutoLayout is the default layout manager delegated by <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a> to\nrender any child Components when no <code><a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout</a></code> is configured into\na <code><a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a>.</code> AutoLayout provides only a passthrough of any layout calls\nto any child containers.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.Panel</a>', {\n    width: 500,\n    height: 280,\n    title: \"AutoLayout Panel\",\n    layout: 'auto',\n    renderTo: document.body,\n    items: [{\n        xtype: 'panel',\n        title: 'Top Inner Panel',\n        width: '75%',\n        height: 90\n    },\n    {\n        xtype: 'panel',\n        title: 'Bottom Inner Panel',\n        width: '75%',\n        height: 90\n    }]\n});\n</code></pre>\n","!type":"fn()"},"Accordion":{"!doc":"<p>This is a layout that manages multiple Panels in an expandable accordion style such that by default only\none Panel can be expanded at any given time (set <a href=\"#!/api/Ext.layout.container.Accordion-cfg-multi\" rel=\"Ext.layout.container.Accordion-cfg-multi\" class=\"docClass\">multi</a> config to have more open). Each Panel has\nbuilt-in support for expanding and collapsing.</p>\n\n<p>Note: Only Ext Panels and all subclasses of <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a> may be used in an accordion layout Container.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Accordion Layout',\n    width: 300,\n    height: 300,\n    defaults: {\n        // applied to each contained panel\n        bodyStyle: 'padding:15px'\n    },\n    layout: {\n        // layout-specific configs go here\n        type: 'accordion',\n        titleCollapse: false,\n        animate: true,\n        activeOnTop: true\n    },\n    items: [{\n        title: 'Panel 1',\n        html: 'Panel content!'\n    },{\n        title: 'Panel 2',\n        html: 'Panel content!'\n    },{\n        title: 'Panel 3',\n        html: 'Panel content!'\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn()","prototype":{"activeOnTop":{"!type":"bool","!doc":"<p>Only valid when <a href=\"#!/api/Ext.layout.container.Accordion-cfg-multi\" rel=\"Ext.layout.container.Accordion-cfg-multi\" class=\"docClass\">multi</a> is <code>false</code> and <a href=\"#!/api/Ext.layout.container.Accordion-cfg-animate\" rel=\"Ext.layout.container.Accordion-cfg-animate\" class=\"docClass\">animate</a> is <code>false</code>.</p>\n\n<p>True to swap the position of each panel as it is expanded so that it becomes the first item in the container,\nfalse to keep the panels in the rendered order.</p>\n"},"align":{"!type":"string","!doc":"<p>Controls how the child items of the container are aligned. The value is used to\nposition items \"perpendicularly\". That is, for horizontal boxes (where <code>vertical</code>\nis <code>false</code>), then this will position items vertically. Otherwise, this will position\nitems horizontally. The acceptable values for this property are best explained in\ncontext with the value of <code>vertical</code>.</p>\n\n<p>If <code>vertical</code> is <code>false</code> then this layout is behaving as an <code>hbox</code> and this config\noperates as follows:</p>\n\n<ul>\n<li><strong>begin</strong> : Child items are aligned vertically at the top of the container.</li>\n<li><strong>middle</strong> : Child items are vertically centered in the container.</li>\n<li><strong>end</strong> : Child items are aligned vertically at the bottom of the container.</li>\n<li><strong>stretch</strong> : Child items are stretched vertically to fill the height of the container.</li>\n<li><strong>stretchmax</strong> : Child items are stretched vertically to the height of the largest item.</li>\n</ul>\n\n\n<p>If <code>vertical</code> is <code>true</code> then this layout is behaving as an <code>vbox</code> and this config\noperates as follows:</p>\n\n<ul>\n<li><strong>begin</strong> : Child items are aligned horizontally at the left side of the container.</li>\n<li><strong>middle</strong> : Child items are horizontally centered in the container.</li>\n<li><strong>end</strong> : Child items are aligned horizontally at the right of the container.</li>\n<li><strong>stretch</strong> : Child items are stretched horizontally to fill the width of the container.</li>\n<li><strong>stretchmax</strong> : Child items are stretched horizontally to the size of the largest item.</li>\n</ul>\n\n\n<p>For backwards compatibility, the following values are also recognized:</p>\n\n<ul>\n<li><strong>left</strong> : Same as <strong>begin</strong>.</li>\n<li><strong>top</strong> : Same as <strong>begin</strong>.</li>\n<li><strong>center</strong> : Same as <strong>middle</strong>.</li>\n<li><strong>right</strong> : Samas as <strong>end</strong>.</li>\n<li><strong>bottom</strong> : Samas as <strong>end</strong>.</li>\n</ul>\n\n"},"animate":{"!type":"bool","!doc":"<p>True to slide the contained panels open and closed during expand/collapse using animation, false to open and\nclose directly with no animation. Note: The layout performs animated collapsing\nand expanding, <em>not</em> the child Panels.</p>\n"},"autoWidth":{"!type":"bool","!doc":"<p>Child Panels have their width actively managed to fit within the accordion's width.</p>\n"},"collapseFirst":{"!type":"bool","!doc":"<p>True to make sure the collapse/expand toggle button always renders first (to the left of) any other tools\nin the contained Panels' title bars, false to render it last. By default, this will use the\n<a href=\"#!/api/Ext.panel.Panel-cfg-collapseFirst\" rel=\"Ext.panel.Panel-cfg-collapseFirst\" class=\"docClass\">Ext.panel.Panel.collapseFirst</a> setting on the panel. If the config option is specified on the layout,\nit will override the panel value.</p>\n"},"enableSplitters":{"!type":"bool","!doc":"<p>This flag can be set to <code>false</code> to ignore the <code>split</code> config on box items. This is\nset to <code>false</code> by <code><a href=\"#!/api/Ext.layout.container.Accordion\" rel=\"Ext.layout.container.Accordion\" class=\"docClass\">Ext.layout.container.Accordion</a></code>.</p>\n"},"fill":{"!type":"bool","!doc":"<p>True to adjust the active item's height to fill the available space in the container, false to use the\nitem's current height, or auto height if not explicitly set.</p>\n"},"hideCollapseTool":{"!type":"bool","!doc":"<p>True to hide the contained Panels' collapse/expand toggle buttons, false to display them.\nWhen set to true, <a href=\"#!/api/Ext.layout.container.Accordion-cfg-titleCollapse\" rel=\"Ext.layout.container.Accordion-cfg-titleCollapse\" class=\"docClass\">titleCollapse</a> is automatically set to true.</p>\n"},"itemCls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to the container. This can be useful for\nadding customized styles to the container or any of its children using standard CSS\nrules. See <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.<a href=\"#!/api/Ext.Component-cfg-componentCls\" rel=\"Ext.Component-cfg-componentCls\" class=\"docClass\">componentCls</a> also.</p>\n"},"multi":{"!type":"bool","!doc":"<p>Set to true to enable multiple accordion items to be open at once.</p>\n"},"titleCollapse":{"!type":"bool","!doc":"<p>True to allow expand/collapse of each contained panel by clicking anywhere on the title bar, false to allow\nexpand/collapse only when the toggle tool button is clicked.  When set to false,\n<a href=\"#!/api/Ext.layout.container.Accordion-cfg-hideCollapseTool\" rel=\"Ext.layout.container.Accordion-cfg-hideCollapseTool\" class=\"docClass\">hideCollapseTool</a> should be false also. An explicit <a href=\"#!/api/Ext.panel.Panel-cfg-titleCollapse\" rel=\"Ext.panel.Panel-cfg-titleCollapse\" class=\"docClass\">Ext.panel.Panel.titleCollapse</a> declared\non the panel will override this setting.</p>\n"},"defaultAnimatePolicy":{},"panelCollapseMode":{"!type":"string"},"targetCls":{"!type":"string"},"type":{"!type":"string"},"beforeRenderItems":{"!type":"fn(items: ?) -> !this"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>In addition to work done by our base classes, containers benefit from some extra\ncached data. The following properties are added to the ownerContext:</p>\n\n<ul>\n<li>visibleItems: the result of <a href=\"#!/api/Ext.layout.container.Accordion-method-getVisibleItems\" rel=\"Ext.layout.container.Accordion-method-getVisibleItems\" class=\"docClass\">getVisibleItems</a></li>\n<li>childItems: the ContextItem[] for each visible item</li>\n<li>targetContext: the ContextItem for the <a href=\"#!/api/Ext.layout.container.Accordion-method-getTarget\" rel=\"Ext.layout.container.Accordion-method-getTarget\" class=\"docClass\">getTarget</a> element</li>\n</ul>\n\n"},"configureItem":{"!type":"fn(item: ?) -> !this","!doc":"<p>Adds layout's itemCls and owning Container's itemCls</p>\n"},"getExpanded":{"!type":"fn(explicitCheck: ?) -> !this"},"getItemsRenderTree":{"!type":"fn(items: ?) -> !this","!doc":"<hr />\n"},"onAdd":{"!type":"fn(item: ?) -> !this"},"onBeforeComponentCollapse":{"!type":"fn(comp: ?) -> !this"},"onBeforeComponentExpand":{"!type":"fn(toExpand: ?) -> !this","!doc":"<p>When a Component expands, adjust the heights of the other Components to be just enough to accommodate\ntheir headers.\nThe expanded Component receives the only flex value, and so gets all remaining space.</p>\n"},"onComponentShow":{"!type":"fn(comp: ?) -> !this"},"onRemove":{"!type":"fn(panel: ?, destroying: ?) -> !this"},"renderItems":{"!type":"fn(items: ?, target: ?) -> !this","!doc":"<p>Iterates over all passed items, ensuring they are rendered.  If the items are already rendered,\nalso determines if the items are in the proper place in the dom.</p>\n"},"updatePanelClasses":{"!type":"fn(ownerContext: ?) -> !this"}}},"VBox":{"prototype":{"vertical":{"!type":"bool","!doc":"<p>Set to <code>true</code> to switch the layout to <code>vbox</code>.</p>\n"},"type":{"!type":"string"}},"!doc":"<p>A layout that arranges items vertically down a Container. This layout optionally divides available vertical space\nbetween child items containing a numeric <code>flex</code> configuration.</p>\n\n<p>This layout may also be used to set the widths of child items by configuring it with the <a href=\"#!/api/Ext.layout.container.VBox-cfg-align\" rel=\"Ext.layout.container.VBox-cfg-align\" class=\"docClass\">align</a> option.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.Panel</a>', {\n    width: 500,\n    height: 400,\n    title: \"VBoxLayout Panel\",\n    layout: {\n        type: 'vbox',\n    },\n    renderTo: document.body,\n    items: [{\n        xtype: 'panel',\n        title: 'Inner Panel One',\n        width: 250,\n        flex: 2\n    },\n    {\n        xtype: 'panel',\n        title: 'Inner Panel Two',\n        width: 250,\n        flex: 4\n    },\n    {\n        xtype: 'panel',\n        title: 'Inner Panel Three',\n        width: '50%',\n        flex: 4\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)"},"Border":{"!doc":"<p>This is a multi-pane, application-oriented UI layout style that supports multiple nested panels, automatic bars\nbetween regions and built-in <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">expanding and collapsing</a> of regions.</p>\n\n<p>This class is intended to be extended or created via the <code>layout:'border'</code> <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">Ext.container.Container.layout</a>\nconfig, and should generally not need to be created directly via the new keyword.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    width: 500,\n    height: 300,\n    title: 'Border Layout',\n    layout: 'border',\n    items: [{\n        title: 'South Region is resizable',\n        region: 'south',     // position for region\n        xtype: 'panel',\n        height: 100,\n        split: true,         // enable resizing\n        margin: '0 5 5 5'\n    },{\n        // xtype: 'panel' implied by default\n        title: 'West Region is collapsible',\n        region:'west',\n        xtype: 'panel',\n        margin: '5 0 0 5',\n        width: 200,\n        collapsible: true,   // make collapsible\n        id: 'west-region-container',\n        layout: 'fit'\n    },{\n        title: 'Center Region',\n        region: 'center',     // center region is required, no width/height specified\n        xtype: 'panel',\n        layout: 'fit',\n        margin: '5 5 0 0'\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<h1>Notes</h1>\n\n<ul>\n<li><p>When using the split option, the layout will automatically insert a <a href=\"#!/api/Ext.resizer.Splitter\" rel=\"Ext.resizer.Splitter\" class=\"docClass\">Ext.resizer.Splitter</a>\ninto the appropriate place. This will modify the underlying\n<a href=\"#!/api/Ext.container.Container-property-items\" rel=\"Ext.container.Container-property-items\" class=\"docClass\">items</a> collection in the container.</p></li>\n<li><p>Any Container using the Border layout <strong>must</strong> have a child item with <code>region:'center'</code>.\nThe child item in the center region will always be resized to fill the remaining space\nnot used by the other regions in the layout.</p></li>\n<li><p>Any child items with a region of <code>west</code> or <code>east</code> may be configured with either an initial\n<code>width</code>, or a <a href=\"#!/api/Ext.layout.container.Box-cfg-flex\" rel=\"Ext.layout.container.Box-cfg-flex\" class=\"docClass\">Ext.layout.container.Box.flex</a> value, or an initial percentage width\n<strong>string</strong> (Which is simply divided by 100 and used as a flex value).\nThe 'center' region has a flex value of <code>1</code>.</p></li>\n<li><p>Any child items with a region of <code>north</code> or <code>south</code> may be configured with either an initial\n<code>height</code>, or a <a href=\"#!/api/Ext.layout.container.Box-cfg-flex\" rel=\"Ext.layout.container.Box-cfg-flex\" class=\"docClass\">Ext.layout.container.Box.flex</a> value, or an initial percentage height\n<strong>string</strong> (Which is simply divided by 100 and used as a flex value).\nThe 'center' region has a flex value of <code>1</code>.</p></li>\n<li><p><strong>There is no BorderLayout.Region class in ExtJS 4.0+</strong></p></li>\n</ul>\n\n","!type":"fn()","prototype":{"itemCls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to the container. This can be useful for\nadding customized styles to the container or any of its children using standard CSS\nrules. See <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.<a href=\"#!/api/Ext.Component-cfg-componentCls\" rel=\"Ext.Component-cfg-componentCls\" class=\"docClass\">componentCls</a> also.</p>\n"},"padding":{"!doc":"<p>Sets the padding to be applied to all child items managed by this layout.</p>\n\n<p>This property can be specified as a string containing space-separated, numeric\npadding values. The order of the sides associated with each value matches the way\nCSS processes padding values:</p>\n\n<ul>\n<li>If there is only one value, it applies to all sides.</li>\n<li>If there are two values, the top and bottom borders are set to the first value\nand the right and left are set to the second.</li>\n<li>If there are three values, the top is set to the first value, the left and right\nare set to the second, and the bottom is set to the third.</li>\n<li>If there are four values, they apply to the top, right, bottom, and left,\nrespectively.</li>\n</ul>\n\n"},"regionWeights":{"!doc":"<p>The default weights to assign to regions in the border layout. These values are\nused when a region does not contain a <code>weight</code> property. This object must have\nproperties for all regions (\"north\", \"south\", \"east\" and \"west\").</p>\n\n<p><strong>IMPORTANT:</strong> Since this is an object, changing its properties will impact ALL\ninstances of Border layout. If this is not desired, provide a replacement object as\na config option instead:</p>\n\n<pre><code> layout: {\n     type: 'border',\n     regionWeights: {\n         west: 20,\n         north: 10,\n         south: -10,\n         east: -20\n     }\n }\n</code></pre>\n\n<p>The region with the highest weight is assigned space from the border before other\nregions. Regions of equal weight are assigned space based on their position in the\nowner's items list (first come, first served).</p>\n"},"split":{"!doc":"<p>This configuration option is to be applied to the <strong>child <code>items</code></strong> managed by this layout.\nEach region with <code>split:true</code> will get a <a href=\"#!/api/Ext.resizer.BorderSplitter\" rel=\"Ext.resizer.BorderSplitter\" class=\"docClass\">Splitter</a> that\nallows for manual resizing of the container. Except for the <code>center</code> region.</p>\n\n<p>This option can also accept an object of configurations from the <a href=\"#!/api/Ext.resizer.BorderSplitter\" rel=\"Ext.resizer.BorderSplitter\" class=\"docClass\">Ext.resizer.BorderSplitter</a>.\nAn example of this would be:</p>\n\n<pre><code>{\n    title: 'North',\n    region: 'north',\n    height: 100,\n    collapsible: true,\n    split: {\n        size: 20\n    }\n}\n</code></pre>\n"},"splitterResize":{"!type":"bool","!doc":"<p>This configuration option is to be applied to the <strong>child <code>items</code></strong> managed by this layout and\nis used in conjunction with <a href=\"#!/api/Ext.layout.container.Border-cfg-split\" rel=\"Ext.layout.container.Border-cfg-split\" class=\"docClass\">split</a>. By default, when specifying <a href=\"#!/api/Ext.layout.container.Border-cfg-split\" rel=\"Ext.layout.container.Border-cfg-split\" class=\"docClass\">split</a>, the region\ncan be dragged to be resized. Set this option to false to show the split bar but prevent resizing.</p>\n"},"axisProps":{"!doc":"<p>Reused meta-data objects that describe axis properties.</p>\n"},"centerRegion":{},"horzPositionProp":{"!type":"string"},"isBorderLayout":{"!type":"bool"},"manageMargins":{"!type":"bool"},"padNotOnContainerProp":{"!type":"string"},"padOnContainerProp":{"!type":"string"},"panelCollapseAnimate":{"!type":"bool"},"panelCollapseMode":{"!type":"string"},"percentageRe":{},"regionFlags":{"!doc":"<p>Flags and configs that get set of regions based on their <code>region</code> property.</p>\n"},"regionMeta":{"!doc":"<hr />\n\n<p> Misc</p>\n"},"sizePolicies":{},"targetCls":{"!type":"string"},"touchedRegions":{"!doc":"<p>Lists the regions that would consider an interior region a neighbor. For example,\na north region would consider an east or west region its neighbords (as well as\nan inner north region).</p>\n"},"type":{"!type":"string"},"beginAxis":{"!type":"fn(ownerContext: ?, regions: ?, name: ?) -> !this","!doc":"<p>Creates the axis objects for the layout. These are only missing size information\nwhich is added during <a href=\"#!/api/Ext.layout.container.Border-method-calculate\" rel=\"Ext.layout.container.Border-method-calculate\" class=\"docClass\">calculate</a>.</p>\n"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>In addition to work done by our base classes, containers benefit from some extra\ncached data. The following properties are added to the ownerContext:</p>\n\n<ul>\n<li>visibleItems: the result of <a href=\"#!/api/Ext.layout.container.Border-method-getVisibleItems\" rel=\"Ext.layout.container.Border-method-getVisibleItems\" class=\"docClass\">getVisibleItems</a></li>\n<li>childItems: the ContextItem[] for each visible item</li>\n<li>targetContext: the ContextItem for the <a href=\"#!/api/Ext.layout.container.Border-method-getTarget\" rel=\"Ext.layout.container.Border-method-getTarget\" class=\"docClass\">getTarget</a> element</li>\n</ul>\n\n"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.layout.container.Border-property-done\" rel=\"Ext.layout.container.Border-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.layout.container.Border-property-done\" rel=\"Ext.layout.container.Border-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"},"calculateChildAxis":{"!type":"fn(childContext: ?, axis: ?) -> !this","!doc":"<p>Performs the calculations for a region on a specified axis.</p>\n"},"eachItem":{"!type":"fn(region: ?, fn: ?, scope: ?) -> !this"},"finishAxis":{"!type":"fn(ownerContext: ?, axis: ?) -> !this","!doc":"<p>Finishes the calculations on an axis. This basically just assigns the remaining\nspace to the center region.</p>\n"},"finishPositions":{"!type":"fn(childItems: ?) -> !this","!doc":"<p>Finishes by setting the positions on the child items.</p>\n"},"getItemSizePolicy":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns an object describing how this layout manages the size of the given component.\nThis method must be implemented by any layout that manages components.</p>\n"},"getLayoutItems":{"!type":"fn() -> ?","!doc":"<p>Returns an array of child components either for a render phase (Performed in the beforeLayout\nmethod of the layout's base class), or the layout phase (onLayout).</p>\n"},"getMaxWeight":{"!type":"fn(region: ?) -> !this"},"getMinMaxWeight":{"!type":"fn(region: ?, min: ?) -> !this"},"getMinWeight":{"!type":"fn(region: ?) -> !this"},"getMoveAfterIndex":{"!type":"fn(after: ?) -> !this"},"getPlaceholder":{"!type":"fn(comp: ?) -> !this"},"getSplitterTarget":{"!type":"fn(splitter: ?) -> !this"},"insertSplitter":{"!type":"fn(item: ?, index: ?, hidden: ?, splitterCfg: ?) -> !this","!doc":"<p>Inserts the splitter for a given region. A reference to the splitter is also stored\non the component as \"splitter\".</p>\n"},"isItemBoxParent":{"!type":"fn(itemContext: ?) -> !this"},"isItemShrinkWrap":{"!type":"fn(item: ?) -> !this"},"moveItemBefore":{"!type":"fn(item: ?, before: ?) -> !this"},"onAdd":{"!type":"fn(item: ?, index: ?) -> !this","!doc":"<p>Called when a region (actually when any component) is added to the container. The\nregion is decorated with some helpful properties (isCenter, isHorz, isVert) and its\nsplitter is added if its \"split\" property is true.</p>\n"},"onDestroy":{"!type":"fn() -> !this"},"onRemove":{"!type":"fn(comp: ?, isDestroying: ?) -> !this"},"setupSplitterNeighbors":{"!type":"fn(items: ?) -> !this"}}},"boxOverflow":{"Menu":{"!type":"fn(config: ?)","prototype":{"menu":{"!doc":"<p>The expand menu - holds items for every item that cannot be shown\nbecause the container is currently not large enough.</p>\n"},"menuCls":{"!type":"string"},"menuItems":{"!doc":"<p>Array of all items that are currently hidden and should go into the dropdown menu</p>\n"},"menuTrigger":{"!doc":"<p>The expand button which triggers the overflow menu to be shown</p>\n"},"noItemsMenuText":{"!type":"string","!doc":"<p>HTML fragment to render into the toolbar overflow menu if there are no items to display</p>\n"},"addComponentToMenu":{"!type":"fn(menu: ?, component: ?) -> !this","!doc":"<p>Adds the given Toolbar item to the given menu. Buttons inside a buttongroup are added individually.</p>\n"},"beforeMenuShow":{"!type":"fn(menu: ?) -> !this","!doc":"<p>Called before the overflow menu is shown. This constructs the menu's items, caching them for as long as it can.</p>\n"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?, firstCycle: ?) -> !this"},"captureChildElements":{"!type":"fn() -> !this","!doc":"<p>Finishes the render operation of the trigger Button.</p>\n"},"clearMenu":{"!type":"fn() -> !this","!doc":"<p>Deletes the sub-menu of each item in the expander menu. Submenus are created for items such as\nsplitbuttons and buttongroups, where the Toolbar item cannot be represented by a single menu item</p>\n"},"clearOverflow":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called by the layout, when it determines that there is no overflow.\nAlso called as an interceptor to the layout's onLayout method to reshow\npreviously hidden overflowing items.</p>\n"},"createMenuConfig":{"!type":"fn(component: ?, hideOnClick: ?) -> !this","!doc":"<p>Returns a menu config for a given component. This config is used to create a menu item\nto be added to the expander menu</p>\n"},"destroy":{"!type":"fn() -> !this"},"getOverflowCls":{"!type":"fn(direction: ?) -> !this"},"getSuffixConfig":{"!type":"fn() -> !this","!doc":"<p>We don't define a prefix in menu overflow.</p>\n"},"handleOverflow":{"!type":"fn(ownerContext: ?) -> !this"},"hideTrigger":{"!type":"fn() -> !this"},"onButtonAttrChange":{"!type":"fn(btn: ?) -> !this"},"onButtonToggle":{"!type":"fn(btn: ?, state: ?) -> !this"},"onRemove":{"!type":"fn(comp: ?) -> !this"},"showTrigger":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Shows the overflow trigger when enableOverflow is set to true and the items\nin the layout are too wide to fit in the space available</p>\n"}}},"None":{"prototype":{"$configPrefixed":{"!type":"bool"},"$configStrict":{"!type":"bool"},"factoryConfig":{},"isBoxOverflowHandler":{"!type":"bool"},"calculate":{"!type":"fn(ownerContext: ?) -> !this"},"completeLayout":{"!type":"fn() -> !this"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this"},"getItem":{"!type":"fn(item: ?) -> ?","!doc":"<p>Normalizes an item reference, string id or numerical index into a reference to the item</p>\n"},"getOwnerType":{"!type":"fn(owner: ?) -> !this"},"getPrefixConfig":{"!type":"fn() -> !this"},"setVertical":{"!type":"fn() -> !this"},"beginLayout":{"!type":"fn() -> !this"},"beginLayoutCycle":{"!type":"fn() -> !this"},"clearOverflow":{"!type":"fn() -> !this"},"getOverflowCls":{"!type":"fn() -> !this"},"getSuffixConfig":{"!type":"fn() -> !this"},"handleOverflow":{"!type":"fn() -> !this"},"onRemove":{"!type":"fn() -> !this"}},"!doc":"<p>Base class for Box Layout overflow handlers. These specialized classes are invoked when a Box Layout\n(either an HBox or a VBox) has child items that are either too wide (for HBox) or too tall (for VBox)\nfor its container.</p>\n","!type":"fn(config: ?)"},"Scroller":{"!type":"fn(config: ?)","prototype":{"animateScroll":{"!type":"bool","!doc":"<p>True to animate the scrolling of items within the layout (ignored if enableScroll is false)</p>\n"},"scrollDuration":{"!type":"number","!doc":"<p>Number of milliseconds that each scroll animation lasts</p>\n"},"scrollIncrement":{"!type":"number","!doc":"<p>The number of pixels to scroll by on scroller click</p>\n"},"scrollRepeatInterval":{"!type":"number","!doc":"<p>Number of milliseconds between each scroll while a scroller button is held down</p>\n"},"wheelIncrement":{"!type":"number","!doc":"<p>The number of pixels to increment on mouse wheel scrolling.</p>\n"},"afterSuffix":{"!type":"string"},"beforeSuffix":{"!type":"string"},"scrollerCls":{"!type":"string","!doc":"<p>private</p>\n"},"atExtremeAfter":{"!type":"fn() -> bool","!doc":"<p>Returns true if the innerCt scroll is already at its right-most point</p>\n"},"atExtremeBefore":{"!type":"fn() -> bool","!doc":"<p>Returns true if the innerCt scroll is already at its left-most point</p>\n"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this"},"clearOverflow":{"!type":"fn() -> !this"},"createScroller":{"!type":"fn(suffix: ?, repeaterName: ?, scrollHandler: ?) -> !this"},"createScrollerCls":{"!type":"fn(xName: ?) -> !this"},"createWheelListener":{"!type":"fn() -> !this","!doc":"<p>Sets up an listener to scroll on the layout's innerCt mousewheel event</p>\n"},"destroy":{"!type":"fn() -> !this"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this"},"getAfterScroller":{"!type":"fn() -> !this","!doc":"<p>Returns a reference to the \"after\" scroller element.  Creates click handlers on\nthe first call.</p>\n"},"getBeforeScroller":{"!type":"fn() -> !this","!doc":"<p>Returns a reference to the \"before\" scroller element.  Creates click handlers on\nthe first call.</p>\n"},"getItemVisibility":{"!type":"fn(item: ?) -> ?","!doc":"<p>For a given item in the container, return an object with information on whether the item is visible\nwith the current innerCt scroll value.</p>\n"},"getMaxScrollPosition":{"!type":"fn() -> number","!doc":"<p>Returns the maximum value we can scrollTo</p>\n"},"getOverflowCls":{"!type":"fn(direction: ?) -> !this"},"getPrefixConfig":{"!type":"fn() -> !this"},"getScrollAnim":{"!type":"fn() -> ?"},"getScrollPosition":{"!type":"fn() -> number","!doc":"<p>Returns the current scroll position of the innerCt element</p>\n"},"getSuffixConfig":{"!type":"fn() -> !this"},"getWheelDelta":{"!type":"fn(e: ?) -> !this"},"handleOverflow":{"!type":"fn(ownerContext: ?) -> !this"},"hideScrollers":{"!type":"fn() -> !this","!doc":"<p>Hides the scroller elements.</p>\n"},"scrollBy":{"!type":"fn(delta: ?) -> !this","!doc":"<p>Scrolls left or right by the number of pixels specified</p>\n"},"scrollLeft":{"!type":"fn() -> !this","!doc":"<p>Scrolls to the left by the configured amount</p>\n"},"scrollRight":{"!type":"fn() -> !this","!doc":"<p>Scrolls to the right by the configured amount</p>\n"},"scrollTo":{"!type":"fn(position: ?, animate: ?) -> !this","!doc":"<p>Scrolls to the given position. Performs bounds checking.</p>\n"},"scrollToItem":{"!type":"fn(item: ?, animate: ?) -> !this","!doc":"<p>Scrolls to the given component.</p>\n"},"setVertical":{"!type":"fn() -> !this"},"showScrollers":{"!type":"fn() -> !this","!doc":"<p>Shows the scroller elements. Creates the scrollers first if they are not already present.</p>\n"},"updateScrollButtons":{"!type":"fn() -> !this","!doc":"<p>Enables or disables each scroller button based on the current scroll position</p>\n"},"scroll":{"!type":"fn(scroller: ?, newPosition: ?, animate: ?, eOpts: ?)"}}}},"Card":{"!doc":"<p>This layout manages multiple child Components, each fitted to the Container, where only a single child Component can be\nvisible at any given time.  This layout style is most commonly used for wizards, tab implementations, etc.\nThis class is intended to be extended or created via the layout:'card' <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">Ext.container.Container.layout</a> config,\nand should generally not need to be created directly via the new keyword.</p>\n\n<p>The CardLayout's focal method is <a href=\"#!/api/Ext.layout.container.Card-method-setActiveItem\" rel=\"Ext.layout.container.Card-method-setActiveItem\" class=\"docClass\">setActiveItem</a>.  Since only one panel is displayed at a time,\nthe only way to move from one Component to the next is by calling setActiveItem, passing the next panel to display\n(or its id or index).  The layout itself does not provide a user interface for handling this navigation,\nso that functionality must be provided by the developer.</p>\n\n<p>To change the active card of a container, call the setActiveItem method of its layout:</p>\n\n<pre class='inline-example '><code>var p = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    layout: 'card',\n    items: [\n        { html: 'Card 1' },\n        { html: 'Card 2' }\n    ],\n    renderTo: Ext.getBody()\n});\n\np.getLayout().setActiveItem(1);\n</code></pre>\n\n<p>The <a href=\"#!/api/Ext.Component-event-beforedeactivate\" rel=\"Ext.Component-event-beforedeactivate\" class=\"docClass\">beforedeactivate</a> and <a href=\"#!/api/Ext.Component-event-beforeactivate\" rel=\"Ext.Component-event-beforeactivate\" class=\"docClass\">beforeactivate</a>\nevents can be used to prevent a card from activating or deactivating by returning <code>false</code>.</p>\n\n<pre class='inline-example '><code>var active = 0;\nvar main = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    renderTo: Ext.getBody(),\n    width: 200,\n    height: 200,\n    layout: 'card',\n    tbar: [{\n        text: 'Next',\n        handler: function(){\n            var layout = main.getLayout();\n            ++active;\n            layout.setActiveItem(active);\n            active = main.items.indexOf(layout.getActiveItem());\n        }\n    }],\n    items: [{\n        title: 'P1'\n    }, {\n        title: 'P2'\n    }, {\n        title: 'P3',\n        listeners: {\n            beforeactivate: function(){\n                return false;\n            }\n        }\n    }]\n});\n</code></pre>\n\n<p>In the following example, a simplistic wizard setup is demonstrated.  A button bar is added\nto the footer of the containing panel to provide navigation buttons.  The buttons will be handled by a\ncommon navigation routine.  Note that other uses of a CardLayout (like a tab control) would require a\ncompletely different implementation.  For serious implementations, a better approach would be to extend\nCardLayout to provide the custom functionality needed.</p>\n\n<pre class='inline-example '><code>var navigate = function(panel, direction){\n    // This routine could contain business logic required to manage the navigation steps.\n    // It would call setActiveItem as needed, manage navigation button state, handle any\n    // branching logic that might be required, handle alternate actions like cancellation\n    // or finalization, etc.  A complete wizard implementation could get pretty\n    // sophisticated depending on the complexity required, and should probably be\n    // done as a subclass of CardLayout in a real-world implementation.\n    var layout = panel.getLayout();\n    layout[direction]();\n    Ext.getCmp('move-prev').setDisabled(!layout.getPrev());\n    Ext.getCmp('move-next').setDisabled(!layout.getNext());\n};\n\nExt.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Example Wizard',\n    width: 300,\n    height: 200,\n    layout: 'card',\n    bodyStyle: 'padding:15px',\n    defaults: {\n        // applied to each contained panel\n        border: false\n    },\n    // just an example of one possible navigation scheme, using buttons\n    bbar: [\n        {\n            id: 'move-prev',\n            text: 'Back',\n            handler: function(btn) {\n                navigate(btn.up(\"panel\"), \"prev\");\n            },\n            disabled: true\n        },\n        '-&gt;', // greedy spacer so that the buttons are aligned to each side\n        {\n            id: 'move-next',\n            text: 'Next',\n            handler: function(btn) {\n                navigate(btn.up(\"panel\"), \"next\");\n            }\n        }\n    ],\n    // the panels (or \"cards\") within the layout\n    items: [{\n        id: 'card-0',\n        html: '&lt;h1&gt;Welcome to the Wizard!&lt;/h1&gt;&lt;p&gt;Step 1 of 3&lt;/p&gt;'\n    },{\n        id: 'card-1',\n        html: '&lt;p&gt;Step 2 of 3&lt;/p&gt;'\n    },{\n        id: 'card-2',\n        html: '&lt;h1&gt;Congratulations!&lt;/h1&gt;&lt;p&gt;Step 3 of 3 - Complete&lt;/p&gt;'\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn()","prototype":{"deferredRender":{"!type":"bool","!doc":"<p>True to render each contained item at the time it becomes active, false to render all contained items\nas soon as the layout is rendered (defaults to false).  If there is a significant amount of content or\na lot of heavy controls being rendered into panels that are not displayed by default, setting this to\ntrue might improve performance.</p>\n"},"hideInactive":{"!type":"bool"},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"configureItem":{"!type":"fn(item: ?) -> !this","!doc":"<p>. Called before both dynamic render, and bulk render.\nEnsure that the active item starts visible, and inactive ones start invisible</p>\n"},"getActiveItem":{"!type":"fn() -> ?","!doc":"<p>Return the active (visible) component in the layout.</p>\n"},"getAnimation":{"!type":"fn(newCard: ?, owner: ?) -> !this"},"getNext":{"!type":"fn() -> ?","!doc":"<p>Return the active (visible) component in the layout to the next card</p>\n"},"getPrev":{"!type":"fn() -> ?","!doc":"<p>Return the active (visible) component in the layout to the previous card</p>\n"},"getRenderTree":{"!type":"fn() -> !this"},"isValidParent":{"!type":"fn(item: ?, target: ?, position: ?) -> !this","!doc":"<p>Validates item is in the proper place in the dom.</p>\n"},"next":{"!type":"fn() -> ?","!doc":"<p>Sets the active (visible) component in the layout to the next card</p>\n"},"onRemove":{"!type":"fn(component: ?) -> !this"},"parseActiveItem":{"!type":"fn(item: ?) -> !this"},"prev":{"!type":"fn() -> ?","!doc":"<p>Sets the active (visible) component in the layout to the previous card</p>\n"},"renderChildren":{"!type":"fn() -> !this"},"setActiveItem":{"!type":"fn(newCard: ?) -> ?","!doc":"<p>Makes the given card active.</p>\n\n<pre><code>var card1 = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {itemId: 'card-1'});\nvar card2 = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {itemId: 'card-2'});\nvar panel = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    layout: 'card',\n    activeItem: 0,\n    items: [card1, card2]\n});\n// These are all equivalent\npanel.getLayout().setActiveItem(card2);\npanel.getLayout().setActiveItem('card-2');\npanel.getLayout().setActiveItem(1);\n</code></pre>\n"}}},"Fit":{"prototype":{"itemCls":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"manageMargins":{"!type":"bool"},"sizePolicies":{},"targetCls":{"!type":"string"},"beginLayoutCycle":{"!type":"fn(ownerContext: ?, firstCycle: ?) -> !this","!doc":"<p>Called before any calculation cycles to reset DOM values and prepare for calculation.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.layout.container.Fit-property-done\" rel=\"Ext.layout.container.Fit-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.layout.container.Fit-property-done\" rel=\"Ext.layout.container.Fit-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"},"fitItem":{"!type":"fn(itemContext: ?, info: ?) -> !this"},"fitItemHeight":{"!type":"fn(itemContext: ?, info: ?) -> !this"},"fitItemWidth":{"!type":"fn(itemContext: ?, info: ?) -> !this"},"getItemSizePolicy":{"!type":"fn(item: ?, ownerSizeModel: ?) -> ?","!doc":"<p>Returns an object describing how this layout manages the size of the given component.\nThis method must be implemented by any layout that manages components.</p>\n"},"positionItemX":{"!type":"fn(itemContext: ?, info: ?) -> !this"},"positionItemY":{"!type":"fn(itemContext: ?, info: ?) -> !this"},"setItemHeight":{"!type":"fn(itemContext: ?, info: ?) -> !this"},"setItemWidth":{"!type":"fn(itemContext: ?, info: ?) -> !this"},"type":{"!type":"string"}},"!doc":"<p>This is a base class for layouts that contain a single item that automatically expands to fill the layout's\ncontainer. This class is intended to be extended or created via the layout:'fit'\n<a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">Ext.container.Container.layout</a> config, and should generally not need to be created directly via the new keyword.</p>\n\n<p>Fit layout does not have any direct config options (other than inherited ones). To fit a panel to a container using\nFit layout, simply set <code>layout: 'fit'</code> on the container and add a single panel to it.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Fit Layout',\n    width: 300,\n    height: 150,\n    layout:'fit',\n    items: {\n        title: 'Inner Panel',\n        html: 'This is the inner panel content',\n        bodyPadding: 20,\n        border: false\n    },\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>If the container has multiple items, all of the items will all be equally sized. This is usually not\ndesired, so to avoid this, place only a <strong>single</strong> item in the container. This sizing of all items\ncan be used to provide a background <a href=\"#!/api/Ext.Img\" rel=\"Ext.Img\" class=\"docClass\">image</a> that is \"behind\" another item\nsuch as a <a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">dataview</a> if you also absolutely position the items.</p>\n","!type":"fn()"},"Center":{"!doc":"<p>This layout manager is used to center contents within a container. As a subclass of\n<a href=\"#!/api/Ext.layout.container.Fit\" rel=\"Ext.layout.container.Fit\" class=\"docClass\">fit layout</a>, CenterLayout expects to have one child\nitem; multiple items will be placed overlapping. The layout does not require any config\noptions. Items in the container can use percentage width or height rather than be fit\nto the full size of the container.</p>\n\n<p>Example usage:</p>\n\n<pre><code> // The content panel is centered in the container\n\n var p = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.Panel</a>', {\n     title: 'Center Layout',\n     layout: 'center',\n     items: [{\n         title: 'Centered Content',\n         width: '75%',  // assign 75% of the container width to the item\n         html: 'Some content'\n     }]\n });\n</code></pre>\n\n<p>If you leave the title blank and specify no border you can create a non-visual, structural\ncontainer just for centering the contents.</p>\n\n<pre><code> var p = Ext.create('<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.Container</a>', {\n     layout: 'center',\n     items: [{\n         title: 'Centered Content',\n         width: 300,\n         height: '90%', // assign 90% of the container height to the item\n         html: 'Some content'\n     }]\n });\n</code></pre>\n","!type":"fn()","prototype":{"itemCls":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"percentRe":{},"targetCls":{"!type":"string"},"getItemSizePolicy":{"!type":"fn(item: ?, ownerSizeModel: ?) -> ?","!doc":"<p>Returns an object describing how this layout manages the size of the given component.\nThis method must be implemented by any layout that manages components.</p>\n"},"getPos":{"!type":"fn(itemContext: ?, info: ?, dimension: ?) -> !this"},"positionItemX":{"!type":"fn(itemContext: ?, info: ?) -> !this"},"positionItemY":{"!type":"fn(itemContext: ?, info: ?) -> !this"},"setItemHeight":{"!type":"fn(itemContext: ?, info: ?) -> !this"},"setItemWidth":{"!type":"fn(itemContext: ?, info: ?) -> !this"}}},"CheckboxGroup":{"!doc":"<p>This layout implements the column arrangement for <a href=\"#!/api/Ext.form.CheckboxGroup\" rel=\"Ext.form.CheckboxGroup\" class=\"docClass\">Ext.form.CheckboxGroup</a> and <a href=\"#!/api/Ext.form.RadioGroup\" rel=\"Ext.form.RadioGroup\" class=\"docClass\">Ext.form.RadioGroup</a>.\nIt groups the component's sub-items into columns based on the component's\n<a href=\"#!/api/Ext.form.CheckboxGroup-cfg-columns\" rel=\"Ext.form.CheckboxGroup-cfg-columns\" class=\"docClass\">columns</a> and <a href=\"#!/api/Ext.form.CheckboxGroup-cfg-vertical\" rel=\"Ext.form.CheckboxGroup-cfg-vertical\" class=\"docClass\">Ext.form.CheckboxGroup.vertical</a> config properties.</p>\n","!type":"fn()","prototype":{"autoFlex":{"!type":"bool","!doc":"<p>By default,  CheckboxGroup allocates all available space to the configured columns meaning that\ncolumn are evenly spaced across the container.</p>\n\n<p>To have each column only be wide enough to fit the container Checkboxes (or Radios), set <code>autoFlex</code> to <code>false</code></p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"createsInnerCt":{"!type":"bool"},"isValidParent":{"!doc":"<p>Always valid. beginLayout ensures the encapsulating elements of all children are in the correct place</p>\n"},"lastOwnerItemsGeneration":{},"renderTpl":{},"type":{"!type":"string"},"addMissingColumns":{"!type":"fn(itemsCount: ?) -> !this"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>In addition to work done by our base classes, containers benefit from some extra\ncached data. The following properties are added to the ownerContext:</p>\n\n<ul>\n<li>visibleItems: the result of <a href=\"#!/api/Ext.layout.container.CheckboxGroup-method-getVisibleItems\" rel=\"Ext.layout.container.CheckboxGroup-method-getVisibleItems\" class=\"docClass\">getVisibleItems</a></li>\n<li>childItems: the ContextItem[] for each visible item</li>\n<li>targetContext: the ContextItem for the <a href=\"#!/api/Ext.layout.container.CheckboxGroup-method-getTarget\" rel=\"Ext.layout.container.CheckboxGroup-method-getTarget\" class=\"docClass\">getTarget</a> element</li>\n</ul>\n\n"},"cacheElements":{"!type":"fn() -> !this"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Just wait for the child items to all lay themselves out in the width we are configured\nto make available to them. Then we can measure our height.</p>\n"},"doRenderColumn":{"!type":"fn(out: ?, renderData: ?, columnIndex: ?) -> !this"},"getColumnCount":{"!type":"fn() -> !this","!doc":"<p>Returns the number of columns in the checkbox group.</p>\n"},"getItemSizePolicy":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns an object describing how this layout manages the size of the given component.\nThis method must be implemented by any layout that manages components.</p>\n"},"getNodeAt":{"!type":"fn(rowIndex: ?, columnIndex: ?) -> !this"},"getRenderColumnIndex":{"!type":"fn(itemIndex: ?, rowCount: ?, columnCount: ?) -> !this"},"getRenderData":{"!type":"fn() -> !this"},"getRenderRowIndex":{"!type":"fn(itemIndex: ?, rowCount: ?, columnCount: ?) -> !this"},"initLayout":{"!type":"fn() -> !this","!doc":"<p>A one-time initialization method called just before rendering.</p>\n"},"isItemAtPosition":{"!type":"fn(item: ?, rowIndex: ?, columnIndex: ?) -> !this"},"moveItem":{"!type":"fn(item: ?, rowIndex: ?, columnIndex: ?) -> !this","!doc":"<p>Moves the given already rendered Component to the specified row and column</p>\n"},"removeExceedingColumns":{"!type":"fn(itemsCount: ?) -> !this"},"renderChildren":{"!type":"fn() -> !this"},"renderItem":{"!type":"fn(item: ?, rowIndex: ?, columnIndex: ?) -> !this","!doc":"<p>Renders the given Component into the specified row and column</p>\n"},"renderItems":{"!type":"fn(items: ?) -> !this","!doc":"<p>Iterates over all passed items, ensuring they are rendered.  If the items are already rendered,\nalso determines if the items are in the proper place in the dom.</p>\n"},"setupRenderTpl":{"!type":"fn(renderTpl: ?) -> !this"}}},"Column":{"!doc":"<p>This is the layout style of choice for creating structural layouts in a multi-column format where the width of each\ncolumn can be specified as a percentage or fixed width, but the height is allowed to vary based on the content. This\nclass is intended to be extended or created via the layout:'column' <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">Ext.container.Container.layout</a> config,\nand should generally not need to be created directly via the new keyword.</p>\n\n<p>ColumnLayout does not have any direct config options (other than inherited ones), but it does support a specific\nconfig property of <code>columnWidth</code> that can be included in the config of any panel added to it. The layout will use\nthe columnWidth (if present) or width of each panel during layout to determine how to size each panel. If width or\ncolumnWidth is not specified for a given panel, its width will default to the panel's width (or auto).</p>\n\n<p>The width property is always evaluated as pixels, and must be a number greater than or equal to 1. The columnWidth\nproperty is always evaluated as a percentage, and must be a decimal value greater than 0 and less than 1 (e.g., .25).</p>\n\n<p>The basic rules for specifying column widths are pretty simple. The logic makes two passes through the set of\ncontained panels. During the first layout pass, all panels that either have a fixed width or none specified (auto)\nare skipped, but their widths are subtracted from the overall container width.</p>\n\n<p>During the second pass, all panels with columnWidths are assigned pixel widths in proportion to their percentages\nbased on the total <strong>remaining</strong> container width. In other words, percentage width panels are designed to fill\nthe space left over by all the fixed-width and/or auto-width panels. Because of this, while you can specify any\nnumber of columns with different percentages, the columnWidths must always add up to 1 (or 100%) when added\ntogether, otherwise your layout may not render as expected.</p>\n\n<pre class='inline-example '><code>// All columns are percentages -- they must add up to 1\nExt.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Column Layout - Percentage Only',\n    width: 350,\n    height: 250,\n    layout:'column',\n    items: [{\n        title: 'Column 1',\n        columnWidth: 0.25\n    },{\n        title: 'Column 2',\n        columnWidth: 0.55\n    },{\n        title: 'Column 3',\n        columnWidth: 0.20\n    }],\n    renderTo: Ext.getBody()\n});\n\n// Mix of width and columnWidth -- all columnWidth values must add up\n// to 1. The first column will take up exactly 120px, and the last two\n// columns will fill the remaining container width.\n\nExt.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.Panel</a>', {\n    title: 'Column Layout - Mixed',\n    width: 350,\n    height: 250,\n    layout:'column',\n    items: [{\n        title: 'Column 1',\n        width: 120\n    },{\n        title: 'Column 2',\n        columnWidth: 0.7\n    },{\n        title: 'Column 3',\n        columnWidth: 0.3\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn()","prototype":{"itemCls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to the container. This can be useful for\nadding customized styles to the container or any of its children using standard CSS\nrules. See <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.<a href=\"#!/api/Ext.Component-cfg-componentCls\" rel=\"Ext.Component-cfg-componentCls\" class=\"docClass\">componentCls</a> also.</p>\n"},"columnWidthSizePolicy":{"!doc":"<p>Columns with a columnWidth have their width managed.</p>\n"},"createsInnerCt":{"!type":"bool"},"manageOverflow":{"!type":"bool","!doc":"<p>true to rerun the layout if scrollbars are needed.</p>\n"},"setsItemSize":{"!type":"bool","!doc":"<p>Column layout does not read the size of individual items in the shrink-wrapping\ndimension(s) because, as a subclass of autocontainer, it measures them as a whole\nusing an outer element.  However, column layout may set the size of its items in\nnon-shrink-wrapping dimension(s).</p>\n"},"targetCls":{"!type":"string"},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"calculateItems":{"!type":"fn(ownerContext: ?, containerSize: ?) -> !this"},"getItemSizePolicy":{"!type":"fn(item: ?, ownerSizeModel: ?) -> ?","!doc":"<p>Returns an object describing how this layout manages the size of the given component.\nThis method must be implemented by any layout that manages components.</p>\n"},"isItemShrinkWrap":{"!type":"fn(ownerContext: ?) -> !this"}}},"ColumnSplitter":{"!type":"fn(config: ?)","prototype":{"cls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element. This can be useful\nfor adding customized styles to the component or any of its children using standard CSS rules.</p>\n"},"height":{"!type":"number","!doc":"<p>The height of this component in pixels.</p>\n"},"width":{"!type":"number","!doc":"<p>The width of this component in pixels.</p>\n"},"collapseDirection":{"!type":"string"},"isSplitter":{"!type":"bool"},"orientation":{"!type":"string","!doc":"<p>Orientation of this Splitter. <code>'vertical'</code> when used in an hbox layout, <code>'horizontal'</code>\nwhen used in a vbox layout.</p>\n"},"synthetic":{"!type":"bool","!doc":"<p>This property is <code>true</code> if the component was created internally by the framework\nand is not explicitly user-defined. This is set for such things as <code>Splitter</code>\ninstances managed by <code>border</code> and <code>box</code> layouts.</p>\n"},"trackerClass":{"!type":"string"},"getTrackerConfig":{"!type":"fn() -> !this","!doc":"<p>Returns the config object (with an <code>xclass</code> property) for the splitter tracker. This\nis overridden by <a href=\"#!/api/Ext.resizer.BorderSplitter\" rel=\"Ext.resizer.BorderSplitter\" class=\"docClass\">BorderSplitter</a> to create a\n<a href=\"#!/api/Ext.resizer.BorderSplitterTracker\" rel=\"Ext.resizer.BorderSplitterTracker\" class=\"docClass\">BorderSplitterTracker</a>.</p>\n"}}},"ColumnSplitterTracker":{"prototype":{"performResize":{"!type":"fn(e: ?, offset: ?) -> !this","!doc":"<p>Performs the actual resizing of the previous and next components</p>\n"}}},"Editor":{"!doc":"<p>Component layout for editors</p>\n","!type":"fn()","prototype":{"autoSizeDefault":{"!doc":"<p>End Definitions</p>\n"},"sizePolicies":{},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.layout.container.Editor-property-done\" rel=\"Ext.layout.container.Editor-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.layout.container.Editor-property-done\" rel=\"Ext.layout.container.Editor-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"},"getDimension":{"!type":"fn(owner: ?, type: ?, getMethod: ?, ownerSize: ?) -> !this"},"getItemSizePolicy":{"!type":"fn(item: ?) -> ?","!doc":"<p>Returns an object describing how this layout manages the size of the given component.\nThis method must be implemented by any layout that manages components.</p>\n"}}},"Form":{"!doc":"<p>This is a layout that will render form Fields, one under the other all stretched to the Container width.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.Panel</a>', {\n    width: 500,\n    height: 300,\n    title: \"FormLayout Panel\",\n    layout: 'form',\n    renderTo: Ext.getBody(),\n    bodyPadding: 5,\n    defaultType: 'textfield',\n    items: [{\n       fieldLabel: 'First Name',\n        name: 'first',\n        allowBlank:false\n    },{\n        fieldLabel: 'Last Name',\n        name: 'last'\n    },{\n        fieldLabel: 'Company',\n        name: 'company'\n    }, {\n        fieldLabel: 'Email',\n        name: 'email',\n        vtype:'email'\n    }, {\n        fieldLabel: 'DOB',\n        name: 'dob',\n        xtype: 'datefield'\n    }, {\n        fieldLabel: 'Age',\n        name: 'age',\n        xtype: 'numberfield',\n        minValue: 0,\n        maxValue: 100\n    }, {\n        xtype: 'timefield',\n        fieldLabel: 'Time',\n        name: 'time',\n        minValue: '8:00am',\n        maxValue: '6:00pm'\n    }]\n});\n</code></pre>\n","!type":"fn()","prototype":{"itemSpacing":{"!type":"number","!doc":"<p>The amount of space, in pixels, to use between the items. Defaults to the value\ninherited from the theme's stylesheet as configured by\n$form-item-margin-bottom.</p>\n"},"labelWidth":{"!doc":"<p>The width of the labels. This can be either a number in pixels, or a valid CSS\n\"width\" style, e.g. <code>'100px'</code>, or <code>'30%'</code>.  When configured, all labels will assume\nthis width, and any <a href=\"#!/api/Ext.form.Labelable-cfg-labelWidth\" rel=\"Ext.form.Labelable-cfg-labelWidth\" class=\"docClass\">labelWidth</a> specified\non the items will be ignored.</p>\n\n<p>The default behavior of this layout when no no labelWidth is specified is to size\nthe labels to the text-width of the label with the longest text.</p>\n"},"afterBodyTpl":{"!type":"string"},"beforeBodyTpl":{"!type":"string"},"formColGroupCls":{"!type":"string"},"formColumnCls":{"!type":"string"},"formLabelColumnCls":{"!type":"string"},"formWrapAutoLabelCls":{"!type":"string"},"formWrapCls":{"!type":"string"},"formWrapSizedLabelCls":{"!type":"string"},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"getRenderData":{"!type":"fn() -> !this"},"getRenderTarget":{"!type":"fn() -> ?","!doc":"<p>Overridden method from <a href=\"#!/api/Ext.layout.container.Container\" rel=\"Ext.layout.container.Container\" class=\"docClass\">Ext.layout.container.Container</a>.\nUsed in the beforeLayout method to render all items into.</p>\n"}}},"SplitColumn":{"!doc":"<p>This layout extends <code><a href=\"#!/api/Ext.layout.container.Column\" rel=\"Ext.layout.container.Column\" class=\"docClass\">Ext.layout.container.Column</a></code> and adds splitters between adjacent\ncolumns allowing the user to resize them.</p>\n","!type":"fn()","prototype":{"firstColumnCls":{"!type":"string"},"lastColumnCls":{"!type":"string"},"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"beforeLayoutCycle":{"!type":"fn(ownerContext: ?) -> !this"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>In addition to work done by our base classes, containers benefit from some extra\ncached data. The following properties are added to the ownerContext:</p>\n\n<ul>\n<li>visibleItems: the result of <a href=\"#!/api/Ext.layout.container.SplitColumn-method-getVisibleItems\" rel=\"Ext.layout.container.SplitColumn-method-getVisibleItems\" class=\"docClass\">getVisibleItems</a></li>\n<li>childItems: the ContextItem[] for each visible item</li>\n<li>targetContext: the ContextItem for the <a href=\"#!/api/Ext.layout.container.SplitColumn-method-getTarget\" rel=\"Ext.layout.container.SplitColumn-method-getTarget\" class=\"docClass\">getTarget</a> element</li>\n</ul>\n\n"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method is called after all layouts are complete and their calculations flushed\nto the DOM. No further layouts will be run and this method is only called once per\nlayout run. The base component layout caches <code>lastComponentSize</code>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"getSplitterConfig":{"!type":"fn() -> !this","!doc":"<p>The geometry of a Column layout with splitters between respective items:</p>\n\n<pre><code>        0        1      2       3        4\n +-----------------------------------------------+\n | +-----------+ || +---------+ || +-----------+ | \\\n | |           | || |         | || |           | |  \\\n | |           | || |         | || |           | |   \\\n | |           | || |         | || |           | |    \\\n | +-----------+ || |         | || |           | |   row[0]\n |               || |         | || |           | |    /\n |               || |         | || |           | |   /\n |               || |         | || +-----------+ |  /\n |               || |         | ||               | /\n |               || +---------+ ||               |\n | +-------------------+ || +------------------+ | \\\n | |                   | || |                  | |  \\\n | |                   | || |                  | |   \\\n | |                   | || |                  | |  row[1]\n | |                   | || |                  | |   /\n | |                   | || +------------------+ |  /\n | +-------------------+ ||                      | /\n +-----------------------------------------------+\n             6           7            8\n</code></pre>\n\n<p>The splitter between 4 and 6 will be hidden but still present in the items. It is\nconsidered part of row[0].</p>\n"},"syncFirstLast":{"!type":"fn() -> !this"},"syncSplitters":{"!type":"fn() -> !this","!doc":"<p>This method synchronizes the splitters so that we have exactly one between each\ncolumn.</p>\n"}}},"Table":{"!doc":"<p>This layout allows you to easily render content into an HTML table. The total number of columns can be specified, and\nrowspan and colspan can be used to create complex layouts within the table. This class is intended to be extended or\ncreated via the <code>layout: {type: 'table'}</code> <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">Ext.container.Container.layout</a> config, and should generally not\nneed to be created directly via the new keyword.</p>\n\n<p>Note that when creating a layout via config, the layout-specific config properties must be passed in via the <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">Ext.container.Container.layout</a> object which will then be applied internally to the layout. In the case of\nTableLayout, the only valid layout config properties are <a href=\"#!/api/Ext.layout.container.Table-cfg-columns\" rel=\"Ext.layout.container.Table-cfg-columns\" class=\"docClass\">columns</a> and <a href=\"#!/api/Ext.layout.container.Table-cfg-tableAttrs\" rel=\"Ext.layout.container.Table-cfg-tableAttrs\" class=\"docClass\">tableAttrs</a>. However, the items\nadded to a TableLayout can supply the following table-specific config properties:</p>\n\n<ul>\n<li><strong>rowspan</strong> Applied to the table cell containing the item.</li>\n<li><strong>colspan</strong> Applied to the table cell containing the item.</li>\n<li><strong>cellId</strong> An id applied to the table cell containing the item.</li>\n<li><strong>cellCls</strong> A CSS class name added to the table cell containing the item.</li>\n</ul>\n\n\n<p>The basic concept of building up a TableLayout is conceptually very similar to building up a standard HTML table. You\nsimply add each panel (or \"cell\") that you want to include along with any span attributes specified as the special\nconfig properties of rowspan and colspan which work exactly like their HTML counterparts. Rather than explicitly\ncreating and nesting rows and columns as you would in HTML, you simply specify the total column count in the\nlayout config and start adding panels in their natural order from left to right, top to bottom. The layout will\nautomatically figure out, based on the column count, rowspans and colspans, how to position each panel within the\ntable. Just like with HTML tables, your rowspans and colspans must add up correctly in your overall layout or you'll\nend up with missing and/or extra cells! Example usage:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Table Layout',\n    width: 300,\n    height: 150,\n    layout: {\n        type: 'table',\n        // The total column count must be specified here\n        columns: 3\n    },\n    defaults: {\n        // applied to each contained panel\n        bodyStyle: 'padding:20px'\n    },\n    items: [{\n        html: 'Cell A content',\n        rowspan: 2\n    },{\n        html: 'Cell B content',\n        colspan: 2\n    },{\n        html: 'Cell C content',\n        cellCls: 'highlight'\n    },{\n        html: 'Cell D content'\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn()","prototype":{"columns":{"!type":"number","!doc":"<p>The total number of columns to create in the table for this layout. If not specified, all Components added to\nthis layout will be rendered into a single row using one column per Component.</p>\n"},"tableAttrs":{"!doc":"<p>An object containing properties which are added to the DomHelper specification used to\ncreate the layout's <code>&lt;table&gt;</code> element. Example:</p>\n\n<pre><code>{\n    xtype: 'panel',\n    layout: {\n        type: 'table',\n        columns: 3,\n        tableAttrs: {\n            style: {\n                width: '100%'\n            }\n        }\n    }\n}\n</code></pre>\n"},"tdAttrs":{"!doc":"<p>An object containing properties which are added to the DomHelper specification used to\ncreate the layout's <code>&lt;td&gt;</code> elements.</p>\n"},"trAttrs":{"!doc":"<p>An object containing properties which are added to the DomHelper specification used to\ncreate the layout's <code>&lt;tr&gt;</code> elements.</p>\n"},"cellCls":{"!type":"string"},"createsInnerCt":{"!type":"bool"},"tableCls":{"!type":"string"},"targetCls":{"!type":"string"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.layout.container.Table-property-done\" rel=\"Ext.layout.container.Table-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.layout.container.Table-property-done\" rel=\"Ext.layout.container.Table-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"},"calculateCells":{"!type":"fn(items: ?) -> ?","!doc":"<p>Determine the row and cell indexes for each component, taking into consideration\nthe number of columns and each item's configured colspan/rowspan values.</p>\n"},"ensureInDocument":{"!type":"fn(el: ?) -> !this"},"finalizeLayout":{"!type":"fn() -> !this","!doc":"<p>This method (if implemented) is called after all layouts have completed. In most\nways this is similar to <a href=\"#!/api/Ext.layout.container.Table-method-completeLayout\" rel=\"Ext.layout.container.Table-method-completeLayout\" class=\"docClass\">completeLayout</a>. This call can cause this (or any\nlayout) to be become invalid (see <a href=\"#!/api/Ext.layout.Context-method-invalidate\" rel=\"Ext.layout.Context-method-invalidate\" class=\"docClass\">Ext.layout.Context.invalidate</a>), but this\nis best avoided. This method is intended to be where final reads are made and so it\nis best to avoid invalidating layouts at this point whenever possible. Even so, this\nmethod can be used to perform final checks that may require all other layouts to be\ncomplete and then invalidate some results.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\nbe flushed at the next opportunity.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"getHiddenItems":{"!type":"fn() -> !this"},"getLayoutItems":{"!type":"fn() -> ?","!doc":"<p>Returns an array of child components either for a render phase (Performed in the beforeLayout\nmethod of the layout's base class), or the layout phase (onLayout).</p>\n"},"getRenderTree":{"!type":"fn() -> !this"},"initInheritedState":{"!type":"fn(inheritedState: ?, inheritedStateInner: ?) -> !this"},"isValidParent":{"!type":"fn(item: ?, target: ?, rowIdx: ?, cellIdx: ?) -> !this","!doc":"<p>Validates item is in the proper place in the dom.</p>\n"},"needsDivWrap":{"!type":"fn() -> !this","!doc":"<p>Opera 10.5 has a bug where if a table cell's child has box-sizing:border-box and padding, it\nwill include that padding in the size of the cell, making it always larger than the\nshrink-wrapped size of its contents. To get around this we have to wrap the contents in a div\nand then set that div's width to match the item rendered within it afterLayout. This method\ndetermines whether we need the wrapper div; it currently does a straight UA sniff as this bug\nseems isolated to just Opera 10.5, but feature detection could be added here if needed.</p>\n"},"renderChildren":{"!type":"fn() -> !this","!doc":"<p>Iterates over all passed items, ensuring they are rendered in a cell in the proper\nlocation in the table structure.</p>\n"}}}},"ClassList":{"!doc":"<p>This class provides a DOM ClassList API to buffer access to an element's class.\nInstances of this class are created by <a href=\"#!/api/Ext.layout.ContextItem-method-getClassList\" rel=\"Ext.layout.ContextItem-method-getClassList\" class=\"docClass\">Ext.layout.ContextItem.getClassList</a>.</p>\n","prototype":{"add":{"!type":"fn(cls: ?) -> !this","!doc":"<p>Adds a single class to the class list.</p>\n"},"addMany":{"!type":"fn(classes: ?) -> !this","!doc":"<p>Adds one or more classes in an array or space-delimited string to the class list.</p>\n"},"remove":{"!type":"fn(cls: ?) -> !this","!doc":"<p>Removes a single class from the class list.</p>\n"},"removeMany":{"!type":"fn(classes: ?) -> !this","!doc":"<p>Removes one or more classes in an array or space-delimited string from the class\nlist.</p>\n"}}},"Context":{"!doc":"<p>Manages context information during a layout.</p>\n\n<h1>Algorithm</h1>\n\n<p>This class performs the following jobs:</p>\n\n<ul>\n<li>Cache DOM reads to avoid reading the same values repeatedly.</li>\n<li>Buffer DOM writes and flush them as a block to avoid read/write interleaving.</li>\n<li>Track layout dependencies so each layout does not have to figure out the source of\nits dependent values.</li>\n<li>Intelligently run layouts when the values on which they depend change (a trigger).</li>\n<li>Allow layouts to avoid processing when required values are unavailable (a block).</li>\n</ul>\n\n\n<p>Work done during layout falls into either a \"read phase\" or a \"write phase\" and it is\nessential to always be aware of the current phase. Most methods in\n<a href=\"#!/api/Ext.layout.Layout\" rel=\"Ext.layout.Layout\" class=\"docClass\">Layout</a> are called during a read phase:\n<a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a>,\n<a href=\"#!/api/Ext.layout.Layout-method-completeLayout\" rel=\"Ext.layout.Layout-method-completeLayout\" class=\"docClass\">completeLayout</a> and\n<a href=\"#!/api/Ext.layout.Layout-method-finalizeLayout\" rel=\"Ext.layout.Layout-method-finalizeLayout\" class=\"docClass\">finalizeLayout</a>. The exceptions to this are\n<a href=\"#!/api/Ext.layout.Layout-method-beginLayout\" rel=\"Ext.layout.Layout-method-beginLayout\" class=\"docClass\">beginLayout</a>,\n<a href=\"#!/api/Ext.layout.Layout-method-beginLayoutCycle\" rel=\"Ext.layout.Layout-method-beginLayoutCycle\" class=\"docClass\">beginLayoutCycle</a> and\n<a href=\"#!/api/Ext.layout.Layout-method-finishedLayout\" rel=\"Ext.layout.Layout-method-finishedLayout\" class=\"docClass\">finishedLayout</a> which are called during\na write phase. While <a href=\"#!/api/Ext.layout.Layout-method-finishedLayout\" rel=\"Ext.layout.Layout-method-finishedLayout\" class=\"docClass\">finishedLayout</a> is called\na write phase, it is really intended to be a catch-all for post-processing after a\nlayout run.</p>\n\n<p>In a read phase, it is OK to read the DOM but this should be done using the appropriate\n<a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">ContextItem</a> where possible since that provides a cache\nto avoid redundant reads. No writes should be made to the DOM in a read phase! Instead,\nthe values should be written to the proper ContextItem for later write-back.</p>\n\n<p>The rules flip-flop in a write phase. The only difference is that ContextItem methods\nlike <a href=\"#!/api/Ext.layout.ContextItem-method-getStyle\" rel=\"Ext.layout.ContextItem-method-getStyle\" class=\"docClass\">getStyle</a> will still read the DOM unless the\nvalue was previously read. This detail is unknowable from the outside of ContextItem, so\nread calls to ContextItem should also be avoided in a write phase.</p>\n\n<p>Calculating interdependent layouts requires a certain amount of iteration. In a given\ncycle, some layouts will contribute results that allow other layouts to proceed. The\ngeneral flow then is to gather all of the layouts (both component and container) in a\ncomponent tree and queue them all for processing. The initial queue order is bottom-up\nand component layout first, then container layout (if applicable) for each component.</p>\n\n<p>This initial step also calls the beginLayout method on all layouts to clear any values\nfrom the DOM that might interfere with calculations and measurements. In other words,\nthis is a \"write phase\" and reads from the DOM should be strictly avoided.</p>\n\n<p>Next the layout enters into its iterations or \"cycles\". Each cycle consists of calling\nthe <a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a> method on all layouts in the\n<a href=\"#!/api/Ext.layout.Context-property-layoutQueue\" rel=\"Ext.layout.Context-property-layoutQueue\" class=\"docClass\">layoutQueue</a>. These calls are part of a \"read phase\" and writes to the DOM should\nbe strictly avoided.</p>\n\n<h1>Considerations</h1>\n\n<p><strong>RULE 1</strong>: Respect the read/write cycles. Always use the <a href=\"#!/api/Ext.layout.ContextItem-method-getProp\" rel=\"Ext.layout.ContextItem-method-getProp\" class=\"docClass\">getProp</a>\nor <a href=\"#!/api/Ext.layout.ContextItem-method-getDomProp\" rel=\"Ext.layout.ContextItem-method-getDomProp\" class=\"docClass\">getDomProp</a> methods to get calculated values;\nonly use the <a href=\"#!/api/Ext.layout.ContextItem-method-getStyle\" rel=\"Ext.layout.ContextItem-method-getStyle\" class=\"docClass\">getStyle</a> method to read styles; use\n<a href=\"#!/api/Ext.layout.ContextItem-method-setProp\" rel=\"Ext.layout.ContextItem-method-setProp\" class=\"docClass\">setProp</a> to set DOM values. Some reads will, of\ncourse, still go directly to the DOM, but if there is a method in\n<a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">ContextItem</a> to do a certain job, it should be used instead\nof a lower-level equivalent.</p>\n\n<p>The basic logic flow in <a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a> consists of gathering\nvalues by calling <a href=\"#!/api/Ext.layout.ContextItem-method-getProp\" rel=\"Ext.layout.ContextItem-method-getProp\" class=\"docClass\">getProp</a> or\n<a href=\"#!/api/Ext.layout.ContextItem-method-getDomProp\" rel=\"Ext.layout.ContextItem-method-getDomProp\" class=\"docClass\">getDomProp</a>, calculating results and publishing\nthem by calling <a href=\"#!/api/Ext.layout.ContextItem-method-setProp\" rel=\"Ext.layout.ContextItem-method-setProp\" class=\"docClass\">setProp</a>. It is important to realize\nthat <a href=\"#!/api/Ext.layout.ContextItem-method-getProp\" rel=\"Ext.layout.ContextItem-method-getProp\" class=\"docClass\">getProp</a> will return <code>undefined</code> if the value\nis not yet known. But the act of calling the method is enough to track the fact that the\ncalling layout depends (in some way) on this value. In other words, the calling layout is\n\"triggered\" by the properties it requests.</p>\n\n<p><strong>RULE 2</strong>: Avoid calling <a href=\"#!/api/Ext.layout.ContextItem-method-getProp\" rel=\"Ext.layout.ContextItem-method-getProp\" class=\"docClass\">getProp</a> unless the value\nis needed. Gratuitous calls cause inefficiency because the layout will appear to depend on\nvalues that it never actually uses. This applies equally to\n<a href=\"#!/api/Ext.layout.ContextItem-method-getDomProp\" rel=\"Ext.layout.ContextItem-method-getDomProp\" class=\"docClass\">getDomProp</a> and the test-only methods\n<a href=\"#!/api/Ext.layout.ContextItem-method-hasProp\" rel=\"Ext.layout.ContextItem-method-hasProp\" class=\"docClass\">hasProp</a> and <a href=\"#!/api/Ext.layout.ContextItem-method-hasDomProp\" rel=\"Ext.layout.ContextItem-method-hasDomProp\" class=\"docClass\">hasDomProp</a>.</p>\n\n<p>Because <a href=\"#!/api/Ext.layout.ContextItem-method-getProp\" rel=\"Ext.layout.ContextItem-method-getProp\" class=\"docClass\">getProp</a> can return <code>undefined</code>, it is often\nthe case that subsequent math will produce NaN's. This is usually not a problem as the\nNaN's simply propagate along and result in final results that are NaN. Both <code>undefined</code>\nand NaN are ignored by <a href=\"#!/api/Ext.layout.ContextItem-method-setProp\" rel=\"Ext.layout.ContextItem-method-setProp\" class=\"docClass\">Ext.layout.ContextItem.setProp</a>, so it is often not necessary\nto even know that this is happening. It does become important for determining if a layout\nis not done or if it might lead to publishing an incorrect (but not NaN or <code>undefined</code>)\nvalue.</p>\n\n<p><strong>RULE 3</strong>: If a layout has not calculated all the values it is required to calculate, it\nmust set <a href=\"#!/api/Ext.layout.Layout-property-done\" rel=\"Ext.layout.Layout-property-done\" class=\"docClass\">done</a> to <code>false</code> before returning from\n<a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a>. This value is always <code>true</code> on entry because\nit is simpler to detect the incomplete state rather than the complete state (especially up\nand down a class hierarchy).</p>\n\n<p><strong>RULE 4</strong>: A layout must never publish an incomplete (wrong) result. Doing so would cause\ndependent layouts to run their calculations on those wrong values, producing more wrong\nvalues and some layouts may even incorrectly flag themselves as <a href=\"#!/api/Ext.layout.Layout-property-done\" rel=\"Ext.layout.Layout-property-done\" class=\"docClass\">done</a>\nbefore the correct values are determined and republished. Doing this will poison the\ncalculations.</p>\n\n<p><strong>RULE 5</strong>: Each value should only be published by one layout. If multiple layouts attempt\nto publish the same values, it would be nearly impossible to avoid breaking <strong>RULE 4</strong>. To\nhelp detect this problem, the layout diagnostics will trap on an attempt to set a value\nfrom different layouts.</p>\n\n<p>Complex layouts can produce many results as part of their calculations. These values are\nimportant for other layouts to proceed and need to be published by the earliest possible\ncall to <a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">Ext.layout.Layout.calculate</a> to avoid unnecessary cycles and poor performance. It is\nalso possible, however, for some results to be related in a way such that publishing them\nmay be an all-or-none proposition (typically to avoid breaking <em>RULE 4</em>).</p>\n\n<p><strong>RULE 6</strong>: Publish results as soon as they are known to be correct rather than wait for\nall values to be calculated. Waiting for everything to be complete can lead to deadlock.\nThe key here is not to forget <strong>RULE 4</strong> in the process.</p>\n\n<p>Some layouts depend on certain critical values as part of their calculations. For example,\nHBox depends on width and cannot do anything until the width is known. In these cases, it\nis best to use <a href=\"#!/api/Ext.layout.ContextItem-method-block\" rel=\"Ext.layout.ContextItem-method-block\" class=\"docClass\">block</a> or\n<a href=\"#!/api/Ext.layout.ContextItem-method-domBlock\" rel=\"Ext.layout.ContextItem-method-domBlock\" class=\"docClass\">domBlock</a> and thereby avoid processing the layout\nuntil the needed value is available.</p>\n\n<p><strong>RULE 7</strong>: Use <a href=\"#!/api/Ext.layout.ContextItem-method-block\" rel=\"Ext.layout.ContextItem-method-block\" class=\"docClass\">block</a> or\n<a href=\"#!/api/Ext.layout.ContextItem-method-domBlock\" rel=\"Ext.layout.ContextItem-method-domBlock\" class=\"docClass\">domBlock</a> when values are required to make progress.\nThis will mimize wasted recalculations.</p>\n\n<p><strong>RULE 8</strong>: Blocks should only be used when no forward progress can be made. If even one\nvalue could still be calculated, a block could result in a deadlock.</p>\n\n<p>Historically, layouts have been invoked directly by component code, sometimes in places\nlike an <code>afterLayout</code> method for a child component. With the flexibility now available\nto solve complex, iterative issues, such things should be done in a responsible layout\n(be it component or container).</p>\n\n<p><strong>RULE 9</strong>: Use layouts to solve layout issues and don't wait for the layout to finish to\nperform further layouts. This is especially important now that layouts process entire\ncomponent trees and not each layout in isolation.</p>\n\n<h1>Sequence Diagram</h1>\n\n<p>The simplest sequence diagram for a layout run looks roughly like this:</p>\n\n<pre><code>  Context         Layout 1     Item 1     Layout 2     Item 2\n     |               |           |           |           |\n----&gt;X--------------&gt;X           |           |           |\nrun  X---------------|-----------|----------&gt;X           |\n     X beginLayout   |           |           |           |\n     X               |           |           |           |\n   A X--------------&gt;X           |           |           |\n     X  calculate    X----------&gt;X           |           |\n     X             C X  getProp  |           |           |\n   B X               X----------&gt;X           |           |\n     X               |  setProp  |           |           |\n     X               |           |           |           |\n   D X---------------|-----------|----------&gt;X           |\n     X  calculate    |           |           X----------&gt;X\n     X               |           |           |  setProp  |\n   E X               |           |           |           |\n     X---------------|-----------|----------&gt;X           |\n     X completeLayout|           |         F |           |\n     X               |           |           |           |\n   G X               |           |           |           |\n   H X--------------&gt;X           |           |           |\n     X  calculate    X----------&gt;X           |           |\n     X             I X  getProp  |           |           |\n     X               X----------&gt;X           |           |\n     X               |  setProp  |           |           |\n   J X--------------&gt;X           |           |           |\n     X completeLayout|           |           |           |\n     X               |           |           |           |\n   K X--------------&gt;X           |           |           |\n     X---------------|-----------|----------&gt;X           |\n     X finalizeLayout|           |           |           |\n     X               |           |           |           |\n   L X--------------&gt;X           |           |           |\n     X---------------|-----------|----------&gt;X           |\n     X finishedLayout|           |           |           |\n     X               |           |           |           |\n   M X--------------&gt;X           |           |           |\n     X---------------|-----------|----------&gt;X           |\n     X notifyOwner   |           |           |           |\n   N |               |           |           |           |\n     -               -           -           -           -\n</code></pre>\n\n<p>Notes:</p>\n\n<p><strong>A.</strong> This is a call from the <a href=\"#!/api/Ext.layout.Context-method-run\" rel=\"Ext.layout.Context-method-run\" class=\"docClass\">run</a> method to the <a href=\"#!/api/Ext.layout.Context-method-runCycle\" rel=\"Ext.layout.Context-method-runCycle\" class=\"docClass\">runCycle</a> method.\nEach layout in the queue will have its <a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a>\nmethod called.</p>\n\n<p><strong>B.</strong> After each <a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a> method is called the\n<a href=\"#!/api/Ext.layout.Layout-property-done\" rel=\"Ext.layout.Layout-property-done\" class=\"docClass\">done</a> flag is checked to see if the Layout has completed.\nIf it has completed and that layout object implements a\n<a href=\"#!/api/Ext.layout.Layout-method-completeLayout\" rel=\"Ext.layout.Layout-method-completeLayout\" class=\"docClass\">completeLayout</a> method, this layout is queued to\nreceive its call. Otherwise, the layout will be queued again unless there are blocks or\ntriggers that govern its requeueing.</p>\n\n<p><strong>C.</strong> The call to <a href=\"#!/api/Ext.layout.ContextItem-method-getProp\" rel=\"Ext.layout.ContextItem-method-getProp\" class=\"docClass\">getProp</a> is made to the Item\nand that will be tracked as a trigger (keyed by the name of the property being requested).\nChanges to this property will cause this layout to be requeued. The call to\n<a href=\"#!/api/Ext.layout.ContextItem-method-setProp\" rel=\"Ext.layout.ContextItem-method-setProp\" class=\"docClass\">setProp</a> will place a value in the item and not\ndirectly into the DOM.</p>\n\n<p><strong>D.</strong> Call the other layouts now in the first cycle (repeat <strong>B</strong> and <strong>C</strong> for each\nlayout).</p>\n\n<p><strong>E.</strong> After completing a cycle, if progress was made (new properties were written to\nthe context) and if the <a href=\"#!/api/Ext.layout.Context-property-layoutQueue\" rel=\"Ext.layout.Context-property-layoutQueue\" class=\"docClass\">layoutQueue</a> is not empty, the next cycle is run. If no\nprogress was made or no layouts are ready to run, all buffered values are written to\nthe DOM (a flush).</p>\n\n<p><strong>F.</strong> After flushing, any layouts that were marked as <a href=\"#!/api/Ext.layout.Layout-property-done\" rel=\"Ext.layout.Layout-property-done\" class=\"docClass\">done</a>\nthat also have a <a href=\"#!/api/Ext.layout.Layout-method-completeLayout\" rel=\"Ext.layout.Layout-method-completeLayout\" class=\"docClass\">completeLayout</a> method are called.\nThis can cause them to become no longer done (see <a href=\"#!/api/Ext.layout.Context-method-invalidate\" rel=\"Ext.layout.Context-method-invalidate\" class=\"docClass\">invalidate</a>). As with\n<a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a>, this is considered a \"read phase\" and\ndirect DOM writes should be avoided.</p>\n\n<p><strong>G.</strong> Flushing and calling any pending <a href=\"#!/api/Ext.layout.Layout-method-completeLayout\" rel=\"Ext.layout.Layout-method-completeLayout\" class=\"docClass\">completeLayout</a>\nmethods will likely trigger layouts that called <a href=\"#!/api/Ext.layout.ContextItem-method-getDomProp\" rel=\"Ext.layout.ContextItem-method-getDomProp\" class=\"docClass\">getDomProp</a>\nand unblock layouts that have called <a href=\"#!/api/Ext.layout.ContextItem-method-domBlock\" rel=\"Ext.layout.ContextItem-method-domBlock\" class=\"docClass\">domBlock</a>.\nThese variants are used when a layout needs the value to be correct in the DOM and not\nsimply known. If this does not cause at least one layout to enter the queue, we have a\nlayout FAILURE. Otherwise, we continue with the next cycle.</p>\n\n<p><strong>H.</strong> Call <a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a> on any layouts in the queue\nat the start of this cycle. Just a repeat of <strong>B</strong> through <strong>G</strong>.</p>\n\n<p><strong>I.</strong> Once the layout has calculated all that it is resposible for, it can leave itself\nin the <a href=\"#!/api/Ext.layout.Layout-property-done\" rel=\"Ext.layout.Layout-property-done\" class=\"docClass\">done</a> state. This is the value on entry to\n<a href=\"#!/api/Ext.layout.Layout-method-calculate\" rel=\"Ext.layout.Layout-method-calculate\" class=\"docClass\">calculate</a> and must be cleared in that call if the\nlayout has more work to do.</p>\n\n<p><strong>J.</strong> Now that all layouts are done, flush any DOM values and\n<a href=\"#!/api/Ext.layout.Layout-method-completeLayout\" rel=\"Ext.layout.Layout-method-completeLayout\" class=\"docClass\">completeLayout</a> calls. This can again cause\nlayouts to become not done, and so we will be back on another cycle if that happens.</p>\n\n<p><strong>K.</strong> After all layouts are done, call the <a href=\"#!/api/Ext.layout.Layout-method-finalizeLayout\" rel=\"Ext.layout.Layout-method-finalizeLayout\" class=\"docClass\">finalizeLayout</a>\nmethod on any layouts that have one. As with <a href=\"#!/api/Ext.layout.Layout-method-completeLayout\" rel=\"Ext.layout.Layout-method-completeLayout\" class=\"docClass\">completeLayout</a>,\nthis can cause layouts to become no longer done. This is less desirable than using\n<a href=\"#!/api/Ext.layout.Layout-method-completeLayout\" rel=\"Ext.layout.Layout-method-completeLayout\" class=\"docClass\">completeLayout</a> because it will cause all\n<a href=\"#!/api/Ext.layout.Layout-method-finalizeLayout\" rel=\"Ext.layout.Layout-method-finalizeLayout\" class=\"docClass\">finalizeLayout</a> methods to be called again\nwhen we think things are all wrapped up.</p>\n\n<p><strong>L.</strong> After finishing the last iteration, layouts that have a\n<a href=\"#!/api/Ext.layout.Layout-method-finishedLayout\" rel=\"Ext.layout.Layout-method-finishedLayout\" class=\"docClass\">finishedLayout</a> method will be called. This\ncall will only happen once per run and cannot cause layouts to be run further.</p>\n\n<p><strong>M.</strong> After calling finahedLayout, layouts that have a\n<a href=\"#!/api/Ext.layout.Layout-method-notifyOwner\" rel=\"Ext.layout.Layout-method-notifyOwner\" class=\"docClass\">notifyOwner</a> method will be called. This\ncall will only happen once per run and cannot cause layouts to be run further.</p>\n\n<p><strong>N.</strong> One last flush to make sure everything has been written to the DOM.</p>\n\n<h1>Inter-Layout Collaboration</h1>\n\n<p>Many layout problems require collaboration between multiple layouts. In some cases, this\nis as simple as a component's container layout providing results used by its component\nlayout or vise-versa. A slightly more distant collaboration occurs in a box layout when\nstretchmax is used: the child item's component layout provides results that are consumed\nby the ownerCt's box layout to determine the size of the children.</p>\n\n<p>The various forms of interdependence between a container and its children are described by\neach components' <a href=\"#!/api/Ext.Component-method-getSizeModel\" rel=\"Ext.Component-method-getSizeModel\" class=\"docClass\">size model</a>.</p>\n\n<p>To facilitate this collaboration, the following pairs of properties are published to the\ncomponent's <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">ContextItem</a>:</p>\n\n<ul>\n<li>width/height: These hold the final size of the component. The layout indicated by the\n<a href=\"#!/api/Ext.Component-method-getSizeModel\" rel=\"Ext.Component-method-getSizeModel\" class=\"docClass\">size model</a> is responsible for setting these.</li>\n<li>contentWidth/contentHeight: These hold size information published by the container\nlayout or from DOM measurement. These describe the content only. These values are\nused by the component layout to determine the outer width/height when that component\nis <a href=\"#!/api/Ext.Component-cfg-shrinkWrap\" rel=\"Ext.Component-cfg-shrinkWrap\" class=\"docClass\">shrink-wrapped</a>. They are also used to\ndetermine overflow. All container layouts must publish these values for dimensions\nthat are shrink-wrapped. If a component has raw content (not container items), the\ncomponentLayout must publish these values instead.</li>\n</ul>\n\n","!type":"fn(config: ?)","prototype":{"debugHooks":{"!doc":"<hr />\n\n<p> Diagnostics</p>\n"},"layoutQueue":{"!doc":"<p>List of layouts to perform.</p>\n"},"remainingLayouts":{"!type":"number"},"state":{"!type":"number","!doc":"<p>One of these values:</p>\n\n<ul>\n<li>0 - Before run</li>\n<li>1 - Running</li>\n<li>2 - Run complete</li>\n</ul>\n\n"},"callLayout":{"!type":"fn(layout: ?, methodName: ?) -> !this"},"cancelComponent":{"!type":"fn(comp: ?, isChild: ?, isDestroying: ?) -> !this"},"cancelLayout":{"!type":"fn(layout: ?) -> !this"},"chainFns":{"!type":"fn(oldOptions: ?, newOptions: ?, funcName: ?) -> !this"},"clearTriggers":{"!type":"fn(layout: ?, inDom: ?) -> !this"},"flush":{"!type":"fn() -> !this","!doc":"<p>Flushes any pending writes to the DOM by calling each ContextItem in the flushQueue.</p>\n"},"flushAnimations":{"!type":"fn() -> !this"},"flushInvalidates":{"!type":"fn() -> !this"},"flushLayouts":{"!type":"fn(queueName: ?, methodName: ?, dontClear: ?) -> !this"},"getCmp":{"!type":"fn(cmp: ?) -> !this","!doc":"<p>Returns the ContextItem for a component.</p>\n"},"getEl":{"!type":"fn(parent: ?, el: ?) -> !this","!doc":"<p>Returns the ContextItem for an element.</p>\n"},"getItem":{"!type":"fn(target: ?, el: ?) -> !this"},"handleFailure":{"!type":"fn() -> !this"},"invalidate":{"!type":"fn(components: ?, full: ?) -> !this","!doc":"<p>Invalidates one or more components' layouts (component and container). This can be\ncalled before run to identify the components that need layout or during the run to\nrestart the layout of a component. This is called internally to flush any queued\ninvalidations at the start of a cycle. If called during a run, it is not expected\nthat new components will be introduced to the layout.</p>\n"},"layoutDone":{"!type":"fn(layout: ?) -> !this"},"newQueue":{"!type":"fn() -> !this"},"processInvalidate":{"!type":"fn(options: ?, item: ?, name: ?) -> !this"},"purgeInvalidates":{"!type":"fn() -> !this"},"queueAnimation":{"!type":"fn(item: ?) -> !this","!doc":"<p>Queues a ContextItem to have its <a href=\"#!/api/Ext.layout.ContextItem-method-flushAnimations\" rel=\"Ext.layout.ContextItem-method-flushAnimations\" class=\"docClass\">Ext.layout.ContextItem.flushAnimations</a> method called.</p>\n"},"queueCompletion":{"!type":"fn(layout: ?) -> !this","!doc":"<p>Queues a layout to have its <a href=\"#!/api/Ext.layout.Layout-method-completeLayout\" rel=\"Ext.layout.Layout-method-completeLayout\" class=\"docClass\">Ext.layout.Layout.completeLayout</a> method called.</p>\n"},"queueFinalize":{"!type":"fn(layout: ?) -> !this","!doc":"<p>Queues a layout to have its <a href=\"#!/api/Ext.layout.Layout-method-finalizeLayout\" rel=\"Ext.layout.Layout-method-finalizeLayout\" class=\"docClass\">Ext.layout.Layout.finalizeLayout</a> method called.</p>\n"},"queueFlush":{"!type":"fn(item: ?) -> !this","!doc":"<p>Queues a ContextItem for the next flush to the DOM. This should only be called by\nthe <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> class.</p>\n"},"queueInvalidate":{"!type":"fn(item: ?, options: ?) -> !this","!doc":"<p>Queue a component (and its tree) to be invalidated on the next cycle.</p>\n"},"queueItemLayouts":{"!type":"fn(item: ?) -> !this"},"queueLayout":{"!type":"fn(layout: ?) -> !this","!doc":"<p>Queues a layout for the next calculation cycle. This should not be called if the\nlayout is done, blocked or already in the queue. The only classes that should call\nthis method are this class and <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a>.</p>\n"},"removeEl":{"!type":"fn(parent: ?, el: ?) -> !this","!doc":"<p>Removes the ContextItem for an element from the cache and from the parent's\n\"children\" array.</p>\n"},"resetLayout":{"!type":"fn(layout: ?, ownerContext: ?, firstTime: ?) -> !this","!doc":"<p>Resets the given layout object. This is called at the start of the run and can also\nbe called during the run by calling <a href=\"#!/api/Ext.layout.Context-method-invalidate\" rel=\"Ext.layout.Context-method-invalidate\" class=\"docClass\">invalidate</a>.</p>\n"},"run":{"!type":"fn() -> bool","!doc":"<p>Runs the layout calculations. This can be called only once on this object.</p>\n"},"runComplete":{"!type":"fn() -> !this"},"runCycle":{"!type":"fn() -> bool","!doc":"<p>Performs one layout cycle by calling each layout in the layout queue.</p>\n"},"runLayout":{"!type":"fn(layout: ?) -> !this","!doc":"<p>Runs one layout as part of a cycle.</p>\n"},"setItemSize":{"!type":"fn(item: ?, width: ?, height: ?) -> !this","!doc":"<p>Set the size of a component, element or composite or an array of components or elements.</p>\n"}}},"ContextItem":{"!doc":"<p>This class manages state information for a component or element during a layout.</p>\n\n<h1>Blocks</h1>\n\n<p>A \"block\" is a required value that is preventing further calculation. When a layout has\nencountered a situation where it cannot possibly calculate results, it can associate\nitself with the context item and missing property so that it will not be rescheduled\nuntil that property is set.</p>\n\n<p>Blocks are a one-shot registration. Once the property changes, the block is removed.</p>\n\n<p>Be careful with blocks. If <em>any</em> further calculations can be made, a block is not the\nright choice.</p>\n\n<h1>Triggers</h1>\n\n<p>Whenever any call to <a href=\"#!/api/Ext.layout.ContextItem-method-getProp\" rel=\"Ext.layout.ContextItem-method-getProp\" class=\"docClass\">getProp</a>, <a href=\"#!/api/Ext.layout.ContextItem-method-getDomProp\" rel=\"Ext.layout.ContextItem-method-getDomProp\" class=\"docClass\">getDomProp</a>, <a href=\"#!/api/Ext.layout.ContextItem-method-hasProp\" rel=\"Ext.layout.ContextItem-method-hasProp\" class=\"docClass\">hasProp</a> or\n<a href=\"#!/api/Ext.layout.ContextItem-method-hasDomProp\" rel=\"Ext.layout.ContextItem-method-hasDomProp\" class=\"docClass\">hasDomProp</a> is made, the current layout is automatically registered as being\ndependent on that property in the appropriate state. Any changes to the property will\ntrigger the layout and it will be queued in the <a href=\"#!/api/Ext.layout.Context\" rel=\"Ext.layout.Context\" class=\"docClass\">Ext.layout.Context</a>.</p>\n\n<p>Triggers, once added, remain for the entire layout. Any changes to the property will\nreschedule all unfinished layouts in their trigger set.</p>\n","!type":"fn(config: ?)","prototype":{"borderNames":{},"boxChildren":{"!doc":"<p>plaed here by Component.getSizeModel</p>\n"},"boxParent":{},"cacheMissHandlers":{},"children":{},"consumersContainerHeight":{"!type":"number"},"consumersContainerWidth":{"!type":"number"},"consumersContentHeight":{"!type":"number"},"consumersContentWidth":{"!type":"number"},"consumersHeight":{"!type":"number"},"consumersWidth":{"!type":"number"},"debugHooks":{"!doc":"<hr />\n\n<p> Diagnostics</p>\n"},"dirty":{},"dirtyCount":{"!type":"number","!doc":"<p>The number of dirty properties</p>\n"},"hasRawContent":{"!type":"bool"},"heightModel":{},"isContextItem":{"!type":"bool"},"isTopLevel":{"!type":"bool"},"marginNames":{},"optOut":{"!type":"bool","!doc":"<p>There are several cases that allow us to skip (opt out) of laying out a component\nand its children as long as its <code>lastBox</code> is not marked as <code>invalid</code>. If anything\nhappens to change things, the <code>lastBox</code> is marked as <code>invalid</code> by <code>updateLayout</code>\nas it ascends the component hierarchy.</p>\n"},"ownerCtContext":{},"ownerSizePolicy":{},"paddingNames":{},"props":{"!doc":"<p>the current set of property values:</p>\n"},"remainingChildDimensions":{"!type":"number"},"sizeModel":{},"state":{"!doc":"<p>State variables that are cleared when invalidated. Only applies to component items.</p>\n"},"styles":{"!doc":"<p>Adds x and y values from a props object to a styles object as \"left\" and \"top\" values.\nOverridden to add the x property as \"right\" in rtl mode.A styles object for an Element</p>\n"},"translateProps":{},"trblNames":{},"widthModel":{},"wrapsComponent":{"!type":"bool","!doc":"<p>True if this item wraps a Component (rather than an Element).</p>\n"},"addBlock":{"!type":"fn(name: ?, layout: ?, propName: ?) -> !this","!doc":"<p>Adds a block.</p>\n"},"addBoxChild":{"!type":"fn(boxChildItem: ?) -> !this"},"addCls":{"!type":"fn(newCls: ?) -> !this","!doc":"<p>Queue the addition of a class name (or array of class names) to this ContextItem's target when next flushed.</p>\n"},"addTrigger":{"!type":"fn(propName: ?, inDom: ?) -> !this","!doc":"<p>Adds a trigger.</p>\n"},"block":{"!type":"fn(layout: ?, propName: ?) -> !this","!doc":"<p>Registers a layout in the block list for the given property. Once the property is\nset in the <a href=\"#!/api/Ext.layout.Context\" rel=\"Ext.layout.Context\" class=\"docClass\">Ext.layout.Context</a>, the layout is unblocked.</p>\n"},"boxChildMeasured":{"!type":"fn() -> !this"},"checkCache":{"!type":"fn(entry: ?) -> !this"},"clearAllBlocks":{"!type":"fn(name: ?) -> !this"},"clearBlocks":{"!type":"fn(name: ?, propName: ?) -> !this","!doc":"<p>Removes any blocks on a property in the specified set. Any layouts that were blocked\nby this property and are not still blocked (by other properties) will be rescheduled.</p>\n"},"clearMarginCache":{"!type":"fn() -> !this","!doc":"<p>clears the margin cache so that marginInfo get re-read from the dom on the next call to getMarginInfo()\nThis is needed in some special cases where the margins have changed since the last layout, making the cached\nvalues invalid.  For example collapsed window headers have different margin than expanded ones.</p>\n"},"domBlock":{"!type":"fn(layout: ?, propName: ?) -> !this","!doc":"<p>Registers a layout in the DOM block list for the given property. Once the property\nflushed to the DOM by the <a href=\"#!/api/Ext.layout.Context\" rel=\"Ext.layout.Context\" class=\"docClass\">Ext.layout.Context</a>, the layout is unblocked.</p>\n"},"fireTriggers":{"!type":"fn(name: ?, propName: ?) -> !this","!doc":"<p>Reschedules any layouts associated with a given trigger.</p>\n"},"flush":{"!type":"fn() -> !this","!doc":"<p>Flushes any updates in the dirty collection to the DOM. This is only called if there\nare dirty entries because this object is only added to the flushQueue of the\n<a href=\"#!/api/Ext.layout.Context\" rel=\"Ext.layout.Context\" class=\"docClass\">Ext.layout.Context</a> when entries become dirty.</p>\n"},"flushAnimations":{"!type":"fn() -> !this"},"getBorderInfo":{"!type":"fn() -> ?","!doc":"<p>Gets the border information for the element as an object with left, top, right and\nbottom properties holding border size in pixels. This object is only read from the\nDOM on first request and is cached.</p>\n"},"getClassList":{"!type":"fn() -> !this","!doc":"<p>Returns a ClassList-like object to buffer access to this item's element's classes.</p>\n"},"getDomProp":{"!type":"fn(propName: ?) -> ?","!doc":"<p>Gets a property of this object if it is correct in the DOM. Also tracks the current\nlayout as dependent on this property so that DOM writes of it will trigger the\nlayout to be recalculated.</p>\n"},"getEl":{"!type":"fn(nameOrEl: ?, owner?: ?) -> ?","!doc":"<p>Returns the context item for an owned element. This should only be called on a\ncomponent's item. The list of child items is used to manage invalidating calculated\nresults.</p>\n"},"getFrameInfo":{"!type":"fn() -> ?","!doc":"<p>Gets the \"frame\" information for the element as an object with left, top, right and\nbottom properties holding border+framing size in pixels. This object is calculated\non first request and is cached.</p>\n"},"getMarginInfo":{"!type":"fn() -> ?","!doc":"<p>Gets the margin information for the element as an object with left, top, right and\nbottom properties holding margin size in pixels. This object is only read from the\nDOM on first request and is cached.</p>\n"},"getPaddingInfo":{"!type":"fn() -> ?","!doc":"<p>Gets the padding information for the element as an object with left, top, right and\nbottom properties holding padding size in pixels. This object is only read from the\nDOM on first request and is cached.</p>\n"},"getProp":{"!type":"fn(propName: ?) -> ?","!doc":"<p>Gets a property of this object. Also tracks the current layout as dependent on this\nproperty so that changes to it will trigger the layout to be recalculated.</p>\n"},"getStyle":{"!type":"fn(styleName: ?) -> ?","!doc":"<p>Returns a style for this item. Each style is read from the DOM only once on first\nrequest and is then cached. If the value is an integer, it is parsed automatically\n(so '5px' is not returned, but rather 5).</p>\n"},"getStyles":{"!type":"fn(styleNames: ?, altNames?: ?) -> ?","!doc":"<p>Returns styles for this item. Each style is read from the DOM only once on first\nrequest and is then cached. If the value is an integer, it is parsed automatically\n(so '5px' is not returned, but rather 5).</p>\n"},"hasDomProp":{"!type":"fn(propName: ?) -> bool","!doc":"<p>Returns true if the given property is correct in the DOM. This is equivalent to\ncalling <a href=\"#!/api/Ext.layout.ContextItem-method-getDomProp\" rel=\"Ext.layout.ContextItem-method-getDomProp\" class=\"docClass\">getDomProp</a> and not getting an undefined result. In particular,\nthis call registers the current layout to be triggered by flushes of this property.</p>\n"},"hasProp":{"!type":"fn(propName: ?) -> bool","!doc":"<p>Returns true if the given property has been set. This is equivalent to calling\n<a href=\"#!/api/Ext.layout.ContextItem-method-getProp\" rel=\"Ext.layout.ContextItem-method-getProp\" class=\"docClass\">getProp</a> and not getting an undefined result. In particular, this call\nregisters the current layout to be triggered by changes to this property.</p>\n"},"init":{"!type":"fn(full: ?) -> ?","!doc":"<p>Clears all properties on this object except (perhaps) those not calculated by this\ncomponent. This is more complex than it would seem because a layout can decide to\ninvalidate its results and run the component's layouts again, but since some of the\nvalues may be calculated by the container, care must be taken to preserve those\nvalues.</p>\n"},"initAnimation":{"!type":"fn() -> !this"},"initContinue":{"!type":"fn(full: ?) -> !this"},"initDone":{"!type":"fn(containerLayoutDone: ?) -> !this"},"invalidate":{"!type":"fn(options: ?) -> !this","!doc":"<p>Invalidates the component associated with this item. The layouts for this component\nand all of its contained items will be re-run after first clearing any computed\nvalues.</p>\n\n<p>If state needs to be carried forward beyond the invalidation, the <code>options</code> parameter\ncan be used.</p>\n"},"markDirty":{"!type":"fn() -> !this"},"onBoxMeasured":{"!type":"fn() -> !this"},"parseMargins":{"!type":"fn(comp: ?, margins: ?) -> !this"},"peek":{"!type":"fn(propName: ?) -> !this"},"recoverProp":{"!type":"fn(propName: ?, oldProps: ?, oldDirty: ?) -> !this","!doc":"<p>Recovers a property value from the last computation and restores its value and\ndirty state.</p>\n"},"redo":{"!type":"fn(deep: ?) -> !this"},"removeCls":{"!type":"fn(removeCls: ?) -> !this","!doc":"<p>Queue the removal of a class name (or array of class names) from this ContextItem's target when next flushed.</p>\n"},"removeEl":{"!type":"fn(nameOrEl: ?, owner?: ?) -> !this","!doc":"<p>Removes a cached ContextItem that was created using <a href=\"#!/api/Ext.layout.ContextItem-method-getEl\" rel=\"Ext.layout.ContextItem-method-getEl\" class=\"docClass\">getEl</a>.  It may be\nnecessary to call this method if the dom reference for owned element changes so\nthat <a href=\"#!/api/Ext.layout.ContextItem-method-getEl\" rel=\"Ext.layout.ContextItem-method-getEl\" class=\"docClass\">getEl</a> can be called again to reinitialize the ContextItem with the\nnew element.</p>\n"},"revertProps":{"!type":"fn(props: ?) -> !this"},"setAttribute":{"!type":"fn(name: ?, value: ?) -> !this","!doc":"<p>Queue the setting of a DOM attribute on this ContextItem's target when next flushed.</p>\n"},"setBox":{"!type":"fn(box: ?) -> !this"},"setContentHeight":{"!type":"fn(height: ?, measured: ?) -> !this","!doc":"<p>Sets the contentHeight property. If the component uses raw content, then only the\nmeasured height is acceptable.</p>\n\n<p>Calculated values can sometimes be NaN or undefined, which generally mean the\ncalculation is not done. To indicate that such as value was passed, 0 is returned.\nOtherwise, 1 is returned.</p>\n\n<p>If the caller is not measuring (i.e., they are calculating) and the component has raw\ncontent, 1 is returned indicating that the caller is done.</p>\n"},"setContentSize":{"!type":"fn(width: ?, height: ?, measured: ?) -> !this","!doc":"<p>Sets the contentWidth and contentHeight properties. If the component uses raw content,\nthen only the measured values are acceptable.</p>\n\n<p>Calculated values can sometimes be NaN or undefined, which generally means that the\ncalculation is not done. To indicate that either passed value was such a value, false\nreturned. Otherwise, true is returned.</p>\n\n<p>If the caller is not measuring (i.e., they are calculating) and the component has raw\ncontent, true is returned indicating that the caller is done.</p>\n"},"setContentWidth":{"!type":"fn(width: ?, measured: ?) -> !this","!doc":"<p>Sets the contentWidth property. If the component uses raw content, then only the\nmeasured width is acceptable.</p>\n\n<p>Calculated values can sometimes be NaN or undefined, which generally means that the\ncalculation is not done. To indicate that such as value was passed, 0 is returned.\nOtherwise, 1 is returned.</p>\n\n<p>If the caller is not measuring (i.e., they are calculating) and the component has raw\ncontent, 1 is returned indicating that the caller is done.</p>\n"},"setHeight":{"!type":"fn(height: ?, dirty?: ?) -> number","!doc":"<p>Sets the height and constrains the height to min/maxHeight range.</p>\n"},"setProp":{"!type":"fn(propName: ?, value: ?, dirty: ?) -> number","!doc":"<p>Sets a property value. This will unblock and/or trigger dependent layouts if the\nproperty value is being changed. Values of NaN and undefined are not accepted by\nthis method.</p>\n"},"setSize":{"!type":"fn(width: ?, height: ?, dirty: ?) -> !this"},"setWidth":{"!type":"fn(width: ?, dirty?: ?) -> number","!doc":"<p>Sets the height and constrains the width to min/maxWidth range.</p>\n"},"undo":{"!type":"fn(deep: ?) -> !this"},"unsetProp":{"!type":"fn(propName: ?) -> !this"},"writeProps":{"!type":"fn(dirtyProps: ?, flushing: ?) -> !this"}}},"SizeModel":{"!doc":"<p>This class describes a size determination strategy or algorithm used by the layout\nsystem. There are special instances of this class stored as static properties to\navoid needless object instantiation. These instances should be treated as readonly.</p>\n\n<ul>\n<li><code>calculated</code></li>\n<li><code>configured</code></li>\n<li><code>constrainedMax</code></li>\n<li><code>constrainedMin</code></li>\n<li><code>natural</code></li>\n<li><code>shrinkWrap</code></li>\n<li><code>calculatedFromConfigured</code></li>\n<li><code>calculatedFromNatural</code></li>\n<li><code>calculatedFromShrinkWrap</code></li>\n</ul>\n\n\n<p>Using one of these instances is simply:</p>\n\n<pre><code>  var calculated = <a href=\"#!/api/Ext.layout.SizeModel-property-calculated\" rel=\"Ext.layout.SizeModel-property-calculated\" class=\"docClass\">Ext.layout.SizeModel.calculated</a>;\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"auto":{"!type":"bool","!doc":"<p>True if the size is either <code>natural</code> or <code>shrinkWrap</code>, otherwise false.</p>\n"},"calculated":{"!type":"bool","!doc":"<p>True if the size is calculated by the <code>ownerLayout</code>.</p>\n"},"calculatedFromConfigured":{"!type":"bool","!doc":"<p>True if the size is calculated by the <code>ownerLayout</code> based on a configured size.</p>\n"},"calculatedFromNatural":{"!type":"bool","!doc":"<p>True if the size is calculated by the <code>ownerLayout</code> based on <code>natural</code> size model\nresults.</p>\n"},"calculatedFromShrinkWrap":{"!type":"bool","!doc":"<p>True if the size is calculated by the <code>ownerLayout</code> based on <code>shrinkWrap</code> size model\nresults.</p>\n"},"configured":{"!type":"bool","!doc":"<p>True if the size is configured (e.g., by a <code>width</code> or <code>minWidth</code>). The names of\nconfiguration properties can be found in the <a href=\"#!/api/Ext.layout.SizeModel-property-names\" rel=\"Ext.layout.SizeModel-property-names\" class=\"docClass\">names</a> property.</p>\n"},"constrainedMax":{"!type":"bool","!doc":"<p>True if the size is constrained by a <code>maxWidth</code> or <code>maxHeight</code> configuration. This\nis a flavor of <code>configured</code> (since <code>maxWidth</code> and <code>maxHeight</code> are config options).\nIf true, the <a href=\"#!/api/Ext.layout.SizeModel-property-names\" rel=\"Ext.layout.SizeModel-property-names\" class=\"docClass\">names</a> property will be defined as well.</p>\n"},"constrainedMin":{"!type":"bool","!doc":"<p>True if the size is constrained by a <code>minWidth</code> or <code>minHeight</code> configuration. This\nis a flavor of <code>configured</code> (since <code>minWidth</code> and <code>minHeight</code> are config options).\nIf true, the <a href=\"#!/api/Ext.layout.SizeModel-property-names\" rel=\"Ext.layout.SizeModel-property-names\" class=\"docClass\">names</a> property will be defined as well.</p>\n"},"fixed":{"!type":"bool","!doc":"<p>True if the size is either <code>calculated</code> or <code>configured</code>, otherwise false.</p>\n"},"name":{"!type":"string","!doc":"<p>The name of this size model (e.g., \"calculated\").</p>\n"},"names":{"!doc":"<p>An object with the config property names that determine the\nsize.</p>\n"},"natural":{"!type":"bool","!doc":"<p>True if the size is determined by CSS and not by content. Such sizes are assumed to\nbe dependent on the container box and measurement occurs on the outer-most element.</p>\n"},"ordinal":{"!type":"number","!doc":"<p>The 0-based ordinal for this <code>SizeModel</code> instance.</p>\n"},"pairsByHeightOrdinal":{"!doc":"<p>An array of objects indexed by the <a href=\"#!/api/Ext.layout.SizeModel-property-ordinal\" rel=\"Ext.layout.SizeModel-property-ordinal\" class=\"docClass\">ordinal</a> of a height <code>SizeModel</code> on\na width <code>SizeModel</code> to yield an object describing both height and width size\nmodels.</p>\n\n<p>Used like this:</p>\n\n<pre><code> widthModel.pairsByHeightOrdinal[heightModel.ordinal]\n</code></pre>\n\n<p>This provides a reusable object equivalent to the following:</p>\n\n<pre><code> {\n     width: widthModel,\n     height: heightModel\n }\n</code></pre>\n"},"shrinkWrap":{"!type":"bool","!doc":"<p>True if the size is determined by content irrespective of the container box.</p>\n"},"sizeModels":{"!doc":"<p>An object containing all SizeModel instances keyed by <code>name</code>.</p>\n"},"sizeModelsArray":{"!doc":"<p>An array of all SizeModel instances.</p>\n"}}},"SizePolicy":{"!doc":"<p>This class describes how a layout will interact with a component it manages.</p>\n\n<p>There are special instances of this class stored as static properties to avoid object\ninstantiation. All instances of this class should be treated as readonly objects.</p>\n","prototype":{"readsHeight":{"!type":"bool","!doc":"<p>Indicates that the <code>height</code> of the component is consumed.</p>\n"},"readsWidth":{"!type":"bool","!doc":"<p>Indicates that the <code>width</code> of the component is consumed.</p>\n"},"setsHeight":{"!type":"bool","!doc":"<p>Indicates that the <code>height</code> of the component will be set (i.e., calculated).</p>\n"},"setsWidth":{"!type":"bool","!doc":"<p>Indicates that the <code>width</code> of the component will be set (i.e., calculated).</p>\n"}}}},"plugin":{"Abstract":{"prototype":{"stateEvents":{"!doc":"<p>The configured list of stateEvents used to (optionally) participate in Owner Component's state management.</p>\n"},"disabled":{"!type":"bool"},"isPlugin":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Plugin, or subclass thereof.</p>\n"},"applyState":{"!type":"fn(state: ?, allState: ?) -> !this","!doc":"<p>The applyState method is invoked by the client Component's State mixin after initComponent method has been run for the client.</p>\n\n<p>The supplied implementation is empty. If plugin Subclasses are to (optionally) participate in the client Component's\nstate management, implementers should provide a suitable method to utilize it.</p>\n"},"clonePlugin":{"!type":"fn(overrideCfg?: ?) -> !this","!doc":"<p>Creates clone of the plugin.</p>\n"},"disable":{"!type":"fn() -> !this","!doc":"<p>The base implementation just sets the plugin's <code>disabled</code> flag to <code>true</code></p>\n\n<p>Plugin subclasses which need more complex processing may implement an overriding implementation.</p>\n"},"enable":{"!type":"fn() -> !this","!doc":"<p>The base implementation just sets the plugin's <code>disabled</code> flag to <code>false</code></p>\n\n<p>Plugin subclasses which need more complex processing may implement an overriding implementation.</p>\n"},"getCmp":{"!type":"fn() -> ?","!doc":"<p>Returns the component to which this plugin is attached.</p>\n"},"getState":{"!type":"fn() -> ?","!doc":"<p>The getState method is invoked by the client Component's State mixin when one or more of the the specified <a href=\"#!/api/Ext.plugin.Abstract-cfg-stateEvents\" rel=\"Ext.plugin.Abstract-cfg-stateEvents\" class=\"docClass\">stateEvents</a> are raised.</p>\n\n<p>The supplied implementation is empty. If plugin Subclasses are to (optionally) participate in the client Component's\nstate management, implementers should provide a suitable method which returns a state object.</p>\n"},"onClassExtended":{"!type":"fn(cls: ?, data: ?, hooks: ?) -> !this","!doc":"<p>Private.\nInject a ptype property so that Component.findPlugin(ptype) works.</p>\n"},"setCmp":{"!type":"fn(cmp: ?) -> !this","!doc":"<p>Sets the component to which this plugin is attached.</p>\n"},"pluginId":{"!type":"string","!doc":"<p>A name for the plugin that can be set at creation time to then retrieve the plugin\nthrough <a href=\"#!/api/Ext.Component-method-getPlugin\" rel=\"Ext.Component-method-getPlugin\" class=\"docClass\">getPlugin</a> method.  For example:</p>\n\n<pre><code>var grid = Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    plugins: [{\n        ptype: 'cellediting',\n        clicksToEdit: 2,\n        pluginId: 'cellplugin'\n    }]\n});\n\n// later on:\nvar plugin = grid.getPlugin('cellplugin');\n</code></pre>\n"},"init":{"!type":"fn(client: ?) -> !this","!doc":"<p>The init method is invoked after initComponent method has been run for the client Component.</p>\n\n<p>The supplied implementation is empty. Subclasses should perform plugin initialization, and set up bidirectional\nlinks between the plugin and its client Component in their own implementation of this method.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>The destroy method is invoked by the owning Component at the time the Component is being destroyed.</p>\n\n<p>The supplied implementation is empty. Subclasses should perform plugin cleanup in their own implementation of\nthis method.</p>\n"}},"!doc":"<p>The AbstractPlugin class is the base class from which user-implemented plugins should inherit.</p>\n\n<p>This class defines the essential API of plugins as used by Components by defining the following methods:</p>\n\n<ul>\n<li><p><code>init</code> : The plugin initialization method which the owning Component calls at Component initialization time.</p>\n\n<p>The Component passes itself as the sole parameter.</p>\n\n<p>Subclasses should set up bidirectional links between the plugin and its client Component here.</p></li>\n<li><p><code>destroy</code> : The plugin cleanup method which the owning Component calls at Component destruction time.</p>\n\n<p>Use this method to break links between the plugin and the Component and to free any allocated resources.</p></li>\n<li><p><code>enable</code> : The base implementation just sets the plugin's <code>disabled</code> flag to <code>false</code></p></li>\n<li><p><code>disable</code> : The base implementation just sets the plugin's <code>disabled</code> flag to <code>true</code></p></li>\n</ul>\n\n","!type":"fn(config?: ?)"},"Manager":{"!doc":"<p>Creates plugin instances.</p>\n\n<p>A plugin may be specified simply as a <em>config object</em> as long as the correct <code>ptype</code> is specified:</p>\n\n<pre><code>{\n    ptype: 'gridviewdragdrop',\n    dragText: 'Drag and drop to reorganize'\n}\n</code></pre>\n\n<p>Or just use the ptype on its own:</p>\n\n<pre><code>'gridviewdragdrop'\n</code></pre>\n\n<p>Alternatively you can instantiate the plugin with Ext.create:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.grid.plugin.DragDrop\" rel=\"Ext.grid.plugin.DragDrop\" class=\"docClass\">Ext.grid.plugin.DragDrop</a>', {\n    dragText: 'Drag and drop to reorganize'\n})\n</code></pre>\n","typeName":{"!type":"string"},"create":{"!type":"fn(config: ?, defaultType?: ?) -> ?","!doc":"<p>Creates a new Plugin from the specified config object using the config object's ptype to determine the class to\ninstantiate.</p>\n"}},"Responsive":{"!doc":"<p>This plugin can be added to component instances to process a <code>responsiveConfig</code>. For\nexample:</p>\n\n<pre><code> Ext.create({\n     xtype: 'viewport',\n     layout: 'border',\n\n     items: [{\n         title: 'Some Title',\n         plugins: 'responsive',\n\n         responsiveConfig: {\n             'width &lt; 800': {\n                 region: 'north'\n             },\n\n             'width &gt;= 800': {\n                 region: 'west'\n             }\n         }\n     }]\n });\n</code></pre>\n\n<p>For details see <code>responsiveConfig</code>.</p>\n","!type":"fn(config: ?)","prototype":{"isPlugin":{"!type":"bool"},"pluginId":{"!type":"string"},"privates":{},"init":{"!type":"fn() -> !this"}}},"Viewport":{"!doc":"<p>This plugin can be applied to any <code>Component</code> (although almost always to a <code>Container</code>)\nto make it fill the browser viewport. This plugin is used internally by the more familiar\n<code><a href=\"#!/api/Ext.container.Viewport\" rel=\"Ext.container.Viewport\" class=\"docClass\">Ext.container.Viewport</a></code> class.</p>\n\n<p>The <code>Viewport</code> container is commonly used but it can be an issue if you need to fill the\nviewport with a container that derives from another class (e.g., <code><a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a></code>). Prior\nto this plugin, you would have to do this:</p>\n\n<pre><code> Ext.create('<a href=\"#!/api/Ext.container.Viewport\" rel=\"Ext.container.Viewport\" class=\"docClass\">Ext.container.Viewport</a>', {\n     layout: 'fit', // full the viewport with the tab panel\n\n     items: [{\n         xtype: 'tabpanel',\n         items: [{\n             ...\n         }]\n     }]\n });\n</code></pre>\n\n<p>With this plugin you can create the <code>tabpanel</code> as the viewport:</p>\n\n<pre><code> Ext.create('<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>', {\n     plugins: 'viewport',\n\n     items: [{\n         ...\n     }]\n });\n</code></pre>\n\n<p>More importantly perhaps is that as a plugin, the view class can be reused in other\ncontexts such as the content of a <code><a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">window</a></code>.</p>\n\n<p>The Viewport renders itself to the document body, and automatically sizes itself to the size of\nthe browser viewport and manages window resizing. There may only be one Viewport created\nin a page.</p>\n\n<h2>Responsive Design</h2>\n\n<p>This plugin enables Ext.mixin.Responsive.responsiveConfig for the target component.</p>\n","!type":"fn(config: ?)","prototype":{"maxUserScale":{"!type":"number","!doc":"<p>The maximum zoom scale. Only applicable for touch devices. Set this to 1 to\ndisable zooming.  Setting this to any value other than \"1\" will disable all\nmulti-touch gestures.</p>\n"},"privates":{},"apply":{"!type":"fn(target: ?) -> !this"}}}},"view":{"Table":{"prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"componentLayout":{"!doc":"<p>The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout\nmanager which sizes a Component's internal structure in response to the Component being sized.</p>\n\n<p>Generally, developers will not use this configuration as all provided Components which need their internal\nelements sizing (Such as <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">input fields</a>) come with their own componentLayout managers.</p>\n\n<p>The <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used on instances of the base <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>\nclass which simply sizes the Component's encapsulating element to the height and width specified in the\n<a href=\"#!/api/Ext.view.Table-method-setSize\" rel=\"Ext.view.Table-method-setSize\" class=\"docClass\">setSize</a> method.</p>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"enableTextSelection":{"!type":"bool","!doc":"<p>True to enable text selections.</p>\n"},"firstCls":{"!type":"string","!doc":"<p>A CSS class to add to the <em>first</em> cell in every row to enable special styling for the first column.\nIf no styling is needed on the first column, this may be configured as <code>null</code>.</p>\n"},"itemSelector":{"!type":"string","!doc":"<p>view item. This wraps a data row</p>\n"},"lastCls":{"!type":"string","!doc":"<p>A CSS class to add to the <em>last</em> cell in every row to enable special styling for the last column.\nIf no styling is needed on the last column, this may be configured as <code>null</code>.</p>\n"},"markDirty":{"!type":"bool","!doc":"<p>True to show the dirty cell indicator when a cell has been modified.</p>\n"},"overItemCls":{"!type":"string","!doc":"<p>A CSS class to apply to each item in the view on mouseover.\nSetting this will automatically set <a href=\"#!/api/Ext.view.Table-cfg-trackOver\" rel=\"Ext.view.Table-cfg-trackOver\" class=\"docClass\">trackOver</a> to <code>true</code>.</p>\n"},"selectedItemCls":{"!type":"string","!doc":"<p>A CSS class to apply to each selected item in the view.</p>\n"},"stripeRows":{"!type":"bool","!doc":"<p>True to stripe the rows.</p>\n\n<p>This causes the CSS class <strong><code>x-grid-row-alt</code></strong> to be added to alternate rows of\nthe grid. A default CSS rule is provided which sets a background color, but you can override this\nwith a rule which either overrides the <strong>background-color</strong> style using the <code>!important</code>\nmodifier, or which uses a CSS selector of higher specificity.</p>\n"},"trackOver":{"!type":"bool","!doc":"<p>cfg docs inherited</p>\n"},"altRowCls":{"!type":"string"},"body":{"!doc":"<p>A flyweight Ext.Element which encapsulates a reference to the view's main row containing element.\n<em>Note that the <code>dom</code> reference will not be present until the first data refresh</em></p>\n"},"bodySelector":{"!type":"string","!doc":"<p>Outer table</p>\n"},"cellRe":{},"cellSelector":{"!type":"string","!doc":"<p>cell</p>\n"},"cellTpl":{},"cellValues":{},"dirtyCls":{"!type":"string"},"focusedItemCls":{"!type":"string"},"innerSelector":{"!type":"string"},"nodeContainerSelector":{"!type":"string","!doc":"<p>Element which contains rows</p>\n"},"outerRowTpl":{},"ownerGrid":{"!doc":"<p>A reference to the top-level owning grid component. This is actually the TablePanel\nso it could be a tree.</p>\n"},"positionBody":{"!type":"bool"},"positionCells":{"!type":"bool"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"refreshSelmodelOnRefresh":{"!type":"bool","!doc":"<p>Flag to disable refreshing SelectionModel on view refresh. Table views render rows with selected CSS class already added if necessary.</p>\n"},"renderBuffer":{"!doc":"<p>/ Private used for buffered rendering</p>\n"},"rowClsRe":{},"rowSelector":{"!type":"string","!doc":"<p>Grid row which contains cells as opposed to wrapping item.</p>\n"},"rowTpl":{},"rowValues":{"!doc":"<p>Private properties used during the row and cell render process.\nThey are allocated here on the prototype, and cleared/re-used to avoid GC churn during repeated rendering.</p>\n"},"selectedCellCls":{"!type":"string"},"sizerSelector":{"!type":"string","!doc":"<p>Select column sizers and cells.\nThis may target <COL> elements as well as <TD> elements\n<COLGROUP> element is inserted if the first row does not have the regular cell patten (eg is a colspanning group header row)</COLGROUP></TD></COL></p>\n"},"stripeOnUpdate":{},"tableValues":{},"tpl":{"!doc":"<p>Outer tpl for TableView just to satisfy the validation within AbstractView.initComponent.</p>\n"},"unselectableCls":{"!type":"string"},"events":{"!doc":"<p>Events a TableView may fire.\nUsed by <a href=\"#!/api/Ext.grid.locking.View\" rel=\"Ext.grid.locking.View\" class=\"docClass\">Ext.grid.locking.View</a> to relay view events to user code</p>\n"},"addCellTpl":{"!type":"fn(newTpl: ?) -> !this"},"addElListener":{"!type":"fn(eventName: ?, fn: ?, scope: ?) -> !this","!doc":"<p>Add a listener to the main view element. It will be destroyed with the view.</p>\n"},"addFooterFn":{"!type":"fn(fn: ?) -> !this"},"addHeaderFn":{"!type":"fn(fn: ?) -> !this","!doc":"<p>Currently, we don't have ordering support for header/footer functions,\nthey will be pushed on at construction time. If the need does arise,\nwe can add this functionality in the future, but for now it's not\nreally necessary since currently only the summary feature uses this.</p>\n"},"addRowCls":{"!type":"fn(rowInfo: ?, cls: ?) -> !this","!doc":"<p>Adds a CSS Class to a specific row.</p>\n"},"addRowTpl":{"!type":"fn(newTpl: ?) -> !this"},"addTpl":{"!type":"fn(newTpl: ?) -> !this"},"autoSizeColumn":{"!type":"fn(header: ?) -> !this","!doc":"<p>Sizes the passed header to fit the max content width.\n<em>Note that group columns shrinkwrap around the size of leaf columns. Auto sizing a group column\nautosizes descendant leaf columns.</em></p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"beforeLayout":{"!type":"fn() -> !this","!doc":"<p>Occurs before componentLayout is run. In previous releases, this method could\nreturn <code>false</code> to prevent its layout but that is not supported in Ext JS 4.1 or\nhigher. This method is simply a notification of the impending layout to give the\ncomponent a chance to adjust the DOM. Ideally, DOM reads should be avoided at this\ntime to reduce expensive document reflows.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"bufferRender":{"!type":"fn(records: ?, index: ?) -> !this","!doc":"<p>private\nOverride so that we can use a quicker way to access the row nodes.\nThey are simply all child nodes of the nodeContainer element.</p>\n"},"clearChangedCells":{"!type":"fn() -> !this"},"clearViewEl":{"!type":"fn(leaveNodeContainer: ?) -> !this"},"collectData":{"!type":"fn(records: ?, startIndex: ?) -> ?","!doc":"<p>Function which can be overridden which returns the data object passed to this\nDataView's <a href=\"#!/api/Ext.view.Table-cfg-tpl\" rel=\"Ext.view.Table-cfg-tpl\" class=\"docClass\">template</a> to render the whole DataView.</p>\n\n<p>This is usually an Array of data objects, each element of which is processed by an\nXTemplate which uses <code>'&amp;lt;tpl for=\".\"&amp;gt;'</code> to iterate over its supplied\ndata object as an Array. However, <i>named</i> properties may be placed into the data object to\nprovide non-repeating data such as headings, totals etc.</p>\n"},"collectNodes":{"!type":"fn(targetEl: ?) -> !this","!doc":"<p>Overridden implementation.\nCalled by refresh to collect the view item nodes.\nNote that these may be wrapping rows which <em>contain</em> rows which map to records</p>\n"},"constructFeatures":{"!type":"fn() -> !this","!doc":"<p>Converts the features array as configured, into an array of instantiated Feature objects.</p>\n\n<p>Must have no side effects other than Feature instantiation.</p>\n\n<p>MUST NOT update the this.features property, and MUST NOT update the instantiated Features.</p>\n"},"constructRowId":{"!type":"fn(internalId: ?) -> !this"},"createRowElement":{"!type":"fn(record: ?, index: ?, updateColumns: ?) -> !this","!doc":"<p>private\nCreate the DOM element which enapsulates the passed record.\nUsed when updating existing rows, so drills down into resulting structure .</p>\n"},"doFocus":{"!type":"fn(rowDom: ?) -> !this","!doc":"<p>Private\nImplementation which saves and restores scroll position on IE.</p>\n"},"doStripeRows":{"!type":"fn(startRow: ?, endRow?: ?) -> !this","!doc":"<p>Stripes rows from a particular row index.</p>\n"},"expandToFit":{"!type":"fn(header: ?) -> !this","!doc":"<p>Expands a particular header to fit the max content width.</p>\n"},"findFeature":{"!type":"fn(ftype: ?) -> !this","!doc":"<p>Finds a features by ftype in the features array</p>\n"},"focusCell":{"!type":"fn(pos: ?, delay?: ?) -> !this","!doc":"<p>Focuses a particular cell and brings it into view. Will fire the rowfocus event.</p>\n"},"focusRow":{"!type":"fn(row: ?, delay?: ?) -> !this","!doc":"<p>Focuses a particular row and brings it into view. Will fire the rowfocus event.</p>\n"},"getBodySelector":{"!type":"fn() -> !this","!doc":"<p>Returns a CSS selector which selects the outermost element(s) in this view.</p>\n"},"getCell":{"!type":"fn(record: ?, column: ?) -> !this","!doc":"<p>Get the cell (td) for a particular record and column.</p>\n"},"getCellByPosition":{"!type":"fn(position: ?, returnDom: ?) -> !this"},"getCellInnerSelector":{"!type":"fn(header: ?) -> !this","!doc":"<p>Returns a CSS selector which selects the content carrying element within cells.</p>\n"},"getCellPaddingAfter":{"!type":"fn(cell: ?) -> !this"},"getCellSelector":{"!type":"fn(header?: ?) -> !this","!doc":"<p>Returns a CSS selector which selects a particular column if the desired header is passed,\nor a general cell selector is no parameter is passed.</p>\n"},"getColumnSizerSelector":{"!type":"fn(header: ?) -> !this","!doc":"<p>Returns a CSS selector which selects the element(s) which define the width of a column.</p>\n\n<p>This is used by the <a href=\"#!/api/Ext.view.TableLayout\" rel=\"Ext.view.TableLayout\" class=\"docClass\">Ext.view.TableLayout</a> when resizing columns.</p>\n"},"getFeature":{"!type":"fn(id: ?) -> ?","!doc":"<p>Get a reference to a feature</p>\n"},"getFirstVisibleRowIndex":{"!type":"fn() -> !this"},"getGridColumns":{"!type":"fn() -> !this","!doc":"<p>Get the leaf columns used for rendering the grid rows.</p>\n"},"getHeaderAtIndex":{"!type":"fn(index: ?) -> !this","!doc":"<p>Get a leaf level header by index regardless of what the nesting\nstructure is.</p>\n"},"getHeaderByCell":{"!type":"fn(cell: ?) -> !this"},"getHeaderCt":{"!type":"fn() -> !this"},"getItemSelector":{"!type":"fn() -> !this","!doc":"<p>Returns a CSS selector which selects items of the view rendered by the outerRowTpl</p>\n"},"getLastVisibleRowIndex":{"!type":"fn() -> !this"},"getMaskTarget":{"!type":"fn() -> !this","!doc":"<p>Masking a TableView masks its owning GridPanel</p>\n"},"getMaxContentWidth":{"!type":"fn(header: ?) -> !this","!doc":"<p>Returns the max contentWidth of the header's text and all cells\nin the grid under this header.</p>\n"},"getNodeById":{"!type":"fn(id: ?) -> !this"},"getNodeByRecord":{"!type":"fn(record: ?) -> !this"},"getPosition":{"!type":"fn(record: ?, header: ?) -> ?","!doc":"<p>Gets the current XY position of the component's underlying element.</p>\n"},"getPositionByEvent":{"!type":"fn(e: ?) -> !this"},"getRecord":{"!type":"fn(node: ?) -> ?","!doc":"<p>Gets a record from a node</p>\n"},"getRow":{"!type":"fn(nodeInfo: ?) -> ?","!doc":"<p>Returns the table row given the passed Record, or index or node.</p>\n"},"getRowById":{"!type":"fn(id: ?) -> !this"},"getRowByRecord":{"!type":"fn(record: ?) -> !this"},"getRowClass":{"!type":"fn(record: ?, index: ?, rowParams: ?, store: ?) -> string","!doc":"<p>Override this function to apply custom CSS classes to rows during rendering. This function should return the\nCSS class name (or empty string '' for none) that will be added to the row's wrapping div. To apply multiple\nclass names, simply return them space-delimited within the string (e.g. 'my-class another-class').\nExample usage:</p>\n\n<pre><code>viewConfig: {\n    getRowClass: function(record, rowIndex, rowParams, store){\n        return record.get(\"valid\") ? \"row-valid\" : \"row-error\";\n    }\n}\n</code></pre>\n"},"getRowFromItem":{"!type":"fn(item: ?) -> !this"},"getRowId":{"!type":"fn(record: ?) -> !this"},"getViewRange":{"!type":"fn() -> !this"},"getVisibleColumnManager":{"!type":"fn() -> !this"},"handleUpdate":{"!type":"fn(store: ?, record: ?, operation: ?, changedFieldNames: ?) -> !this","!doc":"<p>private</p>\n"},"hasActiveFeature":{"!type":"fn() -> !this"},"hasVariableRowHeight":{"!type":"fn() -> !this","!doc":"<p>Returns <code>true</code> if this view has been configured with variableRowHeight (or this has been set by a plugin/feature)\nwhich might insert arbitrary markup into a grid item. Or if at least one visible column has been configured\nwith variableRowHeight. Or if the store is grouped.</p>\n"},"indexInStore":{"!type":"fn(node: ?) -> !this"},"indexOf":{"!type":"fn(node: ?) -> number","!doc":"<p>Finds the index of the passed node.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"initFeatures":{"!type":"fn(grid: ?) -> !this","!doc":"<p>Initializes each feature and bind it to this view.</p>\n"},"insertTpl":{"!type":"fn(which: ?, newTpl: ?) -> !this"},"moveColumn":{"!type":"fn(fromIdx: ?, toIdx: ?, colsToMove?: ?) -> !this","!doc":"<p>Move a grid column from one position to another</p>\n"},"onAdd":{"!type":"fn(store: ?, records: ?, index: ?) -> !this","!doc":"<p>after adding a row stripe rows from then on</p>\n"},"onBeforeCellClick":{"!type":"fn() -> !this"},"onBeforeCellContextMenu":{"!type":"fn() -> !this"},"onBeforeCellDblClick":{"!type":"fn() -> !this"},"onBeforeCellKeyDown":{"!type":"fn() -> !this"},"onBeforeCellLongPress":{"!type":"fn() -> !this"},"onBeforeCellMouseDown":{"!type":"fn() -> !this"},"onBeforeCellMouseUp":{"!type":"fn() -> !this"},"onCellClick":{"!type":"fn() -> !this"},"onCellContextMenu":{"!type":"fn() -> !this"},"onCellDblClick":{"!type":"fn() -> !this"},"onCellDeselect":{"!type":"fn(position: ?) -> !this"},"onCellKeyDown":{"!type":"fn() -> !this"},"onCellLongPress":{"!type":"fn() -> !this"},"onCellMouseDown":{"!type":"fn() -> !this"},"onCellMouseUp":{"!type":"fn() -> !this"},"onCellSelect":{"!type":"fn(position: ?) -> !this"},"onDataRefresh":{"!type":"fn() -> !this","!doc":"<p>When there's a buffered renderer present, store refresh events cause TableViews to go to scrollTop:0</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onEndUpdate":{"!type":"fn() -> !this"},"onRemove":{"!type":"fn(store: ?, records: ?, index: ?) -> !this","!doc":"<p>after removing a row stripe rows from then on</p>\n"},"onReplace":{"!type":"fn(store: ?, startIndex: ?, oldRecords: ?, newRecords: ?) -> !this","!doc":"<p>Private.\nRespond to store replace event which is fired by GroupStore group expand/collapse operations.\nThis saves a layout because a remove and add operation are coalesced in this operation.</p>\n"},"onRowDeselect":{"!type":"fn(rowIdx: ?) -> !this","!doc":"<p>GridSelectionModel invokes onRowDeselect as selection changes</p>\n"},"onRowFocus":{"!type":"fn(rowIdx: ?, highlight: ?, supressFocus: ?) -> !this","!doc":"<p>GridSelectionModel invokes onRowFocus to 'highlight'\nthe last row focused</p>\n"},"onRowSelect":{"!type":"fn(rowIdx: ?) -> !this","!doc":"<p>GridSelectionModel invokes onRowSelect as selection changes</p>\n"},"onUpdate":{"!type":"fn(store: ?, record: ?, operation: ?, modifiedFieldNames: ?) -> !this","!doc":"<p>private</p>\n"},"onViewScroll":{"!type":"fn(e: ?, t: ?) -> !this","!doc":"<p>Private template method implemented starting at the AbstractView class.</p>\n"},"processItemEvent":{"!type":"fn(record: ?, item: ?, rowIndex: ?, e: ?) -> !this","!doc":"<p>Private template method</p>\n"},"processSpecialEvent":{"!type":"fn(e: ?) -> !this"},"refresh":{"!type":"fn() -> !this","!doc":"<p>Refreshes the grid view. Sets the sort state and focuses the previously focused row.</p>\n"},"refreshSelection":{"!type":"fn() -> !this"},"refreshSize":{"!type":"fn(forceLayout: ?) -> !this","!doc":"<p>Private. Called when the table changes height.\nFor example, see examples/grid/group-summary-grid.html\nIf we have flexed column headers, we need to update the header layout\nbecause it may have to accommodate (or cease to accommodate) a vertical scrollbar.\nOnly do this on platforms which have a space-consuming scrollbar.\nOnly do it when vertical scrolling is enabled.</p>\n"},"removeRowCls":{"!type":"fn(rowInfo: ?, cls: ?) -> !this","!doc":"<p>Removes a CSS Class from a specific row.</p>\n"},"renderCell":{"!type":"fn(column: ?, recordIndex: ?, rowIndex: ?, columnIndex: ?, out: ?) -> !this","!doc":"<p>Emits the HTML representing a single grid cell into the passed output stream (which is an array of strings).</p>\n"},"renderColumnSizer":{"!type":"fn(values: ?, out: ?) -> !this","!doc":"<p>Alternative column sizer element renderer.\n   renderTHeadColumnSizer: function(values, out) {\n       var columns = this.getGridColumns(),\n           len = columns.length, i,\n           column, width;</p>\n\n<pre><code>   out.push('&lt;thead&gt;&lt;tr class=\"' + Ext.baseCSSPrefix + 'grid-header-row\"&gt;');\n   for (i = 0; i &lt; len; i++) {\n       column = columns[i];\n       width = column.lastBox ? column.lastBox.width : Ext.grid.header.Container.prototype.defaultWidth;\n       out.push('&lt;th class=\"', Ext.baseCSSPrefix, 'grid-cell-', columns[i].getItemId(), '\" style=\"width:' + width + 'px\"&gt;&lt;/th&gt;');\n   }\n   out.push('&lt;/tr&gt;&lt;/thead&gt;');\n</code></pre>\n\n<p>   },</p>\n"},"renderRow":{"!type":"fn(record: ?, out?: ?) -> string","!doc":"<p>Renders the HTML markup string for a single row into the passed array as a sequence of strings, or\nreturns the HTML markup for a single row.</p>\n"},"renderRows":{"!type":"fn(rows: ?, columns: ?, viewStartIndex: ?, out: ?) -> !this"},"renderTFoot":{"!type":"fn(values: ?, out: ?, parent: ?) -> !this"},"renderTHead":{"!type":"fn(values: ?, out: ?, parent: ?) -> !this"},"retrieveNode":{"!type":"fn(id: ?, dataRow: ?) -> !this"},"scrollCellIntoView":{"!type":"fn(cell: ?, animate: ?) -> !this"},"scrollElIntoView":{"!type":"fn(el: ?, hscroll: ?, animate: ?) -> !this","!doc":"<p>hook for rtl override</p>\n"},"scrollRowIntoView":{"!type":"fn(row: ?, animate: ?) -> !this"},"scrollToTop":{"!type":"fn() -> !this","!doc":"<p>scroll the view to the top</p>\n"},"setPendingStripe":{"!type":"fn(index: ?) -> !this"},"shouldUpdateCell":{"!type":"fn(record: ?, column: ?, changedFieldNames: ?) -> number","!doc":"<p>Decides whether the column needs updating</p>\n"},"syncRowHeights":{"!type":"fn(firstItem: ?, secondItem: ?) -> !this"},"tplApplyOut":{"!type":"fn(values: ?, out: ?, parent: ?) -> !this"},"updateColumns":{"!type":"fn(oldRowDom: ?, newRowDom: ?, columnsToUpdate: ?) -> !this"},"updateIndexes":{"!type":"fn() -> !this","!doc":"<p>Links back from grid rows are installed by the XTemplate as data attributes</p>\n"},"walkCells":{"!type":"fn(position: ?, direction: ?, e: ?, preventWrap: ?, verifierFn: ?, scope: ?) -> ?"},"walkRecs":{"!type":"fn(startRec: ?, distance: ?) -> !this","!doc":"<p>Navigates from the passed record by the passed increment which may be +ve or -ve</p>\n\n<p>Skips hidden records.</p>\n\n<p>If no record is visible in the specified direction, returns the starting record index unchanged.</p>\n"},"walkRows":{"!type":"fn(startRow: ?, distance: ?) -> !this","!doc":"<p>Increments the passed row index by the passed increment which may be +ve or -ve</p>\n\n<p>Skips hidden rows.</p>\n\n<p>If no row is visible in the specified direction, returns the input row index unchanged.</p>\n"},"beforecellclick":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell click is processed. Return false to cancel the default action.</p>\n"},"beforecellcontextmenu":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell right click is processed. Return false to cancel the default action.</p>\n"},"beforecelldblclick":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell double click is processed. Return false to cancel the default action.</p>\n"},"beforecellkeydown":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell key down is processed. Return false to cancel the default action.</p>\n"},"beforecellmousedown":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell mouse down is processed. Return false to cancel the default action.</p>\n"},"beforecellmouseup":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before the cell mouse up is processed. Return false to cancel the default action.</p>\n"},"cellclick":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is clicked.</p>\n"},"cellcontextmenu":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is right clicked.</p>\n"},"celldblclick":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is double clicked.</p>\n"},"cellkeydown":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the keydown event is captured on the cell.</p>\n"},"cellmousedown":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the mousedown event is captured on the cell.</p>\n"},"cellmouseup":{"!type":"fn(this: ?, td: ?, cellIndex: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the mouseup event is captured on the cell.</p>\n"},"rowclick":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is clicked.</p>\n"},"rowcontextmenu":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is right clicked.</p>\n"},"rowdblclick":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when table cell is double clicked.</p>\n"},"rowkeydown":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the keydown event is captured on the cell.</p>\n"},"rowmousedown":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the mousedown event is captured on the cell.</p>\n"},"rowmouseup":{"!type":"fn(this: ?, record: ?, tr: ?, rowIndex: ?, e: ?, eOpts: ?)","!doc":"<p>Fired when the mouseup event is captured on the cell.</p>\n"},"getBoundView":{"!type":"fn(node: ?) -> !this"}},"!doc":"<p>This class encapsulates the user interface for a tabular data set.\nIt acts as a centralized manager for controlling the various interface\nelements of the view. This includes handling events, such as row and cell\nlevel based DOM events. It also reacts to events from the underlying <a href=\"#!/api/Ext.selection.Model\" rel=\"Ext.selection.Model\" class=\"docClass\">Ext.selection.Model</a>\nto provide visual feedback to the user.</p>\n\n<p>This class does not provide ways to manipulate the underlying data of the configured\nExt.data.Store.</p>\n\n<p>This is the base class for both <a href=\"#!/api/Ext.grid.View\" rel=\"Ext.grid.View\" class=\"docClass\">Ext.grid.View</a> and <a href=\"#!/api/Ext.tree.View\" rel=\"Ext.tree.View\" class=\"docClass\">Ext.tree.View</a> and is not\nto be used directly.</p>\n","!type":"fn(config: ?)"},"AbstractView":{"prototype":{"blockRefresh":{"!type":"bool","!doc":"<p>Set this to true to ignore refresh events on the bound store. This is useful if\nyou wish to provide custom transition animations via a plugin</p>\n"},"deferEmptyText":{"!type":"bool","!doc":"<p>True to defer emptyText being applied until the store's first load.</p>\n"},"deferInitialRefresh":{"!type":"bool","!doc":"<p>Configure as 'true` to defer the initial refresh of the view.</p>\n\n<p>This allows the View to execute its render and initial layout more quickly because the process will not be encumbered\nby the update of the view structure.</p>\n"},"disableSelection":{"!type":"bool","!doc":"<p>True to disable selection within the DataView. This configuration will lock the selection model\nthat the DataView uses.</p>\n"},"emptyText":{"!type":"string","!doc":"<p>The text to display in the view when there is no data to display.\nNote that when using local data the emptyText will not be displayed unless you set\nthe <a href=\"#!/api/Ext.view.AbstractView-cfg-deferEmptyText\" rel=\"Ext.view.AbstractView-cfg-deferEmptyText\" class=\"docClass\">deferEmptyText</a> option to false.</p>\n"},"itemCls":{"!type":"string","!doc":"<p>Specifies the class to be assigned to each element in the view when used in conjunction with the\n<a href=\"#!/api/Ext.view.AbstractView-cfg-itemTpl\" rel=\"Ext.view.AbstractView-cfg-itemTpl\" class=\"docClass\">itemTpl</a> configuration.</p>\n"},"itemTpl":{"!doc":"<p>The inner portion of the item template to be rendered. Follows an XTemplate\nstructure and will be placed inside of a tpl.</p>\n"},"loadMask":{"!doc":"<p>False to disable a load mask from displaying while the view is loading. This can also be a\n<a href=\"#!/api/Ext.LoadMask\" rel=\"Ext.LoadMask\" class=\"docClass\">Ext.LoadMask</a> configuration object.</p>\n"},"loadingCls":{"!type":"string","!doc":"<p>The CSS class to apply to the loading message element. Defaults to Ext.LoadMask.prototype.msgCls \"x-mask-loading\".</p>\n"},"loadingHeight":{"!type":"number","!doc":"<p>If specified, gives an explicit height for the data view when it is showing the <a href=\"#!/api/Ext.view.AbstractView-cfg-loadingText\" rel=\"Ext.view.AbstractView-cfg-loadingText\" class=\"docClass\">loadingText</a>,\nif that is specified. This is useful to prevent the view's height from collapsing to zero when the\nloading mask is applied and there are no other contents in the data view.</p>\n"},"loadingText":{"!type":"string","!doc":"<p>A string to display during data load operations.  If specified, this text will be\ndisplayed in a loading div and the view's contents will be cleared while loading, otherwise the view's\ncontents will continue to display normally until the new data is loaded and the contents are replaced.</p>\n"},"loadingUseMsg":{"!type":"bool","!doc":"<p>Whether or not to use the loading message.</p>\n"},"multiSelect":{"!type":"bool","!doc":"<p>True to allow selection of more than one item at a time, false to allow selection of only a single item\nat a time or no selection at all, depending on the value of <a href=\"#!/api/Ext.view.AbstractView-cfg-singleSelect\" rel=\"Ext.view.AbstractView-cfg-singleSelect\" class=\"docClass\">singleSelect</a>.</p>\n"},"preserveScrollOnRefresh":{"!type":"bool","!doc":"<p>True to preserve scroll position across refresh operations.</p>\n"},"simpleSelect":{"!type":"bool","!doc":"<p>True to enable multiselection by clicking on multiple items without requiring the user to hold Shift or Ctrl,\nfalse to force the user to hold Ctrl or Shift to select more than on item.</p>\n"},"singleSelect":{"!type":"bool","!doc":"<p>Allows selection of exactly one item at a time. As this is the default selection mode anyway, this config\nis completely ignored.</p>\n"},"store":{"!doc":"<p>The Ext.data.Store to bind this DataView to.</p>\n"},"throttledUpdate":{"!type":"bool","!doc":"<p>Configure as <code>true</code> to have this view participate in the global throttled update queue which flushes store changes to the UI at a maximum rate\ndetermined by the <a href=\"#!/api/Ext.view.AbstractView-static-cfg-updateDelay\" rel=\"Ext.view.AbstractView-static-cfg-updateDelay\" class=\"docClass\">updateDelay</a> setting.</p>\n"},"tpl":{"!doc":"<p>The HTML fragment or an array of fragments that will make up the template used by this DataView.  This should\nbe specified in the same format expected by the constructor of Ext.XTemplate.</p>\n"},"ariaRole":{"!type":"string"},"defaultBindProperty":{"!type":"string"},"itemAriaRole":{"!type":"string"},"last":{"!type":"bool","!doc":"<p>private</p>\n"},"refreshNeeded":{"!type":"bool","!doc":"<p>Starts as true by default so that pn the leading edge of the first layout a refresh will be triggered.\nA refresh opereration sets this flag to false.\nWhen a refresh is requested using refreshView, the request may be deferred because of hidden or collapsed state.\nThis is done by setting the refreshNeeded flag to true, and the the next layout will trigger  refresh.</p>\n"},"triggerCtEvent":{"!type":"string"},"triggerEvent":{"!type":"string"},"addCmpEvents":{"!type":"fn() -> !this"},"addItemCls":{"!type":"fn(itemInfo: ?, cls: ?) -> !this","!doc":"<p>Adds a CSS Class to a specific item.</p>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"appendNodes":{"!type":"fn(nodes: ?) -> !this"},"clearEmptyEl":{"!type":"fn() -> !this"},"clearSelections":{"!type":"fn() -> !this","!doc":"<p>Deselects all selected records.</p>\n"},"deselect":{"!type":"fn(records: ?, suppressEvent: ?) -> !this","!doc":"<p>Deselects a record instance by record instance or index.</p>\n"},"doAdd":{"!type":"fn(records: ?, index: ?) -> !this"},"doFirstRefresh":{"!type":"fn(store: ?, noDefer: ?) -> !this","!doc":"<p>Perform the first refresh of the View from a newly bound store.</p>\n\n<p>This is called when this View has been sized for the first time.</p>\n"},"doRemove":{"!type":"fn(record: ?, index: ?) -> !this","!doc":"<p>private</p>\n"},"findItemByChild":{"!type":"fn(node: ?) -> ?","!doc":"<p>Returns the template node the passed child belongs to, or null if it doesn't belong to one.</p>\n"},"findTargetByEvent":{"!type":"fn(e: ?) -> !this","!doc":"<p>Returns the template node by the Ext.event.Event or null if it is not found.</p>\n"},"getMaskStore":{"!type":"fn() -> !this"},"getNode":{"!type":"fn(nodeInfo: ?) -> ?","!doc":"<p>Gets a template node.</p>\n"},"getNodeContainer":{"!type":"fn() -> !this"},"getNodeContainerSelector":{"!type":"fn() -> !this","!doc":"<p>Returns a CSS selector which selects the element which contains record nodes.</p>\n"},"getNodes":{"!type":"fn(start?: ?, end?: ?) -> ?","!doc":"<p>Gets a range nodes.</p>\n"},"getRecords":{"!type":"fn(nodes: ?) -> ?","!doc":"<p>Gets an array of the records from an array of nodes</p>\n"},"getRefItems":{"!type":"fn() -> !this"},"getSelectedNodes":{"!type":"fn() -> ?","!doc":"<p>Gets the currently selected nodes.</p>\n"},"getSelectedRecords":{"!type":"fn() -> ?","!doc":"<p>Gets an array of the selected records</p>\n"},"getSelectionCount":{"!type":"fn() -> number","!doc":"<p>Gets the number of selected nodes.</p>\n"},"getSelectionModel":{"!type":"fn() -> ?","!doc":"<p>Gets the selection model for this view.</p>\n"},"getStore":{"!type":"fn() -> ?","!doc":"<p>Returns the store associated with this DataView.</p>\n"},"getStoreListeners":{"!type":"fn() -> ?","!doc":"<p>Gets the listeners to bind to a new store.</p>\n"},"isSelected":{"!type":"fn(node: ?) -> bool","!doc":"<p>Returns true if the passed node is selected, else false.</p>\n"},"onBeginUpdate":{"!type":"fn() -> !this"},"onBindStore":{"!type":"fn(store: ?, initial: ?, propName: ?) -> !this","!doc":"<p>Template method, it is called when a new store is bound\nto the current instance.</p>\n"},"onItemDeselect":{"!type":"fn(record: ?) -> !this","!doc":"<p>invoked by the selection model to maintain visual UI cues</p>\n"},"onItemSelect":{"!type":"fn(record: ?) -> !this","!doc":"<p>invoked by the selection model to maintain visual UI cues</p>\n"},"onMaskBeforeShow":{"!type":"fn() -> !this"},"onMaskHide":{"!type":"fn() -> !this"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"onResize":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the resize operation.</p>\n\n<p>Called when Ext.resizer.Resizer#drag event is fired.</p>\n"},"onUnbindStore":{"!type":"fn(store: ?) -> !this","!doc":"<p>Template method, it is called when an existing store is unbound\nfrom the current instance.</p>\n"},"prepareData":{"!type":"fn(data: ?, recordIndex: ?, record: ?) -> ?","!doc":"<p>Function which can be overridden to provide custom formatting for each Record that is used by this\nDataView's <a href=\"#!/api/Ext.view.AbstractView-cfg-tpl\" rel=\"Ext.view.AbstractView-cfg-tpl\" class=\"docClass\">template</a> to render each node.</p>\n"},"refreshNode":{"!type":"fn(record: ?) -> !this","!doc":"<p>Refreshes an individual node's data from the store.</p>\n"},"refreshView":{"!type":"fn() -> !this"},"removeItemCls":{"!type":"fn(itemInfo: ?, cls: ?) -> !this","!doc":"<p>Removes a CSS Class from a specific item.</p>\n"},"restoreScrollState":{"!type":"fn() -> !this","!doc":"<p>Restores the scrollState.\nMust be used in conjunction with saveScrollState</p>\n"},"saveScrollState":{"!type":"fn() -> !this","!doc":"<p>Saves the scrollState in a private variable. Must be used in conjunction with restoreScrollState.</p>\n"},"select":{"!type":"fn(records: ?, keepExisting: ?, suppressEvent: ?) -> !this","!doc":"<p>Selects a record instance by record instance or index.</p>\n"},"setMaskBind":{"!type":"fn(store: ?) -> !this"},"getRecord":{"!type":"fn(node: ?) -> !this"},"beforerefresh":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires before the view is refreshed</p>\n"},"itemadd":{"!type":"fn(records: ?, index: ?, node: ?, eOpts: ?)","!doc":"<p>Fires when the nodes associated with an recordset have been added to the underlying store</p>\n"},"itemremove":{"!type":"fn(record: ?, index: ?, item: ?, view: ?, eOpts: ?)","!doc":"<p>Fires when the node associated with an individual record is removed</p>\n"},"itemupdate":{"!type":"fn(record: ?, index: ?, node: ?, eOpts: ?)","!doc":"<p>Fires when the node associated with an individual record is updated</p>\n"},"refresh":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when the view is refreshed</p>\n"},"viewready":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when the View's item elements representing Store items has been rendered. No items will be available\nfor selection until this event fires.</p>\n"},"itemSelector":{"!type":"string","!doc":"<p><b>This is a required setting</b>. A simple CSS selector (e.g. <code>div.some-class</code> or\n<code>span:first-child</code>) that will be used to determine what nodes this DataView will be\nworking with. The itemSelector is used to map DOM nodes to records. As such, there should\nonly be one root level element that matches the selector for each record. The itemSelector\nwill be automatically configured if the <a href=\"#!/api/Ext.view.AbstractView-cfg-itemTpl\" rel=\"Ext.view.AbstractView-cfg-itemTpl\" class=\"docClass\">itemTpl</a> config is used.</p>\n\n<pre><code>new <a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a>({\n    renderTo: Ext.getBody(),\n    store: {\n        fields: ['name'],\n        data: [\n            {name: 'Item 1'},\n            {name: 'Item 2'}\n        ]\n    },\n    tpl: [\n        '&lt;ul&gt;',\n        '&lt;tpl for=\".\"&gt;',\n            '&lt;li&gt;{name}&lt;/li&gt;',\n        '&lt;/tpl&gt;',\n        '&lt;/ul&gt;'\n    ],\n    // Match the li, since each one maps to a record\n    itemSelector: 'li'\n});\n</code></pre>\n"},"overItemCls":{"!type":"string","!doc":"<p>A CSS class to apply to each item in the view on mouseover.\nSetting this will automatically set <a href=\"#!/api/Ext.view.AbstractView-cfg-trackOver\" rel=\"Ext.view.AbstractView-cfg-trackOver\" class=\"docClass\">trackOver</a> to <code>true</code>.</p>\n"},"selectedItemCls":{"!type":"string","!doc":"<p>A CSS class to apply to each selected item in the view.</p>\n"},"nodeContainerSelector":{"!doc":"<p>Element which contains rows</p>\n"},"beforeLayout":{"!type":"fn() -> !this","!doc":"<p>Occurs before componentLayout is run. In previous releases, this method could\nreturn <code>false</code> to prevent its layout but that is not supported in Ext JS 4.1 or\nhigher. This method is simply a notification of the impending layout to give the\ncomponent a chance to adjust the DOM. Ideally, DOM reads should be avoided at this\ntime to reduce expensive document reflows.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"bufferRender":{"!type":"fn(records: ?, index: ?) -> !this","!doc":"<p>private</p>\n"},"clearViewEl":{"!type":"fn() -> !this"},"collectData":{"!type":"fn(records: ?, startIndex: ?) -> ?","!doc":"<p>Function which can be overridden which returns the data object passed to this\nDataView's <a href=\"#!/api/Ext.view.AbstractView-cfg-tpl\" rel=\"Ext.view.AbstractView-cfg-tpl\" class=\"docClass\">template</a> to render the whole DataView.</p>\n\n<p>This is usually an Array of data objects, each element of which is processed by an\nXTemplate which uses <code>'&amp;lt;tpl for=\".\"&amp;gt;'</code> to iterate over its supplied\ndata object as an Array. However, <i>named</i> properties may be placed into the data object to\nprovide non-repeating data such as headings, totals etc.</p>\n"},"collectNodes":{"!type":"fn(targetEl: ?) -> !this","!doc":"<p>Private\nCalled by refresh to collect the view item nodes.</p>\n"},"getItemSelector":{"!type":"fn() -> !this"},"getNodeByRecord":{"!type":"fn(record: ?) -> !this"},"getViewRange":{"!type":"fn() -> !this"},"indexOf":{"!type":"fn(nodeInfo: ?) -> number","!doc":"<p>Finds the index of the passed node.</p>\n"},"onAdd":{"!type":"fn(store: ?, records: ?, index: ?) -> !this","!doc":"<p>private</p>\n"},"onDataRefresh":{"!type":"fn() -> !this","!doc":"<p>Calls this.refresh if this.blockRefresh is not true</p>\n"},"onEndUpdate":{"!type":"fn() -> !this"},"onRemove":{"!type":"fn(ds: ?, records: ?, index: ?) -> !this","!doc":"<p>private</p>\n"},"onReplace":{"!type":"fn(store: ?, startIndex: ?, oldRecords: ?, newRecords: ?) -> !this","!doc":"<p>Private.\nRespond to store replace event which is fired by GroupStore group expand/collapse operations.\nThis saves a layout because a remove and add operation are coalesced in this operation.</p>\n"},"onUpdate":{"!type":"fn(store: ?, record: ?, operation: ?, modifiedFieldNames: ?) -> !this","!doc":"<p>private</p>\n"},"onViewScroll":{"!type":"fn() -> !this","!doc":"<p>This is called on animation frame</p>\n"},"refreshSize":{"!type":"fn(forceLayout: ?) -> !this","!doc":"<p>Called by the framework when the view is refreshed, or when rows are added or deleted.</p>\n\n<p>These operations may cause the view's dimensions to change, and if the owning container\nis shrinkwrapping this view, then the layout must be updated to accommodate these new dimensions.</p>\n"},"updateIndexes":{"!type":"fn(startIndex: ?, endIndex: ?) -> !this","!doc":"<p>private</p>\n"},"getBoundView":{"!type":"fn(node: ?) -> !this"},"trackOver":{"!type":"bool","!doc":"<p>When <code>true</code> the <a href=\"#!/api/Ext.view.AbstractView-cfg-overItemCls\" rel=\"Ext.view.AbstractView-cfg-overItemCls\" class=\"docClass\">overItemCls</a> will be applied to rows when hovered over.\nThis in return will also cause <a href=\"#!/api/Ext.view.View-event-highlightitem\" rel=\"Ext.view.View-event-highlightitem\" class=\"docClass\">highlightitem</a> and\n<a href=\"#!/api/Ext.view.View-event-unhighlightitem\" rel=\"Ext.view.View-event-unhighlightitem\" class=\"docClass\">unhighlightitem</a> events to be fired.</p>\n\n<p>Enabled automatically when the <a href=\"#!/api/Ext.view.AbstractView-cfg-overItemCls\" rel=\"Ext.view.AbstractView-cfg-overItemCls\" class=\"docClass\">overItemCls</a> config is set.</p>\n"},"updateDelay":{"!type":"number","!doc":"<p>Global config for use when using <a href=\"#!/api/Ext.view.AbstractView-cfg-throttledUpdate\" rel=\"Ext.view.AbstractView-cfg-throttledUpdate\" class=\"docClass\">throttled view updating</a> if the data in the backing store\nis being changed rapidly, for example receiving changes from the server through a WebSocket connection.</p>\n\n<p>To avoid too-frequent view updates overloading the browser with style recalculation, layout and paint requests, updates can be <a href=\"#!/api/Ext.view.AbstractView-cfg-throttledUpdate\" rel=\"Ext.view.AbstractView-cfg-throttledUpdate\" class=\"docClass\">throttled</a> to\ncoalesced, and applied at the interval specified in milliseconds.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"bindStore":{"!type":"fn(store: ?) -> !this","!doc":"<p>Changes the data store bound to this view and refreshes it.</p>\n"},"handleUpdate":{"!type":"fn(store: ?, record: ?) -> !this","!doc":"<p>private</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"flushChangeQueue":{"!type":"fn() -> !this","!doc":"<p>Flushes all queued field updates to the UI.</p>\n\n<p>Called in the context of the AbstractView class.</p>\n\n<p>The queue is shared across all Views so that there is only one global flush operation.</p>\n"},"onFlushTick":{"!type":"fn() -> !this","!doc":"<p>On every flush (determined by updateDelay setting), ask the animation system to schedule a call to flushChangeQueue at the next animation frame.</p>\n"},"queueRecordChange":{"!type":"fn(view: ?, store: ?, record: ?, operation: ?, modifiedFieldNames: ?) -> !this"}},"!doc":"<p>This is an abstract superclass and should not be used directly. Please see <a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a>.</p>\n","!type":"fn(config: ?)"},"View":{"prototype":{"inputTagRe":{},"EventMap":{},"TouchEventMap":{},"bindStore":{"!type":"fn(store: ?, initial: ?, propertyName: ?) -> !this","!doc":"<p>Changes the data store bound to this view and refreshes it.</p>\n"},"clearHighlight":{"!type":"fn() -> !this","!doc":"<p>Un-highlights the currently highlighted item, if any.</p>\n"},"focusNode":{"!type":"fn(rec: ?) -> !this","!doc":"<p>Focuses a node in the view.</p>\n"},"getTargetSelector":{"!type":"fn() -> !this","!doc":"<p>Can be overridden by features or anything that needs to use a specific selector as a target.</p>\n"},"handleEvent":{"!type":"fn(e: ?) -> !this"},"handleMouseOut":{"!type":"fn(e: ?) -> !this"},"handleMouseOver":{"!type":"fn(e: ?) -> !this"},"highlightItem":{"!type":"fn(item: ?) -> !this","!doc":"<p>Highlights a given item in the View. This is called by the mouseover handler if <a href=\"#!/api/Ext.view.View-cfg-overItemCls\" rel=\"Ext.view.View-cfg-overItemCls\" class=\"docClass\">overItemCls</a>\nand <a href=\"#!/api/Ext.view.View-cfg-trackOver\" rel=\"Ext.view.View-cfg-trackOver\" class=\"docClass\">trackOver</a> are configured, but can also be called manually by other code, for instance to\nhandle stepping through the list via keyboard navigation.</p>\n"},"onBeforeContainerClick":{"!type":"fn() -> !this"},"onBeforeContainerContextMenu":{"!type":"fn() -> !this"},"onBeforeContainerDblClick":{"!type":"fn() -> !this"},"onBeforeContainerKeyDown":{"!type":"fn() -> !this"},"onBeforeContainerLongPress":{"!type":"fn() -> !this"},"onBeforeContainerMouseDown":{"!type":"fn() -> !this"},"onBeforeContainerMouseOut":{"!type":"fn() -> !this"},"onBeforeContainerMouseOver":{"!type":"fn() -> !this"},"onBeforeContainerMouseUp":{"!type":"fn() -> !this"},"onBeforeItemClick":{"!type":"fn() -> !this"},"onBeforeItemContextMenu":{"!type":"fn() -> !this"},"onBeforeItemDblClick":{"!type":"fn() -> !this"},"onBeforeItemFocus":{"!type":"fn() -> !this"},"onBeforeItemKeyDown":{"!type":"fn() -> !this"},"onBeforeItemLongPress":{"!type":"fn() -> !this"},"onBeforeItemMouseDown":{"!type":"fn() -> !this"},"onBeforeItemMouseEnter":{"!type":"fn() -> !this"},"onBeforeItemMouseLeave":{"!type":"fn() -> !this"},"onBeforeItemMouseUp":{"!type":"fn() -> !this"},"onContainerClick":{"!type":"fn() -> !this"},"onContainerContextMenu":{"!type":"fn() -> !this"},"onContainerDblClick":{"!type":"fn() -> !this"},"onContainerKeyDown":{"!type":"fn() -> !this"},"onContainerLongPress":{"!type":"fn() -> !this"},"onContainerMouseDown":{"!type":"fn() -> !this","!doc":"<p>private, template methods</p>\n"},"onContainerMouseOut":{"!type":"fn() -> !this"},"onContainerMouseOver":{"!type":"fn() -> !this"},"onContainerMouseUp":{"!type":"fn() -> !this"},"onItemClick":{"!type":"fn() -> !this"},"onItemContextMenu":{"!type":"fn() -> !this"},"onItemDblClick":{"!type":"fn() -> !this"},"onItemFocus":{"!type":"fn() -> !this"},"onItemKeyDown":{"!type":"fn() -> !this"},"onItemLongPress":{"!type":"fn() -> !this"},"onItemMouseDown":{"!type":"fn() -> !this","!doc":"<p>private, template methods</p>\n"},"onItemMouseEnter":{"!type":"fn(record: ?, item: ?, index: ?, e: ?) -> !this"},"onItemMouseLeave":{"!type":"fn(record: ?, item: ?, index: ?, e: ?) -> !this"},"onItemMouseUp":{"!type":"fn() -> !this"},"processContainerEvent":{"!type":"fn() -> !this"},"processUIEvent":{"!type":"fn(e: ?) -> !this"},"setHighlightedItem":{"!type":"fn(item: ?) -> !this"},"beforecontainerclick":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the click event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainercontextmenu":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the contextmenu event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainerdblclick":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the dblclick event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainerkeydown":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the keydown event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainermousedown":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mousedown event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainermouseout":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseout event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainermouseover":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseover event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforecontainermouseup":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseup event on the container is processed. Returns false to cancel the default action.</p>\n"},"beforedeselect":{"!type":"fn(this: ?, record: ?, eOpts: ?)","!doc":"<p>Fired before a record is deselected. If any listener returns false, the\ndeselection is cancelled.</p>\n"},"beforeitemclick":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the click event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemcontextmenu":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the contextmenu event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemdblclick":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the dblclick event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemkeydown":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the keydown event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemmousedown":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mousedown event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemmouseenter":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseenter event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemmouseleave":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseleave event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeitemmouseup":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires before the mouseup event on an item is processed. Returns false to cancel the default action.</p>\n"},"beforeselect":{"!type":"fn(this: ?, record: ?, eOpts: ?)","!doc":"<p>Fired before a record is selected. If any listener returns false, the\nselection is cancelled.</p>\n"},"containerclick":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the container is clicked.</p>\n"},"containercontextmenu":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the container is right clicked.</p>\n"},"containerdblclick":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the container is double clicked.</p>\n"},"containerkeydown":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when a key is pressed while the container is focused, and no item is currently selected.</p>\n"},"containermousedown":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when there is a mousedown on the container</p>\n"},"containermouseout":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when you move the mouse out of the container.</p>\n"},"containermouseover":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when you move the mouse over the container.</p>\n"},"containermouseup":{"!type":"fn(this: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when there is a mouseup on the container</p>\n"},"deselect":{"!type":"fn(this: ?, record: ?, eOpts: ?)","!doc":"<p>Fired after a record is deselected</p>\n"},"focuschange":{"!type":"fn(this: ?, oldFocused: ?, newFocused: ?, eOpts: ?)","!doc":"<p>Fired when a row is focused</p>\n"},"highlightitem":{"!type":"fn(view: ?, node: ?, eOpts: ?)","!doc":"<p>Fires when a node is highlighted using keyboard navigation, or mouseover.</p>\n"},"itemclick":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when an item is clicked.</p>\n"},"itemcontextmenu":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when an item is right clicked.</p>\n"},"itemdblclick":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when an item is double clicked.</p>\n"},"itemkeydown":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when a key is pressed while an item is currently selected.</p>\n"},"itemmousedown":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when there is a mouse down on an item</p>\n"},"itemmouseenter":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse enters an item.</p>\n"},"itemmouseleave":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse leaves an item.</p>\n"},"itemmouseup":{"!type":"fn(this: ?, record: ?, item: ?, index: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when there is a mouse up on an item</p>\n"},"select":{"!type":"fn(this: ?, record: ?, eOpts: ?)","!doc":"<p>Fired after a record is selected</p>\n"},"selectionchange":{"!type":"fn(this: ?, selected: ?, eOpts: ?)","!doc":"<p>Fired after a selection change has occurred</p>\n"},"unhighlightitem":{"!type":"fn(view: ?, node: ?, eOpts: ?)","!doc":"<p>Fires when a node is unhighlighted using keyboard navigation, or mouseout.</p>\n"},"handleUpdate":{"!type":"fn(store: ?, record: ?) -> !this","!doc":"<p>private</p>\n"},"processItemEvent":{"!type":"fn() -> !this","!doc":"<p>Private template method</p>\n"},"processSpecialEvent":{"!type":"fn() -> !this"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"refresh":{"!type":"fn() -> !this","!doc":"<p>Refreshes the view by reloading the data from the store and re-rendering the template.</p>\n"}},"!doc":"<p>A mechanism for displaying data using custom layout templates and formatting.</p>\n\n<p>The View uses an Ext.XTemplate as its internal templating mechanism, and is bound to an\nExt.data.Store so that as the data in the store changes the view is automatically updated\nto reflect the changes. The view also provides built-in behavior for many common events that can\noccur for its contained items including click, doubleclick, mouseover, mouseout, etc. as well as a\nbuilt-in selection model. <strong>In order to use these features, an <a href=\"#!/api/Ext.view.View-cfg-itemSelector\" rel=\"Ext.view.View-cfg-itemSelector\" class=\"docClass\">itemSelector</a> config must\nbe provided for the View to determine what nodes it will be working with.</strong></p>\n\n<p>The example below binds a View to a Ext.data.Store and renders it into an <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>.</p>\n\n<pre class='inline-example '><code>Ext.define('Image', {\n    extend: 'Ext.data.Model',\n    fields: [\n        { name:'src', type:'string' },\n        { name:'caption', type:'string' }\n    ]\n});\n\nExt.create('Ext.data.Store', {\n    id:'imagesStore',\n    model: 'Image',\n    data: [\n        { src:'http://www.sencha.com/img/20110215-feat-drawing.png', caption:'Drawing &amp; Charts' },\n        { src:'http://www.sencha.com/img/20110215-feat-data.png', caption:'Advanced Data' },\n        { src:'http://www.sencha.com/img/20110215-feat-html5.png', caption:'Overhauled Theme' },\n        { src:'http://www.sencha.com/img/20110215-feat-perf.png', caption:'Performance Tuned' }\n    ]\n});\n\nvar imageTpl = new Ext.XTemplate(\n    '&lt;tpl for=\".\"&gt;',\n        '&lt;div style=\"margin-bottom: 10px;\" class=\"thumb-wrap\"&gt;',\n          '&lt;img src=\"{src}\" /&gt;',\n          '&lt;br/&gt;&lt;span&gt;{caption}&lt;/span&gt;',\n        '&lt;/div&gt;',\n    '&lt;/tpl&gt;'\n);\n\nExt.create('<a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a>', {\n    store: Ext.data.StoreManager.lookup('imagesStore'),\n    tpl: imageTpl,\n    itemSelector: 'div.thumb-wrap',\n    emptyText: 'No images available',\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)"},"DropZone":{"prototype":{"containsRecordAtOffset":{"!type":"fn(records: ?, record: ?, offset: ?) -> bool","!doc":"<p>Determines whether the record at the specified offset from the passed record\nis in the drag payload.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroy this DragDrop instance</p>\n"},"fireViewEvent":{"!type":"fn() -> !this","!doc":"<p>Fire an event through the client DataView. Lock this DropZone during the event processing so that\nits data does not become corrupted by processing mouse events.</p>\n"},"getIndicator":{"!type":"fn() -> !this"},"getPosition":{"!type":"fn(e: ?, node: ?) -> !this"},"getTargetFromEvent":{"!type":"fn(e: ?) -> ?","!doc":"<p>Returns a custom data object associated with the DOM node that is the target of the event.  By default\nthis looks up the event target in the <a href=\"#!/api/Ext.dd.Registry\" rel=\"Ext.dd.Registry\" class=\"docClass\">Ext.dd.Registry</a>, although you can override this method to\nprovide your own custom lookup.</p>\n"},"invalidateDrop":{"!type":"fn() -> !this"},"notifyOut":{"!type":"fn(node: ?, dragZone: ?, e: ?, data: ?) -> !this","!doc":"<p>Moved out of the DropZone without dropping.\nRemove drop position indicator</p>\n"},"onContainerDrop":{"!type":"fn(dd: ?, e: ?, data: ?) -> bool","!doc":"<p>Called when the DropZone determines that a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> has been dropped on it,\nbut not on any of its registered drop nodes.  The default implementation returns false, so it should be\noverridden to provide the appropriate processing of the drop event if you need the drop zone itself to\nbe able to accept drops.  It should return true when valid so that the drag source's repair action does not run.</p>\n"},"onContainerOver":{"!type":"fn(dd: ?, e: ?, data: ?) -> string","!doc":"<p>The mouse is past the end of all nodes (or there are no nodes)</p>\n"},"onNodeDrop":{"!type":"fn(targetNode: ?, dragZone: ?, e: ?, data: ?) -> bool","!doc":"<p>Called when the DropZone determines that a <a href=\"#!/api/Ext.dd.DragSource\" rel=\"Ext.dd.DragSource\" class=\"docClass\">Ext.dd.DragSource</a> has been dropped onto\nthe drop node.  The default implementation returns false, so it should be overridden to provide the\nappropriate processing of the drop event and return true so that the drag source's repair action does not run.</p>\n"},"onNodeOver":{"!type":"fn(node: ?, dragZone: ?, e: ?, data: ?) -> string","!doc":"<p>The mouse is over a View node</p>\n"},"positionIndicator":{"!type":"fn(node: ?, data: ?, e: ?) -> !this"},"indicatorHtml":{},"indicatorCls":{"!type":"string"}},"!type":"fn(config: ?)"},"BoundList":{"prototype":{"autoScroll":{"!type":"bool","!doc":"<p><code>true</code> to use overflow:'auto' on the components layout element and show scroll bars automatically when necessary,\n<code>false</code> to clip any overflowing content.\nThis should not be combined with <a href=\"#!/api/Ext.view.BoundList-cfg-overflowX\" rel=\"Ext.view.BoundList-cfg-overflowX\" class=\"docClass\">overflowX</a> or  <a href=\"#!/api/Ext.view.BoundList-cfg-overflowY\" rel=\"Ext.view.BoundList-cfg-overflowY\" class=\"docClass\">overflowY</a>.</p>\n"},"baseCls":{"!type":"string","!doc":"<p>private overrides</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"componentLayout":{"!doc":"<p>The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout\nmanager which sizes a Component's internal structure in response to the Component being sized.</p>\n\n<p>Generally, developers will not use this configuration as all provided Components which need their internal\nelements sizing (Such as <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">input fields</a>) come with their own componentLayout managers.</p>\n\n<p>The <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used on instances of the base <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>\nclass which simply sizes the Component's encapsulating element to the height and width specified in the\n<a href=\"#!/api/Ext.view.BoundList-method-setSize\" rel=\"Ext.view.BoundList-method-setSize\" class=\"docClass\">setSize</a> method.</p>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"displayField":{"!type":"string","!doc":"<p>The field from the store to show in the view.</p>\n"},"itemCls":{"!type":"string","!doc":"<p>Specifies the class to be assigned to each element in the view when used in conjunction with the\n<a href=\"#!/api/Ext.view.BoundList-cfg-itemTpl\" rel=\"Ext.view.BoundList-cfg-itemTpl\" class=\"docClass\">itemTpl</a> configuration.</p>\n"},"pageSize":{"!type":"number","!doc":"<p>If greater than <code>0</code>, a <a href=\"#!/api/Ext.toolbar.Paging\" rel=\"Ext.toolbar.Paging\" class=\"docClass\">Ext.toolbar.Paging</a> is displayed at the bottom of the list and store\nqueries will execute with page start and\nlimit parameters.</p>\n"},"preserveScrollOnRefresh":{"!type":"bool","!doc":"<p>True to preserve scroll position across refresh operations.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.view.BoundList-method-getEl\" rel=\"Ext.view.BoundList-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.view.BoundList-method-getEl\" rel=\"Ext.view.BoundList-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.view.BoundList-cfg-renderSelectors\" rel=\"Ext.view.BoundList-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.view.BoundList-cfg-childEls\" rel=\"Ext.view.BoundList-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"shadow":{"!doc":"<p>Specifies whether the floating component should be given a shadow. Set to true to automatically create an\n<a href=\"#!/api/Ext.Shadow\" rel=\"Ext.Shadow\" class=\"docClass\">Ext.Shadow</a>, or a string indicating the shadow's display <a href=\"#!/api/Ext.Shadow-cfg-mode\" rel=\"Ext.Shadow-cfg-mode\" class=\"docClass\">Ext.Shadow.mode</a>. Set to false to\ndisable the shadow.</p>\n"},"tpl":{"!doc":"<p>A String or Ext.XTemplate instance to apply to inner template.</p>\n\n<p><a href=\"#!/api/Ext.view.BoundList\" rel=\"Ext.view.BoundList\" class=\"docClass\">Ext.view.BoundList</a> is used for the dropdown list of <a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.field.ComboBox</a>.\nTo customize the template you can do this:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">Ext.form.field.ComboBox</a>', {\n    fieldLabel   : 'State',\n    queryMode    : 'local',\n    displayField : 'text',\n    valueField   : 'abbr',\n    store        : Ext.create('StateStore', {\n        fields : ['abbr', 'text'],\n        data   : [\n            {\"abbr\":\"AL\", \"name\":\"Alabama\"},\n            {\"abbr\":\"AK\", \"name\":\"Alaska\"},\n            {\"abbr\":\"AZ\", \"name\":\"Arizona\"}\n            //...\n        ]\n    }),\n    listConfig : {\n        tpl : '&lt;tpl for=\".\"&gt;&lt;div class=\"x-boundlist-item\"&gt;{abbr}&lt;/div&gt;&lt;/tpl&gt;'\n    }\n});\n</code></pre>\n\n<p>Defaults to:</p>\n\n<pre><code>Ext.create('Ext.XTemplate',\n    '&lt;ul&gt;&lt;tpl for=\".\"&gt;',\n        '&lt;li role=\"option\" class=\"' + itemCls + '\"&gt;' + me.getInnerTpl(me.displayField) + '&lt;/li&gt;',\n    '&lt;/tpl&gt;&lt;/ul&gt;'\n);\n</code></pre>\n"},"trackOver":{"!type":"bool","!doc":"<p>When <code>true</code> the <a href=\"#!/api/Ext.view.BoundList-cfg-overItemCls\" rel=\"Ext.view.BoundList-cfg-overItemCls\" class=\"docClass\">overItemCls</a> will be applied to rows when hovered over.\nThis in return will also cause <a href=\"#!/api/Ext.view.View-event-highlightitem\" rel=\"Ext.view.View-event-highlightitem\" class=\"docClass\">highlightitem</a> and\n<a href=\"#!/api/Ext.view.View-event-unhighlightitem\" rel=\"Ext.view.View-event-unhighlightitem\" class=\"docClass\">unhighlightitem</a> events to be fired.</p>\n\n<p>Enabled automatically when the <a href=\"#!/api/Ext.view.BoundList-cfg-overItemCls\" rel=\"Ext.view.BoundList-cfg-overItemCls\" class=\"docClass\">overItemCls</a> config is set.</p>\n"},"listItemCls":{"!type":"string"},"pagingToolbar":{"!doc":"<p>A reference to the PagingToolbar instance in this view. Only populated if <a href=\"#!/api/Ext.view.BoundList-cfg-pageSize\" rel=\"Ext.view.BoundList-cfg-pageSize\" class=\"docClass\">pageSize</a> is greater\nthan zero and the BoundList has been rendered.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"bindStore":{"!type":"fn(store: ?, initial: ?) -> !this","!doc":"<p>Changes the data store bound to this view and refreshes it.</p>\n"},"createPagingToolbar":{"!type":"fn() -> !this"},"getInnerTpl":{"!type":"fn(displayField: ?) -> string","!doc":"<p>A method that returns the inner template for displaying items in the list.\nThis method is useful to override when using a more complex display value, for example\ninserting an icon along with the text.</p>\n\n<p>The XTemplate is created with a reference to the owning form field in the <code>field</code> property to provide access\nto context. For example to highlight the currently typed value, the getInnerTpl may be configured into a\nComboBox as part of the <a href=\"#!/api/Ext.form.field.ComboBox-cfg-listConfig\" rel=\"Ext.form.field.ComboBox-cfg-listConfig\" class=\"docClass\">listConfig</a>:</p>\n\n<p>   listConfig: {\n       getInnerTpl: function() {\n           return '{[values.name.replace(this.field.getRawValue(), \"<b>\" + this.field.getRawValue() + \"</b>\")]}';\n       }\n   }</p>\n"},"getNodeContainer":{"!type":"fn() -> !this"},"getRefItems":{"!type":"fn() -> !this"},"getRefOwner":{"!type":"fn() -> !this"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"refresh":{"!type":"fn() -> !this","!doc":"<p>Refreshes the view by reloading the data from the store and re-rendering the template.</p>\n"}},"!doc":"<p>An internally used DataView for <a href=\"#!/api/Ext.form.field.ComboBox\" rel=\"Ext.form.field.ComboBox\" class=\"docClass\">ComboBox</a>.</p>\n","!type":"fn(config: ?)"},"DragZone":{"prototype":{"containerScroll":{"!doc":"<p>True to register this container with the Scrollmanager for auto scrolling during drag operations.\nA <a href=\"#!/api/Ext.dd.ScrollManager\" rel=\"Ext.dd.ScrollManager\" class=\"docClass\">Ext.dd.ScrollManager</a> configuration may also be passed.</p>\n"},"getDragData":{"!type":"fn(e: ?) -> ?","!doc":"<p>Called when a mousedown occurs in this container. Looks in <a href=\"#!/api/Ext.dd.Registry\" rel=\"Ext.dd.Registry\" class=\"docClass\">Ext.dd.Registry</a> for a valid target to drag\nbased on the mouse down. Override this method to provide your own lookup logic (e.g. finding a child by class\nname). Make sure your returned object has a \"ddel\" attribute (with an HTML Element) for other functions to work.</p>\n"},"getRepairXY":{"!type":"fn(e: ?, data: ?) -> ?","!doc":"<p>Called before a repair of an invalid drop to get the XY to animate to. By default returns the XY of\nthis.dragData.ddel</p>\n"},"init":{"!type":"fn(id: ?, sGroup: ?, config: ?) -> !this","!doc":"<p>Sets up the DragDrop object.  Must be called in the constructor of any\n<a href=\"#!/api/Ext.dd.DragDrop\" rel=\"Ext.dd.DragDrop\" class=\"docClass\">Ext.dd.DragDrop</a> subclass</p>\n"},"onInitDrag":{"!type":"fn(x: ?, y: ?) -> bool","!doc":"<p>Called once drag threshold has been reached to initialize the proxy element. By default, it clones the\nthis.dragData.ddel</p>\n"},"onItemMouseDown":{"!type":"fn(view: ?, record: ?, item: ?, index: ?, e: ?) -> !this"},"onValidDrop":{"!type":"fn(target: ?, e: ?, id: ?) -> !this"},"getDragText":{"!type":"fn() -> !this"},"isPreventDrag":{"!type":"fn(e: ?, record: ?, item: ?, index: ?) -> !this","!doc":"<p>Template method called upon mousedown. May be overridden in subclasses, or configured\ninto an instance.</p>\n\n<p>Return <code>true</code> to prevent drag start.</p>\n"}},"!type":"fn(config: ?)"},"BoundListKeyNav":{"!doc":"<p>A specialized <a href=\"#!/api/Ext.util.KeyNav\" rel=\"Ext.util.KeyNav\" class=\"docClass\">Ext.util.KeyNav</a> implementation for navigating a <a href=\"#!/api/Ext.view.BoundList\" rel=\"Ext.view.BoundList\" class=\"docClass\">Ext.view.BoundList</a> using\nthe keyboard. The up, down, pageup, pagedown, home, and end keys move the active highlight\nthrough the list. The enter key invokes the selection model's select action using the highlighted item.</p>\n","!type":"fn(config: ?)","prototype":{"boundList":{"!doc":"<p>The <a href=\"#!/api/Ext.view.BoundList\" rel=\"Ext.view.BoundList\" class=\"docClass\">Ext.view.BoundList</a> instance for which key navigation will be managed.</p>\n"},"defaultHandlers":{},"highlightAt":{"!type":"fn(index: ?) -> !this","!doc":"<p>Highlights the item at the given index.</p>\n"},"selectHighlighted":{"!type":"fn(e: ?) -> !this","!doc":"<p>Triggers selection of the currently highlighted item according to the behavior of\nthe configured SelectionModel.</p>\n"}}},"MultiSelector":{"!doc":"<p>This component provides a grid holding selected items from a second store of potential\nmembers. The <code>store</code> of this component represents the selected items. The \"search store\"\nrepresents the potentially selected items.</p>\n\n<p>While this component is a grid and so you can configure <code>columns</code>, it is best to leave\nthat to this class in its <code>initComponent</code> method. That allows this class to create the\nextra column that allows the user to remove rows. Instead use <code><a href=\"#!/api/Ext.view.MultiSelector-cfg-fieldName\" rel=\"Ext.view.MultiSelector-cfg-fieldName\" class=\"docClass\">fieldName</a></code> and\n<code><a href=\"#!/api/Ext.view.MultiSelector-cfg-fieldTitle\" rel=\"Ext.view.MultiSelector-cfg-fieldTitle\" class=\"docClass\">fieldTitle</a></code> to configure the primary column's <code>dataIndex</code> and column <code>text</code>,\nrespectively.</p>\n","!type":"fn(config: ?)","prototype":{"addToolText":{"!type":"string","!doc":"<p>The tooltip to display when the user hovers over the \"+\" tool in the panel header.</p>\n"},"emptyText":{"!type":"string","!doc":"<p>Default text (html tags are accepted) to display in the Panel body when the Store\nis empty. When specified, and the Store is empty, the text will be rendered inside a DIV with the CSS class \"x-grid-empty\".</p>\n"},"fieldName":{"!type":"string","!doc":"<p>The name of the data field to display in the primary column of the grid.</p>\n"},"fieldTitle":{"!type":"string","!doc":"<p>The text to display in the column header for the primary column of the grid.</p>\n"},"removeRowText":{"!type":"string","!doc":"<p>The text to display in the \"remove this row\" column. By default this is a Unicode\n\"X\" looking glyph.</p>\n"},"removeRowTip":{"!type":"string","!doc":"<p>The tooltip to display when the user hovers over the remove cell.</p>\n"},"search":{"!doc":"<p>This object configures the search popup component. By default this contains the\n<code>xtype</code> for a <code><a href=\"#!/api/Ext.view.MultiSelectorSearch\" rel=\"Ext.view.MultiSelectorSearch\" class=\"docClass\">Ext.view.MultiSelectorSearch</a></code> component and specifies <code>autoLoad</code>\nfor its <code>store</code>.</p>\n"},"convertSearchRecord":{},"convertSelectionRecord":{},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"addTools":{"!type":"fn() -> !this","!doc":"<p>Template method to be implemented in subclasses to add their tools after the collapsible tool.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"getSearch":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.view.MultiSelector-cfg-search\" rel=\"Ext.view.MultiSelector-cfg-search\" class=\"docClass\">search</a>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"makeRemoveRowColumn":{"!type":"fn() -> !this"},"processRowEvent":{"!type":"fn(type: ?, view: ?, cell: ?, recordIndex: ?, cellIndex: ?, e: ?, record: ?, row: ?) -> !this"},"renderRemoveRow":{"!type":"fn() -> !this"},"setSearch":{"!type":"fn(search: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.view.MultiSelector-cfg-search\" rel=\"Ext.view.MultiSelector-cfg-search\" class=\"docClass\">search</a>.</p>\n"}}},"MultiSelectorSearch":{"!doc":"<p>This component provides a grid holding selected items from a second store of potential\nmembers. The <code>store</code> of this component represents the selected items. The <code>searchStore</code>\nrepresents the potentially selected items.</p>\n\n<p>The default view defined by this class is intended to be easily replaced by deriving a\nnew class and overriding the appropriate methods. For example, the following is a very\ndifferent view that uses a date range and a data view:</p>\n\n<pre><code> Ext.define('App.view.DateBoundSearch', {\n     extend: '<a href=\"#!/api/Ext.view.MultiSelectorSearch\" rel=\"Ext.view.MultiSelectorSearch\" class=\"docClass\">Ext.view.MultiSelectorSearch</a>',\n\n     makeDockedItems: function () {\n         return {\n             xtype: 'toolbar',\n             items: [{\n                 xtype: 'datefield',\n                 emptyText: 'Start date...',\n                 flex: 1\n             },{\n                 xtype: 'datefield',\n                 emptyText: 'End date...',\n                 flex: 1\n             }]\n         };\n     },\n\n     makeItems: function () {\n         return [{\n             xtype: 'dataview',\n             itemSelector: '.search-item',\n             selType: 'rowselection',\n             store: this.store,\n             autoScroll: true,\n             tpl:\n                 '&lt;tpl for=\".\"&gt;' +\n                     '&lt;div class=\"search-item\"&gt;' +\n                         '&lt;img src=\"{icon}\"&gt;' +\n                         '&lt;div&gt;{name}&lt;/div&gt;' +\n                     '&lt;/div&gt;' +\n                 '&lt;/tpl&gt;'\n         }];\n     },\n\n     getSearchStore: function () {\n         return this.items.getAt(0).getStore();\n     },\n\n     selectRecords: function (records) {\n         var view = this.items.getAt(0);\n         return view.getSelectionModel().select(records);\n     }\n });\n</code></pre>\n\n<p><strong>Important</strong>: This class assumes there are two components with specific <code>reference</code>\nnames assigned to them. These are <code>\"searchField\"</code> and <code>\"searchGrid\"</code>. These components\nare produced by the <code>makeDockedItems</code> and <code>makeItems</code> method, respectively. When\noverriding these it is important to remember to place these <code>reference</code> values on the\nappropriate components.</p>\n","!type":"fn(config: ?)","prototype":{"border":{"!type":"bool","!doc":"<p>Specify as <code>false</code> to render the Panel with zero width borders.</p>\n\n<p>Leaving the value as <code>true</code> uses the selected theme's Ext.panel.Panel.$panel-border-width</p>\n\n<p>Defaults to <code>false</code> when using or extending Neptune.</p>\n"},"floating":{"!type":"bool","!doc":"<p>Specify as true to float the Component outside of the document flow using CSS absolute positioning.</p>\n\n<p>Components such as <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Window</a>s and <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Menu</a>s are floating by default.</p>\n\n<p>Floating Components that are programatically <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">rendered</a> will register\nthemselves with the global <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a></p>\n\n<h3>Floating Components as child items of a Container</h3>\n\n<p>A floating Component may be used as a child item of a Container. This just allows the floating Component to seek\na ZIndexManager by examining the ownerCt chain.</p>\n\n<p>When configured as floating, Components acquire, at render time, a <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a> which\nmanages a stack of related floating Components. The ZIndexManager brings a single floating Component to the top\nof its stack when the Component's <a href=\"#!/api/Ext.view.MultiSelectorSearch-method-toFront\" rel=\"Ext.view.MultiSelectorSearch-method-toFront\" class=\"docClass\">toFront</a> method is called.</p>\n\n<p>The ZIndexManager is found by traversing up the <a href=\"#!/api/Ext.view.MultiSelectorSearch-property-ownerCt\" rel=\"Ext.view.MultiSelectorSearch-property-ownerCt\" class=\"docClass\">ownerCt</a> chain to find an ancestor which itself is\nfloating. This is so that descendant floating Components of floating <em>Containers</em> (Such as a ComboBox dropdown\nwithin a Window) can have its zIndex managed relative to any siblings, but always <strong>above</strong> that floating\nancestor Container.</p>\n\n<p>If no floating ancestor is found, a floating Component registers itself with the default <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a>.</p>\n\n<p>Floating components <em>do not participate in the Container's layout</em>. Because of this, they are not rendered until\nyou explicitly <a href=\"#!/api/Ext.view.MultiSelectorSearch-method-show\" rel=\"Ext.view.MultiSelectorSearch-method-show\" class=\"docClass\">show</a> them.</p>\n\n<p>After rendering, the ownerCt reference is deleted, and the <a href=\"#!/api/Ext.view.MultiSelectorSearch-property-floatParent\" rel=\"Ext.view.MultiSelectorSearch-property-floatParent\" class=\"docClass\">floatParent</a> property is set to the found\nfloating ancestor Container. If no floating ancestor Container was found the <a href=\"#!/api/Ext.view.MultiSelectorSearch-property-floatParent\" rel=\"Ext.view.MultiSelectorSearch-property-floatParent\" class=\"docClass\">floatParent</a> property will\nnot be set.</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.view.MultiSelectorSearch-cfg-items\" rel=\"Ext.view.MultiSelectorSearch-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"minHeight":{"!type":"number","!doc":"<p>The minimum value in pixels which this Component will set its height to.</p>\n\n<p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>\n"},"minWidth":{"!type":"number","!doc":"<p>The minimum value in pixels which this Component will set its width to.</p>\n\n<p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>\n"},"referenceHolder":{"!type":"bool","!doc":"<p>If <code>true</code>, this container will be marked as being a point in the hierarchy where\nreferences to items with a specified <code>reference</code> config will be held. The container\nwill automatically become a referenceHolder if a controller is specified.</p>\n\n<p>See the introductory docs for <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a> for more information\nabout references &amp; reference holders.</p>\n"},"resizable":{"!doc":"<p>Specify as <code>true</code> to apply a <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Resizer</a> to this Component after rendering.</p>\n\n<p>May also be specified as a config object to be passed to the constructor of <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Resizer</a>\nto override any defaults. By default the Component passes its minimum and maximum size, and uses\n<code><a href=\"#!/api/Ext.resizer.Resizer-cfg-dynamic\" rel=\"Ext.resizer.Resizer-cfg-dynamic\" class=\"docClass\">Ext.resizer.Resizer.dynamic</a>: false</code></p>\n"},"searchText":{"!type":"string","!doc":"<p>This text is displayed as the \"emptyText\" of the search <code>textfield</code>.</p>\n"},"defaultListenerScope":{"!type":"bool"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"afterShow":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Component is shown (after <a href=\"#!/api/Ext.view.MultiSelectorSearch-method-onShow\" rel=\"Ext.view.MultiSelectorSearch-method-onShow\" class=\"docClass\">onShow</a> is called).</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.view.MultiSelectorSearch-event-show\" rel=\"Ext.view.MultiSelectorSearch-event-show\" class=\"docClass\">show</a>.</p>\n"},"deselectRecords":{"!type":"fn(records: ?) -> !this"},"getOwnerStore":{"!type":"fn() -> !this"},"getSearchStore":{"!type":"fn() -> ?","!doc":"<p>Returns the store that holds search results. By default this comes from the\n\"search grid\". If this aspect of the view is changed sufficiently so that the\nsearch grid cannot be found, this method should be overridden to return the proper\nstore.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"makeDockedItems":{"!type":"fn() -> !this"},"makeItems":{"!type":"fn() -> !this"},"search":{"!type":"fn(text: ?) -> !this"},"selectRecords":{"!type":"fn(records: ?) -> !this"}}},"NodeCache":{"!doc":"<p>A cache of View elements keyed using the index of the associated record in the store.</p>\n\n<p>This implements the methods of {Ext.dom.CompositeElement} which are used by <a href=\"#!/api/Ext.view.AbstractView\" rel=\"Ext.view.AbstractView\" class=\"docClass\">Ext.view.AbstractView</a>\nto privide a map of record nodes and methods to manipulate the nodes.</p>\n","!type":"fn(view: ?)","prototype":{"importElementMethods":{},"range":{},"clear":{"!type":"fn(removeDom?: ?) -> !this","!doc":"<p>Removes all elements from this NodeCache.</p>\n"},"fill":{"!type":"fn(els: ?) -> ?","!doc":"<p>Clears this NodeCache and adds the elements passed.</p>\n"},"first":{"!type":"fn(asDom: ?) -> !this"},"getCount":{"!type":"fn() -> !this"},"indexOf":{"!type":"fn(el: ?) -> number","!doc":"<p>Find the index of the passed element within the composite collection.</p>\n"},"insert":{"!type":"fn(insertPoint: ?, nodes: ?) -> !this"},"invoke":{"!type":"fn(fn: ?, args: ?) -> ?"},"item":{"!type":"fn(index: ?, asDom: ?) -> !this"},"last":{"!type":"fn(asDom: ?) -> !this"},"removeElement":{"!type":"fn(el: ?, removeDom?: ?) -> !this","!doc":"<p>Removes the specified element(s).</p>\n"},"removeRange":{"!type":"fn(start: ?, end: ?, removeDom: ?) -> !this"},"replaceElement":{"!type":"fn(el: ?, replacement: ?, domReplace?: ?) -> ?","!doc":"<p>Replaces the specified element with the passed element.</p>\n"},"scroll":{"!type":"fn(newRecords: ?, direction: ?, removeCount: ?) -> !this","!doc":"<p>Appends/prepends records depending on direction flag</p>\n"},"slice":{"!type":"fn(start: ?, end: ?) -> !this"},"sumHeights":{"!type":"fn() -> !this"}}},"TableLayout":{"!doc":"<p>Component layout for <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">Ext.view.Table</a></p>\n","!type":"fn(config: ?)","prototype":{"type":{"!type":"string","!doc":"<p>End Definitions</p>\n"},"beginLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called before any calculation cycles to prepare for layout.</p>\n\n<p>This is a write phase and DOM reads should be strictly avoided when overridding\nthis method.</p>\n"},"calculate":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>Called to perform the calculations for this layout. This method will be called at\nleast once and may be called repeatedly if the <a href=\"#!/api/Ext.view.TableLayout-property-done\" rel=\"Ext.view.TableLayout-property-done\" class=\"docClass\">done</a> property is cleared\nbefore return to indicate that this layout is not yet done. The <a href=\"#!/api/Ext.view.TableLayout-property-done\" rel=\"Ext.view.TableLayout-property-done\" class=\"docClass\">done</a> property\nis always set to <code>true</code> before entering this method.</p>\n\n<p>This is a read phase and DOM writes should be strictly avoided in derived classes.\nInstead, DOM writes need to be written to <a href=\"#!/api/Ext.layout.ContextItem\" rel=\"Ext.layout.ContextItem\" class=\"docClass\">Ext.layout.ContextItem</a> objects to\n be flushed at the next opportunity.</p>\n"},"finishedLayout":{"!type":"fn(ownerContext: ?) -> !this","!doc":"<p>This method is called after all layouts are complete and their calculations flushed\nto the DOM. No further layouts will be run and this method is only called once per\nlayout run. The base component layout caches <code>lastComponentSize</code>.</p>\n\n<p>This is a write phase and DOM reads should be avoided if possible when overridding\nthis method.</p>\n\n<p>This method need not be implemented by derived classes and, in fact, should only be\nimplemented when needed.</p>\n"},"measureContentHeight":{"!type":"fn(ownerContext: ?) -> !this"},"setColumnWidths":{"!type":"fn(ownerContext: ?) -> !this"}}}},"Img":{"!doc":"<p>Simple helper class for easily creating image components. This renders an image tag to\nthe DOM with the configured src.</p>\n\n<p><p><img src=\"\" alt=\"Ext.Img component\" width=\"\" height=\"\"></p></p>\n\n<h2>Example usage:</h2>\n\n<pre><code>var changingImage = Ext.create('<a href=\"#!/api/Ext.Img\" rel=\"Ext.Img\" class=\"docClass\">Ext.Img</a>', {\n    src: 'http://www.sencha.com/img/20110215-feat-html5.png',\n    renderTo: Ext.getBody()\n});\n\n// change the src of the image programmatically\nchangingImage.setSrc('http://www.sencha.com/img/20110215-feat-perf.png');\n</code></pre>\n\n<p>By default, only an img element is rendered and that is this component's primary\n<a href=\"#!/api/Ext.Component-method-getEl\" rel=\"Ext.Component-method-getEl\" class=\"docClass\">element</a>. If the <a href=\"#!/api/Ext.Component-cfg-autoEl\" rel=\"Ext.Component-cfg-autoEl\" class=\"docClass\">Ext.Component.autoEl</a> property\nis other than 'img' (the default), the a child img element will be added to the primary\nelement. This can be used to create a wrapper element around the img.</p>\n\n<h2>Wrapping the img in a div:</h2>\n\n<pre><code>var wrappedImage = Ext.create('<a href=\"#!/api/Ext.Img\" rel=\"Ext.Img\" class=\"docClass\">Ext.Img</a>', {\n    src: 'http://www.sencha.com/img/20110215-feat-html5.png',\n    autoEl: 'div', // wrap in a div\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"alt":{"!type":"string","!doc":"<p>The descriptive text for non-visual UI description.</p>\n"},"autoEl":{"!doc":"<p>A tag name or DomHelper spec used to create the <a href=\"#!/api/Ext.Img-method-getEl\" rel=\"Ext.Img-method-getEl\" class=\"docClass\">Element</a> which will\nencapsulate this Component.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong>'div'</strong>. The more complex Sencha classes use a more\ncomplex DOM structure specified by their own <a href=\"#!/api/Ext.Img-cfg-renderTpl\" rel=\"Ext.Img-cfg-renderTpl\" class=\"docClass\">renderTpl</a>s.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components encapsulated by\ndifferent DOM elements. Example usage:</p>\n\n<pre><code>{\n    xtype: 'component',\n    autoEl: {\n        tag: 'img',\n        src: 'http://www.example.com/example.jpg'\n    }\n}, {\n    xtype: 'component',\n    autoEl: {\n        tag: 'blockquote',\n        html: 'autoEl is cool!'\n    }\n}, {\n    xtype: 'container',\n    autoEl: 'ul',\n    cls: 'ux-unordered-list',\n    items: {\n        xtype: 'component',\n        autoEl: 'li',\n        html: 'First list item'\n    }\n}\n</code></pre>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"glyph":{"!doc":"<p>A numeric unicode character code to serve as the image.  If this option is used\nThe image will be rendered using a div with innerHTML set to the html entity\nfor the given character code.  The default font-family for glyphs can be set\nglobally using <a href=\"#!/api/Ext-method-setGlyphFontFamily\" rel=\"Ext-method-setGlyphFontFamily\" class=\"docClass\">Ext.setGlyphFontFamily()</a>. Alternatively,\nthis config option accepts a string with the charCode and font-family separated by\nthe <code>@</code> symbol. For example '65@My Font Family'.</p>\n"},"imgCls":{"!type":"string","!doc":"<p>Optional CSS classes to add to the img element.</p>\n"},"src":{"!type":"string","!doc":"<p>The image src.</p>\n"},"title":{"!type":"string","!doc":"<p>Specifies addtional information about the image.</p>\n"},"maskOnDisable":{"!type":"bool","!doc":"<p>This is an internal flag that you use when creating custom components. By default this is set to <code>true</code> which means\nthat every component gets a mask when it's disabled. Components like FieldContainer, FieldSet, Field, Button, Tab\noverride this property to <code>false</code> since they want to implement custom disable logic.</p>\n"},"getElConfig":{"!type":"fn() -> !this"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"setGlyph":{"!type":"fn(glyph: ?) -> !this"},"setSrc":{"!type":"fn(src: ?) -> !this","!doc":"<p>Updates the <a href=\"#!/api/Ext.Img-cfg-src\" rel=\"Ext.Img-cfg-src\" class=\"docClass\">src</a> of the image.</p>\n"}}},"!doc":"<p>The Ext namespace (global object) encapsulates all classes, singletons, and\nutility methods provided by Sencha's libraries.</p>\n\n<p>Most user interface Components are at a lower level of nesting in the namespace,\nbut many common utility functions are provided as direct properties of the Ext namespace.</p>\n\n<p>Also many frequently used methods from other classes are provided as shortcuts\nwithin the Ext namespace. For example Ext.getCmp aliases\nExt.ComponentManager.get.</p>\n\n<p>Many applications are initiated with Ext.application which is\ncalled once the DOM is ready. This ensures all scripts have been loaded, preventing\ndependency issues. For example:</p>\n\n<pre><code> Ext.application({\n     name: 'MyApp',\n\n     launch: function () {\n         <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>(this.name, 'Ready to go!');\n     }\n });\n</code></pre>\n\n<p><b><a href=\"http://www.sencha.com/products/sencha-cmd/\">Sencha Cmd</a></b> is a free tool\nfor helping you generate and build Ext JS (and Sencha Touch) applications. See\n<code><a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Application</a></code> for more information about creating an app.</p>\n\n<p>A lower-level technique that does not use the <code><a href=\"#!/api/Ext.app.Application\" rel=\"Ext.app.Application\" class=\"docClass\">Ext.app.Application</a></code> architecture is\nExt.onReady.</p>\n\n<p>For more information about how to use the Ext classes, see:</p>\n\n<ul>\n<li><a href=\"http://www.sencha.com/learn/\">The Learning Center</a></li>\n<li><a href=\"http://www.sencha.com/learn/Ext_FAQ\">The FAQ</a></li>\n<li><a href=\"http://www.sencha.com/forum/\">The forums</a></li>\n</ul>\n\n","debugConfig":{"!doc":"<p>This object is used to enable or disable debugging for classes or namespaces. The\ndefault instance looks like this:</p>\n\n<pre><code> <a href=\"#!/api/Ext-cfg-debugConfig\" rel=\"Ext-cfg-debugConfig\" class=\"docClass\">Ext.debugConfig</a> = {\n     hooks: {\n         '*': true\n     }\n };\n</code></pre>\n\n<p>Typically applications will set this in their <code>\"app.json\"</code> like so:</p>\n\n<pre><code> {\n     \"debug\": {\n         \"hooks\": {\n             // Default for all namespaces:\n             '*': true,\n\n             // Except for Ext namespace which is disabled\n             'Ext': false,\n\n             // Except for Ext.layout namespace which is enabled\n             'Ext.layout': true\n         }\n     }\n }\n</code></pre>\n\n<p>Alternatively, because this property is consumed very early in the load process of\nthe framework, this can be set in a <code>script</code> tag that is defined prior to loading\nthe framework itself.</p>\n\n<p>For example, to enable debugging for the <code>Ext.layout</code> namespace only:</p>\n\n<pre><code> var Ext = Ext || {};\n <a href=\"#!/api/Ext-cfg-debugConfig\" rel=\"Ext-cfg-debugConfig\" class=\"docClass\">Ext.debugConfig</a> = {\n     hooks: {\n         //...\n     }\n };\n</code></pre>\n\n<p>For any class declared, the longest matching namespace specified determines if its\n<code>debugHooks</code> will be enabled. The default setting is specified by the '*' property.</p>\n\n<p><strong>NOTE:</strong> This option only applies to debug builds. All debugging is disabled in\nproduction builds.</p>\n"},"manifest":{"!doc":"<p>This object is initialized prior to loading the framework (Ext JS or Sencha\nTouch) and contains settings and other information describing the application.</p>\n\n<p>For applications built using Sencha Cmd, this is produced from the <code>\"app.json\"</code>\nfile with information extracted from all of the required packages' <code>\"package.json\"</code>\nfiles. This can be set to a string when your application is using the\n(microloader)[#/guide/microloader]. In this case, the string of \"foo\" will be\nrequested as <code>\"foo.json\"</code> and the object in that JSON file will parsed and set\nas this object.</p>\n"},"$eventNameMap":{"!doc":"<p>A map of event names which contained the lower-cased verions of any mixed\ncase event names.</p>\n"},"Logger":{},"baseCSSPrefix":{"!type":"string","!doc":"<p>The base prefix to use for all <code>Ext</code> components. To configure this property, you should use the\nExt.buildSettings object before the framework is loaded:</p>\n\n<pre><code>Ext.buildSettings = {\n    baseCSSPrefix : 'abc-'\n};\n</code></pre>\n\n<p>or you can change it before any components are rendered:</p>\n\n<pre><code><a href=\"#!/api/Ext-property-baseCSSPrefix\" rel=\"Ext-property-baseCSSPrefix\" class=\"docClass\">Ext.baseCSSPrefix</a> = Ext.buildSettings.baseCSSPrefix = 'abc-';\n</code></pre>\n\n<p>This will change what CSS classes components will use and you should\nthen recompile the SASS changing the <code>$prefix</code> SASS variable to match.</p>\n"},"emptyFn":{"!doc":"<p>A reusable empty function.</p>\n"},"emptyString":{"!doc":"<p>A zero length string which will pass a truth test. Useful for passing to methods\nwhich use a truth test to reject <i>falsy</i> values where a string value must be cleared.</p>\n"},"enableFx":{"!type":"bool","!doc":"<p>True if the <a href=\"#!/api/Ext.fx.Anim\" rel=\"Ext.fx.Anim\" class=\"docClass\">Ext.fx.Anim</a> Class is available.</p>\n"},"enableGarbageCollector":{"!type":"bool","!doc":"<p><code>true</code> to automatically uncache orphaned Ext.Elements periodically. If set to\n<code>false</code>, the application will be required to clean up orpaned Ext.Elements and\nit's listeners as to not cause memory leakage.</p>\n"},"enableListenerCollection":{"!type":"bool","!doc":"<p>True to automatically purge event listeners during garbageCollection.</p>\n"},"enumerables":{"!doc":"<p>An array containing extra enumerables for old browsers</p>\n"},"frameStartTime":{"!doc":"<p>This indicate the start timestamp of current cycle.\nIt is only reliable during dom-event-initiated cycles and\nExt.draw.Animator initiated cycles.</p>\n"},"functionFactoryCache":{},"identityFn":{"!doc":"<p>A reusable identity function that simply returns its first argument.</p>\n"},"isSecure":{"!type":"bool","!doc":"<p>True if the page is running over SSL</p>\n"},"name":{"!type":"string","!doc":"<p>The name of the property in the global namespace (The <code>window</code> in browser environments) which refers to the current instance of Ext.</p>\n\n\n<p>This is usually <code>\"Ext\"</code>, but if a sandboxed build of ExtJS is being used, this will be an alternative name.</p>\n\n\n<p>If code is being generated for use by <code>eval</code> or to create a <code>new Function</code>, and the global instance\nof Ext must be referenced, this is the name that should be built into the code.</p>\n\n"},"privateFn":{"!doc":"<p>A reusable empty function for use as <code>privates</code> members.</p>\n\n<pre><code> Ext.define('MyClass', {\n     nothing: <a href=\"#!/api/Ext-property-emptyFn\" rel=\"Ext-property-emptyFn\" class=\"docClass\">Ext.emptyFn</a>,\n\n     privates: {\n         privateNothing: <a href=\"#!/api/Ext-property-privateFn\" rel=\"Ext-property-privateFn\" class=\"docClass\">Ext.privateFn</a>\n     }\n });\n</code></pre>\n"},"splitAndUnescape":{},"validIdRe":{"!doc":"<p>Regular expression used for validating identifiers.</p>\n"},"apply":{"!type":"fn(object: ?, config: ?, defaults?: ?) -> ?","!doc":"<p>Copies all the properties of <code>config</code> to the specified <code>object</code>. There are two levels\nof defaulting supported:</p>\n\n<pre><code> <a href=\"#!/api/Ext-method-apply\" rel=\"Ext-method-apply\" class=\"docClass\">Ext.apply</a>(obj, { a: 1 }, { a: 2 });\n //obj.a === 1\n\n <a href=\"#!/api/Ext-method-apply\" rel=\"Ext-method-apply\" class=\"docClass\">Ext.apply</a>(obj, {  }, { a: 2 });\n //obj.a === 2\n</code></pre>\n\n<p>Note that if recursive merging and cloning without referencing the original objects\nor arrays is needed, use Ext.Object.merge instead.</p>\n"},"applyIf":{"!type":"fn(object: ?, config: ?) -> ?","!doc":"<p>Copies all the properties of config to object if they don't already exist.</p>\n"},"batchLayouts":{"!type":"fn(fn: ?, scope?: ?) -> !this","!doc":"<p>Utility wrapper that suspends layouts of all components for the duration of a given\nfunction.</p>\n"},"clone":{"!type":"fn(item: ?) -> ?","!doc":"<p>Clone simple variables including array, {}-like objects, DOM nodes and Date without keeping the old reference.\nA reference for the object itself is returned if it's not a direct decendant of Object. For model cloning,\nsee Model.copy.</p>\n"},"destroy":{"!type":"fn(args: ?) -> !this","!doc":"<p>Destroys all of the given objects. If arrays are passed, the elements of these\nare destroyed recusrively.</p>\n\n<p>What it means to \"destroy\" an object depends on the type of object.</p>\n\n<ul>\n<li><code>Array</code>: Each element of the array is destroyed recursively.</li>\n<li><code>Object</code>: Any object with a <code>destroy</code> method will have that method called.</li>\n</ul>\n\n"},"destroyMembers":{"!type":"fn(object: ?, args: ?) -> !this","!doc":"<p>Destroys the specified named members of the given object using <code><a href=\"#!/api/Ext-method-destroy\" rel=\"Ext-method-destroy\" class=\"docClass\">Ext.destroy</a></code>. These\nproperties will be set to <code>null</code>.</p>\n"},"getUniqueGlobalNamespace":{"!type":"fn() -> !this","!doc":"<p>Generate a unique reference of Ext in the global scope, useful for sandboxing</p>\n"},"id":{"!type":"fn(o?: ?, prefix?: ?) -> string","!doc":"<p>Generates unique ids. If the object/element is passes and it already has an <code>id</code>, it is unchanged.</p>\n"},"isArray":{"!type":"fn(target: ?) -> bool","!doc":"<p>Returns <code>true</code> if the passed value is a JavaScript Array, <code>false</code> otherwise.</p>\n"},"isBoolean":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns <code>true</code> if the passed value is a boolean.</p>\n"},"isDate":{"!type":"fn(object: ?) -> bool","!doc":"<p>Returns <code>true</code> if the passed value is a JavaScript Date object, <code>false</code> otherwise.</p>\n"},"isDebugEnabled":{"!type":"fn(className: ?) -> bool","!doc":"<p>This method returns <code>true</code> if debug is enabled for the specified class. This is\ndone by checking the <code>Ext.debugConfig.hooks</code> config for the closest match to the\ngiven <code>className</code>.</p>\n"},"isDefined":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns <code>true</code> if the passed value is defined.</p>\n"},"isElement":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns <code>true</code> if the passed value is an HTMLElement</p>\n"},"isEmpty":{"!type":"fn(value: ?, allowEmptyString?: ?) -> bool","!doc":"<p>Returns true if the passed value is empty, false otherwise. The value is deemed to be empty if it is either:</p>\n\n<ul>\n<li><code>null</code></li>\n<li><code>undefined</code></li>\n<li>a zero-length array</li>\n<li>a zero-length string (Unless the <code>allowEmptyString</code> parameter is set to <code>true</code>)</li>\n</ul>\n\n"},"isFunction":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns <code>true</code> if the passed value is a JavaScript Function, <code>false</code> otherwise.</p>\n"},"isIterable":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns <code>true</code> if the passed value is iterable, that is, if elements of it are addressable using array\nnotation with numeric indices, <code>false</code> otherwise.</p>\n\n<p>Arrays and function <code>arguments</code> objects are iterable. Also HTML collections such as <code>NodeList</code> and `HTMLCollection'\nare iterable.</p>\n"},"isMSDate":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns 'true' if the passed value is a String that matches the MS Date JSON\nencoding format.</p>\n"},"isNumber":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns <code>true</code> if the passed value is a number. Returns <code>false</code> for non-finite numbers.</p>\n"},"isNumeric":{"!type":"fn(value: ?) -> bool","!doc":"<p>Validates that a value is numeric.</p>\n"},"isObject":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns <code>true</code> if the passed value is a JavaScript Object, <code>false</code> otherwise.</p>\n"},"isPrimitive":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns <code>true</code> if the passed value is a JavaScript 'primitive', a string, number\nor boolean.</p>\n"},"isSimpleObject":{"!type":"fn(value: ?) -> !this"},"isString":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns <code>true</code>if the passed value is a string.</p>\n"},"isTextNode":{"!type":"fn(value: ?) -> bool","!doc":"<p>Returns <code>true</code> if the passed value is a TextNode</p>\n"},"makeIdSelector":{"!type":"fn(id: ?) -> string","!doc":"<p>Converts an id (<code>'foo'</code>) into an id selector (<code>'#foo'</code>).  This method is used\ninternally by the framework whenever an id needs to be converted into a selector\nand is provided as a hook for those that need to escape IDs selectors since,\nas of Ext 5.0, the framework no longer escapes IDs by default.</p>\n"},"now":{"!type":"fn() -> number","!doc":"<p>Returns the current timestamp.</p>\n"},"override":{"!type":"fn(target: ?, overrides: ?) -> !this","!doc":"<p>Overrides members of the specified <code>target</code> with the given values.</p>\n\n<p>If the <code>target</code> is a class declared using Ext.define, the\n<code>override</code> method of that class is called (see Ext.Base.override) given\nthe <code>overrides</code>.</p>\n\n<p>If the <code>target</code> is a function, it is assumed to be a constructor and the contents\nof <code>overrides</code> are applied to its <code>prototype</code> using <a href=\"#!/api/Ext-method-apply\" rel=\"Ext-method-apply\" class=\"docClass\">Ext.apply</a>.</p>\n\n<p>If the <code>target</code> is an instance of a class declared using Ext.define,\nthe <code>overrides</code> are applied to only that instance. In this case, methods are\nspecially processed to allow them to use Ext.Base.callParent.</p>\n\n<pre><code> var panel = new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.Panel</a>({ ... });\n\n <a href=\"#!/api/Ext-method-override\" rel=\"Ext-method-override\" class=\"docClass\">Ext.override</a>(panel, {\n     initComponent: function () {\n         // extra processing...\n\n         this.callParent();\n     }\n });\n</code></pre>\n\n<p>If the <code>target</code> is none of these, the <code>overrides</code> are applied to the <code>target</code>\nusing <a href=\"#!/api/Ext-method-apply\" rel=\"Ext-method-apply\" class=\"docClass\">Ext.apply</a>.</p>\n\n<p>Please refer to Ext.define and Ext.Base.override for\nfurther details.</p>\n"},"resumeLayouts":{"!type":"fn(flush?: ?) -> !this","!doc":"<p>Resumes layout activity in the whole framework.</p>\n\n<p><a href=\"#!/api/Ext-method-suspendLayouts\" rel=\"Ext-method-suspendLayouts\" class=\"docClass\">suspendLayouts</a> is alias of <a href=\"#!/api/Ext.Component-method-suspendLayouts\" rel=\"Ext.Component-method-suspendLayouts\" class=\"docClass\">Ext.Component.suspendLayouts</a>.</p>\n"},"returnId":{"!type":"fn(o: ?) -> !this","!doc":"<p>A reusable function which returns the value of <code>getId()</code> called upon a single passed parameter.\nUseful when creating a Ext.util.MixedCollection of objects keyed by an identifier returned from a <code>getId</code> method.</p>\n"},"returnTrue":{"!type":"fn() -> !this","!doc":"<p>A reusable function which returns <code>true</code>.</p>\n"},"setGlyphFontFamily":{"!type":"fn(fontFamily: ?) -> !this","!doc":"<p>Sets the default font-family to use for components that support a <code>glyph</code> config.</p>\n"},"suspendLayouts":{"!type":"fn() -> !this","!doc":"<p>Stops layouts from happening in the whole framework.</p>\n\n<p>It's useful to suspend the layout activity while updating multiple components and\ncontainers:</p>\n\n<pre><code><a href=\"#!/api/Ext-method-suspendLayouts\" rel=\"Ext-method-suspendLayouts\" class=\"docClass\">Ext.suspendLayouts</a>();\n// batch of updates...\n<a href=\"#!/api/Ext-method-resumeLayouts\" rel=\"Ext-method-resumeLayouts\" class=\"docClass\">Ext.resumeLayouts</a>(true);\n</code></pre>\n\n<p><a href=\"#!/api/Ext-method-suspendLayouts\" rel=\"Ext-method-suspendLayouts\" class=\"docClass\">suspendLayouts</a> is alias of <a href=\"#!/api/Ext.Component-method-suspendLayouts\" rel=\"Ext.Component-method-suspendLayouts\" class=\"docClass\">Ext.Component.suspendLayouts</a>.</p>\n\n<p>See also <a href=\"#!/api/Ext-method-batchLayouts\" rel=\"Ext-method-batchLayouts\" class=\"docClass\">batchLayouts</a> for more abstract way of doing this.</p>\n"},"valueFrom":{"!type":"fn(value: ?, defaultValue: ?, allowBlank?: ?) -> ?","!doc":"<p>Returns the given value itself if it's not empty, as described in <a href=\"#!/api/Ext-method-isEmpty\" rel=\"Ext-method-isEmpty\" class=\"docClass\">isEmpty</a>; returns the default\nvalue (second argument) otherwise.</p>\n"},"resizer":{"Splitter":{"prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"collapseOnDblClick":{"!type":"bool","!doc":"<p>True to enable dblclick to toggle expand and collapse on the <a href=\"#!/api/Ext.resizer.Splitter-cfg-collapseTarget\" rel=\"Ext.resizer.Splitter-cfg-collapseTarget\" class=\"docClass\">collapseTarget</a> Panel.</p>\n"},"collapseTarget":{"!doc":"<p>A string describing the relative position of the immediate sibling Panel to collapse. May be 'prev' or 'next'.</p>\n\n<p>Or the immediate sibling Panel to collapse.</p>\n\n<p>The orientation of the mini-collapse tool will be inferred from this setting.</p>\n\n<p><strong>Note that only Panels may be collapsed.</strong></p>\n"},"collapsedCls":{"!type":"string","!doc":"<p>A class to add to the splitter when it is collapsed. See <a href=\"#!/api/Ext.resizer.Splitter-cfg-collapsible\" rel=\"Ext.resizer.Splitter-cfg-collapsible\" class=\"docClass\">collapsible</a>.</p>\n"},"collapsible":{"!type":"bool","!doc":"<p>True to show a mini-collapse tool in the Splitter to toggle expand and collapse on the <a href=\"#!/api/Ext.resizer.Splitter-cfg-collapseTarget\" rel=\"Ext.resizer.Splitter-cfg-collapseTarget\" class=\"docClass\">collapseTarget</a> Panel.\nDefaults to the <a href=\"#!/api/Ext.panel.Panel-cfg-collapsible\" rel=\"Ext.panel.Panel-cfg-collapsible\" class=\"docClass\">collapsible</a> setting of the Panel.</p>\n"},"defaultSplitMax":{"!type":"number","!doc":"<p>Provides a default maximum width or height for the two components\nthat the splitter is between.</p>\n"},"defaultSplitMin":{"!type":"number","!doc":"<p>Provides a default minimum width or height for the two components\nthat the splitter is between.</p>\n"},"performCollapse":{"!type":"bool","!doc":"<p>Set to false to prevent this Splitter's mini-collapse tool from managing the collapse\nstate of the <a href=\"#!/api/Ext.resizer.Splitter-cfg-collapseTarget\" rel=\"Ext.resizer.Splitter-cfg-collapseTarget\" class=\"docClass\">collapseTarget</a>.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.resizer.Splitter-method-getEl\" rel=\"Ext.resizer.Splitter-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.resizer.Splitter-method-getEl\" rel=\"Ext.resizer.Splitter-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.resizer.Splitter-cfg-renderSelectors\" rel=\"Ext.resizer.Splitter-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.resizer.Splitter-cfg-childEls\" rel=\"Ext.resizer.Splitter-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"size":{"!type":"number","!doc":"<p>The size of the splitter. This becomes the height for vertical splitters and\nwidth for horizontal splitters.</p>\n"},"tracker":{"!doc":"<p>Any configuration options to be passed to the underlying <a href=\"#!/api/Ext.resizer.SplitterTracker\" rel=\"Ext.resizer.SplitterTracker\" class=\"docClass\">Ext.resizer.SplitterTracker</a>.</p>\n"},"ariaRole":{"!type":"string"},"canResize":{"!type":"bool","!doc":"<p>Default to tree, allow internal classes to disable resizing</p>\n"},"collapseDirProps":{},"collapsedClsInternal":{"!type":"string"},"orientationProps":{},"vertical":{"!type":"bool"},"applyCollapseDirection":{"!type":"fn() -> !this"},"applyOrientation":{"!type":"fn() -> !this"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"getCollapseDirection":{"!type":"fn() -> !this"},"getCollapseTarget":{"!type":"fn() -> !this"},"onBeforeTargetCollapse":{"!type":"fn() -> !this"},"onBeforeTargetExpand":{"!type":"fn(target: ?) -> !this"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"onTargetCollapse":{"!type":"fn(target: ?) -> !this"},"onTargetExpand":{"!type":"fn(target: ?) -> !this"},"setCollapseEl":{"!type":"fn(display: ?) -> !this"},"setOrientation":{"!type":"fn(orientation: ?) -> !this"},"setSize":{"!type":"fn() -> ?","!doc":"<p>Work around IE bug. %age margins do not get recalculated on element resize unless repaint called.</p>\n"},"toggleTargetCmp":{"!type":"fn(e: ?, t: ?) -> !this"},"updateOrientation":{"!type":"fn() -> !this"},"isSplitter":{"!type":"bool"},"orientation":{"!type":"string","!doc":"<p>Orientation of this Splitter. <code>'vertical'</code> when used in an hbox layout, <code>'horizontal'</code>\nwhen used in a vbox layout.</p>\n"},"getTrackerConfig":{"!type":"fn() -> !this","!doc":"<p>Returns the config object (with an <code>xclass</code> property) for the splitter tracker. This\nis overridden by <a href=\"#!/api/Ext.resizer.BorderSplitter\" rel=\"Ext.resizer.BorderSplitter\" class=\"docClass\">BorderSplitter</a> to create a\n<a href=\"#!/api/Ext.resizer.BorderSplitterTracker\" rel=\"Ext.resizer.BorderSplitterTracker\" class=\"docClass\">BorderSplitterTracker</a>.</p>\n"}},"!doc":"<p>This class functions <strong>between siblings of a <a href=\"#!/api/Ext.layout.container.VBox\" rel=\"Ext.layout.container.VBox\" class=\"docClass\">VBox</a> or <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">HBox</a>\nlayout</strong> to resize both immediate siblings.</p>\n\n<p>A Splitter will preserve the flex ratio of any flexed siblings it is required to resize. It does this by setting the <code>flex</code> property of <em>all</em> flexed siblings\nto equal their pixel size. The actual numerical <code>flex</code> property in the Components will change, but the <strong>ratio</strong> to the total flex value will be preserved.</p>\n\n<p>A Splitter may be configured to show a centered mini-collapse tool orientated to collapse the <a href=\"#!/api/Ext.resizer.Splitter-cfg-collapseTarget\" rel=\"Ext.resizer.Splitter-cfg-collapseTarget\" class=\"docClass\">collapseTarget</a>.\nThe Splitter will then call that sibling Panel's <a href=\"#!/api/Ext.panel.Panel-method-collapse\" rel=\"Ext.panel.Panel-method-collapse\" class=\"docClass\">collapse</a> or <a href=\"#!/api/Ext.panel.Panel-method-expand\" rel=\"Ext.panel.Panel-method-expand\" class=\"docClass\">expand</a> method\nto perform the appropriate operation (depending on the sibling collapse state). To create the mini-collapse tool but take care\nof collapsing yourself, configure the splitter with <code><a href=\"#!/api/Ext.resizer.Splitter-cfg-performCollapse\" rel=\"Ext.resizer.Splitter-cfg-performCollapse\" class=\"docClass\">performCollapse</a>: false</code>.</p>\n","!type":"fn(config: ?)"},"SplitterTracker":{"prototype":{"enabled":{"!type":"bool"},"overlayCls":{"!type":"string"},"calculateConstrainRegion":{"!type":"fn() -> !this","!doc":"<p>calculate the constrain Region in which the splitter el may be moved.</p>\n"},"createDragOverlay":{"!type":"fn() -> !this"},"endDrag":{"!type":"fn() -> !this","!doc":"<p>Cleans up the overlay (if we have one) and calls the base. This cannot be done in\nonEnd, because onEnd is only called if a drag is detected but the overlay is created\nregardless (by onBeforeStart).</p>\n"},"getNextCmp":{"!type":"fn() -> !this"},"getPrevCmp":{"!type":"fn() -> !this"},"getResizeOffset":{"!type":"fn() -> !this"},"getSplitter":{"!type":"fn() -> !this"},"getVertNextConstrainLeft":{"!type":"fn(o: ?) -> !this"},"getVertNextConstrainRight":{"!type":"fn(o: ?) -> !this"},"getVertPrevConstrainLeft":{"!type":"fn(o: ?) -> !this"},"getVertPrevConstrainRight":{"!type":"fn(o: ?) -> !this"},"onBeforeStart":{"!type":"fn(e: ?) -> !this","!doc":"<p>ensure the tracker is enabled, store boxes of previous and next\ncomponents and calculate the constrain region</p>\n"},"onDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>Track the proxy and set the proper XY coordinates\nwhile constraining the drag</p>\n"},"onEnd":{"!type":"fn(e: ?) -> !this","!doc":"<p>perform the resize and remove the proxy class from the splitter el</p>\n"},"onStart":{"!type":"fn(e: ?) -> !this","!doc":"<p>We move the splitter el. Add the proxy class.</p>\n"},"performResize":{"!type":"fn(e: ?, offset: ?) -> !this","!doc":"<p>Performs the actual resizing of the previous and next components</p>\n"}},"!doc":"<p>Private utility class for Ext.Splitter.</p>\n"},"BorderSplitter":{"!doc":"<p>Private utility class for <a href=\"#!/api/Ext.layout.container.Border\" rel=\"Ext.layout.container.Border\" class=\"docClass\">Ext.layout.container.Border</a>.</p>\n","!type":"fn(config: ?)","prototype":{"collapseTarget":{"!doc":"<p>must be configured in by the border layout:</p>\n"},"getTrackerConfig":{"!type":"fn() -> !this","!doc":"<p>Returns the config object (with an <code>xclass</code> property) for the splitter tracker. This\nis overridden by <a href=\"#!/api/Ext.resizer.BorderSplitter\" rel=\"Ext.resizer.BorderSplitter\" class=\"docClass\">BorderSplitter</a> to create a\n<a href=\"#!/api/Ext.resizer.BorderSplitterTracker\" rel=\"Ext.resizer.BorderSplitterTracker\" class=\"docClass\">BorderSplitterTracker</a>.</p>\n"}}},"BorderSplitterTracker":{"!doc":"<p>Private utility class for Ext.BorderSplitter.</p>\n","prototype":{"constraintAdjusters":{},"getNextCmp":{},"getPrevCmp":{},"splitAdjusters":{},"calculateConstrainRegion":{"!type":"fn() -> !this","!doc":"<p>calculate the constrain Region in which the splitter el may be moved.</p>\n"},"getCollapseDirection":{"!type":"fn() -> !this"},"onBeforeStart":{"!type":"fn(e: ?) -> !this","!doc":"<p>ensure the tracker is enabled, store boxes of previous and next\ncomponents and calculate the constrain region</p>\n"},"performResize":{"!type":"fn(e: ?, offset: ?) -> !this","!doc":"<p>Performs the actual resizing of the previous and next components</p>\n"}}},"Handle":{"!doc":"<p>Provides a handle for 9-point resizing of Elements or Components.</p>\n","!type":"fn(config: ?)","prototype":{"region":{"!doc":"<p>Ext.resizer.Resizer.prototype.possiblePositions define the regions\nwhich will be passed in as a region configuration.</p>\n"},"ariaRole":{"!type":"string"},"baseHandleCls":{"!type":"string"},"handleCls":{"!type":"string"},"beforeRender":{"!type":"fn() -> !this"}}},"Resizer":{"!doc":"<p>Applies drag handles to an element or component to make it resizable. The drag handles are inserted into the element\n(or component's element) and positioned absolute.</p>\n\n<p>Textarea and img elements will be wrapped with an additional div because these elements do not support child nodes.\nThe original element can be accessed through the originalTarget property.</p>\n\n<p>Here is the list of valid resize handles:</p>\n\n<pre><code>Value   Description\n------  -------------------\n 'n'     north\n 's'     south\n 'e'     east\n 'w'     west\n 'nw'    northwest\n 'sw'    southwest\n 'se'    southeast\n 'ne'    northeast\n 'all'   all\n</code></pre>\n\n<p><p><img src=\"\" alt=\"Ext.resizer.Resizer component\" width=\"\" height=\"\"></p></p>\n\n<p>Here's an example showing the creation of a typical Resizer:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Ext.resizer.Resizer</a>', {\n    target: 'elToResize',\n    handles: 'all',\n    minWidth: 200,\n    minHeight: 100,\n    maxWidth: 500,\n    maxHeight: 400,\n    pinned: true\n});\n</code></pre>\n","prototype":{"constrainTo":{"!doc":"<p>An element, or a Region into which the resize operation must be constrained.</p>\n"},"dynamic":{"!type":"bool","!doc":"<p>Specify as true to update the <a href=\"#!/api/Ext.resizer.Resizer-cfg-target\" rel=\"Ext.resizer.Resizer-cfg-target\" class=\"docClass\">target</a> (Element or <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a>) dynamically during\ndragging. This is <code>true</code> by default, but the <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a> class passes <code>false</code> when it is\nconfigured as <a href=\"#!/api/Ext.Component-cfg-resizable\" rel=\"Ext.Component-cfg-resizable\" class=\"docClass\">Ext.Component.resizable</a>.</p>\n\n<p>If specified as <code>false</code>, a proxy element is displayed during the resize operation, and the <a href=\"#!/api/Ext.resizer.Resizer-cfg-target\" rel=\"Ext.resizer.Resizer-cfg-target\" class=\"docClass\">target</a> is\nupdated on mouseup.</p>\n"},"handles":{"!type":"string","!doc":"<p>String consisting of the resize handles to display. Defaults to 's e se' for Elements and fixed position\nComponents. Defaults to 8 point resizing for floating Components (such as Windows). Specify either <code>'all'</code> or any\nof <code>'n s e w ne nw se sw'</code>.</p>\n"},"height":{"!type":"number","!doc":"<p>Optional. The height to set target to in pixels</p>\n"},"heightIncrement":{"!type":"number","!doc":"<p>The increment to snap the height resize in pixels.</p>\n"},"maxHeight":{"!type":"number","!doc":"<p>The maximum height for the element</p>\n"},"maxWidth":{"!type":"number","!doc":"<p>The maximum width for the element</p>\n"},"minHeight":{"!type":"number","!doc":"<p>The minimum height for the element</p>\n"},"minWidth":{"!type":"number","!doc":"<p>The minimum width for the element</p>\n"},"pinned":{"!type":"bool","!doc":"<p>True to ensure that the resize handles are always visible, false indicates resizing by cursor changes only</p>\n"},"preserveRatio":{"!type":"bool","!doc":"<p>True to preserve the original ratio between height and width during resize</p>\n"},"target":{"!doc":"<p>The Element or Component to resize.</p>\n"},"transparent":{"!type":"bool","!doc":"<p>True for transparent handles. This is only applied at config time.</p>\n"},"width":{"!type":"number","!doc":"<p>Optional. The width to set the target to in pixels</p>\n"},"widthIncrement":{"!type":"number","!doc":"<p>The increment to snap the width resize in pixels.</p>\n"},"delimiterRe":{},"el":{"!doc":"<p>Outer element for resizing behavior.</p>\n"},"handleCls":{"!type":"string"},"originalTarget":{"!doc":"<p>Reference to the original resize target if the element of the original resize target was a\n<a href=\"#!/api/Ext.form.field.Field\" rel=\"Ext.form.field.Field\" class=\"docClass\">Field</a>, or an IMG or a TEXTAREA which must be wrapped in a DIV.</p>\n"},"overCls":{"!type":"string"},"pinnedCls":{"!type":"string"},"resizeTracker":{},"wrapCls":{"!type":"string"},"wrappedCls":{"!type":"string"},"destroy":{"!type":"fn() -> !this"},"disable":{"!type":"fn() -> !this"},"enable":{"!type":"fn() -> !this"},"fireResizeEvent":{"!type":"fn(name: ?, The: ?, The: ?) -> !this","!doc":"<p>Fire a resize event, checking if we have listeners before firing.</p>\n"},"getEl":{"!type":"fn() -> ?","!doc":"<p>Returns the element that was configured with the el or target config property. If a component was configured with\nthe target property then this will return the element of this component.</p>\n\n<p>Textarea and img elements will be wrapped with an additional div because these elements do not support child\nnodes. The original element can be accessed through the originalTarget property.</p>\n"},"getTarget":{"!type":"fn() -> ?","!doc":"<p>Returns the element or component that was configured with the target config property.</p>\n\n<p>Textarea and img elements will be wrapped with an additional div because these elements do not support child\nnodes. The original element can be accessed through the originalTarget property.</p>\n"},"onBeforeResize":{"!type":"fn(The: ?, The: ?) -> !this","!doc":"<p>Relay the Tracker's mousedown event as beforeresize</p>\n"},"onResize":{"!type":"fn(The: ?, The: ?) -> !this","!doc":"<p>Relay the Tracker's drag event as resizedrag</p>\n"},"onResizeEnd":{"!type":"fn(The: ?, The: ?) -> !this","!doc":"<p>Relay the Tracker's dragend event as resize</p>\n"},"resizeTo":{"!type":"fn(width: ?, height: ?) -> !this","!doc":"<p>Perform a manual resize and fires the 'resize' event.</p>\n"},"beforeresize":{"!type":"fn(this: ?, width: ?, height: ?, e: ?, eOpts: ?)","!doc":"<p>Fired before resize is allowed. Return false to cancel resize.</p>\n"},"resize":{"!type":"fn(this: ?, width: ?, height: ?, e: ?, eOpts: ?)","!doc":"<p>Fired after a resize.</p>\n"},"resizedrag":{"!type":"fn(this: ?, width: ?, height: ?, e: ?, eOpts: ?)","!doc":"<p>Fires during resizing.</p>\n"}}},"ResizeTracker":{"!doc":"<p>Private utility class for <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Ext.resizer.Resizer</a>.</p>\n","!type":"fn(config: ?)","prototype":{"constrainTo":{"!doc":"<p>Default to no constraint</p>\n"},"dynamic":{"!type":"bool"},"preserveRatio":{"!type":"bool"},"proxyCls":{"!type":"string"},"convertRegionName":{"!type":"fn(name: ?) -> !this"},"createProxy":{"!type":"fn(target: ?) -> ?","!doc":"<p>Create a proxy for this resizer</p>\n"},"getProxy":{"!type":"fn() -> !this","!doc":"<p>Returns the object that will be resized instead of the true target on every mousemove event.\nIf dynamic is false, this will be a proxy, otherwise it will be null target.</p>\n"},"onBeforeStart":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method which should be overridden by each DragTracker instance. Called when the user first clicks and\nholds the mouse button down. Return false to disallow the drag</p>\n"},"onDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method which should be overridden by each DragTracker instance. Called whenever a drag has been detected.</p>\n"},"onEnd":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method which should be overridden by each DragTracker instance. Called when a drag operation has been completed\n(e.g. the user clicked and held the mouse down, dragged the element and then released the mouse button)</p>\n"},"onStart":{"!type":"fn(e: ?) -> !this","!doc":"<p>Template method which should be overridden by each DragTracker instance. Called when a drag operation starts\n(e.g. the user has moved the tracked element beyond the specified tolerance)</p>\n"},"resize":{"!type":"fn(box: ?, atEnd: ?) -> !this"},"updateDimensions":{"!type":"fn(e: ?, atEnd: ?) -> !this"}}}},"LoadMask":{"!doc":"<p>A modal, floating Component which may be shown above a specified <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a> while loading data.\nWhen shown, the configured owning Component will be covered with a modality mask, and the LoadMask's <a href=\"#!/api/Ext.LoadMask-cfg-msg\" rel=\"Ext.LoadMask-cfg-msg\" class=\"docClass\">msg</a> will be\ndisplayed centered, accompanied by a spinner image.</p>\n\n<p>If the <a href=\"#!/api/Ext.LoadMask-cfg-store\" rel=\"Ext.LoadMask-cfg-store\" class=\"docClass\">store</a> config option is specified, the masking will be automatically shown and then hidden synchronized with\nthe Store's loading process.</p>\n\n<p>Because this is a floating Component, its z-index will be managed by the global <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a>\nobject, and upon show, it will place itsef at the top of the hierarchy.</p>\n\n<p>Example usage:</p>\n\n<pre class='inline-example '><code>var myPanel = new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>({\n    renderTo : document.body,\n    height   : 100,\n    width    : 200,\n    title    : 'Foo'\n});\n\nvar myMask = new <a href=\"#!/api/Ext.LoadMask\" rel=\"Ext.LoadMask\" class=\"docClass\">Ext.LoadMask</a>({\n    msg    : 'Please wait...',\n    target : myPanel\n});\n\nmyMask.show();\n</code></pre>\n","!type":"fn(config?: ?)","prototype":{"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"cls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element. This can be useful\nfor adding customized styles to the component or any of its children using standard CSS rules.</p>\n"},"maskCls":{"!type":"string","!doc":"<p>The CSS class to apply to the mask element</p>\n"},"msg":{"!type":"string","!doc":"<p>The text to display in a centered loading message box.</p>\n"},"msgCls":{"!type":"string","!doc":"<p>The CSS class to apply to the loading message element.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.LoadMask-method-getEl\" rel=\"Ext.LoadMask-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.LoadMask-method-getEl\" rel=\"Ext.LoadMask-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.LoadMask-cfg-renderSelectors\" rel=\"Ext.LoadMask-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.LoadMask-cfg-childEls\" rel=\"Ext.LoadMask-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"store":{"!doc":"<p>Optional Store to which the mask is bound. The mask is displayed when a load request is issued, and\nhidden on either load success, or load fail.</p>\n"},"target":{"!doc":"<p>The Component you wish to mask. The the mask will be automatically sized\nupon Component resize, and the message box will be kept centered.</p>\n"},"useMsg":{"!type":"bool","!doc":"<p>Whether or not to use a loading message class or simply mask the bound element.</p>\n"},"useTargetEl":{"!type":"bool","!doc":"<p>True to mask the <a href=\"#!/api/Ext.Component-method-getTargetEl\" rel=\"Ext.Component-method-getTargetEl\" class=\"docClass\">targetEl</a> of the bound Component. By default,\nthe <a href=\"#!/api/Ext.Component-method-getEl\" rel=\"Ext.Component-method-getEl\" class=\"docClass\">el</a> will be masked.</p>\n"},"ariaRole":{"!type":"string"},"isLoadMask":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated LoadMask, or subclass thereof.</p>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"afterShow":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Component is shown (after <a href=\"#!/api/Ext.LoadMask-method-onShow\" rel=\"Ext.LoadMask-method-onShow\" class=\"docClass\">onShow</a> is called).</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.LoadMask-event-show\" rel=\"Ext.LoadMask-event-show\" class=\"docClass\">show</a>.</p>\n"},"bindComponent":{"!type":"fn(comp: ?) -> !this"},"bindStore":{"!type":"fn(store: ?) -> !this","!doc":"<p>Changes the data store bound to this LoadMask.</p>\n"},"getMaskTarget":{"!type":"fn() -> !this","!doc":"<p>Returns the element which is masked by the <a href=\"#!/api/Ext.LoadMask-method-mask\" rel=\"Ext.LoadMask-method-mask\" class=\"docClass\">mask</a> method, or into which the <a href=\"#!/api/Ext.LoadMask-method-setLoading\" rel=\"Ext.LoadMask-method-setLoading\" class=\"docClass\">LoadMask</a> is rendered into.</p>\n\n<p>The default implementation uses the <a href=\"#!/api/Ext.LoadMask-cfg-maskElement\" rel=\"Ext.LoadMask-cfg-maskElement\" class=\"docClass\">maskElement</a> configuration to access the Component's child element by name. By default, <a href=\"#!/api/Ext.LoadMask-cfg-maskElement\" rel=\"Ext.LoadMask-cfg-maskElement\" class=\"docClass\">maskElement</a>\nis <code>null</code> which means that <code>null</code> is returned from this method indicating that the mask needs to be rendered into the document because\ncomponent structure should not be contaminated by mask elements.</p>\n\n<p>Some subclasses may override this method if they have knowledge about external structures where a mask could usefully be rendered.</p>\n\n<p>For example a <a href=\"#!/api/Ext.view.Table\" rel=\"Ext.view.Table\" class=\"docClass\">GridView</a> will request that its owning <a href=\"#!/api/Ext.panel.Table\" rel=\"Ext.panel.Table\" class=\"docClass\">GridPanel</a> be masked. The\nGridPanel will have its own implementation of <code>getMaskTarget</code> which will return the element dicated by its own <a href=\"#!/api/Ext.LoadMask-cfg-maskElement\" rel=\"Ext.LoadMask-cfg-maskElement\" class=\"docClass\">maskElement</a>\nPanels use <code>\"el\"</code> as their <a href=\"#!/api/Ext.LoadMask-cfg-maskElement\" rel=\"Ext.LoadMask-cfg-maskElement\" class=\"docClass\">maskElement</a> by default, but that could be overridden to be <code>\"body\"</code> to leave toolbars and the header\nmouse-accessible.</p>\n"},"getOwner":{"!type":"fn() -> !this"},"getRenderTree":{"!type":"fn() -> !this","!doc":"<p>A mask consists of 2 simultaneously shown/sized/positioned elements.</p>\n"},"getStoreListeners":{"!type":"fn(store: ?) -> ?","!doc":"<p>Gets the listeners to bind to a new store.</p>\n"},"hide":{"!type":"fn() -> ?","!doc":"<p>Hides this Component, setting it to invisible using the configured <a href=\"#!/api/Ext.LoadMask-cfg-hideMode\" rel=\"Ext.LoadMask-cfg-hideMode\" class=\"docClass\">hideMode</a>.</p>\n"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"maybeShow":{"!type":"fn() -> !this"},"onBeforeLoad":{"!type":"fn() -> !this"},"onComponentAdded":{"!type":"fn(owner: ?) -> !this"},"onComponentHide":{"!type":"fn() -> !this"},"onComponentRemoved":{"!type":"fn(owner: ?) -> !this"},"onComponentShow":{"!type":"fn() -> !this"},"onContainerCollapse":{"!type":"fn(container: ?) -> !this","!doc":"<p>Only called if wer are rendered external to the target.\nBest we can do is hide.</p>\n"},"onContainerExpand":{"!type":"fn(container: ?) -> !this","!doc":"<p>Only called if wer are rendered external to the target.\nBest we can do is show.</p>\n"},"onContainerHide":{"!type":"fn(container: ?) -> !this","!doc":"<p>Only called if wer are rendered external to the target.\nBest we can do is hide.</p>\n"},"onContainerShow":{"!type":"fn(container: ?) -> !this","!doc":"<p>Only called if wer are rendered external to the target.\nBest we can do is show.</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onDisable":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the disable operation.\nAfter calling the superclass's <code>onDisable</code>, the Component will be disabled.</p>\n"},"onLoad":{"!type":"fn() -> !this"},"onOwnerToFront":{"!type":"fn(owner: ?, zIndex: ?) -> !this"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"show":{"!type":"fn() -> ?","!doc":"<p>Shows this Component, rendering it first if <a href=\"#!/api/Ext.LoadMask-cfg-autoRender\" rel=\"Ext.LoadMask-cfg-autoRender\" class=\"docClass\">autoRender</a> or <a href=\"#!/api/Ext.LoadMask-cfg-floating\" rel=\"Ext.LoadMask-cfg-floating\" class=\"docClass\">floating</a> are <code>true</code>.</p>\n\n<p>After being shown, a <a href=\"#!/api/Ext.LoadMask-cfg-floating\" rel=\"Ext.LoadMask-cfg-floating\" class=\"docClass\">floating</a> Component (such as a <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>), is activated it and\nbrought to the front of its <a href=\"#!/api/Ext.LoadMask-property-zIndexManager\" rel=\"Ext.LoadMask-property-zIndexManager\" class=\"docClass\">z-index stack</a>.</p>\n"},"sizeMask":{"!type":"fn() -> !this","!doc":"<p>Called when this LoadMask's Component is resized. The toFront method rebases and resizes the modal mask.</p>\n"}}},"menu":{"CheckItem":{"!doc":"<p>A menu item that contains a togglable checkbox by default, but that can also be a part of a radio group.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a>', {\n    width: 100,\n    height: 110,\n    floating: false,  // usually you want this set to True (default)\n    renderTo: Ext.getBody(),  // usually rendered by it's containing component\n    items: [{\n        xtype: 'menucheckitem',\n        text: 'select all'\n    },{\n        xtype: 'menucheckitem',\n        text: 'select specific'\n    },{\n        iconCls: 'add16',\n        text: 'icon item'\n    },{\n        text: 'regular item'\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"ariaRole":{"!type":"string"},"checkChangeDisabled":{"!type":"bool","!doc":"<p>True to prevent the checked item from being toggled. Any submenu will still be accessible.</p>\n"},"checkHandler":{"!doc":"<p>Alternative for the <a href=\"#!/api/Ext.menu.CheckItem-event-checkchange\" rel=\"Ext.menu.CheckItem-event-checkchange\" class=\"docClass\">checkchange</a> event.  Gets called with the same parameters.</p>\n"},"checked":{"!type":"bool","!doc":"<p>True to render the menuitem initially checked.</p>\n"},"checkedCls":{"!type":"string","!doc":"<p>The CSS class used by <a href=\"#!/api/Ext.menu.CheckItem-cfg-cls\" rel=\"Ext.menu.CheckItem-cfg-cls\" class=\"docClass\">cls</a> to show the checked state.\nDefaults to <code>Ext.baseCSSPrefix + 'menu-item-checked'</code>.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"group":{"!type":"string","!doc":"<p>Name of a radio group that the item belongs.</p>\n\n<p>Specifying this option will turn check item into a radio item.</p>\n\n<p>Note that the group name must be globally unique.</p>\n"},"groupCls":{"!type":"string","!doc":"<p>The CSS class applied to this item's icon image to denote being a part of a radio group.\nDefaults to <code>Ext.baseCSSClass + 'menu-group-icon'</code>.\nAny specified <a href=\"#!/api/Ext.menu.CheckItem-cfg-iconCls\" rel=\"Ext.menu.CheckItem-cfg-iconCls\" class=\"docClass\">iconCls</a> overrides this.</p>\n"},"hideOnClick":{"!type":"bool","!doc":"<p>Whether to not to hide the owning menu when this item is clicked.\nDefaults to <code>false</code> for checkbox items, and to <code>true</code> for radio group items.</p>\n"},"scope":{"!doc":"<p>Scope for the <a href=\"#!/api/Ext.menu.CheckItem-cfg-checkHandler\" rel=\"Ext.menu.CheckItem-cfg-checkHandler\" class=\"docClass\">checkHandler</a> callback.</p>\n"},"uncheckedCls":{"!type":"string","!doc":"<p>The CSS class used by <a href=\"#!/api/Ext.menu.CheckItem-cfg-cls\" rel=\"Ext.menu.CheckItem-cfg-cls\" class=\"docClass\">cls</a> to show the unchecked state.\nDefaults to <code>Ext.baseCSSPrefix + 'menu-item-unchecked'</code>.</p>\n"},"checkboxCls":{"!type":"string"},"isMenuCheckItem":{"!type":"bool"},"showCheckbox":{"!type":"bool"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"disableCheckChange":{"!type":"fn() -> !this","!doc":"<p>Disables just the checkbox functionality of this menu Item. If this menu item has a submenu, that submenu\nwill still be accessible</p>\n"},"enableCheckChange":{"!type":"fn() -> !this","!doc":"<p>Reenables the checkbox functionality of this menu item after having been disabled by <a href=\"#!/api/Ext.menu.CheckItem-method-disableCheckChange\" rel=\"Ext.menu.CheckItem-method-disableCheckChange\" class=\"docClass\">disableCheckChange</a></p>\n"},"onClick":{"!type":"fn(e: ?) -> !this"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"setChecked":{"!type":"fn(checked: ?, suppressEvents?: ?) -> !this","!doc":"<p>Sets the checked state of the item</p>\n"},"beforecheckchange":{"!type":"fn(this: ?, checked: ?, eOpts: ?)","!doc":"<p>Fires before a change event. Return false to cancel.</p>\n"},"checkchange":{"!type":"fn(this: ?, checked: ?, eOpts: ?)","!doc":"<p>Fires after a change event.</p>\n"}}},"Item":{"prototype":{"activeCls":{"!type":"string","!doc":"<p>The CSS class added to the menu item when the item is activated (focused/mouseover).</p>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"canActivate":{"!type":"bool","!doc":"<p>Whether or not this menu item can be activated when focused/mouseovered.</p>\n"},"clickHideDelay":{"!type":"number","!doc":"<p>The delay in milliseconds to wait before hiding the menu after clicking the menu item.\nThis only has an effect when <code>hideOnClick: true</code>.</p>\n"},"destroyMenu":{"!type":"bool","!doc":"<p>Whether or not to destroy any associated sub-menu when this item is destroyed.</p>\n"},"disabledCls":{"!type":"string","!doc":"<p>The CSS class added to the menu item when the item is disabled.</p>\n"},"glyph":{"!doc":"<p>A numeric unicode character code to use as the icon for this item. The default\nfont-family for glyphs can be set globally using\n<a href=\"#!/api/Ext-method-setGlyphFontFamily\" rel=\"Ext-method-setGlyphFontFamily\" class=\"docClass\">Ext.setGlyphFontFamily()</a>. Alternatively, this\nconfig option accepts a string with the charCode and font-family separated by the\n<code>@</code> symbol. For example '65@My Font Family'.</p>\n"},"handler":{"!doc":"<p>A function called when the menu item is clicked (can be used instead of <a href=\"#!/api/Ext.menu.Item-event-click\" rel=\"Ext.menu.Item-event-click\" class=\"docClass\">click</a> event).</p>\n"},"href":{"!type":"string","!doc":"<p>The href attribute to use for the underlying anchor link.</p>\n"},"hrefTarget":{"!type":"string","!doc":"<p>The target attribute to use for the underlying anchor link.</p>\n"},"icon":{"!type":"string","!doc":"<p>The path to an icon to display in this item.</p>\n\n<p>There are no default icons that come with Ext JS.</p>\n\n<p>Defaults to <code>Ext.BLANK_IMAGE_URL</code>.</p>\n"},"iconCls":{"!type":"string","!doc":"<p>A CSS class that specifies a <code>background-image</code> to use as the icon for this item.</p>\n\n<p>There are no default icon classes that come with Ext JS.</p>\n"},"menu":{"!doc":"<p>The sub-menu associated with this item, if one was configured.</p>\n"},"menuAlign":{"!type":"string","!doc":"<p>The default Ext.util.Positionable.getAlignToXY anchor position value for this\nitem's sub-menu relative to this item's position.</p>\n"},"menuExpandDelay":{"!type":"number","!doc":"<p>The delay in milliseconds before this item's sub-menu expands after this item is moused over.</p>\n"},"menuHideDelay":{"!type":"number","!doc":"<p>The delay in milliseconds before this item's sub-menu hides after this item is moused out.</p>\n"},"plain":{"!type":"bool","!doc":"<p>Whether or not this item is plain text/html with no icon or visual activation.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.menu.Item-method-getEl\" rel=\"Ext.menu.Item-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.menu.Item-method-getEl\" rel=\"Ext.menu.Item-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.menu.Item-cfg-renderSelectors\" rel=\"Ext.menu.Item-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.menu.Item-cfg-childEls\" rel=\"Ext.menu.Item-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"text":{"!type":"string","!doc":"<p>The text/html to display in this item.</p>\n"},"tooltip":{"!doc":"<p>The tooltip for the button - can be a string to be used as innerHTML (html tags are accepted) or\nQuickTips config object.</p>\n"},"tooltipType":{"!type":"string","!doc":"<p>The type of tooltip to use. Either 'qtip' for QuickTips or 'title' for title attribute.</p>\n"},"activated":{"!type":"bool","!doc":"<p>Whether or not this item is currently activated</p>\n"},"arrowCls":{"!type":"string"},"baseIconCls":{"!type":"string"},"iconAlign":{"!type":"string"},"indentCls":{"!type":"string"},"indentNoSeparatorCls":{"!type":"string"},"indentRightArrowCls":{"!type":"string"},"indentRightIconCls":{"!type":"string"},"isMenuItem":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Menu Item, or subclass thereof.</p>\n"},"linkCls":{"!type":"string"},"linkHrefCls":{"!type":"string"},"maskOnDisable":{"!type":"bool","!doc":"<p>This is an internal flag that you use when creating custom components. By default this is set to <code>true</code> which means\nthat every component gets a mask when it's disabled. Components like FieldContainer, FieldSet, Field, Button, Tab\noverride this property to <code>false</code> since they want to implement custom disable logic.</p>\n"},"parentMenu":{"!doc":"<p>The parent Menu of this item.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"textCls":{"!type":"string"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"cancelDeferHide":{"!type":"fn() -> !this"},"clearTip":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"deactivate":{"!type":"fn(item: ?, eOpts: ?)","!doc":"<p>Fires when this tiem is deactivated</p>\n"},"deferHideMenu":{"!type":"fn() -> !this"},"deferHideParentMenus":{"!type":"fn() -> !this"},"doExpandMenu":{"!type":"fn() -> !this"},"expandMenu":{"!type":"fn(delay: ?) -> !this"},"getRefItems":{"!type":"fn(deep: ?) -> !this"},"getTipAttr":{"!type":"fn() -> !this"},"hideMenu":{"!type":"fn(delay: ?) -> !this"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"onRemoved":{"!type":"fn() -> !this","!doc":"<p>Method to manage awareness of when components are removed from their\nrespective Container, firing a <a href=\"#!/api/Ext.menu.Item-event-removed\" rel=\"Ext.menu.Item-event-removed\" class=\"docClass\">removed</a> event. References are properly\ncleaned up after removing a component from its owning container.</p>\n\n<p>Allows addition of behavior when a Component is removed from\nits parent Container. At this stage, the Component has been\nremoved from its parent Container's collection of child items,\nbut has not been destroyed (It will be destroyed if the parent\nContainer's <code>autoDestroy</code> is <code>true</code>, or if the remove call was\npassed a truthy second parameter). After calling the\nsuperclass's <code>onRemoved</code>, the <code>ownerCt</code> and the <code>refOwner</code> will not\nbe present.</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"setHandler":{"!type":"fn(fn: ?, scope?: ?) -> !this","!doc":"<p>Sets the <a href=\"#!/api/Ext.menu.Item-event-click\" rel=\"Ext.menu.Item-event-click\" class=\"docClass\">click</a> handler of this item</p>\n"},"setIcon":{"!type":"fn(icon: ?) -> !this","!doc":"<p>Sets the <a href=\"#!/api/Ext.menu.Item-cfg-icon\" rel=\"Ext.menu.Item-cfg-icon\" class=\"docClass\">icon</a> on this item.</p>\n"},"setIconCls":{"!type":"fn(iconCls: ?) -> !this","!doc":"<p>Sets the <a href=\"#!/api/Ext.menu.Item-cfg-iconCls\" rel=\"Ext.menu.Item-cfg-iconCls\" class=\"docClass\">iconCls</a> of this item</p>\n"},"setMenu":{"!type":"fn(menu: ?, destroyMenu?: ?) -> !this","!doc":"<p>Set a child menu for this item. See the <a href=\"#!/api/Ext.menu.Item-cfg-menu\" rel=\"Ext.menu.Item-cfg-menu\" class=\"docClass\">menu</a> configuration.</p>\n"},"setText":{"!type":"fn(text: ?) -> !this","!doc":"<p>Sets the <a href=\"#!/api/Ext.menu.Item-cfg-text\" rel=\"Ext.menu.Item-cfg-text\" class=\"docClass\">text</a> of this item</p>\n"},"setTooltip":{"!type":"fn(tooltip: ?) -> ?","!doc":"<p>Sets the tooltip for this menu item.</p>\n"},"activate":{"!type":"fn(item: ?, eOpts: ?)","!doc":"<p>Fires when this item is activated</p>\n"},"click":{"!type":"fn(item: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when this item is clicked</p>\n"},"iconchange":{"!type":"fn(this: ?, oldIcon: ?, newIcon: ?, eOpts: ?)","!doc":"<p>Fired when the item's icon is changed by the <a href=\"#!/api/Ext.menu.Item-method-setIcon\" rel=\"Ext.menu.Item-method-setIcon\" class=\"docClass\">setIcon</a> or <a href=\"#!/api/Ext.menu.Item-method-setIconCls\" rel=\"Ext.menu.Item-method-setIconCls\" class=\"docClass\">setIconCls</a> methods.</p>\n"},"textchange":{"!type":"fn(this: ?, oldText: ?, newText: ?, eOpts: ?)","!doc":"<p>Fired when the item's text is changed by the <a href=\"#!/api/Ext.menu.Item-method-setText\" rel=\"Ext.menu.Item-method-setText\" class=\"docClass\">setText</a> method.</p>\n"},"ariaRole":{"!type":"string"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"hideOnClick":{"!type":"bool","!doc":"<p>Whether to not to hide the owning menu when this item is clicked.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"onClick":{"!type":"fn(e: ?) -> !this"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"}},"!doc":"<p>A base class for all menu items that require menu-related functionality such as click handling,\nsub-menus, icons, etc.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a>', {\n    width: 100,\n    height: 100,\n    floating: false,  // usually you want this set to True (default)\n    renderTo: Ext.getBody(),  // usually rendered by it's containing component\n    items: [{\n        text: 'icon item',\n        iconCls: 'add16'\n    },{\n        text: 'text item'\n    },{\n        text: 'plain item',\n        plain: true\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)"},"ColorPicker":{"!doc":"<p>A menu containing a <a href=\"#!/api/Ext.picker.Color\" rel=\"Ext.picker.Color\" class=\"docClass\">Ext.picker.Color</a> Component.</p>\n\n<p>Notes:</p>\n\n<ul>\n<li>Although not listed here, the <strong>constructor</strong> for this class accepts all of the\nconfiguration options of <a href=\"#!/api/Ext.picker.Color\" rel=\"Ext.picker.Color\" class=\"docClass\">Ext.picker.Color</a>.</li>\n<li>If subclassing ColorMenu, any configuration options for the ColorPicker must be\napplied to the <strong>initialConfig</strong> property of the ColorMenu. Applying\n<a href=\"#!/api/Ext.picker.Color\" rel=\"Ext.picker.Color\" class=\"docClass\">ColorPicker</a> configuration settings to <code>this</code> will <strong>not</strong>\naffect the ColorPicker's configuration.</li>\n</ul>\n\n\n<p>Example:</p>\n\n<pre class='inline-example '><code>var colorPicker = Ext.create('<a href=\"#!/api/Ext.menu.ColorPicker\" rel=\"Ext.menu.ColorPicker\" class=\"docClass\">Ext.menu.ColorPicker</a>', {\n    value: '000000'\n});\n\nExt.create('<a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a>', {\n    items: [{\n        text: 'Choose a color',\n        menu: colorPicker\n    },{\n        iconCls: 'add16',\n        text: 'Icon item'\n    },{\n        text: 'Regular item'\n    }]\n}).showAt([5, 5]);\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"hideOnClick":{"!type":"bool","!doc":"<p>False to continue showing the menu after a color is selected.</p>\n"},"maxHeight":{"!type":"number"},"pickerId":{"!type":"string","!doc":"<p>An id to assign to the underlying color picker.</p>\n"},"picker":{"!doc":"<p>The <a href=\"#!/api/Ext.picker.Color\" rel=\"Ext.picker.Color\" class=\"docClass\">Ext.picker.Color</a> instance for this ColorMenu</p>\n"},"hidePickerOnSelect":{"!type":"fn() -> !this","!doc":"<p>Hides picker on select if hideOnClick is true</p>\n"},"click":{"!type":"fn(eOpts: ?)"},"select":{"!type":"fn(this: ?, color: ?, eOpts: ?)","!doc":"<p>Fires when a color is selected</p>\n"}}},"Menu":{"prototype":{"allowOtherMenus":{"!type":"bool","!doc":"<p>True to allow multiple menus to be displayed at the same time.</p>\n"},"ariaRole":{"!type":"string"},"autoRender":{"!type":"bool","!doc":"<p>Floating is true, so autoRender always happens.</p>\n"},"baseCls":{"!type":"string","!doc":"<p>private</p>\n"},"constrain":{"!type":"bool","!doc":"<p>Menus are constrained to the document body by default.</p>\n"},"defaultAlign":{"!type":"string","!doc":"<p>The default Ext.dom.Element#getAlignToXY anchor position value for this menu\nrelative to its owner. Used in conjunction with <a href=\"#!/api/Ext.menu.Menu-method-showBy\" rel=\"Ext.menu.Menu-method-showBy\" class=\"docClass\">showBy</a>.</p>\n"},"enableKeyNav":{"!type":"bool","!doc":"<p>True to enable keyboard navigation for controlling the menu.\nThis option should generally be disabled when form fields are\nbeing used inside the menu.</p>\n"},"floating":{"!type":"bool","!doc":"<p>A Menu configured as <code>floating: true</code> (the default) will be rendered as an absolutely positioned,\n<a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a> <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a>. If configured as <code>floating: false</code>, the Menu may be\nused as a child item of another <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a>.</p>\n"},"hidden":{"!type":"bool","!doc":"<p>True to initially render the Menu as hidden, requiring to be shown manually.</p>\n\n<p>Defaults to <code>true</code> when <code>floating: true</code>, and defaults to <code>false</code> when <code>floating: false</code>.</p>\n"},"hideMode":{"!type":"string","!doc":"<p>A String which specifies how this Component's encapsulating DOM element will be hidden. Values may be:</p>\n\n<ul>\n<li><code>'display'</code> : The Component will be hidden using the <code>display: none</code> style.</li>\n<li><code>'visibility'</code> : The Component will be hidden using the <code>visibility: hidden</code> style.</li>\n<li><code>'offsets'</code> : The Component will be hidden by absolutely positioning it out of the visible area of the document.\nThis is useful when a hidden Component must maintain measurable dimensions. Hiding using <code>display</code> results in a\nComponent having zero dimensions.</li>\n</ul>\n\n"},"ignoreParentClicks":{"!type":"bool","!doc":"<p>True to ignore clicks on any item in this menu that is a parent item (displays a submenu)\nso that the submenu is not dismissed when clicking the parent item.</p>\n"},"layout":{},"minWidth":{"!type":"number","!doc":"<p>The minimum width of the Menu. The default minWidth only applies when the <a href=\"#!/api/Ext.menu.Menu-cfg-floating\" rel=\"Ext.menu.Menu-cfg-floating\" class=\"docClass\">floating</a> config is true.</p>\n"},"plain":{"!type":"bool","!doc":"<p>True to remove the incised line down the left side of the menu and to not indent general Component items.</p>\n\n<p><a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">MenuItem</a>s will <em>always</em> have space at their start for an icon. With the <code>plain</code> setting,\nnon <a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">MenuItem</a> child components will not be indented to line up.</p>\n\n<p>Basically, <code>plain:true</code> makes a Menu behave more like a regular <a href=\"#!/api/Ext.layout.container.HBox\" rel=\"Ext.layout.container.HBox\" class=\"docClass\">HBox layout</a>\n<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Panel</a> which just has the same background as a Menu.</p>\n\n<p>See also the <a href=\"#!/api/Ext.menu.Menu-cfg-showSeparator\" rel=\"Ext.menu.Menu-cfg-showSeparator\" class=\"docClass\">showSeparator</a> config.</p>\n"},"showSeparator":{"!type":"bool","!doc":"<p>True to show the icon separator.</p>\n"},"_iconSeparatorCls":{"!type":"string"},"_itemCmpCls":{"!type":"string"},"defaultMinWidth":{"!type":"number"},"isMenu":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Menu, or subclass thereof.</p>\n"},"parentMenu":{"!doc":"<p>The parent Menu of this Menu.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"afterShow":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Component is shown (after <a href=\"#!/api/Ext.menu.Menu-method-onShow\" rel=\"Ext.menu.Menu-method-onShow\" class=\"docClass\">onShow</a> is called).</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.menu.Menu-event-show\" rel=\"Ext.menu.Menu-event-show\" class=\"docClass\">show</a>.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"beforeShow":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is shown.</p>\n"},"canActivateItem":{"!type":"fn(item: ?) -> bool","!doc":"<p>Returns whether a menu item can be activated or not.</p>\n"},"configureItem":{"!type":"fn(cmp: ?) -> !this","!doc":"<p>Override applied to the Menu's layout. Runs in the context of the layout.\nAdd special classes to allow non MenuItem components to coexist with MenuItems.\nIf there is only <em>one</em> child, then this Menu is just a vehicle for floating\nand aligning the component, so do not do this.</p>\n"},"deactivateActiveItem":{"!type":"fn(andBlurFocusedItem: ?) -> !this","!doc":"<p>Deactivates the current active item on the menu, if one exists.</p>\n"},"getInherited":{"!type":"fn() -> !this","!doc":"<p>As menus are never contained, a Menu's visibility only ever depends upon its own hidden state.\nIgnore hiddenness from the ancestor hierarchy, override it with local hidden state.</p>\n"},"getItemFromEvent":{"!type":"fn(e: ?) -> !this"},"getRefOwner":{"!type":"fn() -> !this"},"hide":{"!type":"fn() -> ?","!doc":"<p>Hides this Component, setting it to invisible using the configured <a href=\"#!/api/Ext.menu.Menu-cfg-hideMode\" rel=\"Ext.menu.Menu-cfg-hideMode\" class=\"docClass\">hideMode</a>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initHierarchyEvents":{"!type":"fn() -> !this","!doc":"<p>Menus do not have owning containers on which they depend for visibility. They stand outside\nany container hierarchy.</p>\n"},"lookupComponent":{"!type":"fn(cmp: ?) -> !this"},"lookupItemFromObject":{"!type":"fn(cmp: ?) -> !this"},"lookupItemFromString":{"!type":"fn(cmp: ?) -> !this"},"onBoxReady":{"!type":"fn() -> !this","!doc":"<p>Invoked when this component has first achieved size. Occurs after the\n<a href=\"#!/api/Ext.menu.Menu-cfg-componentLayout\" rel=\"Ext.menu.Menu-cfg-componentLayout\" class=\"docClass\">componentLayout</a> has completed its initial run.</p>\n"},"onClick":{"!type":"fn(e: ?) -> !this"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onDocMouseDown":{"!type":"fn(e: ?) -> !this"},"onMouseLeave":{"!type":"fn(e: ?) -> !this"},"onMouseOver":{"!type":"fn(e: ?) -> !this"},"setActiveItem":{"!type":"fn(item: ?) -> ?","!doc":"<p>Sets a component as the active layout item. This only applies when using\na <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> layout.</p>\n\n<pre><code>var card1 = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {itemId: 'card-1'});\nvar card2 = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {itemId: 'card-2'});\nvar panel = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    layout: 'card',\n    items: [card1, card2]\n});\n// These are all equivalent\npanel.getLayout().setActiveItem(card2);\npanel.getLayout().setActiveItem('card-2');\npanel.getLayout().setActiveItem(1);\n</code></pre>\n"},"mouseenter":{"!type":"fn(menu: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse enters this menu</p>\n"},"mouseleave":{"!type":"fn(menu: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse leaves this menu</p>\n"},"mouseover":{"!type":"fn(menu: ?, item: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when the mouse is hovering over this menu</p>\n"},"click":{"!type":"fn(menu: ?, item: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when this menu is clicked</p>\n"}},"!doc":"<p>A menu object. This is the container to which you may add <a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">menu items</a>.</p>\n\n<p>Menus may contain either <a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">menu items</a>, or general <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Components</a>.\nMenus may also contain <a href=\"#!/api/Ext.panel.Panel-cfg-dockedItems\" rel=\"Ext.panel.Panel-cfg-dockedItems\" class=\"docClass\">docked items</a> because it extends <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>.</p>\n\n<p>By default, non <a href=\"#!/api/Ext.menu.Item\" rel=\"Ext.menu.Item\" class=\"docClass\">menu items</a> are indented so that they line up with the text of menu items. clearing\nthe icon column. To make a contained general <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Component</a> left aligned configure the child\nComponent with `indent: false.</p>\n\n<p>By default, Menus are absolutely positioned, floating Components. By configuring a Menu with <code><a href=\"#!/api/Ext.menu.Menu-cfg-floating\" rel=\"Ext.menu.Menu-cfg-floating\" class=\"docClass\">floating</a>: false</code>,\na Menu may be used as a child of a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a>.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a>', {\n    width: 100,\n    margin: '0 0 10 0',\n    floating: false,  // usually you want this set to True (default)\n    renderTo: Ext.getBody(),  // usually rendered by it's containing component\n    items: [{\n        text: 'regular item 1'\n    },{\n        text: 'regular item 2'\n    },{\n        text: 'regular item 3'\n    }]\n});\n\nExt.create('<a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a>', {\n    width: 100,\n    plain: true,\n    floating: false,  // usually you want this set to True (default)\n    renderTo: Ext.getBody(),  // usually rendered by it's containing component\n    items: [{\n        text: 'plain item 1'\n    },{\n        text: 'plain item 2'\n    },{\n        text: 'plain item 3'\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)"},"DatePicker":{"!doc":"<p>A menu containing an <a href=\"#!/api/Ext.picker.Date\" rel=\"Ext.picker.Date\" class=\"docClass\">Ext.picker.Date</a> Component.</p>\n\n<p>Notes:</p>\n\n<ul>\n<li>Although not listed here, the <strong>constructor</strong> for this class accepts all of the\nconfiguration options of <strong><a href=\"#!/api/Ext.picker.Date\" rel=\"Ext.picker.Date\" class=\"docClass\">Ext.picker.Date</a></strong>.</li>\n<li>If subclassing DateMenu, any configuration options for the DatePicker must be applied\nto the <strong>initialConfig</strong> property of the DateMenu. Applying <a href=\"#!/api/Ext.picker.Date\" rel=\"Ext.picker.Date\" class=\"docClass\">Date Picker</a>\nconfiguration settings to <strong>this</strong> will <strong>not</strong> affect the Date Picker's configuration.</li>\n</ul>\n\n\n<p>Example:</p>\n\n<pre class='inline-example '><code>var dateMenu = Ext.create('<a href=\"#!/api/Ext.menu.DatePicker\" rel=\"Ext.menu.DatePicker\" class=\"docClass\">Ext.menu.DatePicker</a>', {\n    handler: function(dp, date){\n        <a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Date Selected', 'You selected ' + Ext.Date.format(date, 'M j, Y'));\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a>', {\n    items: [{\n        text: 'Choose a date',\n        menu: dateMenu\n    },{\n        iconCls: 'add16',\n        text: 'Icon item'\n    },{\n        text: 'Regular item'\n    }]\n}).showAt([5, 5]);\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"hideOnClick":{"!type":"bool","!doc":"<p>False to continue showing the menu after a date is selected.</p>\n"},"maxHeight":{"!type":"number"},"pickerId":{"!type":"string","!doc":"<p>An id to assign to the underlying date picker.</p>\n"},"picker":{"!doc":"<p>The <a href=\"#!/api/Ext.picker.Date\" rel=\"Ext.picker.Date\" class=\"docClass\">Ext.picker.Date</a> instance for this DateMenu</p>\n"},"hidePickerOnSelect":{"!type":"fn() -> !this"},"select":{"!type":"fn(this: ?, date: ?, eOpts: ?)","!doc":"<p>Fires when a date is selected</p>\n"}}},"KeyNav":{"!type":"fn(config: ?)","prototype":{"down":{"!type":"fn(e: ?) -> !this"},"enter":{"!type":"fn(e: ?) -> !this"},"escape":{"!type":"fn(e: ?) -> !this"},"focusNextItem":{"!type":"fn(step: ?) -> !this"},"isWhitelisted":{"!type":"fn(item: ?) -> !this"},"left":{"!type":"fn(e: ?) -> !this"},"right":{"!type":"fn(e: ?) -> !this"},"tab":{"!type":"fn(e: ?) -> !this"},"up":{"!type":"fn(e: ?) -> !this"}}},"Manager":{"!doc":"<p>Provides a common registry of all menus on a page.</p>\n","attached":{"!type":"bool"},"groups":{},"lastShow":{},"menuSelector":{"!type":"string"},"menus":{},"get":{"!type":"fn(menu: ?) -> ?","!doc":"<p>Returns a <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a> object</p>\n"},"hideAll":{"!type":"fn() -> bool","!doc":"<p>Hides all menus that are currently visible</p>\n"},"init":{"!type":"fn() -> !this"},"onBeforeHide":{"!type":"fn(m: ?) -> !this"},"onBeforeShow":{"!type":"fn(m: ?) -> !this"},"onCheckChange":{"!type":"fn(menuItem: ?, state: ?) -> !this"},"onHide":{"!type":"fn(m: ?) -> !this"},"onMouseDown":{"!type":"fn(e: ?) -> !this"},"onShow":{"!type":"fn(m: ?) -> !this"},"register":{"!type":"fn(menu: ?) -> !this"},"registerCheckable":{"!type":"fn(menuItem: ?) -> !this"},"unregister":{"!type":"fn(menu: ?) -> !this"},"unregisterCheckable":{"!type":"fn(menuItem: ?) -> !this"}},"Separator":{"!doc":"<p>Adds a separator bar to a menu, used to divide logical groups of menu items. Generally you will\nadd one of these by using \"-\" in your call to add() or in your items config rather than creating one directly.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Ext.menu.Menu</a>', {\n    width: 100,\n    height: 100,\n    floating: false,  // usually you want this set to True (default)\n    renderTo: Ext.getBody(),  // usually rendered by it's containing component\n    items: [{\n        text: 'icon item',\n        iconCls: 'add16'\n    },{\n        xtype: 'menuseparator'\n    },{\n       text: 'separator above'\n    },{\n       text: 'regular item'\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"activeCls":{"!type":"string"},"ariaRole":{"!type":"string"},"canActivate":{"!type":"bool"},"clickHideDelay":{"!type":"bool"},"destroyMenu":{"!type":"bool"},"disabledCls":{"!type":"bool"},"hideOnClick":{"!type":"bool"},"href":{"!type":"string"},"hrefTarget":{"!type":"string"},"icon":{"!type":"string"},"iconCls":{"!type":"string"},"menu":{},"menuAlign":{"!type":"string"},"menuExpandDelay":{"!type":"number"},"menuHideDelay":{"!type":"number"},"plain":{"!type":"bool"},"separatorCls":{"!type":"string","!doc":"<p>The CSS class used by the separator item to show the incised line.</p>\n"},"text":{"!type":"string"},"beforeRender":{"!type":"fn(ct: ?, pos: ?) -> !this"}}}},"MessageBox":{"!doc":"<p>Utility class for generating different styles of message boxes.  The singleton instance, <a href=\"#!/api/Ext.MessageBox\" rel=\"Ext.MessageBox\" class=\"docClass\">Ext.MessageBox</a>\nalias <code><a href=\"#!/api/Ext.MessageBox\" rel=\"Ext.MessageBox\" class=\"docClass\">Ext.Msg</a></code> can also be used.</p>\n\n<p>Note that a MessageBox is asynchronous.  Unlike a regular JavaScript <code>alert</code> (which will halt\nbrowser execution), showing a MessageBox will not cause the code to stop.  For this reason, if you have code\nthat should only run <em>after</em> some user feedback from the MessageBox, you must use a callback function\n(see the <code>function</code> parameter for <a href=\"#!/api/Ext.MessageBox-method-show\" rel=\"Ext.MessageBox-method-show\" class=\"docClass\">show</a> for more details).</p>\n\n<p>Basic alert</p>\n\n<pre class='inline-example '><code><a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Status', 'Changes saved successfully.');\n</code></pre>\n\n<p>Prompt for user data and process the result using a callback</p>\n\n<pre class='inline-example '><code><a href=\"#!/api/Ext.MessageBox-method-prompt\" rel=\"Ext.MessageBox-method-prompt\" class=\"docClass\">Ext.Msg.prompt</a>('Name', 'Please enter your name:', function(btn, text){\n    if (btn == 'ok'){\n        // process text value and close...\n    }\n});\n</code></pre>\n\n<p>Show a dialog using config options</p>\n\n<pre class='inline-example '><code><a href=\"#!/api/Ext.MessageBox-event-show\" rel=\"Ext.MessageBox-event-show\" class=\"docClass\">Ext.Msg.show</a>({\n    title:'Save Changes?',\n    message: 'You are closing a tab that has unsaved changes. Would you like to save your changes?',\n    buttons: <a href=\"#!/api/Ext.MessageBox-property-YESNOCANCEL\" rel=\"Ext.MessageBox-property-YESNOCANCEL\" class=\"docClass\">Ext.Msg.YESNOCANCEL</a>,\n    icon: <a href=\"#!/api/Ext.MessageBox-property-QUESTION\" rel=\"Ext.MessageBox-property-QUESTION\" class=\"docClass\">Ext.Msg.QUESTION</a>,\n    fn: function(btn) {\n        if (btn === 'yes') {\n            console.log('Yes pressed');\n        } else if (btn === 'no') {\n            console.log('No pressed');\n        } else {\n            console.log('Cancel pressed');\n        } \n    }\n});\n</code></pre>\n","!type":"fn(cfg: ?)"},"window":{"Window":{"animateTarget":{"!doc":"<p>Id or element from which the window should animate while opening.</p>\n"},"autoRender":{"!type":"bool","!doc":"<p>Windows render to the body on first show.</p>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this panel's element.</p>\n"},"closable":{"!type":"bool","!doc":"<p>True to display the 'close' tool button and allow the user to close the window, false to hide the button and\ndisallow closing the window.</p>\n\n<p>By default, when close is requested by either clicking the close button in the header or pressing ESC when the\nWindow has focus, the <a href=\"#!/api/Ext.window.Window-method-close\" rel=\"Ext.window.Window-method-close\" class=\"docClass\">close</a> method will be called. This will <em><a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a></em> the\nWindow and its content meaning that it may not be reused.</p>\n\n<p>To make closing a Window <em>hide</em> the Window so that it may be reused, set <a href=\"#!/api/Ext.window.Window-cfg-closeAction\" rel=\"Ext.window.Window-cfg-closeAction\" class=\"docClass\">closeAction</a> to 'hide'.</p>\n"},"collapsed":{"!type":"bool","!doc":"<p>True to render the window collapsed, false to render it expanded. Note that if <a href=\"#!/api/Ext.window.Window-cfg-expandOnShow\" rel=\"Ext.window.Window-cfg-expandOnShow\" class=\"docClass\">expandOnShow</a>\nis true (the default) it will override the <code>collapsed</code> config and the window will always be\nexpanded when shown.</p>\n"},"collapsible":{"!type":"bool","!doc":"<p>inherited docs, same default</p>\n"},"constrainHeader":{"!type":"bool","!doc":"<p>True to constrain the window header within its containing element (allowing the window body to fall outside of\nits containing element) or false to allow the header to fall outside its containing element.\nOptionally the entire window can be constrained using <a href=\"#!/api/Ext.window.Window-cfg-constrain\" rel=\"Ext.window.Window-cfg-constrain\" class=\"docClass\">constrain</a>.</p>\n"},"defaultFocus":{"!doc":"<p>Specifies a Component to receive focus when this Window is focused.</p>\n\n<p>If a String is provided, the Component will be resolved using the down method which uses Ext.ComponentQuery.\nIf the string begins with an alphanumeric value, it will first attempt to find the Component based on the <a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">Ext.Component.id</a> or <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">Ext.Component.itemId</a>.\nIf a matching component is not found via id, then an attempt to do a query to find a matching component.</p>\n\n<p>An example of finding the Component with an id/itemId:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>', {\n    autoShow     : true,\n    width        : 300,\n    title        : 'Login',\n    defaultFocus : 'username',\n    items        : [\n        {\n            xtype      : 'textfield',\n            fieldLabel : 'Username',\n            itemId     : 'username',\n            name       : 'username'\n        },\n        {\n            xtype      : 'textfield',\n            inputType  : 'password',\n            fieldLabel : 'Password',\n            itemId     : 'password',\n            name       : 'password'\n        }\n    ]\n});\n</code></pre>\n\n<p>If a Number is provided, this will resolve an <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a> at that index. This is very useful if\nthe window has buttons in the <a href=\"#!/api/Ext.window.Window-cfg-buttons\" rel=\"Ext.window.Window-cfg-buttons\" class=\"docClass\">buttons</a> config and you want to provide default focus to one of them.</p>\n\n<p>An example of this would be:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>', {\n    autoShow     : true,\n    width        : 300,\n    title        : 'Login',\n    defaultFocus : 1,\n    items        : [\n        {\n            xtype      : 'textfield',\n            fieldLabel : 'Username',\n            name       : 'username'\n       },\n       {\n            xtype      : 'textfield',\n            inputType  : 'password',\n            fieldLabel : 'Password',\n            name       : 'password'\n        }\n    ],\n    buttons      : [\n        {\n            text : 'Cancel'\n        },\n        {\n            text : 'Login'\n        }\n    ]\n});\n</code></pre>\n\n<p>In summary, defaultFocus may be one of:</p>\n\n<ul>\n<li>The index of a footer Button.</li>\n<li>The id or <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">Ext.Component.itemId</a> of a descendant Component.</li>\n<li>A query to find a <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.</li>\n<li>A descendant <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.</li>\n</ul>\n\n"},"draggable":{"!type":"bool","!doc":"<p>True to allow the window to be dragged by the header bar, false to disable dragging. Note that\nby default the window will be centered in the viewport, so if dragging is disabled the window may need to be\npositioned programmatically after render (e.g., <code>myWindow.setPosition(100, 100);</code>).</p>\n"},"expandOnShow":{"!type":"bool","!doc":"<p>True to always expand the window when it is displayed, false to keep it in its current state (which may be\n<a href=\"#!/api/Ext.window.Window-cfg-collapsed\" rel=\"Ext.window.Window-cfg-collapsed\" class=\"docClass\">collapsed</a>) when displayed.</p>\n"},"ghost":{"!doc":"<p>Set to false to disable the ghost panel during dragging the window.\nDo note that you should not set this to true, by default it is a function.</p>\n"},"hidden":{"!type":"bool","!doc":"<p>Render this Window hidden. If <code>true</code>, the <a href=\"#!/api/Ext.window.Window-method-hide\" rel=\"Ext.window.Window-method-hide\" class=\"docClass\">hide</a> method will be called internally.</p>\n"},"hideShadowOnDeactivate":{"!type":"bool","!doc":"<p>True to hide this Window's shadow when another floating item in the same z-index stack is activated.</p>\n"},"maximizable":{"!type":"bool","!doc":"<p>True to display the 'maximize' tool button and allow the user to maximize the window, false to hide the button\nand disallow maximizing the window. Note that when a window is maximized, the tool button\nwill automatically change to a 'restore' button with the appropriate behavior already built-in that will restore\nthe window to its previous size.</p>\n"},"maximized":{"!type":"bool","!doc":"<p>True to initially display the window in a maximized state.</p>\n"},"minimizable":{"!type":"bool","!doc":"<p>True to display the 'minimize' tool button and allow the user to minimize the window, false to hide the button\nand disallow minimizing the window. Note that this button provides no implementation -- the\nbehavior of minimizing a window is implementation-specific, so the minimize event must be handled and a custom\nminimize behavior implemented for this option to be useful.</p>\n"},"modal":{"!type":"bool","!doc":"<p>True to make the window modal and mask everything behind it when displayed, false to display it without\nrestricting access to other UI elements.</p>\n"},"monitorResize":{"!type":"bool","!doc":"<p><code>true</code> to listen to the viewport resize event and perform any layout updating if necessary.\nThis is useful if using sizes as percentages for the window.</p>\n"},"onEsc":{"!doc":"<p>Allows override of the built-in processing for the escape key. Default action is to close the Window (performing\nwhatever action is specified in <a href=\"#!/api/Ext.window.Window-cfg-closeAction\" rel=\"Ext.window.Window-cfg-closeAction\" class=\"docClass\">closeAction</a>. To prevent the Window closing when the escape key is\npressed, specify this as Ext.emptyFn.</p>\n"},"overlapHeader":{"!type":"bool","!doc":"<p>True to overlap the header in a panel over the framing of the panel itself. This is needed when frame:true (and\nis done automatically for you). Otherwise it is undefined. If you manually add rounded corners to a panel header\nwhich does not have frame:true, this will need to be set to true.</p>\n"},"plain":{"!type":"bool","!doc":"<p>True to render the window body with a transparent background so that it will blend into the framing elements,\nfalse to add a lighter background color to visually highlight the body element and separate it more distinctly\nfrom the surrounding frame.</p>\n"},"x":{"!type":"number","!doc":"<p>The X position of the left edge of the window on initial showing. Defaults to centering the Window within the\nwidth of the Window's container Element (The Element that the Window is rendered to).</p>\n"},"y":{"!type":"number","!doc":"<p>The Y position of the top edge of the window on initial showing. Defaults to centering the Window within the\nheight of the Window's container Element (The Element that the Window is rendered to).</p>\n"},"alwaysFramed":{"!type":"bool","!doc":"<p>Flag to Renderable to always look up the framing styles for this Component</p>\n"},"dd":{"!doc":"<p>If this Window is configured <a href=\"#!/api/Ext.window.Window-cfg-draggable\" rel=\"Ext.window.Window-cfg-draggable\" class=\"docClass\">draggable</a>, this property will contain an instance of\n<a href=\"#!/api/Ext.util.ComponentDragger\" rel=\"Ext.util.ComponentDragger\" class=\"docClass\">Ext.util.ComponentDragger</a> (A subclass of <a href=\"#!/api/Ext.dd.DragTracker\" rel=\"Ext.dd.DragTracker\" class=\"docClass\">DragTracker</a>) which handles dragging\nthe Window's DOM Element, and constraining according to the <a href=\"#!/api/Ext.window.Window-cfg-constrain\" rel=\"Ext.window.Window-cfg-constrain\" class=\"docClass\">constrain</a> and <a href=\"#!/api/Ext.window.Window-cfg-constrainHeader\" rel=\"Ext.window.Window-cfg-constrainHeader\" class=\"docClass\">constrainHeader</a> .</p>\n\n<p>This has implementations of <code>onBeforeStart</code>, <code>onDrag</code> and <code>onEnd</code> which perform the dragging action. If\nextra logic is needed at these points, use createInterceptor or\ncreateSequence to augment the existing implementations.</p>\n"},"floating":{"!type":"bool","!doc":"<p>A Window is always floating.</p>\n"},"ignoreHeaderBorderManagement":{"!type":"bool"},"initialAlphaNum":{},"isRootCfg":{"!doc":"<p>Buffer this so we don't recreate the same object</p>\n"},"isWindow":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Window, or subclass thereof.</p>\n"},"itemCls":{"!type":"string"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"addTools":{"!type":"fn() -> !this","!doc":"<p>Contribute class-specific tools to the header.</p>\n\n<p>Called by Panel's initTools at initialization time.</p>\n\n<p>Implementations should jst add new tool config objects to <code>this.tools</code></p>\n"},"afterCollapse":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Panel is Collapsed.</p>\n"},"afterExpand":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Panel is Expanded.</p>\n"},"afterHide":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Component has been hidden.</p>\n\n<p>Gets passed the same <code>callback</code> and <code>scope</code> parameters that <a href=\"#!/api/Ext.window.Window-method-onHide\" rel=\"Ext.window.Window-method-onHide\" class=\"docClass\">onHide</a> received.</p>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"applyState":{"!type":"fn(state: ?) -> !this","!doc":"<p>Applies the state to the object. This should be overridden in subclasses to do\nmore complex state operations. By default it applies the state properties onto\nthe current object.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"createGhost":{"!type":"fn() -> !this"},"doClose":{"!type":"fn() -> !this"},"getDefaultFocus":{"!type":"fn() -> !this","!doc":"<p>Gets the configured default focus item.  If a <a href=\"#!/api/Ext.window.Window-cfg-defaultFocus\" rel=\"Ext.window.Window-cfg-defaultFocus\" class=\"docClass\">defaultFocus</a> is set, it will\nreceive focus when the Window's <code>focus</code> method is called, otherwise the\nWindow itself will receive focus.</p>\n"},"getElConfig":{"!type":"fn() -> !this"},"getFocusEl":{"!type":"fn() -> ?","!doc":"<p>Returns the focus holder element associated with this Window. By default, this is the Window's element.</p>\n"},"getState":{"!type":"fn() -> ?","!doc":"<p>The supplied default state gathering method for the Component class.</p>\n\n<p>This method returns dimension settings such as <code>flex</code>, <code>anchor</code>, <code>width</code> and <code>height</code> along with <code>collapsed</code>\nstate.</p>\n\n<p>Subclasses which implement more complex state should call the superclass's implementation, and apply their state\nto the result if this basic state is to be saved.</p>\n\n<p>Note that Component state will only be saved if the Component has a <a href=\"#!/api/Ext.window.Window-cfg-stateId\" rel=\"Ext.window.Window-cfg-stateId\" class=\"docClass\">stateId</a> and there as a StateProvider\nconfigured for the document.</p>\n"},"maximize":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the window has been maximized.</p>\n"},"minimize":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the window has been minimized.</p>\n"},"onFocus":{"!type":"fn() -> !this","!doc":"<p>Called when a Component's focusEl receives focus.\nIf there is a valid default focus Component to jump to, focus that,\notherwise continue as usual, focus this Component.</p>\n"},"onRender":{"!type":"fn(ct: ?, position: ?) -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"onWindowResize":{"!type":"fn() -> !this"},"restore":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the window has been restored to its original size after being maximized.</p>\n"},"resumeHeaderLayout":{"!type":"fn(changed: ?) -> !this"},"syncMonitorWindowResize":{"!type":"fn() -> !this","!doc":"<p>Synchronizes the presence of our listener for window resize events. This method\nshould be called whenever this status might change.</p>\n"},"toggleMaximize":{"!type":"fn() -> ?","!doc":"<p>A shortcut method for toggling between <a href=\"#!/api/Ext.window.Window-method-maximize\" rel=\"Ext.window.Window-method-maximize\" class=\"docClass\">maximize</a> and <a href=\"#!/api/Ext.window.Window-method-restore\" rel=\"Ext.window.Window-method-restore\" class=\"docClass\">restore</a> based on the current maximized\nstate of the window.</p>\n"},"activate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the window has been visually activated via <a href=\"#!/api/Ext.window.Window-method-setActive\" rel=\"Ext.window.Window-method-setActive\" class=\"docClass\">setActive</a>.</p>\n"},"deactivate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the window has been visually deactivated via <a href=\"#!/api/Ext.window.Window-method-setActive\" rel=\"Ext.window.Window-method-setActive\" class=\"docClass\">setActive</a>.</p>\n"},"resize":{"!type":"fn(this: ?, width: ?, height: ?, eOpts: ?)","!doc":"<p>Fires after the window has been resized.</p>\n"},"prototype":{"animateTarget":{"!doc":"<p>Id or element from which the window should animate while opening.</p>\n"},"autoRender":{"!type":"bool","!doc":"<p>Windows render to the body on first show.</p>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this panel's element.</p>\n"},"closable":{"!type":"bool","!doc":"<p>True to display the 'close' tool button and allow the user to close the window, false to hide the button and\ndisallow closing the window.</p>\n\n<p>By default, when close is requested by either clicking the close button in the header or pressing ESC when the\nWindow has focus, the <a href=\"#!/api/Ext.window.Window-method-close\" rel=\"Ext.window.Window-method-close\" class=\"docClass\">close</a> method will be called. This will <em><a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a></em> the\nWindow and its content meaning that it may not be reused.</p>\n\n<p>To make closing a Window <em>hide</em> the Window so that it may be reused, set <a href=\"#!/api/Ext.window.Window-cfg-closeAction\" rel=\"Ext.window.Window-cfg-closeAction\" class=\"docClass\">closeAction</a> to 'hide'.</p>\n"},"collapsed":{"!type":"bool","!doc":"<p>True to render the window collapsed, false to render it expanded. Note that if <a href=\"#!/api/Ext.window.Window-cfg-expandOnShow\" rel=\"Ext.window.Window-cfg-expandOnShow\" class=\"docClass\">expandOnShow</a>\nis true (the default) it will override the <code>collapsed</code> config and the window will always be\nexpanded when shown.</p>\n"},"collapsible":{"!type":"bool","!doc":"<p>inherited docs, same default</p>\n"},"constrainHeader":{"!type":"bool","!doc":"<p>True to constrain the window header within its containing element (allowing the window body to fall outside of\nits containing element) or false to allow the header to fall outside its containing element.\nOptionally the entire window can be constrained using <a href=\"#!/api/Ext.window.Window-cfg-constrain\" rel=\"Ext.window.Window-cfg-constrain\" class=\"docClass\">constrain</a>.</p>\n"},"defaultFocus":{"!doc":"<p>Specifies a Component to receive focus when this Window is focused.</p>\n\n<p>If a String is provided, the Component will be resolved using the down method which uses Ext.ComponentQuery.\nIf the string begins with an alphanumeric value, it will first attempt to find the Component based on the <a href=\"#!/api/Ext.Component-cfg-id\" rel=\"Ext.Component-cfg-id\" class=\"docClass\">Ext.Component.id</a> or <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">Ext.Component.itemId</a>.\nIf a matching component is not found via id, then an attempt to do a query to find a matching component.</p>\n\n<p>An example of finding the Component with an id/itemId:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>', {\n    autoShow     : true,\n    width        : 300,\n    title        : 'Login',\n    defaultFocus : 'username',\n    items        : [\n        {\n            xtype      : 'textfield',\n            fieldLabel : 'Username',\n            itemId     : 'username',\n            name       : 'username'\n        },\n        {\n            xtype      : 'textfield',\n            inputType  : 'password',\n            fieldLabel : 'Password',\n            itemId     : 'password',\n            name       : 'password'\n        }\n    ]\n});\n</code></pre>\n\n<p>If a Number is provided, this will resolve an <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a> at that index. This is very useful if\nthe window has buttons in the <a href=\"#!/api/Ext.window.Window-cfg-buttons\" rel=\"Ext.window.Window-cfg-buttons\" class=\"docClass\">buttons</a> config and you want to provide default focus to one of them.</p>\n\n<p>An example of this would be:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>', {\n    autoShow     : true,\n    width        : 300,\n    title        : 'Login',\n    defaultFocus : 1,\n    items        : [\n        {\n            xtype      : 'textfield',\n            fieldLabel : 'Username',\n            name       : 'username'\n       },\n       {\n            xtype      : 'textfield',\n            inputType  : 'password',\n            fieldLabel : 'Password',\n            name       : 'password'\n        }\n    ],\n    buttons      : [\n        {\n            text : 'Cancel'\n        },\n        {\n            text : 'Login'\n        }\n    ]\n});\n</code></pre>\n\n<p>In summary, defaultFocus may be one of:</p>\n\n<ul>\n<li>The index of a footer Button.</li>\n<li>The id or <a href=\"#!/api/Ext.Component-cfg-itemId\" rel=\"Ext.Component-cfg-itemId\" class=\"docClass\">Ext.Component.itemId</a> of a descendant Component.</li>\n<li>A query to find a <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.</li>\n<li>A descendant <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>.</li>\n</ul>\n\n"},"draggable":{"!type":"bool","!doc":"<p>True to allow the window to be dragged by the header bar, false to disable dragging. Note that\nby default the window will be centered in the viewport, so if dragging is disabled the window may need to be\npositioned programmatically after render (e.g., <code>myWindow.setPosition(100, 100);</code>).</p>\n"},"expandOnShow":{"!type":"bool","!doc":"<p>True to always expand the window when it is displayed, false to keep it in its current state (which may be\n<a href=\"#!/api/Ext.window.Window-cfg-collapsed\" rel=\"Ext.window.Window-cfg-collapsed\" class=\"docClass\">collapsed</a>) when displayed.</p>\n"},"ghost":{"!doc":"<p>Set to false to disable the ghost panel during dragging the window.\nDo note that you should not set this to true, by default it is a function.</p>\n"},"hidden":{"!type":"bool","!doc":"<p>Render this Window hidden. If <code>true</code>, the <a href=\"#!/api/Ext.window.Window-method-hide\" rel=\"Ext.window.Window-method-hide\" class=\"docClass\">hide</a> method will be called internally.</p>\n"},"hideShadowOnDeactivate":{"!type":"bool","!doc":"<p>True to hide this Window's shadow when another floating item in the same z-index stack is activated.</p>\n"},"maximizable":{"!type":"bool","!doc":"<p>True to display the 'maximize' tool button and allow the user to maximize the window, false to hide the button\nand disallow maximizing the window. Note that when a window is maximized, the tool button\nwill automatically change to a 'restore' button with the appropriate behavior already built-in that will restore\nthe window to its previous size.</p>\n"},"maximized":{"!type":"bool","!doc":"<p>True to initially display the window in a maximized state.</p>\n"},"minimizable":{"!type":"bool","!doc":"<p>True to display the 'minimize' tool button and allow the user to minimize the window, false to hide the button\nand disallow minimizing the window. Note that this button provides no implementation -- the\nbehavior of minimizing a window is implementation-specific, so the minimize event must be handled and a custom\nminimize behavior implemented for this option to be useful.</p>\n"},"modal":{"!type":"bool","!doc":"<p>True to make the window modal and mask everything behind it when displayed, false to display it without\nrestricting access to other UI elements.</p>\n"},"monitorResize":{"!type":"bool","!doc":"<p><code>true</code> to listen to the viewport resize event and perform any layout updating if necessary.\nThis is useful if using sizes as percentages for the window.</p>\n"},"onEsc":{"!type":"fn(k: ?, e: ?) -> !this"},"overlapHeader":{"!type":"bool","!doc":"<p>True to overlap the header in a panel over the framing of the panel itself. This is needed when frame:true (and\nis done automatically for you). Otherwise it is undefined. If you manually add rounded corners to a panel header\nwhich does not have frame:true, this will need to be set to true.</p>\n"},"plain":{"!type":"bool","!doc":"<p>True to render the window body with a transparent background so that it will blend into the framing elements,\nfalse to add a lighter background color to visually highlight the body element and separate it more distinctly\nfrom the surrounding frame.</p>\n"},"x":{"!type":"number","!doc":"<p>The X position of the left edge of the window on initial showing. Defaults to centering the Window within the\nwidth of the Window's container Element (The Element that the Window is rendered to).</p>\n"},"y":{"!type":"number","!doc":"<p>The Y position of the top edge of the window on initial showing. Defaults to centering the Window within the\nheight of the Window's container Element (The Element that the Window is rendered to).</p>\n"},"alwaysFramed":{"!type":"bool","!doc":"<p>Flag to Renderable to always look up the framing styles for this Component</p>\n"},"dd":{"!doc":"<p>If this Window is configured <a href=\"#!/api/Ext.window.Window-cfg-draggable\" rel=\"Ext.window.Window-cfg-draggable\" class=\"docClass\">draggable</a>, this property will contain an instance of\n<a href=\"#!/api/Ext.util.ComponentDragger\" rel=\"Ext.util.ComponentDragger\" class=\"docClass\">Ext.util.ComponentDragger</a> (A subclass of <a href=\"#!/api/Ext.dd.DragTracker\" rel=\"Ext.dd.DragTracker\" class=\"docClass\">DragTracker</a>) which handles dragging\nthe Window's DOM Element, and constraining according to the <a href=\"#!/api/Ext.window.Window-cfg-constrain\" rel=\"Ext.window.Window-cfg-constrain\" class=\"docClass\">constrain</a> and <a href=\"#!/api/Ext.window.Window-cfg-constrainHeader\" rel=\"Ext.window.Window-cfg-constrainHeader\" class=\"docClass\">constrainHeader</a> .</p>\n\n<p>This has implementations of <code>onBeforeStart</code>, <code>onDrag</code> and <code>onEnd</code> which perform the dragging action. If\nextra logic is needed at these points, use createInterceptor or\ncreateSequence to augment the existing implementations.</p>\n"},"floating":{"!type":"bool","!doc":"<p>A Window is always floating.</p>\n"},"ignoreHeaderBorderManagement":{"!type":"bool"},"initialAlphaNum":{},"isRootCfg":{"!doc":"<p>Buffer this so we don't recreate the same object</p>\n"},"isWindow":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Window, or subclass thereof.</p>\n"},"itemCls":{"!type":"string"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"addTools":{"!type":"fn() -> !this","!doc":"<p>Contribute class-specific tools to the header.</p>\n\n<p>Called by Panel's initTools at initialization time.</p>\n\n<p>Implementations should jst add new tool config objects to <code>this.tools</code></p>\n"},"afterCollapse":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Panel is Collapsed.</p>\n"},"afterExpand":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Panel is Expanded.</p>\n"},"afterHide":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Component has been hidden.</p>\n\n<p>Gets passed the same <code>callback</code> and <code>scope</code> parameters that <a href=\"#!/api/Ext.window.Window-method-onHide\" rel=\"Ext.window.Window-method-onHide\" class=\"docClass\">onHide</a> received.</p>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"applyState":{"!type":"fn(state: ?) -> !this","!doc":"<p>Applies the state to the object. This should be overridden in subclasses to do\nmore complex state operations. By default it applies the state properties onto\nthe current object.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"createGhost":{"!type":"fn() -> !this"},"doClose":{"!type":"fn() -> !this"},"getDefaultFocus":{"!type":"fn() -> !this","!doc":"<p>Gets the configured default focus item.  If a <a href=\"#!/api/Ext.window.Window-cfg-defaultFocus\" rel=\"Ext.window.Window-cfg-defaultFocus\" class=\"docClass\">defaultFocus</a> is set, it will\nreceive focus when the Window's <code>focus</code> method is called, otherwise the\nWindow itself will receive focus.</p>\n"},"getElConfig":{"!type":"fn() -> !this"},"getFocusEl":{"!type":"fn() -> ?","!doc":"<p>Returns the focus holder element associated with this Window. By default, this is the Window's element.</p>\n"},"getState":{"!type":"fn() -> ?","!doc":"<p>The supplied default state gathering method for the Component class.</p>\n\n<p>This method returns dimension settings such as <code>flex</code>, <code>anchor</code>, <code>width</code> and <code>height</code> along with <code>collapsed</code>\nstate.</p>\n\n<p>Subclasses which implement more complex state should call the superclass's implementation, and apply their state\nto the result if this basic state is to be saved.</p>\n\n<p>Note that Component state will only be saved if the Component has a <a href=\"#!/api/Ext.window.Window-cfg-stateId\" rel=\"Ext.window.Window-cfg-stateId\" class=\"docClass\">stateId</a> and there as a StateProvider\nconfigured for the document.</p>\n"},"maximize":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the window has been maximized.</p>\n"},"minimize":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the window has been minimized.</p>\n"},"onFocus":{"!type":"fn() -> !this","!doc":"<p>Called when a Component's focusEl receives focus.\nIf there is a valid default focus Component to jump to, focus that,\notherwise continue as usual, focus this Component.</p>\n"},"onRender":{"!type":"fn(ct: ?, position: ?) -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"onWindowResize":{"!type":"fn() -> !this"},"restore":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the window has been restored to its original size after being maximized.</p>\n"},"resumeHeaderLayout":{"!type":"fn(changed: ?) -> !this"},"syncMonitorWindowResize":{"!type":"fn() -> !this","!doc":"<p>Synchronizes the presence of our listener for window resize events. This method\nshould be called whenever this status might change.</p>\n"},"toggleMaximize":{"!type":"fn() -> ?","!doc":"<p>A shortcut method for toggling between <a href=\"#!/api/Ext.window.Window-method-maximize\" rel=\"Ext.window.Window-method-maximize\" class=\"docClass\">maximize</a> and <a href=\"#!/api/Ext.window.Window-method-restore\" rel=\"Ext.window.Window-method-restore\" class=\"docClass\">restore</a> based on the current maximized\nstate of the window.</p>\n"},"activate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the window has been visually activated via <a href=\"#!/api/Ext.window.Window-method-setActive\" rel=\"Ext.window.Window-method-setActive\" class=\"docClass\">setActive</a>.</p>\n"},"deactivate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires after the window has been visually deactivated via <a href=\"#!/api/Ext.window.Window-method-setActive\" rel=\"Ext.window.Window-method-setActive\" class=\"docClass\">setActive</a>.</p>\n"},"resize":{"!type":"fn(this: ?, width: ?, height: ?, eOpts: ?)","!doc":"<p>Fires after the window has been resized.</p>\n"},"constrain":{"!type":"bool","!doc":"<p>True to constrain the window within its containing element, false to allow it to fall outside of its containing\nelement. By default the window will be rendered to <code>document.body</code>. To render and constrain the window within\nanother element specify <a href=\"#!/api/Ext.window.Window-cfg-renderTo\" rel=\"Ext.window.Window-cfg-renderTo\" class=\"docClass\">renderTo</a>. Optionally the header only can be constrained\nusing <a href=\"#!/api/Ext.window.Window-cfg-constrainHeader\" rel=\"Ext.window.Window-cfg-constrainHeader\" class=\"docClass\">constrainHeader</a>.</p>\n"},"hideMode":{"!type":"string","!doc":"<p>Windows hide using offsets in order to preserve the scroll positions of their descendants.</p>\n"},"minHeight":{"!type":"number","!doc":"<p>inherit docs</p>\n"},"minWidth":{"!type":"number","!doc":"<p>inherit docs</p>\n"},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"onShow":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the show operation. After\ncalling the superclass's onShow, the Component will be visible.</p>\n\n<p>Override in subclasses where more complex behaviour is needed.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.window.Window-event-show\" rel=\"Ext.window.Window-event-show\" class=\"docClass\">show</a>.</p>\n"},"resizable":{"!doc":"<p>Specify as <code>true</code> to allow user resizing at each edge and corner of the window, false to disable resizing.</p>\n\n<p>This may also be specified as a config object to <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Ext.resizer.Resizer</a></p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"}},"!doc":"<p>A specialized panel intended for use as an application window. Windows are floated, <a href=\"#!/api/Ext.window.Window-cfg-resizable\" rel=\"Ext.window.Window-cfg-resizable\" class=\"docClass\">resizable</a>, and\n<a href=\"#!/api/Ext.window.Window-cfg-draggable\" rel=\"Ext.window.Window-cfg-draggable\" class=\"docClass\">draggable</a> by default. Windows can be <a href=\"#!/api/Ext.window.Window-cfg-maximizable\" rel=\"Ext.window.Window-cfg-maximizable\" class=\"docClass\">maximized</a> to fill the viewport, restored to\ntheir prior size, and can be <a href=\"#!/api/Ext.window.Window-method-minimize\" rel=\"Ext.window.Window-method-minimize\" class=\"docClass\">minimize</a>d.</p>\n\n<p>Windows can also be linked to a <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">Ext.ZIndexManager</a> or managed by the <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">Ext.WindowManager</a> to provide\ngrouping, activation, to front, to back and other application-specific behavior.</p>\n\n<p>By default, Windows will be rendered to document.body. To <a href=\"#!/api/Ext.window.Window-cfg-constrain\" rel=\"Ext.window.Window-cfg-constrain\" class=\"docClass\">constrain</a> a Window to another element specify\n<a href=\"#!/api/Ext.Component-cfg-renderTo\" rel=\"Ext.Component-cfg-renderTo\" class=\"docClass\">renderTo</a>.</p>\n\n<p><strong>As with all <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a>s, it is important to consider how you want the Window to size\nand arrange any child Components. Choose an appropriate <a href=\"#!/api/Ext.window.Window-cfg-layout\" rel=\"Ext.window.Window-cfg-layout\" class=\"docClass\">layout</a> configuration which lays out child Components\nin the required manner.</strong></p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Ext.window.Window</a>', {\n    title: 'Hello',\n    height: 200,\n    width: 400,\n    layout: 'fit',\n    items: {  // Let's put an empty grid in just to illustrate fit layout\n        xtype: 'grid',\n        border: false,\n        columns: [{header: 'World'}],                 // One header just for show. There's no data,\n        store: Ext.create('Ext.data.ArrayStore', {}) // A dummy empty data store\n    }\n}).show();\n</code></pre>\n","!type":"fn(config: ?)"},"MessageBox":{"autoScroll":{"!type":"bool","!doc":"<p><code>true</code> to use overflow:'auto' on the components layout element and show scroll bars automatically when necessary,\n<code>false</code> to clip any overflowing content.\nThis should not be combined with <a href=\"#!/api/Ext.window.MessageBox-cfg-overflowX\" rel=\"Ext.window.MessageBox-cfg-overflowX\" class=\"docClass\">overflowX</a> or  <a href=\"#!/api/Ext.window.MessageBox-cfg-overflowY\" rel=\"Ext.window.MessageBox-cfg-overflowY\" class=\"docClass\">overflowY</a>.</p>\n"},"closeAction":{"!type":"string","!doc":"<p>The action to take when the close header tool is clicked:</p>\n\n<ul>\n<li><p><strong><code>'<a href=\"#!/api/Ext.window.MessageBox-method-destroy\" rel=\"Ext.window.MessageBox-method-destroy\" class=\"docClass\">destroy</a>'</code></strong> :</p>\n\n<p><a href=\"#!/api/Ext.window.MessageBox-method-remove\" rel=\"Ext.window.MessageBox-method-remove\" class=\"docClass\">remove</a> the window from the DOM and <a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a> it and all descendant\nComponents. The window will <strong>not</strong> be available to be redisplayed via the <a href=\"#!/api/Ext.window.MessageBox-method-show\" rel=\"Ext.window.MessageBox-method-show\" class=\"docClass\">show</a> method.</p></li>\n<li><p><strong><code>'<a href=\"#!/api/Ext.window.MessageBox-method-hide\" rel=\"Ext.window.MessageBox-method-hide\" class=\"docClass\">hide</a>'</code></strong> :</p>\n\n<p><a href=\"#!/api/Ext.window.MessageBox-method-hide\" rel=\"Ext.window.MessageBox-method-hide\" class=\"docClass\">hide</a> the window by setting visibility to hidden and applying negative offsets. The window will be\navailable to be redisplayed via the <a href=\"#!/api/Ext.window.MessageBox-method-show\" rel=\"Ext.window.MessageBox-method-show\" class=\"docClass\">show</a> method.</p></li>\n</ul>\n\n\n<p><strong>Note:</strong> This behavior has changed! setting <em>does</em> affect the <a href=\"#!/api/Ext.window.MessageBox-method-close\" rel=\"Ext.window.MessageBox-method-close\" class=\"docClass\">close</a> method which will invoke the\napproriate closeAction.</p>\n"},"cls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element. This can be useful\nfor adding customized styles to the component or any of its children using standard CSS rules.</p>\n"},"constrain":{"!type":"bool","!doc":"<p>True to constrain the window within its containing element, false to allow it to fall outside of its containing\nelement. By default the window will be rendered to <code>document.body</code>. To render and constrain the window within\nanother element specify <a href=\"#!/api/Ext.window.MessageBox-cfg-renderTo\" rel=\"Ext.window.MessageBox-cfg-renderTo\" class=\"docClass\">renderTo</a>. Optionally the header only can be constrained\nusing <a href=\"#!/api/Ext.window.MessageBox-cfg-constrainHeader\" rel=\"Ext.window.MessageBox-cfg-constrainHeader\" class=\"docClass\">constrainHeader</a>.</p>\n"},"hideMode":{"!type":"string","!doc":"<p>hide it by offsets. Windows are hidden on render by default.</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.window.MessageBox-cfg-items\" rel=\"Ext.window.MessageBox-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"maxHeight":{"!type":"number","!doc":"<p>The maximum value in pixels which this Component will set its height to.</p>\n\n<p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>\n"},"maxWidth":{"!type":"number","!doc":"<p>The maximum value in pixels which this Component will set its width to.</p>\n\n<p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>\n"},"minHeight":{"!type":"number","!doc":"<p>inherit docs</p>\n"},"minWidth":{"!type":"number","!doc":"<p>Forcibly set these to null on the prototype to override anything set higher in\nthe hierarchy</p>\n"},"resizable":{"!doc":"<p>Specify as <code>true</code> to allow user resizing at each edge and corner of the window, false to disable resizing.</p>\n\n<p>This may also be specified as a config object to <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Ext.resizer.Resizer</a></p>\n"},"shrinkWrapDock":{"!doc":"<p>We want to shrinkWrap around all docked items</p>\n"},"title":{"!type":"string","!doc":"<p>The title text to be used to display in the <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Panel Header</a>.\nOr a config object for a <a href=\"#!/api/Ext.panel.Title\" rel=\"Ext.panel.Title\" class=\"docClass\">Panel Title</a>. When a <code>title</code> is\nspecified the <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Ext.panel.Header</a> will automatically be created and\ndisplayed unless <a href=\"#!/api/Ext.window.MessageBox-cfg-header\" rel=\"Ext.window.MessageBox-cfg-header\" class=\"docClass\">header</a> is set to <code>false</code>.</p>\n"},"CANCEL":{"!type":"number","!doc":"<p>Button config that displays a single Cancel button</p>\n"},"ERROR":{"!type":"string","!doc":"<p>The CSS class that provides the ERROR icon image</p>\n"},"INFO":{"!type":"string","!doc":"<p>The CSS class that provides the INFO icon image</p>\n"},"NO":{"!type":"number","!doc":"<p>Button config that displays a single No button</p>\n"},"OK":{"!type":"number","!doc":"<p>Button config that displays a single OK button</p>\n"},"OKCANCEL":{"!type":"number","!doc":"<p>Button config that displays OK and Cancel buttons</p>\n"},"QUESTION":{"!type":"string","!doc":"<p>The CSS class that provides the QUESTION icon image</p>\n"},"WARNING":{"!type":"string","!doc":"<p>The CSS class that provides the WARNING icon image</p>\n"},"YES":{"!type":"number","!doc":"<p>Button config that displays a single Yes button</p>\n"},"YESNO":{"!type":"number","!doc":"<p>Button config that displays Yes and No buttons</p>\n"},"YESNOCANCEL":{"!type":"number","!doc":"<p>Button config that displays Yes, No and Cancel buttons</p>\n"},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"baseIconCls":{"!type":"string","!doc":"<p></p>\n"},"buttonIds":{"!doc":"<p></p>\n"},"buttonText":{"!doc":"<p>An object containing the default button text strings that can be overriden for localized language support.\nSupported properties are: ok, cancel, yes and no.  Generally you should include a locale-specific\nresource file for handling language support across the framework.\nCustomize the default text like so:</p>\n\n<pre><code>Ext.window.MessageBox.buttonText.yes = \"oui\"; //french\n</code></pre>\n"},"defaultMaxHeight":{"!type":"number"},"defaultMaxWidth":{"!type":"number"},"defaultMinHeight":{"!type":"number"},"defaultMinWidth":{"!type":"number"},"defaultTextHeight":{"!type":"number","!doc":"<p>The default height in pixels of the message box's multiline textarea if displayed.</p>\n"},"minProgressWidth":{"!type":"number","!doc":"<p>The minimum width in pixels of the message box if it is a progress-style dialog.  This is useful\nfor setting a different minimum width than text-only dialogs may need.</p>\n"},"minPromptWidth":{"!type":"number","!doc":"<p>The minimum width in pixels of the message box if it is a prompt dialog.  This is useful\nfor setting a different minimum width than text-only dialogs may need.</p>\n"},"titleText":{"!doc":"<p><locale type=\"object\"></locale></p>\n"},"alert":{"!type":"fn(title: ?, message: ?, fn?: ?, scope?: ?) -> ?","!doc":"<p>Displays a standard read-only message box with an OK button (comparable to the basic JavaScript alert prompt).\nIf a callback function is passed it will be called after the user clicks the button, and the\nid of the button that was clicked will be passed as the only parameter to the callback\n(could also be the top-right close button, which will always report as \"cancel\").</p>\n"},"btnCallback":{"!type":"fn(btn: ?) -> !this"},"confirm":{"!type":"fn(title: ?, message: ?, fn?: ?, scope?: ?) -> ?","!doc":"<p>Displays a confirmation message box with Yes and No buttons (comparable to JavaScript's confirm).\nIf a callback function is passed it will be called after the user clicks either button,\nand the id of the button that was clicked will be passed as the only parameter to the callback\n(could also be the top-right close button, which will always report as \"cancel\").</p>\n"},"hide":{"!type":"fn() -> ?","!doc":"<p>Hides this Component, setting it to invisible using the configured <a href=\"#!/api/Ext.window.MessageBox-cfg-hideMode\" rel=\"Ext.window.MessageBox-cfg-hideMode\" class=\"docClass\">hideMode</a>.</p>\n"},"initComponent":{"!type":"fn(cfg: ?) -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"makeButton":{"!type":"fn(btnIdx: ?) -> !this"},"onClose":{"!type":"fn() -> !this"},"onEsc":{"!type":"fn() -> !this"},"onPromptKey":{"!type":"fn(textField: ?, e: ?) -> !this"},"onShow":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the show operation. After\ncalling the superclass's onShow, the Component will be visible.</p>\n\n<p>Override in subclasses where more complex behaviour is needed.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.window.MessageBox-event-show\" rel=\"Ext.window.MessageBox-event-show\" class=\"docClass\">show</a>.</p>\n"},"progress":{"!type":"fn(title: ?, message: ?, progressText?: ?) -> ?","!doc":"<p>Displays a message box with a progress bar.</p>\n\n<p>You are responsible for updating the progress bar as needed via <a href=\"#!/api/Ext.window.MessageBox-method-updateProgress\" rel=\"Ext.window.MessageBox-method-updateProgress\" class=\"docClass\">updateProgress</a>\nand closing the message box when the process is complete.</p>\n"},"prompt":{"!type":"fn(title: ?, message: ?, fn?: ?, scope?: ?, multiline?: ?, value?: ?) -> ?","!doc":"<p>Displays a message box with OK and Cancel buttons prompting the user to enter some text (comparable to JavaScript's prompt).\nThe prompt can be a single-line or multi-line textbox.  If a callback function is passed it will be called after the user\nclicks either button, and the id of the button that was clicked (could also be the top-right\nclose button, which will always report as \"cancel\") and the text that was entered will be passed as the two parameters to the callback.</p>\n"},"reconfigure":{"!type":"fn(cfg: ?) -> !this"},"setIcon":{"!type":"fn(icon: ?, width?: ?, height?: ?) -> ?","!doc":"<p>Adds the specified icon to the dialog.  By default, the class 'x-messagebox-icon' is applied for default\nstyling, and the class passed in is expected to supply the background image url. Pass in empty string ('')\nto clear any existing icon. This method must be called before the MessageBox is shown.\nThe following built-in icon classes are supported, but you can also pass in a custom class name:</p>\n\n<pre><code><a href=\"#!/api/Ext.window.MessageBox-property-INFO\" rel=\"Ext.window.MessageBox-property-INFO\" class=\"docClass\">Ext.window.MessageBox.INFO</a>\n<a href=\"#!/api/Ext.window.MessageBox-property-WARNING\" rel=\"Ext.window.MessageBox-property-WARNING\" class=\"docClass\">Ext.window.MessageBox.WARNING</a>\n<a href=\"#!/api/Ext.window.MessageBox-property-QUESTION\" rel=\"Ext.window.MessageBox-property-QUESTION\" class=\"docClass\">Ext.window.MessageBox.QUESTION</a>\n<a href=\"#!/api/Ext.window.MessageBox-property-ERROR\" rel=\"Ext.window.MessageBox-property-ERROR\" class=\"docClass\">Ext.window.MessageBox.ERROR</a>\n</code></pre>\n"},"show":{"!type":"fn(config: ?) -> ?","!doc":"<p>Displays a new message box, or reinitializes an existing message box, based on the config options passed in. All\ndisplay functions (e.g. prompt, alert, etc.) on MessageBox call this function internally, although those calls\nare basic shortcuts and do not support all of the config options allowed here.</p>\n\n<p>Example usage:</p>\n\n<pre><code><a href=\"#!/api/Ext.MessageBox-event-show\" rel=\"Ext.MessageBox-event-show\" class=\"docClass\">Ext.Msg.show</a>({\n    title: 'Address',\n    message: 'Please enter your address:',\n    width: 300,\n    buttons: <a href=\"#!/api/Ext.MessageBox-property-OKCANCEL\" rel=\"Ext.MessageBox-property-OKCANCEL\" class=\"docClass\">Ext.Msg.OKCANCEL</a>,\n    multiline: true,\n    fn: saveAddress,\n    animateTarget: 'addAddressBtn',\n    icon: <a href=\"#!/api/Ext.window.MessageBox-property-INFO\" rel=\"Ext.window.MessageBox-property-INFO\" class=\"docClass\">Ext.window.MessageBox.INFO</a>\n});\n</code></pre>\n"},"updateButtonText":{"!type":"fn() -> number","!doc":"<p>Set button text according to current buttonText property object</p>\n"},"updateProgress":{"!type":"fn(value?: ?, progressText?: ?, message?: ?) -> ?","!doc":"<p>Updates a progress-style message box's text and progress bar. Only relevant on message boxes\ninitiated via <a href=\"#!/api/Ext.window.MessageBox-method-progress\" rel=\"Ext.window.MessageBox-method-progress\" class=\"docClass\">progress</a> or <a href=\"#!/api/Ext.window.MessageBox-method-wait\" rel=\"Ext.window.MessageBox-method-wait\" class=\"docClass\">wait</a>,\nor by calling <a href=\"#!/api/Ext.window.MessageBox-method-show\" rel=\"Ext.window.MessageBox-method-show\" class=\"docClass\">show</a> with progress: true.</p>\n"},"updateText":{"!type":"fn(text: ?) -> !this"},"wait":{"!type":"fn(message: ?, title?: ?, config?: ?) -> ?","!doc":"<p>Displays a message box with an infinitely auto-updating progress bar.  This can be used to block user\ninteraction while waiting for a long-running process to complete that does not have defined intervals.\nYou are responsible for closing the message box when the process is complete.</p>\n"},"!doc":"<p>Utility class for generating different styles of message boxes.  The singleton instance, <a href=\"#!/api/Ext.MessageBox\" rel=\"Ext.MessageBox\" class=\"docClass\">Ext.MessageBox</a>\nalias <code><a href=\"#!/api/Ext.MessageBox\" rel=\"Ext.MessageBox\" class=\"docClass\">Ext.Msg</a></code> can also be used.</p>\n\n<p>Note that a MessageBox is asynchronous.  Unlike a regular JavaScript <code>alert</code> (which will halt\nbrowser execution), showing a MessageBox will not cause the code to stop.  For this reason, if you have code\nthat should only run <em>after</em> some user feedback from the MessageBox, you must use a callback function\n(see the <code>function</code> parameter for <a href=\"#!/api/Ext.window.MessageBox-method-show\" rel=\"Ext.window.MessageBox-method-show\" class=\"docClass\">show</a> for more details).</p>\n\n<p>Basic alert</p>\n\n<pre class='inline-example '><code><a href=\"#!/api/Ext.MessageBox-method-alert\" rel=\"Ext.MessageBox-method-alert\" class=\"docClass\">Ext.Msg.alert</a>('Status', 'Changes saved successfully.');\n</code></pre>\n\n<p>Prompt for user data and process the result using a callback</p>\n\n<pre class='inline-example '><code><a href=\"#!/api/Ext.MessageBox-method-prompt\" rel=\"Ext.MessageBox-method-prompt\" class=\"docClass\">Ext.Msg.prompt</a>('Name', 'Please enter your name:', function(btn, text){\n    if (btn == 'ok'){\n        // process text value and close...\n    }\n});\n</code></pre>\n\n<p>Show a dialog using config options</p>\n\n<pre class='inline-example '><code><a href=\"#!/api/Ext.MessageBox-event-show\" rel=\"Ext.MessageBox-event-show\" class=\"docClass\">Ext.Msg.show</a>({\n    title:'Save Changes?',\n    message: 'You are closing a tab that has unsaved changes. Would you like to save your changes?',\n    buttons: <a href=\"#!/api/Ext.MessageBox-property-YESNOCANCEL\" rel=\"Ext.MessageBox-property-YESNOCANCEL\" class=\"docClass\">Ext.Msg.YESNOCANCEL</a>,\n    icon: <a href=\"#!/api/Ext.MessageBox-property-QUESTION\" rel=\"Ext.MessageBox-property-QUESTION\" class=\"docClass\">Ext.Msg.QUESTION</a>,\n    fn: function(btn) {\n        if (btn === 'yes') {\n            console.log('Yes pressed');\n        } else if (btn === 'no') {\n            console.log('No pressed');\n        } else {\n            console.log('Cancel pressed');\n        } \n    }\n});\n</code></pre>\n","!type":"fn(cfg: ?)","prototype":{"autoScroll":{"!type":"bool","!doc":"<p><code>true</code> to use overflow:'auto' on the components layout element and show scroll bars automatically when necessary,\n<code>false</code> to clip any overflowing content.\nThis should not be combined with <a href=\"#!/api/Ext.window.MessageBox-cfg-overflowX\" rel=\"Ext.window.MessageBox-cfg-overflowX\" class=\"docClass\">overflowX</a> or  <a href=\"#!/api/Ext.window.MessageBox-cfg-overflowY\" rel=\"Ext.window.MessageBox-cfg-overflowY\" class=\"docClass\">overflowY</a>.</p>\n"},"closeAction":{"!type":"string","!doc":"<p>The action to take when the close header tool is clicked:</p>\n\n<ul>\n<li><p><strong><code>'<a href=\"#!/api/Ext.window.MessageBox-method-destroy\" rel=\"Ext.window.MessageBox-method-destroy\" class=\"docClass\">destroy</a>'</code></strong> :</p>\n\n<p><a href=\"#!/api/Ext.window.MessageBox-method-remove\" rel=\"Ext.window.MessageBox-method-remove\" class=\"docClass\">remove</a> the window from the DOM and <a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a> it and all descendant\nComponents. The window will <strong>not</strong> be available to be redisplayed via the <a href=\"#!/api/Ext.window.MessageBox-method-show\" rel=\"Ext.window.MessageBox-method-show\" class=\"docClass\">show</a> method.</p></li>\n<li><p><strong><code>'<a href=\"#!/api/Ext.window.MessageBox-method-hide\" rel=\"Ext.window.MessageBox-method-hide\" class=\"docClass\">hide</a>'</code></strong> :</p>\n\n<p><a href=\"#!/api/Ext.window.MessageBox-method-hide\" rel=\"Ext.window.MessageBox-method-hide\" class=\"docClass\">hide</a> the window by setting visibility to hidden and applying negative offsets. The window will be\navailable to be redisplayed via the <a href=\"#!/api/Ext.window.MessageBox-method-show\" rel=\"Ext.window.MessageBox-method-show\" class=\"docClass\">show</a> method.</p></li>\n</ul>\n\n\n<p><strong>Note:</strong> This behavior has changed! setting <em>does</em> affect the <a href=\"#!/api/Ext.window.MessageBox-method-close\" rel=\"Ext.window.MessageBox-method-close\" class=\"docClass\">close</a> method which will invoke the\napproriate closeAction.</p>\n"},"cls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element. This can be useful\nfor adding customized styles to the component or any of its children using standard CSS rules.</p>\n"},"constrain":{"!type":"bool","!doc":"<p>True to constrain the window within its containing element, false to allow it to fall outside of its containing\nelement. By default the window will be rendered to <code>document.body</code>. To render and constrain the window within\nanother element specify <a href=\"#!/api/Ext.window.MessageBox-cfg-renderTo\" rel=\"Ext.window.MessageBox-cfg-renderTo\" class=\"docClass\">renderTo</a>. Optionally the header only can be constrained\nusing <a href=\"#!/api/Ext.window.MessageBox-cfg-constrainHeader\" rel=\"Ext.window.MessageBox-cfg-constrainHeader\" class=\"docClass\">constrainHeader</a>.</p>\n"},"hideMode":{"!type":"string","!doc":"<p>hide it by offsets. Windows are hidden on render by default.</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.window.MessageBox-cfg-items\" rel=\"Ext.window.MessageBox-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"maxHeight":{"!type":"number","!doc":"<p>The maximum value in pixels which this Component will set its height to.</p>\n\n<p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>\n"},"maxWidth":{"!type":"number","!doc":"<p>The maximum value in pixels which this Component will set its width to.</p>\n\n<p><strong>Warning:</strong> This will override any size management applied by layout managers.</p>\n"},"minHeight":{"!type":"number","!doc":"<p>inherit docs</p>\n"},"minWidth":{"!type":"number","!doc":"<p>Forcibly set these to null on the prototype to override anything set higher in\nthe hierarchy</p>\n"},"resizable":{"!doc":"<p>Specify as <code>true</code> to allow user resizing at each edge and corner of the window, false to disable resizing.</p>\n\n<p>This may also be specified as a config object to <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Ext.resizer.Resizer</a></p>\n"},"shrinkWrapDock":{"!doc":"<p>We want to shrinkWrap around all docked items</p>\n"},"title":{"!type":"string","!doc":"<p>The title text to be used to display in the <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Panel Header</a>.\nOr a config object for a <a href=\"#!/api/Ext.panel.Title\" rel=\"Ext.panel.Title\" class=\"docClass\">Panel Title</a>. When a <code>title</code> is\nspecified the <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">Ext.panel.Header</a> will automatically be created and\ndisplayed unless <a href=\"#!/api/Ext.window.MessageBox-cfg-header\" rel=\"Ext.window.MessageBox-cfg-header\" class=\"docClass\">header</a> is set to <code>false</code>.</p>\n"},"CANCEL":{"!type":"number","!doc":"<p>Button config that displays a single Cancel button</p>\n"},"ERROR":{"!type":"string","!doc":"<p>The CSS class that provides the ERROR icon image</p>\n"},"INFO":{"!type":"string","!doc":"<p>The CSS class that provides the INFO icon image</p>\n"},"NO":{"!type":"number","!doc":"<p>Button config that displays a single No button</p>\n"},"OK":{"!type":"number","!doc":"<p>Button config that displays a single OK button</p>\n"},"OKCANCEL":{"!type":"number","!doc":"<p>Button config that displays OK and Cancel buttons</p>\n"},"QUESTION":{"!type":"string","!doc":"<p>The CSS class that provides the QUESTION icon image</p>\n"},"WARNING":{"!type":"string","!doc":"<p>The CSS class that provides the WARNING icon image</p>\n"},"YES":{"!type":"number","!doc":"<p>Button config that displays a single Yes button</p>\n"},"YESNO":{"!type":"number","!doc":"<p>Button config that displays Yes and No buttons</p>\n"},"YESNOCANCEL":{"!type":"number","!doc":"<p>Button config that displays Yes, No and Cancel buttons</p>\n"},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"baseIconCls":{"!type":"string","!doc":"<p></p>\n"},"buttonIds":{"!doc":"<p></p>\n"},"buttonText":{"!doc":"<p>An object containing the default button text strings that can be overriden for localized language support.\nSupported properties are: ok, cancel, yes and no.  Generally you should include a locale-specific\nresource file for handling language support across the framework.\nCustomize the default text like so:</p>\n\n<pre><code>Ext.window.MessageBox.buttonText.yes = \"oui\"; //french\n</code></pre>\n"},"defaultMaxHeight":{"!type":"number"},"defaultMaxWidth":{"!type":"number"},"defaultMinHeight":{"!type":"number"},"defaultMinWidth":{"!type":"number"},"defaultTextHeight":{"!type":"number","!doc":"<p>The default height in pixels of the message box's multiline textarea if displayed.</p>\n"},"minProgressWidth":{"!type":"number","!doc":"<p>The minimum width in pixels of the message box if it is a progress-style dialog.  This is useful\nfor setting a different minimum width than text-only dialogs may need.</p>\n"},"minPromptWidth":{"!type":"number","!doc":"<p>The minimum width in pixels of the message box if it is a prompt dialog.  This is useful\nfor setting a different minimum width than text-only dialogs may need.</p>\n"},"titleText":{"!doc":"<p><locale type=\"object\"></locale></p>\n"},"alert":{"!type":"fn(title: ?, message: ?, fn?: ?, scope?: ?) -> ?","!doc":"<p>Displays a standard read-only message box with an OK button (comparable to the basic JavaScript alert prompt).\nIf a callback function is passed it will be called after the user clicks the button, and the\nid of the button that was clicked will be passed as the only parameter to the callback\n(could also be the top-right close button, which will always report as \"cancel\").</p>\n"},"btnCallback":{"!type":"fn(btn: ?) -> !this"},"confirm":{"!type":"fn(title: ?, message: ?, fn?: ?, scope?: ?) -> ?","!doc":"<p>Displays a confirmation message box with Yes and No buttons (comparable to JavaScript's confirm).\nIf a callback function is passed it will be called after the user clicks either button,\nand the id of the button that was clicked will be passed as the only parameter to the callback\n(could also be the top-right close button, which will always report as \"cancel\").</p>\n"},"hide":{"!type":"fn() -> ?","!doc":"<p>Hides this Component, setting it to invisible using the configured <a href=\"#!/api/Ext.window.MessageBox-cfg-hideMode\" rel=\"Ext.window.MessageBox-cfg-hideMode\" class=\"docClass\">hideMode</a>.</p>\n"},"initComponent":{"!type":"fn(cfg: ?) -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"makeButton":{"!type":"fn(btnIdx: ?) -> !this"},"onClose":{"!type":"fn() -> !this"},"onEsc":{"!type":"fn() -> !this"},"onPromptKey":{"!type":"fn(textField: ?, e: ?) -> !this"},"onShow":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the show operation. After\ncalling the superclass's onShow, the Component will be visible.</p>\n\n<p>Override in subclasses where more complex behaviour is needed.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.window.MessageBox-event-show\" rel=\"Ext.window.MessageBox-event-show\" class=\"docClass\">show</a>.</p>\n"},"progress":{"!type":"fn(title: ?, message: ?, progressText?: ?) -> ?","!doc":"<p>Displays a message box with a progress bar.</p>\n\n<p>You are responsible for updating the progress bar as needed via <a href=\"#!/api/Ext.window.MessageBox-method-updateProgress\" rel=\"Ext.window.MessageBox-method-updateProgress\" class=\"docClass\">updateProgress</a>\nand closing the message box when the process is complete.</p>\n"},"prompt":{"!type":"fn(title: ?, message: ?, fn?: ?, scope?: ?, multiline?: ?, value?: ?) -> ?","!doc":"<p>Displays a message box with OK and Cancel buttons prompting the user to enter some text (comparable to JavaScript's prompt).\nThe prompt can be a single-line or multi-line textbox.  If a callback function is passed it will be called after the user\nclicks either button, and the id of the button that was clicked (could also be the top-right\nclose button, which will always report as \"cancel\") and the text that was entered will be passed as the two parameters to the callback.</p>\n"},"reconfigure":{"!type":"fn(cfg: ?) -> !this"},"setIcon":{"!type":"fn(icon: ?, width?: ?, height?: ?) -> ?","!doc":"<p>Adds the specified icon to the dialog.  By default, the class 'x-messagebox-icon' is applied for default\nstyling, and the class passed in is expected to supply the background image url. Pass in empty string ('')\nto clear any existing icon. This method must be called before the MessageBox is shown.\nThe following built-in icon classes are supported, but you can also pass in a custom class name:</p>\n\n<pre><code><a href=\"#!/api/Ext.window.MessageBox-property-INFO\" rel=\"Ext.window.MessageBox-property-INFO\" class=\"docClass\">Ext.window.MessageBox.INFO</a>\n<a href=\"#!/api/Ext.window.MessageBox-property-WARNING\" rel=\"Ext.window.MessageBox-property-WARNING\" class=\"docClass\">Ext.window.MessageBox.WARNING</a>\n<a href=\"#!/api/Ext.window.MessageBox-property-QUESTION\" rel=\"Ext.window.MessageBox-property-QUESTION\" class=\"docClass\">Ext.window.MessageBox.QUESTION</a>\n<a href=\"#!/api/Ext.window.MessageBox-property-ERROR\" rel=\"Ext.window.MessageBox-property-ERROR\" class=\"docClass\">Ext.window.MessageBox.ERROR</a>\n</code></pre>\n"},"show":{"!type":"fn(config: ?) -> ?","!doc":"<p>Displays a new message box, or reinitializes an existing message box, based on the config options passed in. All\ndisplay functions (e.g. prompt, alert, etc.) on MessageBox call this function internally, although those calls\nare basic shortcuts and do not support all of the config options allowed here.</p>\n\n<p>Example usage:</p>\n\n<pre><code><a href=\"#!/api/Ext.MessageBox-event-show\" rel=\"Ext.MessageBox-event-show\" class=\"docClass\">Ext.Msg.show</a>({\n    title: 'Address',\n    message: 'Please enter your address:',\n    width: 300,\n    buttons: <a href=\"#!/api/Ext.MessageBox-property-OKCANCEL\" rel=\"Ext.MessageBox-property-OKCANCEL\" class=\"docClass\">Ext.Msg.OKCANCEL</a>,\n    multiline: true,\n    fn: saveAddress,\n    animateTarget: 'addAddressBtn',\n    icon: <a href=\"#!/api/Ext.window.MessageBox-property-INFO\" rel=\"Ext.window.MessageBox-property-INFO\" class=\"docClass\">Ext.window.MessageBox.INFO</a>\n});\n</code></pre>\n"},"updateButtonText":{"!type":"fn() -> number","!doc":"<p>Set button text according to current buttonText property object</p>\n"},"updateProgress":{"!type":"fn(value?: ?, progressText?: ?, message?: ?) -> ?","!doc":"<p>Updates a progress-style message box's text and progress bar. Only relevant on message boxes\ninitiated via <a href=\"#!/api/Ext.window.MessageBox-method-progress\" rel=\"Ext.window.MessageBox-method-progress\" class=\"docClass\">progress</a> or <a href=\"#!/api/Ext.window.MessageBox-method-wait\" rel=\"Ext.window.MessageBox-method-wait\" class=\"docClass\">wait</a>,\nor by calling <a href=\"#!/api/Ext.window.MessageBox-method-show\" rel=\"Ext.window.MessageBox-method-show\" class=\"docClass\">show</a> with progress: true.</p>\n"},"updateText":{"!type":"fn(text: ?) -> !this"},"wait":{"!type":"fn(message: ?, title?: ?, config?: ?) -> ?","!doc":"<p>Displays a message box with an infinitely auto-updating progress bar.  This can be used to block user\ninteraction while waiting for a long-running process to complete that does not have defined intervals.\nYou are responsible for closing the message box when the process is complete.</p>\n"}}},"Toast":{"!doc":"<p>This class provides for lightweight, auto-dismissing pop-up notifications called \"toasts\".\nAt the base level, you can display a toast message by calling <code>Ext.toast</code> like so:</p>\n\n<pre><code> Ext.toast('Data saved');\n</code></pre>\n\n<p>This will result in a toast message, which displays in the default location of bottom right in your viewport.</p>\n\n<p>You may expand upon this simple example with the following parameters:</p>\n\n<pre><code> Ext.toast(message, title, align, iconCls);\n</code></pre>\n\n<p>For example, the following toast will appear top-middle in your viewport.  It will display\nthe 'Data Saved' message with a title of 'Title'</p>\n\n<pre><code> Ext.toast('Data Saved', 'Title', 't')\n</code></pre>\n\n<p>It should be noted that the toast's width is determined by the message's width.\nIf you need to set a specific width, or any of the other available configurations for your toast,\nyou can create the toast object as seen below:</p>\n\n<pre><code> Ext.toast({\n     html: 'Data Saved',\n     title: 'My Title',\n     width: 200,\n     align: 't'\n });\n</code></pre>\n\n<p>This component is derived from the excellent work of a Sencha community member, Eirik\nLorentsen.</p>\n","!type":"fn(config: ?)","prototype":{"align":{"!doc":"<p>Specifies the basic alignment of the toast message with its <code>anchor</code>. This controls\nmany aspects of the toast animation as well. For fine grain control of the final\nplacement of the toast and its <code>anchor</code> you may set <code>anchorAlign</code> <em>as well</em>.</p>\n"},"anchor":{"!doc":"<p>The component or the <code>id</code> of the component to which the <code>toast</code> will be anchored.\nThe default behavior is to anchor a <code>toast</code> to the document body (no component).</p>\n"},"anchorAlign":{"!type":"string","!doc":"<p>This string is a full specification of how to position the toast with respect to\nits <code>anchor</code>. This is set to a reasonable value based on <code>align</code> but the <code>align</code>\nalso sets defaults for various other properties. This config controls only the\nfinal position of the toast.</p>\n"},"bodyPadding":{"!doc":"<p>A shortcut for setting a padding style on the body element. The value can either be\na number to be applied to all sides, or a normal css string describing padding.</p>\n"},"cls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element. This can be useful\nfor adding customized styles to the component or any of its children using standard CSS rules.</p>\n"},"draggable":{"!type":"bool","!doc":"<p>True to allow the window to be dragged by the header bar, false to disable dragging. Note that\nby default the window will be centered in the viewport, so if dragging is disabled the window may need to be\npositioned programmatically after render (e.g., <code>myWindow.setPosition(100, 100);</code>).</p>\n"},"plain":{"!type":"bool","!doc":"<p>True to render the window body with a transparent background so that it will blend into the framing elements,\nfalse to add a lighter background color to visually highlight the body element and separate it more distinctly\nfrom the surrounding frame.</p>\n"},"resizable":{"!doc":"<p>Specify as <code>true</code> to allow user resizing at each edge and corner of the window, false to disable resizing.</p>\n\n<p>This may also be specified as a config object to <a href=\"#!/api/Ext.resizer.Resizer\" rel=\"Ext.resizer.Resizer\" class=\"docClass\">Ext.resizer.Resizer</a></p>\n"},"shadow":{"!doc":"<p>Specifies whether the floating component should be given a shadow. Set to true to automatically create an\n<a href=\"#!/api/Ext.Shadow\" rel=\"Ext.Shadow\" class=\"docClass\">Ext.Shadow</a>, or a string indicating the shadow's display <a href=\"#!/api/Ext.Shadow-cfg-mode\" rel=\"Ext.Shadow-cfg-mode\" class=\"docClass\">Ext.Shadow.mode</a>. Set to false to\ndisable the shadow.</p>\n"},"useXAxis":{"!type":"bool","!doc":"<p>Directs the toast message to animate on the x-axis (if <code>true</code>) or y-axis (if <code>false</code>).\nThis value defaults to a value based on the <code>align</code> config.</p>\n"},"alignmentProps":{"!doc":"<p>These properties are keyed by \"align\" and set defaults for various configs.</p>\n"},"autoClose":{"!type":"bool"},"autoCloseDelay":{"!type":"number"},"closeOnMouseDown":{"!type":"bool"},"closeOnMouseOut":{"!type":"bool"},"destroyAfterHide":{"!type":"bool"},"hideDuration":{"!type":"number"},"isFading":{"!type":"bool"},"isHiding":{"!type":"bool","!doc":"<p>Private. Do not override!</p>\n"},"paddingX":{"!type":"number","!doc":"<p>Pixels from the anchor's borders to start the first notification</p>\n"},"paddingY":{"!type":"number"},"slideBackAnimation":{"!type":"string"},"slideBackDuration":{"!type":"number"},"slideInAnimation":{"!type":"string"},"slideInDuration":{"!type":"number"},"spacing":{"!type":"number","!doc":"<p>Pixels between each notification</p>\n"},"stickOnClick":{"!type":"bool"},"stickWhileHover":{"!type":"bool"},"xPos":{"!type":"number","!doc":"<p>Caching coordinates to be able to align to final position of siblings being animated</p>\n"},"yPos":{"!type":"number"},"afterShow":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Component is shown (after <a href=\"#!/api/Ext.window.Toast-method-onShow\" rel=\"Ext.window.Toast-method-onShow\" class=\"docClass\">onShow</a> is called).</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.window.Toast-event-show\" rel=\"Ext.window.Toast-event-show\" class=\"docClass\">show</a>.</p>\n"},"beforeShow":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is shown.</p>\n"},"cancelAutoClose":{"!type":"fn() -> !this"},"doAutoClose":{"!type":"fn() -> !this"},"focus":{"!type":"fn() -> ?","!doc":"<p>Try to focus this component.</p>\n"},"getToasts":{"!type":"fn() -> !this"},"getXposAlignedToAnchor":{"!type":"fn() -> !this"},"getXposAlignedToSibling":{"!type":"fn(sibling: ?) -> !this"},"getYposAlignedToAnchor":{"!type":"fn() -> !this"},"getYposAlignedToSibling":{"!type":"fn(sibling: ?) -> !this"},"hide":{"!type":"fn() -> ?","!doc":"<p>Hides this Component, setting it to invisible using the configured <a href=\"#!/api/Ext.window.Toast-cfg-hideMode\" rel=\"Ext.window.Toast-cfg-hideMode\" class=\"docClass\">hideMode</a>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"onDocumentMousedown":{"!type":"fn(e: ?) -> !this"},"onMouseEnter":{"!type":"fn() -> !this"},"onMouseLeave":{"!type":"fn() -> !this"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"removeFromAnchor":{"!type":"fn() -> !this"},"setAnchor":{"!type":"fn(anchor: ?) -> !this"},"slideBack":{"!type":"fn() -> !this"},"update":{"!type":"fn() -> !this","!doc":"<p>Update the content area of a component.</p>\n"},"updateAlignment":{"!type":"fn(align: ?) -> !this"}}}},"picker":{"Color":{"!doc":"<p>Color picker provides a simple color palette for choosing colors. The picker can be rendered to any container. The\navailable default to a standard 40-color palette; this can be customized with the <a href=\"#!/api/Ext.picker.Color-property-colors\" rel=\"Ext.picker.Color-property-colors\" class=\"docClass\">colors</a> config.</p>\n\n<p>Typically you will need to implement a handler function to be notified when the user chooses a color from the picker;\nyou can register the handler using the <a href=\"#!/api/Ext.picker.Color-event-select\" rel=\"Ext.picker.Color-event-select\" class=\"docClass\">select</a> event, or by implementing the <a href=\"#!/api/Ext.picker.Color-cfg-handler\" rel=\"Ext.picker.Color-cfg-handler\" class=\"docClass\">handler</a> method.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.picker.Color\" rel=\"Ext.picker.Color\" class=\"docClass\">Ext.picker.Color</a>', {\n    value: '993300',  // initial selected color\n    renderTo: Ext.getBody(),\n    listeners: {\n        select: function(picker, selColor) {\n            alert(selColor);\n        }\n    }\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"allowReselect":{"!type":"bool","!doc":"<p>If set to true then reselecting a color that is already selected fires the <a href=\"#!/api/Ext.picker.Color-event-select\" rel=\"Ext.picker.Color-event-select\" class=\"docClass\">select</a> event</p>\n"},"clickEvent":{"!type":"string","!doc":"<p>The DOM event that will cause a color to be selected. This can be any valid event name (dblclick, contextmenu).</p>\n"},"componentCls":{"!type":"string","!doc":"<p>The CSS class to apply to the containing element.</p>\n"},"handler":{"!doc":"<p>A function that will handle the select event of this picker. The handler is passed the following parameters:</p>\n\n<ul>\n<li><p><code>picker</code> : ColorPicker</p>\n\n<p>The <a href=\"#!/api/Ext.picker.Color\" rel=\"Ext.picker.Color\" class=\"docClass\">picker</a>.</p></li>\n<li><p><code>color</code> : String</p>\n\n<p>The 6-digit color hex code (without the # symbol).</p></li>\n</ul>\n\n"},"itemCls":{"!type":"string","!doc":"<p>The CSS class to apply to the color picker's items</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.picker.Color-method-getEl\" rel=\"Ext.picker.Color-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.picker.Color-method-getEl\" rel=\"Ext.picker.Color-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.picker.Color-cfg-renderSelectors\" rel=\"Ext.picker.Color-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.picker.Color-cfg-childEls\" rel=\"Ext.picker.Color-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"scope":{"!doc":"<p>The scope (<code>this</code> reference) in which the <code><a href=\"#!/api/Ext.picker.Color-cfg-handler\" rel=\"Ext.picker.Color-cfg-handler\" class=\"docClass\">handler</a></code> function will be called.</p>\n\n<p>Defaults to this Color picker instance.</p>\n"},"selectedCls":{"!type":"string","!doc":"<p>The CSS class to apply to the selected element</p>\n"},"value":{"!type":"string","!doc":"<p>The initial color to highlight (should be a valid 6-digit color hex code without the # symbol). Note that the hex\ncodes are case-sensitive.</p>\n"},"colors":{"!doc":"<p>An array of 6-digit color hex code strings (without the # symbol). This array can contain any number of colors,\nand each hex code should be unique. The width of the picker is controlled via CSS by adjusting the width property\nof the 'x-color-picker' class (or assigning a custom class), so you can balance the number of colors with the\nwidth setting until the box is symmetrical.</p>\n\n<p>You can override individual colors if needed:</p>\n\n<pre><code>var cp = new <a href=\"#!/api/Ext.picker.Color\" rel=\"Ext.picker.Color\" class=\"docClass\">Ext.picker.Color</a>();\ncp.colors[0] = 'FF0000';  // change the first box to red\n</code></pre>\n\n<p>Or you can provide a custom array of your own for complete control:</p>\n\n<pre><code>var cp = new <a href=\"#!/api/Ext.picker.Color\" rel=\"Ext.picker.Color\" class=\"docClass\">Ext.picker.Color</a>();\ncp.colors = ['000000', '993300', '333300'];\n</code></pre>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"clear":{"!type":"fn() -> !this","!doc":"<p>Clears any selection and sets the value to <code>null</code>.</p>\n"},"getValue":{"!type":"fn() -> string","!doc":"<p>Get the currently selected color value.</p>\n"},"handleClick":{"!type":"fn(event: ?) -> !this"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"select":{"!type":"fn(this: ?, color: ?, eOpts: ?)","!doc":"<p>Fires when a color is selected</p>\n"}}},"Date":{"!doc":"<p>A date picker. This class is used by the <a href=\"#!/api/Ext.form.field.Date\" rel=\"Ext.form.field.Date\" class=\"docClass\">Ext.form.field.Date</a> field to allow browsing and selection of valid\ndates in a popup next to the field, but may also be used with other components.</p>\n\n<p>Typically you will need to implement a handler function to be notified when the user chooses a date from the picker;\nyou can register the handler using the <a href=\"#!/api/Ext.picker.Date-event-select\" rel=\"Ext.picker.Date-event-select\" class=\"docClass\">select</a> event, or by implementing the <a href=\"#!/api/Ext.picker.Date-cfg-handler\" rel=\"Ext.picker.Date-cfg-handler\" class=\"docClass\">handler</a> method.</p>\n\n<p>By default the user will be allowed to pick any date; this can be changed by using the <a href=\"#!/api/Ext.picker.Date-cfg-minDate\" rel=\"Ext.picker.Date-cfg-minDate\" class=\"docClass\">minDate</a>,\n<a href=\"#!/api/Ext.picker.Date-cfg-maxDate\" rel=\"Ext.picker.Date-cfg-maxDate\" class=\"docClass\">maxDate</a>, <a href=\"#!/api/Ext.picker.Date-cfg-disabledDays\" rel=\"Ext.picker.Date-cfg-disabledDays\" class=\"docClass\">disabledDays</a>, <a href=\"#!/api/Ext.picker.Date-cfg-disabledDatesRE\" rel=\"Ext.picker.Date-cfg-disabledDatesRE\" class=\"docClass\">disabledDatesRE</a>, and/or <a href=\"#!/api/Ext.picker.Date-cfg-disabledDates\" rel=\"Ext.picker.Date-cfg-disabledDates\" class=\"docClass\">disabledDates</a> configs.</p>\n\n<p>All the string values documented below may be overridden by including an Ext locale file in your page.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Choose a future date:',\n    width: 200,\n    bodyPadding: 10,\n    renderTo: Ext.getBody(),\n    items: [{\n        xtype: 'datepicker',\n        minDate: new Date(),\n        handler: function(picker, date) {\n            // do something with the selected date\n        }\n    }]\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"ariaTitle":{"!type":"string","!doc":"<p>The text to display for the aria title</p>\n"},"ariaTitleDateFormat":{"!type":"string","!doc":"<p>The date format to display for the current value in the <a href=\"#!/api/Ext.picker.Date-cfg-ariaTitle\" rel=\"Ext.picker.Date-cfg-ariaTitle\" class=\"docClass\">ariaTitle</a></p>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this components element.</p>\n"},"border":{"!doc":"<p>Specifies the border size for this component. The border can be a single numeric value to apply to all sides or it can\nbe a CSS style specification for each style, for example: '10 5 3 10' (top, right, bottom, left).</p>\n\n<p>For components that have no border by default, setting this won't make the border appear by itself.\nYou also need to specify border color and style:</p>\n\n<pre><code>border: 5,\nstyle: {\n    borderColor: 'red',\n    borderStyle: 'solid'\n}\n</code></pre>\n\n<p>To turn off the border, use <code>border: false</code>.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"dayNames":{"!doc":"<p>An array of textual day names which can be overriden for localization support (defaults to Ext.Date.dayNames)</p>\n"},"disableAnim":{"!type":"bool","!doc":"<p>True to disable animations when showing the month picker.</p>\n"},"disabledCellCls":{"!type":"string","!doc":"<p>The class to apply to disabled cells.</p>\n"},"disabledDates":{"!doc":"<p>An array of 'dates' to disable, as strings. These strings will be used to build a dynamic regular expression so\nthey are very powerful. Some examples:</p>\n\n<ul>\n<li>['03/08/2003', '09/16/2003'] would disable those exact dates</li>\n<li>['03/08', '09/16'] would disable those days for every year</li>\n<li>['^03/08'] would only match the beginning (useful if you are using short years)</li>\n<li>['03/../2006'] would disable every day in March 2006</li>\n<li>['^03'] would disable every day in every March</li>\n</ul>\n\n\n<p>Note that the format of the dates included in the array should exactly match the <a href=\"#!/api/Ext.picker.Date-cfg-format\" rel=\"Ext.picker.Date-cfg-format\" class=\"docClass\">format</a> config. In order\nto support regular expressions, if you are using a date format that has '.' in it, you will have to escape the\ndot when restricting dates. For example: ['03\\.08\\.03'].</p>\n"},"disabledDatesRE":{"!doc":"<p>JavaScript regular expression used to disable a pattern of dates. The <a href=\"#!/api/Ext.picker.Date-cfg-disabledDates\" rel=\"Ext.picker.Date-cfg-disabledDates\" class=\"docClass\">disabledDates</a>\nconfig will generate this regex internally, but if you specify disabledDatesRE it will take precedence over the\ndisabledDates value.</p>\n"},"disabledDatesText":{"!type":"string","!doc":"<p>The tooltip text to display when the date falls on a disabled date.</p>\n"},"disabledDays":{"!doc":"<p>An array of days to disable, 0-based. For example, [0, 6] disables Sunday and Saturday.</p>\n"},"disabledDaysText":{"!type":"string","!doc":"<p>The tooltip to display when the date falls on a disabled day.</p>\n"},"focusOnShow":{"!type":"bool","!doc":"<p>True to automatically focus the picker on show.</p>\n"},"format":{"!type":"string","!doc":"<p>The default date format string which can be overriden for localization support. The format must be valid\naccording to Ext.Date.parse (defaults to Ext.Date.defaultFormat).</p>\n"},"handler":{"!doc":"<p>Optional. A function that will handle the select event of this picker. The handler is passed the following\nparameters:</p>\n\n<ul>\n<li><code>picker</code> : <a href=\"#!/api/Ext.picker.Date\" rel=\"Ext.picker.Date\" class=\"docClass\">Ext.picker.Date</a></li>\n</ul>\n\n\n<p>This Date picker.</p>\n\n<ul>\n<li><code>date</code> : Date</li>\n</ul>\n\n\n<p>The selected date.</p>\n"},"keyNavConfig":{"!doc":"<p>Specifies optional custom key event handlers for the <a href=\"#!/api/Ext.util.KeyNav\" rel=\"Ext.util.KeyNav\" class=\"docClass\">Ext.util.KeyNav</a> attached to this date picker. Must\nconform to the config format recognized by the <a href=\"#!/api/Ext.util.KeyNav\" rel=\"Ext.util.KeyNav\" class=\"docClass\">Ext.util.KeyNav</a> constructor. Handlers specified in this\nobject will replace default handlers of the same name.</p>\n"},"longDayFormat":{"!type":"string","!doc":"<p>The format for displaying a date in a longer format.</p>\n"},"maxDate":{"!doc":"<p>Maximum allowable date (JavaScript date object)</p>\n"},"maxText":{"!type":"string","!doc":"<p>The error text to display if the maxDate validation fails.</p>\n"},"minDate":{"!doc":"<p>Minimum allowable date (JavaScript date object)</p>\n"},"minText":{"!type":"string","!doc":"<p>The error text to display if the minDate validation fails.</p>\n"},"monthNames":{"!doc":"<p>An array of textual month names which can be overriden for localization support (defaults to Ext.Date.monthNames)</p>\n"},"monthYearFormat":{"!type":"string","!doc":"<p>The date format for the header month</p>\n"},"monthYearText":{"!type":"string","!doc":"<p>The header month selector tooltip</p>\n"},"nextText":{"!type":"string","!doc":"<p>The next month navigation button tooltip</p>\n"},"prevText":{"!type":"string","!doc":"<p>The previous month navigation button tooltip</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.picker.Date-method-getEl\" rel=\"Ext.picker.Date-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.picker.Date-method-getEl\" rel=\"Ext.picker.Date-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.picker.Date-cfg-renderSelectors\" rel=\"Ext.picker.Date-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.picker.Date-cfg-childEls\" rel=\"Ext.picker.Date-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"scope":{"!doc":"<p>The scope (<code>this</code> reference) in which the <code><a href=\"#!/api/Ext.picker.Date-cfg-handler\" rel=\"Ext.picker.Date-cfg-handler\" class=\"docClass\">handler</a></code> function will be called.</p>\n\n<p>Defaults to this DatePicker instance.</p>\n"},"selectedCls":{"!type":"string","!doc":"<p>The class to apply to the selected cell.</p>\n"},"showToday":{"!type":"bool","!doc":"<p>False to hide the footer area containing the Today button and disable the keyboard handler for spacebar that\nselects the current date.</p>\n"},"startDay":{"!type":"number","!doc":"<p>Day index at which the week should begin, 0-based.</p>\n\n<p>Defaults to <code>0</code> (Sunday).</p>\n"},"todayText":{"!type":"string","!doc":"<p>The text to display on the button that selects the current date</p>\n"},"todayTip":{"!type":"string","!doc":"<p>A string used to format the message for displaying in a tooltip over the button that selects the current date.\nThe <code>{0}</code> token in string is replaced by today's date.</p>\n"},"focusOnSelect":{"!type":"bool","!doc":"<p>Set by other components to stop the picker focus being updated when the value changes.</p>\n"},"initHour":{"!type":"number","!doc":"<p>Default value used to initialise each date in the DatePicker.\n<strong>Note:</strong> 12 noon was chosen because it steers well clear of all DST timezone changes.</p>\n"},"numDays":{"!type":"number","!doc":"<p>24-hour format</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"cacheWidth":{"!type":"fn() -> !this"},"createMonthPicker":{"!type":"fn() -> ?","!doc":"<p>Create the month picker instance</p>\n"},"doHideMonthPicker":{"!type":"fn() -> !this"},"doShowMonthPicker":{"!type":"fn() -> !this"},"focus":{"!type":"fn() -> ?","!doc":"<p>Try to focus this component.</p>\n"},"fullUpdate":{"!type":"fn(date: ?) -> !this","!doc":"<p>Update the contents of the picker for a new month</p>\n"},"getActive":{"!type":"fn() -> ?","!doc":"<p>Get the current active date.</p>\n"},"getDayInitial":{"!type":"fn(value: ?) -> string","!doc":"<p>Gets a single character to represent the day of the week</p>\n"},"getRefItems":{"!type":"fn() -> !this"},"getRefOwner":{"!type":"fn() -> !this","!doc":"<p>Keep the tree structure correct for <a href=\"#!/api/Ext.form.field.Picker\" rel=\"Ext.form.field.Picker\" class=\"docClass\">Ext.form.field.Picker</a> input fields which poke a 'pickerField' reference down into their pop-up pickers.</p>\n"},"getSelectedDate":{"!type":"fn(date: ?) -> !this"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Gets the current selected value of the date field</p>\n"},"handleDateClick":{"!type":"fn(e: ?, t: ?) -> !this","!doc":"<p>Respond to a date being clicked in the picker</p>\n"},"handleMouseWheel":{"!type":"fn(e: ?) -> !this","!doc":"<p>Respond to the mouse wheel event</p>\n"},"handleTabClick":{"!type":"fn(e: ?) -> !this"},"hideMonthPicker":{"!type":"fn(animate?: ?) -> ?","!doc":"<p>Hides the month picker, if it's visible.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>private, inherit docs</p>\n"},"initDisabledDays":{"!type":"fn() -> !this","!doc":"<p>Setup the disabled dates regex based on config options</p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>Initialize any events on this component</p>\n"},"onCancelClick":{"!type":"fn() -> !this","!doc":"<p>Respond to a cancel click on the month picker</p>\n"},"onDisable":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the disable operation.\nAfter calling the superclass's <code>onDisable</code>, the Component will be disabled.</p>\n"},"onEnable":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the enable operation.\nAfter calling the superclass's <code>onEnable</code>, the Component will be enabled.</p>\n"},"onOkClick":{"!type":"fn(picker: ?, value: ?) -> !this","!doc":"<p>Respond to an ok click on the month picker</p>\n"},"onRender":{"!type":"fn(container: ?, position: ?) -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"onSelect":{"!type":"fn() -> !this","!doc":"<p>Perform any post-select actions</p>\n"},"onShow":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the show operation. After\ncalling the superclass's onShow, the Component will be visible.</p>\n\n<p>Override in subclasses where more complex behaviour is needed.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.picker.Date-event-show\" rel=\"Ext.picker.Date-event-show\" class=\"docClass\">show</a>.</p>\n"},"runAnimation":{"!type":"fn(isHide: ?) -> !this","!doc":"<p>Run any animation required to hide/show the month picker.</p>\n"},"selectToday":{"!type":"fn() -> ?","!doc":"<p>Sets the current value to today.</p>\n"},"selectedUpdate":{"!type":"fn(date: ?) -> !this","!doc":"<p>Update the selected cell</p>\n"},"setDisabledDates":{"!type":"fn(disabledDates: ?) -> ?","!doc":"<p>Replaces any existing disabled dates with new values and refreshes the DatePicker.</p>\n"},"setDisabledDays":{"!type":"fn(disabledDays: ?) -> ?","!doc":"<p>Replaces any existing disabled days (by index, 0-6) with new values and refreshes the DatePicker.</p>\n"},"setDisabledStatus":{"!type":"fn(disabled: ?) -> !this","!doc":"<p>Set the disabled state of various internal components</p>\n"},"setMaxDate":{"!type":"fn(value: ?) -> ?","!doc":"<p>Replaces any existing <a href=\"#!/api/Ext.picker.Date-cfg-maxDate\" rel=\"Ext.picker.Date-cfg-maxDate\" class=\"docClass\">maxDate</a> with the new value and refreshes the DatePicker.</p>\n"},"setMinDate":{"!type":"fn(value: ?) -> ?","!doc":"<p>Replaces any existing <a href=\"#!/api/Ext.picker.Date-cfg-minDate\" rel=\"Ext.picker.Date-cfg-minDate\" class=\"docClass\">minDate</a> with the new value and refreshes the DatePicker.</p>\n"},"setValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Sets the value of the date field</p>\n"},"shouldAnimate":{"!type":"fn(animate?: ?) -> bool","!doc":"<p>Checks whether a hide/show action should animate</p>\n"},"showMonthPicker":{"!type":"fn(animate?: ?) -> ?","!doc":"<p>Show the month picker</p>\n"},"showNextMonth":{"!type":"fn(e: ?) -> ?","!doc":"<p>Show the next month.</p>\n"},"showNextYear":{"!type":"fn() -> ?","!doc":"<p>Show the next year.</p>\n"},"showPrevMonth":{"!type":"fn(e: ?) -> ?","!doc":"<p>Show the previous month.</p>\n"},"showPrevYear":{"!type":"fn() -> ?","!doc":"<p>Show the previous year.</p>\n"},"update":{"!type":"fn(date: ?, forceRefresh: ?) -> !this","!doc":"<p>Update the contents of the picker</p>\n"},"select":{"!type":"fn(this: ?, date: ?, eOpts: ?)","!doc":"<p>Fires when a date is selected</p>\n"}}},"Month":{"!doc":"<p>A month picker component. This class is used by the <a href=\"#!/api/Ext.picker.Date\" rel=\"Ext.picker.Date\" class=\"docClass\">Date picker</a> class\nto allow browsing and selection of year/months combinations.</p>\n","!type":"fn(config: ?)","prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to the picker element.</p>\n"},"cancelText":{"!type":"string","!doc":"<p>The text to display on the cancel button.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"okText":{"!type":"string","!doc":"<p>The text to display on the ok button.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.picker.Month-method-getEl\" rel=\"Ext.picker.Month-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.picker.Month-method-getEl\" rel=\"Ext.picker.Month-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.picker.Month-cfg-renderSelectors\" rel=\"Ext.picker.Month-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.picker.Month-cfg-childEls\" rel=\"Ext.picker.Month-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"selectedCls":{"!type":"string","!doc":"<p>The class to be added to selected items in the picker.</p>\n"},"showButtons":{"!type":"bool","!doc":"<p>True to show ok and cancel buttons below the picker.</p>\n"},"value":{"!doc":"<p>The default value to set. See <a href=\"#!/api/Ext.picker.Month-method-setValue\" rel=\"Ext.picker.Month-method-setValue\" class=\"docClass\">setValue</a></p>\n"},"measureMaxHeight":{"!type":"number"},"monthOffset":{"!type":"number","!doc":"<p>10 years in total, 2 per row</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"smallCls":{"!type":"string","!doc":"<p>used when attached to date picker which isnt showing buttons</p>\n"},"totalYears":{"!type":"number"},"yearOffset":{"!type":"number"},"adjustYear":{"!type":"fn(offset?: ?) -> !this","!doc":"<p>Modify the year display by passing an offset.</p>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"beforeRender":{"!type":"fn() -> !this"},"cacheWidth":{"!type":"fn() -> !this"},"calculateMonthMargin":{"!type":"fn() -> !this"},"getLargest":{"!type":"fn(months: ?) -> !this"},"getValue":{"!type":"fn() -> ?","!doc":"<p>Gets the selected value. It is returned as an array [month, year]. It may\nbe a partial value, for example [null, 2010]. The month is returned as\n0 based.</p>\n"},"getYear":{"!type":"fn(defaultValue: ?, offset: ?) -> number","!doc":"<p>Gets the current year value, or the default.</p>\n"},"getYears":{"!type":"fn() -> ?","!doc":"<p>Get an array of years to be pushed in the template. It is not in strict\nnumerical order because we want to show them in columns.</p>\n"},"hasSelection":{"!type":"fn() -> bool","!doc":"<p>Checks whether the picker has a selection</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"onBodyClick":{"!type":"fn(e: ?, t: ?) -> !this","!doc":"<p>React to clicks on the body</p>\n"},"onCancelClick":{"!type":"fn() -> !this","!doc":"<p>React to the cancel button being pressed</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onMonthClick":{"!type":"fn(target: ?, isDouble: ?) -> !this","!doc":"<p>React to a month being clicked</p>\n"},"onOkClick":{"!type":"fn() -> !this","!doc":"<p>React to the ok button being pressed</p>\n"},"onYearClick":{"!type":"fn(target: ?, isDouble: ?) -> !this","!doc":"<p>React to a year being clicked</p>\n"},"resolveOffset":{"!type":"fn(index: ?, offset: ?) -> number","!doc":"<p>Returns an offsetted number based on the position in the collection. Since our collections aren't\nnumerically ordered, this function helps to normalize those differences.</p>\n"},"setValue":{"!type":"fn(value: ?) -> ?","!doc":"<p>Set the value for the picker.</p>\n"},"updateBody":{"!type":"fn() -> !this","!doc":"<p>Update the years in the body based on any change</p>\n"},"cancelclick":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fires when the cancel button is pressed.</p>\n"},"monthclick":{"!type":"fn(this: ?, value: ?, eOpts: ?)","!doc":"<p>Fires when a month is clicked.</p>\n"},"monthdblclick":{"!type":"fn(this: ?, value: ?, eOpts: ?)","!doc":"<p>Fires when a month is clicked.</p>\n"},"okclick":{"!type":"fn(this: ?, value: ?, eOpts: ?)","!doc":"<p>Fires when the ok button is pressed.</p>\n"},"select":{"!type":"fn(this: ?, value: ?, eOpts: ?)","!doc":"<p>Fires when a month/year is selected.</p>\n"},"yearclick":{"!type":"fn(this: ?, value: ?, eOpts: ?)","!doc":"<p>Fires when a year is clicked.</p>\n"},"yeardblclick":{"!type":"fn(this: ?, value: ?, eOpts: ?)","!doc":"<p>Fires when a year is clicked.</p>\n"}}},"Time":{"!doc":"<p>A time picker which provides a list of times from which to choose. This is used by the <a href=\"#!/api/Ext.form.field.Time\" rel=\"Ext.form.field.Time\" class=\"docClass\">Ext.form.field.Time</a>\nclass to allow browsing and selection of valid times, but could also be used with other components.</p>\n\n<p>By default, all times starting at midnight and incrementing every 15 minutes will be presented. This list of\navailable times can be controlled using the <a href=\"#!/api/Ext.picker.Time-cfg-minValue\" rel=\"Ext.picker.Time-cfg-minValue\" class=\"docClass\">minValue</a>, <a href=\"#!/api/Ext.picker.Time-cfg-maxValue\" rel=\"Ext.picker.Time-cfg-maxValue\" class=\"docClass\">maxValue</a>, and <a href=\"#!/api/Ext.picker.Time-cfg-increment\" rel=\"Ext.picker.Time-cfg-increment\" class=\"docClass\">increment</a>\nconfiguration properties. The format of the times presented in the list can be customized with the <a href=\"#!/api/Ext.picker.Time-cfg-format\" rel=\"Ext.picker.Time-cfg-format\" class=\"docClass\">format</a>\nconfig.</p>\n\n<p>To handle when the user selects a time from the list, you can subscribe to the <a href=\"#!/api/Ext.picker.Time-event-selectionchange\" rel=\"Ext.picker.Time-event-selectionchange\" class=\"docClass\">selectionchange</a> event.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.picker.Time\" rel=\"Ext.picker.Time\" class=\"docClass\">Ext.picker.Time</a>', {\n   width: 60,\n   minValue: Ext.Date.parse('04:30:00 AM', 'h:i:s A'),\n   maxValue: Ext.Date.parse('08:00:00 AM', 'h:i:s A'),\n   renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"componentCls":{"!type":"string","!doc":"<p>CSS Class to be added to a components root level element to give distinction to it via styling.</p>\n"},"format":{"!type":"string","!doc":"<p>The default time format string which can be overriden for localization support. The format must be valid\naccording to Ext.Date.parse.</p>\n\n<p>Defaults to <code>'g:i A'</code>, e.g., <code>'3:15 PM'</code>. For 24-hour time format try <code>'H:i'</code> instead.</p>\n"},"increment":{"!type":"number","!doc":"<p>The number of minutes between each time value in the list.</p>\n"},"loadMask":{"!type":"bool"},"maxValue":{"!doc":"<p>The maximum time to be shown in the list of times. This must be a Date object (only the time fields will be\nused); no parsing of String values will be done.</p>\n"},"minValue":{"!doc":"<p>The minimum time to be shown in the list of times. This must be a Date object (only the time fields will be\nused); no parsing of String values will be done.</p>\n"},"displayField":{"!type":"string","!doc":"<p>The field in the implicitly-generated Model objects that gets displayed in the list. This is\nan internal field name only and is not useful to change via config.</p>\n"},"initDate":{"!doc":"<p>Year, month, and day that all times will be normalized into internally.</p>\n"},"createStore":{"!type":"fn() -> !this","!doc":"<p>Creates the internal Ext.data.Store that contains the available times. The store\nis loaded with all possible times, and it is later filtered to hide those times outside\nthe minValue/maxValue.</p>\n"},"focusNode":{"!type":"fn(rec: ?) -> !this","!doc":"<p>Focuses a node in the view.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"normalizeDate":{"!type":"fn(date: ?) -> !this","!doc":"<p>Sets the year/month/day of the given Date object to the <a href=\"#!/api/Ext.picker.Time-property-initDate\" rel=\"Ext.picker.Time-property-initDate\" class=\"docClass\">initDate</a>, so that only\nthe time fields are significant. This makes values suitable for time comparison.</p>\n"},"setMaxValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Set the <a href=\"#!/api/Ext.picker.Time-cfg-maxValue\" rel=\"Ext.picker.Time-cfg-maxValue\" class=\"docClass\">maxValue</a> and update the list of available times. This must be a Date object (only the time\nfields will be used); no parsing of String values will be done.</p>\n"},"setMinValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Set the <a href=\"#!/api/Ext.picker.Time-cfg-minValue\" rel=\"Ext.picker.Time-cfg-minValue\" class=\"docClass\">minValue</a> and update the list of available times. This must be a Date object (only the time\nfields will be used); no parsing of String values will be done.</p>\n"},"updateList":{"!type":"fn() -> !this","!doc":"<p>Update the list of available times in the list to be constrained within the <a href=\"#!/api/Ext.picker.Time-cfg-minValue\" rel=\"Ext.picker.Time-cfg-minValue\" class=\"docClass\">minValue</a>\nand <a href=\"#!/api/Ext.picker.Time-cfg-maxValue\" rel=\"Ext.picker.Time-cfg-maxValue\" class=\"docClass\">maxValue</a>.</p>\n"}}}},"ProgressBar":{"!doc":"<p>An updateable progress bar component. The progress bar supports two different modes: manual and automatic.</p>\n\n<p>In manual mode, you are responsible for showing, updating (via <a href=\"#!/api/Ext.ProgressBar-method-updateProgress\" rel=\"Ext.ProgressBar-method-updateProgress\" class=\"docClass\">updateProgress</a>) and clearing the progress bar\nas needed from your own code. This method is most appropriate when you want to show progress throughout an operation\nthat has predictable points of interest at which you can update the control.</p>\n\n<p>In automatic mode, you simply call <a href=\"#!/api/Ext.ProgressBar-method-wait\" rel=\"Ext.ProgressBar-method-wait\" class=\"docClass\">wait</a> and let the progress bar run indefinitely, only clearing it once the\noperation is complete. You can optionally have the progress bar wait for a specific amount of time and then clear\nitself. Automatic mode is most appropriate for timed operations or asynchronous operations in which you have no need\nfor indicating intermediate progress.</p>\n\n<pre class='inline-example '><code>var p = Ext.create('<a href=\"#!/api/Ext.ProgressBar\" rel=\"Ext.ProgressBar\" class=\"docClass\">Ext.ProgressBar</a>', {\n   renderTo: Ext.getBody(),\n   width: 300\n});\n\n// Wait for 5 seconds, then update the status el (progress bar will auto-reset)\np.wait({\n    interval: 500, //bar will move fast!\n    duration: 50000,\n    increment: 15,\n    text: 'Updating...',\n    scope: this,\n    fn: function(){\n        p.updateText('Done!');\n    }\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"animate":{"!doc":"<p>True to animate the progress bar during transitions, or an animation configuration\n(see the <a href=\"#!/api/Ext.ProgressBar-method-animate\" rel=\"Ext.ProgressBar-method-animate\" class=\"docClass\">animate</a> method for details).</p>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to the progress bar's wrapper element.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"componentLayout":{"!doc":"<p>The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout\nmanager which sizes a Component's internal structure in response to the Component being sized.</p>\n\n<p>Generally, developers will not use this configuration as all provided Components which need their internal\nelements sizing (Such as <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">input fields</a>) come with their own componentLayout managers.</p>\n\n<p>The <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used on instances of the base <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>\nclass which simply sizes the Component's encapsulating element to the height and width specified in the\n<a href=\"#!/api/Ext.ProgressBar-method-setSize\" rel=\"Ext.ProgressBar-method-setSize\" class=\"docClass\">setSize</a> method.</p>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"id":{"!type":"string","!doc":"<p>The progress bar element's id (defaults to an auto-generated id)</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.ProgressBar-method-getEl\" rel=\"Ext.ProgressBar-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.ProgressBar-method-getEl\" rel=\"Ext.ProgressBar-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.ProgressBar-cfg-renderSelectors\" rel=\"Ext.ProgressBar-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.ProgressBar-cfg-childEls\" rel=\"Ext.ProgressBar-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"text":{"!type":"string","!doc":"<p>The text shown in the progress bar.</p>\n"},"textEl":{"!doc":"<p>The element to render the progress text to (defaults to the progress bar's internal text element)</p>\n"},"textTpl":{"!doc":"<p>A template used to create this ProgressBar's background text given two values:</p>\n\n<p>   `value  ' - The raw progress value between 0 and 1\n   'percent' - The value as a percentage between 0 and 100</p>\n"},"value":{"!type":"number","!doc":"<p>A floating point value between 0 and 1 (e.g., .5)</p>\n"},"ariaRole":{"!type":"string"},"defaultBindProperty":{"!type":"string"},"waitTimer":{"!doc":"<p>private</p>\n"},"applyText":{"!type":"fn(text: ?) -> !this"},"applyTextTpl":{"!type":"fn(textTpl: ?) -> !this"},"clearTimer":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"getText":{"!type":"fn() -> !this"},"getTextTpl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.ProgressBar-cfg-textTpl\" rel=\"Ext.ProgressBar-cfg-textTpl\" class=\"docClass\">textTpl</a>.</p>\n"},"getValue":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.ProgressBar-cfg-value\" rel=\"Ext.ProgressBar-cfg-value\" class=\"docClass\">value</a>.</p>\n"},"isWaiting":{"!type":"fn() -> bool","!doc":"<p>Returns true if the progress bar is currently in a <a href=\"#!/api/Ext.ProgressBar-method-wait\" rel=\"Ext.ProgressBar-method-wait\" class=\"docClass\">wait</a> operation</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"reset":{"!type":"fn(hide?: ?) -> ?","!doc":"<p>Resets the progress bar value to 0 and text to empty string. If hide = true, the progress bar will also be hidden\n(using the <a href=\"#!/api/Ext.ProgressBar-cfg-hideMode\" rel=\"Ext.ProgressBar-cfg-hideMode\" class=\"docClass\">hideMode</a> property internally).</p>\n"},"setTextTpl":{"!type":"fn(textTpl: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.ProgressBar-cfg-textTpl\" rel=\"Ext.ProgressBar-cfg-textTpl\" class=\"docClass\">textTpl</a>.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.ProgressBar-cfg-value\" rel=\"Ext.ProgressBar-cfg-value\" class=\"docClass\">value</a>.</p>\n"},"updateProgress":{"!type":"fn(value?: ?, text?: ?, animate?: ?) -> ?","!doc":"<p>Updates the progress bar value, and optionally its text.</p>\n\n<p>If the text argument is not specified, then the <a href=\"#!/api/Ext.ProgressBar-cfg-textTpl\" rel=\"Ext.ProgressBar-cfg-textTpl\" class=\"docClass\">textTpl</a> will be used to generate the text.\nIf there is no <code>textTpl</code>, any existing text value will be unchanged. To blank out existing text, pass <code>\"\"</code>.</p>\n\n<p>Note that even if the progress bar value exceeds 1, it will never automatically reset --\nyou are responsible for determining when the progress is complete and\ncalling <a href=\"#!/api/Ext.ProgressBar-method-reset\" rel=\"Ext.ProgressBar-method-reset\" class=\"docClass\">reset</a> to clear and/or hide the control.</p>\n"},"updateText":{"!type":"fn(text?: ?) -> ?","!doc":"<p>Updates the progress bar text. If specified, textEl will be updated, otherwise the progress bar itself will\ndisplay the updated text.</p>\n"},"updateValue":{"!type":"fn(value: ?) -> !this"},"wait":{"!type":"fn(config?: ?) -> ?","!doc":"<p>Initiates an auto-updating progress bar. A duration can be specified, in which case the progress bar will\nautomatically reset after a fixed amount of time and optionally call a callback function if specified. If no\nduration is passed in, then the progress bar will run indefinitely and must be manually cleared by calling\n<a href=\"#!/api/Ext.ProgressBar-method-reset\" rel=\"Ext.ProgressBar-method-reset\" class=\"docClass\">reset</a>.</p>\n\n<p>Example usage:</p>\n\n<pre><code>var p = new <a href=\"#!/api/Ext.ProgressBar\" rel=\"Ext.ProgressBar\" class=\"docClass\">Ext.ProgressBar</a>({\n   renderTo: 'my-el'\n});\n\n//Wait for 5 seconds, then update the status el (progress bar will auto-reset)\nvar p = Ext.create('<a href=\"#!/api/Ext.ProgressBar\" rel=\"Ext.ProgressBar\" class=\"docClass\">Ext.ProgressBar</a>', {\n   renderTo: Ext.getBody(),\n   width: 300\n});\n\n//Wait for 5 seconds, then update the status el (progress bar will auto-reset)\np.wait({\n   interval: 500, //bar will move fast!\n   duration: 50000,\n   increment: 15,\n   text: 'Updating...',\n   scope: this,\n   fn: function(){\n      p.updateText('Done!');\n   }\n});\n\n//Or update indefinitely until some async action completes, then reset manually\np.wait();\nmyAction.on('complete', function(){\n    p.reset();\n    p.updateText('Done!');\n});\n</code></pre>\n"},"update":{"!type":"fn(this: ?, value: ?, text: ?, eOpts: ?)","!doc":"<p>Fires after each update interval</p>\n"}}},"ProgressBarWidget":{"!doc":"<p>An updateable progress bar widget.</p>\n\n<p>In manual mode, you are responsible for showing, updating (via <a href=\"#!/api/Ext.ProgressBarWidget-method-setValue\" rel=\"Ext.ProgressBarWidget-method-setValue\" class=\"docClass\">setValue</a>)\nand clearing the progress bar as needed from your own code. This method is most\nappropriate when you want to show progress throughout an operation that has predictable\npoints of interest at which you can update the control.</p>\n","prototype":{"animate":{"!type":"bool","!doc":"<p>Specify as <code>true</code> to have this progress bar animate to new extent when updated.</p>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to the progress bar's wrapper element.</p>\n"},"text":{"!type":"string","!doc":"<p>The background text</p>\n"},"textCls":{"!type":"string"},"textTpl":{"!doc":"<p>A template used to create this ProgressBar's background text given two values:</p>\n\n<p>   `value  ' - The raw progress value between 0 and 1\n   'percent' - The value as a percentage between 0 and 100</p>\n"},"ui":{"!type":"string"},"value":{"!type":"number","!doc":"<p>A floating point value between 0 and 1 (e.g., .5)</p>\n"},"defaultBindProperty":{"!type":"string"},"template":{},"applyTextTpl":{"!type":"fn(textTpl: ?) -> !this"},"getAnimate":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-animate\" rel=\"Ext.ProgressBarWidget-cfg-animate\" class=\"docClass\">animate</a>.</p>\n"},"getBaseCls":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-baseCls\" rel=\"Ext.ProgressBarWidget-cfg-baseCls\" class=\"docClass\">baseCls</a>.</p>\n"},"getText":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-text\" rel=\"Ext.ProgressBarWidget-cfg-text\" class=\"docClass\">text</a>.</p>\n"},"getTextCls":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-textCls\" rel=\"Ext.ProgressBarWidget-cfg-textCls\" class=\"docClass\">textCls</a>.</p>\n"},"getTextTpl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-textTpl\" rel=\"Ext.ProgressBarWidget-cfg-textTpl\" class=\"docClass\">textTpl</a>.</p>\n"},"getUi":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-ui\" rel=\"Ext.ProgressBarWidget-cfg-ui\" class=\"docClass\">ui</a>.</p>\n"},"getValue":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-value\" rel=\"Ext.ProgressBarWidget-cfg-value\" class=\"docClass\">value</a>.</p>\n"},"setAnimate":{"!type":"fn(animate: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-animate\" rel=\"Ext.ProgressBarWidget-cfg-animate\" class=\"docClass\">animate</a>.</p>\n"},"setBaseCls":{"!type":"fn(baseCls: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-baseCls\" rel=\"Ext.ProgressBarWidget-cfg-baseCls\" class=\"docClass\">baseCls</a>.</p>\n"},"setText":{"!type":"fn(text: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-text\" rel=\"Ext.ProgressBarWidget-cfg-text\" class=\"docClass\">text</a>.</p>\n"},"setTextCls":{"!type":"fn(textCls: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-textCls\" rel=\"Ext.ProgressBarWidget-cfg-textCls\" class=\"docClass\">textCls</a>.</p>\n"},"setTextTpl":{"!type":"fn(textTpl: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-textTpl\" rel=\"Ext.ProgressBarWidget-cfg-textTpl\" class=\"docClass\">textTpl</a>.</p>\n"},"setUi":{"!type":"fn(ui: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-ui\" rel=\"Ext.ProgressBarWidget-cfg-ui\" class=\"docClass\">ui</a>.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.ProgressBarWidget-cfg-value\" rel=\"Ext.ProgressBarWidget-cfg-value\" class=\"docClass\">value</a>.</p>\n"},"updateBaseCls":{"!type":"fn(baseCls: ?, oldBaseCls: ?) -> !this"},"updateText":{"!type":"fn(text: ?) -> !this"},"updateTextCls":{"!type":"fn(textCls: ?) -> !this"},"updateUi":{"!type":"fn(ui: ?, oldUi: ?) -> !this"},"updateValue":{"!type":"fn(value: ?, oldValue: ?) -> !this"}}},"scroll":{"Indicator":{"!doc":"<p>Provides a visual indicator of scroll position while scrolling using a touch scroller</p>\n","!type":"fn(config: ?)","prototype":{"axis":{"!type":"string","!doc":"<p>('x' or 'y')</p>\n"},"containerEl":{"!doc":"<p>The element in which to render the indicator</p>\n"},"hasOpposite":{"!type":"bool","!doc":"<p><code>true</code> if this indicator must leave room for a\nsecond indicator on the opposite axis</p>\n"},"maxScrollPosition":{"!type":"number","!doc":"<p>The scroller's maximum scroll position on the\nconfigured <a href=\"#!/api/Ext.scroll.Indicator-cfg-axis\" rel=\"Ext.scroll.Indicator-cfg-axis\" class=\"docClass\">axis</a></p>\n"},"minLength":{"!type":"number","!doc":"<p>The minimum length for the indicator. Defaults to the\nindicator's \"size\" (the width of vertical or height of horizontal indicators)</p>\n"},"scrollSize":{"!type":"number","!doc":"<p>The scroller's size on the configured <a href=\"#!/api/Ext.scroll.Indicator-cfg-axis\" rel=\"Ext.scroll.Indicator-cfg-axis\" class=\"docClass\">axis</a></p>\n"},"scroller":{"!doc":"<p>The scroller instance</p>\n"},"cls":{"!type":"string"},"hideAnimConfig":{},"length":{"!type":"number","!doc":"<p>The indicator's \"length\" (height for vertical indicators, or width for\nhorizontal indicators)</p>\n"},"margin":{"!type":"number","!doc":"<p>The indicator's margin (the space between the indicator and the container's edge)</p>\n"},"names":{},"size":{"!type":"number","!doc":"<p>The indicator's size (width if vertical, height if horizontal)</p>\n"},"cacheStyles":{"!type":"fn() -> !this"},"destroy":{"!type":"fn() -> !this"},"getAxis":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-axis\" rel=\"Ext.scroll.Indicator-cfg-axis\" class=\"docClass\">axis</a>.</p>\n"},"getContainerEl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-containerEl\" rel=\"Ext.scroll.Indicator-cfg-containerEl\" class=\"docClass\">containerEl</a>.</p>\n"},"getHasOpposite":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-hasOpposite\" rel=\"Ext.scroll.Indicator-cfg-hasOpposite\" class=\"docClass\">hasOpposite</a>.</p>\n"},"getMaxScrollPosition":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-maxScrollPosition\" rel=\"Ext.scroll.Indicator-cfg-maxScrollPosition\" class=\"docClass\">maxScrollPosition</a>.</p>\n"},"getMinLength":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-minLength\" rel=\"Ext.scroll.Indicator-cfg-minLength\" class=\"docClass\">minLength</a>.</p>\n"},"getScrollSize":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-scrollSize\" rel=\"Ext.scroll.Indicator-cfg-scrollSize\" class=\"docClass\">scrollSize</a>.</p>\n"},"getScroller":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-scroller\" rel=\"Ext.scroll.Indicator-cfg-scroller\" class=\"docClass\">scroller</a>.</p>\n"},"hide":{"!type":"fn() -> !this"},"refreshLength":{"!type":"fn() -> !this"},"setAxis":{"!type":"fn(axis: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-axis\" rel=\"Ext.scroll.Indicator-cfg-axis\" class=\"docClass\">axis</a>.</p>\n"},"setContainerEl":{"!type":"fn(containerEl: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-containerEl\" rel=\"Ext.scroll.Indicator-cfg-containerEl\" class=\"docClass\">containerEl</a>.</p>\n"},"setHasOpposite":{"!type":"fn(hasOpposite: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-hasOpposite\" rel=\"Ext.scroll.Indicator-cfg-hasOpposite\" class=\"docClass\">hasOpposite</a>.</p>\n"},"setMaxScrollPosition":{"!type":"fn(maxScrollPosition: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-maxScrollPosition\" rel=\"Ext.scroll.Indicator-cfg-maxScrollPosition\" class=\"docClass\">maxScrollPosition</a>.</p>\n"},"setMinLength":{"!type":"fn(minLength: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-minLength\" rel=\"Ext.scroll.Indicator-cfg-minLength\" class=\"docClass\">minLength</a>.</p>\n"},"setScrollSize":{"!type":"fn(scrollSize: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-scrollSize\" rel=\"Ext.scroll.Indicator-cfg-scrollSize\" class=\"docClass\">scrollSize</a>.</p>\n"},"setScroller":{"!type":"fn(scroller: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.scroll.Indicator-cfg-scroller\" rel=\"Ext.scroll.Indicator-cfg-scroller\" class=\"docClass\">scroller</a>.</p>\n"},"setValue":{"!type":"fn(value: ?) -> !this","!doc":"<p>Sets the value of this scroll indicator.</p>\n"},"show":{"!type":"fn() -> !this"}}},"Manager":{"!doc":"<p>Framework-internal class for managing touch scrolling of Components and providing\nscroll indicators while scrolling.</p>\n","!type":"fn(config: ?)","prototype":{"direction":{"!type":"string","!doc":"<p>The direction to use for\nthe Scroller.</p>\n"},"el":{"!doc":"<p>The element that gets moved when touch-scrolling.  This should\nbe an single child of an overflowing container element (an element that is styled\nwith overflow:auto), and should shrinkwrap around its contents (display: table, or\nposition: absolute)</p>\n"},"minIndicatorLength":{"!type":"number","!doc":"<p>The minimum length for the scroll indicators.</p>\n"},"owner":{"!doc":"<p>The owning component</p>\n"},"translationMethods":{},"destroy":{"!type":"fn() -> !this"},"doRefresh":{"!type":"fn() -> !this"},"getPosition":{"!type":"fn() -> !this"},"initIndicators":{"!type":"fn() -> !this"},"invokeIndicators":{"!type":"fn(name: ?, args: ?, yArgs: ?) -> !this"},"isAxisEnabled":{"!type":"fn(axis: ?) -> !this"},"onDragEnd":{"!type":"fn() -> !this"},"onElementScroll":{"!type":"fn(event: ?, targetEl: ?) -> !this"},"onMouseWheel":{"!type":"fn(e: ?) -> !this"},"onScroll":{"!type":"fn(scroller: ?, x: ?, y: ?) -> !this"},"onScrollEnd":{"!type":"fn() -> !this"},"onScrollStart":{"!type":"fn() -> !this"},"preventDefault":{"!type":"fn(e: ?) -> !this"},"refresh":{"!type":"fn(immediate: ?) -> !this"},"refreshIndicators":{"!type":"fn() -> !this"},"scrollBy":{"!type":"fn(x: ?, y: ?, animate: ?) -> !this"},"scrollIntoView":{"!type":"fn(el: ?, hscroll?: ?, animate?: ?) -> !this","!doc":"<p>Scrolls a descendant element of the scroller into view.</p>\n"},"scrollTo":{"!type":"fn(x: ?, y: ?, animate: ?) -> !this"},"setScrollX":{"!type":"fn(x: ?) -> !this"},"setScrollY":{"!type":"fn(y: ?) -> !this"},"toggleOthers":{"!type":"fn(disabled: ?) -> !this"},"scroll":{"!type":"fn(scrollManager: ?, x: ?, y: ?, eOpts: ?)","!doc":"<p>Fires when scroller is scrolled.</p>\n"}}}},"selection":{"CellModel":{"!doc":"<p>A selection model for <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">grid panels</a> which allows selection of a single cell at a time.</p>\n\n<p>Implements cell based navigation via keyboard.</p>\n\n<pre class='inline-example '><code>var store = Ext.create('Ext.data.Store', {\n    fields  : ['name', 'email', 'phone'],\n    data    : {\n        items : [\n            { name : 'Lisa',  email : 'lisa@simpsons.com',  phone : '555-111-1224' },\n            { name : 'Bart',  email : 'bart@simpsons.com',  phone : '555-222-1234' },\n            { name : 'Homer', email : 'homer@simpsons.com', phone : '555-222-1244' },\n            { name : 'Marge', email : 'marge@simpsons.com', phone : '555-222-1254' }\n        ]\n    },\n    proxy   : {\n        type   : 'memory',\n        reader : {\n            type : 'json',\n            root : 'items'\n        }\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title    : 'Simpsons',\n    store    : store,\n    width    : 400,\n    renderTo : Ext.getBody(),\n    columns  : [\n        { text : 'Name',  dataIndex : 'name'  },\n        { text : 'Email', dataIndex : 'email', flex : 1 },\n        { text : 'Phone', dataIndex : 'phone' }\n    ],\n    selType: 'cellmodel'\n});\n</code></pre>\n","prototype":{"enableKeyNav":{"!type":"bool","!doc":"<p>Turns on/off keyboard navigation within the grid.</p>\n"},"mode":{"!doc":"<p>Mode of selection.  Valid values are:</p>\n\n<ul>\n<li><strong>\"SINGLE\"</strong> - Only allows selecting one item at a time. This is the default.</li>\n</ul>\n\n"},"preventWrap":{"!type":"bool","!doc":"<p>Set this configuration to true to prevent wrapping around of selection as\na user navigates to the first or last column.</p>\n"},"noSelection":{"!doc":"<p>private property to use when firing a deselect when no old selection exists.</p>\n"},"deselect":{"!type":"fn(this: ?, record: ?, row: ?, column: ?, eOpts: ?)","!doc":"<p>Fired after a cell is deselected</p>\n"},"doMove":{"!type":"fn(direction: ?, e: ?) -> !this"},"getCurrentPosition":{"!type":"fn() -> !this","!doc":"<p>Returns the current position in the format {row: row, column: column}</p>\n"},"getHeaderCt":{"!type":"fn() -> !this"},"initKeyNav":{"!type":"fn(view: ?) -> !this"},"isCellSelected":{"!type":"fn(view: ?, row: ?, column: ?) -> !this"},"move":{"!type":"fn(dir: ?, e: ?) -> !this"},"onCellClick":{"!type":"fn(view: ?, cell: ?, cellIndex: ?, record: ?, row: ?, recordIndex: ?, e: ?) -> !this","!doc":"<p>Set the current position based on where the user clicks.\nIMPORTANT* Due to V4.0.0 history, the cellIndex here is the index within ALL columns, including hidden.</p>\n"},"onCellDeselect":{"!type":"fn(position: ?, supressEvent: ?) -> !this","!doc":"<p>notify view that the cell has been deselected to update the ui\nappropriately</p>\n"},"onCellSelect":{"!type":"fn(position: ?, supressEvent: ?) -> !this","!doc":"<p>notify the view that the cell has been selected to update the ui\nappropriately and bring the cell into focus</p>\n"},"onColumnMove":{"!type":"fn(headerCt: ?, header: ?, fromIdx: ?, toIdx: ?) -> !this","!doc":"<p>When grid uses <a href=\"#!/api/Ext.panel.Table-property-optimizedColumnMove\" rel=\"Ext.panel.Table-property-optimizedColumnMove\" class=\"docClass\">optimizedColumnMove</a> (the default), this is added as a\n<a href=\"#!/api/Ext.panel.Table-event-columnmove\" rel=\"Ext.panel.Table-event-columnmove\" class=\"docClass\">columnmove</a> handler to correctly maintain the\nselected column using the same column header.</p>\n\n<p>If optimizedColumnMove === false, (which some grid Features set) then the view is refreshed,\nso this is not added as a handler because the selected column.</p>\n"},"onEditorTab":{"!type":"fn(editingPlugin: ?, e: ?) -> !this"},"onKeyDown":{"!type":"fn(e: ?) -> !this"},"onKeyLeft":{"!type":"fn(e: ?) -> !this"},"onKeyRight":{"!type":"fn(e: ?) -> !this"},"onKeyTab":{"!type":"fn(e: ?, t: ?) -> !this","!doc":"<p>Tab key from the View's KeyNav, <em>not</em> from an editor.</p>\n"},"onKeyUp":{"!type":"fn(e: ?) -> !this"},"onSelectChange":{"!type":"fn(record: ?, isSelected: ?, suppressEvent: ?, commitFn: ?) -> !this"},"onStoreAdd":{"!type":"fn() -> !this","!doc":"<p>when a record is added to a store</p>\n"},"onStoreClear":{"!type":"fn() -> !this","!doc":"<p>when a store is cleared remove all selections\n(if there were any)</p>\n"},"onStoreRemove":{"!type":"fn(store: ?, records: ?, indexes: ?) -> !this","!doc":"<p>Keep selection model in consistent state upon record deletion.</p>\n"},"onUpdate":{"!type":"fn(record: ?) -> !this","!doc":"<p>Called when the contents of the node are updated, perform any processing here.</p>\n"},"onViewRefresh":{"!type":"fn(view: ?) -> !this"},"refresh":{"!type":"fn() -> !this"},"select":{"!type":"fn(this: ?, record: ?, row: ?, column: ?, eOpts: ?)","!doc":"<p>Fired after a cell is selected</p>\n"},"selectByPosition":{"!type":"fn(position: ?, suppressEvent: ?) -> !this"},"selectWithEvent":{"!type":"fn(record: ?, e: ?) -> !this","!doc":"<p>Provides differentiation of logic between MULTI, SIMPLE and SINGLE\nselection modes. Requires that an event be passed so that we can know\nif user held ctrl or shift.</p>\n"},"setCurrentPosition":{"!type":"fn(position: ?, suppressEvent: ?) -> !this","!doc":"<p>Sets the current position</p>\n"}}},"Model":{"prototype":{"allowDeselect":{"!type":"bool","!doc":"<p>Allow users to deselect a record in a DataView, List or Grid.\nOnly applicable when the <a href=\"#!/api/Ext.selection.Model-cfg-mode\" rel=\"Ext.selection.Model-cfg-mode\" class=\"docClass\">mode</a> is 'SINGLE'.</p>\n"},"pruneRemoved":{"!type":"bool","!doc":"<p>Remove records from the selection when they are removed from the store.</p>\n\n<p><strong>Important:</strong> When using <a href=\"#!/api/Ext.toolbar.Paging\" rel=\"Ext.toolbar.Paging\" class=\"docClass\">paging</a> or a Ext.data.BufferedStore,\nrecords which are cached in the Store's data collection may be removed from the Store when pages change,\nor when rows are scrolled out of view. For this reason <code>pruneRemoved</code> should be set to <code>false</code> when using a buffered Store.</p>\n\n<p>Also, when previously pruned pages are returned to the cache, the records objects in the page will be\n<em>new instances</em>, and will not match the instances in the selection model's collection. For this reason,\nyou MUST ensure that the Model definition's idProperty references a unique\nkey because in this situation, records in the Store have their <strong>IDs</strong> compared to records in the SelectionModel\nin order to re-select a record which is scrolled back into view.</p>\n"},"toggleOnClick":{"!type":"bool","!doc":"<p><code>true</code> to toggle the selection state of an item when clicked.\nOnly applicable when the <a href=\"#!/api/Ext.selection.Model-cfg-mode\" rel=\"Ext.selection.Model-cfg-mode\" class=\"docClass\">mode</a> is 'SINGLE'.\nOnly applicable when the <a href=\"#!/api/Ext.selection.Model-cfg-allowDeselect\" rel=\"Ext.selection.Model-cfg-allowDeselect\" class=\"docClass\">allowDeselect</a> is 'true'.</p>\n"},"selected":{"!doc":"<p>A MixedCollection that maintains all of the currently selected records.</p>\n"},"suspendChange":{"!type":"number"},"afterKeyNavigate":{"!type":"fn(e: ?, record: ?) -> !this","!doc":"<p>Private\nCalled after a new record has been navigated to by a keystroke.\nEvent is passed so that shift and ctrl can be handled.</p>\n"},"beforeViewRender":{"!type":"fn(view: ?) -> !this"},"bindComponent":{"!type":"fn() -> !this"},"bindStore":{"!type":"fn(store: ?, initial: ?) -> !this","!doc":"<p>binds the store to the selModel.</p>\n"},"clearSelections":{"!type":"fn() -> !this","!doc":"<p>A fast reset of the selections without firing events, updating the ui, etc.\nFor private usage only.</p>\n"},"deselectAll":{"!type":"fn(suppressEvent?: ?) -> !this","!doc":"<p>Deselects all records in the view.</p>\n"},"deselectDeletedRecords":{"!type":"fn(records: ?) -> !this","!doc":"<p>Called by subclasses to deselect records upon detection of deletion from the store</p>\n"},"deselectDuringSelect":{"!type":"fn(toSelect: ?, selected: ?, suppressEvent: ?) -> !this"},"deselectRange":{"!type":"fn(startRow: ?, endRow: ?) -> !this","!doc":"<p>Deselects a range of rows if the selection model <a href=\"#!/api/Ext.selection.Model-method-isLocked\" rel=\"Ext.selection.Model-method-isLocked\" class=\"docClass\">is not locked</a>.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>cleanup.</p>\n"},"doDeselect":{"!type":"fn(records: ?, suppressEvent: ?) -> !this","!doc":"<p>records can be an index, a record or an array of records</p>\n"},"doMultiSelect":{"!type":"fn(records: ?, keepExisting: ?, suppressEvent: ?) -> !this"},"doSelect":{"!type":"fn(records: ?, keepExisting: ?, suppressEvent: ?) -> !this"},"doSingleSelect":{"!type":"fn(record: ?, suppressEvent: ?) -> !this"},"getCount":{"!type":"fn() -> number","!doc":"<p>Returns the count of selected records.</p>\n"},"getLastFocused":{"!type":"fn() -> !this"},"getLastSelected":{"!type":"fn() -> ?"},"getSelection":{"!type":"fn() -> ?","!doc":"<p>Returns an array of the currently selected records.</p>\n"},"getSelectionMode":{"!type":"fn() -> string","!doc":"<p>Returns the current selectionMode.</p>\n"},"getSelectionStart":{"!type":"fn() -> !this"},"getStoreListeners":{"!type":"fn() -> ?","!doc":"<p>Gets the listeners to bind to a new store.</p>\n"},"hasSelection":{"!type":"fn() -> bool","!doc":"<p>Returns true if there are any a selected records.</p>\n"},"isFocused":{"!type":"fn(record: ?) -> !this","!doc":"<p>Determines if this record is currently focused.</p>\n"},"isLocked":{"!type":"fn() -> bool","!doc":"<p>Returns true if the selections are locked.</p>\n"},"isRangeSelected":{"!type":"fn(from: ?, to: ?) -> bool","!doc":"<p>Returns true if the specified row is selected.</p>\n"},"isSelected":{"!type":"fn(record: ?) -> bool","!doc":"<p>Returns true if the specified row is selected.</p>\n"},"maybeFireSelectionChange":{"!type":"fn(fireEvent: ?) -> !this","!doc":"<p>fire selection change as long as true is not passed\ninto maybeFireSelectionChange</p>\n"},"normalizeRowRange":{"!type":"fn(startRow: ?, endRow: ?) -> !this"},"onEditorKey":{"!type":"fn() -> !this"},"onLastFocusChanged":{"!type":"fn(oldFocused: ?, newFocused: ?) -> !this"},"onPrune":{"!type":"fn() -> !this"},"onStoreLoad":{"!type":"fn() -> !this"},"onStoreRefresh":{"!type":"fn() -> !this"},"onStoreUpdate":{"!type":"fn() -> !this","!doc":"<p>if records are updated</p>\n"},"onVetoUIEvent":{"!type":"fn() -> !this"},"pruneIf":{"!type":"fn() -> !this"},"resolveListenerScope":{"!type":"fn(defaultScope: ?) -> !this"},"resumeChanges":{"!type":"fn() -> !this"},"selectAll":{"!type":"fn(suppressEvent: ?) -> !this","!doc":"<p>Selects all records in the view.</p>\n"},"selectRange":{"!type":"fn(startRow: ?, endRow: ?, keepExisting?: ?) -> !this","!doc":"<p>Selects a range of rows if the selection model <a href=\"#!/api/Ext.selection.Model-method-isLocked\" rel=\"Ext.selection.Model-method-isLocked\" class=\"docClass\">is not locked</a>.\nAll rows in between startRow and endRow are also selected.</p>\n"},"setLastFocused":{"!type":"fn(record: ?) -> !this","!doc":"<p>Sets a record as the last focused record. This does NOT mean\nthat the record has been selected.</p>\n"},"setLocked":{"!type":"fn(locked: ?) -> !this","!doc":"<p>Locks the current selection and disables any changes from happening to the selection.</p>\n"},"setSelectionMode":{"!type":"fn(selMode: ?) -> !this","!doc":"<p>Sets the current selectionMode.</p>\n"},"setSelectionStart":{"!type":"fn(selection: ?) -> !this"},"suspendChanges":{"!type":"fn() -> !this"},"focuschange":{"!type":"fn(this: ?, oldFocused: ?, newFocused: ?, eOpts: ?)","!doc":"<p>Fired when a row is focused</p>\n"},"selectionchange":{"!type":"fn(this: ?, selected: ?, eOpts: ?)","!doc":"<p>Fired after a selection change has occurred</p>\n"},"deselect":{"!type":"fn(records: ?, suppressEvent?: ?) -> !this","!doc":"<p>Deselects a record instance by record instance or index.</p>\n"},"onStoreClear":{"!type":"fn() -> !this","!doc":"<p>when a store is cleared remove all selections\n(if there were any)</p>\n"},"select":{"!type":"fn(records: ?, keepExisting?: ?, suppressEvent?: ?) -> !this","!doc":"<p>Selects a record instance by record instance or index.</p>\n"},"selectWithEvent":{"!type":"fn(record: ?, e: ?) -> !this","!doc":"<p>Provides differentiation of logic between MULTI, SIMPLE and SINGLE\nselection modes. Requires that an event be passed so that we can know\nif user held ctrl or shift.</p>\n"},"mode":{"!doc":"<p>Mode of selection.  Valid values are:</p>\n\n<ul>\n<li><strong>\"SINGLE\"</strong> - Only allows selecting one item at a time.  Use <a href=\"#!/api/Ext.selection.Model-cfg-allowDeselect\" rel=\"Ext.selection.Model-cfg-allowDeselect\" class=\"docClass\">allowDeselect</a> to allow\ndeselecting that item.  Also see <a href=\"#!/api/Ext.selection.Model-cfg-toggleOnClick\" rel=\"Ext.selection.Model-cfg-toggleOnClick\" class=\"docClass\">toggleOnClick</a>. This is the default.</li>\n<li><strong>\"SIMPLE\"</strong> - Allows simple selection of multiple items one-by-one. Each click in grid will either\nselect or deselect an item.</li>\n<li><strong>\"MULTI\"</strong> - Allows complex selection of multiple items using Ctrl and Shift keys.</li>\n</ul>\n\n"},"onStoreAdd":{"!type":"fn() -> !this","!doc":"<p>when a record is added to a store</p>\n"},"onStoreRemove":{"!type":"fn(store: ?, records: ?, index: ?, isMove: ?) -> !this","!doc":"<p>prune records from the SelectionModel if\nthey were selected at the time they were\nremoved.</p>\n"},"refresh":{"!type":"fn() -> !this"},"onSelectChange":{"!type":"fn(record: ?, isSelected: ?, suppressEvent: ?, commitFn: ?) -> !this"},"onUpdate":{"!type":"fn() -> !this","!doc":"<p>Called when the contents of the node are updated, perform any processing here.</p>\n"}},"!doc":"<p>Tracks what records are currently selected in a databound component.</p>\n\n<p>This is an abstract class and is not meant to be directly used. Databound UI widgets such as\n<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Grid</a> and <a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">Tree</a> should subclass <a href=\"#!/api/Ext.selection.Model\" rel=\"Ext.selection.Model\" class=\"docClass\">Ext.selection.Model</a>\nand provide a way to binding to the component.</p>\n\n<p>The abstract methods <code>onSelectChange</code> and <code>onLastFocusChanged</code> should be implemented in these\nsubclasses to update the UI widget.</p>\n"},"CheckboxModel":{"!doc":"<p>A selection model that renders a column of checkboxes that can be toggled to\nselect or deselect rows. The default mode for this selection model is MULTI.</p>\n\n<p>The selection model will inject a header for the checkboxes in the first view\nand according to the <a href=\"#!/api/Ext.selection.CheckboxModel-cfg-injectCheckbox\" rel=\"Ext.selection.CheckboxModel-cfg-injectCheckbox\" class=\"docClass\">injectCheckbox</a> configuration.</p>\n","!type":"fn()","prototype":{"checkOnly":{"!type":"bool","!doc":"<p>True if rows can only be selected by clicking on the checkbox column.</p>\n"},"checkSelector":{"!type":"string","!doc":"<p>The selector for determining whether the checkbox element is clicked. This may be changed to\nallow for a wider area to be clicked, for example, the whole cell for the selector.</p>\n"},"injectCheckbox":{"!doc":"<p>The index at which to insert the checkbox column.\nSupported values are a numeric index, and the strings 'first' and 'last'.</p>\n"},"mode":{"!doc":"<p>Modes of selection.\nValid values are <code>\"SINGLE\"</code>, <code>\"SIMPLE\"</code>, and <code>\"MULTI\"</code>.</p>\n"},"showHeaderCheckbox":{"!type":"bool","!doc":"<p>Configure as <code>false</code> to not display the header checkbox at the top of the column.\nWhen the store is a BufferedStore, this configuration will\nnot be available because the buffered data set does not always contain all data.</p>\n"},"checkerOnCls":{"!type":"string","!doc":"<p>private</p>\n"},"headerWidth":{"!type":"number"},"tdCls":{"!type":"string"},"addCheckbox":{"!type":"fn(initial: ?) -> !this","!doc":"<p>Add the header checkbox to the header row</p>\n"},"beforeViewRender":{"!type":"fn(view: ?) -> !this"},"bindComponent":{"!type":"fn(view: ?) -> !this"},"getHeaderConfig":{"!type":"fn() -> !this","!doc":"<p>Retrieve a configuration to be used in a HeaderContainer.\nThis should be used when injectCheckbox is set to false.</p>\n"},"hasLockedHeader":{"!type":"fn() -> !this"},"maybeFireSelectionChange":{"!type":"fn(fireEvent: ?) -> !this","!doc":"<p>fire selection change as long as true is not passed\ninto maybeFireSelectionChange</p>\n"},"onHeaderClick":{"!type":"fn(headerCt: ?, header: ?, e: ?) -> !this","!doc":"<p>Toggle between selecting all and deselecting all when clicking on\na checkbox header.</p>\n"},"onReconfigure":{"!type":"fn(grid: ?, store: ?, columns: ?) -> !this","!doc":"<p>Handles the grid's reconfigure event.  Adds the checkbox header if the columns have been reconfigured.</p>\n"},"onSelectChange":{"!type":"fn() -> !this","!doc":"<p>Synchronize header checker value as selection changes.</p>\n"},"onStoreAdd":{"!type":"fn() -> !this","!doc":"<p>when a record is added to a store</p>\n"},"onStoreLoad":{"!type":"fn() -> !this"},"onStoreRefresh":{"!type":"fn() -> !this"},"onStoreRemove":{"!type":"fn() -> !this","!doc":"<p>prune records from the SelectionModel if\nthey were selected at the time they were\nremoved.</p>\n"},"processSelection":{"!type":"fn(view: ?, record: ?, item: ?, index: ?, e: ?) -> !this","!doc":"<p>May be overridden by a subclass to process a click in different ways</p>\n"},"refresh":{"!type":"fn() -> !this","!doc":"<p>After refresh, ensure that the header checkbox state matches</p>\n"},"renderEmpty":{"!type":"fn() -> !this"},"renderer":{"!type":"fn(value: ?, metaData: ?, record: ?, rowIndex: ?, colIndex: ?, store: ?, view: ?) -> !this","!doc":"<p>Generates the HTML to be rendered in the injected checkbox column for each row.\nCreates the standard checkbox markup by default; can be overridden to provide custom rendering.\nSee <a href=\"#!/api/Ext.grid.column.Column-cfg-renderer\" rel=\"Ext.grid.column.Column-cfg-renderer\" class=\"docClass\">Ext.grid.column.Column.renderer</a> for description of allowed parameters.</p>\n"},"resumeChanges":{"!type":"fn() -> !this"},"toggleUiHeader":{"!type":"fn(isChecked: ?) -> !this","!doc":"<p>Toggle the ui header between checked and unchecked state.</p>\n"},"updateHeaderState":{"!type":"fn() -> !this"}}},"RowModel":{"prototype":{"enableKeyNav":{"!type":"bool","!doc":"<p>Turns on/off keyboard navigation within the grid.</p>\n"},"ignoreRightMouseSelection":{"!type":"bool","!doc":"<p>True to ignore selections that are made when using the right mouse button if there are\nrecords that are already selected. If no records are selected, selection will continue\nas normal</p>\n"},"deltaScroll":{"!type":"number","!doc":"<p>Number of pixels to scroll to the left/right when pressing\nleft/right keys.</p>\n"},"afterBufferedScrollTo":{"!type":"fn(newIdx: ?, newRecord: ?) -> !this"},"allowRightMouseSelection":{"!type":"fn(e: ?) -> bool","!doc":"<p>Checks whether a selection should proceed based on the ignoreRightMouseSelection\noption.</p>\n"},"getCurrentPosition":{"!type":"fn() -> !this","!doc":"<p>Returns position of the first selected cell in the selection in the format {row: row, column: column}</p>\n"},"getRowsVisible":{"!type":"fn() -> !this","!doc":"<p>Returns the number of rows currently visible on the screen or\nfalse if there were no rows. This assumes that all rows are\nof the same height and the first view is accurate.</p>\n"},"initKeyNav":{"!type":"fn(view: ?) -> !this"},"isCellSelected":{"!type":"fn(view: ?, record: ?, columnHeader: ?) -> !this"},"isRowSelected":{"!type":"fn(record: ?) -> !this"},"onEditorTab":{"!type":"fn(editingPlugin: ?, e: ?) -> !this"},"onKeyDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Navigate one record down. This could be a selection or\ncould be simply focusing a record for discontiguous\nselection. Provides bounds checking.</p>\n"},"onKeyEnd":{"!type":"fn(e: ?) -> !this","!doc":"<p>go to last visible record in grid.</p>\n"},"onKeyEnter":{"!type":"fn() -> !this"},"onKeyHome":{"!type":"fn(e: ?) -> !this","!doc":"<p>go to first visible record in grid.</p>\n"},"onKeyLeft":{"!type":"fn(e: ?) -> !this"},"onKeyPageDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Go one page down from the lastFocused record in the grid.</p>\n"},"onKeyPageUp":{"!type":"fn(e: ?) -> !this","!doc":"<p>Go one page up from the lastFocused record in the grid.</p>\n"},"onKeyRight":{"!type":"fn(e: ?) -> !this"},"onKeySpace":{"!type":"fn(e: ?) -> !this","!doc":"<p>Select/Deselect based on pressing Spacebar.</p>\n"},"onKeyUp":{"!type":"fn(e: ?) -> !this","!doc":"<p>Navigate one record up. This could be a selection or\ncould be simply focusing a record for discontiguous\nselection. Provides bounds checking.</p>\n"},"onLastFocusChanged":{"!type":"fn(oldFocused: ?, newFocused: ?, supressFocus: ?) -> !this","!doc":"<p>Provide indication of what row was last focused via\nthe gridview.</p>\n"},"onPrune":{"!type":"fn(rec: ?) -> !this"},"onRowClick":{"!type":"fn(view: ?, record: ?, row: ?, index: ?, e: ?) -> !this","!doc":"<p>Select the record with the event included so that\nwe can take into account ctrlKey, shiftKey, etc</p>\n"},"onUpdate":{"!type":"fn(record: ?) -> !this","!doc":"<p>Called when the contents of the node are updated, perform any processing here.</p>\n"},"publishSelection":{"!type":"fn(record: ?) -> !this"},"scrollByDeltaX":{"!type":"fn(delta: ?) -> !this"},"selectByPosition":{"!type":"fn(position: ?, keepExisting: ?) -> !this"},"selectNext":{"!type":"fn(keepExisting?: ?, suppressEvent?: ?) -> bool","!doc":"<p>Selects the record immediately following the currently selected record.</p>\n"},"selectPrevious":{"!type":"fn(keepExisting?: ?, suppressEvent?: ?) -> bool","!doc":"<p>Selects the record that precedes the currently selected record.</p>\n"},"beforedeselect":{"!type":"fn(this: ?, record: ?, index: ?, eOpts: ?)","!doc":"<p>Fired before a record is deselected. If any listener returns false, the\ndeselection is cancelled.</p>\n"},"beforeselect":{"!type":"fn(this: ?, record: ?, index: ?, eOpts: ?)","!doc":"<p>Fired before a record is selected. If any listener returns false, the\nselection is cancelled.</p>\n"},"deselect":{"!type":"fn(this: ?, record: ?, index: ?, eOpts: ?)","!doc":"<p>Fired after a record is deselected</p>\n"},"select":{"!type":"fn(this: ?, record: ?, index: ?, eOpts: ?)","!doc":"<p>Fired after a record is selected</p>\n"},"bindComponent":{"!type":"fn(view: ?) -> !this"},"onSelectChange":{"!type":"fn(record: ?, isSelected: ?, suppressEvent: ?, commitFn: ?) -> !this","!doc":"<p>Allow the GridView to update the UI by\nadding/removing a CSS class from the row.</p>\n"},"processSelection":{"!type":"fn(view: ?, record: ?, item: ?, index: ?, e: ?) -> !this","!doc":"<p>May be overridden by a subclass to process a click in different ways</p>\n"}},"!doc":"<p>A selection model for <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">grid panels</a> which allows selection grid rows..</p>\n\n<p>Implements row based navigation via keyboard.</p>\n\n<pre class='inline-example '><code>var store = Ext.create('Ext.data.Store', {\n    fields  : ['name', 'email', 'phone'],\n    data    : {\n        items : [\n            { name : 'Lisa',  email : 'lisa@simpsons.com',  phone : '555-111-1224' },\n            { name : 'Bart',  email : 'bart@simpsons.com',  phone : '555-222-1234' },\n            { name : 'Homer', email : 'homer@simpsons.com', phone : '555-222-1244' },\n            { name : 'Marge', email : 'marge@simpsons.com', phone : '555-222-1254' }\n        ]\n    },\n    proxy   : {\n        type   : 'memory',\n        reader : {\n            type : 'json',\n            root : 'items'\n        }\n    }\n});\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title    : 'Simpsons',\n    store    : store,\n    width    : 400,\n    renderTo : Ext.getBody(),\n    columns  : [\n        { text : 'Name',  dataIndex : 'name'  },\n        { text : 'Email', dataIndex : 'email', flex : 1 },\n        { text : 'Phone', dataIndex : 'phone' }\n    ]\n});\n</code></pre>\n","!type":"fn()"},"DataViewModel":{"prototype":{"enableKeyNav":{"!type":"bool","!doc":"<p>Turns on/off keyboard navigation within the DataView.</p>\n"},"deselectOnContainerClick":{"!type":"bool"},"destroy":{"!type":"fn() -> !this","!doc":"<p>cleanup.</p>\n"},"initKeyNav":{"!type":"fn(view: ?) -> !this"},"onContainerClick":{"!type":"fn() -> !this"},"onItemClick":{"!type":"fn(view: ?, record: ?, item: ?, index: ?, e: ?) -> !this"},"onLastFocusChanged":{"!type":"fn(oldFocus: ?, newFocus: ?, suppressFocus: ?) -> !this"},"onNavKey":{"!type":"fn(step: ?) -> !this"},"onSelectChange":{"!type":"fn(record: ?, isSelected: ?, suppressEvent: ?, commitFn: ?) -> !this","!doc":"<p>Allow the DataView to update the ui</p>\n"},"onUpdate":{"!type":"fn(record: ?) -> !this","!doc":"<p>Called when the contents of the node are updated, perform any processing here.</p>\n"},"beforedeselect":{"!type":"fn(this: ?, record: ?, eOpts: ?)","!doc":"<p>Fired before a record is deselected. If any listener returns false, the\ndeselection is cancelled.</p>\n"},"beforeselect":{"!type":"fn(this: ?, record: ?, eOpts: ?)","!doc":"<p>Fired before a record is selected. If any listener returns false, the\nselection is cancelled.</p>\n"},"deselect":{"!type":"fn(this: ?, record: ?, eOpts: ?)","!doc":"<p>Fired after a record is deselected</p>\n"},"select":{"!type":"fn(this: ?, record: ?, eOpts: ?)","!doc":"<p>Fired after a record is selected</p>\n"}}},"TreeModel":{"!doc":"<p>Adds custom behavior for left/right keyboard navigation for use with a tree.\nDepends on the view having an expand and collapse method which accepts a\nrecord. This selection model is created by default for <a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">Ext.tree.Panel</a>.</p>\n","!type":"fn()","prototype":{"bindStore":{"!type":"fn(store: ?, initial: ?) -> !this","!doc":"<p>binds the store to the selModel.</p>\n"},"navCollapse":{"!type":"fn(e: ?, t: ?) -> !this"},"navExpand":{"!type":"fn(e: ?, t: ?) -> !this"},"onKeyEnter":{"!type":"fn(e: ?, t: ?) -> !this"},"onKeyLeft":{"!type":"fn(e: ?, t: ?) -> !this"},"onKeyRight":{"!type":"fn(e: ?, t: ?) -> !this"},"onKeySpace":{"!type":"fn(e: ?, t: ?) -> !this","!doc":"<p>Select/Deselect based on pressing Spacebar.</p>\n"},"onNodeRemove":{"!type":"fn(parent: ?, node: ?, isMove: ?) -> !this"},"toggleCheck":{"!type":"fn(e: ?) -> !this"}}}},"Shadow":{"!doc":"<p>Simple class that can provide a shadow effect for any element.  Note that the element\nMUST be absolutely positioned, and the shadow does not provide any shimming.  This\nshould be used only in simple cases - for more advanced functionality that can also\nprovide the same shadow effect, see the <a href=\"#!/api/Ext.dom.Layer\" rel=\"Ext.dom.Layer\" class=\"docClass\">Ext.Layer</a> class.</p>\n","!type":"fn(config?: ?)","prototype":{"mode":{"!type":"string","!doc":"<p>The shadow display mode.  Supports the following options:</p>\n\n<ul>\n<li>sides : Shadow displays on both sides and bottom only</li>\n<li>frame : Shadow displays equally on all four sides</li>\n<li>drop : Traditional bottom-right drop shadow</li>\n</ul>\n\n"},"offset":{"!type":"number","!doc":"<p>The number of pixels to offset the shadow from the element</p>\n"},"boxShadowProperty":{"!doc":"<p>private - CSS property to use to set the box shadow</p>\n"},"defaultMode":{"!type":"string","!doc":"<p>private</p>\n"},"localXYNames":{},"getShadowSize":{"!type":"fn() -> ?","!doc":"<p>Returns the shadow size on each side of the element in standard CSS order: top, right, bottom, left;</p>\n"},"hide":{"!type":"fn() -> !this","!doc":"<p>Hides this shadow</p>\n"},"isVisible":{"!type":"fn() -> !this","!doc":"<p>Returns true if the shadow is visible, else false</p>\n"},"realign":{"!type":"fn(left: ?, top: ?, width: ?, height: ?) -> !this","!doc":"<p>Direct alignment when values are already available. Show must be called at least once before\ncalling this method to ensure it is initialized.</p>\n"},"setOpacity":{"!type":"fn(opacity: ?) -> !this","!doc":"<p>Sets the opacity of the shadow</p>\n"},"setZIndex":{"!type":"fn(zindex: ?) -> !this","!doc":"<p>Adjust the z-index of this shadow</p>\n"},"show":{"!type":"fn(targetEl: ?) -> !this","!doc":"<p>Displays the shadow under the target element</p>\n"}}},"ShadowPool":{"!doc":"<p>Private utility class that manages the internal Shadow cache.</p>\n","markup":{},"shadows":{},"pull":{"!type":"fn() -> !this"},"push":{"!type":"fn(sh: ?) -> !this"},"reset":{"!type":"fn() -> !this"}},"slider":{"Multi":{"!doc":"<p>Slider which supports vertical or horizontal orientation, keyboard adjustments, configurable snapping, axis clicking\nand animation. Can be added as an item to any container.</p>\n\n<p>Sliders can be created with more than one thumb handle by passing an array of values instead of a single one:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.slider.Multi\" rel=\"Ext.slider.Multi\" class=\"docClass\">Ext.slider.Multi</a>', {\n    width: 200,\n    values: [25, 50, 75],\n    increment: 5,\n    minValue: 0,\n    maxValue: 100,\n\n    // this defaults to true, setting to false allows the thumbs to pass each other\n    constrainThumbs: false,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"animate":{"!type":"bool","!doc":"<p>Turn on or off animation.</p>\n"},"childEls":{},"clickToChange":{"!type":"bool","!doc":"<p>Determines whether or not clicking on the Slider axis will change the slider.</p>\n"},"constrainThumbs":{"!type":"bool","!doc":"<p>True to disallow thumbs from overlapping one another.</p>\n"},"decimalPrecision":{"!doc":"<p>The number of decimal places to which to round the Slider's value.</p>\n\n<p>To disable rounding, configure as <strong>false</strong>.</p>\n"},"fieldSubTpl":{"!doc":"<p>note: {id} here is really {inputId}, but {cmpId} is available</p>\n"},"increment":{"!type":"number","!doc":"<p>How many units to change the slider when adjusting by drag and drop. Use this option to enable 'snapping'.</p>\n"},"keyIncrement":{"!type":"number","!doc":"<p>How many units to change the Slider when adjusting with keyboard navigation. If the increment\nconfig is larger, it will be used instead.</p>\n"},"maxValue":{"!type":"number","!doc":"<p>The maximum value for the Slider.</p>\n"},"minValue":{"!type":"number","!doc":"<p>The minimum value for the Slider.</p>\n"},"tipText":{"!doc":"<p>A function used to display custom text for the slider tip.</p>\n\n<p>Defaults to null, which will use the default on the plugin.</p>\n"},"useTips":{"!doc":"<p>True to use an <a href=\"#!/api/Ext.slider.Tip\" rel=\"Ext.slider.Tip\" class=\"docClass\">Ext.slider.Tip</a> to display tips for the value. This option may also\nprovide a configuration object for an <a href=\"#!/api/Ext.slider.Tip\" rel=\"Ext.slider.Tip\" class=\"docClass\">Ext.slider.Tip</a>.</p>\n"},"value":{"!type":"number","!doc":"<p>A value with which to initialize the slider. Setting this will only result in the creation\nof a single slider thumb; if you want multiple thumbs then use the <a href=\"#!/api/Ext.slider.Multi-cfg-values\" rel=\"Ext.slider.Multi-cfg-values\" class=\"docClass\">values</a> config instead.</p>\n\n<p>Defaults to <a href=\"#!/api/Ext.slider.Multi-cfg-minValue\" rel=\"Ext.slider.Multi-cfg-minValue\" class=\"docClass\">minValue</a>.</p>\n"},"values":{"!doc":"<p>Array of Number values with which to initalize the slider. A separate slider thumb will be created for each value\nin this array. This will take precedence over the single <a href=\"#!/api/Ext.slider.Multi-cfg-value\" rel=\"Ext.slider.Multi-cfg-value\" class=\"docClass\">value</a> config.</p>\n"},"vertical":{"!type":"bool","!doc":"<p>Orient the Slider vertically rather than horizontally.</p>\n"},"zeroBasedSnapping":{"!type":"bool","!doc":"<p>Set to <code>true</code> to calculate snap points based on <a href=\"#!/api/Ext.slider.Multi-cfg-increment\" rel=\"Ext.slider.Multi-cfg-increment\" class=\"docClass\">increment</a>s from zero as opposed to\nfrom this Slider's <a href=\"#!/api/Ext.slider.Multi-cfg-minValue\" rel=\"Ext.slider.Multi-cfg-minValue\" class=\"docClass\">minValue</a>.</p>\n\n<p>By Default, valid snap points are calculated starting <a href=\"#!/api/Ext.slider.Multi-cfg-increment\" rel=\"Ext.slider.Multi-cfg-increment\" class=\"docClass\">increment</a>s from the <a href=\"#!/api/Ext.slider.Multi-cfg-minValue\" rel=\"Ext.slider.Multi-cfg-minValue\" class=\"docClass\">minValue</a></p>\n"},"ariaRole":{"!type":"string"},"clickRange":{"!doc":"<p>Determines whether or not a click to the slider component is considered to be a user request to change the value. Specified as an array of [top, bottom],\nthe click event's 'top' property is compared to these numbers and the click only considered a change request if it falls within them. e.g. if the 'top'\nvalue of the click event is 4 or 16, the click is not considered a change request as it falls outside of the [5, 15] range</p>\n"},"dragging":{"!type":"bool","!doc":"<p>True while the thumb is in a drag operation</p>\n"},"horizontalProp":{"!type":"string"},"thumbs":{"!doc":"<p>Array containing references to each thumb</p>\n"},"transformTrackPoints":{},"addThumb":{"!type":"fn(value?: ?) -> ?","!doc":"<p>Creates a new thumb and adds it to the slider</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"calculateThumbPosition":{"!type":"fn(v: ?) -> !this","!doc":"<p>Given a value within this Slider's range, calculates a Thumb's percentage CSS position to map that value.</p>\n"},"getNearest":{"!type":"fn(trackPoint: ?) -> ?","!doc":"<p>Returns the nearest thumb to a click event, along with its distance</p>\n"},"getRange":{"!type":"fn() -> !this"},"getRatio":{"!type":"fn() -> number","!doc":"<p>Returns the ratio of pixels to mapped values. e.g. if the slider is 200px wide and maxValue - minValue is 100,\nthe ratio is 2</p>\n"},"getSubTplData":{"!type":"fn() -> ?","!doc":"<p>private override</p>\n"},"getSubmitValue":{"!type":"fn() -> string","!doc":"<p>Returns the value that would be included in a standard form submit for this field. This will be combined with the\nfield's name to form a name=value pair in the <a href=\"#!/api/Ext.slider.Multi-method-getSubmitData\" rel=\"Ext.slider.Multi-method-getSubmitData\" class=\"docClass\">submitted parameters</a>. If an empty string is\nreturned then just the name= will be submitted; if null is returned then nothing will be submitted.</p>\n\n<p>Note that the value returned will have been <a href=\"#!/api/Ext.slider.Multi-method-processRawValue\" rel=\"Ext.slider.Multi-method-processRawValue\" class=\"docClass\">processed</a> but may or may not have been\nsuccessfully <a href=\"#!/api/Ext.slider.Multi-method-validate\" rel=\"Ext.slider.Multi-method-validate\" class=\"docClass\">validated</a>.</p>\n"},"getTrackpoint":{"!type":"fn(xy: ?) -> !this","!doc":"<p>Given an <code>[x, y]</code> position within the slider's track (Points outside the slider's track are coerced to either the minimum or maximum value),\ncalculate how many pixels <strong>from the slider origin</strong> (left for horizontal Sliders and bottom for vertical Sliders) that point is.</p>\n\n<p>If the point is outside the range of the Slider's track, the return value is <code>undefined</code></p>\n"},"getValue":{"!type":"fn(index: ?) -> ?","!doc":"<p>Returns the current value of the slider</p>\n"},"getValues":{"!type":"fn() -> ?","!doc":"<p>Returns an array of values - one for the location of each thumb</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>private override</p>\n"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>Adds keyboard and mouse listeners on this.el. Ignores click events on the internal focus element.</p>\n"},"initValue":{"!type":"fn() -> !this","!doc":"<p>private override</p>\n"},"normalizeValue":{"!type":"fn(value: ?) -> number","!doc":"<p>Returns a snapped, constrained value when given a desired value</p>\n"},"onClickChange":{"!type":"fn(trackPoint: ?) -> !this","!doc":"<p>Moves the thumb to the indicated position.\nOnly changes the value if the click was within this.clickRange.</p>\n"},"onDisable":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onDragEnd":{"!type":"fn() -> !this"},"onDragStart":{"!type":"fn() -> !this"},"onEnable":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"onKeyDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Handler for any keypresses captured by the slider. If the key is UP or RIGHT, the thumb is moved along to the right\nby this.keyIncrement. If DOWN or LEFT it is moved left. Pressing CTRL moves the slider to the end in either direction</p>\n"},"onMouseDown":{"!type":"fn(e: ?) -> !this","!doc":"<p>Mousedown handler for the slider. If the clickToChange is enabled and the click was not on the draggable 'thumb',\nthis calculates the new value of the slider and tells the implementation (Horizontal or Vertical) to move the thumb</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>private</p>\n"},"promoteThumb":{"!type":"fn(topThumb: ?) -> !this","!doc":"<p>Moves the given thumb above all other by increasing its z-index. This is called when as drag\nany thumb, so that the thumb that was just dragged is always at the highest z-index. This is\nrequired when the thumbs are stacked on top of each other at one of the ends of the slider's\nrange, which can result in the user not being able to move any of them.</p>\n"},"reset":{"!type":"fn() -> !this","!doc":"<p>Resets the current field value to the originally loaded value and clears any validation messages. See <a href=\"#!/api/Ext.form.Basic\" rel=\"Ext.form.Basic\" class=\"docClass\">Ext.form.Basic</a>.<a href=\"#!/api/Ext.form.Basic-cfg-trackResetOnLoad\" rel=\"Ext.form.Basic-cfg-trackResetOnLoad\" class=\"docClass\">trackResetOnLoad</a></p>\n"},"reversePercentageValue":{"!type":"fn(pos: ?) -> number","!doc":"<p>Given a Thumb's percentage position along the slider, returns the mapped slider value for that pixel.\nE.g. if we have a slider 200px wide with minValue = 100 and maxValue = 500, reversePercentageValue(25)\nreturns 200</p>\n"},"reversePixelValue":{"!type":"fn(pos: ?) -> number","!doc":"<p>Given a pixel location along the slider, returns the mapped slider value for that pixel.\nE.g. if we have a slider 200px wide with minValue = 100 and maxValue = 500, reversePixelValue(50)\nreturns 200</p>\n"},"setMaxValue":{"!type":"fn(val: ?) -> !this","!doc":"<p>Sets the maximum value for the slider instance. If the current value is more than the maximum value, the current\nvalue will be changed.</p>\n"},"setMinValue":{"!type":"fn(val: ?) -> !this","!doc":"<p>Sets the minimum value for the slider instance. If the current value is less than the minimum value, the current\nvalue will be changed.</p>\n"},"setReadOnly":{"!type":"fn(readOnly: ?) -> !this","!doc":"<p>Sets the read only state of this field.</p>\n"},"setValue":{"!type":"fn(index: ?, value: ?, animate?: ?) -> ?","!doc":"<p>Programmatically sets the value of the Slider. Ensures that the value is constrained within the minValue and\nmaxValue.</p>\n\n<p>Setting a single value:\n    // Set the second slider value, don't animate\n    mySlider.setValue(1, 50, false);</p>\n\n<p>Setting multiple values at once\n    // Set 3 thumb values, animate\n    mySlider.setValue([20, 40, 60], true);</p>\n"},"syncThumbs":{"!type":"fn() -> !this","!doc":"<p>Synchronizes thumbs position to the proper proportion of the total component width based on the current slider\n<a href=\"#!/api/Ext.slider.Multi-cfg-value\" rel=\"Ext.slider.Multi-cfg-value\" class=\"docClass\">value</a>. This will be called automatically when the Slider is resized by a layout, but if it is rendered\nauto width, this method can be called from another resize handler to sync the Slider if necessary.</p>\n"},"beforechange":{"!type":"fn(slider: ?, newValue: ?, oldValue: ?, eOpts: ?)","!doc":"<p>Fires before the slider value is changed. By returning false from an event handler, you can cancel the\nevent and prevent the slider from changing.</p>\n"},"change":{"!type":"fn(slider: ?, newValue: ?, thumb: ?, eOpts: ?)","!doc":"<p>Fires when the slider value is changed.</p>\n"},"changecomplete":{"!type":"fn(slider: ?, newValue: ?, thumb: ?, eOpts: ?)","!doc":"<p>Fires when the slider value is changed by the user and any drag operations have completed.</p>\n"},"drag":{"!type":"fn(slider: ?, e: ?, eOpts: ?)","!doc":"<p>Fires continuously during the drag operation while the mouse is moving.</p>\n"},"dragend":{"!type":"fn(slider: ?, e: ?, eOpts: ?)","!doc":"<p>Fires after the drag operation has completed.</p>\n"},"dragstart":{"!type":"fn(slider: ?, e: ?, eOpts: ?)","!doc":"<p>Fires after a drag operation has started.</p>\n"}}},"Single":{"!doc":"<p>Slider which supports vertical or horizontal orientation, keyboard adjustments, configurable snapping, axis clicking\nand animation. Can be added as an item to any container.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.slider.Single\" rel=\"Ext.slider.Single\" class=\"docClass\">Ext.slider.Single</a>', {\n    width: 200,\n    value: 50,\n    increment: 10,\n    minValue: 0,\n    maxValue: 100,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>The class <a href=\"#!/api/Ext.slider.Single\" rel=\"Ext.slider.Single\" class=\"docClass\">Ext.slider.Single</a> is aliased to <a href=\"#!/api/Ext.slider.Single\" rel=\"Ext.slider.Single\" class=\"docClass\">Ext.Slider</a> for backwards compatibility.</p>\n","!type":"fn(config: ?)","prototype":{"publishOnComplete":{"!type":"bool","!doc":"<p>This controls when the value of the slider is published to the <code>ViewModel</code>. By\ndefault this is done only when the thumb is released (the change is complete). To\ncause this to happen on every change of the thumb position, specify <code>false</code>. This\nsetting is <code>true</code> by default for improved performance on slower devices (such as\nolder browsers or tablets).</p>\n"},"getNearest":{"!type":"fn(trackPoint: ?) -> ?","!doc":"<p>private</p>\n"},"getValue":{"!type":"fn() -> number","!doc":"<p>Returns the current value of the slider</p>\n"},"setValue":{"!type":"fn(value: ?, animate?: ?) -> !this","!doc":"<p>Programmatically sets the value of the Slider. Ensures that the value is constrained within the minValue and\nmaxValue.</p>\n"}}},"Thumb":{"!doc":"<p>Represents a single thumb element on a Slider. This would not usually be created manually and would instead\nbe created internally by an <a href=\"#!/api/Ext.slider.Multi\" rel=\"Ext.slider.Multi\" class=\"docClass\">Multi slider</a>.</p>\n","!type":"fn(config?: ?)","prototype":{"constrain":{"!type":"bool","!doc":"<p>True to constrain the thumb so that it cannot overlap its siblings</p>\n"},"slider":{"!doc":"<p>The slider this thumb is contained within</p>\n"},"destroy":{"!type":"fn() -> !this"},"disable":{"!type":"fn() -> !this","!doc":"<p>Disables the thumb if it is currently enabled</p>\n"},"enable":{"!type":"fn() -> !this","!doc":"<p>Enables the thumb if it is currently disabled</p>\n"},"getElConfig":{"!type":"fn() -> !this"},"getValueFromTracker":{"!type":"fn() -> !this"},"initEvents":{"!type":"fn() -> !this","!doc":"<p>Sets up an <a href=\"#!/api/Ext.dd.DragTracker\" rel=\"Ext.dd.DragTracker\" class=\"docClass\">Ext.dd.DragTracker</a> for this thumb</p>\n"},"move":{"!type":"fn(v: ?, animate: ?) -> !this","!doc":"<p>move the thumb</p>\n"},"onBeforeDragStart":{"!type":"fn(e: ?) -> bool","!doc":"<p>This is tied into the internal <a href=\"#!/api/Ext.dd.DragTracker\" rel=\"Ext.dd.DragTracker\" class=\"docClass\">Ext.dd.DragTracker</a>. If the slider is currently disabled,\nthis returns false to disable the DragTracker too.</p>\n"},"onDrag":{"!type":"fn(e: ?) -> !this","!doc":"<p>This is tied into the internal <a href=\"#!/api/Ext.dd.DragTracker\" rel=\"Ext.dd.DragTracker\" class=\"docClass\">Ext.dd.DragTracker</a>'s onDrag template method. This is called every time\nthe DragTracker detects a drag movement. It updates the Slider's value using the position of the drag</p>\n"},"onDragEnd":{"!type":"fn(e: ?) -> !this","!doc":"<p>This is tied to the internal <a href=\"#!/api/Ext.dd.DragTracker\" rel=\"Ext.dd.DragTracker\" class=\"docClass\">Ext.dd.DragTracker</a>'s onEnd template method. Removes the drag CSS class and\nfires the 'changecomplete' event with the new value</p>\n"},"onDragStart":{"!type":"fn(e: ?) -> !this","!doc":"<p>This is tied into the internal <a href=\"#!/api/Ext.dd.DragTracker\" rel=\"Ext.dd.DragTracker\" class=\"docClass\">Ext.dd.DragTracker</a>'s onStart template method. Adds the drag CSS class\nto the thumb and fires the 'dragstart' event</p>\n"},"onRender":{"!type":"fn() -> !this"},"render":{"!type":"fn() -> !this","!doc":"<p>Renders the thumb into a slider</p>\n"}}},"Tip":{"!doc":"<p>Simple plugin for using an <a href=\"#!/api/Ext.tip.Tip\" rel=\"Ext.tip.Tip\" class=\"docClass\">Ext.tip.Tip</a> with a slider to show the slider value. In general this class is not created\ndirectly, instead pass the <a href=\"#!/api/Ext.slider.Multi-cfg-useTips\" rel=\"Ext.slider.Multi-cfg-useTips\" class=\"docClass\">Ext.slider.Multi.useTips</a> and <a href=\"#!/api/Ext.slider.Multi-cfg-tipText\" rel=\"Ext.slider.Multi-cfg-tipText\" class=\"docClass\">Ext.slider.Multi.tipText</a> configuration\noptions to the slider directly.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.slider.Single\" rel=\"Ext.slider.Single\" class=\"docClass\">Ext.slider.Single</a>', {\n    width: 214,\n    minValue: 0,\n    maxValue: 100,\n    useTips: true,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>Optionally provide your own tip text by passing tipText:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.slider.Single\" rel=\"Ext.slider.Single\" class=\"docClass\">Ext.slider.Single</a>', {\n    width: 214,\n    minValue: 0,\n    maxValue: 100,\n    useTips: true,\n    tipText: function(thumb){\n        return Ext.String.format('**{0}% complete**', thumb.value);\n    },\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"align":{"!type":"string","!doc":"<p>Alignment configuration for the tip to the slider. See Ext.util.Positionable.alignTo. Default\nvalues for alignment are provided by specifying the <a href=\"#!/api/Ext.slider.Tip-cfg-position\" rel=\"Ext.slider.Tip-cfg-position\" class=\"docClass\">position</a> config.</p>\n"},"minWidth":{"!type":"number","!doc":"<p>The minimum width of the tip in pixels.</p>\n"},"offsets":{"!doc":"<p>Offsets for aligning the tip to the slider. See Ext.util.Positionable.alignTo. Default values\nfor offsets are provided by specifying the <a href=\"#!/api/Ext.slider.Tip-cfg-position\" rel=\"Ext.slider.Tip-cfg-position\" class=\"docClass\">position</a> config.</p>\n"},"position":{"!type":"string","!doc":"<p>Sets the position for where the tip will be displayed related to the thumb. This sets\ndefaults for <a href=\"#!/api/Ext.slider.Tip-cfg-align\" rel=\"Ext.slider.Tip-cfg-align\" class=\"docClass\">align</a> and <a href=\"#!/api/Ext.slider.Tip-cfg-offsets\" rel=\"Ext.slider.Tip-cfg-offsets\" class=\"docClass\">offsets</a> configurations. If <a href=\"#!/api/Ext.slider.Tip-cfg-align\" rel=\"Ext.slider.Tip-cfg-align\" class=\"docClass\">align</a> or\n<a href=\"#!/api/Ext.slider.Tip-cfg-offsets\" rel=\"Ext.slider.Tip-cfg-offsets\" class=\"docClass\">offsets</a> configurations are specified, they will override the defaults defined\nby position.</p>\n"},"defaultHorizontalPosition":{"!type":"string"},"defaultVerticalPosition":{"!type":"string"},"isSliderTip":{"!type":"bool"},"getText":{"!type":"fn(thumb: ?) -> string","!doc":"<p>Used to create the text that appears in the Tip's body. By default this just returns the value of the Slider\nThumb that the Tip is attached to. Override to customize.</p>\n"},"init":{"!type":"fn(slider: ?) -> !this"},"onSlide":{"!type":"fn(slider: ?, e: ?, thumb: ?) -> !this","!doc":"<p>Called whenever a dragstart or drag event is received on the associated Thumb.\nAligns the Tip with the Thumb's new position.</p>\n"}}},"Widget":{"!doc":"<p>A Widget-based implementation of a slider.</p>\n","prototype":{"clickToChange":{"!type":"bool","!doc":"<p>Determines whether or not clicking on the Slider axis will change the slider.</p>\n"},"maxValue":{"!type":"number"},"minValue":{"!type":"number"},"publishOnComplete":{"!type":"bool","!doc":"<p>This controls when the value of the slider is published to the <code>ViewModel</code>. By\ndefault this is done only when the thumb is released (the change is complete). To\ncause this to happen on every change of the thumb position, specify <code>false</code>. This\nsetting is <code>true</code> by default for improved performance on slower devices (such as\nolder browsers or tablets).</p>\n"},"twoWayBindProperties":{"!doc":"<p>This object is a map of config property names holding a <code>true</code> if changes to\nthat config should written back to its binding. Most commonly this is used to\nindicate that the <code>value</code> config should be monitored and changes written back\nto the bound value.</p>\n"},"ui":{"!type":"string"},"value":{"!type":"number"},"vertical":{"!type":"bool"},"animateOnSetValue":{"!doc":"<p>This property is set to false onMouseDown and deleted onMouseUp. It is used only\nby applyValue when it passes the animate parameter to setThumbValue.</p>\n"},"decimalPrecision":{"!type":"number"},"defaultBindProperty":{"!type":"string"},"element":{},"horizontalProp":{"!type":"string"},"thumbCls":{"!type":"string"},"transformTrackPoints":{},"applyValue":{"!type":"fn(value: ?) -> !this"},"calculateThumbPosition":{"!type":"fn(v: ?) -> !this","!doc":"<p>Given a value within this Slider's range, calculates a Thumb's percentage CSS position to map that value.</p>\n"},"cancelDrag":{"!type":"fn(e: ?) -> !this"},"captureMouse":{"!type":"fn(onMouseMove: ?, onMouseUp: ?, args: ?, appendArgs: ?) -> !this"},"doMouseMove":{"!type":"fn(e: ?, thumb: ?, changeComplete: ?) -> !this"},"doSetHeight":{"!type":"fn(height: ?) -> !this"},"getClickToChange":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.slider.Widget-cfg-clickToChange\" rel=\"Ext.slider.Widget-cfg-clickToChange\" class=\"docClass\">clickToChange</a>.</p>\n"},"getMaxValue":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.slider.Widget-cfg-maxValue\" rel=\"Ext.slider.Widget-cfg-maxValue\" class=\"docClass\">maxValue</a>.</p>\n"},"getMinValue":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.slider.Widget-cfg-minValue\" rel=\"Ext.slider.Widget-cfg-minValue\" class=\"docClass\">minValue</a>.</p>\n"},"getNearest":{"!type":"fn(trackPoint: ?) -> ?","!doc":"<p>Returns the nearest thumb to a click event, along with its distance</p>\n"},"getPublishOnComplete":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.slider.Widget-cfg-publishOnComplete\" rel=\"Ext.slider.Widget-cfg-publishOnComplete\" class=\"docClass\">publishOnComplete</a>.</p>\n"},"getRange":{"!type":"fn() -> !this"},"getRatio":{"!type":"fn() -> number","!doc":"<p>Returns the ratio of pixels to mapped values. e.g. if the slider is 200px wide and maxValue - minValue is 100,\nthe ratio is 2</p>\n"},"getThumb":{"!type":"fn(ordinal: ?) -> !this"},"getThumbPositionStyle":{"!type":"fn() -> !this"},"getTrackpoint":{"!type":"fn(xy: ?) -> !this","!doc":"<p>Given an <code>[x, y]</code> position within the slider's track (Points outside the slider's track are coerced to either the minimum or maximum value),\ncalculate how many pixels <strong>from the slider origin</strong> (left for horizontal Sliders and bottom for vertical Sliders) that point is.</p>\n\n<p>If the point is outside the range of the Slider's track, the return value is <code>undefined</code></p>\n"},"getUi":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.slider.Widget-cfg-ui\" rel=\"Ext.slider.Widget-cfg-ui\" class=\"docClass\">ui</a>.</p>\n"},"getValue":{"!type":"fn(index: ?) -> ?","!doc":"<p>Returns the current value of the slider</p>\n"},"getValues":{"!type":"fn() -> ?","!doc":"<p>Returns an array of values - one for the location of each thumb</p>\n"},"getVertical":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.slider.Widget-cfg-vertical\" rel=\"Ext.slider.Widget-cfg-vertical\" class=\"docClass\">vertical</a>.</p>\n"},"moveThumb":{"!type":"fn(thumb: ?, v: ?, animate: ?) -> !this","!doc":"<p>move the thumb</p>\n"},"normalizeValue":{"!type":"fn(value: ?) -> number","!doc":"<p>Returns a snapped, constrained value when given a desired value</p>\n"},"onClickChange":{"!type":"fn(trackPoint: ?) -> !this","!doc":"<p>Moves the thumb to the indicated position.\nOnly changes the value if the click was within this.clickRange.</p>\n"},"onMouseDown":{"!type":"fn(e: ?) -> !this"},"onMouseMove":{"!type":"fn(e: ?, thumb: ?) -> !this"},"onMouseUp":{"!type":"fn(e: ?, thumb: ?) -> !this"},"promoteThumb":{"!type":"fn(topThumb: ?) -> !this","!doc":"<p>Moves the given thumb above all other by increasing its z-index. This is called when as drag\nany thumb, so that the thumb that was just dragged is always at the highest z-index. This is\nrequired when the thumbs are stacked on top of each other at one of the ends of the slider's\nrange, which can result in the user not being able to move any of them.</p>\n"},"reversePercentageValue":{"!type":"fn(pos: ?) -> number","!doc":"<p>Given a Thumb's percentage position along the slider, returns the mapped slider value for that pixel.\nE.g. if we have a slider 200px wide with minValue = 100 and maxValue = 500, reversePercentageValue(25)\nreturns 200</p>\n"},"reversePixelValue":{"!type":"fn(pos: ?) -> number","!doc":"<p>Given a pixel location along the slider, returns the mapped slider value for that pixel.\nE.g. if we have a slider 200px wide with minValue = 100 and maxValue = 500, reversePixelValue(50)\nreturns 200</p>\n"},"setClickToChange":{"!type":"fn(clickToChange: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.slider.Widget-cfg-clickToChange\" rel=\"Ext.slider.Widget-cfg-clickToChange\" class=\"docClass\">clickToChange</a>.</p>\n"},"setMaxValue":{"!type":"fn(maxValue: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.slider.Widget-cfg-maxValue\" rel=\"Ext.slider.Widget-cfg-maxValue\" class=\"docClass\">maxValue</a>.</p>\n"},"setMinValue":{"!type":"fn(minValue: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.slider.Widget-cfg-minValue\" rel=\"Ext.slider.Widget-cfg-minValue\" class=\"docClass\">minValue</a>.</p>\n"},"setPublishOnComplete":{"!type":"fn(publishOnComplete: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.slider.Widget-cfg-publishOnComplete\" rel=\"Ext.slider.Widget-cfg-publishOnComplete\" class=\"docClass\">publishOnComplete</a>.</p>\n"},"setThumbValue":{"!type":"fn(index: ?, value: ?, animate?: ?) -> ?","!doc":"<p>Programmatically sets the value of the Slider. Ensures that the value is constrained within the minValue and\nmaxValue.</p>\n\n<p>Setting a single value:\n    // Set the second slider value, don't animate\n    mySlider.setThumbValue(1, 50, false);</p>\n\n<p>Setting multiple values at once\n    // Set 3 thumb values, animate\n    mySlider.setThumbValue([20, 40, 60], true);</p>\n"},"setUi":{"!type":"fn(ui: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.slider.Widget-cfg-ui\" rel=\"Ext.slider.Widget-cfg-ui\" class=\"docClass\">ui</a>.</p>\n"},"setValue":{"!type":"fn(value: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.slider.Widget-cfg-value\" rel=\"Ext.slider.Widget-cfg-value\" class=\"docClass\">value</a>.</p>\n"},"setVertical":{"!type":"fn(vertical: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.slider.Widget-cfg-vertical\" rel=\"Ext.slider.Widget-cfg-vertical\" class=\"docClass\">vertical</a>.</p>\n"},"update":{"!type":"fn() -> !this","!doc":"<p>// TODO: RTL\ngetRenderTree: function() {\n    var me = this,\n        rtl = me.rtl;</p>\n\n<pre><code>if (rtl &amp;&amp; Ext.rtl) {\n    me.baseCls += ' ' + (Ext.rtl.util.Renderable.prototype._rtlCls);\n    me.horizontalProp = 'right';\n} else if (rtl === false) {\n    me.addCls(Ext.rtl.util.Renderable.prototype._ltrCls);\n}\n\nreturn me.callParent();\n</code></pre>\n\n<p>},</p>\n"},"updateVertical":{"!type":"fn(vertical: ?, oldVertical: ?) -> !this"}}}},"tip":{"Tip":{"prototype":{"autoRender":{"!doc":"<p>private panel overrides</p>\n"},"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"closable":{"!type":"bool","!doc":"<p>True to render a close tool button into the tooltip header.</p>\n"},"closeAction":{"!type":"string","!doc":"<p>The action to take when the close header tool is clicked:</p>\n\n<ul>\n<li><p><strong><a href=\"#!/api/Ext.tip.Tip-method-destroy\" rel=\"Ext.tip.Tip-method-destroy\" class=\"docClass\">destroy</a></strong> : <a href=\"#!/api/Ext.tip.Tip-method-remove\" rel=\"Ext.tip.Tip-method-remove\" class=\"docClass\">remove</a> the window from the DOM and\n<a href=\"#!/api/Ext.Component-method-destroy\" rel=\"Ext.Component-method-destroy\" class=\"docClass\">destroy</a> it and all descendant Components. The\nwindow will <strong>not</strong> be available to be redisplayed via the <a href=\"#!/api/Ext.tip.Tip-method-show\" rel=\"Ext.tip.Tip-method-show\" class=\"docClass\">show</a> method.</p></li>\n<li><p><strong><a href=\"#!/api/Ext.tip.Tip-method-hide\" rel=\"Ext.tip.Tip-method-hide\" class=\"docClass\">hide</a></strong> : <strong>Default.</strong> <a href=\"#!/api/Ext.tip.Tip-method-hide\" rel=\"Ext.tip.Tip-method-hide\" class=\"docClass\">hide</a> the window by setting visibility\nto hidden and applying negative offsets. The window will be available to be\nredisplayed via the <a href=\"#!/api/Ext.tip.Tip-method-show\" rel=\"Ext.tip.Tip-method-show\" class=\"docClass\">show</a> method.</p></li>\n</ul>\n\n\n<p><strong>Note:</strong> This behavior has changed! setting <em>does</em> affect the <a href=\"#!/api/Ext.tip.Tip-method-close\" rel=\"Ext.tip.Tip-method-close\" class=\"docClass\">close</a> method\nwhich will invoke the approriate closeAction.</p>\n"},"constrainPosition":{"!type":"bool","!doc":"<p>If <code>true</code>, then the tooltip will be automatically constrained to stay within\nthe browser viewport.</p>\n"},"defaultAlign":{"!type":"string","!doc":"<p><strong>Experimental</strong>. The default Ext.util.Positionable.alignTo anchor position value\nfor this tip relative to its element of origin.</p>\n"},"floating":{"!type":"bool","!doc":"<p>Specify as true to float the Component outside of the document flow using CSS absolute positioning.</p>\n\n<p>Components such as <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Window</a>s and <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Menu</a>s are floating by default.</p>\n\n<p>Floating Components that are programatically <a href=\"#!/api/Ext.Component-method-render\" rel=\"Ext.Component-method-render\" class=\"docClass\">rendered</a> will register\nthemselves with the global <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a></p>\n\n<h3>Floating Components as child items of a Container</h3>\n\n<p>A floating Component may be used as a child item of a Container. This just allows the floating Component to seek\na ZIndexManager by examining the ownerCt chain.</p>\n\n<p>When configured as floating, Components acquire, at render time, a <a href=\"#!/api/Ext.ZIndexManager\" rel=\"Ext.ZIndexManager\" class=\"docClass\">ZIndexManager</a> which\nmanages a stack of related floating Components. The ZIndexManager brings a single floating Component to the top\nof its stack when the Component's <a href=\"#!/api/Ext.tip.Tip-method-toFront\" rel=\"Ext.tip.Tip-method-toFront\" class=\"docClass\">toFront</a> method is called.</p>\n\n<p>The ZIndexManager is found by traversing up the <a href=\"#!/api/Ext.tip.Tip-property-ownerCt\" rel=\"Ext.tip.Tip-property-ownerCt\" class=\"docClass\">ownerCt</a> chain to find an ancestor which itself is\nfloating. This is so that descendant floating Components of floating <em>Containers</em> (Such as a ComboBox dropdown\nwithin a Window) can have its zIndex managed relative to any siblings, but always <strong>above</strong> that floating\nancestor Container.</p>\n\n<p>If no floating ancestor is found, a floating Component registers itself with the default <a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">ZIndexManager</a>.</p>\n\n<p>Floating components <em>do not participate in the Container's layout</em>. Because of this, they are not rendered until\nyou explicitly <a href=\"#!/api/Ext.tip.Tip-method-show\" rel=\"Ext.tip.Tip-method-show\" class=\"docClass\">show</a> them.</p>\n\n<p>After rendering, the ownerCt reference is deleted, and the <a href=\"#!/api/Ext.tip.Tip-property-floatParent\" rel=\"Ext.tip.Tip-property-floatParent\" class=\"docClass\">floatParent</a> property is set to the found\nfloating ancestor Container. If no floating ancestor Container was found the <a href=\"#!/api/Ext.tip.Tip-property-floatParent\" rel=\"Ext.tip.Tip-property-floatParent\" class=\"docClass\">floatParent</a> property will\nnot be set.</p>\n"},"focusOnToFront":{"!type":"bool","!doc":"<p>Specifies whether the floated component should be automatically <a href=\"#!/api/Ext.Component-method-focus\" rel=\"Ext.Component-method-focus\" class=\"docClass\">focused</a> when\nit is <a href=\"#!/api/Ext.tip.Tip-method-toFront\" rel=\"Ext.tip.Tip-method-toFront\" class=\"docClass\">brought to the front</a>.</p>\n"},"frameHeader":{"!type":"bool","!doc":"<p>True to apply a frame to the panel panels header (if 'frame' is true).</p>\n"},"hidden":{"!type":"bool","!doc":"<p><code>true</code> to hide the component.</p>\n"},"maxWidth":{"!type":"number","!doc":"<p>The maximum width of the tip in pixels.</p>\n"},"shadow":{"!doc":"<p><code>true</code> or \"sides\" for the default effect, \"frame\" for 4-way shadow, and \"drop\"\nfor bottom-right shadow.</p>\n"},"width":{"!type":"number","!doc":"<p>Width in pixels of the tip.  Width will be ignored if it\nexceeds the bounds of <a href=\"#!/api/Ext.tip.Tip-cfg-minWidth\" rel=\"Ext.tip.Tip-cfg-minWidth\" class=\"docClass\">minWidth</a> or <a href=\"#!/api/Ext.tip.Tip-cfg-maxWidth\" rel=\"Ext.tip.Tip-cfg-maxWidth\" class=\"docClass\">maxWidth</a>.</p>\n"},"alwaysFramed":{"!type":"bool","!doc":"<p>Flag to Renderable to always look up the framing styles for this Component</p>\n"},"ghost":{"!doc":"<p>Tip does not ghost. Drag is \"live\"</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"unghost":{},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initDraggable":{"!type":"fn() -> !this","!doc":"<p>Set Tip draggable using base Component's draggability.</p>\n"},"showAt":{"!type":"fn(xy: ?) -> !this","!doc":"<p>Shows this tip at the specified XY position.  Example usage:</p>\n\n<pre><code>// Show the tip at x:50 and y:100\ntip.showAt([50,100]);\n</code></pre>\n"},"minWidth":{"!type":"number","!doc":"<p>The minimum width of the tip in pixels.</p>\n"}},"!doc":"<p>This is the base class for <a href=\"#!/api/Ext.tip.QuickTip\" rel=\"Ext.tip.QuickTip\" class=\"docClass\">Ext.tip.QuickTip</a> and <a href=\"#!/api/Ext.tip.ToolTip\" rel=\"Ext.tip.ToolTip\" class=\"docClass\">Ext.tip.ToolTip</a> that provides the basic layout and\npositioning that all tip-based classes require. This class can be used directly for simple, statically-positioned\ntips that are displayed programmatically, or it can be extended to provide custom tip implementations.</p>\n","!type":"fn(config: ?)"},"QuickTip":{"!doc":"<p>A specialized tooltip class for tooltips that can be specified in markup and automatically managed\nby the global <a href=\"#!/api/Ext.tip.QuickTipManager\" rel=\"Ext.tip.QuickTipManager\" class=\"docClass\">Ext.tip.QuickTipManager</a> instance.  See the QuickTipManager documentation for\nadditional usage details and examples.</p>\n","!type":"fn(config: ?)","prototype":{"interceptTitles":{"!type":"bool","!doc":"<p><code>true</code> to automatically use the element's DOM title value if available.</p>\n"},"shrinkWrapDock":{"!doc":"<p>Allows for this panel to include the <a href=\"#!/api/Ext.tip.QuickTip-cfg-dockedItems\" rel=\"Ext.tip.QuickTip-cfg-dockedItems\" class=\"docClass\">dockedItems</a> when trying to determine the overall\nsize of the panel. This option is only applicable when this panel is also shrink wrapping in the\nsame dimensions. See <a href=\"#!/api/Ext.Component-cfg-shrinkWrap\" rel=\"Ext.Component-cfg-shrinkWrap\" class=\"docClass\">Ext.Component.shrinkWrap</a> for an explanation of the configuration options.</p>\n"},"target":{"!doc":"<p>The target HTMLElement, Ext.dom.Element or id to associate with this Quicktip.</p>\n\n<p>Defaults to the document.</p>\n"},"title":{"!type":"string","!doc":"<p>Force creation of header Component</p>\n"},"tagConfig":{},"cancelShow":{"!type":"fn(el: ?) -> !this","!doc":"<p>Hides a visible tip or cancels an impending show for a particular element.</p>\n"},"doTargetOver":{"!type":"fn(target: ?, xy: ?, event: ?) -> !this"},"getTipCfg":{"!type":"fn(target: ?, event: ?) -> !this","!doc":"<p>Reads the tip text from the closest node to the event target which contains the\nattribute we are configured to look for. Returns an object containing the text\nfrom the attribute, and the target element from which the text was read.</p>\n"},"hide":{"!type":"fn() -> !this","!doc":"<p>Hides this tooltip if visible.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"onTargetOut":{"!type":"fn(e: ?) -> !this"},"onTargetOver":{"!type":"fn(event: ?) -> !this"},"register":{"!type":"fn(config: ?) -> !this","!doc":"<p>Configures a new quick tip instance and assigns it to a target element.</p>\n\n<p>For example usage, see the <a href=\"#!/api/Ext.tip.QuickTipManager\" rel=\"Ext.tip.QuickTipManager\" class=\"docClass\">Ext.tip.QuickTipManager</a> class header.</p>\n"},"show":{"!type":"fn() -> !this","!doc":"<p>Shows this tooltip at the current event target XY position.</p>\n"},"showAt":{"!type":"fn(xy: ?) -> !this","!doc":"<p>Shows this tip at the specified XY position.  Example usage:</p>\n\n<pre><code>// Show the tip at x:50 and y:100\ntip.showAt([50,100]);\n</code></pre>\n"},"targetTextEmpty":{"!type":"fn() -> !this"},"unregister":{"!type":"fn(el: ?) -> !this","!doc":"<p>Removes this quick tip from its element and destroys it.</p>\n"}}},"ToolTip":{"prototype":{"anchor":{"!type":"string","!doc":"<p>If specified, indicates that the tip should be anchored to a\nparticular side of the target element or mouse pointer (\"top\", \"right\", \"bottom\",\nor \"left\"), with an arrow pointing back at the target or mouse pointer. If\n<a href=\"#!/api/Ext.tip.ToolTip-cfg-constrainPosition\" rel=\"Ext.tip.ToolTip-cfg-constrainPosition\" class=\"docClass\">constrainPosition</a> is enabled, this will be used as a preferred value\nonly and may be flipped as needed.</p>\n"},"anchorOffset":{"!type":"number","!doc":"<p>A numeric pixel value used to offset the default position of the anchor arrow.  When the anchor\nposition is on the top or bottom of the tooltip, <code>anchorOffset</code> will be used as a horizontal offset.\nLikewise, when the anchor position is on the left or right side, <code>anchorOffset</code> will be used as\na vertical offset.</p>\n"},"anchorToTarget":{"!type":"bool","!doc":"<p>True to anchor the tooltip to the target element, false to anchor it relative to the mouse coordinates.\nWhen <code>anchorToTarget</code> is true, use <code><a href=\"#!/api/Ext.tip.ToolTip-cfg-defaultAlign\" rel=\"Ext.tip.ToolTip-cfg-defaultAlign\" class=\"docClass\">defaultAlign</a></code> to control tooltip alignment to the\ntarget element.  When <code>anchorToTarget</code> is false, use <code><a href=\"#!/api/Ext.tip.ToolTip-cfg-anchor\" rel=\"Ext.tip.ToolTip-cfg-anchor\" class=\"docClass\">anchor</a></code> instead to control alignment.</p>\n"},"autoHide":{"!type":"bool","!doc":"<p>True to automatically hide the tooltip after the\nmouse exits the target element or after the <code><a href=\"#!/api/Ext.tip.ToolTip-cfg-dismissDelay\" rel=\"Ext.tip.ToolTip-cfg-dismissDelay\" class=\"docClass\">dismissDelay</a></code>\nhas expired if set.  If <code><a href=\"#!/api/Ext.tip.ToolTip-cfg-closable\" rel=\"Ext.tip.ToolTip-cfg-closable\" class=\"docClass\">closable</a> = true</code>\na close tool button will be rendered into the tooltip header.</p>\n"},"delegate":{"!type":"string","!doc":"<p>A DomQuery simple selector which allows selection of individual elements within the\n<code><a href=\"#!/api/Ext.tip.ToolTip-cfg-target\" rel=\"Ext.tip.ToolTip-cfg-target\" class=\"docClass\">target</a></code> element to trigger showing and hiding the ToolTip as the mouse moves within the\ntarget. See Ext.dom.Query for information about simple selectors.</p>\n\n<p>When specified, the child element of the target which caused a show event is placed into the\n<code><a href=\"#!/api/Ext.tip.ToolTip-property-triggerElement\" rel=\"Ext.tip.ToolTip-property-triggerElement\" class=\"docClass\">triggerElement</a></code> property before the ToolTip is shown.</p>\n\n<p>This may be useful when a Component has regular, repeating elements in it, each of which need a\nToolTip which contains information specific to that element.</p>\n\n<p>See the delegate example in class documentation of <a href=\"#!/api/Ext.tip.ToolTip\" rel=\"Ext.tip.ToolTip\" class=\"docClass\">Ext.tip.ToolTip</a>.</p>\n"},"dismissDelay":{"!type":"number","!doc":"<p>Delay in milliseconds before the tooltip automatically hides. To disable automatic hiding, set\ndismissDelay = 0.</p>\n"},"fadeOutDuration":{"!type":"number","!doc":"<p>The number of milliseconds for the <code>fadeOut</code> animation. Only valid if <code>hideAction</code>\nis set to <code>fadeOut</code>.</p>\n"},"hideAction":{"!type":"string","!doc":"<p>The method to use to hide the tooltip. Another useful method for this is <code>fadeOut</code>.</p>\n"},"hideDelay":{"!type":"number","!doc":"<p>Delay in milliseconds after the mouse exits the target element but before the tooltip actually hides.\nSet to 0 for the tooltip to hide immediately.</p>\n"},"mouseOffset":{"!doc":"<p>An XY offset from the mouse position where the tooltip should be shown.</p>\n"},"showDelay":{"!type":"number","!doc":"<p>Delay in milliseconds before the tooltip displays after the mouse enters the target element.</p>\n"},"trackMouse":{"!type":"bool","!doc":"<p>True to have the tooltip follow the mouse as it moves over the target element.</p>\n"},"_timerNames":{},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"quickShowInterval":{"!type":"number"},"targetCounter":{"!type":"number"},"triggerElement":{"!doc":"<p>When a ToolTip is configured with the <code><a href=\"#!/api/Ext.tip.ToolTip-cfg-delegate\" rel=\"Ext.tip.ToolTip-cfg-delegate\" class=\"docClass\">delegate</a></code>\noption to cause selected child elements of the <code><a href=\"#!/api/Ext.tip.ToolTip-cfg-target\" rel=\"Ext.tip.ToolTip-cfg-target\" class=\"docClass\">target</a></code>\nElement to each trigger a separate show event, this property is set to\nthe DOM element which triggered the show.</p>\n"},"beforeDestroy":{"!type":"fn() -> !this","!doc":"<p>Invoked before the Component is destroyed.</p>\n"},"clearTimer":{"!type":"fn(name: ?) -> !this"},"clearTimers":{"!type":"fn() -> !this"},"delayHide":{"!type":"fn() -> !this"},"delayShow":{"!type":"fn(trackMouse: ?) -> !this"},"doEnable":{"!type":"fn() -> !this"},"fadeOut":{"!type":"fn() -> !this"},"getAnchorAlign":{"!type":"fn() -> !this"},"getAnchorPosition":{"!type":"fn() -> !this"},"getMouseOffset":{"!type":"fn() -> !this"},"getOffsets":{"!type":"fn() -> !this"},"getTargetXY":{"!type":"fn() -> !this"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onDisable":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the disable operation.\nAfter calling the superclass's <code>onDisable</code>, the Component will be disabled.</p>\n"},"onDocMouseDown":{"!type":"fn(e: ?) -> !this"},"onHide":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the hide operation. After\ncalling the superclass's onHide, the Component will be hidden.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.tip.ToolTip-event-hide\" rel=\"Ext.tip.ToolTip-event-hide\" class=\"docClass\">hide</a>.</p>\n"},"onMouseMove":{"!type":"fn(e: ?) -> !this"},"onRender":{"!type":"fn(ct: ?, position: ?) -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"onShow":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the show operation. After\ncalling the superclass's onShow, the Component will be visible.</p>\n\n<p>Override in subclasses where more complex behaviour is needed.</p>\n\n<p>Gets passed the same parameters as <a href=\"#!/api/Ext.tip.ToolTip-event-show\" rel=\"Ext.tip.ToolTip-event-show\" class=\"docClass\">show</a>.</p>\n"},"onShowVeto":{"!type":"fn() -> !this"},"setPagePosition":{"!type":"fn(x: ?, y: ?) -> ?","!doc":"<p>Sets the page XY position of the component. To set the left and top instead, use <a href=\"#!/api/Ext.tip.ToolTip-method-setPosition\" rel=\"Ext.tip.ToolTip-method-setPosition\" class=\"docClass\">setPosition</a>.\nThis method fires the <a href=\"#!/api/Ext.tip.ToolTip-event-move\" rel=\"Ext.tip.ToolTip-event-move\" class=\"docClass\">move</a> event.</p>\n"},"setTarget":{"!type":"fn(t: ?) -> !this","!doc":"<p>Binds this ToolTip to the specified element. The tooltip will be displayed when the mouse moves over the element.</p>\n"},"showFromDelay":{"!type":"fn(xy: ?) -> !this"},"syncAnchor":{"!type":"fn() -> !this"},"target":{"!doc":"<p>The target element or string id to monitor for mouseover events to trigger\nshowing this ToolTip.</p>\n"},"hide":{"!type":"fn() -> !this","!doc":"<p>Hides this tooltip if visible.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"onTargetOut":{"!type":"fn(e: ?) -> !this"},"onTargetOver":{"!type":"fn(e: ?) -> !this"},"show":{"!type":"fn(xy: ?) -> !this","!doc":"<p>Shows this tooltip at the current event target XY position.</p>\n"},"showAt":{"!type":"fn(xy: ?) -> !this","!doc":"<p>Shows this tip at the specified XY position.  Example usage:</p>\n\n<pre><code>// Show the tip at x:50 and y:100\ntip.showAt([50,100]);\n</code></pre>\n"}},"!doc":"<p>ToolTip is a <a href=\"#!/api/Ext.tip.Tip\" rel=\"Ext.tip.Tip\" class=\"docClass\">Ext.tip.Tip</a> implementation that handles the common case of displaying a\ntooltip when hovering over a certain element or elements on the page. It allows fine-grained\ncontrol over the tooltip's alignment relative to the target element or mouse, and the timing\nof when it is automatically shown and hidden.</p>\n\n<p>This implementation does <strong>not</strong> have a built-in method of automatically populating the tooltip's\ntext based on the target element; you must either configure a fixed <a href=\"#!/api/Ext.tip.ToolTip-cfg-html\" rel=\"Ext.tip.ToolTip-cfg-html\" class=\"docClass\">html</a> value for each\nToolTip instance, or implement custom logic (e.g. in a <a href=\"#!/api/Ext.tip.ToolTip-event-beforeshow\" rel=\"Ext.tip.ToolTip-event-beforeshow\" class=\"docClass\">beforeshow</a> event listener) to\ngenerate the appropriate tooltip content on the fly. See <a href=\"#!/api/Ext.tip.QuickTip\" rel=\"Ext.tip.QuickTip\" class=\"docClass\">Ext.tip.QuickTip</a> for a more\nconvenient way of automatically populating and configuring a tooltip based on specific DOM\nattributes of each target element.</p>\n\n<h1>Basic Example</h1>\n\n<pre class='inline-example '><code>var tip = Ext.create('<a href=\"#!/api/Ext.tip.ToolTip\" rel=\"Ext.tip.ToolTip\" class=\"docClass\">Ext.tip.ToolTip</a>', {\n    target: 'clearButton',\n    html: 'Press this button to clear the form'\n});\n</code></pre>\n\n<h1>Delegation</h1>\n\n<p>In addition to attaching a ToolTip to a single element, you can also use delegation to attach\none ToolTip to many elements under a common parent. This is more efficient than creating many\nToolTip instances. To do this, point the <a href=\"#!/api/Ext.tip.ToolTip-cfg-target\" rel=\"Ext.tip.ToolTip-cfg-target\" class=\"docClass\">target</a> config to a common ancestor of all the\nelements, and then set the <a href=\"#!/api/Ext.tip.ToolTip-cfg-delegate\" rel=\"Ext.tip.ToolTip-cfg-delegate\" class=\"docClass\">delegate</a> config to a CSS selector that will select all the\nappropriate sub-elements.</p>\n\n<p>When using delegation, it is likely that you will want to programmatically change the content\nof the ToolTip based on each delegate element; you can do this by implementing a custom\nlistener for the <a href=\"#!/api/Ext.tip.ToolTip-event-beforeshow\" rel=\"Ext.tip.ToolTip-event-beforeshow\" class=\"docClass\">beforeshow</a> event. Example:</p>\n\n<pre class='inline-example '><code>var store = Ext.create('Ext.data.ArrayStore', {\n    fields: ['company', 'price', 'change'],\n    data: [\n        ['3m Co',                               71.72, 0.02],\n        ['Alcoa Inc',                           29.01, 0.42],\n        ['Altria Group Inc',                    83.81, 0.28],\n        ['American Express Company',            52.55, 0.01],\n        ['American International Group, Inc.',  64.13, 0.31],\n        ['AT&amp;T Inc.',                           31.61, -0.48]\n    ]\n});\n\nvar grid = Ext.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Array Grid',\n    store: store,\n    columns: [\n        {text: 'Company', flex: 1, dataIndex: 'company'},\n        {text: 'Price', width: 75, dataIndex: 'price'},\n        {text: 'Change', width: 75, dataIndex: 'change'}\n    ],\n    height: 200,\n    width: 400,\n    renderTo: Ext.getBody()\n});\n\nvar view = grid.getView();\nvar tip = Ext.create('<a href=\"#!/api/Ext.tip.ToolTip\" rel=\"Ext.tip.ToolTip\" class=\"docClass\">Ext.tip.ToolTip</a>', {\n    // The overall target element.\n    target: view.el,\n    // Each grid row causes its own separate show and hide.\n    delegate: view.itemSelector,\n    // Moving within the row should not hide the tip.\n    trackMouse: true,\n    // Render immediately so that tip.body can be referenced prior to the first show.\n    renderTo: Ext.getBody(),\n    listeners: {\n        // Change content dynamically depending on which element triggered the show.\n        beforeshow: function updateTipBody(tip) {\n            tip.update('Over company \"' + view.getRecord(tip.triggerElement).get('company') + '\"');\n        }\n    }\n});\n</code></pre>\n\n<h1>Alignment</h1>\n\n<p>The following configuration properties allow control over how the ToolTip is aligned relative to\nthe target element and/or mouse pointer:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.tip.ToolTip-cfg-anchor\" rel=\"Ext.tip.ToolTip-cfg-anchor\" class=\"docClass\">anchor</a></li>\n<li><a href=\"#!/api/Ext.tip.ToolTip-cfg-anchorToTarget\" rel=\"Ext.tip.ToolTip-cfg-anchorToTarget\" class=\"docClass\">anchorToTarget</a></li>\n<li><a href=\"#!/api/Ext.tip.ToolTip-cfg-anchorOffset\" rel=\"Ext.tip.ToolTip-cfg-anchorOffset\" class=\"docClass\">anchorOffset</a></li>\n<li><a href=\"#!/api/Ext.tip.ToolTip-cfg-trackMouse\" rel=\"Ext.tip.ToolTip-cfg-trackMouse\" class=\"docClass\">trackMouse</a></li>\n<li><a href=\"#!/api/Ext.tip.ToolTip-cfg-mouseOffset\" rel=\"Ext.tip.ToolTip-cfg-mouseOffset\" class=\"docClass\">mouseOffset</a></li>\n</ul>\n\n\n<h1>Showing/Hiding</h1>\n\n<p>The following configuration properties allow control over how and when the ToolTip is automatically\nshown and hidden:</p>\n\n<ul>\n<li><a href=\"#!/api/Ext.tip.ToolTip-cfg-autoHide\" rel=\"Ext.tip.ToolTip-cfg-autoHide\" class=\"docClass\">autoHide</a></li>\n<li><a href=\"#!/api/Ext.tip.ToolTip-cfg-showDelay\" rel=\"Ext.tip.ToolTip-cfg-showDelay\" class=\"docClass\">showDelay</a></li>\n<li><a href=\"#!/api/Ext.tip.ToolTip-cfg-hideDelay\" rel=\"Ext.tip.ToolTip-cfg-hideDelay\" class=\"docClass\">hideDelay</a></li>\n<li><a href=\"#!/api/Ext.tip.ToolTip-cfg-dismissDelay\" rel=\"Ext.tip.ToolTip-cfg-dismissDelay\" class=\"docClass\">dismissDelay</a></li>\n</ul>\n\n","!type":"fn(config: ?)"},"QuickTipManager":{"!doc":"<p>Provides attractive and customizable tooltips for any element. The QuickTips\nsingleton is used to configure and manage tooltips globally for multiple elements\nin a generic manner.  To create individual tooltips with maximum customizability,\nyou should consider either <a href=\"#!/api/Ext.tip.Tip\" rel=\"Ext.tip.Tip\" class=\"docClass\">Ext.tip.Tip</a> or <a href=\"#!/api/Ext.tip.ToolTip\" rel=\"Ext.tip.ToolTip\" class=\"docClass\">Ext.tip.ToolTip</a>.</p>\n\n<p>Quicktips can be configured via tag attributes directly in markup, or by\nregistering quick tips programmatically via the <a href=\"#!/api/Ext.tip.QuickTipManager-method-register\" rel=\"Ext.tip.QuickTipManager-method-register\" class=\"docClass\">register</a> method.</p>\n\n<p>The singleton's instance of <a href=\"#!/api/Ext.tip.QuickTip\" rel=\"Ext.tip.QuickTip\" class=\"docClass\">Ext.tip.QuickTip</a> is available via\n<a href=\"#!/api/Ext.tip.QuickTipManager-method-getQuickTip\" rel=\"Ext.tip.QuickTipManager-method-getQuickTip\" class=\"docClass\">getQuickTip</a>, and supports all the methods, and all the all the\nconfiguration properties of <a href=\"#!/api/Ext.tip.QuickTip\" rel=\"Ext.tip.QuickTip\" class=\"docClass\">Ext.tip.QuickTip</a>. These settings will apply to all\ntooltips shown by the singleton.</p>\n\n<p>Below is the summary of the configuration properties which can be used.\nFor detailed descriptions see the config options for the\n<a href=\"#!/api/Ext.tip.QuickTip\" rel=\"Ext.tip.QuickTip\" class=\"docClass\">QuickTip</a> class</p>\n\n<h2>QuickTips singleton configs (all are optional)</h2>\n\n<ul>\n<li><code>dismissDelay</code></li>\n<li><code>hideDelay</code></li>\n<li><code>maxWidth</code></li>\n<li><code>minWidth</code></li>\n<li><code>showDelay</code></li>\n<li><code>trackMouse</code></li>\n</ul>\n\n\n<h2>Target element configs (optional unless otherwise noted)</h2>\n\n<ul>\n<li><code>autoHide</code></li>\n<li><code>cls</code></li>\n<li><code>dismissDelay</code> (overrides singleton value)</li>\n<li><code>target</code> (required)</li>\n<li><code>text</code> (required)</li>\n<li><code>title</code></li>\n<li><code>width</code></li>\n</ul>\n\n\n<p>Here is an example showing how some of these config options could be used:</p>\n\n<pre class='inline-example '><code>// Init the singleton.  Any tag-based quick tips will start working.\n<a href=\"#!/api/Ext.tip.QuickTipManager-method-init\" rel=\"Ext.tip.QuickTipManager-method-init\" class=\"docClass\">Ext.tip.QuickTipManager.init</a>();\n\n// Apply a set of config properties to the singleton\nExt.apply(<a href=\"#!/api/Ext.tip.QuickTipManager-method-getQuickTip\" rel=\"Ext.tip.QuickTipManager-method-getQuickTip\" class=\"docClass\">Ext.tip.QuickTipManager.getQuickTip</a>(), {\n    maxWidth: 200,\n    minWidth: 100,\n    showDelay: 50      // Show 50ms after entering target\n});\n\n// Create a small panel to add a quick tip to\nExt.create('<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>', {\n    id: 'quickTipContainer',\n    width: 200,\n    height: 150,\n    style: {\n        backgroundColor:'#000000'\n    },\n    renderTo: Ext.getBody()\n});\n\n\n// Manually register a quick tip for a specific element\n<a href=\"#!/api/Ext.tip.QuickTipManager-method-register\" rel=\"Ext.tip.QuickTipManager-method-register\" class=\"docClass\">Ext.tip.QuickTipManager.register</a>({\n    target: 'quickTipContainer',\n    title: 'My Tooltip',\n    text: 'This tooltip was added in code',\n    width: 100,\n    dismissDelay: 10000 // Hide after 10 seconds hover\n});\n</code></pre>\n\n<p>To register a quick tip in markup, you simply add one or more of the valid QuickTip\nattributes prefixed with the <strong>data-</strong> namespace.  The HTML element itself is\nautomatically set as the quick tip target. Here is the summary of supported attributes\n(optional unless otherwise noted):</p>\n\n<ul>\n<li><code>hide</code>: Specifying \"user\" is equivalent to setting autoHide = false.\n Any other value will be the same as autoHide = true.</li>\n<li><code>qclass</code>: A CSS class to be applied to the quick tip\n (equivalent to the 'cls' target element config).</li>\n<li><code>qtip (required)</code>: The quick tip text (equivalent to the 'text' target element config).</li>\n<li><code>qtitle</code>: The quick tip title (equivalent to the 'title' target element config).</li>\n<li><code>qwidth</code>: The quick tip width (equivalent to the 'width' target element config).</li>\n</ul>\n\n\n<p>Here is an example of configuring an HTML element to display a tooltip from markup:</p>\n\n<pre><code>// Add a quick tip to an HTML button\n&lt;input type=\"button\" value=\"OK\" data-qtitle=\"OK Button\" data-qwidth=\"100\"\n     data-qtip=\"This is a quick tip from markup!\"&gt;&lt;/input&gt;\n</code></pre>\n","disabled":{"!type":"bool"},"ddDisable":{"!type":"fn() -> !this","!doc":"<p>Protected method called by the dd classes</p>\n"},"ddEnable":{"!type":"fn() -> !this","!doc":"<p>Protected method called by the dd classes</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Destroys the QuickTips instance.</p>\n"},"disable":{"!type":"fn() -> !this","!doc":"<p>Disables quick tips globally.</p>\n"},"enable":{"!type":"fn() -> !this","!doc":"<p>Enables quick tips globally.</p>\n"},"getQuickTip":{"!type":"fn() -> ?","!doc":"<p>Gets the single <a href=\"#!/api/Ext.tip.QuickTip\" rel=\"Ext.tip.QuickTip\" class=\"docClass\">QuickTip</a> instance used to show tips\nfrom all registered elements.</p>\n"},"init":{"!type":"fn(autoRender?: ?, config?: ?) -> !this","!doc":"<p>Initializes the global QuickTips instance and prepare any quick tips.</p>\n"},"isEnabled":{"!type":"fn() -> bool","!doc":"<p>Returns true if quick tips are enabled, else false.</p>\n"},"register":{"!type":"fn(config: ?) -> !this","!doc":"<p>Configures a new quick tip instance and assigns it to a target element.  See\n<a href=\"#!/api/Ext.tip.QuickTip-method-register\" rel=\"Ext.tip.QuickTip-method-register\" class=\"docClass\">Ext.tip.QuickTip.register</a> for details.</p>\n"},"tips":{"!type":"fn(config: ?) -> !this","!doc":"<p>Alias of <a href=\"#!/api/Ext.tip.QuickTipManager-method-register\" rel=\"Ext.tip.QuickTipManager-method-register\" class=\"docClass\">register</a>.</p>\n"},"unregister":{"!type":"fn(el: ?) -> !this","!doc":"<p>Removes any registered quick tip from the target element and destroys it.</p>\n"}}},"sparkline":{"Bar":{"!doc":"<p>Plots a bar chart of the values in the passed values array.</p>\n","prototype":{"barColor":{"!type":"string","!doc":"<p>The bar color for positive values.</p>\n"},"barSpacing":{"!type":"number","!doc":"<p>The pixel spacing between bars.</p>\n"},"barWidth":{"!type":"number","!doc":"<p>The pixel width of bars.</p>\n"},"chartRangeClip":{"!type":"bool","!doc":"<p>If true then the y values supplied to plot will be clipped to fall\nbetween <a href=\"#!/api/Ext.sparkline.Bar-cfg-chartRangeMin\" rel=\"Ext.sparkline.Bar-cfg-chartRangeMin\" class=\"docClass\">chartRangeMin</a> and <a href=\"#!/api/Ext.sparkline.Bar-cfg-chartRangeMax\" rel=\"Ext.sparkline.Bar-cfg-chartRangeMax\" class=\"docClass\">chartRangeMax</a> - By default chartRangeMin/Max just ensure that the chart\nspans at least that range of values, but does not constrain it.</p>\n"},"chartRangeMax":{"!type":"number","!doc":"<p>The maximum value to use for the range of Y values of the chart - Defaults to the minimum value supplied.</p>\n"},"chartRangeMin":{"!type":"number","!doc":"<p>The minimum value to use for the range of Y values of the chart - Defaults to the minimum value supplied.</p>\n"},"colorMap":{},"negBarColor":{"!type":"string","!doc":"<p>The bar color for negative values.</p>\n"},"nullColor":{"!type":"string","!doc":"<p>The bar color for null values. Usually null values are omitted and not plotted. Setting\nthis config causes a very thin bar to be plotted with the special color in the case thath null is a meaningful value in the series.</p>\n"},"stackedBarColor":{"!doc":"<p>An array of colours to use for stacked bar charts.\nThe first series will use the first value in the array, the second series will use the second, etc.</p>\n"},"tipTpl":{},"zeroAxis":{"!type":"bool","!doc":"<p>Centers the Y axis at zero by default.</p>\n"},"zeroColor":{"!type":"string","!doc":"<p>The bar color for zero values.</p>\n"},"all":{"!type":"fn(arr: ?, val: ?, ignoreNull: ?) -> !this","!doc":"<p>determine if all values of an array match a value\nreturns true if the array is empty</p>\n"},"applyColorMap":{"!type":"fn(colorMap: ?) -> !this"},"calcColor":{"!type":"fn(stacknum: ?, value: ?, valuenum: ?) -> !this"},"getBarColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-barColor\" rel=\"Ext.sparkline.Bar-cfg-barColor\" class=\"docClass\">barColor</a>.</p>\n"},"getBarSpacing":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-barSpacing\" rel=\"Ext.sparkline.Bar-cfg-barSpacing\" class=\"docClass\">barSpacing</a>.</p>\n"},"getBarWidth":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-barWidth\" rel=\"Ext.sparkline.Bar-cfg-barWidth\" class=\"docClass\">barWidth</a>.</p>\n"},"getChartRangeClip":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-chartRangeClip\" rel=\"Ext.sparkline.Bar-cfg-chartRangeClip\" class=\"docClass\">chartRangeClip</a>.</p>\n"},"getChartRangeMax":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-chartRangeMax\" rel=\"Ext.sparkline.Bar-cfg-chartRangeMax\" class=\"docClass\">chartRangeMax</a>.</p>\n"},"getChartRangeMin":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-chartRangeMin\" rel=\"Ext.sparkline.Bar-cfg-chartRangeMin\" class=\"docClass\">chartRangeMin</a>.</p>\n"},"getColorMap":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-colorMap\" rel=\"Ext.sparkline.Bar-cfg-colorMap\" class=\"docClass\">colorMap</a>.</p>\n"},"getNegBarColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-negBarColor\" rel=\"Ext.sparkline.Bar-cfg-negBarColor\" class=\"docClass\">negBarColor</a>.</p>\n"},"getNullColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-nullColor\" rel=\"Ext.sparkline.Bar-cfg-nullColor\" class=\"docClass\">nullColor</a>.</p>\n"},"getRegion":{"!type":"fn(x: ?, y: ?) -> !this"},"getRegionFields":{"!type":"fn(region: ?) -> !this"},"getStackedBarColor":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-stackedBarColor\" rel=\"Ext.sparkline.Bar-cfg-stackedBarColor\" class=\"docClass\">stackedBarColor</a>.</p>\n"},"getTipTpl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-tipTpl\" rel=\"Ext.sparkline.Bar-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"getZeroAxis":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-zeroAxis\" rel=\"Ext.sparkline.Bar-cfg-zeroAxis\" class=\"docClass\">zeroAxis</a>.</p>\n"},"getZeroColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-zeroColor\" rel=\"Ext.sparkline.Bar-cfg-zeroColor\" class=\"docClass\">zeroColor</a>.</p>\n"},"onUpdate":{"!type":"fn() -> !this"},"remove":{"!type":"fn(vals: ?, filter: ?) -> !this"},"renderRegion":{"!type":"fn(valuenum: ?, highlight: ?) -> !this","!doc":"<p>Render bar(s) for a region</p>\n"},"setBarColor":{"!type":"fn(barColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-barColor\" rel=\"Ext.sparkline.Bar-cfg-barColor\" class=\"docClass\">barColor</a>.</p>\n"},"setBarSpacing":{"!type":"fn(barSpacing: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-barSpacing\" rel=\"Ext.sparkline.Bar-cfg-barSpacing\" class=\"docClass\">barSpacing</a>.</p>\n"},"setBarWidth":{"!type":"fn(barWidth: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-barWidth\" rel=\"Ext.sparkline.Bar-cfg-barWidth\" class=\"docClass\">barWidth</a>.</p>\n"},"setChartRangeClip":{"!type":"fn(chartRangeClip: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-chartRangeClip\" rel=\"Ext.sparkline.Bar-cfg-chartRangeClip\" class=\"docClass\">chartRangeClip</a>.</p>\n"},"setChartRangeMax":{"!type":"fn(chartRangeMax: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-chartRangeMax\" rel=\"Ext.sparkline.Bar-cfg-chartRangeMax\" class=\"docClass\">chartRangeMax</a>.</p>\n"},"setChartRangeMin":{"!type":"fn(chartRangeMin: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-chartRangeMin\" rel=\"Ext.sparkline.Bar-cfg-chartRangeMin\" class=\"docClass\">chartRangeMin</a>.</p>\n"},"setColorMap":{"!type":"fn(colorMap: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-colorMap\" rel=\"Ext.sparkline.Bar-cfg-colorMap\" class=\"docClass\">colorMap</a>.</p>\n"},"setNegBarColor":{"!type":"fn(negBarColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-negBarColor\" rel=\"Ext.sparkline.Bar-cfg-negBarColor\" class=\"docClass\">negBarColor</a>.</p>\n"},"setNullColor":{"!type":"fn(nullColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-nullColor\" rel=\"Ext.sparkline.Bar-cfg-nullColor\" class=\"docClass\">nullColor</a>.</p>\n"},"setStackedBarColor":{"!type":"fn(stackedBarColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-stackedBarColor\" rel=\"Ext.sparkline.Bar-cfg-stackedBarColor\" class=\"docClass\">stackedBarColor</a>.</p>\n"},"setTipTpl":{"!type":"fn(tipTpl: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-tipTpl\" rel=\"Ext.sparkline.Bar-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"setZeroAxis":{"!type":"fn(zeroAxis: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-zeroAxis\" rel=\"Ext.sparkline.Bar-cfg-zeroAxis\" class=\"docClass\">zeroAxis</a>.</p>\n"},"setZeroColor":{"!type":"fn(zeroColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bar-cfg-zeroColor\" rel=\"Ext.sparkline.Bar-cfg-zeroColor\" class=\"docClass\">zeroColor</a>.</p>\n"}}},"Base":{"!doc":"<p>The base class for ExtJS SparkLines. SparkLines are small, inline graphs used to visually\ndisplay small amounts of data. For large datasets, use the chart package.</p>\n\n<p>The SparkLine subclasses accept an <a href=\"#!/api/Ext.sparkline.Base-cfg-values\" rel=\"Ext.sparkline.Base-cfg-values\" class=\"docClass\">array of values</a>, and present the data in different visualizations.</p>\n\n<pre class='inline-example '><code>new <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.Panel</a>({\n    height: 500,\n    width: 600,\n    frame: true,\n    title: 'Test Sparklines',\n    renderTo:document.body,\n    bodyPadding: 10,\n\n    // Named listeners will resolve to methods in this Panel\n    defaultListenerScope: true,\n\n    // Named references will be collected, and can be access from this Panel\n    referenceHolder: true,\n\n    items: [{\n        reference: 'values',\n        xtype: 'textfield',\n        fieldLabel: 'Values',\n        validator: function(v) {\n            var result = [];\n\n            v = v.replace(/\\s/g, '');\n            v = v.replace(/,$/, '');\n            v = v.split(',');\n            for (var i = 0; i &lt; v.length; i++) {\n                if (!Ext.isNumeric(v[i])) {\n                    return 'Value must be a comma separated array of numbers';\n                }\n                result.push(parseInt(v[i], 10));\n            }\n            this.values = result;\n            return true;\n        },\n        listeners: {\n            change: 'onTypeChange',\n            buffer: 500\n        }\n    }, {\n        reference: 'type',\n        xtype: 'combobox',\n        fieldLabel: 'Type',\n        store: [\n            ['sparklineline',     'Line'],\n            ['sparklinebox',      'Box'],\n            ['sparklinebullet',   'Bullet'],\n            ['sparklinediscrete', 'Discrete'],\n            ['sparklinepie',      'Pie'],\n            ['sparklinetristate', 'TriState']\n        ],\n        value: 'sparklineline',\n        listeners: {\n            change: 'onTypeChange',\n            buffer: 500\n        }\n    }],\n\n    onTypeChange: function() {\n        var me = this,\n            refs = me.getReferences(),\n            config;\n\n        if (me.sparkLine) {\n            me.remove(me.sparkLine, true);\n        }\n        config = {\n            xtype: refs.type.getValue(),\n            values: refs.values.values,\n            height: 25,\n            width: 100                    \n        };\n       me.sparkLine = Ext.create(config);\n        me.add(me.sparkLine);\n\n        // Put under fields\n        me.sparkLine.el.dom.style.marginLeft = refs.type.labelEl.getWidth() + 'px';\n    }\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"baseCls":{"!type":"string"},"defaultPixelsPerValue":{"!type":"number"},"disableInteraction":{"!type":"bool"},"disableTooltips":{"!type":"bool","!doc":"<p>Set to <code>true</code> to disable mouseover tooltips.</p>\n"},"enableHighlight":{"!type":"bool"},"enableTagOptions":{"!type":"bool"},"fillColor":{"!type":"string","!doc":"<p>The hex value for fill color in graphs which fill areas (<a href=\"#!/api/Ext.sparkline.Line\" rel=\"Ext.sparkline.Line\" class=\"docClass\">Line</a>).</p>\n"},"highlightColor":{"!type":"string","!doc":"<p>The hex value for the highlight color to use when mouseing over a graph segment.</p>\n"},"highlightLighten":{"!type":"number","!doc":"<p>How much to lighten the highlight color by when mouseing over a graph segment.</p>\n"},"lineColor":{"!type":"string","!doc":"<p>The hex value for line colors in graphs which display lines (<a href=\"#!/api/Ext.sparkline.Box\" rel=\"Ext.sparkline.Box\" class=\"docClass\">Box</a>, <a href=\"#!/api/Ext.sparkline.Discrete\" rel=\"Ext.sparkline.Discrete\" class=\"docClass\">Discrete and {@link Ext.sparkline.Line Line</a>).</p>\n"},"tagValuesAttribute":{"!type":"string"},"tipTpl":{"!doc":"<p>An XTemplate used to display the value or values in a tooltip when hovering over a Sparkline.</p>\n\n<p>The implemented subclases all define their own <code>tipTpl</code>, but it can be overridden.</p>\n"},"tooltipPrefix":{"!type":"string","!doc":"<p>A string to append to each field displayed in a tooltip.</p>\n"},"tooltipSkipNull":{"!type":"bool","!doc":"<p>Null values will not have a tooltip displayed.</p>\n"},"values":{"!doc":"<p>An array of numbers which define the chart.</p>\n"},"defaultBindProperty":{"!type":"string"},"element":{},"redrawQueue":{"!doc":"<p>When any config is changed, the canvas needs to be redrawn.\nThis is done at the next animation frame when this queue is traversed.</p>\n"},"sparkLineTipClass":{"!type":"string"},"all":{"!type":"fn(val: ?, arr: ?, ignoreNull: ?) -> !this","!doc":"<p>determine if all values of an array match a value\nreturns true if the array is empty</p>\n"},"applyConfigChange":{"!type":"fn(newValue: ?) -> !this","!doc":"<p>generic config value applier.\nAdds this to the queue to do a redraw on the next animation frame</p>\n"},"applyTipTpl":{"!type":"fn(tipTpl: ?) -> !this","!doc":"<p>Appliers convert an incoming config value.\nEnsure the tipTpl is an XTemplate</p>\n"},"calcHighlightColor":{"!type":"fn(color: ?) -> !this"},"destroy":{"!type":"fn() -> !this"},"doSetHeight":{"!type":"fn(height: ?) -> !this"},"doSetWidth":{"!type":"fn(width: ?) -> !this"},"getBaseCls":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-baseCls\" rel=\"Ext.sparkline.Base-cfg-baseCls\" class=\"docClass\">baseCls</a>.</p>\n"},"getDefaultPixelsPerValue":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-defaultPixelsPerValue\" rel=\"Ext.sparkline.Base-cfg-defaultPixelsPerValue\" class=\"docClass\">defaultPixelsPerValue</a>.</p>\n"},"getDisableInteraction":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-disableInteraction\" rel=\"Ext.sparkline.Base-cfg-disableInteraction\" class=\"docClass\">disableInteraction</a>.</p>\n"},"getDisableTooltips":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-disableTooltips\" rel=\"Ext.sparkline.Base-cfg-disableTooltips\" class=\"docClass\">disableTooltips</a>.</p>\n"},"getEnableHighlight":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-enableHighlight\" rel=\"Ext.sparkline.Base-cfg-enableHighlight\" class=\"docClass\">enableHighlight</a>.</p>\n"},"getEnableTagOptions":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-enableTagOptions\" rel=\"Ext.sparkline.Base-cfg-enableTagOptions\" class=\"docClass\">enableTagOptions</a>.</p>\n"},"getFillColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-fillColor\" rel=\"Ext.sparkline.Base-cfg-fillColor\" class=\"docClass\">fillColor</a>.</p>\n"},"getHighlightColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-highlightColor\" rel=\"Ext.sparkline.Base-cfg-highlightColor\" class=\"docClass\">highlightColor</a>.</p>\n"},"getHighlightLighten":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-highlightLighten\" rel=\"Ext.sparkline.Base-cfg-highlightLighten\" class=\"docClass\">highlightLighten</a>.</p>\n"},"getLineColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-lineColor\" rel=\"Ext.sparkline.Base-cfg-lineColor\" class=\"docClass\">lineColor</a>.</p>\n"},"getRegion":{"!type":"fn() -> !this","!doc":"<p>Return a region id for a given x/y co-ordinate</p>\n"},"getRegionFields":{"!type":"fn() -> !this"},"getRegionTooltip":{"!type":"fn(region: ?) -> !this","!doc":"<p>Fetch the HTML to display as a tooltip</p>\n"},"getTagValuesAttribute":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-tagValuesAttribute\" rel=\"Ext.sparkline.Base-cfg-tagValuesAttribute\" class=\"docClass\">tagValuesAttribute</a>.</p>\n"},"getTipTpl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-tipTpl\" rel=\"Ext.sparkline.Base-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"getTooltipPrefix":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-tooltipPrefix\" rel=\"Ext.sparkline.Base-cfg-tooltipPrefix\" class=\"docClass\">tooltipPrefix</a>.</p>\n"},"getTooltipSkipNull":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-tooltipSkipNull\" rel=\"Ext.sparkline.Base-cfg-tooltipSkipNull\" class=\"docClass\">tooltipSkipNull</a>.</p>\n"},"getValues":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-values\" rel=\"Ext.sparkline.Base-cfg-values\" class=\"docClass\">values</a>.</p>\n"},"normalizeValue":{"!type":"fn(val: ?) -> !this"},"normalizeValues":{"!type":"fn(vals: ?) -> !this"},"onMouseEnter":{"!type":"fn(e: ?) -> !this"},"onMouseLeave":{"!type":"fn() -> !this"},"onMouseMove":{"!type":"fn(e: ?) -> !this"},"onUpdate":{"!type":"fn() -> !this"},"redraw":{"!type":"fn() -> !this"},"renderGraph":{"!type":"fn() -> !this","!doc":"<p>Render the chart to the canvas</p>\n"},"setBaseCls":{"!type":"fn(baseCls: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-baseCls\" rel=\"Ext.sparkline.Base-cfg-baseCls\" class=\"docClass\">baseCls</a>.</p>\n"},"setDefaultPixelsPerValue":{"!type":"fn(defaultPixelsPerValue: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-defaultPixelsPerValue\" rel=\"Ext.sparkline.Base-cfg-defaultPixelsPerValue\" class=\"docClass\">defaultPixelsPerValue</a>.</p>\n"},"setDisableInteraction":{"!type":"fn(disableInteraction: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-disableInteraction\" rel=\"Ext.sparkline.Base-cfg-disableInteraction\" class=\"docClass\">disableInteraction</a>.</p>\n"},"setDisableTooltips":{"!type":"fn(disableTooltips: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-disableTooltips\" rel=\"Ext.sparkline.Base-cfg-disableTooltips\" class=\"docClass\">disableTooltips</a>.</p>\n"},"setEnableHighlight":{"!type":"fn(enableHighlight: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-enableHighlight\" rel=\"Ext.sparkline.Base-cfg-enableHighlight\" class=\"docClass\">enableHighlight</a>.</p>\n"},"setEnableTagOptions":{"!type":"fn(enableTagOptions: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-enableTagOptions\" rel=\"Ext.sparkline.Base-cfg-enableTagOptions\" class=\"docClass\">enableTagOptions</a>.</p>\n"},"setFillColor":{"!type":"fn(fillColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-fillColor\" rel=\"Ext.sparkline.Base-cfg-fillColor\" class=\"docClass\">fillColor</a>.</p>\n"},"setHighlightColor":{"!type":"fn(highlightColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-highlightColor\" rel=\"Ext.sparkline.Base-cfg-highlightColor\" class=\"docClass\">highlightColor</a>.</p>\n"},"setHighlightLighten":{"!type":"fn(highlightLighten: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-highlightLighten\" rel=\"Ext.sparkline.Base-cfg-highlightLighten\" class=\"docClass\">highlightLighten</a>.</p>\n"},"setLineColor":{"!type":"fn(lineColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-lineColor\" rel=\"Ext.sparkline.Base-cfg-lineColor\" class=\"docClass\">lineColor</a>.</p>\n"},"setTagValuesAttribute":{"!type":"fn(tagValuesAttribute: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-tagValuesAttribute\" rel=\"Ext.sparkline.Base-cfg-tagValuesAttribute\" class=\"docClass\">tagValuesAttribute</a>.</p>\n"},"setTipTpl":{"!type":"fn(tipTpl: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-tipTpl\" rel=\"Ext.sparkline.Base-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"setTooltipPrefix":{"!type":"fn(tooltipPrefix: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-tooltipPrefix\" rel=\"Ext.sparkline.Base-cfg-tooltipPrefix\" class=\"docClass\">tooltipPrefix</a>.</p>\n"},"setTooltipSkipNull":{"!type":"fn(tooltipSkipNull: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-tooltipSkipNull\" rel=\"Ext.sparkline.Base-cfg-tooltipSkipNull\" class=\"docClass\">tooltipSkipNull</a>.</p>\n"},"setValues":{"!type":"fn(values: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Base-cfg-values\" rel=\"Ext.sparkline.Base-cfg-values\" class=\"docClass\">values</a>.</p>\n"},"updateDisplay":{"!type":"fn() -> !this"},"updateValues":{"!type":"fn(values: ?) -> !this"},"onClassCreated":{"!type":"fn(cls: ?) -> !this"}}},"Box":{"!doc":"<p>Generates a box plot graph from the provided <a href=\"#!/api/Ext.sparkline.Box-cfg-values\" rel=\"Ext.sparkline.Box-cfg-values\" class=\"docClass\">values</a> array.</p>\n\n<p>See <a href=\"http://en.wikipedia.org/wiki/Box_plot\">Wikipedia Box Plots</a></p>\n","!type":"fn(config: ?)","prototype":{"boxFillColor":{"!type":"string","!doc":"<p>The color of the box fill.</p>\n"},"boxLineColor":{"!type":"string","!doc":"<p>The color of the box outline.</p>\n"},"chartRangeMax":{"!type":"number","!doc":"<p>The maximum value to use for the range of Y values of the chart - Defaults to the minimum value supplied.</p>\n"},"chartRangeMin":{"!type":"number","!doc":"<p>The minimum value to use for the range of Y values of the chart - Defaults to the minimum value supplied.</p>\n"},"medianColor":{"!type":"string","!doc":"<p>The color of the median line.</p>\n"},"outlierFillColor":{"!type":"string","!doc":"<p>The fill color of the outlier circles.</p>\n"},"outlierIQR":{"!type":"number","!doc":"<p>The inter-quartile range multipler used to calculate values that qualify as an outlier.</p>\n"},"outlierLineColor":{"!type":"string","!doc":"<p>The color of the outlier circles' outline.</p>\n"},"raw":{"!type":"bool","!doc":"<p>By default the points are calciulated from the input values array. Set this to true to passthe pre-calculated points in the values config.</p>\n"},"showOutliers":{"!type":"bool","!doc":"<p>Configure as <code>false</code> to not show outlier circles.</p>\n"},"spotRadius":{"!type":"number","!doc":"<p>The radius of the outlier circles.</p>\n"},"target":{"!type":"number","!doc":"<p>If set, a crosshair will be drawn at the specified value point.</p>\n"},"targetColor":{"!type":"number","!doc":"<p>The color of the crosshair drawn at the pointe specified by <a href=\"#!/api/Ext.sparkline.Box-cfg-target\" rel=\"Ext.sparkline.Box-cfg-target\" class=\"docClass\">target</a>.</p>\n"},"tipTpl":{"!doc":"<p>An XTemplate used to display the value or values in a tooltip when hovering over a Sparkline.</p>\n\n<p>The implemented subclases all define their own <code>tipTpl</code>, but it can be overridden.</p>\n"},"tooltipFormatFieldlistKey":{"!type":"string"},"whiskerColor":{"!type":"string","!doc":"<p>The color of the whiskers.</p>\n"},"applyValues":{"!type":"fn(newValues: ?) -> !this","!doc":"<p>Ensure values is an array of numbers</p>\n"},"getBoxFillColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-boxFillColor\" rel=\"Ext.sparkline.Box-cfg-boxFillColor\" class=\"docClass\">boxFillColor</a>.</p>\n"},"getBoxLineColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-boxLineColor\" rel=\"Ext.sparkline.Box-cfg-boxLineColor\" class=\"docClass\">boxLineColor</a>.</p>\n"},"getChartRangeMax":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-chartRangeMax\" rel=\"Ext.sparkline.Box-cfg-chartRangeMax\" class=\"docClass\">chartRangeMax</a>.</p>\n"},"getChartRangeMin":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-chartRangeMin\" rel=\"Ext.sparkline.Box-cfg-chartRangeMin\" class=\"docClass\">chartRangeMin</a>.</p>\n"},"getMedianColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-medianColor\" rel=\"Ext.sparkline.Box-cfg-medianColor\" class=\"docClass\">medianColor</a>.</p>\n"},"getOutlierFillColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-outlierFillColor\" rel=\"Ext.sparkline.Box-cfg-outlierFillColor\" class=\"docClass\">outlierFillColor</a>.</p>\n"},"getOutlierIQR":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-outlierIQR\" rel=\"Ext.sparkline.Box-cfg-outlierIQR\" class=\"docClass\">outlierIQR</a>.</p>\n"},"getOutlierLineColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-outlierLineColor\" rel=\"Ext.sparkline.Box-cfg-outlierLineColor\" class=\"docClass\">outlierLineColor</a>.</p>\n"},"getRaw":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-raw\" rel=\"Ext.sparkline.Box-cfg-raw\" class=\"docClass\">raw</a>.</p>\n"},"getRegion":{"!type":"fn() -> !this","!doc":"<p>Simulate a single region</p>\n"},"getRegionFields":{"!type":"fn() -> !this"},"getShowOutliers":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-showOutliers\" rel=\"Ext.sparkline.Box-cfg-showOutliers\" class=\"docClass\">showOutliers</a>.</p>\n"},"getSpotRadius":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-spotRadius\" rel=\"Ext.sparkline.Box-cfg-spotRadius\" class=\"docClass\">spotRadius</a>.</p>\n"},"getTarget":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-target\" rel=\"Ext.sparkline.Box-cfg-target\" class=\"docClass\">target</a>.</p>\n"},"getTargetColor":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-targetColor\" rel=\"Ext.sparkline.Box-cfg-targetColor\" class=\"docClass\">targetColor</a>.</p>\n"},"getTipTpl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-tipTpl\" rel=\"Ext.sparkline.Box-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"getTooltipFormatFieldlistKey":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-tooltipFormatFieldlistKey\" rel=\"Ext.sparkline.Box-cfg-tooltipFormatFieldlistKey\" class=\"docClass\">tooltipFormatFieldlistKey</a>.</p>\n"},"getWhiskerColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-whiskerColor\" rel=\"Ext.sparkline.Box-cfg-whiskerColor\" class=\"docClass\">whiskerColor</a>.</p>\n"},"quartile":{"!type":"fn(values: ?, q: ?) -> !this"},"renderGraph":{"!type":"fn() -> !this","!doc":"<p>Render the chart to the canvas</p>\n"},"renderHighlight":{"!type":"fn() -> !this"},"setBoxFillColor":{"!type":"fn(boxFillColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-boxFillColor\" rel=\"Ext.sparkline.Box-cfg-boxFillColor\" class=\"docClass\">boxFillColor</a>.</p>\n"},"setBoxLineColor":{"!type":"fn(boxLineColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-boxLineColor\" rel=\"Ext.sparkline.Box-cfg-boxLineColor\" class=\"docClass\">boxLineColor</a>.</p>\n"},"setChartRangeMax":{"!type":"fn(chartRangeMax: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-chartRangeMax\" rel=\"Ext.sparkline.Box-cfg-chartRangeMax\" class=\"docClass\">chartRangeMax</a>.</p>\n"},"setChartRangeMin":{"!type":"fn(chartRangeMin: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-chartRangeMin\" rel=\"Ext.sparkline.Box-cfg-chartRangeMin\" class=\"docClass\">chartRangeMin</a>.</p>\n"},"setMedianColor":{"!type":"fn(medianColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-medianColor\" rel=\"Ext.sparkline.Box-cfg-medianColor\" class=\"docClass\">medianColor</a>.</p>\n"},"setOutlierFillColor":{"!type":"fn(outlierFillColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-outlierFillColor\" rel=\"Ext.sparkline.Box-cfg-outlierFillColor\" class=\"docClass\">outlierFillColor</a>.</p>\n"},"setOutlierIQR":{"!type":"fn(outlierIQR: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-outlierIQR\" rel=\"Ext.sparkline.Box-cfg-outlierIQR\" class=\"docClass\">outlierIQR</a>.</p>\n"},"setOutlierLineColor":{"!type":"fn(outlierLineColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-outlierLineColor\" rel=\"Ext.sparkline.Box-cfg-outlierLineColor\" class=\"docClass\">outlierLineColor</a>.</p>\n"},"setRaw":{"!type":"fn(raw: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-raw\" rel=\"Ext.sparkline.Box-cfg-raw\" class=\"docClass\">raw</a>.</p>\n"},"setShowOutliers":{"!type":"fn(showOutliers: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-showOutliers\" rel=\"Ext.sparkline.Box-cfg-showOutliers\" class=\"docClass\">showOutliers</a>.</p>\n"},"setSpotRadius":{"!type":"fn(spotRadius: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-spotRadius\" rel=\"Ext.sparkline.Box-cfg-spotRadius\" class=\"docClass\">spotRadius</a>.</p>\n"},"setTarget":{"!type":"fn(target: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-target\" rel=\"Ext.sparkline.Box-cfg-target\" class=\"docClass\">target</a>.</p>\n"},"setTargetColor":{"!type":"fn(targetColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-targetColor\" rel=\"Ext.sparkline.Box-cfg-targetColor\" class=\"docClass\">targetColor</a>.</p>\n"},"setTipTpl":{"!type":"fn(tipTpl: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-tipTpl\" rel=\"Ext.sparkline.Box-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"setTooltipFormatFieldlistKey":{"!type":"fn(tooltipFormatFieldlistKey: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-tooltipFormatFieldlistKey\" rel=\"Ext.sparkline.Box-cfg-tooltipFormatFieldlistKey\" class=\"docClass\">tooltipFormatFieldlistKey</a>.</p>\n"},"setWhiskerColor":{"!type":"fn(whiskerColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Box-cfg-whiskerColor\" rel=\"Ext.sparkline.Box-cfg-whiskerColor\" class=\"docClass\">whiskerColor</a>.</p>\n"}}},"Bullet":{"!doc":"<p>Plots a bullet graph based upon the input <a href=\"#!/api/Ext.sparkline.Bullet-cfg-values\" rel=\"Ext.sparkline.Bullet-cfg-values\" class=\"docClass\">values</a> array.</p>\n\n<p>See <a href=\"http://en.wikipedia.org/wiki/Bullet_graph\">Bullet graphs Wikipedia Page</a> for more information.</p>\n","!type":"fn(config: ?)","prototype":{"base":{"!type":"number","!doc":"<p>Set this to a number to change the base start number.</p>\n"},"performanceColor":{"!type":"string","!doc":"<p>The color of the performance measure horizontal bar.</p>\n"},"rangeColors":{"!doc":"<p>An array of colors to use for each qualitative range background color.</p>\n"},"targetColor":{"!type":"string","!doc":"<p>The colour of the vertical target marker.</p>\n"},"targetWidth":{"!type":"number","!doc":"<p>Width of the target bar in pixels.</p>\n"},"tipTpl":{"!doc":"<p>An XTemplate used to display the value or values in a tooltip when hovering over a Sparkline.</p>\n\n<p>The implemented subclases all define their own <code>tipTpl</code>, but it can be overridden.</p>\n"},"applyValues":{"!type":"fn(newValues: ?) -> !this","!doc":"<p>Ensure values is an array of normalized values</p>\n"},"getBase":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-base\" rel=\"Ext.sparkline.Bullet-cfg-base\" class=\"docClass\">base</a>.</p>\n"},"getPerformanceColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-performanceColor\" rel=\"Ext.sparkline.Bullet-cfg-performanceColor\" class=\"docClass\">performanceColor</a>.</p>\n"},"getRangeColors":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-rangeColors\" rel=\"Ext.sparkline.Bullet-cfg-rangeColors\" class=\"docClass\">rangeColors</a>.</p>\n"},"getRegion":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Return a region id for a given x/y co-ordinate</p>\n"},"getRegionFields":{"!type":"fn(region: ?) -> !this"},"getTargetColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-targetColor\" rel=\"Ext.sparkline.Bullet-cfg-targetColor\" class=\"docClass\">targetColor</a>.</p>\n"},"getTargetWidth":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-targetWidth\" rel=\"Ext.sparkline.Bullet-cfg-targetWidth\" class=\"docClass\">targetWidth</a>.</p>\n"},"getTipTpl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-tipTpl\" rel=\"Ext.sparkline.Bullet-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"onUpdate":{"!type":"fn() -> !this"},"renderGraph":{"!type":"fn() -> !this","!doc":"<p>Render the chart to the canvas</p>\n"},"renderHighlight":{"!type":"fn(region: ?) -> !this"},"renderPerformance":{"!type":"fn(highlight: ?) -> !this"},"renderRange":{"!type":"fn(region: ?, highlight: ?) -> !this"},"renderTarget":{"!type":"fn(highlight: ?) -> !this"},"setBase":{"!type":"fn(base: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-base\" rel=\"Ext.sparkline.Bullet-cfg-base\" class=\"docClass\">base</a>.</p>\n"},"setPerformanceColor":{"!type":"fn(performanceColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-performanceColor\" rel=\"Ext.sparkline.Bullet-cfg-performanceColor\" class=\"docClass\">performanceColor</a>.</p>\n"},"setRangeColors":{"!type":"fn(rangeColors: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-rangeColors\" rel=\"Ext.sparkline.Bullet-cfg-rangeColors\" class=\"docClass\">rangeColors</a>.</p>\n"},"setTargetColor":{"!type":"fn(targetColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-targetColor\" rel=\"Ext.sparkline.Bullet-cfg-targetColor\" class=\"docClass\">targetColor</a>.</p>\n"},"setTargetWidth":{"!type":"fn(targetWidth: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-targetWidth\" rel=\"Ext.sparkline.Bullet-cfg-targetWidth\" class=\"docClass\">targetWidth</a>.</p>\n"},"setTipTpl":{"!type":"fn(tipTpl: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Bullet-cfg-tipTpl\" rel=\"Ext.sparkline.Bullet-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"}}},"Discrete":{"!doc":"<p>Plots a series of thin vertical lines based upon the input values array.</p>\n","prototype":{"chartRangeClip":{"!type":"bool","!doc":"<p>If true then the y values supplied to plot will be clipped to fall\nbetween <a href=\"#!/api/Ext.sparkline.Discrete-cfg-chartRangeMin\" rel=\"Ext.sparkline.Discrete-cfg-chartRangeMin\" class=\"docClass\">chartRangeMin</a> and <a href=\"#!/api/Ext.sparkline.Discrete-cfg-chartRangeMax\" rel=\"Ext.sparkline.Discrete-cfg-chartRangeMax\" class=\"docClass\">chartRangeMax</a> - By default chartRangeMin/Max just ensure that the chart\nspans at least that range of values, but does not constrain it.</p>\n"},"chartRangeMax":{"!type":"number","!doc":"<p>The maximum value to use for the range of Y values of the chart - Defaults to the maximum value supplied.</p>\n"},"chartRangeMin":{"!type":"number","!doc":"<p>The minimum value to use for the range of Y values of the chart - Defaults to the minimum value supplied.</p>\n"},"lineHeight":{"!type":"number","!doc":"<p>Height of each line in pixels - Defaults to 30% of the graph height.</p>\n"},"thresholdColor":{"!type":"string","!doc":"<p>Colour to use in combination with <a href=\"#!/api/Ext.sparkline.Discrete-cfg-thresholdValue\" rel=\"Ext.sparkline.Discrete-cfg-thresholdValue\" class=\"docClass\">thresholdValue</a></p>\n"},"thresholdValue":{"!type":"number","!doc":"<p>Values less than this value will be drawn using <a href=\"#!/api/Ext.sparkline.Discrete-cfg-thresholdColor\" rel=\"Ext.sparkline.Discrete-cfg-thresholdColor\" class=\"docClass\">thresholdColor</a> instead of lineColor</p>\n"},"tipTpl":{},"applyValues":{"!type":"fn(newValues: ?) -> !this","!doc":"<p>Ensure values is an array of numbers</p>\n"},"getChartRangeClip":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-chartRangeClip\" rel=\"Ext.sparkline.Discrete-cfg-chartRangeClip\" class=\"docClass\">chartRangeClip</a>.</p>\n"},"getChartRangeMax":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-chartRangeMax\" rel=\"Ext.sparkline.Discrete-cfg-chartRangeMax\" class=\"docClass\">chartRangeMax</a>.</p>\n"},"getChartRangeMin":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-chartRangeMin\" rel=\"Ext.sparkline.Discrete-cfg-chartRangeMin\" class=\"docClass\">chartRangeMin</a>.</p>\n"},"getLineHeight":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-lineHeight\" rel=\"Ext.sparkline.Discrete-cfg-lineHeight\" class=\"docClass\">lineHeight</a>.</p>\n"},"getRegion":{"!type":"fn(x: ?, y: ?) -> !this"},"getRegionFields":{"!type":"fn(region: ?) -> !this"},"getThresholdColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-thresholdColor\" rel=\"Ext.sparkline.Discrete-cfg-thresholdColor\" class=\"docClass\">thresholdColor</a>.</p>\n"},"getThresholdValue":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-thresholdValue\" rel=\"Ext.sparkline.Discrete-cfg-thresholdValue\" class=\"docClass\">thresholdValue</a>.</p>\n"},"getTipTpl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-tipTpl\" rel=\"Ext.sparkline.Discrete-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"onUpdate":{"!type":"fn() -> !this"},"renderRegion":{"!type":"fn(valuenum: ?, highlight: ?) -> !this"},"setChartRangeClip":{"!type":"fn(chartRangeClip: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-chartRangeClip\" rel=\"Ext.sparkline.Discrete-cfg-chartRangeClip\" class=\"docClass\">chartRangeClip</a>.</p>\n"},"setChartRangeMax":{"!type":"fn(chartRangeMax: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-chartRangeMax\" rel=\"Ext.sparkline.Discrete-cfg-chartRangeMax\" class=\"docClass\">chartRangeMax</a>.</p>\n"},"setChartRangeMin":{"!type":"fn(chartRangeMin: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-chartRangeMin\" rel=\"Ext.sparkline.Discrete-cfg-chartRangeMin\" class=\"docClass\">chartRangeMin</a>.</p>\n"},"setLineHeight":{"!type":"fn(lineHeight: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-lineHeight\" rel=\"Ext.sparkline.Discrete-cfg-lineHeight\" class=\"docClass\">lineHeight</a>.</p>\n"},"setThresholdColor":{"!type":"fn(thresholdColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-thresholdColor\" rel=\"Ext.sparkline.Discrete-cfg-thresholdColor\" class=\"docClass\">thresholdColor</a>.</p>\n"},"setThresholdValue":{"!type":"fn(thresholdValue: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-thresholdValue\" rel=\"Ext.sparkline.Discrete-cfg-thresholdValue\" class=\"docClass\">thresholdValue</a>.</p>\n"},"setTipTpl":{"!type":"fn(tipTpl: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Discrete-cfg-tipTpl\" rel=\"Ext.sparkline.Discrete-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"}}},"Line":{"!doc":"<p>Plots a line graph based upon the input <a href=\"#!/api/Ext.sparkline.Line-cfg-values\" rel=\"Ext.sparkline.Line-cfg-values\" class=\"docClass\">values</a> array.</p>\n","!type":"fn(config: ?)","prototype":{"chartRangeMax":{"!type":"number","!doc":"<p>The maximum value to use for the range of Y values of the chart - Defaults to the minimum value supplied.</p>\n"},"chartRangeMaxX":{"!type":"number","!doc":"<p>The maximum value to use for the X value of the chart.</p>\n"},"chartRangeMin":{"!type":"number","!doc":"<p>The minimum value to use for the range of Y values of the chart - Defaults to the minimum value supplied.</p>\n"},"chartRangeMinX":{"!type":"number","!doc":"<p>The minimum value to use for the X value of the chart.</p>\n"},"drawNormalOnTop":{"!type":"bool","!doc":"<p>Configure as <code>true</code> to draw the normal range overlaying the chart.</p>\n"},"highlightLineColor":{"!type":"string","!doc":"<p>The colour of value line shown when the graph is mouseovered.</p>\n"},"highlightSpotColor":{"!type":"string","!doc":"<p>The colour of value marker spots when mouseovered.</p>\n"},"lineWidth":{"!type":"number","!doc":"<p>The pixel width of the line plotted.</p>\n"},"maxSpotColor":{"!type":"string","!doc":"<p>The colour of the maximum value marker. Set to false or an empty string to hide it.</p>\n"},"minSpotColor":{"!type":"string","!doc":"<p>The colour of the mimimum value marker. Set to false or an empty string to hide it.</p>\n"},"normalRangeColor":{"!type":"string","!doc":"<p>See <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeMin\" rel=\"Ext.sparkline.Line-cfg-normalRangeMin\" class=\"docClass\">normalRangeMin</a> and <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeMax\" rel=\"Ext.sparkline.Line-cfg-normalRangeMax\" class=\"docClass\">normalRangeMax</a> The color of the undererlayed \"normal range bar\".</p>\n"},"normalRangeMax":{"!type":"number","!doc":"<p>See <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeMin\" rel=\"Ext.sparkline.Line-cfg-normalRangeMin\" class=\"docClass\">normalRangeMin</a> The maximum value to overlay a \"normal range bar\" over the graph using the <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeColor\" rel=\"Ext.sparkline.Line-cfg-normalRangeColor\" class=\"docClass\">normalRangeColor</a>.</p>\n"},"normalRangeMin":{"!type":"number","!doc":"<p>See <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeMax\" rel=\"Ext.sparkline.Line-cfg-normalRangeMax\" class=\"docClass\">normalRangeMax</a> The minimum value to overlay a \"normal range bar\" over the graph using the <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeColor\" rel=\"Ext.sparkline.Line-cfg-normalRangeColor\" class=\"docClass\">normalRangeColor</a>.</p>\n"},"spotColor":{"!type":"string","!doc":"<p>The colour of the final value marker. Set to false or an empty string to hide it.</p>\n"},"spotRadius":{"!type":"number","!doc":"<p>The pixel radius of min, max and final value dots.</p>\n"},"tipTpl":{"!doc":"<p>An XTemplate used to display the value or values in a tooltip when hovering over a Sparkline.</p>\n\n<p>The implemented subclases all define their own <code>tipTpl</code>, but it can be overridden.</p>\n"},"valueSpots":{"!doc":"<p>An object which uses range specifiers as keys to indicate spot color values\nfor range of values. A range specifier is of the form <code>[number]:[number]</code> indicating start and end range.\nOmitting aither means an open ended range. For example to render green spots on all values less than 50\nand red on values higher than 50 use:</p>\n\n<p>   {\n       // Open ended range, with max value 49\n       \":49\": \"green\",</p>\n\n<pre><code>   // Open ended range, with min value 50\n   \"50:\": \"red\"\n</code></pre>\n\n<p>   }</p>\n"},"applyValueSpots":{"!type":"fn(valueSpots: ?) -> !this"},"drawNormalRange":{"!type":"fn(canvasLeft: ?, canvasTop: ?, canvasHeight: ?, canvasWidth: ?, rangey: ?) -> !this"},"getChartRangeMax":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-chartRangeMax\" rel=\"Ext.sparkline.Line-cfg-chartRangeMax\" class=\"docClass\">chartRangeMax</a>.</p>\n"},"getChartRangeMaxX":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-chartRangeMaxX\" rel=\"Ext.sparkline.Line-cfg-chartRangeMaxX\" class=\"docClass\">chartRangeMaxX</a>.</p>\n"},"getChartRangeMin":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-chartRangeMin\" rel=\"Ext.sparkline.Line-cfg-chartRangeMin\" class=\"docClass\">chartRangeMin</a>.</p>\n"},"getChartRangeMinX":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-chartRangeMinX\" rel=\"Ext.sparkline.Line-cfg-chartRangeMinX\" class=\"docClass\">chartRangeMinX</a>.</p>\n"},"getDrawNormalOnTop":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-drawNormalOnTop\" rel=\"Ext.sparkline.Line-cfg-drawNormalOnTop\" class=\"docClass\">drawNormalOnTop</a>.</p>\n"},"getHighlightLineColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-highlightLineColor\" rel=\"Ext.sparkline.Line-cfg-highlightLineColor\" class=\"docClass\">highlightLineColor</a>.</p>\n"},"getHighlightSpotColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-highlightSpotColor\" rel=\"Ext.sparkline.Line-cfg-highlightSpotColor\" class=\"docClass\">highlightSpotColor</a>.</p>\n"},"getLineWidth":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-lineWidth\" rel=\"Ext.sparkline.Line-cfg-lineWidth\" class=\"docClass\">lineWidth</a>.</p>\n"},"getMaxSpotColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-maxSpotColor\" rel=\"Ext.sparkline.Line-cfg-maxSpotColor\" class=\"docClass\">maxSpotColor</a>.</p>\n"},"getMinSpotColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-minSpotColor\" rel=\"Ext.sparkline.Line-cfg-minSpotColor\" class=\"docClass\">minSpotColor</a>.</p>\n"},"getNormalRangeColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeColor\" rel=\"Ext.sparkline.Line-cfg-normalRangeColor\" class=\"docClass\">normalRangeColor</a>.</p>\n"},"getNormalRangeMax":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeMax\" rel=\"Ext.sparkline.Line-cfg-normalRangeMax\" class=\"docClass\">normalRangeMax</a>.</p>\n"},"getNormalRangeMin":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeMin\" rel=\"Ext.sparkline.Line-cfg-normalRangeMin\" class=\"docClass\">normalRangeMin</a>.</p>\n"},"getRegion":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Return a region id for a given x/y co-ordinate</p>\n"},"getRegionFields":{"!type":"fn(region: ?) -> !this"},"getSpotColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-spotColor\" rel=\"Ext.sparkline.Line-cfg-spotColor\" class=\"docClass\">spotColor</a>.</p>\n"},"getSpotRadius":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-spotRadius\" rel=\"Ext.sparkline.Line-cfg-spotRadius\" class=\"docClass\">spotRadius</a>.</p>\n"},"getTipTpl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-tipTpl\" rel=\"Ext.sparkline.Line-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"getValueSpots":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-valueSpots\" rel=\"Ext.sparkline.Line-cfg-valueSpots\" class=\"docClass\">valueSpots</a>.</p>\n"},"onUpdate":{"!type":"fn() -> !this"},"processRangeOptions":{"!type":"fn() -> !this"},"renderGraph":{"!type":"fn() -> !this","!doc":"<p>Render the chart to the canvas</p>\n"},"renderHighlight":{"!type":"fn(region: ?) -> !this"},"scanValues":{"!type":"fn() -> !this"},"setChartRangeMax":{"!type":"fn(chartRangeMax: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-chartRangeMax\" rel=\"Ext.sparkline.Line-cfg-chartRangeMax\" class=\"docClass\">chartRangeMax</a>.</p>\n"},"setChartRangeMaxX":{"!type":"fn(chartRangeMaxX: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-chartRangeMaxX\" rel=\"Ext.sparkline.Line-cfg-chartRangeMaxX\" class=\"docClass\">chartRangeMaxX</a>.</p>\n"},"setChartRangeMin":{"!type":"fn(chartRangeMin: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-chartRangeMin\" rel=\"Ext.sparkline.Line-cfg-chartRangeMin\" class=\"docClass\">chartRangeMin</a>.</p>\n"},"setChartRangeMinX":{"!type":"fn(chartRangeMinX: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-chartRangeMinX\" rel=\"Ext.sparkline.Line-cfg-chartRangeMinX\" class=\"docClass\">chartRangeMinX</a>.</p>\n"},"setDrawNormalOnTop":{"!type":"fn(drawNormalOnTop: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-drawNormalOnTop\" rel=\"Ext.sparkline.Line-cfg-drawNormalOnTop\" class=\"docClass\">drawNormalOnTop</a>.</p>\n"},"setHighlightLineColor":{"!type":"fn(highlightLineColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-highlightLineColor\" rel=\"Ext.sparkline.Line-cfg-highlightLineColor\" class=\"docClass\">highlightLineColor</a>.</p>\n"},"setHighlightSpotColor":{"!type":"fn(highlightSpotColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-highlightSpotColor\" rel=\"Ext.sparkline.Line-cfg-highlightSpotColor\" class=\"docClass\">highlightSpotColor</a>.</p>\n"},"setLineWidth":{"!type":"fn(lineWidth: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-lineWidth\" rel=\"Ext.sparkline.Line-cfg-lineWidth\" class=\"docClass\">lineWidth</a>.</p>\n"},"setMaxSpotColor":{"!type":"fn(maxSpotColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-maxSpotColor\" rel=\"Ext.sparkline.Line-cfg-maxSpotColor\" class=\"docClass\">maxSpotColor</a>.</p>\n"},"setMinSpotColor":{"!type":"fn(minSpotColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-minSpotColor\" rel=\"Ext.sparkline.Line-cfg-minSpotColor\" class=\"docClass\">minSpotColor</a>.</p>\n"},"setNormalRangeColor":{"!type":"fn(normalRangeColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeColor\" rel=\"Ext.sparkline.Line-cfg-normalRangeColor\" class=\"docClass\">normalRangeColor</a>.</p>\n"},"setNormalRangeMax":{"!type":"fn(normalRangeMax: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeMax\" rel=\"Ext.sparkline.Line-cfg-normalRangeMax\" class=\"docClass\">normalRangeMax</a>.</p>\n"},"setNormalRangeMin":{"!type":"fn(normalRangeMin: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-normalRangeMin\" rel=\"Ext.sparkline.Line-cfg-normalRangeMin\" class=\"docClass\">normalRangeMin</a>.</p>\n"},"setSpotColor":{"!type":"fn(spotColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-spotColor\" rel=\"Ext.sparkline.Line-cfg-spotColor\" class=\"docClass\">spotColor</a>.</p>\n"},"setSpotRadius":{"!type":"fn(spotRadius: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-spotRadius\" rel=\"Ext.sparkline.Line-cfg-spotRadius\" class=\"docClass\">spotRadius</a>.</p>\n"},"setTipTpl":{"!type":"fn(tipTpl: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-tipTpl\" rel=\"Ext.sparkline.Line-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"setValueSpots":{"!type":"fn(valueSpots: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Line-cfg-valueSpots\" rel=\"Ext.sparkline.Line-cfg-valueSpots\" class=\"docClass\">valueSpots</a>.</p>\n"}}},"Pie":{"!doc":"<p>Plots a pie chart based upon the input {<a href=\"#!/api/Ext.sparkline.Pie-cfg-values\" rel=\"Ext.sparkline.Pie-cfg-values\" class=\"docClass\">values</a>} array.</p>\n","!type":"fn(config: ?)","prototype":{"borderColor":{"!type":"string","!doc":"<p>Border color of line round slices.</p>\n"},"borderWidth":{"!type":"number","!doc":"<p>Border width in pixels of line round slices.</p>\n"},"offset":{"!type":"number","!doc":"<p>Angle in degrees to offset the first slice.</p>\n"},"sliceColors":{"!doc":"<p>An array of CSS colro values to apply to the chart slices.</p>\n"},"tipTpl":{"!doc":"<p>An XTemplate used to display the value or values in a tooltip when hovering over a Sparkline.</p>\n\n<p>The implemented subclases all define their own <code>tipTpl</code>, but it can be overridden.</p>\n"},"applyValues":{"!type":"fn(newValues: ?) -> !this","!doc":"<p>Ensure values is an array of numbers</p>\n"},"getBorderColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Pie-cfg-borderColor\" rel=\"Ext.sparkline.Pie-cfg-borderColor\" class=\"docClass\">borderColor</a>.</p>\n"},"getBorderWidth":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Pie-cfg-borderWidth\" rel=\"Ext.sparkline.Pie-cfg-borderWidth\" class=\"docClass\">borderWidth</a>.</p>\n"},"getOffset":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Pie-cfg-offset\" rel=\"Ext.sparkline.Pie-cfg-offset\" class=\"docClass\">offset</a>.</p>\n"},"getRegion":{"!type":"fn(x: ?, y: ?) -> !this","!doc":"<p>Return a region id for a given x/y co-ordinate</p>\n"},"getRegionFields":{"!type":"fn(region: ?) -> !this"},"getSliceColors":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Pie-cfg-sliceColors\" rel=\"Ext.sparkline.Pie-cfg-sliceColors\" class=\"docClass\">sliceColors</a>.</p>\n"},"getTipTpl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.Pie-cfg-tipTpl\" rel=\"Ext.sparkline.Pie-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"onUpdate":{"!type":"fn() -> !this"},"renderGraph":{"!type":"fn() -> !this","!doc":"<p>Render the chart to the canvas</p>\n"},"renderHighlight":{"!type":"fn(region: ?) -> !this"},"renderSlice":{"!type":"fn(valuenum: ?, highlight: ?) -> !this"},"setBorderColor":{"!type":"fn(borderColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Pie-cfg-borderColor\" rel=\"Ext.sparkline.Pie-cfg-borderColor\" class=\"docClass\">borderColor</a>.</p>\n"},"setBorderWidth":{"!type":"fn(borderWidth: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Pie-cfg-borderWidth\" rel=\"Ext.sparkline.Pie-cfg-borderWidth\" class=\"docClass\">borderWidth</a>.</p>\n"},"setOffset":{"!type":"fn(offset: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Pie-cfg-offset\" rel=\"Ext.sparkline.Pie-cfg-offset\" class=\"docClass\">offset</a>.</p>\n"},"setSliceColors":{"!type":"fn(sliceColors: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Pie-cfg-sliceColors\" rel=\"Ext.sparkline.Pie-cfg-sliceColors\" class=\"docClass\">sliceColors</a>.</p>\n"},"setTipTpl":{"!type":"fn(tipTpl: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.Pie-cfg-tipTpl\" rel=\"Ext.sparkline.Pie-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"}}},"TriState":{"!doc":"<p>Plots bars based upon \"win\"/\"draw\" or \"lose\" status of the input values array. Positive values mean\na win, zero a draw, and negative a lose.</p>\n","prototype":{"barSpacing":{"!type":"number","!doc":"<p>The pixel spacing between each bar.</p>\n"},"barWidth":{"!type":"number","!doc":"<p>The pixel width of each bar.</p>\n"},"colorMap":{"!doc":"<p>An object which uses range specifiers as keys to indicate bar color values\nfor range of values. A range specifier is of the form <code>[number]:[number]</code> indicating start and end range.\nOmitting aither means an open ended range. For example to render green bars on all values less than -1\nand red on values higher than 1 use:</p>\n\n<p>   {\n       // Open ended range, with max value -1\n       \":-1\": \"green\",</p>\n\n<pre><code>   // Open ended range, with min value 1\n   \"1:\": \"red\"\n</code></pre>\n\n<p>   }</p>\n"},"negBarColor":{"!type":"string","!doc":"<p>The color for negative value bars.</p>\n"},"posBarColor":{"!type":"string","!doc":"<p>The color for positive value bars.</p>\n"},"tipTpl":{},"zeroBarColor":{"!type":"string","!doc":"<p>The color for zero value bars.</p>\n"},"applyColorMap":{"!type":"fn(colorMap: ?) -> !this"},"applyValues":{"!type":"fn(newValues: ?) -> !this","!doc":"<p>Ensure values is an array of numbers</p>\n"},"calcColor":{"!type":"fn(value: ?, valuenum: ?) -> !this"},"getBarSpacing":{"!type":"fn() -> number","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-barSpacing\" rel=\"Ext.sparkline.TriState-cfg-barSpacing\" class=\"docClass\">barSpacing</a>.</p>\n"},"getBarWidth":{"!type":"fn() -> !this"},"getColorMap":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-colorMap\" rel=\"Ext.sparkline.TriState-cfg-colorMap\" class=\"docClass\">colorMap</a>.</p>\n"},"getNegBarColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-negBarColor\" rel=\"Ext.sparkline.TriState-cfg-negBarColor\" class=\"docClass\">negBarColor</a>.</p>\n"},"getPosBarColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-posBarColor\" rel=\"Ext.sparkline.TriState-cfg-posBarColor\" class=\"docClass\">posBarColor</a>.</p>\n"},"getRegion":{"!type":"fn(x: ?, y: ?) -> !this"},"getRegionFields":{"!type":"fn(region: ?) -> !this"},"getTipTpl":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-tipTpl\" rel=\"Ext.sparkline.TriState-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"getZeroBarColor":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-zeroBarColor\" rel=\"Ext.sparkline.TriState-cfg-zeroBarColor\" class=\"docClass\">zeroBarColor</a>.</p>\n"},"onUpdate":{"!type":"fn() -> !this"},"renderRegion":{"!type":"fn(valuenum: ?, highlight: ?) -> !this"},"setBarSpacing":{"!type":"fn(barSpacing: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-barSpacing\" rel=\"Ext.sparkline.TriState-cfg-barSpacing\" class=\"docClass\">barSpacing</a>.</p>\n"},"setBarWidth":{"!type":"fn(barWidth: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-barWidth\" rel=\"Ext.sparkline.TriState-cfg-barWidth\" class=\"docClass\">barWidth</a>.</p>\n"},"setColorMap":{"!type":"fn(colorMap: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-colorMap\" rel=\"Ext.sparkline.TriState-cfg-colorMap\" class=\"docClass\">colorMap</a>.</p>\n"},"setNegBarColor":{"!type":"fn(negBarColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-negBarColor\" rel=\"Ext.sparkline.TriState-cfg-negBarColor\" class=\"docClass\">negBarColor</a>.</p>\n"},"setPosBarColor":{"!type":"fn(posBarColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-posBarColor\" rel=\"Ext.sparkline.TriState-cfg-posBarColor\" class=\"docClass\">posBarColor</a>.</p>\n"},"setTipTpl":{"!type":"fn(tipTpl: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-tipTpl\" rel=\"Ext.sparkline.TriState-cfg-tipTpl\" class=\"docClass\">tipTpl</a>.</p>\n"},"setZeroBarColor":{"!type":"fn(zeroBarColor: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.sparkline.TriState-cfg-zeroBarColor\" rel=\"Ext.sparkline.TriState-cfg-zeroBarColor\" class=\"docClass\">zeroBarColor</a>.</p>\n"}}}},"tab":{"Bar":{"!doc":"<p>TabBar is used internally by a <a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">TabPanel</a> and typically should not\nneed to be created manually.</p>\n","!type":"fn(config: ?)","prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"childEls":{"!doc":"<p>The canonical form of <code>childEls</code> is an object keyed by child's property name\nwith values that are objects with the following properties.</p>\n\n<ul>\n<li><code>itemId</code> - The id to combine with the Component's id that is the id of the\nchild element.</li>\n<li><code>id</code> - The id of the child element.</li>\n<li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This\nshould be set on things like the generated content for an <code><a href=\"#!/api/Ext.view.View\" rel=\"Ext.view.View\" class=\"docClass\">Ext.view.View</a></code>.</li>\n<li><code>select</code>: A selector that will be passed to Ext.dom.Element.select.</li>\n<li><code>selectNode</code>: A selector that will be passed to Ext.dom.Element.selectNode.</li>\n</ul>\n\n\n<p>For example:</p>\n\n<pre><code> childEls: {\n     button: true,\n     buttonText: 'text',\n     buttonImage: {\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above is translated into the following complete form:</p>\n\n<pre><code> childEls: {\n     button: {\n         name: 'button',\n         itemId: 'button'\n     },\n     buttonText: {\n         name: 'buttonText',\n         itemId: 'text'\n     },\n     buttonImage: {\n         name: 'buttonImage',\n         itemId: 'image'\n     }\n }\n</code></pre>\n\n<p>The above can be provided as an array like so:</p>\n\n<pre><code> childEls: [\n     'button',\n     { name: 'buttonText', itemId: 'text' },\n     { name: 'buttonImage', itemId: 'image' }\n }\n</code></pre>\n\n<p>For example, a Component which renders a title and body text:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>', {\n    renderTo: Ext.getBody(),\n    renderTpl: [\n        '&lt;h1 id=\"{id}-title\"&gt;{title}&lt;/h1&gt;',\n        '&lt;p&gt;{msg}&lt;/p&gt;',\n    ],\n    renderData: {\n        title: \"Error\",\n        msg: \"Something went wrong\"\n    },\n    childEls: [\"title\"],\n    listeners: {\n        afterrender: function(cmp){\n            // After rendering the component will have a title property\n            cmp.title.setStyle({color: \"red\"});\n        }\n    }\n});\n</code></pre>\n\n<p>When using <code>select</code>, the property will be an instance of Ext.CompositeElement.\nIn all other cases, the property will be an Ext.dom.Element or <code>null</code>\nif not found.</p>\n\n<p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.\nThe following issues should be considered:</p>\n\n<ul>\n<li>Performance: using selectors can be 10x slower than id lookup.</li>\n<li>Over-selecting: selectors are applied after the DOM elements for all children\nhave been rendered, so selectors can match elements from child components\n(including nested versions of the same component) accidentally.</li>\n</ul>\n\n\n<p>This above issues are most important when using <code>select</code> since it returns multiple\nelements.</p>\n"},"componentLayout":{"!doc":"<p>The sizing and positioning of a Component's internal Elements is the responsibility of the Component's layout\nmanager which sizes a Component's internal structure in response to the Component being sized.</p>\n\n<p>Generally, developers will not use this configuration as all provided Components which need their internal\nelements sizing (Such as <a href=\"#!/api/Ext.form.field.Base\" rel=\"Ext.form.field.Base\" class=\"docClass\">input fields</a>) come with their own componentLayout managers.</p>\n\n<p>The <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used on instances of the base <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a>\nclass which simply sizes the Component's encapsulating element to the height and width specified in the\n<a href=\"#!/api/Ext.tab.Bar-method-setSize\" rel=\"Ext.tab.Bar-method-setSize\" class=\"docClass\">setSize</a> method.</p>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"defaultType":{"!type":"string","!doc":"<p>The default <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">xtype</a> of child Components to create in this Container when\na child item is specified as a raw configuration object, rather than as an instantiated Component.</p>\n"},"maxTabWidth":{"!type":"number","!doc":"<p>The maximum width for a tab in this tab Bar. Defaults to the tab Panel's <a href=\"#!/api/Ext.tab.Panel-cfg-maxTabWidth\" rel=\"Ext.tab.Panel-cfg-maxTabWidth\" class=\"docClass\">maxTabWidth</a> value.</p>\n"},"minTabWidth":{"!type":"number","!doc":"<p>The minimum width for a tab in this tab Bar. Defaults to the tab Panel's <a href=\"#!/api/Ext.tab.Panel-cfg-minTabWidth\" rel=\"Ext.tab.Panel-cfg-minTabWidth\" class=\"docClass\">minTabWidth</a> value.</p>\n"},"plain":{"!type":"bool","!doc":"<p>True to not show the full background on the tabbar</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.tab.Bar-method-getEl\" rel=\"Ext.tab.Bar-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.tab.Bar-method-getEl\" rel=\"Ext.tab.Bar-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.tab.Bar-cfg-renderSelectors\" rel=\"Ext.tab.Bar-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.tab.Bar-cfg-childEls\" rel=\"Ext.tab.Bar-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"tabRotation":{"!doc":"<p>The rotation of the tabs.  Can be one of the following values:</p>\n\n<ul>\n<li><code>null</code> - use the default rotation, depending on the dock position (see below)</li>\n<li><code>0</code> - no rotation</li>\n<li><code>1</code> - rotate 90deg clockwise</li>\n<li><code>2</code> - rotate 90deg counter-clockwise</li>\n</ul>\n\n\n<p>The default behavior of this config depends on the dock position:</p>\n\n<ul>\n<li><code>'top'</code> or <code>'bottom'</code> - <code>0</code></li>\n<li><code>'right'</code> - <code>1</code></li>\n<li><code>'left'</code> - <code>2</code></li>\n</ul>\n\n"},"tabStretchMax":{"!type":"bool","!doc":"<p><code>true</code> to stretch all tabs to the height of the tallest tab when the tabBar\nis docked horizontally, or the width of the widest tab when the tabBar is\ndocked vertically.</p>\n"},"_baseBodyCls":{"!type":"string"},"_layoutAlign":{},"_stripCls":{"!type":"string"},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"isTabBar":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Tab Bar, or subclass thereof.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"adjustTabPositions":{"!type":"fn() -> !this"},"afterComponentLayout":{"!type":"fn(width: ?) -> !this","!doc":"<p>Called by the layout system after the Component has been laid out.</p>\n"},"afterLayout":{"!type":"fn() -> !this","!doc":"<p>Invoked after the Container has laid out (and rendered if necessary)\nits child Components.</p>\n"},"closeTab":{"!type":"fn(toClose: ?) -> !this","!doc":"<p>Closes the given tab by removing it from the TabBar and removing the corresponding card from the TabPanel</p>\n"},"findNextActivatable":{"!type":"fn(toClose: ?) -> !this","!doc":"<p>private - used by TabPanel too.\nWorks out the next tab to activate when one tab is closed.</p>\n"},"getCloseXY":{"!type":"fn(closeEl: ?, tabX: ?, tabY: ?, tabWidth: ?, tabHeight: ?, closeWidth: ?, closeHeight: ?, direction: ?) -> !this"},"getTabInfoFromPoint":{"!type":"fn(xy: ?) -> !this","!doc":"<p>in IE8 and IE9 the clickable region of a rotated element is not its new rotated\nposition, but it's original unrotated position.  The result is that rotated tabs do\nnot capture click and mousenter/mosueleave events correctly.  This method accepts\nan xy position and calculates if the coordinates are within a tab and if they\nare within the tab's close icon (if any)</p>\n"},"getTabRotation":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.tab.Bar-cfg-tabRotation\" rel=\"Ext.tab.Bar-cfg-tabRotation\" class=\"docClass\">tabRotation</a>.</p>\n"},"getTabStretchMax":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.tab.Bar-cfg-tabStretchMax\" rel=\"Ext.tab.Bar-cfg-tabStretchMax\" class=\"docClass\">tabStretchMax</a>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initRenderData":{"!type":"fn() -> ?","!doc":"<p>Initialized the renderData to be used when rendering the renderTpl.</p>\n"},"onAdded":{"!type":"fn(container: ?, pos: ?, instanced: ?) -> !this","!doc":"<p>Method to manage awareness of when components are added to their\nrespective Container, firing an <a href=\"#!/api/Ext.tab.Bar-event-added\" rel=\"Ext.tab.Bar-event-added\" class=\"docClass\">added</a> event. References are\nestablished at add time rather than at render time.</p>\n\n<p>Allows addition of behavior when a Component is added to a\nContainer. At this stage, the Component is in the parent\nContainer's collection of child items. After calling the\nsuperclass's <code>onAdded</code>, the <code>ownerCt</code> reference will be present,\nand if configured with a ref, the <code>refOwner</code> will be set.</p>\n"},"onMouseLeave":{"!type":"fn(e: ?) -> !this"},"onMouseMove":{"!type":"fn(e: ?) -> !this","!doc":"<p>private</p>\n"},"onRemove":{"!type":"fn(tab: ?) -> !this","!doc":"<p>This method is invoked after a new Component has been\nremoved. It is passed the Component which has been\nremoved. This method may be used to update any internal\nstructure which may depend upon the state of the child items.</p>\n"},"onRemoved":{"!type":"fn(destroying: ?) -> !this","!doc":"<p>Method to manage awareness of when components are removed from their\nrespective Container, firing a <a href=\"#!/api/Ext.tab.Bar-event-removed\" rel=\"Ext.tab.Bar-event-removed\" class=\"docClass\">removed</a> event. References are properly\ncleaned up after removing a component from its owning container.</p>\n\n<p>Allows addition of behavior when a Component is removed from\nits parent Container. At this stage, the Component has been\nremoved from its parent Container's collection of child items,\nbut has not been destroyed (It will be destroyed if the parent\nContainer's <code>autoDestroy</code> is <code>true</code>, or if the remove call was\npassed a truthy second parameter). After calling the\nsuperclass's <code>onRemoved</code>, the <code>ownerCt</code> and the <code>refOwner</code> will not\nbe present.</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"setActiveTab":{"!type":"fn(tab: ?, initial: ?) -> !this","!doc":"<p>Marks the given tab as active</p>\n"},"setDock":{"!type":"fn(dock: ?) -> ?","!doc":"<p>Sets the dock position of this component in its parent panel. Note that this only has effect if this item is part\nof the <code>dockedItems</code> collection of a parent that has a DockLayout (note that any Panel has a DockLayout by default)</p>\n"},"setTabRotation":{"!type":"fn(tabRotation: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.tab.Bar-cfg-tabRotation\" rel=\"Ext.tab.Bar-cfg-tabRotation\" class=\"docClass\">tabRotation</a>.</p>\n"},"setTabStretchMax":{"!type":"fn(tabStretchMax: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.tab.Bar-cfg-tabStretchMax\" rel=\"Ext.tab.Bar-cfg-tabStretchMax\" class=\"docClass\">tabStretchMax</a>.</p>\n"},"updateTabRotation":{"!type":"fn(tabRotation: ?) -> !this"},"change":{"!type":"fn(tabBar: ?, tab: ?, card: ?, eOpts: ?)","!doc":"<p>Fired when the currently-active tab has changed</p>\n"}}},"Panel":{"!doc":"<p>A basic tab container. TabPanels can be used exactly like a standard <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a> for\nlayout purposes, but also have special support for containing child Components\n(<code><a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a></code>) that are managed using a\n<a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">CardLayout layout manager</a>, and displayed as separate tabs.</p>\n\n<p><strong>Note:</strong> By default, a tab's close tool <em>destroys</em> the child tab Component and all its descendants.\nThis makes the child tab Component, and all its descendants <strong>unusable</strong>.  To enable re-use of a tab,\nconfigure the TabPanel with <code><a href=\"#!/api/Ext.tab.Panel-cfg-autoDestroy\" rel=\"Ext.tab.Panel-cfg-autoDestroy\" class=\"docClass\">autoDestroy: false</a></code>.</p>\n\n<h2>TabPanel's layout</h2>\n\n<p>TabPanels use a Dock layout to position the <a href=\"#!/api/Ext.tab.Bar\" rel=\"Ext.tab.Bar\" class=\"docClass\">TabBar</a> at the top of the widget.\nPanels added to the TabPanel will have their header hidden by default because the Tab will\nautomatically take the Panel's configured title and icon.</p>\n\n<p>TabPanels use their <a href=\"#!/api/Ext.panel.Header\" rel=\"Ext.panel.Header\" class=\"docClass\">header</a> or <a href=\"#!/api/Ext.panel.Panel-cfg-fbar\" rel=\"Ext.panel.Panel-cfg-fbar\" class=\"docClass\">footer</a>\nelement (depending on the <a href=\"#!/api/Ext.tab.Panel-cfg-tabPosition\" rel=\"Ext.tab.Panel-cfg-tabPosition\" class=\"docClass\">tabPosition</a> configuration) to accommodate the tab selector buttons.\nThis means that a TabPanel will not display any configured title, and will not display any configured\nheader <a href=\"#!/api/Ext.panel.Panel-cfg-tools\" rel=\"Ext.panel.Panel-cfg-tools\" class=\"docClass\">tools</a>.</p>\n\n<p>To display a header, embed the TabPanel in a <a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Panel</a> which uses\n<code><a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">layout: 'fit'</a></code>.</p>\n\n<h2>Controlling tabs</h2>\n\n<p>Configuration options for the <a href=\"#!/api/Ext.tab.Tab\" rel=\"Ext.tab.Tab\" class=\"docClass\">Ext.tab.Tab</a> that represents the component can be passed in\nby specifying the tabConfig option:</p>\n\n<pre class='inline-example '><code><a href=\"#!/api/Ext.tip.QuickTipManager-method-init\" rel=\"Ext.tip.QuickTipManager-method-init\" class=\"docClass\">Ext.tip.QuickTipManager.init</a>();\nExt.create('<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>', {\n    width: 400,\n    height: 400,\n    renderTo: document.body,\n    items: [{\n        title: 'Foo'\n    }, {\n        title: 'Bar',\n        tabConfig: {\n            title: 'Custom Title',\n            tooltip: 'A button tooltip'\n        }\n    }]\n});\n</code></pre>\n\n<h2>Vetoing Changes</h2>\n\n<p>User interaction when changing the tabs can be vetoed by listening to the <a href=\"#!/api/Ext.tab.Panel-event-beforetabchange\" rel=\"Ext.tab.Panel-event-beforetabchange\" class=\"docClass\">beforetabchange</a> event.\nBy returning <code>false</code>, the tab change will not occur.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>', {\n    renderTo: Ext.getBody(),\n    width: 200,\n    height: 200,\n    listeners: {\n        beforetabchange: function(tabs, newTab, oldTab) {\n            return newTab.title != 'P2';\n        }\n    },\n    items: [{\n        title: 'P1'\n    }, {\n        title: 'P2'\n    }, {\n        title: 'P3'\n    }]\n}); \n</code></pre>\n\n<h1>Examples</h1>\n\n<p>Here is a basic TabPanel rendered to the body. This also shows the useful configuration <a href=\"#!/api/Ext.tab.Panel-cfg-activeTab\" rel=\"Ext.tab.Panel-cfg-activeTab\" class=\"docClass\">activeTab</a>,\nwhich allows you to set the active tab on render. If you do not set an <a href=\"#!/api/Ext.tab.Panel-cfg-activeTab\" rel=\"Ext.tab.Panel-cfg-activeTab\" class=\"docClass\">activeTab</a>, no tabs will be\nactive by default.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>', {\n    width: 300,\n    height: 200,\n    activeTab: 0,\n    items: [\n        {\n            title: 'Tab 1',\n            bodyPadding: 10,\n            html : 'A simple tab'\n        },\n        {\n            title: 'Tab 2',\n            html : 'Another one'\n        }\n    ],\n    renderTo : Ext.getBody()\n});\n</code></pre>\n\n<p>It is easy to control the visibility of items in the tab bar. Specify hidden: true to have the\ntab button hidden initially. Items can be subsequently hidden and show by accessing the\ntab property on the child item.</p>\n\n<pre class='inline-example '><code>var tabs = Ext.create('<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>', {\n    width: 400,\n    height: 400,\n    renderTo: document.body,\n    items: [{\n        title: 'Home',\n        html: 'Home',\n        itemId: 'home'\n    }, {\n        title: 'Users',\n        html: 'Users',\n        itemId: 'users',\n        hidden: true\n    }, {\n        title: 'Tickets',\n        html: 'Tickets',\n        itemId: 'tickets'\n    }]\n});\n\nsetTimeout(function(){\n    tabs.child('#home').tab.hide();\n    var users = tabs.child('#users');\n    users.tab.show();\n    tabs.setActiveTab(users);\n}, 1000);\n</code></pre>\n\n<p>You can remove the background of the TabBar by setting the <a href=\"#!/api/Ext.tab.Panel-cfg-plain\" rel=\"Ext.tab.Panel-cfg-plain\" class=\"docClass\">plain</a> property to <code>true</code>.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>', {\n    width: 300,\n    height: 200,\n    activeTab: 0,\n    plain: true,\n    items: [\n        {\n            title: 'Tab 1',\n            bodyPadding: 10,\n            html : 'A simple tab'\n        },\n        {\n            title: 'Tab 2',\n            html : 'Another one'\n        }\n    ],\n    renderTo : Ext.getBody()\n});\n</code></pre>\n\n<p>Another useful configuration of TabPanel is <a href=\"#!/api/Ext.tab.Panel-cfg-tabPosition\" rel=\"Ext.tab.Panel-cfg-tabPosition\" class=\"docClass\">tabPosition</a>. This allows you to change the\nposition where the tabs are displayed. The available options for this are <code>'top'</code> (default) and\n<code>'bottom'</code>.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>', {\n    width: 300,\n    height: 200,\n    activeTab: 0,\n    bodyPadding: 10,\n    tabPosition: 'bottom',\n    items: [\n        {\n            title: 'Tab 1',\n            html : 'A simple tab'\n        },\n        {\n            title: 'Tab 2',\n            html : 'Another one'\n        }\n    ],\n    renderTo : Ext.getBody()\n});\n</code></pre>\n\n<p>The <a href=\"#!/api/Ext.tab.Panel-method-setActiveTab\" rel=\"Ext.tab.Panel-method-setActiveTab\" class=\"docClass\">setActiveTab</a> is a very useful method in TabPanel which will allow you to change the\ncurrent active tab. You can either give it an index or an instance of a tab. For example:</p>\n\n<pre class='inline-example '><code>var tabs = Ext.create('<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>', {\n    items: [\n        {\n            id   : 'my-tab',\n            title: 'Tab 1',\n            html : 'A simple tab'\n        },\n        {\n            title: 'Tab 2',\n            html : 'Another one'\n        }\n    ],\n    renderTo : Ext.getBody()\n});\n\nvar tab = Ext.getCmp('my-tab');\n\nExt.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>', {\n    renderTo: Ext.getBody(),\n    text    : 'Select the first tab',\n    scope   : this,\n    handler : function() {\n        tabs.setActiveTab(tab);\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>', {\n    text    : 'Select the second tab',\n    scope   : this,\n    handler : function() {\n        tabs.setActiveTab(1);\n    },\n    renderTo : Ext.getBody()\n});\n</code></pre>\n\n<p>The <a href=\"#!/api/Ext.tab.Panel-method-getActiveTab\" rel=\"Ext.tab.Panel-method-getActiveTab\" class=\"docClass\">getActiveTab</a> is a another useful method in TabPanel which will return the current active tab.</p>\n\n<pre class='inline-example '><code>var tabs = Ext.create('<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>', {\n    items: [\n        {\n            title: 'Tab 1',\n            html : 'A simple tab'\n        },\n        {\n            title: 'Tab 2',\n            html : 'Another one'\n        }\n    ],\n    renderTo : Ext.getBody()\n});\n\nExt.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>', {\n    text    : 'Get active tab',\n    scope   : this,\n    handler : function() {\n        var tab = tabs.getActiveTab();\n        alert('Current tab: ' + tab.title);\n    },\n    renderTo : Ext.getBody()\n});\n</code></pre>\n\n<p>Adding a new tab is very simple with a TabPanel. You simple call the <a href=\"#!/api/Ext.tab.Panel-method-add\" rel=\"Ext.tab.Panel-method-add\" class=\"docClass\">add</a> method with an config\nobject for a panel.</p>\n\n<pre class='inline-example '><code>var tabs = Ext.create('<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>', {\n    items: [\n        {\n            title: 'Tab 1',\n            html : 'A simple tab'\n        },\n        {\n            title: 'Tab 2',\n            html : 'Another one'\n        }\n    ],\n    renderTo : Ext.getBody()\n});\n\nExt.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>', {\n    text    : 'New tab',\n    scope   : this,\n    handler : function() {\n        var tab = tabs.add({\n            // we use the tabs.items property to get the length of current items/tabs\n            title: 'Tab ' + (tabs.items.length + 1),\n            html : 'Another one'\n        });\n\n        tabs.setActiveTab(tab);\n    },\n    renderTo : Ext.getBody()\n});\n</code></pre>\n\n<p>Additionally, removing a tab is very also simple with a TabPanel. You simple call the <a href=\"#!/api/Ext.tab.Panel-method-remove\" rel=\"Ext.tab.Panel-method-remove\" class=\"docClass\">remove</a> method\nwith an config object for a panel.</p>\n\n<pre class='inline-example '><code>var tabs = Ext.create('<a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a>', {\n    items: [\n        {\n            title: 'Tab 1',\n            html : 'A simple tab'\n        },\n        {\n            id   : 'remove-this-tab',\n            title: 'Tab 2',\n            html : 'Another one'\n        }\n    ],\n    renderTo : Ext.getBody()\n});\n\nExt.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>', {\n    text    : 'Remove tab',\n    scope   : this,\n    handler : function() {\n        var tab = Ext.getCmp('remove-this-tab');\n        tabs.remove(tab);\n    },\n    renderTo : Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"activeItem":{"!doc":"<p>Doesn't apply for <a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">TabPanel</a>, use <a href=\"#!/api/Ext.tab.Panel-cfg-activeTab\" rel=\"Ext.tab.Panel-cfg-activeTab\" class=\"docClass\">activeTab</a> instead.</p>\n"},"activeTab":{"!doc":"<p>The tab to activate initially. Either an ID, index or the tab component itself.</p>\n"},"deferredRender":{"!type":"bool","!doc":"<p>True by default to defer the rendering of child <a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a> to the browsers DOM\nuntil a tab is activated. False will render all contained <a href=\"#!/api/Ext.container.Container-cfg-items\" rel=\"Ext.container.Container-cfg-items\" class=\"docClass\">items</a> as soon as\nthe <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">layout</a> is rendered. If there is a significant amount of content or a lot of\nheavy controls being rendered into panels that are not displayed by default, setting this to true might improve\nperformance.</p>\n\n<p>The deferredRender property is internally passed to the layout manager for TabPanels (<a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a>) as its <a href=\"#!/api/Ext.layout.container.Card-cfg-deferredRender\" rel=\"Ext.layout.container.Card-cfg-deferredRender\" class=\"docClass\">Ext.layout.container.Card.deferredRender</a> configuration value.</p>\n\n<p><strong>Note</strong>: leaving deferredRender as true means that the content within an unactivated tab will not be available</p>\n"},"itemCls":{"!type":"string","!doc":"<p>The class added to each child item of this TabPanel.</p>\n"},"layout":{"!doc":"<p>Optional configuration object for the internal <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">card layout</a>.\nIf present, this is passed straight through to the layout's constructor</p>\n"},"maxTabWidth":{"!type":"number","!doc":"<p>The maximum width for each tab.</p>\n"},"minTabWidth":{"!type":"number","!doc":"<p>The minimum width for a tab in the <a href=\"#!/api/Ext.tab.Panel-cfg-tabBar\" rel=\"Ext.tab.Panel-cfg-tabBar\" class=\"docClass\">tabBar</a>.</p>\n"},"plain":{"!type":"bool","!doc":"<p>True to not show the full background on the TabBar.</p>\n"},"removePanelHeader":{"!type":"bool","!doc":"<p>True to instruct each Panel added to the TabContainer to not render its header element.\nThis is to ensure that the title of the panel does not appear twice.</p>\n"},"tabBar":{"!doc":"<p>Optional configuration object for the internal <a href=\"#!/api/Ext.tab.Bar\" rel=\"Ext.tab.Bar\" class=\"docClass\">Ext.tab.Bar</a>.\nIf present, this is passed straight through to the TabBar's constructor</p>\n"},"tabBarHeaderPosition":{"!type":"number","!doc":"<p>If specified, the <a href=\"#!/api/Ext.tab.Panel-cfg-tabBar\" rel=\"Ext.tab.Panel-cfg-tabBar\" class=\"docClass\">tabBar</a> will be rendered as an item of the TabPanel's\nHeader and the specified <code>tabBarHeaderPosition</code> will be used as the Panel header's\nitemPosition.  If not specified, the <a href=\"#!/api/Ext.tab.Panel-cfg-tabBar\" rel=\"Ext.tab.Panel-cfg-tabBar\" class=\"docClass\">tabBar</a> will be rendered\nas a docked item at <a href=\"#!/api/Ext.tab.Panel-cfg-tabPosition\" rel=\"Ext.tab.Panel-cfg-tabPosition\" class=\"docClass\">tabPosition</a></p>\n"},"tabPosition":{"!doc":"<p>The position where the tab strip should be rendered. Can be <code>top</code>, <code>bottom</code>,\n<code>left</code> or <code>right</code></p>\n"},"tabRotation":{"!doc":"<p>The rotation of the tabs.  Can be one of the following values:</p>\n\n<ul>\n<li><code>'default'</code> use the default rotation, depending on <a href=\"#!/api/Ext.tab.Panel-cfg-tabPosition\" rel=\"Ext.tab.Panel-cfg-tabPosition\" class=\"docClass\">tabPosition</a> (see below)</li>\n<li><code>0</code> - no rotation</li>\n<li><code>1</code> - rotate 90deg clockwise</li>\n<li><code>2</code> - rotate 90deg counter-clockwise</li>\n</ul>\n\n\n<p>The default behavior of this config depends on the <a href=\"#!/api/Ext.tab.Panel-cfg-tabPosition\" rel=\"Ext.tab.Panel-cfg-tabPosition\" class=\"docClass\">tabPosition</a>:</p>\n\n<ul>\n<li><code>'top'</code> or <code>'bottom'</code> - <code>0</code></li>\n<li><code>'right'</code> - <code>1</code></li>\n<li><code>'left'</code> - <code>2</code></li>\n</ul>\n\n"},"tabStretchMax":{"!type":"bool","!doc":"<p><code>true</code> to stretch all tabs to the height of the tallest tab when the tabBar\nis docked horizontally, or the width of the widest tab when the tabBar is\ndocked vertically.</p>\n"},"_defaultTabRotation":{},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"applyTabBar":{"!type":"fn(tabBar: ?) -> !this"},"doRemove":{"!type":"fn(item: ?, autoDestroy: ?) -> !this","!doc":"<p>Unlink the removed child item from its (@link <a href=\"#!/api/Ext.tab.Tab\" rel=\"Ext.tab.Tab\" class=\"docClass\">Ext.tab.Tab</a> Tab}.</p>\n\n<p>If we're removing the currently active tab, activate the nearest one. The item is removed when we call super,\nso we can do preprocessing before then to find the card's index</p>\n"},"getActiveTab":{"!type":"fn() -> ?","!doc":"<p>Returns the item that is currently active inside this TabPanel.</p>\n"},"getTabBar":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.tab.Panel-cfg-tabBar\" rel=\"Ext.tab.Panel-cfg-tabBar\" class=\"docClass\">tabBar</a>.</p>\n"},"getTabPosition":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.tab.Panel-cfg-tabPosition\" rel=\"Ext.tab.Panel-cfg-tabPosition\" class=\"docClass\">tabPosition</a>.</p>\n"},"getTabRotation":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.tab.Panel-cfg-tabRotation\" rel=\"Ext.tab.Panel-cfg-tabRotation\" class=\"docClass\">tabRotation</a>.</p>\n"},"getTabStretchMax":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.tab.Panel-cfg-tabStretchMax\" rel=\"Ext.tab.Panel-cfg-tabStretchMax\" class=\"docClass\">tabStretchMax</a>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>inherit docs</p>\n"},"onAdd":{"!type":"fn(item: ?, index: ?) -> !this","!doc":"<p>Makes sure we have a Tab for each item added to the TabPanel</p>\n"},"onItemBeforeShow":{"!type":"fn(item: ?) -> !this","!doc":"<p>Sets activeTab before item is shown.</p>\n"},"onItemDisable":{"!type":"fn(item: ?) -> !this","!doc":"<p>Disable corresponding tab when item is enabled.</p>\n"},"onItemEnable":{"!type":"fn(item: ?) -> !this","!doc":"<p>Enable corresponding tab when item is enabled.</p>\n"},"onItemIconChange":{"!type":"fn(item: ?, newIcon: ?) -> !this","!doc":"<p>Update the tab icon when panel icon has been set or changed.</p>\n"},"onItemIconClsChange":{"!type":"fn(item: ?, newIconCls: ?) -> !this","!doc":"<p>Update the tab iconCls when panel iconCls has been set or changed.</p>\n"},"onItemTitleChange":{"!type":"fn(item: ?, newTitle: ?) -> !this","!doc":"<p>Update the tab title when panel title has been set or changed.</p>\n"},"onRemove":{"!type":"fn(item: ?, destroying: ?) -> !this","!doc":"<p>Makes sure we remove the corresponding Tab when an item is removed</p>\n"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"setActiveItem":{"!type":"fn(item: ?) -> ?","!doc":"<p>Sets a component as the active layout item. This only applies when using\na <a href=\"#!/api/Ext.layout.container.Card\" rel=\"Ext.layout.container.Card\" class=\"docClass\">Ext.layout.container.Card</a> layout.</p>\n\n<pre><code>var card1 = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {itemId: 'card-1'});\nvar card2 = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {itemId: 'card-2'});\nvar panel = Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    layout: 'card',\n    items: [card1, card2]\n});\n// These are all equivalent\npanel.getLayout().setActiveItem(card2);\npanel.getLayout().setActiveItem('card-2');\npanel.getLayout().setActiveItem(1);\n</code></pre>\n"},"setActiveTab":{"!type":"fn(card: ?) -> ?","!doc":"<p>Makes the given card active. Makes it the visible card in the TabPanel's CardLayout and highlights the Tab.</p>\n"},"setTabBar":{"!type":"fn(tabBar: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.tab.Panel-cfg-tabBar\" rel=\"Ext.tab.Panel-cfg-tabBar\" class=\"docClass\">tabBar</a>.</p>\n"},"setTabPosition":{"!type":"fn(tabPosition: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.tab.Panel-cfg-tabPosition\" rel=\"Ext.tab.Panel-cfg-tabPosition\" class=\"docClass\">tabPosition</a>.</p>\n"},"setTabRotation":{"!type":"fn(tabRotation: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.tab.Panel-cfg-tabRotation\" rel=\"Ext.tab.Panel-cfg-tabRotation\" class=\"docClass\">tabRotation</a>.</p>\n"},"setTabStretchMax":{"!type":"fn(tabStretchMax: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.tab.Panel-cfg-tabStretchMax\" rel=\"Ext.tab.Panel-cfg-tabStretchMax\" class=\"docClass\">tabStretchMax</a>.</p>\n"},"updateHeaderPosition":{"!type":"fn(headerPosition: ?, oldHeaderPosition: ?) -> !this"},"updateTabPosition":{"!type":"fn(tabPosition: ?) -> !this"},"updateTabRotation":{"!type":"fn(tabRotation: ?) -> !this"},"beforetabchange":{"!type":"fn(tabPanel: ?, newCard: ?, oldCard: ?, eOpts: ?)","!doc":"<p>Fires before a tab change (activated by <a href=\"#!/api/Ext.tab.Panel-method-setActiveTab\" rel=\"Ext.tab.Panel-method-setActiveTab\" class=\"docClass\">setActiveTab</a>). Return false in any listener to cancel\nthe tabchange</p>\n"},"tabchange":{"!type":"fn(tabPanel: ?, newCard: ?, oldCard: ?, eOpts: ?)","!doc":"<p>Fires when a new tab has been activated (activated by <a href=\"#!/api/Ext.tab.Panel-method-setActiveTab\" rel=\"Ext.tab.Panel-method-setActiveTab\" class=\"docClass\">setActiveTab</a>).</p>\n"}}},"Tab":{"!doc":"<p>Represents a single Tab in a <a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">TabPanel</a>. A Tab is simply a slightly customized <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Button</a>,\nstyled to look like a tab. Tabs are optionally closable, and can also be disabled. 99% of the time you will not\nneed to create Tabs manually as the framework does so automatically when you use a <a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">TabPanel</a></p>\n","!type":"fn(config: ?)","prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to add to all buttons.</p>\n"},"closable":{"!type":"bool","!doc":"<p>True if the tab is currently closable</p>\n"},"closeText":{"!type":"string","!doc":"<p>The accessible text label for the close button link; only used when <a href=\"#!/api/Ext.tab.Tab-cfg-closable\" rel=\"Ext.tab.Tab-cfg-closable\" class=\"docClass\">closable</a> = true.</p>\n"},"overCls":{"!type":"string","!doc":"<p>An optional extra CSS class that will be added to this component's Element when the mouse moves over the Element,\nand removed when the mouse moves out. This can be useful for adding customized 'active' or 'hover' styles to the\ncomponent or any of its children using standard CSS rules.</p>\n"},"rotation":{"!doc":"<p>The rotation of the tab.  Can be one of the following values:</p>\n\n<ul>\n<li><code>null</code> - use the default rotation, depending on the dock position of the tabbar</li>\n<li><code>0</code> - no rotation</li>\n<li><code>1</code> - rotate 90deg clockwise</li>\n<li><code>2</code> - rotate 90deg counter-clockwise</li>\n</ul>\n\n\n<p>The default behavior of this config depends on the dock position of the tabbar:</p>\n\n<ul>\n<li><code>'top'</code> or <code>'bottom'</code> - <code>0</code></li>\n<li><code>'right'</code> - <code>1</code></li>\n<li><code>'left'</code> - <code>2</code></li>\n</ul>\n\n"},"scale":{"!doc":"<p>The size of the Button. Three values are allowed:</p>\n\n<ul>\n<li>'small' - Results in the button element being 16px high.</li>\n<li>'medium' - Results in the button element being 24px high.</li>\n<li>'large' - Results in the button element being 32px high.</li>\n</ul>\n\n"},"tabPosition":{"!doc":"<p>The tab's position.  Users should not typically need to set this, as it is\nconfigured automatically by the tab bar</p>\n"},"_activeCls":{"!type":"string"},"_baseIconCls":{"!type":"string"},"_btnCls":{"!type":"string"},"_btnWrapCls":{"!type":"string"},"_closableCls":{"!type":"string"},"_defaultRotations":{},"_disabledCls":{"!type":"string"},"_glyphCls":{"!type":"string"},"_hasIconCls":{"!type":"string"},"_innerCls":{"!type":"string"},"_noTextCls":{"!type":"string"},"_positions":{"!doc":"<p>a mapping of the \"ui\" positions.  When \"rotation\" is anything other than 0, a ui\nposition other than the docked side must be used.</p>\n"},"_pressedCls":{"!type":"string"},"_rotateClasses":{},"_textCls":{"!type":"string"},"active":{"!type":"bool","!doc":"<p>Indicates that this tab is currently active. This is NOT a public configuration.</p>\n"},"closeElOverCls":{"!type":"string"},"closeElPressedCls":{"!type":"string"},"isTab":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Tab, or subclass thereof.</p>\n"},"privates":{"!doc":"<hr />\n\n<p> End Positionable methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n\n<p><strong>Overridden in Ext.rtl.Component.</strong></p>\n"},"activate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fired when the tab is activated.</p>\n"},"afterClick":{"!type":"fn(isCloseClick: ?) -> !this"},"beforeRender":{"!type":"fn() -> !this"},"deactivate":{"!type":"fn(this: ?, eOpts: ?)","!doc":"<p>Fired when the tab is deactivated.</p>\n"},"disable":{"!type":"fn(silent: ?) -> !this","!doc":"<p>inherit docs</p>\n"},"enable":{"!type":"fn(silent: ?) -> !this","!doc":"<p>inherit docs</p>\n"},"fireClose":{"!type":"fn() -> !this","!doc":"<p>Fires the close event on the tab.</p>\n"},"getActualRotation":{"!type":"fn() -> !this"},"getRotation":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.tab.Tab-cfg-rotation\" rel=\"Ext.tab.Tab-cfg-rotation\" class=\"docClass\">rotation</a>.</p>\n"},"getTabPosition":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.tab.Tab-cfg-tabPosition\" rel=\"Ext.tab.Tab-cfg-tabPosition\" class=\"docClass\">tabPosition</a>.</p>\n"},"getTemplateArgs":{"!type":"fn() -> ?","!doc":"<p>This method returns an object which provides substitution parameters for the <a href=\"#!/api/Ext.tab.Tab-cfg-renderTpl\" rel=\"Ext.tab.Tab-cfg-renderTpl\" class=\"docClass\">XTemplate</a> used to\ncreate this Button's DOM structure.</p>\n\n<p>Instances or subclasses which use a different Template to create a different DOM structure may need to provide\ntheir own implementation of this method.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initKeyNav":{"!type":"fn() -> !this"},"onAdded":{"!type":"fn(container: ?, pos: ?, instanced: ?) -> !this","!doc":"<p>Method to manage awareness of when components are added to their\nrespective Container, firing an <a href=\"#!/api/Ext.tab.Tab-event-added\" rel=\"Ext.tab.Tab-event-added\" class=\"docClass\">added</a> event. References are\nestablished at add time rather than at render time.</p>\n\n<p>Allows addition of behavior when a Component is added to a\nContainer. At this stage, the Component is in the parent\nContainer's collection of child items. After calling the\nsuperclass's <code>onAdded</code>, the <code>ownerCt</code> reference will be present,\nand if configured with a ref, the <code>refOwner</code> will be set.</p>\n"},"onCloseClick":{"!type":"fn() -> !this","!doc":"<p>Listener attached to click events on the Tab's close button</p>\n"},"onDeleteKey":{"!type":"fn(e: ?) -> !this"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onEnterKey":{"!type":"fn(e: ?) -> !this"},"onRender":{"!type":"fn() -> !this","!doc":"<p>Template method called when this Component's DOM structure is created.</p>\n\n<p>At this point, this Component's (and all descendants') DOM structure <em>exists</em> but it has not\nbeen layed out (positioned and sized).</p>\n\n<p>Subclasses which override this to gain access to the structure at render time should\ncall the parent class's method before attempting to access any child elements of the Component.</p>\n"},"setCard":{"!type":"fn(card: ?) -> !this","!doc":"<p>Sets this tab's attached card. Usually this is handled automatically by the <a href=\"#!/api/Ext.tab.Panel\" rel=\"Ext.tab.Panel\" class=\"docClass\">Ext.tab.Panel</a> that this Tab\nbelongs to and would not need to be done by the developer</p>\n"},"setClosable":{"!type":"fn(closable: ?) -> !this","!doc":"<p>Sets the tab as either closable or not.</p>\n"},"setElOrientation":{"!type":"fn() -> !this"},"setRotation":{"!type":"fn(rotation: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.tab.Tab-cfg-rotation\" rel=\"Ext.tab.Tab-cfg-rotation\" class=\"docClass\">rotation</a>.</p>\n"},"setTabPosition":{"!type":"fn(tabPosition: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.tab.Tab-cfg-tabPosition\" rel=\"Ext.tab.Tab-cfg-tabPosition\" class=\"docClass\">tabPosition</a>.</p>\n"},"syncClosableCls":{"!type":"fn() -> !this","!doc":"<p>This method ensures that the closable cls are added or removed based on 'closable'.</p>\n"},"syncClosableElements":{"!type":"fn() -> !this","!doc":"<p>This method ensures that the closeBtn element exists or not based on 'closable'.</p>\n"},"syncRotationAndPosition":{"!type":"fn() -> !this"},"updateRotation":{"!type":"fn() -> !this"},"updateTabPosition":{"!type":"fn() -> !this"},"beforeclose":{"!type":"fn(tab: ?, eOpts: ?)","!doc":"<p>Fires if the user clicks on the Tab's close button, but before the <a href=\"#!/api/Ext.tab.Tab-event-close\" rel=\"Ext.tab.Tab-event-close\" class=\"docClass\">close</a> event is fired. Return\nfalse from any listener to stop the close event being fired</p>\n"},"close":{"!type":"fn(tab: ?, eOpts: ?)","!doc":"<p>Fires to indicate that the tab is to be closed, usually because the user has clicked the close button.</p>\n"}}}},"toolbar":{"Breadcrumb":{"!doc":"<p>A toolbar that displays hierarchical data from a TreeStore\nas a trail of breadcrumb buttons.  Each button represents a node in the store.  A click\non a button will \"select\" that node in the tree.  Non-leaf nodes will display their\nchild nodes on a dropdown menu of the corresponding button in the breadcrumb trail,\nand a click on an item in the menu will trigger selection of the corresponding child\nnode.</p>\n\n<p>The selection can be set programmatically  using setSelection, or retrieved\nusing getSelection.</p>\n","!type":"fn(config: ?)","prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"buttonUI":{"!type":"string","!doc":"<p>Button UI to use for breadcrumb items.  Use extjs-breadcrumb-arrow to\nadd special styling to the breadcrumb arrows</p>\n"},"displayFieldName":{"!type":"string","!doc":"<p>The name of the field in the data model to display in the navigation items of\nthis breadcrumb toolbar</p>\n"},"layout":{"!doc":"<p><strong>Important</strong>: In order for child items to be correctly sized and\npositioned, typically a layout manager <strong>must</strong> be specified through\nthe <code>layout</code> configuration option.</p>\n\n<p>The sizing and positioning of child <a href=\"#!/api/Ext.toolbar.Breadcrumb-cfg-items\" rel=\"Ext.toolbar.Breadcrumb-cfg-items\" class=\"docClass\">items</a> is the responsibility of\nthe Container's layout manager which creates and manages the type of layout\nyou have in mind.  For example:</p>\n\n<p>If the layout configuration is not explicitly specified for\na general purpose container (e.g. Container or Panel) the\n<a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">default layout manager</a> will be used\nwhich does nothing but render child components sequentially into the\nContainer (no sizing or positioning will be performed in this situation).</p>\n\n<p><strong>layout</strong> may be specified as either as an Object or as a String:</p>\n\n<h2>Specify as an Object</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: {\n    type: 'vbox',\n    align: 'left'\n}\n</code></pre>\n\n<ul>\n<li><p><strong>type</strong></p>\n\n<p>The layout type to be used for this container.  If not specified,\na default <a href=\"#!/api/Ext.layout.container.Auto\" rel=\"Ext.layout.container.Auto\" class=\"docClass\">Ext.layout.container.Auto</a> will be created and used.</p>\n\n<p>Valid layout <code>type</code> values are listed in <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>.</p></li>\n<li><p>Layout specific configuration properties</p>\n\n<p>Additional layout specific configuration properties may also be\nspecified. For complete details regarding the valid config options for\neach layout type, see the layout class corresponding to the <code>type</code>\nspecified.</p></li>\n</ul>\n\n\n<h2>Specify as a String</h2>\n\n<p>Example usage:</p>\n\n<pre><code>layout: 'vbox'\n</code></pre>\n\n<ul>\n<li><p><strong>layout</strong></p>\n\n<p>The layout <code>type</code> to be used for this container (see <a href=\"#!/api/Ext.enums.Layout\" rel=\"Ext.enums.Layout\" class=\"docClass\">Ext.enums.Layout</a>\nfor list of valid values).</p>\n\n<p>Additional layout specific configuration properties. For complete\ndetails regarding the valid config options for each layout type, see the\nlayout class corresponding to the <code>layout</code> specified.</p></li>\n</ul>\n\n\n<h2>Configuring the default layout type</h2>\n\n<p>If a certain Container class has a default layout (For example a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>\nwith a default <code>Box</code> layout), then to simply configure the default layout,\nuse an object, but without the <code>type</code> property:</p>\n\n<pre><code>xtype: 'toolbar',\nlayout: {\n    pack: 'center'\n}\n</code></pre>\n\n<p>@cmd-auto-dependency { aliasPrefix : \"layout.\" }</p>\n"},"overflowHandler":{"!type":"string","!doc":"<p>The overflowHandler for this Breadcrumb:</p>\n\n<ul>\n<li><code>null</code> - hidden overflow</li>\n<li><code>'scroller'</code> to render left/right scroller buttons on either side of the breadcrumb</li>\n<li><code>'menu'</code> to render the overflowing buttons as items of an overflow menu.</li>\n</ul>\n\n"},"selection":{"!doc":"<p>The selected node, or <code>\"root\"</code> to select the root node</p>\n"},"showIcons":{"!type":"bool","!doc":"<p>Controls whether or not icons of tree nodes are displayed in the breadcrumb\nbuttons.  There are 3 possible values for this config:</p>\n\n<ol>\n<li><p>unspecified (<code>null</code>) - the default value. In this mode only icons that are\nspecified in the tree data using (icon\nor iconCls will be displayed, but the\ndefault \"folder\" and \"leaf\" icons will not be displayed.</p></li>\n<li><p><code>true</code> - Icons specified in the tree data will be displayed, and the default\n\"folder\" and \"leaf\" icons will be displayed for nodes that do not specify\nan <code>icon</code> or <code>iconCls</code>.</p></li>\n<li><p><code>false</code> - No icons will be displayed in the breadcrumb buttons, only text.</p></li>\n</ol>\n\n"},"store":{"!doc":"<p>The TreeStore that this breadcrumb toolbar should use as its data source</p>\n"},"useSplitButtons":{"!type":"bool","!doc":"<p><code>false</code> to use regular <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Button</a>s instead of <a href=\"#!/api/Ext.button.Split\" rel=\"Ext.button.Split\" class=\"docClass\">Split Buttons</a>.  When <code>true</code>, a click on the body of a button\nwill navigate to the specified node, and a click on the arrow will show a menu\ncontaining the the child nodes.  When <code>false</code>, the only mode of navigation is\nthe menu, since a click anywhere on the button will show the menu.</p>\n"},"_breadcrumbCls":{"!type":"string"},"_btnCls":{"!type":"string"},"_folderIconCls":{"!type":"string"},"_leafIconCls":{"!type":"string"},"buttons":{"!doc":"<p>Internal cache of buttons that are re-purposed as the items of this container\nas navigation occurs.</p>\n"},"isBreadcrumb":{"!type":"bool"},"privates":{"!doc":"<p></p>\n"},"renderConfig":{},"_onButtonClick":{"!type":"fn(button: ?, e: ?) -> !this","!doc":"<p>Handles a click on a breadcrumb button</p>\n"},"_onMenuBeforeShow":{"!type":"fn(menu: ?) -> !this","!doc":"<p>Handles the <code>beforeshow</code> event of a button menu</p>\n"},"_onMenuClick":{"!type":"fn(menu: ?, item: ?, e: ?) -> !this","!doc":"<p>Handles a click on a button menu</p>\n"},"afterComponentLayout":{"!type":"fn() -> !this","!doc":"<p>Called by the layout system after the Component has been laid out.</p>\n"},"applySelection":{"!type":"fn(node: ?) -> !this"},"applyUseSplitButtons":{"!type":"fn(useSplitButtons: ?, oldUseSplitButtons: ?) -> !this"},"getButtonUI":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.toolbar.Breadcrumb-cfg-buttonUI\" rel=\"Ext.toolbar.Breadcrumb-cfg-buttonUI\" class=\"docClass\">buttonUI</a>.</p>\n"},"getOverflowHandler":{"!type":"fn() -> string","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.toolbar.Breadcrumb-cfg-overflowHandler\" rel=\"Ext.toolbar.Breadcrumb-cfg-overflowHandler\" class=\"docClass\">overflowHandler</a>.</p>\n"},"getShowIcons":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.toolbar.Breadcrumb-cfg-showIcons\" rel=\"Ext.toolbar.Breadcrumb-cfg-showIcons\" class=\"docClass\">showIcons</a>.</p>\n"},"getStore":{"!type":"fn() -> ?","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.toolbar.Breadcrumb-cfg-store\" rel=\"Ext.toolbar.Breadcrumb-cfg-store\" class=\"docClass\">store</a>.</p>\n"},"getUseSplitButtons":{"!type":"fn() -> bool","!doc":"<p>Returns the value of <a href=\"#!/api/Ext.toolbar.Breadcrumb-cfg-useSplitButtons\" rel=\"Ext.toolbar.Breadcrumb-cfg-useSplitButtons\" class=\"docClass\">useSplitButtons</a>.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"setButtonUI":{"!type":"fn(buttonUI: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.toolbar.Breadcrumb-cfg-buttonUI\" rel=\"Ext.toolbar.Breadcrumb-cfg-buttonUI\" class=\"docClass\">buttonUI</a>.</p>\n"},"setOverflowHandler":{"!type":"fn(overflowHandler: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.toolbar.Breadcrumb-cfg-overflowHandler\" rel=\"Ext.toolbar.Breadcrumb-cfg-overflowHandler\" class=\"docClass\">overflowHandler</a>.</p>\n"},"setShowIcons":{"!type":"fn(showIcons: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.toolbar.Breadcrumb-cfg-showIcons\" rel=\"Ext.toolbar.Breadcrumb-cfg-showIcons\" class=\"docClass\">showIcons</a>.</p>\n"},"setStore":{"!type":"fn(store: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.toolbar.Breadcrumb-cfg-store\" rel=\"Ext.toolbar.Breadcrumb-cfg-store\" class=\"docClass\">store</a>.</p>\n"},"setUseSplitButtons":{"!type":"fn(useSplitButtons: ?)","!doc":"<p>Sets the value of <a href=\"#!/api/Ext.toolbar.Breadcrumb-cfg-useSplitButtons\" rel=\"Ext.toolbar.Breadcrumb-cfg-useSplitButtons\" class=\"docClass\">useSplitButtons</a>.</p>\n"},"updateOverflowHandler":{"!type":"fn(overflowHandler: ?) -> !this","!doc":"<p><debug></debug></p>\n"},"updateSelection":{"!type":"fn(node: ?) -> !this"},"updateStore":{"!type":"fn(store: ?, oldStore: ?) -> !this"},"selectionchange":{"!type":"fn(this: ?, node: ?, eOpts: ?)","!doc":"<p>Fires when the selected node changes</p>\n"}}},"Fill":{"!doc":"<p>A non-rendering placeholder item which instructs the Toolbar's Layout to begin using\nthe right-justified button container.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n     title: 'Toolbar Fill Example',\n     width: 300,\n     height: 200,\n     tbar : [\n         'Item 1',\n         { xtype: 'tbfill' },\n         'Item 2'\n     ],\n     renderTo: Ext.getBody()\n });\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"ariaRole":{"!type":"string"},"flex":{"!type":"number"},"isFill":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Fill, or subclass thereof.</p>\n"}}},"Item":{"!doc":"<p>The base class that other non-interacting Toolbar Item classes should extend in order to\nget some basic common toolbar item functionality.</p>\n","!type":"fn(config: ?)","prototype":{"overflowText":{"!type":"string","!doc":"<p>Text to be used for the menu if the item is overflowed.</p>\n"},"disable":{"!type":"fn() -> !this","!doc":"<p>Disable the component.</p>\n"},"enable":{"!type":"fn() -> !this","!doc":"<p>Enable the component</p>\n"},"focus":{"!type":"fn() -> ?","!doc":"<p>Try to focus this component.</p>\n"}}},"Paging":{"!doc":"<p>As the number of records increases, the time required for the browser to render them increases. Paging is used to\nreduce the amount of data exchanged with the client. Note: if there are more records/rows than can be viewed in the\navailable screen area, vertical scrollbars will be added.</p>\n\n<p>Paging is typically handled on the server side (see exception below). The client sends parameters to the server side,\nwhich the server needs to interpret and then respond with the appropriate data.</p>\n\n<p><a href=\"#!/api/Ext.toolbar.Paging\" rel=\"Ext.toolbar.Paging\" class=\"docClass\">Ext.toolbar.Paging</a> is a specialized toolbar that is bound to a Ext.data.Store and provides automatic\npaging control. This Component loads blocks of data into the <a href=\"#!/api/Ext.toolbar.Paging-cfg-store\" rel=\"Ext.toolbar.Paging-cfg-store\" class=\"docClass\">store</a> by passing\nparameters used for paging criteria.</p>\n\n<p><p><img src=\"\" alt=\"Ext.toolbar.Paging component\" width=\"\" height=\"\"></p></p>\n\n<p>Paging Toolbar is typically used as one of the Grid's toolbars:</p>\n\n<pre class='inline-example '><code>var itemsPerPage = 2;   // set the number of items you want per page\n\nvar store = Ext.create('Ext.data.Store', {\n    id:'simpsonsStore',\n    autoLoad: false,\n    fields:['name', 'email', 'phone'],\n    pageSize: itemsPerPage, // items per page\n    proxy: {\n        type: 'ajax',\n        url: 'pagingstore.js',  // url that will load data with respect to start and limit params\n        reader: {\n            type: 'json',\n            rootProperty: 'items',\n            totalProperty: 'total'\n        }\n    }\n});\n\n// specify segment of data you want to load using params\nstore.load({\n    params:{\n        start:0,\n        limit: itemsPerPage\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Ext.grid.Panel</a>', {\n    title: 'Simpsons',\n    store: store,\n    columns: [\n        { header: 'Name',  dataIndex: 'name' },\n        { header: 'Email', dataIndex: 'email', flex: 1 },\n        { header: 'Phone', dataIndex: 'phone' }\n    ],\n    width: 400,\n    height: 125,\n    dockedItems: [{\n        xtype: 'pagingtoolbar',\n        store: store,   // same store GridPanel is using\n        dock: 'bottom',\n        displayInfo: true\n    }],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>To use paging, you need to set a pageSize configuration on the Store, and pass the paging requirements to\nthe server when the store is first loaded.</p>\n\n<pre><code>store.load({\n    params: {\n        // specify params for the first page load if using paging\n        start: 0,\n        limit: myPageSize,\n        // other params\n        foo:   'bar'\n    }\n});\n</code></pre>\n\n<p>If using store's autoLoad configuration:</p>\n\n<pre><code>var myStore = Ext.create('Ext.data.Store', {\n    autoLoad: {start: 0, limit: 25},\n    ...\n});\n</code></pre>\n\n<p>The packet sent back from the server would have this form:</p>\n\n<pre><code>{\n    \"success\": true,\n    \"results\": 2000,\n    \"rows\": [ // ***Note:** this must be an Array\n        { \"id\":  1, \"name\": \"Bill\", \"occupation\": \"Gardener\" },\n        { \"id\":  2, \"name\":  \"Ben\", \"occupation\": \"Horticulturalist\" },\n        ...\n        { \"id\": 25, \"name\":  \"Sue\", \"occupation\": \"Botanist\" }\n    ]\n}\n</code></pre>\n\n<h2>Paging with Local Data</h2>\n\n<p>Paging can also be accomplished with local data using extensions:</p>\n\n<ul>\n<li><a href=\"http://sencha.com/forum/showthread.php?t=71532\">Ext.ux.data.PagingStore</a></li>\n<li>Paging Memory Proxy (examples/ux/PagingMemoryProxy.js)</li>\n</ul>\n\n","!type":"fn(config: ?)","prototype":{"afterPageText":{"!type":"string","!doc":"<p>Customizable piece of the default paging text. Note that this string is formatted using\n{0} as a token that is replaced by the number of total pages. This token should be preserved when overriding this\nstring if showing the total page count is desired.</p>\n"},"beforePageText":{"!type":"string","!doc":"<p>The text displayed before the input item.</p>\n"},"displayInfo":{"!type":"bool","!doc":"<p>true to display the displayMsg</p>\n"},"displayMsg":{"!type":"string","!doc":"<p>The paging status message to display. Note that this string is\nformatted using the braced numbers {0}-{2} as tokens that are replaced by the values for start, end and total\nrespectively. These tokens should be preserved when overriding this string if showing those values is desired.</p>\n"},"emptyMsg":{"!type":"string","!doc":"<p>The message to display when no records are found.</p>\n"},"firstText":{"!type":"string","!doc":"<p>The quicktip text displayed for the first page button.\n<strong>Note</strong>: quick tips must be initialized for the quicktip to show.</p>\n"},"inputItemWidth":{"!type":"number","!doc":"<p>The width in pixels of the input field used to display and change the current page number.</p>\n"},"lastText":{"!type":"string","!doc":"<p>The quicktip text displayed for the last page button.\n<strong>Note</strong>: quick tips must be initialized for the quicktip to show.</p>\n"},"nextText":{"!type":"string","!doc":"<p>The quicktip text displayed for the next page button.\n<strong>Note</strong>: quick tips must be initialized for the quicktip to show.</p>\n"},"prependButtons":{"!type":"bool","!doc":"<p>true to insert any configured items <em>before</em> the paging buttons.</p>\n"},"prevText":{"!type":"string","!doc":"<p>The quicktip text displayed for the previous page button.\n<strong>Note</strong>: quick tips must be initialized for the quicktip to show.</p>\n"},"refreshText":{"!type":"string","!doc":"<p>The quicktip text displayed for the Refresh button.\n<strong>Note</strong>: quick tips must be initialized for the quicktip to show.</p>\n"},"store":{"!doc":"<p>The Ext.data.Store the paging toolbar should use as its data source.</p>\n"},"beforeLoad":{"!type":"fn() -> !this"},"beforeRender":{"!type":"fn() -> !this"},"doRefresh":{"!type":"fn() -> bool","!doc":"<p>Refresh the current page, has the same effect as clicking the 'refresh' button.\nFires the <a href=\"#!/api/Ext.toolbar.Paging-event-beforechange\" rel=\"Ext.toolbar.Paging-event-beforechange\" class=\"docClass\">beforechange</a> event. If the event returns <code>false</code>, then\nthe load will not be attempted.</p>\n"},"getInputItem":{"!type":"fn() -> !this"},"getPageData":{"!type":"fn() -> !this"},"getPagingItems":{"!type":"fn() -> !this","!doc":"<p>Gets the standard paging items in the toolbar</p>\n"},"getStoreListeners":{"!type":"fn() -> ?","!doc":"<p>Gets the listeners to bind to a new store.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"moveFirst":{"!type":"fn() -> bool","!doc":"<p>Move to the first page, has the same effect as clicking the 'first' button.\nFires the <a href=\"#!/api/Ext.toolbar.Paging-event-beforechange\" rel=\"Ext.toolbar.Paging-event-beforechange\" class=\"docClass\">beforechange</a> event. If the event returns <code>false</code>, then\nthe load will not be attempted.</p>\n"},"moveLast":{"!type":"fn() -> bool","!doc":"<p>Move to the last page, has the same effect as clicking the 'last' button.\nFires the <a href=\"#!/api/Ext.toolbar.Paging-event-beforechange\" rel=\"Ext.toolbar.Paging-event-beforechange\" class=\"docClass\">beforechange</a> event. If the event returns <code>false</code>, then\nthe load will not be attempted.</p>\n"},"moveNext":{"!type":"fn() -> bool","!doc":"<p>Move to the next page, has the same effect as clicking the 'next' button.\nFires the <a href=\"#!/api/Ext.toolbar.Paging-event-beforechange\" rel=\"Ext.toolbar.Paging-event-beforechange\" class=\"docClass\">beforechange</a> event. If the event returns <code>false</code>, then\nthe load will not be attempted.</p>\n"},"movePrevious":{"!type":"fn() -> bool","!doc":"<p>Move to the previous page, has the same effect as clicking the 'previous' button.\nFires the <a href=\"#!/api/Ext.toolbar.Paging-event-beforechange\" rel=\"Ext.toolbar.Paging-event-beforechange\" class=\"docClass\">beforechange</a> event. If the event returns <code>false</code>, then\nthe load will not be attempted.</p>\n"},"onDestroy":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior to the destroy operation.\nAfter calling the superclass's onDestroy, the Component will be destroyed.</p>\n"},"onLoad":{"!type":"fn() -> !this"},"onLoadError":{"!type":"fn() -> !this"},"onPagingBlur":{"!type":"fn(e: ?) -> !this"},"onPagingKeyDown":{"!type":"fn(field: ?, e: ?) -> !this"},"processKeyEvent":{"!type":"fn(field: ?, e: ?) -> !this"},"readPageFromInput":{"!type":"fn(pageData: ?) -> !this"},"setChildDisabled":{"!type":"fn(selector: ?, disabled: ?) -> !this"},"updateInfo":{"!type":"fn() -> !this"},"beforechange":{"!type":"fn(this: ?, page: ?, eOpts: ?)","!doc":"<p>Fires just before the active page is changed. Return false to prevent the active page from being changed.</p>\n"},"change":{"!type":"fn(this: ?, pageData: ?, eOpts: ?)","!doc":"<p>Fires after the active page has been changed.</p>\n"}}},"Toolbar":{"prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"defaultButtonUI":{"!type":"string","!doc":"<p>A default <a href=\"#!/api/Ext.Component-cfg-ui\" rel=\"Ext.Component-cfg-ui\" class=\"docClass\">ui</a> to use for <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Button</a> items. This is a quick and simple\nway to change the look of all child <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Buttons</a>.</p>\n\n<p>If there is no value for defaultButtonUI, the button's <a href=\"#!/api/Ext.Component-cfg-ui\" rel=\"Ext.Component-cfg-ui\" class=\"docClass\">ui</a> value will get <code>-toolbar</code>\nappended so the <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Button</a> has a different look when it's a child of a <a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Toolbar</a>.\nTo prevent this and have the same look as buttons outside of a toolbar, you can provide a string value to the defaultButtonUI:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    renderTo    : document.body,\n    width       : 300,\n    title       : 'Panel',\n    html        : 'Some Body',\n    dockedItems : [\n        {\n            xtype           : 'toolbar',\n            dock            : 'top',\n            defaultButtonUI : 'default',\n            items           : [\n                {\n                    text : 'Save'\n                },\n                {\n                    text : 'Remove'\n                }\n            ]\n        }\n    ]\n});\n</code></pre>\n"},"defaultFieldUI":{"!type":"string","!doc":"<p>Default UI for form field items.</p>\n"},"defaultFooterButtonUI":{"!type":"string","!doc":"<p>Default UI for Buttons if the toolbar has a UI of 'footer'</p>\n"},"defaultFooterFieldUI":{"!type":"string","!doc":"<p>Default UI for Form Fields if the toolbar has a UI of 'footer'</p>\n"},"defaultType":{"!type":"string","!doc":"<p>The default <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">xtype</a> of child Components to create in this Container when\na child item is specified as a raw configuration object, rather than as an instantiated Component.</p>\n"},"enableOverflow":{"!type":"bool","!doc":"<p>Configure true to make the toolbar provide a button which activates a dropdown Menu to show\nitems which overflow the Toolbar's width.  Setting this too true is the equivalent\nof setting <code><a href=\"#!/api/Ext.toolbar.Toolbar-cfg-overflowHandler\" rel=\"Ext.toolbar.Toolbar-cfg-overflowHandler\" class=\"docClass\">overflowHandler</a>:'menu'</code>.</p>\n"},"layout":{"!doc":"<p>This class assigns a default layout (<code>layout: 'hbox'</code> or <code>layout: 'vbox'</code> depending upon orientation).</p>\n\n<p>Developers <em>may</em> override this configuration option if another layout is required.\nSee <a href=\"#!/api/Ext.container.Container-cfg-layout\" rel=\"Ext.container.Container-cfg-layout\" class=\"docClass\">Ext.container.Container.layout</a> for additional information.</p>\n"},"overflowHandler":{"!type":"string","!doc":"<ul>\n<li><code>null</code> - hidden overflow</li>\n<li><code>'scroller'</code> to render left/right scroller buttons on either side of the breadcrumb</li>\n<li><code>'menu'</code> to render the overflowing buttons as items of an overflow menu.</li>\n</ul>\n\n"},"vertical":{"!type":"bool","!doc":"<p>Set to <code>true</code> to make the toolbar vertical. The layout will become a <code>vbox</code>.</p>\n"},"ariaRole":{"!type":"string","!doc":"<p><editor-fold desc=\"Properties\"></editor></p>\n\n<hr />\n\n<p> Begin Properties</p>\n\n<hr />\n"},"isToolbar":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated Toolbar, or subclass thereof.</p>\n"},"itemCls":{"!type":"string"},"privates":{"!doc":"<hr />\n\n<p> End Methods</p>\n\n<hr />\n\n<p> </editor-fold></p>\n"},"trackMenus":{"!type":"bool"},"add":{"!type":"fn(args: ?) -> ?","!doc":"<p>Adds element(s) to the toolbar -- this function takes a variable number of\narguments of mixed type and adds them to the toolbar.</p>\n\n<p><strong>Note</strong>: See the notes within <a href=\"#!/api/Ext.container.Container-method-add\" rel=\"Ext.container.Container-method-add\" class=\"docClass\">Ext.container.Container.add</a>.</p>\n"},"getRefItems":{"!type":"fn(deep: ?) -> !this","!doc":"<p>Used by ComponentQuery, child and down to retrieve all of the items\nwhich can potentially be considered a child of this Container.</p>\n\n<p>This may be overriden by Components which have ownership of Components\nthat are not contained in the <a href=\"#!/api/Ext.toolbar.Toolbar-property-items\" rel=\"Ext.toolbar.Toolbar-property-items\" class=\"docClass\">items</a> collection.</p>\n\n<p>NOTE: IMPORTANT note for maintainers:\nItems are returned in tree traversal order. Each item is appended to the result array\nfollowed by the results of that child's getRefItems call.\nFloating child items are appended after internal child items.</p>\n"},"insert":{"!type":"fn(index: ?, component: ?) -> ?","!doc":"<p>Inserts a Component into this Container at a specified index.</p>\n"},"lookupComponent":{"!type":"fn(c: ?) -> !this"},"onAdd":{"!type":"fn(component: ?) -> !this","!doc":"<p>This method is invoked after a new Component has been added. It\nis passed the Component which has been added. This method may\nbe used to update any internal structure which may depend upon\nthe state of the child items.</p>\n"},"onBeforeAdd":{"!type":"fn(component: ?) -> !this","!doc":"<p>This method is invoked before adding a new child Component. It\nis passed the new Component, and may be used to modify the\nComponent, or prepare the Container in some way. Returning\nfalse aborts the add operation.</p>\n"},"onRemove":{"!type":"fn(c: ?) -> !this","!doc":"<p>This method is invoked after a new Component has been\nremoved. It is passed the Component which has been\nremoved. This method may be used to update any internal\nstructure which may depend upon the state of the child items.</p>\n"},"overflowchange":{"!type":"fn(lastHiddenCount: ?, hiddenCount: ?, The: ?, eOpts: ?)","!doc":"<p>Fires after the overflow state has changed if this toolbar has been configured with\nan <code><a href=\"#!/api/Ext.toolbar.Toolbar-cfg-overflowHandler\" rel=\"Ext.toolbar.Toolbar-cfg-overflowHandler\" class=\"docClass\">overflowHandler</a></code>.</p>\n"},"shortcuts":{},"shortcutsHV":{},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"}},"!doc":"<p>Basic Toolbar class. Although the <a href=\"#!/api/Ext.container.Container-cfg-defaultType\" rel=\"Ext.container.Container-cfg-defaultType\" class=\"docClass\">defaultType</a> for\nToolbar is <a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">button</a>, Toolbar elements (child items for the Toolbar container)\nmay be virtually any type of Component. Toolbar elements can be created explicitly via their\nconstructors, or implicitly via their xtypes, and can be <a href=\"#!/api/Ext.toolbar.Toolbar-method-add\" rel=\"Ext.toolbar.Toolbar-method-add\" class=\"docClass\">add</a>ed dynamically.</p>\n\n<h2>Some items have shortcut strings for creation:</h2>\n\n<table>\n<thead>\n<tr>\n<th style=\"text-align:left;\"> Shortcut </th>\n<th style=\"text-align:left;\"> xtype         </th>\n<th style=\"text-align:left;\"> Class                         </th>\n<th style=\"text-align:left;\"> Description</th>\n</tr>\n</thead>\n<tbody>\n<tr>\n<td style=\"text-align:left;\"> '->'     </td>\n<td style=\"text-align:left;\"> <code>tbfill</code>      </td>\n<td style=\"text-align:left;\"> <a href=\"#!/api/Ext.toolbar.Fill\" rel=\"Ext.toolbar.Fill\" class=\"docClass\">Ext.toolbar.Fill</a>      </td>\n<td style=\"text-align:left;\"> begin using the right-justified button container</td>\n</tr>\n<tr>\n<td style=\"text-align:left;\"> '-'      </td>\n<td style=\"text-align:left;\"> <code>tbseparator</code> </td>\n<td style=\"text-align:left;\"> <a href=\"#!/api/Ext.toolbar.Separator\" rel=\"Ext.toolbar.Separator\" class=\"docClass\">Ext.toolbar.Separator</a> </td>\n<td style=\"text-align:left;\"> add a vertical separator bar between toolbar items</td>\n</tr>\n<tr>\n<td style=\"text-align:left;\"> ' '      </td>\n<td style=\"text-align:left;\"> <code>tbspacer</code>    </td>\n<td style=\"text-align:left;\"> <a href=\"#!/api/Ext.toolbar.Spacer\" rel=\"Ext.toolbar.Spacer\" class=\"docClass\">Ext.toolbar.Spacer</a>    </td>\n<td style=\"text-align:left;\"> add horizontal space between elements</td>\n</tr>\n</tbody>\n</table>\n\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Ext.toolbar.Toolbar</a>', {\n    renderTo: document.body,\n    width   : 500,\n    items: [\n        {\n            // xtype: 'button', // default for Toolbars\n            text: 'Button'\n        },\n        {\n            xtype: 'splitbutton',\n            text : 'Split Button'\n        },\n        // begin using the right-justified button container\n        '-&gt;', // same as { xtype: 'tbfill' }\n        {\n            xtype    : 'textfield',\n            name     : 'field1',\n            emptyText: 'enter search term'\n        },\n        // add a vertical separator bar between toolbar items\n        '-', // same as {xtype: 'tbseparator'} to create <a href=\"#!/api/Ext.toolbar.Separator\" rel=\"Ext.toolbar.Separator\" class=\"docClass\">Ext.toolbar.Separator</a>\n        'text 1', // same as {xtype: 'tbtext', text: 'text1'} to create <a href=\"#!/api/Ext.toolbar.TextItem\" rel=\"Ext.toolbar.TextItem\" class=\"docClass\">Ext.toolbar.TextItem</a>\n        { xtype: 'tbspacer' },// same as ' ' to create <a href=\"#!/api/Ext.toolbar.Spacer\" rel=\"Ext.toolbar.Spacer\" class=\"docClass\">Ext.toolbar.Spacer</a>\n        'text 2',\n        { xtype: 'tbspacer', width: 50 }, // add a 50px space\n        'text 3'\n    ]\n});\n</code></pre>\n\n<p>Toolbars have <a href=\"#!/api/Ext.toolbar.Toolbar-method-enable\" rel=\"Ext.toolbar.Toolbar-method-enable\" class=\"docClass\">enable</a> and <a href=\"#!/api/Ext.toolbar.Toolbar-method-disable\" rel=\"Ext.toolbar.Toolbar-method-disable\" class=\"docClass\">disable</a> methods which when called, will\nenable/disable all items within your toolbar.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Ext.toolbar.Toolbar</a>', {\n    renderTo: document.body,\n    width   : 400,\n    items: [\n        {\n            text: 'Button'\n        },\n        {\n            xtype: 'splitbutton',\n            text : 'Split Button'\n        },\n        '-&gt;',\n        {\n            xtype    : 'textfield',\n            name     : 'field1',\n            emptyText: 'enter search term'\n        }\n    ]\n});\n</code></pre>\n\n<p>Example</p>\n\n<pre class='inline-example '><code>var enableBtn = Ext.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>', {\n    text    : 'Enable All Items',\n    disabled: true,\n    scope   : this,\n    handler : function() {\n        //disable the enable button and enable the disable button\n        enableBtn.disable();\n        disableBtn.enable();\n\n        //enable the toolbar\n        toolbar.enable();\n    }\n});\n\nvar disableBtn = Ext.create('<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>', {\n    text    : 'Disable All Items',\n    scope   : this,\n    handler : function() {\n        //enable the enable button and disable button\n        disableBtn.disable();\n        enableBtn.enable();\n\n        //disable the toolbar\n        toolbar.disable();\n    }\n});\n\nvar toolbar = Ext.create('<a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Ext.toolbar.Toolbar</a>', {\n    renderTo: document.body,\n    width   : 400,\n    margin  : '5 0 0 0',\n    items   : [enableBtn, disableBtn]\n});\n</code></pre>\n\n<p>Adding items to and removing items from a toolbar is as simple as calling the <a href=\"#!/api/Ext.toolbar.Toolbar-method-add\" rel=\"Ext.toolbar.Toolbar-method-add\" class=\"docClass\">add</a>\nand <a href=\"#!/api/Ext.toolbar.Toolbar-method-remove\" rel=\"Ext.toolbar.Toolbar-method-remove\" class=\"docClass\">remove</a> methods. There is also a <a href=\"#!/api/Ext.toolbar.Toolbar-method-removeAll\" rel=\"Ext.toolbar.Toolbar-method-removeAll\" class=\"docClass\">removeAll</a> method\nwhich remove all items within the toolbar.</p>\n\n<pre class='inline-example '><code>var toolbar = Ext.create('<a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Ext.toolbar.Toolbar</a>', {\n    renderTo: document.body,\n    width   : 700,\n    items: [\n        {\n            text: 'Example Button'\n        }\n    ]\n});\n\nvar addedItems = [];\n\nExt.create('<a href=\"#!/api/Ext.toolbar.Toolbar\" rel=\"Ext.toolbar.Toolbar\" class=\"docClass\">Ext.toolbar.Toolbar</a>', {\n    renderTo: document.body,\n    width   : 700,\n    margin  : '5 0 0 0',\n    items   : [\n        {\n            text   : 'Add a button',\n            scope  : this,\n            handler: function() {\n                var text = prompt('Please enter the text for your button:');\n                addedItems.push(toolbar.add({\n                    text: text\n                }));\n            }\n        },\n        {\n            text   : 'Add a text item',\n            scope  : this,\n            handler: function() {\n                var text = prompt('Please enter the text for your item:');\n                addedItems.push(toolbar.add(text));\n            }\n        },\n        {\n            text   : 'Add a toolbar separator',\n            scope  : this,\n            handler: function() {\n                addedItems.push(toolbar.add('-'));\n            }\n        },\n        {\n            text   : 'Add a toolbar spacer',\n            scope  : this,\n            handler: function() {\n                addedItems.push(toolbar.add('-&gt;'));\n            }\n        },\n        '-&gt;',\n        {\n            text   : 'Remove last inserted item',\n            scope  : this,\n            handler: function() {\n                if (addedItems.length) {\n                    toolbar.remove(addedItems.pop());\n                } else if (toolbar.items.length) {\n                    toolbar.remove(toolbar.items.last());\n                } else {\n                    alert('No items in the toolbar');\n                }\n            }\n        },\n        {\n            text   : 'Remove all items',\n            scope  : this,\n            handler: function() {\n                toolbar.removeAll();\n            }\n        }\n    ]\n});\n</code></pre>\n","!type":"fn(config: ?)"},"Separator":{"!doc":"<p>A simple class that adds a vertical separator bar between toolbar items (css class: 'x-toolbar-separator').</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Toolbar Separator Example',\n    width: 300,\n    height: 200,\n    tbar : [\n        'Item 1',\n        { xtype: 'tbseparator' },\n        'Item 2'\n    ],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"ariaRole":{"!type":"string"},"focusable":{"!type":"bool"}}},"Spacer":{"!doc":"<p>A simple element that adds extra horizontal space between items in a toolbar.\nBy default a 2px wide space is added via CSS specification:</p>\n\n<pre><code>.x-toolbar .x-toolbar-spacer {\n    width: 2px;\n}\n</code></pre>\n\n<p>Example:</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Toolbar Spacer Example',\n    width: 300,\n    height: 200,\n    tbar : [\n        'Item 1',\n        { xtype: 'tbspacer' }, // or ' '\n        'Item 2',\n        // space width is also configurable via javascript\n        { xtype: 'tbspacer', width: 50 }, // add a 50px space\n        'Item 3'\n    ],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(config: ?)","prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"ariaRole":{"!type":"string"},"focusable":{"!type":"bool"}}},"TextItem":{"!doc":"<p>A simple class that renders text directly into a toolbar.</p>\n\n<pre class='inline-example '><code>Ext.create('<a href=\"#!/api/Ext.panel.Panel\" rel=\"Ext.panel.Panel\" class=\"docClass\">Ext.panel.Panel</a>', {\n    title: 'Panel with TextItem',\n    width: 300,\n    height: 200,\n    tbar: [\n        { xtype: 'tbtext', text: 'Sample Text Item' }\n    ],\n    renderTo: Ext.getBody()\n});\n</code></pre>\n","!type":"fn(text: ?)","prototype":{"baseCls":{"!type":"string","!doc":"<p>The base CSS class to apply to this component's element. This will also be prepended to elements within this\ncomponent like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and\nyou want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use\n<code>componentCls</code> to add specific styling for this component.</p>\n"},"renderTpl":{"!doc":"<p>An XTemplate used to create the internal structure inside this Component's encapsulating\n<a href=\"#!/api/Ext.toolbar.TextItem-method-getEl\" rel=\"Ext.toolbar.TextItem-method-getEl\" class=\"docClass\">Element</a>.</p>\n\n<p>You do not normally need to specify this. For the base classes <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> and\n<a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Ext.container.Container</a>, this defaults to <strong><code>null</code></strong> which means that they will be initially rendered\nwith no internal structure; they render their <a href=\"#!/api/Ext.toolbar.TextItem-method-getEl\" rel=\"Ext.toolbar.TextItem-method-getEl\" class=\"docClass\">Element</a> empty. The more specialized Ext JS and Sencha Touch\nclasses which use a more complex DOM structure, provide their own template definitions.</p>\n\n<p>This is intended to allow the developer to create application-specific utility Components with customized\ninternal structure.</p>\n\n<p>Upon rendering, any created child elements may be automatically imported into object properties using the\n<a href=\"#!/api/Ext.toolbar.TextItem-cfg-renderSelectors\" rel=\"Ext.toolbar.TextItem-cfg-renderSelectors\" class=\"docClass\">renderSelectors</a> and <a href=\"#!/api/Ext.toolbar.TextItem-cfg-childEls\" rel=\"Ext.toolbar.TextItem-cfg-childEls\" class=\"docClass\">childEls</a> options.</p>\n"},"text":{"!type":"string","!doc":"<p>The text to be used as innerHTML (html tags are accepted).</p>\n"},"ariaRole":{},"beforeRender":{"!type":"fn() -> !this"},"setText":{"!type":"fn(text: ?) -> !this","!doc":"<p>Updates this item's text, setting the text to be used as innerHTML.</p>\n"}}}},"tree":{"Column":{"!doc":"<p>Provides indentation and folder structure markup for a Tree taking into account\ndepth and position within the tree hierarchy.</p>\n","!type":"fn(config: ?)","prototype":{"draggable":{"!type":"bool","!doc":"<p>False to disable drag-drop reordering of this column.</p>\n"},"hideable":{"!type":"bool","!doc":"<p>False to prevent the user from hiding this column.</p>\n"},"lockable":{"!type":"bool","!doc":"<p>If the grid is configured with <a href=\"#!/api/Ext.panel.Table-cfg-enableLocking\" rel=\"Ext.panel.Table-cfg-enableLocking\" class=\"docClass\">enableLocking</a>, or has columns which are\nconfigured with a <a href=\"#!/api/Ext.tree.Column-cfg-locked\" rel=\"Ext.tree.Column-cfg-locked\" class=\"docClass\">locked</a> value, this option may be used to disable user-driven locking or unlocking\nof this column. This column will remain in the side into which its own <a href=\"#!/api/Ext.tree.Column-cfg-locked\" rel=\"Ext.tree.Column-cfg-locked\" class=\"docClass\">locked</a> configuration placed it.</p>\n"},"tdCls":{"!type":"string","!doc":"<p>A CSS class names to apply to the table cells for this column.</p>\n"},"autoLock":{"!type":"bool"},"cellTpl":{},"checkboxCls":{"!type":"string"},"elbowCls":{"!type":"string"},"expanderCls":{"!type":"string"},"iconCls":{"!type":"string"},"innerCls":{"!type":"string"},"isTreeColumn":{"!type":"bool"},"textCls":{"!type":"string"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"initTemplateRendererData":{"!type":"fn(value: ?, metaData: ?, record: ?, rowIdx: ?, colIdx: ?, store: ?, view: ?) -> !this"},"treeRenderer":{"!type":"fn(value: ?, metaData: ?, record: ?, rowIdx: ?, colIdx: ?, store: ?, view: ?) -> !this"}}},"Panel":{"!doc":"<p>The TreePanel provides tree-structured UI representation of tree-structured data.\nA TreePanel must be bound to a Ext.data.TreeStore.</p>\n\n<p>TreePanels support multiple columns through the <a href=\"#!/api/Ext.tree.Panel-cfg-columns\" rel=\"Ext.tree.Panel-cfg-columns\" class=\"docClass\">columns</a> configuration.</p>\n\n<p>By default a TreePanel contains a single column which uses the <code>text</code> Field of\nthe store's nodes.</p>\n\n<p>Simple TreePanel using inline data:</p>\n\n<pre class='inline-example '><code>var store = Ext.create('Ext.data.TreeStore', {\n    root: {\n        expanded: true,\n        children: [\n            { text: \"detention\", leaf: true },\n            { text: \"homework\", expanded: true, children: [\n                { text: \"book report\", leaf: true },\n                { text: \"algebra\", leaf: true}\n            ] },\n            { text: \"buy lottery tickets\", leaf: true }\n        ]\n    }\n});\n\nExt.create('<a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">Ext.tree.Panel</a>', {\n    title: 'Simple Tree',\n    width: 200,\n    height: 150,\n    store: store,\n    rootVisible: false,\n    renderTo: Ext.getBody()\n});\n</code></pre>\n\n<p>For the tree node config options (like <code>text</code>, <code>leaf</code>, <code>expanded</code>), see the documentation of\nNodeInterface config options.</p>\n\n<p>Unless the TreeStore is configured with a model of your choosing, nodes in the Ext.data.TreeStore are by default, instances of Ext.data.TreeModel.</p>\n\n<h1>Heterogeneous node types.</h1>\n\n<p>If the tree needs to use different data model classes at different levels there is much flexibility in how to specify this.</p>\n\n<h3>Configuring the Reader.</h3>\n\n<p>If you configure the proxy's reader with a typeProperty, then the server is in control of which data model\ntypes are created. A discriminator field is used in the raw data to decide which class to instantiate.\n<strong>If this is configured, then the data from the server is prioritized over other ways of determining node class</strong>.</p>\n\n<pre class='inline-example '><code>Ext.define('myApp.Territory', {\n    extend: 'Ext.data.TreeModel',\n    fields: [{\n        name: 'text',\n        mapping: 'name'\n    }]\n});\nExt.define('myApp.Country', {\n    extend: 'Ext.data.TreeModel',\n    fields: [{\n        name: 'text',\n        mapping: 'name'\n    }]\n});\nExt.define('myApp.City', {\n    extend: 'Ext.data.TreeModel',\n    fields: [{\n        name: 'text',\n        mapping: 'name'\n    }]\n});\nExt.create('<a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">Ext.tree.Panel</a>', {\n    renderTo: document.body,\n    height: 200,\n    width: 400,\n    title: 'Sales Areas - using typeProperty',\n    rootVisible: false,\n    store: {\n        // Child types use namespace of store's model by default\n        model: 'myApp.Territory',\n        proxy: {\n            type: 'memory',\n            reader: {\n                typeProperty: 'mtype'\n            }\n        },\n        root: {\n            children: [{\n                name: 'Europe, ME, Africa',\n                mtype: 'Territory',\n                children: [{\n                    name: 'UK of GB &amp; NI',\n                    mtype: 'Country',\n                    children: [{\n                        name: 'London',\n                        mtype: 'City',\n                        leaf: true\n                    }]\n                }]\n            }, {\n                name: 'North America',\n                mtype: 'Territory',\n                children: [{\n                    name: 'USA',\n                    mtype: 'Country',\n                    children: [{\n                        name: 'Redwood City',\n                        mtype: 'City',\n                        leaf: true\n                    }]\n                }]\n            }]\n        }\n    }\n});\n</code></pre>\n\n<h3>Node being loaded decides.</h3>\n\n<p>You can declare your TreeModel subclasses with a childType which means that the node being loaded decides the\nclass to instantiate for all of its child nodes.</p>\n\n<p>It is important to note that if the root node is <a href=\"#!/api/Ext.tree.Panel-cfg-rootVisible\" rel=\"Ext.tree.Panel-cfg-rootVisible\" class=\"docClass\">hidden</a>, its type will default to the store's model type, and if left\nas the default (<code>Ext.data.TreeModel</code>) this will have no knowledge of creation of special child node types. So be sure to specify a store model in this case:</p>\n\n<pre class='inline-example '><code>Ext.define('myApp.TerritoryRoot', {\n    extend: 'Ext.data.TreeModel',\n    childType: 'myApp.Territory',\n    fields: [{\n        name: 'text',\n        mapping: 'name'\n    }]\n});\nExt.define('myApp.Territory', {\n    extend: 'Ext.data.TreeModel',\n    childType: 'myApp.Country',\n    fields: [{\n        name: 'text',\n        mapping: 'name'\n    }]\n});\nExt.define('myApp.Country', {\n    extend: 'Ext.data.TreeModel',\n    childType: 'myApp.City',\n    fields: [{\n        name: 'text',\n        mapping: 'name'\n    }]\n});\nExt.define('myApp.City', {\n    extend: 'Ext.data.TreeModel',\n    fields: [{\n        name: 'text',\n        mapping: 'name'\n    }]\n});\nExt.create('<a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">Ext.tree.Panel</a>', {\n    renderTo: document.body,\n    height: 200,\n    width: 400,\n    title: 'Sales Areas',\n    rootVisible: false,\n    store: {\n        model: 'myApp.TerritoryRoot', // Needs to be this so it knows to create 'Country' child nodes\n        root: {\n            children: [{\n                name: 'Europe, ME, Africa',\n                children: [{\n                    name: 'UK of GB &amp; NI',\n                    children: [{\n                        name: 'London',\n                        leaf: true\n                    }]\n                }]\n            }, {\n                name: 'North America',\n                children: [{\n                    name: 'USA',\n                    children: [{\n                        name: 'Redwood City',\n                        leaf: true\n                    }]\n                }]\n            }]\n        }\n    }\n});\n</code></pre>\n\n<h1>Data structure</h1>\n\n<p>The TreeStore maintains a root node and a hierarchical structure of nodes.</p>\n\n<p>The <a href=\"#!/api/Ext.tree.View\" rel=\"Ext.tree.View\" class=\"docClass\">UI</a> of the tree is driven by a {Ext.data.NodeStore NodeStore} which is a flattened view of <em>visible</em> nodes.\nThe NodeStore is dynamically updated to reflect the visibility state of nodes as nodes are added, removed or expanded. The UI\nresponds to mutation events fire by the NodeStore.</p>\n\n<p>Note that nodes have several more fields in order to describe their state within the hierarchy.</p>\n\n<p>If you add store listeners to the update event, then you will recieve notification when any of this state changes.\nYou should check the array of modified field names passed to the listener to decide whether the listener should take action or ignore the event.</p>\n","!type":"fn(config: ?)","prototype":{"animate":{"!type":"bool","!doc":"<p>True to enable animated expand/collapse. Defaults to the value of <a href=\"#!/api/Ext-property-enableFx\" rel=\"Ext-property-enableFx\" class=\"docClass\">Ext.enableFx</a>.</p>\n"},"displayField":{"!type":"string","!doc":"<p>The field inside the model that will be used as the node's text.</p>\n"},"folderSort":{"!type":"bool","!doc":"<p>True to automatically prepend a leaf sorter to the store.</p>\n"},"hideHeaders":{"!type":"bool","!doc":"<p>True to hide the headers.</p>\n"},"lines":{"!type":"bool","!doc":"<p>False to disable tree lines.</p>\n"},"root":{"!doc":"<p>Allows you to not specify a store on this TreePanel. This is useful for creating a simple tree with preloaded\ndata without having to specify a TreeStore and Model. A store and model will be created and root will be passed\nto that store. For example:</p>\n\n<pre><code>Ext.create('<a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">Ext.tree.Panel</a>', {\n    title: 'Simple Tree',\n    root: {\n        text: \"Root node\",\n        expanded: true,\n        children: [\n            { text: \"Child 1\", leaf: true },\n            { text: \"Child 2\", leaf: true }\n        ]\n    },\n    renderTo: Ext.getBody()\n});\n</code></pre>\n"},"rootVisible":{"!type":"bool","!doc":"<p>False to hide the root node.</p>\n\n<p>Note that trees <em>always</em> have a root node. If you do not specify a <a href=\"#!/api/Ext.tree.Panel-cfg-root\" rel=\"Ext.tree.Panel-cfg-root\" class=\"docClass\">root</a> node, one will be created.</p>\n\n<p>If the root node is not visible, then in order for a tree to appear to the end user, the root node is autoloaded with its child nodes.</p>\n"},"rowLines":{"!type":"bool","!doc":"<p>Configure as true to separate rows with visible horizontal lines (depends on theme).</p>\n"},"selType":{"!type":"string","!doc":"<p>An xtype of selection model to use. This is used to create selection model if just\na config object or nothing at all given in <a href=\"#!/api/Ext.tree.Panel-cfg-selModel\" rel=\"Ext.tree.Panel-cfg-selModel\" class=\"docClass\">selModel</a> config.</p>\n"},"singleExpand":{"!type":"bool","!doc":"<p>True if only 1 node per branch may be expanded.</p>\n"},"store":{"!doc":"<p>The Store the tree should use as its data source.</p>\n"},"useArrows":{"!type":"bool","!doc":"<p>True to use Vista-style arrows in the tree.</p>\n"},"viewType":{"!type":"string","!doc":"<p>An xtype of view to use. This is automatically set to 'gridview' by <a href=\"#!/api/Ext.grid.Panel\" rel=\"Ext.grid.Panel\" class=\"docClass\">Grid</a>\nand to 'treeview' by <a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">Tree</a>.</p>\n"},"autoWidthCls":{"!type":"string"},"ddConfig":{},"isTree":{"!type":"bool"},"linesCls":{"!type":"string"},"lockedCfgCopy":{},"noLinesCls":{"!type":"string"},"normalCfgCopy":{"!doc":"<p>Required for the Lockable Mixin. These are the configurations which will be copied to the\nnormal and locked sub tablepanels</p>\n"},"treeCls":{"!type":"string"},"bindStore":{"!type":"fn(store: ?, initial: ?) -> !this","!doc":"<p>Hook into the TreeStore.\nDo not callParent in TreePanel's bindStore\nThe TreeStore is only relevant to the tree - the View has its own NodeStore</p>\n"},"collapseAll":{"!type":"fn(callback?: ?, scope?: ?) -> !this","!doc":"<p>Collapse all nodes</p>\n"},"collapseNode":{"!type":"fn(record: ?, deep?: ?, callback?: ?, scope?: ?) -> !this","!doc":"<p>Collapses a record that is loaded in the tree.</p>\n"},"expandAll":{"!type":"fn(callback?: ?, scope?: ?) -> !this","!doc":"<p>Expand all nodes</p>\n"},"expandNode":{"!type":"fn(record: ?, deep?: ?, callback?: ?, scope?: ?) -> !this","!doc":"<p>Expands a record that is loaded in the tree.</p>\n"},"expandPath":{"!type":"fn(path: ?, field?: ?, separator?: ?, callback?: ?, scope?: ?) -> !this","!doc":"<p>Expand the tree to the path of a particular node.</p>\n"},"getChecked":{"!type":"fn() -> ?","!doc":"<p>Retrieve an array of checked records.</p>\n"},"getRootNode":{"!type":"fn() -> ?","!doc":"<p>Returns the root node for this tree.</p>\n"},"initComponent":{"!type":"fn() -> !this","!doc":"<p>The initComponent template method is an important initialization step for a Component. It is intended to be\nimplemented by each subclass of <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> to provide any needed constructor logic. The\ninitComponent method of the class being created is called first, with each initComponent method\nup the hierarchy to <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> being called thereafter. This makes it easy to implement and,\nif needed, override the constructor logic of the Component at any step in the hierarchy.</p>\n\n<p>The initComponent method <strong>must</strong> contain a call to callParent in order\nto ensure that the parent class' initComponent method is also called.</p>\n\n<p>All config options passed to the constructor are applied to <code>this</code> before initComponent is called,\nso you can simply access them with <code>this.someOption</code>.</p>\n\n<p>The following example demonstrates using a dynamic string for the text of a button at the time of\ninstantiation of the class.</p>\n\n<pre><code>Ext.define('DynamicButtonText', {\n    extend: '<a href=\"#!/api/Ext.button.Button\" rel=\"Ext.button.Button\" class=\"docClass\">Ext.button.Button</a>',\n\n    initComponent: function() {\n        this.text = new Date();\n        this.renderTo = Ext.getBody();\n        this.callParent();\n    }\n});\n\nExt.onReady(function() {\n    Ext.create('DynamicButtonText');\n});\n</code></pre>\n"},"isItemChecked":{"!type":"fn(rec: ?) -> !this"},"onClear":{"!type":"fn() -> !this"},"onRootChange":{"!type":"fn(root: ?) -> !this"},"selectPath":{"!type":"fn(path: ?, field?: ?, separator?: ?, callback?: ?, scope?: ?) -> !this","!doc":"<p>Expand the tree to the path of a particular node, then select it.</p>\n"},"setRootNode":{"!type":"fn(root: ?) -> ?","!doc":"<p>Sets root node of this tree. All trees <em>always</em> have a root node. It may be <a href=\"#!/api/Ext.tree.Panel-cfg-rootVisible\" rel=\"Ext.tree.Panel-cfg-rootVisible\" class=\"docClass\">hidden</a>.</p>\n\n<p>If the passed node has not already been loaded with child nodes, and has its expanded field set, this triggers the <a href=\"#!/api/Ext.tree.Panel-cfg-store\" rel=\"Ext.tree.Panel-cfg-store\" class=\"docClass\">store</a> to load the child nodes of the root.</p>\n"},"unbindStore":{"!type":"fn() -> !this"},"afteritemcollapse":{"!type":"fn(node: ?, index: ?, item: ?, eOpts: ?)","!doc":"<p>Fires after an item has been visually collapsed and is no longer visible in the tree.</p>\n"},"afteritemexpand":{"!type":"fn(node: ?, index: ?, item: ?, eOpts: ?)","!doc":"<p>Fires after an item has been visually expanded and is visible in the tree.</p>\n"},"beforeitemappend":{"!type":"fn(eOpts: ?)"},"beforeitemcollapse":{"!type":"fn(eOpts: ?)"},"beforeitemexpand":{"!type":"fn(eOpts: ?)"},"beforeiteminsert":{"!type":"fn(eOpts: ?)"},"beforeitemmove":{"!type":"fn(eOpts: ?)"},"beforeitemremove":{"!type":"fn(eOpts: ?)"},"beforeload":{"!type":"fn(eOpts: ?)"},"checkchange":{"!type":"fn(node: ?, checked: ?, eOpts: ?)","!doc":"<p>Fires when a node with a checkbox's checked property changes</p>\n"},"itemappend":{"!type":"fn(eOpts: ?)"},"itemcollapse":{"!type":"fn(eOpts: ?)"},"itemexpand":{"!type":"fn(eOpts: ?)"},"iteminsert":{"!type":"fn(eOpts: ?)"},"itemmove":{"!type":"fn(eOpts: ?)"},"itemremove":{"!type":"fn(eOpts: ?)"},"load":{"!type":"fn(eOpts: ?)"}}},"plugin":{"TreeViewDragDrop":{"!doc":"<p>This plugin provides drag and/or drop functionality for a TreeView.</p>\n\n<p>It creates a specialized instance of <a href=\"#!/api/Ext.dd.DragZone\" rel=\"Ext.dd.DragZone\" class=\"docClass\">DragZone</a> which knows how to drag out of a\n<a href=\"#!/api/Ext.tree.View\" rel=\"Ext.tree.View\" class=\"docClass\">TreeView</a> and loads the data object which is passed to a cooperating\n<a href=\"#!/api/Ext.dd.DragZone\" rel=\"Ext.dd.DragZone\" class=\"docClass\">DragZone</a>'s methods with the following properties:</p>\n\n<ul>\n<li><p>copy : Boolean</p>\n\n<p>The value of the TreeView's <code>copy</code> property, or <code>true</code> if the TreeView was configured with <code>allowCopy: true</code> <em>and</em>\nthe control key was pressed when the drag operation was begun.</p></li>\n<li><p>view : TreeView</p>\n\n<p>The source TreeView from which the drag originated.</p></li>\n<li><p>ddel : HTMLElement</p>\n\n<p>The drag proxy element which moves with the mouse</p></li>\n<li><p>item : HTMLElement</p>\n\n<p>The TreeView node upon which the mousedown event was registered.</p></li>\n<li><p>records : Array</p>\n\n<p>An Array of Models representing the selected data being dragged from the source TreeView.</p></li>\n</ul>\n\n\n<p>It also creates a specialized instance of <a href=\"#!/api/Ext.dd.DropZone\" rel=\"Ext.dd.DropZone\" class=\"docClass\">Ext.dd.DropZone</a> which cooperates with other DropZones which are\nmembers of the same ddGroup which processes such data objects.</p>\n\n<p>Adding this plugin to a view means that two new events may be fired from the client TreeView, <a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-event-beforedrop\" rel=\"Ext.tree.plugin.TreeViewDragDrop-event-beforedrop\" class=\"docClass\">beforedrop</a> and\n<a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-event-drop\" rel=\"Ext.tree.plugin.TreeViewDragDrop-event-drop\" class=\"docClass\">drop</a>.</p>\n\n<p>Note that the plugin must be added to the tree view, not to the tree panel. For example using viewConfig:</p>\n\n<pre><code>viewConfig: {\n    plugins: { ptype: 'treeviewdragdrop' }\n}\n</code></pre>\n","!type":"fn(config?: ?)","prototype":{"allowContainerDrops":{"!type":"bool","!doc":"<p>True if drops on the tree container (outside of a specific tree node) are allowed.</p>\n"},"allowParentInserts":{"!type":"bool","!doc":"<p>Allow inserting a dragged node between an expanded parent node and its first child that will become a sibling of\nthe parent when dropped.</p>\n"},"appendOnly":{"!type":"bool","!doc":"<p>True if the tree should only allow append drops (use for trees which are sorted).</p>\n"},"containerScroll":{"!doc":"<p>True to register this container with the Scrollmanager for auto scrolling during drag operations.\nA <a href=\"#!/api/Ext.dd.ScrollManager\" rel=\"Ext.dd.ScrollManager\" class=\"docClass\">Ext.dd.ScrollManager</a> configuration may also be passed.</p>\n"},"ddGroup":{"!type":"string","!doc":"<p>A named drag drop group to which this object belongs. If a group is specified, then both the DragZones and\nDropZone used by this plugin will only interact with other drag drop objects in the same group.</p>\n"},"displayField":{"!type":"string","!doc":"<p>The name of the model field that is used to display the text for the nodes</p>\n"},"dragGroup":{"!type":"string","!doc":"<p>The ddGroup to which the <a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-property-dragZone\" rel=\"Ext.tree.plugin.TreeViewDragDrop-property-dragZone\" class=\"docClass\">DragZone</a> will belong.</p>\n\n<p>This defines which other DropZones the DragZone will interact with. Drag/DropZones only interact with other\nDrag/DropZones which are members of the same ddGroup.</p>\n"},"dragText":{"!type":"string","!doc":"<p>The text to show while dragging.</p>\n\n<p>Two placeholders can be used in the text:</p>\n\n<ul>\n<li><code>{0}</code> The number of selected items.</li>\n<li><code>{1}</code> 's' when more than 1 items (only useful for English).</li>\n</ul>\n\n"},"dragZone":{"!doc":"<p>An <a href=\"#!/api/Ext.view.DragZone\" rel=\"Ext.view.DragZone\" class=\"docClass\">DragZone</a> which handles mousedown and dragging of records from the grid.</p>\n"},"dropGroup":{"!type":"string","!doc":"<p>The ddGroup to which the <a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-property-dropZone\" rel=\"Ext.tree.plugin.TreeViewDragDrop-property-dropZone\" class=\"docClass\">DropZone</a> will belong.</p>\n\n<p>This defines which other DragZones the DropZone will interact with. Drag/DropZones only interact with other\nDrag/DropZones which are members of the same <a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-ddGroup\" rel=\"Ext.tree.plugin.TreeViewDragDrop-cfg-ddGroup\" class=\"docClass\">ddGroup</a>.</p>\n"},"dropZone":{"!doc":"<p>An <a href=\"#!/api/Ext.grid.ViewDropZone\" rel=\"Ext.grid.ViewDropZone\" class=\"docClass\">DropZone</a> which handles mouseover and dropping records in any grid which shares the same <a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-dropGroup\" rel=\"Ext.tree.plugin.TreeViewDragDrop-cfg-dropGroup\" class=\"docClass\">dropGroup</a>.</p>\n"},"enableDrag":{"!type":"bool","!doc":"<p>Set to <code>false</code> to disallow dragging items from the View.</p>\n"},"enableDrop":{"!type":"bool","!doc":"<p>Set to <code>false</code> to disallow the View from accepting drop gestures.</p>\n"},"expandDelay":{"!type":"number","!doc":"<p>The delay in milliseconds to wait before expanding a target tree node while dragging a droppable node over the\ntarget.</p>\n"},"nodeHighlightColor":{"!type":"string","!doc":"<p>The color to use when visually highlighting the dragged or dropped node (default value is light blue).\nThe color must be a 6 digit hex value, without a preceding '#'. See also <a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnDrop\" rel=\"Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnDrop\" class=\"docClass\">nodeHighlightOnDrop</a> and\n<a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnRepair\" rel=\"Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnRepair\" class=\"docClass\">nodeHighlightOnRepair</a>.</p>\n"},"nodeHighlightOnDrop":{"!type":"bool","!doc":"<p>Whether or not to highlight any nodes after they are\nsuccessfully dropped on their target. Defaults to the value of <code><a href=\"#!/api/Ext-property-enableFx\" rel=\"Ext-property-enableFx\" class=\"docClass\">Ext.enableFx</a></code>.\nSee also <a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightColor\" rel=\"Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightColor\" class=\"docClass\">nodeHighlightColor</a> and <a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnRepair\" rel=\"Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnRepair\" class=\"docClass\">nodeHighlightOnRepair</a>.</p>\n"},"nodeHighlightOnRepair":{"!type":"bool","!doc":"<p>Whether or not to highlight any nodes after they are\nrepaired from an unsuccessful drag/drop. Defaults to the value of <code><a href=\"#!/api/Ext-property-enableFx\" rel=\"Ext-property-enableFx\" class=\"docClass\">Ext.enableFx</a></code>.\nSee also <a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightColor\" rel=\"Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightColor\" class=\"docClass\">nodeHighlightColor</a> and <a href=\"#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnDrop\" rel=\"Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnDrop\" class=\"docClass\">nodeHighlightOnDrop</a>.</p>\n"},"sortOnDrop":{"!type":"bool","!doc":"<p>Configure as <code>true</code> to sort the target node into the current tree sort order after the dropped node is added.</p>\n"},"destroy":{"!type":"fn() -> !this","!doc":"<p>Component calls destroy on all its plugins at destroy time.</p>\n"},"onViewRender":{"!type":"fn(view: ?) -> !this"},"beforedrop":{"!type":"fn(node: ?, data: ?, overModel: ?, dropPosition: ?, dropHandlers: ?, eOpts: ?)","!doc":"<p><strong>This event is fired through the TreeView. Add listeners to the TreeView object</strong></p>\n\n<p>Fired when a drop gesture has been triggered by a mouseup event in a valid drop position in the TreeView.</p>\n\n<p>Returning <code>false</code> to this event signals that the drop gesture was invalid, and if the drag proxy will animate\nback to the point from which the drag began.</p>\n\n<p>The dropHandlers parameter can be used to defer the processing of this event. For example to wait for the result of\na message box confirmation or an asynchronous server call. See the details of this property for more information.</p>\n\n<pre class='inline-example '><code>view.on('beforedrop', function(node, data, overModel, dropPosition, dropHandlers) {\n    // Defer the handling\n    dropHandlers.wait = true;\n    <a href=\"#!/api/Ext.MessageBox-method-confirm\" rel=\"Ext.MessageBox-method-confirm\" class=\"docClass\">Ext.MessageBox.confirm</a>('Drop', 'Are you sure', function(btn){\n        if (btn === 'yes') {\n            dropHandlers.processDrop();\n        } else {\n            dropHandlers.cancelDrop();\n        }\n    });\n});\n</code></pre>\n\n<p>Any other return value continues with the data transfer operation, unless the wait property is set.</p>\n"},"drop":{"!type":"fn(node: ?, data: ?, overModel: ?, dropPosition: ?, eOpts: ?)","!doc":"<p><strong>This event is fired through the TreeView. Add listeners to the TreeView object</strong> Fired when a drop operation\nhas been completed and the data has been moved or copied.</p>\n"}}}},"View":{"!doc":"<p>Used as a view by <a href=\"#!/api/Ext.tree.Panel\" rel=\"Ext.tree.Panel\" class=\"docClass\">TreePanel</a>.</p>\n","!type":"fn(config: ?)","prototype":{"animate":{"!type":"bool","!doc":"<p>True to enable animated expand/collapse (defaults to the value of <a href=\"#!/api/Ext-property-enableFx\" rel=\"Ext-property-enableFx\" class=\"docClass\">Ext.enableFx</a>)</p>\n"},"loadMask":{"!type":"bool","!doc":"<p>False to disable a load mask from displaying while the view is loading. This can also be a\n<a href=\"#!/api/Ext.LoadMask\" rel=\"Ext.LoadMask\" class=\"docClass\">Ext.LoadMask</a> configuration object.</p>\n"},"loadingCls":{"!type":"string","!doc":"<p>The CSS class to apply to the loading message element. Defaults to Ext.LoadMask.prototype.msgCls \"x-mask-loading\".</p>\n"},"rootVisible":{"!type":"bool","!doc":"<p>False to hide the root node.</p>\n"},"stripeRows":{"!type":"bool","!doc":"<p>True to stripe the rows.</p>\n\n<p>This causes the CSS class <strong><code>x-grid-row-alt</code></strong> to be added to alternate rows of\nthe grid. A default CSS rule is provided which sets a background color, but you can override this\nwith a rule which either overrides the <strong>background-color</strong> style using the <code>!important</code>\nmodifier, or which uses a CSS selector of higher specificity.</p>\n"},"ariaRole":{"!type":"string"},"checkboxSelector":{"!type":"string"},"collapseDuration":{"!type":"number"},"expandedCls":{"!type":"string"},"expanderIconOverCls":{"!type":"string"},"expanderSelector":{"!type":"string"},"isTreeView":{"!type":"bool","!doc":"<p><code>true</code> in this class to identify an object as an instantiated TreeView, or subclass thereof.</p>\n"},"leafCls":{"!type":"string"},"nodeAnimWrapCls":{"!type":"string","!doc":"<p>Class to add to the node wrap element used to hold nodes when a parent is being\ncollapsed or expanded. During the animation, UI interaction is forbidden by testing\nfor an ancestor node with this class.</p>\n"},"toggleOnDblClick":{"!type":"bool"},"treeRowTpl":{"!doc":"<p>treeRowTpl which is inserted into the rowTpl chain before the base rowTpl. Sets tree-specific classes and attributes</p>\n"},"uiFields":{"!doc":"<p>fields that will trigger a change in the ui that aren't likely to be bound to a column</p>\n"},"afterComponentLayout":{"!type":"fn() -> !this","!doc":"<p>Called by the layout system after the Component has been laid out.</p>\n"},"afterRender":{"!type":"fn() -> !this","!doc":"<p>Allows addition of behavior after rendering is complete. At this stage the Component's Element\nwill have been styled according to the configuration, will have had any configured CSS class\nnames added, and will be in the configured visibility and the configured enable state.</p>\n"},"collapse":{"!type":"fn(record: ?, deep?: ?, callback?: ?, scope?: ?) -> !this","!doc":"<p>Collapses a record that is loaded in the view.</p>\n\n<p>If an animated collapse or expand of the record is in progress, this call will be ignored.</p>\n"},"createAnimWrap":{"!type":"fn(record: ?, index: ?) -> !this"},"doAdd":{"!type":"fn(records: ?, index: ?) -> !this"},"doRemove":{"!type":"fn(record: ?, index: ?) -> !this","!doc":"<p>private</p>\n"},"ensureSingleExpand":{"!type":"fn(node: ?) -> !this"},"expand":{"!type":"fn(record: ?, deep?: ?, callback?: ?, scope?: ?) -> !this","!doc":"<p>Expands a record that is loaded in the view.</p>\n\n<p>If an animated collapse or expand of the record is in progress, this call will be ignored.</p>\n"},"getAnimWrap":{"!type":"fn(parent: ?, bubble?: ?) -> ?","!doc":"<p>Returns the animation wrapper element for the specified parent node, used to wrap the child nodes as\nthey slide up or down during expand/collapse.</p>\n"},"getChecked":{"!type":"fn() -> !this"},"getMaskStore":{"!type":"fn() -> !this"},"getStoreListeners":{"!type":"fn() -> ?","!doc":"<p>Gets the listeners to bind to a new store.</p>\n"},"isAnimating":{"!type":"fn(node: ?) -> bool","!doc":"<p>Checks if a node is currently undergoing animation</p>\n"},"isItemChecked":{"!type":"fn(rec: ?) -> !this"},"onBeforeCollapse":{"!type":"fn(parent: ?, records: ?, index: ?, callback: ?, scope: ?) -> !this","!doc":"<p>Triggered by the TreeStore's beforecollapse event.</p>\n"},"onBeforeExpand":{"!type":"fn(parent: ?, records: ?, index: ?) -> !this"},"onBeforeItemMouseDown":{"!type":"fn(record: ?, item: ?, index: ?, e: ?) -> !this"},"onBeforeSort":{"!type":"fn() -> !this"},"onCheckChange":{"!type":"fn(record: ?) -> !this"},"onCheckboxChange":{"!type":"fn(e: ?, t: ?) -> !this"},"onClear":{"!type":"fn() -> !this"},"onCollapse":{"!type":"fn(parent: ?) -> !this"},"onExpand":{"!type":"fn(parent: ?) -> !this"},"onExpanderMouseOut":{"!type":"fn(e: ?, t: ?) -> !this"},"onExpanderMouseOver":{"!type":"fn(e: ?, t: ?) -> !this"},"onFillComplete":{"!type":"fn(treeStore: ?, fillRoot: ?, newNodes: ?) -> !this"},"onItemClick":{"!type":"fn(record: ?, item: ?, index: ?, e: ?) -> !this"},"onItemDblClick":{"!type":"fn(record: ?, item: ?, index: ?) -> !this"},"onRemove":{"!type":"fn(ds: ?, records: ?, index: ?) -> !this","!doc":"<p>after removing a row stripe rows from then on</p>\n"},"onRootChange":{"!type":"fn(newRoot: ?, oldRoot: ?) -> !this"},"onSort":{"!type":"fn(o: ?) -> !this"},"processUIEvent":{"!type":"fn(e: ?) -> !this"},"refreshPartner":{"!type":"fn() -> !this"},"setRootNode":{"!type":"fn(node: ?, preventSetOnStore: ?) -> !this"},"shouldUpdateCell":{"!type":"fn(record: ?, column: ?, changedFieldNames: ?) -> number","!doc":"<p>Decides whether the column needs updating</p>\n"},"toggle":{"!type":"fn(record: ?, deep?: ?, callback?: ?, scope?: ?) -> !this","!doc":"<p>Toggles a record between expanded and collapsed.</p>\n\n<p>If an animated collapse or expand of the record is in progress, this call will be ignored.</p>\n"},"afteritemcollapse":{"!type":"fn(node: ?, index: ?, item: ?, eOpts: ?)","!doc":"<p>Fires after an item has been visually collapsed and is no longer visible in the tree.</p>\n"},"afteritemexpand":{"!type":"fn(node: ?, index: ?, item: ?, eOpts: ?)","!doc":"<p>Fires after an item has been visually expanded and is visible in the tree.</p>\n"},"nodedragover":{"!type":"fn(targetNode: ?, position: ?, dragData: ?, e: ?, eOpts: ?)","!doc":"<p>Fires when a tree node is being targeted for a drag drop, return false to signal drop not allowed.</p>\n"}}},"ViewDragZone":{"!type":"fn(config: ?)","prototype":{"afterRepair":{"!type":"fn() -> !this"},"getDragText":{"!type":"fn() -> !this"},"isPreventDrag":{"!type":"fn(e: ?, record: ?) -> !this","!doc":"<p>Template method called upon mousedown. May be overridden in subclasses, or configured\ninto an instance.</p>\n\n<p>Return <code>true</code> to prevent drag start.</p>\n"}}},"ViewDropZone":{"!type":"fn(config: ?)","prototype":{"allowContainerDrops":{"!type":"bool","!doc":"<p>True if drops on the tree container (outside of a specific tree node) are allowed.</p>\n\n<p>These are treated as appends to the root node.</p>\n"},"allowParentInserts":{"!type":"bool","!doc":"<p>Allow inserting a dragged node between an expanded parent node and its first child that will become a\nsibling of the parent when dropped.</p>\n"},"appendOnly":{"!type":"bool","!doc":"<p>True if the tree should only allow append drops (use for trees which are sorted).</p>\n"},"expandDelay":{"!type":"number","!doc":"<p>The delay in milliseconds to wait before expanding a target tree node while dragging a droppable node\nover the target.</p>\n"},"indicatorCls":{"!type":"string"},"cancelExpand":{"!type":"fn() -> !this"},"expandNode":{"!type":"fn(node: ?) -> !this"},"getPosition":{"!type":"fn(e: ?, node: ?) -> !this"},"handleNodeDrop":{"!type":"fn(data: ?, targetNode: ?, position: ?) -> !this"},"isValidDropPoint":{"!type":"fn(node: ?, position: ?, dragZone: ?, e: ?, data: ?) -> !this"},"notifyOut":{"!type":"fn() -> !this","!doc":"<p>Moved out of the DropZone without dropping.\nRemove drop position indicator</p>\n"},"onContainerDrop":{"!type":"fn(dragZone: ?, e: ?, data: ?) -> bool","!doc":"<p>This will be called is allowContainerDrops is set.\nThe target node is the root</p>\n"},"onContainerOver":{"!type":"fn(dd: ?, e: ?, data: ?) -> string","!doc":"<p>The mouse is past the end of all nodes (or there are no nodes)</p>\n"},"onNodeOut":{"!type":"fn(n: ?, dd: ?, e: ?, data: ?) -> !this","!doc":"<p>The mouse is no longer over a tree node, so dropping is not valid</p>\n"},"onNodeOver":{"!type":"fn(node: ?, dragZone: ?, e: ?, data: ?) -> string","!doc":"<p>The mouse is over a View node</p>\n"},"queueExpand":{"!type":"fn(node: ?) -> !this"}}}},"WindowManager":{"!doc":"<p>The default global floating Component group that is available automatically.</p>\n\n<p>This manages instances of floating Components which were rendered programatically without\nbeing added to a <a href=\"#!/api/Ext.container.Container\" rel=\"Ext.container.Container\" class=\"docClass\">Container</a>, and for floating Components\nwhich were added into non-floating Containers.</p>\n\n<p><em>Floating</em> Containers create their own instance of ZIndexManager, and floating Components\nadded at any depth below there are managed by that ZIndexManager.</p>\n","!type":"fn(container: ?)"},"ZIndexManager":{"_activateLast":{"!type":"fn() -> !this"},"_hideModalMask":{"!type":"fn() -> !this"},"_onContainerResize":{"!type":"fn() -> !this"},"_onMaskClick":{"!type":"fn() -> !this"},"_setActiveChild":{"!type":"fn(comp: ?, oldFront: ?) -> !this"},"_showModalMask":{"!type":"fn(comp: ?) -> !this"},"assignZIndices":{"!type":"fn() -> !this"},"bringToFront":{"!type":"fn(comp: ?) -> bool","!doc":"<p>Brings the specified Component to the front of any other active Components in this ZIndexManager.</p>\n"},"destroy":{"!type":"fn() -> !this"},"each":{"!type":"fn(fn: ?, scope?: ?) -> !this","!doc":"<p>Executes the specified function once for every Component in this ZIndexManager, passing each\nComponent as the only parameter. Returning false from the function will stop the iteration.</p>\n"},"eachBottomUp":{"!type":"fn(fn: ?, scope?: ?) -> !this","!doc":"<p>Executes the specified function once for every Component in this ZIndexManager, passing each\nComponent as the only parameter. Returning false from the function will stop the iteration.\nThe components are passed to the function starting at the bottom and proceeding to the top.</p>\n"},"eachTopDown":{"!type":"fn(fn: ?, scope?: ?) -> !this","!doc":"<p>Executes the specified function once for every Component in this ZIndexManager, passing each\nComponent as the only parameter. Returning false from the function will stop the iteration.\nThe components are passed to the function starting at the top and proceeding to the bottom.</p>\n"},"get":{"!type":"fn(id: ?) -> ?","!doc":"<p>Gets a registered Component by id.</p>\n"},"getActive":{"!type":"fn() -> ?","!doc":"<p>Gets the currently-active Component in this ZIndexManager.</p>\n"},"getBy":{"!type":"fn(fn: ?, scope?: ?) -> ?","!doc":"<p>Returns zero or more Components in this ZIndexManager using the custom search function passed to this method.\nThe function should accept a single <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> reference as its only argument and should\nreturn true if the Component matches the search criteria, otherwise it should return false.</p>\n"},"getMaskBox":{"!type":"fn() -> !this"},"getNextZSeed":{"!type":"fn() -> !this"},"hide":{"!type":"fn() -> !this","!doc":"<p>Temporarily hides all currently visible managed Components. This is for when\ndragging a Window which may manage a set of floating descendants in its ZIndexManager;\nthey should all be hidden just for the duration of the drag.</p>\n"},"hideAll":{"!type":"fn() -> !this","!doc":"<p>Hides all Components managed by this ZIndexManager.</p>\n"},"onComponentHide":{"!type":"fn(comp: ?) -> !this"},"register":{"!type":"fn(comp: ?) -> !this","!doc":"<p>Registers a floating <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> with this ZIndexManager. This should not\nneed to be called under normal circumstances. Floating Components (such as Windows,\nBoundLists and Menus) are automatically registered with a\n<a href=\"#!/api/Ext.Component-property-zIndexManager\" rel=\"Ext.Component-property-zIndexManager\" class=\"docClass\">zIndexManager</a> at render time.</p>\n\n<p>Where this may be useful is moving Windows between two ZIndexManagers. For example,\nto bring the <a href=\"#!/api/Ext.MessageBox\" rel=\"Ext.MessageBox\" class=\"docClass\">Ext.MessageBox</a> dialog under the same manager as the Desktop's\nZIndexManager in the desktop sample app:</p>\n\n<pre><code>MyDesktop.getDesktop().getManager().register(<a href=\"#!/api/Ext.MessageBox\" rel=\"Ext.MessageBox\" class=\"docClass\">Ext.MessageBox</a>);\n</code></pre>\n"},"sendToBack":{"!type":"fn(comp: ?) -> ?","!doc":"<p>Sends the specified Component to the back of other active Components in this ZIndexManager.</p>\n"},"setBase":{"!type":"fn(baseZIndex: ?) -> !this"},"show":{"!type":"fn() -> !this","!doc":"<p>Restores temporarily hidden managed Components to visibility.</p>\n"},"unregister":{"!type":"fn(comp: ?) -> !this","!doc":"<p>Unregisters a <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> from this ZIndexManager. This should not\nneed to be called. Components are automatically unregistered upon destruction.\nSee <a href=\"#!/api/Ext.ZIndexManager-method-register\" rel=\"Ext.ZIndexManager-method-register\" class=\"docClass\">register</a>.</p>\n"},"!doc":"<p>A class that manages a group of <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">Ext.Component.floating</a> Components and provides z-order management,\nand Component activation behavior, including masking below the active (topmost) Component.</p>\n\n<p><a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">Floating</a> Components which are rendered directly into the document (such as\n<a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Window</a>s) which are <a href=\"#!/api/Ext.Component-method-show\" rel=\"Ext.Component-method-show\" class=\"docClass\">show</a>n are managed by a\n<a href=\"#!/api/Ext.WindowManager\" rel=\"Ext.WindowManager\" class=\"docClass\">global instance</a>.</p>\n\n<p><a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">Floating</a> Components which are descendants of <a href=\"#!/api/Ext.Component-cfg-floating\" rel=\"Ext.Component-cfg-floating\" class=\"docClass\">floating</a>\n<em>Containers</em> (for example a <a href=\"#!/api/Ext.view.BoundList\" rel=\"Ext.view.BoundList\" class=\"docClass\">BoundList</a> within an <a href=\"#!/api/Ext.window.Window\" rel=\"Ext.window.Window\" class=\"docClass\">Window</a>,\nor a <a href=\"#!/api/Ext.menu.Menu\" rel=\"Ext.menu.Menu\" class=\"docClass\">Menu</a>), are managed by a ZIndexManager owned by that floating Container. Therefore\nComboBox dropdowns within Windows will have managed z-indices guaranteed to be correct, relative to the Window.</p>\n","!type":"fn(container: ?)","prototype":{"zBase":{"!type":"number"},"_activateLast":{"!type":"fn() -> !this"},"_hideModalMask":{"!type":"fn() -> !this"},"_onContainerResize":{"!type":"fn() -> !this"},"_onMaskClick":{"!type":"fn() -> !this"},"_setActiveChild":{"!type":"fn(comp: ?, oldFront: ?) -> !this"},"_showModalMask":{"!type":"fn(comp: ?) -> !this"},"assignZIndices":{"!type":"fn() -> !this"},"bringToFront":{"!type":"fn(comp: ?) -> bool","!doc":"<p>Brings the specified Component to the front of any other active Components in this ZIndexManager.</p>\n"},"destroy":{"!type":"fn() -> !this"},"each":{"!type":"fn(fn: ?, scope?: ?) -> !this","!doc":"<p>Executes the specified function once for every Component in this ZIndexManager, passing each\nComponent as the only parameter. Returning false from the function will stop the iteration.</p>\n"},"eachBottomUp":{"!type":"fn(fn: ?, scope?: ?) -> !this","!doc":"<p>Executes the specified function once for every Component in this ZIndexManager, passing each\nComponent as the only parameter. Returning false from the function will stop the iteration.\nThe components are passed to the function starting at the bottom and proceeding to the top.</p>\n"},"eachTopDown":{"!type":"fn(fn: ?, scope?: ?) -> !this","!doc":"<p>Executes the specified function once for every Component in this ZIndexManager, passing each\nComponent as the only parameter. Returning false from the function will stop the iteration.\nThe components are passed to the function starting at the top and proceeding to the bottom.</p>\n"},"get":{"!type":"fn(id: ?) -> ?","!doc":"<p>Gets a registered Component by id.</p>\n"},"getActive":{"!type":"fn() -> ?","!doc":"<p>Gets the currently-active Component in this ZIndexManager.</p>\n"},"getBy":{"!type":"fn(fn: ?, scope?: ?) -> ?","!doc":"<p>Returns zero or more Components in this ZIndexManager using the custom search function passed to this method.\nThe function should accept a single <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> reference as its only argument and should\nreturn true if the Component matches the search criteria, otherwise it should return false.</p>\n"},"getMaskBox":{"!type":"fn() -> !this"},"getNextZSeed":{"!type":"fn() -> !this"},"hide":{"!type":"fn() -> !this","!doc":"<p>Temporarily hides all currently visible managed Components. This is for when\ndragging a Window which may manage a set of floating descendants in its ZIndexManager;\nthey should all be hidden just for the duration of the drag.</p>\n"},"hideAll":{"!type":"fn() -> !this","!doc":"<p>Hides all Components managed by this ZIndexManager.</p>\n"},"onComponentHide":{"!type":"fn(comp: ?) -> !this"},"register":{"!type":"fn(comp: ?) -> !this","!doc":"<p>Registers a floating <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> with this ZIndexManager. This should not\nneed to be called under normal circumstances. Floating Components (such as Windows,\nBoundLists and Menus) are automatically registered with a\n<a href=\"#!/api/Ext.Component-property-zIndexManager\" rel=\"Ext.Component-property-zIndexManager\" class=\"docClass\">zIndexManager</a> at render time.</p>\n\n<p>Where this may be useful is moving Windows between two ZIndexManagers. For example,\nto bring the <a href=\"#!/api/Ext.MessageBox\" rel=\"Ext.MessageBox\" class=\"docClass\">Ext.MessageBox</a> dialog under the same manager as the Desktop's\nZIndexManager in the desktop sample app:</p>\n\n<pre><code>MyDesktop.getDesktop().getManager().register(<a href=\"#!/api/Ext.MessageBox\" rel=\"Ext.MessageBox\" class=\"docClass\">Ext.MessageBox</a>);\n</code></pre>\n"},"sendToBack":{"!type":"fn(comp: ?) -> ?","!doc":"<p>Sends the specified Component to the back of other active Components in this ZIndexManager.</p>\n"},"setBase":{"!type":"fn(baseZIndex: ?) -> !this"},"show":{"!type":"fn() -> !this","!doc":"<p>Restores temporarily hidden managed Components to visibility.</p>\n"},"unregister":{"!type":"fn(comp: ?) -> !this","!doc":"<p>Unregisters a <a href=\"#!/api/Ext.Component\" rel=\"Ext.Component\" class=\"docClass\">Ext.Component</a> from this ZIndexManager. This should not\nneed to be called. Components are automatically unregistered upon destruction.\nSee <a href=\"#!/api/Ext.ZIndexManager-method-register\" rel=\"Ext.ZIndexManager-method-register\" class=\"docClass\">register</a>.</p>\n"}}}}}

});  
